"bundle";
System.registerDynamic('node_modules/@angular/core/testing/async.js', [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var _global = typeof window === 'undefined' ? global : window;
    /**
     * Wraps a test function in an asynchronous test zone. The test will automatically
     * complete when all asynchronous calls within this zone are done. Can be used
     * to wrap an {@link inject} call.
     *
     * Example:
     *
     * ```
     * it('...', async(inject([AClass], (object) => {
     *   object.doSomething.then(() => {
     *     expect(...);
     *   })
     * });
     * ```
     *
     * @stable
     */
    function async(fn) {
        // If we're running using the Jasmine test framework, adapt to call the 'done'
        // function when asynchronous activity is finished.
        if (_global.jasmine) {
            return function (done) {
                runInTestZone(fn, done, function (err) {
                    if (typeof err === 'string') {
                        return done.fail(new Error(err));
                    } else {
                        done.fail(err);
                    }
                });
            };
        }
        // Otherwise, return a promise which will resolve when asynchronous activity
        // is finished. This will be correctly consumed by the Mocha framework with
        // it('...', async(myFn)); or can be used in a custom framework.
        return function () {
            return new Promise(function (finishCallback, failCallback) {
                runInTestZone(fn, finishCallback, failCallback);
            });
        };
    }
    exports.async = async;
    function runInTestZone(fn, finishCallback, failCallback) {
        var AsyncTestZoneSpec = Zone['AsyncTestZoneSpec'];
        if (AsyncTestZoneSpec === undefined) {
            throw new Error('AsyncTestZoneSpec is needed for the async() test helper but could not be found. ' + 'Please make sure that your environment includes zone.js/dist/async-test.js');
        }
        var testZoneSpec = new AsyncTestZoneSpec(finishCallback, failCallback, 'test');
        var testZone = Zone.current.fork(testZoneSpec);
        return testZone.run(fn);
    }
    
});
System.registerDynamic('node_modules/@angular/core/testing/component_fixture.js', ['../index', '../src/facade/async', '../src/facade/exceptions', '../src/facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var index_1 = $__require('../index');
    var async_1 = $__require('../src/facade/async');
    var exceptions_1 = $__require('../src/facade/exceptions');
    var lang_1 = $__require('../src/facade/lang');
    /**
     * Fixture for debugging and testing a component.
     *
     * @stable
     */
    var ComponentFixture = function () {
        function ComponentFixture(componentRef, ngZone, autoDetect) {
            var _this = this;
            this._isStable = true;
            this._completer = null;
            this._onUnstableSubscription = null;
            this._onStableSubscription = null;
            this._onMicrotaskEmptySubscription = null;
            this._onErrorSubscription = null;
            this.changeDetectorRef = componentRef.changeDetectorRef;
            this.elementRef = componentRef.location;
            this.debugElement = index_1.getDebugNode(this.elementRef.nativeElement);
            this.componentInstance = componentRef.instance;
            this.nativeElement = this.elementRef.nativeElement;
            this.componentRef = componentRef;
            this.ngZone = ngZone;
            this._autoDetect = autoDetect;
            if (ngZone != null) {
                this._onUnstableSubscription = async_1.ObservableWrapper.subscribe(ngZone.onUnstable, function (_) {
                    _this._isStable = false;
                });
                this._onMicrotaskEmptySubscription = async_1.ObservableWrapper.subscribe(ngZone.onMicrotaskEmpty, function (_) {
                    if (_this._autoDetect) {
                        // Do a change detection run with checkNoChanges set to true to check
                        // there are no changes on the second run.
                        _this.detectChanges(true);
                    }
                });
                this._onStableSubscription = async_1.ObservableWrapper.subscribe(ngZone.onStable, function (_) {
                    _this._isStable = true;
                    // Check whether there are no pending macrotasks in a microtask so that ngZone gets a chance
                    // to update the state of pending macrotasks.
                    lang_1.scheduleMicroTask(function () {
                        if (!_this.ngZone.hasPendingMacrotasks) {
                            if (_this._completer != null) {
                                _this._completer.resolve(true);
                                _this._completer = null;
                            }
                        }
                    });
                });
                this._onErrorSubscription = async_1.ObservableWrapper.subscribe(ngZone.onError, function (error) {
                    throw error.error;
                });
            }
        }
        ComponentFixture.prototype._tick = function (checkNoChanges) {
            this.changeDetectorRef.detectChanges();
            if (checkNoChanges) {
                this.checkNoChanges();
            }
        };
        /**
         * Trigger a change detection cycle for the component.
         */
        ComponentFixture.prototype.detectChanges = function (checkNoChanges) {
            var _this = this;
            if (checkNoChanges === void 0) {
                checkNoChanges = true;
            }
            if (this.ngZone != null) {
                // Run the change detection inside the NgZone so that any async tasks as part of the change
                // detection are captured by the zone and can be waited for in isStable.
                this.ngZone.run(function () {
                    _this._tick(checkNoChanges);
                });
            } else {
                // Running without zone. Just do the change detection.
                this._tick(checkNoChanges);
            }
        };
        /**
         * Do a change detection run to make sure there were no changes.
         */
        ComponentFixture.prototype.checkNoChanges = function () {
            this.changeDetectorRef.checkNoChanges();
        };
        /**
         * Set whether the fixture should autodetect changes.
         *
         * Also runs detectChanges once so that any existing change is detected.
         */
        ComponentFixture.prototype.autoDetectChanges = function (autoDetect) {
            if (autoDetect === void 0) {
                autoDetect = true;
            }
            if (this.ngZone == null) {
                throw new exceptions_1.BaseException('Cannot call autoDetectChanges when ComponentFixtureNoNgZone is set');
            }
            this._autoDetect = autoDetect;
            this.detectChanges();
        };
        /**
         * Return whether the fixture is currently stable or has async tasks that have not been completed
         * yet.
         */
        ComponentFixture.prototype.isStable = function () {
            return this._isStable && !this.ngZone.hasPendingMacrotasks;
        };
        /**
         * Get a promise that resolves when the fixture is stable.
         *
         * This can be used to resume testing after events have triggered asynchronous activity or
         * asynchronous change detection.
         */
        ComponentFixture.prototype.whenStable = function () {
            if (this.isStable()) {
                return async_1.PromiseWrapper.resolve(false);
            } else if (this._completer !== null) {
                return this._completer.promise;
            } else {
                this._completer = new async_1.PromiseCompleter();
                return this._completer.promise;
            }
        };
        /**
         * Trigger component destruction.
         */
        ComponentFixture.prototype.destroy = function () {
            this.componentRef.destroy();
            if (this._onUnstableSubscription != null) {
                async_1.ObservableWrapper.dispose(this._onUnstableSubscription);
                this._onUnstableSubscription = null;
            }
            if (this._onStableSubscription != null) {
                async_1.ObservableWrapper.dispose(this._onStableSubscription);
                this._onStableSubscription = null;
            }
            if (this._onMicrotaskEmptySubscription != null) {
                async_1.ObservableWrapper.dispose(this._onMicrotaskEmptySubscription);
                this._onMicrotaskEmptySubscription = null;
            }
            if (this._onErrorSubscription != null) {
                async_1.ObservableWrapper.dispose(this._onErrorSubscription);
                this._onErrorSubscription = null;
            }
        };
        return ComponentFixture;
    }();
    exports.ComponentFixture = ComponentFixture;
    
});
System.registerDynamic('node_modules/@angular/core/testing/fake_async.js', ['../index'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var index_1 = $__require('../index');
    var _FakeAsyncTestZoneSpecType = Zone['FakeAsyncTestZoneSpec'];
    /**
     * Wraps a function to be executed in the fakeAsync zone:
     * - microtasks are manually executed by calling `flushMicrotasks()`,
     * - timers are synchronous, `tick()` simulates the asynchronous passage of time.
     *
     * If there are any pending timers at the end of the function, an exception will be thrown.
     *
     * Can be used to wrap inject() calls.
     *
     * ## Example
     *
     * {@example testing/ts/fake_async.ts region='basic'}
     *
     * @param fn
     * @returns {Function} The function wrapped to be executed in the fakeAsync zone
     *
     * @experimental
     */
    function fakeAsync(fn) {
        if (Zone.current.get('FakeAsyncTestZoneSpec') != null) {
            throw new index_1.BaseException('fakeAsync() calls can not be nested');
        }
        var fakeAsyncTestZoneSpec = new _FakeAsyncTestZoneSpecType();
        var fakeAsyncZone = Zone.current.fork(fakeAsyncTestZoneSpec);
        return function () {
            var args = []; /** TODO #9100 */
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i - 0] = arguments[_i];
            }
            var res = fakeAsyncZone.run(function () {
                var res = fn.apply(void 0, args);
                flushMicrotasks();
                return res;
            });
            if (fakeAsyncTestZoneSpec.pendingPeriodicTimers.length > 0) {
                throw new index_1.BaseException(fakeAsyncTestZoneSpec.pendingPeriodicTimers.length + " " + "periodic timer(s) still in the queue.");
            }
            if (fakeAsyncTestZoneSpec.pendingTimers.length > 0) {
                throw new index_1.BaseException(fakeAsyncTestZoneSpec.pendingTimers.length + " timer(s) still in the queue.");
            }
            return res;
        };
    }
    exports.fakeAsync = fakeAsync;
    function _getFakeAsyncZoneSpec() {
        var zoneSpec = Zone.current.get('FakeAsyncTestZoneSpec');
        if (zoneSpec == null) {
            throw new Error('The code should be running in the fakeAsync zone to call this function');
        }
        return zoneSpec;
    }
    /**
     * Simulates the asynchronous passage of time for the timers in the fakeAsync zone.
     *
     * The microtasks queue is drained at the very start of this function and after any timer callback
     * has been executed.
     *
     * ## Example
     *
     * {@example testing/ts/fake_async.ts region='basic'}
     *
     * @experimental
     */
    function tick(millis) {
        if (millis === void 0) {
            millis = 0;
        }
        _getFakeAsyncZoneSpec().tick(millis);
    }
    exports.tick = tick;
    /**
     * Discard all remaining periodic tasks.
     *
     * @experimental
     */
    function discardPeriodicTasks() {
        var zoneSpec = _getFakeAsyncZoneSpec();
        var pendingTimers = zoneSpec.pendingPeriodicTimers;
        zoneSpec.pendingPeriodicTimers.length = 0;
    }
    exports.discardPeriodicTasks = discardPeriodicTasks;
    /**
     * Flush any pending microtasks.
     *
     * @experimental
     */
    function flushMicrotasks() {
        _getFakeAsyncZoneSpec().flushMicrotasks();
    }
    exports.flushMicrotasks = flushMicrotasks;
    
});
System.registerDynamic('node_modules/@angular/core/testing/test_component_builder.js', ['../index', '../src/facade/async', '../src/facade/lang', './component_fixture', './fake_async'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var index_1 = $__require('../index');
    var async_1 = $__require('../src/facade/async');
    var lang_1 = $__require('../src/facade/lang');
    var component_fixture_1 = $__require('./component_fixture');
    var fake_async_1 = $__require('./fake_async');
    /**
     * An abstract class for inserting the root test component element in a platform independent way.
     *
     * @experimental
     */
    var TestComponentRenderer = function () {
        function TestComponentRenderer() {}
        TestComponentRenderer.prototype.insertRootElement = function (rootElementId) {};
        return TestComponentRenderer;
    }();
    exports.TestComponentRenderer = TestComponentRenderer;
    /**
     * @experimental
     */
    exports.ComponentFixtureAutoDetect = new index_1.OpaqueToken('ComponentFixtureAutoDetect');
    /**
     * @experimental
     */
    exports.ComponentFixtureNoNgZone = new index_1.OpaqueToken('ComponentFixtureNoNgZone');
    var _nextRootElementId = 0;
    var TestComponentBuilder = function () {
        function TestComponentBuilder(_injector) {
            this._injector = _injector;
        }
        /**
         * Overrides only the html of a {@link ComponentMetadata}.
         * All the other properties of the component's {@link ViewMetadata} are preserved.
         */
        TestComponentBuilder.prototype.overrideTemplate = function (componentType, template) {
            throw new Error('overrideTemplate is not supported in this implementation of TestComponentBuilder.');
        };
        /**
         * Overrides a component's {@link ViewMetadata}.
         */
        TestComponentBuilder.prototype.overrideView = function (componentType, view) {
            throw new Error('overrideView is not supported in this implementation of TestComponentBuilder.');
        };
        /**
         * Overrides the directives from the component {@link ViewMetadata}.
         */
        TestComponentBuilder.prototype.overrideDirective = function (componentType, from, to) {
            throw new Error('overrideDirective is not supported in this implementation of TestComponentBuilder.');
        };
        /**
         * Overrides one or more injectables configured via `providers` metadata property of a directive
         * or
         * component.
         * Very useful when certain providers need to be mocked out.
         *
         * The providers specified via this method are appended to the existing `providers` causing the
         * duplicated providers to
         * be overridden.
         */
        TestComponentBuilder.prototype.overrideProviders = function (type, providers) {
            throw new Error('overrideProviders is not supported in this implementation of TestComponentBuilder.');
        };
        /**
         * Overrides one or more injectables configured via `providers` metadata property of a directive
         * or
         * component.
         * Very useful when certain providers need to be mocked out.
         *
         * The providers specified via this method are appended to the existing `providers` causing the
         * duplicated providers to
         * be overridden.
         */
        TestComponentBuilder.prototype.overrideViewProviders = function (type, providers) {
            throw new Error('overrideViewProviders is not supported in this implementation of TestComponentBuilder.');
        };
        TestComponentBuilder.prototype.overrideAnimations = function (componentType, animations) {
            throw new Error('overrideAnimations is not supported in this implementation of TestComponentBuilder.');
        };
        TestComponentBuilder.prototype.createFromFactory = function (ngZone, componentFactory) {
            var rootElId = "root" + _nextRootElementId++;
            var testComponentRenderer = this._injector.get(TestComponentRenderer);
            testComponentRenderer.insertRootElement(rootElId);
            var componentRef = componentFactory.create(this._injector, [], "#" + rootElId);
            var autoDetect = this._injector.get(exports.ComponentFixtureAutoDetect, false);
            return new component_fixture_1.ComponentFixture(componentRef, ngZone, autoDetect);
        };
        /**
         * Builds and returns a ComponentFixture.
         */
        TestComponentBuilder.prototype.createAsync = function (rootComponentType) {
            var _this = this;
            var noNgZone = lang_1.IS_DART || this._injector.get(exports.ComponentFixtureNoNgZone, false);
            var ngZone = noNgZone ? null : this._injector.get(index_1.NgZone, null);
            var compiler = this._injector.get(index_1.Compiler);
            var initComponent = function () {
                var promise = compiler.compileComponentAsync(rootComponentType);
                return promise.then(function (componentFactory) {
                    return _this.createFromFactory(ngZone, componentFactory);
                });
            };
            return ngZone == null ? initComponent() : ngZone.run(initComponent);
        };
        TestComponentBuilder.prototype.createFakeAsync = function (rootComponentType) {
            var result;
            var error;
            async_1.PromiseWrapper.then(this.createAsync(rootComponentType), function (_result) {
                result = _result;
            }, function (_error) {
                error = _error;
            });
            fake_async_1.tick();
            if (lang_1.isPresent(error)) {
                throw error;
            }
            return result;
        };
        TestComponentBuilder.prototype.createSync = function (rootComponentType) {
            var _this = this;
            var noNgZone = lang_1.IS_DART || this._injector.get(exports.ComponentFixtureNoNgZone, false);
            var ngZone = noNgZone ? null : this._injector.get(index_1.NgZone, null);
            var compiler = this._injector.get(index_1.Compiler);
            var initComponent = function () {
                return _this.createFromFactory(ngZone, _this._injector.get(index_1.Compiler).compileComponentSync(rootComponentType));
            };
            return ngZone == null ? initComponent() : ngZone.run(initComponent);
        };
        /** @nocollapse */
        TestComponentBuilder.decorators = [{ type: index_1.Injectable }];
        /** @nocollapse */
        TestComponentBuilder.ctorParameters = [{ type: index_1.Injector }];
        return TestComponentBuilder;
    }();
    exports.TestComponentBuilder = TestComponentBuilder;
    
});
System.registerDynamic("node_modules/@angular/core/testing/async_test_completer.js", ["../src/facade/promise"], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var promise_1 = $__require('../src/facade/promise');
    /**
     * Injectable completer that allows signaling completion of an asynchronous test. Used internally.
     */
    var AsyncTestCompleter = function () {
        function AsyncTestCompleter() {
            this._completer = new promise_1.PromiseCompleter();
        }
        AsyncTestCompleter.prototype.done = function (value) {
            this._completer.resolve(value);
        };
        AsyncTestCompleter.prototype.fail = function (error, stackTrace) {
            this._completer.reject(error, stackTrace);
        };
        Object.defineProperty(AsyncTestCompleter.prototype, "promise", {
            get: function () {
                return this._completer.promise;
            },
            enumerable: true,
            configurable: true
        });
        return AsyncTestCompleter;
    }();
    exports.AsyncTestCompleter = AsyncTestCompleter;
    
});
System.registerDynamic('node_modules/@angular/core/testing/test_injector.js', ['../index', '../src/application_ref', '../src/facade/collection', '../src/facade/exceptions', '../src/facade/lang', './async_test_completer'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var index_1 = $__require('../index');
    var application_ref_1 = $__require('../src/application_ref');
    var collection_1 = $__require('../src/facade/collection');
    var exceptions_1 = $__require('../src/facade/exceptions');
    var lang_1 = $__require('../src/facade/lang');
    var async_test_completer_1 = $__require('./async_test_completer');
    /**
     * @experimental
     */
    var TestInjector = function () {
        function TestInjector() {
            this._instantiated = false;
            this._injector = null;
            this._providers = [];
            this.platformProviders = [];
            this.applicationProviders = [];
        }
        TestInjector.prototype.reset = function () {
            this._injector = null;
            this._providers = [];
            this._instantiated = false;
        };
        TestInjector.prototype.addProviders = function (providers) {
            if (this._instantiated) {
                throw new exceptions_1.BaseException('Cannot add providers after test injector is instantiated');
            }
            this._providers = collection_1.ListWrapper.concat(this._providers, providers);
        };
        TestInjector.prototype.createInjector = function () {
            application_ref_1.lockRunMode();
            var rootInjector = index_1.ReflectiveInjector.resolveAndCreate(this.platformProviders);
            this._injector = rootInjector.resolveAndCreateChild(collection_1.ListWrapper.concat(this.applicationProviders, this._providers));
            this._instantiated = true;
            return this._injector;
        };
        TestInjector.prototype.get = function (token) {
            if (!this._instantiated) {
                this.createInjector();
            }
            return this._injector.get(token);
        };
        TestInjector.prototype.execute = function (tokens, fn) {
            var _this = this;
            if (!this._instantiated) {
                this.createInjector();
            }
            var params = tokens.map(function (t) {
                return _this._injector.get(t);
            });
            return lang_1.FunctionWrapper.apply(fn, params);
        };
        return TestInjector;
    }();
    exports.TestInjector = TestInjector;
    var _testInjector = null;
    /**
     * @experimental
     */
    function getTestInjector() {
        if (_testInjector == null) {
            _testInjector = new TestInjector();
        }
        return _testInjector;
    }
    exports.getTestInjector = getTestInjector;
    /**
     * Set the providers that the test injector should use. These should be providers
     * common to every test in the suite.
     *
     * This may only be called once, to set up the common providers for the current test
     * suite on the current platform. If you absolutely need to change the providers,
     * first use `resetBaseTestProviders`.
     *
     * Test Providers for individual platforms are available from
     * 'angular2/platform/testing/<platform_name>'.
     *
     * @experimental
     */
    function setBaseTestProviders(platformProviders, applicationProviders) {
        var testInjector = getTestInjector();
        if (testInjector.platformProviders.length > 0 || testInjector.applicationProviders.length > 0) {
            throw new exceptions_1.BaseException('Cannot set base providers because it has already been called');
        }
        testInjector.platformProviders = platformProviders;
        testInjector.applicationProviders = applicationProviders;
        var injector = testInjector.createInjector();
        var inits = injector.get(index_1.PLATFORM_INITIALIZER, null);
        if (lang_1.isPresent(inits)) {
            inits.forEach(function (init) {
                return init();
            });
        }
        testInjector.reset();
    }
    exports.setBaseTestProviders = setBaseTestProviders;
    /**
     * Reset the providers for the test injector.
     *
     * @experimental
     */
    function resetBaseTestProviders() {
        var testInjector = getTestInjector();
        testInjector.platformProviders = [];
        testInjector.applicationProviders = [];
        testInjector.reset();
    }
    exports.resetBaseTestProviders = resetBaseTestProviders;
    /**
     * Allows injecting dependencies in `beforeEach()` and `it()`.
     *
     * Example:
     *
     * ```
     * beforeEach(inject([Dependency, AClass], (dep, object) => {
     *   // some code that uses `dep` and `object`
     *   // ...
     * }));
     *
     * it('...', inject([AClass], (object) => {
     *   object.doSomething();
     *   expect(...);
     * })
     * ```
     *
     * Notes:
     * - inject is currently a function because of some Traceur limitation the syntax should
     * eventually
     *   becomes `it('...', @Inject (object: AClass, async: AsyncTestCompleter) => { ... });`
     *
     * @stable
     */
    function inject(tokens, fn) {
        var testInjector = getTestInjector();
        if (tokens.indexOf(async_test_completer_1.AsyncTestCompleter) >= 0) {
            // Return an async test method that returns a Promise if AsyncTestCompleter is one of the
            // injected tokens.
            return function () {
                var completer = testInjector.get(async_test_completer_1.AsyncTestCompleter);
                testInjector.execute(tokens, fn);
                return completer.promise;
            };
        } else {
            // Return a synchronous test method with the injected tokens.
            return function () {
                return getTestInjector().execute(tokens, fn);
            };
        }
    }
    exports.inject = inject;
    /**
     * @experimental
     */
    var InjectSetupWrapper = function () {
        function InjectSetupWrapper(_providers) {
            this._providers = _providers;
        }
        InjectSetupWrapper.prototype._addProviders = function () {
            var additionalProviders = this._providers();
            if (additionalProviders.length > 0) {
                getTestInjector().addProviders(additionalProviders);
            }
        };
        InjectSetupWrapper.prototype.inject = function (tokens, fn) {
            var _this = this;
            return function () {
                _this._addProviders();
                return inject_impl(tokens, fn)();
            };
        };
        return InjectSetupWrapper;
    }();
    exports.InjectSetupWrapper = InjectSetupWrapper;
    /**
     * @experimental
     */
    function withProviders(providers) {
        return new InjectSetupWrapper(providers);
    }
    exports.withProviders = withProviders;
    // This is to ensure inject(Async) within InjectSetupWrapper doesn't call itself
    // when transpiled to Dart.
    var inject_impl = inject;
    
});
System.registerDynamic('node_modules/@angular/core/testing/testing.js', ['./test_injector'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * Public Test Library for unit testing Angular2 Applications. Assumes that you are running
   * with Jasmine, Mocha, or a similar framework which exports a beforeEach function and
   * allows tests to be asynchronous by either returning a promise or using a 'done' parameter.
   */

  var global = this || self,
      GLOBAL = global;
  var test_injector_1 = $__require('./test_injector');
  var _global = typeof window === 'undefined' ? global : window;
  /**
   * @deprecated you no longer need to import jasmine functions from @angular/core/testing. Simply use
   * the globals.
   *
   * See http://jasmine.github.io/ for more details.
   */
  exports.expect = _global.expect;
  /**
   * @deprecated you no longer need to import jasmine functions from @angular/core/testing. Simply use
   * the globals.
   *
   * See http://jasmine.github.io/ for more details.
   */
  exports.afterEach = _global.afterEach;
  /**
   * @deprecated you no longer need to import jasmine functions from @angular/core/testing. Simply use
   * the globals.
   *
   * See http://jasmine.github.io/ for more details.
   */
  exports.describe = _global.describe;
  /**
   * @deprecated you no longer need to import jasmine functions from @angular/core/testing. Simply use
   * the globals.
   *
   * See http://jasmine.github.io/ for more details.
   */
  exports.fdescribe = _global.fdescribe;
  /**
   * @deprecated you no longer need to import jasmine functions from @angular/core/testing. Simply use
   * the globals.
   *
   * See http://jasmine.github.io/ for more details.
   */
  exports.ddescribe = _global.ddescribe;
  /**
   * @deprecated you no longer need to import jasmine functions from @angular/core/testing. Simply use
   * the globals.
   *
   * See http://jasmine.github.io/ for more details.
   */
  exports.xdescribe = _global.xdescribe;
  /**
   * @deprecated you no longer need to import jasmine functions from @angular/core/testing. Simply use
   * the globals.
   *
   * See http://jasmine.github.io/ for more details.
   */
  exports.beforeEach = _global.beforeEach;
  /**
   * @deprecated you no longer need to import jasmine functions from @angular/core/testing. Simply use
   * the globals.
   *
   * See http://jasmine.github.io/ for more details.
   */
  exports.it = _global.it;
  /**
   * @deprecated you no longer need to import jasmine functions from @angular/core/testing. Simply use
   * the globals.
   *
   * See http://jasmine.github.io/ for more details.
   */
  exports.fit = _global.fit;
  /**
   * @deprecated you no longer need to import jasmine functions from @angular/core/testing. Simply use
   * the globals.
   *
   * See http://jasmine.github.io/ for more details.
   */
  exports.iit = _global.fit;
  /**
   * @deprecated you no longer need to import jasmine functions from @angular/core/testing. Simply use
   * the globals.
   *
   * See http://jasmine.github.io/ for more details.
   */
  exports.xit = _global.xit;
  var testInjector = test_injector_1.getTestInjector();
  // Reset the test providers before each test.
  if (_global.beforeEach) {
    exports.beforeEach(function () {
      testInjector.reset();
    });
  }
  /**
   * Allows overriding default providers of the test injector,
   * which are defined in test_injector.js
   *
   * @stable
   */
  function addProviders(providers) {
    if (!providers) return;
    try {
      testInjector.addProviders(providers);
    } catch (e) {
      throw new Error('addProviders can\'t be called after the injector has been already created for this test. ' + 'This is most likely because you\'ve already used the injector to inject a beforeEach or the ' + 'current `it` function.');
    }
  }
  exports.addProviders = addProviders;
  /**
   * @deprecated Use beforeEach(() => addProviders())
   */
  function beforeEachProviders(fn) {
    exports.beforeEach(function () {
      addProviders(fn());
    });
  }
  exports.beforeEachProviders = beforeEachProviders;
  
});
System.registerDynamic('node_modules/@angular/core/testing.js', ['./testing/async', './testing/component_fixture', './testing/fake_async', './testing/test_component_builder', './testing/test_injector', './testing/testing'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
  }
  __export($__require('./testing/async'));
  __export($__require('./testing/component_fixture'));
  __export($__require('./testing/fake_async'));
  __export($__require('./testing/test_component_builder'));
  __export($__require('./testing/test_injector'));
  __export($__require('./testing/testing'));
  
});
System.registerDynamic('node_modules/@angular/http/src/backends/browser_jsonp.js', ['@angular/core', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../facade/lang');
    var _nextRequestId = 0;
    exports.JSONP_HOME = '__ng_jsonp__';
    var _jsonpConnections = null;
    function _getJsonpConnections() {
        if (_jsonpConnections === null) {
            _jsonpConnections = lang_1.global[exports.JSONP_HOME] = {};
        }
        return _jsonpConnections;
    }
    var BrowserJsonp = function () {
        function BrowserJsonp() {}
        // Construct a <script> element with the specified URL
        BrowserJsonp.prototype.build = function (url) {
            var node = document.createElement('script');
            node.src = url;
            return node;
        };
        BrowserJsonp.prototype.nextRequestID = function () {
            return "__req" + _nextRequestId++;
        };
        BrowserJsonp.prototype.requestCallback = function (id) {
            return exports.JSONP_HOME + "." + id + ".finished";
        };
        BrowserJsonp.prototype.exposeConnection = function (id, connection) {
            var connections = _getJsonpConnections();
            connections[id] = connection;
        };
        BrowserJsonp.prototype.removeConnection = function (id) {
            var connections = _getJsonpConnections();
            connections[id] = null;
        };
        // Attach the <script> element to the DOM
        BrowserJsonp.prototype.send = function (node) {
            document.body.appendChild(node);
        };
        // Remove <script> element from the DOM
        BrowserJsonp.prototype.cleanup = function (node) {
            if (node.parentNode) {
                node.parentNode.removeChild(node);
            }
        };
        /** @nocollapse */
        BrowserJsonp.decorators = [{ type: core_1.Injectable }];
        return BrowserJsonp;
    }();
    exports.BrowserJsonp = BrowserJsonp;
    
});
System.registerDynamic('node_modules/@angular/http/src/backends/jsonp_backend.js', ['@angular/core', 'rxjs/Observable', '../base_response_options', '../enums', '../facade/exceptions', '../facade/lang', '../interfaces', '../static_response', './browser_jsonp'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var Observable_1 = $__require('rxjs/Observable');
    var base_response_options_1 = $__require('../base_response_options');
    var enums_1 = $__require('../enums');
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    var interfaces_1 = $__require('../interfaces');
    var static_response_1 = $__require('../static_response');
    var browser_jsonp_1 = $__require('./browser_jsonp');
    var JSONP_ERR_NO_CALLBACK = 'JSONP injected script did not invoke callback.';
    var JSONP_ERR_WRONG_METHOD = 'JSONP requests must use GET request method.';
    /**
     * Abstract base class for an in-flight JSONP request.
     *
     * @experimental
     */
    var JSONPConnection = function () {
        function JSONPConnection() {}
        return JSONPConnection;
    }();
    exports.JSONPConnection = JSONPConnection;
    var JSONPConnection_ = function (_super) {
        __extends(JSONPConnection_, _super);
        function JSONPConnection_(req, _dom, baseResponseOptions) {
            var _this = this;
            _super.call(this);
            this._dom = _dom;
            this.baseResponseOptions = baseResponseOptions;
            this._finished = false;
            if (req.method !== enums_1.RequestMethod.Get) {
                throw exceptions_1.makeTypeError(JSONP_ERR_WRONG_METHOD);
            }
            this.request = req;
            this.response = new Observable_1.Observable(function (responseObserver) {
                _this.readyState = enums_1.ReadyState.Loading;
                var id = _this._id = _dom.nextRequestID();
                _dom.exposeConnection(id, _this);
                // Workaround Dart
                // url = url.replace(/=JSONP_CALLBACK(&|$)/, `generated method`);
                var callback = _dom.requestCallback(_this._id);
                var url = req.url;
                if (url.indexOf('=JSONP_CALLBACK&') > -1) {
                    url = lang_1.StringWrapper.replace(url, '=JSONP_CALLBACK&', "=" + callback + "&");
                } else if (url.lastIndexOf('=JSONP_CALLBACK') === url.length - '=JSONP_CALLBACK'.length) {
                    url = url.substring(0, url.length - '=JSONP_CALLBACK'.length) + ("=" + callback);
                }
                var script = _this._script = _dom.build(url);
                var onLoad = function (event) {
                    if (_this.readyState === enums_1.ReadyState.Cancelled) return;
                    _this.readyState = enums_1.ReadyState.Done;
                    _dom.cleanup(script);
                    if (!_this._finished) {
                        var responseOptions_1 = new base_response_options_1.ResponseOptions({ body: JSONP_ERR_NO_CALLBACK, type: enums_1.ResponseType.Error, url: url });
                        if (lang_1.isPresent(baseResponseOptions)) {
                            responseOptions_1 = baseResponseOptions.merge(responseOptions_1);
                        }
                        responseObserver.error(new static_response_1.Response(responseOptions_1));
                        return;
                    }
                    var responseOptions = new base_response_options_1.ResponseOptions({ body: _this._responseData, url: url });
                    if (lang_1.isPresent(_this.baseResponseOptions)) {
                        responseOptions = _this.baseResponseOptions.merge(responseOptions);
                    }
                    responseObserver.next(new static_response_1.Response(responseOptions));
                    responseObserver.complete();
                };
                var onError = function (error) {
                    if (_this.readyState === enums_1.ReadyState.Cancelled) return;
                    _this.readyState = enums_1.ReadyState.Done;
                    _dom.cleanup(script);
                    var responseOptions = new base_response_options_1.ResponseOptions({ body: error.message, type: enums_1.ResponseType.Error });
                    if (lang_1.isPresent(baseResponseOptions)) {
                        responseOptions = baseResponseOptions.merge(responseOptions);
                    }
                    responseObserver.error(new static_response_1.Response(responseOptions));
                };
                script.addEventListener('load', onLoad);
                script.addEventListener('error', onError);
                _dom.send(script);
                return function () {
                    _this.readyState = enums_1.ReadyState.Cancelled;
                    script.removeEventListener('load', onLoad);
                    script.removeEventListener('error', onError);
                    if (lang_1.isPresent(script)) {
                        _this._dom.cleanup(script);
                    }
                };
            });
        }
        JSONPConnection_.prototype.finished = function (data) {
            // Don't leak connections
            this._finished = true;
            this._dom.removeConnection(this._id);
            if (this.readyState === enums_1.ReadyState.Cancelled) return;
            this._responseData = data;
        };
        return JSONPConnection_;
    }(JSONPConnection);
    exports.JSONPConnection_ = JSONPConnection_;
    /**
     * A {@link ConnectionBackend} that uses the JSONP strategy of making requests.
     *
     * @experimental
     */
    var JSONPBackend = function (_super) {
        __extends(JSONPBackend, _super);
        function JSONPBackend() {
            _super.apply(this, arguments);
        }
        return JSONPBackend;
    }(interfaces_1.ConnectionBackend);
    exports.JSONPBackend = JSONPBackend;
    var JSONPBackend_ = function (_super) {
        __extends(JSONPBackend_, _super);
        function JSONPBackend_(_browserJSONP, _baseResponseOptions) {
            _super.call(this);
            this._browserJSONP = _browserJSONP;
            this._baseResponseOptions = _baseResponseOptions;
        }
        JSONPBackend_.prototype.createConnection = function (request) {
            return new JSONPConnection_(request, this._browserJSONP, this._baseResponseOptions);
        };
        /** @nocollapse */
        JSONPBackend_.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        JSONPBackend_.ctorParameters = [{ type: browser_jsonp_1.BrowserJsonp }, { type: base_response_options_1.ResponseOptions }];
        return JSONPBackend_;
    }(JSONPBackend);
    exports.JSONPBackend_ = JSONPBackend_;
    
});
System.registerDynamic("node_modules/@angular/platform-browser/src/browser/title.js", ["../dom/dom_adapter"], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var dom_adapter_1 = $__require('../dom/dom_adapter');
  /**
   * A service that can be used to get and set the title of a current HTML document.
   *
   * Since an Angular 2 application can't be bootstrapped on the entire HTML document (`<html>` tag)
   * it is not possible to bind to the `text` property of the `HTMLTitleElement` elements
   * (representing the `<title>` tag). Instead, this service can be used to set and get the current
   * title value.
   *
   * @experimental
   */
  var Title = function () {
    function Title() {}
    /**
     * Get the title of the current HTML document.
     * @returns {string}
     */
    Title.prototype.getTitle = function () {
      return dom_adapter_1.getDOM().getTitle();
    };
    /**
     * Set the title of the current HTML document.
     * @param newTitle
     */
    Title.prototype.setTitle = function (newTitle) {
      dom_adapter_1.getDOM().setTitle(newTitle);
    };
    return Title;
  }();
  exports.Title = Title;
  
});
System.registerDynamic('node_modules/@angular/platform-browser/src/facade/browser.js', [], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * JS version of browser APIs. This library can only run in the browser.
   */

  var global = this || self,
      GLOBAL = global;
  var win = typeof window !== 'undefined' && window || {};
  exports.window = win;
  exports.document = win.document;
  exports.location = win.location;
  exports.gc = win['gc'] ? function () {
    return win['gc']();
  } : function () {
    return null;
  };
  exports.performance = win['performance'] ? win['performance'] : null;
  exports.Event = win['Event'];
  exports.MouseEvent = win['MouseEvent'];
  exports.KeyboardEvent = win['KeyboardEvent'];
  exports.EventTarget = win['EventTarget'];
  exports.History = win['History'];
  exports.Location = win['Location'];
  exports.EventListener = win['EventListener'];
  
});
System.registerDynamic('node_modules/@angular/platform-browser/src/browser/tools/common_tools.js', ['@angular/core', '../../dom/dom_adapter', '../../facade/browser', '../../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var dom_adapter_1 = $__require('../../dom/dom_adapter');
    var browser_1 = $__require('../../facade/browser');
    var lang_1 = $__require('../../facade/lang');
    var ChangeDetectionPerfRecord = function () {
        function ChangeDetectionPerfRecord(msPerTick, numTicks) {
            this.msPerTick = msPerTick;
            this.numTicks = numTicks;
        }
        return ChangeDetectionPerfRecord;
    }();
    exports.ChangeDetectionPerfRecord = ChangeDetectionPerfRecord;
    /**
     * Entry point for all Angular debug tools. This object corresponds to the `ng`
     * global variable accessible in the dev console.
     */
    var AngularTools = function () {
        function AngularTools(ref) {
            this.profiler = new AngularProfiler(ref);
        }
        return AngularTools;
    }();
    exports.AngularTools = AngularTools;
    /**
     * Entry point for all Angular profiling-related debug tools. This object
     * corresponds to the `ng.profiler` in the dev console.
     */
    var AngularProfiler = function () {
        function AngularProfiler(ref) {
            this.appRef = ref.injector.get(core_1.ApplicationRef);
        }
        /**
         * Exercises change detection in a loop and then prints the average amount of
         * time in milliseconds how long a single round of change detection takes for
         * the current state of the UI. It runs a minimum of 5 rounds for a minimum
         * of 500 milliseconds.
         *
         * Optionally, a user may pass a `config` parameter containing a map of
         * options. Supported options are:
         *
         * `record` (boolean) - causes the profiler to record a CPU profile while
         * it exercises the change detector. Example:
         *
         * ```
         * ng.profiler.timeChangeDetection({record: true})
         * ```
         */
        AngularProfiler.prototype.timeChangeDetection = function (config) {
            var record = lang_1.isPresent(config) && config['record'];
            var profileName = 'Change Detection';
            // Profiler is not available in Android browsers, nor in IE 9 without dev tools opened
            var isProfilerAvailable = lang_1.isPresent(browser_1.window.console.profile);
            if (record && isProfilerAvailable) {
                browser_1.window.console.profile(profileName);
            }
            var start = dom_adapter_1.getDOM().performanceNow();
            var numTicks = 0;
            while (numTicks < 5 || dom_adapter_1.getDOM().performanceNow() - start < 500) {
                this.appRef.tick();
                numTicks++;
            }
            var end = dom_adapter_1.getDOM().performanceNow();
            if (record && isProfilerAvailable) {
                // need to cast to <any> because type checker thinks there's no argument
                // while in fact there is:
                //
                // https://developer.mozilla.org/en-US/docs/Web/API/Console/profileEnd
                browser_1.window.console.profileEnd(profileName);
            }
            var msPerTick = (end - start) / numTicks;
            browser_1.window.console.log("ran " + numTicks + " change detection cycles");
            browser_1.window.console.log(lang_1.NumberWrapper.toFixed(msPerTick, 2) + " ms per check");
            return new ChangeDetectionPerfRecord(msPerTick, numTicks);
        };
        return AngularProfiler;
    }();
    exports.AngularProfiler = AngularProfiler;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/browser/tools/tools.js', ['../../facade/lang', './common_tools'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var lang_1 = $__require('../../facade/lang');
  var common_tools_1 = $__require('./common_tools');
  var context = lang_1.global;
  /**
   * Enabled Angular 2 debug tools that are accessible via your browser's
   * developer console.
   *
   * Usage:
   *
   * 1. Open developer console (e.g. in Chrome Ctrl + Shift + j)
   * 1. Type `ng.` (usually the console will show auto-complete suggestion)
   * 1. Try the change detection profiler `ng.profiler.timeChangeDetection()`
   *    then hit Enter.
   *
   * @experimental All debugging apis are currently experimental.
   */
  function enableDebugTools(ref) {
    context.ng = new common_tools_1.AngularTools(ref);
    return ref;
  }
  exports.enableDebugTools = enableDebugTools;
  /**
   * Disables Angular 2 tools.
   *
   * @experimental All debugging apis are currently experimental.
   */
  function disableDebugTools() {
    delete context.ng;
  }
  exports.disableDebugTools = disableDebugTools;
  
});
System.registerDynamic('node_modules/@angular/platform-browser/src/dom/debug/by.js', ['../../dom/dom_adapter', '../../facade/lang'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var dom_adapter_1 = $__require('../../dom/dom_adapter');
  var lang_1 = $__require('../../facade/lang');
  /**
   * Predicates for use with {@link DebugElement}'s query functions.
   *
   * @experimental All debugging apis are currently experimental.
   */
  var By = function () {
    function By() {}
    /**
     * Match all elements.
     *
     * ## Example
     *
     * {@example platform/dom/debug/ts/by/by.ts region='by_all'}
     */
    By.all = function () {
      return function (debugElement) {
        return true;
      };
    };
    /**
     * Match elements by the given CSS selector.
     *
     * ## Example
     *
     * {@example platform/dom/debug/ts/by/by.ts region='by_css'}
     */
    By.css = function (selector) {
      return function (debugElement) {
        return lang_1.isPresent(debugElement.nativeElement) ? dom_adapter_1.getDOM().elementMatches(debugElement.nativeElement, selector) : false;
      };
    };
    /**
     * Match elements that have the given directive present.
     *
     * ## Example
     *
     * {@example platform/dom/debug/ts/by/by.ts region='by_directive'}
     */
    By.directive = function (type) {
      return function (debugElement) {
        return debugElement.providerTokens.indexOf(type) !== -1;
      };
    };
    return By;
  }();
  exports.By = By;
  
});
System.registerDynamic('node_modules/@angular/platform-browser/src/web_workers/worker/platform_location.js', ['@angular/common', '@angular/core', '../../facade/async', '../../facade/collection', '../../facade/exceptions', '../../facade/lang', '../shared/client_message_broker', '../shared/message_bus', '../shared/messaging_api', '../shared/serialized_types', '../shared/serializer', './event_deserializer'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var common_1 = $__require('@angular/common');
    var core_1 = $__require('@angular/core');
    var async_1 = $__require('../../facade/async');
    var collection_1 = $__require('../../facade/collection');
    var exceptions_1 = $__require('../../facade/exceptions');
    var lang_1 = $__require('../../facade/lang');
    var client_message_broker_1 = $__require('../shared/client_message_broker');
    var message_bus_1 = $__require('../shared/message_bus');
    var messaging_api_1 = $__require('../shared/messaging_api');
    var serialized_types_1 = $__require('../shared/serialized_types');
    var serializer_1 = $__require('../shared/serializer');
    var event_deserializer_1 = $__require('./event_deserializer');
    var WebWorkerPlatformLocation = function (_super) {
        __extends(WebWorkerPlatformLocation, _super);
        function WebWorkerPlatformLocation(brokerFactory, bus, _serializer) {
            var _this = this;
            _super.call(this);
            this._serializer = _serializer;
            this._popStateListeners = [];
            this._hashChangeListeners = [];
            this._location = null;
            this._broker = brokerFactory.createMessageBroker(messaging_api_1.ROUTER_CHANNEL);
            this._channelSource = bus.from(messaging_api_1.ROUTER_CHANNEL);
            async_1.ObservableWrapper.subscribe(this._channelSource, function (msg) {
                var listeners = null;
                if (collection_1.StringMapWrapper.contains(msg, 'event')) {
                    var type = msg['event']['type'];
                    if (lang_1.StringWrapper.equals(type, 'popstate')) {
                        listeners = _this._popStateListeners;
                    } else if (lang_1.StringWrapper.equals(type, 'hashchange')) {
                        listeners = _this._hashChangeListeners;
                    }
                    if (listeners !== null) {
                        var e_1 = event_deserializer_1.deserializeGenericEvent(msg['event']);
                        // There was a popState or hashChange event, so the location object thas been updated
                        _this._location = _this._serializer.deserialize(msg['location'], serialized_types_1.LocationType);
                        listeners.forEach(function (fn) {
                            return fn(e_1);
                        });
                    }
                }
            });
        }
        /** @internal **/
        WebWorkerPlatformLocation.prototype.init = function () {
            var _this = this;
            var args = new client_message_broker_1.UiArguments('getLocation');
            var locationPromise = this._broker.runOnService(args, serialized_types_1.LocationType);
            return async_1.PromiseWrapper.then(locationPromise, function (val) {
                _this._location = val;
                return true;
            }, function (err) {
                throw new exceptions_1.BaseException(err);
            });
        };
        WebWorkerPlatformLocation.prototype.getBaseHrefFromDOM = function () {
            throw new exceptions_1.BaseException('Attempt to get base href from DOM from WebWorker. You must either provide a value for the APP_BASE_HREF token through DI or use the hash location strategy.');
        };
        WebWorkerPlatformLocation.prototype.onPopState = function (fn) {
            this._popStateListeners.push(fn);
        };
        WebWorkerPlatformLocation.prototype.onHashChange = function (fn) {
            this._hashChangeListeners.push(fn);
        };
        Object.defineProperty(WebWorkerPlatformLocation.prototype, "pathname", {
            get: function () {
                if (this._location === null) {
                    return null;
                }
                return this._location.pathname;
            },
            set: function (newPath) {
                if (this._location === null) {
                    throw new exceptions_1.BaseException('Attempt to set pathname before value is obtained from UI');
                }
                this._location.pathname = newPath;
                var fnArgs = [new client_message_broker_1.FnArg(newPath, serializer_1.PRIMITIVE)];
                var args = new client_message_broker_1.UiArguments('setPathname', fnArgs);
                this._broker.runOnService(args, null);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WebWorkerPlatformLocation.prototype, "search", {
            get: function () {
                if (this._location === null) {
                    return null;
                }
                return this._location.search;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WebWorkerPlatformLocation.prototype, "hash", {
            get: function () {
                if (this._location === null) {
                    return null;
                }
                return this._location.hash;
            },
            enumerable: true,
            configurable: true
        });
        WebWorkerPlatformLocation.prototype.pushState = function (state, title, url) {
            var fnArgs = [new client_message_broker_1.FnArg(state, serializer_1.PRIMITIVE), new client_message_broker_1.FnArg(title, serializer_1.PRIMITIVE), new client_message_broker_1.FnArg(url, serializer_1.PRIMITIVE)];
            var args = new client_message_broker_1.UiArguments('pushState', fnArgs);
            this._broker.runOnService(args, null);
        };
        WebWorkerPlatformLocation.prototype.replaceState = function (state, title, url) {
            var fnArgs = [new client_message_broker_1.FnArg(state, serializer_1.PRIMITIVE), new client_message_broker_1.FnArg(title, serializer_1.PRIMITIVE), new client_message_broker_1.FnArg(url, serializer_1.PRIMITIVE)];
            var args = new client_message_broker_1.UiArguments('replaceState', fnArgs);
            this._broker.runOnService(args, null);
        };
        WebWorkerPlatformLocation.prototype.forward = function () {
            var args = new client_message_broker_1.UiArguments('forward');
            this._broker.runOnService(args, null);
        };
        WebWorkerPlatformLocation.prototype.back = function () {
            var args = new client_message_broker_1.UiArguments('back');
            this._broker.runOnService(args, null);
        };
        /** @nocollapse */
        WebWorkerPlatformLocation.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        WebWorkerPlatformLocation.ctorParameters = [{ type: client_message_broker_1.ClientMessageBrokerFactory }, { type: message_bus_1.MessageBus }, { type: serializer_1.Serializer }];
        return WebWorkerPlatformLocation;
    }(common_1.PlatformLocation);
    exports.WebWorkerPlatformLocation = WebWorkerPlatformLocation;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/web_workers/worker/location_providers.js', ['@angular/common', '@angular/core', './platform_location'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var common_1 = $__require('@angular/common');
    var core_1 = $__require('@angular/core');
    var platform_location_1 = $__require('./platform_location');
    /**
     * Those providers should be added when the router is used in a worker context in addition to the
     * {@link ROUTER_PROVIDERS} and after them.
     * @experimental
     */
    exports.WORKER_APP_LOCATION_PROVIDERS = [{ provide: common_1.PlatformLocation, useClass: platform_location_1.WebWorkerPlatformLocation }, {
        provide: core_1.APP_INITIALIZER,
        useFactory: appInitFnFactory,
        multi: true,
        deps: [common_1.PlatformLocation, core_1.NgZone]
    }];
    function appInitFnFactory(platformLocation, zone) {
        return function () {
            return zone.runGuarded(function () {
                return platformLocation.init();
            });
        };
    }
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/web_workers/ui/platform_location.js', ['@angular/core', '../../browser/location/browser_platform_location', '../../facade/async', '../../facade/lang', '../shared/message_bus', '../shared/messaging_api', '../shared/serialized_types', '../shared/serializer', '../shared/service_message_broker'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var browser_platform_location_1 = $__require('../../browser/location/browser_platform_location');
    var async_1 = $__require('../../facade/async');
    var lang_1 = $__require('../../facade/lang');
    var message_bus_1 = $__require('../shared/message_bus');
    var messaging_api_1 = $__require('../shared/messaging_api');
    var serialized_types_1 = $__require('../shared/serialized_types');
    var serializer_1 = $__require('../shared/serializer');
    var service_message_broker_1 = $__require('../shared/service_message_broker');
    var MessageBasedPlatformLocation = function () {
        function MessageBasedPlatformLocation(_brokerFactory, _platformLocation, bus, _serializer) {
            this._brokerFactory = _brokerFactory;
            this._platformLocation = _platformLocation;
            this._serializer = _serializer;
            this._platformLocation.onPopState(lang_1.FunctionWrapper.bind(this._sendUrlChangeEvent, this));
            this._platformLocation.onHashChange(lang_1.FunctionWrapper.bind(this._sendUrlChangeEvent, this));
            this._broker = this._brokerFactory.createMessageBroker(messaging_api_1.ROUTER_CHANNEL);
            this._channelSink = bus.to(messaging_api_1.ROUTER_CHANNEL);
        }
        MessageBasedPlatformLocation.prototype.start = function () {
            this._broker.registerMethod('getLocation', null, lang_1.FunctionWrapper.bind(this._getLocation, this), serialized_types_1.LocationType);
            this._broker.registerMethod('setPathname', [serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._setPathname, this));
            this._broker.registerMethod('pushState', [serializer_1.PRIMITIVE, serializer_1.PRIMITIVE, serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._platformLocation.pushState, this._platformLocation));
            this._broker.registerMethod('replaceState', [serializer_1.PRIMITIVE, serializer_1.PRIMITIVE, serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._platformLocation.replaceState, this._platformLocation));
            this._broker.registerMethod('forward', null, lang_1.FunctionWrapper.bind(this._platformLocation.forward, this._platformLocation));
            this._broker.registerMethod('back', null, lang_1.FunctionWrapper.bind(this._platformLocation.back, this._platformLocation));
        };
        MessageBasedPlatformLocation.prototype._getLocation = function () {
            return async_1.PromiseWrapper.resolve(this._platformLocation.location);
        };
        MessageBasedPlatformLocation.prototype._sendUrlChangeEvent = function (e) {
            var loc = this._serializer.serialize(this._platformLocation.location, serialized_types_1.LocationType);
            var serializedEvent = { 'type': e.type };
            async_1.ObservableWrapper.callEmit(this._channelSink, { 'event': serializedEvent, 'location': loc });
        };
        MessageBasedPlatformLocation.prototype._setPathname = function (pathname) {
            this._platformLocation.pathname = pathname;
        };
        /** @nocollapse */
        MessageBasedPlatformLocation.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        MessageBasedPlatformLocation.ctorParameters = [{ type: service_message_broker_1.ServiceMessageBrokerFactory }, { type: browser_platform_location_1.BrowserPlatformLocation }, { type: message_bus_1.MessageBus }, { type: serializer_1.Serializer }];
        return MessageBasedPlatformLocation;
    }();
    exports.MessageBasedPlatformLocation = MessageBasedPlatformLocation;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/web_workers/ui/location_providers.js', ['@angular/core', '../../browser/location/browser_platform_location', './platform_location'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var browser_platform_location_1 = $__require('../../browser/location/browser_platform_location');
    var platform_location_1 = $__require('./platform_location');
    /**
     * A list of {@link Provider}s. To use the router in a Worker enabled application you must
     * include these providers when setting up the render thread.
     * @experimental
     */
    exports.WORKER_UI_LOCATION_PROVIDERS = [platform_location_1.MessageBasedPlatformLocation, browser_platform_location_1.BrowserPlatformLocation, { provide: core_1.APP_INITIALIZER, useFactory: initUiLocation, multi: true, deps: [core_1.Injector] }];
    function initUiLocation(injector) {
        return function () {
            var zone = injector.get(core_1.NgZone);
            zone.runGuarded(function () {
                return injector.get(platform_location_1.MessageBasedPlatformLocation).start();
            });
        };
    }
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/web_workers/ui/event_serializer.js', ['../../facade/collection', '../../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var collection_1 = $__require('../../facade/collection');
    var lang_1 = $__require('../../facade/lang');
    var MOUSE_EVENT_PROPERTIES = ['altKey', 'button', 'clientX', 'clientY', 'metaKey', 'movementX', 'movementY', 'offsetX', 'offsetY', 'region', 'screenX', 'screenY', 'shiftKey'];
    var KEYBOARD_EVENT_PROPERTIES = ['altkey', 'charCode', 'code', 'ctrlKey', 'isComposing', 'key', 'keyCode', 'location', 'metaKey', 'repeat', 'shiftKey', 'which'];
    var TRANSITION_EVENT_PROPERTIES = ['propertyName', 'elapsedTime', 'pseudoElement'];
    var EVENT_PROPERTIES = ['type', 'bubbles', 'cancelable'];
    var NODES_WITH_VALUE = new collection_1.Set(['input', 'select', 'option', 'button', 'li', 'meter', 'progress', 'param', 'textarea']);
    function serializeGenericEvent(e) {
        return serializeEvent(e, EVENT_PROPERTIES);
    }
    exports.serializeGenericEvent = serializeGenericEvent;
    // TODO(jteplitz602): Allow users to specify the properties they need rather than always
    // adding value and files #3374
    function serializeEventWithTarget(e) {
        var serializedEvent = serializeEvent(e, EVENT_PROPERTIES);
        return addTarget(e, serializedEvent);
    }
    exports.serializeEventWithTarget = serializeEventWithTarget;
    function serializeMouseEvent(e) {
        return serializeEvent(e, MOUSE_EVENT_PROPERTIES);
    }
    exports.serializeMouseEvent = serializeMouseEvent;
    function serializeKeyboardEvent(e) {
        var serializedEvent = serializeEvent(e, KEYBOARD_EVENT_PROPERTIES);
        return addTarget(e, serializedEvent);
    }
    exports.serializeKeyboardEvent = serializeKeyboardEvent;
    function serializeTransitionEvent(e) {
        var serializedEvent = serializeEvent(e, TRANSITION_EVENT_PROPERTIES);
        return addTarget(e, serializedEvent);
    }
    exports.serializeTransitionEvent = serializeTransitionEvent;
    // TODO(jteplitz602): #3374. See above.
    function addTarget(e, serializedEvent) {
        if (NODES_WITH_VALUE.has(e.target.tagName.toLowerCase())) {
            var target = e.target;
            serializedEvent['target'] = { 'value': target.value };
            if (lang_1.isPresent(target.files)) {
                serializedEvent['target']['files'] = target.files;
            }
        }
        return serializedEvent;
    }
    function serializeEvent(e, properties) {
        var serialized = {};
        for (var i = 0; i < properties.length; i++) {
            var prop = properties[i];
            serialized[prop] = e[prop];
        }
        return serialized;
    }
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/web_workers/ui/event_dispatcher.js', ['../../facade/async', '../../facade/exceptions', '../shared/serializer', './event_serializer'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var async_1 = $__require('../../facade/async');
    var exceptions_1 = $__require('../../facade/exceptions');
    var serializer_1 = $__require('../shared/serializer');
    var event_serializer_1 = $__require('./event_serializer');
    var EventDispatcher = function () {
        function EventDispatcher(_sink, _serializer) {
            this._sink = _sink;
            this._serializer = _serializer;
        }
        EventDispatcher.prototype.dispatchRenderEvent = function (element, eventTarget, eventName, event) {
            var serializedEvent;
            // TODO (jteplitz602): support custom events #3350
            switch (event.type) {
                case 'click':
                case 'mouseup':
                case 'mousedown':
                case 'dblclick':
                case 'contextmenu':
                case 'mouseenter':
                case 'mouseleave':
                case 'mousemove':
                case 'mouseout':
                case 'mouseover':
                case 'show':
                    serializedEvent = event_serializer_1.serializeMouseEvent(event);
                    break;
                case 'keydown':
                case 'keypress':
                case 'keyup':
                    serializedEvent = event_serializer_1.serializeKeyboardEvent(event);
                    break;
                case 'input':
                case 'change':
                case 'blur':
                    serializedEvent = event_serializer_1.serializeEventWithTarget(event);
                    break;
                case 'abort':
                case 'afterprint':
                case 'beforeprint':
                case 'cached':
                case 'canplay':
                case 'canplaythrough':
                case 'chargingchange':
                case 'chargingtimechange':
                case 'close':
                case 'dischargingtimechange':
                case 'DOMContentLoaded':
                case 'downloading':
                case 'durationchange':
                case 'emptied':
                case 'ended':
                case 'error':
                case 'fullscreenchange':
                case 'fullscreenerror':
                case 'invalid':
                case 'languagechange':
                case 'levelfchange':
                case 'loadeddata':
                case 'loadedmetadata':
                case 'obsolete':
                case 'offline':
                case 'online':
                case 'open':
                case 'orientatoinchange':
                case 'pause':
                case 'pointerlockchange':
                case 'pointerlockerror':
                case 'play':
                case 'playing':
                case 'ratechange':
                case 'readystatechange':
                case 'reset':
                case 'scroll':
                case 'seeked':
                case 'seeking':
                case 'stalled':
                case 'submit':
                case 'success':
                case 'suspend':
                case 'timeupdate':
                case 'updateready':
                case 'visibilitychange':
                case 'volumechange':
                case 'waiting':
                    serializedEvent = event_serializer_1.serializeGenericEvent(event);
                    break;
                case 'transitionend':
                    serializedEvent = event_serializer_1.serializeTransitionEvent(event);
                    break;
                default:
                    throw new exceptions_1.BaseException(eventName + ' not supported on WebWorkers');
            }
            async_1.ObservableWrapper.callEmit(this._sink, {
                'element': this._serializer.serialize(element, serializer_1.RenderStoreObject),
                'eventName': eventName,
                'eventTarget': eventTarget,
                'event': serializedEvent
            });
            // TODO(kegluneq): Eventually, we want the user to indicate from the UI side whether the event
            // should be canceled, but for now just call `preventDefault` on the original DOM event.
            return false;
        };
        return EventDispatcher;
    }();
    exports.EventDispatcher = EventDispatcher;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/web_workers/ui/renderer.js', ['@angular/core', '../../facade/lang', '../shared/message_bus', '../shared/messaging_api', '../shared/render_store', '../shared/serializer', '../shared/service_message_broker', '../ui/event_dispatcher'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../../facade/lang');
    var message_bus_1 = $__require('../shared/message_bus');
    var messaging_api_1 = $__require('../shared/messaging_api');
    var render_store_1 = $__require('../shared/render_store');
    var serializer_1 = $__require('../shared/serializer');
    var service_message_broker_1 = $__require('../shared/service_message_broker');
    var event_dispatcher_1 = $__require('../ui/event_dispatcher');
    var MessageBasedRenderer = function () {
        function MessageBasedRenderer(_brokerFactory, _bus, _serializer, _renderStore, _rootRenderer) {
            this._brokerFactory = _brokerFactory;
            this._bus = _bus;
            this._serializer = _serializer;
            this._renderStore = _renderStore;
            this._rootRenderer = _rootRenderer;
        }
        MessageBasedRenderer.prototype.start = function () {
            var broker = this._brokerFactory.createMessageBroker(messaging_api_1.RENDERER_CHANNEL);
            this._bus.initChannel(messaging_api_1.EVENT_CHANNEL);
            this._eventDispatcher = new event_dispatcher_1.EventDispatcher(this._bus.to(messaging_api_1.EVENT_CHANNEL), this._serializer);
            broker.registerMethod('renderComponent', [core_1.RenderComponentType, serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._renderComponent, this));
            broker.registerMethod('selectRootElement', [serializer_1.RenderStoreObject, serializer_1.PRIMITIVE, serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._selectRootElement, this));
            broker.registerMethod('createElement', [serializer_1.RenderStoreObject, serializer_1.RenderStoreObject, serializer_1.PRIMITIVE, serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._createElement, this));
            broker.registerMethod('createViewRoot', [serializer_1.RenderStoreObject, serializer_1.RenderStoreObject, serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._createViewRoot, this));
            broker.registerMethod('createTemplateAnchor', [serializer_1.RenderStoreObject, serializer_1.RenderStoreObject, serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._createTemplateAnchor, this));
            broker.registerMethod('createText', [serializer_1.RenderStoreObject, serializer_1.RenderStoreObject, serializer_1.PRIMITIVE, serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._createText, this));
            broker.registerMethod('projectNodes', [serializer_1.RenderStoreObject, serializer_1.RenderStoreObject, serializer_1.RenderStoreObject], lang_1.FunctionWrapper.bind(this._projectNodes, this));
            broker.registerMethod('attachViewAfter', [serializer_1.RenderStoreObject, serializer_1.RenderStoreObject, serializer_1.RenderStoreObject], lang_1.FunctionWrapper.bind(this._attachViewAfter, this));
            broker.registerMethod('detachView', [serializer_1.RenderStoreObject, serializer_1.RenderStoreObject], lang_1.FunctionWrapper.bind(this._detachView, this));
            broker.registerMethod('destroyView', [serializer_1.RenderStoreObject, serializer_1.RenderStoreObject, serializer_1.RenderStoreObject], lang_1.FunctionWrapper.bind(this._destroyView, this));
            broker.registerMethod('setElementProperty', [serializer_1.RenderStoreObject, serializer_1.RenderStoreObject, serializer_1.PRIMITIVE, serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._setElementProperty, this));
            broker.registerMethod('setElementAttribute', [serializer_1.RenderStoreObject, serializer_1.RenderStoreObject, serializer_1.PRIMITIVE, serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._setElementAttribute, this));
            broker.registerMethod('setBindingDebugInfo', [serializer_1.RenderStoreObject, serializer_1.RenderStoreObject, serializer_1.PRIMITIVE, serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._setBindingDebugInfo, this));
            broker.registerMethod('setElementClass', [serializer_1.RenderStoreObject, serializer_1.RenderStoreObject, serializer_1.PRIMITIVE, serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._setElementClass, this));
            broker.registerMethod('setElementStyle', [serializer_1.RenderStoreObject, serializer_1.RenderStoreObject, serializer_1.PRIMITIVE, serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._setElementStyle, this));
            broker.registerMethod('invokeElementMethod', [serializer_1.RenderStoreObject, serializer_1.RenderStoreObject, serializer_1.PRIMITIVE, serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._invokeElementMethod, this));
            broker.registerMethod('setText', [serializer_1.RenderStoreObject, serializer_1.RenderStoreObject, serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._setText, this));
            broker.registerMethod('listen', [serializer_1.RenderStoreObject, serializer_1.RenderStoreObject, serializer_1.PRIMITIVE, serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._listen, this));
            broker.registerMethod('listenGlobal', [serializer_1.RenderStoreObject, serializer_1.PRIMITIVE, serializer_1.PRIMITIVE, serializer_1.PRIMITIVE], lang_1.FunctionWrapper.bind(this._listenGlobal, this));
            broker.registerMethod('listenDone', [serializer_1.RenderStoreObject, serializer_1.RenderStoreObject], lang_1.FunctionWrapper.bind(this._listenDone, this));
        };
        MessageBasedRenderer.prototype._renderComponent = function (renderComponentType, rendererId) {
            var renderer = this._rootRenderer.renderComponent(renderComponentType);
            this._renderStore.store(renderer, rendererId);
        };
        MessageBasedRenderer.prototype._selectRootElement = function (renderer, selector, elId) {
            this._renderStore.store(renderer.selectRootElement(selector, null), elId);
        };
        MessageBasedRenderer.prototype._createElement = function (renderer, parentElement, name, elId) {
            this._renderStore.store(renderer.createElement(parentElement, name, null), elId);
        };
        MessageBasedRenderer.prototype._createViewRoot = function (renderer, hostElement, elId) {
            var viewRoot = renderer.createViewRoot(hostElement);
            if (this._renderStore.serialize(hostElement) !== elId) {
                this._renderStore.store(viewRoot, elId);
            }
        };
        MessageBasedRenderer.prototype._createTemplateAnchor = function (renderer, parentElement, elId) {
            this._renderStore.store(renderer.createTemplateAnchor(parentElement, null), elId);
        };
        MessageBasedRenderer.prototype._createText = function (renderer, parentElement, value, elId) {
            this._renderStore.store(renderer.createText(parentElement, value, null), elId);
        };
        MessageBasedRenderer.prototype._projectNodes = function (renderer, parentElement, nodes) {
            renderer.projectNodes(parentElement, nodes);
        };
        MessageBasedRenderer.prototype._attachViewAfter = function (renderer, node, viewRootNodes) {
            renderer.attachViewAfter(node, viewRootNodes);
        };
        MessageBasedRenderer.prototype._detachView = function (renderer, viewRootNodes) {
            renderer.detachView(viewRootNodes);
        };
        MessageBasedRenderer.prototype._destroyView = function (renderer, hostElement, viewAllNodes) {
            renderer.destroyView(hostElement, viewAllNodes);
            for (var i = 0; i < viewAllNodes.length; i++) {
                this._renderStore.remove(viewAllNodes[i]);
            }
        };
        MessageBasedRenderer.prototype._setElementProperty = function (renderer, renderElement, propertyName, propertyValue) {
            renderer.setElementProperty(renderElement, propertyName, propertyValue);
        };
        MessageBasedRenderer.prototype._setElementAttribute = function (renderer, renderElement, attributeName, attributeValue) {
            renderer.setElementAttribute(renderElement, attributeName, attributeValue);
        };
        MessageBasedRenderer.prototype._setBindingDebugInfo = function (renderer, renderElement, propertyName, propertyValue) {
            renderer.setBindingDebugInfo(renderElement, propertyName, propertyValue);
        };
        MessageBasedRenderer.prototype._setElementClass = function (renderer, renderElement, className, isAdd) {
            renderer.setElementClass(renderElement, className, isAdd);
        };
        MessageBasedRenderer.prototype._setElementStyle = function (renderer, renderElement, styleName, styleValue) {
            renderer.setElementStyle(renderElement, styleName, styleValue);
        };
        MessageBasedRenderer.prototype._invokeElementMethod = function (renderer, renderElement, methodName, args) {
            renderer.invokeElementMethod(renderElement, methodName, args);
        };
        MessageBasedRenderer.prototype._setText = function (renderer, renderNode, text) {
            renderer.setText(renderNode, text);
        };
        MessageBasedRenderer.prototype._listen = function (renderer, renderElement, eventName, unlistenId) {
            var _this = this;
            var unregisterCallback = renderer.listen(renderElement, eventName, function (event /** TODO #9100 */) {
                return _this._eventDispatcher.dispatchRenderEvent(renderElement, null, eventName, event);
            });
            this._renderStore.store(unregisterCallback, unlistenId);
        };
        MessageBasedRenderer.prototype._listenGlobal = function (renderer, eventTarget, eventName, unlistenId) {
            var _this = this;
            var unregisterCallback = renderer.listenGlobal(eventTarget, eventName, function (event /** TODO #9100 */) {
                return _this._eventDispatcher.dispatchRenderEvent(null, eventTarget, eventName, event);
            });
            this._renderStore.store(unregisterCallback, unlistenId);
        };
        MessageBasedRenderer.prototype._listenDone = function (renderer, unlistenCallback) {
            unlistenCallback();
        };
        /** @nocollapse */
        MessageBasedRenderer.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        MessageBasedRenderer.ctorParameters = [{ type: service_message_broker_1.ServiceMessageBrokerFactory }, { type: message_bus_1.MessageBus }, { type: serializer_1.Serializer }, { type: render_store_1.RenderStore }, { type: core_1.RootRenderer }];
        return MessageBasedRenderer;
    }();
    exports.MessageBasedRenderer = MessageBasedRenderer;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/worker_render.js', ['@angular/core', '../core_private', './browser', './browser/browser_adapter', './browser/testability', './dom/dom_adapter', './dom/dom_renderer', './dom/dom_tokens', './dom/events/dom_events', './dom/events/event_manager', './dom/events/hammer_gestures', './dom/events/key_events', './dom/shared_styles_host', './facade/exceptions', './facade/lang', './web_workers/shared/api', './web_workers/shared/client_message_broker', './web_workers/shared/message_bus', './web_workers/shared/post_message_bus', './web_workers/shared/render_store', './web_workers/shared/serializer', './web_workers/shared/service_message_broker', './web_workers/ui/renderer'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var core_private_1 = $__require('../core_private');
    var browser_1 = $__require('./browser');
    var browser_adapter_1 = $__require('./browser/browser_adapter');
    var testability_1 = $__require('./browser/testability');
    var dom_adapter_1 = $__require('./dom/dom_adapter');
    var dom_renderer_1 = $__require('./dom/dom_renderer');
    var dom_tokens_1 = $__require('./dom/dom_tokens');
    var dom_events_1 = $__require('./dom/events/dom_events');
    var event_manager_1 = $__require('./dom/events/event_manager');
    var hammer_gestures_1 = $__require('./dom/events/hammer_gestures');
    var key_events_1 = $__require('./dom/events/key_events');
    var shared_styles_host_1 = $__require('./dom/shared_styles_host');
    var exceptions_1 = $__require('./facade/exceptions');
    var lang_1 = $__require('./facade/lang');
    var api_1 = $__require('./web_workers/shared/api');
    var client_message_broker_1 = $__require('./web_workers/shared/client_message_broker');
    var message_bus_1 = $__require('./web_workers/shared/message_bus');
    var post_message_bus_1 = $__require('./web_workers/shared/post_message_bus');
    var render_store_1 = $__require('./web_workers/shared/render_store');
    var serializer_1 = $__require('./web_workers/shared/serializer');
    var service_message_broker_1 = $__require('./web_workers/shared/service_message_broker');
    var renderer_1 = $__require('./web_workers/ui/renderer');
    var WORKER_RENDER_PLATFORM_MARKER = new core_1.OpaqueToken('WorkerRenderPlatformMarker');
    var WebWorkerInstance = function () {
        function WebWorkerInstance() {}
        /** @internal */
        WebWorkerInstance.prototype.init = function (worker, bus) {
            this.worker = worker;
            this.bus = bus;
        };
        /** @nocollapse */
        WebWorkerInstance.decorators = [{ type: core_1.Injectable }];
        return WebWorkerInstance;
    }();
    exports.WebWorkerInstance = WebWorkerInstance;
    /**
     * @experimental WebWorker support is currently experimental.
     */
    exports.WORKER_SCRIPT = new core_1.OpaqueToken('WebWorkerScript');
    /**
     * A multiple providers used to automatically call the `start()` method after the service is
     * created.
     *
     * TODO(vicb): create an interface for startable services to implement
     * @experimental WebWorker support is currently experimental.
     */
    exports.WORKER_UI_STARTABLE_MESSAGING_SERVICE = new core_1.OpaqueToken('WorkerRenderStartableMsgService');
    /**
     * @experimental WebWorker support is currently experimental.
     */
    exports.WORKER_UI_PLATFORM_PROVIDERS = [core_1.PLATFORM_COMMON_PROVIDERS, { provide: WORKER_RENDER_PLATFORM_MARKER, useValue: true }, { provide: core_1.PLATFORM_INITIALIZER, useValue: initWebWorkerRenderPlatform, multi: true }];
    /**
     * @experimental WebWorker support is currently experimental.
     */
    exports.WORKER_UI_APPLICATION_PROVIDERS = [core_1.APPLICATION_COMMON_PROVIDERS, renderer_1.MessageBasedRenderer, { provide: exports.WORKER_UI_STARTABLE_MESSAGING_SERVICE, useExisting: renderer_1.MessageBasedRenderer, multi: true }, browser_1.BROWSER_SANITIZATION_PROVIDERS, { provide: core_1.ExceptionHandler, useFactory: _exceptionHandler, deps: [] }, { provide: dom_tokens_1.DOCUMENT, useFactory: _document, deps: [] },
    // TODO(jteplitz602): Investigate if we definitely need EVENT_MANAGER on the render thread
    // #5298
    { provide: event_manager_1.EVENT_MANAGER_PLUGINS, useClass: dom_events_1.DomEventsPlugin, multi: true }, { provide: event_manager_1.EVENT_MANAGER_PLUGINS, useClass: key_events_1.KeyEventsPlugin, multi: true }, { provide: event_manager_1.EVENT_MANAGER_PLUGINS, useClass: hammer_gestures_1.HammerGesturesPlugin, multi: true }, { provide: hammer_gestures_1.HAMMER_GESTURE_CONFIG, useClass: hammer_gestures_1.HammerGestureConfig }, { provide: dom_renderer_1.DomRootRenderer, useClass: dom_renderer_1.DomRootRenderer_ }, { provide: core_1.RootRenderer, useExisting: dom_renderer_1.DomRootRenderer }, { provide: shared_styles_host_1.SharedStylesHost, useExisting: shared_styles_host_1.DomSharedStylesHost }, { provide: service_message_broker_1.ServiceMessageBrokerFactory, useClass: service_message_broker_1.ServiceMessageBrokerFactory_ }, { provide: client_message_broker_1.ClientMessageBrokerFactory, useClass: client_message_broker_1.ClientMessageBrokerFactory_ }, { provide: core_private_1.AnimationDriver, useFactory: _resolveDefaultAnimationDriver }, serializer_1.Serializer, { provide: api_1.ON_WEB_WORKER, useValue: false }, render_store_1.RenderStore, shared_styles_host_1.DomSharedStylesHost, core_1.Testability, event_manager_1.EventManager, WebWorkerInstance, { provide: core_1.APP_INITIALIZER, useFactory: initWebWorkerAppFn, multi: true, deps: [core_1.Injector] }, { provide: message_bus_1.MessageBus, useFactory: messageBusFactory, deps: [WebWorkerInstance] }];
    function initializeGenericWorkerRenderer(injector) {
        var bus = injector.get(message_bus_1.MessageBus);
        var zone = injector.get(core_1.NgZone);
        bus.attachToZone(zone);
        // initialize message services after the bus has been created
        var services = injector.get(exports.WORKER_UI_STARTABLE_MESSAGING_SERVICE);
        zone.runGuarded(function () {
            services.forEach(function (svc) {
                svc.start();
            });
        });
    }
    function messageBusFactory(instance) {
        return instance.bus;
    }
    function initWebWorkerRenderPlatform() {
        browser_adapter_1.BrowserDomAdapter.makeCurrent();
        core_private_1.wtfInit();
        testability_1.BrowserGetTestability.init();
    }
    /**
     * @experimental WebWorker support is currently experimental.
     */
    function workerUiPlatform() {
        if (lang_1.isBlank(core_1.getPlatform())) {
            core_1.createPlatform(core_1.ReflectiveInjector.resolveAndCreate(exports.WORKER_UI_PLATFORM_PROVIDERS));
        }
        return core_1.assertPlatform(WORKER_RENDER_PLATFORM_MARKER);
    }
    exports.workerUiPlatform = workerUiPlatform;
    function _exceptionHandler() {
        return new core_1.ExceptionHandler(dom_adapter_1.getDOM());
    }
    function _document() {
        return dom_adapter_1.getDOM().defaultDoc();
    }
    function initWebWorkerAppFn(injector) {
        return function () {
            var scriptUri;
            try {
                scriptUri = injector.get(exports.WORKER_SCRIPT);
            } catch (e) {
                throw new exceptions_1.BaseException('You must provide your WebWorker\'s initialization script with the WORKER_SCRIPT token');
            }
            var instance = injector.get(WebWorkerInstance);
            spawnWebWorker(scriptUri, instance);
            initializeGenericWorkerRenderer(injector);
        };
    }
    /**
     * Spawns a new class and initializes the WebWorkerInstance
     */
    function spawnWebWorker(uri, instance) {
        var webWorker = new Worker(uri);
        var sink = new post_message_bus_1.PostMessageBusSink(webWorker);
        var source = new post_message_bus_1.PostMessageBusSource(webWorker);
        var bus = new post_message_bus_1.PostMessageBus(sink, source);
        instance.init(webWorker, bus);
    }
    function _resolveDefaultAnimationDriver() {
        // web workers have not been tested or configured to
        // work with animations just yet...
        return new core_private_1.NoOpAnimationDriver();
    }
    
});
System.registerDynamic("node_modules/@angular/platform-browser/src/dom/web_animations_player.js", ["../facade/lang"], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('../facade/lang');
    var WebAnimationsPlayer = function () {
        function WebAnimationsPlayer(_player, totalTime) {
            var _this = this;
            this._player = _player;
            this.totalTime = totalTime;
            this._subscriptions = [];
            this._finished = false;
            this.parentPlayer = null;
            // this is required to make the player startable at a later time
            this.reset();
            this._player.onfinish = function () {
                return _this._onFinish();
            };
        }
        WebAnimationsPlayer.prototype._onFinish = function () {
            if (!this._finished) {
                this._finished = true;
                if (!lang_1.isPresent(this.parentPlayer)) {
                    this.destroy();
                }
                this._subscriptions.forEach(function (fn) {
                    return fn();
                });
                this._subscriptions = [];
            }
        };
        WebAnimationsPlayer.prototype.onDone = function (fn) {
            this._subscriptions.push(fn);
        };
        WebAnimationsPlayer.prototype.play = function () {
            this._player.play();
        };
        WebAnimationsPlayer.prototype.pause = function () {
            this._player.pause();
        };
        WebAnimationsPlayer.prototype.finish = function () {
            this._onFinish();
            this._player.finish();
        };
        WebAnimationsPlayer.prototype.reset = function () {
            this._player.cancel();
        };
        WebAnimationsPlayer.prototype.restart = function () {
            this.reset();
            this.play();
        };
        WebAnimationsPlayer.prototype.destroy = function () {
            this.reset();
            this._onFinish();
        };
        WebAnimationsPlayer.prototype.setPosition = function (p /** TODO #9100 */) {
            this._player.currentTime = p * this.totalTime;
        };
        WebAnimationsPlayer.prototype.getPosition = function () {
            return this._player.currentTime / this.totalTime;
        };
        return WebAnimationsPlayer;
    }();
    exports.WebAnimationsPlayer = WebAnimationsPlayer;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/dom/web_animations_driver.js', ['@angular/core', '../facade/collection', '../facade/lang', './dom_adapter', './util', './web_animations_player'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var collection_1 = $__require('../facade/collection');
    var lang_1 = $__require('../facade/lang');
    var dom_adapter_1 = $__require('./dom_adapter');
    var util_1 = $__require('./util');
    var web_animations_player_1 = $__require('./web_animations_player');
    var WebAnimationsDriver = function () {
        function WebAnimationsDriver() {}
        WebAnimationsDriver.prototype.animate = function (element, startingStyles, keyframes, duration, delay, easing) {
            var anyElm = element;
            var formattedSteps = [];
            var startingStyleLookup = {};
            if (lang_1.isPresent(startingStyles) && startingStyles.styles.length > 0) {
                startingStyleLookup = _populateStyles(anyElm, startingStyles, {});
                startingStyleLookup['offset'] = 0;
                formattedSteps.push(startingStyleLookup);
            }
            keyframes.forEach(function (keyframe) {
                var data = _populateStyles(anyElm, keyframe.styles, startingStyleLookup);
                data['offset'] = keyframe.offset;
                formattedSteps.push(data);
            });
            // this is a special case when only styles are applied as an
            // animation. When this occurs we want to animate from start to
            // end with the same values. Removing the offset and having only
            // start/end values is suitable enough for the web-animations API
            if (formattedSteps.length == 1) {
                var start = formattedSteps[0];
                start['offset'] = null;
                formattedSteps = [start, start];
            }
            var playerOptions = {
                'duration': duration,
                'delay': delay,
                'easing': easing,
                'fill': 'both' // we use `both` because it allows for styling at 0% to work with `delay`
            };
            var player = this._triggerWebAnimation(anyElm, formattedSteps, playerOptions);
            return new web_animations_player_1.WebAnimationsPlayer(player, duration);
        };
        /** @internal */
        WebAnimationsDriver.prototype._triggerWebAnimation = function (elm, keyframes, options) {
            return elm.animate(keyframes, options);
        };
        return WebAnimationsDriver;
    }();
    exports.WebAnimationsDriver = WebAnimationsDriver;
    function _populateStyles(element, styles, defaultStyles) {
        var data = {};
        styles.styles.forEach(function (entry) {
            collection_1.StringMapWrapper.forEach(entry, function (val, prop) {
                var formattedProp = util_1.dashCaseToCamelCase(prop);
                data[formattedProp] = val == core_1.AUTO_STYLE ? _computeStyle(element, formattedProp) : val.toString() + _resolveStyleUnit(val, prop, formattedProp);
            });
        });
        collection_1.StringMapWrapper.forEach(defaultStyles, function (value, prop) {
            if (!lang_1.isPresent(data[prop])) {
                data[prop] = value;
            }
        });
        return data;
    }
    function _resolveStyleUnit(val, userProvidedProp, formattedProp) {
        var unit = '';
        if (_isPixelDimensionStyle(formattedProp) && val != 0 && val != '0') {
            if (lang_1.isNumber(val)) {
                unit = 'px';
            } else if (_findDimensionalSuffix(val.toString()).length == 0) {
                throw new core_1.BaseException('Please provide a CSS unit value for ' + userProvidedProp + ':' + val);
            }
        }
        return unit;
    }
    var _$0 = 48;
    var _$9 = 57;
    var _$PERIOD = 46;
    function _findDimensionalSuffix(value) {
        for (var i = 0; i < value.length; i++) {
            var c = lang_1.StringWrapper.charCodeAt(value, i);
            if (c >= _$0 && c <= _$9 || c == _$PERIOD) continue;
            return value.substring(i, value.length);
        }
        return '';
    }
    function _isPixelDimensionStyle(prop) {
        switch (prop) {
            case 'width':
            case 'height':
            case 'minWidth':
            case 'minHeight':
            case 'maxWidth':
            case 'maxHeight':
            case 'left':
            case 'top':
            case 'bottom':
            case 'right':
            case 'fontSize':
            case 'outlineWidth':
            case 'outlineOffset':
            case 'paddingTop':
            case 'paddingLeft':
            case 'paddingBottom':
            case 'paddingRight':
            case 'marginTop':
            case 'marginLeft':
            case 'marginBottom':
            case 'marginRight':
            case 'borderRadius':
            case 'borderWidth':
            case 'borderTopWidth':
            case 'borderLeftWidth':
            case 'borderRightWidth':
            case 'borderBottomWidth':
            case 'textIndent':
                return true;
            default:
                return false;
        }
    }
    function _computeStyle(element, prop) {
        return dom_adapter_1.getDOM().getComputedStyle(element)[prop];
    }
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/browser/generic_browser_adapter.js', ['../dom/dom_adapter', '../facade/collection', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var dom_adapter_1 = $__require('../dom/dom_adapter');
    var collection_1 = $__require('../facade/collection');
    var lang_1 = $__require('../facade/lang');
    /**
     * Provides DOM operations in any browser environment.
     */
    var GenericBrowserDomAdapter = function (_super) {
        __extends(GenericBrowserDomAdapter, _super);
        function GenericBrowserDomAdapter() {
            var _this = this;
            _super.call(this);
            this._animationPrefix = null;
            this._transitionEnd = null;
            try {
                var element = this.createElement('div', this.defaultDoc());
                if (lang_1.isPresent(this.getStyle(element, 'animationName'))) {
                    this._animationPrefix = '';
                } else {
                    var domPrefixes = ['Webkit', 'Moz', 'O', 'ms'];
                    for (var i = 0; i < domPrefixes.length; i++) {
                        if (lang_1.isPresent(this.getStyle(element, domPrefixes[i] + 'AnimationName'))) {
                            this._animationPrefix = '-' + domPrefixes[i].toLowerCase() + '-';
                            break;
                        }
                    }
                }
                var transEndEventNames = {
                    WebkitTransition: 'webkitTransitionEnd',
                    MozTransition: 'transitionend',
                    OTransition: 'oTransitionEnd otransitionend',
                    transition: 'transitionend'
                };
                collection_1.StringMapWrapper.forEach(transEndEventNames, function (value, key) {
                    if (lang_1.isPresent(_this.getStyle(element, key))) {
                        _this._transitionEnd = value;
                    }
                });
            } catch (e) {
                this._animationPrefix = null;
                this._transitionEnd = null;
            }
        }
        GenericBrowserDomAdapter.prototype.getDistributedNodes = function (el) {
            return el.getDistributedNodes();
        };
        GenericBrowserDomAdapter.prototype.resolveAndSetHref = function (el, baseUrl, href) {
            el.href = href == null ? baseUrl : baseUrl + '/../' + href;
        };
        GenericBrowserDomAdapter.prototype.supportsDOMEvents = function () {
            return true;
        };
        GenericBrowserDomAdapter.prototype.supportsNativeShadowDOM = function () {
            return lang_1.isFunction(this.defaultDoc().body.createShadowRoot);
        };
        GenericBrowserDomAdapter.prototype.getAnimationPrefix = function () {
            return lang_1.isPresent(this._animationPrefix) ? this._animationPrefix : '';
        };
        GenericBrowserDomAdapter.prototype.getTransitionEnd = function () {
            return lang_1.isPresent(this._transitionEnd) ? this._transitionEnd : '';
        };
        GenericBrowserDomAdapter.prototype.supportsAnimation = function () {
            return lang_1.isPresent(this._animationPrefix) && lang_1.isPresent(this._transitionEnd);
        };
        return GenericBrowserDomAdapter;
    }(dom_adapter_1.DomAdapter);
    exports.GenericBrowserDomAdapter = GenericBrowserDomAdapter;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/browser/browser_adapter.js', ['../dom/dom_adapter', '../facade/collection', '../facade/lang', './generic_browser_adapter'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var dom_adapter_1 = $__require('../dom/dom_adapter');
    var collection_1 = $__require('../facade/collection');
    var lang_1 = $__require('../facade/lang');
    var generic_browser_adapter_1 = $__require('./generic_browser_adapter');
    var _attrToPropMap = {
        'class': 'className',
        'innerHtml': 'innerHTML',
        'readonly': 'readOnly',
        'tabindex': 'tabIndex'
    };
    var DOM_KEY_LOCATION_NUMPAD = 3;
    // Map to convert some key or keyIdentifier values to what will be returned by getEventKey
    var _keyMap = {
        // The following values are here for cross-browser compatibility and to match the W3C standard
        // cf http://www.w3.org/TR/DOM-Level-3-Events-key/
        '\b': 'Backspace',
        '\t': 'Tab',
        '\x7F': 'Delete',
        '\x1B': 'Escape',
        'Del': 'Delete',
        'Esc': 'Escape',
        'Left': 'ArrowLeft',
        'Right': 'ArrowRight',
        'Up': 'ArrowUp',
        'Down': 'ArrowDown',
        'Menu': 'ContextMenu',
        'Scroll': 'ScrollLock',
        'Win': 'OS'
    };
    // There is a bug in Chrome for numeric keypad keys:
    // https://code.google.com/p/chromium/issues/detail?id=155654
    // 1, 2, 3 ... are reported as A, B, C ...
    var _chromeNumKeyPadMap = {
        'A': '1',
        'B': '2',
        'C': '3',
        'D': '4',
        'E': '5',
        'F': '6',
        'G': '7',
        'H': '8',
        'I': '9',
        'J': '*',
        'K': '+',
        'M': '-',
        'N': '.',
        'O': '/',
        '\x60': '0',
        '\x90': 'NumLock'
    };
    /**
     * A `DomAdapter` powered by full browser DOM APIs.
     */
    /* tslint:disable:requireParameterType */
    var BrowserDomAdapter = function (_super) {
        __extends(BrowserDomAdapter, _super);
        function BrowserDomAdapter() {
            _super.apply(this, arguments);
        }
        BrowserDomAdapter.prototype.parse = function (templateHtml) {
            throw new Error('parse not implemented');
        };
        BrowserDomAdapter.makeCurrent = function () {
            dom_adapter_1.setRootDomAdapter(new BrowserDomAdapter());
        };
        BrowserDomAdapter.prototype.hasProperty = function (element /** TODO #9100 */, name) {
            return name in element;
        };
        BrowserDomAdapter.prototype.setProperty = function (el, name, value) {
            el[name] = value;
        };
        BrowserDomAdapter.prototype.getProperty = function (el, name) {
            return el[name];
        };
        BrowserDomAdapter.prototype.invoke = function (el, methodName, args) {
            el[methodName].apply(el, args);
        };
        // TODO(tbosch): move this into a separate environment class once we have it
        BrowserDomAdapter.prototype.logError = function (error /** TODO #9100 */) {
            if (window.console.error) {
                window.console.error(error);
            } else {
                window.console.log(error);
            }
        };
        BrowserDomAdapter.prototype.log = function (error /** TODO #9100 */) {
            window.console.log(error);
        };
        BrowserDomAdapter.prototype.logGroup = function (error /** TODO #9100 */) {
            if (window.console.group) {
                window.console.group(error);
                this.logError(error);
            } else {
                window.console.log(error);
            }
        };
        BrowserDomAdapter.prototype.logGroupEnd = function () {
            if (window.console.groupEnd) {
                window.console.groupEnd();
            }
        };
        Object.defineProperty(BrowserDomAdapter.prototype, "attrToPropMap", {
            get: function () {
                return _attrToPropMap;
            },
            enumerable: true,
            configurable: true
        });
        BrowserDomAdapter.prototype.query = function (selector) {
            return document.querySelector(selector);
        };
        BrowserDomAdapter.prototype.querySelector = function (el /** TODO #9100 */, selector) {
            return el.querySelector(selector);
        };
        BrowserDomAdapter.prototype.querySelectorAll = function (el /** TODO #9100 */, selector) {
            return el.querySelectorAll(selector);
        };
        BrowserDomAdapter.prototype.on = function (el /** TODO #9100 */, evt /** TODO #9100 */, listener /** TODO #9100 */) {
            el.addEventListener(evt, listener, false);
        };
        BrowserDomAdapter.prototype.onAndCancel = function (el /** TODO #9100 */, evt /** TODO #9100 */, listener /** TODO #9100 */) {
            el.addEventListener(evt, listener, false);
            // Needed to follow Dart's subscription semantic, until fix of
            // https://code.google.com/p/dart/issues/detail?id=17406
            return function () {
                el.removeEventListener(evt, listener, false);
            };
        };
        BrowserDomAdapter.prototype.dispatchEvent = function (el /** TODO #9100 */, evt /** TODO #9100 */) {
            el.dispatchEvent(evt);
        };
        BrowserDomAdapter.prototype.createMouseEvent = function (eventType) {
            var evt = document.createEvent('MouseEvent');
            evt.initEvent(eventType, true, true);
            return evt;
        };
        BrowserDomAdapter.prototype.createEvent = function (eventType /** TODO #9100 */) {
            var evt = document.createEvent('Event');
            evt.initEvent(eventType, true, true);
            return evt;
        };
        BrowserDomAdapter.prototype.preventDefault = function (evt) {
            evt.preventDefault();
            evt.returnValue = false;
        };
        BrowserDomAdapter.prototype.isPrevented = function (evt) {
            return evt.defaultPrevented || lang_1.isPresent(evt.returnValue) && !evt.returnValue;
        };
        BrowserDomAdapter.prototype.getInnerHTML = function (el /** TODO #9100 */) {
            return el.innerHTML;
        };
        BrowserDomAdapter.prototype.getTemplateContent = function (el /** TODO #9100 */) {
            return 'content' in el && el instanceof HTMLTemplateElement ? el.content : null;
        };
        BrowserDomAdapter.prototype.getOuterHTML = function (el /** TODO #9100 */) {
            return el.outerHTML;
        };
        BrowserDomAdapter.prototype.nodeName = function (node) {
            return node.nodeName;
        };
        BrowserDomAdapter.prototype.nodeValue = function (node) {
            return node.nodeValue;
        };
        BrowserDomAdapter.prototype.type = function (node) {
            return node.type;
        };
        BrowserDomAdapter.prototype.content = function (node) {
            if (this.hasProperty(node, 'content')) {
                return node.content;
            } else {
                return node;
            }
        };
        BrowserDomAdapter.prototype.firstChild = function (el /** TODO #9100 */) {
            return el.firstChild;
        };
        BrowserDomAdapter.prototype.nextSibling = function (el /** TODO #9100 */) {
            return el.nextSibling;
        };
        BrowserDomAdapter.prototype.parentElement = function (el /** TODO #9100 */) {
            return el.parentNode;
        };
        BrowserDomAdapter.prototype.childNodes = function (el /** TODO #9100 */) {
            return el.childNodes;
        };
        BrowserDomAdapter.prototype.childNodesAsList = function (el /** TODO #9100 */) {
            var childNodes = el.childNodes;
            var res = collection_1.ListWrapper.createFixedSize(childNodes.length);
            for (var i = 0; i < childNodes.length; i++) {
                res[i] = childNodes[i];
            }
            return res;
        };
        BrowserDomAdapter.prototype.clearNodes = function (el /** TODO #9100 */) {
            while (el.firstChild) {
                el.removeChild(el.firstChild);
            }
        };
        BrowserDomAdapter.prototype.appendChild = function (el /** TODO #9100 */, node /** TODO #9100 */) {
            el.appendChild(node);
        };
        BrowserDomAdapter.prototype.removeChild = function (el /** TODO #9100 */, node /** TODO #9100 */) {
            el.removeChild(node);
        };
        BrowserDomAdapter.prototype.replaceChild = function (el, newChild /** TODO #9100 */, oldChild /** TODO #9100 */) {
            el.replaceChild(newChild, oldChild);
        };
        BrowserDomAdapter.prototype.remove = function (node /** TODO #9100 */) {
            if (node.parentNode) {
                node.parentNode.removeChild(node);
            }
            return node;
        };
        BrowserDomAdapter.prototype.insertBefore = function (el /** TODO #9100 */, node /** TODO #9100 */) {
            el.parentNode.insertBefore(node, el);
        };
        BrowserDomAdapter.prototype.insertAllBefore = function (el /** TODO #9100 */, nodes /** TODO #9100 */) {
            nodes.forEach(function (n /** TODO #9100 */) {
                return el.parentNode.insertBefore(n, el);
            });
        };
        BrowserDomAdapter.prototype.insertAfter = function (el /** TODO #9100 */, node /** TODO #9100 */) {
            el.parentNode.insertBefore(node, el.nextSibling);
        };
        BrowserDomAdapter.prototype.setInnerHTML = function (el /** TODO #9100 */, value /** TODO #9100 */) {
            el.innerHTML = value;
        };
        BrowserDomAdapter.prototype.getText = function (el /** TODO #9100 */) {
            return el.textContent;
        };
        // TODO(vicb): removed Element type because it does not support StyleElement
        BrowserDomAdapter.prototype.setText = function (el /** TODO #9100 */, value) {
            el.textContent = value;
        };
        BrowserDomAdapter.prototype.getValue = function (el /** TODO #9100 */) {
            return el.value;
        };
        BrowserDomAdapter.prototype.setValue = function (el /** TODO #9100 */, value) {
            el.value = value;
        };
        BrowserDomAdapter.prototype.getChecked = function (el /** TODO #9100 */) {
            return el.checked;
        };
        BrowserDomAdapter.prototype.setChecked = function (el /** TODO #9100 */, value) {
            el.checked = value;
        };
        BrowserDomAdapter.prototype.createComment = function (text) {
            return document.createComment(text);
        };
        BrowserDomAdapter.prototype.createTemplate = function (html /** TODO #9100 */) {
            var t = document.createElement('template');
            t.innerHTML = html;
            return t;
        };
        BrowserDomAdapter.prototype.createElement = function (tagName /* TODO #9100 */, doc) {
            if (doc === void 0) {
                doc = document;
            }
            return doc.createElement(tagName);
        };
        BrowserDomAdapter.prototype.createElementNS = function (ns /* TODO #9100 */, tagName /* TODO #9100 */, doc) {
            if (doc === void 0) {
                doc = document;
            }
            return doc.createElementNS(ns, tagName);
        };
        BrowserDomAdapter.prototype.createTextNode = function (text, doc) {
            if (doc === void 0) {
                doc = document;
            }
            return doc.createTextNode(text);
        };
        BrowserDomAdapter.prototype.createScriptTag = function (attrName, attrValue, doc) {
            if (doc === void 0) {
                doc = document;
            }
            var el = doc.createElement('SCRIPT');
            el.setAttribute(attrName, attrValue);
            return el;
        };
        BrowserDomAdapter.prototype.createStyleElement = function (css, doc) {
            if (doc === void 0) {
                doc = document;
            }
            var style = doc.createElement('style');
            this.appendChild(style, this.createTextNode(css));
            return style;
        };
        BrowserDomAdapter.prototype.createShadowRoot = function (el) {
            return el.createShadowRoot();
        };
        BrowserDomAdapter.prototype.getShadowRoot = function (el) {
            return el.shadowRoot;
        };
        BrowserDomAdapter.prototype.getHost = function (el) {
            return el.host;
        };
        BrowserDomAdapter.prototype.clone = function (node) {
            return node.cloneNode(true);
        };
        BrowserDomAdapter.prototype.getElementsByClassName = function (element /** TODO #9100 */, name) {
            return element.getElementsByClassName(name);
        };
        BrowserDomAdapter.prototype.getElementsByTagName = function (element /** TODO #9100 */, name) {
            return element.getElementsByTagName(name);
        };
        BrowserDomAdapter.prototype.classList = function (element /** TODO #9100 */) {
            return Array.prototype.slice.call(element.classList, 0);
        };
        BrowserDomAdapter.prototype.addClass = function (element /** TODO #9100 */, className) {
            element.classList.add(className);
        };
        BrowserDomAdapter.prototype.removeClass = function (element /** TODO #9100 */, className) {
            element.classList.remove(className);
        };
        BrowserDomAdapter.prototype.hasClass = function (element /** TODO #9100 */, className) {
            return element.classList.contains(className);
        };
        BrowserDomAdapter.prototype.setStyle = function (element /** TODO #9100 */, styleName, styleValue) {
            element.style[styleName] = styleValue;
        };
        BrowserDomAdapter.prototype.removeStyle = function (element /** TODO #9100 */, stylename) {
            element.style[stylename] = null;
        };
        BrowserDomAdapter.prototype.getStyle = function (element /** TODO #9100 */, stylename) {
            return element.style[stylename];
        };
        BrowserDomAdapter.prototype.hasStyle = function (element /** TODO #9100 */, styleName, styleValue) {
            if (styleValue === void 0) {
                styleValue = null;
            }
            var value = this.getStyle(element, styleName) || '';
            return styleValue ? value == styleValue : value.length > 0;
        };
        BrowserDomAdapter.prototype.tagName = function (element /** TODO #9100 */) {
            return element.tagName;
        };
        BrowserDomAdapter.prototype.attributeMap = function (element /** TODO #9100 */) {
            var res = new Map();
            var elAttrs = element.attributes;
            for (var i = 0; i < elAttrs.length; i++) {
                var attrib = elAttrs[i];
                res.set(attrib.name, attrib.value);
            }
            return res;
        };
        BrowserDomAdapter.prototype.hasAttribute = function (element /** TODO #9100 */, attribute) {
            return element.hasAttribute(attribute);
        };
        BrowserDomAdapter.prototype.hasAttributeNS = function (element /** TODO #9100 */, ns, attribute) {
            return element.hasAttributeNS(ns, attribute);
        };
        BrowserDomAdapter.prototype.getAttribute = function (element /** TODO #9100 */, attribute) {
            return element.getAttribute(attribute);
        };
        BrowserDomAdapter.prototype.getAttributeNS = function (element /** TODO #9100 */, ns, name) {
            return element.getAttributeNS(ns, name);
        };
        BrowserDomAdapter.prototype.setAttribute = function (element /** TODO #9100 */, name, value) {
            element.setAttribute(name, value);
        };
        BrowserDomAdapter.prototype.setAttributeNS = function (element /** TODO #9100 */, ns, name, value) {
            element.setAttributeNS(ns, name, value);
        };
        BrowserDomAdapter.prototype.removeAttribute = function (element /** TODO #9100 */, attribute) {
            element.removeAttribute(attribute);
        };
        BrowserDomAdapter.prototype.removeAttributeNS = function (element /** TODO #9100 */, ns, name) {
            element.removeAttributeNS(ns, name);
        };
        BrowserDomAdapter.prototype.templateAwareRoot = function (el /** TODO #9100 */) {
            return this.isTemplateElement(el) ? this.content(el) : el;
        };
        BrowserDomAdapter.prototype.createHtmlDocument = function () {
            return document.implementation.createHTMLDocument('fakeTitle');
        };
        BrowserDomAdapter.prototype.defaultDoc = function () {
            return document;
        };
        BrowserDomAdapter.prototype.getBoundingClientRect = function (el /** TODO #9100 */) {
            try {
                return el.getBoundingClientRect();
            } catch (e) {
                return { top: 0, bottom: 0, left: 0, right: 0, width: 0, height: 0 };
            }
        };
        BrowserDomAdapter.prototype.getTitle = function () {
            return document.title;
        };
        BrowserDomAdapter.prototype.setTitle = function (newTitle) {
            document.title = newTitle || '';
        };
        BrowserDomAdapter.prototype.elementMatches = function (n /** TODO #9100 */, selector) {
            var matches = false;
            if (n instanceof HTMLElement) {
                if (n.matches) {
                    matches = n.matches(selector);
                } else if (n.msMatchesSelector) {
                    matches = n.msMatchesSelector(selector);
                } else if (n.webkitMatchesSelector) {
                    matches = n.webkitMatchesSelector(selector);
                }
            }
            return matches;
        };
        BrowserDomAdapter.prototype.isTemplateElement = function (el) {
            return el instanceof HTMLElement && el.nodeName == 'TEMPLATE';
        };
        BrowserDomAdapter.prototype.isTextNode = function (node) {
            return node.nodeType === Node.TEXT_NODE;
        };
        BrowserDomAdapter.prototype.isCommentNode = function (node) {
            return node.nodeType === Node.COMMENT_NODE;
        };
        BrowserDomAdapter.prototype.isElementNode = function (node) {
            return node.nodeType === Node.ELEMENT_NODE;
        };
        BrowserDomAdapter.prototype.hasShadowRoot = function (node /** TODO #9100 */) {
            return node instanceof HTMLElement && lang_1.isPresent(node.shadowRoot);
        };
        BrowserDomAdapter.prototype.isShadowRoot = function (node /** TODO #9100 */) {
            return node instanceof DocumentFragment;
        };
        BrowserDomAdapter.prototype.importIntoDoc = function (node) {
            var toImport = node;
            if (this.isTemplateElement(node)) {
                toImport = this.content(node);
            }
            return document.importNode(toImport, true);
        };
        BrowserDomAdapter.prototype.adoptNode = function (node) {
            return document.adoptNode(node);
        };
        BrowserDomAdapter.prototype.getHref = function (el) {
            return el.href;
        };
        BrowserDomAdapter.prototype.getEventKey = function (event /** TODO #9100 */) {
            var key = event.key;
            if (lang_1.isBlank(key)) {
                key = event.keyIdentifier;
                // keyIdentifier is defined in the old draft of DOM Level 3 Events implemented by Chrome and
                // Safari
                // cf
                // http://www.w3.org/TR/2007/WD-DOM-Level-3-Events-20071221/events.html#Events-KeyboardEvents-Interfaces
                if (lang_1.isBlank(key)) {
                    return 'Unidentified';
                }
                if (key.startsWith('U+')) {
                    key = String.fromCharCode(parseInt(key.substring(2), 16));
                    if (event.location === DOM_KEY_LOCATION_NUMPAD && _chromeNumKeyPadMap.hasOwnProperty(key)) {
                        // There is a bug in Chrome for numeric keypad keys:
                        // https://code.google.com/p/chromium/issues/detail?id=155654
                        // 1, 2, 3 ... are reported as A, B, C ...
                        key = _chromeNumKeyPadMap[key];
                    }
                }
            }
            if (_keyMap.hasOwnProperty(key)) {
                key = _keyMap[key];
            }
            return key;
        };
        BrowserDomAdapter.prototype.getGlobalEventTarget = function (target) {
            if (target == 'window') {
                return window;
            } else if (target == 'document') {
                return document;
            } else if (target == 'body') {
                return document.body;
            }
        };
        BrowserDomAdapter.prototype.getHistory = function () {
            return window.history;
        };
        BrowserDomAdapter.prototype.getLocation = function () {
            return window.location;
        };
        BrowserDomAdapter.prototype.getBaseHref = function () {
            var href = getBaseElementHref();
            if (lang_1.isBlank(href)) {
                return null;
            }
            return relativePath(href);
        };
        BrowserDomAdapter.prototype.resetBaseElement = function () {
            baseElement = null;
        };
        BrowserDomAdapter.prototype.getUserAgent = function () {
            return window.navigator.userAgent;
        };
        BrowserDomAdapter.prototype.setData = function (element /** TODO #9100 */, name, value) {
            this.setAttribute(element, 'data-' + name, value);
        };
        BrowserDomAdapter.prototype.getData = function (element /** TODO #9100 */, name) {
            return this.getAttribute(element, 'data-' + name);
        };
        BrowserDomAdapter.prototype.getComputedStyle = function (element /** TODO #9100 */) {
            return getComputedStyle(element);
        };
        // TODO(tbosch): move this into a separate environment class once we have it
        BrowserDomAdapter.prototype.setGlobalVar = function (path, value) {
            lang_1.setValueOnPath(lang_1.global, path, value);
        };
        BrowserDomAdapter.prototype.requestAnimationFrame = function (callback /** TODO #9100 */) {
            return window.requestAnimationFrame(callback);
        };
        BrowserDomAdapter.prototype.cancelAnimationFrame = function (id) {
            window.cancelAnimationFrame(id);
        };
        BrowserDomAdapter.prototype.supportsWebAnimation = function () {
            return lang_1.isFunction(document.body['animate']);
        };
        BrowserDomAdapter.prototype.performanceNow = function () {
            // performance.now() is not available in all browsers, see
            // http://caniuse.com/#search=performance.now
            if (lang_1.isPresent(window.performance) && lang_1.isPresent(window.performance.now)) {
                return window.performance.now();
            } else {
                return lang_1.DateWrapper.toMillis(lang_1.DateWrapper.now());
            }
        };
        BrowserDomAdapter.prototype.supportsCookies = function () {
            return true;
        };
        BrowserDomAdapter.prototype.getCookie = function (name) {
            return parseCookieValue(document.cookie, name);
        };
        BrowserDomAdapter.prototype.setCookie = function (name, value) {
            // document.cookie is magical, assigning into it assigns/overrides one cookie value, but does
            // not clear other cookies.
            document.cookie = encodeURIComponent(name) + '=' + encodeURIComponent(value);
        };
        return BrowserDomAdapter;
    }(generic_browser_adapter_1.GenericBrowserDomAdapter);
    exports.BrowserDomAdapter = BrowserDomAdapter;
    var baseElement = null;
    function getBaseElementHref() {
        if (lang_1.isBlank(baseElement)) {
            baseElement = document.querySelector('base');
            if (lang_1.isBlank(baseElement)) {
                return null;
            }
        }
        return baseElement.getAttribute('href');
    }
    // based on urlUtils.js in AngularJS 1
    var urlParsingNode = null;
    function relativePath(url /** TODO #9100 */) {
        if (lang_1.isBlank(urlParsingNode)) {
            urlParsingNode = document.createElement('a');
        }
        urlParsingNode.setAttribute('href', url);
        return urlParsingNode.pathname.charAt(0) === '/' ? urlParsingNode.pathname : '/' + urlParsingNode.pathname;
    }
    function parseCookieValue(cookie, name) {
        name = encodeURIComponent(name);
        var cookies = cookie.split(';');
        for (var _i = 0, cookies_1 = cookies; _i < cookies_1.length; _i++) {
            var cookie_1 = cookies_1[_i];
            var _a = cookie_1.split('=', 2),
                key = _a[0],
                value = _a[1];
            if (key.trim() === name) {
                return decodeURIComponent(value);
            }
        }
        return null;
    }
    exports.parseCookieValue = parseCookieValue;
    
});
System.registerDynamic('node_modules/@angular/common/src/pipes/async_pipe.js', ['@angular/core', '../facade/async', '../facade/lang', './invalid_pipe_argument_exception'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var async_1 = $__require('../facade/async');
    var lang_1 = $__require('../facade/lang');
    var invalid_pipe_argument_exception_1 = $__require('./invalid_pipe_argument_exception');
    var ObservableStrategy = function () {
        function ObservableStrategy() {}
        ObservableStrategy.prototype.createSubscription = function (async, updateLatestValue) {
            return async_1.ObservableWrapper.subscribe(async, updateLatestValue, function (e) {
                throw e;
            });
        };
        ObservableStrategy.prototype.dispose = function (subscription) {
            async_1.ObservableWrapper.dispose(subscription);
        };
        ObservableStrategy.prototype.onDestroy = function (subscription) {
            async_1.ObservableWrapper.dispose(subscription);
        };
        return ObservableStrategy;
    }();
    var PromiseStrategy = function () {
        function PromiseStrategy() {}
        PromiseStrategy.prototype.createSubscription = function (async, updateLatestValue) {
            return async.then(updateLatestValue, function (e) {
                throw e;
            });
        };
        PromiseStrategy.prototype.dispose = function (subscription) {};
        PromiseStrategy.prototype.onDestroy = function (subscription) {};
        return PromiseStrategy;
    }();
    var _promiseStrategy = new PromiseStrategy();
    var _observableStrategy = new ObservableStrategy();
    var __unused;
    var AsyncPipe = function () {
        function AsyncPipe(_ref) {
            /** @internal */
            this._latestValue = null;
            /** @internal */
            this._latestReturnedValue = null;
            /** @internal */
            this._subscription = null;
            /** @internal */
            this._obj = null;
            this._strategy = null;
            this._ref = _ref;
        }
        AsyncPipe.prototype.ngOnDestroy = function () {
            if (lang_1.isPresent(this._subscription)) {
                this._dispose();
            }
        };
        AsyncPipe.prototype.transform = function (obj) {
            if (lang_1.isBlank(this._obj)) {
                if (lang_1.isPresent(obj)) {
                    this._subscribe(obj);
                }
                this._latestReturnedValue = this._latestValue;
                return this._latestValue;
            }
            if (obj !== this._obj) {
                this._dispose();
                return this.transform(obj);
            }
            if (this._latestValue === this._latestReturnedValue) {
                return this._latestReturnedValue;
            } else {
                this._latestReturnedValue = this._latestValue;
                return core_1.WrappedValue.wrap(this._latestValue);
            }
        };
        /** @internal */
        AsyncPipe.prototype._subscribe = function (obj) {
            var _this = this;
            this._obj = obj;
            this._strategy = this._selectStrategy(obj);
            this._subscription = this._strategy.createSubscription(obj, function (value) {
                return _this._updateLatestValue(obj, value);
            });
        };
        /** @internal */
        AsyncPipe.prototype._selectStrategy = function (obj) {
            if (lang_1.isPromise(obj)) {
                return _promiseStrategy;
            } else if (async_1.ObservableWrapper.isObservable(obj)) {
                return _observableStrategy;
            } else {
                throw new invalid_pipe_argument_exception_1.InvalidPipeArgumentException(AsyncPipe, obj);
            }
        };
        /** @internal */
        AsyncPipe.prototype._dispose = function () {
            this._strategy.dispose(this._subscription);
            this._latestValue = null;
            this._latestReturnedValue = null;
            this._subscription = null;
            this._obj = null;
        };
        /** @internal */
        AsyncPipe.prototype._updateLatestValue = function (async, value) {
            if (async === this._obj) {
                this._latestValue = value;
                this._ref.markForCheck();
            }
        };
        /** @nocollapse */
        AsyncPipe.decorators = [{ type: core_1.Pipe, args: [{ name: 'async', pure: false }] }];
        /** @nocollapse */
        AsyncPipe.ctorParameters = [{ type: core_1.ChangeDetectorRef }];
        return AsyncPipe;
    }();
    exports.AsyncPipe = AsyncPipe;
    
});
System.registerDynamic('node_modules/@angular/common/src/pipes/common_pipes.js', ['./async_pipe', './date_pipe', './i18n_plural_pipe', './i18n_select_pipe', './json_pipe', './lowercase_pipe', './number_pipe', './replace_pipe', './slice_pipe', './uppercase_pipe'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * @module
   * @description
   * This module provides a set of common Pipes.
   */

  var global = this || self,
      GLOBAL = global;
  var async_pipe_1 = $__require('./async_pipe');
  var date_pipe_1 = $__require('./date_pipe');
  var i18n_plural_pipe_1 = $__require('./i18n_plural_pipe');
  var i18n_select_pipe_1 = $__require('./i18n_select_pipe');
  var json_pipe_1 = $__require('./json_pipe');
  var lowercase_pipe_1 = $__require('./lowercase_pipe');
  var number_pipe_1 = $__require('./number_pipe');
  var replace_pipe_1 = $__require('./replace_pipe');
  var slice_pipe_1 = $__require('./slice_pipe');
  var uppercase_pipe_1 = $__require('./uppercase_pipe');
  /**
   * A collection of Angular core pipes that are likely to be used in each and every
   * application.
   *
   * This collection can be used to quickly enumerate all the built-in pipes in the `pipes`
   * property of the `@Component` decorator.
   *
   * @experimental Contains i18n pipes which are experimental
   */
  exports.COMMON_PIPES = [async_pipe_1.AsyncPipe, uppercase_pipe_1.UpperCasePipe, lowercase_pipe_1.LowerCasePipe, json_pipe_1.JsonPipe, slice_pipe_1.SlicePipe, number_pipe_1.DecimalPipe, number_pipe_1.PercentPipe, number_pipe_1.CurrencyPipe, date_pipe_1.DatePipe, replace_pipe_1.ReplacePipe, i18n_plural_pipe_1.I18nPluralPipe, i18n_select_pipe_1.I18nSelectPipe];
  
});
System.registerDynamic('node_modules/@angular/common/src/pipes/date_pipe.js', ['@angular/core', '../facade/collection', '../facade/intl', '../facade/lang', './invalid_pipe_argument_exception'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var collection_1 = $__require('../facade/collection');
    var intl_1 = $__require('../facade/intl');
    var lang_1 = $__require('../facade/lang');
    var invalid_pipe_argument_exception_1 = $__require('./invalid_pipe_argument_exception');
    // TODO: move to a global configurable location along with other i18n components.
    var defaultLocale = 'en-US';
    var DatePipe = function () {
        function DatePipe() {}
        DatePipe.prototype.transform = function (value, pattern) {
            if (pattern === void 0) {
                pattern = 'mediumDate';
            }
            if (lang_1.isBlank(value)) return null;
            if (!this.supports(value)) {
                throw new invalid_pipe_argument_exception_1.InvalidPipeArgumentException(DatePipe, value);
            }
            if (lang_1.NumberWrapper.isNumeric(value)) {
                value = lang_1.DateWrapper.fromMillis(lang_1.NumberWrapper.parseInt(value, 10));
            } else if (lang_1.isString(value)) {
                value = lang_1.DateWrapper.fromISOString(value);
            }
            if (collection_1.StringMapWrapper.contains(DatePipe._ALIASES, pattern)) {
                pattern = collection_1.StringMapWrapper.get(DatePipe._ALIASES, pattern);
            }
            return intl_1.DateFormatter.format(value, defaultLocale, pattern);
        };
        DatePipe.prototype.supports = function (obj) {
            if (lang_1.isDate(obj) || lang_1.NumberWrapper.isNumeric(obj)) {
                return true;
            }
            if (lang_1.isString(obj) && lang_1.isDate(lang_1.DateWrapper.fromISOString(obj))) {
                return true;
            }
            return false;
        };
        /** @internal */
        DatePipe._ALIASES = {
            'medium': 'yMMMdjms',
            'short': 'yMdjm',
            'fullDate': 'yMMMMEEEEd',
            'longDate': 'yMMMMd',
            'mediumDate': 'yMMMd',
            'shortDate': 'yMd',
            'mediumTime': 'jms',
            'shortTime': 'jm'
        };
        /** @nocollapse */
        DatePipe.decorators = [{ type: core_1.Pipe, args: [{ name: 'date', pure: true }] }];
        return DatePipe;
    }();
    exports.DatePipe = DatePipe;
    
});
System.registerDynamic('node_modules/@angular/common/src/pipes/i18n_plural_pipe.js', ['@angular/core', '../facade/lang', '../localization', './invalid_pipe_argument_exception'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../facade/lang');
    var localization_1 = $__require('../localization');
    var invalid_pipe_argument_exception_1 = $__require('./invalid_pipe_argument_exception');
    var _INTERPOLATION_REGEXP = /#/g;
    var I18nPluralPipe = function () {
        function I18nPluralPipe(_localization) {
            this._localization = _localization;
        }
        I18nPluralPipe.prototype.transform = function (value, pluralMap) {
            if (lang_1.isBlank(value)) return '';
            if (!lang_1.isStringMap(pluralMap)) {
                throw new invalid_pipe_argument_exception_1.InvalidPipeArgumentException(I18nPluralPipe, pluralMap);
            }
            var key = localization_1.getPluralCategory(value, Object.getOwnPropertyNames(pluralMap), this._localization);
            return lang_1.StringWrapper.replaceAll(pluralMap[key], _INTERPOLATION_REGEXP, value.toString());
        };
        /** @nocollapse */
        I18nPluralPipe.decorators = [{ type: core_1.Pipe, args: [{ name: 'i18nPlural', pure: true }] }];
        /** @nocollapse */
        I18nPluralPipe.ctorParameters = [{ type: localization_1.NgLocalization }];
        return I18nPluralPipe;
    }();
    exports.I18nPluralPipe = I18nPluralPipe;
    
});
System.registerDynamic('node_modules/@angular/common/src/pipes/i18n_select_pipe.js', ['@angular/core', '../facade/lang', './invalid_pipe_argument_exception'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../facade/lang');
    var invalid_pipe_argument_exception_1 = $__require('./invalid_pipe_argument_exception');
    var I18nSelectPipe = function () {
        function I18nSelectPipe() {}
        I18nSelectPipe.prototype.transform = function (value, mapping) {
            if (lang_1.isBlank(value)) return '';
            if (!lang_1.isStringMap(mapping)) {
                throw new invalid_pipe_argument_exception_1.InvalidPipeArgumentException(I18nSelectPipe, mapping);
            }
            return mapping.hasOwnProperty(value) ? mapping[value] : '';
        };
        /** @nocollapse */
        I18nSelectPipe.decorators = [{ type: core_1.Pipe, args: [{ name: 'i18nSelect', pure: true }] }];
        return I18nSelectPipe;
    }();
    exports.I18nSelectPipe = I18nSelectPipe;
    
});
System.registerDynamic('node_modules/@angular/common/src/pipes/json_pipe.js', ['@angular/core', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../facade/lang');
    var JsonPipe = function () {
        function JsonPipe() {}
        JsonPipe.prototype.transform = function (value) {
            return lang_1.Json.stringify(value);
        };
        /** @nocollapse */
        JsonPipe.decorators = [{ type: core_1.Pipe, args: [{ name: 'json', pure: false }] }];
        return JsonPipe;
    }();
    exports.JsonPipe = JsonPipe;
    
});
System.registerDynamic('node_modules/@angular/common/src/pipes/lowercase_pipe.js', ['@angular/core', '../facade/lang', './invalid_pipe_argument_exception'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../facade/lang');
    var invalid_pipe_argument_exception_1 = $__require('./invalid_pipe_argument_exception');
    var LowerCasePipe = function () {
        function LowerCasePipe() {}
        LowerCasePipe.prototype.transform = function (value) {
            if (lang_1.isBlank(value)) return value;
            if (!lang_1.isString(value)) {
                throw new invalid_pipe_argument_exception_1.InvalidPipeArgumentException(LowerCasePipe, value);
            }
            return value.toLowerCase();
        };
        /** @nocollapse */
        LowerCasePipe.decorators = [{ type: core_1.Pipe, args: [{ name: 'lowercase' }] }];
        return LowerCasePipe;
    }();
    exports.LowerCasePipe = LowerCasePipe;
    
});
System.registerDynamic("node_modules/@angular/common/src/facade/intl.js", [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    (function (NumberFormatStyle) {
        NumberFormatStyle[NumberFormatStyle["Decimal"] = 0] = "Decimal";
        NumberFormatStyle[NumberFormatStyle["Percent"] = 1] = "Percent";
        NumberFormatStyle[NumberFormatStyle["Currency"] = 2] = "Currency";
    })(exports.NumberFormatStyle || (exports.NumberFormatStyle = {}));
    var NumberFormatStyle = exports.NumberFormatStyle;
    var NumberFormatter = function () {
        function NumberFormatter() {}
        NumberFormatter.format = function (num, locale, style, _a) {
            var _b = _a === void 0 ? {} : _a,
                _c = _b.minimumIntegerDigits,
                minimumIntegerDigits = _c === void 0 ? 1 : _c,
                _d = _b.minimumFractionDigits,
                minimumFractionDigits = _d === void 0 ? 0 : _d,
                _e = _b.maximumFractionDigits,
                maximumFractionDigits = _e === void 0 ? 3 : _e,
                currency = _b.currency,
                _f = _b.currencyAsSymbol,
                currencyAsSymbol = _f === void 0 ? false : _f;
            var intlOptions = {
                minimumIntegerDigits: minimumIntegerDigits,
                minimumFractionDigits: minimumFractionDigits,
                maximumFractionDigits: maximumFractionDigits
            };
            intlOptions.style = NumberFormatStyle[style].toLowerCase();
            if (style == NumberFormatStyle.Currency) {
                intlOptions.currency = currency;
                intlOptions.currencyDisplay = currencyAsSymbol ? 'symbol' : 'code';
            }
            return new Intl.NumberFormat(locale, intlOptions).format(num);
        };
        return NumberFormatter;
    }();
    exports.NumberFormatter = NumberFormatter;
    var DATE_FORMATS_SPLIT = /((?:[^yMLdHhmsaZEwGjJ']+)|(?:'(?:[^']|'')*')|(?:E+|y+|M+|L+|d+|H+|h+|J+|j+|m+|s+|a|Z|G+|w+))(.*)/;
    var PATTERN_ALIASES = {
        yMMMdjms: datePartGetterFactory(combine([digitCondition('year', 1), nameCondition('month', 3), digitCondition('day', 1), digitCondition('hour', 1), digitCondition('minute', 1), digitCondition('second', 1)])),
        yMdjm: datePartGetterFactory(combine([digitCondition('year', 1), digitCondition('month', 1), digitCondition('day', 1), digitCondition('hour', 1), digitCondition('minute', 1)])),
        yMMMMEEEEd: datePartGetterFactory(combine([digitCondition('year', 1), nameCondition('month', 4), nameCondition('weekday', 4), digitCondition('day', 1)])),
        yMMMMd: datePartGetterFactory(combine([digitCondition('year', 1), nameCondition('month', 4), digitCondition('day', 1)])),
        yMMMd: datePartGetterFactory(combine([digitCondition('year', 1), nameCondition('month', 3), digitCondition('day', 1)])),
        yMd: datePartGetterFactory(combine([digitCondition('year', 1), digitCondition('month', 1), digitCondition('day', 1)])),
        jms: datePartGetterFactory(combine([digitCondition('hour', 1), digitCondition('second', 1), digitCondition('minute', 1)])),
        jm: datePartGetterFactory(combine([digitCondition('hour', 1), digitCondition('minute', 1)]))
    };
    var DATE_FORMATS = {
        yyyy: datePartGetterFactory(digitCondition('year', 4)),
        yy: datePartGetterFactory(digitCondition('year', 2)),
        y: datePartGetterFactory(digitCondition('year', 1)),
        MMMM: datePartGetterFactory(nameCondition('month', 4)),
        MMM: datePartGetterFactory(nameCondition('month', 3)),
        MM: datePartGetterFactory(digitCondition('month', 2)),
        M: datePartGetterFactory(digitCondition('month', 1)),
        LLLL: datePartGetterFactory(nameCondition('month', 4)),
        dd: datePartGetterFactory(digitCondition('day', 2)),
        d: datePartGetterFactory(digitCondition('day', 1)),
        HH: hourExtracter(datePartGetterFactory(hour12Modify(digitCondition('hour', 2), false))),
        H: hourExtracter(datePartGetterFactory(hour12Modify(digitCondition('hour', 1), false))),
        hh: hourExtracter(datePartGetterFactory(hour12Modify(digitCondition('hour', 2), true))),
        h: hourExtracter(datePartGetterFactory(hour12Modify(digitCondition('hour', 1), true))),
        jj: datePartGetterFactory(digitCondition('hour', 2)),
        j: datePartGetterFactory(digitCondition('hour', 1)),
        mm: digitModifier(datePartGetterFactory(digitCondition('minute', 2))),
        m: datePartGetterFactory(digitCondition('minute', 1)),
        ss: digitModifier(datePartGetterFactory(digitCondition('second', 2))),
        s: datePartGetterFactory(digitCondition('second', 1)),
        // while ISO 8601 requires fractions to be prefixed with `.` or `,`
        // we can be just safely rely on using `sss` since we currently don't support single or two digit
        // fractions
        sss: datePartGetterFactory(digitCondition('second', 3)),
        EEEE: datePartGetterFactory(nameCondition('weekday', 4)),
        EEE: datePartGetterFactory(nameCondition('weekday', 3)),
        EE: datePartGetterFactory(nameCondition('weekday', 2)),
        E: datePartGetterFactory(nameCondition('weekday', 1)),
        a: hourClockExtracter(datePartGetterFactory(hour12Modify(digitCondition('hour', 1), true))),
        Z: datePartGetterFactory({ timeZoneName: 'long' }),
        z: datePartGetterFactory({ timeZoneName: 'short' }),
        ww: datePartGetterFactory({}),
        // first Thursday of the year. not support ?
        w: datePartGetterFactory({}),
        // of the year not support ?
        G: datePartGetterFactory(nameCondition('era', 1)),
        GG: datePartGetterFactory(nameCondition('era', 2)),
        GGG: datePartGetterFactory(nameCondition('era', 3)),
        GGGG: datePartGetterFactory(nameCondition('era', 4))
    };
    function digitModifier(inner) {
        return function (date, locale) {
            var result = inner(date, locale);
            return result.length == 1 ? '0' + result : result;
        };
    }
    function hourClockExtracter(inner) {
        return function (date, locale) {
            var result = inner(date, locale);
            return result.split(' ')[1];
        };
    }
    function hourExtracter(inner) {
        return function (date, locale) {
            var result = inner(date, locale);
            return result.split(' ')[0];
        };
    }
    function hour12Modify(options, value) {
        options.hour12 = value;
        return options;
    }
    function digitCondition(prop, len) {
        var result = {};
        result[prop] = len == 2 ? '2-digit' : 'numeric';
        return result;
    }
    function nameCondition(prop, len) {
        var result = {};
        result[prop] = len < 4 ? 'short' : 'long';
        return result;
    }
    function combine(options) {
        var result = {};
        options.forEach(function (option) {
            Object.assign(result, option);
        });
        return result;
    }
    function datePartGetterFactory(ret) {
        return function (date, locale) {
            return new Intl.DateTimeFormat(locale, ret).format(date);
        };
    }
    var datePartsFormatterCache = new Map();
    function dateFormatter(format, date, locale) {
        var text = '';
        var match;
        var fn;
        var parts = [];
        if (PATTERN_ALIASES[format]) {
            return PATTERN_ALIASES[format](date, locale);
        }
        if (datePartsFormatterCache.has(format)) {
            parts = datePartsFormatterCache.get(format);
        } else {
            var matchs = DATE_FORMATS_SPLIT.exec(format);
            while (format) {
                match = DATE_FORMATS_SPLIT.exec(format);
                if (match) {
                    parts = concat(parts, match, 1);
                    format = parts.pop();
                } else {
                    parts.push(format);
                    format = null;
                }
            }
            datePartsFormatterCache.set(format, parts);
        }
        parts.forEach(function (part) {
            fn = DATE_FORMATS[part];
            text += fn ? fn(date, locale) : part === '\'\'' ? '\'' : part.replace(/(^'|'$)/g, '').replace(/''/g, '\'');
        });
        return text;
    }
    var slice = [].slice;
    function concat(array1 /** TODO #9100 */, array2 /** TODO #9100 */, index /** TODO #9100 */) {
        return array1.concat(slice.call(array2, index));
    }
    var DateFormatter = function () {
        function DateFormatter() {}
        DateFormatter.format = function (date, locale, pattern) {
            return dateFormatter(pattern, date, locale);
        };
        return DateFormatter;
    }();
    exports.DateFormatter = DateFormatter;
    
});
System.registerDynamic('node_modules/@angular/common/src/pipes/number_pipe.js', ['@angular/core', '../facade/exceptions', '../facade/intl', '../facade/lang', './invalid_pipe_argument_exception'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var exceptions_1 = $__require('../facade/exceptions');
    var intl_1 = $__require('../facade/intl');
    var lang_1 = $__require('../facade/lang');
    var invalid_pipe_argument_exception_1 = $__require('./invalid_pipe_argument_exception');
    var defaultLocale = 'en-US';
    var _NUMBER_FORMAT_REGEXP = /^(\d+)?\.((\d+)(\-(\d+))?)?$/g;
    /**
     * Internal function to format numbers used by Decimal, Percent and Date pipes.
     */
    function formatNumber(pipe, value, style, digits, currency, currencyAsSymbol) {
        if (currency === void 0) {
            currency = null;
        }
        if (currencyAsSymbol === void 0) {
            currencyAsSymbol = false;
        }
        if (lang_1.isBlank(value)) return null;
        if (!lang_1.isNumber(value)) {
            throw new invalid_pipe_argument_exception_1.InvalidPipeArgumentException(pipe, value);
        }
        var minInt = 1,
            minFraction = 0,
            maxFraction = 3;
        if (lang_1.isPresent(digits)) {
            var parts = lang_1.RegExpWrapper.firstMatch(_NUMBER_FORMAT_REGEXP, digits);
            if (lang_1.isBlank(parts)) {
                throw new exceptions_1.BaseException(digits + " is not a valid digit info for number pipes");
            }
            if (lang_1.isPresent(parts[1])) {
                minInt = lang_1.NumberWrapper.parseIntAutoRadix(parts[1]);
            }
            if (lang_1.isPresent(parts[3])) {
                minFraction = lang_1.NumberWrapper.parseIntAutoRadix(parts[3]);
            }
            if (lang_1.isPresent(parts[5])) {
                maxFraction = lang_1.NumberWrapper.parseIntAutoRadix(parts[5]);
            }
        }
        return intl_1.NumberFormatter.format(value, defaultLocale, style, {
            minimumIntegerDigits: minInt,
            minimumFractionDigits: minFraction,
            maximumFractionDigits: maxFraction,
            currency: currency,
            currencyAsSymbol: currencyAsSymbol
        });
    }
    var DecimalPipe = function () {
        function DecimalPipe() {}
        DecimalPipe.prototype.transform = function (value, digits) {
            if (digits === void 0) {
                digits = null;
            }
            return formatNumber(DecimalPipe, value, intl_1.NumberFormatStyle.Decimal, digits);
        };
        /** @nocollapse */
        DecimalPipe.decorators = [{ type: core_1.Pipe, args: [{ name: 'number' }] }];
        return DecimalPipe;
    }();
    exports.DecimalPipe = DecimalPipe;
    var PercentPipe = function () {
        function PercentPipe() {}
        PercentPipe.prototype.transform = function (value, digits) {
            if (digits === void 0) {
                digits = null;
            }
            return formatNumber(PercentPipe, value, intl_1.NumberFormatStyle.Percent, digits);
        };
        /** @nocollapse */
        PercentPipe.decorators = [{ type: core_1.Pipe, args: [{ name: 'percent' }] }];
        return PercentPipe;
    }();
    exports.PercentPipe = PercentPipe;
    var CurrencyPipe = function () {
        function CurrencyPipe() {}
        CurrencyPipe.prototype.transform = function (value, currencyCode, symbolDisplay, digits) {
            if (currencyCode === void 0) {
                currencyCode = 'USD';
            }
            if (symbolDisplay === void 0) {
                symbolDisplay = false;
            }
            if (digits === void 0) {
                digits = null;
            }
            return formatNumber(CurrencyPipe, value, intl_1.NumberFormatStyle.Currency, digits, currencyCode, symbolDisplay);
        };
        /** @nocollapse */
        CurrencyPipe.decorators = [{ type: core_1.Pipe, args: [{ name: 'currency' }] }];
        return CurrencyPipe;
    }();
    exports.CurrencyPipe = CurrencyPipe;
    
});
System.registerDynamic('node_modules/@angular/common/src/pipes/replace_pipe.js', ['@angular/core', '../facade/lang', './invalid_pipe_argument_exception'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../facade/lang');
    var invalid_pipe_argument_exception_1 = $__require('./invalid_pipe_argument_exception');
    var ReplacePipe = function () {
        function ReplacePipe() {}
        ReplacePipe.prototype.transform = function (value, pattern, replacement) {
            if (lang_1.isBlank(value)) {
                return value;
            }
            if (!this._supportedInput(value)) {
                throw new invalid_pipe_argument_exception_1.InvalidPipeArgumentException(ReplacePipe, value);
            }
            var input = value.toString();
            if (!this._supportedPattern(pattern)) {
                throw new invalid_pipe_argument_exception_1.InvalidPipeArgumentException(ReplacePipe, pattern);
            }
            if (!this._supportedReplacement(replacement)) {
                throw new invalid_pipe_argument_exception_1.InvalidPipeArgumentException(ReplacePipe, replacement);
            }
            if (lang_1.isFunction(replacement)) {
                var rgxPattern = lang_1.isString(pattern) ? lang_1.RegExpWrapper.create(pattern) : pattern;
                return lang_1.StringWrapper.replaceAllMapped(input, rgxPattern, replacement);
            }
            if (pattern instanceof RegExp) {
                // use the replaceAll variant
                return lang_1.StringWrapper.replaceAll(input, pattern, replacement);
            }
            return lang_1.StringWrapper.replace(input, pattern, replacement);
        };
        ReplacePipe.prototype._supportedInput = function (input) {
            return lang_1.isString(input) || lang_1.isNumber(input);
        };
        ReplacePipe.prototype._supportedPattern = function (pattern) {
            return lang_1.isString(pattern) || pattern instanceof RegExp;
        };
        ReplacePipe.prototype._supportedReplacement = function (replacement) {
            return lang_1.isString(replacement) || lang_1.isFunction(replacement);
        };
        /** @nocollapse */
        ReplacePipe.decorators = [{ type: core_1.Pipe, args: [{ name: 'replace' }] }];
        return ReplacePipe;
    }();
    exports.ReplacePipe = ReplacePipe;
    
});
System.registerDynamic('node_modules/@angular/common/src/pipes/slice_pipe.js', ['@angular/core', '../facade/collection', '../facade/lang', './invalid_pipe_argument_exception'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var collection_1 = $__require('../facade/collection');
    var lang_1 = $__require('../facade/lang');
    var invalid_pipe_argument_exception_1 = $__require('./invalid_pipe_argument_exception');
    var SlicePipe = function () {
        function SlicePipe() {}
        SlicePipe.prototype.transform = function (value, start, end) {
            if (end === void 0) {
                end = null;
            }
            if (lang_1.isBlank(value)) return value;
            if (!this.supports(value)) {
                throw new invalid_pipe_argument_exception_1.InvalidPipeArgumentException(SlicePipe, value);
            }
            if (lang_1.isString(value)) {
                return lang_1.StringWrapper.slice(value, start, end);
            }
            return collection_1.ListWrapper.slice(value, start, end);
        };
        SlicePipe.prototype.supports = function (obj) {
            return lang_1.isString(obj) || lang_1.isArray(obj);
        };
        /** @nocollapse */
        SlicePipe.decorators = [{ type: core_1.Pipe, args: [{ name: 'slice', pure: false }] }];
        return SlicePipe;
    }();
    exports.SlicePipe = SlicePipe;
    
});
System.registerDynamic('node_modules/@angular/common/src/pipes/invalid_pipe_argument_exception.js', ['../facade/exceptions', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    var InvalidPipeArgumentException = function (_super) {
        __extends(InvalidPipeArgumentException, _super);
        function InvalidPipeArgumentException(type, value) {
            _super.call(this, "Invalid argument '" + value + "' for pipe '" + lang_1.stringify(type) + "'");
        }
        return InvalidPipeArgumentException;
    }(exceptions_1.BaseException);
    exports.InvalidPipeArgumentException = InvalidPipeArgumentException;
    
});
System.registerDynamic('node_modules/@angular/common/src/pipes/uppercase_pipe.js', ['@angular/core', '../facade/lang', './invalid_pipe_argument_exception'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../facade/lang');
    var invalid_pipe_argument_exception_1 = $__require('./invalid_pipe_argument_exception');
    var UpperCasePipe = function () {
        function UpperCasePipe() {}
        UpperCasePipe.prototype.transform = function (value) {
            if (lang_1.isBlank(value)) return value;
            if (!lang_1.isString(value)) {
                throw new invalid_pipe_argument_exception_1.InvalidPipeArgumentException(UpperCasePipe, value);
            }
            return value.toUpperCase();
        };
        /** @nocollapse */
        UpperCasePipe.decorators = [{ type: core_1.Pipe, args: [{ name: 'uppercase' }] }];
        return UpperCasePipe;
    }();
    exports.UpperCasePipe = UpperCasePipe;
    
});
System.registerDynamic('node_modules/@angular/common/src/pipes.js', ['./pipes/async_pipe', './pipes/common_pipes', './pipes/date_pipe', './pipes/i18n_plural_pipe', './pipes/i18n_select_pipe', './pipes/json_pipe', './pipes/lowercase_pipe', './pipes/number_pipe', './pipes/replace_pipe', './pipes/slice_pipe', './pipes/uppercase_pipe'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * @module
   * @description
   * This module provides a set of common Pipes.
   */

  var global = this || self,
      GLOBAL = global;
  var async_pipe_1 = $__require('./pipes/async_pipe');
  exports.AsyncPipe = async_pipe_1.AsyncPipe;
  var common_pipes_1 = $__require('./pipes/common_pipes');
  exports.COMMON_PIPES = common_pipes_1.COMMON_PIPES;
  var date_pipe_1 = $__require('./pipes/date_pipe');
  exports.DatePipe = date_pipe_1.DatePipe;
  var i18n_plural_pipe_1 = $__require('./pipes/i18n_plural_pipe');
  exports.I18nPluralPipe = i18n_plural_pipe_1.I18nPluralPipe;
  var i18n_select_pipe_1 = $__require('./pipes/i18n_select_pipe');
  exports.I18nSelectPipe = i18n_select_pipe_1.I18nSelectPipe;
  var json_pipe_1 = $__require('./pipes/json_pipe');
  exports.JsonPipe = json_pipe_1.JsonPipe;
  var lowercase_pipe_1 = $__require('./pipes/lowercase_pipe');
  exports.LowerCasePipe = lowercase_pipe_1.LowerCasePipe;
  var number_pipe_1 = $__require('./pipes/number_pipe');
  exports.CurrencyPipe = number_pipe_1.CurrencyPipe;
  exports.DecimalPipe = number_pipe_1.DecimalPipe;
  exports.PercentPipe = number_pipe_1.PercentPipe;
  var replace_pipe_1 = $__require('./pipes/replace_pipe');
  exports.ReplacePipe = replace_pipe_1.ReplacePipe;
  var slice_pipe_1 = $__require('./pipes/slice_pipe');
  exports.SlicePipe = slice_pipe_1.SlicePipe;
  var uppercase_pipe_1 = $__require('./pipes/uppercase_pipe');
  exports.UpperCasePipe = uppercase_pipe_1.UpperCasePipe;
  
});
System.registerDynamic('node_modules/@angular/common/src/directives/core_directives.js', ['./ng_class', './ng_for', './ng_if', './ng_plural', './ng_style', './ng_switch', './ng_template_outlet'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var ng_class_1 = $__require('./ng_class');
  var ng_for_1 = $__require('./ng_for');
  var ng_if_1 = $__require('./ng_if');
  var ng_plural_1 = $__require('./ng_plural');
  var ng_style_1 = $__require('./ng_style');
  var ng_switch_1 = $__require('./ng_switch');
  var ng_template_outlet_1 = $__require('./ng_template_outlet');
  /**
   * A collection of Angular core directives that are likely to be used in each and every Angular
   * application.
   *
   * This collection can be used to quickly enumerate all the built-in directives in the `directives`
   * property of the `@Component` annotation.
   *
   * ### Example ([live demo](http://plnkr.co/edit/yakGwpCdUkg0qfzX5m8g?p=preview))
   *
   * Instead of writing:
   *
   * ```typescript
   * import {NgClass, NgIf, NgFor, NgSwitch, NgSwitchWhen, NgSwitchDefault} from '@angular/common';
   * import {OtherDirective} from './myDirectives';
   *
   * @Component({
   *   selector: 'my-component',
   *   templateUrl: 'myComponent.html',
   *   directives: [NgClass, NgIf, NgFor, NgSwitch, NgSwitchWhen, NgSwitchDefault, OtherDirective]
   * })
   * export class MyComponent {
   *   ...
   * }
   * ```
   * one could import all the core directives at once:
   *
   * ```typescript
   * import {CORE_DIRECTIVES} from '@angular/common';
   * import {OtherDirective} from './myDirectives';
   *
   * @Component({
   *   selector: 'my-component',
   *   templateUrl: 'myComponent.html',
   *   directives: [CORE_DIRECTIVES, OtherDirective]
   * })
   * export class MyComponent {
   *   ...
   * }
   * ```
   *
   * @stable
   */
  exports.CORE_DIRECTIVES = [ng_class_1.NgClass, ng_for_1.NgFor, ng_if_1.NgIf, ng_template_outlet_1.NgTemplateOutlet, ng_style_1.NgStyle, ng_switch_1.NgSwitch, ng_switch_1.NgSwitchCase, ng_switch_1.NgSwitchDefault, ng_plural_1.NgPlural, ng_plural_1.NgPluralCase];
  
});
System.registerDynamic('node_modules/@angular/common/src/directives/ng_class.js', ['@angular/core', '../facade/collection', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var collection_1 = $__require('../facade/collection');
    var lang_1 = $__require('../facade/lang');
    var NgClass = function () {
        function NgClass(_iterableDiffers, _keyValueDiffers, _ngEl, _renderer) {
            this._iterableDiffers = _iterableDiffers;
            this._keyValueDiffers = _keyValueDiffers;
            this._ngEl = _ngEl;
            this._renderer = _renderer;
            this._initialClasses = [];
        }
        Object.defineProperty(NgClass.prototype, "initialClasses", {
            set: function (v) {
                this._applyInitialClasses(true);
                this._initialClasses = lang_1.isPresent(v) && lang_1.isString(v) ? v.split(' ') : [];
                this._applyInitialClasses(false);
                this._applyClasses(this._rawClass, false);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgClass.prototype, "rawClass", {
            set: function (v) {
                this._cleanupClasses(this._rawClass);
                if (lang_1.isString(v)) {
                    v = v.split(' ');
                }
                this._rawClass = v;
                this._iterableDiffer = null;
                this._keyValueDiffer = null;
                if (lang_1.isPresent(v)) {
                    if (collection_1.isListLikeIterable(v)) {
                        this._iterableDiffer = this._iterableDiffers.find(v).create(null);
                    } else {
                        this._keyValueDiffer = this._keyValueDiffers.find(v).create(null);
                    }
                }
            },
            enumerable: true,
            configurable: true
        });
        NgClass.prototype.ngDoCheck = function () {
            if (lang_1.isPresent(this._iterableDiffer)) {
                var changes = this._iterableDiffer.diff(this._rawClass);
                if (lang_1.isPresent(changes)) {
                    this._applyIterableChanges(changes);
                }
            }
            if (lang_1.isPresent(this._keyValueDiffer)) {
                var changes = this._keyValueDiffer.diff(this._rawClass);
                if (lang_1.isPresent(changes)) {
                    this._applyKeyValueChanges(changes);
                }
            }
        };
        NgClass.prototype.ngOnDestroy = function () {
            this._cleanupClasses(this._rawClass);
        };
        NgClass.prototype._cleanupClasses = function (rawClassVal) {
            this._applyClasses(rawClassVal, true);
            this._applyInitialClasses(false);
        };
        NgClass.prototype._applyKeyValueChanges = function (changes) {
            var _this = this;
            changes.forEachAddedItem(function (record) {
                _this._toggleClass(record.key, record.currentValue);
            });
            changes.forEachChangedItem(function (record) {
                _this._toggleClass(record.key, record.currentValue);
            });
            changes.forEachRemovedItem(function (record) {
                if (record.previousValue) {
                    _this._toggleClass(record.key, false);
                }
            });
        };
        NgClass.prototype._applyIterableChanges = function (changes) {
            var _this = this;
            changes.forEachAddedItem(function (record) {
                _this._toggleClass(record.item, true);
            });
            changes.forEachRemovedItem(function (record) {
                _this._toggleClass(record.item, false);
            });
        };
        NgClass.prototype._applyInitialClasses = function (isCleanup) {
            var _this = this;
            this._initialClasses.forEach(function (className) {
                return _this._toggleClass(className, !isCleanup);
            });
        };
        NgClass.prototype._applyClasses = function (rawClassVal, isCleanup) {
            var _this = this;
            if (lang_1.isPresent(rawClassVal)) {
                if (lang_1.isArray(rawClassVal)) {
                    rawClassVal.forEach(function (className) {
                        return _this._toggleClass(className, !isCleanup);
                    });
                } else if (rawClassVal instanceof Set) {
                    rawClassVal.forEach(function (className) {
                        return _this._toggleClass(className, !isCleanup);
                    });
                } else {
                    collection_1.StringMapWrapper.forEach(rawClassVal, function (expVal, className) {
                        if (lang_1.isPresent(expVal)) _this._toggleClass(className, !isCleanup);
                    });
                }
            }
        };
        NgClass.prototype._toggleClass = function (className, enabled) {
            className = className.trim();
            if (className.length > 0) {
                if (className.indexOf(' ') > -1) {
                    var classes = className.split(/\s+/g);
                    for (var i = 0, len = classes.length; i < len; i++) {
                        this._renderer.setElementClass(this._ngEl.nativeElement, classes[i], enabled);
                    }
                } else {
                    this._renderer.setElementClass(this._ngEl.nativeElement, className, enabled);
                }
            }
        };
        /** @nocollapse */
        NgClass.decorators = [{ type: core_1.Directive, args: [{ selector: '[ngClass]', inputs: ['rawClass: ngClass', 'initialClasses: class'] }] }];
        /** @nocollapse */
        NgClass.ctorParameters = [{ type: core_1.IterableDiffers }, { type: core_1.KeyValueDiffers }, { type: core_1.ElementRef }, { type: core_1.Renderer }];
        return NgClass;
    }();
    exports.NgClass = NgClass;
    
});
System.registerDynamic('node_modules/@angular/common/src/directives/ng_for.js', ['@angular/core', '../facade/exceptions', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    var NgForRow = function () {
        function NgForRow($implicit, index, count) {
            this.$implicit = $implicit;
            this.index = index;
            this.count = count;
        }
        Object.defineProperty(NgForRow.prototype, "first", {
            get: function () {
                return this.index === 0;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgForRow.prototype, "last", {
            get: function () {
                return this.index === this.count - 1;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgForRow.prototype, "even", {
            get: function () {
                return this.index % 2 === 0;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgForRow.prototype, "odd", {
            get: function () {
                return !this.even;
            },
            enumerable: true,
            configurable: true
        });
        return NgForRow;
    }();
    exports.NgForRow = NgForRow;
    var NgFor = function () {
        function NgFor(_viewContainer, _templateRef, _iterableDiffers, _cdr) {
            this._viewContainer = _viewContainer;
            this._templateRef = _templateRef;
            this._iterableDiffers = _iterableDiffers;
            this._cdr = _cdr;
        }
        Object.defineProperty(NgFor.prototype, "ngForOf", {
            set: function (value) {
                this._ngForOf = value;
                if (lang_1.isBlank(this._differ) && lang_1.isPresent(value)) {
                    try {
                        this._differ = this._iterableDiffers.find(value).create(this._cdr, this._ngForTrackBy);
                    } catch (e) {
                        throw new exceptions_1.BaseException("Cannot find a differ supporting object '" + value + "' of type '" + lang_1.getTypeNameForDebugging(value) + "'. NgFor only supports binding to Iterables such as Arrays.");
                    }
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgFor.prototype, "ngForTemplate", {
            set: function (value) {
                if (lang_1.isPresent(value)) {
                    this._templateRef = value;
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgFor.prototype, "ngForTrackBy", {
            set: function (value) {
                this._ngForTrackBy = value;
            },
            enumerable: true,
            configurable: true
        });
        NgFor.prototype.ngDoCheck = function () {
            if (lang_1.isPresent(this._differ)) {
                var changes = this._differ.diff(this._ngForOf);
                if (lang_1.isPresent(changes)) this._applyChanges(changes);
            }
        };
        NgFor.prototype._applyChanges = function (changes) {
            var _this = this;
            // TODO(rado): check if change detection can produce a change record that is
            // easier to consume than current.
            var recordViewTuples = [];
            changes.forEachRemovedItem(function (removedRecord) {
                return recordViewTuples.push(new RecordViewTuple(removedRecord, null));
            });
            changes.forEachMovedItem(function (movedRecord) {
                return recordViewTuples.push(new RecordViewTuple(movedRecord, null));
            });
            var insertTuples = this._bulkRemove(recordViewTuples);
            changes.forEachAddedItem(function (addedRecord) {
                return insertTuples.push(new RecordViewTuple(addedRecord, null));
            });
            this._bulkInsert(insertTuples);
            for (var i = 0; i < insertTuples.length; i++) {
                this._perViewChange(insertTuples[i].view, insertTuples[i].record);
            }
            for (var i = 0, ilen = this._viewContainer.length; i < ilen; i++) {
                var viewRef = this._viewContainer.get(i);
                viewRef.context.index = i;
                viewRef.context.count = ilen;
            }
            changes.forEachIdentityChange(function (record /** TODO #9100 */) {
                var viewRef = _this._viewContainer.get(record.currentIndex);
                viewRef.context.$implicit = record.item;
            });
        };
        NgFor.prototype._perViewChange = function (view, record) {
            view.context.$implicit = record.item;
        };
        NgFor.prototype._bulkRemove = function (tuples) {
            tuples.sort(function (a, b) {
                return a.record.previousIndex - b.record.previousIndex;
            });
            var movedTuples = [];
            for (var i = tuples.length - 1; i >= 0; i--) {
                var tuple = tuples[i];
                // separate moved views from removed views.
                if (lang_1.isPresent(tuple.record.currentIndex)) {
                    tuple.view = this._viewContainer.detach(tuple.record.previousIndex);
                    movedTuples.push(tuple);
                } else {
                    this._viewContainer.remove(tuple.record.previousIndex);
                }
            }
            return movedTuples;
        };
        NgFor.prototype._bulkInsert = function (tuples) {
            tuples.sort(function (a, b) {
                return a.record.currentIndex - b.record.currentIndex;
            });
            for (var i = 0; i < tuples.length; i++) {
                var tuple = tuples[i];
                if (lang_1.isPresent(tuple.view)) {
                    this._viewContainer.insert(tuple.view, tuple.record.currentIndex);
                } else {
                    tuple.view = this._viewContainer.createEmbeddedView(this._templateRef, new NgForRow(null, null, null), tuple.record.currentIndex);
                }
            }
            return tuples;
        };
        /** @nocollapse */
        NgFor.decorators = [{ type: core_1.Directive, args: [{ selector: '[ngFor][ngForOf]', inputs: ['ngForTrackBy', 'ngForOf', 'ngForTemplate'] }] }];
        /** @nocollapse */
        NgFor.ctorParameters = [{ type: core_1.ViewContainerRef }, { type: core_1.TemplateRef }, { type: core_1.IterableDiffers }, { type: core_1.ChangeDetectorRef }];
        return NgFor;
    }();
    exports.NgFor = NgFor;
    var RecordViewTuple = function () {
        function RecordViewTuple(record, view) {
            this.record = record;
            this.view = view;
        }
        return RecordViewTuple;
    }();
    
});
System.registerDynamic('node_modules/@angular/common/src/directives/ng_if.js', ['@angular/core', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../facade/lang');
    var NgIf = function () {
        function NgIf(_viewContainer, _templateRef) {
            this._viewContainer = _viewContainer;
            this._templateRef = _templateRef;
            this._prevCondition = null;
        }
        Object.defineProperty(NgIf.prototype, "ngIf", {
            set: function (newCondition /* boolean */) {
                if (newCondition && (lang_1.isBlank(this._prevCondition) || !this._prevCondition)) {
                    this._prevCondition = true;
                    this._viewContainer.createEmbeddedView(this._templateRef);
                } else if (!newCondition && (lang_1.isBlank(this._prevCondition) || this._prevCondition)) {
                    this._prevCondition = false;
                    this._viewContainer.clear();
                }
            },
            enumerable: true,
            configurable: true
        });
        /** @nocollapse */
        NgIf.decorators = [{ type: core_1.Directive, args: [{ selector: '[ngIf]', inputs: ['ngIf'] }] }];
        /** @nocollapse */
        NgIf.ctorParameters = [{ type: core_1.ViewContainerRef }, { type: core_1.TemplateRef }];
        return NgIf;
    }();
    exports.NgIf = NgIf;
    
});
System.registerDynamic('node_modules/@angular/common/src/directives/ng_plural.js', ['@angular/core', '../facade/lang', '../localization', './ng_switch'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../facade/lang');
    var localization_1 = $__require('../localization');
    var ng_switch_1 = $__require('./ng_switch');
    var NgPluralCase = function () {
        function NgPluralCase(value, template, viewContainer) {
            this.value = value;
            this._view = new ng_switch_1.SwitchView(viewContainer, template);
        }
        /** @nocollapse */
        NgPluralCase.decorators = [{ type: core_1.Directive, args: [{ selector: '[ngPluralCase]' }] }];
        /** @nocollapse */
        NgPluralCase.ctorParameters = [{ type: undefined, decorators: [{ type: core_1.Attribute, args: ['ngPluralCase'] }] }, { type: core_1.TemplateRef }, { type: core_1.ViewContainerRef }];
        return NgPluralCase;
    }();
    exports.NgPluralCase = NgPluralCase;
    var NgPlural = function () {
        function NgPlural(_localization) {
            this._localization = _localization;
            this._caseViews = {};
            this.cases = null;
        }
        Object.defineProperty(NgPlural.prototype, "ngPlural", {
            set: function (value) {
                this._switchValue = value;
                this._updateView();
            },
            enumerable: true,
            configurable: true
        });
        NgPlural.prototype.ngAfterContentInit = function () {
            var _this = this;
            this.cases.forEach(function (pluralCase) {
                _this._caseViews[pluralCase.value] = pluralCase._view;
            });
            this._updateView();
        };
        /** @internal */
        NgPlural.prototype._updateView = function () {
            this._clearViews();
            var key = localization_1.getPluralCategory(this._switchValue, Object.getOwnPropertyNames(this._caseViews), this._localization);
            this._activateView(this._caseViews[key]);
        };
        /** @internal */
        NgPlural.prototype._clearViews = function () {
            if (lang_1.isPresent(this._activeView)) this._activeView.destroy();
        };
        /** @internal */
        NgPlural.prototype._activateView = function (view) {
            if (!lang_1.isPresent(view)) return;
            this._activeView = view;
            this._activeView.create();
        };
        /** @nocollapse */
        NgPlural.decorators = [{ type: core_1.Directive, args: [{ selector: '[ngPlural]' }] }];
        /** @nocollapse */
        NgPlural.ctorParameters = [{ type: localization_1.NgLocalization }];
        /** @nocollapse */
        NgPlural.propDecorators = {
            'cases': [{ type: core_1.ContentChildren, args: [NgPluralCase] }],
            'ngPlural': [{ type: core_1.Input }]
        };
        return NgPlural;
    }();
    exports.NgPlural = NgPlural;
    
});
System.registerDynamic('node_modules/@angular/common/src/directives/ng_style.js', ['@angular/core', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../facade/lang');
    var NgStyle = function () {
        function NgStyle(_differs, _ngEl, _renderer) {
            this._differs = _differs;
            this._ngEl = _ngEl;
            this._renderer = _renderer;
        }
        Object.defineProperty(NgStyle.prototype, "rawStyle", {
            set: function (v) {
                this._rawStyle = v;
                if (lang_1.isBlank(this._differ) && lang_1.isPresent(v)) {
                    this._differ = this._differs.find(this._rawStyle).create(null);
                }
            },
            enumerable: true,
            configurable: true
        });
        NgStyle.prototype.ngDoCheck = function () {
            if (lang_1.isPresent(this._differ)) {
                var changes = this._differ.diff(this._rawStyle);
                if (lang_1.isPresent(changes)) {
                    this._applyChanges(changes);
                }
            }
        };
        NgStyle.prototype._applyChanges = function (changes) {
            var _this = this;
            changes.forEachAddedItem(function (record) {
                _this._setStyle(record.key, record.currentValue);
            });
            changes.forEachChangedItem(function (record) {
                _this._setStyle(record.key, record.currentValue);
            });
            changes.forEachRemovedItem(function (record) {
                _this._setStyle(record.key, null);
            });
        };
        NgStyle.prototype._setStyle = function (name, val) {
            this._renderer.setElementStyle(this._ngEl.nativeElement, name, val);
        };
        /** @nocollapse */
        NgStyle.decorators = [{ type: core_1.Directive, args: [{ selector: '[ngStyle]', inputs: ['rawStyle: ngStyle'] }] }];
        /** @nocollapse */
        NgStyle.ctorParameters = [{ type: core_1.KeyValueDiffers }, { type: core_1.ElementRef }, { type: core_1.Renderer }];
        return NgStyle;
    }();
    exports.NgStyle = NgStyle;
    
});
System.registerDynamic('node_modules/@angular/common/src/directives/ng_switch.js', ['@angular/core', '../facade/collection', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var collection_1 = $__require('../facade/collection');
    var lang_1 = $__require('../facade/lang');
    var _CASE_DEFAULT = new Object();
    // TODO: remove when fully deprecated
    var _warned = false;
    var SwitchView = function () {
        function SwitchView(_viewContainerRef, _templateRef) {
            this._viewContainerRef = _viewContainerRef;
            this._templateRef = _templateRef;
        }
        SwitchView.prototype.create = function () {
            this._viewContainerRef.createEmbeddedView(this._templateRef);
        };
        SwitchView.prototype.destroy = function () {
            this._viewContainerRef.clear();
        };
        return SwitchView;
    }();
    exports.SwitchView = SwitchView;
    var NgSwitch = function () {
        function NgSwitch() {
            this._useDefault = false;
            this._valueViews = new collection_1.Map();
            this._activeViews = [];
        }
        Object.defineProperty(NgSwitch.prototype, "ngSwitch", {
            set: function (value) {
                // Empty the currently active ViewContainers
                this._emptyAllActiveViews();
                // Add the ViewContainers matching the value (with a fallback to default)
                this._useDefault = false;
                var views = this._valueViews.get(value);
                if (lang_1.isBlank(views)) {
                    this._useDefault = true;
                    views = lang_1.normalizeBlank(this._valueViews.get(_CASE_DEFAULT));
                }
                this._activateViews(views);
                this._switchValue = value;
            },
            enumerable: true,
            configurable: true
        });
        /** @internal */
        NgSwitch.prototype._onCaseValueChanged = function (oldCase, newCase, view) {
            this._deregisterView(oldCase, view);
            this._registerView(newCase, view);
            if (oldCase === this._switchValue) {
                view.destroy();
                collection_1.ListWrapper.remove(this._activeViews, view);
            } else if (newCase === this._switchValue) {
                if (this._useDefault) {
                    this._useDefault = false;
                    this._emptyAllActiveViews();
                }
                view.create();
                this._activeViews.push(view);
            }
            // Switch to default when there is no more active ViewContainers
            if (this._activeViews.length === 0 && !this._useDefault) {
                this._useDefault = true;
                this._activateViews(this._valueViews.get(_CASE_DEFAULT));
            }
        };
        /** @internal */
        NgSwitch.prototype._emptyAllActiveViews = function () {
            var activeContainers = this._activeViews;
            for (var i = 0; i < activeContainers.length; i++) {
                activeContainers[i].destroy();
            }
            this._activeViews = [];
        };
        /** @internal */
        NgSwitch.prototype._activateViews = function (views) {
            // TODO(vicb): assert(this._activeViews.length === 0);
            if (lang_1.isPresent(views)) {
                for (var i = 0; i < views.length; i++) {
                    views[i].create();
                }
                this._activeViews = views;
            }
        };
        /** @internal */
        NgSwitch.prototype._registerView = function (value, view) {
            var views = this._valueViews.get(value);
            if (lang_1.isBlank(views)) {
                views = [];
                this._valueViews.set(value, views);
            }
            views.push(view);
        };
        /** @internal */
        NgSwitch.prototype._deregisterView = function (value, view) {
            // `_CASE_DEFAULT` is used a marker for non-registered cases
            if (value === _CASE_DEFAULT) return;
            var views = this._valueViews.get(value);
            if (views.length == 1) {
                this._valueViews.delete(value);
            } else {
                collection_1.ListWrapper.remove(views, view);
            }
        };
        /** @nocollapse */
        NgSwitch.decorators = [{ type: core_1.Directive, args: [{ selector: '[ngSwitch]', inputs: ['ngSwitch'] }] }];
        return NgSwitch;
    }();
    exports.NgSwitch = NgSwitch;
    var NgSwitchCase = function () {
        function NgSwitchCase(viewContainer, templateRef, ngSwitch) {
            // `_CASE_DEFAULT` is used as a marker for a not yet initialized value
            /** @internal */
            this._value = _CASE_DEFAULT;
            this._switch = ngSwitch;
            this._view = new SwitchView(viewContainer, templateRef);
        }
        Object.defineProperty(NgSwitchCase.prototype, "ngSwitchCase", {
            set: function (value) {
                this._switch._onCaseValueChanged(this._value, value, this._view);
                this._value = value;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgSwitchCase.prototype, "ngSwitchWhen", {
            set: function (value) {
                if (!_warned) {
                    _warned = true;
                    console.warn('*ngSwitchWhen is deprecated and will be removed. Use *ngSwitchCase instead');
                }
                this._switch._onCaseValueChanged(this._value, value, this._view);
                this._value = value;
            },
            enumerable: true,
            configurable: true
        });
        /** @nocollapse */
        NgSwitchCase.decorators = [{ type: core_1.Directive, args: [{ selector: '[ngSwitchCase],[ngSwitchWhen]', inputs: ['ngSwitchCase', 'ngSwitchWhen'] }] }];
        /** @nocollapse */
        NgSwitchCase.ctorParameters = [{ type: core_1.ViewContainerRef }, { type: core_1.TemplateRef }, { type: NgSwitch, decorators: [{ type: core_1.Host }] }];
        return NgSwitchCase;
    }();
    exports.NgSwitchCase = NgSwitchCase;
    var NgSwitchDefault = function () {
        function NgSwitchDefault(viewContainer, templateRef, sswitch) {
            sswitch._registerView(_CASE_DEFAULT, new SwitchView(viewContainer, templateRef));
        }
        /** @nocollapse */
        NgSwitchDefault.decorators = [{ type: core_1.Directive, args: [{ selector: '[ngSwitchDefault]' }] }];
        /** @nocollapse */
        NgSwitchDefault.ctorParameters = [{ type: core_1.ViewContainerRef }, { type: core_1.TemplateRef }, { type: NgSwitch, decorators: [{ type: core_1.Host }] }];
        return NgSwitchDefault;
    }();
    exports.NgSwitchDefault = NgSwitchDefault;
    
});
System.registerDynamic('node_modules/@angular/common/src/directives/ng_template_outlet.js', ['@angular/core', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../facade/lang');
    var NgTemplateOutlet = function () {
        function NgTemplateOutlet(_viewContainerRef) {
            this._viewContainerRef = _viewContainerRef;
        }
        Object.defineProperty(NgTemplateOutlet.prototype, "ngOutletContext", {
            set: function (context) {
                if (this._context !== context) {
                    this._context = context;
                    if (lang_1.isPresent(this._viewRef)) {
                        this.createView();
                    }
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgTemplateOutlet.prototype, "ngTemplateOutlet", {
            set: function (templateRef) {
                if (this._templateRef !== templateRef) {
                    this._templateRef = templateRef;
                    this.createView();
                }
            },
            enumerable: true,
            configurable: true
        });
        NgTemplateOutlet.prototype.createView = function () {
            if (lang_1.isPresent(this._viewRef)) {
                this._viewContainerRef.remove(this._viewContainerRef.indexOf(this._viewRef));
            }
            if (lang_1.isPresent(this._templateRef)) {
                this._viewRef = this._viewContainerRef.createEmbeddedView(this._templateRef, this._context);
            }
        };
        /** @nocollapse */
        NgTemplateOutlet.decorators = [{ type: core_1.Directive, args: [{ selector: '[ngTemplateOutlet]' }] }];
        /** @nocollapse */
        NgTemplateOutlet.ctorParameters = [{ type: core_1.ViewContainerRef }];
        /** @nocollapse */
        NgTemplateOutlet.propDecorators = {
            'ngOutletContext': [{ type: core_1.Input }],
            'ngTemplateOutlet': [{ type: core_1.Input }]
        };
        return NgTemplateOutlet;
    }();
    exports.NgTemplateOutlet = NgTemplateOutlet;
    
});
System.registerDynamic('node_modules/@angular/common/src/directives.js', ['./directives/core_directives', './directives/ng_class', './directives/ng_for', './directives/ng_if', './directives/ng_plural', './directives/ng_style', './directives/ng_switch', './directives/ng_template_outlet'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * @module
   * @description
   * Common directives shipped with Angular.
   */

  var global = this || self,
      GLOBAL = global;
  var core_directives_1 = $__require('./directives/core_directives');
  exports.CORE_DIRECTIVES = core_directives_1.CORE_DIRECTIVES;
  var ng_class_1 = $__require('./directives/ng_class');
  exports.NgClass = ng_class_1.NgClass;
  var ng_for_1 = $__require('./directives/ng_for');
  exports.NgFor = ng_for_1.NgFor;
  var ng_if_1 = $__require('./directives/ng_if');
  exports.NgIf = ng_if_1.NgIf;
  var ng_plural_1 = $__require('./directives/ng_plural');
  exports.NgPlural = ng_plural_1.NgPlural;
  exports.NgPluralCase = ng_plural_1.NgPluralCase;
  var ng_style_1 = $__require('./directives/ng_style');
  exports.NgStyle = ng_style_1.NgStyle;
  var ng_switch_1 = $__require('./directives/ng_switch');
  exports.NgSwitch = ng_switch_1.NgSwitch;
  exports.NgSwitchCase = ng_switch_1.NgSwitchCase;
  exports.NgSwitchDefault = ng_switch_1.NgSwitchDefault;
  var ng_template_outlet_1 = $__require('./directives/ng_template_outlet');
  exports.NgTemplateOutlet = ng_template_outlet_1.NgTemplateOutlet;
  
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/form_builder.js', ['@angular/core', '../facade/collection', '../facade/lang', './model'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var collection_1 = $__require('../facade/collection');
    var lang_1 = $__require('../facade/lang');
    var model_1 = $__require('./model');
    var FormBuilder = function () {
        function FormBuilder() {}
        /**
         * Construct a new {@link ControlGroup} with the given map of configuration.
         * Valid keys for the `extra` parameter map are `optionals` and `validator`.
         *
         * See the {@link ControlGroup} constructor for more details.
         */
        FormBuilder.prototype.group = function (controlsConfig, extra) {
            if (extra === void 0) {
                extra = null;
            }
            var controls = this._reduceControls(controlsConfig);
            var optionals = lang_1.isPresent(extra) ? collection_1.StringMapWrapper.get(extra, 'optionals') : null;
            var validator = lang_1.isPresent(extra) ? collection_1.StringMapWrapper.get(extra, 'validator') : null;
            var asyncValidator = lang_1.isPresent(extra) ? collection_1.StringMapWrapper.get(extra, 'asyncValidator') : null;
            return new model_1.ControlGroup(controls, optionals, validator, asyncValidator);
        };
        /**
         * Construct a new {@link Control} with the given `value`,`validator`, and `asyncValidator`.
         */
        FormBuilder.prototype.control = function (value, validator, asyncValidator) {
            if (validator === void 0) {
                validator = null;
            }
            if (asyncValidator === void 0) {
                asyncValidator = null;
            }
            return new model_1.Control(value, validator, asyncValidator);
        };
        /**
         * Construct an array of {@link Control}s from the given `controlsConfig` array of
         * configuration, with the given optional `validator` and `asyncValidator`.
         */
        FormBuilder.prototype.array = function (controlsConfig, validator, asyncValidator) {
            var _this = this;
            if (validator === void 0) {
                validator = null;
            }
            if (asyncValidator === void 0) {
                asyncValidator = null;
            }
            var controls = controlsConfig.map(function (c) {
                return _this._createControl(c);
            });
            return new model_1.ControlArray(controls, validator, asyncValidator);
        };
        /** @internal */
        FormBuilder.prototype._reduceControls = function (controlsConfig) {
            var _this = this;
            var controls = {};
            collection_1.StringMapWrapper.forEach(controlsConfig, function (controlConfig, controlName) {
                controls[controlName] = _this._createControl(controlConfig);
            });
            return controls;
        };
        /** @internal */
        FormBuilder.prototype._createControl = function (controlConfig) {
            if (controlConfig instanceof model_1.Control || controlConfig instanceof model_1.ControlGroup || controlConfig instanceof model_1.ControlArray) {
                return controlConfig;
            } else if (lang_1.isArray(controlConfig)) {
                var value = controlConfig[0];
                var validator = controlConfig.length > 1 ? controlConfig[1] : null;
                var asyncValidator = controlConfig.length > 2 ? controlConfig[2] : null;
                return this.control(value, validator, asyncValidator);
            } else {
                return this.control(controlConfig);
            }
        };
        /** @nocollapse */
        FormBuilder.decorators = [{ type: core_1.Injectable }];
        return FormBuilder;
    }();
    exports.FormBuilder = FormBuilder;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives.js', ['./directives/checkbox_value_accessor', './directives/default_value_accessor', './directives/ng_control_group', './directives/ng_control_name', './directives/ng_control_status', './directives/ng_form', './directives/ng_form_control', './directives/ng_form_model', './directives/ng_model', './directives/number_value_accessor', './directives/radio_control_value_accessor', './directives/select_control_value_accessor', './directives/select_multiple_control_value_accessor', './directives/validators', './directives/ng_control'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var checkbox_value_accessor_1 = $__require('./directives/checkbox_value_accessor');
  var default_value_accessor_1 = $__require('./directives/default_value_accessor');
  var ng_control_group_1 = $__require('./directives/ng_control_group');
  var ng_control_name_1 = $__require('./directives/ng_control_name');
  var ng_control_status_1 = $__require('./directives/ng_control_status');
  var ng_form_1 = $__require('./directives/ng_form');
  var ng_form_control_1 = $__require('./directives/ng_form_control');
  var ng_form_model_1 = $__require('./directives/ng_form_model');
  var ng_model_1 = $__require('./directives/ng_model');
  var number_value_accessor_1 = $__require('./directives/number_value_accessor');
  var radio_control_value_accessor_1 = $__require('./directives/radio_control_value_accessor');
  var select_control_value_accessor_1 = $__require('./directives/select_control_value_accessor');
  var select_multiple_control_value_accessor_1 = $__require('./directives/select_multiple_control_value_accessor');
  var validators_1 = $__require('./directives/validators');
  var checkbox_value_accessor_2 = $__require('./directives/checkbox_value_accessor');
  exports.CheckboxControlValueAccessor = checkbox_value_accessor_2.CheckboxControlValueAccessor;
  var default_value_accessor_2 = $__require('./directives/default_value_accessor');
  exports.DefaultValueAccessor = default_value_accessor_2.DefaultValueAccessor;
  var ng_control_1 = $__require('./directives/ng_control');
  exports.NgControl = ng_control_1.NgControl;
  var ng_control_group_2 = $__require('./directives/ng_control_group');
  exports.NgControlGroup = ng_control_group_2.NgControlGroup;
  var ng_control_name_2 = $__require('./directives/ng_control_name');
  exports.NgControlName = ng_control_name_2.NgControlName;
  var ng_control_status_2 = $__require('./directives/ng_control_status');
  exports.NgControlStatus = ng_control_status_2.NgControlStatus;
  var ng_form_2 = $__require('./directives/ng_form');
  exports.NgForm = ng_form_2.NgForm;
  var ng_form_control_2 = $__require('./directives/ng_form_control');
  exports.NgFormControl = ng_form_control_2.NgFormControl;
  var ng_form_model_2 = $__require('./directives/ng_form_model');
  exports.NgFormModel = ng_form_model_2.NgFormModel;
  var ng_model_2 = $__require('./directives/ng_model');
  exports.NgModel = ng_model_2.NgModel;
  var number_value_accessor_2 = $__require('./directives/number_value_accessor');
  exports.NumberValueAccessor = number_value_accessor_2.NumberValueAccessor;
  var radio_control_value_accessor_2 = $__require('./directives/radio_control_value_accessor');
  exports.RadioButtonState = radio_control_value_accessor_2.RadioButtonState;
  exports.RadioControlValueAccessor = radio_control_value_accessor_2.RadioControlValueAccessor;
  var select_control_value_accessor_2 = $__require('./directives/select_control_value_accessor');
  exports.NgSelectOption = select_control_value_accessor_2.NgSelectOption;
  exports.SelectControlValueAccessor = select_control_value_accessor_2.SelectControlValueAccessor;
  var select_multiple_control_value_accessor_2 = $__require('./directives/select_multiple_control_value_accessor');
  exports.NgSelectMultipleOption = select_multiple_control_value_accessor_2.NgSelectMultipleOption;
  exports.SelectMultipleControlValueAccessor = select_multiple_control_value_accessor_2.SelectMultipleControlValueAccessor;
  var validators_2 = $__require('./directives/validators');
  exports.MaxLengthValidator = validators_2.MaxLengthValidator;
  exports.MinLengthValidator = validators_2.MinLengthValidator;
  exports.PatternValidator = validators_2.PatternValidator;
  exports.RequiredValidator = validators_2.RequiredValidator;
  /**
   *
   * A list of all the form directives used as part of a `@Component` annotation.
   *
   *  This is a shorthand for importing them each individually.
   *
   * ### Example
   *
   * ```typescript
   * @Component({
   *   selector: 'my-app',
   *   directives: [FORM_DIRECTIVES]
   * })
   * class MyApp {}
   * ```
   * @experimental
   */
  exports.FORM_DIRECTIVES = [ng_control_name_1.NgControlName, ng_control_group_1.NgControlGroup, ng_form_control_1.NgFormControl, ng_model_1.NgModel, ng_form_model_1.NgFormModel, ng_form_1.NgForm, select_control_value_accessor_1.NgSelectOption, select_multiple_control_value_accessor_1.NgSelectMultipleOption, default_value_accessor_1.DefaultValueAccessor, number_value_accessor_1.NumberValueAccessor, checkbox_value_accessor_1.CheckboxControlValueAccessor, select_control_value_accessor_1.SelectControlValueAccessor, select_multiple_control_value_accessor_1.SelectMultipleControlValueAccessor, radio_control_value_accessor_1.RadioControlValueAccessor, ng_control_status_1.NgControlStatus, validators_1.RequiredValidator, validators_1.MinLengthValidator, validators_1.MaxLengthValidator, validators_1.PatternValidator];
  
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/ng_control_group.js', ['@angular/core', '../validators', './control_container', './shared'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var validators_1 = $__require('../validators');
    var control_container_1 = $__require('./control_container');
    var shared_1 = $__require('./shared');
    exports.controlGroupProvider =
    /*@ts2dart_const*/ /* @ts2dart_Provider */{
        provide: control_container_1.ControlContainer,
        useExisting: core_1.forwardRef(function () {
            return NgControlGroup;
        })
    };
    var NgControlGroup = function (_super) {
        __extends(NgControlGroup, _super);
        function NgControlGroup(parent, _validators, _asyncValidators) {
            _super.call(this);
            this._validators = _validators;
            this._asyncValidators = _asyncValidators;
            this._parent = parent;
        }
        NgControlGroup.prototype.ngOnInit = function () {
            this.formDirective.addControlGroup(this);
        };
        NgControlGroup.prototype.ngOnDestroy = function () {
            this.formDirective.removeControlGroup(this);
        };
        Object.defineProperty(NgControlGroup.prototype, "control", {
            /**
             * Get the {@link ControlGroup} backing this binding.
             */
            get: function () {
                return this.formDirective.getControlGroup(this);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgControlGroup.prototype, "path", {
            /**
             * Get the path to this control group.
             */
            get: function () {
                return shared_1.controlPath(this.name, this._parent);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgControlGroup.prototype, "formDirective", {
            /**
             * Get the {@link Form} to which this group belongs.
             */
            get: function () {
                return this._parent.formDirective;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgControlGroup.prototype, "validator", {
            get: function () {
                return shared_1.composeValidators(this._validators);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgControlGroup.prototype, "asyncValidator", {
            get: function () {
                return shared_1.composeAsyncValidators(this._asyncValidators);
            },
            enumerable: true,
            configurable: true
        });
        /** @nocollapse */
        NgControlGroup.decorators = [{ type: core_1.Directive, args: [{
                selector: '[ngControlGroup]',
                providers: [exports.controlGroupProvider],
                inputs: ['name: ngControlGroup'],
                exportAs: 'ngForm'
            }] }];
        /** @nocollapse */
        NgControlGroup.ctorParameters = [{ type: control_container_1.ControlContainer, decorators: [{ type: core_1.Host }, { type: core_1.SkipSelf }] }, { type: Array, decorators: [{ type: core_1.Optional }, { type: core_1.Self }, { type: core_1.Inject, args: [validators_1.NG_VALIDATORS] }] }, { type: Array, decorators: [{ type: core_1.Optional }, { type: core_1.Self }, { type: core_1.Inject, args: [validators_1.NG_ASYNC_VALIDATORS] }] }];
        return NgControlGroup;
    }(control_container_1.ControlContainer);
    exports.NgControlGroup = NgControlGroup;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/ng_control_name.js', ['@angular/core', '../../facade/async', '../validators', './control_container', './control_value_accessor', './ng_control', './shared'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var async_1 = $__require('../../facade/async');
    var validators_1 = $__require('../validators');
    var control_container_1 = $__require('./control_container');
    var control_value_accessor_1 = $__require('./control_value_accessor');
    var ng_control_1 = $__require('./ng_control');
    var shared_1 = $__require('./shared');
    exports.controlNameBinding =
    /*@ts2dart_const*/ /* @ts2dart_Provider */{
        provide: ng_control_1.NgControl,
        useExisting: core_1.forwardRef(function () {
            return NgControlName;
        })
    };
    var NgControlName = function (_super) {
        __extends(NgControlName, _super);
        function NgControlName(_parent, _validators, _asyncValidators, valueAccessors) {
            _super.call(this);
            this._parent = _parent;
            this._validators = _validators;
            this._asyncValidators = _asyncValidators;
            /** @internal */
            this.update = new async_1.EventEmitter();
            this._added = false;
            this.valueAccessor = shared_1.selectValueAccessor(this, valueAccessors);
        }
        NgControlName.prototype.ngOnChanges = function (changes) {
            if (!this._added) {
                this.formDirective.addControl(this);
                this._added = true;
            }
            if (shared_1.isPropertyUpdated(changes, this.viewModel)) {
                this.viewModel = this.model;
                this.formDirective.updateModel(this, this.model);
            }
        };
        NgControlName.prototype.ngOnDestroy = function () {
            this.formDirective.removeControl(this);
        };
        NgControlName.prototype.viewToModelUpdate = function (newValue) {
            this.viewModel = newValue;
            async_1.ObservableWrapper.callEmit(this.update, newValue);
        };
        Object.defineProperty(NgControlName.prototype, "path", {
            get: function () {
                return shared_1.controlPath(this.name, this._parent);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgControlName.prototype, "formDirective", {
            get: function () {
                return this._parent.formDirective;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgControlName.prototype, "validator", {
            get: function () {
                return shared_1.composeValidators(this._validators);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgControlName.prototype, "asyncValidator", {
            get: function () {
                return shared_1.composeAsyncValidators(this._asyncValidators);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgControlName.prototype, "control", {
            get: function () {
                return this.formDirective.getControl(this);
            },
            enumerable: true,
            configurable: true
        });
        /** @nocollapse */
        NgControlName.decorators = [{ type: core_1.Directive, args: [{
                selector: '[ngControl]',
                providers: [exports.controlNameBinding],
                inputs: ['name: ngControl', 'model: ngModel'],
                outputs: ['update: ngModelChange'],
                exportAs: 'ngForm'
            }] }];
        /** @nocollapse */
        NgControlName.ctorParameters = [{ type: control_container_1.ControlContainer, decorators: [{ type: core_1.Host }, { type: core_1.SkipSelf }] }, { type: Array, decorators: [{ type: core_1.Optional }, { type: core_1.Self }, { type: core_1.Inject, args: [validators_1.NG_VALIDATORS] }] }, { type: Array, decorators: [{ type: core_1.Optional }, { type: core_1.Self }, { type: core_1.Inject, args: [validators_1.NG_ASYNC_VALIDATORS] }] }, { type: Array, decorators: [{ type: core_1.Optional }, { type: core_1.Self }, { type: core_1.Inject, args: [control_value_accessor_1.NG_VALUE_ACCESSOR] }] }];
        return NgControlName;
    }(ng_control_1.NgControl);
    exports.NgControlName = NgControlName;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/ng_control_status.js', ['@angular/core', '../../facade/lang', './ng_control'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../../facade/lang');
    var ng_control_1 = $__require('./ng_control');
    var NgControlStatus = function () {
        function NgControlStatus(cd) {
            this._cd = cd;
        }
        Object.defineProperty(NgControlStatus.prototype, "ngClassUntouched", {
            get: function () {
                return lang_1.isPresent(this._cd.control) ? this._cd.control.untouched : false;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgControlStatus.prototype, "ngClassTouched", {
            get: function () {
                return lang_1.isPresent(this._cd.control) ? this._cd.control.touched : false;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgControlStatus.prototype, "ngClassPristine", {
            get: function () {
                return lang_1.isPresent(this._cd.control) ? this._cd.control.pristine : false;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgControlStatus.prototype, "ngClassDirty", {
            get: function () {
                return lang_1.isPresent(this._cd.control) ? this._cd.control.dirty : false;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgControlStatus.prototype, "ngClassValid", {
            get: function () {
                return lang_1.isPresent(this._cd.control) ? this._cd.control.valid : false;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgControlStatus.prototype, "ngClassInvalid", {
            get: function () {
                return lang_1.isPresent(this._cd.control) ? !this._cd.control.valid : false;
            },
            enumerable: true,
            configurable: true
        });
        /** @nocollapse */
        NgControlStatus.decorators = [{ type: core_1.Directive, args: [{
                selector: '[ngControl],[ngModel],[ngFormControl]',
                host: {
                    '[class.ng-untouched]': 'ngClassUntouched',
                    '[class.ng-touched]': 'ngClassTouched',
                    '[class.ng-pristine]': 'ngClassPristine',
                    '[class.ng-dirty]': 'ngClassDirty',
                    '[class.ng-valid]': 'ngClassValid',
                    '[class.ng-invalid]': 'ngClassInvalid'
                }
            }] }];
        /** @nocollapse */
        NgControlStatus.ctorParameters = [{ type: ng_control_1.NgControl, decorators: [{ type: core_1.Self }] }];
        return NgControlStatus;
    }();
    exports.NgControlStatus = NgControlStatus;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/ng_form.js', ['@angular/core', '../../facade/async', '../../facade/collection', '../../facade/lang', '../model', '../validators', './control_container', './shared'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var async_1 = $__require('../../facade/async');
    var collection_1 = $__require('../../facade/collection');
    var lang_1 = $__require('../../facade/lang');
    var model_1 = $__require('../model');
    var validators_1 = $__require('../validators');
    var control_container_1 = $__require('./control_container');
    var shared_1 = $__require('./shared');
    exports.formDirectiveProvider =
    /*@ts2dart_const*/{ provide: control_container_1.ControlContainer, useExisting: core_1.forwardRef(function () {
            return NgForm;
        }) };
    var _formWarningDisplayed = false;
    var NgForm = function (_super) {
        __extends(NgForm, _super);
        function NgForm(validators, asyncValidators) {
            _super.call(this);
            this._submitted = false;
            this.ngSubmit = new async_1.EventEmitter();
            this._displayWarning();
            this.form = new model_1.ControlGroup({}, null, shared_1.composeValidators(validators), shared_1.composeAsyncValidators(asyncValidators));
        }
        NgForm.prototype._displayWarning = function () {
            // TODO(kara): Update this when the new forms module becomes the default
            if (!_formWarningDisplayed) {
                _formWarningDisplayed = true;
                console.warn("\n      *It looks like you're using the old forms module. This will be opt-in in the next RC, and\n      will eventually be removed in favor of the new forms module. For more information, see:\n      https://docs.google.com/document/u/1/d/1RIezQqE4aEhBRmArIAS1mRIZtWFf6JxN_7B4meyWK0Y/pub\n    ");
            }
        };
        Object.defineProperty(NgForm.prototype, "submitted", {
            get: function () {
                return this._submitted;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgForm.prototype, "formDirective", {
            get: function () {
                return this;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgForm.prototype, "control", {
            get: function () {
                return this.form;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgForm.prototype, "path", {
            get: function () {
                return [];
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgForm.prototype, "controls", {
            get: function () {
                return this.form.controls;
            },
            enumerable: true,
            configurable: true
        });
        NgForm.prototype.addControl = function (dir) {
            var _this = this;
            async_1.PromiseWrapper.scheduleMicrotask(function () {
                var container = _this._findContainer(dir.path);
                var ctrl = new model_1.Control();
                shared_1.setUpControl(ctrl, dir);
                container.registerControl(dir.name, ctrl);
                ctrl.updateValueAndValidity({ emitEvent: false });
            });
        };
        NgForm.prototype.getControl = function (dir) {
            return this.form.find(dir.path);
        };
        NgForm.prototype.removeControl = function (dir) {
            var _this = this;
            async_1.PromiseWrapper.scheduleMicrotask(function () {
                var container = _this._findContainer(dir.path);
                if (lang_1.isPresent(container)) {
                    container.removeControl(dir.name);
                }
            });
        };
        NgForm.prototype.addControlGroup = function (dir) {
            var _this = this;
            async_1.PromiseWrapper.scheduleMicrotask(function () {
                var container = _this._findContainer(dir.path);
                var group = new model_1.ControlGroup({});
                shared_1.setUpControlGroup(group, dir);
                container.registerControl(dir.name, group);
                group.updateValueAndValidity({ emitEvent: false });
            });
        };
        NgForm.prototype.removeControlGroup = function (dir) {
            var _this = this;
            async_1.PromiseWrapper.scheduleMicrotask(function () {
                var container = _this._findContainer(dir.path);
                if (lang_1.isPresent(container)) {
                    container.removeControl(dir.name);
                }
            });
        };
        NgForm.prototype.getControlGroup = function (dir) {
            return this.form.find(dir.path);
        };
        NgForm.prototype.updateModel = function (dir, value) {
            var _this = this;
            async_1.PromiseWrapper.scheduleMicrotask(function () {
                var ctrl = _this.form.find(dir.path);
                ctrl.updateValue(value);
            });
        };
        NgForm.prototype.onSubmit = function () {
            this._submitted = true;
            async_1.ObservableWrapper.callEmit(this.ngSubmit, null);
            return false;
        };
        /** @internal */
        NgForm.prototype._findContainer = function (path) {
            path.pop();
            return collection_1.ListWrapper.isEmpty(path) ? this.form : this.form.find(path);
        };
        /** @nocollapse */
        NgForm.decorators = [{ type: core_1.Directive, args: [{
                selector: 'form:not([ngNoForm]):not([ngFormModel]),ngForm,[ngForm]',
                providers: [exports.formDirectiveProvider],
                host: {
                    '(submit)': 'onSubmit()'
                },
                outputs: ['ngSubmit'],
                exportAs: 'ngForm'
            }] }];
        /** @nocollapse */
        NgForm.ctorParameters = [{ type: Array, decorators: [{ type: core_1.Optional }, { type: core_1.Self }, { type: core_1.Inject, args: [validators_1.NG_VALIDATORS] }] }, { type: Array, decorators: [{ type: core_1.Optional }, { type: core_1.Self }, { type: core_1.Inject, args: [validators_1.NG_ASYNC_VALIDATORS] }] }];
        return NgForm;
    }(control_container_1.ControlContainer);
    exports.NgForm = NgForm;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/ng_form_control.js', ['@angular/core', '../../facade/async', '../../facade/collection', '../validators', './control_value_accessor', './ng_control', './shared'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var async_1 = $__require('../../facade/async');
    var collection_1 = $__require('../../facade/collection');
    var validators_1 = $__require('../validators');
    var control_value_accessor_1 = $__require('./control_value_accessor');
    var ng_control_1 = $__require('./ng_control');
    var shared_1 = $__require('./shared');
    exports.formControlBinding =
    /*@ts2dart_const*/ /* @ts2dart_Provider */{
        provide: ng_control_1.NgControl,
        useExisting: core_1.forwardRef(function () {
            return NgFormControl;
        })
    };
    var NgFormControl = function (_super) {
        __extends(NgFormControl, _super);
        function NgFormControl(_validators, _asyncValidators, valueAccessors) {
            _super.call(this);
            this._validators = _validators;
            this._asyncValidators = _asyncValidators;
            this.update = new async_1.EventEmitter();
            this.valueAccessor = shared_1.selectValueAccessor(this, valueAccessors);
        }
        NgFormControl.prototype.ngOnChanges = function (changes) {
            if (this._isControlChanged(changes)) {
                shared_1.setUpControl(this.form, this);
                this.form.updateValueAndValidity({ emitEvent: false });
            }
            if (shared_1.isPropertyUpdated(changes, this.viewModel)) {
                this.form.updateValue(this.model);
                this.viewModel = this.model;
            }
        };
        Object.defineProperty(NgFormControl.prototype, "path", {
            get: function () {
                return [];
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgFormControl.prototype, "validator", {
            get: function () {
                return shared_1.composeValidators(this._validators);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgFormControl.prototype, "asyncValidator", {
            get: function () {
                return shared_1.composeAsyncValidators(this._asyncValidators);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgFormControl.prototype, "control", {
            get: function () {
                return this.form;
            },
            enumerable: true,
            configurable: true
        });
        NgFormControl.prototype.viewToModelUpdate = function (newValue) {
            this.viewModel = newValue;
            async_1.ObservableWrapper.callEmit(this.update, newValue);
        };
        NgFormControl.prototype._isControlChanged = function (changes) {
            return collection_1.StringMapWrapper.contains(changes, 'form');
        };
        /** @nocollapse */
        NgFormControl.decorators = [{ type: core_1.Directive, args: [{
                selector: '[ngFormControl]',
                providers: [exports.formControlBinding],
                inputs: ['form: ngFormControl', 'model: ngModel'],
                outputs: ['update: ngModelChange'],
                exportAs: 'ngForm'
            }] }];
        /** @nocollapse */
        NgFormControl.ctorParameters = [{ type: Array, decorators: [{ type: core_1.Optional }, { type: core_1.Self }, { type: core_1.Inject, args: [validators_1.NG_VALIDATORS] }] }, { type: Array, decorators: [{ type: core_1.Optional }, { type: core_1.Self }, { type: core_1.Inject, args: [validators_1.NG_ASYNC_VALIDATORS] }] }, { type: Array, decorators: [{ type: core_1.Optional }, { type: core_1.Self }, { type: core_1.Inject, args: [control_value_accessor_1.NG_VALUE_ACCESSOR] }] }];
        return NgFormControl;
    }(ng_control_1.NgControl);
    exports.NgFormControl = NgFormControl;
    
});
System.registerDynamic("node_modules/@angular/common/src/forms-deprecated/directives/control_container.js", ["./abstract_control_directive"], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var abstract_control_directive_1 = $__require('./abstract_control_directive');
    /**
     * A directive that contains multiple {@link NgControl}s.
     *
     * Only used by the forms module.
     *
     * @experimental
     */
    var ControlContainer = function (_super) {
        __extends(ControlContainer, _super);
        function ControlContainer() {
            _super.apply(this, arguments);
        }
        Object.defineProperty(ControlContainer.prototype, "formDirective", {
            /**
             * Get the form to which this container belongs.
             */
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ControlContainer.prototype, "path", {
            /**
             * Get the path to this container.
             */
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        return ControlContainer;
    }(abstract_control_directive_1.AbstractControlDirective);
    exports.ControlContainer = ControlContainer;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/ng_form_model.js', ['@angular/core', '../../facade/async', '../../facade/collection', '../../facade/exceptions', '../../facade/lang', '../validators', './control_container', './shared'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var async_1 = $__require('../../facade/async');
    var collection_1 = $__require('../../facade/collection');
    var exceptions_1 = $__require('../../facade/exceptions');
    var lang_1 = $__require('../../facade/lang');
    var validators_1 = $__require('../validators');
    var control_container_1 = $__require('./control_container');
    var shared_1 = $__require('./shared');
    exports.formDirectiveProvider =
    /*@ts2dart_const*/ /* @ts2dart_Provider */{
        provide: control_container_1.ControlContainer,
        useExisting: core_1.forwardRef(function () {
            return NgFormModel;
        })
    };
    var _formModelWarningDisplayed = false;
    var NgFormModel = function (_super) {
        __extends(NgFormModel, _super);
        function NgFormModel(_validators, _asyncValidators) {
            _super.call(this);
            this._validators = _validators;
            this._asyncValidators = _asyncValidators;
            this._submitted = false;
            this.form = null;
            this.directives = [];
            this.ngSubmit = new async_1.EventEmitter();
            this._displayWarning();
        }
        NgFormModel.prototype._displayWarning = function () {
            // TODO(kara): Update this when the new forms module becomes the default
            if (!_formModelWarningDisplayed) {
                _formModelWarningDisplayed = true;
                console.warn("\n      *It looks like you're using the old forms module. This will be opt-in in the next RC, and\n      will eventually be removed in favor of the new forms module. For more information, see:\n      https://docs.google.com/document/u/1/d/1RIezQqE4aEhBRmArIAS1mRIZtWFf6JxN_7B4meyWK0Y/pub\n    ");
            }
        };
        NgFormModel.prototype.ngOnChanges = function (changes) {
            this._checkFormPresent();
            if (collection_1.StringMapWrapper.contains(changes, 'form')) {
                var sync = shared_1.composeValidators(this._validators);
                this.form.validator = validators_1.Validators.compose([this.form.validator, sync]);
                var async = shared_1.composeAsyncValidators(this._asyncValidators);
                this.form.asyncValidator = validators_1.Validators.composeAsync([this.form.asyncValidator, async]);
                this.form.updateValueAndValidity({ onlySelf: true, emitEvent: false });
            }
            this._updateDomValue();
        };
        Object.defineProperty(NgFormModel.prototype, "submitted", {
            get: function () {
                return this._submitted;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgFormModel.prototype, "formDirective", {
            get: function () {
                return this;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgFormModel.prototype, "control", {
            get: function () {
                return this.form;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgFormModel.prototype, "path", {
            get: function () {
                return [];
            },
            enumerable: true,
            configurable: true
        });
        NgFormModel.prototype.addControl = function (dir) {
            var ctrl = this.form.find(dir.path);
            shared_1.setUpControl(ctrl, dir);
            ctrl.updateValueAndValidity({ emitEvent: false });
            this.directives.push(dir);
        };
        NgFormModel.prototype.getControl = function (dir) {
            return this.form.find(dir.path);
        };
        NgFormModel.prototype.removeControl = function (dir) {
            collection_1.ListWrapper.remove(this.directives, dir);
        };
        NgFormModel.prototype.addControlGroup = function (dir) {
            var ctrl = this.form.find(dir.path);
            shared_1.setUpControlGroup(ctrl, dir);
            ctrl.updateValueAndValidity({ emitEvent: false });
        };
        NgFormModel.prototype.removeControlGroup = function (dir) {};
        NgFormModel.prototype.getControlGroup = function (dir) {
            return this.form.find(dir.path);
        };
        NgFormModel.prototype.updateModel = function (dir, value) {
            var ctrl = this.form.find(dir.path);
            ctrl.updateValue(value);
        };
        NgFormModel.prototype.onSubmit = function () {
            this._submitted = true;
            async_1.ObservableWrapper.callEmit(this.ngSubmit, null);
            return false;
        };
        /** @internal */
        NgFormModel.prototype._updateDomValue = function () {
            var _this = this;
            this.directives.forEach(function (dir) {
                var ctrl = _this.form.find(dir.path);
                dir.valueAccessor.writeValue(ctrl.value);
            });
        };
        NgFormModel.prototype._checkFormPresent = function () {
            if (lang_1.isBlank(this.form)) {
                throw new exceptions_1.BaseException("ngFormModel expects a form. Please pass one in. Example: <form [ngFormModel]=\"myCoolForm\">");
            }
        };
        /** @nocollapse */
        NgFormModel.decorators = [{ type: core_1.Directive, args: [{
                selector: '[ngFormModel]',
                providers: [exports.formDirectiveProvider],
                inputs: ['form: ngFormModel'],
                host: { '(submit)': 'onSubmit()' },
                outputs: ['ngSubmit'],
                exportAs: 'ngForm'
            }] }];
        /** @nocollapse */
        NgFormModel.ctorParameters = [{ type: Array, decorators: [{ type: core_1.Optional }, { type: core_1.Self }, { type: core_1.Inject, args: [validators_1.NG_VALIDATORS] }] }, { type: Array, decorators: [{ type: core_1.Optional }, { type: core_1.Self }, { type: core_1.Inject, args: [validators_1.NG_ASYNC_VALIDATORS] }] }];
        return NgFormModel;
    }(control_container_1.ControlContainer);
    exports.NgFormModel = NgFormModel;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/checkbox_value_accessor.js', ['@angular/core', './control_value_accessor'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var control_value_accessor_1 = $__require('./control_value_accessor');
    exports.CHECKBOX_VALUE_ACCESSOR = {
        provide: control_value_accessor_1.NG_VALUE_ACCESSOR,
        useExisting: core_1.forwardRef(function () {
            return CheckboxControlValueAccessor;
        }),
        multi: true
    };
    var CheckboxControlValueAccessor = function () {
        function CheckboxControlValueAccessor(_renderer, _elementRef) {
            this._renderer = _renderer;
            this._elementRef = _elementRef;
            this.onChange = function (_) {};
            this.onTouched = function () {};
        }
        CheckboxControlValueAccessor.prototype.writeValue = function (value) {
            this._renderer.setElementProperty(this._elementRef.nativeElement, 'checked', value);
        };
        CheckboxControlValueAccessor.prototype.registerOnChange = function (fn) {
            this.onChange = fn;
        };
        CheckboxControlValueAccessor.prototype.registerOnTouched = function (fn) {
            this.onTouched = fn;
        };
        /** @nocollapse */
        CheckboxControlValueAccessor.decorators = [{ type: core_1.Directive, args: [{
                selector: 'input[type=checkbox][ngControl],input[type=checkbox][ngFormControl],input[type=checkbox][ngModel]',
                host: { '(change)': 'onChange($event.target.checked)', '(blur)': 'onTouched()' },
                providers: [exports.CHECKBOX_VALUE_ACCESSOR]
            }] }];
        /** @nocollapse */
        CheckboxControlValueAccessor.ctorParameters = [{ type: core_1.Renderer }, { type: core_1.ElementRef }];
        return CheckboxControlValueAccessor;
    }();
    exports.CheckboxControlValueAccessor = CheckboxControlValueAccessor;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/default_value_accessor.js', ['@angular/core', '../../facade/lang', './control_value_accessor'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../../facade/lang');
    var control_value_accessor_1 = $__require('./control_value_accessor');
    exports.DEFAULT_VALUE_ACCESSOR =
    /* @ts2dart_Provider */{
        provide: control_value_accessor_1.NG_VALUE_ACCESSOR,
        useExisting: core_1.forwardRef(function () {
            return DefaultValueAccessor;
        }),
        multi: true
    };
    var DefaultValueAccessor = function () {
        function DefaultValueAccessor(_renderer, _elementRef) {
            this._renderer = _renderer;
            this._elementRef = _elementRef;
            this.onChange = function (_) {};
            this.onTouched = function () {};
        }
        DefaultValueAccessor.prototype.writeValue = function (value) {
            var normalizedValue = lang_1.isBlank(value) ? '' : value;
            this._renderer.setElementProperty(this._elementRef.nativeElement, 'value', normalizedValue);
        };
        DefaultValueAccessor.prototype.registerOnChange = function (fn) {
            this.onChange = fn;
        };
        DefaultValueAccessor.prototype.registerOnTouched = function (fn) {
            this.onTouched = fn;
        };
        /** @nocollapse */
        DefaultValueAccessor.decorators = [{ type: core_1.Directive, args: [{
                selector: 'input:not([type=checkbox])[ngControl],textarea[ngControl],input:not([type=checkbox])[ngFormControl],textarea[ngFormControl],input:not([type=checkbox])[ngModel],textarea[ngModel],[ngDefaultControl]',
                // TODO: vsavkin replace the above selector with the one below it once
                // https://github.com/angular/angular/issues/3011 is implemented
                // selector: '[ngControl],[ngModel],[ngFormControl]',
                host: { '(input)': 'onChange($event.target.value)', '(blur)': 'onTouched()' },
                providers: [exports.DEFAULT_VALUE_ACCESSOR]
            }] }];
        /** @nocollapse */
        DefaultValueAccessor.ctorParameters = [{ type: core_1.Renderer }, { type: core_1.ElementRef }];
        return DefaultValueAccessor;
    }();
    exports.DefaultValueAccessor = DefaultValueAccessor;
    
});
System.registerDynamic("node_modules/@angular/common/src/forms-deprecated/directives/normalize_validator.js", [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    function normalizeValidator(validator) {
        if (validator.validate !== undefined) {
            return function (c) {
                return validator.validate(c);
            };
        } else {
            return validator;
        }
    }
    exports.normalizeValidator = normalizeValidator;
    function normalizeAsyncValidator(validator) {
        if (validator.validate !== undefined) {
            return function (c) {
                return validator.validate(c);
            };
        } else {
            return validator;
        }
    }
    exports.normalizeAsyncValidator = normalizeAsyncValidator;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/number_value_accessor.js', ['@angular/core', '../../facade/lang', './control_value_accessor'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../../facade/lang');
    var control_value_accessor_1 = $__require('./control_value_accessor');
    exports.NUMBER_VALUE_ACCESSOR = {
        provide: control_value_accessor_1.NG_VALUE_ACCESSOR,
        useExisting: core_1.forwardRef(function () {
            return NumberValueAccessor;
        }),
        multi: true
    };
    var NumberValueAccessor = function () {
        function NumberValueAccessor(_renderer, _elementRef) {
            this._renderer = _renderer;
            this._elementRef = _elementRef;
            this.onChange = function (_) {};
            this.onTouched = function () {};
        }
        NumberValueAccessor.prototype.writeValue = function (value) {
            this._renderer.setElementProperty(this._elementRef.nativeElement, 'value', value);
        };
        NumberValueAccessor.prototype.registerOnChange = function (fn) {
            this.onChange = function (value) {
                fn(value == '' ? null : lang_1.NumberWrapper.parseFloat(value));
            };
        };
        NumberValueAccessor.prototype.registerOnTouched = function (fn) {
            this.onTouched = fn;
        };
        /** @nocollapse */
        NumberValueAccessor.decorators = [{ type: core_1.Directive, args: [{
                selector: 'input[type=number][ngControl],input[type=number][ngFormControl],input[type=number][ngModel]',
                host: {
                    '(change)': 'onChange($event.target.value)',
                    '(input)': 'onChange($event.target.value)',
                    '(blur)': 'onTouched()'
                },
                providers: [exports.NUMBER_VALUE_ACCESSOR]
            }] }];
        /** @nocollapse */
        NumberValueAccessor.ctorParameters = [{ type: core_1.Renderer }, { type: core_1.ElementRef }];
        return NumberValueAccessor;
    }();
    exports.NumberValueAccessor = NumberValueAccessor;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/abstract_control_directive.js', ['../../facade/exceptions', '../../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var exceptions_1 = $__require('../../facade/exceptions');
    var lang_1 = $__require('../../facade/lang');
    /**
     * Base class for control directives.
     *
     * Only used internally in the forms module.
     *
     * @experimental
     */
    var AbstractControlDirective = function () {
        function AbstractControlDirective() {}
        Object.defineProperty(AbstractControlDirective.prototype, "control", {
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControlDirective.prototype, "value", {
            get: function () {
                return lang_1.isPresent(this.control) ? this.control.value : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControlDirective.prototype, "valid", {
            get: function () {
                return lang_1.isPresent(this.control) ? this.control.valid : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControlDirective.prototype, "errors", {
            get: function () {
                return lang_1.isPresent(this.control) ? this.control.errors : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControlDirective.prototype, "pristine", {
            get: function () {
                return lang_1.isPresent(this.control) ? this.control.pristine : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControlDirective.prototype, "dirty", {
            get: function () {
                return lang_1.isPresent(this.control) ? this.control.dirty : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControlDirective.prototype, "touched", {
            get: function () {
                return lang_1.isPresent(this.control) ? this.control.touched : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControlDirective.prototype, "untouched", {
            get: function () {
                return lang_1.isPresent(this.control) ? this.control.untouched : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControlDirective.prototype, "path", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        return AbstractControlDirective;
    }();
    exports.AbstractControlDirective = AbstractControlDirective;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/ng_control.js', ['../../facade/exceptions', './abstract_control_directive'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var exceptions_1 = $__require('../../facade/exceptions');
    var abstract_control_directive_1 = $__require('./abstract_control_directive');
    /**
     * A base class that all control directive extend.
     * It binds a {@link Control} object to a DOM element.
     *
     * Used internally by Angular forms.
     *
     * @experimental
     */
    var NgControl = function (_super) {
        __extends(NgControl, _super);
        function NgControl() {
            _super.apply(this, arguments);
            this.name = null;
            this.valueAccessor = null;
        }
        Object.defineProperty(NgControl.prototype, "validator", {
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgControl.prototype, "asyncValidator", {
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        return NgControl;
    }(abstract_control_directive_1.AbstractControlDirective);
    exports.NgControl = NgControl;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/radio_control_value_accessor.js', ['@angular/core', '../../facade/collection', '../../facade/lang', './control_value_accessor', './ng_control'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var collection_1 = $__require('../../facade/collection');
    var lang_1 = $__require('../../facade/lang');
    var control_value_accessor_1 = $__require('./control_value_accessor');
    var ng_control_1 = $__require('./ng_control');
    exports.RADIO_VALUE_ACCESSOR = {
        provide: control_value_accessor_1.NG_VALUE_ACCESSOR,
        useExisting: core_1.forwardRef(function () {
            return RadioControlValueAccessor;
        }),
        multi: true
    };
    var RadioControlRegistry = function () {
        function RadioControlRegistry() {
            this._accessors = [];
        }
        RadioControlRegistry.prototype.add = function (control, accessor) {
            this._accessors.push([control, accessor]);
        };
        RadioControlRegistry.prototype.remove = function (accessor) {
            var indexToRemove = -1;
            for (var i = 0; i < this._accessors.length; ++i) {
                if (this._accessors[i][1] === accessor) {
                    indexToRemove = i;
                }
            }
            collection_1.ListWrapper.removeAt(this._accessors, indexToRemove);
        };
        RadioControlRegistry.prototype.select = function (accessor) {
            var _this = this;
            this._accessors.forEach(function (c) {
                if (_this._isSameGroup(c, accessor) && c[1] !== accessor) {
                    c[1].fireUncheck();
                }
            });
        };
        RadioControlRegistry.prototype._isSameGroup = function (controlPair, accessor) {
            return controlPair[0].control.root === accessor._control.control.root && controlPair[1].name === accessor.name;
        };
        /** @nocollapse */
        RadioControlRegistry.decorators = [{ type: core_1.Injectable }];
        return RadioControlRegistry;
    }();
    exports.RadioControlRegistry = RadioControlRegistry;
    /**
     * The value provided by the forms API for radio buttons.
     *
     * @experimental
     */
    var RadioButtonState = function () {
        function RadioButtonState(checked, value) {
            this.checked = checked;
            this.value = value;
        }
        return RadioButtonState;
    }();
    exports.RadioButtonState = RadioButtonState;
    var RadioControlValueAccessor = function () {
        function RadioControlValueAccessor(_renderer, _elementRef, _registry, _injector) {
            this._renderer = _renderer;
            this._elementRef = _elementRef;
            this._registry = _registry;
            this._injector = _injector;
            this.onChange = function () {};
            this.onTouched = function () {};
        }
        RadioControlValueAccessor.prototype.ngOnInit = function () {
            this._control = this._injector.get(ng_control_1.NgControl);
            this._registry.add(this._control, this);
        };
        RadioControlValueAccessor.prototype.ngOnDestroy = function () {
            this._registry.remove(this);
        };
        RadioControlValueAccessor.prototype.writeValue = function (value) {
            this._state = value;
            if (lang_1.isPresent(value) && value.checked) {
                this._renderer.setElementProperty(this._elementRef.nativeElement, 'checked', true);
            }
        };
        RadioControlValueAccessor.prototype.registerOnChange = function (fn) {
            var _this = this;
            this._fn = fn;
            this.onChange = function () {
                fn(new RadioButtonState(true, _this._state.value));
                _this._registry.select(_this);
            };
        };
        RadioControlValueAccessor.prototype.fireUncheck = function () {
            this._fn(new RadioButtonState(false, this._state.value));
        };
        RadioControlValueAccessor.prototype.registerOnTouched = function (fn) {
            this.onTouched = fn;
        };
        /** @nocollapse */
        RadioControlValueAccessor.decorators = [{ type: core_1.Directive, args: [{
                selector: 'input[type=radio][ngControl],input[type=radio][ngFormControl],input[type=radio][ngModel]',
                host: { '(change)': 'onChange()', '(blur)': 'onTouched()' },
                providers: [exports.RADIO_VALUE_ACCESSOR]
            }] }];
        /** @nocollapse */
        RadioControlValueAccessor.ctorParameters = [{ type: core_1.Renderer }, { type: core_1.ElementRef }, { type: RadioControlRegistry }, { type: core_1.Injector }];
        /** @nocollapse */
        RadioControlValueAccessor.propDecorators = {
            'name': [{ type: core_1.Input }]
        };
        return RadioControlValueAccessor;
    }();
    exports.RadioControlValueAccessor = RadioControlValueAccessor;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/select_multiple_control_value_accessor.js', ['@angular/core', '../../facade/collection', '../../facade/lang', './control_value_accessor'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var collection_1 = $__require('../../facade/collection');
    var lang_1 = $__require('../../facade/lang');
    var control_value_accessor_1 = $__require('./control_value_accessor');
    var SELECT_MULTIPLE_VALUE_ACCESSOR = {
        provide: control_value_accessor_1.NG_VALUE_ACCESSOR,
        useExisting: core_1.forwardRef(function () {
            return SelectMultipleControlValueAccessor;
        }),
        multi: true
    };
    function _buildValueString(id, value) {
        if (lang_1.isBlank(id)) return "" + value;
        if (lang_1.isString(value)) value = "'" + value + "'";
        if (!lang_1.isPrimitive(value)) value = 'Object';
        return lang_1.StringWrapper.slice(id + ": " + value, 0, 50);
    }
    function _extractId(valueString) {
        return valueString.split(':')[0];
    }
    /** Mock interface for HTMLCollection */
    var HTMLCollection = function () {
        function HTMLCollection() {}
        return HTMLCollection;
    }();
    var SelectMultipleControlValueAccessor = function () {
        function SelectMultipleControlValueAccessor() {
            /** @internal */
            this._optionMap = new Map();
            /** @internal */
            this._idCounter = 0;
            this.onChange = function (_) {};
            this.onTouched = function () {};
        }
        SelectMultipleControlValueAccessor.prototype.writeValue = function (value) {
            var _this = this;
            this.value = value;
            if (value == null) return;
            var values = value;
            // convert values to ids
            var ids = values.map(function (v) {
                return _this._getOptionId(v);
            });
            this._optionMap.forEach(function (opt, o) {
                opt._setSelected(ids.indexOf(o.toString()) > -1);
            });
        };
        SelectMultipleControlValueAccessor.prototype.registerOnChange = function (fn) {
            var _this = this;
            this.onChange = function (_) {
                var selected = [];
                if (_.hasOwnProperty('selectedOptions')) {
                    var options = _.selectedOptions;
                    for (var i = 0; i < options.length; i++) {
                        var opt = options.item(i);
                        var val = _this._getOptionValue(opt.value);
                        selected.push(val);
                    }
                } else {
                    var options = _.options;
                    for (var i = 0; i < options.length; i++) {
                        var opt = options.item(i);
                        if (opt.selected) {
                            var val = _this._getOptionValue(opt.value);
                            selected.push(val);
                        }
                    }
                }
                fn(selected);
            };
        };
        SelectMultipleControlValueAccessor.prototype.registerOnTouched = function (fn) {
            this.onTouched = fn;
        };
        /** @internal */
        SelectMultipleControlValueAccessor.prototype._registerOption = function (value) {
            var id = (this._idCounter++).toString();
            this._optionMap.set(id, value);
            return id;
        };
        /** @internal */
        SelectMultipleControlValueAccessor.prototype._getOptionId = function (value) {
            for (var _i = 0, _a = collection_1.MapWrapper.keys(this._optionMap); _i < _a.length; _i++) {
                var id = _a[_i];
                if (lang_1.looseIdentical(this._optionMap.get(id)._value, value)) return id;
            }
            return null;
        };
        /** @internal */
        SelectMultipleControlValueAccessor.prototype._getOptionValue = function (valueString) {
            var opt = this._optionMap.get(_extractId(valueString));
            return lang_1.isPresent(opt) ? opt._value : valueString;
        };
        /** @nocollapse */
        SelectMultipleControlValueAccessor.decorators = [{ type: core_1.Directive, args: [{
                selector: 'select[multiple][ngControl],select[multiple][ngFormControl],select[multiple][ngModel]',
                host: { '(input)': 'onChange($event.target)', '(blur)': 'onTouched()' },
                providers: [SELECT_MULTIPLE_VALUE_ACCESSOR]
            }] }];
        /** @nocollapse */
        SelectMultipleControlValueAccessor.ctorParameters = [];
        return SelectMultipleControlValueAccessor;
    }();
    exports.SelectMultipleControlValueAccessor = SelectMultipleControlValueAccessor;
    var NgSelectMultipleOption = function () {
        function NgSelectMultipleOption(_element, _renderer, _select) {
            this._element = _element;
            this._renderer = _renderer;
            this._select = _select;
            if (lang_1.isPresent(this._select)) {
                this.id = this._select._registerOption(this);
            }
        }
        Object.defineProperty(NgSelectMultipleOption.prototype, "ngValue", {
            set: function (value) {
                if (this._select == null) return;
                this._value = value;
                this._setElementValue(_buildValueString(this.id, value));
                this._select.writeValue(this._select.value);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgSelectMultipleOption.prototype, "value", {
            set: function (value) {
                if (lang_1.isPresent(this._select)) {
                    this._value = value;
                    this._setElementValue(_buildValueString(this.id, value));
                    this._select.writeValue(this._select.value);
                } else {
                    this._setElementValue(value);
                }
            },
            enumerable: true,
            configurable: true
        });
        /** @internal */
        NgSelectMultipleOption.prototype._setElementValue = function (value) {
            this._renderer.setElementProperty(this._element.nativeElement, 'value', value);
        };
        /** @internal */
        NgSelectMultipleOption.prototype._setSelected = function (selected) {
            this._renderer.setElementProperty(this._element.nativeElement, 'selected', selected);
        };
        NgSelectMultipleOption.prototype.ngOnDestroy = function () {
            if (lang_1.isPresent(this._select)) {
                this._select._optionMap.delete(this.id);
                this._select.writeValue(this._select.value);
            }
        };
        /** @nocollapse */
        NgSelectMultipleOption.decorators = [{ type: core_1.Directive, args: [{ selector: 'option' }] }];
        /** @nocollapse */
        NgSelectMultipleOption.ctorParameters = [{ type: core_1.ElementRef }, { type: core_1.Renderer }, { type: SelectMultipleControlValueAccessor, decorators: [{ type: core_1.Optional }, { type: core_1.Host }] }];
        /** @nocollapse */
        NgSelectMultipleOption.propDecorators = {
            'ngValue': [{ type: core_1.Input, args: ['ngValue'] }],
            'value': [{ type: core_1.Input, args: ['value'] }]
        };
        return NgSelectMultipleOption;
    }();
    exports.NgSelectMultipleOption = NgSelectMultipleOption;
    exports.SELECT_DIRECTIVES = [SelectMultipleControlValueAccessor, NgSelectMultipleOption];
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/shared.js', ['../../facade/collection', '../../facade/exceptions', '../../facade/lang', '../validators', './checkbox_value_accessor', './default_value_accessor', './normalize_validator', './number_value_accessor', './radio_control_value_accessor', './select_control_value_accessor', './select_multiple_control_value_accessor'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var collection_1 = $__require('../../facade/collection');
    var exceptions_1 = $__require('../../facade/exceptions');
    var lang_1 = $__require('../../facade/lang');
    var validators_1 = $__require('../validators');
    var checkbox_value_accessor_1 = $__require('./checkbox_value_accessor');
    var default_value_accessor_1 = $__require('./default_value_accessor');
    var normalize_validator_1 = $__require('./normalize_validator');
    var number_value_accessor_1 = $__require('./number_value_accessor');
    var radio_control_value_accessor_1 = $__require('./radio_control_value_accessor');
    var select_control_value_accessor_1 = $__require('./select_control_value_accessor');
    var select_multiple_control_value_accessor_1 = $__require('./select_multiple_control_value_accessor');
    function controlPath(name, parent) {
        var p = collection_1.ListWrapper.clone(parent.path);
        p.push(name);
        return p;
    }
    exports.controlPath = controlPath;
    function setUpControl(control, dir) {
        if (lang_1.isBlank(control)) _throwError(dir, 'Cannot find control');
        if (lang_1.isBlank(dir.valueAccessor)) _throwError(dir, 'No value accessor for');
        control.validator = validators_1.Validators.compose([control.validator, dir.validator]);
        control.asyncValidator = validators_1.Validators.composeAsync([control.asyncValidator, dir.asyncValidator]);
        dir.valueAccessor.writeValue(control.value);
        // view -> model
        dir.valueAccessor.registerOnChange(function (newValue) {
            dir.viewToModelUpdate(newValue);
            control.updateValue(newValue, { emitModelToViewChange: false });
            control.markAsDirty();
        });
        // model -> view
        control.registerOnChange(function (newValue) {
            return dir.valueAccessor.writeValue(newValue);
        });
        // touched
        dir.valueAccessor.registerOnTouched(function () {
            return control.markAsTouched();
        });
    }
    exports.setUpControl = setUpControl;
    function setUpControlGroup(control, dir) {
        if (lang_1.isBlank(control)) _throwError(dir, 'Cannot find control');
        control.validator = validators_1.Validators.compose([control.validator, dir.validator]);
        control.asyncValidator = validators_1.Validators.composeAsync([control.asyncValidator, dir.asyncValidator]);
    }
    exports.setUpControlGroup = setUpControlGroup;
    function _throwError(dir, message) {
        var path = dir.path.join(' -> ');
        throw new exceptions_1.BaseException(message + " '" + path + "'");
    }
    function composeValidators(validators) {
        return lang_1.isPresent(validators) ? validators_1.Validators.compose(validators.map(normalize_validator_1.normalizeValidator)) : null;
    }
    exports.composeValidators = composeValidators;
    function composeAsyncValidators(validators) {
        return lang_1.isPresent(validators) ? validators_1.Validators.composeAsync(validators.map(normalize_validator_1.normalizeAsyncValidator)) : null;
    }
    exports.composeAsyncValidators = composeAsyncValidators;
    function isPropertyUpdated(changes, viewModel) {
        if (!collection_1.StringMapWrapper.contains(changes, 'model')) return false;
        var change = changes['model'];
        if (change.isFirstChange()) return true;
        return !lang_1.looseIdentical(viewModel, change.currentValue);
    }
    exports.isPropertyUpdated = isPropertyUpdated;
    // TODO: vsavkin remove it once https://github.com/angular/angular/issues/3011 is implemented
    function selectValueAccessor(dir, valueAccessors) {
        if (lang_1.isBlank(valueAccessors)) return null;
        var defaultAccessor;
        var builtinAccessor;
        var customAccessor;
        valueAccessors.forEach(function (v) {
            if (lang_1.hasConstructor(v, default_value_accessor_1.DefaultValueAccessor)) {
                defaultAccessor = v;
            } else if (lang_1.hasConstructor(v, checkbox_value_accessor_1.CheckboxControlValueAccessor) || lang_1.hasConstructor(v, number_value_accessor_1.NumberValueAccessor) || lang_1.hasConstructor(v, select_control_value_accessor_1.SelectControlValueAccessor) || lang_1.hasConstructor(v, select_multiple_control_value_accessor_1.SelectMultipleControlValueAccessor) || lang_1.hasConstructor(v, radio_control_value_accessor_1.RadioControlValueAccessor)) {
                if (lang_1.isPresent(builtinAccessor)) _throwError(dir, 'More than one built-in value accessor matches');
                builtinAccessor = v;
            } else {
                if (lang_1.isPresent(customAccessor)) _throwError(dir, 'More than one custom value accessor matches');
                customAccessor = v;
            }
        });
        if (lang_1.isPresent(customAccessor)) return customAccessor;
        if (lang_1.isPresent(builtinAccessor)) return builtinAccessor;
        if (lang_1.isPresent(defaultAccessor)) return defaultAccessor;
        _throwError(dir, 'No valid value accessor for');
        return null;
    }
    exports.selectValueAccessor = selectValueAccessor;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/ng_model.js', ['@angular/core', '../../facade/async', '../model', '../validators', './control_value_accessor', './ng_control', './shared'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var async_1 = $__require('../../facade/async');
    var model_1 = $__require('../model');
    var validators_1 = $__require('../validators');
    var control_value_accessor_1 = $__require('./control_value_accessor');
    var ng_control_1 = $__require('./ng_control');
    var shared_1 = $__require('./shared');
    exports.formControlBinding =
    /*@ts2dart_const*/ /* @ts2dart_Provider */{
        provide: ng_control_1.NgControl,
        useExisting: core_1.forwardRef(function () {
            return NgModel;
        })
    };
    var NgModel = function (_super) {
        __extends(NgModel, _super);
        function NgModel(_validators, _asyncValidators, valueAccessors) {
            _super.call(this);
            this._validators = _validators;
            this._asyncValidators = _asyncValidators;
            /** @internal */
            this._control = new model_1.Control();
            /** @internal */
            this._added = false;
            this.update = new async_1.EventEmitter();
            this.valueAccessor = shared_1.selectValueAccessor(this, valueAccessors);
        }
        NgModel.prototype.ngOnChanges = function (changes) {
            if (!this._added) {
                shared_1.setUpControl(this._control, this);
                this._control.updateValueAndValidity({ emitEvent: false });
                this._added = true;
            }
            if (shared_1.isPropertyUpdated(changes, this.viewModel)) {
                this._control.updateValue(this.model);
                this.viewModel = this.model;
            }
        };
        Object.defineProperty(NgModel.prototype, "control", {
            get: function () {
                return this._control;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgModel.prototype, "path", {
            get: function () {
                return [];
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgModel.prototype, "validator", {
            get: function () {
                return shared_1.composeValidators(this._validators);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgModel.prototype, "asyncValidator", {
            get: function () {
                return shared_1.composeAsyncValidators(this._asyncValidators);
            },
            enumerable: true,
            configurable: true
        });
        NgModel.prototype.viewToModelUpdate = function (newValue) {
            this.viewModel = newValue;
            async_1.ObservableWrapper.callEmit(this.update, newValue);
        };
        /** @nocollapse */
        NgModel.decorators = [{ type: core_1.Directive, args: [{
                selector: '[ngModel]:not([ngControl]):not([ngFormControl])',
                providers: [exports.formControlBinding],
                inputs: ['model: ngModel'],
                outputs: ['update: ngModelChange'],
                exportAs: 'ngForm'
            }] }];
        /** @nocollapse */
        NgModel.ctorParameters = [{ type: Array, decorators: [{ type: core_1.Optional }, { type: core_1.Self }, { type: core_1.Inject, args: [validators_1.NG_VALIDATORS] }] }, { type: Array, decorators: [{ type: core_1.Optional }, { type: core_1.Self }, { type: core_1.Inject, args: [validators_1.NG_ASYNC_VALIDATORS] }] }, { type: Array, decorators: [{ type: core_1.Optional }, { type: core_1.Self }, { type: core_1.Inject, args: [control_value_accessor_1.NG_VALUE_ACCESSOR] }] }];
        return NgModel;
    }(ng_control_1.NgControl);
    exports.NgModel = NgModel;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/control_value_accessor.js', ['@angular/core'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var core_1 = $__require('@angular/core');
  /**
   * Used to provide a {@link ControlValueAccessor} for form controls.
   *
   * See {@link DefaultValueAccessor} for how to implement one.
   * @experimental
   */
  exports.NG_VALUE_ACCESSOR =
  /*@ts2dart_const*/new core_1.OpaqueToken('NgValueAccessor');
  
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/select_control_value_accessor.js', ['@angular/core', '../../facade/collection', '../../facade/lang', './control_value_accessor'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var collection_1 = $__require('../../facade/collection');
    var lang_1 = $__require('../../facade/lang');
    var control_value_accessor_1 = $__require('./control_value_accessor');
    exports.SELECT_VALUE_ACCESSOR = {
        provide: control_value_accessor_1.NG_VALUE_ACCESSOR,
        useExisting: core_1.forwardRef(function () {
            return SelectControlValueAccessor;
        }),
        multi: true
    };
    function _buildValueString(id, value) {
        if (lang_1.isBlank(id)) return "" + value;
        if (!lang_1.isPrimitive(value)) value = 'Object';
        return lang_1.StringWrapper.slice(id + ": " + value, 0, 50);
    }
    function _extractId(valueString) {
        return valueString.split(':')[0];
    }
    var SelectControlValueAccessor = function () {
        function SelectControlValueAccessor(_renderer, _elementRef) {
            this._renderer = _renderer;
            this._elementRef = _elementRef;
            /** @internal */
            this._optionMap = new Map();
            /** @internal */
            this._idCounter = 0;
            this.onChange = function (_) {};
            this.onTouched = function () {};
        }
        SelectControlValueAccessor.prototype.writeValue = function (value) {
            this.value = value;
            var valueString = _buildValueString(this._getOptionId(value), value);
            this._renderer.setElementProperty(this._elementRef.nativeElement, 'value', valueString);
        };
        SelectControlValueAccessor.prototype.registerOnChange = function (fn) {
            var _this = this;
            this.onChange = function (valueString) {
                _this.value = valueString;
                fn(_this._getOptionValue(valueString));
            };
        };
        SelectControlValueAccessor.prototype.registerOnTouched = function (fn) {
            this.onTouched = fn;
        };
        /** @internal */
        SelectControlValueAccessor.prototype._registerOption = function () {
            return (this._idCounter++).toString();
        };
        /** @internal */
        SelectControlValueAccessor.prototype._getOptionId = function (value) {
            for (var _i = 0, _a = collection_1.MapWrapper.keys(this._optionMap); _i < _a.length; _i++) {
                var id = _a[_i];
                if (lang_1.looseIdentical(this._optionMap.get(id), value)) return id;
            }
            return null;
        };
        /** @internal */
        SelectControlValueAccessor.prototype._getOptionValue = function (valueString) {
            var value = this._optionMap.get(_extractId(valueString));
            return lang_1.isPresent(value) ? value : valueString;
        };
        /** @nocollapse */
        SelectControlValueAccessor.decorators = [{ type: core_1.Directive, args: [{
                selector: 'select:not([multiple])[ngControl],select:not([multiple])[ngFormControl],select:not([multiple])[ngModel]',
                host: { '(change)': 'onChange($event.target.value)', '(blur)': 'onTouched()' },
                providers: [exports.SELECT_VALUE_ACCESSOR]
            }] }];
        /** @nocollapse */
        SelectControlValueAccessor.ctorParameters = [{ type: core_1.Renderer }, { type: core_1.ElementRef }];
        return SelectControlValueAccessor;
    }();
    exports.SelectControlValueAccessor = SelectControlValueAccessor;
    var NgSelectOption = function () {
        function NgSelectOption(_element, _renderer, _select) {
            this._element = _element;
            this._renderer = _renderer;
            this._select = _select;
            if (lang_1.isPresent(this._select)) this.id = this._select._registerOption();
        }
        Object.defineProperty(NgSelectOption.prototype, "ngValue", {
            set: function (value) {
                if (this._select == null) return;
                this._select._optionMap.set(this.id, value);
                this._setElementValue(_buildValueString(this.id, value));
                this._select.writeValue(this._select.value);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgSelectOption.prototype, "value", {
            set: function (value) {
                this._setElementValue(value);
                if (lang_1.isPresent(this._select)) this._select.writeValue(this._select.value);
            },
            enumerable: true,
            configurable: true
        });
        /** @internal */
        NgSelectOption.prototype._setElementValue = function (value) {
            this._renderer.setElementProperty(this._element.nativeElement, 'value', value);
        };
        NgSelectOption.prototype.ngOnDestroy = function () {
            if (lang_1.isPresent(this._select)) {
                this._select._optionMap.delete(this.id);
                this._select.writeValue(this._select.value);
            }
        };
        /** @nocollapse */
        NgSelectOption.decorators = [{ type: core_1.Directive, args: [{ selector: 'option' }] }];
        /** @nocollapse */
        NgSelectOption.ctorParameters = [{ type: core_1.ElementRef }, { type: core_1.Renderer }, { type: SelectControlValueAccessor, decorators: [{ type: core_1.Optional }, { type: core_1.Host }] }];
        /** @nocollapse */
        NgSelectOption.propDecorators = {
            'ngValue': [{ type: core_1.Input, args: ['ngValue'] }],
            'value': [{ type: core_1.Input, args: ['value'] }]
        };
        return NgSelectOption;
    }();
    exports.NgSelectOption = NgSelectOption;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/directives/validators.js', ['@angular/core', '../../facade/lang', '../validators'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../../facade/lang');
    var validators_1 = $__require('../validators');
    var REQUIRED = validators_1.Validators.required;
    exports.REQUIRED_VALIDATOR = {
        provide: validators_1.NG_VALIDATORS,
        useValue: REQUIRED,
        multi: true
    };
    var RequiredValidator = function () {
        function RequiredValidator() {}
        /** @nocollapse */
        RequiredValidator.decorators = [{ type: core_1.Directive, args: [{
                selector: '[required][ngControl],[required][ngFormControl],[required][ngModel]',
                providers: [exports.REQUIRED_VALIDATOR]
            }] }];
        return RequiredValidator;
    }();
    exports.RequiredValidator = RequiredValidator;
    /**
     * Provivder which adds {@link MinLengthValidator} to {@link NG_VALIDATORS}.
     *
     * ## Example:
     *
     * {@example common/forms/ts/validators/validators.ts region='min'}
     */
    exports.MIN_LENGTH_VALIDATOR = {
        provide: validators_1.NG_VALIDATORS,
        useExisting: core_1.forwardRef(function () {
            return MinLengthValidator;
        }),
        multi: true
    };
    var MinLengthValidator = function () {
        function MinLengthValidator(minLength) {
            this._validator = validators_1.Validators.minLength(lang_1.NumberWrapper.parseInt(minLength, 10));
        }
        MinLengthValidator.prototype.validate = function (c) {
            return this._validator(c);
        };
        /** @nocollapse */
        MinLengthValidator.decorators = [{ type: core_1.Directive, args: [{
                selector: '[minlength][ngControl],[minlength][ngFormControl],[minlength][ngModel]',
                providers: [exports.MIN_LENGTH_VALIDATOR]
            }] }];
        /** @nocollapse */
        MinLengthValidator.ctorParameters = [{ type: undefined, decorators: [{ type: core_1.Attribute, args: ['minlength'] }] }];
        return MinLengthValidator;
    }();
    exports.MinLengthValidator = MinLengthValidator;
    /**
     * Provider which adds {@link MaxLengthValidator} to {@link NG_VALIDATORS}.
     *
     * ## Example:
     *
     * {@example common/forms/ts/validators/validators.ts region='max'}
     */
    exports.MAX_LENGTH_VALIDATOR = {
        provide: validators_1.NG_VALIDATORS,
        useExisting: core_1.forwardRef(function () {
            return MaxLengthValidator;
        }),
        multi: true
    };
    var MaxLengthValidator = function () {
        function MaxLengthValidator(maxLength) {
            this._validator = validators_1.Validators.maxLength(lang_1.NumberWrapper.parseInt(maxLength, 10));
        }
        MaxLengthValidator.prototype.validate = function (c) {
            return this._validator(c);
        };
        /** @nocollapse */
        MaxLengthValidator.decorators = [{ type: core_1.Directive, args: [{
                selector: '[maxlength][ngControl],[maxlength][ngFormControl],[maxlength][ngModel]',
                providers: [exports.MAX_LENGTH_VALIDATOR]
            }] }];
        /** @nocollapse */
        MaxLengthValidator.ctorParameters = [{ type: undefined, decorators: [{ type: core_1.Attribute, args: ['maxlength'] }] }];
        return MaxLengthValidator;
    }();
    exports.MaxLengthValidator = MaxLengthValidator;
    exports.PATTERN_VALIDATOR = {
        provide: validators_1.NG_VALIDATORS,
        useExisting: core_1.forwardRef(function () {
            return PatternValidator;
        }),
        multi: true
    };
    var PatternValidator = function () {
        function PatternValidator(pattern) {
            this._validator = validators_1.Validators.pattern(pattern);
        }
        PatternValidator.prototype.validate = function (c) {
            return this._validator(c);
        };
        /** @nocollapse */
        PatternValidator.decorators = [{ type: core_1.Directive, args: [{
                selector: '[pattern][ngControl],[pattern][ngFormControl],[pattern][ngModel]',
                providers: [exports.PATTERN_VALIDATOR]
            }] }];
        /** @nocollapse */
        PatternValidator.ctorParameters = [{ type: undefined, decorators: [{ type: core_1.Attribute, args: ['pattern'] }] }];
        return PatternValidator;
    }();
    exports.PatternValidator = PatternValidator;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/model.js', ['../facade/async', '../facade/collection', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var async_1 = $__require('../facade/async');
    var collection_1 = $__require('../facade/collection');
    var lang_1 = $__require('../facade/lang');
    /**
     * Indicates that a Control is valid, i.e. that no errors exist in the input value.
     */
    exports.VALID = 'VALID';
    /**
     * Indicates that a Control is invalid, i.e. that an error exists in the input value.
     */
    exports.INVALID = 'INVALID';
    /**
     * Indicates that a Control is pending, i.e. that async validation is occurring and
     * errors are not yet available for the input value.
     */
    exports.PENDING = 'PENDING';
    function isControl(control) {
        return control instanceof AbstractControl;
    }
    exports.isControl = isControl;
    function _find(control, path) {
        if (lang_1.isBlank(path)) return null;
        if (!(path instanceof Array)) {
            path = path.split('/');
        }
        if (path instanceof Array && collection_1.ListWrapper.isEmpty(path)) return null;
        return path.reduce(function (v, name) {
            if (v instanceof ControlGroup) {
                return lang_1.isPresent(v.controls[name]) ? v.controls[name] : null;
            } else if (v instanceof ControlArray) {
                var index = name;
                return lang_1.isPresent(v.at(index)) ? v.at(index) : null;
            } else {
                return null;
            }
        }, control);
    }
    function toObservable(r) {
        return lang_1.isPromise(r) ? async_1.ObservableWrapper.fromPromise(r) : r;
    }
    /**
     * @experimental
     */
    var AbstractControl = function () {
        function AbstractControl(validator, asyncValidator) {
            this.validator = validator;
            this.asyncValidator = asyncValidator;
            this._pristine = true;
            this._touched = false;
        }
        Object.defineProperty(AbstractControl.prototype, "value", {
            get: function () {
                return this._value;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControl.prototype, "status", {
            get: function () {
                return this._status;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControl.prototype, "valid", {
            get: function () {
                return this._status === exports.VALID;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControl.prototype, "errors", {
            /**
             * Returns the errors of this control.
             */
            get: function () {
                return this._errors;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControl.prototype, "pristine", {
            get: function () {
                return this._pristine;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControl.prototype, "dirty", {
            get: function () {
                return !this.pristine;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControl.prototype, "touched", {
            get: function () {
                return this._touched;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControl.prototype, "untouched", {
            get: function () {
                return !this._touched;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControl.prototype, "valueChanges", {
            get: function () {
                return this._valueChanges;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControl.prototype, "statusChanges", {
            get: function () {
                return this._statusChanges;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AbstractControl.prototype, "pending", {
            get: function () {
                return this._status == exports.PENDING;
            },
            enumerable: true,
            configurable: true
        });
        AbstractControl.prototype.markAsTouched = function () {
            this._touched = true;
        };
        AbstractControl.prototype.markAsDirty = function (_a) {
            var onlySelf = (_a === void 0 ? {} : _a).onlySelf;
            onlySelf = lang_1.normalizeBool(onlySelf);
            this._pristine = false;
            if (lang_1.isPresent(this._parent) && !onlySelf) {
                this._parent.markAsDirty({ onlySelf: onlySelf });
            }
        };
        AbstractControl.prototype.markAsPending = function (_a) {
            var onlySelf = (_a === void 0 ? {} : _a).onlySelf;
            onlySelf = lang_1.normalizeBool(onlySelf);
            this._status = exports.PENDING;
            if (lang_1.isPresent(this._parent) && !onlySelf) {
                this._parent.markAsPending({ onlySelf: onlySelf });
            }
        };
        AbstractControl.prototype.setParent = function (parent) {
            this._parent = parent;
        };
        AbstractControl.prototype.updateValueAndValidity = function (_a) {
            var _b = _a === void 0 ? {} : _a,
                onlySelf = _b.onlySelf,
                emitEvent = _b.emitEvent;
            onlySelf = lang_1.normalizeBool(onlySelf);
            emitEvent = lang_1.isPresent(emitEvent) ? emitEvent : true;
            this._updateValue();
            this._errors = this._runValidator();
            this._status = this._calculateStatus();
            if (this._status == exports.VALID || this._status == exports.PENDING) {
                this._runAsyncValidator(emitEvent);
            }
            if (emitEvent) {
                async_1.ObservableWrapper.callEmit(this._valueChanges, this._value);
                async_1.ObservableWrapper.callEmit(this._statusChanges, this._status);
            }
            if (lang_1.isPresent(this._parent) && !onlySelf) {
                this._parent.updateValueAndValidity({ onlySelf: onlySelf, emitEvent: emitEvent });
            }
        };
        AbstractControl.prototype._runValidator = function () {
            return lang_1.isPresent(this.validator) ? this.validator(this) : null;
        };
        AbstractControl.prototype._runAsyncValidator = function (emitEvent) {
            var _this = this;
            if (lang_1.isPresent(this.asyncValidator)) {
                this._status = exports.PENDING;
                this._cancelExistingSubscription();
                var obs = toObservable(this.asyncValidator(this));
                this._asyncValidationSubscription = async_1.ObservableWrapper.subscribe(obs, function (res) {
                    return _this.setErrors(res, { emitEvent: emitEvent });
                });
            }
        };
        AbstractControl.prototype._cancelExistingSubscription = function () {
            if (lang_1.isPresent(this._asyncValidationSubscription)) {
                async_1.ObservableWrapper.dispose(this._asyncValidationSubscription);
            }
        };
        /**
         * Sets errors on a control.
         *
         * This is used when validations are run not automatically, but manually by the user.
         *
         * Calling `setErrors` will also update the validity of the parent control.
         *
         * ## Usage
         *
         * ```
         * var login = new Control("someLogin");
         * login.setErrors({
         *   "notUnique": true
         * });
         *
         * expect(login.valid).toEqual(false);
         * expect(login.errors).toEqual({"notUnique": true});
         *
         * login.updateValue("someOtherLogin");
         *
         * expect(login.valid).toEqual(true);
         * ```
         */
        AbstractControl.prototype.setErrors = function (errors, _a) {
            var emitEvent = (_a === void 0 ? {} : _a).emitEvent;
            emitEvent = lang_1.isPresent(emitEvent) ? emitEvent : true;
            this._errors = errors;
            this._status = this._calculateStatus();
            if (emitEvent) {
                async_1.ObservableWrapper.callEmit(this._statusChanges, this._status);
            }
            if (lang_1.isPresent(this._parent)) {
                this._parent._updateControlsErrors();
            }
        };
        AbstractControl.prototype.find = function (path) {
            return _find(this, path);
        };
        AbstractControl.prototype.getError = function (errorCode, path) {
            if (path === void 0) {
                path = null;
            }
            var control = lang_1.isPresent(path) && !collection_1.ListWrapper.isEmpty(path) ? this.find(path) : this;
            if (lang_1.isPresent(control) && lang_1.isPresent(control._errors)) {
                return collection_1.StringMapWrapper.get(control._errors, errorCode);
            } else {
                return null;
            }
        };
        AbstractControl.prototype.hasError = function (errorCode, path) {
            if (path === void 0) {
                path = null;
            }
            return lang_1.isPresent(this.getError(errorCode, path));
        };
        Object.defineProperty(AbstractControl.prototype, "root", {
            get: function () {
                var x = this;
                while (lang_1.isPresent(x._parent)) {
                    x = x._parent;
                }
                return x;
            },
            enumerable: true,
            configurable: true
        });
        /** @internal */
        AbstractControl.prototype._updateControlsErrors = function () {
            this._status = this._calculateStatus();
            if (lang_1.isPresent(this._parent)) {
                this._parent._updateControlsErrors();
            }
        };
        /** @internal */
        AbstractControl.prototype._initObservables = function () {
            this._valueChanges = new async_1.EventEmitter();
            this._statusChanges = new async_1.EventEmitter();
        };
        AbstractControl.prototype._calculateStatus = function () {
            if (lang_1.isPresent(this._errors)) return exports.INVALID;
            if (this._anyControlsHaveStatus(exports.PENDING)) return exports.PENDING;
            if (this._anyControlsHaveStatus(exports.INVALID)) return exports.INVALID;
            return exports.VALID;
        };
        return AbstractControl;
    }();
    exports.AbstractControl = AbstractControl;
    /**
     * Defines a part of a form that cannot be divided into other controls. `Control`s have values and
     * validation state, which is determined by an optional validation function.
     *
     * `Control` is one of the three fundamental building blocks used to define forms in Angular, along
     * with {@link ControlGroup} and {@link ControlArray}.
     *
     * ## Usage
     *
     * By default, a `Control` is created for every `<input>` or other form component.
     * With {@link NgFormControl} or {@link NgFormModel} an existing {@link Control} can be
     * bound to a DOM element instead. This `Control` can be configured with a custom
     * validation function.
     *
     * ### Example ([live demo](http://plnkr.co/edit/23DESOpbNnBpBHZt1BR4?p=preview))
     *
     * @experimental
     */
    var Control = function (_super) {
        __extends(Control, _super);
        function Control(value, validator, asyncValidator) {
            if (value === void 0) {
                value = null;
            }
            if (validator === void 0) {
                validator = null;
            }
            if (asyncValidator === void 0) {
                asyncValidator = null;
            }
            _super.call(this, validator, asyncValidator);
            this._value = value;
            this.updateValueAndValidity({ onlySelf: true, emitEvent: false });
            this._initObservables();
        }
        /**
         * Set the value of the control to `value`.
         *
         * If `onlySelf` is `true`, this change will only affect the validation of this `Control`
         * and not its parent component. If `emitEvent` is `true`, this change will cause a
         * `valueChanges` event on the `Control` to be emitted. Both of these options default to
         * `false`.
         *
         * If `emitModelToViewChange` is `true`, the view will be notified about the new value
         * via an `onChange` event. This is the default behavior if `emitModelToViewChange` is not
         * specified.
         */
        Control.prototype.updateValue = function (value, _a) {
            var _b = _a === void 0 ? {} : _a,
                onlySelf = _b.onlySelf,
                emitEvent = _b.emitEvent,
                emitModelToViewChange = _b.emitModelToViewChange;
            emitModelToViewChange = lang_1.isPresent(emitModelToViewChange) ? emitModelToViewChange : true;
            this._value = value;
            if (lang_1.isPresent(this._onChange) && emitModelToViewChange) this._onChange(this._value);
            this.updateValueAndValidity({ onlySelf: onlySelf, emitEvent: emitEvent });
        };
        /**
         * @internal
         */
        Control.prototype._updateValue = function () {};
        /**
         * @internal
         */
        Control.prototype._anyControlsHaveStatus = function (status) {
            return false;
        };
        /**
         * Register a listener for change events.
         */
        Control.prototype.registerOnChange = function (fn) {
            this._onChange = fn;
        };
        return Control;
    }(AbstractControl);
    exports.Control = Control;
    /**
     * Defines a part of a form, of fixed length, that can contain other controls.
     *
     * A `ControlGroup` aggregates the values of each {@link Control} in the group.
     * The status of a `ControlGroup` depends on the status of its children.
     * If one of the controls in a group is invalid, the entire group is invalid.
     * Similarly, if a control changes its value, the entire group changes as well.
     *
     * `ControlGroup` is one of the three fundamental building blocks used to define forms in Angular,
     * along with {@link Control} and {@link ControlArray}. {@link ControlArray} can also contain other
     * controls, but is of variable length.
     *
     * ### Example ([live demo](http://plnkr.co/edit/23DESOpbNnBpBHZt1BR4?p=preview))
     *
     * @experimental
     */
    var ControlGroup = function (_super) {
        __extends(ControlGroup, _super);
        function ControlGroup(controls, optionals, validator, asyncValidator) {
            if (optionals === void 0) {
                optionals = null;
            }
            if (validator === void 0) {
                validator = null;
            }
            if (asyncValidator === void 0) {
                asyncValidator = null;
            }
            _super.call(this, validator, asyncValidator);
            this.controls = controls;
            this._optionals = lang_1.isPresent(optionals) ? optionals : {};
            this._initObservables();
            this._setParentForControls();
            this.updateValueAndValidity({ onlySelf: true, emitEvent: false });
        }
        /**
         * Register a control with the group's list of controls.
         */
        ControlGroup.prototype.registerControl = function (name, control) {
            this.controls[name] = control;
            control.setParent(this);
        };
        /**
         * Add a control to this group.
         */
        ControlGroup.prototype.addControl = function (name, control) {
            this.registerControl(name, control);
            this.updateValueAndValidity();
        };
        /**
         * Remove a control from this group.
         */
        ControlGroup.prototype.removeControl = function (name) {
            collection_1.StringMapWrapper.delete(this.controls, name);
            this.updateValueAndValidity();
        };
        /**
         * Mark the named control as non-optional.
         */
        ControlGroup.prototype.include = function (controlName) {
            collection_1.StringMapWrapper.set(this._optionals, controlName, true);
            this.updateValueAndValidity();
        };
        /**
         * Mark the named control as optional.
         */
        ControlGroup.prototype.exclude = function (controlName) {
            collection_1.StringMapWrapper.set(this._optionals, controlName, false);
            this.updateValueAndValidity();
        };
        /**
         * Check whether there is a control with the given name in the group.
         */
        ControlGroup.prototype.contains = function (controlName) {
            var c = collection_1.StringMapWrapper.contains(this.controls, controlName);
            return c && this._included(controlName);
        };
        /** @internal */
        ControlGroup.prototype._setParentForControls = function () {
            var _this = this;
            collection_1.StringMapWrapper.forEach(this.controls, function (control, name) {
                control.setParent(_this);
            });
        };
        /** @internal */
        ControlGroup.prototype._updateValue = function () {
            this._value = this._reduceValue();
        };
        /** @internal */
        ControlGroup.prototype._anyControlsHaveStatus = function (status) {
            var _this = this;
            var res = false;
            collection_1.StringMapWrapper.forEach(this.controls, function (control, name) {
                res = res || _this.contains(name) && control.status == status;
            });
            return res;
        };
        /** @internal */
        ControlGroup.prototype._reduceValue = function () {
            return this._reduceChildren({}, function (acc, control, name) {
                acc[name] = control.value;
                return acc;
            });
        };
        /** @internal */
        ControlGroup.prototype._reduceChildren = function (initValue, fn) {
            var _this = this;
            var res = initValue;
            collection_1.StringMapWrapper.forEach(this.controls, function (control, name) {
                if (_this._included(name)) {
                    res = fn(res, control, name);
                }
            });
            return res;
        };
        /** @internal */
        ControlGroup.prototype._included = function (controlName) {
            var isOptional = collection_1.StringMapWrapper.contains(this._optionals, controlName);
            return !isOptional || collection_1.StringMapWrapper.get(this._optionals, controlName);
        };
        return ControlGroup;
    }(AbstractControl);
    exports.ControlGroup = ControlGroup;
    /**
     * Defines a part of a form, of variable length, that can contain other controls.
     *
     * A `ControlArray` aggregates the values of each {@link Control} in the group.
     * The status of a `ControlArray` depends on the status of its children.
     * If one of the controls in a group is invalid, the entire array is invalid.
     * Similarly, if a control changes its value, the entire array changes as well.
     *
     * `ControlArray` is one of the three fundamental building blocks used to define forms in Angular,
     * along with {@link Control} and {@link ControlGroup}. {@link ControlGroup} can also contain
     * other controls, but is of fixed length.
     *
     * ## Adding or removing controls
     *
     * To change the controls in the array, use the `push`, `insert`, or `removeAt` methods
     * in `ControlArray` itself. These methods ensure the controls are properly tracked in the
     * form's hierarchy. Do not modify the array of `AbstractControl`s used to instantiate
     * the `ControlArray` directly, as that will result in strange and unexpected behavior such
     * as broken change detection.
     *
     * ### Example ([live demo](http://plnkr.co/edit/23DESOpbNnBpBHZt1BR4?p=preview))
     *
     * @experimental
     */
    var ControlArray = function (_super) {
        __extends(ControlArray, _super);
        function ControlArray(controls, validator, asyncValidator) {
            if (validator === void 0) {
                validator = null;
            }
            if (asyncValidator === void 0) {
                asyncValidator = null;
            }
            _super.call(this, validator, asyncValidator);
            this.controls = controls;
            this._initObservables();
            this._setParentForControls();
            this.updateValueAndValidity({ onlySelf: true, emitEvent: false });
        }
        /**
         * Get the {@link AbstractControl} at the given `index` in the array.
         */
        ControlArray.prototype.at = function (index) {
            return this.controls[index];
        };
        /**
         * Insert a new {@link AbstractControl} at the end of the array.
         */
        ControlArray.prototype.push = function (control) {
            this.controls.push(control);
            control.setParent(this);
            this.updateValueAndValidity();
        };
        /**
         * Insert a new {@link AbstractControl} at the given `index` in the array.
         */
        ControlArray.prototype.insert = function (index, control) {
            collection_1.ListWrapper.insert(this.controls, index, control);
            control.setParent(this);
            this.updateValueAndValidity();
        };
        /**
         * Remove the control at the given `index` in the array.
         */
        ControlArray.prototype.removeAt = function (index) {
            collection_1.ListWrapper.removeAt(this.controls, index);
            this.updateValueAndValidity();
        };
        Object.defineProperty(ControlArray.prototype, "length", {
            /**
             * Length of the control array.
             */
            get: function () {
                return this.controls.length;
            },
            enumerable: true,
            configurable: true
        });
        /** @internal */
        ControlArray.prototype._updateValue = function () {
            this._value = this.controls.map(function (control) {
                return control.value;
            });
        };
        /** @internal */
        ControlArray.prototype._anyControlsHaveStatus = function (status) {
            return this.controls.some(function (c) {
                return c.status == status;
            });
        };
        /** @internal */
        ControlArray.prototype._setParentForControls = function () {
            var _this = this;
            this.controls.forEach(function (control) {
                control.setParent(_this);
            });
        };
        return ControlArray;
    }(AbstractControl);
    exports.ControlArray = ControlArray;
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated/validators.js', ['@angular/core', '../facade/async', '../facade/collection', '../facade/lang', '../facade/promise'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var async_1 = $__require('../facade/async');
    var collection_1 = $__require('../facade/collection');
    var lang_1 = $__require('../facade/lang');
    var promise_1 = $__require('../facade/promise');
    /**
     * Providers for validators to be used for {@link Control}s in a form.
     *
     * Provide this using `multi: true` to add validators.
     *
     * ### Example
     *
     * {@example core/forms/ts/ng_validators/ng_validators.ts region='ng_validators'}
     * @experimental
     */
    exports.NG_VALIDATORS = new core_1.OpaqueToken('NgValidators');
    /**
     * Providers for asynchronous validators to be used for {@link Control}s
     * in a form.
     *
     * Provide this using `multi: true` to add validators.
     *
     * See {@link NG_VALIDATORS} for more details.
     *
     * @experimental
     */
    exports.NG_ASYNC_VALIDATORS =
    /*@ts2dart_const*/new core_1.OpaqueToken('NgAsyncValidators');
    /**
     * Provides a set of validators used by form controls.
     *
     * A validator is a function that processes a {@link Control} or collection of
     * controls and returns a map of errors. A null map means that validation has passed.
     *
     * ### Example
     *
     * ```typescript
     * var loginControl = new Control("", Validators.required)
     * ```
     *
     * @experimental
     */
    var Validators = function () {
        function Validators() {}
        /**
         * Validator that requires controls to have a non-empty value.
         */
        Validators.required = function (control) {
            return lang_1.isBlank(control.value) || lang_1.isString(control.value) && control.value == '' ? { 'required': true } : null;
        };
        /**
         * Validator that requires controls to have a value of a minimum length.
         */
        Validators.minLength = function (minLength) {
            return function (control) {
                if (lang_1.isPresent(Validators.required(control))) return null;
                var v = control.value;
                return v.length < minLength ? { 'minlength': { 'requiredLength': minLength, 'actualLength': v.length } } : null;
            };
        };
        /**
         * Validator that requires controls to have a value of a maximum length.
         */
        Validators.maxLength = function (maxLength) {
            return function (control) {
                if (lang_1.isPresent(Validators.required(control))) return null;
                var v = control.value;
                return v.length > maxLength ? { 'maxlength': { 'requiredLength': maxLength, 'actualLength': v.length } } : null;
            };
        };
        /**
         * Validator that requires a control to match a regex to its value.
         */
        Validators.pattern = function (pattern) {
            return function (control) {
                if (lang_1.isPresent(Validators.required(control))) return null;
                var regex = new RegExp("^" + pattern + "$");
                var v = control.value;
                return regex.test(v) ? null : { 'pattern': { 'requiredPattern': "^" + pattern + "$", 'actualValue': v } };
            };
        };
        /**
         * No-op validator.
         */
        Validators.nullValidator = function (c) {
            return null;
        };
        /**
         * Compose multiple validators into a single function that returns the union
         * of the individual error maps.
         */
        Validators.compose = function (validators) {
            if (lang_1.isBlank(validators)) return null;
            var presentValidators = validators.filter(lang_1.isPresent);
            if (presentValidators.length == 0) return null;
            return function (control) {
                return _mergeErrors(_executeValidators(control, presentValidators));
            };
        };
        Validators.composeAsync = function (validators) {
            if (lang_1.isBlank(validators)) return null;
            var presentValidators = validators.filter(lang_1.isPresent);
            if (presentValidators.length == 0) return null;
            return function (control) {
                var promises = _executeAsyncValidators(control, presentValidators).map(_convertToPromise);
                return promise_1.PromiseWrapper.all(promises).then(_mergeErrors);
            };
        };
        return Validators;
    }();
    exports.Validators = Validators;
    function _convertToPromise(obj) {
        return lang_1.isPromise(obj) ? obj : async_1.ObservableWrapper.toPromise(obj);
    }
    function _executeValidators(control, validators) {
        return validators.map(function (v) {
            return v(control);
        });
    }
    function _executeAsyncValidators(control, validators) {
        return validators.map(function (v) {
            return v(control);
        });
    }
    function _mergeErrors(arrayOfErrors) {
        var res = arrayOfErrors.reduce(function (res, errors) {
            return lang_1.isPresent(errors) ? collection_1.StringMapWrapper.merge(res, errors) : res;
        }, {});
        return collection_1.StringMapWrapper.isEmpty(res) ? null : res;
    }
    
});
System.registerDynamic('node_modules/@angular/common/src/forms-deprecated.js', ['./forms-deprecated/directives/radio_control_value_accessor', './forms-deprecated/form_builder', './forms-deprecated/directives', './forms-deprecated/directives/abstract_control_directive', './forms-deprecated/directives/checkbox_value_accessor', './forms-deprecated/directives/control_container', './forms-deprecated/directives/control_value_accessor', './forms-deprecated/directives/default_value_accessor', './forms-deprecated/directives/ng_control', './forms-deprecated/directives/ng_control_group', './forms-deprecated/directives/ng_control_name', './forms-deprecated/directives/ng_control_status', './forms-deprecated/directives/ng_form', './forms-deprecated/directives/ng_form_control', './forms-deprecated/directives/ng_form_model', './forms-deprecated/directives/ng_model', './forms-deprecated/directives/select_control_value_accessor', './forms-deprecated/directives/validators', './forms-deprecated/model', './forms-deprecated/validators'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var radio_control_value_accessor_1 = $__require('./forms-deprecated/directives/radio_control_value_accessor');
  var form_builder_1 = $__require('./forms-deprecated/form_builder');
  var directives_1 = $__require('./forms-deprecated/directives');
  exports.FORM_DIRECTIVES = directives_1.FORM_DIRECTIVES;
  exports.RadioButtonState = directives_1.RadioButtonState;
  var abstract_control_directive_1 = $__require('./forms-deprecated/directives/abstract_control_directive');
  exports.AbstractControlDirective = abstract_control_directive_1.AbstractControlDirective;
  var checkbox_value_accessor_1 = $__require('./forms-deprecated/directives/checkbox_value_accessor');
  exports.CheckboxControlValueAccessor = checkbox_value_accessor_1.CheckboxControlValueAccessor;
  var control_container_1 = $__require('./forms-deprecated/directives/control_container');
  exports.ControlContainer = control_container_1.ControlContainer;
  var control_value_accessor_1 = $__require('./forms-deprecated/directives/control_value_accessor');
  exports.NG_VALUE_ACCESSOR = control_value_accessor_1.NG_VALUE_ACCESSOR;
  var default_value_accessor_1 = $__require('./forms-deprecated/directives/default_value_accessor');
  exports.DefaultValueAccessor = default_value_accessor_1.DefaultValueAccessor;
  var ng_control_1 = $__require('./forms-deprecated/directives/ng_control');
  exports.NgControl = ng_control_1.NgControl;
  var ng_control_group_1 = $__require('./forms-deprecated/directives/ng_control_group');
  exports.NgControlGroup = ng_control_group_1.NgControlGroup;
  var ng_control_name_1 = $__require('./forms-deprecated/directives/ng_control_name');
  exports.NgControlName = ng_control_name_1.NgControlName;
  var ng_control_status_1 = $__require('./forms-deprecated/directives/ng_control_status');
  exports.NgControlStatus = ng_control_status_1.NgControlStatus;
  var ng_form_1 = $__require('./forms-deprecated/directives/ng_form');
  exports.NgForm = ng_form_1.NgForm;
  var ng_form_control_1 = $__require('./forms-deprecated/directives/ng_form_control');
  exports.NgFormControl = ng_form_control_1.NgFormControl;
  var ng_form_model_1 = $__require('./forms-deprecated/directives/ng_form_model');
  exports.NgFormModel = ng_form_model_1.NgFormModel;
  var ng_model_1 = $__require('./forms-deprecated/directives/ng_model');
  exports.NgModel = ng_model_1.NgModel;
  var select_control_value_accessor_1 = $__require('./forms-deprecated/directives/select_control_value_accessor');
  exports.NgSelectOption = select_control_value_accessor_1.NgSelectOption;
  exports.SelectControlValueAccessor = select_control_value_accessor_1.SelectControlValueAccessor;
  var validators_1 = $__require('./forms-deprecated/directives/validators');
  exports.MaxLengthValidator = validators_1.MaxLengthValidator;
  exports.MinLengthValidator = validators_1.MinLengthValidator;
  exports.PatternValidator = validators_1.PatternValidator;
  exports.RequiredValidator = validators_1.RequiredValidator;
  var form_builder_2 = $__require('./forms-deprecated/form_builder');
  exports.FormBuilder = form_builder_2.FormBuilder;
  var model_1 = $__require('./forms-deprecated/model');
  exports.AbstractControl = model_1.AbstractControl;
  exports.Control = model_1.Control;
  exports.ControlArray = model_1.ControlArray;
  exports.ControlGroup = model_1.ControlGroup;
  var validators_2 = $__require('./forms-deprecated/validators');
  exports.NG_ASYNC_VALIDATORS = validators_2.NG_ASYNC_VALIDATORS;
  exports.NG_VALIDATORS = validators_2.NG_VALIDATORS;
  exports.Validators = validators_2.Validators;
  /**
   * Shorthand set of providers used for building Angular forms.
   *
   * ### Example
   *
   * ```typescript
   * bootstrap(MyApp, [FORM_PROVIDERS]);
   * ```
   *
   * @experimental
   */
  exports.FORM_PROVIDERS = [form_builder_1.FormBuilder, radio_control_value_accessor_1.RadioControlRegistry];
  
});
System.registerDynamic('node_modules/@angular/common/src/common_directives.js', ['./directives', './forms-deprecated'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var directives_1 = $__require('./directives');
  var forms_deprecated_1 = $__require('./forms-deprecated');
  /**
   * A collection of Angular core directives that are likely to be used in each and every Angular
   * application. This includes core directives (e.g., NgIf and NgFor), and forms directives (e.g.,
   * NgModel).
   *
   * This collection can be used to quickly enumerate all the built-in directives in the `directives`
   * property of the `@Component` decorator.
   *
   * ### Example
   *
   * Instead of writing:
   *
   * ```typescript
   * import {NgClass, NgIf, NgFor, NgSwitch, NgSwitchWhen, NgSwitchDefault, NgModel, NgForm} from
   * '@angular/common';
   * import {OtherDirective} from './myDirectives';
   *
   * @Component({
   *   selector: 'my-component',
   *   templateUrl: 'myComponent.html',
   *   directives: [NgClass, NgIf, NgFor, NgSwitch, NgSwitchWhen, NgSwitchDefault, NgModel, NgForm,
   * OtherDirective]
   * })
   * export class MyComponent {
   *   ...
   * }
   * ```
   * one could import all the common directives at once:
   *
   * ```typescript
   * import {COMMON_DIRECTIVES} from '@angular/common';
   * import {OtherDirective} from './myDirectives';
   *
   * @Component({
   *   selector: 'my-component',
   *   templateUrl: 'myComponent.html',
   *   directives: [COMMON_DIRECTIVES, OtherDirective]
   * })
   * export class MyComponent {
   *   ...
   * }
   * ```
   *
   * @experimental Contains forms which are experimental.
   */
  exports.COMMON_DIRECTIVES = [directives_1.CORE_DIRECTIVES, forms_deprecated_1.FORM_DIRECTIVES];
  
});
System.registerDynamic('node_modules/@angular/common/src/location/hash_location_strategy.js', ['@angular/core', '../facade/lang', './location', './location_strategy', './platform_location'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../facade/lang');
    var location_1 = $__require('./location');
    var location_strategy_1 = $__require('./location_strategy');
    var platform_location_1 = $__require('./platform_location');
    var HashLocationStrategy = function (_super) {
        __extends(HashLocationStrategy, _super);
        function HashLocationStrategy(_platformLocation, _baseHref) {
            _super.call(this);
            this._platformLocation = _platformLocation;
            this._baseHref = '';
            if (lang_1.isPresent(_baseHref)) {
                this._baseHref = _baseHref;
            }
        }
        HashLocationStrategy.prototype.onPopState = function (fn) {
            this._platformLocation.onPopState(fn);
            this._platformLocation.onHashChange(fn);
        };
        HashLocationStrategy.prototype.getBaseHref = function () {
            return this._baseHref;
        };
        HashLocationStrategy.prototype.path = function (includeHash) {
            if (includeHash === void 0) {
                includeHash = false;
            }
            // the hash value is always prefixed with a `#`
            // and if it is empty then it will stay empty
            var path = this._platformLocation.hash;
            if (!lang_1.isPresent(path)) path = '#';
            // Dart will complain if a call to substring is
            // executed with a position value that extends the
            // length of string.
            return path.length > 0 ? path.substring(1) : path;
        };
        HashLocationStrategy.prototype.prepareExternalUrl = function (internal) {
            var url = location_1.Location.joinWithSlash(this._baseHref, internal);
            return url.length > 0 ? '#' + url : url;
        };
        HashLocationStrategy.prototype.pushState = function (state, title, path, queryParams) {
            var url = this.prepareExternalUrl(path + location_1.Location.normalizeQueryParams(queryParams));
            if (url.length == 0) {
                url = this._platformLocation.pathname;
            }
            this._platformLocation.pushState(state, title, url);
        };
        HashLocationStrategy.prototype.replaceState = function (state, title, path, queryParams) {
            var url = this.prepareExternalUrl(path + location_1.Location.normalizeQueryParams(queryParams));
            if (url.length == 0) {
                url = this._platformLocation.pathname;
            }
            this._platformLocation.replaceState(state, title, url);
        };
        HashLocationStrategy.prototype.forward = function () {
            this._platformLocation.forward();
        };
        HashLocationStrategy.prototype.back = function () {
            this._platformLocation.back();
        };
        /** @nocollapse */
        HashLocationStrategy.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        HashLocationStrategy.ctorParameters = [{ type: platform_location_1.PlatformLocation }, { type: undefined, decorators: [{ type: core_1.Optional }, { type: core_1.Inject, args: [location_strategy_1.APP_BASE_HREF] }] }];
        return HashLocationStrategy;
    }(location_strategy_1.LocationStrategy);
    exports.HashLocationStrategy = HashLocationStrategy;
    
});
System.registerDynamic("node_modules/@angular/common/src/facade/base_wrapped_exception.js", [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    /**
     * A base class for the WrappedException that can be used to identify
     * a WrappedException from ExceptionHandler without adding circular
     * dependency.
     */
    var BaseWrappedException = function (_super) {
        __extends(BaseWrappedException, _super);
        function BaseWrappedException(message) {
            _super.call(this, message);
        }
        Object.defineProperty(BaseWrappedException.prototype, "wrapperMessage", {
            get: function () {
                return '';
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "wrapperStack", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "originalException", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "originalStack", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "context", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "message", {
            get: function () {
                return '';
            },
            enumerable: true,
            configurable: true
        });
        return BaseWrappedException;
    }(Error);
    exports.BaseWrappedException = BaseWrappedException;
    
});
System.registerDynamic("node_modules/@angular/common/src/facade/collection.js", ["./lang"], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('./lang');
    exports.Map = lang_1.global.Map;
    exports.Set = lang_1.global.Set;
    // Safari and Internet Explorer do not support the iterable parameter to the
    // Map constructor.  We work around that by manually adding the items.
    var createMapFromPairs = function () {
        try {
            if (new exports.Map([[1, 2]]).size === 1) {
                return function createMapFromPairs(pairs) {
                    return new exports.Map(pairs);
                };
            }
        } catch (e) {}
        return function createMapAndPopulateFromPairs(pairs) {
            var map = new exports.Map();
            for (var i = 0; i < pairs.length; i++) {
                var pair = pairs[i];
                map.set(pair[0], pair[1]);
            }
            return map;
        };
    }();
    var createMapFromMap = function () {
        try {
            if (new exports.Map(new exports.Map())) {
                return function createMapFromMap(m) {
                    return new exports.Map(m);
                };
            }
        } catch (e) {}
        return function createMapAndPopulateFromMap(m) {
            var map = new exports.Map();
            m.forEach(function (v, k) {
                map.set(k, v);
            });
            return map;
        };
    }();
    var _clearValues = function () {
        if (new exports.Map().keys().next) {
            return function _clearValues(m) {
                var keyIterator = m.keys();
                var k;
                while (!(k = keyIterator.next()).done) {
                    m.set(k.value, null);
                }
            };
        } else {
            return function _clearValuesWithForeEach(m) {
                m.forEach(function (v, k) {
                    m.set(k, null);
                });
            };
        }
    }();
    // Safari doesn't implement MapIterator.next(), which is used is Traceur's polyfill of Array.from
    // TODO(mlaval): remove the work around once we have a working polyfill of Array.from
    var _arrayFromMap = function () {
        try {
            if (new exports.Map().values().next) {
                return function createArrayFromMap(m, getValues) {
                    return getValues ? Array.from(m.values()) : Array.from(m.keys());
                };
            }
        } catch (e) {}
        return function createArrayFromMapWithForeach(m, getValues) {
            var res = ListWrapper.createFixedSize(m.size),
                i = 0;
            m.forEach(function (v, k) {
                res[i] = getValues ? v : k;
                i++;
            });
            return res;
        };
    }();
    var MapWrapper = function () {
        function MapWrapper() {}
        MapWrapper.clone = function (m) {
            return createMapFromMap(m);
        };
        MapWrapper.createFromStringMap = function (stringMap) {
            var result = new exports.Map();
            for (var prop in stringMap) {
                result.set(prop, stringMap[prop]);
            }
            return result;
        };
        MapWrapper.toStringMap = function (m) {
            var r = {};
            m.forEach(function (v, k) {
                return r[k] = v;
            });
            return r;
        };
        MapWrapper.createFromPairs = function (pairs) {
            return createMapFromPairs(pairs);
        };
        MapWrapper.clearValues = function (m) {
            _clearValues(m);
        };
        MapWrapper.iterable = function (m) {
            return m;
        };
        MapWrapper.keys = function (m) {
            return _arrayFromMap(m, false);
        };
        MapWrapper.values = function (m) {
            return _arrayFromMap(m, true);
        };
        return MapWrapper;
    }();
    exports.MapWrapper = MapWrapper;
    /**
     * Wraps Javascript Objects
     */
    var StringMapWrapper = function () {
        function StringMapWrapper() {}
        StringMapWrapper.create = function () {
            // Note: We are not using Object.create(null) here due to
            // performance!
            // http://jsperf.com/ng2-object-create-null
            return {};
        };
        StringMapWrapper.contains = function (map, key) {
            return map.hasOwnProperty(key);
        };
        StringMapWrapper.get = function (map, key) {
            return map.hasOwnProperty(key) ? map[key] : undefined;
        };
        StringMapWrapper.set = function (map, key, value) {
            map[key] = value;
        };
        StringMapWrapper.keys = function (map) {
            return Object.keys(map);
        };
        StringMapWrapper.values = function (map) {
            return Object.keys(map).reduce(function (r, a) {
                r.push(map[a]);
                return r;
            }, []);
        };
        StringMapWrapper.isEmpty = function (map) {
            for (var prop in map) {
                return false;
            }
            return true;
        };
        StringMapWrapper.delete = function (map, key) {
            delete map[key];
        };
        StringMapWrapper.forEach = function (map, callback) {
            for (var prop in map) {
                if (map.hasOwnProperty(prop)) {
                    callback(map[prop], prop);
                }
            }
        };
        StringMapWrapper.merge = function (m1, m2) {
            var m = {};
            for (var attr in m1) {
                if (m1.hasOwnProperty(attr)) {
                    m[attr] = m1[attr];
                }
            }
            for (var attr in m2) {
                if (m2.hasOwnProperty(attr)) {
                    m[attr] = m2[attr];
                }
            }
            return m;
        };
        StringMapWrapper.equals = function (m1, m2) {
            var k1 = Object.keys(m1);
            var k2 = Object.keys(m2);
            if (k1.length != k2.length) {
                return false;
            }
            var key;
            for (var i = 0; i < k1.length; i++) {
                key = k1[i];
                if (m1[key] !== m2[key]) {
                    return false;
                }
            }
            return true;
        };
        return StringMapWrapper;
    }();
    exports.StringMapWrapper = StringMapWrapper;
    var ListWrapper = function () {
        function ListWrapper() {}
        // JS has no way to express a statically fixed size list, but dart does so we
        // keep both methods.
        ListWrapper.createFixedSize = function (size) {
            return new Array(size);
        };
        ListWrapper.createGrowableSize = function (size) {
            return new Array(size);
        };
        ListWrapper.clone = function (array) {
            return array.slice(0);
        };
        ListWrapper.forEachWithIndex = function (array, fn) {
            for (var i = 0; i < array.length; i++) {
                fn(array[i], i);
            }
        };
        ListWrapper.first = function (array) {
            if (!array) return null;
            return array[0];
        };
        ListWrapper.last = function (array) {
            if (!array || array.length == 0) return null;
            return array[array.length - 1];
        };
        ListWrapper.indexOf = function (array, value, startIndex) {
            if (startIndex === void 0) {
                startIndex = 0;
            }
            return array.indexOf(value, startIndex);
        };
        ListWrapper.contains = function (list, el) {
            return list.indexOf(el) !== -1;
        };
        ListWrapper.reversed = function (array) {
            var a = ListWrapper.clone(array);
            return a.reverse();
        };
        ListWrapper.concat = function (a, b) {
            return a.concat(b);
        };
        ListWrapper.insert = function (list, index, value) {
            list.splice(index, 0, value);
        };
        ListWrapper.removeAt = function (list, index) {
            var res = list[index];
            list.splice(index, 1);
            return res;
        };
        ListWrapper.removeAll = function (list, items) {
            for (var i = 0; i < items.length; ++i) {
                var index = list.indexOf(items[i]);
                list.splice(index, 1);
            }
        };
        ListWrapper.remove = function (list, el) {
            var index = list.indexOf(el);
            if (index > -1) {
                list.splice(index, 1);
                return true;
            }
            return false;
        };
        ListWrapper.clear = function (list) {
            list.length = 0;
        };
        ListWrapper.isEmpty = function (list) {
            return list.length == 0;
        };
        ListWrapper.fill = function (list, value, start, end) {
            if (start === void 0) {
                start = 0;
            }
            if (end === void 0) {
                end = null;
            }
            list.fill(value, start, end === null ? list.length : end);
        };
        ListWrapper.equals = function (a, b) {
            if (a.length != b.length) return false;
            for (var i = 0; i < a.length; ++i) {
                if (a[i] !== b[i]) return false;
            }
            return true;
        };
        ListWrapper.slice = function (l, from, to) {
            if (from === void 0) {
                from = 0;
            }
            if (to === void 0) {
                to = null;
            }
            return l.slice(from, to === null ? undefined : to);
        };
        ListWrapper.splice = function (l, from, length) {
            return l.splice(from, length);
        };
        ListWrapper.sort = function (l, compareFn) {
            if (lang_1.isPresent(compareFn)) {
                l.sort(compareFn);
            } else {
                l.sort();
            }
        };
        ListWrapper.toString = function (l) {
            return l.toString();
        };
        ListWrapper.toJSON = function (l) {
            return JSON.stringify(l);
        };
        ListWrapper.maximum = function (list, predicate) {
            if (list.length == 0) {
                return null;
            }
            var solution = null;
            var maxValue = -Infinity;
            for (var index = 0; index < list.length; index++) {
                var candidate = list[index];
                if (lang_1.isBlank(candidate)) {
                    continue;
                }
                var candidateValue = predicate(candidate);
                if (candidateValue > maxValue) {
                    solution = candidate;
                    maxValue = candidateValue;
                }
            }
            return solution;
        };
        ListWrapper.flatten = function (list) {
            var target = [];
            _flattenArray(list, target);
            return target;
        };
        ListWrapper.addAll = function (list, source) {
            for (var i = 0; i < source.length; i++) {
                list.push(source[i]);
            }
        };
        return ListWrapper;
    }();
    exports.ListWrapper = ListWrapper;
    function _flattenArray(source, target) {
        if (lang_1.isPresent(source)) {
            for (var i = 0; i < source.length; i++) {
                var item = source[i];
                if (lang_1.isArray(item)) {
                    _flattenArray(item, target);
                } else {
                    target.push(item);
                }
            }
        }
        return target;
    }
    function isListLikeIterable(obj) {
        if (!lang_1.isJsObject(obj)) return false;
        return lang_1.isArray(obj) || !(obj instanceof exports.Map) && lang_1.getSymbolIterator() in obj; // JS Iterable have a Symbol.iterator prop
    }
    exports.isListLikeIterable = isListLikeIterable;
    function areIterablesEqual(a, b, comparator) {
        var iterator1 = a[lang_1.getSymbolIterator()]();
        var iterator2 = b[lang_1.getSymbolIterator()]();
        while (true) {
            var item1 = iterator1.next();
            var item2 = iterator2.next();
            if (item1.done && item2.done) return true;
            if (item1.done || item2.done) return false;
            if (!comparator(item1.value, item2.value)) return false;
        }
    }
    exports.areIterablesEqual = areIterablesEqual;
    function iterateListLike(obj, fn) {
        if (lang_1.isArray(obj)) {
            for (var i = 0; i < obj.length; i++) {
                fn(obj[i]);
            }
        } else {
            var iterator = obj[lang_1.getSymbolIterator()]();
            var item;
            while (!(item = iterator.next()).done) {
                fn(item.value);
            }
        }
    }
    exports.iterateListLike = iterateListLike;
    // Safari and Internet Explorer do not support the iterable parameter to the
    // Set constructor.  We work around that by manually adding the items.
    var createSetFromList = function () {
        var test = new exports.Set([1, 2, 3]);
        if (test.size === 3) {
            return function createSetFromList(lst) {
                return new exports.Set(lst);
            };
        } else {
            return function createSetAndPopulateFromList(lst) {
                var res = new exports.Set(lst);
                if (res.size !== lst.length) {
                    for (var i = 0; i < lst.length; i++) {
                        res.add(lst[i]);
                    }
                }
                return res;
            };
        }
    }();
    var SetWrapper = function () {
        function SetWrapper() {}
        SetWrapper.createFromList = function (lst) {
            return createSetFromList(lst);
        };
        SetWrapper.has = function (s, key) {
            return s.has(key);
        };
        SetWrapper.delete = function (m, k) {
            m.delete(k);
        };
        return SetWrapper;
    }();
    exports.SetWrapper = SetWrapper;
    
});
System.registerDynamic('node_modules/@angular/common/src/facade/exception_handler.js', ['./base_wrapped_exception', './collection', './lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var base_wrapped_exception_1 = $__require('./base_wrapped_exception');
    var collection_1 = $__require('./collection');
    var lang_1 = $__require('./lang');
    var _ArrayLogger = function () {
        function _ArrayLogger() {
            this.res = [];
        }
        _ArrayLogger.prototype.log = function (s) {
            this.res.push(s);
        };
        _ArrayLogger.prototype.logError = function (s) {
            this.res.push(s);
        };
        _ArrayLogger.prototype.logGroup = function (s) {
            this.res.push(s);
        };
        _ArrayLogger.prototype.logGroupEnd = function () {};
        ;
        return _ArrayLogger;
    }();
    /**
     * Provides a hook for centralized exception handling.
     *
     * The default implementation of `ExceptionHandler` prints error messages to the `Console`. To
     * intercept error handling,
     * write a custom exception handler that replaces this default as appropriate for your app.
     *
     * ### Example
     *
     * ```javascript
     *
     * class MyExceptionHandler implements ExceptionHandler {
     *   call(error, stackTrace = null, reason = null) {
     *     // do something with the exception
     *   }
     * }
     *
     * bootstrap(MyApp, {provide: ExceptionHandler, useClass: MyExceptionHandler}])
     *
     * ```
     * @stable
     */
    var ExceptionHandler = function () {
        function ExceptionHandler(_logger, _rethrowException) {
            if (_rethrowException === void 0) {
                _rethrowException = true;
            }
            this._logger = _logger;
            this._rethrowException = _rethrowException;
        }
        ExceptionHandler.exceptionToString = function (exception, stackTrace, reason) {
            if (stackTrace === void 0) {
                stackTrace = null;
            }
            if (reason === void 0) {
                reason = null;
            }
            var l = new _ArrayLogger();
            var e = new ExceptionHandler(l, false);
            e.call(exception, stackTrace, reason);
            return l.res.join('\n');
        };
        ExceptionHandler.prototype.call = function (exception, stackTrace, reason) {
            if (stackTrace === void 0) {
                stackTrace = null;
            }
            if (reason === void 0) {
                reason = null;
            }
            var originalException = this._findOriginalException(exception);
            var originalStack = this._findOriginalStack(exception);
            var context = this._findContext(exception);
            this._logger.logGroup("EXCEPTION: " + this._extractMessage(exception));
            if (lang_1.isPresent(stackTrace) && lang_1.isBlank(originalStack)) {
                this._logger.logError('STACKTRACE:');
                this._logger.logError(this._longStackTrace(stackTrace));
            }
            if (lang_1.isPresent(reason)) {
                this._logger.logError("REASON: " + reason);
            }
            if (lang_1.isPresent(originalException)) {
                this._logger.logError("ORIGINAL EXCEPTION: " + this._extractMessage(originalException));
            }
            if (lang_1.isPresent(originalStack)) {
                this._logger.logError('ORIGINAL STACKTRACE:');
                this._logger.logError(this._longStackTrace(originalStack));
            }
            if (lang_1.isPresent(context)) {
                this._logger.logError('ERROR CONTEXT:');
                this._logger.logError(context);
            }
            this._logger.logGroupEnd();
            // We rethrow exceptions, so operations like 'bootstrap' will result in an error
            // when an exception happens. If we do not rethrow, bootstrap will always succeed.
            if (this._rethrowException) throw exception;
        };
        /** @internal */
        ExceptionHandler.prototype._extractMessage = function (exception) {
            return exception instanceof base_wrapped_exception_1.BaseWrappedException ? exception.wrapperMessage : exception.toString();
        };
        /** @internal */
        ExceptionHandler.prototype._longStackTrace = function (stackTrace) {
            return collection_1.isListLikeIterable(stackTrace) ? stackTrace.join('\n\n-----async gap-----\n') : stackTrace.toString();
        };
        /** @internal */
        ExceptionHandler.prototype._findContext = function (exception) {
            try {
                if (!(exception instanceof base_wrapped_exception_1.BaseWrappedException)) return null;
                return lang_1.isPresent(exception.context) ? exception.context : this._findContext(exception.originalException);
            } catch (e) {
                // exception.context can throw an exception. if it happens, we ignore the context.
                return null;
            }
        };
        /** @internal */
        ExceptionHandler.prototype._findOriginalException = function (exception) {
            if (!(exception instanceof base_wrapped_exception_1.BaseWrappedException)) return null;
            var e = exception.originalException;
            while (e instanceof base_wrapped_exception_1.BaseWrappedException && lang_1.isPresent(e.originalException)) {
                e = e.originalException;
            }
            return e;
        };
        /** @internal */
        ExceptionHandler.prototype._findOriginalStack = function (exception) {
            if (!(exception instanceof base_wrapped_exception_1.BaseWrappedException)) return null;
            var e = exception;
            var stack = exception.originalStack;
            while (e instanceof base_wrapped_exception_1.BaseWrappedException && lang_1.isPresent(e.originalException)) {
                e = e.originalException;
                if (e instanceof base_wrapped_exception_1.BaseWrappedException && lang_1.isPresent(e.originalException)) {
                    stack = e.originalStack;
                }
            }
            return stack;
        };
        return ExceptionHandler;
    }();
    exports.ExceptionHandler = ExceptionHandler;
    
});
System.registerDynamic('node_modules/@angular/common/src/facade/exceptions.js', ['./base_wrapped_exception', './exception_handler'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var base_wrapped_exception_1 = $__require('./base_wrapped_exception');
    var exception_handler_1 = $__require('./exception_handler');
    var exception_handler_2 = $__require('./exception_handler');
    exports.ExceptionHandler = exception_handler_2.ExceptionHandler;
    /**
     * @stable
     */
    var BaseException = function (_super) {
        __extends(BaseException, _super);
        function BaseException(message) {
            if (message === void 0) {
                message = '--';
            }
            _super.call(this, message);
            this.message = message;
            this.stack = new Error(message).stack;
        }
        BaseException.prototype.toString = function () {
            return this.message;
        };
        return BaseException;
    }(Error);
    exports.BaseException = BaseException;
    /**
     * Wraps an exception and provides additional context or information.
     * @stable
     */
    var WrappedException = function (_super) {
        __extends(WrappedException, _super);
        function WrappedException(_wrapperMessage, _originalException /** TODO #9100 */, _originalStack /** TODO #9100 */, _context /** TODO #9100 */) {
            _super.call(this, _wrapperMessage);
            this._wrapperMessage = _wrapperMessage;
            this._originalException = _originalException;
            this._originalStack = _originalStack;
            this._context = _context;
            this._wrapperStack = new Error(_wrapperMessage).stack;
        }
        Object.defineProperty(WrappedException.prototype, "wrapperMessage", {
            get: function () {
                return this._wrapperMessage;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "wrapperStack", {
            get: function () {
                return this._wrapperStack;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "originalException", {
            get: function () {
                return this._originalException;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "originalStack", {
            get: function () {
                return this._originalStack;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "context", {
            get: function () {
                return this._context;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "message", {
            get: function () {
                return exception_handler_1.ExceptionHandler.exceptionToString(this);
            },
            enumerable: true,
            configurable: true
        });
        WrappedException.prototype.toString = function () {
            return this.message;
        };
        return WrappedException;
    }(base_wrapped_exception_1.BaseWrappedException);
    exports.WrappedException = WrappedException;
    function makeTypeError(message) {
        return new TypeError(message);
    }
    exports.makeTypeError = makeTypeError;
    function unimplemented() {
        throw new BaseException('unimplemented');
    }
    exports.unimplemented = unimplemented;
    
});
System.registerDynamic("node_modules/@angular/common/src/location/platform_location.js", [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";
    /**
     * This class should not be used directly by an application developer. Instead, use
     * {@link Location}.
     *
     * `PlatformLocation` encapsulates all calls to DOM apis, which allows the Router to be platform
     * agnostic.
     * This means that we can have different implementation of `PlatformLocation` for the different
     * platforms
     * that angular supports. For example, the default `PlatformLocation` is {@link
     * BrowserPlatformLocation},
     * however when you run your app in a WebWorker you use {@link WebWorkerPlatformLocation}.
     *
     * The `PlatformLocation` class is used directly by all implementations of {@link LocationStrategy}
     * when
     * they need to interact with the DOM apis like pushState, popState, etc...
     *
     * {@link LocationStrategy} in turn is used by the {@link Location} service which is used directly
     * by
     * the {@link Router} in order to navigate between routes. Since all interactions between {@link
     * Router} /
     * {@link Location} / {@link LocationStrategy} and DOM apis flow through the `PlatformLocation`
     * class
     * they are all platform independent.
     *
     * @stable
     */

    var global = this || self,
        GLOBAL = global;
    var PlatformLocation = function () {
        function PlatformLocation() {}
        Object.defineProperty(PlatformLocation.prototype, "pathname", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(PlatformLocation.prototype, "search", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(PlatformLocation.prototype, "hash", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        return PlatformLocation;
    }();
    exports.PlatformLocation = PlatformLocation;
    
});
System.registerDynamic('node_modules/@angular/common/src/location/path_location_strategy.js', ['@angular/core', '../facade/exceptions', '../facade/lang', './location', './location_strategy', './platform_location'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    var location_1 = $__require('./location');
    var location_strategy_1 = $__require('./location_strategy');
    var platform_location_1 = $__require('./platform_location');
    var PathLocationStrategy = function (_super) {
        __extends(PathLocationStrategy, _super);
        function PathLocationStrategy(_platformLocation, href) {
            _super.call(this);
            this._platformLocation = _platformLocation;
            if (lang_1.isBlank(href)) {
                href = this._platformLocation.getBaseHrefFromDOM();
            }
            if (lang_1.isBlank(href)) {
                throw new exceptions_1.BaseException("No base href set. Please provide a value for the APP_BASE_HREF token or add a base element to the document.");
            }
            this._baseHref = href;
        }
        PathLocationStrategy.prototype.onPopState = function (fn) {
            this._platformLocation.onPopState(fn);
            this._platformLocation.onHashChange(fn);
        };
        PathLocationStrategy.prototype.getBaseHref = function () {
            return this._baseHref;
        };
        PathLocationStrategy.prototype.prepareExternalUrl = function (internal) {
            return location_1.Location.joinWithSlash(this._baseHref, internal);
        };
        PathLocationStrategy.prototype.path = function (includeHash) {
            if (includeHash === void 0) {
                includeHash = false;
            }
            var pathname = this._platformLocation.pathname + location_1.Location.normalizeQueryParams(this._platformLocation.search);
            var hash = this._platformLocation.hash;
            return hash && includeHash ? "" + pathname + hash : pathname;
        };
        PathLocationStrategy.prototype.pushState = function (state, title, url, queryParams) {
            var externalUrl = this.prepareExternalUrl(url + location_1.Location.normalizeQueryParams(queryParams));
            this._platformLocation.pushState(state, title, externalUrl);
        };
        PathLocationStrategy.prototype.replaceState = function (state, title, url, queryParams) {
            var externalUrl = this.prepareExternalUrl(url + location_1.Location.normalizeQueryParams(queryParams));
            this._platformLocation.replaceState(state, title, externalUrl);
        };
        PathLocationStrategy.prototype.forward = function () {
            this._platformLocation.forward();
        };
        PathLocationStrategy.prototype.back = function () {
            this._platformLocation.back();
        };
        /** @nocollapse */
        PathLocationStrategy.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        PathLocationStrategy.ctorParameters = [{ type: platform_location_1.PlatformLocation }, { type: undefined, decorators: [{ type: core_1.Optional }, { type: core_1.Inject, args: [location_strategy_1.APP_BASE_HREF] }] }];
        return PathLocationStrategy;
    }(location_strategy_1.LocationStrategy);
    exports.PathLocationStrategy = PathLocationStrategy;
    
});
System.registerDynamic('node_modules/@angular/common/src/facade/lang.js', [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var globalScope;
    if (typeof window === 'undefined') {
        if (typeof WorkerGlobalScope !== 'undefined' && self instanceof WorkerGlobalScope) {
            // TODO: Replace any with WorkerGlobalScope from lib.webworker.d.ts #3492
            globalScope = self;
        } else {
            globalScope = global;
        }
    } else {
        globalScope = window;
    }
    function scheduleMicroTask(fn) {
        Zone.current.scheduleMicroTask('scheduleMicrotask', fn);
    }
    exports.scheduleMicroTask = scheduleMicroTask;
    exports.IS_DART = false;
    // Need to declare a new variable for global here since TypeScript
    // exports the original value of the symbol.
    var _global = globalScope;
    exports.global = _global;
    /**
     * Runtime representation a type that a Component or other object is instances of.
     *
     * An example of a `Type` is `MyCustomComponent` class, which in JavaScript is be represented by
     * the `MyCustomComponent` constructor function.
     *
     * @stable
     */
    exports.Type = Function;
    function getTypeNameForDebugging(type) {
        if (type['name']) {
            return type['name'];
        }
        return typeof type;
    }
    exports.getTypeNameForDebugging = getTypeNameForDebugging;
    exports.Math = _global.Math;
    exports.Date = _global.Date;
    // TODO: remove calls to assert in production environment
    // Note: Can't just export this and import in in other files
    // as `assert` is a reserved keyword in Dart
    _global.assert = function assert(condition) {
        // TODO: to be fixed properly via #2830, noop for now
    };
    function isPresent(obj) {
        return obj !== undefined && obj !== null;
    }
    exports.isPresent = isPresent;
    function isBlank(obj) {
        return obj === undefined || obj === null;
    }
    exports.isBlank = isBlank;
    function isBoolean(obj) {
        return typeof obj === 'boolean';
    }
    exports.isBoolean = isBoolean;
    function isNumber(obj) {
        return typeof obj === 'number';
    }
    exports.isNumber = isNumber;
    function isString(obj) {
        return typeof obj === 'string';
    }
    exports.isString = isString;
    function isFunction(obj) {
        return typeof obj === 'function';
    }
    exports.isFunction = isFunction;
    function isType(obj) {
        return isFunction(obj);
    }
    exports.isType = isType;
    function isStringMap(obj) {
        return typeof obj === 'object' && obj !== null;
    }
    exports.isStringMap = isStringMap;
    var STRING_MAP_PROTO = Object.getPrototypeOf({});
    function isStrictStringMap(obj) {
        return isStringMap(obj) && Object.getPrototypeOf(obj) === STRING_MAP_PROTO;
    }
    exports.isStrictStringMap = isStrictStringMap;
    function isPromise(obj) {
        return obj instanceof _global.Promise;
    }
    exports.isPromise = isPromise;
    function isArray(obj) {
        return Array.isArray(obj);
    }
    exports.isArray = isArray;
    function isDate(obj) {
        return obj instanceof exports.Date && !isNaN(obj.valueOf());
    }
    exports.isDate = isDate;
    function noop() {}
    exports.noop = noop;
    function stringify(token) {
        if (typeof token === 'string') {
            return token;
        }
        if (token === undefined || token === null) {
            return '' + token;
        }
        if (token.name) {
            return token.name;
        }
        if (token.overriddenName) {
            return token.overriddenName;
        }
        var res = token.toString();
        var newLineIndex = res.indexOf('\n');
        return newLineIndex === -1 ? res : res.substring(0, newLineIndex);
    }
    exports.stringify = stringify;
    // serialize / deserialize enum exist only for consistency with dart API
    // enums in typescript don't need to be serialized
    function serializeEnum(val) {
        return val;
    }
    exports.serializeEnum = serializeEnum;
    function deserializeEnum(val, values) {
        return val;
    }
    exports.deserializeEnum = deserializeEnum;
    function resolveEnumToken(enumValue, val) {
        return enumValue[val];
    }
    exports.resolveEnumToken = resolveEnumToken;
    var StringWrapper = function () {
        function StringWrapper() {}
        StringWrapper.fromCharCode = function (code) {
            return String.fromCharCode(code);
        };
        StringWrapper.charCodeAt = function (s, index) {
            return s.charCodeAt(index);
        };
        StringWrapper.split = function (s, regExp) {
            return s.split(regExp);
        };
        StringWrapper.equals = function (s, s2) {
            return s === s2;
        };
        StringWrapper.stripLeft = function (s, charVal) {
            if (s && s.length) {
                var pos = 0;
                for (var i = 0; i < s.length; i++) {
                    if (s[i] != charVal) break;
                    pos++;
                }
                s = s.substring(pos);
            }
            return s;
        };
        StringWrapper.stripRight = function (s, charVal) {
            if (s && s.length) {
                var pos = s.length;
                for (var i = s.length - 1; i >= 0; i--) {
                    if (s[i] != charVal) break;
                    pos--;
                }
                s = s.substring(0, pos);
            }
            return s;
        };
        StringWrapper.replace = function (s, from, replace) {
            return s.replace(from, replace);
        };
        StringWrapper.replaceAll = function (s, from, replace) {
            return s.replace(from, replace);
        };
        StringWrapper.slice = function (s, from, to) {
            if (from === void 0) {
                from = 0;
            }
            if (to === void 0) {
                to = null;
            }
            return s.slice(from, to === null ? undefined : to);
        };
        StringWrapper.replaceAllMapped = function (s, from, cb) {
            return s.replace(from, function () {
                var matches = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    matches[_i - 0] = arguments[_i];
                }
                // Remove offset & string from the result array
                matches.splice(-2, 2);
                // The callback receives match, p1, ..., pn
                return cb(matches);
            });
        };
        StringWrapper.contains = function (s, substr) {
            return s.indexOf(substr) != -1;
        };
        StringWrapper.compare = function (a, b) {
            if (a < b) {
                return -1;
            } else if (a > b) {
                return 1;
            } else {
                return 0;
            }
        };
        return StringWrapper;
    }();
    exports.StringWrapper = StringWrapper;
    var StringJoiner = function () {
        function StringJoiner(parts) {
            if (parts === void 0) {
                parts = [];
            }
            this.parts = parts;
        }
        StringJoiner.prototype.add = function (part) {
            this.parts.push(part);
        };
        StringJoiner.prototype.toString = function () {
            return this.parts.join('');
        };
        return StringJoiner;
    }();
    exports.StringJoiner = StringJoiner;
    var NumberParseError = function (_super) {
        __extends(NumberParseError, _super);
        function NumberParseError(message) {
            _super.call(this);
            this.message = message;
        }
        NumberParseError.prototype.toString = function () {
            return this.message;
        };
        return NumberParseError;
    }(Error);
    exports.NumberParseError = NumberParseError;
    var NumberWrapper = function () {
        function NumberWrapper() {}
        NumberWrapper.toFixed = function (n, fractionDigits) {
            return n.toFixed(fractionDigits);
        };
        NumberWrapper.equal = function (a, b) {
            return a === b;
        };
        NumberWrapper.parseIntAutoRadix = function (text) {
            var result = parseInt(text);
            if (isNaN(result)) {
                throw new NumberParseError('Invalid integer literal when parsing ' + text);
            }
            return result;
        };
        NumberWrapper.parseInt = function (text, radix) {
            if (radix == 10) {
                if (/^(\-|\+)?[0-9]+$/.test(text)) {
                    return parseInt(text, radix);
                }
            } else if (radix == 16) {
                if (/^(\-|\+)?[0-9ABCDEFabcdef]+$/.test(text)) {
                    return parseInt(text, radix);
                }
            } else {
                var result = parseInt(text, radix);
                if (!isNaN(result)) {
                    return result;
                }
            }
            throw new NumberParseError('Invalid integer literal when parsing ' + text + ' in base ' + radix);
        };
        // TODO: NaN is a valid literal but is returned by parseFloat to indicate an error.
        NumberWrapper.parseFloat = function (text) {
            return parseFloat(text);
        };
        Object.defineProperty(NumberWrapper, "NaN", {
            get: function () {
                return NaN;
            },
            enumerable: true,
            configurable: true
        });
        NumberWrapper.isNumeric = function (value) {
            return !isNaN(value - parseFloat(value));
        };
        NumberWrapper.isNaN = function (value) {
            return isNaN(value);
        };
        NumberWrapper.isInteger = function (value) {
            return Number.isInteger(value);
        };
        return NumberWrapper;
    }();
    exports.NumberWrapper = NumberWrapper;
    exports.RegExp = _global.RegExp;
    var RegExpWrapper = function () {
        function RegExpWrapper() {}
        RegExpWrapper.create = function (regExpStr, flags) {
            if (flags === void 0) {
                flags = '';
            }
            flags = flags.replace(/g/g, '');
            return new _global.RegExp(regExpStr, flags + 'g');
        };
        RegExpWrapper.firstMatch = function (regExp, input) {
            // Reset multimatch regex state
            regExp.lastIndex = 0;
            return regExp.exec(input);
        };
        RegExpWrapper.test = function (regExp, input) {
            regExp.lastIndex = 0;
            return regExp.test(input);
        };
        RegExpWrapper.matcher = function (regExp, input) {
            // Reset regex state for the case
            // someone did not loop over all matches
            // last time.
            regExp.lastIndex = 0;
            return { re: regExp, input: input };
        };
        RegExpWrapper.replaceAll = function (regExp, input, replace) {
            var c = regExp.exec(input);
            var res = '';
            regExp.lastIndex = 0;
            var prev = 0;
            while (c) {
                res += input.substring(prev, c.index);
                res += replace(c);
                prev = c.index + c[0].length;
                regExp.lastIndex = prev;
                c = regExp.exec(input);
            }
            res += input.substring(prev);
            return res;
        };
        return RegExpWrapper;
    }();
    exports.RegExpWrapper = RegExpWrapper;
    var RegExpMatcherWrapper = function () {
        function RegExpMatcherWrapper() {}
        RegExpMatcherWrapper.next = function (matcher) {
            return matcher.re.exec(matcher.input);
        };
        return RegExpMatcherWrapper;
    }();
    exports.RegExpMatcherWrapper = RegExpMatcherWrapper;
    var FunctionWrapper = function () {
        function FunctionWrapper() {}
        FunctionWrapper.apply = function (fn, posArgs) {
            return fn.apply(null, posArgs);
        };
        FunctionWrapper.bind = function (fn, scope) {
            return fn.bind(scope);
        };
        return FunctionWrapper;
    }();
    exports.FunctionWrapper = FunctionWrapper;
    // JS has NaN !== NaN
    function looseIdentical(a, b) {
        return a === b || typeof a === 'number' && typeof b === 'number' && isNaN(a) && isNaN(b);
    }
    exports.looseIdentical = looseIdentical;
    // JS considers NaN is the same as NaN for map Key (while NaN !== NaN otherwise)
    // see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Map
    function getMapKey(value) {
        return value;
    }
    exports.getMapKey = getMapKey;
    function normalizeBlank(obj) {
        return isBlank(obj) ? null : obj;
    }
    exports.normalizeBlank = normalizeBlank;
    function normalizeBool(obj) {
        return isBlank(obj) ? false : obj;
    }
    exports.normalizeBool = normalizeBool;
    function isJsObject(o) {
        return o !== null && (typeof o === 'function' || typeof o === 'object');
    }
    exports.isJsObject = isJsObject;
    function print(obj) {
        console.log(obj);
    }
    exports.print = print;
    function warn(obj) {
        console.warn(obj);
    }
    exports.warn = warn;
    // Can't be all uppercase as our transpiler would think it is a special directive...
    var Json = function () {
        function Json() {}
        Json.parse = function (s) {
            return _global.JSON.parse(s);
        };
        Json.stringify = function (data) {
            // Dart doesn't take 3 arguments
            return _global.JSON.stringify(data, null, 2);
        };
        return Json;
    }();
    exports.Json = Json;
    var DateWrapper = function () {
        function DateWrapper() {}
        DateWrapper.create = function (year, month, day, hour, minutes, seconds, milliseconds) {
            if (month === void 0) {
                month = 1;
            }
            if (day === void 0) {
                day = 1;
            }
            if (hour === void 0) {
                hour = 0;
            }
            if (minutes === void 0) {
                minutes = 0;
            }
            if (seconds === void 0) {
                seconds = 0;
            }
            if (milliseconds === void 0) {
                milliseconds = 0;
            }
            return new exports.Date(year, month - 1, day, hour, minutes, seconds, milliseconds);
        };
        DateWrapper.fromISOString = function (str) {
            return new exports.Date(str);
        };
        DateWrapper.fromMillis = function (ms) {
            return new exports.Date(ms);
        };
        DateWrapper.toMillis = function (date) {
            return date.getTime();
        };
        DateWrapper.now = function () {
            return new exports.Date();
        };
        DateWrapper.toJson = function (date) {
            return date.toJSON();
        };
        return DateWrapper;
    }();
    exports.DateWrapper = DateWrapper;
    function setValueOnPath(global, path, value) {
        var parts = path.split('.');
        var obj = global;
        while (parts.length > 1) {
            var name = parts.shift();
            if (obj.hasOwnProperty(name) && isPresent(obj[name])) {
                obj = obj[name];
            } else {
                obj = obj[name] = {};
            }
        }
        if (obj === undefined || obj === null) {
            obj = {};
        }
        obj[parts.shift()] = value;
    }
    exports.setValueOnPath = setValueOnPath;
    var _symbolIterator = null;
    function getSymbolIterator() {
        if (isBlank(_symbolIterator)) {
            if (isPresent(globalScope.Symbol) && isPresent(Symbol.iterator)) {
                _symbolIterator = Symbol.iterator;
            } else {
                // es6-shim specific logic
                var keys = Object.getOwnPropertyNames(Map.prototype);
                for (var i = 0; i < keys.length; ++i) {
                    var key = keys[i];
                    if (key !== 'entries' && key !== 'size' && Map.prototype[key] === Map.prototype['entries']) {
                        _symbolIterator = key;
                    }
                }
            }
        }
        return _symbolIterator;
    }
    exports.getSymbolIterator = getSymbolIterator;
    function evalExpression(sourceUrl, expr, declarations, vars) {
        var fnBody = declarations + "\nreturn " + expr + "\n//# sourceURL=" + sourceUrl;
        var fnArgNames = [];
        var fnArgValues = [];
        for (var argName in vars) {
            fnArgNames.push(argName);
            fnArgValues.push(vars[argName]);
        }
        return new (Function.bind.apply(Function, [void 0].concat(fnArgNames.concat(fnBody))))().apply(void 0, fnArgValues);
    }
    exports.evalExpression = evalExpression;
    function isPrimitive(obj) {
        return !isJsObject(obj);
    }
    exports.isPrimitive = isPrimitive;
    function hasConstructor(value, type) {
        return value.constructor === type;
    }
    exports.hasConstructor = hasConstructor;
    function escape(s) {
        return _global.encodeURI(s);
    }
    exports.escape = escape;
    function escapeRegExp(s) {
        return s.replace(/([.*+?^=!:${}()|[\]\/\\])/g, '\\$1');
    }
    exports.escapeRegExp = escapeRegExp;
    
});
System.registerDynamic("node_modules/@angular/common/src/facade/promise.js", [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var PromiseCompleter = function () {
        function PromiseCompleter() {
            var _this = this;
            this.promise = new Promise(function (res, rej) {
                _this.resolve = res;
                _this.reject = rej;
            });
        }
        return PromiseCompleter;
    }();
    exports.PromiseCompleter = PromiseCompleter;
    var PromiseWrapper = function () {
        function PromiseWrapper() {}
        PromiseWrapper.resolve = function (obj) {
            return Promise.resolve(obj);
        };
        PromiseWrapper.reject = function (obj, _) {
            return Promise.reject(obj);
        };
        // Note: We can't rename this method into `catch`, as this is not a valid
        // method name in Dart.
        PromiseWrapper.catchError = function (promise, onError) {
            return promise.catch(onError);
        };
        PromiseWrapper.all = function (promises) {
            if (promises.length == 0) return Promise.resolve([]);
            return Promise.all(promises);
        };
        PromiseWrapper.then = function (promise, success, rejection) {
            return promise.then(success, rejection);
        };
        PromiseWrapper.wrap = function (computation) {
            return new Promise(function (res, rej) {
                try {
                    res(computation());
                } catch (e) {
                    rej(e);
                }
            });
        };
        PromiseWrapper.scheduleMicrotask = function (computation) {
            PromiseWrapper.then(PromiseWrapper.resolve(null), computation, function (_) {});
        };
        PromiseWrapper.completer = function () {
            return new PromiseCompleter();
        };
        return PromiseWrapper;
    }();
    exports.PromiseWrapper = PromiseWrapper;
    
});
System.registerDynamic('node_modules/@angular/common/src/facade/async.js', ['rxjs/Subject', 'rxjs/observable/PromiseObservable', 'rxjs/operator/toPromise', './lang', 'rxjs/Observable', './promise'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subject_1 = $__require('rxjs/Subject');
    var PromiseObservable_1 = $__require('rxjs/observable/PromiseObservable');
    var toPromise_1 = $__require('rxjs/operator/toPromise');
    var lang_1 = $__require('./lang');
    var Observable_1 = $__require('rxjs/Observable');
    exports.Observable = Observable_1.Observable;
    var Subject_2 = $__require('rxjs/Subject');
    exports.Subject = Subject_2.Subject;
    var promise_1 = $__require('./promise');
    exports.PromiseCompleter = promise_1.PromiseCompleter;
    exports.PromiseWrapper = promise_1.PromiseWrapper;
    var TimerWrapper = function () {
        function TimerWrapper() {}
        TimerWrapper.setTimeout = function (fn, millis) {
            return lang_1.global.setTimeout(fn, millis);
        };
        TimerWrapper.clearTimeout = function (id) {
            lang_1.global.clearTimeout(id);
        };
        TimerWrapper.setInterval = function (fn, millis) {
            return lang_1.global.setInterval(fn, millis);
        };
        TimerWrapper.clearInterval = function (id) {
            lang_1.global.clearInterval(id);
        };
        return TimerWrapper;
    }();
    exports.TimerWrapper = TimerWrapper;
    var ObservableWrapper = function () {
        function ObservableWrapper() {}
        // TODO(vsavkin): when we use rxnext, try inferring the generic type from the first arg
        ObservableWrapper.subscribe = function (emitter, onNext, onError, onComplete) {
            if (onComplete === void 0) {
                onComplete = function () {};
            }
            onError = typeof onError === 'function' && onError || lang_1.noop;
            onComplete = typeof onComplete === 'function' && onComplete || lang_1.noop;
            return emitter.subscribe({ next: onNext, error: onError, complete: onComplete });
        };
        ObservableWrapper.isObservable = function (obs) {
            return !!obs.subscribe;
        };
        /**
         * Returns whether `obs` has any subscribers listening to events.
         */
        ObservableWrapper.hasSubscribers = function (obs) {
            return obs.observers.length > 0;
        };
        ObservableWrapper.dispose = function (subscription) {
            subscription.unsubscribe();
        };
        /**
         * @deprecated - use callEmit() instead
         */
        ObservableWrapper.callNext = function (emitter, value) {
            emitter.emit(value);
        };
        ObservableWrapper.callEmit = function (emitter, value) {
            emitter.emit(value);
        };
        ObservableWrapper.callError = function (emitter, error) {
            emitter.error(error);
        };
        ObservableWrapper.callComplete = function (emitter) {
            emitter.complete();
        };
        ObservableWrapper.fromPromise = function (promise) {
            return PromiseObservable_1.PromiseObservable.create(promise);
        };
        ObservableWrapper.toPromise = function (obj) {
            return toPromise_1.toPromise.call(obj);
        };
        return ObservableWrapper;
    }();
    exports.ObservableWrapper = ObservableWrapper;
    /**
     * Use by directives and components to emit custom Events.
     *
     * ### Examples
     *
     * In the following example, `Zippy` alternatively emits `open` and `close` events when its
     * title gets clicked:
     *
     * ```
     * @Component({
     *   selector: 'zippy',
     *   template: `
     *   <div class="zippy">
     *     <div (click)="toggle()">Toggle</div>
     *     <div [hidden]="!visible">
     *       <ng-content></ng-content>
     *     </div>
     *  </div>`})
     * export class Zippy {
     *   visible: boolean = true;
     *   @Output() open: EventEmitter<any> = new EventEmitter();
     *   @Output() close: EventEmitter<any> = new EventEmitter();
     *
     *   toggle() {
     *     this.visible = !this.visible;
     *     if (this.visible) {
     *       this.open.emit(null);
     *     } else {
     *       this.close.emit(null);
     *     }
     *   }
     * }
     * ```
     *
     * The events payload can be accessed by the parameter `$event` on the components output event
     * handler:
     *
     * ```
     * <zippy (open)="onOpen($event)" (close)="onClose($event)"></zippy>
     * ```
     *
     * Uses Rx.Observable but provides an adapter to make it work as specified here:
     * https://github.com/jhusain/observable-spec
     *
     * Once a reference implementation of the spec is available, switch to it.
     * @stable
     */
    var EventEmitter = function (_super) {
        __extends(EventEmitter, _super);
        /**
         * Creates an instance of [EventEmitter], which depending on [isAsync],
         * delivers events synchronously or asynchronously.
         */
        function EventEmitter(isAsync) {
            if (isAsync === void 0) {
                isAsync = false;
            }
            _super.call(this);
            this.__isAsync = isAsync;
        }
        EventEmitter.prototype.emit = function (value) {
            _super.prototype.next.call(this, value);
        };
        /**
         * @deprecated - use .emit(value) instead
         */
        EventEmitter.prototype.next = function (value) {
            _super.prototype.next.call(this, value);
        };
        EventEmitter.prototype.subscribe = function (generatorOrNext, error, complete) {
            var schedulerFn;
            var errorFn = function (err) {
                return null;
            };
            var completeFn = function () {
                return null;
            };
            if (generatorOrNext && typeof generatorOrNext === 'object') {
                schedulerFn = this.__isAsync ? function (value /** TODO #9100 */) {
                    setTimeout(function () {
                        return generatorOrNext.next(value);
                    });
                } : function (value /** TODO #9100 */) {
                    generatorOrNext.next(value);
                };
                if (generatorOrNext.error) {
                    errorFn = this.__isAsync ? function (err) {
                        setTimeout(function () {
                            return generatorOrNext.error(err);
                        });
                    } : function (err) {
                        generatorOrNext.error(err);
                    };
                }
                if (generatorOrNext.complete) {
                    completeFn = this.__isAsync ? function () {
                        setTimeout(function () {
                            return generatorOrNext.complete();
                        });
                    } : function () {
                        generatorOrNext.complete();
                    };
                }
            } else {
                schedulerFn = this.__isAsync ? function (value /** TODO #9100 */) {
                    setTimeout(function () {
                        return generatorOrNext(value);
                    });
                } : function (value /** TODO #9100 */) {
                    generatorOrNext(value);
                };
                if (error) {
                    errorFn = this.__isAsync ? function (err) {
                        setTimeout(function () {
                            return error(err);
                        });
                    } : function (err) {
                        error(err);
                    };
                }
                if (complete) {
                    completeFn = this.__isAsync ? function () {
                        setTimeout(function () {
                            return complete();
                        });
                    } : function () {
                        complete();
                    };
                }
            }
            return _super.prototype.subscribe.call(this, schedulerFn, errorFn, completeFn);
        };
        return EventEmitter;
    }(Subject_1.Subject);
    exports.EventEmitter = EventEmitter;
    
});
System.registerDynamic('node_modules/@angular/common/src/location/location_strategy.js', ['@angular/core'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var core_1 = $__require('@angular/core');
  /**
   * `LocationStrategy` is responsible for representing and reading route state
   * from the browser's URL. Angular provides two strategies:
   * {@link HashLocationStrategy} and {@link PathLocationStrategy} (default).
   *
   * This is used under the hood of the {@link Location} service.
   *
   * Applications should use the {@link Router} or {@link Location} services to
   * interact with application route state.
   *
   * For instance, {@link HashLocationStrategy} produces URLs like
   * `http://example.com#/foo`, and {@link PathLocationStrategy} produces
   * `http://example.com/foo` as an equivalent URL.
   *
   * See these two classes for more.
   *
   * @stable
   */
  var LocationStrategy = function () {
    function LocationStrategy() {}
    return LocationStrategy;
  }();
  exports.LocationStrategy = LocationStrategy;
  /**
   * The `APP_BASE_HREF` token represents the base href to be used with the
   * {@link PathLocationStrategy}.
   *
   * If you're using {@link PathLocationStrategy}, you must provide a provider to a string
   * representing the URL prefix that should be preserved when generating and recognizing
   * URLs.
   *
   * ### Example
   *
   * ```
   * import {Component} from '@angular/core';
   * import {ROUTER_DIRECTIVES, ROUTER_PROVIDERS, RouteConfig} from '@angular/router';
   * import {APP_BASE_HREF} from '@angular/common';
   *
   * @Component({directives: [ROUTER_DIRECTIVES]})
   * @RouteConfig([
   *  {...},
   * ])
   * class AppCmp {
   *   // ...
   * }
   *
   * bootstrap(AppCmp, [
   *   ROUTER_PROVIDERS,
   *   {provide: APP_BASE_HREF, useValue: '/my/app'}
   * ]);
   * ```
   * @stable
   */
  exports.APP_BASE_HREF = new core_1.OpaqueToken('appBaseHref');
  
});
System.registerDynamic('node_modules/@angular/common/src/location/location.js', ['@angular/core', '../facade/async', './location_strategy'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var async_1 = $__require('../facade/async');
    var location_strategy_1 = $__require('./location_strategy');
    var Location = function () {
        function Location(platformStrategy) {
            var _this = this;
            /** @internal */
            this._subject = new core_1.EventEmitter();
            this._platformStrategy = platformStrategy;
            var browserBaseHref = this._platformStrategy.getBaseHref();
            this._baseHref = Location.stripTrailingSlash(_stripIndexHtml(browserBaseHref));
            this._platformStrategy.onPopState(function (ev) {
                async_1.ObservableWrapper.callEmit(_this._subject, { 'url': _this.path(true), 'pop': true, 'type': ev.type });
            });
        }
        /**
         * Returns the normalized URL path.
         */
        // TODO: vsavkin. Remove the boolean flag and always include hash once the deprecated router is
        // removed.
        Location.prototype.path = function (includeHash) {
            if (includeHash === void 0) {
                includeHash = false;
            }
            return this.normalize(this._platformStrategy.path(includeHash));
        };
        /**
         * Normalizes the given path and compares to the current normalized path.
         */
        Location.prototype.isCurrentPathEqualTo = function (path, query) {
            if (query === void 0) {
                query = '';
            }
            return this.path() == this.normalize(path + Location.normalizeQueryParams(query));
        };
        /**
         * Given a string representing a URL, returns the normalized URL path without leading or
         * trailing slashes
         */
        Location.prototype.normalize = function (url) {
            return Location.stripTrailingSlash(_stripBaseHref(this._baseHref, _stripIndexHtml(url)));
        };
        /**
         * Given a string representing a URL, returns the platform-specific external URL path.
         * If the given URL doesn't begin with a leading slash (`'/'`), this method adds one
         * before normalizing. This method will also add a hash if `HashLocationStrategy` is
         * used, or the `APP_BASE_HREF` if the `PathLocationStrategy` is in use.
         */
        Location.prototype.prepareExternalUrl = function (url) {
            if (url.length > 0 && !url.startsWith('/')) {
                url = '/' + url;
            }
            return this._platformStrategy.prepareExternalUrl(url);
        };
        // TODO: rename this method to pushState
        /**
         * Changes the browsers URL to the normalized version of the given URL, and pushes a
         * new item onto the platform's history.
         */
        Location.prototype.go = function (path, query) {
            if (query === void 0) {
                query = '';
            }
            this._platformStrategy.pushState(null, '', path, query);
        };
        /**
         * Changes the browsers URL to the normalized version of the given URL, and replaces
         * the top item on the platform's history stack.
         */
        Location.prototype.replaceState = function (path, query) {
            if (query === void 0) {
                query = '';
            }
            this._platformStrategy.replaceState(null, '', path, query);
        };
        /**
         * Navigates forward in the platform's history.
         */
        Location.prototype.forward = function () {
            this._platformStrategy.forward();
        };
        /**
         * Navigates back in the platform's history.
         */
        Location.prototype.back = function () {
            this._platformStrategy.back();
        };
        /**
         * Subscribe to the platform's `popState` events.
         */
        Location.prototype.subscribe = function (onNext, onThrow, onReturn) {
            if (onThrow === void 0) {
                onThrow = null;
            }
            if (onReturn === void 0) {
                onReturn = null;
            }
            return async_1.ObservableWrapper.subscribe(this._subject, onNext, onThrow, onReturn);
        };
        /**
         * Given a string of url parameters, prepend with '?' if needed, otherwise return parameters as
         * is.
         */
        Location.normalizeQueryParams = function (params) {
            return params.length > 0 && params.substring(0, 1) != '?' ? '?' + params : params;
        };
        /**
         * Given 2 parts of a url, join them with a slash if needed.
         */
        Location.joinWithSlash = function (start, end) {
            if (start.length == 0) {
                return end;
            }
            if (end.length == 0) {
                return start;
            }
            var slashes = 0;
            if (start.endsWith('/')) {
                slashes++;
            }
            if (end.startsWith('/')) {
                slashes++;
            }
            if (slashes == 2) {
                return start + end.substring(1);
            }
            if (slashes == 1) {
                return start + end;
            }
            return start + '/' + end;
        };
        /**
         * If url has a trailing slash, remove it, otherwise return url as is.
         */
        Location.stripTrailingSlash = function (url) {
            if (/\/$/g.test(url)) {
                url = url.substring(0, url.length - 1);
            }
            return url;
        };
        /** @nocollapse */
        Location.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        Location.ctorParameters = [{ type: location_strategy_1.LocationStrategy }];
        return Location;
    }();
    exports.Location = Location;
    function _stripBaseHref(baseHref, url) {
        if (baseHref.length > 0 && url.startsWith(baseHref)) {
            return url.substring(baseHref.length);
        }
        return url;
    }
    function _stripIndexHtml(url) {
        if (/\/index.html$/g.test(url)) {
            // '/index.html'.length == 11
            return url.substring(0, url.length - 11);
        }
        return url;
    }
    
});
System.registerDynamic('node_modules/@angular/common/src/location.js', ['./location/platform_location', './location/location_strategy', './location/hash_location_strategy', './location/path_location_strategy', './location/location'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
  }
  __export($__require('./location/platform_location'));
  __export($__require('./location/location_strategy'));
  __export($__require('./location/hash_location_strategy'));
  __export($__require('./location/path_location_strategy'));
  __export($__require('./location/location'));
  
});
System.registerDynamic("node_modules/@angular/common/src/localization.js", [], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * @experimental
   */

  var global = this || self,
      GLOBAL = global;
  var NgLocalization = function () {
    function NgLocalization() {}
    return NgLocalization;
  }();
  exports.NgLocalization = NgLocalization;
  /**
   * Returns the plural category for a given value.
   * - "=value" when the case exists,
   * - the plural category otherwise
   *
   * @internal
   */
  function getPluralCategory(value, cases, ngLocalization) {
    var nbCase = "=" + value;
    return cases.indexOf(nbCase) > -1 ? nbCase : ngLocalization.getPluralCategory(value);
  }
  exports.getPluralCategory = getPluralCategory;
  
});
System.registerDynamic('node_modules/@angular/common/index.js', ['./src/pipes', './src/directives', './src/forms-deprecated', './src/common_directives', './src/location', './src/localization'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
  }
  __export($__require('./src/pipes'));
  __export($__require('./src/directives'));
  __export($__require('./src/forms-deprecated'));
  __export($__require('./src/common_directives'));
  __export($__require('./src/location'));
  var localization_1 = $__require('./src/localization');
  exports.NgLocalization = localization_1.NgLocalization;
  
});
System.registerDynamic("node_modules/@angular/platform-browser/src/browser/location/history.js", [], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  function supportsState() {
    return !!window.history.pushState;
  }
  exports.supportsState = supportsState;
  
});
System.registerDynamic('node_modules/@angular/platform-browser/src/browser/location/browser_platform_location.js', ['@angular/common', '@angular/core', '../../dom/dom_adapter', './history'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var common_1 = $__require('@angular/common');
    var core_1 = $__require('@angular/core');
    var dom_adapter_1 = $__require('../../dom/dom_adapter');
    var history_1 = $__require('./history');
    var BrowserPlatformLocation = function (_super) {
        __extends(BrowserPlatformLocation, _super);
        function BrowserPlatformLocation() {
            _super.call(this);
            this._init();
        }
        // This is moved to its own method so that `MockPlatformLocationStrategy` can overwrite it
        /** @internal */
        BrowserPlatformLocation.prototype._init = function () {
            this._location = dom_adapter_1.getDOM().getLocation();
            this._history = dom_adapter_1.getDOM().getHistory();
        };
        Object.defineProperty(BrowserPlatformLocation.prototype, "location", {
            /** @internal */
            get: function () {
                return this._location;
            },
            enumerable: true,
            configurable: true
        });
        BrowserPlatformLocation.prototype.getBaseHrefFromDOM = function () {
            return dom_adapter_1.getDOM().getBaseHref();
        };
        BrowserPlatformLocation.prototype.onPopState = function (fn) {
            dom_adapter_1.getDOM().getGlobalEventTarget('window').addEventListener('popstate', fn, false);
        };
        BrowserPlatformLocation.prototype.onHashChange = function (fn) {
            dom_adapter_1.getDOM().getGlobalEventTarget('window').addEventListener('hashchange', fn, false);
        };
        Object.defineProperty(BrowserPlatformLocation.prototype, "pathname", {
            get: function () {
                return this._location.pathname;
            },
            set: function (newPath) {
                this._location.pathname = newPath;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BrowserPlatformLocation.prototype, "search", {
            get: function () {
                return this._location.search;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BrowserPlatformLocation.prototype, "hash", {
            get: function () {
                return this._location.hash;
            },
            enumerable: true,
            configurable: true
        });
        BrowserPlatformLocation.prototype.pushState = function (state, title, url) {
            if (history_1.supportsState()) {
                this._history.pushState(state, title, url);
            } else {
                this._location.hash = url;
            }
        };
        BrowserPlatformLocation.prototype.replaceState = function (state, title, url) {
            if (history_1.supportsState()) {
                this._history.replaceState(state, title, url);
            } else {
                this._location.hash = url;
            }
        };
        BrowserPlatformLocation.prototype.forward = function () {
            this._history.forward();
        };
        BrowserPlatformLocation.prototype.back = function () {
            this._history.back();
        };
        /** @nocollapse */
        BrowserPlatformLocation.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        BrowserPlatformLocation.ctorParameters = [];
        return BrowserPlatformLocation;
    }(common_1.PlatformLocation);
    exports.BrowserPlatformLocation = BrowserPlatformLocation;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/browser/testability.js', ['@angular/core', '../dom/dom_adapter', '../facade/collection', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var dom_adapter_1 = $__require('../dom/dom_adapter');
    var collection_1 = $__require('../facade/collection');
    var lang_1 = $__require('../facade/lang');
    var PublicTestability = function () {
        function PublicTestability(testability) {
            this._testability = testability;
        }
        PublicTestability.prototype.isStable = function () {
            return this._testability.isStable();
        };
        PublicTestability.prototype.whenStable = function (callback) {
            this._testability.whenStable(callback);
        };
        PublicTestability.prototype.findBindings = function (using, provider, exactMatch) {
            return this.findProviders(using, provider, exactMatch);
        };
        PublicTestability.prototype.findProviders = function (using, provider, exactMatch) {
            return this._testability.findBindings(using, provider, exactMatch);
        };
        return PublicTestability;
    }();
    var BrowserGetTestability = function () {
        function BrowserGetTestability() {}
        BrowserGetTestability.init = function () {
            core_1.setTestabilityGetter(new BrowserGetTestability());
        };
        BrowserGetTestability.prototype.addToWindow = function (registry) {
            lang_1.global.getAngularTestability = function (elem, findInAncestors) {
                if (findInAncestors === void 0) {
                    findInAncestors = true;
                }
                var testability = registry.findTestabilityInTree(elem, findInAncestors);
                if (testability == null) {
                    throw new Error('Could not find testability for element.');
                }
                return new PublicTestability(testability);
            };
            lang_1.global.getAllAngularTestabilities = function () {
                var testabilities = registry.getAllTestabilities();
                return testabilities.map(function (testability) {
                    return new PublicTestability(testability);
                });
            };
            lang_1.global.getAllAngularRootElements = function () {
                return registry.getAllRootElements();
            };
            var whenAllStable = function (callback /** TODO #9100 */) {
                var testabilities = lang_1.global.getAllAngularTestabilities();
                var count = testabilities.length;
                var didWork = false;
                var decrement = function (didWork_ /** TODO #9100 */) {
                    didWork = didWork || didWork_;
                    count--;
                    if (count == 0) {
                        callback(didWork);
                    }
                };
                testabilities.forEach(function (testability /** TODO #9100 */) {
                    testability.whenStable(decrement);
                });
            };
            if (!lang_1.global.frameworkStabilizers) {
                lang_1.global.frameworkStabilizers = collection_1.ListWrapper.createGrowableSize(0);
            }
            lang_1.global.frameworkStabilizers.push(whenAllStable);
        };
        BrowserGetTestability.prototype.findTestabilityInTree = function (registry, elem, findInAncestors) {
            if (elem == null) {
                return null;
            }
            var t = registry.getTestability(elem);
            if (lang_1.isPresent(t)) {
                return t;
            } else if (!findInAncestors) {
                return null;
            }
            if (dom_adapter_1.getDOM().isShadowRoot(elem)) {
                return this.findTestabilityInTree(registry, dom_adapter_1.getDOM().getHost(elem), true);
            }
            return this.findTestabilityInTree(registry, dom_adapter_1.getDOM().parentElement(elem), true);
        };
        return BrowserGetTestability;
    }();
    exports.BrowserGetTestability = BrowserGetTestability;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/dom/events/hammer_common.js', ['../../facade/collection', './event_manager'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var collection_1 = $__require('../../facade/collection');
    var event_manager_1 = $__require('./event_manager');
    var _eventNames = {
        // pan
        'pan': true,
        'panstart': true,
        'panmove': true,
        'panend': true,
        'pancancel': true,
        'panleft': true,
        'panright': true,
        'panup': true,
        'pandown': true,
        // pinch
        'pinch': true,
        'pinchstart': true,
        'pinchmove': true,
        'pinchend': true,
        'pinchcancel': true,
        'pinchin': true,
        'pinchout': true,
        // press
        'press': true,
        'pressup': true,
        // rotate
        'rotate': true,
        'rotatestart': true,
        'rotatemove': true,
        'rotateend': true,
        'rotatecancel': true,
        // swipe
        'swipe': true,
        'swipeleft': true,
        'swiperight': true,
        'swipeup': true,
        'swipedown': true,
        // tap
        'tap': true
    };
    var HammerGesturesPluginCommon = function (_super) {
        __extends(HammerGesturesPluginCommon, _super);
        function HammerGesturesPluginCommon() {
            _super.call(this);
        }
        HammerGesturesPluginCommon.prototype.supports = function (eventName) {
            eventName = eventName.toLowerCase();
            return collection_1.StringMapWrapper.contains(_eventNames, eventName);
        };
        return HammerGesturesPluginCommon;
    }(event_manager_1.EventManagerPlugin);
    exports.HammerGesturesPluginCommon = HammerGesturesPluginCommon;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/dom/events/hammer_gestures.js', ['@angular/core', '../../facade/exceptions', '../../facade/lang', './hammer_common'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var exceptions_1 = $__require('../../facade/exceptions');
    var lang_1 = $__require('../../facade/lang');
    var hammer_common_1 = $__require('./hammer_common');
    /**
     * A DI token that you can use to provide{@link HammerGestureConfig} to Angular. Use it to configure
     * Hammer gestures.
     *
     * @experimental
     */
    exports.HAMMER_GESTURE_CONFIG = new core_1.OpaqueToken('HammerGestureConfig');
    var HammerGestureConfig = function () {
        function HammerGestureConfig() {
            this.events = [];
            this.overrides = {};
        }
        HammerGestureConfig.prototype.buildHammer = function (element) {
            var mc = new Hammer(element);
            mc.get('pinch').set({ enable: true });
            mc.get('rotate').set({ enable: true });
            for (var eventName in this.overrides) {
                mc.get(eventName).set(this.overrides[eventName]);
            }
            return mc;
        };
        /** @nocollapse */
        HammerGestureConfig.decorators = [{ type: core_1.Injectable }];
        return HammerGestureConfig;
    }();
    exports.HammerGestureConfig = HammerGestureConfig;
    var HammerGesturesPlugin = function (_super) {
        __extends(HammerGesturesPlugin, _super);
        function HammerGesturesPlugin(_config) {
            _super.call(this);
            this._config = _config;
        }
        HammerGesturesPlugin.prototype.supports = function (eventName) {
            if (!_super.prototype.supports.call(this, eventName) && !this.isCustomEvent(eventName)) return false;
            if (!lang_1.isPresent(window['Hammer'])) {
                throw new exceptions_1.BaseException("Hammer.js is not loaded, can not bind " + eventName + " event");
            }
            return true;
        };
        HammerGesturesPlugin.prototype.addEventListener = function (element, eventName, handler) {
            var _this = this;
            var zone = this.manager.getZone();
            eventName = eventName.toLowerCase();
            return zone.runOutsideAngular(function () {
                // Creating the manager bind events, must be done outside of angular
                var mc = _this._config.buildHammer(element);
                var callback = function (eventObj /** TODO #???? */) {
                    zone.runGuarded(function () {
                        handler(eventObj);
                    });
                };
                mc.on(eventName, callback);
                return function () {
                    mc.off(eventName, callback);
                };
            });
        };
        HammerGesturesPlugin.prototype.isCustomEvent = function (eventName) {
            return this._config.events.indexOf(eventName) > -1;
        };
        /** @nocollapse */
        HammerGesturesPlugin.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        HammerGesturesPlugin.ctorParameters = [{ type: HammerGestureConfig, decorators: [{ type: core_1.Inject, args: [exports.HAMMER_GESTURE_CONFIG] }] }];
        return HammerGesturesPlugin;
    }(hammer_common_1.HammerGesturesPluginCommon);
    exports.HammerGesturesPlugin = HammerGesturesPlugin;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/dom/events/key_events.js', ['@angular/core', '../../facade/lang', '../../facade/collection', '../dom_adapter', './event_manager'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../../facade/lang');
    var collection_1 = $__require('../../facade/collection');
    var dom_adapter_1 = $__require('../dom_adapter');
    var event_manager_1 = $__require('./event_manager');
    var modifierKeys = ['alt', 'control', 'meta', 'shift'];
    var modifierKeyGetters = {
        'alt': function (event) {
            return event.altKey;
        },
        'control': function (event) {
            return event.ctrlKey;
        },
        'meta': function (event) {
            return event.metaKey;
        },
        'shift': function (event) {
            return event.shiftKey;
        }
    };
    var KeyEventsPlugin = function (_super) {
        __extends(KeyEventsPlugin, _super);
        function KeyEventsPlugin() {
            _super.call(this);
        }
        KeyEventsPlugin.prototype.supports = function (eventName) {
            return lang_1.isPresent(KeyEventsPlugin.parseEventName(eventName));
        };
        KeyEventsPlugin.prototype.addEventListener = function (element, eventName, handler) {
            var parsedEvent = KeyEventsPlugin.parseEventName(eventName);
            var outsideHandler = KeyEventsPlugin.eventCallback(element, collection_1.StringMapWrapper.get(parsedEvent, 'fullKey'), handler, this.manager.getZone());
            return this.manager.getZone().runOutsideAngular(function () {
                return dom_adapter_1.getDOM().onAndCancel(element, collection_1.StringMapWrapper.get(parsedEvent, 'domEventName'), outsideHandler);
            });
        };
        KeyEventsPlugin.parseEventName = function (eventName) {
            var parts = eventName.toLowerCase().split('.');
            var domEventName = parts.shift();
            if (parts.length === 0 || !(lang_1.StringWrapper.equals(domEventName, 'keydown') || lang_1.StringWrapper.equals(domEventName, 'keyup'))) {
                return null;
            }
            var key = KeyEventsPlugin._normalizeKey(parts.pop());
            var fullKey = '';
            modifierKeys.forEach(function (modifierName) {
                if (collection_1.ListWrapper.contains(parts, modifierName)) {
                    collection_1.ListWrapper.remove(parts, modifierName);
                    fullKey += modifierName + '.';
                }
            });
            fullKey += key;
            if (parts.length != 0 || key.length === 0) {
                // returning null instead of throwing to let another plugin process the event
                return null;
            }
            var result = collection_1.StringMapWrapper.create();
            collection_1.StringMapWrapper.set(result, 'domEventName', domEventName);
            collection_1.StringMapWrapper.set(result, 'fullKey', fullKey);
            return result;
        };
        KeyEventsPlugin.getEventFullKey = function (event) {
            var fullKey = '';
            var key = dom_adapter_1.getDOM().getEventKey(event);
            key = key.toLowerCase();
            if (lang_1.StringWrapper.equals(key, ' ')) {
                key = 'space'; // for readability
            } else if (lang_1.StringWrapper.equals(key, '.')) {
                key = 'dot'; // because '.' is used as a separator in event names
            }
            modifierKeys.forEach(function (modifierName) {
                if (modifierName != key) {
                    var modifierGetter = collection_1.StringMapWrapper.get(modifierKeyGetters, modifierName);
                    if (modifierGetter(event)) {
                        fullKey += modifierName + '.';
                    }
                }
            });
            fullKey += key;
            return fullKey;
        };
        KeyEventsPlugin.eventCallback = function (element, fullKey, handler, zone) {
            return function (event /** TODO #9100 */) {
                if (lang_1.StringWrapper.equals(KeyEventsPlugin.getEventFullKey(event), fullKey)) {
                    zone.runGuarded(function () {
                        return handler(event);
                    });
                }
            };
        };
        /** @internal */
        KeyEventsPlugin._normalizeKey = function (keyName) {
            // TODO: switch to a StringMap if the mapping grows too much
            switch (keyName) {
                case 'esc':
                    return 'escape';
                default:
                    return keyName;
            }
        };
        /** @nocollapse */
        KeyEventsPlugin.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        KeyEventsPlugin.ctorParameters = [];
        return KeyEventsPlugin;
    }(event_manager_1.EventManagerPlugin);
    exports.KeyEventsPlugin = KeyEventsPlugin;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/security/html_sanitizer.js', ['@angular/core', '../dom/dom_adapter', './url_sanitizer'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var dom_adapter_1 = $__require('../dom/dom_adapter');
    var url_sanitizer_1 = $__require('./url_sanitizer');
    /** A <body> element that can be safely used to parse untrusted HTML. Lazily initialized below. */
    var inertElement = null;
    /** Lazily initialized to make sure the DOM adapter gets set before use. */
    var DOM = null;
    /** Returns an HTML element that is guaranteed to not execute code when creating elements in it. */
    function getInertElement() {
        if (inertElement) return inertElement;
        DOM = dom_adapter_1.getDOM();
        // Prefer using <template> element if supported.
        var templateEl = DOM.createElement('template');
        if ('content' in templateEl) return templateEl;
        var doc = DOM.createHtmlDocument();
        inertElement = DOM.querySelector(doc, 'body');
        if (inertElement == null) {
            // usually there should be only one body element in the document, but IE doesn't have any, so we
            // need to create one.
            var html = DOM.createElement('html', doc);
            inertElement = DOM.createElement('body', doc);
            DOM.appendChild(html, inertElement);
            DOM.appendChild(doc, html);
        }
        return inertElement;
    }
    function tagSet(tags) {
        var res = {};
        for (var _i = 0, _a = tags.split(','); _i < _a.length; _i++) {
            var t = _a[_i];
            res[t.toLowerCase()] = true;
        }
        return res;
    }
    function merge() {
        var sets = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            sets[_i - 0] = arguments[_i];
        }
        var res = {};
        for (var _a = 0, sets_1 = sets; _a < sets_1.length; _a++) {
            var s = sets_1[_a];
            for (var v in s) {
                if (s.hasOwnProperty(v)) res[v] = true;
            }
        }
        return res;
    }
    // Good source of info about elements and attributes
    // http://dev.w3.org/html5/spec/Overview.html#semantics
    // http://simon.html5.org/html-elements
    // Safe Void Elements - HTML5
    // http://dev.w3.org/html5/spec/Overview.html#void-elements
    var VOID_ELEMENTS = tagSet('area,br,col,hr,img,wbr');
    // Elements that you can, intentionally, leave open (and which close themselves)
    // http://dev.w3.org/html5/spec/Overview.html#optional-tags
    var OPTIONAL_END_TAG_BLOCK_ELEMENTS = tagSet('colgroup,dd,dt,li,p,tbody,td,tfoot,th,thead,tr');
    var OPTIONAL_END_TAG_INLINE_ELEMENTS = tagSet('rp,rt');
    var OPTIONAL_END_TAG_ELEMENTS = merge(OPTIONAL_END_TAG_INLINE_ELEMENTS, OPTIONAL_END_TAG_BLOCK_ELEMENTS);
    // Safe Block Elements - HTML5
    var BLOCK_ELEMENTS = merge(OPTIONAL_END_TAG_BLOCK_ELEMENTS, tagSet('address,article,' + 'aside,blockquote,caption,center,del,details,dialog,dir,div,dl,figure,figcaption,footer,h1,h2,h3,h4,h5,' + 'h6,header,hgroup,hr,ins,main,map,menu,nav,ol,pre,section,summary,table,ul'));
    // Inline Elements - HTML5
    var INLINE_ELEMENTS = merge(OPTIONAL_END_TAG_INLINE_ELEMENTS, tagSet('a,abbr,acronym,audio,b,' + 'bdi,bdo,big,br,cite,code,del,dfn,em,font,i,img,ins,kbd,label,map,mark,picture,q,ruby,rp,rt,s,' + 'samp,small,source,span,strike,strong,sub,sup,time,track,tt,u,var,video'));
    var VALID_ELEMENTS = merge(VOID_ELEMENTS, BLOCK_ELEMENTS, INLINE_ELEMENTS, OPTIONAL_END_TAG_ELEMENTS);
    // Attributes that have href and hence need to be sanitized
    var URI_ATTRS = tagSet('background,cite,href,itemtype,longdesc,poster,src,xlink:href');
    // Attributes that have special href set hence need to be sanitized
    var SRCSET_ATTRS = tagSet('srcset');
    var HTML_ATTRS = tagSet('abbr,accesskey,align,alt,autoplay,axis,bgcolor,border,cellpadding,cellspacing,class,clear,color,cols,colspan,' + 'compact,controls,coords,datetime,default,dir,download,face,headers,height,hidden,hreflang,hspace,' + 'ismap,itemscope,itemprop,kind,label,lang,language,loop,media,muted,nohref,nowrap,open,preload,rel,rev,role,rows,rowspan,rules,' + 'scope,scrolling,shape,size,sizes,span,srclang,start,summary,tabindex,target,title,translate,type,usemap,' + 'valign,value,vspace,width');
    // NB: This currently conciously doesn't support SVG. SVG sanitization has had several security
    // issues in the past, so it seems safer to leave it out if possible. If support for binding SVG via
    // innerHTML is required, SVG attributes should be added here.
    // NB: Sanitization does not allow <form> elements or other active elements (<button> etc). Those
    // can be sanitized, but they increase security surface area without a legitimate use case, so they
    // are left out here.
    var VALID_ATTRS = merge(URI_ATTRS, SRCSET_ATTRS, HTML_ATTRS);
    /**
     * SanitizingHtmlSerializer serializes a DOM fragment, stripping out any unsafe elements and unsafe
     * attributes.
     */
    var SanitizingHtmlSerializer = function () {
        function SanitizingHtmlSerializer() {
            this.buf = [];
        }
        SanitizingHtmlSerializer.prototype.sanitizeChildren = function (el) {
            // This cannot use a TreeWalker, as it has to run on Angular's various DOM adapters.
            // However this code never accesses properties off of `document` before deleting its contents
            // again, so it shouldn't be vulnerable to DOM clobbering.
            var current = el.firstChild;
            while (current) {
                if (DOM.isElementNode(current)) {
                    this.startElement(current);
                } else if (DOM.isTextNode(current)) {
                    this.chars(DOM.nodeValue(current));
                }
                if (DOM.firstChild(current)) {
                    current = DOM.firstChild(current);
                    continue;
                }
                while (current) {
                    // Leaving the element. Walk up and to the right, closing tags as we go.
                    if (DOM.isElementNode(current)) {
                        this.endElement(DOM.nodeName(current).toLowerCase());
                    }
                    if (DOM.nextSibling(current)) {
                        current = DOM.nextSibling(current);
                        break;
                    }
                    current = DOM.parentElement(current);
                }
            }
            return this.buf.join('');
        };
        SanitizingHtmlSerializer.prototype.startElement = function (element) {
            var _this = this;
            var tagName = DOM.nodeName(element).toLowerCase();
            tagName = tagName.toLowerCase();
            if (VALID_ELEMENTS.hasOwnProperty(tagName)) {
                this.buf.push('<');
                this.buf.push(tagName);
                DOM.attributeMap(element).forEach(function (value, attrName) {
                    var lower = attrName.toLowerCase();
                    if (!VALID_ATTRS.hasOwnProperty(lower)) return;
                    // TODO(martinprobst): Special case image URIs for data:image/...
                    if (URI_ATTRS[lower]) value = url_sanitizer_1.sanitizeUrl(value);
                    if (SRCSET_ATTRS[lower]) value = url_sanitizer_1.sanitizeSrcset(value);
                    _this.buf.push(' ');
                    _this.buf.push(attrName);
                    _this.buf.push('="');
                    _this.buf.push(encodeEntities(value));
                    _this.buf.push('"');
                });
                this.buf.push('>');
            }
        };
        SanitizingHtmlSerializer.prototype.endElement = function (tagName) {
            tagName = tagName.toLowerCase();
            if (VALID_ELEMENTS.hasOwnProperty(tagName) && !VOID_ELEMENTS.hasOwnProperty(tagName)) {
                this.buf.push('</');
                this.buf.push(tagName);
                this.buf.push('>');
            }
        };
        SanitizingHtmlSerializer.prototype.chars = function (chars /** TODO #9100 */) {
            this.buf.push(encodeEntities(chars));
        };
        return SanitizingHtmlSerializer;
    }();
    // Regular Expressions for parsing tags and attributes
    var SURROGATE_PAIR_REGEXP = /[\uD800-\uDBFF][\uDC00-\uDFFF]/g;
    // ! to ~ is the ASCII range.
    var NON_ALPHANUMERIC_REGEXP = /([^\#-~ |!])/g;
    /**
     * Escapes all potentially dangerous characters, so that the
     * resulting string can be safely inserted into attribute or
     * element text.
     * @param value
     * @returns {string} escaped text
     */
    function encodeEntities(value /** TODO #9100 */) {
        return value.replace(/&/g, '&amp;').replace(SURROGATE_PAIR_REGEXP, function (match /** TODO #9100 */) {
            var hi = match.charCodeAt(0);
            var low = match.charCodeAt(1);
            return '&#' + ((hi - 0xD800) * 0x400 + (low - 0xDC00) + 0x10000) + ';';
        }).replace(NON_ALPHANUMERIC_REGEXP, function (match /** TODO #9100 */) {
            return '&#' + match.charCodeAt(0) + ';';
        }).replace(/</g, '&lt;').replace(/>/g, '&gt;');
    }
    /**
     * When IE9-11 comes across an unknown namespaced attribute e.g. 'xlink:foo' it adds 'xmlns:ns1'
     * attribute to declare ns1 namespace and prefixes the attribute with 'ns1' (e.g. 'ns1:xlink:foo').
     *
     * This is undesirable since we don't want to allow any of these custom attributes. This method
     * strips them all.
     */
    function stripCustomNsAttrs(el) {
        DOM.attributeMap(el).forEach(function (_, attrName) {
            if (attrName === 'xmlns:ns1' || attrName.indexOf('ns1:') === 0) {
                DOM.removeAttribute(el, attrName);
            }
        });
        for (var _i = 0, _a = DOM.childNodesAsList(el); _i < _a.length; _i++) {
            var n = _a[_i];
            if (DOM.isElementNode(n)) stripCustomNsAttrs(n);
        }
    }
    /**
     * Sanitizes the given unsafe, untrusted HTML fragment, and returns HTML text that is safe to add to
     * the DOM in a browser environment.
     */
    function sanitizeHtml(unsafeHtmlInput) {
        try {
            var containerEl = getInertElement();
            // Make sure unsafeHtml is actually a string (TypeScript types are not enforced at runtime).
            var unsafeHtml = unsafeHtmlInput ? String(unsafeHtmlInput) : '';
            // mXSS protection. Repeatedly parse the document to make sure it stabilizes, so that a browser
            // trying to auto-correct incorrect HTML cannot cause formerly inert HTML to become dangerous.
            var mXSSAttempts = 5;
            var parsedHtml = unsafeHtml;
            do {
                if (mXSSAttempts === 0) {
                    throw new Error('Failed to sanitize html because the input is unstable');
                }
                mXSSAttempts--;
                unsafeHtml = parsedHtml;
                DOM.setInnerHTML(containerEl, unsafeHtml);
                if (DOM.defaultDoc().documentMode) {
                    // strip custom-namespaced attributes on IE<=11
                    stripCustomNsAttrs(containerEl);
                }
                parsedHtml = DOM.getInnerHTML(containerEl);
            } while (unsafeHtml !== parsedHtml);
            var sanitizer = new SanitizingHtmlSerializer();
            var safeHtml = sanitizer.sanitizeChildren(DOM.getTemplateContent(containerEl) || containerEl);
            // Clear out the body element.
            var parent_1 = DOM.getTemplateContent(containerEl) || containerEl;
            for (var _i = 0, _a = DOM.childNodesAsList(parent_1); _i < _a.length; _i++) {
                var child = _a[_i];
                DOM.removeChild(parent_1, child);
            }
            if (core_1.isDevMode() && safeHtml !== unsafeHtmlInput) {
                DOM.log('WARNING: sanitizing HTML stripped some content (see http://g.co/ng/security#xss).');
            }
            return safeHtml;
        } catch (e) {
            // In case anything goes wrong, clear out inertElement to reset the entire DOM structure.
            inertElement = null;
            throw e;
        }
    }
    exports.sanitizeHtml = sanitizeHtml;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/security/style_sanitizer.js', ['@angular/core', '../dom/dom_adapter', './url_sanitizer'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var dom_adapter_1 = $__require('../dom/dom_adapter');
    var url_sanitizer_1 = $__require('./url_sanitizer');
    /**
     * Regular expression for safe style values.
     *
     * Quotes (" and ') are allowed, but a check must be done elsewhere to ensure they're balanced.
     *
     * ',' allows multiple values to be assigned to the same property (e.g. background-attachment or
     * font-family) and hence could allow multiple values to get injected, but that should pose no risk
     * of XSS.
     *
     * The function expression checks only for XSS safety, not for CSS validity.
     *
     * This regular expression was taken from the Closure sanitization library, and augmented for
     * transformation values.
     */
    var VALUES = '[-,."\'%_!# a-zA-Z0-9]+';
    var TRANSFORMATION_FNS = '(?:matrix|translate|scale|rotate|skew|perspective)(?:X|Y|3d)?';
    var COLOR_FNS = '(?:rgb|hsl)a?';
    var FN_ARGS = '\\([-0-9.%, a-zA-Z]+\\)';
    var SAFE_STYLE_VALUE = new RegExp("^(" + VALUES + "|(?:" + TRANSFORMATION_FNS + "|" + COLOR_FNS + ")" + FN_ARGS + ")$", 'g');
    /**
     * Matches a `url(...)` value with an arbitrary argument as long as it does
     * not contain parentheses.
     *
     * The URL value still needs to be sanitized separately.
     *
     * `url(...)` values are a very common use case, e.g. for `background-image`. With carefully crafted
     * CSS style rules, it is possible to construct an information leak with `url` values in CSS, e.g.
     * by observing whether scroll bars are displayed, or character ranges used by a font face
     * definition.
     *
     * Angular only allows binding CSS values (as opposed to entire CSS rules), so it is unlikely that
     * binding a URL value without further cooperation from the page will cause an information leak, and
     * if so, it is just a leak, not a full blown XSS vulnerability.
     *
     * Given the common use case, low likelihood of attack vector, and low impact of an attack, this
     * code is permissive and allows URLs that sanitize otherwise.
     */
    var URL_RE = /^url\(([^)]+)\)$/;
    /**
     * Checks that quotes (" and ') are properly balanced inside a string. Assumes
     * that neither escape (\) nor any other character that could result in
     * breaking out of a string parsing context are allowed;
     * see http://www.w3.org/TR/css3-syntax/#string-token-diagram.
     *
     * This code was taken from the Closure sanitization library.
     */
    function hasBalancedQuotes(value) {
        var outsideSingle = true;
        var outsideDouble = true;
        for (var i = 0; i < value.length; i++) {
            var c = value.charAt(i);
            if (c === '\'' && outsideDouble) {
                outsideSingle = !outsideSingle;
            } else if (c === '"' && outsideSingle) {
                outsideDouble = !outsideDouble;
            }
        }
        return outsideSingle && outsideDouble;
    }
    /**
     * Sanitizes the given untrusted CSS style property value (i.e. not an entire object, just a single
     * value) and returns a value that is safe to use in a browser environment.
     */
    function sanitizeStyle(value) {
        value = String(value).trim(); // Make sure it's actually a string.
        if (!value) return '';
        // Single url(...) values are supported, but only for URLs that sanitize cleanly. See above for
        // reasoning behind this.
        var urlMatch = URL_RE.exec(value);
        if (urlMatch && url_sanitizer_1.sanitizeUrl(urlMatch[1]) === urlMatch[1] || value.match(SAFE_STYLE_VALUE) && hasBalancedQuotes(value)) {
            return value; // Safe style values.
        }
        if (core_1.isDevMode()) {
            dom_adapter_1.getDOM().log("WARNING: sanitizing unsafe style value " + value + " (see http://g.co/ng/security#xss).");
        }
        return 'unsafe';
    }
    exports.sanitizeStyle = sanitizeStyle;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/security/url_sanitizer.js', ['@angular/core', '../dom/dom_adapter'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var dom_adapter_1 = $__require('../dom/dom_adapter');
    /**
     * A pattern that recognizes a commonly useful subset of URLs that are safe.
     *
     * This regular expression matches a subset of URLs that will not cause script
     * execution if used in URL context within a HTML document. Specifically, this
     * regular expression matches if (comment from here on and regex copied from
     * Soy's EscapingConventions):
     * (1) Either a protocol in a whitelist (http, https, mailto or ftp).
     * (2) or no protocol.  A protocol must be followed by a colon. The below
     *     allows that by allowing colons only after one of the characters [/?#].
     *     A colon after a hash (#) must be in the fragment.
     *     Otherwise, a colon after a (?) must be in a query.
     *     Otherwise, a colon after a single solidus (/) must be in a path.
     *     Otherwise, a colon after a double solidus (//) must be in the authority
     *     (before port).
     *
     * The pattern disallows &, used in HTML entity declarations before
     * one of the characters in [/?#]. This disallows HTML entities used in the
     * protocol name, which should never happen, e.g. "h&#116;tp" for "http".
     * It also disallows HTML entities in the first path part of a relative path,
     * e.g. "foo&lt;bar/baz".  Our existing escaping functions should not produce
     * that. More importantly, it disallows masking of a colon,
     * e.g. "javascript&#58;...".
     *
     * This regular expression was taken from the Closure sanitization library.
     */
    var SAFE_URL_PATTERN = /^(?:(?:https?|mailto|ftp|tel|file):|[^&:/?#]*(?:[/?#]|$))/gi;
    /* A pattern that matches safe srcset values */
    var SAFE_SRCSET_PATTERN = /^(?:(?:https?|file):|[^&:/?#]*(?:[/?#]|$))/gi;
    /** A pattern that matches safe data URLs. Only matches image, video and audio types. */
    var DATA_URL_PATTERN = /^data:(?:image\/(?:bmp|gif|jpeg|jpg|png|tiff|webp)|video\/(?:mpeg|mp4|ogg|webm)|audio\/(?:mp3|oga|ogg|opus));base64,[a-z0-9+\/]+=*$/i;
    function sanitizeUrl(url) {
        url = String(url);
        if (url.match(SAFE_URL_PATTERN) || url.match(DATA_URL_PATTERN)) return url;
        if (core_1.isDevMode()) {
            dom_adapter_1.getDOM().log("WARNING: sanitizing unsafe URL value " + url + " (see http://g.co/ng/security#xss)");
        }
        return 'unsafe:' + url;
    }
    exports.sanitizeUrl = sanitizeUrl;
    function sanitizeSrcset(srcset) {
        srcset = String(srcset);
        return srcset.split(',').map(function (srcset) {
            return sanitizeUrl(srcset.trim());
        }).join(', ');
    }
    exports.sanitizeSrcset = sanitizeSrcset;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/security/dom_sanitization_service.js', ['@angular/core', '../../core_private', './html_sanitizer', './style_sanitizer', './url_sanitizer'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var core_private_1 = $__require('../../core_private');
    exports.SecurityContext = core_private_1.SecurityContext;
    var html_sanitizer_1 = $__require('./html_sanitizer');
    var style_sanitizer_1 = $__require('./style_sanitizer');
    var url_sanitizer_1 = $__require('./url_sanitizer');
    /**
     * DomSanitizationService helps preventing Cross Site Scripting Security bugs (XSS) by sanitizing
     * values to be safe to use in the different DOM contexts.
     *
     * For example, when binding a URL in an `<a [href]="someValue">` hyperlink, `someValue` will be
     * sanitized so that an attacker cannot inject e.g. a `javascript:` URL that would execute code on
     * the website.
     *
     * In specific situations, it might be necessary to disable sanitization, for example if the
     * application genuinely needs to produce a `javascript:` style link with a dynamic value in it.
     * Users can bypass security by constructing a value with one of the `bypassSecurityTrust...`
     * methods, and then binding to that value from the template.
     *
     * These situations should be very rare, and extraordinary care must be taken to avoid creating a
     * Cross Site Scripting (XSS) security bug!
     *
     * When using `bypassSecurityTrust...`, make sure to call the method as early as possible and as
     * close as possible to the source of the value, to make it easy to verify no security bug is
     * created by its use.
     *
     * It is not required (and not recommended) to bypass security if the value is safe, e.g. a URL that
     * does not start with a suspicious protocol, or an HTML snippet that does not contain dangerous
     * code. The sanitizer leaves safe values intact.
     *
     * @security Calling any of the `bypassSecurityTrust...` APIs disables Angular's built-in
     * sanitization for the value passed in. Carefully check and audit all values and code paths going
     * into this call. Make sure any user data is appropriately escaped for this security context.
     * For more detail, see the [Security Guide](http://g.co/ng/security).
     *
     * @stable
     */
    var DomSanitizationService = function () {
        function DomSanitizationService() {}
        return DomSanitizationService;
    }();
    exports.DomSanitizationService = DomSanitizationService;
    var DomSanitizationServiceImpl = function (_super) {
        __extends(DomSanitizationServiceImpl, _super);
        function DomSanitizationServiceImpl() {
            _super.apply(this, arguments);
        }
        DomSanitizationServiceImpl.prototype.sanitize = function (ctx, value) {
            if (value == null) return null;
            switch (ctx) {
                case core_private_1.SecurityContext.NONE:
                    return value;
                case core_private_1.SecurityContext.HTML:
                    if (value instanceof SafeHtmlImpl) return value.changingThisBreaksApplicationSecurity;
                    this.checkNotSafeValue(value, 'HTML');
                    return html_sanitizer_1.sanitizeHtml(String(value));
                case core_private_1.SecurityContext.STYLE:
                    if (value instanceof SafeStyleImpl) return value.changingThisBreaksApplicationSecurity;
                    this.checkNotSafeValue(value, 'Style');
                    return style_sanitizer_1.sanitizeStyle(value);
                case core_private_1.SecurityContext.SCRIPT:
                    if (value instanceof SafeScriptImpl) return value.changingThisBreaksApplicationSecurity;
                    this.checkNotSafeValue(value, 'Script');
                    throw new Error('unsafe value used in a script context');
                case core_private_1.SecurityContext.URL:
                    if (value instanceof SafeUrlImpl) return value.changingThisBreaksApplicationSecurity;
                    this.checkNotSafeValue(value, 'URL');
                    return url_sanitizer_1.sanitizeUrl(String(value));
                case core_private_1.SecurityContext.RESOURCE_URL:
                    if (value instanceof SafeResourceUrlImpl) {
                        return value.changingThisBreaksApplicationSecurity;
                    }
                    this.checkNotSafeValue(value, 'ResourceURL');
                    throw new Error('unsafe value used in a resource URL context (see http://g.co/ng/security#xss)');
                default:
                    throw new Error("Unexpected SecurityContext " + ctx + " (see http://g.co/ng/security#xss)");
            }
        };
        DomSanitizationServiceImpl.prototype.checkNotSafeValue = function (value, expectedType) {
            if (value instanceof SafeValueImpl) {
                throw new Error("Required a safe " + expectedType + ", got a " + value.getTypeName() + " " + "(see http://g.co/ng/security#xss)");
            }
        };
        DomSanitizationServiceImpl.prototype.bypassSecurityTrustHtml = function (value) {
            return new SafeHtmlImpl(value);
        };
        DomSanitizationServiceImpl.prototype.bypassSecurityTrustStyle = function (value) {
            return new SafeStyleImpl(value);
        };
        DomSanitizationServiceImpl.prototype.bypassSecurityTrustScript = function (value) {
            return new SafeScriptImpl(value);
        };
        DomSanitizationServiceImpl.prototype.bypassSecurityTrustUrl = function (value) {
            return new SafeUrlImpl(value);
        };
        DomSanitizationServiceImpl.prototype.bypassSecurityTrustResourceUrl = function (value) {
            return new SafeResourceUrlImpl(value);
        };
        /** @nocollapse */
        DomSanitizationServiceImpl.decorators = [{ type: core_1.Injectable }];
        return DomSanitizationServiceImpl;
    }(DomSanitizationService);
    exports.DomSanitizationServiceImpl = DomSanitizationServiceImpl;
    var SafeValueImpl = function () {
        function SafeValueImpl(changingThisBreaksApplicationSecurity) {
            this.changingThisBreaksApplicationSecurity = changingThisBreaksApplicationSecurity;
            // empty
        }
        SafeValueImpl.prototype.toString = function () {
            return "SafeValue must use [property]=binding: " + this.changingThisBreaksApplicationSecurity + " (see http://g.co/ng/security#xss)";
        };
        return SafeValueImpl;
    }();
    var SafeHtmlImpl = function (_super) {
        __extends(SafeHtmlImpl, _super);
        function SafeHtmlImpl() {
            _super.apply(this, arguments);
        }
        SafeHtmlImpl.prototype.getTypeName = function () {
            return 'HTML';
        };
        return SafeHtmlImpl;
    }(SafeValueImpl);
    var SafeStyleImpl = function (_super) {
        __extends(SafeStyleImpl, _super);
        function SafeStyleImpl() {
            _super.apply(this, arguments);
        }
        SafeStyleImpl.prototype.getTypeName = function () {
            return 'Style';
        };
        return SafeStyleImpl;
    }(SafeValueImpl);
    var SafeScriptImpl = function (_super) {
        __extends(SafeScriptImpl, _super);
        function SafeScriptImpl() {
            _super.apply(this, arguments);
        }
        SafeScriptImpl.prototype.getTypeName = function () {
            return 'Script';
        };
        return SafeScriptImpl;
    }(SafeValueImpl);
    var SafeUrlImpl = function (_super) {
        __extends(SafeUrlImpl, _super);
        function SafeUrlImpl() {
            _super.apply(this, arguments);
        }
        SafeUrlImpl.prototype.getTypeName = function () {
            return 'URL';
        };
        return SafeUrlImpl;
    }(SafeValueImpl);
    var SafeResourceUrlImpl = function (_super) {
        __extends(SafeResourceUrlImpl, _super);
        function SafeResourceUrlImpl() {
            _super.apply(this, arguments);
        }
        SafeResourceUrlImpl.prototype.getTypeName = function () {
            return 'ResourceURL';
        };
        return SafeResourceUrlImpl;
    }(SafeValueImpl);
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/browser.js', ['@angular/common', '@angular/core', '../core_private', '../src/dom/web_animations_driver', './browser/browser_adapter', './browser/location/browser_platform_location', './browser/testability', './dom/debug/ng_probe', './dom/dom_adapter', './dom/dom_renderer', './dom/dom_tokens', './dom/events/dom_events', './dom/events/event_manager', './dom/events/hammer_gestures', './dom/events/key_events', './dom/shared_styles_host', './facade/lang', './security/dom_sanitization_service'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var common_1 = $__require('@angular/common');
    var core_1 = $__require('@angular/core');
    var core_private_1 = $__require('../core_private');
    var web_animations_driver_1 = $__require('../src/dom/web_animations_driver');
    var browser_adapter_1 = $__require('./browser/browser_adapter');
    var browser_platform_location_1 = $__require('./browser/location/browser_platform_location');
    var testability_1 = $__require('./browser/testability');
    var ng_probe_1 = $__require('./dom/debug/ng_probe');
    var dom_adapter_1 = $__require('./dom/dom_adapter');
    var dom_renderer_1 = $__require('./dom/dom_renderer');
    var dom_tokens_1 = $__require('./dom/dom_tokens');
    var dom_events_1 = $__require('./dom/events/dom_events');
    var event_manager_1 = $__require('./dom/events/event_manager');
    var hammer_gestures_1 = $__require('./dom/events/hammer_gestures');
    var key_events_1 = $__require('./dom/events/key_events');
    var shared_styles_host_1 = $__require('./dom/shared_styles_host');
    var lang_1 = $__require('./facade/lang');
    var dom_sanitization_service_1 = $__require('./security/dom_sanitization_service');
    var BROWSER_PLATFORM_MARKER = new core_1.OpaqueToken('BrowserPlatformMarker');
    /**
     * A set of providers to initialize the Angular platform in a web browser.
     *
     * Used automatically by `bootstrap`, or can be passed to {@link platform}.
     *
     * @experimental API related to bootstrapping are still under review.
     */
    exports.BROWSER_PLATFORM_PROVIDERS = [{ provide: BROWSER_PLATFORM_MARKER, useValue: true }, core_1.PLATFORM_COMMON_PROVIDERS, { provide: core_1.PLATFORM_INITIALIZER, useValue: initDomAdapter, multi: true }, { provide: common_1.PlatformLocation, useClass: browser_platform_location_1.BrowserPlatformLocation }];
    /**
     * @security Replacing built-in sanitization providers exposes the application to XSS risks.
     * Attacker-controlled data introduced by an unsanitized provider could expose your
     * application to XSS risks. For more detail, see the [Security Guide](http://g.co/ng/security).
     * @experimental
     */
    exports.BROWSER_SANITIZATION_PROVIDERS = [{ provide: core_private_1.SanitizationService, useExisting: dom_sanitization_service_1.DomSanitizationService }, { provide: dom_sanitization_service_1.DomSanitizationService, useClass: dom_sanitization_service_1.DomSanitizationServiceImpl }];
    /**
     * A set of providers to initialize an Angular application in a web browser.
     *
     * Used automatically by `bootstrap`, or can be passed to {@link PlatformRef.application}.
     *
     * @experimental API related to bootstrapping are still under review.
     */
    exports.BROWSER_APP_PROVIDERS = [core_1.APPLICATION_COMMON_PROVIDERS, common_1.FORM_PROVIDERS, exports.BROWSER_SANITIZATION_PROVIDERS, { provide: core_1.ExceptionHandler, useFactory: _exceptionHandler, deps: [] }, { provide: dom_tokens_1.DOCUMENT, useFactory: _document, deps: [] }, { provide: event_manager_1.EVENT_MANAGER_PLUGINS, useClass: dom_events_1.DomEventsPlugin, multi: true }, { provide: event_manager_1.EVENT_MANAGER_PLUGINS, useClass: key_events_1.KeyEventsPlugin, multi: true }, { provide: event_manager_1.EVENT_MANAGER_PLUGINS, useClass: hammer_gestures_1.HammerGesturesPlugin, multi: true }, { provide: hammer_gestures_1.HAMMER_GESTURE_CONFIG, useClass: hammer_gestures_1.HammerGestureConfig }, { provide: dom_renderer_1.DomRootRenderer, useClass: dom_renderer_1.DomRootRenderer_ }, { provide: core_1.RootRenderer, useExisting: dom_renderer_1.DomRootRenderer }, { provide: shared_styles_host_1.SharedStylesHost, useExisting: shared_styles_host_1.DomSharedStylesHost }, { provide: core_private_1.AnimationDriver, useFactory: _resolveDefaultAnimationDriver }, shared_styles_host_1.DomSharedStylesHost, core_1.Testability, event_manager_1.EventManager, ng_probe_1.ELEMENT_PROBE_PROVIDERS];
    /**
     * @experimental API related to bootstrapping are still under review.
     */
    function browserPlatform() {
        if (lang_1.isBlank(core_1.getPlatform())) {
            core_1.createPlatform(core_1.ReflectiveInjector.resolveAndCreate(exports.BROWSER_PLATFORM_PROVIDERS));
        }
        return core_1.assertPlatform(BROWSER_PLATFORM_MARKER);
    }
    exports.browserPlatform = browserPlatform;
    function initDomAdapter() {
        browser_adapter_1.BrowserDomAdapter.makeCurrent();
        core_private_1.wtfInit();
        testability_1.BrowserGetTestability.init();
    }
    function _exceptionHandler() {
        return new core_1.ExceptionHandler(dom_adapter_1.getDOM());
    }
    function _document() {
        return dom_adapter_1.getDOM().defaultDoc();
    }
    function _resolveDefaultAnimationDriver() {
        if (dom_adapter_1.getDOM().supportsWebAnimation()) {
            return new web_animations_driver_1.WebAnimationsDriver();
        }
        return new core_private_1.NoOpAnimationDriver();
    }
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/web_workers/shared/api.js', ['@angular/core'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var core_1 = $__require('@angular/core');
  exports.ON_WEB_WORKER = new core_1.OpaqueToken('WebWorker.onWebWorker');
  
});
System.registerDynamic('node_modules/@angular/platform-browser/src/web_workers/shared/post_message_bus.js', ['@angular/core', '../../facade/async', '../../facade/collection', '../../facade/exceptions'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var async_1 = $__require('../../facade/async');
    var collection_1 = $__require('../../facade/collection');
    var exceptions_1 = $__require('../../facade/exceptions');
    var PostMessageBusSink = function () {
        function PostMessageBusSink(_postMessageTarget) {
            this._postMessageTarget = _postMessageTarget;
            this._channels = collection_1.StringMapWrapper.create();
            this._messageBuffer = [];
        }
        PostMessageBusSink.prototype.attachToZone = function (zone) {
            var _this = this;
            this._zone = zone;
            this._zone.runOutsideAngular(function () {
                async_1.ObservableWrapper.subscribe(_this._zone.onStable, function (_) {
                    _this._handleOnEventDone();
                });
            });
        };
        PostMessageBusSink.prototype.initChannel = function (channel, runInZone) {
            var _this = this;
            if (runInZone === void 0) {
                runInZone = true;
            }
            if (collection_1.StringMapWrapper.contains(this._channels, channel)) {
                throw new exceptions_1.BaseException(channel + " has already been initialized");
            }
            var emitter = new async_1.EventEmitter(false);
            var channelInfo = new _Channel(emitter, runInZone);
            this._channels[channel] = channelInfo;
            emitter.subscribe(function (data) {
                var message = { channel: channel, message: data };
                if (runInZone) {
                    _this._messageBuffer.push(message);
                } else {
                    _this._sendMessages([message]);
                }
            });
        };
        PostMessageBusSink.prototype.to = function (channel) {
            if (collection_1.StringMapWrapper.contains(this._channels, channel)) {
                return this._channels[channel].emitter;
            } else {
                throw new exceptions_1.BaseException(channel + " is not set up. Did you forget to call initChannel?");
            }
        };
        PostMessageBusSink.prototype._handleOnEventDone = function () {
            if (this._messageBuffer.length > 0) {
                this._sendMessages(this._messageBuffer);
                this._messageBuffer = [];
            }
        };
        PostMessageBusSink.prototype._sendMessages = function (messages) {
            this._postMessageTarget.postMessage(messages);
        };
        return PostMessageBusSink;
    }();
    exports.PostMessageBusSink = PostMessageBusSink;
    var PostMessageBusSource = function () {
        function PostMessageBusSource(eventTarget) {
            var _this = this;
            this._channels = collection_1.StringMapWrapper.create();
            if (eventTarget) {
                eventTarget.addEventListener('message', function (ev) {
                    return _this._handleMessages(ev);
                });
            } else {
                // if no eventTarget is given we assume we're in a WebWorker and listen on the global scope
                var workerScope = self;
                workerScope.addEventListener('message', function (ev) {
                    return _this._handleMessages(ev);
                });
            }
        }
        PostMessageBusSource.prototype.attachToZone = function (zone) {
            this._zone = zone;
        };
        PostMessageBusSource.prototype.initChannel = function (channel, runInZone) {
            if (runInZone === void 0) {
                runInZone = true;
            }
            if (collection_1.StringMapWrapper.contains(this._channels, channel)) {
                throw new exceptions_1.BaseException(channel + " has already been initialized");
            }
            var emitter = new async_1.EventEmitter(false);
            var channelInfo = new _Channel(emitter, runInZone);
            this._channels[channel] = channelInfo;
        };
        PostMessageBusSource.prototype.from = function (channel) {
            if (collection_1.StringMapWrapper.contains(this._channels, channel)) {
                return this._channels[channel].emitter;
            } else {
                throw new exceptions_1.BaseException(channel + " is not set up. Did you forget to call initChannel?");
            }
        };
        PostMessageBusSource.prototype._handleMessages = function (ev) {
            var messages = ev.data;
            for (var i = 0; i < messages.length; i++) {
                this._handleMessage(messages[i]);
            }
        };
        PostMessageBusSource.prototype._handleMessage = function (data) {
            var channel = data.channel;
            if (collection_1.StringMapWrapper.contains(this._channels, channel)) {
                var channelInfo = this._channels[channel];
                if (channelInfo.runInZone) {
                    this._zone.run(function () {
                        channelInfo.emitter.emit(data.message);
                    });
                } else {
                    channelInfo.emitter.emit(data.message);
                }
            }
        };
        return PostMessageBusSource;
    }();
    exports.PostMessageBusSource = PostMessageBusSource;
    var PostMessageBus = function () {
        function PostMessageBus(sink, source) {
            this.sink = sink;
            this.source = source;
        }
        PostMessageBus.prototype.attachToZone = function (zone) {
            this.source.attachToZone(zone);
            this.sink.attachToZone(zone);
        };
        PostMessageBus.prototype.initChannel = function (channel, runInZone) {
            if (runInZone === void 0) {
                runInZone = true;
            }
            this.source.initChannel(channel, runInZone);
            this.sink.initChannel(channel, runInZone);
        };
        PostMessageBus.prototype.from = function (channel) {
            return this.source.from(channel);
        };
        PostMessageBus.prototype.to = function (channel) {
            return this.sink.to(channel);
        };
        /** @nocollapse */
        PostMessageBus.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        PostMessageBus.ctorParameters = [{ type: PostMessageBusSink }, { type: PostMessageBusSource }];
        return PostMessageBus;
    }();
    exports.PostMessageBus = PostMessageBus;
    /**
     * Helper class that wraps a channel's {@link EventEmitter} and
     * keeps track of if it should run in the zone.
     */
    var _Channel = function () {
        function _Channel(emitter, runInZone) {
            this.emitter = emitter;
            this.runInZone = runInZone;
        }
        return _Channel;
    }();
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/web_workers/shared/service_message_broker.js', ['@angular/core', '../../facade/async', '../../facade/collection', '../../facade/lang', '../shared/message_bus', '../shared/serializer'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var async_1 = $__require('../../facade/async');
    var collection_1 = $__require('../../facade/collection');
    var lang_1 = $__require('../../facade/lang');
    var message_bus_1 = $__require('../shared/message_bus');
    var serializer_1 = $__require('../shared/serializer');
    /**
     * @experimental WebWorker support in Angular is currently experimental.
     */
    var ServiceMessageBrokerFactory = function () {
        function ServiceMessageBrokerFactory() {}
        return ServiceMessageBrokerFactory;
    }();
    exports.ServiceMessageBrokerFactory = ServiceMessageBrokerFactory;
    var ServiceMessageBrokerFactory_ = function (_super) {
        __extends(ServiceMessageBrokerFactory_, _super);
        function ServiceMessageBrokerFactory_(_messageBus, _serializer) {
            _super.call(this);
            this._messageBus = _messageBus;
            this._serializer = _serializer;
        }
        ServiceMessageBrokerFactory_.prototype.createMessageBroker = function (channel, runInZone) {
            if (runInZone === void 0) {
                runInZone = true;
            }
            this._messageBus.initChannel(channel, runInZone);
            return new ServiceMessageBroker_(this._messageBus, this._serializer, channel);
        };
        /** @nocollapse */
        ServiceMessageBrokerFactory_.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        ServiceMessageBrokerFactory_.ctorParameters = [{ type: message_bus_1.MessageBus }, { type: serializer_1.Serializer }];
        return ServiceMessageBrokerFactory_;
    }(ServiceMessageBrokerFactory);
    exports.ServiceMessageBrokerFactory_ = ServiceMessageBrokerFactory_;
    /**
     * Helper class for UIComponents that allows components to register methods.
     * If a registered method message is received from the broker on the worker,
     * the UIMessageBroker deserializes its arguments and calls the registered method.
     * If that method returns a promise, the UIMessageBroker returns the result to the worker.
     *
     * @experimental WebWorker support in Angular is currently experimental.
     */
    var ServiceMessageBroker = function () {
        function ServiceMessageBroker() {}
        return ServiceMessageBroker;
    }();
    exports.ServiceMessageBroker = ServiceMessageBroker;
    var ServiceMessageBroker_ = function (_super) {
        __extends(ServiceMessageBroker_, _super);
        function ServiceMessageBroker_(messageBus, _serializer, channel /** TODO #9100 */) {
            var _this = this;
            _super.call(this);
            this._serializer = _serializer;
            this.channel = channel;
            this._methods = new collection_1.Map();
            this._sink = messageBus.to(channel);
            var source = messageBus.from(channel);
            async_1.ObservableWrapper.subscribe(source, function (message) {
                return _this._handleMessage(message);
            });
        }
        ServiceMessageBroker_.prototype.registerMethod = function (methodName, signature, method, returnType) {
            var _this = this;
            this._methods.set(methodName, function (message) {
                var serializedArgs = message.args;
                var numArgs = signature === null ? 0 : signature.length;
                var deserializedArgs = collection_1.ListWrapper.createFixedSize(numArgs);
                for (var i = 0; i < numArgs; i++) {
                    var serializedArg = serializedArgs[i];
                    deserializedArgs[i] = _this._serializer.deserialize(serializedArg, signature[i]);
                }
                var promise = lang_1.FunctionWrapper.apply(method, deserializedArgs);
                if (lang_1.isPresent(returnType) && lang_1.isPresent(promise)) {
                    _this._wrapWebWorkerPromise(message.id, promise, returnType);
                }
            });
        };
        ServiceMessageBroker_.prototype._handleMessage = function (map) {
            var message = new ReceivedMessage(map);
            if (this._methods.has(message.method)) {
                this._methods.get(message.method)(message);
            }
        };
        ServiceMessageBroker_.prototype._wrapWebWorkerPromise = function (id, promise, type) {
            var _this = this;
            async_1.PromiseWrapper.then(promise, function (result) {
                async_1.ObservableWrapper.callEmit(_this._sink, { 'type': 'result', 'value': _this._serializer.serialize(result, type), 'id': id });
            });
        };
        return ServiceMessageBroker_;
    }(ServiceMessageBroker);
    exports.ServiceMessageBroker_ = ServiceMessageBroker_;
    /**
     * @experimental WebWorker support in Angular is currently experimental.
     */
    var ReceivedMessage = function () {
        function ReceivedMessage(data) {
            this.method = data['method'];
            this.args = data['args'];
            this.id = data['id'];
            this.type = data['type'];
        }
        return ReceivedMessage;
    }();
    exports.ReceivedMessage = ReceivedMessage;
    
});
System.registerDynamic("node_modules/@angular/platform-browser/src/facade/promise.js", [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var PromiseCompleter = function () {
        function PromiseCompleter() {
            var _this = this;
            this.promise = new Promise(function (res, rej) {
                _this.resolve = res;
                _this.reject = rej;
            });
        }
        return PromiseCompleter;
    }();
    exports.PromiseCompleter = PromiseCompleter;
    var PromiseWrapper = function () {
        function PromiseWrapper() {}
        PromiseWrapper.resolve = function (obj) {
            return Promise.resolve(obj);
        };
        PromiseWrapper.reject = function (obj, _) {
            return Promise.reject(obj);
        };
        // Note: We can't rename this method into `catch`, as this is not a valid
        // method name in Dart.
        PromiseWrapper.catchError = function (promise, onError) {
            return promise.catch(onError);
        };
        PromiseWrapper.all = function (promises) {
            if (promises.length == 0) return Promise.resolve([]);
            return Promise.all(promises);
        };
        PromiseWrapper.then = function (promise, success, rejection) {
            return promise.then(success, rejection);
        };
        PromiseWrapper.wrap = function (computation) {
            return new Promise(function (res, rej) {
                try {
                    res(computation());
                } catch (e) {
                    rej(e);
                }
            });
        };
        PromiseWrapper.scheduleMicrotask = function (computation) {
            PromiseWrapper.then(PromiseWrapper.resolve(null), computation, function (_) {});
        };
        PromiseWrapper.completer = function () {
            return new PromiseCompleter();
        };
        return PromiseWrapper;
    }();
    exports.PromiseWrapper = PromiseWrapper;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/facade/async.js', ['rxjs/Subject', 'rxjs/observable/PromiseObservable', 'rxjs/operator/toPromise', './lang', 'rxjs/Observable', './promise'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subject_1 = $__require('rxjs/Subject');
    var PromiseObservable_1 = $__require('rxjs/observable/PromiseObservable');
    var toPromise_1 = $__require('rxjs/operator/toPromise');
    var lang_1 = $__require('./lang');
    var Observable_1 = $__require('rxjs/Observable');
    exports.Observable = Observable_1.Observable;
    var Subject_2 = $__require('rxjs/Subject');
    exports.Subject = Subject_2.Subject;
    var promise_1 = $__require('./promise');
    exports.PromiseCompleter = promise_1.PromiseCompleter;
    exports.PromiseWrapper = promise_1.PromiseWrapper;
    var TimerWrapper = function () {
        function TimerWrapper() {}
        TimerWrapper.setTimeout = function (fn, millis) {
            return lang_1.global.setTimeout(fn, millis);
        };
        TimerWrapper.clearTimeout = function (id) {
            lang_1.global.clearTimeout(id);
        };
        TimerWrapper.setInterval = function (fn, millis) {
            return lang_1.global.setInterval(fn, millis);
        };
        TimerWrapper.clearInterval = function (id) {
            lang_1.global.clearInterval(id);
        };
        return TimerWrapper;
    }();
    exports.TimerWrapper = TimerWrapper;
    var ObservableWrapper = function () {
        function ObservableWrapper() {}
        // TODO(vsavkin): when we use rxnext, try inferring the generic type from the first arg
        ObservableWrapper.subscribe = function (emitter, onNext, onError, onComplete) {
            if (onComplete === void 0) {
                onComplete = function () {};
            }
            onError = typeof onError === 'function' && onError || lang_1.noop;
            onComplete = typeof onComplete === 'function' && onComplete || lang_1.noop;
            return emitter.subscribe({ next: onNext, error: onError, complete: onComplete });
        };
        ObservableWrapper.isObservable = function (obs) {
            return !!obs.subscribe;
        };
        /**
         * Returns whether `obs` has any subscribers listening to events.
         */
        ObservableWrapper.hasSubscribers = function (obs) {
            return obs.observers.length > 0;
        };
        ObservableWrapper.dispose = function (subscription) {
            subscription.unsubscribe();
        };
        /**
         * @deprecated - use callEmit() instead
         */
        ObservableWrapper.callNext = function (emitter, value) {
            emitter.emit(value);
        };
        ObservableWrapper.callEmit = function (emitter, value) {
            emitter.emit(value);
        };
        ObservableWrapper.callError = function (emitter, error) {
            emitter.error(error);
        };
        ObservableWrapper.callComplete = function (emitter) {
            emitter.complete();
        };
        ObservableWrapper.fromPromise = function (promise) {
            return PromiseObservable_1.PromiseObservable.create(promise);
        };
        ObservableWrapper.toPromise = function (obj) {
            return toPromise_1.toPromise.call(obj);
        };
        return ObservableWrapper;
    }();
    exports.ObservableWrapper = ObservableWrapper;
    /**
     * Use by directives and components to emit custom Events.
     *
     * ### Examples
     *
     * In the following example, `Zippy` alternatively emits `open` and `close` events when its
     * title gets clicked:
     *
     * ```
     * @Component({
     *   selector: 'zippy',
     *   template: `
     *   <div class="zippy">
     *     <div (click)="toggle()">Toggle</div>
     *     <div [hidden]="!visible">
     *       <ng-content></ng-content>
     *     </div>
     *  </div>`})
     * export class Zippy {
     *   visible: boolean = true;
     *   @Output() open: EventEmitter<any> = new EventEmitter();
     *   @Output() close: EventEmitter<any> = new EventEmitter();
     *
     *   toggle() {
     *     this.visible = !this.visible;
     *     if (this.visible) {
     *       this.open.emit(null);
     *     } else {
     *       this.close.emit(null);
     *     }
     *   }
     * }
     * ```
     *
     * The events payload can be accessed by the parameter `$event` on the components output event
     * handler:
     *
     * ```
     * <zippy (open)="onOpen($event)" (close)="onClose($event)"></zippy>
     * ```
     *
     * Uses Rx.Observable but provides an adapter to make it work as specified here:
     * https://github.com/jhusain/observable-spec
     *
     * Once a reference implementation of the spec is available, switch to it.
     * @stable
     */
    var EventEmitter = function (_super) {
        __extends(EventEmitter, _super);
        /**
         * Creates an instance of [EventEmitter], which depending on [isAsync],
         * delivers events synchronously or asynchronously.
         */
        function EventEmitter(isAsync) {
            if (isAsync === void 0) {
                isAsync = false;
            }
            _super.call(this);
            this.__isAsync = isAsync;
        }
        EventEmitter.prototype.emit = function (value) {
            _super.prototype.next.call(this, value);
        };
        /**
         * @deprecated - use .emit(value) instead
         */
        EventEmitter.prototype.next = function (value) {
            _super.prototype.next.call(this, value);
        };
        EventEmitter.prototype.subscribe = function (generatorOrNext, error, complete) {
            var schedulerFn;
            var errorFn = function (err) {
                return null;
            };
            var completeFn = function () {
                return null;
            };
            if (generatorOrNext && typeof generatorOrNext === 'object') {
                schedulerFn = this.__isAsync ? function (value /** TODO #9100 */) {
                    setTimeout(function () {
                        return generatorOrNext.next(value);
                    });
                } : function (value /** TODO #9100 */) {
                    generatorOrNext.next(value);
                };
                if (generatorOrNext.error) {
                    errorFn = this.__isAsync ? function (err) {
                        setTimeout(function () {
                            return generatorOrNext.error(err);
                        });
                    } : function (err) {
                        generatorOrNext.error(err);
                    };
                }
                if (generatorOrNext.complete) {
                    completeFn = this.__isAsync ? function () {
                        setTimeout(function () {
                            return generatorOrNext.complete();
                        });
                    } : function () {
                        generatorOrNext.complete();
                    };
                }
            } else {
                schedulerFn = this.__isAsync ? function (value /** TODO #9100 */) {
                    setTimeout(function () {
                        return generatorOrNext(value);
                    });
                } : function (value /** TODO #9100 */) {
                    generatorOrNext(value);
                };
                if (error) {
                    errorFn = this.__isAsync ? function (err) {
                        setTimeout(function () {
                            return error(err);
                        });
                    } : function (err) {
                        error(err);
                    };
                }
                if (complete) {
                    completeFn = this.__isAsync ? function () {
                        setTimeout(function () {
                            return complete();
                        });
                    } : function () {
                        complete();
                    };
                }
            }
            return _super.prototype.subscribe.call(this, schedulerFn, errorFn, completeFn);
        };
        return EventEmitter;
    }(Subject_1.Subject);
    exports.EventEmitter = EventEmitter;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/web_workers/shared/client_message_broker.js', ['@angular/core', '../../facade/async', '../../facade/collection', '../../facade/lang', './message_bus', './serializer'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var async_1 = $__require('../../facade/async');
    var collection_1 = $__require('../../facade/collection');
    var lang_1 = $__require('../../facade/lang');
    var message_bus_1 = $__require('./message_bus');
    var serializer_1 = $__require('./serializer');
    /**
     * @experimental WebWorker support in Angular is experimental.
     */
    var ClientMessageBrokerFactory = function () {
        function ClientMessageBrokerFactory() {}
        return ClientMessageBrokerFactory;
    }();
    exports.ClientMessageBrokerFactory = ClientMessageBrokerFactory;
    var ClientMessageBrokerFactory_ = function (_super) {
        __extends(ClientMessageBrokerFactory_, _super);
        function ClientMessageBrokerFactory_(_messageBus, _serializer) {
            _super.call(this);
            this._messageBus = _messageBus;
            this._serializer = _serializer;
        }
        /**
         * Initializes the given channel and attaches a new {@link ClientMessageBroker} to it.
         */
        ClientMessageBrokerFactory_.prototype.createMessageBroker = function (channel, runInZone) {
            if (runInZone === void 0) {
                runInZone = true;
            }
            this._messageBus.initChannel(channel, runInZone);
            return new ClientMessageBroker_(this._messageBus, this._serializer, channel);
        };
        /** @nocollapse */
        ClientMessageBrokerFactory_.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        ClientMessageBrokerFactory_.ctorParameters = [{ type: message_bus_1.MessageBus }, { type: serializer_1.Serializer }];
        return ClientMessageBrokerFactory_;
    }(ClientMessageBrokerFactory);
    exports.ClientMessageBrokerFactory_ = ClientMessageBrokerFactory_;
    /**
     * @experimental WebWorker support in Angular is experimental.
     */
    var ClientMessageBroker = function () {
        function ClientMessageBroker() {}
        return ClientMessageBroker;
    }();
    exports.ClientMessageBroker = ClientMessageBroker;
    var ClientMessageBroker_ = function (_super) {
        __extends(ClientMessageBroker_, _super);
        function ClientMessageBroker_(messageBus, _serializer, channel /** TODO #9100 */) {
            var _this = this;
            _super.call(this);
            this.channel = channel;
            this._pending = new Map();
            this._sink = messageBus.to(channel);
            this._serializer = _serializer;
            var source = messageBus.from(channel);
            async_1.ObservableWrapper.subscribe(source, function (message) {
                return _this._handleMessage(message);
            });
        }
        ClientMessageBroker_.prototype._generateMessageId = function (name) {
            var time = lang_1.stringify(lang_1.DateWrapper.toMillis(lang_1.DateWrapper.now()));
            var iteration = 0;
            var id = name + time + lang_1.stringify(iteration);
            while (lang_1.isPresent(this._pending[id])) {
                id = "" + name + time + iteration;
                iteration++;
            }
            return id;
        };
        ClientMessageBroker_.prototype.runOnService = function (args, returnType) {
            var _this = this;
            var fnArgs = [];
            if (lang_1.isPresent(args.args)) {
                args.args.forEach(function (argument) {
                    if (argument.type != null) {
                        fnArgs.push(_this._serializer.serialize(argument.value, argument.type));
                    } else {
                        fnArgs.push(argument.value);
                    }
                });
            }
            var promise;
            var id = null;
            if (returnType != null) {
                var completer = async_1.PromiseWrapper.completer();
                id = this._generateMessageId(args.method);
                this._pending.set(id, completer);
                async_1.PromiseWrapper.catchError(completer.promise, function (err, stack) {
                    lang_1.print(err);
                    completer.reject(err, stack);
                });
                promise = async_1.PromiseWrapper.then(completer.promise, function (value) {
                    if (_this._serializer == null) {
                        return value;
                    } else {
                        return _this._serializer.deserialize(value, returnType);
                    }
                });
            } else {
                promise = null;
            }
            // TODO(jteplitz602): Create a class for these messages so we don't keep using StringMap #3685
            var message = { 'method': args.method, 'args': fnArgs };
            if (id != null) {
                message['id'] = id;
            }
            async_1.ObservableWrapper.callEmit(this._sink, message);
            return promise;
        };
        ClientMessageBroker_.prototype._handleMessage = function (message) {
            var data = new MessageData(message);
            // TODO(jteplitz602): replace these strings with messaging constants #3685
            if (lang_1.StringWrapper.equals(data.type, 'result') || lang_1.StringWrapper.equals(data.type, 'error')) {
                var id = data.id;
                if (this._pending.has(id)) {
                    if (lang_1.StringWrapper.equals(data.type, 'result')) {
                        this._pending.get(id).resolve(data.value);
                    } else {
                        this._pending.get(id).reject(data.value, null);
                    }
                    this._pending.delete(id);
                }
            }
        };
        return ClientMessageBroker_;
    }(ClientMessageBroker);
    exports.ClientMessageBroker_ = ClientMessageBroker_;
    var MessageData = function () {
        function MessageData(data) {
            this.type = collection_1.StringMapWrapper.get(data, 'type');
            this.id = this._getValueIfPresent(data, 'id');
            this.value = this._getValueIfPresent(data, 'value');
        }
        /**
         * Returns the value from the StringMap if present. Otherwise returns null
         * @internal
         */
        MessageData.prototype._getValueIfPresent = function (data, key) {
            if (collection_1.StringMapWrapper.contains(data, key)) {
                return collection_1.StringMapWrapper.get(data, key);
            } else {
                return null;
            }
        };
        return MessageData;
    }();
    /**
     * @experimental WebWorker support in Angular is experimental.
     */
    var FnArg = function () {
        function FnArg(value /** TODO #9100 */, type) {
            this.value = value;
            this.type = type;
        }
        return FnArg;
    }();
    exports.FnArg = FnArg;
    /**
     * @experimental WebWorker support in Angular is experimental.
     */
    var UiArguments = function () {
        function UiArguments(method, args) {
            this.method = method;
            this.args = args;
        }
        return UiArguments;
    }();
    exports.UiArguments = UiArguments;
    
});
System.registerDynamic("node_modules/@angular/platform-browser/src/web_workers/shared/message_bus.js", [], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * Message Bus is a low level API used to communicate between the UI and the background.
   * Communication is based on a channel abstraction. Messages published in a
   * given channel to one MessageBusSink are received on the same channel
   * by the corresponding MessageBusSource.
   *
   * @experimental WebWorker support in Angular is currenlty experimental.
   */

  var global = this || self,
      GLOBAL = global;
  var MessageBus = function () {
    function MessageBus() {}
    return MessageBus;
  }();
  exports.MessageBus = MessageBus;
  
});
System.registerDynamic('node_modules/@angular/platform-browser/src/web_workers/shared/messaging_api.js', [], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * All channels used by angular's WebWorker components are listed here.
   * You should not use these channels in your application code.
   */

  var global = this || self,
      GLOBAL = global;
  exports.RENDERER_CHANNEL = 'ng-Renderer';
  exports.EVENT_CHANNEL = 'ng-Events';
  exports.ROUTER_CHANNEL = 'ng-Router';
  
});
System.registerDynamic("node_modules/@angular/platform-browser/src/web_workers/shared/render_store.js", ["@angular/core"], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var RenderStore = function () {
        function RenderStore() {
            this._nextIndex = 0;
            this._lookupById = new Map();
            this._lookupByObject = new Map();
        }
        RenderStore.prototype.allocateId = function () {
            return this._nextIndex++;
        };
        RenderStore.prototype.store = function (obj, id) {
            this._lookupById.set(id, obj);
            this._lookupByObject.set(obj, id);
        };
        RenderStore.prototype.remove = function (obj) {
            var index = this._lookupByObject.get(obj);
            this._lookupByObject.delete(obj);
            this._lookupById.delete(index);
        };
        RenderStore.prototype.deserialize = function (id) {
            if (id == null) {
                return null;
            }
            if (!this._lookupById.has(id)) {
                return null;
            }
            return this._lookupById.get(id);
        };
        RenderStore.prototype.serialize = function (obj) {
            if (obj == null) {
                return null;
            }
            return this._lookupByObject.get(obj);
        };
        /** @nocollapse */
        RenderStore.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        RenderStore.ctorParameters = [];
        return RenderStore;
    }();
    exports.RenderStore = RenderStore;
    
});
System.registerDynamic("node_modules/@angular/platform-browser/src/web_workers/shared/serialized_types.js", [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";
    // This file contains interface versions of browser types that can be serialized to Plain Old
    // JavaScript Objects

    var global = this || self,
        GLOBAL = global;
    var LocationType = function () {
        function LocationType(href, protocol, host, hostname, port, pathname, search, hash, origin) {
            this.href = href;
            this.protocol = protocol;
            this.host = host;
            this.hostname = hostname;
            this.port = port;
            this.pathname = pathname;
            this.search = search;
            this.hash = hash;
            this.origin = origin;
        }
        return LocationType;
    }();
    exports.LocationType = LocationType;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/web_workers/shared/serializer.js', ['@angular/core', '../../../core_private', '../../facade/exceptions', '../../facade/lang', './render_store', './serialized_types'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var core_private_1 = $__require('../../../core_private');
    var exceptions_1 = $__require('../../facade/exceptions');
    var lang_1 = $__require('../../facade/lang');
    var render_store_1 = $__require('./render_store');
    var serialized_types_1 = $__require('./serialized_types');
    // PRIMITIVE is any type that does not need to be serialized (string, number, boolean)
    // We set it to String so that it is considered a Type.
    /**
     * @experimental WebWorker support in Angular is currently experimental.
     */
    exports.PRIMITIVE = String;
    var Serializer = function () {
        function Serializer(_renderStore) {
            this._renderStore = _renderStore;
        }
        Serializer.prototype.serialize = function (obj, type) {
            var _this = this;
            if (!lang_1.isPresent(obj)) {
                return null;
            }
            if (lang_1.isArray(obj)) {
                return obj.map(function (v) {
                    return _this.serialize(v, type);
                });
            }
            if (type == exports.PRIMITIVE) {
                return obj;
            }
            if (type == RenderStoreObject) {
                return this._renderStore.serialize(obj);
            } else if (type === core_1.RenderComponentType) {
                return this._serializeRenderComponentType(obj);
            } else if (type === core_1.ViewEncapsulation) {
                return lang_1.serializeEnum(obj);
            } else if (type === serialized_types_1.LocationType) {
                return this._serializeLocation(obj);
            } else {
                throw new exceptions_1.BaseException('No serializer for ' + type.toString());
            }
        };
        Serializer.prototype.deserialize = function (map, type, data) {
            var _this = this;
            if (!lang_1.isPresent(map)) {
                return null;
            }
            if (lang_1.isArray(map)) {
                var obj = [];
                map.forEach(function (val) {
                    return obj.push(_this.deserialize(val, type, data));
                });
                return obj;
            }
            if (type == exports.PRIMITIVE) {
                return map;
            }
            if (type == RenderStoreObject) {
                return this._renderStore.deserialize(map);
            } else if (type === core_1.RenderComponentType) {
                return this._deserializeRenderComponentType(map);
            } else if (type === core_1.ViewEncapsulation) {
                return core_private_1.VIEW_ENCAPSULATION_VALUES[map];
            } else if (type === serialized_types_1.LocationType) {
                return this._deserializeLocation(map);
            } else {
                throw new exceptions_1.BaseException('No deserializer for ' + type.toString());
            }
        };
        Serializer.prototype._serializeLocation = function (loc) {
            return {
                'href': loc.href,
                'protocol': loc.protocol,
                'host': loc.host,
                'hostname': loc.hostname,
                'port': loc.port,
                'pathname': loc.pathname,
                'search': loc.search,
                'hash': loc.hash,
                'origin': loc.origin
            };
        };
        Serializer.prototype._deserializeLocation = function (loc) {
            return new serialized_types_1.LocationType(loc['href'], loc['protocol'], loc['host'], loc['hostname'], loc['port'], loc['pathname'], loc['search'], loc['hash'], loc['origin']);
        };
        Serializer.prototype._serializeRenderComponentType = function (obj) {
            return {
                'id': obj.id,
                'templateUrl': obj.templateUrl,
                'slotCount': obj.slotCount,
                'encapsulation': this.serialize(obj.encapsulation, core_1.ViewEncapsulation),
                'styles': this.serialize(obj.styles, exports.PRIMITIVE)
            };
        };
        Serializer.prototype._deserializeRenderComponentType = function (map) {
            return new core_1.RenderComponentType(map['id'], map['templateUrl'], map['slotCount'], this.deserialize(map['encapsulation'], core_1.ViewEncapsulation), this.deserialize(map['styles'], exports.PRIMITIVE));
        };
        /** @nocollapse */
        Serializer.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        Serializer.ctorParameters = [{ type: render_store_1.RenderStore }];
        return Serializer;
    }();
    exports.Serializer = Serializer;
    var RenderStoreObject = function () {
        function RenderStoreObject() {}
        return RenderStoreObject;
    }();
    exports.RenderStoreObject = RenderStoreObject;
    
});
System.registerDynamic("node_modules/@angular/platform-browser/src/web_workers/worker/event_deserializer.js", [], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  // no deserialization is necessary in TS.
  // This is only here to match dart interface

  var global = this || self,
      GLOBAL = global;
  function deserializeGenericEvent(serializedEvent) {
    return serializedEvent;
  }
  exports.deserializeGenericEvent = deserializeGenericEvent;
  
});
System.registerDynamic('node_modules/@angular/platform-browser/src/web_workers/worker/renderer.js', ['@angular/core', '../../facade/async', '../../facade/collection', '../../facade/lang', '../shared/client_message_broker', '../shared/message_bus', '../shared/messaging_api', '../shared/render_store', '../shared/serializer', './event_deserializer'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var async_1 = $__require('../../facade/async');
    var collection_1 = $__require('../../facade/collection');
    var lang_1 = $__require('../../facade/lang');
    var client_message_broker_1 = $__require('../shared/client_message_broker');
    var message_bus_1 = $__require('../shared/message_bus');
    var messaging_api_1 = $__require('../shared/messaging_api');
    var render_store_1 = $__require('../shared/render_store');
    var serializer_1 = $__require('../shared/serializer');
    var event_deserializer_1 = $__require('./event_deserializer');
    var WebWorkerRootRenderer = function () {
        function WebWorkerRootRenderer(messageBrokerFactory, bus, _serializer, _renderStore) {
            var _this = this;
            this._serializer = _serializer;
            this._renderStore = _renderStore;
            this.globalEvents = new NamedEventEmitter();
            this._componentRenderers = new Map();
            this._messageBroker = messageBrokerFactory.createMessageBroker(messaging_api_1.RENDERER_CHANNEL);
            bus.initChannel(messaging_api_1.EVENT_CHANNEL);
            var source = bus.from(messaging_api_1.EVENT_CHANNEL);
            async_1.ObservableWrapper.subscribe(source, function (message) {
                return _this._dispatchEvent(message);
            });
        }
        WebWorkerRootRenderer.prototype._dispatchEvent = function (message) {
            var eventName = message['eventName'];
            var target = message['eventTarget'];
            var event = event_deserializer_1.deserializeGenericEvent(message['event']);
            if (lang_1.isPresent(target)) {
                this.globalEvents.dispatchEvent(eventNameWithTarget(target, eventName), event);
            } else {
                var element = this._serializer.deserialize(message['element'], serializer_1.RenderStoreObject);
                element.events.dispatchEvent(eventName, event);
            }
        };
        WebWorkerRootRenderer.prototype.renderComponent = function (componentType) {
            var result = this._componentRenderers.get(componentType.id);
            if (lang_1.isBlank(result)) {
                result = new WebWorkerRenderer(this, componentType);
                this._componentRenderers.set(componentType.id, result);
                var id = this._renderStore.allocateId();
                this._renderStore.store(result, id);
                this.runOnService('renderComponent', [new client_message_broker_1.FnArg(componentType, core_1.RenderComponentType), new client_message_broker_1.FnArg(result, serializer_1.RenderStoreObject)]);
            }
            return result;
        };
        WebWorkerRootRenderer.prototype.runOnService = function (fnName, fnArgs) {
            var args = new client_message_broker_1.UiArguments(fnName, fnArgs);
            this._messageBroker.runOnService(args, null);
        };
        WebWorkerRootRenderer.prototype.allocateNode = function () {
            var result = new WebWorkerRenderNode();
            var id = this._renderStore.allocateId();
            this._renderStore.store(result, id);
            return result;
        };
        WebWorkerRootRenderer.prototype.allocateId = function () {
            return this._renderStore.allocateId();
        };
        WebWorkerRootRenderer.prototype.destroyNodes = function (nodes) {
            for (var i = 0; i < nodes.length; i++) {
                this._renderStore.remove(nodes[i]);
            }
        };
        /** @nocollapse */
        WebWorkerRootRenderer.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        WebWorkerRootRenderer.ctorParameters = [{ type: client_message_broker_1.ClientMessageBrokerFactory }, { type: message_bus_1.MessageBus }, { type: serializer_1.Serializer }, { type: render_store_1.RenderStore }];
        return WebWorkerRootRenderer;
    }();
    exports.WebWorkerRootRenderer = WebWorkerRootRenderer;
    var WebWorkerRenderer = function () {
        function WebWorkerRenderer(_rootRenderer, _componentType) {
            this._rootRenderer = _rootRenderer;
            this._componentType = _componentType;
        }
        WebWorkerRenderer.prototype._runOnService = function (fnName, fnArgs) {
            var fnArgsWithRenderer = [new client_message_broker_1.FnArg(this, serializer_1.RenderStoreObject)].concat(fnArgs);
            this._rootRenderer.runOnService(fnName, fnArgsWithRenderer);
        };
        WebWorkerRenderer.prototype.selectRootElement = function (selectorOrNode, debugInfo) {
            var node = this._rootRenderer.allocateNode();
            this._runOnService('selectRootElement', [new client_message_broker_1.FnArg(selectorOrNode, null), new client_message_broker_1.FnArg(node, serializer_1.RenderStoreObject)]);
            return node;
        };
        WebWorkerRenderer.prototype.createElement = function (parentElement, name, debugInfo) {
            var node = this._rootRenderer.allocateNode();
            this._runOnService('createElement', [new client_message_broker_1.FnArg(parentElement, serializer_1.RenderStoreObject), new client_message_broker_1.FnArg(name, null), new client_message_broker_1.FnArg(node, serializer_1.RenderStoreObject)]);
            return node;
        };
        WebWorkerRenderer.prototype.createViewRoot = function (hostElement) {
            var viewRoot = this._componentType.encapsulation === core_1.ViewEncapsulation.Native ? this._rootRenderer.allocateNode() : hostElement;
            this._runOnService('createViewRoot', [new client_message_broker_1.FnArg(hostElement, serializer_1.RenderStoreObject), new client_message_broker_1.FnArg(viewRoot, serializer_1.RenderStoreObject)]);
            return viewRoot;
        };
        WebWorkerRenderer.prototype.createTemplateAnchor = function (parentElement, debugInfo) {
            var node = this._rootRenderer.allocateNode();
            this._runOnService('createTemplateAnchor', [new client_message_broker_1.FnArg(parentElement, serializer_1.RenderStoreObject), new client_message_broker_1.FnArg(node, serializer_1.RenderStoreObject)]);
            return node;
        };
        WebWorkerRenderer.prototype.createText = function (parentElement, value, debugInfo) {
            var node = this._rootRenderer.allocateNode();
            this._runOnService('createText', [new client_message_broker_1.FnArg(parentElement, serializer_1.RenderStoreObject), new client_message_broker_1.FnArg(value, null), new client_message_broker_1.FnArg(node, serializer_1.RenderStoreObject)]);
            return node;
        };
        WebWorkerRenderer.prototype.projectNodes = function (parentElement, nodes) {
            this._runOnService('projectNodes', [new client_message_broker_1.FnArg(parentElement, serializer_1.RenderStoreObject), new client_message_broker_1.FnArg(nodes, serializer_1.RenderStoreObject)]);
        };
        WebWorkerRenderer.prototype.attachViewAfter = function (node, viewRootNodes) {
            this._runOnService('attachViewAfter', [new client_message_broker_1.FnArg(node, serializer_1.RenderStoreObject), new client_message_broker_1.FnArg(viewRootNodes, serializer_1.RenderStoreObject)]);
        };
        WebWorkerRenderer.prototype.detachView = function (viewRootNodes) {
            this._runOnService('detachView', [new client_message_broker_1.FnArg(viewRootNodes, serializer_1.RenderStoreObject)]);
        };
        WebWorkerRenderer.prototype.destroyView = function (hostElement, viewAllNodes) {
            this._runOnService('destroyView', [new client_message_broker_1.FnArg(hostElement, serializer_1.RenderStoreObject), new client_message_broker_1.FnArg(viewAllNodes, serializer_1.RenderStoreObject)]);
            this._rootRenderer.destroyNodes(viewAllNodes);
        };
        WebWorkerRenderer.prototype.setElementProperty = function (renderElement, propertyName, propertyValue) {
            this._runOnService('setElementProperty', [new client_message_broker_1.FnArg(renderElement, serializer_1.RenderStoreObject), new client_message_broker_1.FnArg(propertyName, null), new client_message_broker_1.FnArg(propertyValue, null)]);
        };
        WebWorkerRenderer.prototype.setElementAttribute = function (renderElement, attributeName, attributeValue) {
            this._runOnService('setElementAttribute', [new client_message_broker_1.FnArg(renderElement, serializer_1.RenderStoreObject), new client_message_broker_1.FnArg(attributeName, null), new client_message_broker_1.FnArg(attributeValue, null)]);
        };
        WebWorkerRenderer.prototype.setBindingDebugInfo = function (renderElement, propertyName, propertyValue) {
            this._runOnService('setBindingDebugInfo', [new client_message_broker_1.FnArg(renderElement, serializer_1.RenderStoreObject), new client_message_broker_1.FnArg(propertyName, null), new client_message_broker_1.FnArg(propertyValue, null)]);
        };
        WebWorkerRenderer.prototype.setElementClass = function (renderElement, className, isAdd) {
            this._runOnService('setElementClass', [new client_message_broker_1.FnArg(renderElement, serializer_1.RenderStoreObject), new client_message_broker_1.FnArg(className, null), new client_message_broker_1.FnArg(isAdd, null)]);
        };
        WebWorkerRenderer.prototype.setElementStyle = function (renderElement, styleName, styleValue) {
            this._runOnService('setElementStyle', [new client_message_broker_1.FnArg(renderElement, serializer_1.RenderStoreObject), new client_message_broker_1.FnArg(styleName, null), new client_message_broker_1.FnArg(styleValue, null)]);
        };
        WebWorkerRenderer.prototype.invokeElementMethod = function (renderElement, methodName, args) {
            this._runOnService('invokeElementMethod', [new client_message_broker_1.FnArg(renderElement, serializer_1.RenderStoreObject), new client_message_broker_1.FnArg(methodName, null), new client_message_broker_1.FnArg(args, null)]);
        };
        WebWorkerRenderer.prototype.setText = function (renderNode, text) {
            this._runOnService('setText', [new client_message_broker_1.FnArg(renderNode, serializer_1.RenderStoreObject), new client_message_broker_1.FnArg(text, null)]);
        };
        WebWorkerRenderer.prototype.listen = function (renderElement, name, callback) {
            var _this = this;
            renderElement.events.listen(name, callback);
            var unlistenCallbackId = this._rootRenderer.allocateId();
            this._runOnService('listen', [new client_message_broker_1.FnArg(renderElement, serializer_1.RenderStoreObject), new client_message_broker_1.FnArg(name, null), new client_message_broker_1.FnArg(unlistenCallbackId, null)]);
            return function () {
                renderElement.events.unlisten(name, callback);
                _this._runOnService('listenDone', [new client_message_broker_1.FnArg(unlistenCallbackId, null)]);
            };
        };
        WebWorkerRenderer.prototype.listenGlobal = function (target, name, callback) {
            var _this = this;
            this._rootRenderer.globalEvents.listen(eventNameWithTarget(target, name), callback);
            var unlistenCallbackId = this._rootRenderer.allocateId();
            this._runOnService('listenGlobal', [new client_message_broker_1.FnArg(target, null), new client_message_broker_1.FnArg(name, null), new client_message_broker_1.FnArg(unlistenCallbackId, null)]);
            return function () {
                _this._rootRenderer.globalEvents.unlisten(eventNameWithTarget(target, name), callback);
                _this._runOnService('listenDone', [new client_message_broker_1.FnArg(unlistenCallbackId, null)]);
            };
        };
        WebWorkerRenderer.prototype.animate = function (element, startingStyles, keyframes, duration, delay, easing) {
            // TODO
            return null;
        };
        return WebWorkerRenderer;
    }();
    exports.WebWorkerRenderer = WebWorkerRenderer;
    var NamedEventEmitter = function () {
        function NamedEventEmitter() {}
        NamedEventEmitter.prototype._getListeners = function (eventName) {
            if (lang_1.isBlank(this._listeners)) {
                this._listeners = new Map();
            }
            var listeners = this._listeners.get(eventName);
            if (lang_1.isBlank(listeners)) {
                listeners = [];
                this._listeners.set(eventName, listeners);
            }
            return listeners;
        };
        NamedEventEmitter.prototype.listen = function (eventName, callback) {
            this._getListeners(eventName).push(callback);
        };
        NamedEventEmitter.prototype.unlisten = function (eventName, callback) {
            collection_1.ListWrapper.remove(this._getListeners(eventName), callback);
        };
        NamedEventEmitter.prototype.dispatchEvent = function (eventName, event) {
            var listeners = this._getListeners(eventName);
            for (var i = 0; i < listeners.length; i++) {
                listeners[i](event);
            }
        };
        return NamedEventEmitter;
    }();
    exports.NamedEventEmitter = NamedEventEmitter;
    function eventNameWithTarget(target, eventName) {
        return target + ":" + eventName;
    }
    var WebWorkerRenderNode = function () {
        function WebWorkerRenderNode() {
            this.events = new NamedEventEmitter();
        }
        return WebWorkerRenderNode;
    }();
    exports.WebWorkerRenderNode = WebWorkerRenderNode;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/web_workers/worker/worker_adapter.js', ['../../dom/dom_adapter'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var dom_adapter_1 = $__require('../../dom/dom_adapter');
    /**
     * This adapter is required to log error messages.
     *
     * Note: other methods all throw as the DOM is not accessible directly in web worker context.
     */
    var WorkerDomAdapter = function (_super) {
        __extends(WorkerDomAdapter, _super);
        function WorkerDomAdapter() {
            _super.apply(this, arguments);
        }
        WorkerDomAdapter.makeCurrent = function () {
            dom_adapter_1.setRootDomAdapter(new WorkerDomAdapter());
        };
        WorkerDomAdapter.prototype.logError = function (error /** TODO #9100 */) {
            if (console.error) {
                console.error(error);
            } else {
                console.log(error);
            }
        };
        WorkerDomAdapter.prototype.log = function (error /** TODO #9100 */) {
            console.log(error);
        };
        WorkerDomAdapter.prototype.logGroup = function (error /** TODO #9100 */) {
            if (console.group) {
                console.group(error);
                this.logError(error);
            } else {
                console.log(error);
            }
        };
        WorkerDomAdapter.prototype.logGroupEnd = function () {
            if (console.groupEnd) {
                console.groupEnd();
            }
        };
        WorkerDomAdapter.prototype.hasProperty = function (element /** TODO #9100 */, name) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.setProperty = function (el, name, value) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getProperty = function (el, name) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.invoke = function (el, methodName, args) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getXHR = function () {
            throw 'not implemented';
        };
        Object.defineProperty(WorkerDomAdapter.prototype, "attrToPropMap", {
            get: function () {
                throw 'not implemented';
            },
            set: function (value) {
                throw 'not implemented';
            },
            enumerable: true,
            configurable: true
        });
        WorkerDomAdapter.prototype.parse = function (templateHtml) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.query = function (selector) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.querySelector = function (el /** TODO #9100 */, selector) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.querySelectorAll = function (el /** TODO #9100 */, selector) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.on = function (el /** TODO #9100 */, evt /** TODO #9100 */, listener /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.onAndCancel = function (el /** TODO #9100 */, evt /** TODO #9100 */, listener /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.dispatchEvent = function (el /** TODO #9100 */, evt /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.createMouseEvent = function (eventType /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.createEvent = function (eventType) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.preventDefault = function (evt /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.isPrevented = function (evt /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getInnerHTML = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getTemplateContent = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getOuterHTML = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.nodeName = function (node /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.nodeValue = function (node /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.type = function (node /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.content = function (node /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.firstChild = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.nextSibling = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.parentElement = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.childNodes = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.childNodesAsList = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.clearNodes = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.appendChild = function (el /** TODO #9100 */, node /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.removeChild = function (el /** TODO #9100 */, node /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.replaceChild = function (el /** TODO #9100 */, newNode /** TODO #9100 */, oldNode /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.remove = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.insertBefore = function (el /** TODO #9100 */, node /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.insertAllBefore = function (el /** TODO #9100 */, nodes /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.insertAfter = function (el /** TODO #9100 */, node /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.setInnerHTML = function (el /** TODO #9100 */, value /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getText = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.setText = function (el /** TODO #9100 */, value) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getValue = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.setValue = function (el /** TODO #9100 */, value) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getChecked = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.setChecked = function (el /** TODO #9100 */, value) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.createComment = function (text) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.createTemplate = function (html /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.createElement = function (tagName /** TODO #9100 */, doc /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.createElementNS = function (ns, tagName, doc /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.createTextNode = function (text, doc /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.createScriptTag = function (attrName, attrValue, doc /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.createStyleElement = function (css, doc /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.createShadowRoot = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getShadowRoot = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getHost = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getDistributedNodes = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.clone = function (node) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getElementsByClassName = function (element /** TODO #9100 */, name) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getElementsByTagName = function (element /** TODO #9100 */, name) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.classList = function (element /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.addClass = function (element /** TODO #9100 */, className) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.removeClass = function (element /** TODO #9100 */, className) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.hasClass = function (element /** TODO #9100 */, className) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.setStyle = function (element /** TODO #9100 */, styleName, styleValue) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.removeStyle = function (element /** TODO #9100 */, styleName) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getStyle = function (element /** TODO #9100 */, styleName) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.hasStyle = function (element /** TODO #9100 */, styleName, styleValue) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.tagName = function (element /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.attributeMap = function (element /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.hasAttribute = function (element /** TODO #9100 */, attribute) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.hasAttributeNS = function (element /** TODO #9100 */, ns, attribute) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getAttribute = function (element /** TODO #9100 */, attribute) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getAttributeNS = function (element /** TODO #9100 */, ns, attribute) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.setAttribute = function (element /** TODO #9100 */, name, value) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.setAttributeNS = function (element /** TODO #9100 */, ns, name, value) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.removeAttribute = function (element /** TODO #9100 */, attribute) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.removeAttributeNS = function (element /** TODO #9100 */, ns, attribute) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.templateAwareRoot = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.createHtmlDocument = function () {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.defaultDoc = function () {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getBoundingClientRect = function (el /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getTitle = function () {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.setTitle = function (newTitle) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.elementMatches = function (n /** TODO #9100 */, selector) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.isTemplateElement = function (el) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.isTextNode = function (node /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.isCommentNode = function (node /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.isElementNode = function (node /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.hasShadowRoot = function (node /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.isShadowRoot = function (node /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.importIntoDoc = function (node) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.adoptNode = function (node) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getHref = function (element /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getEventKey = function (event /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.resolveAndSetHref = function (element /** TODO #9100 */, baseUrl, href) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.supportsDOMEvents = function () {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.supportsNativeShadowDOM = function () {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getGlobalEventTarget = function (target) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getHistory = function () {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getLocation = function () {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getBaseHref = function () {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.resetBaseElement = function () {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getUserAgent = function () {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.setData = function (element /** TODO #9100 */, name, value) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getComputedStyle = function (element /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getData = function (element /** TODO #9100 */, name) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.setGlobalVar = function (name, value) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.requestAnimationFrame = function (callback /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.cancelAnimationFrame = function (id /** TODO #9100 */) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.performanceNow = function () {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getAnimationPrefix = function () {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.getTransitionEnd = function () {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.supportsAnimation = function () {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.supportsWebAnimation = function () {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.supportsCookies = function () {
            return false;
        };
        WorkerDomAdapter.prototype.getCookie = function (name) {
            throw 'not implemented';
        };
        WorkerDomAdapter.prototype.setCookie = function (name, value) {
            throw 'not implemented';
        };
        return WorkerDomAdapter;
    }(dom_adapter_1.DomAdapter);
    exports.WorkerDomAdapter = WorkerDomAdapter;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/worker_app.js', ['@angular/common', '@angular/core', './browser', './facade/lang', './web_workers/shared/api', './web_workers/shared/client_message_broker', './web_workers/shared/message_bus', './web_workers/shared/post_message_bus', './web_workers/shared/render_store', './web_workers/shared/serializer', './web_workers/shared/service_message_broker', './web_workers/worker/renderer', './web_workers/worker/worker_adapter'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var common_1 = $__require('@angular/common');
    var core_1 = $__require('@angular/core');
    var browser_1 = $__require('./browser');
    var lang_1 = $__require('./facade/lang');
    var api_1 = $__require('./web_workers/shared/api');
    var client_message_broker_1 = $__require('./web_workers/shared/client_message_broker');
    var message_bus_1 = $__require('./web_workers/shared/message_bus');
    var post_message_bus_1 = $__require('./web_workers/shared/post_message_bus');
    var render_store_1 = $__require('./web_workers/shared/render_store');
    var serializer_1 = $__require('./web_workers/shared/serializer');
    var service_message_broker_1 = $__require('./web_workers/shared/service_message_broker');
    var renderer_1 = $__require('./web_workers/worker/renderer');
    var worker_adapter_1 = $__require('./web_workers/worker/worker_adapter');
    var PrintLogger = function () {
        function PrintLogger() {
            this.log = lang_1.print;
            this.logError = lang_1.print;
            this.logGroup = lang_1.print;
        }
        PrintLogger.prototype.logGroupEnd = function () {};
        return PrintLogger;
    }();
    var WORKER_APP_PLATFORM_MARKER = new core_1.OpaqueToken('WorkerAppPlatformMarker');
    /**
     * @experimental
     */
    exports.WORKER_APP_PLATFORM_PROVIDERS = [core_1.PLATFORM_COMMON_PROVIDERS, { provide: WORKER_APP_PLATFORM_MARKER, useValue: true }];
    /**
     * @experimental
     */
    exports.WORKER_APP_APPLICATION_PROVIDERS = [core_1.APPLICATION_COMMON_PROVIDERS, common_1.FORM_PROVIDERS, browser_1.BROWSER_SANITIZATION_PROVIDERS, serializer_1.Serializer, { provide: client_message_broker_1.ClientMessageBrokerFactory, useClass: client_message_broker_1.ClientMessageBrokerFactory_ }, { provide: service_message_broker_1.ServiceMessageBrokerFactory, useClass: service_message_broker_1.ServiceMessageBrokerFactory_ }, renderer_1.WebWorkerRootRenderer, { provide: core_1.RootRenderer, useExisting: renderer_1.WebWorkerRootRenderer }, { provide: api_1.ON_WEB_WORKER, useValue: true }, render_store_1.RenderStore, { provide: core_1.ExceptionHandler, useFactory: _exceptionHandler, deps: [] }, { provide: message_bus_1.MessageBus, useFactory: createMessageBus, deps: [core_1.NgZone] }, { provide: core_1.APP_INITIALIZER, useValue: setupWebWorker, multi: true }];
    /**
     * @experimental
     */
    function workerAppPlatform() {
        if (lang_1.isBlank(core_1.getPlatform())) {
            core_1.createPlatform(core_1.ReflectiveInjector.resolveAndCreate(exports.WORKER_APP_PLATFORM_PROVIDERS));
        }
        return core_1.assertPlatform(WORKER_APP_PLATFORM_MARKER);
    }
    exports.workerAppPlatform = workerAppPlatform;
    function _exceptionHandler() {
        return new core_1.ExceptionHandler(new PrintLogger());
    }
    // TODO(jteplitz602) remove this and compile with lib.webworker.d.ts (#3492)
    var _postMessage = {
        postMessage: function (message, transferrables) {
            postMessage(message, transferrables);
        }
    };
    function createMessageBus(zone) {
        var sink = new post_message_bus_1.PostMessageBusSink(_postMessage);
        var source = new post_message_bus_1.PostMessageBusSource();
        var bus = new post_message_bus_1.PostMessageBus(sink, source);
        bus.attachToZone(zone);
        return bus;
    }
    function setupWebWorker() {
        worker_adapter_1.WorkerDomAdapter.makeCurrent();
    }
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/dom/debug/ng_probe.js', ['@angular/core', '../../../core_private', '../dom_adapter', '../dom_renderer'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var core_private_1 = $__require('../../../core_private');
    var dom_adapter_1 = $__require('../dom_adapter');
    var dom_renderer_1 = $__require('../dom_renderer');
    var CORE_TOKENS = {
        'ApplicationRef': core_1.ApplicationRef,
        'NgZone': core_1.NgZone
    };
    var INSPECT_GLOBAL_NAME = 'ng.probe';
    var CORE_TOKENS_GLOBAL_NAME = 'ng.coreTokens';
    /**
     * Returns a {@link DebugElement} for the given native DOM element, or
     * null if the given native element does not have an Angular view associated
     * with it.
     */
    function inspectNativeElement(element /** TODO #9100 */) {
        return core_1.getDebugNode(element);
    }
    exports.inspectNativeElement = inspectNativeElement;
    function _createConditionalRootRenderer(rootRenderer /** TODO #9100 */) {
        if (core_1.isDevMode()) {
            return _createRootRenderer(rootRenderer);
        }
        return rootRenderer;
    }
    function _createRootRenderer(rootRenderer /** TODO #9100 */) {
        dom_adapter_1.getDOM().setGlobalVar(INSPECT_GLOBAL_NAME, inspectNativeElement);
        dom_adapter_1.getDOM().setGlobalVar(CORE_TOKENS_GLOBAL_NAME, CORE_TOKENS);
        return new core_private_1.DebugDomRootRenderer(rootRenderer);
    }
    /**
     * Providers which support debugging Angular applications (e.g. via `ng.probe`).
     */
    exports.ELEMENT_PROBE_PROVIDERS = [{ provide: core_1.RootRenderer, useFactory: _createConditionalRootRenderer, deps: [dom_renderer_1.DomRootRenderer] }];
    exports.ELEMENT_PROBE_PROVIDERS_PROD_MODE = [{ provide: core_1.RootRenderer, useFactory: _createRootRenderer, deps: [dom_renderer_1.DomRootRenderer] }];
    
});
System.registerDynamic("node_modules/@angular/platform-browser/core_private.js", ["@angular/core"], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var core_1 = $__require('@angular/core');
  exports.RenderDebugInfo = core_1.__core_private__.RenderDebugInfo;
  exports.wtfInit = core_1.__core_private__.wtfInit;
  exports.ReflectionCapabilities = core_1.__core_private__.ReflectionCapabilities;
  exports.VIEW_ENCAPSULATION_VALUES = core_1.__core_private__.VIEW_ENCAPSULATION_VALUES;
  exports.DebugDomRootRenderer = core_1.__core_private__.DebugDomRootRenderer;
  /**
   * @experimental bogus marker to pass the ts-api-guardian's check - this api should be public so
   * this line will go away when that happens
   */
  exports.SecurityContext = core_1.__core_private__.SecurityContext;
  exports.SanitizationService = core_1.__core_private__.SanitizationService;
  exports.reflector = core_1.__core_private__.reflector;
  exports.NoOpAnimationPlayer = core_1.__core_private__.NoOpAnimationPlayer;
  exports.AnimationPlayer = core_1.__core_private__.AnimationPlayer;
  exports.NoOpAnimationDriver = core_1.__core_private__.NoOpAnimationDriver;
  exports.AnimationDriver = core_1.__core_private__.AnimationDriver;
  exports.AnimationSequencePlayer = core_1.__core_private__.AnimationSequencePlayer;
  exports.AnimationGroupPlayer = core_1.__core_private__.AnimationGroupPlayer;
  exports.AnimationKeyframe = core_1.__core_private__.AnimationKeyframe;
  exports.AnimationStyles = core_1.__core_private__.AnimationStyles;
  exports.prepareFinalAnimationStyles = core_1.__core_private__.prepareFinalAnimationStyles;
  exports.balanceAnimationKeyframes = core_1.__core_private__.balanceAnimationKeyframes;
  exports.flattenStyles = core_1.__core_private__.flattenStyles;
  exports.clearStyles = core_1.__core_private__.clearStyles;
  exports.collectAndResolveStyles = core_1.__core_private__.collectAndResolveStyles;
  
});
System.registerDynamic('node_modules/@angular/platform-browser/src/dom/util.js', ['../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('../facade/lang');
    var CAMEL_CASE_REGEXP = /([A-Z])/g;
    var DASH_CASE_REGEXP = /-([a-z])/g;
    function camelCaseToDashCase(input) {
        return lang_1.StringWrapper.replaceAllMapped(input, CAMEL_CASE_REGEXP, function (m /** TODO #9100 */) {
            return '-' + m[1].toLowerCase();
        });
    }
    exports.camelCaseToDashCase = camelCaseToDashCase;
    function dashCaseToCamelCase(input) {
        return lang_1.StringWrapper.replaceAllMapped(input, DASH_CASE_REGEXP, function (m /** TODO #9100 */) {
            return m[1].toUpperCase();
        });
    }
    exports.dashCaseToCamelCase = dashCaseToCamelCase;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/dom/dom_renderer.js', ['@angular/core', '../facade/exceptions', '../facade/lang', './shared_styles_host', '../../core_private', './events/event_manager', './dom_tokens', './dom_adapter', './util'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    var shared_styles_host_1 = $__require('./shared_styles_host');
    var core_private_1 = $__require('../../core_private');
    var event_manager_1 = $__require('./events/event_manager');
    var dom_tokens_1 = $__require('./dom_tokens');
    var dom_adapter_1 = $__require('./dom_adapter');
    var util_1 = $__require('./util');
    var NAMESPACE_URIS = {
        'xlink': 'http://www.w3.org/1999/xlink',
        'svg': 'http://www.w3.org/2000/svg',
        'xhtml': 'http://www.w3.org/1999/xhtml'
    };
    var TEMPLATE_COMMENT_TEXT = 'template bindings={}';
    var TEMPLATE_BINDINGS_EXP = /^template bindings=(.*)$/g;
    var DomRootRenderer = function () {
        function DomRootRenderer(document, eventManager, sharedStylesHost, animationDriver) {
            this.document = document;
            this.eventManager = eventManager;
            this.sharedStylesHost = sharedStylesHost;
            this.animationDriver = animationDriver;
            this.registeredComponents = new Map();
        }
        DomRootRenderer.prototype.renderComponent = function (componentProto) {
            var renderer = this.registeredComponents.get(componentProto.id);
            if (lang_1.isBlank(renderer)) {
                renderer = new DomRenderer(this, componentProto, this.animationDriver);
                this.registeredComponents.set(componentProto.id, renderer);
            }
            return renderer;
        };
        return DomRootRenderer;
    }();
    exports.DomRootRenderer = DomRootRenderer;
    var DomRootRenderer_ = function (_super) {
        __extends(DomRootRenderer_, _super);
        function DomRootRenderer_(_document, _eventManager, sharedStylesHost, animationDriver) {
            _super.call(this, _document, _eventManager, sharedStylesHost, animationDriver);
        }
        /** @nocollapse */
        DomRootRenderer_.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        DomRootRenderer_.ctorParameters = [{ type: undefined, decorators: [{ type: core_1.Inject, args: [dom_tokens_1.DOCUMENT] }] }, { type: event_manager_1.EventManager }, { type: shared_styles_host_1.DomSharedStylesHost }, { type: core_private_1.AnimationDriver }];
        return DomRootRenderer_;
    }(DomRootRenderer);
    exports.DomRootRenderer_ = DomRootRenderer_;
    var DomRenderer = function () {
        function DomRenderer(_rootRenderer, componentProto, _animationDriver) {
            this._rootRenderer = _rootRenderer;
            this.componentProto = componentProto;
            this._animationDriver = _animationDriver;
            this._styles = _flattenStyles(componentProto.id, componentProto.styles, []);
            if (componentProto.encapsulation !== core_1.ViewEncapsulation.Native) {
                this._rootRenderer.sharedStylesHost.addStyles(this._styles);
            }
            if (this.componentProto.encapsulation === core_1.ViewEncapsulation.Emulated) {
                this._contentAttr = _shimContentAttribute(componentProto.id);
                this._hostAttr = _shimHostAttribute(componentProto.id);
            } else {
                this._contentAttr = null;
                this._hostAttr = null;
            }
        }
        DomRenderer.prototype.selectRootElement = function (selectorOrNode, debugInfo) {
            var el;
            if (lang_1.isString(selectorOrNode)) {
                el = dom_adapter_1.getDOM().querySelector(this._rootRenderer.document, selectorOrNode);
                if (lang_1.isBlank(el)) {
                    throw new exceptions_1.BaseException("The selector \"" + selectorOrNode + "\" did not match any elements");
                }
            } else {
                el = selectorOrNode;
            }
            dom_adapter_1.getDOM().clearNodes(el);
            return el;
        };
        DomRenderer.prototype.createElement = function (parent, name, debugInfo) {
            var nsAndName = splitNamespace(name);
            var el = lang_1.isPresent(nsAndName[0]) ? dom_adapter_1.getDOM().createElementNS(NAMESPACE_URIS[nsAndName[0]], nsAndName[1]) : dom_adapter_1.getDOM().createElement(nsAndName[1]);
            if (lang_1.isPresent(this._contentAttr)) {
                dom_adapter_1.getDOM().setAttribute(el, this._contentAttr, '');
            }
            if (lang_1.isPresent(parent)) {
                dom_adapter_1.getDOM().appendChild(parent, el);
            }
            return el;
        };
        DomRenderer.prototype.createViewRoot = function (hostElement) {
            var nodesParent;
            if (this.componentProto.encapsulation === core_1.ViewEncapsulation.Native) {
                nodesParent = dom_adapter_1.getDOM().createShadowRoot(hostElement);
                this._rootRenderer.sharedStylesHost.addHost(nodesParent);
                for (var i = 0; i < this._styles.length; i++) {
                    dom_adapter_1.getDOM().appendChild(nodesParent, dom_adapter_1.getDOM().createStyleElement(this._styles[i]));
                }
            } else {
                if (lang_1.isPresent(this._hostAttr)) {
                    dom_adapter_1.getDOM().setAttribute(hostElement, this._hostAttr, '');
                }
                nodesParent = hostElement;
            }
            return nodesParent;
        };
        DomRenderer.prototype.createTemplateAnchor = function (parentElement, debugInfo) {
            var comment = dom_adapter_1.getDOM().createComment(TEMPLATE_COMMENT_TEXT);
            if (lang_1.isPresent(parentElement)) {
                dom_adapter_1.getDOM().appendChild(parentElement, comment);
            }
            return comment;
        };
        DomRenderer.prototype.createText = function (parentElement, value, debugInfo) {
            var node = dom_adapter_1.getDOM().createTextNode(value);
            if (lang_1.isPresent(parentElement)) {
                dom_adapter_1.getDOM().appendChild(parentElement, node);
            }
            return node;
        };
        DomRenderer.prototype.projectNodes = function (parentElement, nodes) {
            if (lang_1.isBlank(parentElement)) return;
            appendNodes(parentElement, nodes);
        };
        DomRenderer.prototype.attachViewAfter = function (node, viewRootNodes) {
            moveNodesAfterSibling(node, viewRootNodes);
        };
        DomRenderer.prototype.detachView = function (viewRootNodes) {
            for (var i = 0; i < viewRootNodes.length; i++) {
                dom_adapter_1.getDOM().remove(viewRootNodes[i]);
            }
        };
        DomRenderer.prototype.destroyView = function (hostElement, viewAllNodes) {
            if (this.componentProto.encapsulation === core_1.ViewEncapsulation.Native && lang_1.isPresent(hostElement)) {
                this._rootRenderer.sharedStylesHost.removeHost(dom_adapter_1.getDOM().getShadowRoot(hostElement));
            }
        };
        DomRenderer.prototype.listen = function (renderElement, name, callback) {
            return this._rootRenderer.eventManager.addEventListener(renderElement, name, decoratePreventDefault(callback));
        };
        DomRenderer.prototype.listenGlobal = function (target, name, callback) {
            return this._rootRenderer.eventManager.addGlobalEventListener(target, name, decoratePreventDefault(callback));
        };
        DomRenderer.prototype.setElementProperty = function (renderElement, propertyName, propertyValue) {
            dom_adapter_1.getDOM().setProperty(renderElement, propertyName, propertyValue);
        };
        DomRenderer.prototype.setElementAttribute = function (renderElement, attributeName, attributeValue) {
            var attrNs;
            var nsAndName = splitNamespace(attributeName);
            if (lang_1.isPresent(nsAndName[0])) {
                attributeName = nsAndName[0] + ':' + nsAndName[1];
                attrNs = NAMESPACE_URIS[nsAndName[0]];
            }
            if (lang_1.isPresent(attributeValue)) {
                if (lang_1.isPresent(attrNs)) {
                    dom_adapter_1.getDOM().setAttributeNS(renderElement, attrNs, attributeName, attributeValue);
                } else {
                    dom_adapter_1.getDOM().setAttribute(renderElement, attributeName, attributeValue);
                }
            } else {
                if (lang_1.isPresent(attrNs)) {
                    dom_adapter_1.getDOM().removeAttributeNS(renderElement, attrNs, nsAndName[1]);
                } else {
                    dom_adapter_1.getDOM().removeAttribute(renderElement, attributeName);
                }
            }
        };
        DomRenderer.prototype.setBindingDebugInfo = function (renderElement, propertyName, propertyValue) {
            var dashCasedPropertyName = util_1.camelCaseToDashCase(propertyName);
            if (dom_adapter_1.getDOM().isCommentNode(renderElement)) {
                var existingBindings = lang_1.RegExpWrapper.firstMatch(TEMPLATE_BINDINGS_EXP, lang_1.StringWrapper.replaceAll(dom_adapter_1.getDOM().getText(renderElement), /\n/g, ''));
                var parsedBindings = lang_1.Json.parse(existingBindings[1]);
                parsedBindings[dashCasedPropertyName] = propertyValue;
                dom_adapter_1.getDOM().setText(renderElement, lang_1.StringWrapper.replace(TEMPLATE_COMMENT_TEXT, '{}', lang_1.Json.stringify(parsedBindings)));
            } else {
                this.setElementAttribute(renderElement, propertyName, propertyValue);
            }
        };
        DomRenderer.prototype.setElementClass = function (renderElement, className, isAdd) {
            if (isAdd) {
                dom_adapter_1.getDOM().addClass(renderElement, className);
            } else {
                dom_adapter_1.getDOM().removeClass(renderElement, className);
            }
        };
        DomRenderer.prototype.setElementStyle = function (renderElement, styleName, styleValue) {
            if (lang_1.isPresent(styleValue)) {
                dom_adapter_1.getDOM().setStyle(renderElement, styleName, lang_1.stringify(styleValue));
            } else {
                dom_adapter_1.getDOM().removeStyle(renderElement, styleName);
            }
        };
        DomRenderer.prototype.invokeElementMethod = function (renderElement, methodName, args) {
            dom_adapter_1.getDOM().invoke(renderElement, methodName, args);
        };
        DomRenderer.prototype.setText = function (renderNode, text) {
            dom_adapter_1.getDOM().setText(renderNode, text);
        };
        DomRenderer.prototype.animate = function (element, startingStyles, keyframes, duration, delay, easing) {
            return this._animationDriver.animate(element, startingStyles, keyframes, duration, delay, easing);
        };
        return DomRenderer;
    }();
    exports.DomRenderer = DomRenderer;
    function moveNodesAfterSibling(sibling /** TODO #9100 */, nodes /** TODO #9100 */) {
        var parent = dom_adapter_1.getDOM().parentElement(sibling);
        if (nodes.length > 0 && lang_1.isPresent(parent)) {
            var nextSibling = dom_adapter_1.getDOM().nextSibling(sibling);
            if (lang_1.isPresent(nextSibling)) {
                for (var i = 0; i < nodes.length; i++) {
                    dom_adapter_1.getDOM().insertBefore(nextSibling, nodes[i]);
                }
            } else {
                for (var i = 0; i < nodes.length; i++) {
                    dom_adapter_1.getDOM().appendChild(parent, nodes[i]);
                }
            }
        }
    }
    function appendNodes(parent /** TODO #9100 */, nodes /** TODO #9100 */) {
        for (var i = 0; i < nodes.length; i++) {
            dom_adapter_1.getDOM().appendChild(parent, nodes[i]);
        }
    }
    function decoratePreventDefault(eventHandler) {
        return function (event /** TODO #9100 */) {
            var allowDefaultBehavior = eventHandler(event);
            if (allowDefaultBehavior === false) {
                // TODO(tbosch): move preventDefault into event plugins...
                dom_adapter_1.getDOM().preventDefault(event);
            }
        };
    }
    var COMPONENT_REGEX = /%COMP%/g;
    exports.COMPONENT_VARIABLE = '%COMP%';
    exports.HOST_ATTR = "_nghost-" + exports.COMPONENT_VARIABLE;
    exports.CONTENT_ATTR = "_ngcontent-" + exports.COMPONENT_VARIABLE;
    function _shimContentAttribute(componentShortId) {
        return lang_1.StringWrapper.replaceAll(exports.CONTENT_ATTR, COMPONENT_REGEX, componentShortId);
    }
    function _shimHostAttribute(componentShortId) {
        return lang_1.StringWrapper.replaceAll(exports.HOST_ATTR, COMPONENT_REGEX, componentShortId);
    }
    function _flattenStyles(compId, styles, target) {
        for (var i = 0; i < styles.length; i++) {
            var style = styles[i];
            if (lang_1.isArray(style)) {
                _flattenStyles(compId, style, target);
            } else {
                style = lang_1.StringWrapper.replaceAll(style, COMPONENT_REGEX, compId);
                target.push(style);
            }
        }
        return target;
    }
    var NS_PREFIX_RE = /^:([^:]+):(.+)/g;
    function splitNamespace(name) {
        if (name[0] != ':') {
            return [null, name];
        }
        var match = lang_1.RegExpWrapper.firstMatch(NS_PREFIX_RE, name);
        return [match[1], match[2]];
    }
    
});
System.registerDynamic("node_modules/@angular/platform-browser/src/facade/base_wrapped_exception.js", [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    /**
     * A base class for the WrappedException that can be used to identify
     * a WrappedException from ExceptionHandler without adding circular
     * dependency.
     */
    var BaseWrappedException = function (_super) {
        __extends(BaseWrappedException, _super);
        function BaseWrappedException(message) {
            _super.call(this, message);
        }
        Object.defineProperty(BaseWrappedException.prototype, "wrapperMessage", {
            get: function () {
                return '';
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "wrapperStack", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "originalException", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "originalStack", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "context", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "message", {
            get: function () {
                return '';
            },
            enumerable: true,
            configurable: true
        });
        return BaseWrappedException;
    }(Error);
    exports.BaseWrappedException = BaseWrappedException;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/facade/exception_handler.js', ['./base_wrapped_exception', './collection', './lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var base_wrapped_exception_1 = $__require('./base_wrapped_exception');
    var collection_1 = $__require('./collection');
    var lang_1 = $__require('./lang');
    var _ArrayLogger = function () {
        function _ArrayLogger() {
            this.res = [];
        }
        _ArrayLogger.prototype.log = function (s) {
            this.res.push(s);
        };
        _ArrayLogger.prototype.logError = function (s) {
            this.res.push(s);
        };
        _ArrayLogger.prototype.logGroup = function (s) {
            this.res.push(s);
        };
        _ArrayLogger.prototype.logGroupEnd = function () {};
        ;
        return _ArrayLogger;
    }();
    /**
     * Provides a hook for centralized exception handling.
     *
     * The default implementation of `ExceptionHandler` prints error messages to the `Console`. To
     * intercept error handling,
     * write a custom exception handler that replaces this default as appropriate for your app.
     *
     * ### Example
     *
     * ```javascript
     *
     * class MyExceptionHandler implements ExceptionHandler {
     *   call(error, stackTrace = null, reason = null) {
     *     // do something with the exception
     *   }
     * }
     *
     * bootstrap(MyApp, {provide: ExceptionHandler, useClass: MyExceptionHandler}])
     *
     * ```
     * @stable
     */
    var ExceptionHandler = function () {
        function ExceptionHandler(_logger, _rethrowException) {
            if (_rethrowException === void 0) {
                _rethrowException = true;
            }
            this._logger = _logger;
            this._rethrowException = _rethrowException;
        }
        ExceptionHandler.exceptionToString = function (exception, stackTrace, reason) {
            if (stackTrace === void 0) {
                stackTrace = null;
            }
            if (reason === void 0) {
                reason = null;
            }
            var l = new _ArrayLogger();
            var e = new ExceptionHandler(l, false);
            e.call(exception, stackTrace, reason);
            return l.res.join('\n');
        };
        ExceptionHandler.prototype.call = function (exception, stackTrace, reason) {
            if (stackTrace === void 0) {
                stackTrace = null;
            }
            if (reason === void 0) {
                reason = null;
            }
            var originalException = this._findOriginalException(exception);
            var originalStack = this._findOriginalStack(exception);
            var context = this._findContext(exception);
            this._logger.logGroup("EXCEPTION: " + this._extractMessage(exception));
            if (lang_1.isPresent(stackTrace) && lang_1.isBlank(originalStack)) {
                this._logger.logError('STACKTRACE:');
                this._logger.logError(this._longStackTrace(stackTrace));
            }
            if (lang_1.isPresent(reason)) {
                this._logger.logError("REASON: " + reason);
            }
            if (lang_1.isPresent(originalException)) {
                this._logger.logError("ORIGINAL EXCEPTION: " + this._extractMessage(originalException));
            }
            if (lang_1.isPresent(originalStack)) {
                this._logger.logError('ORIGINAL STACKTRACE:');
                this._logger.logError(this._longStackTrace(originalStack));
            }
            if (lang_1.isPresent(context)) {
                this._logger.logError('ERROR CONTEXT:');
                this._logger.logError(context);
            }
            this._logger.logGroupEnd();
            // We rethrow exceptions, so operations like 'bootstrap' will result in an error
            // when an exception happens. If we do not rethrow, bootstrap will always succeed.
            if (this._rethrowException) throw exception;
        };
        /** @internal */
        ExceptionHandler.prototype._extractMessage = function (exception) {
            return exception instanceof base_wrapped_exception_1.BaseWrappedException ? exception.wrapperMessage : exception.toString();
        };
        /** @internal */
        ExceptionHandler.prototype._longStackTrace = function (stackTrace) {
            return collection_1.isListLikeIterable(stackTrace) ? stackTrace.join('\n\n-----async gap-----\n') : stackTrace.toString();
        };
        /** @internal */
        ExceptionHandler.prototype._findContext = function (exception) {
            try {
                if (!(exception instanceof base_wrapped_exception_1.BaseWrappedException)) return null;
                return lang_1.isPresent(exception.context) ? exception.context : this._findContext(exception.originalException);
            } catch (e) {
                // exception.context can throw an exception. if it happens, we ignore the context.
                return null;
            }
        };
        /** @internal */
        ExceptionHandler.prototype._findOriginalException = function (exception) {
            if (!(exception instanceof base_wrapped_exception_1.BaseWrappedException)) return null;
            var e = exception.originalException;
            while (e instanceof base_wrapped_exception_1.BaseWrappedException && lang_1.isPresent(e.originalException)) {
                e = e.originalException;
            }
            return e;
        };
        /** @internal */
        ExceptionHandler.prototype._findOriginalStack = function (exception) {
            if (!(exception instanceof base_wrapped_exception_1.BaseWrappedException)) return null;
            var e = exception;
            var stack = exception.originalStack;
            while (e instanceof base_wrapped_exception_1.BaseWrappedException && lang_1.isPresent(e.originalException)) {
                e = e.originalException;
                if (e instanceof base_wrapped_exception_1.BaseWrappedException && lang_1.isPresent(e.originalException)) {
                    stack = e.originalStack;
                }
            }
            return stack;
        };
        return ExceptionHandler;
    }();
    exports.ExceptionHandler = ExceptionHandler;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/facade/exceptions.js', ['./base_wrapped_exception', './exception_handler'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var base_wrapped_exception_1 = $__require('./base_wrapped_exception');
    var exception_handler_1 = $__require('./exception_handler');
    var exception_handler_2 = $__require('./exception_handler');
    exports.ExceptionHandler = exception_handler_2.ExceptionHandler;
    /**
     * @stable
     */
    var BaseException = function (_super) {
        __extends(BaseException, _super);
        function BaseException(message) {
            if (message === void 0) {
                message = '--';
            }
            _super.call(this, message);
            this.message = message;
            this.stack = new Error(message).stack;
        }
        BaseException.prototype.toString = function () {
            return this.message;
        };
        return BaseException;
    }(Error);
    exports.BaseException = BaseException;
    /**
     * Wraps an exception and provides additional context or information.
     * @stable
     */
    var WrappedException = function (_super) {
        __extends(WrappedException, _super);
        function WrappedException(_wrapperMessage, _originalException /** TODO #9100 */, _originalStack /** TODO #9100 */, _context /** TODO #9100 */) {
            _super.call(this, _wrapperMessage);
            this._wrapperMessage = _wrapperMessage;
            this._originalException = _originalException;
            this._originalStack = _originalStack;
            this._context = _context;
            this._wrapperStack = new Error(_wrapperMessage).stack;
        }
        Object.defineProperty(WrappedException.prototype, "wrapperMessage", {
            get: function () {
                return this._wrapperMessage;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "wrapperStack", {
            get: function () {
                return this._wrapperStack;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "originalException", {
            get: function () {
                return this._originalException;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "originalStack", {
            get: function () {
                return this._originalStack;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "context", {
            get: function () {
                return this._context;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "message", {
            get: function () {
                return exception_handler_1.ExceptionHandler.exceptionToString(this);
            },
            enumerable: true,
            configurable: true
        });
        WrappedException.prototype.toString = function () {
            return this.message;
        };
        return WrappedException;
    }(base_wrapped_exception_1.BaseWrappedException);
    exports.WrappedException = WrappedException;
    function makeTypeError(message) {
        return new TypeError(message);
    }
    exports.makeTypeError = makeTypeError;
    function unimplemented() {
        throw new BaseException('unimplemented');
    }
    exports.unimplemented = unimplemented;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/dom/events/event_manager.js', ['@angular/core', '../../facade/collection', '../../facade/exceptions'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var collection_1 = $__require('../../facade/collection');
    var exceptions_1 = $__require('../../facade/exceptions');
    /**
     * @stable
     */
    exports.EVENT_MANAGER_PLUGINS = new core_1.OpaqueToken('EventManagerPlugins');
    var EventManager = function () {
        function EventManager(plugins, _zone) {
            var _this = this;
            this._zone = _zone;
            plugins.forEach(function (p) {
                return p.manager = _this;
            });
            this._plugins = collection_1.ListWrapper.reversed(plugins);
        }
        EventManager.prototype.addEventListener = function (element, eventName, handler) {
            var plugin = this._findPluginFor(eventName);
            return plugin.addEventListener(element, eventName, handler);
        };
        EventManager.prototype.addGlobalEventListener = function (target, eventName, handler) {
            var plugin = this._findPluginFor(eventName);
            return plugin.addGlobalEventListener(target, eventName, handler);
        };
        EventManager.prototype.getZone = function () {
            return this._zone;
        };
        /** @internal */
        EventManager.prototype._findPluginFor = function (eventName) {
            var plugins = this._plugins;
            for (var i = 0; i < plugins.length; i++) {
                var plugin = plugins[i];
                if (plugin.supports(eventName)) {
                    return plugin;
                }
            }
            throw new exceptions_1.BaseException("No event manager plugin found for event " + eventName);
        };
        /** @nocollapse */
        EventManager.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        EventManager.ctorParameters = [{ type: Array, decorators: [{ type: core_1.Inject, args: [exports.EVENT_MANAGER_PLUGINS] }] }, { type: core_1.NgZone }];
        return EventManager;
    }();
    exports.EventManager = EventManager;
    var EventManagerPlugin = function () {
        function EventManagerPlugin() {}
        // That is equivalent to having supporting $event.target
        EventManagerPlugin.prototype.supports = function (eventName) {
            return false;
        };
        EventManagerPlugin.prototype.addEventListener = function (element, eventName, handler) {
            throw 'not implemented';
        };
        EventManagerPlugin.prototype.addGlobalEventListener = function (element, eventName, handler) {
            throw 'not implemented';
        };
        return EventManagerPlugin;
    }();
    exports.EventManagerPlugin = EventManagerPlugin;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/dom/events/dom_events.js', ['@angular/core', '../dom_adapter', './event_manager'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var dom_adapter_1 = $__require('../dom_adapter');
    var event_manager_1 = $__require('./event_manager');
    var DomEventsPlugin = function (_super) {
        __extends(DomEventsPlugin, _super);
        function DomEventsPlugin() {
            _super.apply(this, arguments);
        }
        // This plugin should come last in the list of plugins, because it accepts all
        // events.
        DomEventsPlugin.prototype.supports = function (eventName) {
            return true;
        };
        DomEventsPlugin.prototype.addEventListener = function (element, eventName, handler) {
            var zone = this.manager.getZone();
            var outsideHandler = function (event /** TODO #9100 */) {
                return zone.runGuarded(function () {
                    return handler(event);
                });
            };
            return this.manager.getZone().runOutsideAngular(function () {
                return dom_adapter_1.getDOM().onAndCancel(element, eventName, outsideHandler);
            });
        };
        DomEventsPlugin.prototype.addGlobalEventListener = function (target, eventName, handler) {
            var element = dom_adapter_1.getDOM().getGlobalEventTarget(target);
            var zone = this.manager.getZone();
            var outsideHandler = function (event /** TODO #9100 */) {
                return zone.runGuarded(function () {
                    return handler(event);
                });
            };
            return this.manager.getZone().runOutsideAngular(function () {
                return dom_adapter_1.getDOM().onAndCancel(element, eventName, outsideHandler);
            });
        };
        /** @nocollapse */
        DomEventsPlugin.decorators = [{ type: core_1.Injectable }];
        return DomEventsPlugin;
    }(event_manager_1.EventManagerPlugin);
    exports.DomEventsPlugin = DomEventsPlugin;
    
});
System.registerDynamic("node_modules/@angular/platform-browser/src/facade/collection.js", ["./lang"], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('./lang');
    exports.Map = lang_1.global.Map;
    exports.Set = lang_1.global.Set;
    // Safari and Internet Explorer do not support the iterable parameter to the
    // Map constructor.  We work around that by manually adding the items.
    var createMapFromPairs = function () {
        try {
            if (new exports.Map([[1, 2]]).size === 1) {
                return function createMapFromPairs(pairs) {
                    return new exports.Map(pairs);
                };
            }
        } catch (e) {}
        return function createMapAndPopulateFromPairs(pairs) {
            var map = new exports.Map();
            for (var i = 0; i < pairs.length; i++) {
                var pair = pairs[i];
                map.set(pair[0], pair[1]);
            }
            return map;
        };
    }();
    var createMapFromMap = function () {
        try {
            if (new exports.Map(new exports.Map())) {
                return function createMapFromMap(m) {
                    return new exports.Map(m);
                };
            }
        } catch (e) {}
        return function createMapAndPopulateFromMap(m) {
            var map = new exports.Map();
            m.forEach(function (v, k) {
                map.set(k, v);
            });
            return map;
        };
    }();
    var _clearValues = function () {
        if (new exports.Map().keys().next) {
            return function _clearValues(m) {
                var keyIterator = m.keys();
                var k;
                while (!(k = keyIterator.next()).done) {
                    m.set(k.value, null);
                }
            };
        } else {
            return function _clearValuesWithForeEach(m) {
                m.forEach(function (v, k) {
                    m.set(k, null);
                });
            };
        }
    }();
    // Safari doesn't implement MapIterator.next(), which is used is Traceur's polyfill of Array.from
    // TODO(mlaval): remove the work around once we have a working polyfill of Array.from
    var _arrayFromMap = function () {
        try {
            if (new exports.Map().values().next) {
                return function createArrayFromMap(m, getValues) {
                    return getValues ? Array.from(m.values()) : Array.from(m.keys());
                };
            }
        } catch (e) {}
        return function createArrayFromMapWithForeach(m, getValues) {
            var res = ListWrapper.createFixedSize(m.size),
                i = 0;
            m.forEach(function (v, k) {
                res[i] = getValues ? v : k;
                i++;
            });
            return res;
        };
    }();
    var MapWrapper = function () {
        function MapWrapper() {}
        MapWrapper.clone = function (m) {
            return createMapFromMap(m);
        };
        MapWrapper.createFromStringMap = function (stringMap) {
            var result = new exports.Map();
            for (var prop in stringMap) {
                result.set(prop, stringMap[prop]);
            }
            return result;
        };
        MapWrapper.toStringMap = function (m) {
            var r = {};
            m.forEach(function (v, k) {
                return r[k] = v;
            });
            return r;
        };
        MapWrapper.createFromPairs = function (pairs) {
            return createMapFromPairs(pairs);
        };
        MapWrapper.clearValues = function (m) {
            _clearValues(m);
        };
        MapWrapper.iterable = function (m) {
            return m;
        };
        MapWrapper.keys = function (m) {
            return _arrayFromMap(m, false);
        };
        MapWrapper.values = function (m) {
            return _arrayFromMap(m, true);
        };
        return MapWrapper;
    }();
    exports.MapWrapper = MapWrapper;
    /**
     * Wraps Javascript Objects
     */
    var StringMapWrapper = function () {
        function StringMapWrapper() {}
        StringMapWrapper.create = function () {
            // Note: We are not using Object.create(null) here due to
            // performance!
            // http://jsperf.com/ng2-object-create-null
            return {};
        };
        StringMapWrapper.contains = function (map, key) {
            return map.hasOwnProperty(key);
        };
        StringMapWrapper.get = function (map, key) {
            return map.hasOwnProperty(key) ? map[key] : undefined;
        };
        StringMapWrapper.set = function (map, key, value) {
            map[key] = value;
        };
        StringMapWrapper.keys = function (map) {
            return Object.keys(map);
        };
        StringMapWrapper.values = function (map) {
            return Object.keys(map).reduce(function (r, a) {
                r.push(map[a]);
                return r;
            }, []);
        };
        StringMapWrapper.isEmpty = function (map) {
            for (var prop in map) {
                return false;
            }
            return true;
        };
        StringMapWrapper.delete = function (map, key) {
            delete map[key];
        };
        StringMapWrapper.forEach = function (map, callback) {
            for (var prop in map) {
                if (map.hasOwnProperty(prop)) {
                    callback(map[prop], prop);
                }
            }
        };
        StringMapWrapper.merge = function (m1, m2) {
            var m = {};
            for (var attr in m1) {
                if (m1.hasOwnProperty(attr)) {
                    m[attr] = m1[attr];
                }
            }
            for (var attr in m2) {
                if (m2.hasOwnProperty(attr)) {
                    m[attr] = m2[attr];
                }
            }
            return m;
        };
        StringMapWrapper.equals = function (m1, m2) {
            var k1 = Object.keys(m1);
            var k2 = Object.keys(m2);
            if (k1.length != k2.length) {
                return false;
            }
            var key;
            for (var i = 0; i < k1.length; i++) {
                key = k1[i];
                if (m1[key] !== m2[key]) {
                    return false;
                }
            }
            return true;
        };
        return StringMapWrapper;
    }();
    exports.StringMapWrapper = StringMapWrapper;
    var ListWrapper = function () {
        function ListWrapper() {}
        // JS has no way to express a statically fixed size list, but dart does so we
        // keep both methods.
        ListWrapper.createFixedSize = function (size) {
            return new Array(size);
        };
        ListWrapper.createGrowableSize = function (size) {
            return new Array(size);
        };
        ListWrapper.clone = function (array) {
            return array.slice(0);
        };
        ListWrapper.forEachWithIndex = function (array, fn) {
            for (var i = 0; i < array.length; i++) {
                fn(array[i], i);
            }
        };
        ListWrapper.first = function (array) {
            if (!array) return null;
            return array[0];
        };
        ListWrapper.last = function (array) {
            if (!array || array.length == 0) return null;
            return array[array.length - 1];
        };
        ListWrapper.indexOf = function (array, value, startIndex) {
            if (startIndex === void 0) {
                startIndex = 0;
            }
            return array.indexOf(value, startIndex);
        };
        ListWrapper.contains = function (list, el) {
            return list.indexOf(el) !== -1;
        };
        ListWrapper.reversed = function (array) {
            var a = ListWrapper.clone(array);
            return a.reverse();
        };
        ListWrapper.concat = function (a, b) {
            return a.concat(b);
        };
        ListWrapper.insert = function (list, index, value) {
            list.splice(index, 0, value);
        };
        ListWrapper.removeAt = function (list, index) {
            var res = list[index];
            list.splice(index, 1);
            return res;
        };
        ListWrapper.removeAll = function (list, items) {
            for (var i = 0; i < items.length; ++i) {
                var index = list.indexOf(items[i]);
                list.splice(index, 1);
            }
        };
        ListWrapper.remove = function (list, el) {
            var index = list.indexOf(el);
            if (index > -1) {
                list.splice(index, 1);
                return true;
            }
            return false;
        };
        ListWrapper.clear = function (list) {
            list.length = 0;
        };
        ListWrapper.isEmpty = function (list) {
            return list.length == 0;
        };
        ListWrapper.fill = function (list, value, start, end) {
            if (start === void 0) {
                start = 0;
            }
            if (end === void 0) {
                end = null;
            }
            list.fill(value, start, end === null ? list.length : end);
        };
        ListWrapper.equals = function (a, b) {
            if (a.length != b.length) return false;
            for (var i = 0; i < a.length; ++i) {
                if (a[i] !== b[i]) return false;
            }
            return true;
        };
        ListWrapper.slice = function (l, from, to) {
            if (from === void 0) {
                from = 0;
            }
            if (to === void 0) {
                to = null;
            }
            return l.slice(from, to === null ? undefined : to);
        };
        ListWrapper.splice = function (l, from, length) {
            return l.splice(from, length);
        };
        ListWrapper.sort = function (l, compareFn) {
            if (lang_1.isPresent(compareFn)) {
                l.sort(compareFn);
            } else {
                l.sort();
            }
        };
        ListWrapper.toString = function (l) {
            return l.toString();
        };
        ListWrapper.toJSON = function (l) {
            return JSON.stringify(l);
        };
        ListWrapper.maximum = function (list, predicate) {
            if (list.length == 0) {
                return null;
            }
            var solution = null;
            var maxValue = -Infinity;
            for (var index = 0; index < list.length; index++) {
                var candidate = list[index];
                if (lang_1.isBlank(candidate)) {
                    continue;
                }
                var candidateValue = predicate(candidate);
                if (candidateValue > maxValue) {
                    solution = candidate;
                    maxValue = candidateValue;
                }
            }
            return solution;
        };
        ListWrapper.flatten = function (list) {
            var target = [];
            _flattenArray(list, target);
            return target;
        };
        ListWrapper.addAll = function (list, source) {
            for (var i = 0; i < source.length; i++) {
                list.push(source[i]);
            }
        };
        return ListWrapper;
    }();
    exports.ListWrapper = ListWrapper;
    function _flattenArray(source, target) {
        if (lang_1.isPresent(source)) {
            for (var i = 0; i < source.length; i++) {
                var item = source[i];
                if (lang_1.isArray(item)) {
                    _flattenArray(item, target);
                } else {
                    target.push(item);
                }
            }
        }
        return target;
    }
    function isListLikeIterable(obj) {
        if (!lang_1.isJsObject(obj)) return false;
        return lang_1.isArray(obj) || !(obj instanceof exports.Map) && lang_1.getSymbolIterator() in obj; // JS Iterable have a Symbol.iterator prop
    }
    exports.isListLikeIterable = isListLikeIterable;
    function areIterablesEqual(a, b, comparator) {
        var iterator1 = a[lang_1.getSymbolIterator()]();
        var iterator2 = b[lang_1.getSymbolIterator()]();
        while (true) {
            var item1 = iterator1.next();
            var item2 = iterator2.next();
            if (item1.done && item2.done) return true;
            if (item1.done || item2.done) return false;
            if (!comparator(item1.value, item2.value)) return false;
        }
    }
    exports.areIterablesEqual = areIterablesEqual;
    function iterateListLike(obj, fn) {
        if (lang_1.isArray(obj)) {
            for (var i = 0; i < obj.length; i++) {
                fn(obj[i]);
            }
        } else {
            var iterator = obj[lang_1.getSymbolIterator()]();
            var item;
            while (!(item = iterator.next()).done) {
                fn(item.value);
            }
        }
    }
    exports.iterateListLike = iterateListLike;
    // Safari and Internet Explorer do not support the iterable parameter to the
    // Set constructor.  We work around that by manually adding the items.
    var createSetFromList = function () {
        var test = new exports.Set([1, 2, 3]);
        if (test.size === 3) {
            return function createSetFromList(lst) {
                return new exports.Set(lst);
            };
        } else {
            return function createSetAndPopulateFromList(lst) {
                var res = new exports.Set(lst);
                if (res.size !== lst.length) {
                    for (var i = 0; i < lst.length; i++) {
                        res.add(lst[i]);
                    }
                }
                return res;
            };
        }
    }();
    var SetWrapper = function () {
        function SetWrapper() {}
        SetWrapper.createFromList = function (lst) {
            return createSetFromList(lst);
        };
        SetWrapper.has = function (s, key) {
            return s.has(key);
        };
        SetWrapper.delete = function (m, k) {
            m.delete(k);
        };
        return SetWrapper;
    }();
    exports.SetWrapper = SetWrapper;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/facade/lang.js', [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var globalScope;
    if (typeof window === 'undefined') {
        if (typeof WorkerGlobalScope !== 'undefined' && self instanceof WorkerGlobalScope) {
            // TODO: Replace any with WorkerGlobalScope from lib.webworker.d.ts #3492
            globalScope = self;
        } else {
            globalScope = global;
        }
    } else {
        globalScope = window;
    }
    function scheduleMicroTask(fn) {
        Zone.current.scheduleMicroTask('scheduleMicrotask', fn);
    }
    exports.scheduleMicroTask = scheduleMicroTask;
    exports.IS_DART = false;
    // Need to declare a new variable for global here since TypeScript
    // exports the original value of the symbol.
    var _global = globalScope;
    exports.global = _global;
    /**
     * Runtime representation a type that a Component or other object is instances of.
     *
     * An example of a `Type` is `MyCustomComponent` class, which in JavaScript is be represented by
     * the `MyCustomComponent` constructor function.
     *
     * @stable
     */
    exports.Type = Function;
    function getTypeNameForDebugging(type) {
        if (type['name']) {
            return type['name'];
        }
        return typeof type;
    }
    exports.getTypeNameForDebugging = getTypeNameForDebugging;
    exports.Math = _global.Math;
    exports.Date = _global.Date;
    // TODO: remove calls to assert in production environment
    // Note: Can't just export this and import in in other files
    // as `assert` is a reserved keyword in Dart
    _global.assert = function assert(condition) {
        // TODO: to be fixed properly via #2830, noop for now
    };
    function isPresent(obj) {
        return obj !== undefined && obj !== null;
    }
    exports.isPresent = isPresent;
    function isBlank(obj) {
        return obj === undefined || obj === null;
    }
    exports.isBlank = isBlank;
    function isBoolean(obj) {
        return typeof obj === 'boolean';
    }
    exports.isBoolean = isBoolean;
    function isNumber(obj) {
        return typeof obj === 'number';
    }
    exports.isNumber = isNumber;
    function isString(obj) {
        return typeof obj === 'string';
    }
    exports.isString = isString;
    function isFunction(obj) {
        return typeof obj === 'function';
    }
    exports.isFunction = isFunction;
    function isType(obj) {
        return isFunction(obj);
    }
    exports.isType = isType;
    function isStringMap(obj) {
        return typeof obj === 'object' && obj !== null;
    }
    exports.isStringMap = isStringMap;
    var STRING_MAP_PROTO = Object.getPrototypeOf({});
    function isStrictStringMap(obj) {
        return isStringMap(obj) && Object.getPrototypeOf(obj) === STRING_MAP_PROTO;
    }
    exports.isStrictStringMap = isStrictStringMap;
    function isPromise(obj) {
        return obj instanceof _global.Promise;
    }
    exports.isPromise = isPromise;
    function isArray(obj) {
        return Array.isArray(obj);
    }
    exports.isArray = isArray;
    function isDate(obj) {
        return obj instanceof exports.Date && !isNaN(obj.valueOf());
    }
    exports.isDate = isDate;
    function noop() {}
    exports.noop = noop;
    function stringify(token) {
        if (typeof token === 'string') {
            return token;
        }
        if (token === undefined || token === null) {
            return '' + token;
        }
        if (token.name) {
            return token.name;
        }
        if (token.overriddenName) {
            return token.overriddenName;
        }
        var res = token.toString();
        var newLineIndex = res.indexOf('\n');
        return newLineIndex === -1 ? res : res.substring(0, newLineIndex);
    }
    exports.stringify = stringify;
    // serialize / deserialize enum exist only for consistency with dart API
    // enums in typescript don't need to be serialized
    function serializeEnum(val) {
        return val;
    }
    exports.serializeEnum = serializeEnum;
    function deserializeEnum(val, values) {
        return val;
    }
    exports.deserializeEnum = deserializeEnum;
    function resolveEnumToken(enumValue, val) {
        return enumValue[val];
    }
    exports.resolveEnumToken = resolveEnumToken;
    var StringWrapper = function () {
        function StringWrapper() {}
        StringWrapper.fromCharCode = function (code) {
            return String.fromCharCode(code);
        };
        StringWrapper.charCodeAt = function (s, index) {
            return s.charCodeAt(index);
        };
        StringWrapper.split = function (s, regExp) {
            return s.split(regExp);
        };
        StringWrapper.equals = function (s, s2) {
            return s === s2;
        };
        StringWrapper.stripLeft = function (s, charVal) {
            if (s && s.length) {
                var pos = 0;
                for (var i = 0; i < s.length; i++) {
                    if (s[i] != charVal) break;
                    pos++;
                }
                s = s.substring(pos);
            }
            return s;
        };
        StringWrapper.stripRight = function (s, charVal) {
            if (s && s.length) {
                var pos = s.length;
                for (var i = s.length - 1; i >= 0; i--) {
                    if (s[i] != charVal) break;
                    pos--;
                }
                s = s.substring(0, pos);
            }
            return s;
        };
        StringWrapper.replace = function (s, from, replace) {
            return s.replace(from, replace);
        };
        StringWrapper.replaceAll = function (s, from, replace) {
            return s.replace(from, replace);
        };
        StringWrapper.slice = function (s, from, to) {
            if (from === void 0) {
                from = 0;
            }
            if (to === void 0) {
                to = null;
            }
            return s.slice(from, to === null ? undefined : to);
        };
        StringWrapper.replaceAllMapped = function (s, from, cb) {
            return s.replace(from, function () {
                var matches = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    matches[_i - 0] = arguments[_i];
                }
                // Remove offset & string from the result array
                matches.splice(-2, 2);
                // The callback receives match, p1, ..., pn
                return cb(matches);
            });
        };
        StringWrapper.contains = function (s, substr) {
            return s.indexOf(substr) != -1;
        };
        StringWrapper.compare = function (a, b) {
            if (a < b) {
                return -1;
            } else if (a > b) {
                return 1;
            } else {
                return 0;
            }
        };
        return StringWrapper;
    }();
    exports.StringWrapper = StringWrapper;
    var StringJoiner = function () {
        function StringJoiner(parts) {
            if (parts === void 0) {
                parts = [];
            }
            this.parts = parts;
        }
        StringJoiner.prototype.add = function (part) {
            this.parts.push(part);
        };
        StringJoiner.prototype.toString = function () {
            return this.parts.join('');
        };
        return StringJoiner;
    }();
    exports.StringJoiner = StringJoiner;
    var NumberParseError = function (_super) {
        __extends(NumberParseError, _super);
        function NumberParseError(message) {
            _super.call(this);
            this.message = message;
        }
        NumberParseError.prototype.toString = function () {
            return this.message;
        };
        return NumberParseError;
    }(Error);
    exports.NumberParseError = NumberParseError;
    var NumberWrapper = function () {
        function NumberWrapper() {}
        NumberWrapper.toFixed = function (n, fractionDigits) {
            return n.toFixed(fractionDigits);
        };
        NumberWrapper.equal = function (a, b) {
            return a === b;
        };
        NumberWrapper.parseIntAutoRadix = function (text) {
            var result = parseInt(text);
            if (isNaN(result)) {
                throw new NumberParseError('Invalid integer literal when parsing ' + text);
            }
            return result;
        };
        NumberWrapper.parseInt = function (text, radix) {
            if (radix == 10) {
                if (/^(\-|\+)?[0-9]+$/.test(text)) {
                    return parseInt(text, radix);
                }
            } else if (radix == 16) {
                if (/^(\-|\+)?[0-9ABCDEFabcdef]+$/.test(text)) {
                    return parseInt(text, radix);
                }
            } else {
                var result = parseInt(text, radix);
                if (!isNaN(result)) {
                    return result;
                }
            }
            throw new NumberParseError('Invalid integer literal when parsing ' + text + ' in base ' + radix);
        };
        // TODO: NaN is a valid literal but is returned by parseFloat to indicate an error.
        NumberWrapper.parseFloat = function (text) {
            return parseFloat(text);
        };
        Object.defineProperty(NumberWrapper, "NaN", {
            get: function () {
                return NaN;
            },
            enumerable: true,
            configurable: true
        });
        NumberWrapper.isNumeric = function (value) {
            return !isNaN(value - parseFloat(value));
        };
        NumberWrapper.isNaN = function (value) {
            return isNaN(value);
        };
        NumberWrapper.isInteger = function (value) {
            return Number.isInteger(value);
        };
        return NumberWrapper;
    }();
    exports.NumberWrapper = NumberWrapper;
    exports.RegExp = _global.RegExp;
    var RegExpWrapper = function () {
        function RegExpWrapper() {}
        RegExpWrapper.create = function (regExpStr, flags) {
            if (flags === void 0) {
                flags = '';
            }
            flags = flags.replace(/g/g, '');
            return new _global.RegExp(regExpStr, flags + 'g');
        };
        RegExpWrapper.firstMatch = function (regExp, input) {
            // Reset multimatch regex state
            regExp.lastIndex = 0;
            return regExp.exec(input);
        };
        RegExpWrapper.test = function (regExp, input) {
            regExp.lastIndex = 0;
            return regExp.test(input);
        };
        RegExpWrapper.matcher = function (regExp, input) {
            // Reset regex state for the case
            // someone did not loop over all matches
            // last time.
            regExp.lastIndex = 0;
            return { re: regExp, input: input };
        };
        RegExpWrapper.replaceAll = function (regExp, input, replace) {
            var c = regExp.exec(input);
            var res = '';
            regExp.lastIndex = 0;
            var prev = 0;
            while (c) {
                res += input.substring(prev, c.index);
                res += replace(c);
                prev = c.index + c[0].length;
                regExp.lastIndex = prev;
                c = regExp.exec(input);
            }
            res += input.substring(prev);
            return res;
        };
        return RegExpWrapper;
    }();
    exports.RegExpWrapper = RegExpWrapper;
    var RegExpMatcherWrapper = function () {
        function RegExpMatcherWrapper() {}
        RegExpMatcherWrapper.next = function (matcher) {
            return matcher.re.exec(matcher.input);
        };
        return RegExpMatcherWrapper;
    }();
    exports.RegExpMatcherWrapper = RegExpMatcherWrapper;
    var FunctionWrapper = function () {
        function FunctionWrapper() {}
        FunctionWrapper.apply = function (fn, posArgs) {
            return fn.apply(null, posArgs);
        };
        FunctionWrapper.bind = function (fn, scope) {
            return fn.bind(scope);
        };
        return FunctionWrapper;
    }();
    exports.FunctionWrapper = FunctionWrapper;
    // JS has NaN !== NaN
    function looseIdentical(a, b) {
        return a === b || typeof a === 'number' && typeof b === 'number' && isNaN(a) && isNaN(b);
    }
    exports.looseIdentical = looseIdentical;
    // JS considers NaN is the same as NaN for map Key (while NaN !== NaN otherwise)
    // see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Map
    function getMapKey(value) {
        return value;
    }
    exports.getMapKey = getMapKey;
    function normalizeBlank(obj) {
        return isBlank(obj) ? null : obj;
    }
    exports.normalizeBlank = normalizeBlank;
    function normalizeBool(obj) {
        return isBlank(obj) ? false : obj;
    }
    exports.normalizeBool = normalizeBool;
    function isJsObject(o) {
        return o !== null && (typeof o === 'function' || typeof o === 'object');
    }
    exports.isJsObject = isJsObject;
    function print(obj) {
        console.log(obj);
    }
    exports.print = print;
    function warn(obj) {
        console.warn(obj);
    }
    exports.warn = warn;
    // Can't be all uppercase as our transpiler would think it is a special directive...
    var Json = function () {
        function Json() {}
        Json.parse = function (s) {
            return _global.JSON.parse(s);
        };
        Json.stringify = function (data) {
            // Dart doesn't take 3 arguments
            return _global.JSON.stringify(data, null, 2);
        };
        return Json;
    }();
    exports.Json = Json;
    var DateWrapper = function () {
        function DateWrapper() {}
        DateWrapper.create = function (year, month, day, hour, minutes, seconds, milliseconds) {
            if (month === void 0) {
                month = 1;
            }
            if (day === void 0) {
                day = 1;
            }
            if (hour === void 0) {
                hour = 0;
            }
            if (minutes === void 0) {
                minutes = 0;
            }
            if (seconds === void 0) {
                seconds = 0;
            }
            if (milliseconds === void 0) {
                milliseconds = 0;
            }
            return new exports.Date(year, month - 1, day, hour, minutes, seconds, milliseconds);
        };
        DateWrapper.fromISOString = function (str) {
            return new exports.Date(str);
        };
        DateWrapper.fromMillis = function (ms) {
            return new exports.Date(ms);
        };
        DateWrapper.toMillis = function (date) {
            return date.getTime();
        };
        DateWrapper.now = function () {
            return new exports.Date();
        };
        DateWrapper.toJson = function (date) {
            return date.toJSON();
        };
        return DateWrapper;
    }();
    exports.DateWrapper = DateWrapper;
    function setValueOnPath(global, path, value) {
        var parts = path.split('.');
        var obj = global;
        while (parts.length > 1) {
            var name = parts.shift();
            if (obj.hasOwnProperty(name) && isPresent(obj[name])) {
                obj = obj[name];
            } else {
                obj = obj[name] = {};
            }
        }
        if (obj === undefined || obj === null) {
            obj = {};
        }
        obj[parts.shift()] = value;
    }
    exports.setValueOnPath = setValueOnPath;
    var _symbolIterator = null;
    function getSymbolIterator() {
        if (isBlank(_symbolIterator)) {
            if (isPresent(globalScope.Symbol) && isPresent(Symbol.iterator)) {
                _symbolIterator = Symbol.iterator;
            } else {
                // es6-shim specific logic
                var keys = Object.getOwnPropertyNames(Map.prototype);
                for (var i = 0; i < keys.length; ++i) {
                    var key = keys[i];
                    if (key !== 'entries' && key !== 'size' && Map.prototype[key] === Map.prototype['entries']) {
                        _symbolIterator = key;
                    }
                }
            }
        }
        return _symbolIterator;
    }
    exports.getSymbolIterator = getSymbolIterator;
    function evalExpression(sourceUrl, expr, declarations, vars) {
        var fnBody = declarations + "\nreturn " + expr + "\n//# sourceURL=" + sourceUrl;
        var fnArgNames = [];
        var fnArgValues = [];
        for (var argName in vars) {
            fnArgNames.push(argName);
            fnArgValues.push(vars[argName]);
        }
        return new (Function.bind.apply(Function, [void 0].concat(fnArgNames.concat(fnBody))))().apply(void 0, fnArgValues);
    }
    exports.evalExpression = evalExpression;
    function isPrimitive(obj) {
        return !isJsObject(obj);
    }
    exports.isPrimitive = isPrimitive;
    function hasConstructor(value, type) {
        return value.constructor === type;
    }
    exports.hasConstructor = hasConstructor;
    function escape(s) {
        return _global.encodeURI(s);
    }
    exports.escape = escape;
    function escapeRegExp(s) {
        return s.replace(/([.*+?^=!:${}()|[\]\/\\])/g, '\\$1');
    }
    exports.escapeRegExp = escapeRegExp;
    
});
System.registerDynamic("node_modules/@angular/platform-browser/src/dom/dom_adapter.js", ["../facade/lang"], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('../facade/lang');
    var _DOM = null;
    function getDOM() {
        return _DOM;
    }
    exports.getDOM = getDOM;
    function setDOM(adapter) {
        _DOM = adapter;
    }
    exports.setDOM = setDOM;
    function setRootDomAdapter(adapter) {
        if (lang_1.isBlank(_DOM)) {
            _DOM = adapter;
        }
    }
    exports.setRootDomAdapter = setRootDomAdapter;
    /* tslint:disable:requireParameterType */
    /**
     * Provides DOM operations in an environment-agnostic way.
     */
    var DomAdapter = function () {
        function DomAdapter() {
            this.xhrType = null;
        }
        /** @deprecated */
        DomAdapter.prototype.getXHR = function () {
            return this.xhrType;
        };
        Object.defineProperty(DomAdapter.prototype, "attrToPropMap", {
            /**
             * Maps attribute names to their corresponding property names for cases
             * where attribute name doesn't match property name.
             */
            get: function () {
                return this._attrToPropMap;
            },
            set: function (value) {
                this._attrToPropMap = value;
            },
            enumerable: true,
            configurable: true
        });
        ;
        ;
        return DomAdapter;
    }();
    exports.DomAdapter = DomAdapter;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/src/dom/dom_tokens.js', ['@angular/core'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var core_1 = $__require('@angular/core');
  /**
   * A DI Token representing the main rendering context. In a browser this is the DOM Document.
   *
   * Note: Document might not be available in the Application Context when Application and Rendering
   * Contexts are not the same (e.g. when running the application into a Web Worker).
   *
   * @stable
   */
  exports.DOCUMENT = new core_1.OpaqueToken('DocumentToken');
  
});
System.registerDynamic('node_modules/@angular/platform-browser/src/dom/shared_styles_host.js', ['@angular/core', '../facade/collection', './dom_adapter', './dom_tokens'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var collection_1 = $__require('../facade/collection');
    var dom_adapter_1 = $__require('./dom_adapter');
    var dom_tokens_1 = $__require('./dom_tokens');
    var SharedStylesHost = function () {
        function SharedStylesHost() {
            /** @internal */
            this._styles = [];
            /** @internal */
            this._stylesSet = new Set();
        }
        SharedStylesHost.prototype.addStyles = function (styles) {
            var _this = this;
            var additions = [];
            styles.forEach(function (style) {
                if (!collection_1.SetWrapper.has(_this._stylesSet, style)) {
                    _this._stylesSet.add(style);
                    _this._styles.push(style);
                    additions.push(style);
                }
            });
            this.onStylesAdded(additions);
        };
        SharedStylesHost.prototype.onStylesAdded = function (additions) {};
        SharedStylesHost.prototype.getAllStyles = function () {
            return this._styles;
        };
        /** @nocollapse */
        SharedStylesHost.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        SharedStylesHost.ctorParameters = [];
        return SharedStylesHost;
    }();
    exports.SharedStylesHost = SharedStylesHost;
    var DomSharedStylesHost = function (_super) {
        __extends(DomSharedStylesHost, _super);
        function DomSharedStylesHost(doc) {
            _super.call(this);
            this._hostNodes = new Set();
            this._hostNodes.add(doc.head);
        }
        /** @internal */
        DomSharedStylesHost.prototype._addStylesToHost = function (styles, host) {
            for (var i = 0; i < styles.length; i++) {
                var style = styles[i];
                dom_adapter_1.getDOM().appendChild(host, dom_adapter_1.getDOM().createStyleElement(style));
            }
        };
        DomSharedStylesHost.prototype.addHost = function (hostNode) {
            this._addStylesToHost(this._styles, hostNode);
            this._hostNodes.add(hostNode);
        };
        DomSharedStylesHost.prototype.removeHost = function (hostNode) {
            collection_1.SetWrapper.delete(this._hostNodes, hostNode);
        };
        DomSharedStylesHost.prototype.onStylesAdded = function (additions) {
            var _this = this;
            this._hostNodes.forEach(function (hostNode) {
                _this._addStylesToHost(additions, hostNode);
            });
        };
        /** @nocollapse */
        DomSharedStylesHost.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        DomSharedStylesHost.ctorParameters = [{ type: undefined, decorators: [{ type: core_1.Inject, args: [dom_tokens_1.DOCUMENT] }] }];
        return DomSharedStylesHost;
    }(SharedStylesHost);
    exports.DomSharedStylesHost = DomSharedStylesHost;
    
});
System.registerDynamic('node_modules/@angular/platform-browser/private_export.js', ['./src/dom/debug/ng_probe', './src/dom/dom_adapter', './src/dom/dom_renderer', './src/dom/events/dom_events', './src/dom/shared_styles_host'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var ng_proble = $__require('./src/dom/debug/ng_probe');
    var dom_adapter = $__require('./src/dom/dom_adapter');
    var dom_renderer = $__require('./src/dom/dom_renderer');
    var dom_events = $__require('./src/dom/events/dom_events');
    var shared_styles_host = $__require('./src/dom/shared_styles_host');
    exports.__platform_browser_private__ = {
        DomAdapter: dom_adapter.DomAdapter,
        getDOM: dom_adapter.getDOM,
        setRootDomAdapter: dom_adapter.setRootDomAdapter,
        DomRootRenderer: dom_renderer.DomRootRenderer,
        DomRootRenderer_: dom_renderer.DomRootRenderer_,
        DomSharedStylesHost: shared_styles_host.DomSharedStylesHost,
        SharedStylesHost: shared_styles_host.SharedStylesHost,
        ELEMENT_PROBE_PROVIDERS: ng_proble.ELEMENT_PROBE_PROVIDERS,
        DomEventsPlugin: dom_events.DomEventsPlugin
    };
    
});
System.registerDynamic('node_modules/@angular/platform-browser/index.js', ['./src/browser/location/browser_platform_location', './src/browser/title', './src/browser/tools/tools', './src/dom/debug/by', './src/dom/dom_tokens', './src/dom/events/event_manager', './src/dom/events/hammer_gestures', './src/security/dom_sanitization_service', './src/browser', './src/web_workers/shared/client_message_broker', './src/web_workers/shared/service_message_broker', './src/web_workers/shared/serializer', './src/web_workers/shared/message_bus', './src/web_workers/worker/location_providers', './src/web_workers/ui/location_providers', './src/worker_render', './src/worker_app', './private_export'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
  }
  var browser_platform_location_1 = $__require('./src/browser/location/browser_platform_location');
  exports.BrowserPlatformLocation = browser_platform_location_1.BrowserPlatformLocation;
  var title_1 = $__require('./src/browser/title');
  exports.Title = title_1.Title;
  var tools_1 = $__require('./src/browser/tools/tools');
  exports.disableDebugTools = tools_1.disableDebugTools;
  exports.enableDebugTools = tools_1.enableDebugTools;
  var by_1 = $__require('./src/dom/debug/by');
  exports.By = by_1.By;
  var dom_tokens_1 = $__require('./src/dom/dom_tokens');
  exports.DOCUMENT = dom_tokens_1.DOCUMENT;
  var event_manager_1 = $__require('./src/dom/events/event_manager');
  exports.EVENT_MANAGER_PLUGINS = event_manager_1.EVENT_MANAGER_PLUGINS;
  exports.EventManager = event_manager_1.EventManager;
  var hammer_gestures_1 = $__require('./src/dom/events/hammer_gestures');
  exports.HAMMER_GESTURE_CONFIG = hammer_gestures_1.HAMMER_GESTURE_CONFIG;
  exports.HammerGestureConfig = hammer_gestures_1.HammerGestureConfig;
  var dom_sanitization_service_1 = $__require('./src/security/dom_sanitization_service');
  exports.DomSanitizationService = dom_sanitization_service_1.DomSanitizationService;
  exports.SecurityContext = dom_sanitization_service_1.SecurityContext;
  __export($__require('./src/browser'));
  // Web Workers
  var client_message_broker_1 = $__require('./src/web_workers/shared/client_message_broker');
  exports.ClientMessageBroker = client_message_broker_1.ClientMessageBroker;
  exports.ClientMessageBrokerFactory = client_message_broker_1.ClientMessageBrokerFactory;
  exports.FnArg = client_message_broker_1.FnArg;
  exports.UiArguments = client_message_broker_1.UiArguments;
  var service_message_broker_1 = $__require('./src/web_workers/shared/service_message_broker');
  exports.ReceivedMessage = service_message_broker_1.ReceivedMessage;
  exports.ServiceMessageBroker = service_message_broker_1.ServiceMessageBroker;
  exports.ServiceMessageBrokerFactory = service_message_broker_1.ServiceMessageBrokerFactory;
  var serializer_1 = $__require('./src/web_workers/shared/serializer');
  exports.PRIMITIVE = serializer_1.PRIMITIVE;
  __export($__require('./src/web_workers/shared/message_bus'));
  var location_providers_1 = $__require('./src/web_workers/worker/location_providers');
  exports.WORKER_APP_LOCATION_PROVIDERS = location_providers_1.WORKER_APP_LOCATION_PROVIDERS;
  var location_providers_2 = $__require('./src/web_workers/ui/location_providers');
  exports.WORKER_UI_LOCATION_PROVIDERS = location_providers_2.WORKER_UI_LOCATION_PROVIDERS;
  __export($__require('./src/worker_render'));
  __export($__require('./src/worker_app'));
  __export($__require('./private_export'));
  
});
System.registerDynamic("node_modules/@angular/http/src/backends/browser_xhr.js", ["@angular/core"], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var BrowserXhr = function () {
        function BrowserXhr() {}
        BrowserXhr.prototype.build = function () {
            return new XMLHttpRequest();
        };
        /** @nocollapse */
        BrowserXhr.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        BrowserXhr.ctorParameters = [];
        return BrowserXhr;
    }();
    exports.BrowserXhr = BrowserXhr;
    
});
System.registerDynamic('node_modules/@angular/http/src/backends/xhr_backend.js', ['@angular/core', '@angular/platform-browser', 'rxjs/Observable', '../base_response_options', '../enums', '../facade/lang', '../headers', '../http_utils', '../interfaces', '../static_response', './browser_xhr'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var core_1 = $__require('@angular/core');
    var platform_browser_1 = $__require('@angular/platform-browser');
    var Observable_1 = $__require('rxjs/Observable');
    var base_response_options_1 = $__require('../base_response_options');
    var enums_1 = $__require('../enums');
    var lang_1 = $__require('../facade/lang');
    var headers_1 = $__require('../headers');
    var http_utils_1 = $__require('../http_utils');
    var interfaces_1 = $__require('../interfaces');
    var static_response_1 = $__require('../static_response');
    var browser_xhr_1 = $__require('./browser_xhr');
    var XSSI_PREFIX = /^\)\]\}',?\n/;
    /**
     * Creates connections using `XMLHttpRequest`. Given a fully-qualified
     * request, an `XHRConnection` will immediately create an `XMLHttpRequest` object and send the
     * request.
     *
     * This class would typically not be created or interacted with directly inside applications, though
     * the {@link MockConnection} may be interacted with in tests.
     *
     * @experimental
     */
    var XHRConnection = function () {
        function XHRConnection(req, browserXHR, baseResponseOptions) {
            var _this = this;
            this.request = req;
            this.response = new Observable_1.Observable(function (responseObserver) {
                var _xhr = browserXHR.build();
                _xhr.open(enums_1.RequestMethod[req.method].toUpperCase(), req.url);
                if (lang_1.isPresent(req.withCredentials)) {
                    _xhr.withCredentials = req.withCredentials;
                }
                // load event handler
                var onLoad = function () {
                    // responseText is the old-school way of retrieving response (supported by IE8 & 9)
                    // response/responseType properties were introduced in XHR Level2 spec (supported by
                    // IE10)
                    var body = lang_1.isPresent(_xhr.response) ? _xhr.response : _xhr.responseText;
                    // Implicitly strip a potential XSSI prefix.
                    if (lang_1.isString(body)) body = body.replace(XSSI_PREFIX, '');
                    var headers = headers_1.Headers.fromResponseHeaderString(_xhr.getAllResponseHeaders());
                    var url = http_utils_1.getResponseURL(_xhr);
                    // normalize IE9 bug (http://bugs.jquery.com/ticket/1450)
                    var status = _xhr.status === 1223 ? 204 : _xhr.status;
                    // fix status code when it is 0 (0 status is undocumented).
                    // Occurs when accessing file resources or on Android 4.1 stock browser
                    // while retrieving files from application cache.
                    if (status === 0) {
                        status = body ? 200 : 0;
                    }
                    var statusText = _xhr.statusText || 'OK';
                    var responseOptions = new base_response_options_1.ResponseOptions({ body: body, status: status, headers: headers, statusText: statusText, url: url });
                    if (lang_1.isPresent(baseResponseOptions)) {
                        responseOptions = baseResponseOptions.merge(responseOptions);
                    }
                    var response = new static_response_1.Response(responseOptions);
                    response.ok = http_utils_1.isSuccess(status);
                    if (response.ok) {
                        responseObserver.next(response);
                        // TODO(gdi2290): defer complete if array buffer until done
                        responseObserver.complete();
                        return;
                    }
                    responseObserver.error(response);
                };
                // error event handler
                var onError = function (err) {
                    var responseOptions = new base_response_options_1.ResponseOptions({
                        body: err,
                        type: enums_1.ResponseType.Error,
                        status: _xhr.status,
                        statusText: _xhr.statusText
                    });
                    if (lang_1.isPresent(baseResponseOptions)) {
                        responseOptions = baseResponseOptions.merge(responseOptions);
                    }
                    responseObserver.error(new static_response_1.Response(responseOptions));
                };
                _this.setDetectedContentType(req, _xhr);
                if (lang_1.isPresent(req.headers)) {
                    req.headers.forEach(function (values, name) {
                        return _xhr.setRequestHeader(name, values.join(','));
                    });
                }
                _xhr.addEventListener('load', onLoad);
                _xhr.addEventListener('error', onError);
                _xhr.send(_this.request.getBody());
                return function () {
                    _xhr.removeEventListener('load', onLoad);
                    _xhr.removeEventListener('error', onError);
                    _xhr.abort();
                };
            });
        }
        XHRConnection.prototype.setDetectedContentType = function (req /** TODO #9100 */, _xhr /** TODO #9100 */) {
            // Skip if a custom Content-Type header is provided
            if (lang_1.isPresent(req.headers) && lang_1.isPresent(req.headers.get('Content-Type'))) {
                return;
            }
            // Set the detected content type
            switch (req.contentType) {
                case enums_1.ContentType.NONE:
                    break;
                case enums_1.ContentType.JSON:
                    _xhr.setRequestHeader('Content-Type', 'application/json');
                    break;
                case enums_1.ContentType.FORM:
                    _xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded;charset=UTF-8');
                    break;
                case enums_1.ContentType.TEXT:
                    _xhr.setRequestHeader('Content-Type', 'text/plain');
                    break;
                case enums_1.ContentType.BLOB:
                    var blob = req.blob();
                    if (blob.type) {
                        _xhr.setRequestHeader('Content-Type', blob.type);
                    }
                    break;
            }
        };
        return XHRConnection;
    }();
    exports.XHRConnection = XHRConnection;
    /**
     * `XSRFConfiguration` sets up Cross Site Request Forgery (XSRF) protection for the application
     * using a cookie. See https://www.owasp.org/index.php/Cross-Site_Request_Forgery_(CSRF) for more
     * information on XSRF.
     *
     * Applications can configure custom cookie and header names by binding an instance of this class
     * with different `cookieName` and `headerName` values. See the main HTTP documentation for more
     * details.
     *
     * @experimental
     */
    var CookieXSRFStrategy = function () {
        function CookieXSRFStrategy(_cookieName, _headerName) {
            if (_cookieName === void 0) {
                _cookieName = 'XSRF-TOKEN';
            }
            if (_headerName === void 0) {
                _headerName = 'X-XSRF-TOKEN';
            }
            this._cookieName = _cookieName;
            this._headerName = _headerName;
        }
        CookieXSRFStrategy.prototype.configureRequest = function (req) {
            var xsrfToken = platform_browser_1.__platform_browser_private__.getDOM().getCookie(this._cookieName);
            if (xsrfToken && !req.headers.has(this._headerName)) {
                req.headers.set(this._headerName, xsrfToken);
            }
        };
        return CookieXSRFStrategy;
    }();
    exports.CookieXSRFStrategy = CookieXSRFStrategy;
    var XHRBackend = function () {
        function XHRBackend(_browserXHR, _baseResponseOptions, _xsrfStrategy) {
            this._browserXHR = _browserXHR;
            this._baseResponseOptions = _baseResponseOptions;
            this._xsrfStrategy = _xsrfStrategy;
        }
        XHRBackend.prototype.createConnection = function (request) {
            this._xsrfStrategy.configureRequest(request);
            return new XHRConnection(request, this._browserXHR, this._baseResponseOptions);
        };
        /** @nocollapse */
        XHRBackend.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        XHRBackend.ctorParameters = [{ type: browser_xhr_1.BrowserXhr }, { type: base_response_options_1.ResponseOptions }, { type: interfaces_1.XSRFStrategy }];
        return XHRBackend;
    }();
    exports.XHRBackend = XHRBackend;
    
});
System.registerDynamic('node_modules/@angular/http/src/base_response_options.js', ['@angular/core', '../src/facade/lang', './enums', './headers'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../src/facade/lang');
    var enums_1 = $__require('./enums');
    var headers_1 = $__require('./headers');
    /**
     * Creates a response options object to be optionally provided when instantiating a
     * {@link Response}.
     *
     * This class is based on the `ResponseInit` description in the [Fetch
     * Spec](https://fetch.spec.whatwg.org/#responseinit).
     *
     * All values are null by default. Typical defaults can be found in the
     * {@link BaseResponseOptions} class, which sub-classes `ResponseOptions`.
     *
     * This class may be used in tests to build {@link Response Responses} for
     * mock responses (see {@link MockBackend}).
     *
     * ### Example ([live demo](http://plnkr.co/edit/P9Jkk8e8cz6NVzbcxEsD?p=preview))
     *
     * ```typescript
     * import {ResponseOptions, Response} from '@angular/http';
     *
     * var options = new ResponseOptions({
     *   body: '{"name":"Jeff"}'
     * });
     * var res = new Response(options);
     *
     * console.log('res.json():', res.json()); // Object {name: "Jeff"}
     * ```
     *
     * @experimental
     */
    var ResponseOptions = function () {
        function ResponseOptions(_a) {
            var _b = _a === void 0 ? {} : _a,
                body = _b.body,
                status = _b.status,
                headers = _b.headers,
                statusText = _b.statusText,
                type = _b.type,
                url = _b.url;
            this.body = lang_1.isPresent(body) ? body : null;
            this.status = lang_1.isPresent(status) ? status : null;
            this.headers = lang_1.isPresent(headers) ? headers : null;
            this.statusText = lang_1.isPresent(statusText) ? statusText : null;
            this.type = lang_1.isPresent(type) ? type : null;
            this.url = lang_1.isPresent(url) ? url : null;
        }
        /**
         * Creates a copy of the `ResponseOptions` instance, using the optional input as values to
         * override
         * existing values. This method will not change the values of the instance on which it is being
         * called.
         *
         * This may be useful when sharing a base `ResponseOptions` object inside tests,
         * where certain properties may change from test to test.
         *
         * ### Example ([live demo](http://plnkr.co/edit/1lXquqFfgduTFBWjNoRE?p=preview))
         *
         * ```typescript
         * import {ResponseOptions, Response} from '@angular/http';
         *
         * var options = new ResponseOptions({
         *   body: {name: 'Jeff'}
         * });
         * var res = new Response(options.merge({
         *   url: 'https://google.com'
         * }));
         * console.log('options.url:', options.url); // null
         * console.log('res.json():', res.json()); // Object {name: "Jeff"}
         * console.log('res.url:', res.url); // https://google.com
         * ```
         */
        ResponseOptions.prototype.merge = function (options) {
            return new ResponseOptions({
                body: lang_1.isPresent(options) && lang_1.isPresent(options.body) ? options.body : this.body,
                status: lang_1.isPresent(options) && lang_1.isPresent(options.status) ? options.status : this.status,
                headers: lang_1.isPresent(options) && lang_1.isPresent(options.headers) ? options.headers : this.headers,
                statusText: lang_1.isPresent(options) && lang_1.isPresent(options.statusText) ? options.statusText : this.statusText,
                type: lang_1.isPresent(options) && lang_1.isPresent(options.type) ? options.type : this.type,
                url: lang_1.isPresent(options) && lang_1.isPresent(options.url) ? options.url : this.url
            });
        };
        return ResponseOptions;
    }();
    exports.ResponseOptions = ResponseOptions;
    var BaseResponseOptions = function (_super) {
        __extends(BaseResponseOptions, _super);
        function BaseResponseOptions() {
            _super.call(this, { status: 200, statusText: 'Ok', type: enums_1.ResponseType.Default, headers: new headers_1.Headers() });
        }
        /** @nocollapse */
        BaseResponseOptions.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        BaseResponseOptions.ctorParameters = [];
        return BaseResponseOptions;
    }(ResponseOptions);
    exports.BaseResponseOptions = BaseResponseOptions;
    
});
System.registerDynamic('node_modules/@angular/core/src/metadata/di.js', ['../di/forward_ref', '../di/metadata', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var forward_ref_1 = $__require('../di/forward_ref');
    var metadata_1 = $__require('../di/metadata');
    var lang_1 = $__require('../facade/lang');
    /**
     * Specifies that a constant attribute value should be injected.
     *
     * The directive can inject constant string literals of host element attributes.
     *
     * ### Example
     *
     * Suppose we have an `<input>` element and want to know its `type`.
     *
     * ```html
     * <input type="text">
     * ```
     *
     * A decorator can inject string literal `text` like so:
     *
     * {@example core/ts/metadata/metadata.ts region='attributeMetadata'}
     * @ts2dart_const
     * @stable
     */
    var AttributeMetadata = function (_super) {
        __extends(AttributeMetadata, _super);
        function AttributeMetadata(attributeName) {
            _super.call(this);
            this.attributeName = attributeName;
        }
        Object.defineProperty(AttributeMetadata.prototype, "token", {
            get: function () {
                // Normally one would default a token to a type of an injected value but here
                // the type of a variable is "string" and we can't use primitive type as a return value
                // so we use instance of Attribute instead. This doesn't matter much in practice as arguments
                // with @Attribute annotation are injected by ElementInjector that doesn't take tokens into
                // account.
                return this;
            },
            enumerable: true,
            configurable: true
        });
        AttributeMetadata.prototype.toString = function () {
            return "@Attribute(" + lang_1.stringify(this.attributeName) + ")";
        };
        return AttributeMetadata;
    }(metadata_1.DependencyMetadata);
    exports.AttributeMetadata = AttributeMetadata;
    /**
     * Declares an injectable parameter to be a live list of directives or variable
     * bindings from the content children of a directive.
     *
     * ### Example ([live demo](http://plnkr.co/edit/lY9m8HLy7z06vDoUaSN2?p=preview))
     *
     * Assume that `<tabs>` component would like to get a list its children `<pane>`
     * components as shown in this example:
     *
     * ```html
     * <tabs>
     *   <pane title="Overview">...</pane>
     *   <pane *ngFor="let o of objects" [title]="o.title">{{o.text}}</pane>
     * </tabs>
     * ```
     *
     * The preferred solution is to query for `Pane` directives using this decorator.
     *
     * ```javascript
     * @Component({
     *   selector: 'pane',
     *   inputs: ['title']
     * })
     * class Pane {
     *   title:string;
     * }
     *
     * @Component({
     *  selector: 'tabs',
     *  template: `
     *    <ul>
     *      <li *ngFor="let pane of panes">{{pane.title}}</li>
     *    </ul>
     *    <ng-content></ng-content>
     *  `
     * })
     * class Tabs {
     *   panes: QueryList<Pane>;
     *   constructor(@Query(Pane) panes:QueryList<Pane>) {
      *    this.panes = panes;
      *  }
     * }
     * ```
     *
     * A query can look for variable bindings by passing in a string with desired binding symbol.
     *
     * ### Example ([live demo](http://plnkr.co/edit/sT2j25cH1dURAyBRCKx1?p=preview))
     * ```html
     * <seeker>
     *   <div #findme>...</div>
     * </seeker>
     *
     * @Component({ selector: 'seeker' })
     * class Seeker {
     *   constructor(@Query('findme') elList: QueryList<ElementRef>) {...}
     * }
     * ```
     *
     * In this case the object that is injected depend on the type of the variable
     * binding. It can be an ElementRef, a directive or a component.
     *
     * Passing in a comma separated list of variable bindings will query for all of them.
     *
     * ```html
     * <seeker>
     *   <div #find-me>...</div>
     *   <div #find-me-too>...</div>
     * </seeker>
     *
     *  @Component({
     *   selector: 'seeker'
     * })
     * class Seeker {
     *   constructor(@Query('findMe, findMeToo') elList: QueryList<ElementRef>) {...}
     * }
     * ```
     *
     * Configure whether query looks for direct children or all descendants
     * of the querying element, by using the `descendants` parameter.
     * It is set to `false` by default.
     *
     * ### Example ([live demo](http://plnkr.co/edit/wtGeB977bv7qvA5FTYl9?p=preview))
     * ```html
     * <container #first>
     *   <item>a</item>
     *   <item>b</item>
     *   <container #second>
     *     <item>c</item>
     *   </container>
     * </container>
     * ```
     *
     * When querying for items, the first container will see only `a` and `b` by default,
     * but with `Query(TextDirective, {descendants: true})` it will see `c` too.
     *
     * The queried directives are kept in a depth-first pre-order with respect to their
     * positions in the DOM.
     *
     * Query does not look deep into any subcomponent views.
     *
     * Query is updated as part of the change-detection cycle. Since change detection
     * happens after construction of a directive, QueryList will always be empty when observed in the
     * constructor.
     *
     * The injected object is an unmodifiable live list.
     * See {@link QueryList} for more details.
     * @ts2dart_const
     * @deprecated
     */
    var QueryMetadata = function (_super) {
        __extends(QueryMetadata, _super);
        function QueryMetadata(_selector, _a) {
            var _b = _a === void 0 ? {} : _a,
                _c = _b.descendants,
                descendants = _c === void 0 ? false : _c,
                _d = _b.first,
                first = _d === void 0 ? false : _d,
                _e = _b.read,
                read = _e === void 0 ? null : _e;
            _super.call(this);
            this._selector = _selector;
            this.descendants = descendants;
            this.first = first;
            this.read = read;
        }
        Object.defineProperty(QueryMetadata.prototype, "isViewQuery", {
            /**
             * always `false` to differentiate it with {@link ViewQueryMetadata}.
             */
            get: function () {
                return false;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(QueryMetadata.prototype, "selector", {
            /**
             * what this is querying for.
             */
            get: function () {
                return forward_ref_1.resolveForwardRef(this._selector);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(QueryMetadata.prototype, "isVarBindingQuery", {
            /**
             * whether this is querying for a variable binding or a directive.
             */
            get: function () {
                return lang_1.isString(this.selector);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(QueryMetadata.prototype, "varBindings", {
            /**
             * returns a list of variable bindings this is querying for.
             * Only applicable if this is a variable bindings query.
             */
            get: function () {
                return lang_1.StringWrapper.split(this.selector, /\s*,\s*/g);
            },
            enumerable: true,
            configurable: true
        });
        QueryMetadata.prototype.toString = function () {
            return "@Query(" + lang_1.stringify(this.selector) + ")";
        };
        return QueryMetadata;
    }(metadata_1.DependencyMetadata);
    exports.QueryMetadata = QueryMetadata;
    // TODO: add an example after ContentChildren and ViewChildren are in master
    /**
     * Configures a content query.
     *
     * Content queries are set before the `ngAfterContentInit` callback is called.
     *
     * ### Example
     *
     * ```
     * @Directive({
     *   selector: 'someDir'
     * })
     * class SomeDir {
     *   @ContentChildren(ChildDirective) contentChildren: QueryList<ChildDirective>;
     *
     *   ngAfterContentInit() {
     *     // contentChildren is set
     *   }
     * }
     * ```
     * @ts2dart_const
     * @stable
     */
    var ContentChildrenMetadata = function (_super) {
        __extends(ContentChildrenMetadata, _super);
        function ContentChildrenMetadata(_selector, _a) {
            var _b = _a === void 0 ? {} : _a,
                _c = _b.descendants,
                descendants = _c === void 0 ? false : _c,
                _d = _b.read,
                read = _d === void 0 ? null : _d;
            _super.call(this, _selector, { descendants: descendants, read: read });
        }
        return ContentChildrenMetadata;
    }(QueryMetadata);
    exports.ContentChildrenMetadata = ContentChildrenMetadata;
    // TODO: add an example after ContentChild and ViewChild are in master
    /**
     * Configures a content query.
     *
     * Content queries are set before the `ngAfterContentInit` callback is called.
     *
     * ### Example
     *
     * ```
     * @Directive({
     *   selector: 'someDir'
     * })
     * class SomeDir {
     *   @ContentChild(ChildDirective) contentChild;
     *
     *   ngAfterContentInit() {
     *     // contentChild is set
     *   }
     * }
     * ```
     * @ts2dart_const
     * @stable
     */
    var ContentChildMetadata = function (_super) {
        __extends(ContentChildMetadata, _super);
        function ContentChildMetadata(_selector, _a) {
            var _b = (_a === void 0 ? {} : _a).read,
                read = _b === void 0 ? null : _b;
            _super.call(this, _selector, { descendants: true, first: true, read: read });
        }
        return ContentChildMetadata;
    }(QueryMetadata);
    exports.ContentChildMetadata = ContentChildMetadata;
    /**
     * Similar to {@link QueryMetadata}, but querying the component view, instead of
     * the content children.
     *
     * ### Example ([live demo](http://plnkr.co/edit/eNsFHDf7YjyM6IzKxM1j?p=preview))
     *
     * ```javascript
     * @Component({
     *   ...,
     *   template: `
     *     <item> a </item>
     *     <item> b </item>
     *     <item> c </item>
     *   `
     * })
     * class MyComponent {
     *   shown: boolean;
     *
     *   constructor(private @ViewQuery(Item) items:QueryList<Item>) {
     *     items.changes.subscribe(() => console.log(items.length));
     *   }
     * }
     * ```
     *
     * Supports the same querying parameters as {@link QueryMetadata}, except
     * `descendants`. This always queries the whole view.
     *
     * As `shown` is flipped between true and false, items will contain zero of one
     * items.
     *
     * Specifies that a {@link QueryList} should be injected.
     *
     * The injected object is an iterable and observable live list.
     * See {@link QueryList} for more details.
     * @ts2dart_const
     * @deprecated
     */
    var ViewQueryMetadata = function (_super) {
        __extends(ViewQueryMetadata, _super);
        function ViewQueryMetadata(_selector, _a) {
            var _b = _a === void 0 ? {} : _a,
                _c = _b.descendants,
                descendants = _c === void 0 ? false : _c,
                _d = _b.first,
                first = _d === void 0 ? false : _d,
                _e = _b.read,
                read = _e === void 0 ? null : _e;
            _super.call(this, _selector, { descendants: descendants, first: first, read: read });
        }
        Object.defineProperty(ViewQueryMetadata.prototype, "isViewQuery", {
            /**
             * always `true` to differentiate it with {@link QueryMetadata}.
             */
            get: function () {
                return true;
            },
            enumerable: true,
            configurable: true
        });
        ViewQueryMetadata.prototype.toString = function () {
            return "@ViewQuery(" + lang_1.stringify(this.selector) + ")";
        };
        return ViewQueryMetadata;
    }(QueryMetadata);
    exports.ViewQueryMetadata = ViewQueryMetadata;
    /**
     * Declares a list of child element references.
     *
     * Angular automatically updates the list when the DOM is updated.
     *
     * `ViewChildren` takes an argument to select elements.
     *
     * - If the argument is a type, directives or components with the type will be bound.
     *
     * - If the argument is a string, the string is interpreted as a list of comma-separated selectors.
     * For each selector, an element containing the matching template variable (e.g. `#child`) will be
     * bound.
     *
     * View children are set before the `ngAfterViewInit` callback is called.
     *
     * ### Example
     *
     * With type selector:
     *
     * ```
     * @Component({
     *   selector: 'child-cmp',
     *   template: '<p>child</p>'
     * })
     * class ChildCmp {
     *   doSomething() {}
     * }
     *
     * @Component({
     *   selector: 'some-cmp',
     *   template: `
     *     <child-cmp></child-cmp>
     *     <child-cmp></child-cmp>
     *     <child-cmp></child-cmp>
     *   `,
     *   directives: [ChildCmp]
     * })
     * class SomeCmp {
     *   @ViewChildren(ChildCmp) children:QueryList<ChildCmp>;
     *
     *   ngAfterViewInit() {
     *     // children are set
     *     this.children.toArray().forEach((child)=>child.doSomething());
     *   }
     * }
     * ```
     *
     * With string selector:
     *
     * ```
     * @Component({
     *   selector: 'child-cmp',
     *   template: '<p>child</p>'
     * })
     * class ChildCmp {
     *   doSomething() {}
     * }
     *
     * @Component({
     *   selector: 'some-cmp',
     *   template: `
     *     <child-cmp #child1></child-cmp>
     *     <child-cmp #child2></child-cmp>
     *     <child-cmp #child3></child-cmp>
     *   `,
     *   directives: [ChildCmp]
     * })
     * class SomeCmp {
     *   @ViewChildren('child1,child2,child3') children:QueryList<ChildCmp>;
     *
     *   ngAfterViewInit() {
     *     // children are set
     *     this.children.toArray().forEach((child)=>child.doSomething());
     *   }
     * }
     * ```
     * @ts2dart_const
     * @stable
     */
    var ViewChildrenMetadata = function (_super) {
        __extends(ViewChildrenMetadata, _super);
        function ViewChildrenMetadata(_selector, _a) {
            var _b = (_a === void 0 ? {} : _a).read,
                read = _b === void 0 ? null : _b;
            _super.call(this, _selector, { descendants: true, read: read });
        }
        return ViewChildrenMetadata;
    }(ViewQueryMetadata);
    exports.ViewChildrenMetadata = ViewChildrenMetadata;
    /**
     *
     * Declares a reference of child element.
     *
     * `ViewChildren` takes an argument to select elements.
     *
     * - If the argument is a type, a directive or a component with the type will be bound.
     *
     If the argument is a string, the string is interpreted as a selector. An element containing the
     matching template variable (e.g. `#child`) will be bound.
     *
     * In either case, `@ViewChild()` assigns the first (looking from above) element if there are
     multiple matches.
     *
     * View child is set before the `ngAfterViewInit` callback is called.
     *
     * ### Example
     *
     * With type selector:
     *
     * ```
     * @Component({
     *   selector: 'child-cmp',
     *   template: '<p>child</p>'
     * })
     * class ChildCmp {
     *   doSomething() {}
     * }
     *
     * @Component({
     *   selector: 'some-cmp',
     *   template: '<child-cmp></child-cmp>',
     *   directives: [ChildCmp]
     * })
     * class SomeCmp {
     *   @ViewChild(ChildCmp) child:ChildCmp;
     *
     *   ngAfterViewInit() {
     *     // child is set
     *     this.child.doSomething();
     *   }
     * }
     * ```
     *
     * With string selector:
     *
     * ```
     * @Component({
     *   selector: 'child-cmp',
     *   template: '<p>child</p>'
     * })
     * class ChildCmp {
     *   doSomething() {}
     * }
     *
     * @Component({
     *   selector: 'some-cmp',
     *   template: '<child-cmp #child></child-cmp>',
     *   directives: [ChildCmp]
     * })
     * class SomeCmp {
     *   @ViewChild('child') child:ChildCmp;
     *
     *   ngAfterViewInit() {
     *     // child is set
     *     this.child.doSomething();
     *   }
     * }
     * ```
     * @ts2dart_const
     * @stable
     */
    var ViewChildMetadata = function (_super) {
        __extends(ViewChildMetadata, _super);
        function ViewChildMetadata(_selector, _a) {
            var _b = (_a === void 0 ? {} : _a).read,
                read = _b === void 0 ? null : _b;
            _super.call(this, _selector, { descendants: true, first: true, read: read });
        }
        return ViewChildMetadata;
    }(ViewQueryMetadata);
    exports.ViewChildMetadata = ViewChildMetadata;
    
});
System.registerDynamic('node_modules/@angular/core/src/metadata/directives.js', ['../change_detection/constants', '../di/metadata', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var constants_1 = $__require('../change_detection/constants');
    var metadata_1 = $__require('../di/metadata');
    var lang_1 = $__require('../facade/lang');
    /**
     * Directives allow you to attach behavior to elements in the DOM.
     *
     * {@link DirectiveMetadata}s with an embedded view are called {@link ComponentMetadata}s.
     *
     * A directive consists of a single directive annotation and a controller class. When the
     * directive's `selector` matches
     * elements in the DOM, the following steps occur:
     *
     * 1. For each directive, the `ElementInjector` attempts to resolve the directive's constructor
     * arguments.
     * 2. Angular instantiates directives for each matched element using `ElementInjector` in a
     * depth-first order,
     *    as declared in the HTML.
     *
     * ## Understanding How Injection Works
     *
     * There are three stages of injection resolution.
     * - *Pre-existing Injectors*:
     *   - The terminal {@link Injector} cannot resolve dependencies. It either throws an error or, if
     * the dependency was
     *     specified as `@Optional`, returns `null`.
     *   - The platform injector resolves browser singleton resources, such as: cookies, title,
     * location, and others.
     * - *Component Injectors*: Each component instance has its own {@link Injector}, and they follow
     * the same parent-child hierarchy
     *     as the component instances in the DOM.
     * - *Element Injectors*: Each component instance has a Shadow DOM. Within the Shadow DOM each
     * element has an `ElementInjector`
     *     which follow the same parent-child hierarchy as the DOM elements themselves.
     *
     * When a template is instantiated, it also must instantiate the corresponding directives in a
     * depth-first order. The
     * current `ElementInjector` resolves the constructor dependencies for each directive.
     *
     * Angular then resolves dependencies as follows, according to the order in which they appear in the
     * {@link ViewMetadata}:
     *
     * 1. Dependencies on the current element
     * 2. Dependencies on element injectors and their parents until it encounters a Shadow DOM boundary
     * 3. Dependencies on component injectors and their parents until it encounters the root component
     * 4. Dependencies on pre-existing injectors
     *
     *
     * The `ElementInjector` can inject other directives, element-specific special objects, or it can
     * delegate to the parent
     * injector.
     *
     * To inject other directives, declare the constructor parameter as:
     * - `directive:DirectiveType`: a directive on the current element only
     * - `@Host() directive:DirectiveType`: any directive that matches the type between the current
     * element and the
     *    Shadow DOM root.
     * - `@Query(DirectiveType) query:QueryList<DirectiveType>`: A live collection of direct child
     * directives.
     * - `@QueryDescendants(DirectiveType) query:QueryList<DirectiveType>`: A live collection of any
     * child directives.
     *
     * To inject element-specific special objects, declare the constructor parameter as:
     * - `element: ElementRef` to obtain a reference to logical element in the view.
     * - `viewContainer: ViewContainerRef` to control child template instantiation, for
     * {@link DirectiveMetadata} directives only
     * - `bindingPropagation: BindingPropagation` to control change detection in a more granular way.
     *
     * ### Example
     *
     * The following example demonstrates how dependency injection resolves constructor arguments in
     * practice.
     *
     *
     * Assume this HTML template:
     *
     * ```
     * <div dependency="1">
     *   <div dependency="2">
     *     <div dependency="3" my-directive>
     *       <div dependency="4">
     *         <div dependency="5"></div>
     *       </div>
     *       <div dependency="6"></div>
     *     </div>
     *   </div>
     * </div>
     * ```
     *
     * With the following `dependency` decorator and `SomeService` injectable class.
     *
     * ```
     * @Injectable()
     * class SomeService {
     * }
     *
     * @Directive({
     *   selector: '[dependency]',
     *   inputs: [
     *     'id: dependency'
     *   ]
     * })
     * class Dependency {
     *   id:string;
     * }
     * ```
     *
     * Let's step through the different ways in which `MyDirective` could be declared...
     *
     *
     * ### No injection
     *
     * Here the constructor is declared with no arguments, therefore nothing is injected into
     * `MyDirective`.
     *
     * ```
     * @Directive({ selector: '[my-directive]' })
     * class MyDirective {
     *   constructor() {
     *   }
     * }
     * ```
     *
     * This directive would be instantiated with no dependencies.
     *
     *
     * ### Component-level injection
     *
     * Directives can inject any injectable instance from the closest component injector or any of its
     * parents.
     *
     * Here, the constructor declares a parameter, `someService`, and injects the `SomeService` type
     * from the parent
     * component's injector.
     * ```
     * @Directive({ selector: '[my-directive]' })
     * class MyDirective {
     *   constructor(someService: SomeService) {
     *   }
     * }
     * ```
     *
     * This directive would be instantiated with a dependency on `SomeService`.
     *
     *
     * ### Injecting a directive from the current element
     *
     * Directives can inject other directives declared on the current element.
     *
     * ```
     * @Directive({ selector: '[my-directive]' })
     * class MyDirective {
     *   constructor(dependency: Dependency) {
     *     expect(dependency.id).toEqual(3);
     *   }
     * }
     * ```
     * This directive would be instantiated with `Dependency` declared at the same element, in this case
     * `dependency="3"`.
     *
     * ### Injecting a directive from any ancestor elements
     *
     * Directives can inject other directives declared on any ancestor element (in the current Shadow
     * DOM), i.e. on the current element, the
     * parent element, or its parents.
     * ```
     * @Directive({ selector: '[my-directive]' })
     * class MyDirective {
     *   constructor(@Host() dependency: Dependency) {
     *     expect(dependency.id).toEqual(2);
     *   }
     * }
     * ```
     *
     * `@Host` checks the current element, the parent, as well as its parents recursively. If
     * `dependency="2"` didn't
     * exist on the direct parent, this injection would
     * have returned
     * `dependency="1"`.
     *
     *
     * ### Injecting a live collection of direct child directives
     *
     *
     * A directive can also query for other child directives. Since parent directives are instantiated
     * before child directives, a directive can't simply inject the list of child directives. Instead,
     * the directive injects a {@link QueryList}, which updates its contents as children are added,
     * removed, or moved by a directive that uses a {@link ViewContainerRef} such as a `ngFor`, an
     * `ngIf`, or an `ngSwitch`.
     *
     * ```
     * @Directive({ selector: '[my-directive]' })
     * class MyDirective {
     *   constructor(@Query(Dependency) dependencies:QueryList<Dependency>) {
     *   }
     * }
     * ```
     *
     * This directive would be instantiated with a {@link QueryList} which contains `Dependency` 4 and
     * `Dependency` 6. Here, `Dependency` 5 would not be included, because it is not a direct child.
     *
     * ### Injecting a live collection of descendant directives
     *
     * By passing the descendant flag to `@Query` above, we can include the children of the child
     * elements.
     *
     * ```
     * @Directive({ selector: '[my-directive]' })
     * class MyDirective {
     *   constructor(@Query(Dependency, {descendants: true}) dependencies:QueryList<Dependency>) {
     *   }
     * }
     * ```
     *
     * This directive would be instantiated with a Query which would contain `Dependency` 4, 5 and 6.
     *
     * ### Optional injection
     *
     * The normal behavior of directives is to return an error when a specified dependency cannot be
     * resolved. If you
     * would like to inject `null` on unresolved dependency instead, you can annotate that dependency
     * with `@Optional()`.
     * This explicitly permits the author of a template to treat some of the surrounding directives as
     * optional.
     *
     * ```
     * @Directive({ selector: '[my-directive]' })
     * class MyDirective {
     *   constructor(@Optional() dependency:Dependency) {
     *   }
     * }
     * ```
     *
     * This directive would be instantiated with a `Dependency` directive found on the current element.
     * If none can be
     * found, the injector supplies `null` instead of throwing an error.
     *
     * ### Example
     *
     * Here we use a decorator directive to simply define basic tool-tip behavior.
     *
     * ```
     * @Directive({
     *   selector: '[tooltip]',
     *   inputs: [
     *     'text: tooltip'
     *   ],
     *   host: {
     *     '(mouseenter)': 'onMouseEnter()',
     *     '(mouseleave)': 'onMouseLeave()'
     *   }
     * })
     * class Tooltip{
     *   text:string;
     *   overlay:Overlay; // NOT YET IMPLEMENTED
     *   overlayManager:OverlayManager; // NOT YET IMPLEMENTED
     *
     *   constructor(overlayManager:OverlayManager) {
     *     this.overlay = overlay;
     *   }
     *
     *   onMouseEnter() {
     *     // exact signature to be determined
     *     this.overlay = this.overlayManager.open(text, ...);
     *   }
     *
     *   onMouseLeave() {
     *     this.overlay.close();
     *     this.overlay = null;
     *   }
     * }
     * ```
     * In our HTML template, we can then add this behavior to a `<div>` or any other element with the
     * `tooltip` selector,
     * like so:
     *
     * ```
     * <div tooltip="some text here"></div>
     * ```
     *
     * Directives can also control the instantiation, destruction, and positioning of inline template
     * elements:
     *
     * A directive uses a {@link ViewContainerRef} to instantiate, insert, move, and destroy views at
     * runtime.
     * The {@link ViewContainerRef} is created as a result of `<template>` element, and represents a
     * location in the current view
     * where these actions are performed.
     *
     * Views are always created as children of the current {@link ViewMetadata}, and as siblings of the
     * `<template>` element. Thus a
     * directive in a child view cannot inject the directive that created it.
     *
     * Since directives that create views via ViewContainers are common in Angular, and using the full
     * `<template>` element syntax is wordy, Angular
     * also supports a shorthand notation: `<li *foo="bar">` and `<li template="foo: bar">` are
     * equivalent.
     *
     * Thus,
     *
     * ```
     * <ul>
     *   <li *foo="bar" title="text"></li>
     * </ul>
     * ```
     *
     * Expands in use to:
     *
     * ```
     * <ul>
     *   <template [foo]="bar">
     *     <li title="text"></li>
     *   </template>
     * </ul>
     * ```
     *
     * Notice that although the shorthand places `*foo="bar"` within the `<li>` element, the binding for
     * the directive
     * controller is correctly instantiated on the `<template>` element rather than the `<li>` element.
     *
     * ## Lifecycle hooks
     *
     * When the directive class implements some {@link ../../guide/lifecycle-hooks.html} the callbacks
     * are called by the change detection at defined points in time during the life of the directive.
     *
     * ### Example
     *
     * Let's suppose we want to implement the `unless` behavior, to conditionally include a template.
     *
     * Here is a simple directive that triggers on an `unless` selector:
     *
     * ```
     * @Directive({
     *   selector: '[unless]',
     *   inputs: ['unless']
     * })
     * export class Unless {
     *   viewContainer: ViewContainerRef;
     *   templateRef: TemplateRef;
     *   prevCondition: boolean;
     *
     *   constructor(viewContainer: ViewContainerRef, templateRef: TemplateRef) {
     *     this.viewContainer = viewContainer;
     *     this.templateRef = templateRef;
     *     this.prevCondition = null;
     *   }
     *
     *   set unless(newCondition) {
     *     if (newCondition && (isBlank(this.prevCondition) || !this.prevCondition)) {
     *       this.prevCondition = true;
     *       this.viewContainer.clear();
     *     } else if (!newCondition && (isBlank(this.prevCondition) || this.prevCondition)) {
     *       this.prevCondition = false;
     *       this.viewContainer.create(this.templateRef);
     *     }
     *   }
     * }
     * ```
     *
     * We can then use this `unless` selector in a template:
     * ```
     * <ul>
     *   <li *unless="expr"></li>
     * </ul>
     * ```
     *
     * Once the directive instantiates the child view, the shorthand notation for the template expands
     * and the result is:
     *
     * ```
     * <ul>
     *   <template [unless]="exp">
     *     <li></li>
     *   </template>
     *   <li></li>
     * </ul>
     * ```
     *
     * Note also that although the `<li></li>` template still exists inside the `<template></template>`,
     * the instantiated
     * view occurs on the second `<li></li>` which is a sibling to the `<template>` element.
     * @ts2dart_const
     * @stable
     */
    var DirectiveMetadata = function (_super) {
        __extends(DirectiveMetadata, _super);
        function DirectiveMetadata(_a) {
            var _b = _a === void 0 ? {} : _a,
                selector = _b.selector,
                inputs = _b.inputs,
                outputs = _b.outputs,
                properties = _b.properties,
                events = _b.events,
                host = _b.host,
                providers = _b.providers,
                exportAs = _b.exportAs,
                queries = _b.queries;
            _super.call(this);
            this.selector = selector;
            this._inputs = inputs;
            this._properties = properties;
            this._outputs = outputs;
            this._events = events;
            this.host = host;
            this.exportAs = exportAs;
            this.queries = queries;
            this._providers = providers;
        }
        Object.defineProperty(DirectiveMetadata.prototype, "inputs", {
            /**
             * Enumerates the set of data-bound input properties for a directive
             *
             * Angular automatically updates input properties during change detection.
             *
             * The `inputs` property defines a set of `directiveProperty` to `bindingProperty`
             * configuration:
             *
             * - `directiveProperty` specifies the component property where the value is written.
             * - `bindingProperty` specifies the DOM property where the value is read from.
             *
             * When `bindingProperty` is not provided, it is assumed to be equal to `directiveProperty`.
             *
             * ### Example ([live demo](http://plnkr.co/edit/ivhfXY?p=preview))
             *
             * The following example creates a component with two data-bound properties.
             *
             * ```typescript
             * @Component({
             *   selector: 'bank-account',
             *   inputs: ['bankName', 'id: account-id'],
             *   template: `
             *     Bank Name: {{bankName}}
             *     Account Id: {{id}}
             *   `
             * })
             * class BankAccount {
             *   bankName: string;
             *   id: string;
             *
             *   // this property is not bound, and won't be automatically updated by Angular
             *   normalizedBankName: string;
             * }
             *
             * @Component({
             *   selector: 'app',
             *   template: `
             *     <bank-account bank-name="RBC" account-id="4747"></bank-account>
             *   `,
             *   directives: [BankAccount]
             * })
             * class App {}
             *
             * bootstrap(App);
             * ```
             *
             */
            get: function () {
                return lang_1.isPresent(this._properties) && this._properties.length > 0 ? this._properties : this._inputs;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DirectiveMetadata.prototype, "properties", {
            /**
             * Use `inputs` instead
             *
             * @deprecated
             */
            get: function () {
                return this.inputs;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DirectiveMetadata.prototype, "outputs", {
            /**
             * Enumerates the set of event-bound output properties.
             *
             * When an output property emits an event, an event handler attached to that event
             * the template is invoked.
             *
             * The `outputs` property defines a set of `directiveProperty` to `bindingProperty`
             * configuration:
             *
             * - `directiveProperty` specifies the component property that emits events.
             * - `bindingProperty` specifies the DOM property the event handler is attached to.
             *
             * ### Example ([live demo](http://plnkr.co/edit/d5CNq7?p=preview))
             *
             * ```typescript
             * @Directive({
             *   selector: 'interval-dir',
             *   outputs: ['everySecond', 'five5Secs: everyFiveSeconds']
             * })
             * class IntervalDir {
             *   everySecond = new EventEmitter();
             *   five5Secs = new EventEmitter();
             *
             *   constructor() {
             *     setInterval(() => this.everySecond.emit("event"), 1000);
             *     setInterval(() => this.five5Secs.emit("event"), 5000);
             *   }
             * }
             *
             * @Component({
             *   selector: 'app',
             *   template: `
             *     <interval-dir (everySecond)="everySecond()" (everyFiveSeconds)="everyFiveSeconds()">
             *     </interval-dir>
             *   `,
             *   directives: [IntervalDir]
             * })
             * class App {
             *   everySecond() { console.log('second'); }
             *   everyFiveSeconds() { console.log('five seconds'); }
             * }
             * bootstrap(App);
             * ```
             *
             */
            get: function () {
                return lang_1.isPresent(this._events) && this._events.length > 0 ? this._events : this._outputs;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DirectiveMetadata.prototype, "events", {
            /**
             * Use `outputs` instead
             *
             * @deprecated
             */
            get: function () {
                return this.outputs;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DirectiveMetadata.prototype, "providers", {
            /**
             * Defines the set of injectable objects that are visible to a Directive and its light DOM
             * children.
             *
             * ## Simple Example
             *
             * Here is an example of a class that can be injected:
             *
             * ```
             * class Greeter {
             *    greet(name:string) {
             *      return 'Hello ' + name + '!';
             *    }
             * }
             *
             * @Directive({
             *   selector: 'greet',
             *   providers: [
             *     Greeter
             *   ]
             * })
             * class HelloWorld {
             *   greeter:Greeter;
             *
             *   constructor(greeter:Greeter) {
             *     this.greeter = greeter;
             *   }
             * }
             * ```
             */
            get: function () {
                return this._providers;
            },
            enumerable: true,
            configurable: true
        });
        return DirectiveMetadata;
    }(metadata_1.InjectableMetadata);
    exports.DirectiveMetadata = DirectiveMetadata;
    /**
     * Declare reusable UI building blocks for an application.
     *
     * Each Angular component requires a single `@Component` annotation. The
     * `@Component`
     * annotation specifies when a component is instantiated, and which properties and hostListeners it
     * binds to.
     *
     * When a component is instantiated, Angular
     * - creates a shadow DOM for the component.
     * - loads the selected template into the shadow DOM.
     * - creates all the injectable objects configured with `providers` and `viewProviders`.
     *
     * All template expressions and statements are then evaluated against the component instance.
     *
     * For details on the `@View` annotation, see {@link ViewMetadata}.
     *
     * ## Lifecycle hooks
     *
     * When the component class implements some {@link ../../guide/lifecycle-hooks.html} the callbacks
     * are called by the change detection at defined points in time during the life of the component.
     *
     * ### Example
     *
     * {@example core/ts/metadata/metadata.ts region='component'}
     * @ts2dart_const
     * @stable
     */
    var ComponentMetadata = function (_super) {
        __extends(ComponentMetadata, _super);
        function ComponentMetadata(_a) {
            var _b = _a === void 0 ? {} : _a,
                selector = _b.selector,
                inputs = _b.inputs,
                outputs = _b.outputs,
                properties = _b.properties,
                events = _b.events,
                host = _b.host,
                exportAs = _b.exportAs,
                moduleId = _b.moduleId,
                providers = _b.providers,
                viewProviders = _b.viewProviders,
                _c = _b.changeDetection,
                changeDetection = _c === void 0 ? constants_1.ChangeDetectionStrategy.Default : _c,
                queries = _b.queries,
                templateUrl = _b.templateUrl,
                template = _b.template,
                styleUrls = _b.styleUrls,
                styles = _b.styles,
                animations = _b.animations,
                directives = _b.directives,
                pipes = _b.pipes,
                encapsulation = _b.encapsulation,
                interpolation = _b.interpolation,
                precompile = _b.precompile;
            _super.call(this, {
                selector: selector,
                inputs: inputs,
                outputs: outputs,
                properties: properties,
                events: events,
                host: host,
                exportAs: exportAs,
                providers: providers,
                queries: queries
            });
            this.changeDetection = changeDetection;
            this._viewProviders = viewProviders;
            this.templateUrl = templateUrl;
            this.template = template;
            this.styleUrls = styleUrls;
            this.styles = styles;
            this.directives = directives;
            this.pipes = pipes;
            this.encapsulation = encapsulation;
            this.moduleId = moduleId;
            this.animations = animations;
            this.interpolation = interpolation;
            this.precompile = precompile;
        }
        Object.defineProperty(ComponentMetadata.prototype, "viewProviders", {
            /**
             * Defines the set of injectable objects that are visible to its view DOM children.
             *
             * ## Simple Example
             *
             * Here is an example of a class that can be injected:
             *
             * ```
             * class Greeter {
             *    greet(name:string) {
             *      return 'Hello ' + name + '!';
             *    }
             * }
             *
             * @Directive({
             *   selector: 'needs-greeter'
             * })
             * class NeedsGreeter {
             *   greeter:Greeter;
             *
             *   constructor(greeter:Greeter) {
             *     this.greeter = greeter;
             *   }
             * }
             *
             * @Component({
             *   selector: 'greet',
             *   viewProviders: [
             *     Greeter
             *   ],
             *   template: `<needs-greeter></needs-greeter>`,
             *   directives: [NeedsGreeter]
             * })
             * class HelloWorld {
             * }
             *
             * ```
             */
            get: function () {
                return this._viewProviders;
            },
            enumerable: true,
            configurable: true
        });
        return ComponentMetadata;
    }(DirectiveMetadata);
    exports.ComponentMetadata = ComponentMetadata;
    /**
     * Declare reusable pipe function.
     *
     * A "pure" pipe is only re-evaluated when either the input or any of the arguments change.
     *
     * When not specified, pipes default to being pure.
     *
     * ### Example
     *
     * {@example core/ts/metadata/metadata.ts region='pipe'}
     * @ts2dart_const
     * @stable
     */
    var PipeMetadata = function (_super) {
        __extends(PipeMetadata, _super);
        function PipeMetadata(_a) {
            var name = _a.name,
                pure = _a.pure;
            _super.call(this);
            this.name = name;
            this._pure = pure;
        }
        Object.defineProperty(PipeMetadata.prototype, "pure", {
            get: function () {
                return lang_1.isPresent(this._pure) ? this._pure : true;
            },
            enumerable: true,
            configurable: true
        });
        return PipeMetadata;
    }(metadata_1.InjectableMetadata);
    exports.PipeMetadata = PipeMetadata;
    /**
     * Declares a data-bound input property.
     *
     * Angular automatically updates data-bound properties during change detection.
     *
     * `InputMetadata` takes an optional parameter that specifies the name
     * used when instantiating a component in the template. When not provided,
     * the name of the decorated property is used.
     *
     * ### Example
     *
     * The following example creates a component with two input properties.
     *
     * ```typescript
     * @Component({
     *   selector: 'bank-account',
     *   template: `
     *     Bank Name: {{bankName}}
     *     Account Id: {{id}}
     *   `
     * })
     * class BankAccount {
     *   @Input() bankName: string;
     *   @Input('account-id') id: string;
     *
     *   // this property is not bound, and won't be automatically updated by Angular
     *   normalizedBankName: string;
     * }
     *
     * @Component({
     *   selector: 'app',
     *   template: `
     *     <bank-account bank-name="RBC" account-id="4747"></bank-account>
     *   `,
     *   directives: [BankAccount]
     * })
     * class App {}
     *
     * bootstrap(App);
     * ```
     * @ts2dart_const
     * @stable
     */
    var InputMetadata = function () {
        function InputMetadata(
        /**
         * Name used when instantiating a component in the template.
         */
        bindingPropertyName) {
            this.bindingPropertyName = bindingPropertyName;
        }
        return InputMetadata;
    }();
    exports.InputMetadata = InputMetadata;
    /**
     * Declares an event-bound output property.
     *
     * When an output property emits an event, an event handler attached to that event
     * the template is invoked.
     *
     * `OutputMetadata` takes an optional parameter that specifies the name
     * used when instantiating a component in the template. When not provided,
     * the name of the decorated property is used.
     *
     * ### Example
     *
     * ```typescript
     * @Directive({
     *   selector: 'interval-dir',
     * })
     * class IntervalDir {
     *   @Output() everySecond = new EventEmitter();
     *   @Output('everyFiveSeconds') five5Secs = new EventEmitter();
     *
     *   constructor() {
     *     setInterval(() => this.everySecond.emit("event"), 1000);
     *     setInterval(() => this.five5Secs.emit("event"), 5000);
     *   }
     * }
     *
     * @Component({
     *   selector: 'app',
     *   template: `
     *     <interval-dir (everySecond)="everySecond()" (everyFiveSeconds)="everyFiveSeconds()">
     *     </interval-dir>
     *   `,
     *   directives: [IntervalDir]
     * })
     * class App {
     *   everySecond() { console.log('second'); }
     *   everyFiveSeconds() { console.log('five seconds'); }
     * }
     * bootstrap(App);
     * ```
     * @ts2dart_const
     * @stable
     */
    var OutputMetadata = function () {
        function OutputMetadata(bindingPropertyName) {
            this.bindingPropertyName = bindingPropertyName;
        }
        return OutputMetadata;
    }();
    exports.OutputMetadata = OutputMetadata;
    /**
     * Declares a host property binding.
     *
     * Angular automatically checks host property bindings during change detection.
     * If a binding changes, it will update the host element of the directive.
     *
     * `HostBindingMetadata` takes an optional parameter that specifies the property
     * name of the host element that will be updated. When not provided,
     * the class property name is used.
     *
     * ### Example
     *
     * The following example creates a directive that sets the `valid` and `invalid` classes
     * on the DOM element that has ngModel directive on it.
     *
     * ```typescript
     * @Directive({selector: '[ngModel]'})
     * class NgModelStatus {
     *   constructor(public control:NgModel) {}
     *   @HostBinding('class.valid') get valid { return this.control.valid; }
     *   @HostBinding('class.invalid') get invalid { return this.control.invalid; }
     * }
     *
     * @Component({
     *   selector: 'app',
     *   template: `<input [(ngModel)]="prop">`,
     *   directives: [FORM_DIRECTIVES, NgModelStatus]
     * })
     * class App {
     *   prop;
     * }
     *
     * bootstrap(App);
     * ```
     * @ts2dart_const
     * @stable
     */
    var HostBindingMetadata = function () {
        function HostBindingMetadata(hostPropertyName) {
            this.hostPropertyName = hostPropertyName;
        }
        return HostBindingMetadata;
    }();
    exports.HostBindingMetadata = HostBindingMetadata;
    /**
     * Declares a host listener.
     *
     * Angular will invoke the decorated method when the host element emits the specified event.
     *
     * If the decorated method returns `false`, then `preventDefault` is applied on the DOM
     * event.
     *
     * ### Example
     *
     * The following example declares a directive that attaches a click listener to the button and
     * counts clicks.
     *
     * ```typescript
     * @Directive({selector: 'button[counting]'})
     * class CountClicks {
     *   numberOfClicks = 0;
     *
     *   @HostListener('click', ['$event.target'])
     *   onClick(btn) {
     *     console.log("button", btn, "number of clicks:", this.numberOfClicks++);
     *   }
     * }
     *
     * @Component({
     *   selector: 'app',
     *   template: `<button counting>Increment</button>`,
     *   directives: [CountClicks]
     * })
     * class App {}
     *
     * bootstrap(App);
     * ```
     * @ts2dart_const
     * @stable
     */
    var HostListenerMetadata = function () {
        function HostListenerMetadata(eventName, args) {
            this.eventName = eventName;
            this.args = args;
        }
        return HostListenerMetadata;
    }();
    exports.HostListenerMetadata = HostListenerMetadata;
    
});
System.registerDynamic('node_modules/@angular/core/src/metadata.js', ['./metadata/di', './metadata/directives', './metadata/view', './metadata/lifecycle_hooks', './util/decorators'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var di_1 = $__require('./metadata/di');
  var directives_1 = $__require('./metadata/directives');
  var view_1 = $__require('./metadata/view');
  var di_2 = $__require('./metadata/di');
  exports.AttributeMetadata = di_2.AttributeMetadata;
  exports.ContentChildMetadata = di_2.ContentChildMetadata;
  exports.ContentChildrenMetadata = di_2.ContentChildrenMetadata;
  exports.QueryMetadata = di_2.QueryMetadata;
  exports.ViewChildMetadata = di_2.ViewChildMetadata;
  exports.ViewChildrenMetadata = di_2.ViewChildrenMetadata;
  exports.ViewQueryMetadata = di_2.ViewQueryMetadata;
  var directives_2 = $__require('./metadata/directives');
  exports.ComponentMetadata = directives_2.ComponentMetadata;
  exports.DirectiveMetadata = directives_2.DirectiveMetadata;
  exports.HostBindingMetadata = directives_2.HostBindingMetadata;
  exports.HostListenerMetadata = directives_2.HostListenerMetadata;
  exports.InputMetadata = directives_2.InputMetadata;
  exports.OutputMetadata = directives_2.OutputMetadata;
  exports.PipeMetadata = directives_2.PipeMetadata;
  var lifecycle_hooks_1 = $__require('./metadata/lifecycle_hooks');
  exports.AfterContentChecked = lifecycle_hooks_1.AfterContentChecked;
  exports.AfterContentInit = lifecycle_hooks_1.AfterContentInit;
  exports.AfterViewChecked = lifecycle_hooks_1.AfterViewChecked;
  exports.AfterViewInit = lifecycle_hooks_1.AfterViewInit;
  exports.DoCheck = lifecycle_hooks_1.DoCheck;
  exports.OnChanges = lifecycle_hooks_1.OnChanges;
  exports.OnDestroy = lifecycle_hooks_1.OnDestroy;
  exports.OnInit = lifecycle_hooks_1.OnInit;
  var view_2 = $__require('./metadata/view');
  exports.ViewEncapsulation = view_2.ViewEncapsulation;
  exports.ViewMetadata = view_2.ViewMetadata;
  var decorators_1 = $__require('./util/decorators');
  // TODO(alexeagle): remove the duplication of this doc. It is copied from ComponentMetadata.
  /**
   * Declare reusable UI building blocks for an application.
   *
   * Each Angular component requires a single `@Component` annotation. The `@Component`
   * annotation specifies when a component is instantiated, and which properties and hostListeners it
   * binds to.
   *
   * When a component is instantiated, Angular
   * - creates a shadow DOM for the component.
   * - loads the selected template into the shadow DOM.
   * - creates all the injectable objects configured with `providers` and `viewProviders`.
   *
   * All template expressions and statements are then evaluated against the component instance.
   *
   * ## Lifecycle hooks
   *
   * When the component class implements some {@link ../../guide/lifecycle-hooks.html} the callbacks
   * are called by the change detection at defined points in time during the life of the component.
   *
   * ### Example
   *
   * {@example core/ts/metadata/metadata.ts region='component'}
   * @stable
   * @Annotation
   */
  exports.Component = decorators_1.makeDecorator(directives_1.ComponentMetadata, function (fn) {
    return fn.View = View;
  });
  // TODO(alexeagle): remove the duplication of this doc. It is copied from DirectiveMetadata.
  /**
   * Directives allow you to attach behavior to elements in the DOM.
   *
   * {@link DirectiveMetadata}s with an embedded view are called {@link ComponentMetadata}s.
   *
   * A directive consists of a single directive annotation and a controller class. When the
   * directive's `selector` matches
   * elements in the DOM, the following steps occur:
   *
   * 1. For each directive, the `ElementInjector` attempts to resolve the directive's constructor
   * arguments.
   * 2. Angular instantiates directives for each matched element using `ElementInjector` in a
   * depth-first order,
   *    as declared in the HTML.
   *
   * ## Understanding How Injection Works
   *
   * There are three stages of injection resolution.
   * - *Pre-existing Injectors*:
   *   - The terminal {@link Injector} cannot resolve dependencies. It either throws an error or, if
   * the dependency was
   *     specified as `@Optional`, returns `null`.
   *   - The platform injector resolves browser singleton resources, such as: cookies, title,
   * location, and others.
   * - *Component Injectors*: Each component instance has its own {@link Injector}, and they follow
   * the same parent-child hierarchy
   *     as the component instances in the DOM.
   * - *Element Injectors*: Each component instance has a Shadow DOM. Within the Shadow DOM each
   * element has an `ElementInjector`
   *     which follow the same parent-child hierarchy as the DOM elements themselves.
   *
   * When a template is instantiated, it also must instantiate the corresponding directives in a
   * depth-first order. The
   * current `ElementInjector` resolves the constructor dependencies for each directive.
   *
   * Angular then resolves dependencies as follows, according to the order in which they appear in the
   * {@link ViewMetadata}:
   *
   * 1. Dependencies on the current element
   * 2. Dependencies on element injectors and their parents until it encounters a Shadow DOM boundary
   * 3. Dependencies on component injectors and their parents until it encounters the root component
   * 4. Dependencies on pre-existing injectors
   *
   *
   * The `ElementInjector` can inject other directives, element-specific special objects, or it can
   * delegate to the parent
   * injector.
   *
   * To inject other directives, declare the constructor parameter as:
   * - `directive:DirectiveType`: a directive on the current element only
   * - `@Host() directive:DirectiveType`: any directive that matches the type between the current
   * element and the
   *    Shadow DOM root.
   * - `@Query(DirectiveType) query:QueryList<DirectiveType>`: A live collection of direct child
   * directives.
   * - `@QueryDescendants(DirectiveType) query:QueryList<DirectiveType>`: A live collection of any
   * child directives.
   *
   * To inject element-specific special objects, declare the constructor parameter as:
   * - `element: ElementRef` to obtain a reference to logical element in the view.
   * - `viewContainer: ViewContainerRef` to control child template instantiation, for
   * {@link DirectiveMetadata} directives only
   * - `bindingPropagation: BindingPropagation` to control change detection in a more granular way.
   *
   * ### Example
   *
   * The following example demonstrates how dependency injection resolves constructor arguments in
   * practice.
   *
   *
   * Assume this HTML template:
   *
   * ```
   * <div dependency="1">
   *   <div dependency="2">
   *     <div dependency="3" my-directive>
   *       <div dependency="4">
   *         <div dependency="5"></div>
   *       </div>
   *       <div dependency="6"></div>
   *     </div>
   *   </div>
   * </div>
   * ```
   *
   * With the following `dependency` decorator and `SomeService` injectable class.
   *
   * ```
   * @Injectable()
   * class SomeService {
   * }
   *
   * @Directive({
   *   selector: '[dependency]',
   *   inputs: [
   *     'id: dependency'
   *   ]
   * })
   * class Dependency {
   *   id:string;
   * }
   * ```
   *
   * Let's step through the different ways in which `MyDirective` could be declared...
   *
   *
   * ### No injection
   *
   * Here the constructor is declared with no arguments, therefore nothing is injected into
   * `MyDirective`.
   *
   * ```
   * @Directive({ selector: '[my-directive]' })
   * class MyDirective {
   *   constructor() {
   *   }
   * }
   * ```
   *
   * This directive would be instantiated with no dependencies.
   *
   *
   * ### Component-level injection
   *
   * Directives can inject any injectable instance from the closest component injector or any of its
   * parents.
   *
   * Here, the constructor declares a parameter, `someService`, and injects the `SomeService` type
   * from the parent
   * component's injector.
   * ```
   * @Directive({ selector: '[my-directive]' })
   * class MyDirective {
   *   constructor(someService: SomeService) {
   *   }
   * }
   * ```
   *
   * This directive would be instantiated with a dependency on `SomeService`.
   *
   *
   * ### Injecting a directive from the current element
   *
   * Directives can inject other directives declared on the current element.
   *
   * ```
   * @Directive({ selector: '[my-directive]' })
   * class MyDirective {
   *   constructor(dependency: Dependency) {
   *     expect(dependency.id).toEqual(3);
   *   }
   * }
   * ```
   * This directive would be instantiated with `Dependency` declared at the same element, in this case
   * `dependency="3"`.
   *
   * ### Injecting a directive from any ancestor elements
   *
   * Directives can inject other directives declared on any ancestor element (in the current Shadow
   * DOM), i.e. on the current element, the
   * parent element, or its parents.
   * ```
   * @Directive({ selector: '[my-directive]' })
   * class MyDirective {
   *   constructor(@Host() dependency: Dependency) {
   *     expect(dependency.id).toEqual(2);
   *   }
   * }
   * ```
   *
   * `@Host` checks the current element, the parent, as well as its parents recursively. If
   * `dependency="2"` didn't
   * exist on the direct parent, this injection would
   * have returned
   * `dependency="1"`.
   *
   *
   * ### Injecting a live collection of direct child directives
   *
   *
   * A directive can also query for other child directives. Since parent directives are instantiated
   * before child directives, a directive can't simply inject the list of child directives. Instead,
   * the directive injects a {@link QueryList}, which updates its contents as children are added,
   * removed, or moved by a directive that uses a {@link ViewContainerRef} such as a `ngFor`, an
   * `ngIf`, or an `ngSwitch`.
   *
   * ```
   * @Directive({ selector: '[my-directive]' })
   * class MyDirective {
   *   constructor(@Query(Dependency) dependencies:QueryList<Dependency>) {
   *   }
   * }
   * ```
   *
   * This directive would be instantiated with a {@link QueryList} which contains `Dependency` 4 and
   * 6. Here, `Dependency` 5 would not be included, because it is not a direct child.
   *
   * ### Injecting a live collection of descendant directives
   *
   * By passing the descendant flag to `@Query` above, we can include the children of the child
   * elements.
   *
   * ```
   * @Directive({ selector: '[my-directive]' })
   * class MyDirective {
   *   constructor(@Query(Dependency, {descendants: true}) dependencies:QueryList<Dependency>) {
   *   }
   * }
   * ```
   *
   * This directive would be instantiated with a Query which would contain `Dependency` 4, 5 and 6.
   *
   * ### Optional injection
   *
   * The normal behavior of directives is to return an error when a specified dependency cannot be
   * resolved. If you
   * would like to inject `null` on unresolved dependency instead, you can annotate that dependency
   * with `@Optional()`.
   * This explicitly permits the author of a template to treat some of the surrounding directives as
   * optional.
   *
   * ```
   * @Directive({ selector: '[my-directive]' })
   * class MyDirective {
   *   constructor(@Optional() dependency:Dependency) {
   *   }
   * }
   * ```
   *
   * This directive would be instantiated with a `Dependency` directive found on the current element.
   * If none can be
   * found, the injector supplies `null` instead of throwing an error.
   *
   * ### Example
   *
   * Here we use a decorator directive to simply define basic tool-tip behavior.
   *
   * ```
   * @Directive({
   *   selector: '[tooltip]',
   *   inputs: [
   *     'text: tooltip'
   *   ],
   *   host: {
   *     '(mouseenter)': 'onMouseEnter()',
   *     '(mouseleave)': 'onMouseLeave()'
   *   }
   * })
   * class Tooltip{
   *   text:string;
   *   overlay:Overlay; // NOT YET IMPLEMENTED
   *   overlayManager:OverlayManager; // NOT YET IMPLEMENTED
   *
   *   constructor(overlayManager:OverlayManager) {
   *     this.overlayManager = overlayManager;
   *   }
   *
   *   onMouseEnter() {
   *     // exact signature to be determined
   *     this.overlay = this.overlayManager.open(text, ...);
   *   }
   *
   *   onMouseLeave() {
   *     this.overlay.close();
   *     this.overlay = null;
   *   }
   * }
   * ```
   * In our HTML template, we can then add this behavior to a `<div>` or any other element with the
   * `tooltip` selector,
   * like so:
   *
   * ```
   * <div tooltip="some text here"></div>
   * ```
   *
   * Directives can also control the instantiation, destruction, and positioning of inline template
   * elements:
   *
   * A directive uses a {@link ViewContainerRef} to instantiate, insert, move, and destroy views at
   * runtime.
   * The {@link ViewContainerRef} is created as a result of `<template>` element, and represents a
   * location in the current view
   * where these actions are performed.
   *
   * Views are always created as children of the current {@link ViewMetadata}, and as siblings of the
   * `<template>` element. Thus a
   * directive in a child view cannot inject the directive that created it.
   *
   * Since directives that create views via ViewContainers are common in Angular, and using the full
   * `<template>` element syntax is wordy, Angular
   * also supports a shorthand notation: `<li *foo="bar">` and `<li template="foo: bar">` are
   * equivalent.
   *
   * Thus,
   *
   * ```
   * <ul>
   *   <li *foo="bar" title="text"></li>
   * </ul>
   * ```
   *
   * Expands in use to:
   *
   * ```
   * <ul>
   *   <template [foo]="bar">
   *     <li title="text"></li>
   *   </template>
   * </ul>
   * ```
   *
   * Notice that although the shorthand places `*foo="bar"` within the `<li>` element, the binding for
   * the directive
   * controller is correctly instantiated on the `<template>` element rather than the `<li>` element.
   *
   * ## Lifecycle hooks
   *
   * When the directive class implements some {@link ../../guide/lifecycle-hooks.html} the callbacks
   * are called by the change detection at defined points in time during the life of the directive.
   *
   * ### Example
   *
   * Let's suppose we want to implement the `unless` behavior, to conditionally include a template.
   *
   * Here is a simple directive that triggers on an `unless` selector:
   *
   * ```
   * @Directive({
   *   selector: '[unless]',
   *   inputs: ['unless']
   * })
   * export class Unless {
   *   viewContainer: ViewContainerRef;
   *   templateRef: TemplateRef;
   *   prevCondition: boolean;
   *
   *   constructor(viewContainer: ViewContainerRef, templateRef: TemplateRef) {
   *     this.viewContainer = viewContainer;
   *     this.templateRef = templateRef;
   *     this.prevCondition = null;
   *   }
   *
   *   set unless(newCondition) {
   *     if (newCondition && (isBlank(this.prevCondition) || !this.prevCondition)) {
   *       this.prevCondition = true;
   *       this.viewContainer.clear();
   *     } else if (!newCondition && (isBlank(this.prevCondition) || this.prevCondition)) {
   *       this.prevCondition = false;
   *       this.viewContainer.create(this.templateRef);
   *     }
   *   }
   * }
   * ```
   *
   * We can then use this `unless` selector in a template:
   * ```
   * <ul>
   *   <li *unless="expr"></li>
   * </ul>
   * ```
   *
   * Once the directive instantiates the child view, the shorthand notation for the template expands
   * and the result is:
   *
   * ```
   * <ul>
   *   <template [unless]="exp">
   *     <li></li>
   *   </template>
   *   <li></li>
   * </ul>
   * ```
   *
   * Note also that although the `<li></li>` template still exists inside the `<template></template>`,
   * the instantiated
   * view occurs on the second `<li></li>` which is a sibling to the `<template>` element.
   * @stable
   * @Annotation
   */
  exports.Directive = decorators_1.makeDecorator(directives_1.DirectiveMetadata);
  // TODO(alexeagle): remove the duplication of this doc. It is copied from ViewMetadata.
  /**
   * Metadata properties available for configuring Views.
   *
   * Each Angular component requires a single `@Component` and at least one `@View` annotation. The
   * `@View` annotation specifies the HTML template to use, and lists the directives that are active
   * within the template.
   *
   * When a component is instantiated, the template is loaded into the component's shadow root, and
   * the expressions and statements in the template are evaluated against the component.
   *
   * For details on the `@Component` annotation, see {@link ComponentMetadata}.
   *
   * ### Example
   *
   * ```
   * @Component({
   *   selector: 'greet',
   *   template: 'Hello {{name}}!',
   *   directives: [GreetUser, Bold]
   * })
   * class Greet {
   *   name: string;
   *
   *   constructor() {
   *     this.name = 'World';
   *   }
   * }
   * ```
   * @deprecated
   * @Annotation
   */
  var View = decorators_1.makeDecorator(view_1.ViewMetadata, function (fn) {
    return fn.View = View;
  });
  /**
   * Specifies that a constant attribute value should be injected.
   *
   * The directive can inject constant string literals of host element attributes.
   *
   * ### Example
   *
   * Suppose we have an `<input>` element and want to know its `type`.
   *
   * ```html
   * <input type="text">
   * ```
   *
   * A decorator can inject string literal `text` like so:
   *
   * {@example core/ts/metadata/metadata.ts region='attributeMetadata'}
   * @stable
   * @Annotation
   */
  exports.Attribute = decorators_1.makeParamDecorator(di_1.AttributeMetadata);
  // TODO(alexeagle): remove the duplication of this doc. It is copied from QueryMetadata.
  /**
   * Declares an injectable parameter to be a live list of directives or variable
   * bindings from the content children of a directive.
   *
   * ### Example ([live demo](http://plnkr.co/edit/lY9m8HLy7z06vDoUaSN2?p=preview))
   *
   * Assume that `<tabs>` component would like to get a list its children `<pane>`
   * components as shown in this example:
   *
   * ```html
   * <tabs>
   *   <pane title="Overview">...</pane>
   *   <pane *ngFor="let o of objects" [title]="o.title">{{o.text}}</pane>
   * </tabs>
   * ```
   *
   * The preferred solution is to query for `Pane` directives using this decorator.
   *
   * ```javascript
   * @Component({
   *   selector: 'pane',
   *   inputs: ['title']
   * })
   * class Pane {
   *   title:string;
   * }
   *
   * @Component({
   *  selector: 'tabs',
   *  template: `
   *    <ul>
   *      <li *ngFor="let pane of panes">{{pane.title}}</li>
   *    </ul>
   *    <ng-content></ng-content>
   *  `
   * })
   * class Tabs {
   *   panes: QueryList<Pane>;
   *   constructor(@Query(Pane) panes:QueryList<Pane>) {
   *     this.panes = panes;
   *   }
   * }
   * ```
   *
   * A query can look for variable bindings by passing in a string with desired binding symbol.
   *
   * ### Example ([live demo](http://plnkr.co/edit/sT2j25cH1dURAyBRCKx1?p=preview))
   * ```html
   * <seeker>
   *   <div #findme>...</div>
   * </seeker>
   *
   * @Component({ selector: 'seeker' })
   * class seeker {
   *   constructor(@Query('findme') elList: QueryList<ElementRef>) {...}
   * }
   * ```
   *
   * In this case the object that is injected depend on the type of the variable
   * binding. It can be an ElementRef, a directive or a component.
   *
   * Passing in a comma separated list of variable bindings will query for all of them.
   *
   * ```html
   * <seeker>
   *   <div #findMe>...</div>
   *   <div #findMeToo>...</div>
   * </seeker>
   *
   *  @Component({
   *   selector: 'seeker'
   * })
   * class Seeker {
   *   constructor(@Query('findMe, findMeToo') elList: QueryList<ElementRef>) {...}
   * }
   * ```
   *
   * Configure whether query looks for direct children or all descendants
   * of the querying element, by using the `descendants` parameter.
   * It is set to `false` by default.
   *
   * ### Example ([live demo](http://plnkr.co/edit/wtGeB977bv7qvA5FTYl9?p=preview))
   * ```html
   * <container #first>
   *   <item>a</item>
   *   <item>b</item>
   *   <container #second>
   *     <item>c</item>
   *   </container>
   * </container>
   * ```
   *
   * When querying for items, the first container will see only `a` and `b` by default,
   * but with `Query(TextDirective, {descendants: true})` it will see `c` too.
   *
   * The queried directives are kept in a depth-first pre-order with respect to their
   * positions in the DOM.
   *
   * Query does not look deep into any subcomponent views.
   *
   * Query is updated as part of the change-detection cycle. Since change detection
   * happens after construction of a directive, QueryList will always be empty when observed in the
   * constructor.
   *
   * The injected object is an unmodifiable live list.
   * See {@link QueryList} for more details.
   * @deprecated
   * @Annotation
   */
  exports.Query = decorators_1.makeParamDecorator(di_1.QueryMetadata);
  // TODO(alexeagle): remove the duplication of this doc. It is copied from ContentChildrenMetadata.
  /**
   * Configures a content query.
   *
   * Content queries are set before the `ngAfterContentInit` callback is called.
   *
   * ### Example
   *
   * ```
   * @Directive({
   *   selector: 'someDir'
   * })
   * class SomeDir {
   *   @ContentChildren(ChildDirective) contentChildren: QueryList<ChildDirective>;
   *
   *   ngAfterContentInit() {
   *     // contentChildren is set
   *   }
   * }
   * ```
   * @stable
   * @Annotation
   */
  exports.ContentChildren = decorators_1.makePropDecorator(di_1.ContentChildrenMetadata);
  // TODO(alexeagle): remove the duplication of this doc. It is copied from ContentChildMetadata.
  /**
   * Configures a content query.
   *
   * Content queries are set before the `ngAfterContentInit` callback is called.
   *
   * ### Example
   *
   * ```
   * @Directive({
   *   selector: 'someDir'
   * })
   * class SomeDir {
   *   @ContentChild(ChildDirective) contentChild;
   *   @ContentChild('container_ref') containerChild
   *
   *   ngAfterContentInit() {
   *     // contentChild is set
   *     // containerChild is set
   *   }
   * }
   * ```
   *
   * ```html
   * <container #container_ref>
   *   <item>a</item>
   *   <item>b</item>
   * </container>
   * ```
   * @stable
   * @Annotation
   */
  exports.ContentChild = decorators_1.makePropDecorator(di_1.ContentChildMetadata);
  // TODO(alexeagle): remove the duplication of this doc. It is copied from ViewChildrenMetadata.
  /**
   * Declares a list of child element references.
   *
   * Angular automatically updates the list when the DOM is updated.
   *
   * `ViewChildren` takes a argument to select elements.
   *
   * - If the argument is a type, directives or components with the type will be bound.
   *
   * - If the argument is a string, the string is interpreted as a list of comma-separated selectors.
   * For each selector, an element containing the matching template variable (e.g. `#child`) will be
   * bound.
   *
   * View children are set before the `ngAfterViewInit` callback is called.
   *
   * ### Example
   *
   * With type selector:
   *
   * ```
   * @Component({
   *   selector: 'child-cmp',
   *   template: '<p>child</p>'
   * })
   * class ChildCmp {
   *   doSomething() {}
   * }
   *
   * @Component({
   *   selector: 'some-cmp',
   *   template: `
   *     <child-cmp></child-cmp>
   *     <child-cmp></child-cmp>
   *     <child-cmp></child-cmp>
   *   `,
   *   directives: [ChildCmp]
   * })
   * class SomeCmp {
   *   @ViewChildren(ChildCmp) children:QueryList<ChildCmp>;
   *
   *   ngAfterViewInit() {
   *     // children are set
   *     this.children.toArray().forEach((child)=>child.doSomething());
   *   }
   * }
   * ```
   *
   * With string selector:
   *
   * ```
   * @Component({
   *   selector: 'child-cmp',
   *   template: '<p>child</p>'
   * })
   * class ChildCmp {
   *   doSomething() {}
   * }
   *
   * @Component({
   *   selector: 'some-cmp',
   *   template: `
   *     <child-cmp #child1></child-cmp>
   *     <child-cmp #child2></child-cmp>
   *     <child-cmp #child3></child-cmp>
   *   `,
   *   directives: [ChildCmp]
   * })
   * class SomeCmp {
   *   @ViewChildren('child1,child2,child3') children:QueryList<ChildCmp>;
   *
   *   ngAfterViewInit() {
   *     // children are set
   *     this.children.toArray().forEach((child)=>child.doSomething());
   *   }
   * }
   * ```
   *
   * See also: [ViewChildrenMetadata]
   * @stable
   * @Annotation
   */
  exports.ViewChildren = decorators_1.makePropDecorator(di_1.ViewChildrenMetadata);
  // TODO(alexeagle): remove the duplication of this doc. It is copied from ViewChildMetadata.
  /**
   * Declares a reference to a child element.
   *
   * `ViewChildren` takes a argument to select elements.
   *
   * - If the argument is a type, a directive or a component with the type will be bound.
   *
   * - If the argument is a string, the string is interpreted as a selector. An element containing the
   * matching template variable (e.g. `#child`) will be bound.
   *
   * In either case, `@ViewChild()` assigns the first (looking from above) element if there are
   * multiple matches.
   *
   * View child is set before the `ngAfterViewInit` callback is called.
   *
   * ### Example
   *
   * With type selector:
   *
   * ```
   * @Component({
   *   selector: 'child-cmp',
   *   template: '<p>child</p>'
   * })
   * class ChildCmp {
   *   doSomething() {}
   * }
   *
   * @Component({
   *   selector: 'some-cmp',
   *   template: '<child-cmp></child-cmp>',
   *   directives: [ChildCmp]
   * })
   * class SomeCmp {
   *   @ViewChild(ChildCmp) child:ChildCmp;
   *
   *   ngAfterViewInit() {
   *     // child is set
   *     this.child.doSomething();
   *   }
   * }
   * ```
   *
   * With string selector:
   *
   * ```
   * @Component({
   *   selector: 'child-cmp',
   *   template: '<p>child</p>'
   * })
   * class ChildCmp {
   *   doSomething() {}
   * }
   *
   * @Component({
   *   selector: 'some-cmp',
   *   template: '<child-cmp #child></child-cmp>',
   *   directives: [ChildCmp]
   * })
   * class SomeCmp {
   *   @ViewChild('child') child:ChildCmp;
   *
   *   ngAfterViewInit() {
   *     // child is set
   *     this.child.doSomething();
   *   }
   * }
   * ```
   * See also: [ViewChildMetadata]
   * @stable
   * @Annotation
   */
  exports.ViewChild = decorators_1.makePropDecorator(di_1.ViewChildMetadata);
  // TODO(alexeagle): remove the duplication of this doc. It is copied from ViewQueryMetadata.
  /**
   * Similar to {@link QueryMetadata}, but querying the component view, instead of
   * the content children.
   *
   * ### Example ([live demo](http://plnkr.co/edit/eNsFHDf7YjyM6IzKxM1j?p=preview))
   *
   * ```javascript
   * @Component({
   *   ...,
   *   template: `
   *     <item> a </item>
   *     <item> b </item>
   *     <item> c </item>
   *   `
   * })
   * class MyComponent {
   *   shown: boolean;
   *
   *   constructor(private @Query(Item) items:QueryList<Item>) {
   *     items.changes.subscribe(() => console.log(items.length));
   *   }
   * }
   * ```
   *
   * Supports the same querying parameters as {@link QueryMetadata}, except
   * `descendants`. This always queries the whole view.
   *
   * As `shown` is flipped between true and false, items will contain zero of one
   * items.
   *
   * Specifies that a {@link QueryList} should be injected.
   *
   * The injected object is an iterable and observable live list.
   * See {@link QueryList} for more details.
   * @deprecated
   * @Annotation
   */
  exports.ViewQuery = decorators_1.makeParamDecorator(di_1.ViewQueryMetadata);
  // TODO(alexeagle): remove the duplication of this doc. It is copied from PipeMetadata.
  /**
   * Declare reusable pipe function.
   *
   * ### Example
   *
   * {@example core/ts/metadata/metadata.ts region='pipe'}
   * @stable
   * @Annotation
   */
  exports.Pipe = decorators_1.makeDecorator(directives_1.PipeMetadata);
  // TODO(alexeagle): remove the duplication of this doc. It is copied from InputMetadata.
  /**
   * Declares a data-bound input property.
   *
   * Angular automatically updates data-bound properties during change detection.
   *
   * `InputMetadata` takes an optional parameter that specifies the name
   * used when instantiating a component in the template. When not provided,
   * the name of the decorated property is used.
   *
   * ### Example
   *
   * The following example creates a component with two input properties.
   *
   * ```typescript
   * @Component({
   *   selector: 'bank-account',
   *   template: `
   *     Bank Name: {{bankName}}
   *     Account Id: {{id}}
   *   `
   * })
   * class BankAccount {
   *   @Input() bankName: string;
   *   @Input('account-id') id: string;
   *
   *   // this property is not bound, and won't be automatically updated by Angular
   *   normalizedBankName: string;
   * }
   *
   * @Component({
   *   selector: 'app',
   *   template: `
   *     <bank-account bank-name="RBC" account-id="4747"></bank-account>
   *   `,
   *   directives: [BankAccount]
   * })
   * class App {}
   *
   * bootstrap(App);
   * ```
   * @stable
   * @Annotation
   */
  exports.Input = decorators_1.makePropDecorator(directives_1.InputMetadata);
  // TODO(alexeagle): remove the duplication of this doc. It is copied from OutputMetadata.
  /**
   * Declares an event-bound output property.
   *
   * When an output property emits an event, an event handler attached to that event
   * the template is invoked.
   *
   * `OutputMetadata` takes an optional parameter that specifies the name
   * used when instantiating a component in the template. When not provided,
   * the name of the decorated property is used.
   *
   * ### Example
   *
   * ```typescript
   * @Directive({
   *   selector: 'interval-dir',
   * })
   * class IntervalDir {
   *   @Output() everySecond = new EventEmitter();
   *   @Output('everyFiveSeconds') five5Secs = new EventEmitter();
   *
   *   constructor() {
   *     setInterval(() => this.everySecond.emit("event"), 1000);
   *     setInterval(() => this.five5Secs.emit("event"), 5000);
   *   }
   * }
   *
   * @Component({
   *   selector: 'app',
   *   template: `
   *     <interval-dir (everySecond)="everySecond()" (everyFiveSeconds)="everyFiveSeconds()">
   *     </interval-dir>
   *   `,
   *   directives: [IntervalDir]
   * })
   * class App {
   *   everySecond() { console.log('second'); }
   *   everyFiveSeconds() { console.log('five seconds'); }
   * }
   * bootstrap(App);
   * ```
   * @stable
   * @Annotation
   */
  exports.Output = decorators_1.makePropDecorator(directives_1.OutputMetadata);
  // TODO(alexeagle): remove the duplication of this doc. It is copied from HostBindingMetadata.
  /**
   * Declares a host property binding.
   *
   * Angular automatically checks host property bindings during change detection.
   * If a binding changes, it will update the host element of the directive.
   *
   * `HostBindingMetadata` takes an optional parameter that specifies the property
   * name of the host element that will be updated. When not provided,
   * the class property name is used.
   *
   * ### Example
   *
   * The following example creates a directive that sets the `valid` and `invalid` classes
   * on the DOM element that has ngModel directive on it.
   *
   * ```typescript
   * @Directive({selector: '[ngModel]'})
   * class NgModelStatus {
   *   constructor(public control:NgModel) {}
   *   @HostBinding('class.valid') get valid() { return this.control.valid; }
   *   @HostBinding('class.invalid') get invalid() { return this.control.invalid; }
   * }
   *
   * @Component({
   *   selector: 'app',
   *   template: `<input [(ngModel)]="prop">`,
   *   directives: [FORM_DIRECTIVES, NgModelStatus]
   * })
   * class App {
   *   prop;
   * }
   *
   * bootstrap(App);
   * ```
   * @stable
   * @Annotation
   */
  exports.HostBinding = decorators_1.makePropDecorator(directives_1.HostBindingMetadata);
  // TODO(alexeagle): remove the duplication of this doc. It is copied from HostListenerMetadata.
  /**
   * Declares a host listener.
   *
   * Angular will invoke the decorated method when the host element emits the specified event.
   *
   * If the decorated method returns `false`, then `preventDefault` is applied on the DOM
   * event.
   *
   * ### Example
   *
   * The following example declares a directive that attaches a click listener to the button and
   * counts clicks.
   *
   * ```typescript
   * @Directive({selector: 'button[counting]'})
   * class CountClicks {
   *   numberOfClicks = 0;
   *
   *   @HostListener('click', ['$event.target'])
   *   onClick(btn) {
   *     console.log("button", btn, "number of clicks:", this.numberOfClicks++);
   *   }
   * }
   *
   * @Component({
   *   selector: 'app',
   *   template: `<button counting>Increment</button>`,
   *   directives: [CountClicks]
   * })
   * class App {}
   *
   * bootstrap(App);
   * ```
   * @stable
   * @Annotation
   */
  exports.HostListener = decorators_1.makePropDecorator(directives_1.HostListenerMetadata);
  
});
System.registerDynamic("node_modules/@angular/core/src/util.js", ["./util/decorators"], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  // Public API for util

  var global = this || self,
      GLOBAL = global;
  var decorators_1 = $__require('./util/decorators');
  exports.Class = decorators_1.Class;
  
});
System.registerDynamic("node_modules/@angular/core/src/zone.js", ["./zone/ng_zone"], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  // Public API for Zone

  var global = this || self,
      GLOBAL = global;
  var ng_zone_1 = $__require('./zone/ng_zone');
  exports.NgZone = ng_zone_1.NgZone;
  exports.NgZoneError = ng_zone_1.NgZoneError;
  
});
System.registerDynamic("node_modules/@angular/core/src/render.js", ["./render/api"], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  // Public API for render

  var global = this || self,
      GLOBAL = global;
  var api_1 = $__require('./render/api');
  exports.RenderComponentType = api_1.RenderComponentType;
  exports.Renderer = api_1.Renderer;
  exports.RootRenderer = api_1.RootRenderer;
  
});
System.registerDynamic('node_modules/@angular/core/src/linker/compiler.js', ['../facade/exceptions', '../facade/lang'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var exceptions_1 = $__require('../facade/exceptions');
  var lang_1 = $__require('../facade/lang');
  /**
   * Low-level service for running the angular compiler duirng runtime
   * to create {@link ComponentFactory}s, which
   * can later be used to create and render a Component instance.
   * @stable
   */
  var Compiler = function () {
    function Compiler() {}
    /**
     * Loads the template and styles of a component and returns the associated `ComponentFactory`.
     */
    Compiler.prototype.compileComponentAsync = function (component) {
      throw new exceptions_1.BaseException("Runtime compiler is not loaded. Tried to compile " + lang_1.stringify(component));
    };
    /**
     * Compiles the given component. All templates have to be either inline or compiled via
     * `compileComponentAsync` before.
     */
    Compiler.prototype.compileComponentSync = function (component) {
      throw new exceptions_1.BaseException("Runtime compiler is not loaded. Tried to compile " + lang_1.stringify(component));
    };
    /**
     * Clears all caches
     */
    Compiler.prototype.clearCache = function () {};
    /**
     * Clears the cache for the given component.
     */
    Compiler.prototype.clearCacheFor = function (compType) {};
    return Compiler;
  }();
  exports.Compiler = Compiler;
  
});
System.registerDynamic('node_modules/@angular/core/src/linker/query_list.js', ['../facade/async', '../facade/collection', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var async_1 = $__require('../facade/async');
    var collection_1 = $__require('../facade/collection');
    var lang_1 = $__require('../facade/lang');
    /**
     * An unmodifiable list of items that Angular keeps up to date when the state
     * of the application changes.
     *
     * The type of object that {@link QueryMetadata} and {@link ViewQueryMetadata} provide.
     *
     * Implements an iterable interface, therefore it can be used in both ES6
     * javascript `for (var i of items)` loops as well as in Angular templates with
     * `*ngFor="let i of myList"`.
     *
     * Changes can be observed by subscribing to the changes `Observable`.
     *
     * NOTE: In the future this class will implement an `Observable` interface.
     *
     * ### Example ([live demo](http://plnkr.co/edit/RX8sJnQYl9FWuSCWme5z?p=preview))
     * ```typescript
     * @Component({...})
     * class Container {
     *   @ViewChildren(Item) items:QueryList<Item>;
     * }
     * ```
     * @stable
     */
    var QueryList = function () {
        function QueryList() {
            this._dirty = true;
            this._results = [];
            this._emitter = new async_1.EventEmitter();
        }
        Object.defineProperty(QueryList.prototype, "changes", {
            get: function () {
                return this._emitter;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(QueryList.prototype, "length", {
            get: function () {
                return this._results.length;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(QueryList.prototype, "first", {
            get: function () {
                return this._results[0];
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(QueryList.prototype, "last", {
            get: function () {
                return this._results[this.length - 1];
            },
            enumerable: true,
            configurable: true
        });
        /**
         * See
         * [Array.map](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/map)
         */
        QueryList.prototype.map = function (fn) {
            return this._results.map(fn);
        };
        /**
         * See
         * [Array.filter](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/filter)
         */
        QueryList.prototype.filter = function (fn) {
            return this._results.filter(fn);
        };
        /**
         * See
         * [Array.reduce](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/reduce)
         */
        QueryList.prototype.reduce = function (fn, init) {
            return this._results.reduce(fn, init);
        };
        /**
         * See
         * [Array.forEach](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/forEach)
         */
        QueryList.prototype.forEach = function (fn) {
            this._results.forEach(fn);
        };
        /**
         * See
         * [Array.some](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/some)
         */
        QueryList.prototype.some = function (fn) {
            return this._results.some(fn);
        };
        QueryList.prototype.toArray = function () {
            return this._results.slice();
        };
        QueryList.prototype[lang_1.getSymbolIterator()] = function () {
            return this._results[lang_1.getSymbolIterator()]();
        };
        QueryList.prototype.toString = function () {
            return this._results.toString();
        };
        QueryList.prototype.reset = function (res) {
            this._results = collection_1.ListWrapper.flatten(res);
            this._dirty = false;
        };
        QueryList.prototype.notifyOnChanges = function () {
            this._emitter.emit(this);
        };
        /** internal */
        QueryList.prototype.setDirty = function () {
            this._dirty = true;
        };
        Object.defineProperty(QueryList.prototype, "dirty", {
            /** internal */
            get: function () {
                return this._dirty;
            },
            enumerable: true,
            configurable: true
        });
        return QueryList;
    }();
    exports.QueryList = QueryList;
    
});
System.registerDynamic('node_modules/@angular/core/src/linker/systemjs_component_resolver.js', ['../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('../facade/lang');
    var _SEPARATOR = '#';
    /**
     * Component resolver that can load components lazily
     * @experimental
     */
    var SystemJsComponentResolver = function () {
        function SystemJsComponentResolver(_resolver) {
            this._resolver = _resolver;
        }
        SystemJsComponentResolver.prototype.resolveComponent = function (componentType) {
            var _this = this;
            if (lang_1.isString(componentType)) {
                var _a = componentType.split(_SEPARATOR),
                    module = _a[0],
                    component_1 = _a[1];
                if (component_1 === void 0) {
                    // Use the default export when no component is specified
                    component_1 = 'default';
                }
                return lang_1.global.System.import(module).then(function (module) {
                    return _this._resolver.resolveComponent(module[component_1]);
                });
            }
            return this._resolver.resolveComponent(componentType);
        };
        SystemJsComponentResolver.prototype.clearCache = function () {};
        return SystemJsComponentResolver;
    }();
    exports.SystemJsComponentResolver = SystemJsComponentResolver;
    var FACTORY_MODULE_SUFFIX = '.ngfactory';
    var FACTORY_CLASS_SUFFIX = 'NgFactory';
    /**
     * Component resolver that can load component factories lazily
     * @experimental
     */
    var SystemJsCmpFactoryResolver = function () {
        function SystemJsCmpFactoryResolver() {}
        SystemJsCmpFactoryResolver.prototype.resolveComponent = function (componentType) {
            if (lang_1.isString(componentType)) {
                var _a = componentType.split(_SEPARATOR),
                    module = _a[0],
                    factory_1 = _a[1];
                return lang_1.global.System.import(module + FACTORY_MODULE_SUFFIX).then(function (module) {
                    return module[factory_1 + FACTORY_CLASS_SUFFIX];
                });
            }
            return Promise.resolve(null);
        };
        SystemJsCmpFactoryResolver.prototype.clearCache = function () {};
        return SystemJsCmpFactoryResolver;
    }();
    exports.SystemJsCmpFactoryResolver = SystemJsCmpFactoryResolver;
    
});
System.registerDynamic('node_modules/@angular/core/src/linker.js', ['./linker/compiler', './linker/component_factory', './linker/component_factory_resolver', './linker/component_resolver', './linker/dynamic_component_loader', './linker/element_ref', './linker/exceptions', './linker/query_list', './linker/systemjs_component_resolver', './linker/template_ref', './linker/view_container_ref', './linker/view_ref'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  // Public API for compiler

  var global = this || self,
      GLOBAL = global;
  var compiler_1 = $__require('./linker/compiler');
  exports.Compiler = compiler_1.Compiler;
  var component_factory_1 = $__require('./linker/component_factory');
  exports.ComponentFactory = component_factory_1.ComponentFactory;
  exports.ComponentRef = component_factory_1.ComponentRef;
  var component_factory_resolver_1 = $__require('./linker/component_factory_resolver');
  exports.ComponentFactoryResolver = component_factory_resolver_1.ComponentFactoryResolver;
  exports.NoComponentFactoryError = component_factory_resolver_1.NoComponentFactoryError;
  var component_resolver_1 = $__require('./linker/component_resolver');
  exports.ComponentResolver = component_resolver_1.ComponentResolver;
  var dynamic_component_loader_1 = $__require('./linker/dynamic_component_loader');
  exports.DynamicComponentLoader = dynamic_component_loader_1.DynamicComponentLoader;
  var element_ref_1 = $__require('./linker/element_ref');
  exports.ElementRef = element_ref_1.ElementRef;
  var exceptions_1 = $__require('./linker/exceptions');
  exports.ExpressionChangedAfterItHasBeenCheckedException = exceptions_1.ExpressionChangedAfterItHasBeenCheckedException;
  var query_list_1 = $__require('./linker/query_list');
  exports.QueryList = query_list_1.QueryList;
  var systemjs_component_resolver_1 = $__require('./linker/systemjs_component_resolver');
  exports.SystemJsCmpFactoryResolver = systemjs_component_resolver_1.SystemJsCmpFactoryResolver;
  exports.SystemJsComponentResolver = systemjs_component_resolver_1.SystemJsComponentResolver;
  var template_ref_1 = $__require('./linker/template_ref');
  exports.TemplateRef = template_ref_1.TemplateRef;
  var view_container_ref_1 = $__require('./linker/view_container_ref');
  exports.ViewContainerRef = view_container_ref_1.ViewContainerRef;
  var view_ref_1 = $__require('./linker/view_ref');
  exports.EmbeddedViewRef = view_ref_1.EmbeddedViewRef;
  exports.ViewRef = view_ref_1.ViewRef;
  
});
System.registerDynamic("node_modules/@angular/core/src/change_detection.js", ["./change_detection/change_detection"], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * @module
   * @description
   * Change detection enables data binding in Angular.
   */

  var global = this || self,
      GLOBAL = global;
  var change_detection_1 = $__require('./change_detection/change_detection');
  exports.ChangeDetectionStrategy = change_detection_1.ChangeDetectionStrategy;
  exports.ChangeDetectorRef = change_detection_1.ChangeDetectorRef;
  exports.CollectionChangeRecord = change_detection_1.CollectionChangeRecord;
  exports.DefaultIterableDiffer = change_detection_1.DefaultIterableDiffer;
  exports.IterableDiffers = change_detection_1.IterableDiffers;
  exports.KeyValueChangeRecord = change_detection_1.KeyValueChangeRecord;
  exports.KeyValueDiffers = change_detection_1.KeyValueDiffers;
  exports.SimpleChange = change_detection_1.SimpleChange;
  exports.WrappedValue = change_detection_1.WrappedValue;
  
});
System.registerDynamic('node_modules/@angular/core/src/platform_directives_and_pipes.js', ['./di'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var di_1 = $__require('./di');
  /**
     A token that can be provided when bootstrapping an application to make an array of directives
    * available in every component of the application.
    *
    * ### Example
    *
    * ```typescript
    * import {PLATFORM_DIRECTIVES} from '@angular/core';
    * import {OtherDirective} from './myDirectives';
    *
    * @Component({
    *   selector: 'my-component',
    *   template: `
    *     <!-- can use other directive even though the component does not list it in `directives` -->
    *     <other-directive></other-directive>
    *   `
    * })
    * export class MyComponent {
    *   ...
    * }
    *
    * bootstrap(MyComponent, [{provide: PLATFORM_DIRECTIVES, useValue: [OtherDirective],
    multi:true}]);
    * ```
    * @stable
    */
  exports.PLATFORM_DIRECTIVES =
  /*@ts2dart_const*/new di_1.OpaqueToken('Platform Directives');
  /**
    * A token that can be provided when bootstraping an application to make an array of pipes
    * available in every component of the application.
    *
    * ### Example
    *
    * ```typescript
    * import {PLATFORM_PIPES} from '@angular/core';
    * import {OtherPipe} from './myPipe';
    *
    * @Component({
    *   selector: 'my-component',
    *   template: `
    *     {{123 | other-pipe}}
    *   `
    * })
    * export class MyComponent {
    *   ...
    * }
    *
    * bootstrap(MyComponent, [{provide: PLATFORM_PIPES, useValue: [OtherPipe], multi:true}]);
    * ```
    * @stable
    */
  exports.PLATFORM_PIPES = new di_1.OpaqueToken('Platform Pipes');
  
});
System.registerDynamic('node_modules/@angular/core/src/platform_common_providers.js', ['./application_ref', './console', './reflection/reflection', './reflection/reflector_reader', './testability/testability'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var application_ref_1 = $__require('./application_ref');
  var console_1 = $__require('./console');
  var reflection_1 = $__require('./reflection/reflection');
  var reflector_reader_1 = $__require('./reflection/reflector_reader');
  var testability_1 = $__require('./testability/testability');
  function _reflector() {
    return reflection_1.reflector;
  }
  var __unused; // prevent missing use Dart warning.
  /**
   * A default set of providers which should be included in any Angular platform.
   * @experimental
   */
  exports.PLATFORM_COMMON_PROVIDERS = [application_ref_1.PLATFORM_CORE_PROVIDERS,
  /*@ts2dart_Provider*/{ provide: reflection_1.Reflector, useFactory: _reflector, deps: [] },
  /*@ts2dart_Provider*/{ provide: reflector_reader_1.ReflectorReader, useExisting: reflection_1.Reflector }, testability_1.TestabilityRegistry, console_1.Console];
  
});
System.registerDynamic('node_modules/@angular/core/src/testability/testability.js', ['../di/decorators', '../facade/async', '../facade/collection', '../facade/exceptions', '../facade/lang', '../zone/ng_zone'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var decorators_1 = $__require('../di/decorators');
    var async_1 = $__require('../facade/async');
    var collection_1 = $__require('../facade/collection');
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    var ng_zone_1 = $__require('../zone/ng_zone');
    var Testability = function () {
        function Testability(_ngZone) {
            this._ngZone = _ngZone;
            /** @internal */
            this._pendingCount = 0;
            /** @internal */
            this._isZoneStable = true;
            /**
             * Whether any work was done since the last 'whenStable' callback. This is
             * useful to detect if this could have potentially destabilized another
             * component while it is stabilizing.
             * @internal
             */
            this._didWork = false;
            /** @internal */
            this._callbacks = [];
            this._watchAngularEvents();
        }
        /** @internal */
        Testability.prototype._watchAngularEvents = function () {
            var _this = this;
            async_1.ObservableWrapper.subscribe(this._ngZone.onUnstable, function (_) {
                _this._didWork = true;
                _this._isZoneStable = false;
            });
            this._ngZone.runOutsideAngular(function () {
                async_1.ObservableWrapper.subscribe(_this._ngZone.onStable, function (_) {
                    ng_zone_1.NgZone.assertNotInAngularZone();
                    lang_1.scheduleMicroTask(function () {
                        _this._isZoneStable = true;
                        _this._runCallbacksIfReady();
                    });
                });
            });
        };
        Testability.prototype.increasePendingRequestCount = function () {
            this._pendingCount += 1;
            this._didWork = true;
            return this._pendingCount;
        };
        Testability.prototype.decreasePendingRequestCount = function () {
            this._pendingCount -= 1;
            if (this._pendingCount < 0) {
                throw new exceptions_1.BaseException('pending async requests below zero');
            }
            this._runCallbacksIfReady();
            return this._pendingCount;
        };
        Testability.prototype.isStable = function () {
            return this._isZoneStable && this._pendingCount == 0 && !this._ngZone.hasPendingMacrotasks;
        };
        /** @internal */
        Testability.prototype._runCallbacksIfReady = function () {
            var _this = this;
            if (this.isStable()) {
                // Schedules the call backs in a new frame so that it is always async.
                lang_1.scheduleMicroTask(function () {
                    while (_this._callbacks.length !== 0) {
                        _this._callbacks.pop()(_this._didWork);
                    }
                    _this._didWork = false;
                });
            } else {
                // Not Ready
                this._didWork = true;
            }
        };
        Testability.prototype.whenStable = function (callback) {
            this._callbacks.push(callback);
            this._runCallbacksIfReady();
        };
        Testability.prototype.getPendingRequestCount = function () {
            return this._pendingCount;
        };
        Testability.prototype.findBindings = function (using, provider, exactMatch) {
            // TODO(juliemr): implement.
            return [];
        };
        Testability.prototype.findProviders = function (using, provider, exactMatch) {
            // TODO(juliemr): implement.
            return [];
        };
        /** @nocollapse */
        Testability.decorators = [{ type: decorators_1.Injectable }];
        /** @nocollapse */
        Testability.ctorParameters = [{ type: ng_zone_1.NgZone }];
        return Testability;
    }();
    exports.Testability = Testability;
    var TestabilityRegistry = function () {
        function TestabilityRegistry() {
            /** @internal */
            this._applications = new collection_1.Map();
            _testabilityGetter.addToWindow(this);
        }
        TestabilityRegistry.prototype.registerApplication = function (token, testability) {
            this._applications.set(token, testability);
        };
        TestabilityRegistry.prototype.getTestability = function (elem) {
            return this._applications.get(elem);
        };
        TestabilityRegistry.prototype.getAllTestabilities = function () {
            return collection_1.MapWrapper.values(this._applications);
        };
        TestabilityRegistry.prototype.getAllRootElements = function () {
            return collection_1.MapWrapper.keys(this._applications);
        };
        TestabilityRegistry.prototype.findTestabilityInTree = function (elem, findInAncestors) {
            if (findInAncestors === void 0) {
                findInAncestors = true;
            }
            return _testabilityGetter.findTestabilityInTree(this, elem, findInAncestors);
        };
        /** @nocollapse */
        TestabilityRegistry.decorators = [{ type: decorators_1.Injectable }];
        /** @nocollapse */
        TestabilityRegistry.ctorParameters = [];
        return TestabilityRegistry;
    }();
    exports.TestabilityRegistry = TestabilityRegistry;
    /* @ts2dart_const */
    var _NoopGetTestability = function () {
        function _NoopGetTestability() {}
        _NoopGetTestability.prototype.addToWindow = function (registry) {};
        _NoopGetTestability.prototype.findTestabilityInTree = function (registry, elem, findInAncestors) {
            return null;
        };
        return _NoopGetTestability;
    }();
    /**
     * Set the {@link GetTestability} implementation used by the Angular testing framework.
     * @experimental
     */
    function setTestabilityGetter(getter) {
        _testabilityGetter = getter;
    }
    exports.setTestabilityGetter = setTestabilityGetter;
    var _testabilityGetter = new _NoopGetTestability();
    
});
System.registerDynamic('node_modules/@angular/core/src/zone/ng_zone_impl.js', [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";
    /**
     * Stores error information; delivered via [NgZone.onError] stream.
     * @deprecated
     */

    var global = this || self,
        GLOBAL = global;
    var NgZoneError = function () {
        function NgZoneError(error, stackTrace) {
            this.error = error;
            this.stackTrace = stackTrace;
        }
        return NgZoneError;
    }();
    exports.NgZoneError = NgZoneError;
    var NgZoneImpl = function () {
        function NgZoneImpl(_a) {
            var _this = this;
            var trace = _a.trace,
                onEnter = _a.onEnter,
                onLeave = _a.onLeave,
                setMicrotask = _a.setMicrotask,
                setMacrotask = _a.setMacrotask,
                onError = _a.onError;
            this.onEnter = onEnter;
            this.onLeave = onLeave;
            this.setMicrotask = setMicrotask;
            this.setMacrotask = setMacrotask;
            this.onError = onError;
            if (Zone) {
                this.outer = this.inner = Zone.current;
                if (Zone['wtfZoneSpec']) {
                    this.inner = this.inner.fork(Zone['wtfZoneSpec']);
                }
                if (trace && Zone['longStackTraceZoneSpec']) {
                    this.inner = this.inner.fork(Zone['longStackTraceZoneSpec']);
                }
                this.inner = this.inner.fork({
                    name: 'angular',
                    properties: { 'isAngularZone': true },
                    onInvokeTask: function (delegate, current, target, task, applyThis, applyArgs) {
                        try {
                            _this.onEnter();
                            return delegate.invokeTask(target, task, applyThis, applyArgs);
                        } finally {
                            _this.onLeave();
                        }
                    },
                    onInvoke: function (delegate, current, target, callback, applyThis, applyArgs, source) {
                        try {
                            _this.onEnter();
                            return delegate.invoke(target, callback, applyThis, applyArgs, source);
                        } finally {
                            _this.onLeave();
                        }
                    },
                    onHasTask: function (delegate, current, target, hasTaskState) {
                        delegate.hasTask(target, hasTaskState);
                        if (current == target) {
                            // We are only interested in hasTask events which originate from our zone
                            // (A child hasTask event is not interesting to us)
                            if (hasTaskState.change == 'microTask') {
                                _this.setMicrotask(hasTaskState.microTask);
                            } else if (hasTaskState.change == 'macroTask') {
                                _this.setMacrotask(hasTaskState.macroTask);
                            }
                        }
                    },
                    onHandleError: function (delegate, current, target, error) {
                        delegate.handleError(target, error);
                        _this.onError(new NgZoneError(error, error.stack));
                        return false;
                    }
                });
            } else {
                throw new Error('Angular requires Zone.js polyfill.');
            }
        }
        NgZoneImpl.isInAngularZone = function () {
            return Zone.current.get('isAngularZone') === true;
        };
        NgZoneImpl.prototype.runInner = function (fn) {
            return this.inner.run(fn);
        };
        ;
        NgZoneImpl.prototype.runInnerGuarded = function (fn) {
            return this.inner.runGuarded(fn);
        };
        ;
        NgZoneImpl.prototype.runOuter = function (fn) {
            return this.outer.run(fn);
        };
        ;
        return NgZoneImpl;
    }();
    exports.NgZoneImpl = NgZoneImpl;
    
});
System.registerDynamic('node_modules/@angular/core/src/zone/ng_zone.js', ['../facade/async', '../facade/exceptions', './ng_zone_impl'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var async_1 = $__require('../facade/async');
    var exceptions_1 = $__require('../facade/exceptions');
    var ng_zone_impl_1 = $__require('./ng_zone_impl');
    var ng_zone_impl_2 = $__require('./ng_zone_impl');
    exports.NgZoneError = ng_zone_impl_2.NgZoneError;
    /**
     * An injectable service for executing work inside or outside of the Angular zone.
     *
     * The most common use of this service is to optimize performance when starting a work consisting of
     * one or more asynchronous tasks that don't require UI updates or error handling to be handled by
     * Angular. Such tasks can be kicked off via {@link #runOutsideAngular} and if needed, these tasks
     * can reenter the Angular zone via {@link #run}.
     *
     * <!-- TODO: add/fix links to:
     *   - docs explaining zones and the use of zones in Angular and change-detection
     *   - link to runOutsideAngular/run (throughout this file!)
     *   -->
     *
     * ### Example ([live demo](http://plnkr.co/edit/lY9m8HLy7z06vDoUaSN2?p=preview))
     * ```
     * import {Component, View, NgZone} from '@angular/core';
     * import {NgIf} from '@angular/common';
     *
     * @Component({
     *   selector: 'ng-zone-demo'.
     *   template: `
     *     <h2>Demo: NgZone</h2>
     *
     *     <p>Progress: {{progress}}%</p>
     *     <p *ngIf="progress >= 100">Done processing {{label}} of Angular zone!</p>
     *
     *     <button (click)="processWithinAngularZone()">Process within Angular zone</button>
     *     <button (click)="processOutsideOfAngularZone()">Process outside of Angular zone</button>
     *   `,
     *   directives: [NgIf]
     * })
     * export class NgZoneDemo {
     *   progress: number = 0;
     *   label: string;
     *
     *   constructor(private _ngZone: NgZone) {}
     *
     *   // Loop inside the Angular zone
     *   // so the UI DOES refresh after each setTimeout cycle
     *   processWithinAngularZone() {
     *     this.label = 'inside';
     *     this.progress = 0;
     *     this._increaseProgress(() => console.log('Inside Done!'));
     *   }
     *
     *   // Loop outside of the Angular zone
     *   // so the UI DOES NOT refresh after each setTimeout cycle
     *   processOutsideOfAngularZone() {
     *     this.label = 'outside';
     *     this.progress = 0;
     *     this._ngZone.runOutsideAngular(() => {
     *       this._increaseProgress(() => {
     *       // reenter the Angular zone and display done
     *       this._ngZone.run(() => {console.log('Outside Done!') });
     *     }}));
     *   }
     *
     *
     *   _increaseProgress(doneCallback: () => void) {
     *     this.progress += 1;
     *     console.log(`Current progress: ${this.progress}%`);
     *
     *     if (this.progress < 100) {
     *       window.setTimeout(() => this._increaseProgress(doneCallback)), 10)
     *     } else {
     *       doneCallback();
     *     }
     *   }
     * }
     * ```
     * @experimental
     */
    var NgZone = function () {
        function NgZone(_a) {
            var _this = this;
            var _b = _a.enableLongStackTrace,
                enableLongStackTrace = _b === void 0 ? false : _b;
            this._hasPendingMicrotasks = false;
            this._hasPendingMacrotasks = false;
            /** @internal */
            this._isStable = true;
            /** @internal */
            this._nesting = 0;
            /** @internal */
            this._onUnstable = new async_1.EventEmitter(false);
            /** @internal */
            this._onMicrotaskEmpty = new async_1.EventEmitter(false);
            /** @internal */
            this._onStable = new async_1.EventEmitter(false);
            /** @internal */
            this._onErrorEvents = new async_1.EventEmitter(false);
            this._zoneImpl = new ng_zone_impl_1.NgZoneImpl({
                trace: enableLongStackTrace,
                onEnter: function () {
                    // console.log('ZONE.enter', this._nesting, this._isStable);
                    _this._nesting++;
                    if (_this._isStable) {
                        _this._isStable = false;
                        _this._onUnstable.emit(null);
                    }
                },
                onLeave: function () {
                    _this._nesting--;
                    // console.log('ZONE.leave', this._nesting, this._isStable);
                    _this._checkStable();
                },
                setMicrotask: function (hasMicrotasks) {
                    _this._hasPendingMicrotasks = hasMicrotasks;
                    _this._checkStable();
                },
                setMacrotask: function (hasMacrotasks) {
                    _this._hasPendingMacrotasks = hasMacrotasks;
                },
                onError: function (error) {
                    return _this._onErrorEvents.emit(error);
                }
            });
        }
        NgZone.isInAngularZone = function () {
            return ng_zone_impl_1.NgZoneImpl.isInAngularZone();
        };
        NgZone.assertInAngularZone = function () {
            if (!ng_zone_impl_1.NgZoneImpl.isInAngularZone()) {
                throw new exceptions_1.BaseException('Expected to be in Angular Zone, but it is not!');
            }
        };
        NgZone.assertNotInAngularZone = function () {
            if (ng_zone_impl_1.NgZoneImpl.isInAngularZone()) {
                throw new exceptions_1.BaseException('Expected to not be in Angular Zone, but it is!');
            }
        };
        NgZone.prototype._checkStable = function () {
            var _this = this;
            if (this._nesting == 0) {
                if (!this._hasPendingMicrotasks && !this._isStable) {
                    try {
                        // console.log('ZONE.microtaskEmpty');
                        this._nesting++;
                        this._onMicrotaskEmpty.emit(null);
                    } finally {
                        this._nesting--;
                        if (!this._hasPendingMicrotasks) {
                            try {
                                // console.log('ZONE.stable', this._nesting, this._isStable);
                                this.runOutsideAngular(function () {
                                    return _this._onStable.emit(null);
                                });
                            } finally {
                                this._isStable = true;
                            }
                        }
                    }
                }
            }
        };
        ;
        Object.defineProperty(NgZone.prototype, "onUnstable", {
            /**
             * Notifies when code enters Angular Zone. This gets fired first on VM Turn.
             */
            get: function () {
                return this._onUnstable;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgZone.prototype, "onMicrotaskEmpty", {
            /**
             * Notifies when there is no more microtasks enqueue in the current VM Turn.
             * This is a hint for Angular to do change detection, which may enqueue more microtasks.
             * For this reason this event can fire multiple times per VM Turn.
             */
            get: function () {
                return this._onMicrotaskEmpty;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgZone.prototype, "onStable", {
            /**
             * Notifies when the last `onMicrotaskEmpty` has run and there are no more microtasks, which
             * implies we are about to relinquish VM turn.
             * This event gets called just once.
             */
            get: function () {
                return this._onStable;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgZone.prototype, "onError", {
            /**
             * Notify that an error has been delivered.
             */
            get: function () {
                return this._onErrorEvents;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgZone.prototype, "isStable", {
            /**
             * Whether there are no outstanding microtasks or microtasks.
             */
            get: function () {
                return this._isStable;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgZone.prototype, "hasPendingMicrotasks", {
            /**
             * Whether there are any outstanding microtasks.
             */
            get: function () {
                return this._hasPendingMicrotasks;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(NgZone.prototype, "hasPendingMacrotasks", {
            /**
             * Whether there are any outstanding microtasks.
             */
            get: function () {
                return this._hasPendingMacrotasks;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Executes the `fn` function synchronously within the Angular zone and returns value returned by
         * the function.
         *
         * Running functions via `run` allows you to reenter Angular zone from a task that was executed
         * outside of the Angular zone (typically started via {@link #runOutsideAngular}).
         *
         * Any future tasks or microtasks scheduled from within this function will continue executing from
         * within the Angular zone.
         *
         * If a synchronous error happens it will be rethrown and not reported via `onError`.
         */
        NgZone.prototype.run = function (fn) {
            return this._zoneImpl.runInner(fn);
        };
        /**
         * Same as #run, except that synchronous errors are caught and forwarded
         * via `onError` and not rethrown.
         */
        NgZone.prototype.runGuarded = function (fn) {
            return this._zoneImpl.runInnerGuarded(fn);
        };
        /**
         * Executes the `fn` function synchronously in Angular's parent zone and returns value returned by
         * the function.
         *
         * Running functions via `runOutsideAngular` allows you to escape Angular's zone and do work that
         * doesn't trigger Angular change-detection or is subject to Angular's error handling.
         *
         * Any future tasks or microtasks scheduled from within this function will continue executing from
         * outside of the Angular zone.
         *
         * Use {@link #run} to reenter the Angular zone and do work that updates the application model.
         */
        NgZone.prototype.runOutsideAngular = function (fn) {
            return this._zoneImpl.runOuter(fn);
        };
        return NgZone;
    }();
    exports.NgZone = NgZone;
    
});
System.registerDynamic('node_modules/@angular/core/src/application_ref.js', ['../src/facade/async', '../src/facade/collection', '../src/facade/exceptions', '../src/facade/lang', './application_tokens', './console', './di', './linker/component_resolver', './profile/profile', './testability/testability', './zone/ng_zone'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var async_1 = $__require('../src/facade/async');
    var collection_1 = $__require('../src/facade/collection');
    var exceptions_1 = $__require('../src/facade/exceptions');
    var lang_1 = $__require('../src/facade/lang');
    var application_tokens_1 = $__require('./application_tokens');
    var console_1 = $__require('./console');
    var di_1 = $__require('./di');
    var component_resolver_1 = $__require('./linker/component_resolver');
    var profile_1 = $__require('./profile/profile');
    var testability_1 = $__require('./testability/testability');
    var ng_zone_1 = $__require('./zone/ng_zone');
    /**
     * Create an Angular zone.
     * @experimental
     */
    function createNgZone() {
        return new ng_zone_1.NgZone({ enableLongStackTrace: isDevMode() });
    }
    exports.createNgZone = createNgZone;
    var _devMode = true;
    var _runModeLocked = false;
    var _platform;
    var _inPlatformCreate = false;
    /**
     * Disable Angular's development mode, which turns off assertions and other
     * checks within the framework.
     *
     * One important assertion this disables verifies that a change detection pass
     * does not result in additional changes to any bindings (also known as
     * unidirectional data flow).
     *
     * @experimental APIs related to application bootstrap are currently under review.
     */
    function enableProdMode() {
        if (_runModeLocked) {
            // Cannot use BaseException as that ends up importing from facade/lang.
            throw new exceptions_1.BaseException('Cannot enable prod mode after platform setup.');
        }
        _devMode = false;
    }
    exports.enableProdMode = enableProdMode;
    /**
     * Returns whether Angular is in development mode.
     * This can only be read after `lockRunMode` has been called.
     *
     * By default, this is true, unless a user calls `enableProdMode`.
     *
     * @experimental APIs related to application bootstrap are currently under review.
     */
    function isDevMode() {
        if (!_runModeLocked) {
            throw new exceptions_1.BaseException("Dev mode can't be read before bootstrap!");
        }
        return _devMode;
    }
    exports.isDevMode = isDevMode;
    /**
     * Locks the run mode of Angular. After this has been called,
     * it can't be changed any more. I.e. `isDevMode()` will always
     * return the same value.
     *
     * @experimental APIs related to application bootstrap are currently under review.
     */
    function lockRunMode() {
        _runModeLocked = true;
    }
    exports.lockRunMode = lockRunMode;
    /**
     * Creates a platform.
     * Platforms have to be eagerly created via this function.
     *
     * @experimental APIs related to application bootstrap are currently under review.
     */
    function createPlatform(injector) {
        if (_inPlatformCreate) {
            throw new exceptions_1.BaseException('Already creating a platform...');
        }
        if (lang_1.isPresent(_platform) && !_platform.disposed) {
            throw new exceptions_1.BaseException('There can be only one platform. Destroy the previous one to create a new one.');
        }
        lockRunMode();
        _inPlatformCreate = true;
        try {
            _platform = injector.get(PlatformRef);
        } finally {
            _inPlatformCreate = false;
        }
        return _platform;
    }
    exports.createPlatform = createPlatform;
    /**
     * Checks that there currently is a platform
     * which contains the given token as a provider.
     *
     * @experimental APIs related to application bootstrap are currently under review.
     */
    function assertPlatform(requiredToken) {
        var platform = getPlatform();
        if (lang_1.isBlank(platform)) {
            throw new exceptions_1.BaseException('No platform exists!');
        }
        if (lang_1.isPresent(platform) && lang_1.isBlank(platform.injector.get(requiredToken, null))) {
            throw new exceptions_1.BaseException('A platform with a different configuration has been created. Please destroy it first.');
        }
        return platform;
    }
    exports.assertPlatform = assertPlatform;
    /**
     * Dispose the existing platform.
     *
     * @experimental APIs related to application bootstrap are currently under review.
     */
    function disposePlatform() {
        if (lang_1.isPresent(_platform) && !_platform.disposed) {
            _platform.dispose();
        }
    }
    exports.disposePlatform = disposePlatform;
    /**
     * Returns the current platform.
     *
     * @experimental APIs related to application bootstrap are currently under review.
     */
    function getPlatform() {
        return lang_1.isPresent(_platform) && !_platform.disposed ? _platform : null;
    }
    exports.getPlatform = getPlatform;
    /**
     * Shortcut for ApplicationRef.bootstrap.
     * Requires a platform to be created first.
     *
     * @experimental APIs related to application bootstrap are currently under review.
     */
    function coreBootstrap(componentFactory, injector) {
        var appRef = injector.get(ApplicationRef);
        return appRef.bootstrap(componentFactory);
    }
    exports.coreBootstrap = coreBootstrap;
    /**
     * Resolves the componentFactory for the given component,
     * waits for asynchronous initializers and bootstraps the component.
     * Requires a platform to be created first.
     *
     * @experimental APIs related to application bootstrap are currently under review.
     */
    function coreLoadAndBootstrap(componentType, injector) {
        var appRef = injector.get(ApplicationRef);
        return appRef.run(function () {
            var componentResolver = injector.get(component_resolver_1.ComponentResolver);
            return async_1.PromiseWrapper.all([componentResolver.resolveComponent(componentType), appRef.waitForAsyncInitializers()]).then(function (arr) {
                return appRef.bootstrap(arr[0]);
            });
        });
    }
    exports.coreLoadAndBootstrap = coreLoadAndBootstrap;
    /**
     * The Angular platform is the entry point for Angular on a web page. Each page
     * has exactly one platform, and services (such as reflection) which are common
     * to every Angular application running on the page are bound in its scope.
     *
     * A page's platform is initialized implicitly when {@link bootstrap}() is called, or
     * explicitly by calling {@link createPlatform}().
     *
     * @experimental APIs related to application bootstrap are currently under review.
     */
    var PlatformRef = function () {
        function PlatformRef() {}
        Object.defineProperty(PlatformRef.prototype, "injector", {
            /**
             * Retrieve the platform {@link Injector}, which is the parent injector for
             * every Angular application on the page and provides singleton providers.
             */
            get: function () {
                throw exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(PlatformRef.prototype, "disposed", {
            get: function () {
                throw exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        return PlatformRef;
    }();
    exports.PlatformRef = PlatformRef;
    var PlatformRef_ = function (_super) {
        __extends(PlatformRef_, _super);
        function PlatformRef_(_injector) {
            _super.call(this);
            this._injector = _injector;
            /** @internal */
            this._applications = [];
            /** @internal */
            this._disposeListeners = [];
            this._disposed = false;
            if (!_inPlatformCreate) {
                throw new exceptions_1.BaseException('Platforms have to be created via `createPlatform`!');
            }
            var inits = _injector.get(application_tokens_1.PLATFORM_INITIALIZER, null);
            if (lang_1.isPresent(inits)) inits.forEach(function (init) {
                return init();
            });
        }
        PlatformRef_.prototype.registerDisposeListener = function (dispose) {
            this._disposeListeners.push(dispose);
        };
        Object.defineProperty(PlatformRef_.prototype, "injector", {
            get: function () {
                return this._injector;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(PlatformRef_.prototype, "disposed", {
            get: function () {
                return this._disposed;
            },
            enumerable: true,
            configurable: true
        });
        PlatformRef_.prototype.addApplication = function (appRef) {
            this._applications.push(appRef);
        };
        PlatformRef_.prototype.dispose = function () {
            collection_1.ListWrapper.clone(this._applications).forEach(function (app) {
                return app.dispose();
            });
            this._disposeListeners.forEach(function (dispose) {
                return dispose();
            });
            this._disposed = true;
        };
        /** @internal */
        PlatformRef_.prototype._applicationDisposed = function (app) {
            collection_1.ListWrapper.remove(this._applications, app);
        };
        /** @nocollapse */
        PlatformRef_.decorators = [{ type: di_1.Injectable }];
        /** @nocollapse */
        PlatformRef_.ctorParameters = [{ type: di_1.Injector }];
        return PlatformRef_;
    }(PlatformRef);
    exports.PlatformRef_ = PlatformRef_;
    /**
     * A reference to an Angular application running on a page.
     *
     * For more about Angular applications, see the documentation for {@link bootstrap}.
     *
     * @experimental APIs related to application bootstrap are currently under review.
     */
    var ApplicationRef = function () {
        function ApplicationRef() {}
        Object.defineProperty(ApplicationRef.prototype, "injector", {
            /**
             * Retrieve the application {@link Injector}.
             */
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(ApplicationRef.prototype, "zone", {
            /**
             * Retrieve the application {@link NgZone}.
             */
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(ApplicationRef.prototype, "componentTypes", {
            /**
             * Get a list of component types registered to this application.
             */
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        ;
        return ApplicationRef;
    }();
    exports.ApplicationRef = ApplicationRef;
    var ApplicationRef_ = function (_super) {
        __extends(ApplicationRef_, _super);
        function ApplicationRef_(_platform, _zone, _injector) {
            var _this = this;
            _super.call(this);
            this._platform = _platform;
            this._zone = _zone;
            this._injector = _injector;
            /** @internal */
            this._bootstrapListeners = [];
            /** @internal */
            this._disposeListeners = [];
            /** @internal */
            this._rootComponents = [];
            /** @internal */
            this._rootComponentTypes = [];
            /** @internal */
            this._changeDetectorRefs = [];
            /** @internal */
            this._runningTick = false;
            /** @internal */
            this._enforceNoNewChanges = false;
            var zone = _injector.get(ng_zone_1.NgZone);
            this._enforceNoNewChanges = isDevMode();
            zone.run(function () {
                _this._exceptionHandler = _injector.get(exceptions_1.ExceptionHandler);
            });
            this._asyncInitDonePromise = this.run(function () {
                var inits = _injector.get(application_tokens_1.APP_INITIALIZER, null);
                var asyncInitResults = [];
                var asyncInitDonePromise;
                if (lang_1.isPresent(inits)) {
                    for (var i = 0; i < inits.length; i++) {
                        var initResult = inits[i]();
                        if (lang_1.isPromise(initResult)) {
                            asyncInitResults.push(initResult);
                        }
                    }
                }
                if (asyncInitResults.length > 0) {
                    asyncInitDonePromise = async_1.PromiseWrapper.all(asyncInitResults).then(function (_) {
                        return _this._asyncInitDone = true;
                    });
                    _this._asyncInitDone = false;
                } else {
                    _this._asyncInitDone = true;
                    asyncInitDonePromise = async_1.PromiseWrapper.resolve(true);
                }
                return asyncInitDonePromise;
            });
            async_1.ObservableWrapper.subscribe(zone.onError, function (error) {
                _this._exceptionHandler.call(error.error, error.stackTrace);
            });
            async_1.ObservableWrapper.subscribe(this._zone.onMicrotaskEmpty, function (_) {
                _this._zone.run(function () {
                    _this.tick();
                });
            });
        }
        ApplicationRef_.prototype.registerBootstrapListener = function (listener) {
            this._bootstrapListeners.push(listener);
        };
        ApplicationRef_.prototype.registerDisposeListener = function (dispose) {
            this._disposeListeners.push(dispose);
        };
        ApplicationRef_.prototype.registerChangeDetector = function (changeDetector) {
            this._changeDetectorRefs.push(changeDetector);
        };
        ApplicationRef_.prototype.unregisterChangeDetector = function (changeDetector) {
            collection_1.ListWrapper.remove(this._changeDetectorRefs, changeDetector);
        };
        ApplicationRef_.prototype.waitForAsyncInitializers = function () {
            return this._asyncInitDonePromise;
        };
        ApplicationRef_.prototype.run = function (callback) {
            var _this = this;
            var zone = this.injector.get(ng_zone_1.NgZone);
            var result;
            // Note: Don't use zone.runGuarded as we want to know about
            // the thrown exception!
            // Note: the completer needs to be created outside
            // of `zone.run` as Dart swallows rejected promises
            // via the onError callback of the promise.
            var completer = async_1.PromiseWrapper.completer();
            zone.run(function () {
                try {
                    result = callback();
                    if (lang_1.isPromise(result)) {
                        async_1.PromiseWrapper.then(result, function (ref) {
                            completer.resolve(ref);
                        }, function (err, stackTrace) {
                            completer.reject(err, stackTrace);
                            _this._exceptionHandler.call(err, stackTrace);
                        });
                    }
                } catch (e) {
                    _this._exceptionHandler.call(e, e.stack);
                    throw e;
                }
            });
            return lang_1.isPromise(result) ? completer.promise : result;
        };
        ApplicationRef_.prototype.bootstrap = function (componentFactory) {
            var _this = this;
            if (!this._asyncInitDone) {
                throw new exceptions_1.BaseException('Cannot bootstrap as there are still asynchronous initializers running. Wait for them using waitForAsyncInitializers().');
            }
            return this.run(function () {
                _this._rootComponentTypes.push(componentFactory.componentType);
                var compRef = componentFactory.create(_this._injector, [], componentFactory.selector);
                compRef.onDestroy(function () {
                    _this._unloadComponent(compRef);
                });
                var testability = compRef.injector.get(testability_1.Testability, null);
                if (lang_1.isPresent(testability)) {
                    compRef.injector.get(testability_1.TestabilityRegistry).registerApplication(compRef.location.nativeElement, testability);
                }
                _this._loadComponent(compRef);
                var c = _this._injector.get(console_1.Console);
                if (isDevMode()) {
                    var prodDescription = lang_1.IS_DART ? 'Production mode is disabled in Dart.' : 'Call enableProdMode() to enable the production mode.';
                    c.log("Angular 2 is running in the development mode. " + prodDescription);
                }
                return compRef;
            });
        };
        /** @internal */
        ApplicationRef_.prototype._loadComponent = function (componentRef) {
            this._changeDetectorRefs.push(componentRef.changeDetectorRef);
            this.tick();
            this._rootComponents.push(componentRef);
            this._bootstrapListeners.forEach(function (listener) {
                return listener(componentRef);
            });
        };
        /** @internal */
        ApplicationRef_.prototype._unloadComponent = function (componentRef) {
            if (!collection_1.ListWrapper.contains(this._rootComponents, componentRef)) {
                return;
            }
            this.unregisterChangeDetector(componentRef.changeDetectorRef);
            collection_1.ListWrapper.remove(this._rootComponents, componentRef);
        };
        Object.defineProperty(ApplicationRef_.prototype, "injector", {
            get: function () {
                return this._injector;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ApplicationRef_.prototype, "zone", {
            get: function () {
                return this._zone;
            },
            enumerable: true,
            configurable: true
        });
        ApplicationRef_.prototype.tick = function () {
            if (this._runningTick) {
                throw new exceptions_1.BaseException('ApplicationRef.tick is called recursively');
            }
            var s = ApplicationRef_._tickScope();
            try {
                this._runningTick = true;
                this._changeDetectorRefs.forEach(function (detector) {
                    return detector.detectChanges();
                });
                if (this._enforceNoNewChanges) {
                    this._changeDetectorRefs.forEach(function (detector) {
                        return detector.checkNoChanges();
                    });
                }
            } finally {
                this._runningTick = false;
                profile_1.wtfLeave(s);
            }
        };
        ApplicationRef_.prototype.dispose = function () {
            // TODO(alxhub): Dispose of the NgZone.
            collection_1.ListWrapper.clone(this._rootComponents).forEach(function (ref) {
                return ref.destroy();
            });
            this._disposeListeners.forEach(function (dispose) {
                return dispose();
            });
            this._platform._applicationDisposed(this);
        };
        Object.defineProperty(ApplicationRef_.prototype, "componentTypes", {
            get: function () {
                return this._rootComponentTypes;
            },
            enumerable: true,
            configurable: true
        });
        /** @internal */
        ApplicationRef_._tickScope = profile_1.wtfCreateScope('ApplicationRef#tick()');
        /** @nocollapse */
        ApplicationRef_.decorators = [{ type: di_1.Injectable }];
        /** @nocollapse */
        ApplicationRef_.ctorParameters = [{ type: PlatformRef_ }, { type: ng_zone_1.NgZone }, { type: di_1.Injector }];
        return ApplicationRef_;
    }(ApplicationRef);
    exports.ApplicationRef_ = ApplicationRef_;
    exports.PLATFORM_CORE_PROVIDERS =
    /*@ts2dart_const*/[PlatformRef_,
    /*@ts2dart_const*/
    /* @ts2dart_Provider */{ provide: PlatformRef, useExisting: PlatformRef_ }];
    exports.APPLICATION_CORE_PROVIDERS = [
    /* @ts2dart_Provider */{ provide: ng_zone_1.NgZone, useFactory: createNgZone, deps: [] }, ApplicationRef_,
    /* @ts2dart_Provider */{ provide: ApplicationRef, useExisting: ApplicationRef_ }];
    
});
System.registerDynamic('node_modules/@angular/core/src/linker/dynamic_component_loader.js', ['../di/decorators', '../di/reflective_injector', '../facade/lang', './component_resolver'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var decorators_1 = $__require('../di/decorators');
    var reflective_injector_1 = $__require('../di/reflective_injector');
    var lang_1 = $__require('../facade/lang');
    var component_resolver_1 = $__require('./component_resolver');
    /**
     * Use ComponentResolver and ViewContainerRef directly.
     *
     * @deprecated
     */
    var DynamicComponentLoader = function () {
        function DynamicComponentLoader() {}
        return DynamicComponentLoader;
    }();
    exports.DynamicComponentLoader = DynamicComponentLoader;
    var DynamicComponentLoader_ = function (_super) {
        __extends(DynamicComponentLoader_, _super);
        function DynamicComponentLoader_(_compiler) {
            _super.call(this);
            this._compiler = _compiler;
        }
        DynamicComponentLoader_.prototype.loadAsRoot = function (type, overrideSelectorOrNode, injector, onDispose, projectableNodes) {
            return this._compiler.resolveComponent(type).then(function (componentFactory) {
                var componentRef = componentFactory.create(injector, projectableNodes, lang_1.isPresent(overrideSelectorOrNode) ? overrideSelectorOrNode : componentFactory.selector);
                if (lang_1.isPresent(onDispose)) {
                    componentRef.onDestroy(onDispose);
                }
                return componentRef;
            });
        };
        DynamicComponentLoader_.prototype.loadNextToLocation = function (type, location, providers, projectableNodes) {
            if (providers === void 0) {
                providers = null;
            }
            if (projectableNodes === void 0) {
                projectableNodes = null;
            }
            return this._compiler.resolveComponent(type).then(function (componentFactory) {
                var contextInjector = location.parentInjector;
                var childInjector = lang_1.isPresent(providers) && providers.length > 0 ? reflective_injector_1.ReflectiveInjector.fromResolvedProviders(providers, contextInjector) : contextInjector;
                return location.createComponent(componentFactory, location.length, childInjector, projectableNodes);
            });
        };
        /** @nocollapse */
        DynamicComponentLoader_.decorators = [{ type: decorators_1.Injectable }];
        /** @nocollapse */
        DynamicComponentLoader_.ctorParameters = [{ type: component_resolver_1.ComponentResolver }];
        return DynamicComponentLoader_;
    }(DynamicComponentLoader);
    exports.DynamicComponentLoader_ = DynamicComponentLoader_;
    
});
System.registerDynamic('node_modules/@angular/core/src/application_common_providers.js', ['./application_ref', './application_tokens', './change_detection/change_detection', './linker/component_factory_resolver', './linker/component_resolver', './linker/dynamic_component_loader', './linker/view_utils'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var application_ref_1 = $__require('./application_ref');
  var application_tokens_1 = $__require('./application_tokens');
  var change_detection_1 = $__require('./change_detection/change_detection');
  var component_factory_resolver_1 = $__require('./linker/component_factory_resolver');
  var component_resolver_1 = $__require('./linker/component_resolver');
  var dynamic_component_loader_1 = $__require('./linker/dynamic_component_loader');
  var view_utils_1 = $__require('./linker/view_utils');
  var __unused; // avoid unused import when Type union types are erased
  /**
   * A default set of providers which should be included in any Angular
   * application, regardless of the platform it runs onto.
   * @stable
   */
  exports.APPLICATION_COMMON_PROVIDERS =
  /*@ts2dart_const*/[application_ref_1.APPLICATION_CORE_PROVIDERS,
  /* @ts2dart_Provider */{ provide: component_resolver_1.ComponentResolver, useClass: component_resolver_1.ReflectorComponentResolver }, { provide: component_factory_resolver_1.ComponentFactoryResolver, useValue: component_factory_resolver_1.ComponentFactoryResolver.NULL }, application_tokens_1.APP_ID_RANDOM_PROVIDER, view_utils_1.ViewUtils,
  /* @ts2dart_Provider */{ provide: change_detection_1.IterableDiffers, useValue: change_detection_1.defaultIterableDiffers },
  /* @ts2dart_Provider */{ provide: change_detection_1.KeyValueDiffers, useValue: change_detection_1.defaultKeyValueDiffers },
  /* @ts2dart_Provider */{ provide: dynamic_component_loader_1.DynamicComponentLoader, useClass: dynamic_component_loader_1.DynamicComponentLoader_ }];
  
});
System.registerDynamic("node_modules/@angular/core/src/animation/animation_driver.js", ["./animation_player"], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var animation_player_1 = $__require('./animation_player');
    var AnimationDriver = function () {
        function AnimationDriver() {}
        return AnimationDriver;
    }();
    exports.AnimationDriver = AnimationDriver;
    var NoOpAnimationDriver = function (_super) {
        __extends(NoOpAnimationDriver, _super);
        function NoOpAnimationDriver() {
            _super.apply(this, arguments);
        }
        NoOpAnimationDriver.prototype.animate = function (element, startingStyles, keyframes, duration, delay, easing) {
            return new animation_player_1.NoOpAnimationPlayer();
        };
        return NoOpAnimationDriver;
    }(AnimationDriver);
    exports.NoOpAnimationDriver = NoOpAnimationDriver;
    
});
System.registerDynamic("node_modules/@angular/core/src/animation/animation_keyframe.js", [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var AnimationKeyframe = function () {
        function AnimationKeyframe(offset, styles) {
            this.offset = offset;
            this.styles = styles;
        }
        return AnimationKeyframe;
    }();
    exports.AnimationKeyframe = AnimationKeyframe;
    
});
System.registerDynamic('node_modules/@angular/core/src/animation/animation_sequence_player.js', ['../facade/lang', './animation_player'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('../facade/lang');
    var animation_player_1 = $__require('./animation_player');
    var AnimationSequencePlayer = function () {
        function AnimationSequencePlayer(_players) {
            var _this = this;
            this._players = _players;
            this._currentIndex = 0;
            this._subscriptions = [];
            this._finished = false;
            this.parentPlayer = null;
            this._players.forEach(function (player) {
                player.parentPlayer = _this;
            });
            this._onNext(false);
        }
        AnimationSequencePlayer.prototype._onNext = function (start) {
            var _this = this;
            if (this._finished) return;
            if (this._players.length == 0) {
                this._activePlayer = new animation_player_1.NoOpAnimationPlayer();
                lang_1.scheduleMicroTask(function () {
                    return _this._onFinish();
                });
            } else if (this._currentIndex >= this._players.length) {
                this._activePlayer = new animation_player_1.NoOpAnimationPlayer();
                this._onFinish();
            } else {
                var player = this._players[this._currentIndex++];
                player.onDone(function () {
                    return _this._onNext(true);
                });
                this._activePlayer = player;
                if (start) {
                    player.play();
                }
            }
        };
        AnimationSequencePlayer.prototype._onFinish = function () {
            if (!this._finished) {
                this._finished = true;
                if (!lang_1.isPresent(this.parentPlayer)) {
                    this.destroy();
                }
                this._subscriptions.forEach(function (subscription) {
                    return subscription();
                });
                this._subscriptions = [];
            }
        };
        AnimationSequencePlayer.prototype.onDone = function (fn) {
            this._subscriptions.push(fn);
        };
        AnimationSequencePlayer.prototype.play = function () {
            this._activePlayer.play();
        };
        AnimationSequencePlayer.prototype.pause = function () {
            this._activePlayer.pause();
        };
        AnimationSequencePlayer.prototype.restart = function () {
            if (this._players.length > 0) {
                this.reset();
                this._players[0].restart();
            }
        };
        AnimationSequencePlayer.prototype.reset = function () {
            this._players.forEach(function (player) {
                return player.reset();
            });
        };
        AnimationSequencePlayer.prototype.finish = function () {
            this._onFinish();
            this._players.forEach(function (player) {
                return player.finish();
            });
        };
        AnimationSequencePlayer.prototype.destroy = function () {
            this._onFinish();
            this._players.forEach(function (player) {
                return player.destroy();
            });
        };
        AnimationSequencePlayer.prototype.setPosition = function (p /** TODO #9100 */) {
            this._players[0].setPosition(p);
        };
        AnimationSequencePlayer.prototype.getPosition = function () {
            return this._players[0].getPosition();
        };
        return AnimationSequencePlayer;
    }();
    exports.AnimationSequencePlayer = AnimationSequencePlayer;
    
});
System.registerDynamic('node_modules/@angular/core/src/animation/animation_constants.js', [], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  exports.FILL_STYLE_FLAG = 'true'; // TODO (matsko): change to boolean
  exports.ANY_STATE = '*';
  exports.DEFAULT_STATE = '*';
  exports.EMPTY_STATE = 'void';
  
});
System.registerDynamic('node_modules/@angular/core/src/animation/animation_style_util.js', ['../facade/collection', '../facade/lang', './animation_constants', './metadata'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var collection_1 = $__require('../facade/collection');
    var lang_1 = $__require('../facade/lang');
    var animation_constants_1 = $__require('./animation_constants');
    var metadata_1 = $__require('./metadata');
    function prepareFinalAnimationStyles(previousStyles, newStyles, nullValue) {
        if (nullValue === void 0) {
            nullValue = null;
        }
        var finalStyles = {};
        collection_1.StringMapWrapper.forEach(newStyles, function (value, prop) {
            finalStyles[prop] = value == metadata_1.AUTO_STYLE ? nullValue : value.toString();
        });
        collection_1.StringMapWrapper.forEach(previousStyles, function (value, prop) {
            if (!lang_1.isPresent(finalStyles[prop])) {
                finalStyles[prop] = nullValue;
            }
        });
        return finalStyles;
    }
    exports.prepareFinalAnimationStyles = prepareFinalAnimationStyles;
    function balanceAnimationKeyframes(collectedStyles, finalStateStyles, keyframes) {
        var limit = keyframes.length - 1;
        var firstKeyframe = keyframes[0];
        // phase 1: copy all the styles from the first keyframe into the lookup map
        var flatenedFirstKeyframeStyles = flattenStyles(firstKeyframe.styles.styles);
        var extraFirstKeyframeStyles = {};
        var hasExtraFirstStyles = false;
        collection_1.StringMapWrapper.forEach(collectedStyles, function (value, prop) {
            // if the style is already defined in the first keyframe then
            // we do not replace it.
            if (!flatenedFirstKeyframeStyles[prop]) {
                flatenedFirstKeyframeStyles[prop] = value;
                extraFirstKeyframeStyles[prop] = value;
                hasExtraFirstStyles = true;
            }
        });
        var keyframeCollectedStyles = collection_1.StringMapWrapper.merge({}, flatenedFirstKeyframeStyles);
        // phase 2: normalize the final keyframe
        var finalKeyframe = keyframes[limit];
        collection_1.ListWrapper.insert(finalKeyframe.styles.styles, 0, finalStateStyles);
        var flatenedFinalKeyframeStyles = flattenStyles(finalKeyframe.styles.styles);
        var extraFinalKeyframeStyles = {};
        var hasExtraFinalStyles = false;
        collection_1.StringMapWrapper.forEach(keyframeCollectedStyles, function (value, prop) {
            if (!lang_1.isPresent(flatenedFinalKeyframeStyles[prop])) {
                extraFinalKeyframeStyles[prop] = metadata_1.AUTO_STYLE;
                hasExtraFinalStyles = true;
            }
        });
        if (hasExtraFinalStyles) {
            finalKeyframe.styles.styles.push(extraFinalKeyframeStyles);
        }
        collection_1.StringMapWrapper.forEach(flatenedFinalKeyframeStyles, function (value, prop) {
            if (!lang_1.isPresent(flatenedFirstKeyframeStyles[prop])) {
                extraFirstKeyframeStyles[prop] = metadata_1.AUTO_STYLE;
                hasExtraFirstStyles = true;
            }
        });
        if (hasExtraFirstStyles) {
            firstKeyframe.styles.styles.push(extraFirstKeyframeStyles);
        }
        return keyframes;
    }
    exports.balanceAnimationKeyframes = balanceAnimationKeyframes;
    function clearStyles(styles) {
        var finalStyles = {};
        collection_1.StringMapWrapper.keys(styles).forEach(function (key) {
            finalStyles[key] = null;
        });
        return finalStyles;
    }
    exports.clearStyles = clearStyles;
    function collectAndResolveStyles(collection, styles) {
        return styles.map(function (entry) {
            var stylesObj = {};
            collection_1.StringMapWrapper.forEach(entry, function (value, prop) {
                if (value == animation_constants_1.FILL_STYLE_FLAG) {
                    value = collection[prop];
                    if (!lang_1.isPresent(value)) {
                        value = metadata_1.AUTO_STYLE;
                    }
                }
                collection[prop] = value;
                stylesObj[prop] = value;
            });
            return stylesObj;
        });
    }
    exports.collectAndResolveStyles = collectAndResolveStyles;
    function renderStyles(element, renderer, styles) {
        collection_1.StringMapWrapper.forEach(styles, function (value, prop) {
            renderer.setElementStyle(element, prop, value);
        });
    }
    exports.renderStyles = renderStyles;
    function flattenStyles(styles) {
        var finalStyles = {};
        styles.forEach(function (entry) {
            collection_1.StringMapWrapper.forEach(entry, function (value, prop) {
                finalStyles[prop] = value;
            });
        });
        return finalStyles;
    }
    exports.flattenStyles = flattenStyles;
    
});
System.registerDynamic("node_modules/@angular/core/src/animation/animation_styles.js", [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var AnimationStyles = function () {
        function AnimationStyles(styles) {
            this.styles = styles;
        }
        return AnimationStyles;
    }();
    exports.AnimationStyles = AnimationStyles;
    
});
System.registerDynamic('node_modules/@angular/core/src/console.js', ['./di/decorators', './facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var decorators_1 = $__require('./di/decorators');
    var lang_1 = $__require('./facade/lang');
    // Note: Need to rename warn as in Dart
    // class members and imports can't use the same name.
    var _warnImpl = lang_1.warn;
    var Console = function () {
        function Console() {}
        Console.prototype.log = function (message) {
            lang_1.print(message);
        };
        // Note: for reporting errors use `DOM.logError()` as it is platform specific
        Console.prototype.warn = function (message) {
            _warnImpl(message);
        };
        /** @nocollapse */
        Console.decorators = [{ type: decorators_1.Injectable }];
        return Console;
    }();
    exports.Console = Console;
    
});
System.registerDynamic('node_modules/@angular/core/src/debug/debug_node.js', ['../facade/collection', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var collection_1 = $__require('../facade/collection');
    var lang_1 = $__require('../facade/lang');
    var EventListener = function () {
        function EventListener(name, callback) {
            this.name = name;
            this.callback = callback;
        }
        ;
        return EventListener;
    }();
    exports.EventListener = EventListener;
    /**
     * @experimental All debugging apis are currently experimental.
     */
    var DebugNode = function () {
        function DebugNode(nativeNode, parent, _debugInfo) {
            this._debugInfo = _debugInfo;
            this.nativeNode = nativeNode;
            if (lang_1.isPresent(parent) && parent instanceof DebugElement) {
                parent.addChild(this);
            } else {
                this.parent = null;
            }
            this.listeners = [];
        }
        Object.defineProperty(DebugNode.prototype, "injector", {
            get: function () {
                return lang_1.isPresent(this._debugInfo) ? this._debugInfo.injector : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DebugNode.prototype, "componentInstance", {
            get: function () {
                return lang_1.isPresent(this._debugInfo) ? this._debugInfo.component : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DebugNode.prototype, "context", {
            get: function () {
                return lang_1.isPresent(this._debugInfo) ? this._debugInfo.context : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DebugNode.prototype, "references", {
            get: function () {
                return lang_1.isPresent(this._debugInfo) ? this._debugInfo.references : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DebugNode.prototype, "providerTokens", {
            get: function () {
                return lang_1.isPresent(this._debugInfo) ? this._debugInfo.providerTokens : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DebugNode.prototype, "source", {
            get: function () {
                return lang_1.isPresent(this._debugInfo) ? this._debugInfo.source : null;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Use injector.get(token) instead.
         *
         * @deprecated
         */
        DebugNode.prototype.inject = function (token) {
            return this.injector.get(token);
        };
        return DebugNode;
    }();
    exports.DebugNode = DebugNode;
    /**
     * @experimental All debugging apis are currently experimental.
     */
    var DebugElement = function (_super) {
        __extends(DebugElement, _super);
        function DebugElement(nativeNode, parent, _debugInfo) {
            _super.call(this, nativeNode, parent, _debugInfo);
            this.properties = {};
            this.attributes = {};
            this.classes = {};
            this.styles = {};
            this.childNodes = [];
            this.nativeElement = nativeNode;
        }
        DebugElement.prototype.addChild = function (child) {
            if (lang_1.isPresent(child)) {
                this.childNodes.push(child);
                child.parent = this;
            }
        };
        DebugElement.prototype.removeChild = function (child) {
            var childIndex = this.childNodes.indexOf(child);
            if (childIndex !== -1) {
                child.parent = null;
                this.childNodes.splice(childIndex, 1);
            }
        };
        DebugElement.prototype.insertChildrenAfter = function (child, newChildren) {
            var siblingIndex = this.childNodes.indexOf(child);
            if (siblingIndex !== -1) {
                var previousChildren = this.childNodes.slice(0, siblingIndex + 1);
                var nextChildren = this.childNodes.slice(siblingIndex + 1);
                this.childNodes = collection_1.ListWrapper.concat(collection_1.ListWrapper.concat(previousChildren, newChildren), nextChildren);
                for (var i = 0; i < newChildren.length; ++i) {
                    var newChild = newChildren[i];
                    if (lang_1.isPresent(newChild.parent)) {
                        newChild.parent.removeChild(newChild);
                    }
                    newChild.parent = this;
                }
            }
        };
        DebugElement.prototype.query = function (predicate) {
            var results = this.queryAll(predicate);
            return results.length > 0 ? results[0] : null;
        };
        DebugElement.prototype.queryAll = function (predicate) {
            var matches = [];
            _queryElementChildren(this, predicate, matches);
            return matches;
        };
        DebugElement.prototype.queryAllNodes = function (predicate) {
            var matches = [];
            _queryNodeChildren(this, predicate, matches);
            return matches;
        };
        Object.defineProperty(DebugElement.prototype, "children", {
            get: function () {
                var children = [];
                this.childNodes.forEach(function (node) {
                    if (node instanceof DebugElement) {
                        children.push(node);
                    }
                });
                return children;
            },
            enumerable: true,
            configurable: true
        });
        DebugElement.prototype.triggerEventHandler = function (eventName, eventObj) {
            this.listeners.forEach(function (listener) {
                if (listener.name == eventName) {
                    listener.callback(eventObj);
                }
            });
        };
        return DebugElement;
    }(DebugNode);
    exports.DebugElement = DebugElement;
    /**
     * @experimental
     */
    function asNativeElements(debugEls) {
        return debugEls.map(function (el) {
            return el.nativeElement;
        });
    }
    exports.asNativeElements = asNativeElements;
    function _queryElementChildren(element, predicate, matches) {
        element.childNodes.forEach(function (node) {
            if (node instanceof DebugElement) {
                if (predicate(node)) {
                    matches.push(node);
                }
                _queryElementChildren(node, predicate, matches);
            }
        });
    }
    function _queryNodeChildren(parentNode, predicate, matches) {
        if (parentNode instanceof DebugElement) {
            parentNode.childNodes.forEach(function (node) {
                if (predicate(node)) {
                    matches.push(node);
                }
                if (node instanceof DebugElement) {
                    _queryNodeChildren(node, predicate, matches);
                }
            });
        }
    }
    // Need to keep the nodes in a global Map so that multiple angular apps are supported.
    var _nativeNodeToDebugNode = new Map();
    /**
     * @experimental
     */
    function getDebugNode(nativeNode) {
        return _nativeNodeToDebugNode.get(nativeNode);
    }
    exports.getDebugNode = getDebugNode;
    function getAllDebugNodes() {
        return collection_1.MapWrapper.values(_nativeNodeToDebugNode);
    }
    exports.getAllDebugNodes = getAllDebugNodes;
    function indexDebugNode(node) {
        _nativeNodeToDebugNode.set(node.nativeNode, node);
    }
    exports.indexDebugNode = indexDebugNode;
    function removeDebugNodeFromIndex(node) {
        _nativeNodeToDebugNode.delete(node.nativeNode);
    }
    exports.removeDebugNodeFromIndex = removeDebugNodeFromIndex;
    
});
System.registerDynamic('node_modules/@angular/core/src/debug/debug_renderer.js', ['../facade/lang', './debug_node'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('../facade/lang');
    var debug_node_1 = $__require('./debug_node');
    var DebugDomRootRenderer = function () {
        function DebugDomRootRenderer(_delegate) {
            this._delegate = _delegate;
        }
        DebugDomRootRenderer.prototype.renderComponent = function (componentProto) {
            return new DebugDomRenderer(this._delegate.renderComponent(componentProto));
        };
        return DebugDomRootRenderer;
    }();
    exports.DebugDomRootRenderer = DebugDomRootRenderer;
    var DebugDomRenderer = function () {
        function DebugDomRenderer(_delegate) {
            this._delegate = _delegate;
        }
        DebugDomRenderer.prototype.selectRootElement = function (selectorOrNode, debugInfo) {
            var nativeEl = this._delegate.selectRootElement(selectorOrNode, debugInfo);
            var debugEl = new debug_node_1.DebugElement(nativeEl, null, debugInfo);
            debug_node_1.indexDebugNode(debugEl);
            return nativeEl;
        };
        DebugDomRenderer.prototype.createElement = function (parentElement, name, debugInfo) {
            var nativeEl = this._delegate.createElement(parentElement, name, debugInfo);
            var debugEl = new debug_node_1.DebugElement(nativeEl, debug_node_1.getDebugNode(parentElement), debugInfo);
            debugEl.name = name;
            debug_node_1.indexDebugNode(debugEl);
            return nativeEl;
        };
        DebugDomRenderer.prototype.createViewRoot = function (hostElement) {
            return this._delegate.createViewRoot(hostElement);
        };
        DebugDomRenderer.prototype.createTemplateAnchor = function (parentElement, debugInfo) {
            var comment = this._delegate.createTemplateAnchor(parentElement, debugInfo);
            var debugEl = new debug_node_1.DebugNode(comment, debug_node_1.getDebugNode(parentElement), debugInfo);
            debug_node_1.indexDebugNode(debugEl);
            return comment;
        };
        DebugDomRenderer.prototype.createText = function (parentElement, value, debugInfo) {
            var text = this._delegate.createText(parentElement, value, debugInfo);
            var debugEl = new debug_node_1.DebugNode(text, debug_node_1.getDebugNode(parentElement), debugInfo);
            debug_node_1.indexDebugNode(debugEl);
            return text;
        };
        DebugDomRenderer.prototype.projectNodes = function (parentElement, nodes) {
            var debugParent = debug_node_1.getDebugNode(parentElement);
            if (lang_1.isPresent(debugParent) && debugParent instanceof debug_node_1.DebugElement) {
                var debugElement_1 = debugParent;
                nodes.forEach(function (node) {
                    debugElement_1.addChild(debug_node_1.getDebugNode(node));
                });
            }
            this._delegate.projectNodes(parentElement, nodes);
        };
        DebugDomRenderer.prototype.attachViewAfter = function (node, viewRootNodes) {
            var debugNode = debug_node_1.getDebugNode(node);
            if (lang_1.isPresent(debugNode)) {
                var debugParent = debugNode.parent;
                if (viewRootNodes.length > 0 && lang_1.isPresent(debugParent)) {
                    var debugViewRootNodes = [];
                    viewRootNodes.forEach(function (rootNode) {
                        return debugViewRootNodes.push(debug_node_1.getDebugNode(rootNode));
                    });
                    debugParent.insertChildrenAfter(debugNode, debugViewRootNodes);
                }
            }
            this._delegate.attachViewAfter(node, viewRootNodes);
        };
        DebugDomRenderer.prototype.detachView = function (viewRootNodes) {
            viewRootNodes.forEach(function (node) {
                var debugNode = debug_node_1.getDebugNode(node);
                if (lang_1.isPresent(debugNode) && lang_1.isPresent(debugNode.parent)) {
                    debugNode.parent.removeChild(debugNode);
                }
            });
            this._delegate.detachView(viewRootNodes);
        };
        DebugDomRenderer.prototype.destroyView = function (hostElement, viewAllNodes) {
            viewAllNodes.forEach(function (node) {
                debug_node_1.removeDebugNodeFromIndex(debug_node_1.getDebugNode(node));
            });
            this._delegate.destroyView(hostElement, viewAllNodes);
        };
        DebugDomRenderer.prototype.listen = function (renderElement, name, callback) {
            var debugEl = debug_node_1.getDebugNode(renderElement);
            if (lang_1.isPresent(debugEl)) {
                debugEl.listeners.push(new debug_node_1.EventListener(name, callback));
            }
            return this._delegate.listen(renderElement, name, callback);
        };
        DebugDomRenderer.prototype.listenGlobal = function (target, name, callback) {
            return this._delegate.listenGlobal(target, name, callback);
        };
        DebugDomRenderer.prototype.setElementProperty = function (renderElement, propertyName, propertyValue) {
            var debugEl = debug_node_1.getDebugNode(renderElement);
            if (lang_1.isPresent(debugEl) && debugEl instanceof debug_node_1.DebugElement) {
                debugEl.properties[propertyName] = propertyValue;
            }
            this._delegate.setElementProperty(renderElement, propertyName, propertyValue);
        };
        DebugDomRenderer.prototype.setElementAttribute = function (renderElement, attributeName, attributeValue) {
            var debugEl = debug_node_1.getDebugNode(renderElement);
            if (lang_1.isPresent(debugEl) && debugEl instanceof debug_node_1.DebugElement) {
                debugEl.attributes[attributeName] = attributeValue;
            }
            this._delegate.setElementAttribute(renderElement, attributeName, attributeValue);
        };
        DebugDomRenderer.prototype.setBindingDebugInfo = function (renderElement, propertyName, propertyValue) {
            this._delegate.setBindingDebugInfo(renderElement, propertyName, propertyValue);
        };
        DebugDomRenderer.prototype.setElementClass = function (renderElement, className, isAdd) {
            var debugEl = debug_node_1.getDebugNode(renderElement);
            if (lang_1.isPresent(debugEl) && debugEl instanceof debug_node_1.DebugElement) {
                debugEl.classes[className] = isAdd;
            }
            this._delegate.setElementClass(renderElement, className, isAdd);
        };
        DebugDomRenderer.prototype.setElementStyle = function (renderElement, styleName, styleValue) {
            var debugEl = debug_node_1.getDebugNode(renderElement);
            if (lang_1.isPresent(debugEl) && debugEl instanceof debug_node_1.DebugElement) {
                debugEl.styles[styleName] = styleValue;
            }
            this._delegate.setElementStyle(renderElement, styleName, styleValue);
        };
        DebugDomRenderer.prototype.invokeElementMethod = function (renderElement, methodName, args) {
            this._delegate.invokeElementMethod(renderElement, methodName, args);
        };
        DebugDomRenderer.prototype.setText = function (renderNode, text) {
            this._delegate.setText(renderNode, text);
        };
        DebugDomRenderer.prototype.animate = function (element, startingStyles, keyframes, duration, delay, easing) {
            return this._delegate.animate(element, startingStyles, keyframes, duration, delay, easing);
        };
        return DebugDomRenderer;
    }();
    exports.DebugDomRenderer = DebugDomRenderer;
    
});
System.registerDynamic('node_modules/@angular/core/src/linker/component_factory_resolver.js', ['../facade/exceptions', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    /**
     * @stable
     */
    var NoComponentFactoryError = function (_super) {
        __extends(NoComponentFactoryError, _super);
        function NoComponentFactoryError(component) {
            _super.call(this, "No component factory found for " + lang_1.stringify(component));
            this.component = component;
        }
        return NoComponentFactoryError;
    }(exceptions_1.BaseException);
    exports.NoComponentFactoryError = NoComponentFactoryError;
    var _NullComponentFactoryResolver = function () {
        function _NullComponentFactoryResolver() {}
        _NullComponentFactoryResolver.prototype.resolveComponentFactory = function (component) {
            throw new NoComponentFactoryError(component);
        };
        return _NullComponentFactoryResolver;
    }();
    /**
     * @stable
     */
    var ComponentFactoryResolver = function () {
        function ComponentFactoryResolver() {}
        ComponentFactoryResolver.NULL = new _NullComponentFactoryResolver();
        return ComponentFactoryResolver;
    }();
    exports.ComponentFactoryResolver = ComponentFactoryResolver;
    var CodegenComponentFactoryResolver = function () {
        function CodegenComponentFactoryResolver(factories, _parent) {
            this._parent = _parent;
            this._factories = new Map();
            for (var i = 0; i < factories.length; i++) {
                var factory = factories[i];
                this._factories.set(factory.componentType, factory);
            }
        }
        CodegenComponentFactoryResolver.prototype.resolveComponentFactory = function (component) {
            var result = this._factories.get(component);
            if (!result) {
                result = this._parent.resolveComponentFactory(component);
            }
            return result;
        };
        return CodegenComponentFactoryResolver;
    }();
    exports.CodegenComponentFactoryResolver = CodegenComponentFactoryResolver;
    
});
System.registerDynamic('node_modules/@angular/core/src/linker/component_factory.js', ['../facade/exceptions', '../facade/lang', './view_utils'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    var view_utils_1 = $__require('./view_utils');
    /**
     * Represents an instance of a Component created via a {@link ComponentFactory}.
     *
     * `ComponentRef` provides access to the Component Instance as well other objects related to this
     * Component Instance and allows you to destroy the Component Instance via the {@link #destroy}
     * method.
     * @stable
     */
    var ComponentRef = function () {
        function ComponentRef() {}
        Object.defineProperty(ComponentRef.prototype, "location", {
            /**
             * Location of the Host Element of this Component Instance.
             */
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ComponentRef.prototype, "injector", {
            /**
             * The injector on which the component instance exists.
             */
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ComponentRef.prototype, "instance", {
            /**
             * The instance of the Component.
             */
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(ComponentRef.prototype, "hostView", {
            /**
             * The {@link ViewRef} of the Host View of this Component instance.
             */
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(ComponentRef.prototype, "changeDetectorRef", {
            /**
             * The {@link ChangeDetectorRef} of the Component instance.
             */
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ComponentRef.prototype, "componentType", {
            /**
             * The component type.
             */
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        return ComponentRef;
    }();
    exports.ComponentRef = ComponentRef;
    var ComponentRef_ = function (_super) {
        __extends(ComponentRef_, _super);
        function ComponentRef_(_hostElement, _componentType) {
            _super.call(this);
            this._hostElement = _hostElement;
            this._componentType = _componentType;
        }
        Object.defineProperty(ComponentRef_.prototype, "location", {
            get: function () {
                return this._hostElement.elementRef;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ComponentRef_.prototype, "injector", {
            get: function () {
                return this._hostElement.injector;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ComponentRef_.prototype, "instance", {
            get: function () {
                return this._hostElement.component;
            },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(ComponentRef_.prototype, "hostView", {
            get: function () {
                return this._hostElement.parentView.ref;
            },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(ComponentRef_.prototype, "changeDetectorRef", {
            get: function () {
                return this._hostElement.parentView.ref;
            },
            enumerable: true,
            configurable: true
        });
        ;
        Object.defineProperty(ComponentRef_.prototype, "componentType", {
            get: function () {
                return this._componentType;
            },
            enumerable: true,
            configurable: true
        });
        ComponentRef_.prototype.destroy = function () {
            this._hostElement.parentView.destroy();
        };
        ComponentRef_.prototype.onDestroy = function (callback) {
            this.hostView.onDestroy(callback);
        };
        return ComponentRef_;
    }(ComponentRef);
    exports.ComponentRef_ = ComponentRef_;
    /**
     * @experimental
     * @ts2dart_const
     */
    var EMPTY_CONTEXT = new Object();
    /**
     * @stable
     */
    var ComponentFactory = function () {
        function ComponentFactory(selector, _viewFactory, _componentType) {
            this.selector = selector;
            this._viewFactory = _viewFactory;
            this._componentType = _componentType;
        }
        Object.defineProperty(ComponentFactory.prototype, "componentType", {
            get: function () {
                return this._componentType;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Creates a new component.
         */
        ComponentFactory.prototype.create = function (injector, projectableNodes, rootSelectorOrNode) {
            if (projectableNodes === void 0) {
                projectableNodes = null;
            }
            if (rootSelectorOrNode === void 0) {
                rootSelectorOrNode = null;
            }
            var vu = injector.get(view_utils_1.ViewUtils);
            if (lang_1.isBlank(projectableNodes)) {
                projectableNodes = [];
            }
            // Note: Host views don't need a declarationAppElement!
            var hostView = this._viewFactory(vu, injector, null);
            var hostElement = hostView.create(EMPTY_CONTEXT, projectableNodes, rootSelectorOrNode);
            return new ComponentRef_(hostElement, this._componentType);
        };
        return ComponentFactory;
    }();
    exports.ComponentFactory = ComponentFactory;
    
});
System.registerDynamic('node_modules/@angular/core/src/linker/component_resolver.js', ['../di/decorators', '../facade/async', '../facade/exceptions', '../facade/lang', '../reflection/reflection', './component_factory'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var decorators_1 = $__require('../di/decorators');
    var async_1 = $__require('../facade/async');
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    var reflection_1 = $__require('../reflection/reflection');
    var component_factory_1 = $__require('./component_factory');
    /**
     * Low-level service for loading {@link ComponentFactory}s, which
     * can later be used to create and render a Component instance.
     * @experimental
     */
    var ComponentResolver = function () {
        function ComponentResolver() {}
        return ComponentResolver;
    }();
    exports.ComponentResolver = ComponentResolver;
    function _isComponentFactory(type) {
        return type instanceof component_factory_1.ComponentFactory;
    }
    var ReflectorComponentResolver = function (_super) {
        __extends(ReflectorComponentResolver, _super);
        function ReflectorComponentResolver() {
            _super.apply(this, arguments);
        }
        ReflectorComponentResolver.prototype.resolveComponent = function (component) {
            if (lang_1.isString(component)) {
                return async_1.PromiseWrapper.reject(new exceptions_1.BaseException("Cannot resolve component using '" + component + "'."), null);
            }
            var metadatas = reflection_1.reflector.annotations(component);
            var componentFactory = metadatas.find(_isComponentFactory);
            if (lang_1.isBlank(componentFactory)) {
                throw new exceptions_1.BaseException("No precompiled component " + lang_1.stringify(component) + " found");
            }
            return async_1.PromiseWrapper.resolve(componentFactory);
        };
        ReflectorComponentResolver.prototype.clearCache = function () {};
        /** @nocollapse */
        ReflectorComponentResolver.decorators = [{ type: decorators_1.Injectable }];
        return ReflectorComponentResolver;
    }(ComponentResolver);
    exports.ReflectorComponentResolver = ReflectorComponentResolver;
    
});
System.registerDynamic("node_modules/@angular/core/src/linker/template_ref.js", ["../facade/lang"], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var lang_1 = $__require('../facade/lang');
    var EMPTY_CONTEXT = new Object();
    /**
     * Represents an Embedded Template that can be used to instantiate Embedded Views.
     *
     * You can access a `TemplateRef`, in two ways. Via a directive placed on a `<template>` element (or
     * directive prefixed with `*`) and have the `TemplateRef` for this Embedded View injected into the
     * constructor of the directive using the `TemplateRef` Token. Alternatively you can query for the
     * `TemplateRef` from a Component or a Directive via {@link Query}.
     *
     * To instantiate Embedded Views based on a Template, use
     * {@link ViewContainerRef#createEmbeddedView}, which will create the View and attach it to the
     * View Container.
     * @stable
     */
    var TemplateRef = function () {
        function TemplateRef() {}
        Object.defineProperty(TemplateRef.prototype, "elementRef", {
            /**
             * The location in the View where the Embedded View logically belongs to.
             *
             * The data-binding and injection contexts of Embedded Views created from this `TemplateRef`
             * inherit from the contexts of this location.
             *
             * Typically new Embedded Views are attached to the View Container of this location, but in
             * advanced use-cases, the View can be attached to a different container while keeping the
             * data-binding and injection context from the original location.
             *
             */
            // TODO(i): rename to anchor or location
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        return TemplateRef;
    }();
    exports.TemplateRef = TemplateRef;
    var TemplateRef_ = function (_super) {
        __extends(TemplateRef_, _super);
        function TemplateRef_(_appElement, _viewFactory) {
            _super.call(this);
            this._appElement = _appElement;
            this._viewFactory = _viewFactory;
        }
        TemplateRef_.prototype.createEmbeddedView = function (context) {
            var view = this._viewFactory(this._appElement.parentView.viewUtils, this._appElement.parentInjector, this._appElement);
            if (lang_1.isBlank(context)) {
                context = EMPTY_CONTEXT;
            }
            view.create(context, null, null);
            return view.ref;
        };
        Object.defineProperty(TemplateRef_.prototype, "elementRef", {
            get: function () {
                return this._appElement.elementRef;
            },
            enumerable: true,
            configurable: true
        });
        return TemplateRef_;
    }(TemplateRef);
    exports.TemplateRef_ = TemplateRef_;
    
});
System.registerDynamic("node_modules/@angular/core/src/facade/promise.js", [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var PromiseCompleter = function () {
        function PromiseCompleter() {
            var _this = this;
            this.promise = new Promise(function (res, rej) {
                _this.resolve = res;
                _this.reject = rej;
            });
        }
        return PromiseCompleter;
    }();
    exports.PromiseCompleter = PromiseCompleter;
    var PromiseWrapper = function () {
        function PromiseWrapper() {}
        PromiseWrapper.resolve = function (obj) {
            return Promise.resolve(obj);
        };
        PromiseWrapper.reject = function (obj, _) {
            return Promise.reject(obj);
        };
        // Note: We can't rename this method into `catch`, as this is not a valid
        // method name in Dart.
        PromiseWrapper.catchError = function (promise, onError) {
            return promise.catch(onError);
        };
        PromiseWrapper.all = function (promises) {
            if (promises.length == 0) return Promise.resolve([]);
            return Promise.all(promises);
        };
        PromiseWrapper.then = function (promise, success, rejection) {
            return promise.then(success, rejection);
        };
        PromiseWrapper.wrap = function (computation) {
            return new Promise(function (res, rej) {
                try {
                    res(computation());
                } catch (e) {
                    rej(e);
                }
            });
        };
        PromiseWrapper.scheduleMicrotask = function (computation) {
            PromiseWrapper.then(PromiseWrapper.resolve(null), computation, function (_) {});
        };
        PromiseWrapper.completer = function () {
            return new PromiseCompleter();
        };
        return PromiseWrapper;
    }();
    exports.PromiseWrapper = PromiseWrapper;
    
});
System.registerDynamic('node_modules/@angular/core/src/facade/async.js', ['rxjs/Subject', 'rxjs/observable/PromiseObservable', 'rxjs/operator/toPromise', './lang', 'rxjs/Observable', './promise'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subject_1 = $__require('rxjs/Subject');
    var PromiseObservable_1 = $__require('rxjs/observable/PromiseObservable');
    var toPromise_1 = $__require('rxjs/operator/toPromise');
    var lang_1 = $__require('./lang');
    var Observable_1 = $__require('rxjs/Observable');
    exports.Observable = Observable_1.Observable;
    var Subject_2 = $__require('rxjs/Subject');
    exports.Subject = Subject_2.Subject;
    var promise_1 = $__require('./promise');
    exports.PromiseCompleter = promise_1.PromiseCompleter;
    exports.PromiseWrapper = promise_1.PromiseWrapper;
    var TimerWrapper = function () {
        function TimerWrapper() {}
        TimerWrapper.setTimeout = function (fn, millis) {
            return lang_1.global.setTimeout(fn, millis);
        };
        TimerWrapper.clearTimeout = function (id) {
            lang_1.global.clearTimeout(id);
        };
        TimerWrapper.setInterval = function (fn, millis) {
            return lang_1.global.setInterval(fn, millis);
        };
        TimerWrapper.clearInterval = function (id) {
            lang_1.global.clearInterval(id);
        };
        return TimerWrapper;
    }();
    exports.TimerWrapper = TimerWrapper;
    var ObservableWrapper = function () {
        function ObservableWrapper() {}
        // TODO(vsavkin): when we use rxnext, try inferring the generic type from the first arg
        ObservableWrapper.subscribe = function (emitter, onNext, onError, onComplete) {
            if (onComplete === void 0) {
                onComplete = function () {};
            }
            onError = typeof onError === 'function' && onError || lang_1.noop;
            onComplete = typeof onComplete === 'function' && onComplete || lang_1.noop;
            return emitter.subscribe({ next: onNext, error: onError, complete: onComplete });
        };
        ObservableWrapper.isObservable = function (obs) {
            return !!obs.subscribe;
        };
        /**
         * Returns whether `obs` has any subscribers listening to events.
         */
        ObservableWrapper.hasSubscribers = function (obs) {
            return obs.observers.length > 0;
        };
        ObservableWrapper.dispose = function (subscription) {
            subscription.unsubscribe();
        };
        /**
         * @deprecated - use callEmit() instead
         */
        ObservableWrapper.callNext = function (emitter, value) {
            emitter.emit(value);
        };
        ObservableWrapper.callEmit = function (emitter, value) {
            emitter.emit(value);
        };
        ObservableWrapper.callError = function (emitter, error) {
            emitter.error(error);
        };
        ObservableWrapper.callComplete = function (emitter) {
            emitter.complete();
        };
        ObservableWrapper.fromPromise = function (promise) {
            return PromiseObservable_1.PromiseObservable.create(promise);
        };
        ObservableWrapper.toPromise = function (obj) {
            return toPromise_1.toPromise.call(obj);
        };
        return ObservableWrapper;
    }();
    exports.ObservableWrapper = ObservableWrapper;
    /**
     * Use by directives and components to emit custom Events.
     *
     * ### Examples
     *
     * In the following example, `Zippy` alternatively emits `open` and `close` events when its
     * title gets clicked:
     *
     * ```
     * @Component({
     *   selector: 'zippy',
     *   template: `
     *   <div class="zippy">
     *     <div (click)="toggle()">Toggle</div>
     *     <div [hidden]="!visible">
     *       <ng-content></ng-content>
     *     </div>
     *  </div>`})
     * export class Zippy {
     *   visible: boolean = true;
     *   @Output() open: EventEmitter<any> = new EventEmitter();
     *   @Output() close: EventEmitter<any> = new EventEmitter();
     *
     *   toggle() {
     *     this.visible = !this.visible;
     *     if (this.visible) {
     *       this.open.emit(null);
     *     } else {
     *       this.close.emit(null);
     *     }
     *   }
     * }
     * ```
     *
     * The events payload can be accessed by the parameter `$event` on the components output event
     * handler:
     *
     * ```
     * <zippy (open)="onOpen($event)" (close)="onClose($event)"></zippy>
     * ```
     *
     * Uses Rx.Observable but provides an adapter to make it work as specified here:
     * https://github.com/jhusain/observable-spec
     *
     * Once a reference implementation of the spec is available, switch to it.
     * @stable
     */
    var EventEmitter = function (_super) {
        __extends(EventEmitter, _super);
        /**
         * Creates an instance of [EventEmitter], which depending on [isAsync],
         * delivers events synchronously or asynchronously.
         */
        function EventEmitter(isAsync) {
            if (isAsync === void 0) {
                isAsync = false;
            }
            _super.call(this);
            this.__isAsync = isAsync;
        }
        EventEmitter.prototype.emit = function (value) {
            _super.prototype.next.call(this, value);
        };
        /**
         * @deprecated - use .emit(value) instead
         */
        EventEmitter.prototype.next = function (value) {
            _super.prototype.next.call(this, value);
        };
        EventEmitter.prototype.subscribe = function (generatorOrNext, error, complete) {
            var schedulerFn;
            var errorFn = function (err) {
                return null;
            };
            var completeFn = function () {
                return null;
            };
            if (generatorOrNext && typeof generatorOrNext === 'object') {
                schedulerFn = this.__isAsync ? function (value /** TODO #9100 */) {
                    setTimeout(function () {
                        return generatorOrNext.next(value);
                    });
                } : function (value /** TODO #9100 */) {
                    generatorOrNext.next(value);
                };
                if (generatorOrNext.error) {
                    errorFn = this.__isAsync ? function (err) {
                        setTimeout(function () {
                            return generatorOrNext.error(err);
                        });
                    } : function (err) {
                        generatorOrNext.error(err);
                    };
                }
                if (generatorOrNext.complete) {
                    completeFn = this.__isAsync ? function () {
                        setTimeout(function () {
                            return generatorOrNext.complete();
                        });
                    } : function () {
                        generatorOrNext.complete();
                    };
                }
            } else {
                schedulerFn = this.__isAsync ? function (value /** TODO #9100 */) {
                    setTimeout(function () {
                        return generatorOrNext(value);
                    });
                } : function (value /** TODO #9100 */) {
                    generatorOrNext(value);
                };
                if (error) {
                    errorFn = this.__isAsync ? function (err) {
                        setTimeout(function () {
                            return error(err);
                        });
                    } : function (err) {
                        error(err);
                    };
                }
                if (complete) {
                    completeFn = this.__isAsync ? function () {
                        setTimeout(function () {
                            return complete();
                        });
                    } : function () {
                        complete();
                    };
                }
            }
            return _super.prototype.subscribe.call(this, schedulerFn, errorFn, completeFn);
        };
        return EventEmitter;
    }(Subject_1.Subject);
    exports.EventEmitter = EventEmitter;
    
});
System.registerDynamic('node_modules/@angular/core/src/linker/view_ref.js', ['../change_detection/constants', '../facade/exceptions'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var constants_1 = $__require('../change_detection/constants');
    var exceptions_1 = $__require('../facade/exceptions');
    /**
     * @stable
     */
    var ViewRef = function () {
        function ViewRef() {}
        Object.defineProperty(ViewRef.prototype, "destroyed", {
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        return ViewRef;
    }();
    exports.ViewRef = ViewRef;
    /**
     * Represents an Angular View.
     *
     * <!-- TODO: move the next two paragraphs to the dev guide -->
     * A View is a fundamental building block of the application UI. It is the smallest grouping of
     * Elements which are created and destroyed together.
     *
     * Properties of elements in a View can change, but the structure (number and order) of elements in
     * a View cannot. Changing the structure of Elements can only be done by inserting, moving or
     * removing nested Views via a {@link ViewContainerRef}. Each View can contain many View Containers.
     * <!-- /TODO -->
     *
     * ### Example
     *
     * Given this template...
     *
     * ```
     * Count: {{items.length}}
     * <ul>
     *   <li *ngFor="let  item of items">{{item}}</li>
     * </ul>
     * ```
     *
     * We have two {@link TemplateRef}s:
     *
     * Outer {@link TemplateRef}:
     * ```
     * Count: {{items.length}}
     * <ul>
     *   <template ngFor let-item [ngForOf]="items"></template>
     * </ul>
     * ```
     *
     * Inner {@link TemplateRef}:
     * ```
     *   <li>{{item}}</li>
     * ```
     *
     * Notice that the original template is broken down into two separate {@link TemplateRef}s.
     *
     * The outer/inner {@link TemplateRef}s are then assembled into views like so:
     *
     * ```
     * <!-- ViewRef: outer-0 -->
     * Count: 2
     * <ul>
     *   <template view-container-ref></template>
     *   <!-- ViewRef: inner-1 --><li>first</li><!-- /ViewRef: inner-1 -->
     *   <!-- ViewRef: inner-2 --><li>second</li><!-- /ViewRef: inner-2 -->
     * </ul>
     * <!-- /ViewRef: outer-0 -->
     * ```
     * @experimental
     */
    var EmbeddedViewRef = function (_super) {
        __extends(EmbeddedViewRef, _super);
        function EmbeddedViewRef() {
            _super.apply(this, arguments);
        }
        Object.defineProperty(EmbeddedViewRef.prototype, "context", {
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(EmbeddedViewRef.prototype, "rootNodes", {
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        ;
        return EmbeddedViewRef;
    }(ViewRef);
    exports.EmbeddedViewRef = EmbeddedViewRef;
    var ViewRef_ = function () {
        function ViewRef_(_view) {
            this._view = _view;
            this._view = _view;
            this._originalMode = this._view.cdMode;
        }
        Object.defineProperty(ViewRef_.prototype, "internalView", {
            get: function () {
                return this._view;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ViewRef_.prototype, "rootNodes", {
            get: function () {
                return this._view.flatRootNodes;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ViewRef_.prototype, "context", {
            get: function () {
                return this._view.context;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ViewRef_.prototype, "destroyed", {
            get: function () {
                return this._view.destroyed;
            },
            enumerable: true,
            configurable: true
        });
        ViewRef_.prototype.markForCheck = function () {
            this._view.markPathToRootAsCheckOnce();
        };
        ViewRef_.prototype.detach = function () {
            this._view.cdMode = constants_1.ChangeDetectorStatus.Detached;
        };
        ViewRef_.prototype.detectChanges = function () {
            this._view.detectChanges(false);
        };
        ViewRef_.prototype.checkNoChanges = function () {
            this._view.detectChanges(true);
        };
        ViewRef_.prototype.reattach = function () {
            this._view.cdMode = this._originalMode;
            this.markForCheck();
        };
        ViewRef_.prototype.onDestroy = function (callback) {
            this._view.disposables.push(callback);
        };
        ViewRef_.prototype.destroy = function () {
            this._view.destroy();
        };
        return ViewRef_;
    }();
    exports.ViewRef_ = ViewRef_;
    
});
System.registerDynamic('node_modules/@angular/core/src/linker/debug_context.js', ['../facade/collection', '../facade/lang', './view_type'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var collection_1 = $__require('../facade/collection');
    var lang_1 = $__require('../facade/lang');
    var view_type_1 = $__require('./view_type');
    /* @ts2dart_const */
    var StaticNodeDebugInfo = function () {
        function StaticNodeDebugInfo(providerTokens, componentToken, refTokens) {
            this.providerTokens = providerTokens;
            this.componentToken = componentToken;
            this.refTokens = refTokens;
        }
        return StaticNodeDebugInfo;
    }();
    exports.StaticNodeDebugInfo = StaticNodeDebugInfo;
    var DebugContext = function () {
        function DebugContext(_view, _nodeIndex, _tplRow, _tplCol) {
            this._view = _view;
            this._nodeIndex = _nodeIndex;
            this._tplRow = _tplRow;
            this._tplCol = _tplCol;
        }
        Object.defineProperty(DebugContext.prototype, "_staticNodeInfo", {
            get: function () {
                return lang_1.isPresent(this._nodeIndex) ? this._view.staticNodeDebugInfos[this._nodeIndex] : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DebugContext.prototype, "context", {
            get: function () {
                return this._view.context;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DebugContext.prototype, "component", {
            get: function () {
                var staticNodeInfo = this._staticNodeInfo;
                if (lang_1.isPresent(staticNodeInfo) && lang_1.isPresent(staticNodeInfo.componentToken)) {
                    return this.injector.get(staticNodeInfo.componentToken);
                }
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DebugContext.prototype, "componentRenderElement", {
            get: function () {
                var componentView = this._view;
                while (lang_1.isPresent(componentView.declarationAppElement) && componentView.type !== view_type_1.ViewType.COMPONENT) {
                    componentView = componentView.declarationAppElement.parentView;
                }
                return lang_1.isPresent(componentView.declarationAppElement) ? componentView.declarationAppElement.nativeElement : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DebugContext.prototype, "injector", {
            get: function () {
                return this._view.injector(this._nodeIndex);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DebugContext.prototype, "renderNode", {
            get: function () {
                if (lang_1.isPresent(this._nodeIndex) && lang_1.isPresent(this._view.allNodes)) {
                    return this._view.allNodes[this._nodeIndex];
                } else {
                    return null;
                }
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DebugContext.prototype, "providerTokens", {
            get: function () {
                var staticNodeInfo = this._staticNodeInfo;
                return lang_1.isPresent(staticNodeInfo) ? staticNodeInfo.providerTokens : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DebugContext.prototype, "source", {
            get: function () {
                return this._view.componentType.templateUrl + ":" + this._tplRow + ":" + this._tplCol;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DebugContext.prototype, "references", {
            get: function () {
                var _this = this;
                var varValues = {};
                var staticNodeInfo = this._staticNodeInfo;
                if (lang_1.isPresent(staticNodeInfo)) {
                    var refs = staticNodeInfo.refTokens;
                    collection_1.StringMapWrapper.forEach(refs, function (refToken /** TODO #9100 */, refName /** TODO #9100 */) {
                        var varValue;
                        if (lang_1.isBlank(refToken)) {
                            varValue = lang_1.isPresent(_this._view.allNodes) ? _this._view.allNodes[_this._nodeIndex] : null;
                        } else {
                            varValue = _this._view.injectorGet(refToken, _this._nodeIndex, null);
                        }
                        varValues[refName] = varValue;
                    });
                }
                return varValues;
            },
            enumerable: true,
            configurable: true
        });
        return DebugContext;
    }();
    exports.DebugContext = DebugContext;
    
});
System.registerDynamic("node_modules/@angular/core/src/linker/element_injector.js", ["../di/injector"], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var injector_1 = $__require('../di/injector');
    var _UNDEFINED = new Object();
    var ElementInjector = function (_super) {
        __extends(ElementInjector, _super);
        function ElementInjector(_view, _nodeIndex) {
            _super.call(this);
            this._view = _view;
            this._nodeIndex = _nodeIndex;
        }
        ElementInjector.prototype.get = function (token, notFoundValue) {
            if (notFoundValue === void 0) {
                notFoundValue = injector_1.THROW_IF_NOT_FOUND;
            }
            var result = _UNDEFINED;
            if (result === _UNDEFINED) {
                result = this._view.injectorGet(token, this._nodeIndex, _UNDEFINED);
            }
            if (result === _UNDEFINED) {
                result = this._view.parentInjector.get(token, notFoundValue);
            }
            return result;
        };
        return ElementInjector;
    }(injector_1.Injector);
    exports.ElementInjector = ElementInjector;
    
});
System.registerDynamic("node_modules/@angular/core/src/facade/math.js", ["./lang"], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var lang_1 = $__require('./lang');
  exports.Math = lang_1.global.Math;
  exports.NaN = typeof exports.NaN;
  
});
System.registerDynamic('node_modules/@angular/core/src/animation/animation_group_player.js', ['../facade/lang', '../facade/math'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('../facade/lang');
    var math_1 = $__require('../facade/math');
    var AnimationGroupPlayer = function () {
        function AnimationGroupPlayer(_players) {
            var _this = this;
            this._players = _players;
            this._subscriptions = [];
            this._finished = false;
            this.parentPlayer = null;
            var count = 0;
            var total = this._players.length;
            if (total == 0) {
                lang_1.scheduleMicroTask(function () {
                    return _this._onFinish();
                });
            } else {
                this._players.forEach(function (player) {
                    player.parentPlayer = _this;
                    player.onDone(function () {
                        if (++count >= total) {
                            _this._onFinish();
                        }
                    });
                });
            }
        }
        AnimationGroupPlayer.prototype._onFinish = function () {
            if (!this._finished) {
                this._finished = true;
                if (!lang_1.isPresent(this.parentPlayer)) {
                    this.destroy();
                }
                this._subscriptions.forEach(function (subscription) {
                    return subscription();
                });
                this._subscriptions = [];
            }
        };
        AnimationGroupPlayer.prototype.onDone = function (fn) {
            this._subscriptions.push(fn);
        };
        AnimationGroupPlayer.prototype.play = function () {
            this._players.forEach(function (player) {
                return player.play();
            });
        };
        AnimationGroupPlayer.prototype.pause = function () {
            this._players.forEach(function (player) {
                return player.pause();
            });
        };
        AnimationGroupPlayer.prototype.restart = function () {
            this._players.forEach(function (player) {
                return player.restart();
            });
        };
        AnimationGroupPlayer.prototype.finish = function () {
            this._onFinish();
            this._players.forEach(function (player) {
                return player.finish();
            });
        };
        AnimationGroupPlayer.prototype.destroy = function () {
            this._onFinish();
            this._players.forEach(function (player) {
                return player.destroy();
            });
        };
        AnimationGroupPlayer.prototype.reset = function () {
            this._players.forEach(function (player) {
                return player.reset();
            });
        };
        AnimationGroupPlayer.prototype.setPosition = function (p /** TODO #9100 */) {
            this._players.forEach(function (player) {
                player.setPosition(p);
            });
        };
        AnimationGroupPlayer.prototype.getPosition = function () {
            var min = 0;
            this._players.forEach(function (player) {
                var p = player.getPosition();
                min = math_1.Math.min(p, min);
            });
            return min;
        };
        return AnimationGroupPlayer;
    }();
    exports.AnimationGroupPlayer = AnimationGroupPlayer;
    
});
System.registerDynamic('node_modules/@angular/core/src/animation/active_animation_players_map.js', ['../facade/collection', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var collection_1 = $__require('../facade/collection');
    var lang_1 = $__require('../facade/lang');
    var ActiveAnimationPlayersMap = function () {
        function ActiveAnimationPlayersMap() {
            this._map = new collection_1.Map();
            this._allPlayers = [];
        }
        Object.defineProperty(ActiveAnimationPlayersMap.prototype, "length", {
            get: function () {
                return this.getAllPlayers().length;
            },
            enumerable: true,
            configurable: true
        });
        ActiveAnimationPlayersMap.prototype.find = function (element, animationName) {
            var playersByAnimation = this._map.get(element);
            if (lang_1.isPresent(playersByAnimation)) {
                return playersByAnimation[animationName];
            }
        };
        ActiveAnimationPlayersMap.prototype.findAllPlayersByElement = function (element) {
            var players = [];
            collection_1.StringMapWrapper.forEach(this._map.get(element), function (player /** TODO #9100 */) {
                return players.push(player);
            });
            return players;
        };
        ActiveAnimationPlayersMap.prototype.set = function (element, animationName, player) {
            var playersByAnimation = this._map.get(element);
            if (!lang_1.isPresent(playersByAnimation)) {
                playersByAnimation = {};
            }
            var existingEntry = playersByAnimation[animationName];
            if (lang_1.isPresent(existingEntry)) {
                this.remove(element, animationName);
            }
            playersByAnimation[animationName] = player;
            this._allPlayers.push(player);
            this._map.set(element, playersByAnimation);
        };
        ActiveAnimationPlayersMap.prototype.getAllPlayers = function () {
            return this._allPlayers;
        };
        ActiveAnimationPlayersMap.prototype.remove = function (element, animationName) {
            var playersByAnimation = this._map.get(element);
            if (lang_1.isPresent(playersByAnimation)) {
                var player = playersByAnimation[animationName];
                delete playersByAnimation[animationName];
                var index = this._allPlayers.indexOf(player);
                collection_1.ListWrapper.removeAt(this._allPlayers, index);
                if (collection_1.StringMapWrapper.isEmpty(playersByAnimation)) {
                    this._map.delete(element);
                }
            }
        };
        return ActiveAnimationPlayersMap;
    }();
    exports.ActiveAnimationPlayersMap = ActiveAnimationPlayersMap;
    
});
System.registerDynamic('node_modules/@angular/core/src/linker/view.js', ['../facade/async', '../facade/collection', '../facade/lang', './element', './view_ref', './view_type', './view_utils', '../change_detection/change_detection', '../profile/profile', './exceptions', './debug_context', './element_injector', '../animation/animation_group_player', '../animation/active_animation_players_map'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var async_1 = $__require('../facade/async');
    var collection_1 = $__require('../facade/collection');
    var lang_1 = $__require('../facade/lang');
    var element_1 = $__require('./element');
    var view_ref_1 = $__require('./view_ref');
    var view_type_1 = $__require('./view_type');
    var view_utils_1 = $__require('./view_utils');
    var change_detection_1 = $__require('../change_detection/change_detection');
    var profile_1 = $__require('../profile/profile');
    var exceptions_1 = $__require('./exceptions');
    var debug_context_1 = $__require('./debug_context');
    var element_injector_1 = $__require('./element_injector');
    var animation_group_player_1 = $__require('../animation/animation_group_player');
    var active_animation_players_map_1 = $__require('../animation/active_animation_players_map');
    var _scope_check = profile_1.wtfCreateScope("AppView#check(ascii id)");
    /**
     * Cost of making objects: http://jsperf.com/instantiate-size-of-object
     *
     */
    var AppView = function () {
        function AppView(clazz, componentType, type, viewUtils, parentInjector, declarationAppElement, cdMode) {
            this.clazz = clazz;
            this.componentType = componentType;
            this.type = type;
            this.viewUtils = viewUtils;
            this.parentInjector = parentInjector;
            this.declarationAppElement = declarationAppElement;
            this.cdMode = cdMode;
            this.contentChildren = [];
            this.viewChildren = [];
            this.viewContainerElement = null;
            this.numberOfChecks = 0;
            this.activeAnimationPlayers = new active_animation_players_map_1.ActiveAnimationPlayersMap();
            this.ref = new view_ref_1.ViewRef_(this);
            if (type === view_type_1.ViewType.COMPONENT || type === view_type_1.ViewType.HOST) {
                this.renderer = viewUtils.renderComponent(componentType);
            } else {
                this.renderer = declarationAppElement.parentView.renderer;
            }
        }
        Object.defineProperty(AppView.prototype, "destroyed", {
            get: function () {
                return this.cdMode === change_detection_1.ChangeDetectorStatus.Destroyed;
            },
            enumerable: true,
            configurable: true
        });
        AppView.prototype.cancelActiveAnimation = function (element, animationName, removeAllAnimations) {
            if (removeAllAnimations === void 0) {
                removeAllAnimations = false;
            }
            if (removeAllAnimations) {
                this.activeAnimationPlayers.findAllPlayersByElement(element).forEach(function (player) {
                    return player.destroy();
                });
            } else {
                var player = this.activeAnimationPlayers.find(element, animationName);
                if (lang_1.isPresent(player)) {
                    player.destroy();
                }
            }
        };
        AppView.prototype.registerAndStartAnimation = function (element, animationName, player) {
            var _this = this;
            this.activeAnimationPlayers.set(element, animationName, player);
            player.onDone(function () {
                _this.activeAnimationPlayers.remove(element, animationName);
            });
            player.play();
        };
        AppView.prototype.create = function (context, givenProjectableNodes, rootSelectorOrNode) {
            this.context = context;
            var projectableNodes;
            switch (this.type) {
                case view_type_1.ViewType.COMPONENT:
                    projectableNodes = view_utils_1.ensureSlotCount(givenProjectableNodes, this.componentType.slotCount);
                    break;
                case view_type_1.ViewType.EMBEDDED:
                    projectableNodes = this.declarationAppElement.parentView.projectableNodes;
                    break;
                case view_type_1.ViewType.HOST:
                    // Note: Don't ensure the slot count for the projectableNodes as we store
                    // them only for the contained component view (which will later check the slot count...)
                    projectableNodes = givenProjectableNodes;
                    break;
            }
            this._hasExternalHostElement = lang_1.isPresent(rootSelectorOrNode);
            this.projectableNodes = projectableNodes;
            return this.createInternal(rootSelectorOrNode);
        };
        /**
         * Overwritten by implementations.
         * Returns the AppElement for the host element for ViewType.HOST.
         */
        AppView.prototype.createInternal = function (rootSelectorOrNode) {
            return null;
        };
        AppView.prototype.init = function (rootNodesOrAppElements, allNodes, disposables, subscriptions) {
            this.rootNodesOrAppElements = rootNodesOrAppElements;
            this.allNodes = allNodes;
            this.disposables = disposables;
            this.subscriptions = subscriptions;
            if (this.type === view_type_1.ViewType.COMPONENT) {
                // Note: the render nodes have been attached to their host element
                // in the ViewFactory already.
                this.declarationAppElement.parentView.viewChildren.push(this);
                this.dirtyParentQueriesInternal();
            }
        };
        AppView.prototype.selectOrCreateHostElement = function (elementName, rootSelectorOrNode, debugInfo) {
            var hostElement;
            if (lang_1.isPresent(rootSelectorOrNode)) {
                hostElement = this.renderer.selectRootElement(rootSelectorOrNode, debugInfo);
            } else {
                hostElement = this.renderer.createElement(null, elementName, debugInfo);
            }
            return hostElement;
        };
        AppView.prototype.injectorGet = function (token, nodeIndex, notFoundResult) {
            return this.injectorGetInternal(token, nodeIndex, notFoundResult);
        };
        /**
         * Overwritten by implementations
         */
        AppView.prototype.injectorGetInternal = function (token, nodeIndex, notFoundResult) {
            return notFoundResult;
        };
        AppView.prototype.injector = function (nodeIndex) {
            if (lang_1.isPresent(nodeIndex)) {
                return new element_injector_1.ElementInjector(this, nodeIndex);
            } else {
                return this.parentInjector;
            }
        };
        AppView.prototype.destroy = function () {
            if (this._hasExternalHostElement) {
                this.renderer.detachView(this.flatRootNodes);
            } else if (lang_1.isPresent(this.viewContainerElement)) {
                this.viewContainerElement.detachView(this.viewContainerElement.nestedViews.indexOf(this));
            }
            this._destroyRecurse();
        };
        AppView.prototype._destroyRecurse = function () {
            if (this.cdMode === change_detection_1.ChangeDetectorStatus.Destroyed) {
                return;
            }
            var children = this.contentChildren;
            for (var i = 0; i < children.length; i++) {
                children[i]._destroyRecurse();
            }
            children = this.viewChildren;
            for (var i = 0; i < children.length; i++) {
                children[i]._destroyRecurse();
            }
            this.destroyLocal();
            this.cdMode = change_detection_1.ChangeDetectorStatus.Destroyed;
        };
        AppView.prototype.destroyLocal = function () {
            var _this = this;
            var hostElement = this.type === view_type_1.ViewType.COMPONENT ? this.declarationAppElement.nativeElement : null;
            for (var i = 0; i < this.disposables.length; i++) {
                this.disposables[i]();
            }
            for (var i = 0; i < this.subscriptions.length; i++) {
                async_1.ObservableWrapper.dispose(this.subscriptions[i]);
            }
            this.destroyInternal();
            this.dirtyParentQueriesInternal();
            if (this.activeAnimationPlayers.length == 0) {
                this.renderer.destroyView(hostElement, this.allNodes);
            } else {
                var player = new animation_group_player_1.AnimationGroupPlayer(this.activeAnimationPlayers.getAllPlayers());
                player.onDone(function () {
                    _this.renderer.destroyView(hostElement, _this.allNodes);
                });
            }
        };
        /**
         * Overwritten by implementations
         */
        AppView.prototype.destroyInternal = function () {};
        /**
         * Overwritten by implementations
         */
        AppView.prototype.detachInternal = function () {};
        AppView.prototype.detach = function () {
            var _this = this;
            this.detachInternal();
            if (this.activeAnimationPlayers.length == 0) {
                this.renderer.detachView(this.flatRootNodes);
            } else {
                var player = new animation_group_player_1.AnimationGroupPlayer(this.activeAnimationPlayers.getAllPlayers());
                player.onDone(function () {
                    _this.renderer.detachView(_this.flatRootNodes);
                });
            }
        };
        Object.defineProperty(AppView.prototype, "changeDetectorRef", {
            get: function () {
                return this.ref;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AppView.prototype, "parent", {
            get: function () {
                return lang_1.isPresent(this.declarationAppElement) ? this.declarationAppElement.parentView : null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AppView.prototype, "flatRootNodes", {
            get: function () {
                return view_utils_1.flattenNestedViewRenderNodes(this.rootNodesOrAppElements);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AppView.prototype, "lastRootNode", {
            get: function () {
                var lastNode = this.rootNodesOrAppElements.length > 0 ? this.rootNodesOrAppElements[this.rootNodesOrAppElements.length - 1] : null;
                return _findLastRenderNode(lastNode);
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Overwritten by implementations
         */
        AppView.prototype.dirtyParentQueriesInternal = function () {};
        AppView.prototype.detectChanges = function (throwOnChange) {
            var s = _scope_check(this.clazz);
            if (this.cdMode === change_detection_1.ChangeDetectorStatus.Checked || this.cdMode === change_detection_1.ChangeDetectorStatus.Errored) return;
            if (this.cdMode === change_detection_1.ChangeDetectorStatus.Destroyed) {
                this.throwDestroyedError('detectChanges');
            }
            this.detectChangesInternal(throwOnChange);
            if (this.cdMode === change_detection_1.ChangeDetectorStatus.CheckOnce) this.cdMode = change_detection_1.ChangeDetectorStatus.Checked;
            this.numberOfChecks++;
            profile_1.wtfLeave(s);
        };
        /**
         * Overwritten by implementations
         */
        AppView.prototype.detectChangesInternal = function (throwOnChange) {
            this.detectContentChildrenChanges(throwOnChange);
            this.detectViewChildrenChanges(throwOnChange);
        };
        AppView.prototype.detectContentChildrenChanges = function (throwOnChange) {
            for (var i = 0; i < this.contentChildren.length; ++i) {
                var child = this.contentChildren[i];
                if (child.cdMode === change_detection_1.ChangeDetectorStatus.Detached) continue;
                child.detectChanges(throwOnChange);
            }
        };
        AppView.prototype.detectViewChildrenChanges = function (throwOnChange) {
            for (var i = 0; i < this.viewChildren.length; ++i) {
                var child = this.viewChildren[i];
                if (child.cdMode === change_detection_1.ChangeDetectorStatus.Detached) continue;
                child.detectChanges(throwOnChange);
            }
        };
        AppView.prototype.addToContentChildren = function (renderAppElement) {
            renderAppElement.parentView.contentChildren.push(this);
            this.viewContainerElement = renderAppElement;
            this.dirtyParentQueriesInternal();
        };
        AppView.prototype.removeFromContentChildren = function (renderAppElement) {
            collection_1.ListWrapper.remove(renderAppElement.parentView.contentChildren, this);
            this.dirtyParentQueriesInternal();
            this.viewContainerElement = null;
        };
        AppView.prototype.markAsCheckOnce = function () {
            this.cdMode = change_detection_1.ChangeDetectorStatus.CheckOnce;
        };
        AppView.prototype.markPathToRootAsCheckOnce = function () {
            var c = this;
            while (lang_1.isPresent(c) && c.cdMode !== change_detection_1.ChangeDetectorStatus.Detached) {
                if (c.cdMode === change_detection_1.ChangeDetectorStatus.Checked) {
                    c.cdMode = change_detection_1.ChangeDetectorStatus.CheckOnce;
                }
                var parentEl = c.type === view_type_1.ViewType.COMPONENT ? c.declarationAppElement : c.viewContainerElement;
                c = lang_1.isPresent(parentEl) ? parentEl.parentView : null;
            }
        };
        AppView.prototype.eventHandler = function (cb) {
            return cb;
        };
        AppView.prototype.throwDestroyedError = function (details) {
            throw new exceptions_1.ViewDestroyedException(details);
        };
        return AppView;
    }();
    exports.AppView = AppView;
    var DebugAppView = function (_super) {
        __extends(DebugAppView, _super);
        function DebugAppView(clazz, componentType, type, viewUtils, parentInjector, declarationAppElement, cdMode, staticNodeDebugInfos) {
            _super.call(this, clazz, componentType, type, viewUtils, parentInjector, declarationAppElement, cdMode);
            this.staticNodeDebugInfos = staticNodeDebugInfos;
            this._currentDebugContext = null;
        }
        DebugAppView.prototype.create = function (context, givenProjectableNodes, rootSelectorOrNode) {
            this._resetDebug();
            try {
                return _super.prototype.create.call(this, context, givenProjectableNodes, rootSelectorOrNode);
            } catch (e) {
                this._rethrowWithContext(e, e.stack);
                throw e;
            }
        };
        DebugAppView.prototype.injectorGet = function (token, nodeIndex, notFoundResult) {
            this._resetDebug();
            try {
                return _super.prototype.injectorGet.call(this, token, nodeIndex, notFoundResult);
            } catch (e) {
                this._rethrowWithContext(e, e.stack);
                throw e;
            }
        };
        DebugAppView.prototype.detach = function () {
            this._resetDebug();
            try {
                _super.prototype.detach.call(this);
            } catch (e) {
                this._rethrowWithContext(e, e.stack);
                throw e;
            }
        };
        DebugAppView.prototype.destroyLocal = function () {
            this._resetDebug();
            try {
                _super.prototype.destroyLocal.call(this);
            } catch (e) {
                this._rethrowWithContext(e, e.stack);
                throw e;
            }
        };
        DebugAppView.prototype.detectChanges = function (throwOnChange) {
            this._resetDebug();
            try {
                _super.prototype.detectChanges.call(this, throwOnChange);
            } catch (e) {
                this._rethrowWithContext(e, e.stack);
                throw e;
            }
        };
        DebugAppView.prototype._resetDebug = function () {
            this._currentDebugContext = null;
        };
        DebugAppView.prototype.debug = function (nodeIndex, rowNum, colNum) {
            return this._currentDebugContext = new debug_context_1.DebugContext(this, nodeIndex, rowNum, colNum);
        };
        DebugAppView.prototype._rethrowWithContext = function (e, stack) {
            if (!(e instanceof exceptions_1.ViewWrappedException)) {
                if (!(e instanceof exceptions_1.ExpressionChangedAfterItHasBeenCheckedException)) {
                    this.cdMode = change_detection_1.ChangeDetectorStatus.Errored;
                }
                if (lang_1.isPresent(this._currentDebugContext)) {
                    throw new exceptions_1.ViewWrappedException(e, stack, this._currentDebugContext);
                }
            }
        };
        DebugAppView.prototype.eventHandler = function (cb) {
            var _this = this;
            var superHandler = _super.prototype.eventHandler.call(this, cb);
            return function (event /** TODO #9100 */) {
                _this._resetDebug();
                try {
                    return superHandler(event);
                } catch (e) {
                    _this._rethrowWithContext(e, e.stack);
                    throw e;
                }
            };
        };
        return DebugAppView;
    }(AppView);
    exports.DebugAppView = DebugAppView;
    function _findLastRenderNode(node) {
        var lastNode;
        if (node instanceof element_1.AppElement) {
            var appEl = node;
            lastNode = appEl.nativeElement;
            if (lang_1.isPresent(appEl.nestedViews)) {
                // Note: Views might have no root nodes at all!
                for (var i = appEl.nestedViews.length - 1; i >= 0; i--) {
                    var nestedView = appEl.nestedViews[i];
                    if (nestedView.rootNodesOrAppElements.length > 0) {
                        lastNode = _findLastRenderNode(nestedView.rootNodesOrAppElements[nestedView.rootNodesOrAppElements.length - 1]);
                    }
                }
            }
        } else {
            lastNode = node;
        }
        return lastNode;
    }
    
});
System.registerDynamic('node_modules/@angular/core/src/application_tokens.js', ['../src/facade/lang', './di'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var lang_1 = $__require('../src/facade/lang');
  var di_1 = $__require('./di');
  /**
   * A DI Token representing a unique string id assigned to the application by Angular and used
   * primarily for prefixing application attributes and CSS styles when
   * {@link ViewEncapsulation#Emulated} is being used.
   *
   * If you need to avoid randomly generated value to be used as an application id, you can provide
   * a custom value via a DI provider <!-- TODO: provider --> configuring the root {@link Injector}
   * using this token.
   * @experimental
   */
  exports.APP_ID = new di_1.OpaqueToken('AppId');
  function _appIdRandomProviderFactory() {
    return "" + _randomChar() + _randomChar() + _randomChar();
  }
  /**
   * Providers that will generate a random APP_ID_TOKEN.
   * @experimental
   */
  exports.APP_ID_RANDOM_PROVIDER =
  /*@ts2dart_const*/ /* @ts2dart_Provider */{
    provide: exports.APP_ID,
    useFactory: _appIdRandomProviderFactory,
    deps: []
  };
  function _randomChar() {
    return lang_1.StringWrapper.fromCharCode(97 + lang_1.Math.floor(lang_1.Math.random() * 25));
  }
  /**
   * A function that will be executed when a platform is initialized.
   * @experimental
   */
  exports.PLATFORM_INITIALIZER =
  /*@ts2dart_const*/new di_1.OpaqueToken('Platform Initializer');
  /**
   * A function that will be executed when an application is initialized.
   * @experimental
   */
  exports.APP_INITIALIZER =
  /*@ts2dart_const*/new di_1.OpaqueToken('Application Initializer');
  /**
   * A token which indicates the root directory of the application
   * @experimental
   */
  exports.PACKAGE_ROOT_URL =
  /*@ts2dart_const*/new di_1.OpaqueToken('Application Packages Root URL');
  
});
System.registerDynamic('node_modules/@angular/core/src/change_detection/differs/default_iterable_differ.js', ['../../facade/collection', '../../facade/exceptions', '../../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var collection_1 = $__require('../../facade/collection');
    var exceptions_1 = $__require('../../facade/exceptions');
    var lang_1 = $__require('../../facade/lang');
    /* @ts2dart_const */
    var DefaultIterableDifferFactory = function () {
        function DefaultIterableDifferFactory() {}
        DefaultIterableDifferFactory.prototype.supports = function (obj) {
            return collection_1.isListLikeIterable(obj);
        };
        DefaultIterableDifferFactory.prototype.create = function (cdRef, trackByFn) {
            return new DefaultIterableDiffer(trackByFn);
        };
        return DefaultIterableDifferFactory;
    }();
    exports.DefaultIterableDifferFactory = DefaultIterableDifferFactory;
    var trackByIdentity = function (index, item) {
        return item;
    };
    /**
     * @stable
     */
    var DefaultIterableDiffer = function () {
        function DefaultIterableDiffer(_trackByFn) {
            this._trackByFn = _trackByFn;
            this._length = null;
            this._collection = null;
            // Keeps track of the used records at any point in time (during & across `_check()` calls)
            this._linkedRecords = null;
            // Keeps track of the removed records at any point in time during `_check()` calls.
            this._unlinkedRecords = null;
            this._previousItHead = null;
            this._itHead = null;
            this._itTail = null;
            this._additionsHead = null;
            this._additionsTail = null;
            this._movesHead = null;
            this._movesTail = null;
            this._removalsHead = null;
            this._removalsTail = null;
            // Keeps track of records where custom track by is the same, but item identity has changed
            this._identityChangesHead = null;
            this._identityChangesTail = null;
            this._trackByFn = lang_1.isPresent(this._trackByFn) ? this._trackByFn : trackByIdentity;
        }
        Object.defineProperty(DefaultIterableDiffer.prototype, "collection", {
            get: function () {
                return this._collection;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(DefaultIterableDiffer.prototype, "length", {
            get: function () {
                return this._length;
            },
            enumerable: true,
            configurable: true
        });
        DefaultIterableDiffer.prototype.forEachItem = function (fn) {
            var record;
            for (record = this._itHead; record !== null; record = record._next) {
                fn(record);
            }
        };
        DefaultIterableDiffer.prototype.forEachPreviousItem = function (fn) {
            var record;
            for (record = this._previousItHead; record !== null; record = record._nextPrevious) {
                fn(record);
            }
        };
        DefaultIterableDiffer.prototype.forEachAddedItem = function (fn) {
            var record;
            for (record = this._additionsHead; record !== null; record = record._nextAdded) {
                fn(record);
            }
        };
        DefaultIterableDiffer.prototype.forEachMovedItem = function (fn) {
            var record;
            for (record = this._movesHead; record !== null; record = record._nextMoved) {
                fn(record);
            }
        };
        DefaultIterableDiffer.prototype.forEachRemovedItem = function (fn) {
            var record;
            for (record = this._removalsHead; record !== null; record = record._nextRemoved) {
                fn(record);
            }
        };
        DefaultIterableDiffer.prototype.forEachIdentityChange = function (fn) {
            var record;
            for (record = this._identityChangesHead; record !== null; record = record._nextIdentityChange) {
                fn(record);
            }
        };
        DefaultIterableDiffer.prototype.diff = function (collection) {
            if (lang_1.isBlank(collection)) collection = [];
            if (!collection_1.isListLikeIterable(collection)) {
                throw new exceptions_1.BaseException("Error trying to diff '" + collection + "'");
            }
            if (this.check(collection)) {
                return this;
            } else {
                return null;
            }
        };
        DefaultIterableDiffer.prototype.onDestroy = function () {};
        // todo(vicb): optim for UnmodifiableListView (frozen arrays)
        DefaultIterableDiffer.prototype.check = function (collection) {
            var _this = this;
            this._reset();
            var record = this._itHead;
            var mayBeDirty = false;
            var index;
            var item;
            var itemTrackBy;
            if (lang_1.isArray(collection)) {
                var list = collection;
                this._length = collection.length;
                for (index = 0; index < this._length; index++) {
                    item = list[index];
                    itemTrackBy = this._trackByFn(index, item);
                    if (record === null || !lang_1.looseIdentical(record.trackById, itemTrackBy)) {
                        record = this._mismatch(record, item, itemTrackBy, index);
                        mayBeDirty = true;
                    } else {
                        if (mayBeDirty) {
                            // TODO(misko): can we limit this to duplicates only?
                            record = this._verifyReinsertion(record, item, itemTrackBy, index);
                        }
                        if (!lang_1.looseIdentical(record.item, item)) this._addIdentityChange(record, item);
                    }
                    record = record._next;
                }
            } else {
                index = 0;
                collection_1.iterateListLike(collection, function (item /** TODO #9100 */) {
                    itemTrackBy = _this._trackByFn(index, item);
                    if (record === null || !lang_1.looseIdentical(record.trackById, itemTrackBy)) {
                        record = _this._mismatch(record, item, itemTrackBy, index);
                        mayBeDirty = true;
                    } else {
                        if (mayBeDirty) {
                            // TODO(misko): can we limit this to duplicates only?
                            record = _this._verifyReinsertion(record, item, itemTrackBy, index);
                        }
                        if (!lang_1.looseIdentical(record.item, item)) _this._addIdentityChange(record, item);
                    }
                    record = record._next;
                    index++;
                });
                this._length = index;
            }
            this._truncate(record);
            this._collection = collection;
            return this.isDirty;
        };
        Object.defineProperty(DefaultIterableDiffer.prototype, "isDirty", {
            /* CollectionChanges is considered dirty if it has any additions, moves, removals, or identity
             * changes.
             */
            get: function () {
                return this._additionsHead !== null || this._movesHead !== null || this._removalsHead !== null || this._identityChangesHead !== null;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Reset the state of the change objects to show no changes. This means set previousKey to
         * currentKey, and clear all of the queues (additions, moves, removals).
         * Set the previousIndexes of moved and added items to their currentIndexes
         * Reset the list of additions, moves and removals
         *
         * @internal
         */
        DefaultIterableDiffer.prototype._reset = function () {
            if (this.isDirty) {
                var record;
                var nextRecord;
                for (record = this._previousItHead = this._itHead; record !== null; record = record._next) {
                    record._nextPrevious = record._next;
                }
                for (record = this._additionsHead; record !== null; record = record._nextAdded) {
                    record.previousIndex = record.currentIndex;
                }
                this._additionsHead = this._additionsTail = null;
                for (record = this._movesHead; record !== null; record = nextRecord) {
                    record.previousIndex = record.currentIndex;
                    nextRecord = record._nextMoved;
                }
                this._movesHead = this._movesTail = null;
                this._removalsHead = this._removalsTail = null;
                this._identityChangesHead = this._identityChangesTail = null;
            }
        };
        /**
         * This is the core function which handles differences between collections.
         *
         * - `record` is the record which we saw at this position last time. If null then it is a new
         *   item.
         * - `item` is the current item in the collection
         * - `index` is the position of the item in the collection
         *
         * @internal
         */
        DefaultIterableDiffer.prototype._mismatch = function (record, item, itemTrackBy, index) {
            // The previous record after which we will append the current one.
            var previousRecord;
            if (record === null) {
                previousRecord = this._itTail;
            } else {
                previousRecord = record._prev;
                // Remove the record from the collection since we know it does not match the item.
                this._remove(record);
            }
            // Attempt to see if we have seen the item before.
            record = this._linkedRecords === null ? null : this._linkedRecords.get(itemTrackBy, index);
            if (record !== null) {
                // We have seen this before, we need to move it forward in the collection.
                // But first we need to check if identity changed, so we can update in view if necessary
                if (!lang_1.looseIdentical(record.item, item)) this._addIdentityChange(record, item);
                this._moveAfter(record, previousRecord, index);
            } else {
                // Never seen it, check evicted list.
                record = this._unlinkedRecords === null ? null : this._unlinkedRecords.get(itemTrackBy);
                if (record !== null) {
                    // It is an item which we have evicted earlier: reinsert it back into the list.
                    // But first we need to check if identity changed, so we can update in view if necessary
                    if (!lang_1.looseIdentical(record.item, item)) this._addIdentityChange(record, item);
                    this._reinsertAfter(record, previousRecord, index);
                } else {
                    // It is a new item: add it.
                    record = this._addAfter(new CollectionChangeRecord(item, itemTrackBy), previousRecord, index);
                }
            }
            return record;
        };
        /**
         * This check is only needed if an array contains duplicates. (Short circuit of nothing dirty)
         *
         * Use case: `[a, a]` => `[b, a, a]`
         *
         * If we did not have this check then the insertion of `b` would:
         *   1) evict first `a`
         *   2) insert `b` at `0` index.
         *   3) leave `a` at index `1` as is. <-- this is wrong!
         *   3) reinsert `a` at index 2. <-- this is wrong!
         *
         * The correct behavior is:
         *   1) evict first `a`
         *   2) insert `b` at `0` index.
         *   3) reinsert `a` at index 1.
         *   3) move `a` at from `1` to `2`.
         *
         *
         * Double check that we have not evicted a duplicate item. We need to check if the item type may
         * have already been removed:
         * The insertion of b will evict the first 'a'. If we don't reinsert it now it will be reinserted
         * at the end. Which will show up as the two 'a's switching position. This is incorrect, since a
         * better way to think of it is as insert of 'b' rather then switch 'a' with 'b' and then add 'a'
         * at the end.
         *
         * @internal
         */
        DefaultIterableDiffer.prototype._verifyReinsertion = function (record, item, itemTrackBy, index) {
            var reinsertRecord = this._unlinkedRecords === null ? null : this._unlinkedRecords.get(itemTrackBy);
            if (reinsertRecord !== null) {
                record = this._reinsertAfter(reinsertRecord, record._prev, index);
            } else if (record.currentIndex != index) {
                record.currentIndex = index;
                this._addToMoves(record, index);
            }
            return record;
        };
        /**
         * Get rid of any excess {@link CollectionChangeRecord}s from the previous collection
         *
         * - `record` The first excess {@link CollectionChangeRecord}.
         *
         * @internal
         */
        DefaultIterableDiffer.prototype._truncate = function (record) {
            // Anything after that needs to be removed;
            while (record !== null) {
                var nextRecord = record._next;
                this._addToRemovals(this._unlink(record));
                record = nextRecord;
            }
            if (this._unlinkedRecords !== null) {
                this._unlinkedRecords.clear();
            }
            if (this._additionsTail !== null) {
                this._additionsTail._nextAdded = null;
            }
            if (this._movesTail !== null) {
                this._movesTail._nextMoved = null;
            }
            if (this._itTail !== null) {
                this._itTail._next = null;
            }
            if (this._removalsTail !== null) {
                this._removalsTail._nextRemoved = null;
            }
            if (this._identityChangesTail !== null) {
                this._identityChangesTail._nextIdentityChange = null;
            }
        };
        /** @internal */
        DefaultIterableDiffer.prototype._reinsertAfter = function (record, prevRecord, index) {
            if (this._unlinkedRecords !== null) {
                this._unlinkedRecords.remove(record);
            }
            var prev = record._prevRemoved;
            var next = record._nextRemoved;
            if (prev === null) {
                this._removalsHead = next;
            } else {
                prev._nextRemoved = next;
            }
            if (next === null) {
                this._removalsTail = prev;
            } else {
                next._prevRemoved = prev;
            }
            this._insertAfter(record, prevRecord, index);
            this._addToMoves(record, index);
            return record;
        };
        /** @internal */
        DefaultIterableDiffer.prototype._moveAfter = function (record, prevRecord, index) {
            this._unlink(record);
            this._insertAfter(record, prevRecord, index);
            this._addToMoves(record, index);
            return record;
        };
        /** @internal */
        DefaultIterableDiffer.prototype._addAfter = function (record, prevRecord, index) {
            this._insertAfter(record, prevRecord, index);
            if (this._additionsTail === null) {
                // todo(vicb)
                // assert(this._additionsHead === null);
                this._additionsTail = this._additionsHead = record;
            } else {
                // todo(vicb)
                // assert(_additionsTail._nextAdded === null);
                // assert(record._nextAdded === null);
                this._additionsTail = this._additionsTail._nextAdded = record;
            }
            return record;
        };
        /** @internal */
        DefaultIterableDiffer.prototype._insertAfter = function (record, prevRecord, index) {
            // todo(vicb)
            // assert(record != prevRecord);
            // assert(record._next === null);
            // assert(record._prev === null);
            var next = prevRecord === null ? this._itHead : prevRecord._next;
            // todo(vicb)
            // assert(next != record);
            // assert(prevRecord != record);
            record._next = next;
            record._prev = prevRecord;
            if (next === null) {
                this._itTail = record;
            } else {
                next._prev = record;
            }
            if (prevRecord === null) {
                this._itHead = record;
            } else {
                prevRecord._next = record;
            }
            if (this._linkedRecords === null) {
                this._linkedRecords = new _DuplicateMap();
            }
            this._linkedRecords.put(record);
            record.currentIndex = index;
            return record;
        };
        /** @internal */
        DefaultIterableDiffer.prototype._remove = function (record) {
            return this._addToRemovals(this._unlink(record));
        };
        /** @internal */
        DefaultIterableDiffer.prototype._unlink = function (record) {
            if (this._linkedRecords !== null) {
                this._linkedRecords.remove(record);
            }
            var prev = record._prev;
            var next = record._next;
            // todo(vicb)
            // assert((record._prev = null) === null);
            // assert((record._next = null) === null);
            if (prev === null) {
                this._itHead = next;
            } else {
                prev._next = next;
            }
            if (next === null) {
                this._itTail = prev;
            } else {
                next._prev = prev;
            }
            return record;
        };
        /** @internal */
        DefaultIterableDiffer.prototype._addToMoves = function (record, toIndex) {
            // todo(vicb)
            // assert(record._nextMoved === null);
            if (record.previousIndex === toIndex) {
                return record;
            }
            if (this._movesTail === null) {
                // todo(vicb)
                // assert(_movesHead === null);
                this._movesTail = this._movesHead = record;
            } else {
                // todo(vicb)
                // assert(_movesTail._nextMoved === null);
                this._movesTail = this._movesTail._nextMoved = record;
            }
            return record;
        };
        /** @internal */
        DefaultIterableDiffer.prototype._addToRemovals = function (record) {
            if (this._unlinkedRecords === null) {
                this._unlinkedRecords = new _DuplicateMap();
            }
            this._unlinkedRecords.put(record);
            record.currentIndex = null;
            record._nextRemoved = null;
            if (this._removalsTail === null) {
                // todo(vicb)
                // assert(_removalsHead === null);
                this._removalsTail = this._removalsHead = record;
                record._prevRemoved = null;
            } else {
                // todo(vicb)
                // assert(_removalsTail._nextRemoved === null);
                // assert(record._nextRemoved === null);
                record._prevRemoved = this._removalsTail;
                this._removalsTail = this._removalsTail._nextRemoved = record;
            }
            return record;
        };
        /** @internal */
        DefaultIterableDiffer.prototype._addIdentityChange = function (record, item) {
            record.item = item;
            if (this._identityChangesTail === null) {
                this._identityChangesTail = this._identityChangesHead = record;
            } else {
                this._identityChangesTail = this._identityChangesTail._nextIdentityChange = record;
            }
            return record;
        };
        DefaultIterableDiffer.prototype.toString = function () {
            var list = [];
            this.forEachItem(function (record /** TODO #9100 */) {
                return list.push(record);
            });
            var previous = [];
            this.forEachPreviousItem(function (record /** TODO #9100 */) {
                return previous.push(record);
            });
            var additions = [];
            this.forEachAddedItem(function (record /** TODO #9100 */) {
                return additions.push(record);
            });
            var moves = [];
            this.forEachMovedItem(function (record /** TODO #9100 */) {
                return moves.push(record);
            });
            var removals = [];
            this.forEachRemovedItem(function (record /** TODO #9100 */) {
                return removals.push(record);
            });
            var identityChanges = [];
            this.forEachIdentityChange(function (record /** TODO #9100 */) {
                return identityChanges.push(record);
            });
            return 'collection: ' + list.join(', ') + '\n' + 'previous: ' + previous.join(', ') + '\n' + 'additions: ' + additions.join(', ') + '\n' + 'moves: ' + moves.join(', ') + '\n' + 'removals: ' + removals.join(', ') + '\n' + 'identityChanges: ' + identityChanges.join(', ') + '\n';
        };
        return DefaultIterableDiffer;
    }();
    exports.DefaultIterableDiffer = DefaultIterableDiffer;
    /**
     * @stable
     */
    var CollectionChangeRecord = function () {
        function CollectionChangeRecord(item, trackById) {
            this.item = item;
            this.trackById = trackById;
            this.currentIndex = null;
            this.previousIndex = null;
            /** @internal */
            this._nextPrevious = null;
            /** @internal */
            this._prev = null;
            /** @internal */
            this._next = null;
            /** @internal */
            this._prevDup = null;
            /** @internal */
            this._nextDup = null;
            /** @internal */
            this._prevRemoved = null;
            /** @internal */
            this._nextRemoved = null;
            /** @internal */
            this._nextAdded = null;
            /** @internal */
            this._nextMoved = null;
            /** @internal */
            this._nextIdentityChange = null;
        }
        CollectionChangeRecord.prototype.toString = function () {
            return this.previousIndex === this.currentIndex ? lang_1.stringify(this.item) : lang_1.stringify(this.item) + '[' + lang_1.stringify(this.previousIndex) + '->' + lang_1.stringify(this.currentIndex) + ']';
        };
        return CollectionChangeRecord;
    }();
    exports.CollectionChangeRecord = CollectionChangeRecord;
    // A linked list of CollectionChangeRecords with the same CollectionChangeRecord.item
    var _DuplicateItemRecordList = function () {
        function _DuplicateItemRecordList() {
            /** @internal */
            this._head = null;
            /** @internal */
            this._tail = null;
        }
        /**
         * Append the record to the list of duplicates.
         *
         * Note: by design all records in the list of duplicates hold the same value in record.item.
         */
        _DuplicateItemRecordList.prototype.add = function (record) {
            if (this._head === null) {
                this._head = this._tail = record;
                record._nextDup = null;
                record._prevDup = null;
            } else {
                // todo(vicb)
                // assert(record.item ==  _head.item ||
                //       record.item is num && record.item.isNaN && _head.item is num && _head.item.isNaN);
                this._tail._nextDup = record;
                record._prevDup = this._tail;
                record._nextDup = null;
                this._tail = record;
            }
        };
        // Returns a CollectionChangeRecord having CollectionChangeRecord.trackById == trackById and
        // CollectionChangeRecord.currentIndex >= afterIndex
        _DuplicateItemRecordList.prototype.get = function (trackById, afterIndex) {
            var record;
            for (record = this._head; record !== null; record = record._nextDup) {
                if ((afterIndex === null || afterIndex < record.currentIndex) && lang_1.looseIdentical(record.trackById, trackById)) {
                    return record;
                }
            }
            return null;
        };
        /**
         * Remove one {@link CollectionChangeRecord} from the list of duplicates.
         *
         * Returns whether the list of duplicates is empty.
         */
        _DuplicateItemRecordList.prototype.remove = function (record) {
            // todo(vicb)
            // assert(() {
            //  // verify that the record being removed is in the list.
            //  for (CollectionChangeRecord cursor = _head; cursor != null; cursor = cursor._nextDup) {
            //    if (identical(cursor, record)) return true;
            //  }
            //  return false;
            //});
            var prev = record._prevDup;
            var next = record._nextDup;
            if (prev === null) {
                this._head = next;
            } else {
                prev._nextDup = next;
            }
            if (next === null) {
                this._tail = prev;
            } else {
                next._prevDup = prev;
            }
            return this._head === null;
        };
        return _DuplicateItemRecordList;
    }();
    var _DuplicateMap = function () {
        function _DuplicateMap() {
            this.map = new Map();
        }
        _DuplicateMap.prototype.put = function (record) {
            // todo(vicb) handle corner cases
            var key = lang_1.getMapKey(record.trackById);
            var duplicates = this.map.get(key);
            if (!lang_1.isPresent(duplicates)) {
                duplicates = new _DuplicateItemRecordList();
                this.map.set(key, duplicates);
            }
            duplicates.add(record);
        };
        /**
         * Retrieve the `value` using key. Because the CollectionChangeRecord value may be one which we
         * have already iterated over, we use the afterIndex to pretend it is not there.
         *
         * Use case: `[a, b, c, a, a]` if we are at index `3` which is the second `a` then asking if we
         * have any more `a`s needs to return the last `a` not the first or second.
         */
        _DuplicateMap.prototype.get = function (trackById, afterIndex) {
            if (afterIndex === void 0) {
                afterIndex = null;
            }
            var key = lang_1.getMapKey(trackById);
            var recordList = this.map.get(key);
            return lang_1.isBlank(recordList) ? null : recordList.get(trackById, afterIndex);
        };
        /**
         * Removes a {@link CollectionChangeRecord} from the list of duplicates.
         *
         * The list of duplicates also is removed from the map if it gets empty.
         */
        _DuplicateMap.prototype.remove = function (record) {
            var key = lang_1.getMapKey(record.trackById);
            // todo(vicb)
            // assert(this.map.containsKey(key));
            var recordList = this.map.get(key);
            // Remove the list of duplicates when it gets empty
            if (recordList.remove(record)) {
                this.map.delete(key);
            }
            return record;
        };
        Object.defineProperty(_DuplicateMap.prototype, "isEmpty", {
            get: function () {
                return this.map.size === 0;
            },
            enumerable: true,
            configurable: true
        });
        _DuplicateMap.prototype.clear = function () {
            this.map.clear();
        };
        _DuplicateMap.prototype.toString = function () {
            return '_DuplicateMap(' + lang_1.stringify(this.map) + ')';
        };
        return _DuplicateMap;
    }();
    
});
System.registerDynamic('node_modules/@angular/core/src/change_detection/differs/default_keyvalue_differ.js', ['../../facade/collection', '../../facade/exceptions', '../../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var collection_1 = $__require('../../facade/collection');
    var exceptions_1 = $__require('../../facade/exceptions');
    var lang_1 = $__require('../../facade/lang');
    /* @ts2dart_const */
    var DefaultKeyValueDifferFactory = function () {
        function DefaultKeyValueDifferFactory() {}
        DefaultKeyValueDifferFactory.prototype.supports = function (obj) {
            return obj instanceof Map || lang_1.isJsObject(obj);
        };
        DefaultKeyValueDifferFactory.prototype.create = function (cdRef) {
            return new DefaultKeyValueDiffer();
        };
        return DefaultKeyValueDifferFactory;
    }();
    exports.DefaultKeyValueDifferFactory = DefaultKeyValueDifferFactory;
    var DefaultKeyValueDiffer = function () {
        function DefaultKeyValueDiffer() {
            this._records = new Map();
            this._mapHead = null;
            this._previousMapHead = null;
            this._changesHead = null;
            this._changesTail = null;
            this._additionsHead = null;
            this._additionsTail = null;
            this._removalsHead = null;
            this._removalsTail = null;
        }
        Object.defineProperty(DefaultKeyValueDiffer.prototype, "isDirty", {
            get: function () {
                return this._additionsHead !== null || this._changesHead !== null || this._removalsHead !== null;
            },
            enumerable: true,
            configurable: true
        });
        DefaultKeyValueDiffer.prototype.forEachItem = function (fn) {
            var record;
            for (record = this._mapHead; record !== null; record = record._next) {
                fn(record);
            }
        };
        DefaultKeyValueDiffer.prototype.forEachPreviousItem = function (fn) {
            var record;
            for (record = this._previousMapHead; record !== null; record = record._nextPrevious) {
                fn(record);
            }
        };
        DefaultKeyValueDiffer.prototype.forEachChangedItem = function (fn) {
            var record;
            for (record = this._changesHead; record !== null; record = record._nextChanged) {
                fn(record);
            }
        };
        DefaultKeyValueDiffer.prototype.forEachAddedItem = function (fn) {
            var record;
            for (record = this._additionsHead; record !== null; record = record._nextAdded) {
                fn(record);
            }
        };
        DefaultKeyValueDiffer.prototype.forEachRemovedItem = function (fn) {
            var record;
            for (record = this._removalsHead; record !== null; record = record._nextRemoved) {
                fn(record);
            }
        };
        DefaultKeyValueDiffer.prototype.diff = function (map) {
            if (lang_1.isBlank(map)) map = collection_1.MapWrapper.createFromPairs([]);
            if (!(map instanceof Map || lang_1.isJsObject(map))) {
                throw new exceptions_1.BaseException("Error trying to diff '" + map + "'");
            }
            if (this.check(map)) {
                return this;
            } else {
                return null;
            }
        };
        DefaultKeyValueDiffer.prototype.onDestroy = function () {};
        DefaultKeyValueDiffer.prototype.check = function (map) {
            var _this = this;
            this._reset();
            var records = this._records;
            var oldSeqRecord = this._mapHead;
            var lastOldSeqRecord = null;
            var lastNewSeqRecord = null;
            var seqChanged = false;
            this._forEach(map, function (value /** TODO #9100 */, key /** TODO #9100 */) {
                var newSeqRecord;
                if (oldSeqRecord !== null && key === oldSeqRecord.key) {
                    newSeqRecord = oldSeqRecord;
                    if (!lang_1.looseIdentical(value, oldSeqRecord.currentValue)) {
                        oldSeqRecord.previousValue = oldSeqRecord.currentValue;
                        oldSeqRecord.currentValue = value;
                        _this._addToChanges(oldSeqRecord);
                    }
                } else {
                    seqChanged = true;
                    if (oldSeqRecord !== null) {
                        oldSeqRecord._next = null;
                        _this._removeFromSeq(lastOldSeqRecord, oldSeqRecord);
                        _this._addToRemovals(oldSeqRecord);
                    }
                    if (records.has(key)) {
                        newSeqRecord = records.get(key);
                    } else {
                        newSeqRecord = new KeyValueChangeRecord(key);
                        records.set(key, newSeqRecord);
                        newSeqRecord.currentValue = value;
                        _this._addToAdditions(newSeqRecord);
                    }
                }
                if (seqChanged) {
                    if (_this._isInRemovals(newSeqRecord)) {
                        _this._removeFromRemovals(newSeqRecord);
                    }
                    if (lastNewSeqRecord == null) {
                        _this._mapHead = newSeqRecord;
                    } else {
                        lastNewSeqRecord._next = newSeqRecord;
                    }
                }
                lastOldSeqRecord = oldSeqRecord;
                lastNewSeqRecord = newSeqRecord;
                oldSeqRecord = oldSeqRecord === null ? null : oldSeqRecord._next;
            });
            this._truncate(lastOldSeqRecord, oldSeqRecord);
            return this.isDirty;
        };
        /** @internal */
        DefaultKeyValueDiffer.prototype._reset = function () {
            if (this.isDirty) {
                var record;
                // Record the state of the mapping
                for (record = this._previousMapHead = this._mapHead; record !== null; record = record._next) {
                    record._nextPrevious = record._next;
                }
                for (record = this._changesHead; record !== null; record = record._nextChanged) {
                    record.previousValue = record.currentValue;
                }
                for (record = this._additionsHead; record != null; record = record._nextAdded) {
                    record.previousValue = record.currentValue;
                }
                // todo(vicb) once assert is supported
                // assert(() {
                //  var r = _changesHead;
                //  while (r != null) {
                //    var nextRecord = r._nextChanged;
                //    r._nextChanged = null;
                //    r = nextRecord;
                //  }
                //
                //  r = _additionsHead;
                //  while (r != null) {
                //    var nextRecord = r._nextAdded;
                //    r._nextAdded = null;
                //    r = nextRecord;
                //  }
                //
                //  r = _removalsHead;
                //  while (r != null) {
                //    var nextRecord = r._nextRemoved;
                //    r._nextRemoved = null;
                //    r = nextRecord;
                //  }
                //
                //  return true;
                //});
                this._changesHead = this._changesTail = null;
                this._additionsHead = this._additionsTail = null;
                this._removalsHead = this._removalsTail = null;
            }
        };
        /** @internal */
        DefaultKeyValueDiffer.prototype._truncate = function (lastRecord, record) {
            while (record !== null) {
                if (lastRecord === null) {
                    this._mapHead = null;
                } else {
                    lastRecord._next = null;
                }
                var nextRecord = record._next;
                // todo(vicb) assert
                // assert((() {
                //  record._next = null;
                //  return true;
                //}));
                this._addToRemovals(record);
                lastRecord = record;
                record = nextRecord;
            }
            for (var rec = this._removalsHead; rec !== null; rec = rec._nextRemoved) {
                rec.previousValue = rec.currentValue;
                rec.currentValue = null;
                this._records.delete(rec.key);
            }
        };
        /** @internal */
        DefaultKeyValueDiffer.prototype._isInRemovals = function (record) {
            return record === this._removalsHead || record._nextRemoved !== null || record._prevRemoved !== null;
        };
        /** @internal */
        DefaultKeyValueDiffer.prototype._addToRemovals = function (record) {
            // todo(vicb) assert
            // assert(record._next == null);
            // assert(record._nextAdded == null);
            // assert(record._nextChanged == null);
            // assert(record._nextRemoved == null);
            // assert(record._prevRemoved == null);
            if (this._removalsHead === null) {
                this._removalsHead = this._removalsTail = record;
            } else {
                this._removalsTail._nextRemoved = record;
                record._prevRemoved = this._removalsTail;
                this._removalsTail = record;
            }
        };
        /** @internal */
        DefaultKeyValueDiffer.prototype._removeFromSeq = function (prev, record) {
            var next = record._next;
            if (prev === null) {
                this._mapHead = next;
            } else {
                prev._next = next;
            }
            // todo(vicb) assert
            // assert((() {
            //  record._next = null;
            //  return true;
            //})());
        };
        /** @internal */
        DefaultKeyValueDiffer.prototype._removeFromRemovals = function (record) {
            // todo(vicb) assert
            // assert(record._next == null);
            // assert(record._nextAdded == null);
            // assert(record._nextChanged == null);
            var prev = record._prevRemoved;
            var next = record._nextRemoved;
            if (prev === null) {
                this._removalsHead = next;
            } else {
                prev._nextRemoved = next;
            }
            if (next === null) {
                this._removalsTail = prev;
            } else {
                next._prevRemoved = prev;
            }
            record._prevRemoved = record._nextRemoved = null;
        };
        /** @internal */
        DefaultKeyValueDiffer.prototype._addToAdditions = function (record) {
            // todo(vicb): assert
            // assert(record._next == null);
            // assert(record._nextAdded == null);
            // assert(record._nextChanged == null);
            // assert(record._nextRemoved == null);
            // assert(record._prevRemoved == null);
            if (this._additionsHead === null) {
                this._additionsHead = this._additionsTail = record;
            } else {
                this._additionsTail._nextAdded = record;
                this._additionsTail = record;
            }
        };
        /** @internal */
        DefaultKeyValueDiffer.prototype._addToChanges = function (record) {
            // todo(vicb) assert
            // assert(record._nextAdded == null);
            // assert(record._nextChanged == null);
            // assert(record._nextRemoved == null);
            // assert(record._prevRemoved == null);
            if (this._changesHead === null) {
                this._changesHead = this._changesTail = record;
            } else {
                this._changesTail._nextChanged = record;
                this._changesTail = record;
            }
        };
        DefaultKeyValueDiffer.prototype.toString = function () {
            var items = [];
            var previous = [];
            var changes = [];
            var additions = [];
            var removals = [];
            var record;
            for (record = this._mapHead; record !== null; record = record._next) {
                items.push(lang_1.stringify(record));
            }
            for (record = this._previousMapHead; record !== null; record = record._nextPrevious) {
                previous.push(lang_1.stringify(record));
            }
            for (record = this._changesHead; record !== null; record = record._nextChanged) {
                changes.push(lang_1.stringify(record));
            }
            for (record = this._additionsHead; record !== null; record = record._nextAdded) {
                additions.push(lang_1.stringify(record));
            }
            for (record = this._removalsHead; record !== null; record = record._nextRemoved) {
                removals.push(lang_1.stringify(record));
            }
            return 'map: ' + items.join(', ') + '\n' + 'previous: ' + previous.join(', ') + '\n' + 'additions: ' + additions.join(', ') + '\n' + 'changes: ' + changes.join(', ') + '\n' + 'removals: ' + removals.join(', ') + '\n';
        };
        /** @internal */
        DefaultKeyValueDiffer.prototype._forEach = function (obj /** TODO #9100 */, fn) {
            if (obj instanceof Map) {
                obj.forEach(fn);
            } else {
                collection_1.StringMapWrapper.forEach(obj, fn);
            }
        };
        return DefaultKeyValueDiffer;
    }();
    exports.DefaultKeyValueDiffer = DefaultKeyValueDiffer;
    /**
     * @stable
     */
    var KeyValueChangeRecord = function () {
        function KeyValueChangeRecord(key) {
            this.key = key;
            this.previousValue = null;
            this.currentValue = null;
            /** @internal */
            this._nextPrevious = null;
            /** @internal */
            this._next = null;
            /** @internal */
            this._nextAdded = null;
            /** @internal */
            this._nextRemoved = null;
            /** @internal */
            this._prevRemoved = null;
            /** @internal */
            this._nextChanged = null;
        }
        KeyValueChangeRecord.prototype.toString = function () {
            return lang_1.looseIdentical(this.previousValue, this.currentValue) ? lang_1.stringify(this.key) : lang_1.stringify(this.key) + '[' + lang_1.stringify(this.previousValue) + '->' + lang_1.stringify(this.currentValue) + ']';
        };
        return KeyValueChangeRecord;
    }();
    exports.KeyValueChangeRecord = KeyValueChangeRecord;
    
});
System.registerDynamic('node_modules/@angular/core/src/change_detection/differs/iterable_differs.js', ['../../di', '../../facade/collection', '../../facade/exceptions', '../../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var di_1 = $__require('../../di');
    var collection_1 = $__require('../../facade/collection');
    var exceptions_1 = $__require('../../facade/exceptions');
    var lang_1 = $__require('../../facade/lang');
    /**
     * A repository of different iterable diffing strategies used by NgFor, NgClass, and others.
     * @ts2dart_const
     * @stable
     */
    var IterableDiffers = function () {
        /*@ts2dart_const*/
        function IterableDiffers(factories) {
            this.factories = factories;
        }
        IterableDiffers.create = function (factories, parent) {
            if (lang_1.isPresent(parent)) {
                var copied = collection_1.ListWrapper.clone(parent.factories);
                factories = factories.concat(copied);
                return new IterableDiffers(factories);
            } else {
                return new IterableDiffers(factories);
            }
        };
        /**
         * Takes an array of {@link IterableDifferFactory} and returns a provider used to extend the
         * inherited {@link IterableDiffers} instance with the provided factories and return a new
         * {@link IterableDiffers} instance.
         *
         * The following example shows how to extend an existing list of factories,
               * which will only be applied to the injector for this component and its children.
               * This step is all that's required to make a new {@link IterableDiffer} available.
         *
         * ### Example
         *
         * ```
         * @Component({
         *   viewProviders: [
         *     IterableDiffers.extend([new ImmutableListDiffer()])
         *   ]
         * })
         * ```
         */
        IterableDiffers.extend = function (factories) {
            return new di_1.Provider(IterableDiffers, {
                useFactory: function (parent) {
                    if (lang_1.isBlank(parent)) {
                        // Typically would occur when calling IterableDiffers.extend inside of dependencies passed
                        // to
                        // bootstrap(), which would override default pipes instead of extending them.
                        throw new exceptions_1.BaseException('Cannot extend IterableDiffers without a parent injector');
                    }
                    return IterableDiffers.create(factories, parent);
                },
                // Dependency technically isn't optional, but we can provide a better error message this way.
                deps: [[IterableDiffers, new di_1.SkipSelfMetadata(), new di_1.OptionalMetadata()]]
            });
        };
        IterableDiffers.prototype.find = function (iterable) {
            var factory = this.factories.find(function (f) {
                return f.supports(iterable);
            });
            if (lang_1.isPresent(factory)) {
                return factory;
            } else {
                throw new exceptions_1.BaseException("Cannot find a differ supporting object '" + iterable + "' of type '" + lang_1.getTypeNameForDebugging(iterable) + "'");
            }
        };
        return IterableDiffers;
    }();
    exports.IterableDiffers = IterableDiffers;
    
});
System.registerDynamic("node_modules/@angular/core/src/di/injector.js", ["../facade/exceptions"], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var exceptions_1 = $__require('../facade/exceptions');
  var _THROW_IF_NOT_FOUND = new Object();
  exports.THROW_IF_NOT_FOUND = _THROW_IF_NOT_FOUND;
  /**
   * @stable
   */
  var Injector = function () {
    function Injector() {}
    /**
     * Retrieves an instance from the injector based on the provided token.
     * If not found:
     * - Throws {@link NoProviderError} if no `notFoundValue` that is not equal to
     * Injector.THROW_IF_NOT_FOUND is given
     * - Returns the `notFoundValue` otherwise
     *
     * ### Example ([live demo](http://plnkr.co/edit/HeXSHg?p=preview))
     *
     * ```typescript
     * var injector = ReflectiveInjector.resolveAndCreate([
     *   {provide: "validToken", useValue: "Value"}
     * ]);
     * expect(injector.get("validToken")).toEqual("Value");
     * expect(() => injector.get("invalidToken")).toThrowError();
     * ```
     *
     * `Injector` returns itself when given `Injector` as a token.
     *
     * ```typescript
     * var injector = ReflectiveInjector.resolveAndCreate([]);
     * expect(injector.get(Injector)).toBe(injector);
     * ```
     */
    Injector.prototype.get = function (token, notFoundValue) {
      return exceptions_1.unimplemented();
    };
    Injector.THROW_IF_NOT_FOUND = _THROW_IF_NOT_FOUND;
    return Injector;
  }();
  exports.Injector = Injector;
  
});
System.registerDynamic('node_modules/@angular/core/src/di/reflective_injector.js', ['../facade/collection', '../facade/exceptions', './injector', './metadata', './reflective_exceptions', './reflective_key', './reflective_provider'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var collection_1 = $__require('../facade/collection');
    var exceptions_1 = $__require('../facade/exceptions');
    var injector_1 = $__require('./injector');
    var metadata_1 = $__require('./metadata');
    var reflective_exceptions_1 = $__require('./reflective_exceptions');
    var reflective_key_1 = $__require('./reflective_key');
    var reflective_provider_1 = $__require('./reflective_provider');
    var __unused; // avoid unused import when Type union types are erased
    // Threshold for the dynamic version
    var _MAX_CONSTRUCTION_COUNTER = 10;
    var UNDEFINED = new Object();
    var ReflectiveProtoInjectorInlineStrategy = function () {
        function ReflectiveProtoInjectorInlineStrategy(protoEI, providers) {
            this.provider0 = null;
            this.provider1 = null;
            this.provider2 = null;
            this.provider3 = null;
            this.provider4 = null;
            this.provider5 = null;
            this.provider6 = null;
            this.provider7 = null;
            this.provider8 = null;
            this.provider9 = null;
            this.keyId0 = null;
            this.keyId1 = null;
            this.keyId2 = null;
            this.keyId3 = null;
            this.keyId4 = null;
            this.keyId5 = null;
            this.keyId6 = null;
            this.keyId7 = null;
            this.keyId8 = null;
            this.keyId9 = null;
            var length = providers.length;
            if (length > 0) {
                this.provider0 = providers[0];
                this.keyId0 = providers[0].key.id;
            }
            if (length > 1) {
                this.provider1 = providers[1];
                this.keyId1 = providers[1].key.id;
            }
            if (length > 2) {
                this.provider2 = providers[2];
                this.keyId2 = providers[2].key.id;
            }
            if (length > 3) {
                this.provider3 = providers[3];
                this.keyId3 = providers[3].key.id;
            }
            if (length > 4) {
                this.provider4 = providers[4];
                this.keyId4 = providers[4].key.id;
            }
            if (length > 5) {
                this.provider5 = providers[5];
                this.keyId5 = providers[5].key.id;
            }
            if (length > 6) {
                this.provider6 = providers[6];
                this.keyId6 = providers[6].key.id;
            }
            if (length > 7) {
                this.provider7 = providers[7];
                this.keyId7 = providers[7].key.id;
            }
            if (length > 8) {
                this.provider8 = providers[8];
                this.keyId8 = providers[8].key.id;
            }
            if (length > 9) {
                this.provider9 = providers[9];
                this.keyId9 = providers[9].key.id;
            }
        }
        ReflectiveProtoInjectorInlineStrategy.prototype.getProviderAtIndex = function (index) {
            if (index == 0) return this.provider0;
            if (index == 1) return this.provider1;
            if (index == 2) return this.provider2;
            if (index == 3) return this.provider3;
            if (index == 4) return this.provider4;
            if (index == 5) return this.provider5;
            if (index == 6) return this.provider6;
            if (index == 7) return this.provider7;
            if (index == 8) return this.provider8;
            if (index == 9) return this.provider9;
            throw new reflective_exceptions_1.OutOfBoundsError(index);
        };
        ReflectiveProtoInjectorInlineStrategy.prototype.createInjectorStrategy = function (injector) {
            return new ReflectiveInjectorInlineStrategy(injector, this);
        };
        return ReflectiveProtoInjectorInlineStrategy;
    }();
    exports.ReflectiveProtoInjectorInlineStrategy = ReflectiveProtoInjectorInlineStrategy;
    var ReflectiveProtoInjectorDynamicStrategy = function () {
        function ReflectiveProtoInjectorDynamicStrategy(protoInj, providers) {
            this.providers = providers;
            var len = providers.length;
            this.keyIds = collection_1.ListWrapper.createFixedSize(len);
            for (var i = 0; i < len; i++) {
                this.keyIds[i] = providers[i].key.id;
            }
        }
        ReflectiveProtoInjectorDynamicStrategy.prototype.getProviderAtIndex = function (index) {
            if (index < 0 || index >= this.providers.length) {
                throw new reflective_exceptions_1.OutOfBoundsError(index);
            }
            return this.providers[index];
        };
        ReflectiveProtoInjectorDynamicStrategy.prototype.createInjectorStrategy = function (ei) {
            return new ReflectiveInjectorDynamicStrategy(this, ei);
        };
        return ReflectiveProtoInjectorDynamicStrategy;
    }();
    exports.ReflectiveProtoInjectorDynamicStrategy = ReflectiveProtoInjectorDynamicStrategy;
    var ReflectiveProtoInjector = function () {
        function ReflectiveProtoInjector(providers) {
            this.numberOfProviders = providers.length;
            this._strategy = providers.length > _MAX_CONSTRUCTION_COUNTER ? new ReflectiveProtoInjectorDynamicStrategy(this, providers) : new ReflectiveProtoInjectorInlineStrategy(this, providers);
        }
        ReflectiveProtoInjector.fromResolvedProviders = function (providers) {
            return new ReflectiveProtoInjector(providers);
        };
        ReflectiveProtoInjector.prototype.getProviderAtIndex = function (index) {
            return this._strategy.getProviderAtIndex(index);
        };
        return ReflectiveProtoInjector;
    }();
    exports.ReflectiveProtoInjector = ReflectiveProtoInjector;
    var ReflectiveInjectorInlineStrategy = function () {
        function ReflectiveInjectorInlineStrategy(injector, protoStrategy) {
            this.injector = injector;
            this.protoStrategy = protoStrategy;
            this.obj0 = UNDEFINED;
            this.obj1 = UNDEFINED;
            this.obj2 = UNDEFINED;
            this.obj3 = UNDEFINED;
            this.obj4 = UNDEFINED;
            this.obj5 = UNDEFINED;
            this.obj6 = UNDEFINED;
            this.obj7 = UNDEFINED;
            this.obj8 = UNDEFINED;
            this.obj9 = UNDEFINED;
        }
        ReflectiveInjectorInlineStrategy.prototype.resetConstructionCounter = function () {
            this.injector._constructionCounter = 0;
        };
        ReflectiveInjectorInlineStrategy.prototype.instantiateProvider = function (provider) {
            return this.injector._new(provider);
        };
        ReflectiveInjectorInlineStrategy.prototype.getObjByKeyId = function (keyId) {
            var p = this.protoStrategy;
            var inj = this.injector;
            if (p.keyId0 === keyId) {
                if (this.obj0 === UNDEFINED) {
                    this.obj0 = inj._new(p.provider0);
                }
                return this.obj0;
            }
            if (p.keyId1 === keyId) {
                if (this.obj1 === UNDEFINED) {
                    this.obj1 = inj._new(p.provider1);
                }
                return this.obj1;
            }
            if (p.keyId2 === keyId) {
                if (this.obj2 === UNDEFINED) {
                    this.obj2 = inj._new(p.provider2);
                }
                return this.obj2;
            }
            if (p.keyId3 === keyId) {
                if (this.obj3 === UNDEFINED) {
                    this.obj3 = inj._new(p.provider3);
                }
                return this.obj3;
            }
            if (p.keyId4 === keyId) {
                if (this.obj4 === UNDEFINED) {
                    this.obj4 = inj._new(p.provider4);
                }
                return this.obj4;
            }
            if (p.keyId5 === keyId) {
                if (this.obj5 === UNDEFINED) {
                    this.obj5 = inj._new(p.provider5);
                }
                return this.obj5;
            }
            if (p.keyId6 === keyId) {
                if (this.obj6 === UNDEFINED) {
                    this.obj6 = inj._new(p.provider6);
                }
                return this.obj6;
            }
            if (p.keyId7 === keyId) {
                if (this.obj7 === UNDEFINED) {
                    this.obj7 = inj._new(p.provider7);
                }
                return this.obj7;
            }
            if (p.keyId8 === keyId) {
                if (this.obj8 === UNDEFINED) {
                    this.obj8 = inj._new(p.provider8);
                }
                return this.obj8;
            }
            if (p.keyId9 === keyId) {
                if (this.obj9 === UNDEFINED) {
                    this.obj9 = inj._new(p.provider9);
                }
                return this.obj9;
            }
            return UNDEFINED;
        };
        ReflectiveInjectorInlineStrategy.prototype.getObjAtIndex = function (index) {
            if (index == 0) return this.obj0;
            if (index == 1) return this.obj1;
            if (index == 2) return this.obj2;
            if (index == 3) return this.obj3;
            if (index == 4) return this.obj4;
            if (index == 5) return this.obj5;
            if (index == 6) return this.obj6;
            if (index == 7) return this.obj7;
            if (index == 8) return this.obj8;
            if (index == 9) return this.obj9;
            throw new reflective_exceptions_1.OutOfBoundsError(index);
        };
        ReflectiveInjectorInlineStrategy.prototype.getMaxNumberOfObjects = function () {
            return _MAX_CONSTRUCTION_COUNTER;
        };
        return ReflectiveInjectorInlineStrategy;
    }();
    exports.ReflectiveInjectorInlineStrategy = ReflectiveInjectorInlineStrategy;
    var ReflectiveInjectorDynamicStrategy = function () {
        function ReflectiveInjectorDynamicStrategy(protoStrategy, injector) {
            this.protoStrategy = protoStrategy;
            this.injector = injector;
            this.objs = collection_1.ListWrapper.createFixedSize(protoStrategy.providers.length);
            collection_1.ListWrapper.fill(this.objs, UNDEFINED);
        }
        ReflectiveInjectorDynamicStrategy.prototype.resetConstructionCounter = function () {
            this.injector._constructionCounter = 0;
        };
        ReflectiveInjectorDynamicStrategy.prototype.instantiateProvider = function (provider) {
            return this.injector._new(provider);
        };
        ReflectiveInjectorDynamicStrategy.prototype.getObjByKeyId = function (keyId) {
            var p = this.protoStrategy;
            for (var i = 0; i < p.keyIds.length; i++) {
                if (p.keyIds[i] === keyId) {
                    if (this.objs[i] === UNDEFINED) {
                        this.objs[i] = this.injector._new(p.providers[i]);
                    }
                    return this.objs[i];
                }
            }
            return UNDEFINED;
        };
        ReflectiveInjectorDynamicStrategy.prototype.getObjAtIndex = function (index) {
            if (index < 0 || index >= this.objs.length) {
                throw new reflective_exceptions_1.OutOfBoundsError(index);
            }
            return this.objs[index];
        };
        ReflectiveInjectorDynamicStrategy.prototype.getMaxNumberOfObjects = function () {
            return this.objs.length;
        };
        return ReflectiveInjectorDynamicStrategy;
    }();
    exports.ReflectiveInjectorDynamicStrategy = ReflectiveInjectorDynamicStrategy;
    /**
     * A ReflectiveDependency injection container used for instantiating objects and resolving
     * dependencies.
     *
     * An `Injector` is a replacement for a `new` operator, which can automatically resolve the
     * constructor dependencies.
     *
     * In typical use, application code asks for the dependencies in the constructor and they are
     * resolved by the `Injector`.
     *
     * ### Example ([live demo](http://plnkr.co/edit/jzjec0?p=preview))
     *
     * The following example creates an `Injector` configured to create `Engine` and `Car`.
     *
     * ```typescript
     * @Injectable()
     * class Engine {
     * }
     *
     * @Injectable()
     * class Car {
     *   constructor(public engine:Engine) {}
     * }
     *
     * var injector = ReflectiveInjector.resolveAndCreate([Car, Engine]);
     * var car = injector.get(Car);
     * expect(car instanceof Car).toBe(true);
     * expect(car.engine instanceof Engine).toBe(true);
     * ```
     *
     * Notice, we don't use the `new` operator because we explicitly want to have the `Injector`
     * resolve all of the object's dependencies automatically.
     *
     * @stable
     */
    var ReflectiveInjector = function () {
        function ReflectiveInjector() {}
        /**
         * Turns an array of provider definitions into an array of resolved providers.
         *
         * A resolution is a process of flattening multiple nested arrays and converting individual
         * providers into an array of {@link ResolvedReflectiveProvider}s.
         *
         * ### Example ([live demo](http://plnkr.co/edit/AiXTHi?p=preview))
         *
         * ```typescript
         * @Injectable()
         * class Engine {
         * }
         *
         * @Injectable()
         * class Car {
         *   constructor(public engine:Engine) {}
         * }
         *
         * var providers = ReflectiveInjector.resolve([Car, [[Engine]]]);
         *
         * expect(providers.length).toEqual(2);
         *
         * expect(providers[0] instanceof ResolvedReflectiveProvider).toBe(true);
         * expect(providers[0].key.displayName).toBe("Car");
         * expect(providers[0].dependencies.length).toEqual(1);
         * expect(providers[0].factory).toBeDefined();
         *
         * expect(providers[1].key.displayName).toBe("Engine");
         * });
         * ```
         *
         * See {@link ReflectiveInjector#fromResolvedProviders} for more info.
         */
        ReflectiveInjector.resolve = function (providers) {
            return reflective_provider_1.resolveReflectiveProviders(providers);
        };
        /**
         * Resolves an array of providers and creates an injector from those providers.
         *
         * The passed-in providers can be an array of `Type`, {@link Provider},
         * or a recursive array of more providers.
         *
         * ### Example ([live demo](http://plnkr.co/edit/ePOccA?p=preview))
         *
         * ```typescript
         * @Injectable()
         * class Engine {
         * }
         *
         * @Injectable()
         * class Car {
         *   constructor(public engine:Engine) {}
         * }
         *
         * var injector = ReflectiveInjector.resolveAndCreate([Car, Engine]);
         * expect(injector.get(Car) instanceof Car).toBe(true);
         * ```
         *
         * This function is slower than the corresponding `fromResolvedProviders`
         * because it needs to resolve the passed-in providers first.
         * See {@link Injector#resolve} and {@link Injector#fromResolvedProviders}.
         */
        ReflectiveInjector.resolveAndCreate = function (providers, parent) {
            if (parent === void 0) {
                parent = null;
            }
            var ResolvedReflectiveProviders = ReflectiveInjector.resolve(providers);
            return ReflectiveInjector.fromResolvedProviders(ResolvedReflectiveProviders, parent);
        };
        /**
         * Creates an injector from previously resolved providers.
         *
         * This API is the recommended way to construct injectors in performance-sensitive parts.
         *
         * ### Example ([live demo](http://plnkr.co/edit/KrSMci?p=preview))
         *
         * ```typescript
         * @Injectable()
         * class Engine {
         * }
         *
         * @Injectable()
         * class Car {
         *   constructor(public engine:Engine) {}
         * }
         *
         * var providers = ReflectiveInjector.resolve([Car, Engine]);
         * var injector = ReflectiveInjector.fromResolvedProviders(providers);
         * expect(injector.get(Car) instanceof Car).toBe(true);
         * ```
         * @experimental
         */
        ReflectiveInjector.fromResolvedProviders = function (providers, parent) {
            if (parent === void 0) {
                parent = null;
            }
            return new ReflectiveInjector_(ReflectiveProtoInjector.fromResolvedProviders(providers), parent);
        };
        /**
         * @deprecated
         */
        ReflectiveInjector.fromResolvedBindings = function (providers) {
            return ReflectiveInjector.fromResolvedProviders(providers);
        };
        Object.defineProperty(ReflectiveInjector.prototype, "parent", {
            /**
             * Parent of this injector.
             *
             * <!-- TODO: Add a link to the section of the user guide talking about hierarchical injection.
             * -->
             *
             * ### Example ([live demo](http://plnkr.co/edit/eosMGo?p=preview))
             *
             * ```typescript
             * var parent = ReflectiveInjector.resolveAndCreate([]);
             * var child = parent.resolveAndCreateChild([]);
             * expect(child.parent).toBe(parent);
             * ```
             */
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        /**
         * @internal
         */
        ReflectiveInjector.prototype.debugContext = function () {
            return null;
        };
        /**
         * Resolves an array of providers and creates a child injector from those providers.
         *
         * <!-- TODO: Add a link to the section of the user guide talking about hierarchical injection.
         * -->
         *
         * The passed-in providers can be an array of `Type`, {@link Provider},
         * or a recursive array of more providers.
         *
         * ### Example ([live demo](http://plnkr.co/edit/opB3T4?p=preview))
         *
         * ```typescript
         * class ParentProvider {}
         * class ChildProvider {}
         *
         * var parent = ReflectiveInjector.resolveAndCreate([ParentProvider]);
         * var child = parent.resolveAndCreateChild([ChildProvider]);
         *
         * expect(child.get(ParentProvider) instanceof ParentProvider).toBe(true);
         * expect(child.get(ChildProvider) instanceof ChildProvider).toBe(true);
         * expect(child.get(ParentProvider)).toBe(parent.get(ParentProvider));
         * ```
         *
         * This function is slower than the corresponding `createChildFromResolved`
         * because it needs to resolve the passed-in providers first.
         * See {@link Injector#resolve} and {@link Injector#createChildFromResolved}.
         */
        ReflectiveInjector.prototype.resolveAndCreateChild = function (providers) {
            return exceptions_1.unimplemented();
        };
        /**
         * Creates a child injector from previously resolved providers.
         *
         * <!-- TODO: Add a link to the section of the user guide talking about hierarchical injection.
         * -->
         *
         * This API is the recommended way to construct injectors in performance-sensitive parts.
         *
         * ### Example ([live demo](http://plnkr.co/edit/VhyfjN?p=preview))
         *
         * ```typescript
         * class ParentProvider {}
         * class ChildProvider {}
         *
         * var parentProviders = ReflectiveInjector.resolve([ParentProvider]);
         * var childProviders = ReflectiveInjector.resolve([ChildProvider]);
         *
         * var parent = ReflectiveInjector.fromResolvedProviders(parentProviders);
         * var child = parent.createChildFromResolved(childProviders);
         *
         * expect(child.get(ParentProvider) instanceof ParentProvider).toBe(true);
         * expect(child.get(ChildProvider) instanceof ChildProvider).toBe(true);
         * expect(child.get(ParentProvider)).toBe(parent.get(ParentProvider));
         * ```
         */
        ReflectiveInjector.prototype.createChildFromResolved = function (providers) {
            return exceptions_1.unimplemented();
        };
        /**
         * Resolves a provider and instantiates an object in the context of the injector.
         *
         * The created object does not get cached by the injector.
         *
         * ### Example ([live demo](http://plnkr.co/edit/yvVXoB?p=preview))
         *
         * ```typescript
         * @Injectable()
         * class Engine {
         * }
         *
         * @Injectable()
         * class Car {
         *   constructor(public engine:Engine) {}
         * }
         *
         * var injector = ReflectiveInjector.resolveAndCreate([Engine]);
         *
         * var car = injector.resolveAndInstantiate(Car);
         * expect(car.engine).toBe(injector.get(Engine));
         * expect(car).not.toBe(injector.resolveAndInstantiate(Car));
         * ```
         */
        ReflectiveInjector.prototype.resolveAndInstantiate = function (provider) {
            return exceptions_1.unimplemented();
        };
        /**
         * Instantiates an object using a resolved provider in the context of the injector.
         *
         * The created object does not get cached by the injector.
         *
         * ### Example ([live demo](http://plnkr.co/edit/ptCImQ?p=preview))
         *
         * ```typescript
         * @Injectable()
         * class Engine {
         * }
         *
         * @Injectable()
         * class Car {
         *   constructor(public engine:Engine) {}
         * }
         *
         * var injector = ReflectiveInjector.resolveAndCreate([Engine]);
         * var carProvider = ReflectiveInjector.resolve([Car])[0];
         * var car = injector.instantiateResolved(carProvider);
         * expect(car.engine).toBe(injector.get(Engine));
         * expect(car).not.toBe(injector.instantiateResolved(carProvider));
         * ```
         */
        ReflectiveInjector.prototype.instantiateResolved = function (provider) {
            return exceptions_1.unimplemented();
        };
        return ReflectiveInjector;
    }();
    exports.ReflectiveInjector = ReflectiveInjector;
    var ReflectiveInjector_ = function () {
        /**
         * Private
         */
        function ReflectiveInjector_(_proto /* ProtoInjector */, _parent, _debugContext) {
            if (_parent === void 0) {
                _parent = null;
            }
            if (_debugContext === void 0) {
                _debugContext = null;
            }
            this._debugContext = _debugContext;
            /** @internal */
            this._constructionCounter = 0;
            this._proto = _proto;
            this._parent = _parent;
            this._strategy = _proto._strategy.createInjectorStrategy(this);
        }
        /**
         * @internal
         */
        ReflectiveInjector_.prototype.debugContext = function () {
            return this._debugContext();
        };
        ReflectiveInjector_.prototype.get = function (token, notFoundValue) {
            if (notFoundValue === void 0) {
                notFoundValue = injector_1.THROW_IF_NOT_FOUND;
            }
            return this._getByKey(reflective_key_1.ReflectiveKey.get(token), null, null, notFoundValue);
        };
        ReflectiveInjector_.prototype.getAt = function (index) {
            return this._strategy.getObjAtIndex(index);
        };
        Object.defineProperty(ReflectiveInjector_.prototype, "parent", {
            get: function () {
                return this._parent;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ReflectiveInjector_.prototype, "internalStrategy", {
            /**
             * @internal
             * Internal. Do not use.
             * We return `any` not to export the InjectorStrategy type.
             */
            get: function () {
                return this._strategy;
            },
            enumerable: true,
            configurable: true
        });
        ReflectiveInjector_.prototype.resolveAndCreateChild = function (providers) {
            var ResolvedReflectiveProviders = ReflectiveInjector.resolve(providers);
            return this.createChildFromResolved(ResolvedReflectiveProviders);
        };
        ReflectiveInjector_.prototype.createChildFromResolved = function (providers) {
            var proto = new ReflectiveProtoInjector(providers);
            var inj = new ReflectiveInjector_(proto);
            inj._parent = this;
            return inj;
        };
        ReflectiveInjector_.prototype.resolveAndInstantiate = function (provider) {
            return this.instantiateResolved(ReflectiveInjector.resolve([provider])[0]);
        };
        ReflectiveInjector_.prototype.instantiateResolved = function (provider) {
            return this._instantiateProvider(provider);
        };
        /** @internal */
        ReflectiveInjector_.prototype._new = function (provider) {
            if (this._constructionCounter++ > this._strategy.getMaxNumberOfObjects()) {
                throw new reflective_exceptions_1.CyclicDependencyError(this, provider.key);
            }
            return this._instantiateProvider(provider);
        };
        ReflectiveInjector_.prototype._instantiateProvider = function (provider) {
            if (provider.multiProvider) {
                var res = collection_1.ListWrapper.createFixedSize(provider.resolvedFactories.length);
                for (var i = 0; i < provider.resolvedFactories.length; ++i) {
                    res[i] = this._instantiate(provider, provider.resolvedFactories[i]);
                }
                return res;
            } else {
                return this._instantiate(provider, provider.resolvedFactories[0]);
            }
        };
        ReflectiveInjector_.prototype._instantiate = function (provider, ResolvedReflectiveFactory) {
            var factory = ResolvedReflectiveFactory.factory;
            var deps = ResolvedReflectiveFactory.dependencies;
            var length = deps.length;
            var d0;
            var d1;
            var d2;
            var d3;
            var d4;
            var d5;
            var d6;
            var d7;
            var d8;
            var d9;
            var d10;
            var d11;
            var d12;
            var d13;
            var d14;
            var d15;
            var d16;
            var d17;
            var d18;
            var d19;
            try {
                d0 = length > 0 ? this._getByReflectiveDependency(provider, deps[0]) : null;
                d1 = length > 1 ? this._getByReflectiveDependency(provider, deps[1]) : null;
                d2 = length > 2 ? this._getByReflectiveDependency(provider, deps[2]) : null;
                d3 = length > 3 ? this._getByReflectiveDependency(provider, deps[3]) : null;
                d4 = length > 4 ? this._getByReflectiveDependency(provider, deps[4]) : null;
                d5 = length > 5 ? this._getByReflectiveDependency(provider, deps[5]) : null;
                d6 = length > 6 ? this._getByReflectiveDependency(provider, deps[6]) : null;
                d7 = length > 7 ? this._getByReflectiveDependency(provider, deps[7]) : null;
                d8 = length > 8 ? this._getByReflectiveDependency(provider, deps[8]) : null;
                d9 = length > 9 ? this._getByReflectiveDependency(provider, deps[9]) : null;
                d10 = length > 10 ? this._getByReflectiveDependency(provider, deps[10]) : null;
                d11 = length > 11 ? this._getByReflectiveDependency(provider, deps[11]) : null;
                d12 = length > 12 ? this._getByReflectiveDependency(provider, deps[12]) : null;
                d13 = length > 13 ? this._getByReflectiveDependency(provider, deps[13]) : null;
                d14 = length > 14 ? this._getByReflectiveDependency(provider, deps[14]) : null;
                d15 = length > 15 ? this._getByReflectiveDependency(provider, deps[15]) : null;
                d16 = length > 16 ? this._getByReflectiveDependency(provider, deps[16]) : null;
                d17 = length > 17 ? this._getByReflectiveDependency(provider, deps[17]) : null;
                d18 = length > 18 ? this._getByReflectiveDependency(provider, deps[18]) : null;
                d19 = length > 19 ? this._getByReflectiveDependency(provider, deps[19]) : null;
            } catch (e) {
                if (e instanceof reflective_exceptions_1.AbstractProviderError || e instanceof reflective_exceptions_1.InstantiationError) {
                    e.addKey(this, provider.key);
                }
                throw e;
            }
            var obj;
            try {
                switch (length) {
                    case 0:
                        obj = factory();
                        break;
                    case 1:
                        obj = factory(d0);
                        break;
                    case 2:
                        obj = factory(d0, d1);
                        break;
                    case 3:
                        obj = factory(d0, d1, d2);
                        break;
                    case 4:
                        obj = factory(d0, d1, d2, d3);
                        break;
                    case 5:
                        obj = factory(d0, d1, d2, d3, d4);
                        break;
                    case 6:
                        obj = factory(d0, d1, d2, d3, d4, d5);
                        break;
                    case 7:
                        obj = factory(d0, d1, d2, d3, d4, d5, d6);
                        break;
                    case 8:
                        obj = factory(d0, d1, d2, d3, d4, d5, d6, d7);
                        break;
                    case 9:
                        obj = factory(d0, d1, d2, d3, d4, d5, d6, d7, d8);
                        break;
                    case 10:
                        obj = factory(d0, d1, d2, d3, d4, d5, d6, d7, d8, d9);
                        break;
                    case 11:
                        obj = factory(d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10);
                        break;
                    case 12:
                        obj = factory(d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11);
                        break;
                    case 13:
                        obj = factory(d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11, d12);
                        break;
                    case 14:
                        obj = factory(d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11, d12, d13);
                        break;
                    case 15:
                        obj = factory(d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11, d12, d13, d14);
                        break;
                    case 16:
                        obj = factory(d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11, d12, d13, d14, d15);
                        break;
                    case 17:
                        obj = factory(d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11, d12, d13, d14, d15, d16);
                        break;
                    case 18:
                        obj = factory(d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11, d12, d13, d14, d15, d16, d17);
                        break;
                    case 19:
                        obj = factory(d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11, d12, d13, d14, d15, d16, d17, d18);
                        break;
                    case 20:
                        obj = factory(d0, d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11, d12, d13, d14, d15, d16, d17, d18, d19);
                        break;
                    default:
                        throw new exceptions_1.BaseException("Cannot instantiate '" + provider.key.displayName + "' because it has more than 20 dependencies");
                }
            } catch (e) {
                throw new reflective_exceptions_1.InstantiationError(this, e, e.stack, provider.key);
            }
            return obj;
        };
        ReflectiveInjector_.prototype._getByReflectiveDependency = function (provider, dep) {
            return this._getByKey(dep.key, dep.lowerBoundVisibility, dep.upperBoundVisibility, dep.optional ? null : injector_1.THROW_IF_NOT_FOUND);
        };
        ReflectiveInjector_.prototype._getByKey = function (key, lowerBoundVisibility, upperBoundVisibility, notFoundValue) {
            if (key === INJECTOR_KEY) {
                return this;
            }
            if (upperBoundVisibility instanceof metadata_1.SelfMetadata) {
                return this._getByKeySelf(key, notFoundValue);
            } else {
                return this._getByKeyDefault(key, notFoundValue, lowerBoundVisibility);
            }
        };
        /** @internal */
        ReflectiveInjector_.prototype._throwOrNull = function (key, notFoundValue) {
            if (notFoundValue !== injector_1.THROW_IF_NOT_FOUND) {
                return notFoundValue;
            } else {
                throw new reflective_exceptions_1.NoProviderError(this, key);
            }
        };
        /** @internal */
        ReflectiveInjector_.prototype._getByKeySelf = function (key, notFoundValue) {
            var obj = this._strategy.getObjByKeyId(key.id);
            return obj !== UNDEFINED ? obj : this._throwOrNull(key, notFoundValue);
        };
        /** @internal */
        ReflectiveInjector_.prototype._getByKeyDefault = function (key, notFoundValue, lowerBoundVisibility) {
            var inj;
            if (lowerBoundVisibility instanceof metadata_1.SkipSelfMetadata) {
                inj = this._parent;
            } else {
                inj = this;
            }
            while (inj instanceof ReflectiveInjector_) {
                var inj_ = inj;
                var obj = inj_._strategy.getObjByKeyId(key.id);
                if (obj !== UNDEFINED) return obj;
                inj = inj_._parent;
            }
            if (inj !== null) {
                return inj.get(key.token, notFoundValue);
            } else {
                return this._throwOrNull(key, notFoundValue);
            }
        };
        Object.defineProperty(ReflectiveInjector_.prototype, "displayName", {
            get: function () {
                var providers = _mapProviders(this, function (b) {
                    return ' "' + b.key.displayName + '" ';
                }).join(', ');
                return "ReflectiveInjector(providers: [" + providers + "])";
            },
            enumerable: true,
            configurable: true
        });
        ReflectiveInjector_.prototype.toString = function () {
            return this.displayName;
        };
        return ReflectiveInjector_;
    }();
    exports.ReflectiveInjector_ = ReflectiveInjector_;
    var INJECTOR_KEY = reflective_key_1.ReflectiveKey.get(injector_1.Injector);
    function _mapProviders(injector, fn) {
        var res = new Array(injector._proto.numberOfProviders);
        for (var i = 0; i < injector._proto.numberOfProviders; ++i) {
            res[i] = fn(injector._proto.getProviderAtIndex(i));
        }
        return res;
    }
    
});
System.registerDynamic('node_modules/@angular/core/src/di/provider.js', ['../facade/exceptions', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    /**
     * Describes how the {@link Injector} should instantiate a given token.
     *
     * See {@link provide}.
     *
     * ### Example ([live demo](http://plnkr.co/edit/GNAyj6K6PfYg2NBzgwZ5?p%3Dpreview&p=preview))
     *
     * ```javascript
     * var injector = Injector.resolveAndCreate([
     *   new Provider("message", { useValue: 'Hello' })
     * ]);
     *
     * expect(injector.get("message")).toEqual('Hello');
     * ```
     * @ts2dart_const
     * @deprecated
     */
    var Provider = function () {
        function Provider(token, _a) {
            var useClass = _a.useClass,
                useValue = _a.useValue,
                useExisting = _a.useExisting,
                useFactory = _a.useFactory,
                deps = _a.deps,
                multi = _a.multi;
            this.token = token;
            this.useClass = useClass;
            this.useValue = useValue;
            this.useExisting = useExisting;
            this.useFactory = useFactory;
            this.dependencies = deps;
            this._multi = multi;
        }
        Object.defineProperty(Provider.prototype, "multi", {
            // TODO: Provide a full working example after alpha38 is released.
            /**
             * Creates multiple providers matching the same token (a multi-provider).
             *
             * Multi-providers are used for creating pluggable service, where the system comes
             * with some default providers, and the user can register additional providers.
             * The combination of the default providers and the additional providers will be
             * used to drive the behavior of the system.
             *
             * ### Example
             *
             * ```typescript
             * var injector = Injector.resolveAndCreate([
             *   new Provider("Strings", { useValue: "String1", multi: true}),
             *   new Provider("Strings", { useValue: "String2", multi: true})
             * ]);
             *
             * expect(injector.get("Strings")).toEqual(["String1", "String2"]);
             * ```
             *
             * Multi-providers and regular providers cannot be mixed. The following
             * will throw an exception:
             *
             * ```typescript
             * var injector = Injector.resolveAndCreate([
             *   new Provider("Strings", { useValue: "String1", multi: true }),
             *   new Provider("Strings", { useValue: "String2"})
             * ]);
             * ```
             */
            get: function () {
                return lang_1.normalizeBool(this._multi);
            },
            enumerable: true,
            configurable: true
        });
        return Provider;
    }();
    exports.Provider = Provider;
    /**
     * See {@link Provider} instead.
     *
     * @deprecated
     * @ts2dart_const
     */
    var Binding = function (_super) {
        __extends(Binding, _super);
        function Binding(token, _a) {
            var toClass = _a.toClass,
                toValue = _a.toValue,
                toAlias = _a.toAlias,
                toFactory = _a.toFactory,
                deps = _a.deps,
                multi = _a.multi;
            _super.call(this, token, {
                useClass: toClass,
                useValue: toValue,
                useExisting: toAlias,
                useFactory: toFactory,
                deps: deps,
                multi: multi
            });
        }
        Object.defineProperty(Binding.prototype, "toClass", {
            /**
             * @deprecated
             */
            get: function () {
                return this.useClass;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Binding.prototype, "toAlias", {
            /**
             * @deprecated
             */
            get: function () {
                return this.useExisting;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Binding.prototype, "toFactory", {
            /**
             * @deprecated
             */
            get: function () {
                return this.useFactory;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Binding.prototype, "toValue", {
            /**
             * @deprecated
             */
            get: function () {
                return this.useValue;
            },
            enumerable: true,
            configurable: true
        });
        return Binding;
    }(Provider);
    exports.Binding = Binding;
    /**
     * Creates a {@link Provider}.
     *
     * To construct a {@link Provider}, bind a `token` to either a class, a value, a factory function,
     * or
     * to an existing `token`.
     * See {@link ProviderBuilder} for more details.
     *
     * The `token` is most commonly a class or {@link OpaqueToken-class.html}.
     *
     * @deprecated
     */
    function bind(token) {
        return new ProviderBuilder(token);
    }
    exports.bind = bind;
    /**
     * Helper class for the {@link bind} function.
     * @deprecated
     */
    var ProviderBuilder = function () {
        function ProviderBuilder(token) {
            this.token = token;
        }
        /**
         * Binds a DI token to a class.
         *
         * ### Example ([live demo](http://plnkr.co/edit/ZpBCSYqv6e2ud5KXLdxQ?p=preview))
         *
         * Because `toAlias` and `toClass` are often confused, the example contains
         * both use cases for easy comparison.
         *
         * ```typescript
         * class Vehicle {}
         *
         * class Car extends Vehicle {}
         *
         * var injectorClass = Injector.resolveAndCreate([
         *   Car,
         *   {provide: Vehicle, useClass: Car}
         * ]);
         * var injectorAlias = Injector.resolveAndCreate([
         *   Car,
         *   {provide: Vehicle, useExisting: Car}
         * ]);
         *
         * expect(injectorClass.get(Vehicle)).not.toBe(injectorClass.get(Car));
         * expect(injectorClass.get(Vehicle) instanceof Car).toBe(true);
         *
         * expect(injectorAlias.get(Vehicle)).toBe(injectorAlias.get(Car));
         * expect(injectorAlias.get(Vehicle) instanceof Car).toBe(true);
         * ```
         */
        ProviderBuilder.prototype.toClass = function (type) {
            if (!lang_1.isType(type)) {
                throw new exceptions_1.BaseException("Trying to create a class provider but \"" + lang_1.stringify(type) + "\" is not a class!");
            }
            return new Provider(this.token, { useClass: type });
        };
        /**
         * Binds a DI token to a value.
         *
         * ### Example ([live demo](http://plnkr.co/edit/G024PFHmDL0cJFgfZK8O?p=preview))
         *
         * ```typescript
         * var injector = Injector.resolveAndCreate([
         *   {provide: 'message', useValue: 'Hello'}
         * ]);
         *
         * expect(injector.get('message')).toEqual('Hello');
         * ```
         */
        ProviderBuilder.prototype.toValue = function (value) {
            return new Provider(this.token, { useValue: value });
        };
        /**
         * Binds a DI token to an existing token.
         *
         * Angular will return the same instance as if the provided token was used. (This is
         * in contrast to `useClass` where a separate instance of `useClass` will be returned.)
         *
         * ### Example ([live demo](http://plnkr.co/edit/uBaoF2pN5cfc5AfZapNw?p=preview))
         *
         * Because `toAlias` and `toClass` are often confused, the example contains
         * both use cases for easy comparison.
         *
         * ```typescript
         * class Vehicle {}
         *
         * class Car extends Vehicle {}
         *
         * var injectorAlias = Injector.resolveAndCreate([
         *   Car,
         *   {provide: Vehicle, useExisting: Car}
         * ]);
         * var injectorClass = Injector.resolveAndCreate([
         *   Car,
         *   {provide: Vehicle, useClass: Car})
         * ]);
         *
         * expect(injectorAlias.get(Vehicle)).toBe(injectorAlias.get(Car));
         * expect(injectorAlias.get(Vehicle) instanceof Car).toBe(true);
         *
         * expect(injectorClass.get(Vehicle)).not.toBe(injectorClass.get(Car));
         * expect(injectorClass.get(Vehicle) instanceof Car).toBe(true);
         * ```
         */
        ProviderBuilder.prototype.toAlias = function (aliasToken) {
            if (lang_1.isBlank(aliasToken)) {
                throw new exceptions_1.BaseException("Can not alias " + lang_1.stringify(this.token) + " to a blank value!");
            }
            return new Provider(this.token, { useExisting: aliasToken });
        };
        /**
         * Binds a DI token to a function which computes the value.
         *
         * ### Example ([live demo](http://plnkr.co/edit/OejNIfTT3zb1iBxaIYOb?p=preview))
         *
         * ```typescript
         * var injector = Injector.resolveAndCreate([
         *   {provide: Number, useFactory: () => { return 1+2; }},
         *   {provide: String, useFactory: (v) => { return "Value: " + v; }, deps: [Number]}
         * ]);
         *
         * expect(injector.get(Number)).toEqual(3);
         * expect(injector.get(String)).toEqual('Value: 3');
         * ```
         */
        ProviderBuilder.prototype.toFactory = function (factory, dependencies) {
            if (!lang_1.isFunction(factory)) {
                throw new exceptions_1.BaseException("Trying to create a factory provider but \"" + lang_1.stringify(factory) + "\" is not a function!");
            }
            return new Provider(this.token, { useFactory: factory, deps: dependencies });
        };
        return ProviderBuilder;
    }();
    exports.ProviderBuilder = ProviderBuilder;
    /**
     * Creates a {@link Provider}.
     *
     * See {@link Provider} for more details.
     *
     * <!-- TODO: improve the docs -->
     * @deprecated
     */
    function provide(token, _a) {
        var useClass = _a.useClass,
            useValue = _a.useValue,
            useExisting = _a.useExisting,
            useFactory = _a.useFactory,
            deps = _a.deps,
            multi = _a.multi;
        return new Provider(token, {
            useClass: useClass,
            useValue: useValue,
            useExisting: useExisting,
            useFactory: useFactory,
            deps: deps,
            multi: multi
        });
    }
    exports.provide = provide;
    
});
System.registerDynamic('node_modules/@angular/core/src/di/provider_util.js', ['./provider'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var provider_1 = $__require('./provider');
    function isProviderLiteral(obj) {
        return obj && typeof obj == 'object' && obj.provide;
    }
    exports.isProviderLiteral = isProviderLiteral;
    function createProvider(obj) {
        return new provider_1.Provider(obj.provide, obj);
    }
    exports.createProvider = createProvider;
    
});
System.registerDynamic('node_modules/@angular/core/src/di/reflective_provider.js', ['../facade/lang', '../facade/collection', '../reflection/reflection', './reflective_key', './metadata', './reflective_exceptions', './forward_ref', './provider', './provider_util'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('../facade/lang');
    var collection_1 = $__require('../facade/collection');
    var reflection_1 = $__require('../reflection/reflection');
    var reflective_key_1 = $__require('./reflective_key');
    var metadata_1 = $__require('./metadata');
    var reflective_exceptions_1 = $__require('./reflective_exceptions');
    var forward_ref_1 = $__require('./forward_ref');
    var provider_1 = $__require('./provider');
    var provider_util_1 = $__require('./provider_util');
    /**
     * `Dependency` is used by the framework to extend DI.
     * This is internal to Angular and should not be used directly.
     */
    var ReflectiveDependency = function () {
        function ReflectiveDependency(key, optional, lowerBoundVisibility, upperBoundVisibility, properties) {
            this.key = key;
            this.optional = optional;
            this.lowerBoundVisibility = lowerBoundVisibility;
            this.upperBoundVisibility = upperBoundVisibility;
            this.properties = properties;
        }
        ReflectiveDependency.fromKey = function (key) {
            return new ReflectiveDependency(key, false, null, null, []);
        };
        return ReflectiveDependency;
    }();
    exports.ReflectiveDependency = ReflectiveDependency;
    var _EMPTY_LIST = [];
    var ResolvedReflectiveProvider_ = function () {
        function ResolvedReflectiveProvider_(key, resolvedFactories, multiProvider) {
            this.key = key;
            this.resolvedFactories = resolvedFactories;
            this.multiProvider = multiProvider;
        }
        Object.defineProperty(ResolvedReflectiveProvider_.prototype, "resolvedFactory", {
            get: function () {
                return this.resolvedFactories[0];
            },
            enumerable: true,
            configurable: true
        });
        return ResolvedReflectiveProvider_;
    }();
    exports.ResolvedReflectiveProvider_ = ResolvedReflectiveProvider_;
    /**
     * An internal resolved representation of a factory function created by resolving {@link Provider}.
     * @experimental
     */
    var ResolvedReflectiveFactory = function () {
        function ResolvedReflectiveFactory(
        /**
         * Factory function which can return an instance of an object represented by a key.
         */
        factory,
        /**
         * Arguments (dependencies) to the `factory` function.
         */
        dependencies) {
            this.factory = factory;
            this.dependencies = dependencies;
        }
        return ResolvedReflectiveFactory;
    }();
    exports.ResolvedReflectiveFactory = ResolvedReflectiveFactory;
    /**
     * Resolve a single provider.
     */
    function resolveReflectiveFactory(provider) {
        var factoryFn;
        var resolvedDeps;
        if (lang_1.isPresent(provider.useClass)) {
            var useClass = forward_ref_1.resolveForwardRef(provider.useClass);
            factoryFn = reflection_1.reflector.factory(useClass);
            resolvedDeps = _dependenciesFor(useClass);
        } else if (lang_1.isPresent(provider.useExisting)) {
            factoryFn = function (aliasInstance) {
                return aliasInstance;
            };
            resolvedDeps = [ReflectiveDependency.fromKey(reflective_key_1.ReflectiveKey.get(provider.useExisting))];
        } else if (lang_1.isPresent(provider.useFactory)) {
            factoryFn = provider.useFactory;
            resolvedDeps = constructDependencies(provider.useFactory, provider.dependencies);
        } else {
            factoryFn = function () {
                return provider.useValue;
            };
            resolvedDeps = _EMPTY_LIST;
        }
        return new ResolvedReflectiveFactory(factoryFn, resolvedDeps);
    }
    exports.resolveReflectiveFactory = resolveReflectiveFactory;
    /**
     * Converts the {@link Provider} into {@link ResolvedProvider}.
     *
     * {@link Injector} internally only uses {@link ResolvedProvider}, {@link Provider} contains
     * convenience provider syntax.
     */
    function resolveReflectiveProvider(provider) {
        return new ResolvedReflectiveProvider_(reflective_key_1.ReflectiveKey.get(provider.token), [resolveReflectiveFactory(provider)], provider.multi);
    }
    exports.resolveReflectiveProvider = resolveReflectiveProvider;
    /**
     * Resolve a list of Providers.
     */
    function resolveReflectiveProviders(providers) {
        var normalized = _normalizeProviders(providers, []);
        var resolved = normalized.map(resolveReflectiveProvider);
        return collection_1.MapWrapper.values(mergeResolvedReflectiveProviders(resolved, new Map()));
    }
    exports.resolveReflectiveProviders = resolveReflectiveProviders;
    /**
     * Merges a list of ResolvedProviders into a list where
     * each key is contained exactly once and multi providers
     * have been merged.
     */
    function mergeResolvedReflectiveProviders(providers, normalizedProvidersMap) {
        for (var i = 0; i < providers.length; i++) {
            var provider = providers[i];
            var existing = normalizedProvidersMap.get(provider.key.id);
            if (lang_1.isPresent(existing)) {
                if (provider.multiProvider !== existing.multiProvider) {
                    throw new reflective_exceptions_1.MixingMultiProvidersWithRegularProvidersError(existing, provider);
                }
                if (provider.multiProvider) {
                    for (var j = 0; j < provider.resolvedFactories.length; j++) {
                        existing.resolvedFactories.push(provider.resolvedFactories[j]);
                    }
                } else {
                    normalizedProvidersMap.set(provider.key.id, provider);
                }
            } else {
                var resolvedProvider;
                if (provider.multiProvider) {
                    resolvedProvider = new ResolvedReflectiveProvider_(provider.key, collection_1.ListWrapper.clone(provider.resolvedFactories), provider.multiProvider);
                } else {
                    resolvedProvider = provider;
                }
                normalizedProvidersMap.set(provider.key.id, resolvedProvider);
            }
        }
        return normalizedProvidersMap;
    }
    exports.mergeResolvedReflectiveProviders = mergeResolvedReflectiveProviders;
    function _normalizeProviders(providers, res) {
        providers.forEach(function (b) {
            if (b instanceof lang_1.Type) {
                res.push(provider_1.provide(b, { useClass: b }));
            } else if (b instanceof provider_1.Provider) {
                res.push(b);
            } else if (provider_util_1.isProviderLiteral(b)) {
                res.push(provider_util_1.createProvider(b));
            } else if (b instanceof Array) {
                _normalizeProviders(b, res);
            } else if (b instanceof provider_1.ProviderBuilder) {
                throw new reflective_exceptions_1.InvalidProviderError(b.token);
            } else {
                throw new reflective_exceptions_1.InvalidProviderError(b);
            }
        });
        return res;
    }
    function constructDependencies(typeOrFunc, dependencies) {
        if (lang_1.isBlank(dependencies)) {
            return _dependenciesFor(typeOrFunc);
        } else {
            var params = dependencies.map(function (t) {
                return [t];
            });
            return dependencies.map(function (t) {
                return _extractToken(typeOrFunc, t, params);
            });
        }
    }
    exports.constructDependencies = constructDependencies;
    function _dependenciesFor(typeOrFunc) {
        var params = reflection_1.reflector.parameters(typeOrFunc);
        if (lang_1.isBlank(params)) return [];
        if (params.some(lang_1.isBlank)) {
            throw new reflective_exceptions_1.NoAnnotationError(typeOrFunc, params);
        }
        return params.map(function (p) {
            return _extractToken(typeOrFunc, p, params);
        });
    }
    function _extractToken(typeOrFunc /** TODO #9100 */, metadata /** TODO #9100 */ /*any[] | any*/, params) {
        var depProps = [];
        var token = null;
        var optional = false;
        if (!lang_1.isArray(metadata)) {
            if (metadata instanceof metadata_1.InjectMetadata) {
                return _createDependency(metadata.token, optional, null, null, depProps);
            } else {
                return _createDependency(metadata, optional, null, null, depProps);
            }
        }
        var lowerBoundVisibility = null;
        var upperBoundVisibility = null;
        for (var i = 0; i < metadata.length; ++i) {
            var paramMetadata = metadata[i];
            if (paramMetadata instanceof lang_1.Type) {
                token = paramMetadata;
            } else if (paramMetadata instanceof metadata_1.InjectMetadata) {
                token = paramMetadata.token;
            } else if (paramMetadata instanceof metadata_1.OptionalMetadata) {
                optional = true;
            } else if (paramMetadata instanceof metadata_1.SelfMetadata) {
                upperBoundVisibility = paramMetadata;
            } else if (paramMetadata instanceof metadata_1.HostMetadata) {
                upperBoundVisibility = paramMetadata;
            } else if (paramMetadata instanceof metadata_1.SkipSelfMetadata) {
                lowerBoundVisibility = paramMetadata;
            } else if (paramMetadata instanceof metadata_1.DependencyMetadata) {
                if (lang_1.isPresent(paramMetadata.token)) {
                    token = paramMetadata.token;
                }
                depProps.push(paramMetadata);
            }
        }
        token = forward_ref_1.resolveForwardRef(token);
        if (lang_1.isPresent(token)) {
            return _createDependency(token, optional, lowerBoundVisibility, upperBoundVisibility, depProps);
        } else {
            throw new reflective_exceptions_1.NoAnnotationError(typeOrFunc, params);
        }
    }
    function _createDependency(token /** TODO #9100 */, optional /** TODO #9100 */, lowerBoundVisibility /** TODO #9100 */, upperBoundVisibility /** TODO #9100 */, depProps /** TODO #9100 */) {
        return new ReflectiveDependency(reflective_key_1.ReflectiveKey.get(token), optional, lowerBoundVisibility, upperBoundVisibility, depProps);
    }
    
});
System.registerDynamic('node_modules/@angular/core/src/di/forward_ref.js', ['../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('../facade/lang');
    /**
     * Allows to refer to references which are not yet defined.
     *
     * For instance, `forwardRef` is used when the `token` which we need to refer to for the purposes of
     * DI is declared,
     * but not yet defined. It is also used when the `token` which we use when creating a query is not
     * yet defined.
     *
     * ### Example
     * {@example core/di/ts/forward_ref/forward_ref.ts region='forward_ref'}
     * @experimental
     */
    function forwardRef(forwardRefFn) {
        forwardRefFn.__forward_ref__ = forwardRef;
        forwardRefFn.toString = function () {
            return lang_1.stringify(this());
        };
        return forwardRefFn;
    }
    exports.forwardRef = forwardRef;
    /**
     * Lazily retrieves the reference value from a forwardRef.
     *
     * Acts as the identity function when given a non-forward-ref value.
     *
     * ### Example ([live demo](http://plnkr.co/edit/GU72mJrk1fiodChcmiDR?p=preview))
     *
     * ```typescript
     * var ref = forwardRef(() => "refValue");
     * expect(resolveForwardRef(ref)).toEqual("refValue");
     * expect(resolveForwardRef("regularValue")).toEqual("regularValue");
     * ```
     *
     * See: {@link forwardRef}
     * @experimental
     */
    function resolveForwardRef(type) {
        if (lang_1.isFunction(type) && type.hasOwnProperty('__forward_ref__') && type.__forward_ref__ === forwardRef) {
            return type();
        } else {
            return type;
        }
    }
    exports.resolveForwardRef = resolveForwardRef;
    
});
System.registerDynamic('node_modules/@angular/core/src/di/reflective_key.js', ['../facade/exceptions', '../facade/lang', './forward_ref'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    var forward_ref_1 = $__require('./forward_ref');
    /**
     * A unique object used for retrieving items from the {@link ReflectiveInjector}.
     *
     * Keys have:
     * - a system-wide unique `id`.
     * - a `token`.
     *
     * `Key` is used internally by {@link ReflectiveInjector} because its system-wide unique `id` allows
     * the
     * injector to store created objects in a more efficient way.
     *
     * `Key` should not be created directly. {@link ReflectiveInjector} creates keys automatically when
     * resolving
     * providers.
     * @experimental
     */
    var ReflectiveKey = function () {
        /**
         * Private
         */
        function ReflectiveKey(token, id) {
            this.token = token;
            this.id = id;
            if (lang_1.isBlank(token)) {
                throw new exceptions_1.BaseException('Token must be defined!');
            }
        }
        Object.defineProperty(ReflectiveKey.prototype, "displayName", {
            /**
             * Returns a stringified token.
             */
            get: function () {
                return lang_1.stringify(this.token);
            },
            enumerable: true,
            configurable: true
        });
        /**
         * Retrieves a `Key` for a token.
         */
        ReflectiveKey.get = function (token) {
            return _globalKeyRegistry.get(forward_ref_1.resolveForwardRef(token));
        };
        Object.defineProperty(ReflectiveKey, "numberOfKeys", {
            /**
             * @returns the number of keys registered in the system.
             */
            get: function () {
                return _globalKeyRegistry.numberOfKeys;
            },
            enumerable: true,
            configurable: true
        });
        return ReflectiveKey;
    }();
    exports.ReflectiveKey = ReflectiveKey;
    /**
     * @internal
     */
    var KeyRegistry = function () {
        function KeyRegistry() {
            this._allKeys = new Map();
        }
        KeyRegistry.prototype.get = function (token) {
            if (token instanceof ReflectiveKey) return token;
            if (this._allKeys.has(token)) {
                return this._allKeys.get(token);
            }
            var newKey = new ReflectiveKey(token, ReflectiveKey.numberOfKeys);
            this._allKeys.set(token, newKey);
            return newKey;
        };
        Object.defineProperty(KeyRegistry.prototype, "numberOfKeys", {
            get: function () {
                return this._allKeys.size;
            },
            enumerable: true,
            configurable: true
        });
        return KeyRegistry;
    }();
    exports.KeyRegistry = KeyRegistry;
    var _globalKeyRegistry = new KeyRegistry();
    
});
System.registerDynamic('node_modules/@angular/core/src/di/reflective_exceptions.js', ['../facade/collection', '../facade/exceptions', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var collection_1 = $__require('../facade/collection');
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    function findFirstClosedCycle(keys) {
        var res = [];
        for (var i = 0; i < keys.length; ++i) {
            if (collection_1.ListWrapper.contains(res, keys[i])) {
                res.push(keys[i]);
                return res;
            }
            res.push(keys[i]);
        }
        return res;
    }
    function constructResolvingPath(keys) {
        if (keys.length > 1) {
            var reversed = findFirstClosedCycle(collection_1.ListWrapper.reversed(keys));
            var tokenStrs = reversed.map(function (k) {
                return lang_1.stringify(k.token);
            });
            return ' (' + tokenStrs.join(' -> ') + ')';
        }
        return '';
    }
    /**
     * Base class for all errors arising from misconfigured providers.
     * @stable
     */
    var AbstractProviderError = function (_super) {
        __extends(AbstractProviderError, _super);
        function AbstractProviderError(injector, key, constructResolvingMessage) {
            _super.call(this, 'DI Exception');
            this.keys = [key];
            this.injectors = [injector];
            this.constructResolvingMessage = constructResolvingMessage;
            this.message = this.constructResolvingMessage(this.keys);
        }
        AbstractProviderError.prototype.addKey = function (injector, key) {
            this.injectors.push(injector);
            this.keys.push(key);
            this.message = this.constructResolvingMessage(this.keys);
        };
        Object.defineProperty(AbstractProviderError.prototype, "context", {
            get: function () {
                return this.injectors[this.injectors.length - 1].debugContext();
            },
            enumerable: true,
            configurable: true
        });
        return AbstractProviderError;
    }(exceptions_1.BaseException);
    exports.AbstractProviderError = AbstractProviderError;
    /**
     * Thrown when trying to retrieve a dependency by `Key` from {@link Injector}, but the
     * {@link Injector} does not have a {@link Provider} for {@link Key}.
     *
     * ### Example ([live demo](http://plnkr.co/edit/vq8D3FRB9aGbnWJqtEPE?p=preview))
     *
     * ```typescript
     * class A {
     *   constructor(b:B) {}
     * }
     *
     * expect(() => Injector.resolveAndCreate([A])).toThrowError();
     * ```
     * @stable
     */
    var NoProviderError = function (_super) {
        __extends(NoProviderError, _super);
        function NoProviderError(injector, key) {
            _super.call(this, injector, key, function (keys) {
                var first = lang_1.stringify(collection_1.ListWrapper.first(keys).token);
                return "No provider for " + first + "!" + constructResolvingPath(keys);
            });
        }
        return NoProviderError;
    }(AbstractProviderError);
    exports.NoProviderError = NoProviderError;
    /**
     * Thrown when dependencies form a cycle.
     *
     * ### Example ([live demo](http://plnkr.co/edit/wYQdNos0Tzql3ei1EV9j?p=info))
     *
     * ```typescript
     * var injector = Injector.resolveAndCreate([
     *   {provide: "one", useFactory: (two) => "two", deps: [[new Inject("two")]]},
     *   {provide: "two", useFactory: (one) => "one", deps: [[new Inject("one")]]}
     * ]);
     *
     * expect(() => injector.get("one")).toThrowError();
     * ```
     *
     * Retrieving `A` or `B` throws a `CyclicDependencyError` as the graph above cannot be constructed.
     * @stable
     */
    var CyclicDependencyError = function (_super) {
        __extends(CyclicDependencyError, _super);
        function CyclicDependencyError(injector, key) {
            _super.call(this, injector, key, function (keys) {
                return "Cannot instantiate cyclic dependency!" + constructResolvingPath(keys);
            });
        }
        return CyclicDependencyError;
    }(AbstractProviderError);
    exports.CyclicDependencyError = CyclicDependencyError;
    /**
     * Thrown when a constructing type returns with an Error.
     *
     * The `InstantiationError` class contains the original error plus the dependency graph which caused
     * this object to be instantiated.
     *
     * ### Example ([live demo](http://plnkr.co/edit/7aWYdcqTQsP0eNqEdUAf?p=preview))
     *
     * ```typescript
     * class A {
     *   constructor() {
     *     throw new Error('message');
     *   }
     * }
     *
     * var injector = Injector.resolveAndCreate([A]);
    
     * try {
     *   injector.get(A);
     * } catch (e) {
     *   expect(e instanceof InstantiationError).toBe(true);
     *   expect(e.originalException.message).toEqual("message");
     *   expect(e.originalStack).toBeDefined();
     * }
     * ```
     * @stable
     */
    var InstantiationError = function (_super) {
        __extends(InstantiationError, _super);
        function InstantiationError(injector, originalException, originalStack, key) {
            _super.call(this, 'DI Exception', originalException, originalStack, null);
            this.keys = [key];
            this.injectors = [injector];
        }
        InstantiationError.prototype.addKey = function (injector, key) {
            this.injectors.push(injector);
            this.keys.push(key);
        };
        Object.defineProperty(InstantiationError.prototype, "wrapperMessage", {
            get: function () {
                var first = lang_1.stringify(collection_1.ListWrapper.first(this.keys).token);
                return "Error during instantiation of " + first + "!" + constructResolvingPath(this.keys) + ".";
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(InstantiationError.prototype, "causeKey", {
            get: function () {
                return this.keys[0];
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(InstantiationError.prototype, "context", {
            get: function () {
                return this.injectors[this.injectors.length - 1].debugContext();
            },
            enumerable: true,
            configurable: true
        });
        return InstantiationError;
    }(exceptions_1.WrappedException);
    exports.InstantiationError = InstantiationError;
    /**
     * Thrown when an object other then {@link Provider} (or `Type`) is passed to {@link Injector}
     * creation.
     *
     * ### Example ([live demo](http://plnkr.co/edit/YatCFbPAMCL0JSSQ4mvH?p=preview))
     *
     * ```typescript
     * expect(() => Injector.resolveAndCreate(["not a type"])).toThrowError();
     * ```
     * @stable
     */
    var InvalidProviderError = function (_super) {
        __extends(InvalidProviderError, _super);
        function InvalidProviderError(provider) {
            _super.call(this, "Invalid provider - only instances of Provider and Type are allowed, got: " + provider);
        }
        return InvalidProviderError;
    }(exceptions_1.BaseException);
    exports.InvalidProviderError = InvalidProviderError;
    /**
     * Thrown when the class has no annotation information.
     *
     * Lack of annotation information prevents the {@link Injector} from determining which dependencies
     * need to be injected into the constructor.
     *
     * ### Example ([live demo](http://plnkr.co/edit/rHnZtlNS7vJOPQ6pcVkm?p=preview))
     *
     * ```typescript
     * class A {
     *   constructor(b) {}
     * }
     *
     * expect(() => Injector.resolveAndCreate([A])).toThrowError();
     * ```
     *
     * This error is also thrown when the class not marked with {@link Injectable} has parameter types.
     *
     * ```typescript
     * class B {}
     *
     * class A {
     *   constructor(b:B) {} // no information about the parameter types of A is available at runtime.
     * }
     *
     * expect(() => Injector.resolveAndCreate([A,B])).toThrowError();
     * ```
     * @stable
     */
    var NoAnnotationError = function (_super) {
        __extends(NoAnnotationError, _super);
        function NoAnnotationError(typeOrFunc, params) {
            _super.call(this, NoAnnotationError._genMessage(typeOrFunc, params));
        }
        NoAnnotationError._genMessage = function (typeOrFunc, params) {
            var signature = [];
            for (var i = 0, ii = params.length; i < ii; i++) {
                var parameter = params[i];
                if (lang_1.isBlank(parameter) || parameter.length == 0) {
                    signature.push('?');
                } else {
                    signature.push(parameter.map(lang_1.stringify).join(' '));
                }
            }
            return 'Cannot resolve all parameters for \'' + lang_1.stringify(typeOrFunc) + '\'(' + signature.join(', ') + '). ' + 'Make sure that all the parameters are decorated with Inject or have valid type annotations and that \'' + lang_1.stringify(typeOrFunc) + '\' is decorated with Injectable.';
        };
        return NoAnnotationError;
    }(exceptions_1.BaseException);
    exports.NoAnnotationError = NoAnnotationError;
    /**
     * Thrown when getting an object by index.
     *
     * ### Example ([live demo](http://plnkr.co/edit/bRs0SX2OTQiJzqvjgl8P?p=preview))
     *
     * ```typescript
     * class A {}
     *
     * var injector = Injector.resolveAndCreate([A]);
     *
     * expect(() => injector.getAt(100)).toThrowError();
     * ```
     * @stable
     */
    var OutOfBoundsError = function (_super) {
        __extends(OutOfBoundsError, _super);
        function OutOfBoundsError(index) {
            _super.call(this, "Index " + index + " is out-of-bounds.");
        }
        return OutOfBoundsError;
    }(exceptions_1.BaseException);
    exports.OutOfBoundsError = OutOfBoundsError;
    // TODO: add a working example after alpha38 is released
    /**
     * Thrown when a multi provider and a regular provider are bound to the same token.
     *
     * ### Example
     *
     * ```typescript
     * expect(() => Injector.resolveAndCreate([
     *   new Provider("Strings", {useValue: "string1", multi: true}),
     *   new Provider("Strings", {useValue: "string2", multi: false})
     * ])).toThrowError();
     * ```
     */
    var MixingMultiProvidersWithRegularProvidersError = function (_super) {
        __extends(MixingMultiProvidersWithRegularProvidersError, _super);
        function MixingMultiProvidersWithRegularProvidersError(provider1, provider2) {
            _super.call(this, 'Cannot mix multi providers and regular providers, got: ' + provider1.toString() + ' ' + provider2.toString());
        }
        return MixingMultiProvidersWithRegularProvidersError;
    }(exceptions_1.BaseException);
    exports.MixingMultiProvidersWithRegularProvidersError = MixingMultiProvidersWithRegularProvidersError;
    
});
System.registerDynamic("node_modules/@angular/core/src/di/opaque_token.js", [], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * Creates a token that can be used in a DI Provider.
   *
   * ### Example ([live demo](http://plnkr.co/edit/Ys9ezXpj2Mnoy3Uc8KBp?p=preview))
   *
   * ```typescript
   * var t = new OpaqueToken("value");
   *
   * var injector = Injector.resolveAndCreate([
   *   {provide: t, useValue: "bindingValue"}
   * ]);
   *
   * expect(injector.get(t)).toEqual("bindingValue");
   * ```
   *
   * Using an `OpaqueToken` is preferable to using strings as tokens because of possible collisions
   * caused by multiple providers using the same string as two different tokens.
   *
   * Using an `OpaqueToken` is preferable to using an `Object` as tokens because it provides better
   * error messages.
   * @ts2dart_const
   * @stable
   */

  var global = this || self,
      GLOBAL = global;
  var OpaqueToken = function () {
    function OpaqueToken(_desc) {
      this._desc = _desc;
    }
    OpaqueToken.prototype.toString = function () {
      return "Token " + this._desc;
    };
    return OpaqueToken;
  }();
  exports.OpaqueToken = OpaqueToken;
  
});
System.registerDynamic('node_modules/@angular/core/src/di.js', ['./di/metadata', './di/decorators', './di/forward_ref', './di/injector', './di/reflective_injector', './di/provider', './di/reflective_provider', './di/reflective_key', './di/reflective_exceptions', './di/opaque_token'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
  }
  /**
   * @module
   * @description
   * The `di` module provides dependency injection container services.
   */
  var metadata_1 = $__require('./di/metadata');
  exports.HostMetadata = metadata_1.HostMetadata;
  exports.InjectMetadata = metadata_1.InjectMetadata;
  exports.InjectableMetadata = metadata_1.InjectableMetadata;
  exports.OptionalMetadata = metadata_1.OptionalMetadata;
  exports.SelfMetadata = metadata_1.SelfMetadata;
  exports.SkipSelfMetadata = metadata_1.SkipSelfMetadata;
  // we have to reexport * because Dart and TS export two different sets of types
  __export($__require('./di/decorators'));
  var forward_ref_1 = $__require('./di/forward_ref');
  exports.forwardRef = forward_ref_1.forwardRef;
  exports.resolveForwardRef = forward_ref_1.resolveForwardRef;
  var injector_1 = $__require('./di/injector');
  exports.Injector = injector_1.Injector;
  var reflective_injector_1 = $__require('./di/reflective_injector');
  exports.ReflectiveInjector = reflective_injector_1.ReflectiveInjector;
  var provider_1 = $__require('./di/provider');
  exports.Binding = provider_1.Binding;
  exports.ProviderBuilder = provider_1.ProviderBuilder;
  exports.bind = provider_1.bind;
  exports.Provider = provider_1.Provider;
  exports.provide = provider_1.provide;
  var reflective_provider_1 = $__require('./di/reflective_provider');
  exports.ResolvedReflectiveFactory = reflective_provider_1.ResolvedReflectiveFactory;
  var reflective_key_1 = $__require('./di/reflective_key');
  exports.ReflectiveKey = reflective_key_1.ReflectiveKey;
  var reflective_exceptions_1 = $__require('./di/reflective_exceptions');
  exports.NoProviderError = reflective_exceptions_1.NoProviderError;
  exports.AbstractProviderError = reflective_exceptions_1.AbstractProviderError;
  exports.CyclicDependencyError = reflective_exceptions_1.CyclicDependencyError;
  exports.InstantiationError = reflective_exceptions_1.InstantiationError;
  exports.InvalidProviderError = reflective_exceptions_1.InvalidProviderError;
  exports.NoAnnotationError = reflective_exceptions_1.NoAnnotationError;
  exports.OutOfBoundsError = reflective_exceptions_1.OutOfBoundsError;
  var opaque_token_1 = $__require('./di/opaque_token');
  exports.OpaqueToken = opaque_token_1.OpaqueToken;
  
});
System.registerDynamic('node_modules/@angular/core/src/change_detection/differs/keyvalue_differs.js', ['../../di', '../../facade/collection', '../../facade/exceptions', '../../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var di_1 = $__require('../../di');
    var collection_1 = $__require('../../facade/collection');
    var exceptions_1 = $__require('../../facade/exceptions');
    var lang_1 = $__require('../../facade/lang');
    /**
     * A repository of different Map diffing strategies used by NgClass, NgStyle, and others.
     * @ts2dart_const
     * @stable
     */
    var KeyValueDiffers = function () {
        /*@ts2dart_const*/
        function KeyValueDiffers(factories) {
            this.factories = factories;
        }
        KeyValueDiffers.create = function (factories, parent) {
            if (lang_1.isPresent(parent)) {
                var copied = collection_1.ListWrapper.clone(parent.factories);
                factories = factories.concat(copied);
                return new KeyValueDiffers(factories);
            } else {
                return new KeyValueDiffers(factories);
            }
        };
        /**
         * Takes an array of {@link KeyValueDifferFactory} and returns a provider used to extend the
         * inherited {@link KeyValueDiffers} instance with the provided factories and return a new
         * {@link KeyValueDiffers} instance.
         *
         * The following example shows how to extend an existing list of factories,
               * which will only be applied to the injector for this component and its children.
               * This step is all that's required to make a new {@link KeyValueDiffer} available.
         *
         * ### Example
         *
         * ```
         * @Component({
         *   viewProviders: [
         *     KeyValueDiffers.extend([new ImmutableMapDiffer()])
         *   ]
         * })
         * ```
         */
        KeyValueDiffers.extend = function (factories) {
            return new di_1.Provider(KeyValueDiffers, {
                useFactory: function (parent) {
                    if (lang_1.isBlank(parent)) {
                        // Typically would occur when calling KeyValueDiffers.extend inside of dependencies passed
                        // to
                        // bootstrap(), which would override default pipes instead of extending them.
                        throw new exceptions_1.BaseException('Cannot extend KeyValueDiffers without a parent injector');
                    }
                    return KeyValueDiffers.create(factories, parent);
                },
                // Dependency technically isn't optional, but we can provide a better error message this way.
                deps: [[KeyValueDiffers, new di_1.SkipSelfMetadata(), new di_1.OptionalMetadata()]]
            });
        };
        KeyValueDiffers.prototype.find = function (kv) {
            var factory = this.factories.find(function (f) {
                return f.supports(kv);
            });
            if (lang_1.isPresent(factory)) {
                return factory;
            } else {
                throw new exceptions_1.BaseException("Cannot find a differ supporting object '" + kv + "'");
            }
        };
        return KeyValueDiffers;
    }();
    exports.KeyValueDiffers = KeyValueDiffers;
    
});
System.registerDynamic("node_modules/@angular/core/src/change_detection/change_detector_ref.js", [], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * @stable
   */

  var global = this || self,
      GLOBAL = global;
  var ChangeDetectorRef = function () {
    function ChangeDetectorRef() {}
    return ChangeDetectorRef;
  }();
  exports.ChangeDetectorRef = ChangeDetectorRef;
  
});
System.registerDynamic("node_modules/@angular/core/src/change_detection/constants.js", ["../facade/lang"], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var lang_1 = $__require('../facade/lang');
  /**
   * Describes within the change detector which strategy will be used the next time change
   * detection is triggered.
   * @stable
   */
  (function (ChangeDetectionStrategy) {
    /**
     * `OnPush` means that the change detector's mode will be set to `CheckOnce` during hydration.
     */
    ChangeDetectionStrategy[ChangeDetectionStrategy["OnPush"] = 0] = "OnPush";
    /**
     * `Default` means that the change detector's mode will be set to `CheckAlways` during hydration.
     */
    ChangeDetectionStrategy[ChangeDetectionStrategy["Default"] = 1] = "Default";
  })(exports.ChangeDetectionStrategy || (exports.ChangeDetectionStrategy = {}));
  var ChangeDetectionStrategy = exports.ChangeDetectionStrategy;
  /**
   * Describes the status of the detector.
   */
  (function (ChangeDetectorStatus) {
    /**
     * `CheckedOnce` means that after calling detectChanges the mode of the change detector
     * will become `Checked`.
     */
    ChangeDetectorStatus[ChangeDetectorStatus["CheckOnce"] = 0] = "CheckOnce";
    /**
     * `Checked` means that the change detector should be skipped until its mode changes to
     * `CheckOnce`.
     */
    ChangeDetectorStatus[ChangeDetectorStatus["Checked"] = 1] = "Checked";
    /**
     * `CheckAlways` means that after calling detectChanges the mode of the change detector
     * will remain `CheckAlways`.
     */
    ChangeDetectorStatus[ChangeDetectorStatus["CheckAlways"] = 2] = "CheckAlways";
    /**
     * `Detached` means that the change detector sub tree is not a part of the main tree and
     * should be skipped.
     */
    ChangeDetectorStatus[ChangeDetectorStatus["Detached"] = 3] = "Detached";
    /**
     * `Errored` means that the change detector encountered an error checking a binding
     * or calling a directive lifecycle method and is now in an inconsistent state. Change
     * detectors in this state will no longer detect changes.
     */
    ChangeDetectorStatus[ChangeDetectorStatus["Errored"] = 4] = "Errored";
    /**
     * `Destroyed` means that the change detector is destroyed.
     */
    ChangeDetectorStatus[ChangeDetectorStatus["Destroyed"] = 5] = "Destroyed";
  })(exports.ChangeDetectorStatus || (exports.ChangeDetectorStatus = {}));
  var ChangeDetectorStatus = exports.ChangeDetectorStatus;
  /**
   * List of possible {@link ChangeDetectionStrategy} values.
   */
  exports.CHANGE_DETECTION_STRATEGY_VALUES = [ChangeDetectionStrategy.OnPush, ChangeDetectionStrategy.Default];
  /**
   * List of possible {@link ChangeDetectorStatus} values.
   */
  exports.CHANGE_DETECTOR_STATUS_VALUES = [ChangeDetectorStatus.CheckOnce, ChangeDetectorStatus.Checked, ChangeDetectorStatus.CheckAlways, ChangeDetectorStatus.Detached, ChangeDetectorStatus.Errored, ChangeDetectorStatus.Destroyed];
  function isDefaultChangeDetectionStrategy(changeDetectionStrategy) {
    return lang_1.isBlank(changeDetectionStrategy) || changeDetectionStrategy === ChangeDetectionStrategy.Default;
  }
  exports.isDefaultChangeDetectionStrategy = isDefaultChangeDetectionStrategy;
  
});
System.registerDynamic('node_modules/@angular/core/src/change_detection/change_detection.js', ['./differs/default_iterable_differ', './differs/default_keyvalue_differ', './differs/iterable_differs', './differs/keyvalue_differs', './change_detection_util', './change_detector_ref', './constants'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var default_iterable_differ_1 = $__require('./differs/default_iterable_differ');
  var default_keyvalue_differ_1 = $__require('./differs/default_keyvalue_differ');
  var iterable_differs_1 = $__require('./differs/iterable_differs');
  var keyvalue_differs_1 = $__require('./differs/keyvalue_differs');
  var change_detection_util_1 = $__require('./change_detection_util');
  exports.SimpleChange = change_detection_util_1.SimpleChange;
  exports.ValueUnwrapper = change_detection_util_1.ValueUnwrapper;
  exports.WrappedValue = change_detection_util_1.WrappedValue;
  exports.devModeEqual = change_detection_util_1.devModeEqual;
  exports.looseIdentical = change_detection_util_1.looseIdentical;
  exports.uninitialized = change_detection_util_1.uninitialized;
  var change_detector_ref_1 = $__require('./change_detector_ref');
  exports.ChangeDetectorRef = change_detector_ref_1.ChangeDetectorRef;
  var constants_1 = $__require('./constants');
  exports.CHANGE_DETECTION_STRATEGY_VALUES = constants_1.CHANGE_DETECTION_STRATEGY_VALUES;
  exports.ChangeDetectionStrategy = constants_1.ChangeDetectionStrategy;
  exports.ChangeDetectorStatus = constants_1.ChangeDetectorStatus;
  exports.isDefaultChangeDetectionStrategy = constants_1.isDefaultChangeDetectionStrategy;
  var default_iterable_differ_2 = $__require('./differs/default_iterable_differ');
  exports.CollectionChangeRecord = default_iterable_differ_2.CollectionChangeRecord;
  exports.DefaultIterableDifferFactory = default_iterable_differ_2.DefaultIterableDifferFactory;
  var default_iterable_differ_3 = $__require('./differs/default_iterable_differ');
  exports.DefaultIterableDiffer = default_iterable_differ_3.DefaultIterableDiffer;
  var default_keyvalue_differ_2 = $__require('./differs/default_keyvalue_differ');
  exports.DefaultKeyValueDifferFactory = default_keyvalue_differ_2.DefaultKeyValueDifferFactory;
  exports.KeyValueChangeRecord = default_keyvalue_differ_2.KeyValueChangeRecord;
  var iterable_differs_2 = $__require('./differs/iterable_differs');
  exports.IterableDiffers = iterable_differs_2.IterableDiffers;
  var keyvalue_differs_2 = $__require('./differs/keyvalue_differs');
  exports.KeyValueDiffers = keyvalue_differs_2.KeyValueDiffers;
  /**
   * Structural diffing for `Object`s and `Map`s.
   */
  exports.keyValDiff =
  /*@ts2dart_const*/[new default_keyvalue_differ_1.DefaultKeyValueDifferFactory()];
  /**
   * Structural diffing for `Iterable` types such as `Array`s.
   */
  exports.iterableDiff =
  /*@ts2dart_const*/[new default_iterable_differ_1.DefaultIterableDifferFactory()];
  exports.defaultIterableDiffers = new iterable_differs_1.IterableDiffers(exports.iterableDiff);
  exports.defaultKeyValueDiffers = new keyvalue_differs_1.KeyValueDiffers(exports.keyValDiff);
  
});
System.registerDynamic('node_modules/@angular/core/src/change_detection/change_detection_util.js', ['../facade/collection', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var collection_1 = $__require('../facade/collection');
    var lang_1 = $__require('../facade/lang');
    var lang_2 = $__require('../facade/lang');
    exports.looseIdentical = lang_2.looseIdentical;
    exports.uninitialized = new Object();
    function devModeEqual(a, b) {
        if (collection_1.isListLikeIterable(a) && collection_1.isListLikeIterable(b)) {
            return collection_1.areIterablesEqual(a, b, devModeEqual);
        } else if (!collection_1.isListLikeIterable(a) && !lang_1.isPrimitive(a) && !collection_1.isListLikeIterable(b) && !lang_1.isPrimitive(b)) {
            return true;
        } else {
            return lang_1.looseIdentical(a, b);
        }
    }
    exports.devModeEqual = devModeEqual;
    /**
     * Indicates that the result of a {@link PipeMetadata} transformation has changed even though the
     * reference
     * has not changed.
     *
     * The wrapped value will be unwrapped by change detection, and the unwrapped value will be stored.
     *
     * Example:
     *
     * ```
     * if (this._latestValue === this._latestReturnedValue) {
     *    return this._latestReturnedValue;
     *  } else {
     *    this._latestReturnedValue = this._latestValue;
     *    return WrappedValue.wrap(this._latestValue); // this will force update
     *  }
     * ```
     * @stable
     */
    var WrappedValue = function () {
        function WrappedValue(wrapped) {
            this.wrapped = wrapped;
        }
        WrappedValue.wrap = function (value) {
            return new WrappedValue(value);
        };
        return WrappedValue;
    }();
    exports.WrappedValue = WrappedValue;
    /**
     * Helper class for unwrapping WrappedValue s
     */
    var ValueUnwrapper = function () {
        function ValueUnwrapper() {
            this.hasWrappedValue = false;
        }
        ValueUnwrapper.prototype.unwrap = function (value) {
            if (value instanceof WrappedValue) {
                this.hasWrappedValue = true;
                return value.wrapped;
            }
            return value;
        };
        ValueUnwrapper.prototype.reset = function () {
            this.hasWrappedValue = false;
        };
        return ValueUnwrapper;
    }();
    exports.ValueUnwrapper = ValueUnwrapper;
    /**
     * Represents a basic change from a previous to a new value.
     * @stable
     */
    var SimpleChange = function () {
        function SimpleChange(previousValue, currentValue) {
            this.previousValue = previousValue;
            this.currentValue = currentValue;
        }
        /**
         * Check whether the new value is the first value assigned.
         */
        SimpleChange.prototype.isFirstChange = function () {
            return this.previousValue === exports.uninitialized;
        };
        return SimpleChange;
    }();
    exports.SimpleChange = SimpleChange;
    
});
System.registerDynamic("node_modules/@angular/core/src/di/metadata.js", ["../facade/lang"], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('../facade/lang');
    /**
     * A parameter metadata that specifies a dependency.
     *
     * ### Example ([live demo](http://plnkr.co/edit/6uHYJK?p=preview))
     *
     * ```typescript
     * class Engine {}
     *
     * @Injectable()
     * class Car {
     *   engine;
     *   constructor(@Inject("MyEngine") engine:Engine) {
     *     this.engine = engine;
     *   }
     * }
     *
     * var injector = Injector.resolveAndCreate([
     *  {provide: "MyEngine", useClass: Engine},
     *  Car
     * ]);
     *
     * expect(injector.get(Car).engine instanceof Engine).toBe(true);
     * ```
     *
     * When `@Inject()` is not present, {@link Injector} will use the type annotation of the parameter.
     *
     * ### Example
     *
     * ```typescript
     * class Engine {}
     *
     * @Injectable()
     * class Car {
     *   constructor(public engine: Engine) {} //same as constructor(@Inject(Engine) engine:Engine)
     * }
     *
     * var injector = Injector.resolveAndCreate([Engine, Car]);
     * expect(injector.get(Car).engine instanceof Engine).toBe(true);
     * ```
     * @ts2dart_const
     * @stable
     */
    var InjectMetadata = function () {
        function InjectMetadata(token) {
            this.token = token;
        }
        InjectMetadata.prototype.toString = function () {
            return "@Inject(" + lang_1.stringify(this.token) + ")";
        };
        return InjectMetadata;
    }();
    exports.InjectMetadata = InjectMetadata;
    /**
     * A parameter metadata that marks a dependency as optional. {@link Injector} provides `null` if
     * the dependency is not found.
     *
     * ### Example ([live demo](http://plnkr.co/edit/AsryOm?p=preview))
     *
     * ```typescript
     * class Engine {}
     *
     * @Injectable()
     * class Car {
     *   engine;
     *   constructor(@Optional() engine:Engine) {
     *     this.engine = engine;
     *   }
     * }
     *
     * var injector = Injector.resolveAndCreate([Car]);
     * expect(injector.get(Car).engine).toBeNull();
     * ```
     * @ts2dart_const
     * @stable
     */
    var OptionalMetadata = function () {
        function OptionalMetadata() {}
        OptionalMetadata.prototype.toString = function () {
            return "@Optional()";
        };
        return OptionalMetadata;
    }();
    exports.OptionalMetadata = OptionalMetadata;
    /**
     * `DependencyMetadata` is used by the framework to extend DI.
     * This is internal to Angular and should not be used directly.
     * @ts2dart_const
     * @stable
     */
    var DependencyMetadata = function () {
        function DependencyMetadata() {}
        Object.defineProperty(DependencyMetadata.prototype, "token", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        return DependencyMetadata;
    }();
    exports.DependencyMetadata = DependencyMetadata;
    /**
     * A marker metadata that marks a class as available to {@link Injector} for creation.
     *
     * ### Example ([live demo](http://plnkr.co/edit/Wk4DMQ?p=preview))
     *
     * ```typescript
     * @Injectable()
     * class UsefulService {}
     *
     * @Injectable()
     * class NeedsService {
     *   constructor(public service:UsefulService) {}
     * }
     *
     * var injector = Injector.resolveAndCreate([NeedsService, UsefulService]);
     * expect(injector.get(NeedsService).service instanceof UsefulService).toBe(true);
     * ```
     * {@link Injector} will throw {@link NoAnnotationError} when trying to instantiate a class that
     * does not have `@Injectable` marker, as shown in the example below.
     *
     * ```typescript
     * class UsefulService {}
     *
     * class NeedsService {
     *   constructor(public service:UsefulService) {}
     * }
     *
     * var injector = Injector.resolveAndCreate([NeedsService, UsefulService]);
     * expect(() => injector.get(NeedsService)).toThrowError();
     * ```
     * @ts2dart_const
     * @stable
     */
    var InjectableMetadata = function () {
        function InjectableMetadata() {}
        return InjectableMetadata;
    }();
    exports.InjectableMetadata = InjectableMetadata;
    /**
     * Specifies that an {@link Injector} should retrieve a dependency only from itself.
     *
     * ### Example ([live demo](http://plnkr.co/edit/NeagAg?p=preview))
     *
     * ```typescript
     * class Dependency {
     * }
     *
     * @Injectable()
     * class NeedsDependency {
     *   dependency;
     *   constructor(@Self() dependency:Dependency) {
     *     this.dependency = dependency;
     *   }
     * }
     *
     * var inj = Injector.resolveAndCreate([Dependency, NeedsDependency]);
     * var nd = inj.get(NeedsDependency);
     *
     * expect(nd.dependency instanceof Dependency).toBe(true);
     *
     * var inj = Injector.resolveAndCreate([Dependency]);
     * var child = inj.resolveAndCreateChild([NeedsDependency]);
     * expect(() => child.get(NeedsDependency)).toThrowError();
     * ```
     * @ts2dart_const
     * @stable
     */
    var SelfMetadata = function () {
        function SelfMetadata() {}
        SelfMetadata.prototype.toString = function () {
            return "@Self()";
        };
        return SelfMetadata;
    }();
    exports.SelfMetadata = SelfMetadata;
    /**
     * Specifies that the dependency resolution should start from the parent injector.
     *
     * ### Example ([live demo](http://plnkr.co/edit/Wchdzb?p=preview))
     *
     * ```typescript
     * class Dependency {
     * }
     *
     * @Injectable()
     * class NeedsDependency {
     *   dependency;
     *   constructor(@SkipSelf() dependency:Dependency) {
     *     this.dependency = dependency;
     *   }
     * }
     *
     * var parent = Injector.resolveAndCreate([Dependency]);
     * var child = parent.resolveAndCreateChild([NeedsDependency]);
     * expect(child.get(NeedsDependency).dependency instanceof Depedency).toBe(true);
     *
     * var inj = Injector.resolveAndCreate([Dependency, NeedsDependency]);
     * expect(() => inj.get(NeedsDependency)).toThrowError();
     * ```
     * @ts2dart_const
     * @stable
     */
    var SkipSelfMetadata = function () {
        function SkipSelfMetadata() {}
        SkipSelfMetadata.prototype.toString = function () {
            return "@SkipSelf()";
        };
        return SkipSelfMetadata;
    }();
    exports.SkipSelfMetadata = SkipSelfMetadata;
    /**
     * Specifies that an injector should retrieve a dependency from any injector until reaching the
     * closest host.
     *
     * In Angular, a component element is automatically declared as a host for all the injectors in
     * its view.
     *
     * ### Example ([live demo](http://plnkr.co/edit/GX79pV?p=preview))
     *
     * In the following example `App` contains `ParentCmp`, which contains `ChildDirective`.
     * So `ParentCmp` is the host of `ChildDirective`.
     *
     * `ChildDirective` depends on two services: `HostService` and `OtherService`.
     * `HostService` is defined at `ParentCmp`, and `OtherService` is defined at `App`.
     *
     *```typescript
     * class OtherService {}
     * class HostService {}
     *
     * @Directive({
     *   selector: 'child-directive'
     * })
     * class ChildDirective {
     *   constructor(@Optional() @Host() os:OtherService, @Optional() @Host() hs:HostService){
     *     console.log("os is null", os);
     *     console.log("hs is NOT null", hs);
     *   }
     * }
     *
     * @Component({
     *   selector: 'parent-cmp',
     *   providers: [HostService],
     *   template: `
     *     Dir: <child-directive></child-directive>
     *   `,
     *   directives: [ChildDirective]
     * })
     * class ParentCmp {
     * }
     *
     * @Component({
     *   selector: 'app',
     *   providers: [OtherService],
     *   template: `
     *     Parent: <parent-cmp></parent-cmp>
     *   `,
     *   directives: [ParentCmp]
     * })
     * class App {
     * }
     *
     * bootstrap(App);
     *```
     * @ts2dart_const
     * @stable
     */
    var HostMetadata = function () {
        function HostMetadata() {}
        HostMetadata.prototype.toString = function () {
            return "@Host()";
        };
        return HostMetadata;
    }();
    exports.HostMetadata = HostMetadata;
    
});
System.registerDynamic('node_modules/@angular/core/src/di/decorators.js', ['../util/decorators', './metadata'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var decorators_1 = $__require('../util/decorators');
  var metadata_1 = $__require('./metadata');
  /**
   * Factory for creating {@link InjectMetadata}.
   * @stable
   * @Annotation
   */
  exports.Inject = decorators_1.makeParamDecorator(metadata_1.InjectMetadata);
  /**
   * Factory for creating {@link OptionalMetadata}.
   * @stable
   * @Annotation
   */
  exports.Optional = decorators_1.makeParamDecorator(metadata_1.OptionalMetadata);
  /**
   * Factory for creating {@link InjectableMetadata}.
   * @stable
   * @Annotation
   */
  exports.Injectable = decorators_1.makeDecorator(metadata_1.InjectableMetadata);
  /**
   * Factory for creating {@link SelfMetadata}.
   * @stable
   * @Annotation
   */
  exports.Self = decorators_1.makeParamDecorator(metadata_1.SelfMetadata);
  /**
   * Factory for creating {@link HostMetadata}.
   * @stable
   * @Annotation
   */
  exports.Host = decorators_1.makeParamDecorator(metadata_1.HostMetadata);
  /**
   * Factory for creating {@link SkipSelfMetadata}.
   * @stable
   * @Annotation
   */
  exports.SkipSelf = decorators_1.makeParamDecorator(metadata_1.SkipSelfMetadata);
  
});
System.registerDynamic("node_modules/@angular/core/src/linker/element_ref.js", [], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * A wrapper around a native element inside of a View.
   *
   * An `ElementRef` is backed by a render-specific element. In the browser, this is usually a DOM
   * element.
   *
   * @security Permitting direct access to the DOM can make your application more vulnerable to
   * XSS attacks. Carefully review any use of `ElementRef` in your code. For more detail, see the
   * [Security Guide](http://g.co/ng/security).
   *
   * @stable
   */
  // Note: We don't expose things like `Injector`, `ViewContainer`, ... here,
  // i.e. users have to ask for what they need. With that, we can build better analysis tools
  // and could do better codegen in the future.

  var global = this || self,
      GLOBAL = global;
  var ElementRef = function () {
    function ElementRef(nativeElement) {
      this.nativeElement = nativeElement;
    }
    return ElementRef;
  }();
  exports.ElementRef = ElementRef;
  
});
System.registerDynamic('node_modules/@angular/core/src/profile/wtf_impl.js', ['../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('../facade/lang');
    var trace;
    var events;
    function detectWTF() {
        var wtf = lang_1.global['wtf'];
        if (wtf) {
            trace = wtf['trace'];
            if (trace) {
                events = trace['events'];
                return true;
            }
        }
        return false;
    }
    exports.detectWTF = detectWTF;
    function createScope(signature, flags) {
        if (flags === void 0) {
            flags = null;
        }
        return events.createScope(signature, flags);
    }
    exports.createScope = createScope;
    function leave(scope, returnValue) {
        trace.leaveScope(scope, returnValue);
        return returnValue;
    }
    exports.leave = leave;
    function startTimeRange(rangeType, action) {
        return trace.beginTimeRange(rangeType, action);
    }
    exports.startTimeRange = startTimeRange;
    function endTimeRange(range) {
        trace.endTimeRange(range);
    }
    exports.endTimeRange = endTimeRange;
    
});
System.registerDynamic("node_modules/@angular/core/src/profile/profile.js", ["./wtf_impl"], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var wtf_impl_1 = $__require('./wtf_impl');
  // Change exports to const once https://github.com/angular/ts2dart/issues/150
  /**
   * True if WTF is enabled.
   */
  exports.wtfEnabled = wtf_impl_1.detectWTF();
  function noopScope(arg0, arg1) {
    return null;
  }
  /**
   * Create trace scope.
   *
   * Scopes must be strictly nested and are analogous to stack frames, but
   * do not have to follow the stack frames. Instead it is recommended that they follow logical
   * nesting. You may want to use
   * [Event
   * Signatures](http://google.github.io/tracing-framework/instrumenting-code.html#custom-events)
   * as they are defined in WTF.
   *
   * Used to mark scope entry. The return value is used to leave the scope.
   *
   *     var myScope = wtfCreateScope('MyClass#myMethod(ascii someVal)');
   *
   *     someMethod() {
   *        var s = myScope('Foo'); // 'Foo' gets stored in tracing UI
   *        // DO SOME WORK HERE
   *        return wtfLeave(s, 123); // Return value 123
   *     }
   *
   * Note, adding try-finally block around the work to ensure that `wtfLeave` gets called can
   * negatively impact the performance of your application. For this reason we recommend that
   * you don't add them to ensure that `wtfLeave` gets called. In production `wtfLeave` is a noop and
   * so try-finally block has no value. When debugging perf issues, skipping `wtfLeave`, do to
   * exception, will produce incorrect trace, but presence of exception signifies logic error which
   * needs to be fixed before the app should be profiled. Add try-finally only when you expect that
   * an exception is expected during normal execution while profiling.
   *
   * @experimental
   */
  exports.wtfCreateScope = exports.wtfEnabled ? wtf_impl_1.createScope : function (signature, flags) {
    return noopScope;
  };
  /**
   * Used to mark end of Scope.
   *
   * - `scope` to end.
   * - `returnValue` (optional) to be passed to the WTF.
   *
   * Returns the `returnValue for easy chaining.
   * @experimental
   */
  exports.wtfLeave = exports.wtfEnabled ? wtf_impl_1.leave : function (s, r) {
    return r;
  };
  /**
   * Used to mark Async start. Async are similar to scope but they don't have to be strictly nested.
   * The return value is used in the call to [endAsync]. Async ranges only work if WTF has been
   * enabled.
   *
   *     someMethod() {
   *        var s = wtfStartTimeRange('HTTP:GET', 'some.url');
   *        var future = new Future.delay(5).then((_) {
   *          wtfEndTimeRange(s);
   *        });
   *     }
   * @experimental
   */
  exports.wtfStartTimeRange = exports.wtfEnabled ? wtf_impl_1.startTimeRange : function (rangeType, action) {
    return null;
  };
  /**
   * Ends a async time range operation.
   * [range] is the return value from [wtfStartTimeRange] Async ranges only work if WTF has been
   * enabled.
   * @experimental
   */
  exports.wtfEndTimeRange = exports.wtfEnabled ? wtf_impl_1.endTimeRange : function (r) {
    return null;
  };
  
});
System.registerDynamic('node_modules/@angular/core/src/linker/view_container_ref.js', ['../facade/collection', '../facade/exceptions', '../facade/lang', '../profile/profile'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var collection_1 = $__require('../facade/collection');
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    var profile_1 = $__require('../profile/profile');
    /**
     * Represents a container where one or more Views can be attached.
     *
     * The container can contain two kinds of Views. Host Views, created by instantiating a
     * {@link Component} via {@link #createComponent}, and Embedded Views, created by instantiating an
     * {@link TemplateRef Embedded Template} via {@link #createEmbeddedView}.
     *
     * The location of the View Container within the containing View is specified by the Anchor
     * `element`. Each View Container can have only one Anchor Element and each Anchor Element can only
     * have a single View Container.
     *
     * Root elements of Views attached to this container become siblings of the Anchor Element in
     * the Rendered View.
     *
     * To access a `ViewContainerRef` of an Element, you can either place a {@link Directive} injected
     * with `ViewContainerRef` on the Element, or you obtain it via a {@link ViewChild} query.
     * @stable
     */
    var ViewContainerRef = function () {
        function ViewContainerRef() {}
        Object.defineProperty(ViewContainerRef.prototype, "element", {
            /**
             * Anchor element that specifies the location of this container in the containing View.
             * <!-- TODO: rename to anchorElement -->
             */
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ViewContainerRef.prototype, "injector", {
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ViewContainerRef.prototype, "parentInjector", {
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ViewContainerRef.prototype, "length", {
            /**
             * Returns the number of Views currently attached to this container.
             */
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        ;
        return ViewContainerRef;
    }();
    exports.ViewContainerRef = ViewContainerRef;
    var ViewContainerRef_ = function () {
        function ViewContainerRef_(_element) {
            this._element = _element;
            /** @internal */
            this._createComponentInContainerScope = profile_1.wtfCreateScope('ViewContainerRef#createComponent()');
            /** @internal */
            this._insertScope = profile_1.wtfCreateScope('ViewContainerRef#insert()');
            /** @internal */
            this._removeScope = profile_1.wtfCreateScope('ViewContainerRef#remove()');
            /** @internal */
            this._detachScope = profile_1.wtfCreateScope('ViewContainerRef#detach()');
        }
        ViewContainerRef_.prototype.get = function (index) {
            return this._element.nestedViews[index].ref;
        };
        Object.defineProperty(ViewContainerRef_.prototype, "length", {
            get: function () {
                var views = this._element.nestedViews;
                return lang_1.isPresent(views) ? views.length : 0;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ViewContainerRef_.prototype, "element", {
            get: function () {
                return this._element.elementRef;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ViewContainerRef_.prototype, "injector", {
            get: function () {
                return this._element.injector;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ViewContainerRef_.prototype, "parentInjector", {
            get: function () {
                return this._element.parentInjector;
            },
            enumerable: true,
            configurable: true
        });
        // TODO(rado): profile and decide whether bounds checks should be added
        // to the methods below.
        ViewContainerRef_.prototype.createEmbeddedView = function (templateRef, context, index) {
            if (context === void 0) {
                context = null;
            }
            if (index === void 0) {
                index = -1;
            }
            var viewRef = templateRef.createEmbeddedView(context);
            this.insert(viewRef, index);
            return viewRef;
        };
        ViewContainerRef_.prototype.createComponent = function (componentFactory, index, injector, projectableNodes) {
            if (index === void 0) {
                index = -1;
            }
            if (injector === void 0) {
                injector = null;
            }
            if (projectableNodes === void 0) {
                projectableNodes = null;
            }
            var s = this._createComponentInContainerScope();
            var contextInjector = lang_1.isPresent(injector) ? injector : this._element.parentInjector;
            var componentRef = componentFactory.create(contextInjector, projectableNodes);
            this.insert(componentRef.hostView, index);
            return profile_1.wtfLeave(s, componentRef);
        };
        // TODO(i): refactor insert+remove into move
        ViewContainerRef_.prototype.insert = function (viewRef, index) {
            if (index === void 0) {
                index = -1;
            }
            var s = this._insertScope();
            if (index == -1) index = this.length;
            var viewRef_ = viewRef;
            this._element.attachView(viewRef_.internalView, index);
            return profile_1.wtfLeave(s, viewRef_);
        };
        ViewContainerRef_.prototype.indexOf = function (viewRef) {
            return collection_1.ListWrapper.indexOf(this._element.nestedViews, viewRef.internalView);
        };
        // TODO(i): rename to destroy
        ViewContainerRef_.prototype.remove = function (index) {
            if (index === void 0) {
                index = -1;
            }
            var s = this._removeScope();
            if (index == -1) index = this.length - 1;
            var view = this._element.detachView(index);
            view.destroy();
            // view is intentionally not returned to the client.
            profile_1.wtfLeave(s);
        };
        // TODO(i): refactor insert+remove into move
        ViewContainerRef_.prototype.detach = function (index) {
            if (index === void 0) {
                index = -1;
            }
            var s = this._detachScope();
            if (index == -1) index = this.length - 1;
            var view = this._element.detachView(index);
            return profile_1.wtfLeave(s, view.ref);
        };
        ViewContainerRef_.prototype.clear = function () {
            for (var i = this.length - 1; i >= 0; i--) {
                this.remove(i);
            }
        };
        return ViewContainerRef_;
    }();
    exports.ViewContainerRef_ = ViewContainerRef_;
    
});
System.registerDynamic("node_modules/@angular/core/src/linker/view_type.js", [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    (function (ViewType) {
        // A view that contains the host element with bound component directive.
        // Contains a COMPONENT view
        ViewType[ViewType["HOST"] = 0] = "HOST";
        // The view of the component
        // Can contain 0 to n EMBEDDED views
        ViewType[ViewType["COMPONENT"] = 1] = "COMPONENT";
        // A view that is embedded into another View via a <template> element
        // inside of a COMPONENT view
        ViewType[ViewType["EMBEDDED"] = 2] = "EMBEDDED";
    })(exports.ViewType || (exports.ViewType = {}));
    var ViewType = exports.ViewType;
    
});
System.registerDynamic('node_modules/@angular/core/src/linker/element.js', ['../facade/collection', '../facade/exceptions', '../facade/lang', './element_ref', './view_container_ref', './view_type'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var collection_1 = $__require('../facade/collection');
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    var element_ref_1 = $__require('./element_ref');
    var view_container_ref_1 = $__require('./view_container_ref');
    var view_type_1 = $__require('./view_type');
    /**
     * An AppElement is created for elements that have a ViewContainerRef,
     * a nested component or a <template> element to keep data around
     * that is needed for later instantiations.
     */
    var AppElement = function () {
        function AppElement(index, parentIndex, parentView, nativeElement) {
            this.index = index;
            this.parentIndex = parentIndex;
            this.parentView = parentView;
            this.nativeElement = nativeElement;
            this.nestedViews = null;
            this.componentView = null;
        }
        Object.defineProperty(AppElement.prototype, "elementRef", {
            get: function () {
                return new element_ref_1.ElementRef(this.nativeElement);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AppElement.prototype, "vcRef", {
            get: function () {
                return new view_container_ref_1.ViewContainerRef_(this);
            },
            enumerable: true,
            configurable: true
        });
        AppElement.prototype.initComponent = function (component, componentConstructorViewQueries, view) {
            this.component = component;
            this.componentConstructorViewQueries = componentConstructorViewQueries;
            this.componentView = view;
        };
        Object.defineProperty(AppElement.prototype, "parentInjector", {
            get: function () {
                return this.parentView.injector(this.parentIndex);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(AppElement.prototype, "injector", {
            get: function () {
                return this.parentView.injector(this.index);
            },
            enumerable: true,
            configurable: true
        });
        AppElement.prototype.mapNestedViews = function (nestedViewClass, callback) {
            var result = [];
            if (lang_1.isPresent(this.nestedViews)) {
                this.nestedViews.forEach(function (nestedView) {
                    if (nestedView.clazz === nestedViewClass) {
                        result.push(callback(nestedView));
                    }
                });
            }
            return result;
        };
        AppElement.prototype.attachView = function (view, viewIndex) {
            if (view.type === view_type_1.ViewType.COMPONENT) {
                throw new exceptions_1.BaseException("Component views can't be moved!");
            }
            var nestedViews = this.nestedViews;
            if (nestedViews == null) {
                nestedViews = [];
                this.nestedViews = nestedViews;
            }
            collection_1.ListWrapper.insert(nestedViews, viewIndex, view);
            var refRenderNode;
            if (viewIndex > 0) {
                var prevView = nestedViews[viewIndex - 1];
                refRenderNode = prevView.lastRootNode;
            } else {
                refRenderNode = this.nativeElement;
            }
            if (lang_1.isPresent(refRenderNode)) {
                view.renderer.attachViewAfter(refRenderNode, view.flatRootNodes);
            }
            view.addToContentChildren(this);
        };
        AppElement.prototype.detachView = function (viewIndex) {
            var view = collection_1.ListWrapper.removeAt(this.nestedViews, viewIndex);
            if (view.type === view_type_1.ViewType.COMPONENT) {
                throw new exceptions_1.BaseException("Component views can't be moved!");
            }
            view.detach();
            view.removeFromContentChildren(this);
            return view;
        };
        return AppElement;
    }();
    exports.AppElement = AppElement;
    
});
System.registerDynamic("node_modules/@angular/core/src/linker/exceptions.js", ["../facade/exceptions"], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var exceptions_1 = $__require('../facade/exceptions');
    /**
     * An error thrown if application changes model breaking the top-down data flow.
     *
     * This exception is only thrown in dev mode.
     *
     * <!-- TODO: Add a link once the dev mode option is configurable -->
     *
     * ### Example
     *
     * ```typescript
     * @Component({
     *   selector: 'parent',
     *   template: `
     *     <child [prop]="parentProp"></child>
     *   `,
     *   directives: [forwardRef(() => Child)]
     * })
     * class Parent {
     *   parentProp = "init";
     * }
     *
     * @Directive({selector: 'child', inputs: ['prop']})
     * class Child {
     *   constructor(public parent: Parent) {}
     *
     *   set prop(v) {
     *     // this updates the parent property, which is disallowed during change detection
     *     // this will result in ExpressionChangedAfterItHasBeenCheckedException
     *     this.parent.parentProp = "updated";
     *   }
     * }
     * ```
     * @stable
     */
    var ExpressionChangedAfterItHasBeenCheckedException = function (_super) {
        __extends(ExpressionChangedAfterItHasBeenCheckedException, _super);
        function ExpressionChangedAfterItHasBeenCheckedException(oldValue, currValue, context) {
            _super.call(this, "Expression has changed after it was checked. " + ("Previous value: '" + oldValue + "'. Current value: '" + currValue + "'"));
        }
        return ExpressionChangedAfterItHasBeenCheckedException;
    }(exceptions_1.BaseException);
    exports.ExpressionChangedAfterItHasBeenCheckedException = ExpressionChangedAfterItHasBeenCheckedException;
    /**
     * Thrown when an exception was raised during view creation, change detection or destruction.
     *
     * This error wraps the original exception to attach additional contextual information that can
     * be useful for debugging.
     * @stable
     */
    var ViewWrappedException = function (_super) {
        __extends(ViewWrappedException, _super);
        function ViewWrappedException(originalException, originalStack, context) {
            _super.call(this, "Error in " + context.source, originalException, originalStack, context);
        }
        return ViewWrappedException;
    }(exceptions_1.WrappedException);
    exports.ViewWrappedException = ViewWrappedException;
    /**
     * Thrown when a destroyed view is used.
     *
     * This error indicates a bug in the framework.
     *
     * This is an internal Angular error.
     * @stable
     */
    var ViewDestroyedException = function (_super) {
        __extends(ViewDestroyedException, _super);
        function ViewDestroyedException(details) {
            _super.call(this, "Attempt to use a destroyed view: " + details);
        }
        return ViewDestroyedException;
    }(exceptions_1.BaseException);
    exports.ViewDestroyedException = ViewDestroyedException;
    
});
System.registerDynamic('node_modules/@angular/core/src/linker/view_utils.js', ['../application_tokens', '../change_detection/change_detection', '../change_detection/change_detection_util', '../di/decorators', '../facade/collection', '../facade/exceptions', '../facade/lang', '../render/api', '../security', './element', './exceptions'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var application_tokens_1 = $__require('../application_tokens');
    var change_detection_1 = $__require('../change_detection/change_detection');
    var change_detection_util_1 = $__require('../change_detection/change_detection_util');
    var decorators_1 = $__require('../di/decorators');
    var collection_1 = $__require('../facade/collection');
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    var api_1 = $__require('../render/api');
    var security_1 = $__require('../security');
    var element_1 = $__require('./element');
    var exceptions_2 = $__require('./exceptions');
    var ViewUtils = function () {
        function ViewUtils(_renderer, _appId, sanitizer) {
            this._renderer = _renderer;
            this._appId = _appId;
            this._nextCompTypeId = 0;
            this.sanitizer = sanitizer;
        }
        /**
         * Used by the generated code
         */
        ViewUtils.prototype.createRenderComponentType = function (templateUrl, slotCount, encapsulation, styles) {
            return new api_1.RenderComponentType(this._appId + "-" + this._nextCompTypeId++, templateUrl, slotCount, encapsulation, styles);
        };
        /** @internal */
        ViewUtils.prototype.renderComponent = function (renderComponentType) {
            return this._renderer.renderComponent(renderComponentType);
        };
        /** @nocollapse */
        ViewUtils.decorators = [{ type: decorators_1.Injectable }];
        /** @nocollapse */
        ViewUtils.ctorParameters = [{ type: api_1.RootRenderer }, { type: undefined, decorators: [{ type: decorators_1.Inject, args: [application_tokens_1.APP_ID] }] }, { type: security_1.SanitizationService }];
        return ViewUtils;
    }();
    exports.ViewUtils = ViewUtils;
    function flattenNestedViewRenderNodes(nodes) {
        return _flattenNestedViewRenderNodes(nodes, []);
    }
    exports.flattenNestedViewRenderNodes = flattenNestedViewRenderNodes;
    function _flattenNestedViewRenderNodes(nodes, renderNodes) {
        for (var i = 0; i < nodes.length; i++) {
            var node = nodes[i];
            if (node instanceof element_1.AppElement) {
                var appEl = node;
                renderNodes.push(appEl.nativeElement);
                if (lang_1.isPresent(appEl.nestedViews)) {
                    for (var k = 0; k < appEl.nestedViews.length; k++) {
                        _flattenNestedViewRenderNodes(appEl.nestedViews[k].rootNodesOrAppElements, renderNodes);
                    }
                }
            } else {
                renderNodes.push(node);
            }
        }
        return renderNodes;
    }
    var EMPTY_ARR = [];
    function ensureSlotCount(projectableNodes, expectedSlotCount) {
        var res;
        if (lang_1.isBlank(projectableNodes)) {
            res = EMPTY_ARR;
        } else if (projectableNodes.length < expectedSlotCount) {
            var givenSlotCount = projectableNodes.length;
            res = collection_1.ListWrapper.createFixedSize(expectedSlotCount);
            for (var i = 0; i < expectedSlotCount; i++) {
                res[i] = i < givenSlotCount ? projectableNodes[i] : EMPTY_ARR;
            }
        } else {
            res = projectableNodes;
        }
        return res;
    }
    exports.ensureSlotCount = ensureSlotCount;
    exports.MAX_INTERPOLATION_VALUES = 9;
    function interpolate(valueCount, c0, a1, c1, a2, c2, a3, c3, a4, c4, a5, c5, a6, c6, a7, c7, a8, c8, a9, c9) {
        switch (valueCount) {
            case 1:
                return c0 + _toStringWithNull(a1) + c1;
            case 2:
                return c0 + _toStringWithNull(a1) + c1 + _toStringWithNull(a2) + c2;
            case 3:
                return c0 + _toStringWithNull(a1) + c1 + _toStringWithNull(a2) + c2 + _toStringWithNull(a3) + c3;
            case 4:
                return c0 + _toStringWithNull(a1) + c1 + _toStringWithNull(a2) + c2 + _toStringWithNull(a3) + c3 + _toStringWithNull(a4) + c4;
            case 5:
                return c0 + _toStringWithNull(a1) + c1 + _toStringWithNull(a2) + c2 + _toStringWithNull(a3) + c3 + _toStringWithNull(a4) + c4 + _toStringWithNull(a5) + c5;
            case 6:
                return c0 + _toStringWithNull(a1) + c1 + _toStringWithNull(a2) + c2 + _toStringWithNull(a3) + c3 + _toStringWithNull(a4) + c4 + _toStringWithNull(a5) + c5 + _toStringWithNull(a6) + c6;
            case 7:
                return c0 + _toStringWithNull(a1) + c1 + _toStringWithNull(a2) + c2 + _toStringWithNull(a3) + c3 + _toStringWithNull(a4) + c4 + _toStringWithNull(a5) + c5 + _toStringWithNull(a6) + c6 + _toStringWithNull(a7) + c7;
            case 8:
                return c0 + _toStringWithNull(a1) + c1 + _toStringWithNull(a2) + c2 + _toStringWithNull(a3) + c3 + _toStringWithNull(a4) + c4 + _toStringWithNull(a5) + c5 + _toStringWithNull(a6) + c6 + _toStringWithNull(a7) + c7 + _toStringWithNull(a8) + c8;
            case 9:
                return c0 + _toStringWithNull(a1) + c1 + _toStringWithNull(a2) + c2 + _toStringWithNull(a3) + c3 + _toStringWithNull(a4) + c4 + _toStringWithNull(a5) + c5 + _toStringWithNull(a6) + c6 + _toStringWithNull(a7) + c7 + _toStringWithNull(a8) + c8 + _toStringWithNull(a9) + c9;
            default:
                throw new exceptions_1.BaseException("Does not support more than 9 expressions");
        }
    }
    exports.interpolate = interpolate;
    function _toStringWithNull(v) {
        return v != null ? v.toString() : '';
    }
    function checkBinding(throwOnChange, oldValue, newValue) {
        if (throwOnChange) {
            if (!change_detection_1.devModeEqual(oldValue, newValue)) {
                throw new exceptions_2.ExpressionChangedAfterItHasBeenCheckedException(oldValue, newValue, null);
            }
            return false;
        } else {
            return !lang_1.looseIdentical(oldValue, newValue);
        }
    }
    exports.checkBinding = checkBinding;
    function arrayLooseIdentical(a, b) {
        if (a.length != b.length) return false;
        for (var i = 0; i < a.length; ++i) {
            if (!lang_1.looseIdentical(a[i], b[i])) return false;
        }
        return true;
    }
    exports.arrayLooseIdentical = arrayLooseIdentical;
    function mapLooseIdentical(m1, m2) {
        var k1 = collection_1.StringMapWrapper.keys(m1);
        var k2 = collection_1.StringMapWrapper.keys(m2);
        if (k1.length != k2.length) {
            return false;
        }
        var key;
        for (var i = 0; i < k1.length; i++) {
            key = k1[i];
            if (!lang_1.looseIdentical(m1[key], m2[key])) {
                return false;
            }
        }
        return true;
    }
    exports.mapLooseIdentical = mapLooseIdentical;
    function castByValue(input, value) {
        return input;
    }
    exports.castByValue = castByValue;
    exports.EMPTY_ARRAY = [];
    exports.EMPTY_MAP = {};
    function pureProxy1(fn) {
        var result;
        var v0;
        v0 = change_detection_util_1.uninitialized;
        return function (p0) {
            if (!lang_1.looseIdentical(v0, p0)) {
                v0 = p0;
                result = fn(p0);
            }
            return result;
        };
    }
    exports.pureProxy1 = pureProxy1;
    function pureProxy2(fn) {
        var result;
        var v0 /** TODO #9100 */, v1;
        v0 = v1 = change_detection_util_1.uninitialized;
        return function (p0, p1) {
            if (!lang_1.looseIdentical(v0, p0) || !lang_1.looseIdentical(v1, p1)) {
                v0 = p0;
                v1 = p1;
                result = fn(p0, p1);
            }
            return result;
        };
    }
    exports.pureProxy2 = pureProxy2;
    function pureProxy3(fn) {
        var result;
        var v0 /** TODO #9100 */, v1 /** TODO #9100 */, v2;
        v0 = v1 = v2 = change_detection_util_1.uninitialized;
        return function (p0, p1, p2) {
            if (!lang_1.looseIdentical(v0, p0) || !lang_1.looseIdentical(v1, p1) || !lang_1.looseIdentical(v2, p2)) {
                v0 = p0;
                v1 = p1;
                v2 = p2;
                result = fn(p0, p1, p2);
            }
            return result;
        };
    }
    exports.pureProxy3 = pureProxy3;
    function pureProxy4(fn) {
        var result;
        var v0 /** TODO #9100 */, v1 /** TODO #9100 */, v2 /** TODO #9100 */, v3;
        v0 = v1 = v2 = v3 = change_detection_util_1.uninitialized;
        return function (p0, p1, p2, p3) {
            if (!lang_1.looseIdentical(v0, p0) || !lang_1.looseIdentical(v1, p1) || !lang_1.looseIdentical(v2, p2) || !lang_1.looseIdentical(v3, p3)) {
                v0 = p0;
                v1 = p1;
                v2 = p2;
                v3 = p3;
                result = fn(p0, p1, p2, p3);
            }
            return result;
        };
    }
    exports.pureProxy4 = pureProxy4;
    function pureProxy5(fn) {
        var result;
        var v0 /** TODO #9100 */, v1 /** TODO #9100 */, v2 /** TODO #9100 */, v3 /** TODO #9100 */, v4;
        v0 = v1 = v2 = v3 = v4 = change_detection_util_1.uninitialized;
        return function (p0, p1, p2, p3, p4) {
            if (!lang_1.looseIdentical(v0, p0) || !lang_1.looseIdentical(v1, p1) || !lang_1.looseIdentical(v2, p2) || !lang_1.looseIdentical(v3, p3) || !lang_1.looseIdentical(v4, p4)) {
                v0 = p0;
                v1 = p1;
                v2 = p2;
                v3 = p3;
                v4 = p4;
                result = fn(p0, p1, p2, p3, p4);
            }
            return result;
        };
    }
    exports.pureProxy5 = pureProxy5;
    function pureProxy6(fn) {
        var result;
        var v0 /** TODO #9100 */, v1 /** TODO #9100 */, v2 /** TODO #9100 */, v3 /** TODO #9100 */, v4 /** TODO #9100 */, v5;
        v0 = v1 = v2 = v3 = v4 = v5 = change_detection_util_1.uninitialized;
        return function (p0, p1, p2, p3, p4, p5) {
            if (!lang_1.looseIdentical(v0, p0) || !lang_1.looseIdentical(v1, p1) || !lang_1.looseIdentical(v2, p2) || !lang_1.looseIdentical(v3, p3) || !lang_1.looseIdentical(v4, p4) || !lang_1.looseIdentical(v5, p5)) {
                v0 = p0;
                v1 = p1;
                v2 = p2;
                v3 = p3;
                v4 = p4;
                v5 = p5;
                result = fn(p0, p1, p2, p3, p4, p5);
            }
            return result;
        };
    }
    exports.pureProxy6 = pureProxy6;
    function pureProxy7(fn) {
        var result;
        var v0 /** TODO #9100 */, v1 /** TODO #9100 */, v2 /** TODO #9100 */, v3 /** TODO #9100 */, v4 /** TODO #9100 */, v5 /** TODO #9100 */, v6;
        v0 = v1 = v2 = v3 = v4 = v5 = v6 = change_detection_util_1.uninitialized;
        return function (p0, p1, p2, p3, p4, p5, p6) {
            if (!lang_1.looseIdentical(v0, p0) || !lang_1.looseIdentical(v1, p1) || !lang_1.looseIdentical(v2, p2) || !lang_1.looseIdentical(v3, p3) || !lang_1.looseIdentical(v4, p4) || !lang_1.looseIdentical(v5, p5) || !lang_1.looseIdentical(v6, p6)) {
                v0 = p0;
                v1 = p1;
                v2 = p2;
                v3 = p3;
                v4 = p4;
                v5 = p5;
                v6 = p6;
                result = fn(p0, p1, p2, p3, p4, p5, p6);
            }
            return result;
        };
    }
    exports.pureProxy7 = pureProxy7;
    function pureProxy8(fn) {
        var result;
        var v0 /** TODO #9100 */, v1 /** TODO #9100 */, v2 /** TODO #9100 */, v3 /** TODO #9100 */, v4 /** TODO #9100 */, v5 /** TODO #9100 */, v6 /** TODO #9100 */, v7;
        v0 = v1 = v2 = v3 = v4 = v5 = v6 = v7 = change_detection_util_1.uninitialized;
        return function (p0, p1, p2, p3, p4, p5, p6, p7) {
            if (!lang_1.looseIdentical(v0, p0) || !lang_1.looseIdentical(v1, p1) || !lang_1.looseIdentical(v2, p2) || !lang_1.looseIdentical(v3, p3) || !lang_1.looseIdentical(v4, p4) || !lang_1.looseIdentical(v5, p5) || !lang_1.looseIdentical(v6, p6) || !lang_1.looseIdentical(v7, p7)) {
                v0 = p0;
                v1 = p1;
                v2 = p2;
                v3 = p3;
                v4 = p4;
                v5 = p5;
                v6 = p6;
                v7 = p7;
                result = fn(p0, p1, p2, p3, p4, p5, p6, p7);
            }
            return result;
        };
    }
    exports.pureProxy8 = pureProxy8;
    function pureProxy9(fn) {
        var result;
        var v0 /** TODO #9100 */, v1 /** TODO #9100 */, v2 /** TODO #9100 */, v3 /** TODO #9100 */, v4 /** TODO #9100 */, v5 /** TODO #9100 */, v6 /** TODO #9100 */, v7 /** TODO #9100 */, v8;
        v0 = v1 = v2 = v3 = v4 = v5 = v6 = v7 = v8 = change_detection_util_1.uninitialized;
        return function (p0, p1, p2, p3, p4, p5, p6, p7, p8) {
            if (!lang_1.looseIdentical(v0, p0) || !lang_1.looseIdentical(v1, p1) || !lang_1.looseIdentical(v2, p2) || !lang_1.looseIdentical(v3, p3) || !lang_1.looseIdentical(v4, p4) || !lang_1.looseIdentical(v5, p5) || !lang_1.looseIdentical(v6, p6) || !lang_1.looseIdentical(v7, p7) || !lang_1.looseIdentical(v8, p8)) {
                v0 = p0;
                v1 = p1;
                v2 = p2;
                v3 = p3;
                v4 = p4;
                v5 = p5;
                v6 = p6;
                v7 = p7;
                v8 = p8;
                result = fn(p0, p1, p2, p3, p4, p5, p6, p7, p8);
            }
            return result;
        };
    }
    exports.pureProxy9 = pureProxy9;
    function pureProxy10(fn) {
        var result;
        var v0 /** TODO #9100 */, v1 /** TODO #9100 */, v2 /** TODO #9100 */, v3 /** TODO #9100 */, v4 /** TODO #9100 */, v5 /** TODO #9100 */, v6 /** TODO #9100 */, v7 /** TODO #9100 */, v8 /** TODO #9100 */, v9;
        v0 = v1 = v2 = v3 = v4 = v5 = v6 = v7 = v8 = v9 = change_detection_util_1.uninitialized;
        return function (p0, p1, p2, p3, p4, p5, p6, p7, p8, p9) {
            if (!lang_1.looseIdentical(v0, p0) || !lang_1.looseIdentical(v1, p1) || !lang_1.looseIdentical(v2, p2) || !lang_1.looseIdentical(v3, p3) || !lang_1.looseIdentical(v4, p4) || !lang_1.looseIdentical(v5, p5) || !lang_1.looseIdentical(v6, p6) || !lang_1.looseIdentical(v7, p7) || !lang_1.looseIdentical(v8, p8) || !lang_1.looseIdentical(v9, p9)) {
                v0 = p0;
                v1 = p1;
                v2 = p2;
                v3 = p3;
                v4 = p4;
                v5 = p5;
                v6 = p6;
                v7 = p7;
                v8 = p8;
                v9 = p9;
                result = fn(p0, p1, p2, p3, p4, p5, p6, p7, p8, p9);
            }
            return result;
        };
    }
    exports.pureProxy10 = pureProxy10;
    
});
System.registerDynamic("node_modules/@angular/core/src/metadata/lifecycle_hooks.js", [], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * @stable
   */

  var global = this || self,
      GLOBAL = global;
  (function (LifecycleHooks) {
    LifecycleHooks[LifecycleHooks["OnInit"] = 0] = "OnInit";
    LifecycleHooks[LifecycleHooks["OnDestroy"] = 1] = "OnDestroy";
    LifecycleHooks[LifecycleHooks["DoCheck"] = 2] = "DoCheck";
    LifecycleHooks[LifecycleHooks["OnChanges"] = 3] = "OnChanges";
    LifecycleHooks[LifecycleHooks["AfterContentInit"] = 4] = "AfterContentInit";
    LifecycleHooks[LifecycleHooks["AfterContentChecked"] = 5] = "AfterContentChecked";
    LifecycleHooks[LifecycleHooks["AfterViewInit"] = 6] = "AfterViewInit";
    LifecycleHooks[LifecycleHooks["AfterViewChecked"] = 7] = "AfterViewChecked";
  })(exports.LifecycleHooks || (exports.LifecycleHooks = {}));
  var LifecycleHooks = exports.LifecycleHooks;
  exports.LIFECYCLE_HOOKS_VALUES = [LifecycleHooks.OnInit, LifecycleHooks.OnDestroy, LifecycleHooks.DoCheck, LifecycleHooks.OnChanges, LifecycleHooks.AfterContentInit, LifecycleHooks.AfterContentChecked, LifecycleHooks.AfterViewInit, LifecycleHooks.AfterViewChecked];
  /**
   * Lifecycle hooks are guaranteed to be called in the following order:
   * - `OnChanges` (if any bindings have changed),
   * - `OnInit` (after the first check only),
   * - `DoCheck`,
   * - `AfterContentInit`,
   * - `AfterContentChecked`,
   * - `AfterViewInit`,
   * - `AfterViewChecked`,
   * - `OnDestroy` (at the very end before destruction)
   */
  /**
   * Implement this interface to get notified when any data-bound property of your directive changes.
   *
   * `ngOnChanges` is called right after the data-bound properties have been checked and before view
   * and content children are checked if at least one of them has changed.
   *
   * The `changes` parameter contains an entry for each of the changed data-bound property. The key is
   * the property name and the value is an instance of {@link SimpleChange}.
   *
   * ### Example ([live example](http://plnkr.co/edit/AHrB6opLqHDBPkt4KpdT?p=preview)):
   *
   * ```typescript
   * @Component({
   *   selector: 'my-cmp',
   *   template: `<p>myProp = {{myProp}}</p>`
   * })
   * class MyComponent implements OnChanges {
   *   @Input() myProp: any;
   *
   *   ngOnChanges(changes: SimpleChanges) {
   *     console.log('ngOnChanges - myProp = ' + changes['myProp'].currentValue);
   *   }
   * }
   *
   * @Component({
   *   selector: 'app',
   *   template: `
   *     <button (click)="value = value + 1">Change MyComponent</button>
   *     <my-cmp [my-prop]="value"></my-cmp>`,
   *   directives: [MyComponent]
   * })
   * export class App {
   *   value = 0;
   * }
   *
   * bootstrap(App).catch(err => console.error(err));
   * ```
   * @stable
   */
  var OnChanges = function () {
    function OnChanges() {}
    return OnChanges;
  }();
  exports.OnChanges = OnChanges;
  /**
   * Implement this interface to execute custom initialization logic after your directive's
   * data-bound properties have been initialized.
   *
   * `ngOnInit` is called right after the directive's data-bound properties have been checked for the
   * first time, and before any of its children have been checked. It is invoked only once when the
   * directive is instantiated.
   *
   * ### Example ([live example](http://plnkr.co/edit/1MBypRryXd64v4pV03Yn?p=preview))
   *
   * ```typescript
   * @Component({
   *   selector: 'my-cmp',
   *   template: `<p>my-component</p>`
   * })
   * class MyComponent implements OnInit, OnDestroy {
   *   ngOnInit() {
   *     console.log('ngOnInit');
   *   }
   *
   *   ngOnDestroy() {
   *     console.log('ngOnDestroy');
   *   }
   * }
   *
   * @Component({
   *   selector: 'app',
   *   template: `
   *     <button (click)="hasChild = !hasChild">
   *       {{hasChild ? 'Destroy' : 'Create'}} MyComponent
   *     </button>
   *     <my-cmp *ngIf="hasChild"></my-cmp>`,
   *   directives: [MyComponent, NgIf]
   * })
   * export class App {
   *   hasChild = true;
   * }
   *
   * bootstrap(App).catch(err => console.error(err));
   *  ```
   * @stable
   */
  var OnInit = function () {
    function OnInit() {}
    return OnInit;
  }();
  exports.OnInit = OnInit;
  /**
   * Implement this interface to supplement the default change detection algorithm in your directive.
   *
   * `ngDoCheck` gets called to check the changes in the directives in addition to the default
   * algorithm.
   *
   * The default change detection algorithm looks for differences by comparing bound-property values
   * by reference across change detection runs.
   *
   * Note that a directive typically should not use both `DoCheck` and {@link OnChanges} to respond to
   * changes on the same input. `ngOnChanges` will continue to be called when the default change
   * detector
   * detects changes, so it is usually unnecessary to respond to changes on the same input in both
   * hooks.
   * Reaction to the changes have to be handled from within the `ngDoCheck` callback.
   *
   * You can use {@link KeyValueDiffers} and {@link IterableDiffers} to help add your custom check
   * mechanisms.
   *
   * ### Example ([live demo](http://plnkr.co/edit/QpnIlF0CR2i5bcYbHEUJ?p=preview))
   *
   * In the following example `ngDoCheck` uses an {@link IterableDiffers} to detect the updates to the
   * array `list`:
   *
   * ```typescript
   * @Component({
   *   selector: 'custom-check',
   *   template: `
   *     <p>Changes:</p>
   *     <ul>
   *       <li *ngFor="let line of logs">{{line}}</li>
   *     </ul>`,
   *   directives: [NgFor]
   * })
   * class CustomCheckComponent implements DoCheck {
   *   @Input() list: any[];
   *   differ: any;
   *   logs = [];
   *
   *   constructor(differs: IterableDiffers) {
   *     this.differ = differs.find([]).create(null);
   *   }
   *
   *   ngDoCheck() {
   *     var changes = this.differ.diff(this.list);
   *
   *     if (changes) {
   *       changes.forEachAddedItem(r => this.logs.push('added ' + r.item));
   *       changes.forEachRemovedItem(r => this.logs.push('removed ' + r.item))
   *     }
   *   }
   * }
   *
   * @Component({
   *   selector: 'app',
   *   template: `
   *     <button (click)="list.push(list.length)">Push</button>
   *     <button (click)="list.pop()">Pop</button>
   *     <custom-check [list]="list"></custom-check>`,
   *   directives: [CustomCheckComponent]
   * })
   * export class App {
   *   list = [];
   * }
   * ```
   * @stable
   */
  var DoCheck = function () {
    function DoCheck() {}
    return DoCheck;
  }();
  exports.DoCheck = DoCheck;
  /**
   * Implement this interface to get notified when your directive is destroyed.
   *
   * `ngOnDestroy` callback is typically used for any custom cleanup that needs to occur when the
   * instance is destroyed
   *
   * ### Example ([live example](http://plnkr.co/edit/1MBypRryXd64v4pV03Yn?p=preview))
   *
   * ```typesript
   * @Component({
   *   selector: 'my-cmp',
   *   template: `<p>my-component</p>`
   * })
   * class MyComponent implements OnInit, OnDestroy {
   *   ngOnInit() {
   *     console.log('ngOnInit');
   *   }
   *
   *   ngOnDestroy() {
   *     console.log('ngOnDestroy');
   *   }
   * }
   *
   * @Component({
   *   selector: 'app',
   *   template: `
   *     <button (click)="hasChild = !hasChild">
   *       {{hasChild ? 'Destroy' : 'Create'}} MyComponent
   *     </button>
   *     <my-cmp *ngIf="hasChild"></my-cmp>`,
   *   directives: [MyComponent, NgIf]
   * })
   * export class App {
   *   hasChild = true;
   * }
   *
   * bootstrap(App).catch(err => console.error(err));
   * ```
   *
   *
   * To create a stateful Pipe, you should implement this interface and set the `pure`
   * parameter to `false` in the {@link PipeMetadata}.
   *
   * A stateful pipe may produce different output, given the same input. It is
   * likely that a stateful pipe may contain state that should be cleaned up when
   * a binding is destroyed. For example, a subscription to a stream of data may need to
   * be disposed, or an interval may need to be cleared.
   *
   * ### Example ([live demo](http://plnkr.co/edit/i8pm5brO4sPaLxBx56MR?p=preview))
   *
   * In this example, a pipe is created to countdown its input value, updating it every
   * 50ms. Because it maintains an internal interval, it automatically clears
   * the interval when the binding is destroyed or the countdown completes.
   *
   * ```
   * import {OnDestroy, Pipe, PipeTransform} from '@angular/core'
   * @Pipe({name: 'countdown', pure: false})
   * class CountDown implements PipeTransform, OnDestroy {
   *   remainingTime:Number;
   *   interval:SetInterval;
   *   ngOnDestroy() {
   *     if (this.interval) {
   *       clearInterval(this.interval);
   *     }
   *   }
   *   transform(value: any, args: any[] = []) {
   *     if (!parseInt(value, 10)) return null;
   *     if (typeof this.remainingTime !== 'number') {
   *       this.remainingTime = parseInt(value, 10);
   *     }
   *     if (!this.interval) {
   *       this.interval = setInterval(() => {
   *         this.remainingTime-=50;
   *         if (this.remainingTime <= 0) {
   *           this.remainingTime = 0;
   *           clearInterval(this.interval);
   *           delete this.interval;
   *         }
   *       }, 50);
   *     }
   *     return this.remainingTime;
   *   }
   * }
   * ```
   *
   * Invoking `{{ 10000 | countdown }}` would cause the value to be decremented by 50,
   * every 50ms, until it reaches 0.
   *
   * @stable
   */
  var OnDestroy = function () {
    function OnDestroy() {}
    return OnDestroy;
  }();
  exports.OnDestroy = OnDestroy;
  /**
   * Implement this interface to get notified when your directive's content has been fully
   * initialized.
   *
   * ### Example ([live demo](http://plnkr.co/edit/plamXUpsLQbIXpViZhUO?p=preview))
   *
   * ```typescript
   * @Component({
   *   selector: 'child-cmp',
   *   template: `{{where}} child`
   * })
   * class ChildComponent {
   *   @Input() where: string;
   * }
   *
   * @Component({
   *   selector: 'parent-cmp',
   *   template: `<ng-content></ng-content>`
   * })
   * class ParentComponent implements AfterContentInit {
   *   @ContentChild(ChildComponent) contentChild: ChildComponent;
   *
   *   constructor() {
   *     // contentChild is not initialized yet
   *     console.log(this.getMessage(this.contentChild));
   *   }
   *
   *   ngAfterContentInit() {
   *     // contentChild is updated after the content has been checked
   *     console.log('AfterContentInit: ' + this.getMessage(this.contentChild));
   *   }
   *
   *   private getMessage(cmp: ChildComponent): string {
   *     return cmp ? cmp.where + ' child' : 'no child';
   *   }
   * }
   *
   * @Component({
   *   selector: 'app',
   *   template: `
   *     <parent-cmp>
   *       <child-cmp where="content"></child-cmp>
   *     </parent-cmp>`,
   *   directives: [ParentComponent, ChildComponent]
   * })
   * export class App {
   * }
   *
   * bootstrap(App).catch(err => console.error(err));
   * ```
   * @stable
   */
  var AfterContentInit = function () {
    function AfterContentInit() {}
    return AfterContentInit;
  }();
  exports.AfterContentInit = AfterContentInit;
  /**
   * Implement this interface to get notified after every check of your directive's content.
   *
   * ### Example ([live demo](http://plnkr.co/edit/tGdrytNEKQnecIPkD7NU?p=preview))
   *
   * ```typescript
   * @Component({selector: 'child-cmp', template: `{{where}} child`})
   * class ChildComponent {
   *   @Input() where: string;
   * }
   *
   * @Component({selector: 'parent-cmp', template: `<ng-content></ng-content>`})
   * class ParentComponent implements AfterContentChecked {
   *   @ContentChild(ChildComponent) contentChild: ChildComponent;
   *
   *   constructor() {
   *     // contentChild is not initialized yet
   *     console.log(this.getMessage(this.contentChild));
   *   }
   *
   *   ngAfterContentChecked() {
   *     // contentChild is updated after the content has been checked
   *     console.log('AfterContentChecked: ' + this.getMessage(this.contentChild));
   *   }
   *
   *   private getMessage(cmp: ChildComponent): string {
   *     return cmp ? cmp.where + ' child' : 'no child';
   *   }
   * }
   *
   * @Component({
   *   selector: 'app',
   *   template: `
   *     <parent-cmp>
   *       <button (click)="hasContent = !hasContent">Toggle content child</button>
   *       <child-cmp *ngIf="hasContent" where="content"></child-cmp>
   *     </parent-cmp>`,
   *   directives: [NgIf, ParentComponent, ChildComponent]
   * })
   * export class App {
   *   hasContent = true;
   * }
   *
   * bootstrap(App).catch(err => console.error(err));
   * ```
   * @stable
   */
  var AfterContentChecked = function () {
    function AfterContentChecked() {}
    return AfterContentChecked;
  }();
  exports.AfterContentChecked = AfterContentChecked;
  /**
   * Implement this interface to get notified when your component's view has been fully initialized.
   *
   * ### Example ([live demo](http://plnkr.co/edit/LhTKVMEM0fkJgyp4CI1W?p=preview))
   *
   * ```typescript
   * @Component({selector: 'child-cmp', template: `{{where}} child`})
   * class ChildComponent {
   *   @Input() where: string;
   * }
   *
   * @Component({
   *   selector: 'parent-cmp',
   *   template: `<child-cmp where="view"></child-cmp>`,
   *   directives: [ChildComponent]
   * })
   * class ParentComponent implements AfterViewInit {
   *   @ViewChild(ChildComponent) viewChild: ChildComponent;
   *
   *   constructor() {
   *     // viewChild is not initialized yet
   *     console.log(this.getMessage(this.viewChild));
   *   }
   *
   *   ngAfterViewInit() {
   *     // viewChild is updated after the view has been initialized
   *     console.log('ngAfterViewInit: ' + this.getMessage(this.viewChild));
   *   }
   *
   *   private getMessage(cmp: ChildComponent): string {
   *     return cmp ? cmp.where + ' child' : 'no child';
   *   }
   * }
   *
   * @Component({
   *   selector: 'app',
   *   template: `<parent-cmp></parent-cmp>`,
   *   directives: [ParentComponent]
   * })
   * export class App {
   * }
   *
   * bootstrap(App).catch(err => console.error(err));
   * ```
   * @stable
   */
  var AfterViewInit = function () {
    function AfterViewInit() {}
    return AfterViewInit;
  }();
  exports.AfterViewInit = AfterViewInit;
  /**
   * Implement this interface to get notified after every check of your component's view.
   *
   * ### Example ([live demo](http://plnkr.co/edit/0qDGHcPQkc25CXhTNzKU?p=preview))
   *
   * ```typescript
   * @Component({selector: 'child-cmp', template: `{{where}} child`})
   * class ChildComponent {
   *   @Input() where: string;
   * }
   *
   * @Component({
   *   selector: 'parent-cmp',
   *   template: `
   *     <button (click)="showView = !showView">Toggle view child</button>
   *     <child-cmp *ngIf="showView" where="view"></child-cmp>`,
   *   directives: [NgIf, ChildComponent]
   * })
   * class ParentComponent implements AfterViewChecked {
   *   @ViewChild(ChildComponent) viewChild: ChildComponent;
   *   showView = true;
   *
   *   constructor() {
   *     // viewChild is not initialized yet
   *     console.log(this.getMessage(this.viewChild));
   *   }
   *
   *   ngAfterViewChecked() {
   *     // viewChild is updated after the view has been checked
   *     console.log('AfterViewChecked: ' + this.getMessage(this.viewChild));
   *   }
   *
   *   private getMessage(cmp: ChildComponent): string {
   *     return cmp ? cmp.where + ' child' : 'no child';
   *   }
   * }
   *
   * @Component({
   *   selector: 'app',
   *   template: `<parent-cmp></parent-cmp>`,
   *   directives: [ParentComponent]
   * })
   * export class App {
   * }
   *
   * bootstrap(App).catch(err => console.error(err));
   * ```
   * @stable
   */
  var AfterViewChecked = function () {
    function AfterViewChecked() {}
    return AfterViewChecked;
  }();
  exports.AfterViewChecked = AfterViewChecked;
  
});
System.registerDynamic("node_modules/@angular/core/src/metadata/view.js", [], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * Defines template and style encapsulation options available for Component's {@link View}.
   *
   * See {@link ViewMetadata#encapsulation}.
   * @stable
   */

  var global = this || self,
      GLOBAL = global;
  (function (ViewEncapsulation) {
    /**
     * Emulate `Native` scoping of styles by adding an attribute containing surrogate id to the Host
     * Element and pre-processing the style rules provided via
     * {@link ViewMetadata#styles} or {@link ViewMetadata#stylesUrls}, and adding the new Host Element
     * attribute to all selectors.
     *
     * This is the default option.
     */
    ViewEncapsulation[ViewEncapsulation["Emulated"] = 0] = "Emulated";
    /**
     * Use the native encapsulation mechanism of the renderer.
     *
     * For the DOM this means using [Shadow DOM](https://w3c.github.io/webcomponents/spec/shadow/) and
     * creating a ShadowRoot for Component's Host Element.
     */
    ViewEncapsulation[ViewEncapsulation["Native"] = 1] = "Native";
    /**
     * Don't provide any template or style encapsulation.
     */
    ViewEncapsulation[ViewEncapsulation["None"] = 2] = "None";
  })(exports.ViewEncapsulation || (exports.ViewEncapsulation = {}));
  var ViewEncapsulation = exports.ViewEncapsulation;
  exports.VIEW_ENCAPSULATION_VALUES = [ViewEncapsulation.Emulated, ViewEncapsulation.Native, ViewEncapsulation.None];
  /**
   * Metadata properties available for configuring Views.
   *
   * Each Angular component requires a single `@Component` and at least one `@View` annotation. The
   * `@View` annotation specifies the HTML template to use, and lists the directives that are active
   * within the template.
   *
   * When a component is instantiated, the template is loaded into the component's shadow root, and
   * the expressions and statements in the template are evaluated against the component.
   *
   * For details on the `@Component` annotation, see {@link ComponentMetadata}.
   *
   * ### Example
   *
   * ```
   * @Component({
   *   selector: 'greet',
   *   template: 'Hello {{name}}!',
   *   directives: [GreetUser, Bold]
   * })
   * class Greet {
   *   name: string;
   *
   *   constructor() {
   *     this.name = 'World';
   *   }
   * }
   * ```
   * @ts2dart_const
   *
   * @experimental You should most likely be using ComponentMetadata instead.
   */
  var ViewMetadata = function () {
    function ViewMetadata(_a) {
      var _b = _a === void 0 ? {} : _a,
          templateUrl = _b.templateUrl,
          template = _b.template,
          directives = _b.directives,
          pipes = _b.pipes,
          encapsulation = _b.encapsulation,
          styles = _b.styles,
          styleUrls = _b.styleUrls,
          animations = _b.animations,
          interpolation = _b.interpolation;
      this.templateUrl = templateUrl;
      this.template = template;
      this.styleUrls = styleUrls;
      this.styles = styles;
      this.directives = directives;
      this.pipes = pipes;
      this.encapsulation = encapsulation;
      this.animations = animations;
      this.interpolation = interpolation;
    }
    return ViewMetadata;
  }();
  exports.ViewMetadata = ViewMetadata;
  
});
System.registerDynamic("node_modules/@angular/core/src/profile/wtf_init.js", [], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * This is here because DART requires it. It is noop in JS.
   */

  var global = this || self,
      GLOBAL = global;
  function wtfInit() {}
  exports.wtfInit = wtfInit;
  
});
System.registerDynamic('node_modules/@angular/core/src/reflection/reflector.js', ['../facade/collection', '../facade/exceptions', '../facade/lang', './reflector_reader'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var collection_1 = $__require('../facade/collection');
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    var reflector_reader_1 = $__require('./reflector_reader');
    /**
     * Reflective information about a symbol, including annotations, interfaces, and other metadata.
     */
    var ReflectionInfo = function () {
        function ReflectionInfo(annotations, parameters, factory, interfaces, propMetadata) {
            this.annotations = annotations;
            this.parameters = parameters;
            this.factory = factory;
            this.interfaces = interfaces;
            this.propMetadata = propMetadata;
        }
        return ReflectionInfo;
    }();
    exports.ReflectionInfo = ReflectionInfo;
    /**
     * Provides access to reflection data about symbols. Used internally by Angular
     * to power dependency injection and compilation.
     */
    var Reflector = function (_super) {
        __extends(Reflector, _super);
        function Reflector(reflectionCapabilities) {
            _super.call(this);
            /** @internal */
            this._injectableInfo = new collection_1.Map();
            /** @internal */
            this._getters = new collection_1.Map();
            /** @internal */
            this._setters = new collection_1.Map();
            /** @internal */
            this._methods = new collection_1.Map();
            this._usedKeys = null;
            this.reflectionCapabilities = reflectionCapabilities;
        }
        Reflector.prototype.updateCapabilities = function (caps) {
            this.reflectionCapabilities = caps;
        };
        Reflector.prototype.isReflectionEnabled = function () {
            return this.reflectionCapabilities.isReflectionEnabled();
        };
        /**
         * Causes `this` reflector to track keys used to access
         * {@link ReflectionInfo} objects.
         */
        Reflector.prototype.trackUsage = function () {
            this._usedKeys = new collection_1.Set();
        };
        /**
         * Lists types for which reflection information was not requested since
         * {@link #trackUsage} was called. This list could later be audited as
         * potential dead code.
         */
        Reflector.prototype.listUnusedKeys = function () {
            var _this = this;
            if (this._usedKeys == null) {
                throw new exceptions_1.BaseException('Usage tracking is disabled');
            }
            var allTypes = collection_1.MapWrapper.keys(this._injectableInfo);
            return allTypes.filter(function (key) {
                return !collection_1.SetWrapper.has(_this._usedKeys, key);
            });
        };
        Reflector.prototype.registerFunction = function (func, funcInfo) {
            this._injectableInfo.set(func, funcInfo);
        };
        Reflector.prototype.registerType = function (type, typeInfo) {
            this._injectableInfo.set(type, typeInfo);
        };
        Reflector.prototype.registerGetters = function (getters) {
            _mergeMaps(this._getters, getters);
        };
        Reflector.prototype.registerSetters = function (setters) {
            _mergeMaps(this._setters, setters);
        };
        Reflector.prototype.registerMethods = function (methods) {
            _mergeMaps(this._methods, methods);
        };
        Reflector.prototype.factory = function (type) {
            if (this._containsReflectionInfo(type)) {
                var res = this._getReflectionInfo(type).factory;
                return lang_1.isPresent(res) ? res : null;
            } else {
                return this.reflectionCapabilities.factory(type);
            }
        };
        Reflector.prototype.parameters = function (typeOrFunc) {
            if (this._injectableInfo.has(typeOrFunc)) {
                var res = this._getReflectionInfo(typeOrFunc).parameters;
                return lang_1.isPresent(res) ? res : [];
            } else {
                return this.reflectionCapabilities.parameters(typeOrFunc);
            }
        };
        Reflector.prototype.annotations = function (typeOrFunc) {
            if (this._injectableInfo.has(typeOrFunc)) {
                var res = this._getReflectionInfo(typeOrFunc).annotations;
                return lang_1.isPresent(res) ? res : [];
            } else {
                return this.reflectionCapabilities.annotations(typeOrFunc);
            }
        };
        Reflector.prototype.propMetadata = function (typeOrFunc) {
            if (this._injectableInfo.has(typeOrFunc)) {
                var res = this._getReflectionInfo(typeOrFunc).propMetadata;
                return lang_1.isPresent(res) ? res : {};
            } else {
                return this.reflectionCapabilities.propMetadata(typeOrFunc);
            }
        };
        Reflector.prototype.interfaces = function (type) {
            if (this._injectableInfo.has(type)) {
                var res = this._getReflectionInfo(type).interfaces;
                return lang_1.isPresent(res) ? res : [];
            } else {
                return this.reflectionCapabilities.interfaces(type);
            }
        };
        Reflector.prototype.hasLifecycleHook = function (type, lcInterface, lcProperty) {
            var interfaces = this.interfaces(type);
            if (interfaces.indexOf(lcInterface) !== -1) {
                return true;
            } else {
                return this.reflectionCapabilities.hasLifecycleHook(type, lcInterface, lcProperty);
            }
        };
        Reflector.prototype.getter = function (name) {
            if (this._getters.has(name)) {
                return this._getters.get(name);
            } else {
                return this.reflectionCapabilities.getter(name);
            }
        };
        Reflector.prototype.setter = function (name) {
            if (this._setters.has(name)) {
                return this._setters.get(name);
            } else {
                return this.reflectionCapabilities.setter(name);
            }
        };
        Reflector.prototype.method = function (name) {
            if (this._methods.has(name)) {
                return this._methods.get(name);
            } else {
                return this.reflectionCapabilities.method(name);
            }
        };
        /** @internal */
        Reflector.prototype._getReflectionInfo = function (typeOrFunc) {
            if (lang_1.isPresent(this._usedKeys)) {
                this._usedKeys.add(typeOrFunc);
            }
            return this._injectableInfo.get(typeOrFunc);
        };
        /** @internal */
        Reflector.prototype._containsReflectionInfo = function (typeOrFunc) {
            return this._injectableInfo.has(typeOrFunc);
        };
        Reflector.prototype.importUri = function (type) {
            return this.reflectionCapabilities.importUri(type);
        };
        return Reflector;
    }(reflector_reader_1.ReflectorReader);
    exports.Reflector = Reflector;
    function _mergeMaps(target, config) {
        collection_1.StringMapWrapper.forEach(config, function (v, k) {
            return target.set(k, v);
        });
    }
    
});
System.registerDynamic('node_modules/@angular/core/src/reflection/reflection.js', ['./reflection_capabilities', './reflector'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var reflection_capabilities_1 = $__require('./reflection_capabilities');
  var reflector_1 = $__require('./reflector');
  var reflector_2 = $__require('./reflector');
  exports.ReflectionInfo = reflector_2.ReflectionInfo;
  exports.Reflector = reflector_2.Reflector;
  /**
   * The {@link Reflector} used internally in Angular to access metadata
   * about symbols.
   */
  exports.reflector = new reflector_1.Reflector(new reflection_capabilities_1.ReflectionCapabilities());
  
});
System.registerDynamic("node_modules/@angular/core/src/reflection/reflection_capabilities.js", ["../facade/lang"], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('../facade/lang');
    var ReflectionCapabilities = function () {
        function ReflectionCapabilities(reflect) {
            this._reflect = lang_1.isPresent(reflect) ? reflect : lang_1.global.Reflect;
        }
        ReflectionCapabilities.prototype.isReflectionEnabled = function () {
            return true;
        };
        ReflectionCapabilities.prototype.factory = function (t) {
            switch (t.length) {
                case 0:
                    return function () {
                        return new t();
                    };
                case 1:
                    return function (a1) {
                        return new t(a1);
                    };
                case 2:
                    return function (a1, a2) {
                        return new t(a1, a2);
                    };
                case 3:
                    return function (a1, a2, a3) {
                        return new t(a1, a2, a3);
                    };
                case 4:
                    return function (a1, a2, a3, a4) {
                        return new t(a1, a2, a3, a4);
                    };
                case 5:
                    return function (a1, a2, a3, a4, a5) {
                        return new t(a1, a2, a3, a4, a5);
                    };
                case 6:
                    return function (a1, a2, a3, a4, a5, a6) {
                        return new t(a1, a2, a3, a4, a5, a6);
                    };
                case 7:
                    return function (a1, a2, a3, a4, a5, a6, a7) {
                        return new t(a1, a2, a3, a4, a5, a6, a7);
                    };
                case 8:
                    return function (a1, a2, a3, a4, a5, a6, a7, a8) {
                        return new t(a1, a2, a3, a4, a5, a6, a7, a8);
                    };
                case 9:
                    return function (a1, a2, a3, a4, a5, a6, a7, a8, a9) {
                        return new t(a1, a2, a3, a4, a5, a6, a7, a8, a9);
                    };
                case 10:
                    return function (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10) {
                        return new t(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10);
                    };
                case 11:
                    return function (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11) {
                        return new t(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11);
                    };
                case 12:
                    return function (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12) {
                        return new t(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12);
                    };
                case 13:
                    return function (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13) {
                        return new t(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13);
                    };
                case 14:
                    return function (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14) {
                        return new t(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14);
                    };
                case 15:
                    return function (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15) {
                        return new t(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15);
                    };
                case 16:
                    return function (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15, a16) {
                        return new t(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15, a16);
                    };
                case 17:
                    return function (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15, a16, a17) {
                        return new t(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15, a16, a17);
                    };
                case 18:
                    return function (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15, a16, a17, a18) {
                        return new t(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15, a16, a17, a18);
                    };
                case 19:
                    return function (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15, a16, a17, a18, a19) {
                        return new t(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15, a16, a17, a18, a19);
                    };
                case 20:
                    return function (a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15, a16, a17, a18, a19, a20) {
                        return new t(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15, a16, a17, a18, a19, a20);
                    };
            }
            ;
            throw new Error("Cannot create a factory for '" + lang_1.stringify(t) + "' because its constructor has more than 20 arguments");
        };
        /** @internal */
        ReflectionCapabilities.prototype._zipTypesAndAnnotations = function (paramTypes /** TODO #9100 */, paramAnnotations /** TODO #9100 */) {
            var result;
            if (typeof paramTypes === 'undefined') {
                result = new Array(paramAnnotations.length);
            } else {
                result = new Array(paramTypes.length);
            }
            for (var i = 0; i < result.length; i++) {
                // TS outputs Object for parameters without types, while Traceur omits
                // the annotations. For now we preserve the Traceur behavior to aid
                // migration, but this can be revisited.
                if (typeof paramTypes === 'undefined') {
                    result[i] = [];
                } else if (paramTypes[i] != Object) {
                    result[i] = [paramTypes[i]];
                } else {
                    result[i] = [];
                }
                if (lang_1.isPresent(paramAnnotations) && lang_1.isPresent(paramAnnotations[i])) {
                    result[i] = result[i].concat(paramAnnotations[i]);
                }
            }
            return result;
        };
        ReflectionCapabilities.prototype.parameters = function (typeOrFunc) {
            // Prefer the direct API.
            if (lang_1.isPresent(typeOrFunc.parameters)) {
                return typeOrFunc.parameters;
            }
            // API of tsickle for lowering decorators to properties on the class.
            if (lang_1.isPresent(typeOrFunc.ctorParameters)) {
                var ctorParameters = typeOrFunc.ctorParameters;
                var paramTypes_1 = ctorParameters.map(function (ctorParam /** TODO #9100 */) {
                    return ctorParam && ctorParam.type;
                });
                var paramAnnotations_1 = ctorParameters.map(function (ctorParam /** TODO #9100 */) {
                    return ctorParam && convertTsickleDecoratorIntoMetadata(ctorParam.decorators);
                });
                return this._zipTypesAndAnnotations(paramTypes_1, paramAnnotations_1);
            }
            // API for metadata created by invoking the decorators.
            if (lang_1.isPresent(this._reflect) && lang_1.isPresent(this._reflect.getMetadata)) {
                var paramAnnotations = this._reflect.getMetadata('parameters', typeOrFunc);
                var paramTypes = this._reflect.getMetadata('design:paramtypes', typeOrFunc);
                if (lang_1.isPresent(paramTypes) || lang_1.isPresent(paramAnnotations)) {
                    return this._zipTypesAndAnnotations(paramTypes, paramAnnotations);
                }
            }
            // The array has to be filled with `undefined` because holes would be skipped by `some`
            var parameters = new Array(typeOrFunc.length);
            parameters.fill(undefined);
            return parameters;
        };
        ReflectionCapabilities.prototype.annotations = function (typeOrFunc) {
            // Prefer the direct API.
            if (lang_1.isPresent(typeOrFunc.annotations)) {
                var annotations = typeOrFunc.annotations;
                if (lang_1.isFunction(annotations) && annotations.annotations) {
                    annotations = annotations.annotations;
                }
                return annotations;
            }
            // API of tsickle for lowering decorators to properties on the class.
            if (lang_1.isPresent(typeOrFunc.decorators)) {
                return convertTsickleDecoratorIntoMetadata(typeOrFunc.decorators);
            }
            // API for metadata created by invoking the decorators.
            if (lang_1.isPresent(this._reflect) && lang_1.isPresent(this._reflect.getMetadata)) {
                var annotations = this._reflect.getMetadata('annotations', typeOrFunc);
                if (lang_1.isPresent(annotations)) return annotations;
            }
            return [];
        };
        ReflectionCapabilities.prototype.propMetadata = function (typeOrFunc) {
            // Prefer the direct API.
            if (lang_1.isPresent(typeOrFunc.propMetadata)) {
                var propMetadata = typeOrFunc.propMetadata;
                if (lang_1.isFunction(propMetadata) && propMetadata.propMetadata) {
                    propMetadata = propMetadata.propMetadata;
                }
                return propMetadata;
            }
            // API of tsickle for lowering decorators to properties on the class.
            if (lang_1.isPresent(typeOrFunc.propDecorators)) {
                var propDecorators_1 = typeOrFunc.propDecorators;
                var propMetadata_1 = {};
                Object.keys(propDecorators_1).forEach(function (prop) {
                    propMetadata_1[prop] = convertTsickleDecoratorIntoMetadata(propDecorators_1[prop]);
                });
                return propMetadata_1;
            }
            // API for metadata created by invoking the decorators.
            if (lang_1.isPresent(this._reflect) && lang_1.isPresent(this._reflect.getMetadata)) {
                var propMetadata = this._reflect.getMetadata('propMetadata', typeOrFunc);
                if (lang_1.isPresent(propMetadata)) return propMetadata;
            }
            return {};
        };
        // Note: JavaScript does not support to query for interfaces during runtime.
        // However, we can't throw here as the reflector will always call this method
        // when asked for a lifecycle interface as this is what we check in Dart.
        ReflectionCapabilities.prototype.interfaces = function (type) {
            return [];
        };
        ReflectionCapabilities.prototype.hasLifecycleHook = function (type, lcInterface, lcProperty) {
            if (!(type instanceof lang_1.Type)) return false;
            var proto = type.prototype;
            return !!proto[lcProperty];
        };
        ReflectionCapabilities.prototype.getter = function (name) {
            return new Function('o', 'return o.' + name + ';');
        };
        ReflectionCapabilities.prototype.setter = function (name) {
            return new Function('o', 'v', 'return o.' + name + ' = v;');
        };
        ReflectionCapabilities.prototype.method = function (name) {
            var functionBody = "if (!o." + name + ") throw new Error('\"" + name + "\" is undefined');\n        return o." + name + ".apply(o, args);";
            return new Function('o', 'args', functionBody);
        };
        // There is not a concept of import uri in Js, but this is useful in developing Dart applications.
        ReflectionCapabilities.prototype.importUri = function (type) {
            // StaticSymbol
            if (typeof type === 'object' && type['filePath']) {
                return type['filePath'];
            }
            // Runtime type
            return "./" + lang_1.stringify(type);
        };
        return ReflectionCapabilities;
    }();
    exports.ReflectionCapabilities = ReflectionCapabilities;
    function convertTsickleDecoratorIntoMetadata(decoratorInvocations) {
        if (!decoratorInvocations) {
            return [];
        }
        return decoratorInvocations.map(function (decoratorInvocation) {
            var decoratorType = decoratorInvocation.type;
            var annotationCls = decoratorType.annotationCls;
            var annotationArgs = decoratorInvocation.args ? decoratorInvocation.args : [];
            var annotation = Object.create(annotationCls.prototype);
            annotationCls.apply(annotation, annotationArgs);
            return annotation;
        });
    }
    
});
System.registerDynamic("node_modules/@angular/core/src/reflection/reflector_reader.js", [], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * Provides read-only access to reflection data about symbols. Used internally by Angular
   * to power dependency injection and compilation.
   */

  var global = this || self,
      GLOBAL = global;
  var ReflectorReader = function () {
    function ReflectorReader() {}
    return ReflectorReader;
  }();
  exports.ReflectorReader = ReflectorReader;
  
});
System.registerDynamic("node_modules/@angular/core/src/render/api.js", ["../facade/exceptions"], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var exceptions_1 = $__require('../facade/exceptions');
    /**
     * @experimental
     */
    var RenderComponentType = function () {
        function RenderComponentType(id, templateUrl, slotCount, encapsulation, styles) {
            this.id = id;
            this.templateUrl = templateUrl;
            this.slotCount = slotCount;
            this.encapsulation = encapsulation;
            this.styles = styles;
        }
        return RenderComponentType;
    }();
    exports.RenderComponentType = RenderComponentType;
    var RenderDebugInfo = function () {
        function RenderDebugInfo() {}
        Object.defineProperty(RenderDebugInfo.prototype, "injector", {
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(RenderDebugInfo.prototype, "component", {
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(RenderDebugInfo.prototype, "providerTokens", {
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(RenderDebugInfo.prototype, "references", {
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(RenderDebugInfo.prototype, "context", {
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(RenderDebugInfo.prototype, "source", {
            get: function () {
                return exceptions_1.unimplemented();
            },
            enumerable: true,
            configurable: true
        });
        return RenderDebugInfo;
    }();
    exports.RenderDebugInfo = RenderDebugInfo;
    /**
     * @experimental
     */
    var Renderer = function () {
        function Renderer() {}
        return Renderer;
    }();
    exports.Renderer = Renderer;
    /**
     * Injectable service that provides a low-level interface for modifying the UI.
     *
     * Use this service to bypass Angular's templating and make custom UI changes that can't be
     * expressed declaratively. For example if you need to set a property or an attribute whose name is
     * not statically known, use {@link #setElementProperty} or {@link #setElementAttribute}
     * respectively.
     *
     * If you are implementing a custom renderer, you must implement this interface.
     *
     * The default Renderer implementation is `DomRenderer`. Also available is `WebWorkerRenderer`.
     * @experimental
     */
    var RootRenderer = function () {
        function RootRenderer() {}
        return RootRenderer;
    }();
    exports.RootRenderer = RootRenderer;
    
});
System.registerDynamic("node_modules/@angular/core/src/security.js", [], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * A SecurityContext marks a location that has dangerous security implications, e.g. a DOM property
   * like `innerHTML` that could cause Cross Site Scripting (XSS) security bugs when improperly
   * handled.
   *
   * See DomSanitizationService for more details on security in Angular applications.
   *
   * @stable
   */

  var global = this || self,
      GLOBAL = global;
  (function (SecurityContext) {
    SecurityContext[SecurityContext["NONE"] = 0] = "NONE";
    SecurityContext[SecurityContext["HTML"] = 1] = "HTML";
    SecurityContext[SecurityContext["STYLE"] = 2] = "STYLE";
    SecurityContext[SecurityContext["SCRIPT"] = 3] = "SCRIPT";
    SecurityContext[SecurityContext["URL"] = 4] = "URL";
    SecurityContext[SecurityContext["RESOURCE_URL"] = 5] = "RESOURCE_URL";
  })(exports.SecurityContext || (exports.SecurityContext = {}));
  var SecurityContext = exports.SecurityContext;
  /**
   * SanitizationService is used by the views to sanitize potentially dangerous values. This is a
   * private API, use code should only refer to DomSanitizationService.
   *
   * @stable
   */
  var SanitizationService = function () {
    function SanitizationService() {}
    return SanitizationService;
  }();
  exports.SanitizationService = SanitizationService;
  
});
System.registerDynamic('node_modules/@angular/core/src/util/decorators.js', ['../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('../facade/lang');
    var _nextClassId = 0;
    function extractAnnotation(annotation) {
        if (lang_1.isFunction(annotation) && annotation.hasOwnProperty('annotation')) {
            // it is a decorator, extract annotation
            annotation = annotation.annotation;
        }
        return annotation;
    }
    function applyParams(fnOrArray, key) {
        if (fnOrArray === Object || fnOrArray === String || fnOrArray === Function || fnOrArray === Number || fnOrArray === Array) {
            throw new Error("Can not use native " + lang_1.stringify(fnOrArray) + " as constructor");
        }
        if (lang_1.isFunction(fnOrArray)) {
            return fnOrArray;
        } else if (fnOrArray instanceof Array) {
            var annotations = fnOrArray;
            var fn = fnOrArray[fnOrArray.length - 1];
            if (!lang_1.isFunction(fn)) {
                throw new Error("Last position of Class method array must be Function in key " + key + " was '" + lang_1.stringify(fn) + "'");
            }
            var annoLength = annotations.length - 1;
            if (annoLength != fn.length) {
                throw new Error("Number of annotations (" + annoLength + ") does not match number of arguments (" + fn.length + ") in the function: " + lang_1.stringify(fn));
            }
            var paramsAnnotations = [];
            for (var i = 0, ii = annotations.length - 1; i < ii; i++) {
                var paramAnnotations = [];
                paramsAnnotations.push(paramAnnotations);
                var annotation = annotations[i];
                if (annotation instanceof Array) {
                    for (var j = 0; j < annotation.length; j++) {
                        paramAnnotations.push(extractAnnotation(annotation[j]));
                    }
                } else if (lang_1.isFunction(annotation)) {
                    paramAnnotations.push(extractAnnotation(annotation));
                } else {
                    paramAnnotations.push(annotation);
                }
            }
            Reflect.defineMetadata('parameters', paramsAnnotations, fn);
            return fn;
        } else {
            throw new Error("Only Function or Array is supported in Class definition for key '" + key + "' is '" + lang_1.stringify(fnOrArray) + "'");
        }
    }
    /**
     * Provides a way for expressing ES6 classes with parameter annotations in ES5.
     *
     * ## Basic Example
     *
     * ```
     * var Greeter = ng.Class({
     *   constructor: function(name) {
     *     this.name = name;
     *   },
     *
     *   greet: function() {
     *     alert('Hello ' + this.name + '!');
     *   }
     * });
     * ```
     *
     * is equivalent to ES6:
     *
     * ```
     * class Greeter {
     *   constructor(name) {
     *     this.name = name;
     *   }
     *
     *   greet() {
     *     alert('Hello ' + this.name + '!');
     *   }
     * }
     * ```
     *
     * or equivalent to ES5:
     *
     * ```
     * var Greeter = function (name) {
     *   this.name = name;
     * }
     *
     * Greeter.prototype.greet = function () {
     *   alert('Hello ' + this.name + '!');
     * }
     * ```
     *
     * ### Example with parameter annotations
     *
     * ```
     * var MyService = ng.Class({
     *   constructor: [String, [new Query(), QueryList], function(name, queryList) {
     *     ...
     *   }]
     * });
     * ```
     *
     * is equivalent to ES6:
     *
     * ```
     * class MyService {
     *   constructor(name: string, @Query() queryList: QueryList) {
     *     ...
     *   }
     * }
     * ```
     *
     * ### Example with inheritance
     *
     * ```
     * var Shape = ng.Class({
     *   constructor: (color) {
     *     this.color = color;
     *   }
     * });
     *
     * var Square = ng.Class({
     *   extends: Shape,
     *   constructor: function(color, size) {
     *     Shape.call(this, color);
     *     this.size = size;
     *   }
     * });
     * ```
     * @stable
     */
    function Class(clsDef) {
        var constructor = applyParams(clsDef.hasOwnProperty('constructor') ? clsDef.constructor : undefined, 'constructor');
        var proto = constructor.prototype;
        if (clsDef.hasOwnProperty('extends')) {
            if (lang_1.isFunction(clsDef.extends)) {
                constructor.prototype = proto = Object.create(clsDef.extends.prototype);
            } else {
                throw new Error("Class definition 'extends' property must be a constructor function was: " + lang_1.stringify(clsDef.extends));
            }
        }
        for (var key in clsDef) {
            if (key != 'extends' && key != 'prototype' && clsDef.hasOwnProperty(key)) {
                proto[key] = applyParams(clsDef[key], key);
            }
        }
        if (this && this.annotations instanceof Array) {
            Reflect.defineMetadata('annotations', this.annotations, constructor);
        }
        if (!constructor['name']) {
            constructor['overriddenName'] = "class" + _nextClassId++;
        }
        return constructor;
    }
    exports.Class = Class;
    var Reflect = lang_1.global.Reflect;
    // Throw statement at top-level is disallowed by closure compiler in ES6 input.
    // Wrap in an IIFE as a work-around.
    (function checkReflect() {
        if (!(Reflect && Reflect.getMetadata)) {
            throw 'reflect-metadata shim is required when using class decorators';
        }
    })();
    function makeDecorator(annotationCls /* TODO #9100 */, chainFn) {
        if (chainFn === void 0) {
            chainFn = null;
        }
        function DecoratorFactory(objOrType /** TODO #9100 */) {
            var annotationInstance = new annotationCls(objOrType);
            if (this instanceof annotationCls) {
                return annotationInstance;
            } else {
                var chainAnnotation = lang_1.isFunction(this) && this.annotations instanceof Array ? this.annotations : [];
                chainAnnotation.push(annotationInstance);
                var TypeDecorator = function TypeDecorator(cls /** TODO #9100 */) {
                    var annotations = Reflect.getOwnMetadata('annotations', cls);
                    annotations = annotations || [];
                    annotations.push(annotationInstance);
                    Reflect.defineMetadata('annotations', annotations, cls);
                    return cls;
                };
                TypeDecorator.annotations = chainAnnotation;
                TypeDecorator.Class = Class;
                if (chainFn) chainFn(TypeDecorator);
                return TypeDecorator;
            }
        }
        DecoratorFactory.prototype = Object.create(annotationCls.prototype);
        DecoratorFactory.annotationCls = annotationCls;
        return DecoratorFactory;
    }
    exports.makeDecorator = makeDecorator;
    function makeParamDecorator(annotationCls /** TODO #9100 */) {
        function ParamDecoratorFactory() {
            var args = []; /** TODO #9100 */
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i - 0] = arguments[_i];
            }
            var annotationInstance = Object.create(annotationCls.prototype);
            annotationCls.apply(annotationInstance, args);
            if (this instanceof annotationCls) {
                return annotationInstance;
            } else {
                ParamDecorator.annotation = annotationInstance;
                return ParamDecorator;
            }
            function ParamDecorator(cls /** TODO #9100 */, unusedKey /** TODO #9100 */, index /** TODO #9100 */) {
                var parameters = Reflect.getMetadata('parameters', cls);
                parameters = parameters || [];
                // there might be gaps if some in between parameters do not have annotations.
                // we pad with nulls.
                while (parameters.length <= index) {
                    parameters.push(null);
                }
                parameters[index] = parameters[index] || [];
                var annotationsForParam = parameters[index];
                annotationsForParam.push(annotationInstance);
                Reflect.defineMetadata('parameters', parameters, cls);
                return cls;
            }
        }
        ParamDecoratorFactory.prototype = Object.create(annotationCls.prototype);
        ParamDecoratorFactory.annotationCls = annotationCls;
        return ParamDecoratorFactory;
    }
    exports.makeParamDecorator = makeParamDecorator;
    function makePropDecorator(annotationCls /** TODO #9100 */) {
        function PropDecoratorFactory() {
            var args = []; /** TODO #9100 */
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i - 0] = arguments[_i];
            }
            var decoratorInstance = Object.create(annotationCls.prototype);
            annotationCls.apply(decoratorInstance, args);
            if (this instanceof annotationCls) {
                return decoratorInstance;
            } else {
                return function PropDecorator(target, name) {
                    var meta = Reflect.getOwnMetadata('propMetadata', target.constructor);
                    meta = meta || {};
                    meta[name] = meta[name] || [];
                    meta[name].unshift(decoratorInstance);
                    Reflect.defineMetadata('propMetadata', meta, target.constructor);
                };
            }
        }
        PropDecoratorFactory.prototype = Object.create(annotationCls.prototype);
        PropDecoratorFactory.annotationCls = annotationCls;
        return PropDecoratorFactory;
    }
    exports.makePropDecorator = makePropDecorator;
    
});
System.registerDynamic('node_modules/@angular/core/private_export.js', ['./src/animation/animation_constants', './src/animation/animation_driver', './src/animation/animation_group_player', './src/animation/animation_keyframe', './src/animation/animation_player', './src/animation/animation_sequence_player', './src/animation/animation_style_util', './src/animation/animation_styles', './src/change_detection/change_detection_util', './src/change_detection/constants', './src/console', './src/debug/debug_renderer', './src/di/provider_util', './src/di/reflective_provider', './src/linker/component_factory_resolver', './src/linker/component_resolver', './src/linker/debug_context', './src/linker/element', './src/linker/template_ref', './src/linker/view', './src/linker/view_type', './src/linker/view_utils', './src/metadata/lifecycle_hooks', './src/metadata/view', './src/profile/wtf_init', './src/reflection/reflection', './src/reflection/reflection_capabilities', './src/reflection/reflector_reader', './src/render/api', './src/security', './src/util/decorators'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var animation_constants_1 = $__require('./src/animation/animation_constants');
    var animation_driver_1 = $__require('./src/animation/animation_driver');
    var animation_group_player_1 = $__require('./src/animation/animation_group_player');
    var animation_keyframe_1 = $__require('./src/animation/animation_keyframe');
    var animation_player_1 = $__require('./src/animation/animation_player');
    var animation_sequence_player_1 = $__require('./src/animation/animation_sequence_player');
    var animationUtils = $__require('./src/animation/animation_style_util');
    var animation_styles_1 = $__require('./src/animation/animation_styles');
    var change_detection_util = $__require('./src/change_detection/change_detection_util');
    var constants = $__require('./src/change_detection/constants');
    var console = $__require('./src/console');
    var debug = $__require('./src/debug/debug_renderer');
    var provider_util = $__require('./src/di/provider_util');
    var reflective_provider = $__require('./src/di/reflective_provider');
    var component_factory_resolver = $__require('./src/linker/component_factory_resolver');
    var component_resolver = $__require('./src/linker/component_resolver');
    var debug_context = $__require('./src/linker/debug_context');
    var element = $__require('./src/linker/element');
    var template_ref = $__require('./src/linker/template_ref');
    var view = $__require('./src/linker/view');
    var view_type = $__require('./src/linker/view_type');
    var view_utils = $__require('./src/linker/view_utils');
    var lifecycle_hooks = $__require('./src/metadata/lifecycle_hooks');
    var metadata_view = $__require('./src/metadata/view');
    var wtf_init = $__require('./src/profile/wtf_init');
    var reflection = $__require('./src/reflection/reflection');
    var reflection_capabilities = $__require('./src/reflection/reflection_capabilities');
    var reflector_reader = $__require('./src/reflection/reflector_reader');
    var api = $__require('./src/render/api');
    var security = $__require('./src/security');
    var decorators = $__require('./src/util/decorators');
    exports.__core_private__ = {
        isDefaultChangeDetectionStrategy: constants.isDefaultChangeDetectionStrategy,
        ChangeDetectorStatus: constants.ChangeDetectorStatus,
        CHANGE_DETECTION_STRATEGY_VALUES: constants.CHANGE_DETECTION_STRATEGY_VALUES,
        constructDependencies: reflective_provider.constructDependencies,
        LifecycleHooks: lifecycle_hooks.LifecycleHooks,
        LIFECYCLE_HOOKS_VALUES: lifecycle_hooks.LIFECYCLE_HOOKS_VALUES,
        ReflectorReader: reflector_reader.ReflectorReader,
        ReflectorComponentResolver: component_resolver.ReflectorComponentResolver,
        CodegenComponentFactoryResolver: component_factory_resolver.CodegenComponentFactoryResolver,
        AppElement: element.AppElement,
        AppView: view.AppView,
        DebugAppView: view.DebugAppView,
        ViewType: view_type.ViewType,
        MAX_INTERPOLATION_VALUES: view_utils.MAX_INTERPOLATION_VALUES,
        checkBinding: view_utils.checkBinding,
        flattenNestedViewRenderNodes: view_utils.flattenNestedViewRenderNodes,
        interpolate: view_utils.interpolate,
        ViewUtils: view_utils.ViewUtils,
        VIEW_ENCAPSULATION_VALUES: metadata_view.VIEW_ENCAPSULATION_VALUES,
        DebugContext: debug_context.DebugContext,
        StaticNodeDebugInfo: debug_context.StaticNodeDebugInfo,
        devModeEqual: change_detection_util.devModeEqual,
        uninitialized: change_detection_util.uninitialized,
        ValueUnwrapper: change_detection_util.ValueUnwrapper,
        RenderDebugInfo: api.RenderDebugInfo,
        SecurityContext: security.SecurityContext,
        SanitizationService: security.SanitizationService,
        TemplateRef_: template_ref.TemplateRef_,
        wtfInit: wtf_init.wtfInit,
        ReflectionCapabilities: reflection_capabilities.ReflectionCapabilities,
        makeDecorator: decorators.makeDecorator,
        DebugDomRootRenderer: debug.DebugDomRootRenderer,
        createProvider: provider_util.createProvider,
        isProviderLiteral: provider_util.isProviderLiteral,
        EMPTY_ARRAY: view_utils.EMPTY_ARRAY,
        EMPTY_MAP: view_utils.EMPTY_MAP,
        pureProxy1: view_utils.pureProxy1,
        pureProxy2: view_utils.pureProxy2,
        pureProxy3: view_utils.pureProxy3,
        pureProxy4: view_utils.pureProxy4,
        pureProxy5: view_utils.pureProxy5,
        pureProxy6: view_utils.pureProxy6,
        pureProxy7: view_utils.pureProxy7,
        pureProxy8: view_utils.pureProxy8,
        pureProxy9: view_utils.pureProxy9,
        pureProxy10: view_utils.pureProxy10,
        castByValue: view_utils.castByValue,
        Console: console.Console,
        reflector: reflection.reflector,
        Reflector: reflection.Reflector,
        NoOpAnimationPlayer: animation_player_1.NoOpAnimationPlayer,
        AnimationPlayer: animation_player_1.AnimationPlayer,
        NoOpAnimationDriver: animation_driver_1.NoOpAnimationDriver,
        AnimationDriver: animation_driver_1.AnimationDriver,
        AnimationSequencePlayer: animation_sequence_player_1.AnimationSequencePlayer,
        AnimationGroupPlayer: animation_group_player_1.AnimationGroupPlayer,
        AnimationKeyframe: animation_keyframe_1.AnimationKeyframe,
        prepareFinalAnimationStyles: animationUtils.prepareFinalAnimationStyles,
        balanceAnimationKeyframes: animationUtils.balanceAnimationKeyframes,
        flattenStyles: animationUtils.flattenStyles,
        clearStyles: animationUtils.clearStyles,
        renderStyles: animationUtils.renderStyles,
        collectAndResolveStyles: animationUtils.collectAndResolveStyles,
        AnimationStyles: animation_styles_1.AnimationStyles,
        ANY_STATE: animation_constants_1.ANY_STATE,
        DEFAULT_STATE: animation_constants_1.DEFAULT_STATE,
        EMPTY_STATE: animation_constants_1.EMPTY_STATE,
        FILL_STYLE_FLAG: animation_constants_1.FILL_STYLE_FLAG
    };
    
});
System.registerDynamic('node_modules/@angular/core/src/animation/metadata.js', ['../facade/exceptions', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    /**
     * @experimental Animation support is experimental.
     */
    exports.AUTO_STYLE = '*';
    /**
     * Metadata representing the entry of animations.
     * Instances of this class are provided via the animation DSL when the {@link trigger trigger
     * animation function} is called.
     *
     * @experimental Animation support is experimental.
     */
    var AnimationEntryMetadata = function () {
        function AnimationEntryMetadata(name, definitions) {
            this.name = name;
            this.definitions = definitions;
        }
        return AnimationEntryMetadata;
    }();
    exports.AnimationEntryMetadata = AnimationEntryMetadata;
    /**
     * @experimental Animation support is experimental.
     */
    var AnimationStateMetadata = function () {
        function AnimationStateMetadata() {}
        return AnimationStateMetadata;
    }();
    exports.AnimationStateMetadata = AnimationStateMetadata;
    /**
     * Metadata representing the entry of animations.
     * Instances of this class are provided via the animation DSL when the {@link state state animation
     * function} is called.
     *
     * @experimental Animation support is experimental.
     */
    var AnimationStateDeclarationMetadata = function (_super) {
        __extends(AnimationStateDeclarationMetadata, _super);
        function AnimationStateDeclarationMetadata(stateNameExpr, styles) {
            _super.call(this);
            this.stateNameExpr = stateNameExpr;
            this.styles = styles;
        }
        return AnimationStateDeclarationMetadata;
    }(AnimationStateMetadata);
    exports.AnimationStateDeclarationMetadata = AnimationStateDeclarationMetadata;
    /**
     * Metadata representing the entry of animations.
     * Instances of this class are provided via the animation DSL when the
     * {@link transition transition animation function} is called.
     *
     * @experimental Animation support is experimental.
     */
    var AnimationStateTransitionMetadata = function (_super) {
        __extends(AnimationStateTransitionMetadata, _super);
        function AnimationStateTransitionMetadata(stateChangeExpr, steps) {
            _super.call(this);
            this.stateChangeExpr = stateChangeExpr;
            this.steps = steps;
        }
        return AnimationStateTransitionMetadata;
    }(AnimationStateMetadata);
    exports.AnimationStateTransitionMetadata = AnimationStateTransitionMetadata;
    /**
     * @experimental Animation support is experimental.
     */
    var AnimationMetadata = function () {
        function AnimationMetadata() {}
        return AnimationMetadata;
    }();
    exports.AnimationMetadata = AnimationMetadata;
    /**
     * Metadata representing the entry of animations.
     * Instances of this class are provided via the animation DSL when the {@link keyframes keyframes
     * animation function} is called.
     *
     * @experimental Animation support is experimental.
     */
    var AnimationKeyframesSequenceMetadata = function (_super) {
        __extends(AnimationKeyframesSequenceMetadata, _super);
        function AnimationKeyframesSequenceMetadata(steps) {
            _super.call(this);
            this.steps = steps;
        }
        return AnimationKeyframesSequenceMetadata;
    }(AnimationMetadata);
    exports.AnimationKeyframesSequenceMetadata = AnimationKeyframesSequenceMetadata;
    /**
     * Metadata representing the entry of animations.
     * Instances of this class are provided via the animation DSL when the {@link style style animation
     * function} is called.
     *
     * @experimental Animation support is experimental.
     */
    var AnimationStyleMetadata = function (_super) {
        __extends(AnimationStyleMetadata, _super);
        function AnimationStyleMetadata(styles, offset) {
            if (offset === void 0) {
                offset = null;
            }
            _super.call(this);
            this.styles = styles;
            this.offset = offset;
        }
        return AnimationStyleMetadata;
    }(AnimationMetadata);
    exports.AnimationStyleMetadata = AnimationStyleMetadata;
    /**
     * Metadata representing the entry of animations.
     * Instances of this class are provided via the animation DSL when the {@link animate animate
     * animation function} is called.
     *
     * @experimental Animation support is experimental.
     */
    var AnimationAnimateMetadata = function (_super) {
        __extends(AnimationAnimateMetadata, _super);
        function AnimationAnimateMetadata(timings, styles) {
            _super.call(this);
            this.timings = timings;
            this.styles = styles;
        }
        return AnimationAnimateMetadata;
    }(AnimationMetadata);
    exports.AnimationAnimateMetadata = AnimationAnimateMetadata;
    /**
     * @experimental Animation support is experimental.
     */
    var AnimationWithStepsMetadata = function (_super) {
        __extends(AnimationWithStepsMetadata, _super);
        function AnimationWithStepsMetadata() {
            _super.call(this);
        }
        Object.defineProperty(AnimationWithStepsMetadata.prototype, "steps", {
            get: function () {
                throw new exceptions_1.BaseException('NOT IMPLEMENTED: Base Class');
            },
            enumerable: true,
            configurable: true
        });
        return AnimationWithStepsMetadata;
    }(AnimationMetadata);
    exports.AnimationWithStepsMetadata = AnimationWithStepsMetadata;
    /**
     * Metadata representing the entry of animations.
     * Instances of this class are provided via the animation DSL when the {@link sequence sequence
     * animation function} is called.
     *
     * @experimental Animation support is experimental.
     */
    var AnimationSequenceMetadata = function (_super) {
        __extends(AnimationSequenceMetadata, _super);
        function AnimationSequenceMetadata(_steps) {
            _super.call(this);
            this._steps = _steps;
        }
        Object.defineProperty(AnimationSequenceMetadata.prototype, "steps", {
            get: function () {
                return this._steps;
            },
            enumerable: true,
            configurable: true
        });
        return AnimationSequenceMetadata;
    }(AnimationWithStepsMetadata);
    exports.AnimationSequenceMetadata = AnimationSequenceMetadata;
    /**
     * Metadata representing the entry of animations.
     * Instances of this class are provided via the animation DSL when the {@link group group animation
     * function} is called.
     *
     * @experimental Animation support is experimental.
     */
    var AnimationGroupMetadata = function (_super) {
        __extends(AnimationGroupMetadata, _super);
        function AnimationGroupMetadata(_steps) {
            _super.call(this);
            this._steps = _steps;
        }
        Object.defineProperty(AnimationGroupMetadata.prototype, "steps", {
            get: function () {
                return this._steps;
            },
            enumerable: true,
            configurable: true
        });
        return AnimationGroupMetadata;
    }(AnimationWithStepsMetadata);
    exports.AnimationGroupMetadata = AnimationGroupMetadata;
    /**
     * `animate` is an animation-specific function that is designed to be used inside of Angular2's
     * animation
     * DSL language. If this information is new, please navigate to the
     * {@link ComponentMetadata#animations-anchor component animations metadata
     * page} to gain a better understanding of how animations in Angular2 are used.
     *
     * `animate` specifies an animation step that will apply the provided `styles` data for a given
     * amount of
     * time based on the provided `timing` expression value. Calls to `animate` are expected to be
     * used within {@link sequence an animation sequence}, {@link group group}, or {@link transition
     * transition}.
     *
     * ### Usage
     *
     * The `animate` function accepts two input parameters: `timing` and `styles`:
     *
     * - `timing` is a string based value that can be a combination of a duration with optional
     * delay and easing values. The format for the expression breaks down to `duration delay easing`
     * (therefore a value such as `1s 100ms ease-out` will be parse itself into `duration=1000,
     * delay=100, easing=ease-out`.
     * If a numeric value is provided then that will be used as the `duration` value in millisecond
     * form.
     * - `styles` is the style input data which can either be a call to {@link style style} or {@link
     * keyframes keyframes}.
     * If left empty then the styles from the destination state will be collected and used (this is
     * useful when
     * describing an animation step that will complete an animation by {@link
     * transition#the-final-animate-call animating to the final state}).
     *
     * ```typescript
     * // various functions for specifying timing data
     * animate(500, style(...))
     * animate("1s", style(...))
     * animate("100ms 0.5s", style(...))
     * animate("5s ease", style(...))
     * animate("5s 10ms cubic-bezier(.17,.67,.88,.1)", style(...))
     *
     * // either style() of keyframes() can be used
     * animate(500, style({ background: "red" }))
     * animate(500, keyframes([
     *   style({ background: "blue" })),
     *   style({ background: "red" }))
     * ])
     * ```
     *
     * ### Example ([live demo](http://plnkr.co/edit/Kez8XGWBxWue7qP7nNvF?p=preview))
     *
     * {@example core/animation/ts/dsl/animation_example.ts region='Component'}
     *
     * @experimental Animation support is experimental.
     */
    function animate(timing, styles) {
        if (styles === void 0) {
            styles = null;
        }
        var stylesEntry = styles;
        if (!lang_1.isPresent(stylesEntry)) {
            var EMPTY_STYLE = {};
            stylesEntry = new AnimationStyleMetadata([EMPTY_STYLE], 1);
        }
        return new AnimationAnimateMetadata(timing, stylesEntry);
    }
    exports.animate = animate;
    /**
     * `group` is an animation-specific function that is designed to be used inside of Angular2's
     * animation
     * DSL language. If this information is new, please navigate to the
     * {@link ComponentMetadata#animations-anchor component animations metadata
     * page} to gain a better understanding of how animations in Angular2 are used.
     *
     * `group` specifies a list of animation steps that are all run in parallel. Grouped animations
     * are useful when a series of styles must be animated/closed off
     * at different statrting/ending times.
     *
     * The `group` function can either be used within a {@link sequence sequence} or a {@link transition
     * transition}
     * and it will only continue to the next instruction once all of the inner animation steps
     * have completed.
     *
     * ### Usage
     *
     * The `steps` data that is passed into the `group` animation function can either consist
     * of {@link style style} or {@link animate animate} function calls. Each call to `style()` or
     * `animate()`
     * within a group will be executed instantly (use {@link keyframes keyframes} or a
     * {@link animate#usage animate() with a delay value} to offset styles to be applied at a later
     * time).
     *
     * ```typescript
     * group([
     *   animate("1s", { background: "black" }))
     *   animate("2s", { color: "white" }))
     * ])
     * ```
     *
     * ### Example ([live demo](http://plnkr.co/edit/Kez8XGWBxWue7qP7nNvF?p=preview))
     *
     * {@example core/animation/ts/dsl/animation_example.ts region='Component'}
     *
     * @experimental Animation support is experimental.
     */
    function group(steps) {
        return new AnimationGroupMetadata(steps);
    }
    exports.group = group;
    /**
     * `sequence` is an animation-specific function that is designed to be used inside of Angular2's
     * animation
     * DSL language. If this information is new, please navigate to the
     * {@link ComponentMetadata#animations-anchor component animations metadata
     * page} to gain a better understanding of how animations in Angular2 are used.
     *
     * `sequence` Specifies a list of animation steps that are run one by one. (`sequence` is used
     * by default when an array is passed as animation data into {@link transition transition}.)
     *
     * The `sequence` function can either be used within a {@link group group} or a {@link transition
     * transition}
     * and it will only continue to the next instruction once each of the inner animation steps
     * have completed.
     *
     * To perform animation styling in parallel with other animation steps then
     * have a look at the {@link group group} animation function.
     *
     * ### Usage
     *
     * The `steps` data that is passed into the `sequence` animation function can either consist
     * of {@link style style} or {@link animate animate} function calls. A call to `style()` will apply
     * the
     * provided styling data immediately while a call to `animate()` will apply its styling
     * data over a given time depending on its timing data.
     *
     * ```typescript
     * sequence([
     *   style({ opacity: 0 })),
     *   animate("1s", { opacity: 1 }))
     * ])
     * ```
     *
     * ### Example ([live demo](http://plnkr.co/edit/Kez8XGWBxWue7qP7nNvF?p=preview))
     *
     * {@example core/animation/ts/dsl/animation_example.ts region='Component'}
     *
     * @experimental Animation support is experimental.
     */
    function sequence(steps) {
        return new AnimationSequenceMetadata(steps);
    }
    exports.sequence = sequence;
    /**
     * `style` is an animation-specific function that is designed to be used inside of Angular2's
     * animation
     * DSL language. If this information is new, please navigate to the
     * {@link ComponentMetadata#animations-anchor component animations metadata
     * page} to gain a better understanding of how animations in Angular2 are used.
     *
     * `style` declares a key/value object containing CSS properties/styles that can then
     * be used for {@link state animation states}, within an {@link sequence animation sequence}, or as
     * styling data for both {@link animate animate} and {@link keyframes keyframes}.
     *
     * ### Usage
     *
     * `style` takes in a key/value string map as data and expects one or more CSS property/value
     * pairs to be defined.
     *
     * ```typescript
     * // string values are used for css properties
     * style({ background: "red", color: "blue" })
     *
     * // numerical (pixel) values are also supported
     * style({ width: 100, height: 0 })
     * ```
     *
     * #### Auto-styles (using `*`)
     *
     * When an asterix (`*`) character is used as a value then it will be detected from the element
     * being animated
     * and applied as animation data when the animation starts.
     *
     * This feature proves useful for a state depending on layout and/or environment factors; in such
     * cases
     * the styles are calculated just before the animation starts.
     *
     * ```typescript
     * // the steps below will animate from 0 to the
     * // actual height of the element
     * style({ height: 0 }),
     * animate("1s", style({ height: "*" }))
     * ```
     *
     * ### Example ([live demo](http://plnkr.co/edit/Kez8XGWBxWue7qP7nNvF?p=preview))
     *
     * {@example core/animation/ts/dsl/animation_example.ts region='Component'}
     *
     * @experimental Animation support is experimental.
     */
    function style(tokens) {
        var input;
        var offset = null;
        if (lang_1.isString(tokens)) {
            input = [tokens];
        } else {
            if (lang_1.isArray(tokens)) {
                input = tokens;
            } else {
                input = [tokens];
            }
            input.forEach(function (entry) {
                var entryOffset = entry['offset'];
                if (lang_1.isPresent(entryOffset)) {
                    offset = offset == null ? lang_1.NumberWrapper.parseFloat(entryOffset) : offset;
                }
            });
        }
        return new AnimationStyleMetadata(input, offset);
    }
    exports.style = style;
    /**
     * `state` is an animation-specific function that is designed to be used inside of Angular2's
     * animation
     * DSL language. If this information is new, please navigate to the
     * {@link ComponentMetadata#animations-anchor component animations metadata
     * page} to gain a better understanding of how animations in Angular2 are used.
     *
     * `state` declares an animation state within the given trigger. When a state is
     * active within a component then its associated styles will persist on
     * the element that the trigger is attached to (even when the animation ends).
     *
     * To animate between states, have a look at the animation {@link transition transition}
     * DSL function. To register states to an animation trigger please have a look
     * at the {@link trigger trigger} function.
     *
     * #### The `void` state
     *
     * The `void` state value is a reserved word that angular uses to determine when the element is not
     * apart
     * of the application anymore (e.g. when an `ngIf` evaluates to false then the state of the
     * associated element
     * is void).
     *
     * #### The `*` (default) state
     *
     * The `*` state (when styled) is a fallback state that will be used if
     * the state that is being animated is not declared within the trigger.
     *
     * ### Usage
     *
     * `state` will declare an animation state with its associated styles
     * within the given trigger.
     *
     * - `stateNameExpr` can be one or more state names separated by commas.
     * - `styles` refers to the {@link style styling data} that will be persisted on the element once
     * the state
     * has been reached.
     *
     * ```typescript
     * // "void" is a reserved name for a state and is used to represent
     * // the state in which an element is detached from from the application.
     * state("void", style({ height: 0 }))
     *
     * // user-defined states
     * state("closed", style({ height: 0 }))
     * state("open, visible", style({ height: "*" }))
     * ```
     *
     * ### Example ([live demo](http://plnkr.co/edit/Kez8XGWBxWue7qP7nNvF?p=preview))
     *
     * {@example core/animation/ts/dsl/animation_example.ts region='Component'}
     *
     * @experimental Animation support is experimental.
     */
    function state(stateNameExpr, styles) {
        return new AnimationStateDeclarationMetadata(stateNameExpr, styles);
    }
    exports.state = state;
    /**
     * `keyframes` is an animation-specific function that is designed to be used inside of Angular2's
     * animation
     * DSL language. If this information is new, please navigate to the
     * {@link ComponentMetadata#animations-anchor component animations metadata
     * page} to gain a better understanding of how animations in Angular2 are used.
     *
     * `keyframes` specifies a collection of {@link style style} entries each optionally characterized
     * by an `offset` value.
     *
     * ### Usage
     *
     * The `keyframes` animation function is designed to be used alongside the {@link animate animate}
     * animation function. Instead of applying animations from where they are
     * currently to their destination, keyframes can describe how each style entry is applied
     * and at what point within the animation arc (much like CSS Keyframe Animations do).
     *
     * For each `style()` entry an `offset` value can be set. Doing so allows to specifiy at
     * what percentage of the animate time the styles will be applied.
     *
     * ```typescript
     * // the provided offset values describe when each backgroundColor value is applied.
     * animate("5s", keyframes([
     *   style({ backgroundColor: "red", offset: 0 }),
     *   style({ backgroundColor: "blue", offset: 0.2 }),
     *   style({ backgroundColor: "orange", offset: 0.3 }),
     *   style({ backgroundColor: "black", offset: 1 })
     * ]))
     * ```
     *
     * Alternatively, if there are no `offset` values used within the style entries then the offsets
     * will
     * be calculated automatically.
     *
     * ```typescript
     * animate("5s", keyframes([
     *   style({ backgroundColor: "red" }) // offset = 0
     *   style({ backgroundColor: "blue" }) // offset = 0.33
     *   style({ backgroundColor: "orange" }) // offset = 0.66
     *   style({ backgroundColor: "black" }) // offset = 1
     * ]))
     * ```
     *
     * ### Example ([live demo](http://plnkr.co/edit/Kez8XGWBxWue7qP7nNvF?p=preview))
     *
     * {@example core/animation/ts/dsl/animation_example.ts region='Component'}
     *
     * @experimental Animation support is experimental.
     */
    function keyframes(steps) {
        return new AnimationKeyframesSequenceMetadata(steps);
    }
    exports.keyframes = keyframes;
    /**
     * `transition` is an animation-specific function that is designed to be used inside of Angular2's
     * animation
     * DSL language. If this information is new, please navigate to the
     * {@link ComponentMetadata#animations-anchor component animations metadata
     * page} to gain a better understanding of how animations in Angular2 are used.
     *
     * `transition` declares the {@link sequence sequence of animation steps} that will be run when the
     * provided
     * `stateChangeExpr` value is satisfied. The `stateChangeExpr` consists of a `state1 => state2`
     * which consists
     * of two known states (use an asterix (`*`) to refer to a dynamic starting and/or ending state).
     *
     * Animation transitions are placed within an {@link trigger animation trigger}. For an transition
     * to animate to
     * a state value and persist its styles then one or more {@link state animation states} is expected
     * to be defined.
     *
     * ### Usage
     *
     * An animation transition is kicked off the `stateChangeExpr` predicate evaluates to true based on
     * what the
     * previous state is and what the current state has become. In other words, if a transition is
     * defined that
     * matches the old/current state criteria then the associated animation will be triggered.
     *
     * ```typescript
     * // all transition/state changes are defined within an animation trigger
     * trigger("myAnimationTrigger", [
     *   // if a state is defined then its styles will be persisted when the
     *   // animation has fully completed itself
     *   state("on", style({ background: "green" })),
     *   state("off", style({ background: "grey" })),
     *
     *   // a transition animation that will be kicked off when the state value
     *   // bound to "myAnimationTrigger" changes from "on" to "off"
     *   transition("on => off", animate(500)),
     *
     *   // it is also possible to do run the same animation for both directions
     *   transition("on <=> off", animate(500)),
     *
     *   // or to define multiple states pairs separated by commas
     *   transition("on => off, off => void", animate(500)),
     *
     *   // this is a catch-all state change for when an element is inserted into
     *   // the page and the destination state is unknown
     *   transition("void => *", [
     *     style({ opacity: 0 }),
     *     animate(500)
     *   ]),
     *
     *   // this will capture a state change between any states
     *   transition("* => *", animate("1s 0s")),
     * ])
     * ```
     *
     * The template associated with this component will make use of the `myAnimationTrigger`
     * animation trigger by binding to an element within its template code.
     *
     * ```html
     * <!-- somewhere inside of my-component-tpl.html -->
     * <div @myAnimationTrigger="myStatusExp">...</div>
     * ```
     *
     * #### The final `animate` call
     *
     * If the final step within the transition steps is a call to `animate()` that **only**
     * uses a timing value with **no style data** then it will be automatically used as the final
     * animation
     * arc for the element to animate itself to the final state. This involves an automatic mix of
     * adding/removing CSS styles so that the element will be in the exact state it should be for the
     * applied state to be presented correctly.
     *
     * ```
     * // start off by hiding the element, but make sure that it animates properly to whatever state
     * // is currently active for "myAnimationTrigger"
     * transition("void => *", [
     *   style({ opacity: 0 }),
     *   animate(500)
     * ])
     * ```
     *
     * ### Example ([live demo](http://plnkr.co/edit/Kez8XGWBxWue7qP7nNvF?p=preview))
     *
     * {@example core/animation/ts/dsl/animation_example.ts region='Component'}
     *
     * @experimental Animation support is experimental.
     */
    function transition(stateChangeExpr, steps) {
        var animationData = lang_1.isArray(steps) ? new AnimationSequenceMetadata(steps) : steps;
        return new AnimationStateTransitionMetadata(stateChangeExpr, animationData);
    }
    exports.transition = transition;
    /**
     * `trigger` is an animation-specific function that is designed to be used inside of Angular2's
     * animation
     * DSL language. If this information is new, please navigate to the
     * {@link ComponentMetadata#animations-anchor component animations metadata
     * page} to gain a better understanding of how animations in Angular2 are used.
     *
     * `trigger` Creates an animation trigger which will a list of {@link state state} and {@link
     * transition transition}
     * entries that will be evaluated when the expression bound to the trigger changes.
     *
     * Triggers are registered within the component annotation data under the
     * {@link ComponentMetadata#animations-anchor animations section}. An animation trigger can
     * be placed on an element within a template by referencing the name of the
     * trigger followed by the expression value that the trigger is bound to
     * (in the form of `@triggerName="expression"`.
     *
     * ### Usage
     *
     * `trigger` will create an animation trigger reference based on the provided `name` value.
     * The provided `animation` value is expected to be an array consisting of {@link state state} and
     * {@link transition transition}
     * declarations.
     *
     * ```typescript
     * @Component({
     *   selector: 'my-component',
     *   templateUrl: 'my-component-tpl.html',
     *   animations: [
     *     trigger("myAnimationTrigger", [
     *       state(...),
     *       state(...),
     *       transition(...),
     *       transition(...)
     *     ])
     *   ]
     * })
     * class MyComponent {
     *   myStatusExp = "something";
     * }
     * ```
     *
     * The template associated with this component will make use of the `myAnimationTrigger`
     * animation trigger by binding to an element within its template code.
     *
     * ```html
     * <!-- somewhere inside of my-component-tpl.html -->
     * <div @myAnimationTrigger="myStatusExp">...</div>
     * ```
     *
     * ### Example ([live demo](http://plnkr.co/edit/Kez8XGWBxWue7qP7nNvF?p=preview))
     *
     * {@example core/animation/ts/dsl/animation_example.ts region='Component'}
     *
     * @experimental Animation support is experimental.
     */
    function trigger(name, animation) {
        return new AnimationEntryMetadata(name, animation);
    }
    exports.trigger = trigger;
    
});
System.registerDynamic("node_modules/@angular/core/src/facade/base_wrapped_exception.js", [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    /**
     * A base class for the WrappedException that can be used to identify
     * a WrappedException from ExceptionHandler without adding circular
     * dependency.
     */
    var BaseWrappedException = function (_super) {
        __extends(BaseWrappedException, _super);
        function BaseWrappedException(message) {
            _super.call(this, message);
        }
        Object.defineProperty(BaseWrappedException.prototype, "wrapperMessage", {
            get: function () {
                return '';
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "wrapperStack", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "originalException", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "originalStack", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "context", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "message", {
            get: function () {
                return '';
            },
            enumerable: true,
            configurable: true
        });
        return BaseWrappedException;
    }(Error);
    exports.BaseWrappedException = BaseWrappedException;
    
});
System.registerDynamic("node_modules/@angular/core/src/facade/collection.js", ["./lang"], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('./lang');
    exports.Map = lang_1.global.Map;
    exports.Set = lang_1.global.Set;
    // Safari and Internet Explorer do not support the iterable parameter to the
    // Map constructor.  We work around that by manually adding the items.
    var createMapFromPairs = function () {
        try {
            if (new exports.Map([[1, 2]]).size === 1) {
                return function createMapFromPairs(pairs) {
                    return new exports.Map(pairs);
                };
            }
        } catch (e) {}
        return function createMapAndPopulateFromPairs(pairs) {
            var map = new exports.Map();
            for (var i = 0; i < pairs.length; i++) {
                var pair = pairs[i];
                map.set(pair[0], pair[1]);
            }
            return map;
        };
    }();
    var createMapFromMap = function () {
        try {
            if (new exports.Map(new exports.Map())) {
                return function createMapFromMap(m) {
                    return new exports.Map(m);
                };
            }
        } catch (e) {}
        return function createMapAndPopulateFromMap(m) {
            var map = new exports.Map();
            m.forEach(function (v, k) {
                map.set(k, v);
            });
            return map;
        };
    }();
    var _clearValues = function () {
        if (new exports.Map().keys().next) {
            return function _clearValues(m) {
                var keyIterator = m.keys();
                var k;
                while (!(k = keyIterator.next()).done) {
                    m.set(k.value, null);
                }
            };
        } else {
            return function _clearValuesWithForeEach(m) {
                m.forEach(function (v, k) {
                    m.set(k, null);
                });
            };
        }
    }();
    // Safari doesn't implement MapIterator.next(), which is used is Traceur's polyfill of Array.from
    // TODO(mlaval): remove the work around once we have a working polyfill of Array.from
    var _arrayFromMap = function () {
        try {
            if (new exports.Map().values().next) {
                return function createArrayFromMap(m, getValues) {
                    return getValues ? Array.from(m.values()) : Array.from(m.keys());
                };
            }
        } catch (e) {}
        return function createArrayFromMapWithForeach(m, getValues) {
            var res = ListWrapper.createFixedSize(m.size),
                i = 0;
            m.forEach(function (v, k) {
                res[i] = getValues ? v : k;
                i++;
            });
            return res;
        };
    }();
    var MapWrapper = function () {
        function MapWrapper() {}
        MapWrapper.clone = function (m) {
            return createMapFromMap(m);
        };
        MapWrapper.createFromStringMap = function (stringMap) {
            var result = new exports.Map();
            for (var prop in stringMap) {
                result.set(prop, stringMap[prop]);
            }
            return result;
        };
        MapWrapper.toStringMap = function (m) {
            var r = {};
            m.forEach(function (v, k) {
                return r[k] = v;
            });
            return r;
        };
        MapWrapper.createFromPairs = function (pairs) {
            return createMapFromPairs(pairs);
        };
        MapWrapper.clearValues = function (m) {
            _clearValues(m);
        };
        MapWrapper.iterable = function (m) {
            return m;
        };
        MapWrapper.keys = function (m) {
            return _arrayFromMap(m, false);
        };
        MapWrapper.values = function (m) {
            return _arrayFromMap(m, true);
        };
        return MapWrapper;
    }();
    exports.MapWrapper = MapWrapper;
    /**
     * Wraps Javascript Objects
     */
    var StringMapWrapper = function () {
        function StringMapWrapper() {}
        StringMapWrapper.create = function () {
            // Note: We are not using Object.create(null) here due to
            // performance!
            // http://jsperf.com/ng2-object-create-null
            return {};
        };
        StringMapWrapper.contains = function (map, key) {
            return map.hasOwnProperty(key);
        };
        StringMapWrapper.get = function (map, key) {
            return map.hasOwnProperty(key) ? map[key] : undefined;
        };
        StringMapWrapper.set = function (map, key, value) {
            map[key] = value;
        };
        StringMapWrapper.keys = function (map) {
            return Object.keys(map);
        };
        StringMapWrapper.values = function (map) {
            return Object.keys(map).reduce(function (r, a) {
                r.push(map[a]);
                return r;
            }, []);
        };
        StringMapWrapper.isEmpty = function (map) {
            for (var prop in map) {
                return false;
            }
            return true;
        };
        StringMapWrapper.delete = function (map, key) {
            delete map[key];
        };
        StringMapWrapper.forEach = function (map, callback) {
            for (var prop in map) {
                if (map.hasOwnProperty(prop)) {
                    callback(map[prop], prop);
                }
            }
        };
        StringMapWrapper.merge = function (m1, m2) {
            var m = {};
            for (var attr in m1) {
                if (m1.hasOwnProperty(attr)) {
                    m[attr] = m1[attr];
                }
            }
            for (var attr in m2) {
                if (m2.hasOwnProperty(attr)) {
                    m[attr] = m2[attr];
                }
            }
            return m;
        };
        StringMapWrapper.equals = function (m1, m2) {
            var k1 = Object.keys(m1);
            var k2 = Object.keys(m2);
            if (k1.length != k2.length) {
                return false;
            }
            var key;
            for (var i = 0; i < k1.length; i++) {
                key = k1[i];
                if (m1[key] !== m2[key]) {
                    return false;
                }
            }
            return true;
        };
        return StringMapWrapper;
    }();
    exports.StringMapWrapper = StringMapWrapper;
    var ListWrapper = function () {
        function ListWrapper() {}
        // JS has no way to express a statically fixed size list, but dart does so we
        // keep both methods.
        ListWrapper.createFixedSize = function (size) {
            return new Array(size);
        };
        ListWrapper.createGrowableSize = function (size) {
            return new Array(size);
        };
        ListWrapper.clone = function (array) {
            return array.slice(0);
        };
        ListWrapper.forEachWithIndex = function (array, fn) {
            for (var i = 0; i < array.length; i++) {
                fn(array[i], i);
            }
        };
        ListWrapper.first = function (array) {
            if (!array) return null;
            return array[0];
        };
        ListWrapper.last = function (array) {
            if (!array || array.length == 0) return null;
            return array[array.length - 1];
        };
        ListWrapper.indexOf = function (array, value, startIndex) {
            if (startIndex === void 0) {
                startIndex = 0;
            }
            return array.indexOf(value, startIndex);
        };
        ListWrapper.contains = function (list, el) {
            return list.indexOf(el) !== -1;
        };
        ListWrapper.reversed = function (array) {
            var a = ListWrapper.clone(array);
            return a.reverse();
        };
        ListWrapper.concat = function (a, b) {
            return a.concat(b);
        };
        ListWrapper.insert = function (list, index, value) {
            list.splice(index, 0, value);
        };
        ListWrapper.removeAt = function (list, index) {
            var res = list[index];
            list.splice(index, 1);
            return res;
        };
        ListWrapper.removeAll = function (list, items) {
            for (var i = 0; i < items.length; ++i) {
                var index = list.indexOf(items[i]);
                list.splice(index, 1);
            }
        };
        ListWrapper.remove = function (list, el) {
            var index = list.indexOf(el);
            if (index > -1) {
                list.splice(index, 1);
                return true;
            }
            return false;
        };
        ListWrapper.clear = function (list) {
            list.length = 0;
        };
        ListWrapper.isEmpty = function (list) {
            return list.length == 0;
        };
        ListWrapper.fill = function (list, value, start, end) {
            if (start === void 0) {
                start = 0;
            }
            if (end === void 0) {
                end = null;
            }
            list.fill(value, start, end === null ? list.length : end);
        };
        ListWrapper.equals = function (a, b) {
            if (a.length != b.length) return false;
            for (var i = 0; i < a.length; ++i) {
                if (a[i] !== b[i]) return false;
            }
            return true;
        };
        ListWrapper.slice = function (l, from, to) {
            if (from === void 0) {
                from = 0;
            }
            if (to === void 0) {
                to = null;
            }
            return l.slice(from, to === null ? undefined : to);
        };
        ListWrapper.splice = function (l, from, length) {
            return l.splice(from, length);
        };
        ListWrapper.sort = function (l, compareFn) {
            if (lang_1.isPresent(compareFn)) {
                l.sort(compareFn);
            } else {
                l.sort();
            }
        };
        ListWrapper.toString = function (l) {
            return l.toString();
        };
        ListWrapper.toJSON = function (l) {
            return JSON.stringify(l);
        };
        ListWrapper.maximum = function (list, predicate) {
            if (list.length == 0) {
                return null;
            }
            var solution = null;
            var maxValue = -Infinity;
            for (var index = 0; index < list.length; index++) {
                var candidate = list[index];
                if (lang_1.isBlank(candidate)) {
                    continue;
                }
                var candidateValue = predicate(candidate);
                if (candidateValue > maxValue) {
                    solution = candidate;
                    maxValue = candidateValue;
                }
            }
            return solution;
        };
        ListWrapper.flatten = function (list) {
            var target = [];
            _flattenArray(list, target);
            return target;
        };
        ListWrapper.addAll = function (list, source) {
            for (var i = 0; i < source.length; i++) {
                list.push(source[i]);
            }
        };
        return ListWrapper;
    }();
    exports.ListWrapper = ListWrapper;
    function _flattenArray(source, target) {
        if (lang_1.isPresent(source)) {
            for (var i = 0; i < source.length; i++) {
                var item = source[i];
                if (lang_1.isArray(item)) {
                    _flattenArray(item, target);
                } else {
                    target.push(item);
                }
            }
        }
        return target;
    }
    function isListLikeIterable(obj) {
        if (!lang_1.isJsObject(obj)) return false;
        return lang_1.isArray(obj) || !(obj instanceof exports.Map) && lang_1.getSymbolIterator() in obj; // JS Iterable have a Symbol.iterator prop
    }
    exports.isListLikeIterable = isListLikeIterable;
    function areIterablesEqual(a, b, comparator) {
        var iterator1 = a[lang_1.getSymbolIterator()]();
        var iterator2 = b[lang_1.getSymbolIterator()]();
        while (true) {
            var item1 = iterator1.next();
            var item2 = iterator2.next();
            if (item1.done && item2.done) return true;
            if (item1.done || item2.done) return false;
            if (!comparator(item1.value, item2.value)) return false;
        }
    }
    exports.areIterablesEqual = areIterablesEqual;
    function iterateListLike(obj, fn) {
        if (lang_1.isArray(obj)) {
            for (var i = 0; i < obj.length; i++) {
                fn(obj[i]);
            }
        } else {
            var iterator = obj[lang_1.getSymbolIterator()]();
            var item;
            while (!(item = iterator.next()).done) {
                fn(item.value);
            }
        }
    }
    exports.iterateListLike = iterateListLike;
    // Safari and Internet Explorer do not support the iterable parameter to the
    // Set constructor.  We work around that by manually adding the items.
    var createSetFromList = function () {
        var test = new exports.Set([1, 2, 3]);
        if (test.size === 3) {
            return function createSetFromList(lst) {
                return new exports.Set(lst);
            };
        } else {
            return function createSetAndPopulateFromList(lst) {
                var res = new exports.Set(lst);
                if (res.size !== lst.length) {
                    for (var i = 0; i < lst.length; i++) {
                        res.add(lst[i]);
                    }
                }
                return res;
            };
        }
    }();
    var SetWrapper = function () {
        function SetWrapper() {}
        SetWrapper.createFromList = function (lst) {
            return createSetFromList(lst);
        };
        SetWrapper.has = function (s, key) {
            return s.has(key);
        };
        SetWrapper.delete = function (m, k) {
            m.delete(k);
        };
        return SetWrapper;
    }();
    exports.SetWrapper = SetWrapper;
    
});
System.registerDynamic('node_modules/@angular/core/src/facade/exception_handler.js', ['./base_wrapped_exception', './collection', './lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var base_wrapped_exception_1 = $__require('./base_wrapped_exception');
    var collection_1 = $__require('./collection');
    var lang_1 = $__require('./lang');
    var _ArrayLogger = function () {
        function _ArrayLogger() {
            this.res = [];
        }
        _ArrayLogger.prototype.log = function (s) {
            this.res.push(s);
        };
        _ArrayLogger.prototype.logError = function (s) {
            this.res.push(s);
        };
        _ArrayLogger.prototype.logGroup = function (s) {
            this.res.push(s);
        };
        _ArrayLogger.prototype.logGroupEnd = function () {};
        ;
        return _ArrayLogger;
    }();
    /**
     * Provides a hook for centralized exception handling.
     *
     * The default implementation of `ExceptionHandler` prints error messages to the `Console`. To
     * intercept error handling,
     * write a custom exception handler that replaces this default as appropriate for your app.
     *
     * ### Example
     *
     * ```javascript
     *
     * class MyExceptionHandler implements ExceptionHandler {
     *   call(error, stackTrace = null, reason = null) {
     *     // do something with the exception
     *   }
     * }
     *
     * bootstrap(MyApp, {provide: ExceptionHandler, useClass: MyExceptionHandler}])
     *
     * ```
     * @stable
     */
    var ExceptionHandler = function () {
        function ExceptionHandler(_logger, _rethrowException) {
            if (_rethrowException === void 0) {
                _rethrowException = true;
            }
            this._logger = _logger;
            this._rethrowException = _rethrowException;
        }
        ExceptionHandler.exceptionToString = function (exception, stackTrace, reason) {
            if (stackTrace === void 0) {
                stackTrace = null;
            }
            if (reason === void 0) {
                reason = null;
            }
            var l = new _ArrayLogger();
            var e = new ExceptionHandler(l, false);
            e.call(exception, stackTrace, reason);
            return l.res.join('\n');
        };
        ExceptionHandler.prototype.call = function (exception, stackTrace, reason) {
            if (stackTrace === void 0) {
                stackTrace = null;
            }
            if (reason === void 0) {
                reason = null;
            }
            var originalException = this._findOriginalException(exception);
            var originalStack = this._findOriginalStack(exception);
            var context = this._findContext(exception);
            this._logger.logGroup("EXCEPTION: " + this._extractMessage(exception));
            if (lang_1.isPresent(stackTrace) && lang_1.isBlank(originalStack)) {
                this._logger.logError('STACKTRACE:');
                this._logger.logError(this._longStackTrace(stackTrace));
            }
            if (lang_1.isPresent(reason)) {
                this._logger.logError("REASON: " + reason);
            }
            if (lang_1.isPresent(originalException)) {
                this._logger.logError("ORIGINAL EXCEPTION: " + this._extractMessage(originalException));
            }
            if (lang_1.isPresent(originalStack)) {
                this._logger.logError('ORIGINAL STACKTRACE:');
                this._logger.logError(this._longStackTrace(originalStack));
            }
            if (lang_1.isPresent(context)) {
                this._logger.logError('ERROR CONTEXT:');
                this._logger.logError(context);
            }
            this._logger.logGroupEnd();
            // We rethrow exceptions, so operations like 'bootstrap' will result in an error
            // when an exception happens. If we do not rethrow, bootstrap will always succeed.
            if (this._rethrowException) throw exception;
        };
        /** @internal */
        ExceptionHandler.prototype._extractMessage = function (exception) {
            return exception instanceof base_wrapped_exception_1.BaseWrappedException ? exception.wrapperMessage : exception.toString();
        };
        /** @internal */
        ExceptionHandler.prototype._longStackTrace = function (stackTrace) {
            return collection_1.isListLikeIterable(stackTrace) ? stackTrace.join('\n\n-----async gap-----\n') : stackTrace.toString();
        };
        /** @internal */
        ExceptionHandler.prototype._findContext = function (exception) {
            try {
                if (!(exception instanceof base_wrapped_exception_1.BaseWrappedException)) return null;
                return lang_1.isPresent(exception.context) ? exception.context : this._findContext(exception.originalException);
            } catch (e) {
                // exception.context can throw an exception. if it happens, we ignore the context.
                return null;
            }
        };
        /** @internal */
        ExceptionHandler.prototype._findOriginalException = function (exception) {
            if (!(exception instanceof base_wrapped_exception_1.BaseWrappedException)) return null;
            var e = exception.originalException;
            while (e instanceof base_wrapped_exception_1.BaseWrappedException && lang_1.isPresent(e.originalException)) {
                e = e.originalException;
            }
            return e;
        };
        /** @internal */
        ExceptionHandler.prototype._findOriginalStack = function (exception) {
            if (!(exception instanceof base_wrapped_exception_1.BaseWrappedException)) return null;
            var e = exception;
            var stack = exception.originalStack;
            while (e instanceof base_wrapped_exception_1.BaseWrappedException && lang_1.isPresent(e.originalException)) {
                e = e.originalException;
                if (e instanceof base_wrapped_exception_1.BaseWrappedException && lang_1.isPresent(e.originalException)) {
                    stack = e.originalStack;
                }
            }
            return stack;
        };
        return ExceptionHandler;
    }();
    exports.ExceptionHandler = ExceptionHandler;
    
});
System.registerDynamic('node_modules/@angular/core/src/facade/exceptions.js', ['./base_wrapped_exception', './exception_handler'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var base_wrapped_exception_1 = $__require('./base_wrapped_exception');
    var exception_handler_1 = $__require('./exception_handler');
    var exception_handler_2 = $__require('./exception_handler');
    exports.ExceptionHandler = exception_handler_2.ExceptionHandler;
    /**
     * @stable
     */
    var BaseException = function (_super) {
        __extends(BaseException, _super);
        function BaseException(message) {
            if (message === void 0) {
                message = '--';
            }
            _super.call(this, message);
            this.message = message;
            this.stack = new Error(message).stack;
        }
        BaseException.prototype.toString = function () {
            return this.message;
        };
        return BaseException;
    }(Error);
    exports.BaseException = BaseException;
    /**
     * Wraps an exception and provides additional context or information.
     * @stable
     */
    var WrappedException = function (_super) {
        __extends(WrappedException, _super);
        function WrappedException(_wrapperMessage, _originalException /** TODO #9100 */, _originalStack /** TODO #9100 */, _context /** TODO #9100 */) {
            _super.call(this, _wrapperMessage);
            this._wrapperMessage = _wrapperMessage;
            this._originalException = _originalException;
            this._originalStack = _originalStack;
            this._context = _context;
            this._wrapperStack = new Error(_wrapperMessage).stack;
        }
        Object.defineProperty(WrappedException.prototype, "wrapperMessage", {
            get: function () {
                return this._wrapperMessage;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "wrapperStack", {
            get: function () {
                return this._wrapperStack;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "originalException", {
            get: function () {
                return this._originalException;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "originalStack", {
            get: function () {
                return this._originalStack;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "context", {
            get: function () {
                return this._context;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "message", {
            get: function () {
                return exception_handler_1.ExceptionHandler.exceptionToString(this);
            },
            enumerable: true,
            configurable: true
        });
        WrappedException.prototype.toString = function () {
            return this.message;
        };
        return WrappedException;
    }(base_wrapped_exception_1.BaseWrappedException);
    exports.WrappedException = WrappedException;
    function makeTypeError(message) {
        return new TypeError(message);
    }
    exports.makeTypeError = makeTypeError;
    function unimplemented() {
        throw new BaseException('unimplemented');
    }
    exports.unimplemented = unimplemented;
    
});
System.registerDynamic('node_modules/@angular/core/src/facade/lang.js', [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var globalScope;
    if (typeof window === 'undefined') {
        if (typeof WorkerGlobalScope !== 'undefined' && self instanceof WorkerGlobalScope) {
            // TODO: Replace any with WorkerGlobalScope from lib.webworker.d.ts #3492
            globalScope = self;
        } else {
            globalScope = global;
        }
    } else {
        globalScope = window;
    }
    function scheduleMicroTask(fn) {
        Zone.current.scheduleMicroTask('scheduleMicrotask', fn);
    }
    exports.scheduleMicroTask = scheduleMicroTask;
    exports.IS_DART = false;
    // Need to declare a new variable for global here since TypeScript
    // exports the original value of the symbol.
    var _global = globalScope;
    exports.global = _global;
    /**
     * Runtime representation a type that a Component or other object is instances of.
     *
     * An example of a `Type` is `MyCustomComponent` class, which in JavaScript is be represented by
     * the `MyCustomComponent` constructor function.
     *
     * @stable
     */
    exports.Type = Function;
    function getTypeNameForDebugging(type) {
        if (type['name']) {
            return type['name'];
        }
        return typeof type;
    }
    exports.getTypeNameForDebugging = getTypeNameForDebugging;
    exports.Math = _global.Math;
    exports.Date = _global.Date;
    // TODO: remove calls to assert in production environment
    // Note: Can't just export this and import in in other files
    // as `assert` is a reserved keyword in Dart
    _global.assert = function assert(condition) {
        // TODO: to be fixed properly via #2830, noop for now
    };
    function isPresent(obj) {
        return obj !== undefined && obj !== null;
    }
    exports.isPresent = isPresent;
    function isBlank(obj) {
        return obj === undefined || obj === null;
    }
    exports.isBlank = isBlank;
    function isBoolean(obj) {
        return typeof obj === 'boolean';
    }
    exports.isBoolean = isBoolean;
    function isNumber(obj) {
        return typeof obj === 'number';
    }
    exports.isNumber = isNumber;
    function isString(obj) {
        return typeof obj === 'string';
    }
    exports.isString = isString;
    function isFunction(obj) {
        return typeof obj === 'function';
    }
    exports.isFunction = isFunction;
    function isType(obj) {
        return isFunction(obj);
    }
    exports.isType = isType;
    function isStringMap(obj) {
        return typeof obj === 'object' && obj !== null;
    }
    exports.isStringMap = isStringMap;
    var STRING_MAP_PROTO = Object.getPrototypeOf({});
    function isStrictStringMap(obj) {
        return isStringMap(obj) && Object.getPrototypeOf(obj) === STRING_MAP_PROTO;
    }
    exports.isStrictStringMap = isStrictStringMap;
    function isPromise(obj) {
        return obj instanceof _global.Promise;
    }
    exports.isPromise = isPromise;
    function isArray(obj) {
        return Array.isArray(obj);
    }
    exports.isArray = isArray;
    function isDate(obj) {
        return obj instanceof exports.Date && !isNaN(obj.valueOf());
    }
    exports.isDate = isDate;
    function noop() {}
    exports.noop = noop;
    function stringify(token) {
        if (typeof token === 'string') {
            return token;
        }
        if (token === undefined || token === null) {
            return '' + token;
        }
        if (token.name) {
            return token.name;
        }
        if (token.overriddenName) {
            return token.overriddenName;
        }
        var res = token.toString();
        var newLineIndex = res.indexOf('\n');
        return newLineIndex === -1 ? res : res.substring(0, newLineIndex);
    }
    exports.stringify = stringify;
    // serialize / deserialize enum exist only for consistency with dart API
    // enums in typescript don't need to be serialized
    function serializeEnum(val) {
        return val;
    }
    exports.serializeEnum = serializeEnum;
    function deserializeEnum(val, values) {
        return val;
    }
    exports.deserializeEnum = deserializeEnum;
    function resolveEnumToken(enumValue, val) {
        return enumValue[val];
    }
    exports.resolveEnumToken = resolveEnumToken;
    var StringWrapper = function () {
        function StringWrapper() {}
        StringWrapper.fromCharCode = function (code) {
            return String.fromCharCode(code);
        };
        StringWrapper.charCodeAt = function (s, index) {
            return s.charCodeAt(index);
        };
        StringWrapper.split = function (s, regExp) {
            return s.split(regExp);
        };
        StringWrapper.equals = function (s, s2) {
            return s === s2;
        };
        StringWrapper.stripLeft = function (s, charVal) {
            if (s && s.length) {
                var pos = 0;
                for (var i = 0; i < s.length; i++) {
                    if (s[i] != charVal) break;
                    pos++;
                }
                s = s.substring(pos);
            }
            return s;
        };
        StringWrapper.stripRight = function (s, charVal) {
            if (s && s.length) {
                var pos = s.length;
                for (var i = s.length - 1; i >= 0; i--) {
                    if (s[i] != charVal) break;
                    pos--;
                }
                s = s.substring(0, pos);
            }
            return s;
        };
        StringWrapper.replace = function (s, from, replace) {
            return s.replace(from, replace);
        };
        StringWrapper.replaceAll = function (s, from, replace) {
            return s.replace(from, replace);
        };
        StringWrapper.slice = function (s, from, to) {
            if (from === void 0) {
                from = 0;
            }
            if (to === void 0) {
                to = null;
            }
            return s.slice(from, to === null ? undefined : to);
        };
        StringWrapper.replaceAllMapped = function (s, from, cb) {
            return s.replace(from, function () {
                var matches = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    matches[_i - 0] = arguments[_i];
                }
                // Remove offset & string from the result array
                matches.splice(-2, 2);
                // The callback receives match, p1, ..., pn
                return cb(matches);
            });
        };
        StringWrapper.contains = function (s, substr) {
            return s.indexOf(substr) != -1;
        };
        StringWrapper.compare = function (a, b) {
            if (a < b) {
                return -1;
            } else if (a > b) {
                return 1;
            } else {
                return 0;
            }
        };
        return StringWrapper;
    }();
    exports.StringWrapper = StringWrapper;
    var StringJoiner = function () {
        function StringJoiner(parts) {
            if (parts === void 0) {
                parts = [];
            }
            this.parts = parts;
        }
        StringJoiner.prototype.add = function (part) {
            this.parts.push(part);
        };
        StringJoiner.prototype.toString = function () {
            return this.parts.join('');
        };
        return StringJoiner;
    }();
    exports.StringJoiner = StringJoiner;
    var NumberParseError = function (_super) {
        __extends(NumberParseError, _super);
        function NumberParseError(message) {
            _super.call(this);
            this.message = message;
        }
        NumberParseError.prototype.toString = function () {
            return this.message;
        };
        return NumberParseError;
    }(Error);
    exports.NumberParseError = NumberParseError;
    var NumberWrapper = function () {
        function NumberWrapper() {}
        NumberWrapper.toFixed = function (n, fractionDigits) {
            return n.toFixed(fractionDigits);
        };
        NumberWrapper.equal = function (a, b) {
            return a === b;
        };
        NumberWrapper.parseIntAutoRadix = function (text) {
            var result = parseInt(text);
            if (isNaN(result)) {
                throw new NumberParseError('Invalid integer literal when parsing ' + text);
            }
            return result;
        };
        NumberWrapper.parseInt = function (text, radix) {
            if (radix == 10) {
                if (/^(\-|\+)?[0-9]+$/.test(text)) {
                    return parseInt(text, radix);
                }
            } else if (radix == 16) {
                if (/^(\-|\+)?[0-9ABCDEFabcdef]+$/.test(text)) {
                    return parseInt(text, radix);
                }
            } else {
                var result = parseInt(text, radix);
                if (!isNaN(result)) {
                    return result;
                }
            }
            throw new NumberParseError('Invalid integer literal when parsing ' + text + ' in base ' + radix);
        };
        // TODO: NaN is a valid literal but is returned by parseFloat to indicate an error.
        NumberWrapper.parseFloat = function (text) {
            return parseFloat(text);
        };
        Object.defineProperty(NumberWrapper, "NaN", {
            get: function () {
                return NaN;
            },
            enumerable: true,
            configurable: true
        });
        NumberWrapper.isNumeric = function (value) {
            return !isNaN(value - parseFloat(value));
        };
        NumberWrapper.isNaN = function (value) {
            return isNaN(value);
        };
        NumberWrapper.isInteger = function (value) {
            return Number.isInteger(value);
        };
        return NumberWrapper;
    }();
    exports.NumberWrapper = NumberWrapper;
    exports.RegExp = _global.RegExp;
    var RegExpWrapper = function () {
        function RegExpWrapper() {}
        RegExpWrapper.create = function (regExpStr, flags) {
            if (flags === void 0) {
                flags = '';
            }
            flags = flags.replace(/g/g, '');
            return new _global.RegExp(regExpStr, flags + 'g');
        };
        RegExpWrapper.firstMatch = function (regExp, input) {
            // Reset multimatch regex state
            regExp.lastIndex = 0;
            return regExp.exec(input);
        };
        RegExpWrapper.test = function (regExp, input) {
            regExp.lastIndex = 0;
            return regExp.test(input);
        };
        RegExpWrapper.matcher = function (regExp, input) {
            // Reset regex state for the case
            // someone did not loop over all matches
            // last time.
            regExp.lastIndex = 0;
            return { re: regExp, input: input };
        };
        RegExpWrapper.replaceAll = function (regExp, input, replace) {
            var c = regExp.exec(input);
            var res = '';
            regExp.lastIndex = 0;
            var prev = 0;
            while (c) {
                res += input.substring(prev, c.index);
                res += replace(c);
                prev = c.index + c[0].length;
                regExp.lastIndex = prev;
                c = regExp.exec(input);
            }
            res += input.substring(prev);
            return res;
        };
        return RegExpWrapper;
    }();
    exports.RegExpWrapper = RegExpWrapper;
    var RegExpMatcherWrapper = function () {
        function RegExpMatcherWrapper() {}
        RegExpMatcherWrapper.next = function (matcher) {
            return matcher.re.exec(matcher.input);
        };
        return RegExpMatcherWrapper;
    }();
    exports.RegExpMatcherWrapper = RegExpMatcherWrapper;
    var FunctionWrapper = function () {
        function FunctionWrapper() {}
        FunctionWrapper.apply = function (fn, posArgs) {
            return fn.apply(null, posArgs);
        };
        FunctionWrapper.bind = function (fn, scope) {
            return fn.bind(scope);
        };
        return FunctionWrapper;
    }();
    exports.FunctionWrapper = FunctionWrapper;
    // JS has NaN !== NaN
    function looseIdentical(a, b) {
        return a === b || typeof a === 'number' && typeof b === 'number' && isNaN(a) && isNaN(b);
    }
    exports.looseIdentical = looseIdentical;
    // JS considers NaN is the same as NaN for map Key (while NaN !== NaN otherwise)
    // see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Map
    function getMapKey(value) {
        return value;
    }
    exports.getMapKey = getMapKey;
    function normalizeBlank(obj) {
        return isBlank(obj) ? null : obj;
    }
    exports.normalizeBlank = normalizeBlank;
    function normalizeBool(obj) {
        return isBlank(obj) ? false : obj;
    }
    exports.normalizeBool = normalizeBool;
    function isJsObject(o) {
        return o !== null && (typeof o === 'function' || typeof o === 'object');
    }
    exports.isJsObject = isJsObject;
    function print(obj) {
        console.log(obj);
    }
    exports.print = print;
    function warn(obj) {
        console.warn(obj);
    }
    exports.warn = warn;
    // Can't be all uppercase as our transpiler would think it is a special directive...
    var Json = function () {
        function Json() {}
        Json.parse = function (s) {
            return _global.JSON.parse(s);
        };
        Json.stringify = function (data) {
            // Dart doesn't take 3 arguments
            return _global.JSON.stringify(data, null, 2);
        };
        return Json;
    }();
    exports.Json = Json;
    var DateWrapper = function () {
        function DateWrapper() {}
        DateWrapper.create = function (year, month, day, hour, minutes, seconds, milliseconds) {
            if (month === void 0) {
                month = 1;
            }
            if (day === void 0) {
                day = 1;
            }
            if (hour === void 0) {
                hour = 0;
            }
            if (minutes === void 0) {
                minutes = 0;
            }
            if (seconds === void 0) {
                seconds = 0;
            }
            if (milliseconds === void 0) {
                milliseconds = 0;
            }
            return new exports.Date(year, month - 1, day, hour, minutes, seconds, milliseconds);
        };
        DateWrapper.fromISOString = function (str) {
            return new exports.Date(str);
        };
        DateWrapper.fromMillis = function (ms) {
            return new exports.Date(ms);
        };
        DateWrapper.toMillis = function (date) {
            return date.getTime();
        };
        DateWrapper.now = function () {
            return new exports.Date();
        };
        DateWrapper.toJson = function (date) {
            return date.toJSON();
        };
        return DateWrapper;
    }();
    exports.DateWrapper = DateWrapper;
    function setValueOnPath(global, path, value) {
        var parts = path.split('.');
        var obj = global;
        while (parts.length > 1) {
            var name = parts.shift();
            if (obj.hasOwnProperty(name) && isPresent(obj[name])) {
                obj = obj[name];
            } else {
                obj = obj[name] = {};
            }
        }
        if (obj === undefined || obj === null) {
            obj = {};
        }
        obj[parts.shift()] = value;
    }
    exports.setValueOnPath = setValueOnPath;
    var _symbolIterator = null;
    function getSymbolIterator() {
        if (isBlank(_symbolIterator)) {
            if (isPresent(globalScope.Symbol) && isPresent(Symbol.iterator)) {
                _symbolIterator = Symbol.iterator;
            } else {
                // es6-shim specific logic
                var keys = Object.getOwnPropertyNames(Map.prototype);
                for (var i = 0; i < keys.length; ++i) {
                    var key = keys[i];
                    if (key !== 'entries' && key !== 'size' && Map.prototype[key] === Map.prototype['entries']) {
                        _symbolIterator = key;
                    }
                }
            }
        }
        return _symbolIterator;
    }
    exports.getSymbolIterator = getSymbolIterator;
    function evalExpression(sourceUrl, expr, declarations, vars) {
        var fnBody = declarations + "\nreturn " + expr + "\n//# sourceURL=" + sourceUrl;
        var fnArgNames = [];
        var fnArgValues = [];
        for (var argName in vars) {
            fnArgNames.push(argName);
            fnArgValues.push(vars[argName]);
        }
        return new (Function.bind.apply(Function, [void 0].concat(fnArgNames.concat(fnBody))))().apply(void 0, fnArgValues);
    }
    exports.evalExpression = evalExpression;
    function isPrimitive(obj) {
        return !isJsObject(obj);
    }
    exports.isPrimitive = isPrimitive;
    function hasConstructor(value, type) {
        return value.constructor === type;
    }
    exports.hasConstructor = hasConstructor;
    function escape(s) {
        return _global.encodeURI(s);
    }
    exports.escape = escape;
    function escapeRegExp(s) {
        return s.replace(/([.*+?^=!:${}()|[\]\/\\])/g, '\\$1');
    }
    exports.escapeRegExp = escapeRegExp;
    
});
System.registerDynamic('node_modules/@angular/core/src/animation/animation_player.js', ['../facade/exceptions', '../facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var exceptions_1 = $__require('../facade/exceptions');
    var lang_1 = $__require('../facade/lang');
    /**
     * @experimental Animation support is experimental.
     */
    var AnimationPlayer = function () {
        function AnimationPlayer() {}
        Object.defineProperty(AnimationPlayer.prototype, "parentPlayer", {
            get: function () {
                throw new exceptions_1.BaseException('NOT IMPLEMENTED: Base Class');
            },
            set: function (player) {
                throw new exceptions_1.BaseException('NOT IMPLEMENTED: Base Class');
            },
            enumerable: true,
            configurable: true
        });
        return AnimationPlayer;
    }();
    exports.AnimationPlayer = AnimationPlayer;
    var NoOpAnimationPlayer = function () {
        function NoOpAnimationPlayer() {
            var _this = this;
            this._subscriptions = [];
            this.parentPlayer = null;
            lang_1.scheduleMicroTask(function () {
                return _this._onFinish();
            });
        }
        /** @internal */
        NoOpAnimationPlayer.prototype._onFinish = function () {
            this._subscriptions.forEach(function (entry) {
                entry();
            });
            this._subscriptions = [];
        };
        NoOpAnimationPlayer.prototype.onDone = function (fn) {
            this._subscriptions.push(fn);
        };
        NoOpAnimationPlayer.prototype.play = function () {};
        NoOpAnimationPlayer.prototype.pause = function () {};
        NoOpAnimationPlayer.prototype.restart = function () {};
        NoOpAnimationPlayer.prototype.finish = function () {
            this._onFinish();
        };
        NoOpAnimationPlayer.prototype.destroy = function () {};
        NoOpAnimationPlayer.prototype.reset = function () {};
        NoOpAnimationPlayer.prototype.setPosition = function (p /** TODO #9100 */) {};
        NoOpAnimationPlayer.prototype.getPosition = function () {
            return 0;
        };
        return NoOpAnimationPlayer;
    }();
    exports.NoOpAnimationPlayer = NoOpAnimationPlayer;
    
});
System.registerDynamic('node_modules/@angular/core/index.js', ['./src/metadata', './src/util', './src/di', './src/application_ref', './src/application_tokens', './src/zone', './src/render', './src/linker', './src/debug/debug_node', './src/testability/testability', './src/change_detection', './src/platform_directives_and_pipes', './src/platform_common_providers', './src/application_common_providers', './src/profile/profile', './src/facade/lang', './src/facade/async', './src/facade/exceptions', './private_export', './src/animation/metadata', './src/animation/animation_player'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
  }
  /**
   * @module
   * @description
   * Entry point from which you should import all public core APIs.
   */
  __export($__require('./src/metadata'));
  __export($__require('./src/util'));
  __export($__require('./src/di'));
  var application_ref_1 = $__require('./src/application_ref');
  exports.createPlatform = application_ref_1.createPlatform;
  exports.assertPlatform = application_ref_1.assertPlatform;
  exports.disposePlatform = application_ref_1.disposePlatform;
  exports.getPlatform = application_ref_1.getPlatform;
  exports.coreBootstrap = application_ref_1.coreBootstrap;
  exports.coreLoadAndBootstrap = application_ref_1.coreLoadAndBootstrap;
  exports.createNgZone = application_ref_1.createNgZone;
  exports.PlatformRef = application_ref_1.PlatformRef;
  exports.ApplicationRef = application_ref_1.ApplicationRef;
  exports.enableProdMode = application_ref_1.enableProdMode;
  exports.lockRunMode = application_ref_1.lockRunMode;
  exports.isDevMode = application_ref_1.isDevMode;
  var application_tokens_1 = $__require('./src/application_tokens');
  exports.APP_ID = application_tokens_1.APP_ID;
  exports.APP_INITIALIZER = application_tokens_1.APP_INITIALIZER;
  exports.PACKAGE_ROOT_URL = application_tokens_1.PACKAGE_ROOT_URL;
  exports.PLATFORM_INITIALIZER = application_tokens_1.PLATFORM_INITIALIZER;
  __export($__require('./src/zone'));
  __export($__require('./src/render'));
  __export($__require('./src/linker'));
  var debug_node_1 = $__require('./src/debug/debug_node');
  exports.DebugElement = debug_node_1.DebugElement;
  exports.DebugNode = debug_node_1.DebugNode;
  exports.asNativeElements = debug_node_1.asNativeElements;
  exports.getDebugNode = debug_node_1.getDebugNode;
  __export($__require('./src/testability/testability'));
  __export($__require('./src/change_detection'));
  __export($__require('./src/platform_directives_and_pipes'));
  __export($__require('./src/platform_common_providers'));
  __export($__require('./src/application_common_providers'));
  var profile_1 = $__require('./src/profile/profile');
  exports.wtfCreateScope = profile_1.wtfCreateScope;
  exports.wtfLeave = profile_1.wtfLeave;
  exports.wtfStartTimeRange = profile_1.wtfStartTimeRange;
  exports.wtfEndTimeRange = profile_1.wtfEndTimeRange;
  var lang_1 = $__require('./src/facade/lang');
  exports.Type = lang_1.Type;
  var async_1 = $__require('./src/facade/async');
  exports.EventEmitter = async_1.EventEmitter;
  var exceptions_1 = $__require('./src/facade/exceptions');
  exports.ExceptionHandler = exceptions_1.ExceptionHandler;
  exports.WrappedException = exceptions_1.WrappedException;
  exports.BaseException = exceptions_1.BaseException;
  __export($__require('./private_export'));
  __export($__require('./src/animation/metadata'));
  var animation_player_1 = $__require('./src/animation/animation_player');
  exports.AnimationPlayer = animation_player_1.AnimationPlayer;
  
});
System.registerDynamic('node_modules/@angular/http/src/base_request_options.js', ['@angular/core', '../src/facade/lang', './enums', './headers', './http_utils', './url_search_params'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var lang_1 = $__require('../src/facade/lang');
    var enums_1 = $__require('./enums');
    var headers_1 = $__require('./headers');
    var http_utils_1 = $__require('./http_utils');
    var url_search_params_1 = $__require('./url_search_params');
    /**
     * Creates a request options object to be optionally provided when instantiating a
     * {@link Request}.
     *
     * This class is based on the `RequestInit` description in the [Fetch
     * Spec](https://fetch.spec.whatwg.org/#requestinit).
     *
     * All values are null by default. Typical defaults can be found in the {@link BaseRequestOptions}
     * class, which sub-classes `RequestOptions`.
     *
     * ### Example ([live demo](http://plnkr.co/edit/7Wvi3lfLq41aQPKlxB4O?p=preview))
     *
     * ```typescript
     * import {RequestOptions, Request, RequestMethod} from '@angular/http';
     *
     * var options = new RequestOptions({
     *   method: RequestMethod.Post,
     *   url: 'https://google.com'
     * });
     * var req = new Request(options);
     * console.log('req.method:', RequestMethod[req.method]); // Post
     * console.log('options.url:', options.url); // https://google.com
     * ```
     *
     * @experimental
     */
    var RequestOptions = function () {
        function RequestOptions(_a) {
            var _b = _a === void 0 ? {} : _a,
                method = _b.method,
                headers = _b.headers,
                body = _b.body,
                url = _b.url,
                search = _b.search,
                withCredentials = _b.withCredentials;
            this.method = lang_1.isPresent(method) ? http_utils_1.normalizeMethodName(method) : null;
            this.headers = lang_1.isPresent(headers) ? headers : null;
            this.body = lang_1.isPresent(body) ? body : null;
            this.url = lang_1.isPresent(url) ? url : null;
            this.search = lang_1.isPresent(search) ? lang_1.isString(search) ? new url_search_params_1.URLSearchParams(search) : search : null;
            this.withCredentials = lang_1.isPresent(withCredentials) ? withCredentials : null;
        }
        /**
         * Creates a copy of the `RequestOptions` instance, using the optional input as values to override
         * existing values. This method will not change the values of the instance on which it is being
         * called.
         *
         * Note that `headers` and `search` will override existing values completely if present in
         * the `options` object. If these values should be merged, it should be done prior to calling
         * `merge` on the `RequestOptions` instance.
         *
         * ### Example ([live demo](http://plnkr.co/edit/6w8XA8YTkDRcPYpdB9dk?p=preview))
         *
         * ```typescript
         * import {RequestOptions, Request, RequestMethod} from '@angular/http';
         *
         * var options = new RequestOptions({
         *   method: RequestMethod.Post
         * });
         * var req = new Request(options.merge({
         *   url: 'https://google.com'
         * }));
         * console.log('req.method:', RequestMethod[req.method]); // Post
         * console.log('options.url:', options.url); // null
         * console.log('req.url:', req.url); // https://google.com
         * ```
         */
        RequestOptions.prototype.merge = function (options) {
            return new RequestOptions({
                method: lang_1.isPresent(options) && lang_1.isPresent(options.method) ? options.method : this.method,
                headers: lang_1.isPresent(options) && lang_1.isPresent(options.headers) ? options.headers : this.headers,
                body: lang_1.isPresent(options) && lang_1.isPresent(options.body) ? options.body : this.body,
                url: lang_1.isPresent(options) && lang_1.isPresent(options.url) ? options.url : this.url,
                search: lang_1.isPresent(options) && lang_1.isPresent(options.search) ? lang_1.isString(options.search) ? new url_search_params_1.URLSearchParams(options.search) : options.search.clone() : this.search,
                withCredentials: lang_1.isPresent(options) && lang_1.isPresent(options.withCredentials) ? options.withCredentials : this.withCredentials
            });
        };
        return RequestOptions;
    }();
    exports.RequestOptions = RequestOptions;
    var BaseRequestOptions = function (_super) {
        __extends(BaseRequestOptions, _super);
        function BaseRequestOptions() {
            _super.call(this, { method: enums_1.RequestMethod.Get, headers: new headers_1.Headers() });
        }
        /** @nocollapse */
        BaseRequestOptions.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        BaseRequestOptions.ctorParameters = [];
        return BaseRequestOptions;
    }(RequestOptions);
    exports.BaseRequestOptions = BaseRequestOptions;
    
});
System.registerDynamic('node_modules/@angular/http/src/http.js', ['@angular/core', '../src/facade/exceptions', '../src/facade/lang', './base_request_options', './enums', './interfaces', './static_request'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var core_1 = $__require('@angular/core');
    var exceptions_1 = $__require('../src/facade/exceptions');
    var lang_1 = $__require('../src/facade/lang');
    var base_request_options_1 = $__require('./base_request_options');
    var enums_1 = $__require('./enums');
    var interfaces_1 = $__require('./interfaces');
    var static_request_1 = $__require('./static_request');
    function httpRequest(backend, request) {
        return backend.createConnection(request).response;
    }
    function mergeOptions(defaultOpts, providedOpts, method, url) {
        var newOptions = defaultOpts;
        if (lang_1.isPresent(providedOpts)) {
            // Hack so Dart can used named parameters
            return newOptions.merge(new base_request_options_1.RequestOptions({
                method: providedOpts.method || method,
                url: providedOpts.url || url,
                search: providedOpts.search,
                headers: providedOpts.headers,
                body: providedOpts.body,
                withCredentials: providedOpts.withCredentials
            }));
        }
        if (lang_1.isPresent(method)) {
            return newOptions.merge(new base_request_options_1.RequestOptions({ method: method, url: url }));
        } else {
            return newOptions.merge(new base_request_options_1.RequestOptions({ url: url }));
        }
    }
    var Http = function () {
        function Http(_backend, _defaultOptions) {
            this._backend = _backend;
            this._defaultOptions = _defaultOptions;
        }
        /**
         * Performs any type of http request. First argument is required, and can either be a url or
         * a {@link Request} instance. If the first argument is a url, an optional {@link RequestOptions}
         * object can be provided as the 2nd argument. The options object will be merged with the values
         * of {@link BaseRequestOptions} before performing the request.
         */
        Http.prototype.request = function (url, options) {
            var responseObservable;
            if (lang_1.isString(url)) {
                responseObservable = httpRequest(this._backend, new static_request_1.Request(mergeOptions(this._defaultOptions, options, enums_1.RequestMethod.Get, url)));
            } else if (url instanceof static_request_1.Request) {
                responseObservable = httpRequest(this._backend, url);
            } else {
                throw exceptions_1.makeTypeError('First argument must be a url string or Request instance.');
            }
            return responseObservable;
        };
        /**
         * Performs a request with `get` http method.
         */
        Http.prototype.get = function (url, options) {
            return httpRequest(this._backend, new static_request_1.Request(mergeOptions(this._defaultOptions, options, enums_1.RequestMethod.Get, url)));
        };
        /**
         * Performs a request with `post` http method.
         */
        Http.prototype.post = function (url, body, options) {
            return httpRequest(this._backend, new static_request_1.Request(mergeOptions(this._defaultOptions.merge(new base_request_options_1.RequestOptions({ body: body })), options, enums_1.RequestMethod.Post, url)));
        };
        /**
         * Performs a request with `put` http method.
         */
        Http.prototype.put = function (url, body, options) {
            return httpRequest(this._backend, new static_request_1.Request(mergeOptions(this._defaultOptions.merge(new base_request_options_1.RequestOptions({ body: body })), options, enums_1.RequestMethod.Put, url)));
        };
        /**
         * Performs a request with `delete` http method.
         */
        Http.prototype.delete = function (url, options) {
            return httpRequest(this._backend, new static_request_1.Request(mergeOptions(this._defaultOptions, options, enums_1.RequestMethod.Delete, url)));
        };
        /**
         * Performs a request with `patch` http method.
         */
        Http.prototype.patch = function (url, body, options) {
            return httpRequest(this._backend, new static_request_1.Request(mergeOptions(this._defaultOptions.merge(new base_request_options_1.RequestOptions({ body: body })), options, enums_1.RequestMethod.Patch, url)));
        };
        /**
         * Performs a request with `head` http method.
         */
        Http.prototype.head = function (url, options) {
            return httpRequest(this._backend, new static_request_1.Request(mergeOptions(this._defaultOptions, options, enums_1.RequestMethod.Head, url)));
        };
        /** @nocollapse */
        Http.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        Http.ctorParameters = [{ type: interfaces_1.ConnectionBackend }, { type: base_request_options_1.RequestOptions }];
        return Http;
    }();
    exports.Http = Http;
    var Jsonp = function (_super) {
        __extends(Jsonp, _super);
        function Jsonp(backend, defaultOptions) {
            _super.call(this, backend, defaultOptions);
        }
        /**
         * Performs any type of http request. First argument is required, and can either be a url or
         * a {@link Request} instance. If the first argument is a url, an optional {@link RequestOptions}
         * object can be provided as the 2nd argument. The options object will be merged with the values
         * of {@link BaseRequestOptions} before performing the request.
         *
         * @security Regular XHR is the safest alternative to JSONP for most applications, and is
         * supported by all current browsers. Because JSONP creates a `<script>` element with
         * contents retrieved from a remote source, attacker-controlled data introduced by an untrusted
         * source could expose your application to XSS risks. Data exposed by JSONP may also be
         * readable by malicious third-party websites. In addition, JSONP introduces potential risk for
         * future security issues (e.g. content sniffing).  For more detail, see the
         * [Security Guide](http://g.co/ng/security).
         */
        Jsonp.prototype.request = function (url, options) {
            var responseObservable;
            if (lang_1.isString(url)) {
                url = new static_request_1.Request(mergeOptions(this._defaultOptions, options, enums_1.RequestMethod.Get, url));
            }
            if (url instanceof static_request_1.Request) {
                if (url.method !== enums_1.RequestMethod.Get) {
                    exceptions_1.makeTypeError('JSONP requests must use GET request method.');
                }
                responseObservable = httpRequest(this._backend, url);
            } else {
                throw exceptions_1.makeTypeError('First argument must be a url string or Request instance.');
            }
            return responseObservable;
        };
        /** @nocollapse */
        Jsonp.decorators = [{ type: core_1.Injectable }];
        /** @nocollapse */
        Jsonp.ctorParameters = [{ type: interfaces_1.ConnectionBackend }, { type: base_request_options_1.RequestOptions }];
        return Jsonp;
    }(Http);
    exports.Jsonp = Jsonp;
    
});
System.registerDynamic("node_modules/@angular/http/src/interfaces.js", [], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * Abstract class from which real backends are derived.
   *
   * The primary purpose of a `ConnectionBackend` is to create new connections to fulfill a given
   * {@link Request}.
   *
   * @experimental
   */

  var global = this || self,
      GLOBAL = global;
  var ConnectionBackend = function () {
    function ConnectionBackend() {}
    return ConnectionBackend;
  }();
  exports.ConnectionBackend = ConnectionBackend;
  /**
   * Abstract class from which real connections are derived.
   *
   * @experimental
   */
  var Connection = function () {
    function Connection() {}
    return Connection;
  }();
  exports.Connection = Connection;
  /**
   * An XSRFStrategy configures XSRF protection (e.g. via headers) on an HTTP request.
   *
   * @experimental
   */
  var XSRFStrategy = function () {
    function XSRFStrategy() {}
    return XSRFStrategy;
  }();
  exports.XSRFStrategy = XSRFStrategy;
  
});
System.registerDynamic('node_modules/@angular/http/src/headers.js', ['../src/facade/exceptions', '../src/facade/lang', '../src/facade/collection'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var exceptions_1 = $__require('../src/facade/exceptions');
    var lang_1 = $__require('../src/facade/lang');
    var collection_1 = $__require('../src/facade/collection');
    /**
     * Polyfill for [Headers](https://developer.mozilla.org/en-US/docs/Web/API/Headers/Headers), as
     * specified in the [Fetch Spec](https://fetch.spec.whatwg.org/#headers-class).
     *
     * The only known difference between this `Headers` implementation and the spec is the
     * lack of an `entries` method.
     *
     * ### Example ([live demo](http://plnkr.co/edit/MTdwT6?p=preview))
     *
     * ```
     * import {Headers} from '@angular/http';
     *
     * var firstHeaders = new Headers();
     * firstHeaders.append('Content-Type', 'image/jpeg');
     * console.log(firstHeaders.get('Content-Type')) //'image/jpeg'
     *
     * // Create headers from Plain Old JavaScript Object
     * var secondHeaders = new Headers({
     *   'X-My-Custom-Header': 'Angular'
     * });
     * console.log(secondHeaders.get('X-My-Custom-Header')); //'Angular'
     *
     * var thirdHeaders = new Headers(secondHeaders);
     * console.log(thirdHeaders.get('X-My-Custom-Header')); //'Angular'
     * ```
     *
     * @experimental
     */
    var Headers = function () {
        function Headers(headers) {
            var _this = this;
            if (headers instanceof Headers) {
                this._headersMap = headers._headersMap;
                return;
            }
            this._headersMap = new collection_1.Map();
            if (lang_1.isBlank(headers)) {
                return;
            }
            // headers instanceof StringMap
            collection_1.StringMapWrapper.forEach(headers, function (v, k) {
                _this._headersMap.set(k, collection_1.isListLikeIterable(v) ? v : [v]);
            });
        }
        /**
         * Returns a new Headers instance from the given DOMString of Response Headers
         */
        Headers.fromResponseHeaderString = function (headersString) {
            return headersString.trim().split('\n').map(function (val) {
                return val.split(':');
            }).map(function (_a) {
                var key = _a[0],
                    parts = _a.slice(1);
                return [key.trim(), parts.join(':').trim()];
            }).reduce(function (headers, _a) {
                var key = _a[0],
                    value = _a[1];
                return !headers.set(key, value) && headers;
            }, new Headers());
        };
        /**
         * Appends a header to existing list of header values for a given header name.
         */
        Headers.prototype.append = function (name, value) {
            var mapName = this._headersMap.get(name);
            var list = collection_1.isListLikeIterable(mapName) ? mapName : [];
            list.push(value);
            this._headersMap.set(name, list);
        };
        /**
         * Deletes all header values for the given name.
         */
        Headers.prototype.delete = function (name) {
            this._headersMap.delete(name);
        };
        Headers.prototype.forEach = function (fn) {
            this._headersMap.forEach(fn);
        };
        /**
         * Returns first header that matches given name.
         */
        Headers.prototype.get = function (header) {
            return collection_1.ListWrapper.first(this._headersMap.get(header));
        };
        /**
         * Check for existence of header by given name.
         */
        Headers.prototype.has = function (header) {
            return this._headersMap.has(header);
        };
        /**
         * Provides names of set headers
         */
        Headers.prototype.keys = function () {
            return collection_1.MapWrapper.keys(this._headersMap);
        };
        /**
         * Sets or overrides header value for given name.
         */
        Headers.prototype.set = function (header, value) {
            var list = [];
            if (collection_1.isListLikeIterable(value)) {
                var pushValue = value.join(',');
                list.push(pushValue);
            } else {
                list.push(value);
            }
            this._headersMap.set(header, list);
        };
        /**
         * Returns values of all headers.
         */
        Headers.prototype.values = function () {
            return collection_1.MapWrapper.values(this._headersMap);
        };
        /**
         * Returns string of all headers.
         */
        Headers.prototype.toJSON = function () {
            var serializableHeaders = {};
            this._headersMap.forEach(function (values, name) {
                var list = [];
                collection_1.iterateListLike(values, function (val /** TODO #9100 */) {
                    return list = collection_1.ListWrapper.concat(list, val.split(','));
                });
                serializableHeaders[name] = list;
            });
            return serializableHeaders;
        };
        /**
         * Returns list of header values for a given name.
         */
        Headers.prototype.getAll = function (header) {
            var headers = this._headersMap.get(header);
            return collection_1.isListLikeIterable(headers) ? headers : [];
        };
        /**
         * This method is not implemented.
         */
        Headers.prototype.entries = function () {
            throw new exceptions_1.BaseException('"entries" method is not implemented on Headers class');
        };
        return Headers;
    }();
    exports.Headers = Headers;
    
});
System.registerDynamic('node_modules/@angular/http/src/static_request.js', ['../src/facade/lang', './enums', './headers', './http_utils', './url_search_params'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('../src/facade/lang');
    var enums_1 = $__require('./enums');
    var headers_1 = $__require('./headers');
    var http_utils_1 = $__require('./http_utils');
    var url_search_params_1 = $__require('./url_search_params');
    // TODO(jeffbcross): properly implement body accessors
    /**
     * Creates `Request` instances from provided values.
     *
     * The Request's interface is inspired by the Request constructor defined in the [Fetch
     * Spec](https://fetch.spec.whatwg.org/#request-class),
     * but is considered a static value whose body can be accessed many times. There are other
     * differences in the implementation, but this is the most significant.
     *
     * `Request` instances are typically created by higher-level classes, like {@link Http} and
     * {@link Jsonp}, but it may occasionally be useful to explicitly create `Request` instances.
     * One such example is when creating services that wrap higher-level services, like {@link Http},
     * where it may be useful to generate a `Request` with arbitrary headers and search params.
     *
     * ```typescript
     * import {Injectable, Injector} from '@angular/core';
     * import {HTTP_PROVIDERS, Http, Request, RequestMethod} from '@angular/http';
     *
     * @Injectable()
     * class AutoAuthenticator {
     *   constructor(public http:Http) {}
     *   request(url:string) {
     *     return this.http.request(new Request({
     *       method: RequestMethod.Get,
     *       url: url,
     *       search: 'password=123'
     *     }));
     *   }
     * }
     *
     * var injector = Injector.resolveAndCreate([HTTP_PROVIDERS, AutoAuthenticator]);
     * var authenticator = injector.get(AutoAuthenticator);
     * authenticator.request('people.json').subscribe(res => {
     *   //URL should have included '?password=123'
     *   console.log('people', res.json());
     * });
     * ```
     *
     * @experimental
     */
    var Request = function () {
        function Request(requestOptions) {
            // TODO: assert that url is present
            var url = requestOptions.url;
            this.url = requestOptions.url;
            if (lang_1.isPresent(requestOptions.search)) {
                var search = requestOptions.search.toString();
                if (search.length > 0) {
                    var prefix = '?';
                    if (lang_1.StringWrapper.contains(this.url, '?')) {
                        prefix = this.url[this.url.length - 1] == '&' ? '' : '&';
                    }
                    // TODO: just delete search-query-looking string in url?
                    this.url = url + prefix + search;
                }
            }
            this._body = requestOptions.body;
            this.contentType = this.detectContentType();
            this.method = http_utils_1.normalizeMethodName(requestOptions.method);
            // TODO(jeffbcross): implement behavior
            // Defaults to 'omit', consistent with browser
            // TODO(jeffbcross): implement behavior
            this.headers = new headers_1.Headers(requestOptions.headers);
            this.withCredentials = requestOptions.withCredentials;
        }
        /**
         * Returns the request's body as string, assuming that body exists. If body is undefined, return
         * empty
         * string.
         */
        Request.prototype.text = function () {
            return lang_1.isPresent(this._body) ? this._body.toString() : '';
        };
        /**
         * Returns the request's body as JSON string, assuming that body exists. If body is undefined,
         * return
         * empty
         * string.
         */
        Request.prototype.json = function () {
            return lang_1.isPresent(this._body) ? JSON.stringify(this._body) : '';
        };
        /**
         * Returns the request's body as array buffer, assuming that body exists. If body is undefined,
         * return
         * null.
         */
        Request.prototype.arrayBuffer = function () {
            if (this._body instanceof ArrayBuffer) return this._body;
            throw 'The request body isn\'t an array buffer';
        };
        /**
         * Returns the request's body as blob, assuming that body exists. If body is undefined, return
         * null.
         */
        Request.prototype.blob = function () {
            if (this._body instanceof Blob) return this._body;
            if (this._body instanceof ArrayBuffer) return new Blob([this._body]);
            throw 'The request body isn\'t either a blob or an array buffer';
        };
        /**
         * Returns the content type of request's body based on its type.
         */
        Request.prototype.detectContentType = function () {
            if (this._body == null) {
                return enums_1.ContentType.NONE;
            } else if (this._body instanceof url_search_params_1.URLSearchParams) {
                return enums_1.ContentType.FORM;
            } else if (this._body instanceof FormData) {
                return enums_1.ContentType.FORM_DATA;
            } else if (this._body instanceof Blob) {
                return enums_1.ContentType.BLOB;
            } else if (this._body instanceof ArrayBuffer) {
                return enums_1.ContentType.ARRAY_BUFFER;
            } else if (this._body && typeof this._body == 'object') {
                return enums_1.ContentType.JSON;
            } else {
                return enums_1.ContentType.TEXT;
            }
        };
        /**
         * Returns the request's body according to its type. If body is undefined, return
         * null.
         */
        Request.prototype.getBody = function () {
            switch (this.contentType) {
                case enums_1.ContentType.JSON:
                    return this.json();
                case enums_1.ContentType.FORM:
                    return this.text();
                case enums_1.ContentType.FORM_DATA:
                    return this._body;
                case enums_1.ContentType.TEXT:
                    return this.text();
                case enums_1.ContentType.BLOB:
                    return this.blob();
                case enums_1.ContentType.ARRAY_BUFFER:
                    return this.arrayBuffer();
                default:
                    return null;
            }
        };
        return Request;
    }();
    exports.Request = Request;
    var noop = function () {};
    var w = typeof window == 'object' ? window : noop;
    var FormData = w['FormData'] || noop;
    var Blob = w['Blob'] || noop;
    var ArrayBuffer = w['ArrayBuffer'] || noop;
    
});
System.registerDynamic("node_modules/@angular/http/src/facade/base_wrapped_exception.js", [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    /**
     * A base class for the WrappedException that can be used to identify
     * a WrappedException from ExceptionHandler without adding circular
     * dependency.
     */
    var BaseWrappedException = function (_super) {
        __extends(BaseWrappedException, _super);
        function BaseWrappedException(message) {
            _super.call(this, message);
        }
        Object.defineProperty(BaseWrappedException.prototype, "wrapperMessage", {
            get: function () {
                return '';
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "wrapperStack", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "originalException", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "originalStack", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "context", {
            get: function () {
                return null;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(BaseWrappedException.prototype, "message", {
            get: function () {
                return '';
            },
            enumerable: true,
            configurable: true
        });
        return BaseWrappedException;
    }(Error);
    exports.BaseWrappedException = BaseWrappedException;
    
});
System.registerDynamic('node_modules/@angular/http/src/facade/exception_handler.js', ['./base_wrapped_exception', './collection', './lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var base_wrapped_exception_1 = $__require('./base_wrapped_exception');
    var collection_1 = $__require('./collection');
    var lang_1 = $__require('./lang');
    var _ArrayLogger = function () {
        function _ArrayLogger() {
            this.res = [];
        }
        _ArrayLogger.prototype.log = function (s) {
            this.res.push(s);
        };
        _ArrayLogger.prototype.logError = function (s) {
            this.res.push(s);
        };
        _ArrayLogger.prototype.logGroup = function (s) {
            this.res.push(s);
        };
        _ArrayLogger.prototype.logGroupEnd = function () {};
        ;
        return _ArrayLogger;
    }();
    /**
     * Provides a hook for centralized exception handling.
     *
     * The default implementation of `ExceptionHandler` prints error messages to the `Console`. To
     * intercept error handling,
     * write a custom exception handler that replaces this default as appropriate for your app.
     *
     * ### Example
     *
     * ```javascript
     *
     * class MyExceptionHandler implements ExceptionHandler {
     *   call(error, stackTrace = null, reason = null) {
     *     // do something with the exception
     *   }
     * }
     *
     * bootstrap(MyApp, {provide: ExceptionHandler, useClass: MyExceptionHandler}])
     *
     * ```
     * @stable
     */
    var ExceptionHandler = function () {
        function ExceptionHandler(_logger, _rethrowException) {
            if (_rethrowException === void 0) {
                _rethrowException = true;
            }
            this._logger = _logger;
            this._rethrowException = _rethrowException;
        }
        ExceptionHandler.exceptionToString = function (exception, stackTrace, reason) {
            if (stackTrace === void 0) {
                stackTrace = null;
            }
            if (reason === void 0) {
                reason = null;
            }
            var l = new _ArrayLogger();
            var e = new ExceptionHandler(l, false);
            e.call(exception, stackTrace, reason);
            return l.res.join('\n');
        };
        ExceptionHandler.prototype.call = function (exception, stackTrace, reason) {
            if (stackTrace === void 0) {
                stackTrace = null;
            }
            if (reason === void 0) {
                reason = null;
            }
            var originalException = this._findOriginalException(exception);
            var originalStack = this._findOriginalStack(exception);
            var context = this._findContext(exception);
            this._logger.logGroup("EXCEPTION: " + this._extractMessage(exception));
            if (lang_1.isPresent(stackTrace) && lang_1.isBlank(originalStack)) {
                this._logger.logError('STACKTRACE:');
                this._logger.logError(this._longStackTrace(stackTrace));
            }
            if (lang_1.isPresent(reason)) {
                this._logger.logError("REASON: " + reason);
            }
            if (lang_1.isPresent(originalException)) {
                this._logger.logError("ORIGINAL EXCEPTION: " + this._extractMessage(originalException));
            }
            if (lang_1.isPresent(originalStack)) {
                this._logger.logError('ORIGINAL STACKTRACE:');
                this._logger.logError(this._longStackTrace(originalStack));
            }
            if (lang_1.isPresent(context)) {
                this._logger.logError('ERROR CONTEXT:');
                this._logger.logError(context);
            }
            this._logger.logGroupEnd();
            // We rethrow exceptions, so operations like 'bootstrap' will result in an error
            // when an exception happens. If we do not rethrow, bootstrap will always succeed.
            if (this._rethrowException) throw exception;
        };
        /** @internal */
        ExceptionHandler.prototype._extractMessage = function (exception) {
            return exception instanceof base_wrapped_exception_1.BaseWrappedException ? exception.wrapperMessage : exception.toString();
        };
        /** @internal */
        ExceptionHandler.prototype._longStackTrace = function (stackTrace) {
            return collection_1.isListLikeIterable(stackTrace) ? stackTrace.join('\n\n-----async gap-----\n') : stackTrace.toString();
        };
        /** @internal */
        ExceptionHandler.prototype._findContext = function (exception) {
            try {
                if (!(exception instanceof base_wrapped_exception_1.BaseWrappedException)) return null;
                return lang_1.isPresent(exception.context) ? exception.context : this._findContext(exception.originalException);
            } catch (e) {
                // exception.context can throw an exception. if it happens, we ignore the context.
                return null;
            }
        };
        /** @internal */
        ExceptionHandler.prototype._findOriginalException = function (exception) {
            if (!(exception instanceof base_wrapped_exception_1.BaseWrappedException)) return null;
            var e = exception.originalException;
            while (e instanceof base_wrapped_exception_1.BaseWrappedException && lang_1.isPresent(e.originalException)) {
                e = e.originalException;
            }
            return e;
        };
        /** @internal */
        ExceptionHandler.prototype._findOriginalStack = function (exception) {
            if (!(exception instanceof base_wrapped_exception_1.BaseWrappedException)) return null;
            var e = exception;
            var stack = exception.originalStack;
            while (e instanceof base_wrapped_exception_1.BaseWrappedException && lang_1.isPresent(e.originalException)) {
                e = e.originalException;
                if (e instanceof base_wrapped_exception_1.BaseWrappedException && lang_1.isPresent(e.originalException)) {
                    stack = e.originalStack;
                }
            }
            return stack;
        };
        return ExceptionHandler;
    }();
    exports.ExceptionHandler = ExceptionHandler;
    
});
System.registerDynamic('node_modules/@angular/http/src/facade/exceptions.js', ['./base_wrapped_exception', './exception_handler'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var base_wrapped_exception_1 = $__require('./base_wrapped_exception');
    var exception_handler_1 = $__require('./exception_handler');
    var exception_handler_2 = $__require('./exception_handler');
    exports.ExceptionHandler = exception_handler_2.ExceptionHandler;
    /**
     * @stable
     */
    var BaseException = function (_super) {
        __extends(BaseException, _super);
        function BaseException(message) {
            if (message === void 0) {
                message = '--';
            }
            _super.call(this, message);
            this.message = message;
            this.stack = new Error(message).stack;
        }
        BaseException.prototype.toString = function () {
            return this.message;
        };
        return BaseException;
    }(Error);
    exports.BaseException = BaseException;
    /**
     * Wraps an exception and provides additional context or information.
     * @stable
     */
    var WrappedException = function (_super) {
        __extends(WrappedException, _super);
        function WrappedException(_wrapperMessage, _originalException /** TODO #9100 */, _originalStack /** TODO #9100 */, _context /** TODO #9100 */) {
            _super.call(this, _wrapperMessage);
            this._wrapperMessage = _wrapperMessage;
            this._originalException = _originalException;
            this._originalStack = _originalStack;
            this._context = _context;
            this._wrapperStack = new Error(_wrapperMessage).stack;
        }
        Object.defineProperty(WrappedException.prototype, "wrapperMessage", {
            get: function () {
                return this._wrapperMessage;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "wrapperStack", {
            get: function () {
                return this._wrapperStack;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "originalException", {
            get: function () {
                return this._originalException;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "originalStack", {
            get: function () {
                return this._originalStack;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "context", {
            get: function () {
                return this._context;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(WrappedException.prototype, "message", {
            get: function () {
                return exception_handler_1.ExceptionHandler.exceptionToString(this);
            },
            enumerable: true,
            configurable: true
        });
        WrappedException.prototype.toString = function () {
            return this.message;
        };
        return WrappedException;
    }(base_wrapped_exception_1.BaseWrappedException);
    exports.WrappedException = WrappedException;
    function makeTypeError(message) {
        return new TypeError(message);
    }
    exports.makeTypeError = makeTypeError;
    function unimplemented() {
        throw new BaseException('unimplemented');
    }
    exports.unimplemented = unimplemented;
    
});
System.registerDynamic("node_modules/@angular/http/src/enums.js", [], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";
  /**
   * Supported http methods.
   * @experimental
   */

  var global = this || self,
      GLOBAL = global;
  (function (RequestMethod) {
    RequestMethod[RequestMethod["Get"] = 0] = "Get";
    RequestMethod[RequestMethod["Post"] = 1] = "Post";
    RequestMethod[RequestMethod["Put"] = 2] = "Put";
    RequestMethod[RequestMethod["Delete"] = 3] = "Delete";
    RequestMethod[RequestMethod["Options"] = 4] = "Options";
    RequestMethod[RequestMethod["Head"] = 5] = "Head";
    RequestMethod[RequestMethod["Patch"] = 6] = "Patch";
  })(exports.RequestMethod || (exports.RequestMethod = {}));
  var RequestMethod = exports.RequestMethod;
  /**
   * All possible states in which a connection can be, based on
   * [States](http://www.w3.org/TR/XMLHttpRequest/#states) from the `XMLHttpRequest` spec, but with an
   * additional "CANCELLED" state.
   * @experimental
   */
  (function (ReadyState) {
    ReadyState[ReadyState["Unsent"] = 0] = "Unsent";
    ReadyState[ReadyState["Open"] = 1] = "Open";
    ReadyState[ReadyState["HeadersReceived"] = 2] = "HeadersReceived";
    ReadyState[ReadyState["Loading"] = 3] = "Loading";
    ReadyState[ReadyState["Done"] = 4] = "Done";
    ReadyState[ReadyState["Cancelled"] = 5] = "Cancelled";
  })(exports.ReadyState || (exports.ReadyState = {}));
  var ReadyState = exports.ReadyState;
  /**
   * Acceptable response types to be associated with a {@link Response}, based on
   * [ResponseType](https://fetch.spec.whatwg.org/#responsetype) from the Fetch spec.
   * @experimental
   */
  (function (ResponseType) {
    ResponseType[ResponseType["Basic"] = 0] = "Basic";
    ResponseType[ResponseType["Cors"] = 1] = "Cors";
    ResponseType[ResponseType["Default"] = 2] = "Default";
    ResponseType[ResponseType["Error"] = 3] = "Error";
    ResponseType[ResponseType["Opaque"] = 4] = "Opaque";
  })(exports.ResponseType || (exports.ResponseType = {}));
  var ResponseType = exports.ResponseType;
  /**
   * Supported content type to be automatically associated with a {@link Request}.
   * @experimental
   */
  (function (ContentType) {
    ContentType[ContentType["NONE"] = 0] = "NONE";
    ContentType[ContentType["JSON"] = 1] = "JSON";
    ContentType[ContentType["FORM"] = 2] = "FORM";
    ContentType[ContentType["FORM_DATA"] = 3] = "FORM_DATA";
    ContentType[ContentType["TEXT"] = 4] = "TEXT";
    ContentType[ContentType["BLOB"] = 5] = "BLOB";
    ContentType[ContentType["ARRAY_BUFFER"] = 6] = "ARRAY_BUFFER";
  })(exports.ContentType || (exports.ContentType = {}));
  var ContentType = exports.ContentType;
  
});
System.registerDynamic('node_modules/@angular/http/src/http_utils.js', ['../src/facade/exceptions', '../src/facade/lang', './enums'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var exceptions_1 = $__require('../src/facade/exceptions');
    var lang_1 = $__require('../src/facade/lang');
    var enums_1 = $__require('./enums');
    function normalizeMethodName(method) {
        if (lang_1.isString(method)) {
            var originalMethod = method;
            method = method.replace(/(\w)(\w*)/g, function (g0, g1, g2) {
                return g1.toUpperCase() + g2.toLowerCase();
            });
            method = enums_1.RequestMethod[method];
            if (typeof method !== 'number') throw exceptions_1.makeTypeError("Invalid request method. The method \"" + originalMethod + "\" is not supported.");
        }
        return method;
    }
    exports.normalizeMethodName = normalizeMethodName;
    exports.isSuccess = function (status) {
        return status >= 200 && status < 300;
    };
    function getResponseURL(xhr) {
        if ('responseURL' in xhr) {
            return xhr.responseURL;
        }
        if (/^X-Request-URL:/m.test(xhr.getAllResponseHeaders())) {
            return xhr.getResponseHeader('X-Request-URL');
        }
        return;
    }
    exports.getResponseURL = getResponseURL;
    var lang_2 = $__require('../src/facade/lang');
    exports.isJsObject = lang_2.isJsObject;
    
});
System.registerDynamic('node_modules/@angular/http/src/static_response.js', ['../src/facade/exceptions', '../src/facade/lang', './http_utils'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var exceptions_1 = $__require('../src/facade/exceptions');
    var lang_1 = $__require('../src/facade/lang');
    var http_utils_1 = $__require('./http_utils');
    /**
     * Creates `Response` instances from provided values.
     *
     * Though this object isn't
     * usually instantiated by end-users, it is the primary object interacted with when it comes time to
     * add data to a view.
     *
     * ### Example
     *
     * ```
     * http.request('my-friends.txt').subscribe(response => this.friends = response.text());
     * ```
     *
     * The Response's interface is inspired by the Response constructor defined in the [Fetch
     * Spec](https://fetch.spec.whatwg.org/#response-class), but is considered a static value whose body
     * can be accessed many times. There are other differences in the implementation, but this is the
     * most significant.
     *
     * @experimental
     */
    var Response = function () {
        function Response(responseOptions) {
            this._body = responseOptions.body;
            this.status = responseOptions.status;
            this.ok = this.status >= 200 && this.status <= 299;
            this.statusText = responseOptions.statusText;
            this.headers = responseOptions.headers;
            this.type = responseOptions.type;
            this.url = responseOptions.url;
        }
        /**
         * Not yet implemented
         */
        // TODO: Blob return type
        Response.prototype.blob = function () {
            throw new exceptions_1.BaseException('"blob()" method not implemented on Response superclass');
        };
        /**
         * Attempts to return body as parsed `JSON` object, or raises an exception.
         */
        Response.prototype.json = function () {
            var jsonResponse;
            if (http_utils_1.isJsObject(this._body)) {
                jsonResponse = this._body;
            } else if (lang_1.isString(this._body)) {
                jsonResponse = lang_1.Json.parse(this._body);
            }
            return jsonResponse;
        };
        /**
         * Returns the body as a string, presuming `toString()` can be called on the response body.
         */
        Response.prototype.text = function () {
            return this._body.toString();
        };
        /**
         * Not yet implemented
         */
        // TODO: ArrayBuffer return type
        Response.prototype.arrayBuffer = function () {
            throw new exceptions_1.BaseException('"arrayBuffer()" method not implemented on Response superclass');
        };
        Response.prototype.toString = function () {
            return "Response with status: " + this.status + " " + this.statusText + " for URL: " + this.url;
        };
        return Response;
    }();
    exports.Response = Response;
    
});
System.registerDynamic("node_modules/@angular/http/src/facade/collection.js", ["./lang"], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var lang_1 = $__require('./lang');
    exports.Map = lang_1.global.Map;
    exports.Set = lang_1.global.Set;
    // Safari and Internet Explorer do not support the iterable parameter to the
    // Map constructor.  We work around that by manually adding the items.
    var createMapFromPairs = function () {
        try {
            if (new exports.Map([[1, 2]]).size === 1) {
                return function createMapFromPairs(pairs) {
                    return new exports.Map(pairs);
                };
            }
        } catch (e) {}
        return function createMapAndPopulateFromPairs(pairs) {
            var map = new exports.Map();
            for (var i = 0; i < pairs.length; i++) {
                var pair = pairs[i];
                map.set(pair[0], pair[1]);
            }
            return map;
        };
    }();
    var createMapFromMap = function () {
        try {
            if (new exports.Map(new exports.Map())) {
                return function createMapFromMap(m) {
                    return new exports.Map(m);
                };
            }
        } catch (e) {}
        return function createMapAndPopulateFromMap(m) {
            var map = new exports.Map();
            m.forEach(function (v, k) {
                map.set(k, v);
            });
            return map;
        };
    }();
    var _clearValues = function () {
        if (new exports.Map().keys().next) {
            return function _clearValues(m) {
                var keyIterator = m.keys();
                var k;
                while (!(k = keyIterator.next()).done) {
                    m.set(k.value, null);
                }
            };
        } else {
            return function _clearValuesWithForeEach(m) {
                m.forEach(function (v, k) {
                    m.set(k, null);
                });
            };
        }
    }();
    // Safari doesn't implement MapIterator.next(), which is used is Traceur's polyfill of Array.from
    // TODO(mlaval): remove the work around once we have a working polyfill of Array.from
    var _arrayFromMap = function () {
        try {
            if (new exports.Map().values().next) {
                return function createArrayFromMap(m, getValues) {
                    return getValues ? Array.from(m.values()) : Array.from(m.keys());
                };
            }
        } catch (e) {}
        return function createArrayFromMapWithForeach(m, getValues) {
            var res = ListWrapper.createFixedSize(m.size),
                i = 0;
            m.forEach(function (v, k) {
                res[i] = getValues ? v : k;
                i++;
            });
            return res;
        };
    }();
    var MapWrapper = function () {
        function MapWrapper() {}
        MapWrapper.clone = function (m) {
            return createMapFromMap(m);
        };
        MapWrapper.createFromStringMap = function (stringMap) {
            var result = new exports.Map();
            for (var prop in stringMap) {
                result.set(prop, stringMap[prop]);
            }
            return result;
        };
        MapWrapper.toStringMap = function (m) {
            var r = {};
            m.forEach(function (v, k) {
                return r[k] = v;
            });
            return r;
        };
        MapWrapper.createFromPairs = function (pairs) {
            return createMapFromPairs(pairs);
        };
        MapWrapper.clearValues = function (m) {
            _clearValues(m);
        };
        MapWrapper.iterable = function (m) {
            return m;
        };
        MapWrapper.keys = function (m) {
            return _arrayFromMap(m, false);
        };
        MapWrapper.values = function (m) {
            return _arrayFromMap(m, true);
        };
        return MapWrapper;
    }();
    exports.MapWrapper = MapWrapper;
    /**
     * Wraps Javascript Objects
     */
    var StringMapWrapper = function () {
        function StringMapWrapper() {}
        StringMapWrapper.create = function () {
            // Note: We are not using Object.create(null) here due to
            // performance!
            // http://jsperf.com/ng2-object-create-null
            return {};
        };
        StringMapWrapper.contains = function (map, key) {
            return map.hasOwnProperty(key);
        };
        StringMapWrapper.get = function (map, key) {
            return map.hasOwnProperty(key) ? map[key] : undefined;
        };
        StringMapWrapper.set = function (map, key, value) {
            map[key] = value;
        };
        StringMapWrapper.keys = function (map) {
            return Object.keys(map);
        };
        StringMapWrapper.values = function (map) {
            return Object.keys(map).reduce(function (r, a) {
                r.push(map[a]);
                return r;
            }, []);
        };
        StringMapWrapper.isEmpty = function (map) {
            for (var prop in map) {
                return false;
            }
            return true;
        };
        StringMapWrapper.delete = function (map, key) {
            delete map[key];
        };
        StringMapWrapper.forEach = function (map, callback) {
            for (var prop in map) {
                if (map.hasOwnProperty(prop)) {
                    callback(map[prop], prop);
                }
            }
        };
        StringMapWrapper.merge = function (m1, m2) {
            var m = {};
            for (var attr in m1) {
                if (m1.hasOwnProperty(attr)) {
                    m[attr] = m1[attr];
                }
            }
            for (var attr in m2) {
                if (m2.hasOwnProperty(attr)) {
                    m[attr] = m2[attr];
                }
            }
            return m;
        };
        StringMapWrapper.equals = function (m1, m2) {
            var k1 = Object.keys(m1);
            var k2 = Object.keys(m2);
            if (k1.length != k2.length) {
                return false;
            }
            var key;
            for (var i = 0; i < k1.length; i++) {
                key = k1[i];
                if (m1[key] !== m2[key]) {
                    return false;
                }
            }
            return true;
        };
        return StringMapWrapper;
    }();
    exports.StringMapWrapper = StringMapWrapper;
    var ListWrapper = function () {
        function ListWrapper() {}
        // JS has no way to express a statically fixed size list, but dart does so we
        // keep both methods.
        ListWrapper.createFixedSize = function (size) {
            return new Array(size);
        };
        ListWrapper.createGrowableSize = function (size) {
            return new Array(size);
        };
        ListWrapper.clone = function (array) {
            return array.slice(0);
        };
        ListWrapper.forEachWithIndex = function (array, fn) {
            for (var i = 0; i < array.length; i++) {
                fn(array[i], i);
            }
        };
        ListWrapper.first = function (array) {
            if (!array) return null;
            return array[0];
        };
        ListWrapper.last = function (array) {
            if (!array || array.length == 0) return null;
            return array[array.length - 1];
        };
        ListWrapper.indexOf = function (array, value, startIndex) {
            if (startIndex === void 0) {
                startIndex = 0;
            }
            return array.indexOf(value, startIndex);
        };
        ListWrapper.contains = function (list, el) {
            return list.indexOf(el) !== -1;
        };
        ListWrapper.reversed = function (array) {
            var a = ListWrapper.clone(array);
            return a.reverse();
        };
        ListWrapper.concat = function (a, b) {
            return a.concat(b);
        };
        ListWrapper.insert = function (list, index, value) {
            list.splice(index, 0, value);
        };
        ListWrapper.removeAt = function (list, index) {
            var res = list[index];
            list.splice(index, 1);
            return res;
        };
        ListWrapper.removeAll = function (list, items) {
            for (var i = 0; i < items.length; ++i) {
                var index = list.indexOf(items[i]);
                list.splice(index, 1);
            }
        };
        ListWrapper.remove = function (list, el) {
            var index = list.indexOf(el);
            if (index > -1) {
                list.splice(index, 1);
                return true;
            }
            return false;
        };
        ListWrapper.clear = function (list) {
            list.length = 0;
        };
        ListWrapper.isEmpty = function (list) {
            return list.length == 0;
        };
        ListWrapper.fill = function (list, value, start, end) {
            if (start === void 0) {
                start = 0;
            }
            if (end === void 0) {
                end = null;
            }
            list.fill(value, start, end === null ? list.length : end);
        };
        ListWrapper.equals = function (a, b) {
            if (a.length != b.length) return false;
            for (var i = 0; i < a.length; ++i) {
                if (a[i] !== b[i]) return false;
            }
            return true;
        };
        ListWrapper.slice = function (l, from, to) {
            if (from === void 0) {
                from = 0;
            }
            if (to === void 0) {
                to = null;
            }
            return l.slice(from, to === null ? undefined : to);
        };
        ListWrapper.splice = function (l, from, length) {
            return l.splice(from, length);
        };
        ListWrapper.sort = function (l, compareFn) {
            if (lang_1.isPresent(compareFn)) {
                l.sort(compareFn);
            } else {
                l.sort();
            }
        };
        ListWrapper.toString = function (l) {
            return l.toString();
        };
        ListWrapper.toJSON = function (l) {
            return JSON.stringify(l);
        };
        ListWrapper.maximum = function (list, predicate) {
            if (list.length == 0) {
                return null;
            }
            var solution = null;
            var maxValue = -Infinity;
            for (var index = 0; index < list.length; index++) {
                var candidate = list[index];
                if (lang_1.isBlank(candidate)) {
                    continue;
                }
                var candidateValue = predicate(candidate);
                if (candidateValue > maxValue) {
                    solution = candidate;
                    maxValue = candidateValue;
                }
            }
            return solution;
        };
        ListWrapper.flatten = function (list) {
            var target = [];
            _flattenArray(list, target);
            return target;
        };
        ListWrapper.addAll = function (list, source) {
            for (var i = 0; i < source.length; i++) {
                list.push(source[i]);
            }
        };
        return ListWrapper;
    }();
    exports.ListWrapper = ListWrapper;
    function _flattenArray(source, target) {
        if (lang_1.isPresent(source)) {
            for (var i = 0; i < source.length; i++) {
                var item = source[i];
                if (lang_1.isArray(item)) {
                    _flattenArray(item, target);
                } else {
                    target.push(item);
                }
            }
        }
        return target;
    }
    function isListLikeIterable(obj) {
        if (!lang_1.isJsObject(obj)) return false;
        return lang_1.isArray(obj) || !(obj instanceof exports.Map) && lang_1.getSymbolIterator() in obj; // JS Iterable have a Symbol.iterator prop
    }
    exports.isListLikeIterable = isListLikeIterable;
    function areIterablesEqual(a, b, comparator) {
        var iterator1 = a[lang_1.getSymbolIterator()]();
        var iterator2 = b[lang_1.getSymbolIterator()]();
        while (true) {
            var item1 = iterator1.next();
            var item2 = iterator2.next();
            if (item1.done && item2.done) return true;
            if (item1.done || item2.done) return false;
            if (!comparator(item1.value, item2.value)) return false;
        }
    }
    exports.areIterablesEqual = areIterablesEqual;
    function iterateListLike(obj, fn) {
        if (lang_1.isArray(obj)) {
            for (var i = 0; i < obj.length; i++) {
                fn(obj[i]);
            }
        } else {
            var iterator = obj[lang_1.getSymbolIterator()]();
            var item;
            while (!(item = iterator.next()).done) {
                fn(item.value);
            }
        }
    }
    exports.iterateListLike = iterateListLike;
    // Safari and Internet Explorer do not support the iterable parameter to the
    // Set constructor.  We work around that by manually adding the items.
    var createSetFromList = function () {
        var test = new exports.Set([1, 2, 3]);
        if (test.size === 3) {
            return function createSetFromList(lst) {
                return new exports.Set(lst);
            };
        } else {
            return function createSetAndPopulateFromList(lst) {
                var res = new exports.Set(lst);
                if (res.size !== lst.length) {
                    for (var i = 0; i < lst.length; i++) {
                        res.add(lst[i]);
                    }
                }
                return res;
            };
        }
    }();
    var SetWrapper = function () {
        function SetWrapper() {}
        SetWrapper.createFromList = function (lst) {
            return createSetFromList(lst);
        };
        SetWrapper.has = function (s, key) {
            return s.has(key);
        };
        SetWrapper.delete = function (m, k) {
            m.delete(k);
        };
        return SetWrapper;
    }();
    exports.SetWrapper = SetWrapper;
    
});
System.registerDynamic('node_modules/@angular/http/src/facade/lang.js', [], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var globalScope;
    if (typeof window === 'undefined') {
        if (typeof WorkerGlobalScope !== 'undefined' && self instanceof WorkerGlobalScope) {
            // TODO: Replace any with WorkerGlobalScope from lib.webworker.d.ts #3492
            globalScope = self;
        } else {
            globalScope = global;
        }
    } else {
        globalScope = window;
    }
    function scheduleMicroTask(fn) {
        Zone.current.scheduleMicroTask('scheduleMicrotask', fn);
    }
    exports.scheduleMicroTask = scheduleMicroTask;
    exports.IS_DART = false;
    // Need to declare a new variable for global here since TypeScript
    // exports the original value of the symbol.
    var _global = globalScope;
    exports.global = _global;
    /**
     * Runtime representation a type that a Component or other object is instances of.
     *
     * An example of a `Type` is `MyCustomComponent` class, which in JavaScript is be represented by
     * the `MyCustomComponent` constructor function.
     *
     * @stable
     */
    exports.Type = Function;
    function getTypeNameForDebugging(type) {
        if (type['name']) {
            return type['name'];
        }
        return typeof type;
    }
    exports.getTypeNameForDebugging = getTypeNameForDebugging;
    exports.Math = _global.Math;
    exports.Date = _global.Date;
    // TODO: remove calls to assert in production environment
    // Note: Can't just export this and import in in other files
    // as `assert` is a reserved keyword in Dart
    _global.assert = function assert(condition) {
        // TODO: to be fixed properly via #2830, noop for now
    };
    function isPresent(obj) {
        return obj !== undefined && obj !== null;
    }
    exports.isPresent = isPresent;
    function isBlank(obj) {
        return obj === undefined || obj === null;
    }
    exports.isBlank = isBlank;
    function isBoolean(obj) {
        return typeof obj === 'boolean';
    }
    exports.isBoolean = isBoolean;
    function isNumber(obj) {
        return typeof obj === 'number';
    }
    exports.isNumber = isNumber;
    function isString(obj) {
        return typeof obj === 'string';
    }
    exports.isString = isString;
    function isFunction(obj) {
        return typeof obj === 'function';
    }
    exports.isFunction = isFunction;
    function isType(obj) {
        return isFunction(obj);
    }
    exports.isType = isType;
    function isStringMap(obj) {
        return typeof obj === 'object' && obj !== null;
    }
    exports.isStringMap = isStringMap;
    var STRING_MAP_PROTO = Object.getPrototypeOf({});
    function isStrictStringMap(obj) {
        return isStringMap(obj) && Object.getPrototypeOf(obj) === STRING_MAP_PROTO;
    }
    exports.isStrictStringMap = isStrictStringMap;
    function isPromise(obj) {
        return obj instanceof _global.Promise;
    }
    exports.isPromise = isPromise;
    function isArray(obj) {
        return Array.isArray(obj);
    }
    exports.isArray = isArray;
    function isDate(obj) {
        return obj instanceof exports.Date && !isNaN(obj.valueOf());
    }
    exports.isDate = isDate;
    function noop() {}
    exports.noop = noop;
    function stringify(token) {
        if (typeof token === 'string') {
            return token;
        }
        if (token === undefined || token === null) {
            return '' + token;
        }
        if (token.name) {
            return token.name;
        }
        if (token.overriddenName) {
            return token.overriddenName;
        }
        var res = token.toString();
        var newLineIndex = res.indexOf('\n');
        return newLineIndex === -1 ? res : res.substring(0, newLineIndex);
    }
    exports.stringify = stringify;
    // serialize / deserialize enum exist only for consistency with dart API
    // enums in typescript don't need to be serialized
    function serializeEnum(val) {
        return val;
    }
    exports.serializeEnum = serializeEnum;
    function deserializeEnum(val, values) {
        return val;
    }
    exports.deserializeEnum = deserializeEnum;
    function resolveEnumToken(enumValue, val) {
        return enumValue[val];
    }
    exports.resolveEnumToken = resolveEnumToken;
    var StringWrapper = function () {
        function StringWrapper() {}
        StringWrapper.fromCharCode = function (code) {
            return String.fromCharCode(code);
        };
        StringWrapper.charCodeAt = function (s, index) {
            return s.charCodeAt(index);
        };
        StringWrapper.split = function (s, regExp) {
            return s.split(regExp);
        };
        StringWrapper.equals = function (s, s2) {
            return s === s2;
        };
        StringWrapper.stripLeft = function (s, charVal) {
            if (s && s.length) {
                var pos = 0;
                for (var i = 0; i < s.length; i++) {
                    if (s[i] != charVal) break;
                    pos++;
                }
                s = s.substring(pos);
            }
            return s;
        };
        StringWrapper.stripRight = function (s, charVal) {
            if (s && s.length) {
                var pos = s.length;
                for (var i = s.length - 1; i >= 0; i--) {
                    if (s[i] != charVal) break;
                    pos--;
                }
                s = s.substring(0, pos);
            }
            return s;
        };
        StringWrapper.replace = function (s, from, replace) {
            return s.replace(from, replace);
        };
        StringWrapper.replaceAll = function (s, from, replace) {
            return s.replace(from, replace);
        };
        StringWrapper.slice = function (s, from, to) {
            if (from === void 0) {
                from = 0;
            }
            if (to === void 0) {
                to = null;
            }
            return s.slice(from, to === null ? undefined : to);
        };
        StringWrapper.replaceAllMapped = function (s, from, cb) {
            return s.replace(from, function () {
                var matches = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    matches[_i - 0] = arguments[_i];
                }
                // Remove offset & string from the result array
                matches.splice(-2, 2);
                // The callback receives match, p1, ..., pn
                return cb(matches);
            });
        };
        StringWrapper.contains = function (s, substr) {
            return s.indexOf(substr) != -1;
        };
        StringWrapper.compare = function (a, b) {
            if (a < b) {
                return -1;
            } else if (a > b) {
                return 1;
            } else {
                return 0;
            }
        };
        return StringWrapper;
    }();
    exports.StringWrapper = StringWrapper;
    var StringJoiner = function () {
        function StringJoiner(parts) {
            if (parts === void 0) {
                parts = [];
            }
            this.parts = parts;
        }
        StringJoiner.prototype.add = function (part) {
            this.parts.push(part);
        };
        StringJoiner.prototype.toString = function () {
            return this.parts.join('');
        };
        return StringJoiner;
    }();
    exports.StringJoiner = StringJoiner;
    var NumberParseError = function (_super) {
        __extends(NumberParseError, _super);
        function NumberParseError(message) {
            _super.call(this);
            this.message = message;
        }
        NumberParseError.prototype.toString = function () {
            return this.message;
        };
        return NumberParseError;
    }(Error);
    exports.NumberParseError = NumberParseError;
    var NumberWrapper = function () {
        function NumberWrapper() {}
        NumberWrapper.toFixed = function (n, fractionDigits) {
            return n.toFixed(fractionDigits);
        };
        NumberWrapper.equal = function (a, b) {
            return a === b;
        };
        NumberWrapper.parseIntAutoRadix = function (text) {
            var result = parseInt(text);
            if (isNaN(result)) {
                throw new NumberParseError('Invalid integer literal when parsing ' + text);
            }
            return result;
        };
        NumberWrapper.parseInt = function (text, radix) {
            if (radix == 10) {
                if (/^(\-|\+)?[0-9]+$/.test(text)) {
                    return parseInt(text, radix);
                }
            } else if (radix == 16) {
                if (/^(\-|\+)?[0-9ABCDEFabcdef]+$/.test(text)) {
                    return parseInt(text, radix);
                }
            } else {
                var result = parseInt(text, radix);
                if (!isNaN(result)) {
                    return result;
                }
            }
            throw new NumberParseError('Invalid integer literal when parsing ' + text + ' in base ' + radix);
        };
        // TODO: NaN is a valid literal but is returned by parseFloat to indicate an error.
        NumberWrapper.parseFloat = function (text) {
            return parseFloat(text);
        };
        Object.defineProperty(NumberWrapper, "NaN", {
            get: function () {
                return NaN;
            },
            enumerable: true,
            configurable: true
        });
        NumberWrapper.isNumeric = function (value) {
            return !isNaN(value - parseFloat(value));
        };
        NumberWrapper.isNaN = function (value) {
            return isNaN(value);
        };
        NumberWrapper.isInteger = function (value) {
            return Number.isInteger(value);
        };
        return NumberWrapper;
    }();
    exports.NumberWrapper = NumberWrapper;
    exports.RegExp = _global.RegExp;
    var RegExpWrapper = function () {
        function RegExpWrapper() {}
        RegExpWrapper.create = function (regExpStr, flags) {
            if (flags === void 0) {
                flags = '';
            }
            flags = flags.replace(/g/g, '');
            return new _global.RegExp(regExpStr, flags + 'g');
        };
        RegExpWrapper.firstMatch = function (regExp, input) {
            // Reset multimatch regex state
            regExp.lastIndex = 0;
            return regExp.exec(input);
        };
        RegExpWrapper.test = function (regExp, input) {
            regExp.lastIndex = 0;
            return regExp.test(input);
        };
        RegExpWrapper.matcher = function (regExp, input) {
            // Reset regex state for the case
            // someone did not loop over all matches
            // last time.
            regExp.lastIndex = 0;
            return { re: regExp, input: input };
        };
        RegExpWrapper.replaceAll = function (regExp, input, replace) {
            var c = regExp.exec(input);
            var res = '';
            regExp.lastIndex = 0;
            var prev = 0;
            while (c) {
                res += input.substring(prev, c.index);
                res += replace(c);
                prev = c.index + c[0].length;
                regExp.lastIndex = prev;
                c = regExp.exec(input);
            }
            res += input.substring(prev);
            return res;
        };
        return RegExpWrapper;
    }();
    exports.RegExpWrapper = RegExpWrapper;
    var RegExpMatcherWrapper = function () {
        function RegExpMatcherWrapper() {}
        RegExpMatcherWrapper.next = function (matcher) {
            return matcher.re.exec(matcher.input);
        };
        return RegExpMatcherWrapper;
    }();
    exports.RegExpMatcherWrapper = RegExpMatcherWrapper;
    var FunctionWrapper = function () {
        function FunctionWrapper() {}
        FunctionWrapper.apply = function (fn, posArgs) {
            return fn.apply(null, posArgs);
        };
        FunctionWrapper.bind = function (fn, scope) {
            return fn.bind(scope);
        };
        return FunctionWrapper;
    }();
    exports.FunctionWrapper = FunctionWrapper;
    // JS has NaN !== NaN
    function looseIdentical(a, b) {
        return a === b || typeof a === 'number' && typeof b === 'number' && isNaN(a) && isNaN(b);
    }
    exports.looseIdentical = looseIdentical;
    // JS considers NaN is the same as NaN for map Key (while NaN !== NaN otherwise)
    // see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Map
    function getMapKey(value) {
        return value;
    }
    exports.getMapKey = getMapKey;
    function normalizeBlank(obj) {
        return isBlank(obj) ? null : obj;
    }
    exports.normalizeBlank = normalizeBlank;
    function normalizeBool(obj) {
        return isBlank(obj) ? false : obj;
    }
    exports.normalizeBool = normalizeBool;
    function isJsObject(o) {
        return o !== null && (typeof o === 'function' || typeof o === 'object');
    }
    exports.isJsObject = isJsObject;
    function print(obj) {
        console.log(obj);
    }
    exports.print = print;
    function warn(obj) {
        console.warn(obj);
    }
    exports.warn = warn;
    // Can't be all uppercase as our transpiler would think it is a special directive...
    var Json = function () {
        function Json() {}
        Json.parse = function (s) {
            return _global.JSON.parse(s);
        };
        Json.stringify = function (data) {
            // Dart doesn't take 3 arguments
            return _global.JSON.stringify(data, null, 2);
        };
        return Json;
    }();
    exports.Json = Json;
    var DateWrapper = function () {
        function DateWrapper() {}
        DateWrapper.create = function (year, month, day, hour, minutes, seconds, milliseconds) {
            if (month === void 0) {
                month = 1;
            }
            if (day === void 0) {
                day = 1;
            }
            if (hour === void 0) {
                hour = 0;
            }
            if (minutes === void 0) {
                minutes = 0;
            }
            if (seconds === void 0) {
                seconds = 0;
            }
            if (milliseconds === void 0) {
                milliseconds = 0;
            }
            return new exports.Date(year, month - 1, day, hour, minutes, seconds, milliseconds);
        };
        DateWrapper.fromISOString = function (str) {
            return new exports.Date(str);
        };
        DateWrapper.fromMillis = function (ms) {
            return new exports.Date(ms);
        };
        DateWrapper.toMillis = function (date) {
            return date.getTime();
        };
        DateWrapper.now = function () {
            return new exports.Date();
        };
        DateWrapper.toJson = function (date) {
            return date.toJSON();
        };
        return DateWrapper;
    }();
    exports.DateWrapper = DateWrapper;
    function setValueOnPath(global, path, value) {
        var parts = path.split('.');
        var obj = global;
        while (parts.length > 1) {
            var name = parts.shift();
            if (obj.hasOwnProperty(name) && isPresent(obj[name])) {
                obj = obj[name];
            } else {
                obj = obj[name] = {};
            }
        }
        if (obj === undefined || obj === null) {
            obj = {};
        }
        obj[parts.shift()] = value;
    }
    exports.setValueOnPath = setValueOnPath;
    var _symbolIterator = null;
    function getSymbolIterator() {
        if (isBlank(_symbolIterator)) {
            if (isPresent(globalScope.Symbol) && isPresent(Symbol.iterator)) {
                _symbolIterator = Symbol.iterator;
            } else {
                // es6-shim specific logic
                var keys = Object.getOwnPropertyNames(Map.prototype);
                for (var i = 0; i < keys.length; ++i) {
                    var key = keys[i];
                    if (key !== 'entries' && key !== 'size' && Map.prototype[key] === Map.prototype['entries']) {
                        _symbolIterator = key;
                    }
                }
            }
        }
        return _symbolIterator;
    }
    exports.getSymbolIterator = getSymbolIterator;
    function evalExpression(sourceUrl, expr, declarations, vars) {
        var fnBody = declarations + "\nreturn " + expr + "\n//# sourceURL=" + sourceUrl;
        var fnArgNames = [];
        var fnArgValues = [];
        for (var argName in vars) {
            fnArgNames.push(argName);
            fnArgValues.push(vars[argName]);
        }
        return new (Function.bind.apply(Function, [void 0].concat(fnArgNames.concat(fnBody))))().apply(void 0, fnArgValues);
    }
    exports.evalExpression = evalExpression;
    function isPrimitive(obj) {
        return !isJsObject(obj);
    }
    exports.isPrimitive = isPrimitive;
    function hasConstructor(value, type) {
        return value.constructor === type;
    }
    exports.hasConstructor = hasConstructor;
    function escape(s) {
        return _global.encodeURI(s);
    }
    exports.escape = escape;
    function escapeRegExp(s) {
        return s.replace(/([.*+?^=!:${}()|[\]\/\\])/g, '\\$1');
    }
    exports.escapeRegExp = escapeRegExp;
    
});
System.registerDynamic('node_modules/@angular/http/src/url_search_params.js', ['../src/facade/collection', '../src/facade/lang'], true, function ($__require, exports, module) {
    /**
     * @license
     * Copyright Google Inc. All Rights Reserved.
     *
     * Use of this source code is governed by an MIT-style license that can be
     * found in the LICENSE file at https://angular.io/license
     */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var collection_1 = $__require('../src/facade/collection');
    var lang_1 = $__require('../src/facade/lang');
    function paramParser(rawParams) {
        if (rawParams === void 0) {
            rawParams = '';
        }
        var map = new collection_1.Map();
        if (rawParams.length > 0) {
            var params = rawParams.split('&');
            params.forEach(function (param) {
                var split = param.split('=', 2);
                var key = split[0];
                var val = split[1];
                var list = lang_1.isPresent(map.get(key)) ? map.get(key) : [];
                list.push(val);
                map.set(key, list);
            });
        }
        return map;
    }
    /**
     * @experimental
     **/
    var QueryEncoder = function () {
        function QueryEncoder() {}
        QueryEncoder.prototype.encodeKey = function (k) {
            return standardEncoding(k);
        };
        QueryEncoder.prototype.encodeValue = function (v) {
            return standardEncoding(v);
        };
        return QueryEncoder;
    }();
    exports.QueryEncoder = QueryEncoder;
    function standardEncoding(v) {
        return encodeURIComponent(v).replace(/%40/gi, '@').replace(/%3A/gi, ':').replace(/%24/gi, '$').replace(/%2C/gi, ',').replace(/%3B/gi, ';').replace(/%2B/gi, '+').replace(/%3D/gi, ';').replace(/%3F/gi, '?').replace(/%2F/gi, '/');
    }
    /**
     * Map-like representation of url search parameters, based on
     * [URLSearchParams](https://url.spec.whatwg.org/#urlsearchparams) in the url living standard,
     * with several extensions for merging URLSearchParams objects:
     *   - setAll()
     *   - appendAll()
     *   - replaceAll()
     *
     * This class accepts an optional second parameter of ${@link QueryEncoder},
     * which is used to serialize parameters before making a request. By default,
     * `QueryEncoder` encodes keys and values of parameters using `encodeURIComponent`,
     * and then un-encodes certain characters that are allowed to be part of the query
     * according to IETF RFC 3986: https://tools.ietf.org/html/rfc3986.
     *
     * These are the characters that are not encoded: `! $ \' ( ) * + , ; A 9 - . _ ~ ? /`
     *
     * If the set of allowed query characters is not acceptable for a particular backend,
     * `QueryEncoder` can be subclassed and provided as the 2nd argument to URLSearchParams.
     *
     * ```
     * import {URLSearchParams, QueryEncoder} from '@angular/http';
     * class MyQueryEncoder extends QueryEncoder {
     *   encodeKey(k: string): string {
     *     return myEncodingFunction(k);
     *   }
     *
     *   encodeValue(v: string): string {
     *     return myEncodingFunction(v);
     *   }
     * }
     *
     * let params = new URLSearchParams('', new MyQueryEncoder());
     * ```
     * @experimental
     */
    var URLSearchParams = function () {
        function URLSearchParams(rawParams, queryEncoder) {
            if (rawParams === void 0) {
                rawParams = '';
            }
            if (queryEncoder === void 0) {
                queryEncoder = new QueryEncoder();
            }
            this.rawParams = rawParams;
            this.queryEncoder = queryEncoder;
            this.paramsMap = paramParser(rawParams);
        }
        URLSearchParams.prototype.clone = function () {
            var clone = new URLSearchParams();
            clone.appendAll(this);
            return clone;
        };
        URLSearchParams.prototype.has = function (param) {
            return this.paramsMap.has(param);
        };
        URLSearchParams.prototype.get = function (param) {
            var storedParam = this.paramsMap.get(param);
            if (collection_1.isListLikeIterable(storedParam)) {
                return collection_1.ListWrapper.first(storedParam);
            } else {
                return null;
            }
        };
        URLSearchParams.prototype.getAll = function (param) {
            var mapParam = this.paramsMap.get(param);
            return lang_1.isPresent(mapParam) ? mapParam : [];
        };
        URLSearchParams.prototype.set = function (param, val) {
            var mapParam = this.paramsMap.get(param);
            var list = lang_1.isPresent(mapParam) ? mapParam : [];
            collection_1.ListWrapper.clear(list);
            list.push(val);
            this.paramsMap.set(param, list);
        };
        // A merge operation
        // For each name-values pair in `searchParams`, perform `set(name, values[0])`
        //
        // E.g: "a=[1,2,3], c=[8]" + "a=[4,5,6], b=[7]" = "a=[4], c=[8], b=[7]"
        //
        // TODO(@caitp): document this better
        URLSearchParams.prototype.setAll = function (searchParams) {
            var _this = this;
            searchParams.paramsMap.forEach(function (value, param) {
                var mapParam = _this.paramsMap.get(param);
                var list = lang_1.isPresent(mapParam) ? mapParam : [];
                collection_1.ListWrapper.clear(list);
                list.push(value[0]);
                _this.paramsMap.set(param, list);
            });
        };
        URLSearchParams.prototype.append = function (param, val) {
            var mapParam = this.paramsMap.get(param);
            var list = lang_1.isPresent(mapParam) ? mapParam : [];
            list.push(val);
            this.paramsMap.set(param, list);
        };
        // A merge operation
        // For each name-values pair in `searchParams`, perform `append(name, value)`
        // for each value in `values`.
        //
        // E.g: "a=[1,2], c=[8]" + "a=[3,4], b=[7]" = "a=[1,2,3,4], c=[8], b=[7]"
        //
        // TODO(@caitp): document this better
        URLSearchParams.prototype.appendAll = function (searchParams) {
            var _this = this;
            searchParams.paramsMap.forEach(function (value, param) {
                var mapParam = _this.paramsMap.get(param);
                var list = lang_1.isPresent(mapParam) ? mapParam : [];
                for (var i = 0; i < value.length; ++i) {
                    list.push(value[i]);
                }
                _this.paramsMap.set(param, list);
            });
        };
        // A merge operation
        // For each name-values pair in `searchParams`, perform `delete(name)`,
        // followed by `set(name, values)`
        //
        // E.g: "a=[1,2,3], c=[8]" + "a=[4,5,6], b=[7]" = "a=[4,5,6], c=[8], b=[7]"
        //
        // TODO(@caitp): document this better
        URLSearchParams.prototype.replaceAll = function (searchParams) {
            var _this = this;
            searchParams.paramsMap.forEach(function (value, param) {
                var mapParam = _this.paramsMap.get(param);
                var list = lang_1.isPresent(mapParam) ? mapParam : [];
                collection_1.ListWrapper.clear(list);
                for (var i = 0; i < value.length; ++i) {
                    list.push(value[i]);
                }
                _this.paramsMap.set(param, list);
            });
        };
        URLSearchParams.prototype.toString = function () {
            var _this = this;
            var paramsList = [];
            this.paramsMap.forEach(function (values, k) {
                values.forEach(function (v) {
                    return paramsList.push(_this.queryEncoder.encodeKey(k) + '=' + _this.queryEncoder.encodeValue(v));
                });
            });
            return paramsList.join('&');
        };
        URLSearchParams.prototype.delete = function (param) {
            this.paramsMap.delete(param);
        };
        return URLSearchParams;
    }();
    exports.URLSearchParams = URLSearchParams;
    
});
System.registerDynamic('node_modules/@angular/http/http.js', ['./src/backends/browser_jsonp', './src/backends/browser_xhr', './src/backends/jsonp_backend', './src/backends/xhr_backend', './src/base_request_options', './src/base_response_options', './src/http', './src/interfaces', './src/enums', './src/headers', './src/static_request', './src/static_response', './src/url_search_params'], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var browser_jsonp_1 = $__require('./src/backends/browser_jsonp');
  var browser_xhr_1 = $__require('./src/backends/browser_xhr');
  var jsonp_backend_1 = $__require('./src/backends/jsonp_backend');
  var xhr_backend_1 = $__require('./src/backends/xhr_backend');
  var base_request_options_1 = $__require('./src/base_request_options');
  var base_response_options_1 = $__require('./src/base_response_options');
  var http_1 = $__require('./src/http');
  var interfaces_1 = $__require('./src/interfaces');
  var browser_xhr_2 = $__require('./src/backends/browser_xhr');
  exports.BrowserXhr = browser_xhr_2.BrowserXhr;
  var jsonp_backend_2 = $__require('./src/backends/jsonp_backend');
  exports.JSONPBackend = jsonp_backend_2.JSONPBackend;
  exports.JSONPConnection = jsonp_backend_2.JSONPConnection;
  var xhr_backend_2 = $__require('./src/backends/xhr_backend');
  exports.CookieXSRFStrategy = xhr_backend_2.CookieXSRFStrategy;
  exports.XHRBackend = xhr_backend_2.XHRBackend;
  exports.XHRConnection = xhr_backend_2.XHRConnection;
  var base_request_options_2 = $__require('./src/base_request_options');
  exports.BaseRequestOptions = base_request_options_2.BaseRequestOptions;
  exports.RequestOptions = base_request_options_2.RequestOptions;
  var base_response_options_2 = $__require('./src/base_response_options');
  exports.BaseResponseOptions = base_response_options_2.BaseResponseOptions;
  exports.ResponseOptions = base_response_options_2.ResponseOptions;
  var enums_1 = $__require('./src/enums');
  exports.ReadyState = enums_1.ReadyState;
  exports.RequestMethod = enums_1.RequestMethod;
  exports.ResponseType = enums_1.ResponseType;
  var headers_1 = $__require('./src/headers');
  exports.Headers = headers_1.Headers;
  var http_2 = $__require('./src/http');
  exports.Http = http_2.Http;
  exports.Jsonp = http_2.Jsonp;
  var interfaces_2 = $__require('./src/interfaces');
  exports.Connection = interfaces_2.Connection;
  exports.ConnectionBackend = interfaces_2.ConnectionBackend;
  exports.XSRFStrategy = interfaces_2.XSRFStrategy;
  var static_request_1 = $__require('./src/static_request');
  exports.Request = static_request_1.Request;
  var static_response_1 = $__require('./src/static_response');
  exports.Response = static_response_1.Response;
  var url_search_params_1 = $__require('./src/url_search_params');
  exports.QueryEncoder = url_search_params_1.QueryEncoder;
  exports.URLSearchParams = url_search_params_1.URLSearchParams;
  /**
   * Provides a basic set of injectables to use the {@link Http} service in any application.
   *
   * The `HTTP_PROVIDERS` should be included either in a component's injector,
   * or in the root injector when bootstrapping an application.
   *
   * ### Example ([live demo](http://plnkr.co/edit/snj7Nv?p=preview))
   *
   * ```
   * import {Component} from '@angular/core';
   * import {bootstrap} from '@angular/platform-browser/browser';
   * import {NgFor} from '@angular/common';
   * import {HTTP_PROVIDERS, Http} from '@angular/http';
   *
   * @Component({
   *   selector: 'app',
   *   providers: [HTTP_PROVIDERS],
   *   template: `
   *     <div>
   *       <h1>People</h1>
   *       <ul>
   *         <li *ngFor="let person of people">
   *           {{person.name}}
   *         </li>
   *       </ul>
   *     </div>
   *   `,
   *   directives: [NgFor]
   * })
   * export class App {
   *   people: Object[];
   *   constructor(http:Http) {
   *     http.get('people.json').subscribe(res => {
   *       this.people = res.json();
   *     });
   *   }
   *   active:boolean = false;
   *   toggleActiveState() {
   *     this.active = !this.active;
   *   }
   * }
   *
   * bootstrap(App)
   *   .catch(err => console.error(err));
   * ```
   *
   * The primary public API included in `HTTP_PROVIDERS` is the {@link Http} class.
   * However, other providers required by `Http` are included,
   * which may be beneficial to override in certain cases.
   *
   * The providers included in `HTTP_PROVIDERS` include:
   *  * {@link Http}
   *  * {@link XHRBackend}
   *  * {@link XSRFStrategy} - Bound to {@link CookieXSRFStrategy} class (see below)
   *  * `BrowserXHR` - Private factory to create `XMLHttpRequest` instances
   *  * {@link RequestOptions} - Bound to {@link BaseRequestOptions} class
   *  * {@link ResponseOptions} - Bound to {@link BaseResponseOptions} class
   *
   * There may be cases where it makes sense to extend the base request options,
   * such as to add a search string to be appended to all URLs.
   * To accomplish this, a new provider for {@link RequestOptions} should
   * be added in the same injector as `HTTP_PROVIDERS`.
   *
   * ### Example ([live demo](http://plnkr.co/edit/aCMEXi?p=preview))
   *
   * ```
   * import {provide} from '@angular/core';
   * import {bootstrap} from '@angular/platform-browser/browser';
   * import {HTTP_PROVIDERS, BaseRequestOptions, RequestOptions} from '@angular/http';
   *
   * class MyOptions extends BaseRequestOptions {
   *   search: string = 'coreTeam=true';
   * }
   *
   * bootstrap(App, [HTTP_PROVIDERS, {provide: RequestOptions, useClass: MyOptions}])
   *   .catch(err => console.error(err));
   * ```
   *
   * Likewise, to use a mock backend for unit tests, the {@link XHRBackend}
   * provider should be bound to {@link MockBackend}.
   *
   * ### Example ([live demo](http://plnkr.co/edit/7LWALD?p=preview))
   *
   * ```
   * import {provide} from '@angular/core';
   * import {bootstrap} from '@angular/platform-browser/browser';
   * import {HTTP_PROVIDERS, Http, Response, XHRBackend} from '@angular/http';
   * import {MockBackend} from '@angular/http/testing';
   *
   * var people = [{name: 'Jeff'}, {name: 'Tobias'}];
   *
   * var injector = Injector.resolveAndCreate([
   *   HTTP_PROVIDERS,
   *   MockBackend,
   *   {provide: XHRBackend, useExisting: MockBackend}
   * ]);
   * var http = injector.get(Http);
   * var backend = injector.get(MockBackend);
   *
   * // Listen for any new requests
   * backend.connections.observer({
   *   next: connection => {
   *     var response = new Response({body: people});
   *     setTimeout(() => {
   *       // Send a response to the request
   *       connection.mockRespond(response);
   *     });
   *   }
   * });
   *
   * http.get('people.json').observer({
   *   next: res => {
   *     // Response came from mock backend
   *     console.log('first person', res.json()[0].name);
   *   }
   * });
   * ```
   *
   * `XSRFStrategy` allows customizing how the application protects itself against Cross Site Request
   * Forgery (XSRF) attacks. By default, Angular will look for a cookie called `'XSRF-TOKEN'`, and set
   * an HTTP request header called `'X-XSRF-TOKEN'` with the value of the cookie on each request,
   * allowing the server side to validate that the request comes from its own front end.
   *
   * Applications can override the names used by configuring a different `XSRFStrategy` instance. Most
   * commonly, applications will configure a `CookieXSRFStrategy` with different cookie or header
   * names, but if needed, they can supply a completely custom implementation.
   *
   * See the security documentation for more information.
   *
   * ### Example
   *
   * ```
   * import {provide} from '@angular/core';
   * import {bootstrap} from '@angular/platform-browser/browser';
   * import {HTTP_PROVIDERS, XSRFStrategy, CookieXSRFStrategy} from '@angular/http';
   *
   * bootstrap(
   *     App,
   *     [HTTP_PROVIDERS, {provide: XSRFStrategy,
   *         useValue: new CookieXSRFStrategy('MY-XSRF-COOKIE-NAME', 'X-MY-XSRF-HEADER-NAME')}])
   *   .catch(err => console.error(err));
   * ```
   *
   * @experimental
   */
  exports.HTTP_PROVIDERS = [
  // TODO(pascal): use factory type annotations once supported in DI
  // issue: https://github.com/angular/angular/issues/3183
  { provide: http_1.Http, useFactory: httpFactory, deps: [xhr_backend_1.XHRBackend, base_request_options_1.RequestOptions] }, browser_xhr_1.BrowserXhr, { provide: base_request_options_1.RequestOptions, useClass: base_request_options_1.BaseRequestOptions }, { provide: base_response_options_1.ResponseOptions, useClass: base_response_options_1.BaseResponseOptions }, xhr_backend_1.XHRBackend, { provide: interfaces_1.XSRFStrategy, useValue: new xhr_backend_1.CookieXSRFStrategy() }];
  /**
   * @experimental
   */
  function httpFactory(xhrBackend, requestOptions) {
    return new http_1.Http(xhrBackend, requestOptions);
  }
  exports.httpFactory = httpFactory;
  /**
   * See {@link HTTP_PROVIDERS} instead.
   *
   * @deprecated
   */
  exports.HTTP_BINDINGS = exports.HTTP_PROVIDERS;
  /**
   * Provides a basic set of providers to use the {@link Jsonp} service in any application.
   *
   * The `JSONP_PROVIDERS` should be included either in a component's injector,
   * or in the root injector when bootstrapping an application.
   *
   * ### Example ([live demo](http://plnkr.co/edit/vmeN4F?p=preview))
   *
   * ```
   * import {Component} from '@angular/core';
   * import {NgFor} from '@angular/common';
   * import {JSONP_PROVIDERS, Jsonp} from '@angular/http';
   *
   * @Component({
   *   selector: 'app',
   *   providers: [JSONP_PROVIDERS],
   *   template: `
   *     <div>
   *       <h1>People</h1>
   *       <ul>
   *         <li *ngFor="let person of people">
   *           {{person.name}}
   *         </li>
   *       </ul>
   *     </div>
   *   `,
   *   directives: [NgFor]
   * })
   * export class App {
   *   people: Array<Object>;
   *   constructor(jsonp:Jsonp) {
   *     jsonp.request('people.json').subscribe(res => {
   *       this.people = res.json();
   *     })
   *   }
   * }
   * ```
   *
   * The primary public API included in `JSONP_PROVIDERS` is the {@link Jsonp} class.
   * However, other providers required by `Jsonp` are included,
   * which may be beneficial to override in certain cases.
   *
   * The providers included in `JSONP_PROVIDERS` include:
   *  * {@link Jsonp}
   *  * {@link JSONPBackend}
   *  * `BrowserJsonp` - Private factory
   *  * {@link RequestOptions} - Bound to {@link BaseRequestOptions} class
   *  * {@link ResponseOptions} - Bound to {@link BaseResponseOptions} class
   *
   * There may be cases where it makes sense to extend the base request options,
   * such as to add a search string to be appended to all URLs.
   * To accomplish this, a new provider for {@link RequestOptions} should
   * be added in the same injector as `JSONP_PROVIDERS`.
   *
   * ### Example ([live demo](http://plnkr.co/edit/TFug7x?p=preview))
   *
   * ```
   * import {provide} from '@angular/core';
   * import {bootstrap} from '@angular/platform-browser/browser';
   * import {JSONP_PROVIDERS, BaseRequestOptions, RequestOptions} from '@angular/http';
   *
   * class MyOptions extends BaseRequestOptions {
   *   search: string = 'coreTeam=true';
   * }
   *
   * bootstrap(App, [JSONP_PROVIDERS, {provide: RequestOptions, useClass: MyOptions}])
   *   .catch(err => console.error(err));
   * ```
   *
   * Likewise, to use a mock backend for unit tests, the {@link JSONPBackend}
   * provider should be bound to {@link MockBackend}.
   *
   * ### Example ([live demo](http://plnkr.co/edit/HDqZWL?p=preview))
   *
   * ```
   * import {provide, Injector} from '@angular/core';
   * import {JSONP_PROVIDERS, Jsonp, Response, JSONPBackend} from '@angular/http';
   * import {MockBackend} from '@angular/http/testing';
   *
   * var people = [{name: 'Jeff'}, {name: 'Tobias'}];
   * var injector = Injector.resolveAndCreate([
   *   JSONP_PROVIDERS,
   *   MockBackend,
   *   {provide: JSONPBackend, useExisting: MockBackend}
   * ]);
   * var jsonp = injector.get(Jsonp);
   * var backend = injector.get(MockBackend);
   *
   * // Listen for any new requests
   * backend.connections.observer({
   *   next: connection => {
   *     var response = new Response({body: people});
   *     setTimeout(() => {
   *       // Send a response to the request
   *       connection.mockRespond(response);
   *     });
   *   }
   * });
  
   * jsonp.get('people.json').observer({
   *   next: res => {
   *     // Response came from mock backend
   *     console.log('first person', res.json()[0].name);
   *   }
   * });
   * ```
   *
   * @experimental
   */
  exports.JSONP_PROVIDERS = [
  // TODO(pascal): use factory type annotations once supported in DI
  // issue: https://github.com/angular/angular/issues/3183
  { provide: http_1.Jsonp, useFactory: jsonpFactory, deps: [jsonp_backend_1.JSONPBackend, base_request_options_1.RequestOptions] }, browser_jsonp_1.BrowserJsonp, { provide: base_request_options_1.RequestOptions, useClass: base_request_options_1.BaseRequestOptions }, { provide: base_response_options_1.ResponseOptions, useClass: base_response_options_1.BaseResponseOptions }, { provide: jsonp_backend_1.JSONPBackend, useClass: jsonp_backend_1.JSONPBackend_ }];
  function jsonpFactory(jsonpBackend, requestOptions) {
    return new http_1.Jsonp(jsonpBackend, requestOptions);
  }
  /**
   * See {@link JSONP_PROVIDERS} instead.
   *
   * @deprecated
   */
  exports.JSON_BINDINGS = exports.JSONP_PROVIDERS;
  
});
System.registerDynamic("node_modules/@angular/http/index.js", ["./http"], true, function ($__require, exports, module) {
  /**
   * @license
   * Copyright Google Inc. All Rights Reserved.
   *
   * Use of this source code is governed by an MIT-style license that can be
   * found in the LICENSE file at https://angular.io/license
   */
  "use strict";

  var global = this || self,
      GLOBAL = global;
  function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
  }
  __export($__require('./http'));
  
});
System.registerDynamic('node_modules/angular-mocks/angular-mocks.js', [], false, function ($__require, $__exports, $__module) {
  var _retrieveGlobal = System.get("@@global-helpers").prepareGlobal($__module.id, null, null);

  (function ($__global) {
    /**
     * @license AngularJS v1.5.11
     * (c) 2010-2017 Google, Inc. http://angularjs.org
     * License: MIT
     */
    (function (window, angular) {

      'use strict';

      /**
       * @ngdoc object
       * @name angular.mock
       * @description
       *
       * Namespace from 'angular-mocks.js' which contains testing related code.
       *
       */

      angular.mock = {};

      /**
       * ! This is a private undocumented service !
       *
       * @name $browser
       *
       * @description
       * This service is a mock implementation of {@link ng.$browser}. It provides fake
       * implementation for commonly used browser apis that are hard to test, e.g. setTimeout, xhr,
       * cookies, etc.
       *
       * The api of this service is the same as that of the real {@link ng.$browser $browser}, except
       * that there are several helper methods available which can be used in tests.
       */
      angular.mock.$BrowserProvider = function () {
        this.$get = function () {
          return new angular.mock.$Browser();
        };
      };

      angular.mock.$Browser = function () {
        var self = this;

        this.isMock = true;
        self.$$url = 'http://server/';
        self.$$lastUrl = self.$$url; // used by url polling fn
        self.pollFns = [];

        // TODO(vojta): remove this temporary api
        self.$$completeOutstandingRequest = angular.noop;
        self.$$incOutstandingRequestCount = angular.noop;

        // register url polling fn

        self.onUrlChange = function (listener) {
          self.pollFns.push(function () {
            if (self.$$lastUrl !== self.$$url || self.$$state !== self.$$lastState) {
              self.$$lastUrl = self.$$url;
              self.$$lastState = self.$$state;
              listener(self.$$url, self.$$state);
            }
          });

          return listener;
        };

        self.$$applicationDestroyed = angular.noop;
        self.$$checkUrlChange = angular.noop;

        self.deferredFns = [];
        self.deferredNextId = 0;

        self.defer = function (fn, delay) {
          delay = delay || 0;
          self.deferredFns.push({ time: self.defer.now + delay, fn: fn, id: self.deferredNextId });
          self.deferredFns.sort(function (a, b) {
            return a.time - b.time;
          });
          return self.deferredNextId++;
        };

        /**
         * @name $browser#defer.now
         *
         * @description
         * Current milliseconds mock time.
         */
        self.defer.now = 0;

        self.defer.cancel = function (deferId) {
          var fnIndex;

          angular.forEach(self.deferredFns, function (fn, index) {
            if (fn.id === deferId) fnIndex = index;
          });

          if (angular.isDefined(fnIndex)) {
            self.deferredFns.splice(fnIndex, 1);
            return true;
          }

          return false;
        };

        /**
         * @name $browser#defer.flush
         *
         * @description
         * Flushes all pending requests and executes the defer callbacks.
         *
         * @param {number=} number of milliseconds to flush. See {@link #defer.now}
         */
        self.defer.flush = function (delay) {
          var nextTime;

          if (angular.isDefined(delay)) {
            // A delay was passed so compute the next time
            nextTime = self.defer.now + delay;
          } else {
            if (self.deferredFns.length) {
              // No delay was passed so set the next time so that it clears the deferred queue
              nextTime = self.deferredFns[self.deferredFns.length - 1].time;
            } else {
              // No delay passed, but there are no deferred tasks so flush - indicates an error!
              throw new Error('No deferred tasks to be flushed');
            }
          }

          while (self.deferredFns.length && self.deferredFns[0].time <= nextTime) {
            // Increment the time and call the next deferred function
            self.defer.now = self.deferredFns[0].time;
            self.deferredFns.shift().fn();
          }

          // Ensure that the current time is correct
          self.defer.now = nextTime;
        };

        self.$$baseHref = '/';
        self.baseHref = function () {
          return this.$$baseHref;
        };
      };
      angular.mock.$Browser.prototype = {

        /**
         * @name $browser#poll
         *
         * @description
         * run all fns in pollFns
         */
        poll: function poll() {
          angular.forEach(this.pollFns, function (pollFn) {
            pollFn();
          });
        },

        url: function (url, replace, state) {
          if (angular.isUndefined(state)) {
            state = null;
          }
          if (url) {
            this.$$url = url;
            // Native pushState serializes & copies the object; simulate it.
            this.$$state = angular.copy(state);
            return this;
          }

          return this.$$url;
        },

        state: function () {
          return this.$$state;
        },

        notifyWhenNoOutstandingRequests: function (fn) {
          fn();
        }
      };

      /**
       * @ngdoc provider
       * @name $exceptionHandlerProvider
       *
       * @description
       * Configures the mock implementation of {@link ng.$exceptionHandler} to rethrow or to log errors
       * passed to the `$exceptionHandler`.
       */

      /**
       * @ngdoc service
       * @name $exceptionHandler
       *
       * @description
       * Mock implementation of {@link ng.$exceptionHandler} that rethrows or logs errors passed
       * to it. See {@link ngMock.$exceptionHandlerProvider $exceptionHandlerProvider} for configuration
       * information.
       *
       *
       * ```js
       *   describe('$exceptionHandlerProvider', function() {
       *
       *     it('should capture log messages and exceptions', function() {
       *
       *       module(function($exceptionHandlerProvider) {
       *         $exceptionHandlerProvider.mode('log');
       *       });
       *
       *       inject(function($log, $exceptionHandler, $timeout) {
       *         $timeout(function() { $log.log(1); });
       *         $timeout(function() { $log.log(2); throw 'banana peel'; });
       *         $timeout(function() { $log.log(3); });
       *         expect($exceptionHandler.errors).toEqual([]);
       *         expect($log.assertEmpty());
       *         $timeout.flush();
       *         expect($exceptionHandler.errors).toEqual(['banana peel']);
       *         expect($log.log.logs).toEqual([[1], [2], [3]]);
       *       });
       *     });
       *   });
       * ```
       */

      angular.mock.$ExceptionHandlerProvider = function () {
        var handler;

        /**
         * @ngdoc method
         * @name $exceptionHandlerProvider#mode
         *
         * @description
         * Sets the logging mode.
         *
         * @param {string} mode Mode of operation, defaults to `rethrow`.
         *
         *   - `log`: Sometimes it is desirable to test that an error is thrown, for this case the `log`
         *     mode stores an array of errors in `$exceptionHandler.errors`, to allow later assertion of
         *     them. See {@link ngMock.$log#assertEmpty assertEmpty()} and
         *     {@link ngMock.$log#reset reset()}.
         *   - `rethrow`: If any errors are passed to the handler in tests, it typically means that there
         *     is a bug in the application or test, so this mock will make these tests fail. For any
         *     implementations that expect exceptions to be thrown, the `rethrow` mode will also maintain
         *     a log of thrown errors in `$exceptionHandler.errors`.
         */
        this.mode = function (mode) {

          switch (mode) {
            case 'log':
            case 'rethrow':
              var errors = [];
              handler = function (e) {
                if (arguments.length === 1) {
                  errors.push(e);
                } else {
                  errors.push([].slice.call(arguments, 0));
                }
                if (mode === 'rethrow') {
                  throw e;
                }
              };
              handler.errors = errors;
              break;
            default:
              throw new Error('Unknown mode \'' + mode + '\', only \'log\'/\'rethrow\' modes are allowed!');
          }
        };

        this.$get = function () {
          return handler;
        };

        this.mode('rethrow');
      };

      /**
       * @ngdoc service
       * @name $log
       *
       * @description
       * Mock implementation of {@link ng.$log} that gathers all logged messages in arrays
       * (one array per logging level). These arrays are exposed as `logs` property of each of the
       * level-specific log function, e.g. for level `error` the array is exposed as `$log.error.logs`.
       *
       */
      angular.mock.$LogProvider = function () {
        var debug = true;

        function concat(array1, array2, index) {
          return array1.concat(Array.prototype.slice.call(array2, index));
        }

        this.debugEnabled = function (flag) {
          if (angular.isDefined(flag)) {
            debug = flag;
            return this;
          } else {
            return debug;
          }
        };

        this.$get = function () {
          var $log = {
            log: function () {
              $log.log.logs.push(concat([], arguments, 0));
            },
            warn: function () {
              $log.warn.logs.push(concat([], arguments, 0));
            },
            info: function () {
              $log.info.logs.push(concat([], arguments, 0));
            },
            error: function () {
              $log.error.logs.push(concat([], arguments, 0));
            },
            debug: function () {
              if (debug) {
                $log.debug.logs.push(concat([], arguments, 0));
              }
            }
          };

          /**
           * @ngdoc method
           * @name $log#reset
           *
           * @description
           * Reset all of the logging arrays to empty.
           */
          $log.reset = function () {
            /**
             * @ngdoc property
             * @name $log#log.logs
             *
             * @description
             * Array of messages logged using {@link ng.$log#log `log()`}.
             *
             * @example
             * ```js
             * $log.log('Some Log');
             * var first = $log.log.logs.unshift();
             * ```
             */
            $log.log.logs = [];
            /**
             * @ngdoc property
             * @name $log#info.logs
             *
             * @description
             * Array of messages logged using {@link ng.$log#info `info()`}.
             *
             * @example
             * ```js
             * $log.info('Some Info');
             * var first = $log.info.logs.unshift();
             * ```
             */
            $log.info.logs = [];
            /**
             * @ngdoc property
             * @name $log#warn.logs
             *
             * @description
             * Array of messages logged using {@link ng.$log#warn `warn()`}.
             *
             * @example
             * ```js
             * $log.warn('Some Warning');
             * var first = $log.warn.logs.unshift();
             * ```
             */
            $log.warn.logs = [];
            /**
             * @ngdoc property
             * @name $log#error.logs
             *
             * @description
             * Array of messages logged using {@link ng.$log#error `error()`}.
             *
             * @example
             * ```js
             * $log.error('Some Error');
             * var first = $log.error.logs.unshift();
             * ```
             */
            $log.error.logs = [];
            /**
            * @ngdoc property
            * @name $log#debug.logs
            *
            * @description
            * Array of messages logged using {@link ng.$log#debug `debug()`}.
            *
            * @example
            * ```js
            * $log.debug('Some Error');
            * var first = $log.debug.logs.unshift();
            * ```
            */
            $log.debug.logs = [];
          };

          /**
           * @ngdoc method
           * @name $log#assertEmpty
           *
           * @description
           * Assert that all of the logging methods have no logged messages. If any messages are present,
           * an exception is thrown.
           */
          $log.assertEmpty = function () {
            var errors = [];
            angular.forEach(['error', 'warn', 'info', 'log', 'debug'], function (logLevel) {
              angular.forEach($log[logLevel].logs, function (log) {
                angular.forEach(log, function (logItem) {
                  errors.push('MOCK $log (' + logLevel + '): ' + String(logItem) + '\n' + (logItem.stack || ''));
                });
              });
            });
            if (errors.length) {
              errors.unshift('Expected $log to be empty! Either a message was logged unexpectedly, or ' + 'an expected log message was not checked and removed:');
              errors.push('');
              throw new Error(errors.join('\n---------\n'));
            }
          };

          $log.reset();
          return $log;
        };
      };

      /**
       * @ngdoc service
       * @name $interval
       *
       * @description
       * Mock implementation of the $interval service.
       *
       * Use {@link ngMock.$interval#flush `$interval.flush(millis)`} to
       * move forward by `millis` milliseconds and trigger any functions scheduled to run in that
       * time.
       *
       * @param {function()} fn A function that should be called repeatedly.
       * @param {number} delay Number of milliseconds between each function call.
       * @param {number=} [count=0] Number of times to repeat. If not set, or 0, will repeat
       *   indefinitely.
       * @param {boolean=} [invokeApply=true] If set to `false` skips model dirty checking, otherwise
       *   will invoke `fn` within the {@link ng.$rootScope.Scope#$apply $apply} block.
       * @param {...*=} Pass additional parameters to the executed function.
       * @returns {promise} A promise which will be notified on each iteration.
       */
      angular.mock.$IntervalProvider = function () {
        this.$get = ['$browser', '$rootScope', '$q', '$$q', function ($browser, $rootScope, $q, $$q) {
          var repeatFns = [],
              nextRepeatId = 0,
              now = 0;

          var $interval = function (fn, delay, count, invokeApply) {
            var hasParams = arguments.length > 4,
                args = hasParams ? Array.prototype.slice.call(arguments, 4) : [],
                iteration = 0,
                skipApply = angular.isDefined(invokeApply) && !invokeApply,
                deferred = (skipApply ? $$q : $q).defer(),
                promise = deferred.promise;

            count = angular.isDefined(count) ? count : 0;
            promise.then(null, null, !hasParams ? fn : function () {
              fn.apply(null, args);
            });

            promise.$$intervalId = nextRepeatId;

            function tick() {
              deferred.notify(iteration++);

              if (count > 0 && iteration >= count) {
                var fnIndex;
                deferred.resolve(iteration);

                angular.forEach(repeatFns, function (fn, index) {
                  if (fn.id === promise.$$intervalId) fnIndex = index;
                });

                if (angular.isDefined(fnIndex)) {
                  repeatFns.splice(fnIndex, 1);
                }
              }

              if (skipApply) {
                $browser.defer.flush();
              } else {
                $rootScope.$apply();
              }
            }

            repeatFns.push({
              nextTime: now + delay,
              delay: delay,
              fn: tick,
              id: nextRepeatId,
              deferred: deferred
            });
            repeatFns.sort(function (a, b) {
              return a.nextTime - b.nextTime;
            });

            nextRepeatId++;
            return promise;
          };
          /**
           * @ngdoc method
           * @name $interval#cancel
           *
           * @description
           * Cancels a task associated with the `promise`.
           *
           * @param {promise} promise A promise from calling the `$interval` function.
           * @returns {boolean} Returns `true` if the task was successfully cancelled.
           */
          $interval.cancel = function (promise) {
            if (!promise) return false;
            var fnIndex;

            angular.forEach(repeatFns, function (fn, index) {
              if (fn.id === promise.$$intervalId) fnIndex = index;
            });

            if (angular.isDefined(fnIndex)) {
              repeatFns[fnIndex].deferred.reject('canceled');
              repeatFns.splice(fnIndex, 1);
              return true;
            }

            return false;
          };

          /**
           * @ngdoc method
           * @name $interval#flush
           * @description
           *
           * Runs interval tasks scheduled to be run in the next `millis` milliseconds.
           *
           * @param {number=} millis maximum timeout amount to flush up until.
           *
           * @return {number} The amount of time moved forward.
           */
          $interval.flush = function (millis) {
            now += millis;
            while (repeatFns.length && repeatFns[0].nextTime <= now) {
              var task = repeatFns[0];
              task.fn();
              task.nextTime += task.delay;
              repeatFns.sort(function (a, b) {
                return a.nextTime - b.nextTime;
              });
            }
            return millis;
          };

          return $interval;
        }];
      };

      function jsonStringToDate(string) {
        // The R_ISO8061_STR regex is never going to fit into the 100 char limit!
        // eslit-disable-next-line max-len
        var R_ISO8061_STR = /^(-?\d{4})-?(\d\d)-?(\d\d)(?:T(\d\d)(?::?(\d\d)(?::?(\d\d)(?:\.(\d{3}))?)?)?(Z|([+-])(\d\d):?(\d\d)))?$/;

        var match;
        if (match = string.match(R_ISO8061_STR)) {
          var date = new Date(0),
              tzHour = 0,
              tzMin = 0;
          if (match[9]) {
            tzHour = toInt(match[9] + match[10]);
            tzMin = toInt(match[9] + match[11]);
          }
          date.setUTCFullYear(toInt(match[1]), toInt(match[2]) - 1, toInt(match[3]));
          date.setUTCHours(toInt(match[4] || 0) - tzHour, toInt(match[5] || 0) - tzMin, toInt(match[6] || 0), toInt(match[7] || 0));
          return date;
        }
        return string;
      }

      function toInt(str) {
        return parseInt(str, 10);
      }

      function padNumberInMock(num, digits, trim) {
        var neg = '';
        if (num < 0) {
          neg = '-';
          num = -num;
        }
        num = '' + num;
        while (num.length < digits) num = '0' + num;
        if (trim) {
          num = num.substr(num.length - digits);
        }
        return neg + num;
      }

      /**
       * @ngdoc type
       * @name angular.mock.TzDate
       * @description
       *
       * *NOTE*: this is not an injectable instance, just a globally available mock class of `Date`.
       *
       * Mock of the Date type which has its timezone specified via constructor arg.
       *
       * The main purpose is to create Date-like instances with timezone fixed to the specified timezone
       * offset, so that we can test code that depends on local timezone settings without dependency on
       * the time zone settings of the machine where the code is running.
       *
       * @param {number} offset Offset of the *desired* timezone in hours (fractions will be honored)
       * @param {(number|string)} timestamp Timestamp representing the desired time in *UTC*
       *
       * @example
       * !!!! WARNING !!!!!
       * This is not a complete Date object so only methods that were implemented can be called safely.
       * To make matters worse, TzDate instances inherit stuff from Date via a prototype.
       *
       * We do our best to intercept calls to "unimplemented" methods, but since the list of methods is
       * incomplete we might be missing some non-standard methods. This can result in errors like:
       * "Date.prototype.foo called on incompatible Object".
       *
       * ```js
       * var newYearInBratislava = new TzDate(-1, '2009-12-31T23:00:00Z');
       * newYearInBratislava.getTimezoneOffset() => -60;
       * newYearInBratislava.getFullYear() => 2010;
       * newYearInBratislava.getMonth() => 0;
       * newYearInBratislava.getDate() => 1;
       * newYearInBratislava.getHours() => 0;
       * newYearInBratislava.getMinutes() => 0;
       * newYearInBratislava.getSeconds() => 0;
       * ```
       *
       */
      angular.mock.TzDate = function (offset, timestamp) {
        var self = new Date(0);
        if (angular.isString(timestamp)) {
          var tsStr = timestamp;

          self.origDate = jsonStringToDate(timestamp);

          timestamp = self.origDate.getTime();
          if (isNaN(timestamp)) {
            // eslint-disable-next-line no-throw-literal
            throw {
              name: 'Illegal Argument',
              message: 'Arg \'' + tsStr + '\' passed into TzDate constructor is not a valid date string'
            };
          }
        } else {
          self.origDate = new Date(timestamp);
        }

        var localOffset = new Date(timestamp).getTimezoneOffset();
        self.offsetDiff = localOffset * 60 * 1000 - offset * 1000 * 60 * 60;
        self.date = new Date(timestamp + self.offsetDiff);

        self.getTime = function () {
          return self.date.getTime() - self.offsetDiff;
        };

        self.toLocaleDateString = function () {
          return self.date.toLocaleDateString();
        };

        self.getFullYear = function () {
          return self.date.getFullYear();
        };

        self.getMonth = function () {
          return self.date.getMonth();
        };

        self.getDate = function () {
          return self.date.getDate();
        };

        self.getHours = function () {
          return self.date.getHours();
        };

        self.getMinutes = function () {
          return self.date.getMinutes();
        };

        self.getSeconds = function () {
          return self.date.getSeconds();
        };

        self.getMilliseconds = function () {
          return self.date.getMilliseconds();
        };

        self.getTimezoneOffset = function () {
          return offset * 60;
        };

        self.getUTCFullYear = function () {
          return self.origDate.getUTCFullYear();
        };

        self.getUTCMonth = function () {
          return self.origDate.getUTCMonth();
        };

        self.getUTCDate = function () {
          return self.origDate.getUTCDate();
        };

        self.getUTCHours = function () {
          return self.origDate.getUTCHours();
        };

        self.getUTCMinutes = function () {
          return self.origDate.getUTCMinutes();
        };

        self.getUTCSeconds = function () {
          return self.origDate.getUTCSeconds();
        };

        self.getUTCMilliseconds = function () {
          return self.origDate.getUTCMilliseconds();
        };

        self.getDay = function () {
          return self.date.getDay();
        };

        // provide this method only on browsers that already have it
        if (self.toISOString) {
          self.toISOString = function () {
            return padNumberInMock(self.origDate.getUTCFullYear(), 4) + '-' + padNumberInMock(self.origDate.getUTCMonth() + 1, 2) + '-' + padNumberInMock(self.origDate.getUTCDate(), 2) + 'T' + padNumberInMock(self.origDate.getUTCHours(), 2) + ':' + padNumberInMock(self.origDate.getUTCMinutes(), 2) + ':' + padNumberInMock(self.origDate.getUTCSeconds(), 2) + '.' + padNumberInMock(self.origDate.getUTCMilliseconds(), 3) + 'Z';
          };
        }

        //hide all methods not implemented in this mock that the Date prototype exposes
        var unimplementedMethods = ['getUTCDay', 'getYear', 'setDate', 'setFullYear', 'setHours', 'setMilliseconds', 'setMinutes', 'setMonth', 'setSeconds', 'setTime', 'setUTCDate', 'setUTCFullYear', 'setUTCHours', 'setUTCMilliseconds', 'setUTCMinutes', 'setUTCMonth', 'setUTCSeconds', 'setYear', 'toDateString', 'toGMTString', 'toJSON', 'toLocaleFormat', 'toLocaleString', 'toLocaleTimeString', 'toSource', 'toString', 'toTimeString', 'toUTCString', 'valueOf'];

        angular.forEach(unimplementedMethods, function (methodName) {
          self[methodName] = function () {
            throw new Error('Method \'' + methodName + '\' is not implemented in the TzDate mock');
          };
        });

        return self;
      };

      //make "tzDateInstance instanceof Date" return true
      angular.mock.TzDate.prototype = Date.prototype;

      /**
       * @ngdoc service
       * @name $animate
       *
       * @description
       * Mock implementation of the {@link ng.$animate `$animate`} service. Exposes two additional methods
       * for testing animations.
       *
       * You need to require the `ngAnimateMock` module in your test suite for instance `beforeEach(module('ngAnimateMock'))`
       */
      angular.mock.animate = angular.module('ngAnimateMock', ['ng']).config(['$provide', function ($provide) {

        $provide.factory('$$forceReflow', function () {
          function reflowFn() {
            reflowFn.totalReflows++;
          }
          reflowFn.totalReflows = 0;
          return reflowFn;
        });

        $provide.factory('$$animateAsyncRun', function () {
          var queue = [];
          var queueFn = function () {
            return function (fn) {
              queue.push(fn);
            };
          };
          queueFn.flush = function () {
            if (queue.length === 0) return false;

            for (var i = 0; i < queue.length; i++) {
              queue[i]();
            }
            queue = [];

            return true;
          };
          return queueFn;
        });

        $provide.decorator('$$animateJs', ['$delegate', function ($delegate) {
          var runners = [];

          var animateJsConstructor = function () {
            var animator = $delegate.apply($delegate, arguments);
            // If no javascript animation is found, animator is undefined
            if (animator) {
              runners.push(animator);
            }
            return animator;
          };

          animateJsConstructor.$closeAndFlush = function () {
            runners.forEach(function (runner) {
              runner.end();
            });
            runners = [];
          };

          return animateJsConstructor;
        }]);

        $provide.decorator('$animateCss', ['$delegate', function ($delegate) {
          var runners = [];

          var animateCssConstructor = function (element, options) {
            var animator = $delegate(element, options);
            runners.push(animator);
            return animator;
          };

          animateCssConstructor.$closeAndFlush = function () {
            runners.forEach(function (runner) {
              runner.end();
            });
            runners = [];
          };

          return animateCssConstructor;
        }]);

        $provide.decorator('$animate', ['$delegate', '$timeout', '$browser', '$$rAF', '$animateCss', '$$animateJs', '$$forceReflow', '$$animateAsyncRun', '$rootScope', function ($delegate, $timeout, $browser, $$rAF, $animateCss, $$animateJs, $$forceReflow, $$animateAsyncRun, $rootScope) {
          var animate = {
            queue: [],
            cancel: $delegate.cancel,
            on: $delegate.on,
            off: $delegate.off,
            pin: $delegate.pin,
            get reflows() {
              return $$forceReflow.totalReflows;
            },
            enabled: $delegate.enabled,
            /**
             * @ngdoc method
             * @name $animate#closeAndFlush
             * @description
             *
             * This method will close all pending animations (both {@link ngAnimate#javascript-based-animations Javascript}
             * and {@link ngAnimate.$animateCss CSS}) and it will also flush any remaining animation frames and/or callbacks.
             */
            closeAndFlush: function () {
              // we allow the flush command to swallow the errors
              // because depending on whether CSS or JS animations are
              // used, there may not be a RAF flush. The primary flush
              // at the end of this function must throw an exception
              // because it will track if there were pending animations
              this.flush(true);
              $animateCss.$closeAndFlush();
              $$animateJs.$closeAndFlush();
              this.flush();
            },
            /**
             * @ngdoc method
             * @name $animate#flush
             * @description
             *
             * This method is used to flush the pending callbacks and animation frames to either start
             * an animation or conclude an animation. Note that this will not actually close an
             * actively running animation (see {@link ngMock.$animate#closeAndFlush `closeAndFlush()`} for that).
             */
            flush: function (hideErrors) {
              $rootScope.$digest();

              var doNextRun,
                  somethingFlushed = false;
              do {
                doNextRun = false;

                if ($$rAF.queue.length) {
                  $$rAF.flush();
                  doNextRun = somethingFlushed = true;
                }

                if ($$animateAsyncRun.flush()) {
                  doNextRun = somethingFlushed = true;
                }
              } while (doNextRun);

              if (!somethingFlushed && !hideErrors) {
                throw new Error('No pending animations ready to be closed or flushed');
              }

              $rootScope.$digest();
            }
          };

          angular.forEach(['animate', 'enter', 'leave', 'move', 'addClass', 'removeClass', 'setClass'], function (method) {
            animate[method] = function () {
              animate.queue.push({
                event: method,
                element: arguments[0],
                options: arguments[arguments.length - 1],
                args: arguments
              });
              return $delegate[method].apply($delegate, arguments);
            };
          });

          return animate;
        }]);
      }]);

      /**
       * @ngdoc function
       * @name angular.mock.dump
       * @description
       *
       * *NOTE*: This is not an injectable instance, just a globally available function.
       *
       * Method for serializing common angular objects (scope, elements, etc..) into strings.
       * It is useful for logging objects to the console when debugging.
       *
       * @param {*} object - any object to turn into string.
       * @return {string} a serialized string of the argument
       */
      angular.mock.dump = function (object) {
        return serialize(object);

        function serialize(object) {
          var out;

          if (angular.isElement(object)) {
            object = angular.element(object);
            out = angular.element('<div></div>');
            angular.forEach(object, function (element) {
              out.append(angular.element(element).clone());
            });
            out = out.html();
          } else if (angular.isArray(object)) {
            out = [];
            angular.forEach(object, function (o) {
              out.push(serialize(o));
            });
            out = '[ ' + out.join(', ') + ' ]';
          } else if (angular.isObject(object)) {
            if (angular.isFunction(object.$eval) && angular.isFunction(object.$apply)) {
              out = serializeScope(object);
            } else if (object instanceof Error) {
              out = object.stack || '' + object.name + ': ' + object.message;
            } else {
              // TODO(i): this prevents methods being logged,
              // we should have a better way to serialize objects
              out = angular.toJson(object, true);
            }
          } else {
            out = String(object);
          }

          return out;
        }

        function serializeScope(scope, offset) {
          offset = offset || '  ';
          var log = [offset + 'Scope(' + scope.$id + '): {'];
          for (var key in scope) {
            if (Object.prototype.hasOwnProperty.call(scope, key) && !key.match(/^(\$|this)/)) {
              log.push('  ' + key + ': ' + angular.toJson(scope[key]));
            }
          }
          var child = scope.$$childHead;
          while (child) {
            log.push(serializeScope(child, offset + '  '));
            child = child.$$nextSibling;
          }
          log.push('}');
          return log.join('\n' + offset);
        }
      };

      /**
       * @ngdoc service
       * @name $httpBackend
       * @description
       * Fake HTTP backend implementation suitable for unit testing applications that use the
       * {@link ng.$http $http service}.
       *
       * <div class="alert alert-info">
       * **Note**: For fake HTTP backend implementation suitable for end-to-end testing or backend-less
       * development please see {@link ngMockE2E.$httpBackend e2e $httpBackend mock}.
       * </div>
       *
       * During unit testing, we want our unit tests to run quickly and have no external dependencies so
       * we don’t want to send [XHR](https://developer.mozilla.org/en/xmlhttprequest) or
       * [JSONP](http://en.wikipedia.org/wiki/JSONP) requests to a real server. All we really need is
       * to verify whether a certain request has been sent or not, or alternatively just let the
       * application make requests, respond with pre-trained responses and assert that the end result is
       * what we expect it to be.
       *
       * This mock implementation can be used to respond with static or dynamic responses via the
       * `expect` and `when` apis and their shortcuts (`expectGET`, `whenPOST`, etc).
       *
       * When an Angular application needs some data from a server, it calls the $http service, which
       * sends the request to a real server using $httpBackend service. With dependency injection, it is
       * easy to inject $httpBackend mock (which has the same API as $httpBackend) and use it to verify
       * the requests and respond with some testing data without sending a request to a real server.
       *
       * There are two ways to specify what test data should be returned as http responses by the mock
       * backend when the code under test makes http requests:
       *
       * - `$httpBackend.expect` - specifies a request expectation
       * - `$httpBackend.when` - specifies a backend definition
       *
       *
       * ## Request Expectations vs Backend Definitions
       *
       * Request expectations provide a way to make assertions about requests made by the application and
       * to define responses for those requests. The test will fail if the expected requests are not made
       * or they are made in the wrong order.
       *
       * Backend definitions allow you to define a fake backend for your application which doesn't assert
       * if a particular request was made or not, it just returns a trained response if a request is made.
       * The test will pass whether or not the request gets made during testing.
       *
       *
       * <table class="table">
       *   <tr><th width="220px"></th><th>Request expectations</th><th>Backend definitions</th></tr>
       *   <tr>
       *     <th>Syntax</th>
       *     <td>.expect(...).respond(...)</td>
       *     <td>.when(...).respond(...)</td>
       *   </tr>
       *   <tr>
       *     <th>Typical usage</th>
       *     <td>strict unit tests</td>
       *     <td>loose (black-box) unit testing</td>
       *   </tr>
       *   <tr>
       *     <th>Fulfills multiple requests</th>
       *     <td>NO</td>
       *     <td>YES</td>
       *   </tr>
       *   <tr>
       *     <th>Order of requests matters</th>
       *     <td>YES</td>
       *     <td>NO</td>
       *   </tr>
       *   <tr>
       *     <th>Request required</th>
       *     <td>YES</td>
       *     <td>NO</td>
       *   </tr>
       *   <tr>
       *     <th>Response required</th>
       *     <td>optional (see below)</td>
       *     <td>YES</td>
       *   </tr>
       * </table>
       *
       * In cases where both backend definitions and request expectations are specified during unit
       * testing, the request expectations are evaluated first.
       *
       * If a request expectation has no response specified, the algorithm will search your backend
       * definitions for an appropriate response.
       *
       * If a request didn't match any expectation or if the expectation doesn't have the response
       * defined, the backend definitions are evaluated in sequential order to see if any of them match
       * the request. The response from the first matched definition is returned.
       *
       *
       * ## Flushing HTTP requests
       *
       * The $httpBackend used in production always responds to requests asynchronously. If we preserved
       * this behavior in unit testing, we'd have to create async unit tests, which are hard to write,
       * to follow and to maintain. But neither can the testing mock respond synchronously; that would
       * change the execution of the code under test. For this reason, the mock $httpBackend has a
       * `flush()` method, which allows the test to explicitly flush pending requests. This preserves
       * the async api of the backend, while allowing the test to execute synchronously.
       *
       *
       * ## Unit testing with mock $httpBackend
       * The following code shows how to setup and use the mock backend when unit testing a controller.
       * First we create the controller under test:
       *
        ```js
        // The module code
        angular
          .module('MyApp', [])
          .controller('MyController', MyController);
      
        // The controller code
        function MyController($scope, $http) {
          var authToken;
      
          $http.get('/auth.py').then(function(response) {
            authToken = response.headers('A-Token');
            $scope.user = response.data;
          });
      
          $scope.saveMessage = function(message) {
            var headers = { 'Authorization': authToken };
            $scope.status = 'Saving...';
      
            $http.post('/add-msg.py', message, { headers: headers } ).then(function(response) {
              $scope.status = '';
            })['catch'](function() {
              $scope.status = 'Failed...';
            });
          };
        }
        ```
       *
       * Now we setup the mock backend and create the test specs:
       *
        ```js
          // testing controller
          describe('MyController', function() {
             var $httpBackend, $rootScope, createController, authRequestHandler;
      
             // Set up the module
             beforeEach(module('MyApp'));
      
             beforeEach(inject(function($injector) {
               // Set up the mock http service responses
               $httpBackend = $injector.get('$httpBackend');
               // backend definition common for all tests
               authRequestHandler = $httpBackend.when('GET', '/auth.py')
                                      .respond({userId: 'userX'}, {'A-Token': 'xxx'});
      
               // Get hold of a scope (i.e. the root scope)
               $rootScope = $injector.get('$rootScope');
               // The $controller service is used to create instances of controllers
               var $controller = $injector.get('$controller');
      
               createController = function() {
                 return $controller('MyController', {'$scope' : $rootScope });
               };
             }));
      
      
             afterEach(function() {
               $httpBackend.verifyNoOutstandingExpectation();
               $httpBackend.verifyNoOutstandingRequest();
             });
      
      
             it('should fetch authentication token', function() {
               $httpBackend.expectGET('/auth.py');
               var controller = createController();
               $httpBackend.flush();
             });
      
      
             it('should fail authentication', function() {
      
               // Notice how you can change the response even after it was set
               authRequestHandler.respond(401, '');
      
               $httpBackend.expectGET('/auth.py');
               var controller = createController();
               $httpBackend.flush();
               expect($rootScope.status).toBe('Failed...');
             });
      
      
             it('should send msg to server', function() {
               var controller = createController();
               $httpBackend.flush();
      
               // now you don’t care about the authentication, but
               // the controller will still send the request and
               // $httpBackend will respond without you having to
               // specify the expectation and response for this request
      
               $httpBackend.expectPOST('/add-msg.py', 'message content').respond(201, '');
               $rootScope.saveMessage('message content');
               expect($rootScope.status).toBe('Saving...');
               $httpBackend.flush();
               expect($rootScope.status).toBe('');
             });
      
      
             it('should send auth header', function() {
               var controller = createController();
               $httpBackend.flush();
      
               $httpBackend.expectPOST('/add-msg.py', undefined, function(headers) {
                 // check if the header was sent, if it wasn't the expectation won't
                 // match the request and the test will fail
                 return headers['Authorization'] === 'xxx';
               }).respond(201, '');
      
               $rootScope.saveMessage('whatever');
               $httpBackend.flush();
             });
          });
        ```
       *
       * ## Dynamic responses
       *
       * You define a response to a request by chaining a call to `respond()` onto a definition or expectation.
       * If you provide a **callback** as the first parameter to `respond(callback)` then you can dynamically generate
       * a response based on the properties of the request.
       *
       * The `callback` function should be of the form `function(method, url, data, headers, params)`.
       *
       * ### Query parameters
       *
       * By default, query parameters on request URLs are parsed into the `params` object. So a request URL
       * of `/list?q=searchstr&orderby=-name` would set `params` to be `{q: 'searchstr', orderby: '-name'}`.
       *
       * ### Regex parameter matching
       *
       * If an expectation or definition uses a **regex** to match the URL, you can provide an array of **keys** via a
       * `params` argument. The index of each **key** in the array will match the index of a **group** in the
       * **regex**.
       *
       * The `params` object in the **callback** will now have properties with these keys, which hold the value of the
       * corresponding **group** in the **regex**.
       *
       * This also applies to the `when` and `expect` shortcut methods.
       *
       *
       * ```js
       *   $httpBackend.expect('GET', /\/user\/(.+)/, undefined, undefined, ['id'])
       *     .respond(function(method, url, data, headers, params) {
       *       // for requested url of '/user/1234' params is {id: '1234'}
       *     });
       *
       *   $httpBackend.whenPATCH(/\/user\/(.+)\/article\/(.+)/, undefined, undefined, ['user', 'article'])
       *     .respond(function(method, url, data, headers, params) {
       *       // for url of '/user/1234/article/567' params is {user: '1234', article: '567'}
       *     });
       * ```
       *
       * ## Matching route requests
       *
       * For extra convenience, `whenRoute` and `expectRoute` shortcuts are available. These methods offer colon
       * delimited matching of the url path, ignoring the query string. This allows declarations
       * similar to how application routes are configured with `$routeProvider`. Because these methods convert
       * the definition url to regex, declaration order is important. Combined with query parameter parsing,
       * the following is possible:
       *
        ```js
          $httpBackend.whenRoute('GET', '/users/:id')
            .respond(function(method, url, data, headers, params) {
              return [200, MockUserList[Number(params.id)]];
            });
      
          $httpBackend.whenRoute('GET', '/users')
            .respond(function(method, url, data, headers, params) {
              var userList = angular.copy(MockUserList),
                defaultSort = 'lastName',
                count, pages, isPrevious, isNext;
      
              // paged api response '/v1/users?page=2'
              params.page = Number(params.page) || 1;
      
              // query for last names '/v1/users?q=Archer'
              if (params.q) {
                userList = $filter('filter')({lastName: params.q});
              }
      
              pages = Math.ceil(userList.length / pagingLength);
              isPrevious = params.page > 1;
              isNext = params.page < pages;
      
              return [200, {
                count:    userList.length,
                previous: isPrevious,
                next:     isNext,
                // sort field -> '/v1/users?sortBy=firstName'
                results:  $filter('orderBy')(userList, params.sortBy || defaultSort)
                            .splice((params.page - 1) * pagingLength, pagingLength)
              }];
            });
        ```
       */
      angular.mock.$HttpBackendProvider = function () {
        this.$get = ['$rootScope', '$timeout', createHttpBackendMock];
      };

      /**
       * General factory function for $httpBackend mock.
       * Returns instance for unit testing (when no arguments specified):
       *   - passing through is disabled
       *   - auto flushing is disabled
       *
       * Returns instance for e2e testing (when `$delegate` and `$browser` specified):
       *   - passing through (delegating request to real backend) is enabled
       *   - auto flushing is enabled
       *
       * @param {Object=} $delegate Real $httpBackend instance (allow passing through if specified)
       * @param {Object=} $browser Auto-flushing enabled if specified
       * @return {Object} Instance of $httpBackend mock
       */
      function createHttpBackendMock($rootScope, $timeout, $delegate, $browser) {
        var definitions = [],
            expectations = [],
            responses = [],
            responsesPush = angular.bind(responses, responses.push),
            copy = angular.copy;

        function createResponse(status, data, headers, statusText) {
          if (angular.isFunction(status)) return status;

          return function () {
            return angular.isNumber(status) ? [status, data, headers, statusText] : [200, status, data, headers];
          };
        }

        // TODO(vojta): change params to: method, url, data, headers, callback
        function $httpBackend(method, url, data, callback, headers, timeout, withCredentials, responseType, eventHandlers, uploadEventHandlers) {

          var xhr = new MockXhr(),
              expectation = expectations[0],
              wasExpected = false;

          xhr.$$events = eventHandlers;
          xhr.upload.$$events = uploadEventHandlers;

          function prettyPrint(data) {
            return angular.isString(data) || angular.isFunction(data) || data instanceof RegExp ? data : angular.toJson(data);
          }

          function wrapResponse(wrapped) {
            if (!$browser && timeout) {
              if (timeout.then) {
                timeout.then(handleTimeout);
              } else {
                $timeout(handleTimeout, timeout);
              }
            }

            return handleResponse;

            function handleResponse() {
              var response = wrapped.response(method, url, data, headers, wrapped.params(url));
              xhr.$$respHeaders = response[2];
              callback(copy(response[0]), copy(response[1]), xhr.getAllResponseHeaders(), copy(response[3] || ''));
            }

            function handleTimeout() {
              for (var i = 0, ii = responses.length; i < ii; i++) {
                if (responses[i] === handleResponse) {
                  responses.splice(i, 1);
                  callback(-1, undefined, '');
                  break;
                }
              }
            }
          }

          if (expectation && expectation.match(method, url)) {
            if (!expectation.matchData(data)) {
              throw new Error('Expected ' + expectation + ' with different data\n' + 'EXPECTED: ' + prettyPrint(expectation.data) + '\nGOT:      ' + data);
            }

            if (!expectation.matchHeaders(headers)) {
              throw new Error('Expected ' + expectation + ' with different headers\n' + 'EXPECTED: ' + prettyPrint(expectation.headers) + '\nGOT:      ' + prettyPrint(headers));
            }

            expectations.shift();

            if (expectation.response) {
              responses.push(wrapResponse(expectation));
              return;
            }
            wasExpected = true;
          }

          var i = -1,
              definition;
          while (definition = definitions[++i]) {
            if (definition.match(method, url, data, headers || {})) {
              if (definition.response) {
                // if $browser specified, we do auto flush all requests
                ($browser ? $browser.defer : responsesPush)(wrapResponse(definition));
              } else if (definition.passThrough) {
                $delegate(method, url, data, callback, headers, timeout, withCredentials, responseType, eventHandlers, uploadEventHandlers);
              } else throw new Error('No response defined !');
              return;
            }
          }
          throw wasExpected ? new Error('No response defined !') : new Error('Unexpected request: ' + method + ' ' + url + '\n' + (expectation ? 'Expected ' + expectation : 'No more request expected'));
        }

        /**
         * @ngdoc method
         * @name $httpBackend#when
         * @description
         * Creates a new backend definition.
         *
         * @param {string} method HTTP method.
         * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
         *   and returns true if the url matches the current definition.
         * @param {(string|RegExp|function(string))=} data HTTP request body or function that receives
         *   data string and returns true if the data is as expected.
         * @param {(Object|function(Object))=} headers HTTP headers or function that receives http header
         *   object and returns true if the headers match the current definition.
         * @param {(Array)=} keys Array of keys to assign to regex matches in request url described above.
         * @returns {requestHandler} Returns an object with `respond` method that controls how a matched
         *   request is handled. You can save this object for later use and invoke `respond` again in
         *   order to change how a matched request is handled.
         *
         *  - respond –
         *      ```js
         *      {function([status,] data[, headers, statusText])
         *      | function(function(method, url, data, headers, params)}
         *      ```
         *    – The respond method takes a set of static data to be returned or a function that can
         *    return an array containing response status (number), response data (Array|Object|string),
         *    response headers (Object), and the text for the status (string). The respond method returns
         *    the `requestHandler` object for possible overrides.
         */
        $httpBackend.when = function (method, url, data, headers, keys) {
          var definition = new MockHttpExpectation(method, url, data, headers, keys),
              chain = {
            respond: function (status, data, headers, statusText) {
              definition.passThrough = undefined;
              definition.response = createResponse(status, data, headers, statusText);
              return chain;
            }
          };

          if ($browser) {
            chain.passThrough = function () {
              definition.response = undefined;
              definition.passThrough = true;
              return chain;
            };
          }

          definitions.push(definition);
          return chain;
        };

        /**
         * @ngdoc method
         * @name $httpBackend#whenGET
         * @description
         * Creates a new backend definition for GET requests. For more info see `when()`.
         *
         * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
         *   and returns true if the url matches the current definition.
         * @param {(Object|function(Object))=} headers HTTP headers.
         * @param {(Array)=} keys Array of keys to assign to regex matches in request url described above.
         * @returns {requestHandler} Returns an object with `respond` method that controls how a matched
         * request is handled. You can save this object for later use and invoke `respond` again in
         * order to change how a matched request is handled.
         */

        /**
         * @ngdoc method
         * @name $httpBackend#whenHEAD
         * @description
         * Creates a new backend definition for HEAD requests. For more info see `when()`.
         *
         * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
         *   and returns true if the url matches the current definition.
         * @param {(Object|function(Object))=} headers HTTP headers.
         * @param {(Array)=} keys Array of keys to assign to regex matches in request url described above.
         * @returns {requestHandler} Returns an object with `respond` method that controls how a matched
         * request is handled. You can save this object for later use and invoke `respond` again in
         * order to change how a matched request is handled.
         */

        /**
         * @ngdoc method
         * @name $httpBackend#whenDELETE
         * @description
         * Creates a new backend definition for DELETE requests. For more info see `when()`.
         *
         * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
         *   and returns true if the url matches the current definition.
         * @param {(Object|function(Object))=} headers HTTP headers.
         * @param {(Array)=} keys Array of keys to assign to regex matches in request url described above.
         * @returns {requestHandler} Returns an object with `respond` method that controls how a matched
         * request is handled. You can save this object for later use and invoke `respond` again in
         * order to change how a matched request is handled.
         */

        /**
         * @ngdoc method
         * @name $httpBackend#whenPOST
         * @description
         * Creates a new backend definition for POST requests. For more info see `when()`.
         *
         * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
         *   and returns true if the url matches the current definition.
         * @param {(string|RegExp|function(string))=} data HTTP request body or function that receives
         *   data string and returns true if the data is as expected.
         * @param {(Object|function(Object))=} headers HTTP headers.
         * @param {(Array)=} keys Array of keys to assign to regex matches in request url described above.
         * @returns {requestHandler} Returns an object with `respond` method that controls how a matched
         * request is handled. You can save this object for later use and invoke `respond` again in
         * order to change how a matched request is handled.
         */

        /**
         * @ngdoc method
         * @name $httpBackend#whenPUT
         * @description
         * Creates a new backend definition for PUT requests.  For more info see `when()`.
         *
         * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
         *   and returns true if the url matches the current definition.
         * @param {(string|RegExp|function(string))=} data HTTP request body or function that receives
         *   data string and returns true if the data is as expected.
         * @param {(Object|function(Object))=} headers HTTP headers.
         * @param {(Array)=} keys Array of keys to assign to regex matches in request url described above.
         * @returns {requestHandler} Returns an object with `respond` method that controls how a matched
         * request is handled. You can save this object for later use and invoke `respond` again in
         * order to change how a matched request is handled.
         */

        /**
         * @ngdoc method
         * @name $httpBackend#whenJSONP
         * @description
         * Creates a new backend definition for JSONP requests. For more info see `when()`.
         *
         * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
         *   and returns true if the url matches the current definition.
         * @param {(Array)=} keys Array of keys to assign to regex matches in request url described above.
         * @returns {requestHandler} Returns an object with `respond` method that controls how a matched
         * request is handled. You can save this object for later use and invoke `respond` again in
         * order to change how a matched request is handled.
         */
        createShortMethods('when');

        /**
         * @ngdoc method
         * @name $httpBackend#whenRoute
         * @description
         * Creates a new backend definition that compares only with the requested route.
         *
         * @param {string} method HTTP method.
         * @param {string} url HTTP url string that supports colon param matching.
         * @returns {requestHandler} Returns an object with `respond` method that controls how a matched
         * request is handled. You can save this object for later use and invoke `respond` again in
         * order to change how a matched request is handled. See #when for more info.
         */
        $httpBackend.whenRoute = function (method, url) {
          var pathObj = parseRoute(url);
          return $httpBackend.when(method, pathObj.regexp, undefined, undefined, pathObj.keys);
        };

        function parseRoute(url) {
          var ret = {
            regexp: url
          },
              keys = ret.keys = [];

          if (!url || !angular.isString(url)) return ret;

          url = url.replace(/([().])/g, '\\$1').replace(/(\/)?:(\w+)([?*])?/g, function (_, slash, key, option) {
            var optional = option === '?' ? option : null;
            var star = option === '*' ? option : null;
            keys.push({ name: key, optional: !!optional });
            slash = slash || '';
            return '' + (optional ? '' : slash) + '(?:' + (optional ? slash : '') + (star && '(.+?)' || '([^/]+)') + (optional || '') + ')' + (optional || '');
          }).replace(/([/$*])/g, '\\$1');

          ret.regexp = new RegExp('^' + url, 'i');
          return ret;
        }

        /**
         * @ngdoc method
         * @name $httpBackend#expect
         * @description
         * Creates a new request expectation.
         *
         * @param {string} method HTTP method.
         * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
         *   and returns true if the url matches the current definition.
         * @param {(string|RegExp|function(string)|Object)=} data HTTP request body or function that
         *  receives data string and returns true if the data is as expected, or Object if request body
         *  is in JSON format.
         * @param {(Object|function(Object))=} headers HTTP headers or function that receives http header
         *   object and returns true if the headers match the current expectation.
         * @param {(Array)=} keys Array of keys to assign to regex matches in request url described above.
         * @returns {requestHandler} Returns an object with `respond` method that controls how a matched
         *  request is handled. You can save this object for later use and invoke `respond` again in
         *  order to change how a matched request is handled.
         *
         *  - respond –
         *    ```
         *    { function([status,] data[, headers, statusText])
         *    | function(function(method, url, data, headers, params)}
         *    ```
         *    – The respond method takes a set of static data to be returned or a function that can
         *    return an array containing response status (number), response data (Array|Object|string),
         *    response headers (Object), and the text for the status (string). The respond method returns
         *    the `requestHandler` object for possible overrides.
         */
        $httpBackend.expect = function (method, url, data, headers, keys) {
          var expectation = new MockHttpExpectation(method, url, data, headers, keys),
              chain = {
            respond: function (status, data, headers, statusText) {
              expectation.response = createResponse(status, data, headers, statusText);
              return chain;
            }
          };

          expectations.push(expectation);
          return chain;
        };

        /**
         * @ngdoc method
         * @name $httpBackend#expectGET
         * @description
         * Creates a new request expectation for GET requests. For more info see `expect()`.
         *
         * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
         *   and returns true if the url matches the current definition.
         * @param {Object=} headers HTTP headers.
         * @param {(Array)=} keys Array of keys to assign to regex matches in request url described above.
         * @returns {requestHandler} Returns an object with `respond` method that controls how a matched
         * request is handled. You can save this object for later use and invoke `respond` again in
         * order to change how a matched request is handled. See #expect for more info.
         */

        /**
         * @ngdoc method
         * @name $httpBackend#expectHEAD
         * @description
         * Creates a new request expectation for HEAD requests. For more info see `expect()`.
         *
         * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
         *   and returns true if the url matches the current definition.
         * @param {Object=} headers HTTP headers.
         * @param {(Array)=} keys Array of keys to assign to regex matches in request url described above.
         * @returns {requestHandler} Returns an object with `respond` method that controls how a matched
         *   request is handled. You can save this object for later use and invoke `respond` again in
         *   order to change how a matched request is handled.
         */

        /**
         * @ngdoc method
         * @name $httpBackend#expectDELETE
         * @description
         * Creates a new request expectation for DELETE requests. For more info see `expect()`.
         *
         * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
         *   and returns true if the url matches the current definition.
         * @param {Object=} headers HTTP headers.
         * @param {(Array)=} keys Array of keys to assign to regex matches in request url described above.
         * @returns {requestHandler} Returns an object with `respond` method that controls how a matched
         *   request is handled. You can save this object for later use and invoke `respond` again in
         *   order to change how a matched request is handled.
         */

        /**
         * @ngdoc method
         * @name $httpBackend#expectPOST
         * @description
         * Creates a new request expectation for POST requests. For more info see `expect()`.
         *
         * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
         *   and returns true if the url matches the current definition.
         * @param {(string|RegExp|function(string)|Object)=} data HTTP request body or function that
         *  receives data string and returns true if the data is as expected, or Object if request body
         *  is in JSON format.
         * @param {Object=} headers HTTP headers.
         * @param {(Array)=} keys Array of keys to assign to regex matches in request url described above.
         * @returns {requestHandler} Returns an object with `respond` method that controls how a matched
         *   request is handled. You can save this object for later use and invoke `respond` again in
         *   order to change how a matched request is handled.
         */

        /**
         * @ngdoc method
         * @name $httpBackend#expectPUT
         * @description
         * Creates a new request expectation for PUT requests. For more info see `expect()`.
         *
         * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
         *   and returns true if the url matches the current definition.
         * @param {(string|RegExp|function(string)|Object)=} data HTTP request body or function that
         *  receives data string and returns true if the data is as expected, or Object if request body
         *  is in JSON format.
         * @param {Object=} headers HTTP headers.
         * @param {(Array)=} keys Array of keys to assign to regex matches in request url described above.
         * @returns {requestHandler} Returns an object with `respond` method that controls how a matched
         *   request is handled. You can save this object for later use and invoke `respond` again in
         *   order to change how a matched request is handled.
         */

        /**
         * @ngdoc method
         * @name $httpBackend#expectPATCH
         * @description
         * Creates a new request expectation for PATCH requests. For more info see `expect()`.
         *
         * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
         *   and returns true if the url matches the current definition.
         * @param {(string|RegExp|function(string)|Object)=} data HTTP request body or function that
         *  receives data string and returns true if the data is as expected, or Object if request body
         *  is in JSON format.
         * @param {Object=} headers HTTP headers.
         * @param {(Array)=} keys Array of keys to assign to regex matches in request url described above.
         * @returns {requestHandler} Returns an object with `respond` method that controls how a matched
         *   request is handled. You can save this object for later use and invoke `respond` again in
         *   order to change how a matched request is handled.
         */

        /**
         * @ngdoc method
         * @name $httpBackend#expectJSONP
         * @description
         * Creates a new request expectation for JSONP requests. For more info see `expect()`.
         *
         * @param {string|RegExp|function(string)} url HTTP url or function that receives an url
         *   and returns true if the url matches the current definition.
         * @param {(Array)=} keys Array of keys to assign to regex matches in request url described above.
         * @returns {requestHandler} Returns an object with `respond` method that controls how a matched
         *   request is handled. You can save this object for later use and invoke `respond` again in
         *   order to change how a matched request is handled.
         */
        createShortMethods('expect');

        /**
         * @ngdoc method
         * @name $httpBackend#expectRoute
         * @description
         * Creates a new request expectation that compares only with the requested route.
         *
         * @param {string} method HTTP method.
         * @param {string} url HTTP url string that supports colon param matching.
         * @returns {requestHandler} Returns an object with `respond` method that controls how a matched
         * request is handled. You can save this object for later use and invoke `respond` again in
         * order to change how a matched request is handled. See #expect for more info.
         */
        $httpBackend.expectRoute = function (method, url) {
          var pathObj = parseRoute(url);
          return $httpBackend.expect(method, pathObj.regexp, undefined, undefined, pathObj.keys);
        };

        /**
         * @ngdoc method
         * @name $httpBackend#flush
         * @description
         * Flushes pending requests using the trained responses. Requests are flushed in the order they
         * were made, but it is also possible to skip one or more requests (for example to have them
         * flushed later). This is useful for simulating scenarios where responses arrive from the server
         * in any order.
         *
         * If there are no pending requests to flush when the method is called, an exception is thrown (as
         * this is typically a sign of programming error).
         *
         * @param {number=} count - Number of responses to flush. If undefined/null, all pending requests
         *     (starting after `skip`) will be flushed.
         * @param {number=} [skip=0] - Number of pending requests to skip. For example, a value of `5`
         *     would skip the first 5 pending requests and start flushing from the 6th onwards.
         */
        $httpBackend.flush = function (count, skip, digest) {
          if (digest !== false) $rootScope.$digest();

          skip = skip || 0;
          if (skip >= responses.length) throw new Error('No pending request to flush !');

          if (angular.isDefined(count) && count !== null) {
            while (count--) {
              var part = responses.splice(skip, 1);
              if (!part.length) throw new Error('No more pending request to flush !');
              part[0]();
            }
          } else {
            while (responses.length > skip) {
              responses.splice(skip, 1)[0]();
            }
          }
          $httpBackend.verifyNoOutstandingExpectation(digest);
        };

        /**
         * @ngdoc method
         * @name $httpBackend#verifyNoOutstandingExpectation
         * @description
         * Verifies that all of the requests defined via the `expect` api were made. If any of the
         * requests were not made, verifyNoOutstandingExpectation throws an exception.
         *
         * Typically, you would call this method following each test case that asserts requests using an
         * "afterEach" clause.
         *
         * ```js
         *   afterEach($httpBackend.verifyNoOutstandingExpectation);
         * ```
         */
        $httpBackend.verifyNoOutstandingExpectation = function (digest) {
          if (digest !== false) $rootScope.$digest();
          if (expectations.length) {
            throw new Error('Unsatisfied requests: ' + expectations.join(', '));
          }
        };

        /**
         * @ngdoc method
         * @name $httpBackend#verifyNoOutstandingRequest
         * @description
         * Verifies that there are no outstanding requests that need to be flushed.
         *
         * Typically, you would call this method following each test case that asserts requests using an
         * "afterEach" clause.
         *
         * ```js
         *   afterEach($httpBackend.verifyNoOutstandingRequest);
         * ```
         */
        $httpBackend.verifyNoOutstandingRequest = function () {
          if (responses.length) {
            throw new Error('Unflushed requests: ' + responses.length);
          }
        };

        /**
         * @ngdoc method
         * @name $httpBackend#resetExpectations
         * @description
         * Resets all request expectations, but preserves all backend definitions. Typically, you would
         * call resetExpectations during a multiple-phase test when you want to reuse the same instance of
         * $httpBackend mock.
         */
        $httpBackend.resetExpectations = function () {
          expectations.length = 0;
          responses.length = 0;
        };

        return $httpBackend;

        function createShortMethods(prefix) {
          angular.forEach(['GET', 'DELETE', 'JSONP', 'HEAD'], function (method) {
            $httpBackend[prefix + method] = function (url, headers, keys) {
              return $httpBackend[prefix](method, url, undefined, headers, keys);
            };
          });

          angular.forEach(['PUT', 'POST', 'PATCH'], function (method) {
            $httpBackend[prefix + method] = function (url, data, headers, keys) {
              return $httpBackend[prefix](method, url, data, headers, keys);
            };
          });
        }
      }

      function MockHttpExpectation(method, url, data, headers, keys) {

        function getUrlParams(u) {
          var params = u.slice(u.indexOf('?') + 1).split('&');
          return params.sort();
        }

        function compareUrl(u) {
          return url.slice(0, url.indexOf('?')) === u.slice(0, u.indexOf('?')) && getUrlParams(url).join() === getUrlParams(u).join();
        }

        this.data = data;
        this.headers = headers;

        this.match = function (m, u, d, h) {
          if (method !== m) return false;
          if (!this.matchUrl(u)) return false;
          if (angular.isDefined(d) && !this.matchData(d)) return false;
          if (angular.isDefined(h) && !this.matchHeaders(h)) return false;
          return true;
        };

        this.matchUrl = function (u) {
          if (!url) return true;
          if (angular.isFunction(url.test)) return url.test(u);
          if (angular.isFunction(url)) return url(u);
          return url === u || compareUrl(u);
        };

        this.matchHeaders = function (h) {
          if (angular.isUndefined(headers)) return true;
          if (angular.isFunction(headers)) return headers(h);
          return angular.equals(headers, h);
        };

        this.matchData = function (d) {
          if (angular.isUndefined(data)) return true;
          if (data && angular.isFunction(data.test)) return data.test(d);
          if (data && angular.isFunction(data)) return data(d);
          if (data && !angular.isString(data)) {
            return angular.equals(angular.fromJson(angular.toJson(data)), angular.fromJson(d));
          }
          // eslint-disable-next-line eqeqeq
          return data == d;
        };

        this.toString = function () {
          return method + ' ' + url;
        };

        this.params = function (u) {
          return angular.extend(parseQuery(), pathParams());

          function pathParams() {
            var keyObj = {};
            if (!url || !angular.isFunction(url.test) || !keys || keys.length === 0) return keyObj;

            var m = url.exec(u);
            if (!m) return keyObj;
            for (var i = 1, len = m.length; i < len; ++i) {
              var key = keys[i - 1];
              var val = m[i];
              if (key && val) {
                keyObj[key.name || key] = val;
              }
            }

            return keyObj;
          }

          function parseQuery() {
            var obj = {},
                key_value,
                key,
                queryStr = u.indexOf('?') > -1 ? u.substring(u.indexOf('?') + 1) : '';

            angular.forEach(queryStr.split('&'), function (keyValue) {
              if (keyValue) {
                key_value = keyValue.replace(/\+/g, '%20').split('=');
                key = tryDecodeURIComponent(key_value[0]);
                if (angular.isDefined(key)) {
                  var val = angular.isDefined(key_value[1]) ? tryDecodeURIComponent(key_value[1]) : true;
                  if (!hasOwnProperty.call(obj, key)) {
                    obj[key] = val;
                  } else if (angular.isArray(obj[key])) {
                    obj[key].push(val);
                  } else {
                    obj[key] = [obj[key], val];
                  }
                }
              }
            });
            return obj;
          }
          function tryDecodeURIComponent(value) {
            try {
              return decodeURIComponent(value);
            } catch (e) {
              // Ignore any invalid uri component
            }
          }
        };
      }

      function createMockXhr() {
        return new MockXhr();
      }

      function MockXhr() {

        // hack for testing $http, $httpBackend
        MockXhr.$$lastInstance = this;

        this.open = function (method, url, async) {
          this.$$method = method;
          this.$$url = url;
          this.$$async = async;
          this.$$reqHeaders = {};
          this.$$respHeaders = {};
        };

        this.send = function (data) {
          this.$$data = data;
        };

        this.setRequestHeader = function (key, value) {
          this.$$reqHeaders[key] = value;
        };

        this.getResponseHeader = function (name) {
          // the lookup must be case insensitive,
          // that's why we try two quick lookups first and full scan last
          var header = this.$$respHeaders[name];
          if (header) return header;

          name = angular.lowercase(name);
          header = this.$$respHeaders[name];
          if (header) return header;

          header = undefined;
          angular.forEach(this.$$respHeaders, function (headerVal, headerName) {
            if (!header && angular.lowercase(headerName) === name) header = headerVal;
          });
          return header;
        };

        this.getAllResponseHeaders = function () {
          var lines = [];

          angular.forEach(this.$$respHeaders, function (value, key) {
            lines.push(key + ': ' + value);
          });
          return lines.join('\n');
        };

        this.abort = angular.noop;

        // This section simulates the events on a real XHR object (and the upload object)
        // When we are testing $httpBackend (inside the angular project) we make partial use of this
        // but store the events directly ourselves on `$$events`, instead of going through the `addEventListener`
        this.$$events = {};
        this.addEventListener = function (name, listener) {
          if (angular.isUndefined(this.$$events[name])) this.$$events[name] = [];
          this.$$events[name].push(listener);
        };

        this.upload = {
          $$events: {},
          addEventListener: this.addEventListener
        };
      }

      /**
       * @ngdoc service
       * @name $timeout
       * @description
       *
       * This service is just a simple decorator for {@link ng.$timeout $timeout} service
       * that adds a "flush" and "verifyNoPendingTasks" methods.
       */

      angular.mock.$TimeoutDecorator = ['$delegate', '$browser', function ($delegate, $browser) {

        /**
         * @ngdoc method
         * @name $timeout#flush
         * @description
         *
         * Flushes the queue of pending tasks.
         *
         * @param {number=} delay maximum timeout amount to flush up until
         */
        $delegate.flush = function (delay) {
          $browser.defer.flush(delay);
        };

        /**
         * @ngdoc method
         * @name $timeout#verifyNoPendingTasks
         * @description
         *
         * Verifies that there are no pending tasks that need to be flushed.
         */
        $delegate.verifyNoPendingTasks = function () {
          if ($browser.deferredFns.length) {
            throw new Error('Deferred tasks to flush (' + $browser.deferredFns.length + '): ' + formatPendingTasksAsString($browser.deferredFns));
          }
        };

        function formatPendingTasksAsString(tasks) {
          var result = [];
          angular.forEach(tasks, function (task) {
            result.push('{id: ' + task.id + ', time: ' + task.time + '}');
          });

          return result.join(', ');
        }

        return $delegate;
      }];

      angular.mock.$RAFDecorator = ['$delegate', function ($delegate) {
        var rafFn = function (fn) {
          var index = rafFn.queue.length;
          rafFn.queue.push(fn);
          return function () {
            rafFn.queue.splice(index, 1);
          };
        };

        rafFn.queue = [];
        rafFn.supported = $delegate.supported;

        rafFn.flush = function () {
          if (rafFn.queue.length === 0) {
            throw new Error('No rAF callbacks present');
          }

          var length = rafFn.queue.length;
          for (var i = 0; i < length; i++) {
            rafFn.queue[i]();
          }

          rafFn.queue = rafFn.queue.slice(i);
        };

        return rafFn;
      }];

      /**
       *
       */
      var originalRootElement;
      angular.mock.$RootElementProvider = function () {
        this.$get = ['$injector', function ($injector) {
          originalRootElement = angular.element('<div ng-app></div>').data('$injector', $injector);
          return originalRootElement;
        }];
      };

      /**
       * @ngdoc service
       * @name $controller
       * @description
       * A decorator for {@link ng.$controller} with additional `bindings` parameter, useful when testing
       * controllers of directives that use {@link $compile#-bindtocontroller- `bindToController`}.
       *
       * Depending on the value of
       * {@link ng.$compileProvider#preAssignBindingsEnabled `preAssignBindingsEnabled()`}, the properties
       * will be bound before or after invoking the constructor.
       *
       *
       * ## Example
       *
       * ```js
       *
       * // Directive definition ...
       *
       * myMod.directive('myDirective', {
       *   controller: 'MyDirectiveController',
       *   bindToController: {
       *     name: '@'
       *   }
       * });
       *
       *
       * // Controller definition ...
       *
       * myMod.controller('MyDirectiveController', ['$log', function($log) {
       *   this.log = function() {
       *     $log.info(this.name);
       *   };
       * }]);
       *
       *
       * // In a test ...
       *
       * describe('myDirectiveController', function() {
       *   describe('log()', function() {
       *     it('should write the bound name to the log', inject(function($controller, $log) {
       *       var ctrl = $controller('MyDirectiveController', { /* no locals &#42;/ }, { name: 'Clark Kent' });
       *       ctrl.log();
       *
       *       expect(ctrl.name).toEqual('Clark Kent');
       *       expect($log.info.logs).toEqual(['Clark Kent']);
       *     }));
       *   });
       * });
       *
       * ```
       *
       * @param {Function|string} constructor If called with a function then it's considered to be the
       *    controller constructor function. Otherwise it's considered to be a string which is used
       *    to retrieve the controller constructor using the following steps:
       *
       *    * check if a controller with given name is registered via `$controllerProvider`
       *    * check if evaluating the string on the current scope returns a constructor
       *    * if $controllerProvider#allowGlobals, check `window[constructor]` on the global
       *      `window` object (not recommended)
       *
       *    The string can use the `controller as property` syntax, where the controller instance is published
       *    as the specified property on the `scope`; the `scope` must be injected into `locals` param for this
       *    to work correctly.
       *
       * @param {Object} locals Injection locals for Controller.
       * @param {Object=} bindings Properties to add to the controller instance. This is used to simulate
       *                           the `bindToController` feature and simplify certain kinds of tests.
       * @return {Object} Instance of given controller.
       */
      function createControllerDecorator(compileProvider) {
        angular.mock.$ControllerDecorator = ['$delegate', function ($delegate) {
          return function (expression, locals, later, ident) {
            if (later && typeof later === 'object') {
              var preAssignBindingsEnabled = compileProvider.preAssignBindingsEnabled();

              var instantiate = $delegate(expression, locals, true, ident);
              if (preAssignBindingsEnabled) {
                angular.extend(instantiate.instance, later);
              }

              var instance = instantiate();
              if (!preAssignBindingsEnabled || instance !== instantiate.instance) {
                angular.extend(instance, later);
              }

              return instance;
            }
            return $delegate(expression, locals, later, ident);
          };
        }];

        return angular.mock.$ControllerDecorator;
      }

      /**
       * @ngdoc service
       * @name $componentController
       * @description
       * A service that can be used to create instances of component controllers. Useful for unit-testing.
       *
       * Be aware that the controller will be instantiated and attached to the scope as specified in
       * the component definition object. If you do not provide a `$scope` object in the `locals` param
       * then the helper will create a new isolated scope as a child of `$rootScope`.
       *
       * If you are using `$element` or `$attrs` in the controller, make sure to provide them as `locals`.
       * The `$element` must be a jqLite-wrapped DOM element, and `$attrs` should be an object that
       * has all properties / functions that you are using in the controller. If this is getting too complex,
       * you should compile the component instead and access the component's controller via the
       * {@link angular.element#methods `controller`} function.
       *
       * See also the section on {@link guide/component#unit-testing-component-controllers unit-testing component controllers}
       * in the guide.
       *
       * @param {string} componentName the name of the component whose controller we want to instantiate
       * @param {Object} locals Injection locals for Controller.
       * @param {Object=} bindings Properties to add to the controller before invoking the constructor. This is used
       *                           to simulate the `bindToController` feature and simplify certain kinds of tests.
       * @param {string=} ident Override the property name to use when attaching the controller to the scope.
       * @return {Object} Instance of requested controller.
       */
      angular.mock.$ComponentControllerProvider = ['$compileProvider', function ComponentControllerProvider($compileProvider) {
        this.$get = ['$controller', '$injector', '$rootScope', function ($controller, $injector, $rootScope) {
          return function $componentController(componentName, locals, bindings, ident) {
            // get all directives associated to the component name
            var directives = $injector.get(componentName + 'Directive');
            // look for those directives that are components
            var candidateDirectives = directives.filter(function (directiveInfo) {
              // components have controller, controllerAs and restrict:'E'
              return directiveInfo.controller && directiveInfo.controllerAs && directiveInfo.restrict === 'E';
            });
            // check if valid directives found
            if (candidateDirectives.length === 0) {
              throw new Error('No component found');
            }
            if (candidateDirectives.length > 1) {
              throw new Error('Too many components found');
            }
            // get the info of the component
            var directiveInfo = candidateDirectives[0];
            // create a scope if needed
            locals = locals || {};
            locals.$scope = locals.$scope || $rootScope.$new(true);
            return $controller(directiveInfo.controller, locals, bindings, ident || directiveInfo.controllerAs);
          };
        }];
      }];

      /**
       * @ngdoc module
       * @name ngMock
       * @packageName angular-mocks
       * @description
       *
       * # ngMock
       *
       * The `ngMock` module provides support to inject and mock Angular services into unit tests.
       * In addition, ngMock also extends various core ng services such that they can be
       * inspected and controlled in a synchronous manner within test code.
       *
       *
       * <div doc-module-components="ngMock"></div>
       *
       * @installation
       *
       *  First, download the file:
       *  * [Google CDN](https://developers.google.com/speed/libraries/devguide#angularjs) e.g.
       *    `"//ajax.googleapis.com/ajax/libs/angularjs/X.Y.Z/angular-mocks.js"`
       *  * [NPM](https://www.npmjs.com/) e.g. `npm install angular-mocks@X.Y.Z`
       *  * [Yarn](https://yarnpkg.com) e.g. `yarn add angular-mocks@X.Y.Z`
       *  * [Bower](http://bower.io) e.g. `bower install angular-mocks#X.Y.Z`
       *  * [code.angularjs.org](https://code.angularjs.org/) (discouraged for production use)  e.g.
       *    `"//code.angularjs.org/X.Y.Z/angular-mocks.js"`
       *
       * where X.Y.Z is the AngularJS version you are running.
       *
       * Then, configure your test runner to load `angular-mocks.js` after `angular.js`.
       * This example uses <a href="http://karma-runner.github.io/">Karma</a>:
       *
       * ```
       * config.set({
       *   files: [
       *     'build/angular.js', // and other module files you need
       *     'build/angular-mocks.js',
       *     '<path/to/application/files>',
       *     '<path/to/spec/files>'
       *   ]
       * });
       * ```
       *
       * Including the `angular-mocks.js` file automatically adds the `ngMock` module, so your tests
       *  are ready to go!
       */
      angular.module('ngMock', ['ng']).provider({
        $browser: angular.mock.$BrowserProvider,
        $exceptionHandler: angular.mock.$ExceptionHandlerProvider,
        $log: angular.mock.$LogProvider,
        $interval: angular.mock.$IntervalProvider,
        $httpBackend: angular.mock.$HttpBackendProvider,
        $rootElement: angular.mock.$RootElementProvider,
        $componentController: angular.mock.$ComponentControllerProvider
      }).config(['$provide', '$compileProvider', function ($provide, $compileProvider) {
        $provide.decorator('$timeout', angular.mock.$TimeoutDecorator);
        $provide.decorator('$$rAF', angular.mock.$RAFDecorator);
        $provide.decorator('$rootScope', angular.mock.$RootScopeDecorator);
        $provide.decorator('$controller', createControllerDecorator($compileProvider));
      }]);

      /**
       * @ngdoc module
       * @name ngMockE2E
       * @module ngMockE2E
       * @packageName angular-mocks
       * @description
       *
       * The `ngMockE2E` is an angular module which contains mocks suitable for end-to-end testing.
       * Currently there is only one mock present in this module -
       * the {@link ngMockE2E.$httpBackend e2e $httpBackend} mock.
       */
      angular.module('ngMockE2E', ['ng']).config(['$provide', function ($provide) {
        $provide.value('$httpBackend', angular.injector(['ng']).get('$httpBackend'));
        $provide.decorator('$httpBackend', angular.mock.e2e.$httpBackendDecorator);
      }]);

      /**
       * @ngdoc service
       * @name $httpBackend
       * @module ngMockE2E
       * @description
       * Fake HTTP backend implementation suitable for end-to-end testing or backend-less development of
       * applications that use the {@link ng.$http $http service}.
       *
       * <div class="alert alert-info">
       * **Note**: For fake http backend implementation suitable for unit testing please see
       * {@link ngMock.$httpBackend unit-testing $httpBackend mock}.
       * </div>
       *
       * This implementation can be used to respond with static or dynamic responses via the `when` api
       * and its shortcuts (`whenGET`, `whenPOST`, etc) and optionally pass through requests to the
       * real $httpBackend for specific requests (e.g. to interact with certain remote apis or to fetch
       * templates from a webserver).
       *
       * As opposed to unit-testing, in an end-to-end testing scenario or in scenario when an application
       * is being developed with the real backend api replaced with a mock, it is often desirable for
       * certain category of requests to bypass the mock and issue a real http request (e.g. to fetch
       * templates or static files from the webserver). To configure the backend with this behavior
       * use the `passThrough` request handler of `when` instead of `respond`.
       *
       * Additionally, we don't want to manually have to flush mocked out requests like we do during unit
       * testing. For this reason the e2e $httpBackend flushes mocked out requests
       * automatically, closely simulating the behavior of the XMLHttpRequest object.
       *
       * To setup the application to run with this http backend, you have to create a module that depends
       * on the `ngMockE2E` and your application modules and defines the fake backend:
       *
       * ```js
       *   var myAppDev = angular.module('myAppDev', ['myApp', 'ngMockE2E']);
       *   myAppDev.run(function($httpBackend) {
       *     var phones = [{name: 'phone1'}, {name: 'phone2'}];
       *
       *     // returns the current list of phones
       *     $httpBackend.whenGET('/phones').respond(phones);
       *
       *     // adds a new phone to the phones array
       *     $httpBackend.whenPOST('/phones').respond(function(method, url, data) {
       *       var phone = angular.fromJson(data);
       *       phones.push(phone);
       *       return [200, phone, {}];
       *     });
       *     $httpBackend.whenGET(/^\/templates\//).passThrough(); // Requests for templare are handled by the real server
       *     //...
       *   });
       * ```
       *
       * Afterwards, bootstrap your app with this new module.
       *
       * ## Example
       * <example name="httpbackend-e2e-testing" module="myAppE2E" deps="angular-mocks.js">
       * <file name="app.js">
       *   var myApp = angular.module('myApp', []);
       *
       *   myApp.controller('MainCtrl', function MainCtrl($http) {
       *     var ctrl = this;
       *
       *     ctrl.phones = [];
       *     ctrl.newPhone = {
       *       name: ''
       *     };
       *
       *     ctrl.getPhones = function() {
       *       $http.get('/phones').then(function(response) {
       *         ctrl.phones = response.data;
       *       });
       *     };
       *
       *     ctrl.addPhone = function(phone) {
       *       $http.post('/phones', phone).then(function() {
       *         ctrl.newPhone = {name: ''};
       *         return ctrl.getPhones();
       *       });
       *     };
       *
       *     ctrl.getPhones();
       *   });
       * </file>
       * <file name="e2e.js">
       *   var myAppDev = angular.module('myAppE2E', ['myApp', 'ngMockE2E']);
       *
       *   myAppDev.run(function($httpBackend) {
       *     var phones = [{name: 'phone1'}, {name: 'phone2'}];
       *
       *     // returns the current list of phones
       *     $httpBackend.whenGET('/phones').respond(phones);
       *
       *     // adds a new phone to the phones array
       *     $httpBackend.whenPOST('/phones').respond(function(method, url, data) {
       *       var phone = angular.fromJson(data);
       *       phones.push(phone);
       *       return [200, phone, {}];
       *     });
       *   });
       * </file>
       * <file name="index.html">
       *   <div ng-controller="MainCtrl as $ctrl">
       *   <form name="newPhoneForm" ng-submit="$ctrl.addPhone($ctrl.newPhone)">
       *     <input type="text" ng-model="$ctrl.newPhone.name">
       *     <input type="submit" value="Add Phone">
       *   </form>
       *   <h1>Phones</h1>
       *   <ul>
       *     <li ng-repeat="phone in $ctrl.phones">{{phone.name}}</li>
       *   </ul>
       *   </div>
       * </file>
       * </example>
       *
       *
       */

      /**
       * @ngdoc method
       * @name $httpBackend#when
       * @module ngMockE2E
       * @description
       * Creates a new backend definition.
       *
       * @param {string} method HTTP method.
       * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
       *   and returns true if the url matches the current definition.
       * @param {(string|RegExp|function(string))=} data HTTP request body or function that receives
       *   data string and returns true if the data is as expected.
       * @param {(Object|function(Object))=} headers HTTP headers or function that receives http header
       *   object and returns true if the headers match the current definition.
       * @param {(Array)=} keys Array of keys to assign to regex matches in request url described on
       *   {@link ngMock.$httpBackend $httpBackend mock}.
       * @returns {requestHandler} Returns an object with `respond` and `passThrough` methods that
       *   control how a matched request is handled. You can save this object for later use and invoke
       *   `respond` or `passThrough` again in order to change how a matched request is handled.
       *
       *  - respond –
       *    ```
       *    { function([status,] data[, headers, statusText])
       *    | function(function(method, url, data, headers, params)}
       *    ```
       *    – The respond method takes a set of static data to be returned or a function that can return
       *    an array containing response status (number), response data (Array|Object|string), response
       *    headers (Object), and the text for the status (string).
       *  - passThrough – `{function()}` – Any request matching a backend definition with
       *    `passThrough` handler will be passed through to the real backend (an XHR request will be made
       *    to the server.)
       *  - Both methods return the `requestHandler` object for possible overrides.
       */

      /**
       * @ngdoc method
       * @name $httpBackend#whenGET
       * @module ngMockE2E
       * @description
       * Creates a new backend definition for GET requests. For more info see `when()`.
       *
       * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
       *   and returns true if the url matches the current definition.
       * @param {(Object|function(Object))=} headers HTTP headers.
       * @param {(Array)=} keys Array of keys to assign to regex matches in request url described on
       *   {@link ngMock.$httpBackend $httpBackend mock}.
       * @returns {requestHandler} Returns an object with `respond` and `passThrough` methods that
       *   control how a matched request is handled. You can save this object for later use and invoke
       *   `respond` or `passThrough` again in order to change how a matched request is handled.
       */

      /**
       * @ngdoc method
       * @name $httpBackend#whenHEAD
       * @module ngMockE2E
       * @description
       * Creates a new backend definition for HEAD requests. For more info see `when()`.
       *
       * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
       *   and returns true if the url matches the current definition.
       * @param {(Object|function(Object))=} headers HTTP headers.
       * @param {(Array)=} keys Array of keys to assign to regex matches in request url described on
       *   {@link ngMock.$httpBackend $httpBackend mock}.
       * @returns {requestHandler} Returns an object with `respond` and `passThrough` methods that
       *   control how a matched request is handled. You can save this object for later use and invoke
       *   `respond` or `passThrough` again in order to change how a matched request is handled.
       */

      /**
       * @ngdoc method
       * @name $httpBackend#whenDELETE
       * @module ngMockE2E
       * @description
       * Creates a new backend definition for DELETE requests. For more info see `when()`.
       *
       * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
       *   and returns true if the url matches the current definition.
       * @param {(Object|function(Object))=} headers HTTP headers.
       * @param {(Array)=} keys Array of keys to assign to regex matches in request url described on
       *   {@link ngMock.$httpBackend $httpBackend mock}.
       * @returns {requestHandler} Returns an object with `respond` and `passThrough` methods that
       *   control how a matched request is handled. You can save this object for later use and invoke
       *   `respond` or `passThrough` again in order to change how a matched request is handled.
       */

      /**
       * @ngdoc method
       * @name $httpBackend#whenPOST
       * @module ngMockE2E
       * @description
       * Creates a new backend definition for POST requests. For more info see `when()`.
       *
       * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
       *   and returns true if the url matches the current definition.
       * @param {(string|RegExp|function(string))=} data HTTP request body or function that receives
       *   data string and returns true if the data is as expected.
       * @param {(Object|function(Object))=} headers HTTP headers.
       * @param {(Array)=} keys Array of keys to assign to regex matches in request url described on
       *   {@link ngMock.$httpBackend $httpBackend mock}.
       * @returns {requestHandler} Returns an object with `respond` and `passThrough` methods that
       *   control how a matched request is handled. You can save this object for later use and invoke
       *   `respond` or `passThrough` again in order to change how a matched request is handled.
       */

      /**
       * @ngdoc method
       * @name $httpBackend#whenPUT
       * @module ngMockE2E
       * @description
       * Creates a new backend definition for PUT requests.  For more info see `when()`.
       *
       * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
       *   and returns true if the url matches the current definition.
       * @param {(string|RegExp|function(string))=} data HTTP request body or function that receives
       *   data string and returns true if the data is as expected.
       * @param {(Object|function(Object))=} headers HTTP headers.
       * @param {(Array)=} keys Array of keys to assign to regex matches in request url described on
       *   {@link ngMock.$httpBackend $httpBackend mock}.
       * @returns {requestHandler} Returns an object with `respond` and `passThrough` methods that
       *   control how a matched request is handled. You can save this object for later use and invoke
       *   `respond` or `passThrough` again in order to change how a matched request is handled.
       */

      /**
       * @ngdoc method
       * @name $httpBackend#whenPATCH
       * @module ngMockE2E
       * @description
       * Creates a new backend definition for PATCH requests.  For more info see `when()`.
       *
       * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
       *   and returns true if the url matches the current definition.
       * @param {(string|RegExp|function(string))=} data HTTP request body or function that receives
       *   data string and returns true if the data is as expected.
       * @param {(Object|function(Object))=} headers HTTP headers.
       * @param {(Array)=} keys Array of keys to assign to regex matches in request url described on
       *   {@link ngMock.$httpBackend $httpBackend mock}.
       * @returns {requestHandler} Returns an object with `respond` and `passThrough` methods that
       *   control how a matched request is handled. You can save this object for later use and invoke
       *   `respond` or `passThrough` again in order to change how a matched request is handled.
       */

      /**
       * @ngdoc method
       * @name $httpBackend#whenJSONP
       * @module ngMockE2E
       * @description
       * Creates a new backend definition for JSONP requests. For more info see `when()`.
       *
       * @param {string|RegExp|function(string)} url HTTP url or function that receives a url
       *   and returns true if the url matches the current definition.
       * @param {(Array)=} keys Array of keys to assign to regex matches in request url described on
       *   {@link ngMock.$httpBackend $httpBackend mock}.
       * @returns {requestHandler} Returns an object with `respond` and `passThrough` methods that
       *   control how a matched request is handled. You can save this object for later use and invoke
       *   `respond` or `passThrough` again in order to change how a matched request is handled.
       */
      /**
       * @ngdoc method
       * @name $httpBackend#whenRoute
       * @module ngMockE2E
       * @description
       * Creates a new backend definition that compares only with the requested route.
       *
       * @param {string} method HTTP method.
       * @param {string} url HTTP url string that supports colon param matching.
       * @returns {requestHandler} Returns an object with `respond` and `passThrough` methods that
       *   control how a matched request is handled. You can save this object for later use and invoke
       *   `respond` or `passThrough` again in order to change how a matched request is handled.
       */
      angular.mock.e2e = {};
      angular.mock.e2e.$httpBackendDecorator = ['$rootScope', '$timeout', '$delegate', '$browser', createHttpBackendMock];

      /**
       * @ngdoc type
       * @name $rootScope.Scope
       * @module ngMock
       * @description
       * {@link ng.$rootScope.Scope Scope} type decorated with helper methods useful for testing. These
       * methods are automatically available on any {@link ng.$rootScope.Scope Scope} instance when
       * `ngMock` module is loaded.
       *
       * In addition to all the regular `Scope` methods, the following helper methods are available:
       */
      angular.mock.$RootScopeDecorator = ['$delegate', function ($delegate) {

        var $rootScopePrototype = Object.getPrototypeOf($delegate);

        $rootScopePrototype.$countChildScopes = countChildScopes;
        $rootScopePrototype.$countWatchers = countWatchers;

        return $delegate;

        // ------------------------------------------------------------------------------------------ //

        /**
         * @ngdoc method
         * @name $rootScope.Scope#$countChildScopes
         * @module ngMock
         * @this $rootScope.Scope
         * @description
         * Counts all the direct and indirect child scopes of the current scope.
         *
         * The current scope is excluded from the count. The count includes all isolate child scopes.
         *
         * @returns {number} Total number of child scopes.
         */
        function countChildScopes() {
          var count = 0; // exclude the current scope
          var pendingChildHeads = [this.$$childHead];
          var currentScope;

          while (pendingChildHeads.length) {
            currentScope = pendingChildHeads.shift();

            while (currentScope) {
              count += 1;
              pendingChildHeads.push(currentScope.$$childHead);
              currentScope = currentScope.$$nextSibling;
            }
          }

          return count;
        }

        /**
         * @ngdoc method
         * @name $rootScope.Scope#$countWatchers
         * @this $rootScope.Scope
         * @module ngMock
         * @description
         * Counts all the watchers of direct and indirect child scopes of the current scope.
         *
         * The watchers of the current scope are included in the count and so are all the watchers of
         * isolate child scopes.
         *
         * @returns {number} Total number of watchers.
         */
        function countWatchers() {
          var count = this.$$watchers ? this.$$watchers.length : 0; // include the current scope
          var pendingChildHeads = [this.$$childHead];
          var currentScope;

          while (pendingChildHeads.length) {
            currentScope = pendingChildHeads.shift();

            while (currentScope) {
              count += currentScope.$$watchers ? currentScope.$$watchers.length : 0;
              pendingChildHeads.push(currentScope.$$childHead);
              currentScope = currentScope.$$nextSibling;
            }
          }

          return count;
        }
      }];

      (function (jasmineOrMocha) {

        if (!jasmineOrMocha) {
          return;
        }

        var currentSpec = null,
            injectorState = new InjectorState(),
            annotatedFunctions = [],
            wasInjectorCreated = function () {
          return !!currentSpec;
        };

        angular.mock.$$annotate = angular.injector.$$annotate;
        angular.injector.$$annotate = function (fn) {
          if (typeof fn === 'function' && !fn.$inject) {
            annotatedFunctions.push(fn);
          }
          return angular.mock.$$annotate.apply(this, arguments);
        };

        /**
         * @ngdoc function
         * @name angular.mock.module
         * @description
         *
         * *NOTE*: This function is also published on window for easy access.<br>
         * *NOTE*: This function is declared ONLY WHEN running tests with jasmine or mocha
         *
         * This function registers a module configuration code. It collects the configuration information
         * which will be used when the injector is created by {@link angular.mock.inject inject}.
         *
         * See {@link angular.mock.inject inject} for usage example
         *
         * @param {...(string|Function|Object)} fns any number of modules which are represented as string
         *        aliases or as anonymous module initialization functions. The modules are used to
         *        configure the injector. The 'ng' and 'ngMock' modules are automatically loaded. If an
         *        object literal is passed each key-value pair will be registered on the module via
         *        {@link auto.$provide $provide}.value, the key being the string name (or token) to associate
         *        with the value on the injector.
         */
        var module = window.module = angular.mock.module = function () {
          var moduleFns = Array.prototype.slice.call(arguments, 0);
          return wasInjectorCreated() ? workFn() : workFn;
          /////////////////////
          function workFn() {
            if (currentSpec.$injector) {
              throw new Error('Injector already created, can not register a module!');
            } else {
              var fn,
                  modules = currentSpec.$modules || (currentSpec.$modules = []);
              angular.forEach(moduleFns, function (module) {
                if (angular.isObject(module) && !angular.isArray(module)) {
                  fn = ['$provide', function ($provide) {
                    angular.forEach(module, function (value, key) {
                      $provide.value(key, value);
                    });
                  }];
                } else {
                  fn = module;
                }
                if (currentSpec.$providerInjector) {
                  currentSpec.$providerInjector.invoke(fn);
                } else {
                  modules.push(fn);
                }
              });
            }
          }
        };

        module.$$beforeAllHook = window.before || window.beforeAll;
        module.$$afterAllHook = window.after || window.afterAll;

        // purely for testing ngMock itself
        module.$$currentSpec = function (to) {
          if (arguments.length === 0) return to;
          currentSpec = to;
        };

        /**
         * @ngdoc function
         * @name angular.mock.module.sharedInjector
         * @description
         *
         * *NOTE*: This function is declared ONLY WHEN running tests with jasmine or mocha
         *
         * This function ensures a single injector will be used for all tests in a given describe context.
         * This contrasts with the default behaviour where a new injector is created per test case.
         *
         * Use sharedInjector when you want to take advantage of Jasmine's `beforeAll()`, or mocha's
         * `before()` methods. Call `module.sharedInjector()` before you setup any other hooks that
         * will create (i.e call `module()`) or use (i.e call `inject()`) the injector.
         *
         * You cannot call `sharedInjector()` from within a context already using `sharedInjector()`.
         *
         * ## Example
         *
         * Typically beforeAll is used to make many assertions about a single operation. This can
         * cut down test run-time as the test setup doesn't need to be re-run, and enabling focussed
         * tests each with a single assertion.
         *
         * ```js
         * describe("Deep Thought", function() {
         *
         *   module.sharedInjector();
         *
         *   beforeAll(module("UltimateQuestion"));
         *
         *   beforeAll(inject(function(DeepThought) {
         *     expect(DeepThought.answer).toBeUndefined();
         *     DeepThought.generateAnswer();
         *   }));
         *
         *   it("has calculated the answer correctly", inject(function(DeepThought) {
         *     // Because of sharedInjector, we have access to the instance of the DeepThought service
         *     // that was provided to the beforeAll() hook. Therefore we can test the generated answer
         *     expect(DeepThought.answer).toBe(42);
         *   }));
         *
         *   it("has calculated the answer within the expected time", inject(function(DeepThought) {
         *     expect(DeepThought.runTimeMillennia).toBeLessThan(8000);
         *   }));
         *
         *   it("has double checked the answer", inject(function(DeepThought) {
         *     expect(DeepThought.absolutelySureItIsTheRightAnswer).toBe(true);
         *   }));
         *
         * });
         *
         * ```
         */
        module.sharedInjector = function () {
          if (!(module.$$beforeAllHook && module.$$afterAllHook)) {
            throw Error('sharedInjector() cannot be used unless your test runner defines beforeAll/afterAll');
          }

          var initialized = false;

          module.$$beforeAllHook( /** @this */function () {
            if (injectorState.shared) {
              injectorState.sharedError = Error('sharedInjector() cannot be called inside a context that has already called sharedInjector()');
              throw injectorState.sharedError;
            }
            initialized = true;
            currentSpec = this;
            injectorState.shared = true;
          });

          module.$$afterAllHook(function () {
            if (initialized) {
              injectorState = new InjectorState();
              module.$$cleanup();
            } else {
              injectorState.sharedError = null;
            }
          });
        };

        module.$$beforeEach = function () {
          if (injectorState.shared && currentSpec && currentSpec !== this) {
            var state = currentSpec;
            currentSpec = this;
            angular.forEach(['$injector', '$modules', '$providerInjector', '$injectorStrict'], function (k) {
              currentSpec[k] = state[k];
              state[k] = null;
            });
          } else {
            currentSpec = this;
            originalRootElement = null;
            annotatedFunctions = [];
          }
        };

        module.$$afterEach = function () {
          if (injectorState.cleanupAfterEach()) {
            module.$$cleanup();
          }
        };

        module.$$cleanup = function () {
          var injector = currentSpec.$injector;

          annotatedFunctions.forEach(function (fn) {
            delete fn.$inject;
          });

          angular.forEach(currentSpec.$modules, function (module) {
            if (module && module.$$hashKey) {
              module.$$hashKey = undefined;
            }
          });

          currentSpec.$injector = null;
          currentSpec.$modules = null;
          currentSpec.$providerInjector = null;
          currentSpec = null;

          if (injector) {
            // Ensure `$rootElement` is instantiated, before checking `originalRootElement`
            var $rootElement = injector.get('$rootElement');
            var rootNode = $rootElement && $rootElement[0];
            var cleanUpNodes = !originalRootElement ? [] : [originalRootElement[0]];
            if (rootNode && (!originalRootElement || rootNode !== originalRootElement[0])) {
              cleanUpNodes.push(rootNode);
            }
            angular.element.cleanData(cleanUpNodes);

            // Ensure `$destroy()` is available, before calling it
            // (a mocked `$rootScope` might not implement it (or not even be an object at all))
            var $rootScope = injector.get('$rootScope');
            if ($rootScope && $rootScope.$destroy) $rootScope.$destroy();
          }

          // clean up jquery's fragment cache
          angular.forEach(angular.element.fragments, function (val, key) {
            delete angular.element.fragments[key];
          });

          MockXhr.$$lastInstance = null;

          angular.forEach(angular.callbacks, function (val, key) {
            delete angular.callbacks[key];
          });
          angular.callbacks.$$counter = 0;
        };

        (window.beforeEach || window.setup)(module.$$beforeEach);
        (window.afterEach || window.teardown)(module.$$afterEach);

        /**
         * @ngdoc function
         * @name angular.mock.inject
         * @description
         *
         * *NOTE*: This function is also published on window for easy access.<br>
         * *NOTE*: This function is declared ONLY WHEN running tests with jasmine or mocha
         *
         * The inject function wraps a function into an injectable function. The inject() creates new
         * instance of {@link auto.$injector $injector} per test, which is then used for
         * resolving references.
         *
         *
         * ## Resolving References (Underscore Wrapping)
         * Often, we would like to inject a reference once, in a `beforeEach()` block and reuse this
         * in multiple `it()` clauses. To be able to do this we must assign the reference to a variable
         * that is declared in the scope of the `describe()` block. Since we would, most likely, want
         * the variable to have the same name of the reference we have a problem, since the parameter
         * to the `inject()` function would hide the outer variable.
         *
         * To help with this, the injected parameters can, optionally, be enclosed with underscores.
         * These are ignored by the injector when the reference name is resolved.
         *
         * For example, the parameter `_myService_` would be resolved as the reference `myService`.
         * Since it is available in the function body as `_myService_`, we can then assign it to a variable
         * defined in an outer scope.
         *
         * ```
         * // Defined out reference variable outside
         * var myService;
         *
         * // Wrap the parameter in underscores
         * beforeEach( inject( function(_myService_){
         *   myService = _myService_;
         * }));
         *
         * // Use myService in a series of tests.
         * it('makes use of myService', function() {
         *   myService.doStuff();
         * });
         *
         * ```
         *
         * See also {@link angular.mock.module angular.mock.module}
         *
         * ## Example
         * Example of what a typical jasmine tests looks like with the inject method.
         * ```js
         *
         *   angular.module('myApplicationModule', [])
         *       .value('mode', 'app')
         *       .value('version', 'v1.0.1');
         *
         *
         *   describe('MyApp', function() {
         *
         *     // You need to load modules that you want to test,
         *     // it loads only the "ng" module by default.
         *     beforeEach(module('myApplicationModule'));
         *
         *
         *     // inject() is used to inject arguments of all given functions
         *     it('should provide a version', inject(function(mode, version) {
         *       expect(version).toEqual('v1.0.1');
         *       expect(mode).toEqual('app');
         *     }));
         *
         *
         *     // The inject and module method can also be used inside of the it or beforeEach
         *     it('should override a version and test the new version is injected', function() {
         *       // module() takes functions or strings (module aliases)
         *       module(function($provide) {
         *         $provide.value('version', 'overridden'); // override version here
         *       });
         *
         *       inject(function(version) {
         *         expect(version).toEqual('overridden');
         *       });
         *     });
         *   });
         *
         * ```
         *
         * @param {...Function} fns any number of functions which will be injected using the injector.
         */

        var ErrorAddingDeclarationLocationStack = function ErrorAddingDeclarationLocationStack(e, errorForStack) {
          this.message = e.message;
          this.name = e.name;
          if (e.line) this.line = e.line;
          if (e.sourceId) this.sourceId = e.sourceId;
          if (e.stack && errorForStack) this.stack = e.stack + '\n' + errorForStack.stack;
          if (e.stackArray) this.stackArray = e.stackArray;
        };
        ErrorAddingDeclarationLocationStack.prototype = Error.prototype;

        window.inject = angular.mock.inject = function () {
          var blockFns = Array.prototype.slice.call(arguments, 0);
          var errorForStack = new Error('Declaration Location');
          // IE10+ and PhanthomJS do not set stack trace information, until the error is thrown
          if (!errorForStack.stack) {
            try {
              throw errorForStack;
            } catch (e) {/* empty */}
          }
          return wasInjectorCreated() ? WorkFn.call(currentSpec) : WorkFn;
          /////////////////////
          function WorkFn() {
            var modules = currentSpec.$modules || [];
            var strictDi = !!currentSpec.$injectorStrict;
            modules.unshift(['$injector', function ($injector) {
              currentSpec.$providerInjector = $injector;
            }]);
            modules.unshift('ngMock');
            modules.unshift('ng');
            var injector = currentSpec.$injector;
            if (!injector) {
              if (strictDi) {
                // If strictDi is enabled, annotate the providerInjector blocks
                angular.forEach(modules, function (moduleFn) {
                  if (typeof moduleFn === 'function') {
                    angular.injector.$$annotate(moduleFn);
                  }
                });
              }
              injector = currentSpec.$injector = angular.injector(modules, strictDi);
              currentSpec.$injectorStrict = strictDi;
            }
            for (var i = 0, ii = blockFns.length; i < ii; i++) {
              if (currentSpec.$injectorStrict) {
                // If the injector is strict / strictDi, and the spec wants to inject using automatic
                // annotation, then annotate the function here.
                injector.annotate(blockFns[i]);
              }
              try {
                injector.invoke(blockFns[i] || angular.noop, this);
              } catch (e) {
                if (e.stack && errorForStack) {
                  throw new ErrorAddingDeclarationLocationStack(e, errorForStack);
                }
                throw e;
              } finally {
                errorForStack = null;
              }
            }
          }
        };

        angular.mock.inject.strictDi = function (value) {
          value = arguments.length ? !!value : true;
          return wasInjectorCreated() ? workFn() : workFn;

          function workFn() {
            if (value !== currentSpec.$injectorStrict) {
              if (currentSpec.$injector) {
                throw new Error('Injector already created, can not modify strict annotations');
              } else {
                currentSpec.$injectorStrict = value;
              }
            }
          }
        };

        function InjectorState() {
          this.shared = false;
          this.sharedError = null;

          this.cleanupAfterEach = function () {
            return !this.shared || this.sharedError;
          };
        }
      })(window.jasmine || window.mocha);
    })(window, window.angular);
  })(this);

  return _retrieveGlobal();
});
System.registerDynamic('node_modules/angular/angular.js', [], true, function ($__require, exports, module) {
  var global = this || self,
      GLOBAL = global;
  /**
   * @license AngularJS v1.6.3
   * (c) 2010-2017 Google, Inc. http://angularjs.org
   * License: MIT
   */
  (function (window) {
    'use strict';

    /**
     * @description
     *
     * This object provides a utility for producing rich Error messages within
     * Angular. It can be called as follows:
     *
     * var exampleMinErr = minErr('example');
     * throw exampleMinErr('one', 'This {0} is {1}', foo, bar);
     *
     * The above creates an instance of minErr in the example namespace. The
     * resulting error will have a namespaced error code of example.one.  The
     * resulting error will replace {0} with the value of foo, and {1} with the
     * value of bar. The object is not restricted in the number of arguments it can
     * take.
     *
     * If fewer arguments are specified than necessary for interpolation, the extra
     * interpolation markers will be preserved in the final string.
     *
     * Since data will be parsed statically during a build step, some restrictions
     * are applied with respect to how minErr instances are created and called.
     * Instances should have names of the form namespaceMinErr for a minErr created
     * using minErr('namespace') . Error codes, namespaces and template strings
     * should all be static strings, not variables or general expressions.
     *
     * @param {string} module The namespace to use for the new minErr instance.
     * @param {function} ErrorConstructor Custom error constructor to be instantiated when returning
     *   error from returned function, for cases when a particular type of error is useful.
     * @returns {function(code:string, template:string, ...templateArgs): Error} minErr instance
     */

    function minErr(module, ErrorConstructor) {
      ErrorConstructor = ErrorConstructor || Error;
      return function () {
        var code = arguments[0],
            template = arguments[1],
            message = '[' + (module ? module + ':' : '') + code + '] ',
            templateArgs = sliceArgs(arguments, 2).map(function (arg) {
          return toDebugString(arg, minErrConfig.objectMaxDepth);
        }),
            paramPrefix,
            i;

        message += template.replace(/\{\d+\}/g, function (match) {
          var index = +match.slice(1, -1);

          if (index < templateArgs.length) {
            return templateArgs[index];
          }

          return match;
        });

        message += '\nhttp://errors.angularjs.org/1.6.3/' + (module ? module + '/' : '') + code;

        for (i = 0, paramPrefix = '?'; i < templateArgs.length; i++, paramPrefix = '&') {
          message += paramPrefix + 'p' + i + '=' + encodeURIComponent(templateArgs[i]);
        }

        return new ErrorConstructor(message);
      };
    }

    /* We need to tell ESLint what variables are being exported */
    /* exported
      angular,
      msie,
      jqLite,
      jQuery,
      slice,
      splice,
      push,
      toString,
      minErrConfig,
      errorHandlingConfig,
      isValidObjectMaxDepth,
      ngMinErr,
      angularModule,
      uid,
      REGEX_STRING_REGEXP,
      VALIDITY_STATE_PROPERTY,
    
      lowercase,
      uppercase,
      manualLowercase,
      manualUppercase,
      nodeName_,
      isArrayLike,
      forEach,
      forEachSorted,
      reverseParams,
      nextUid,
      setHashKey,
      extend,
      toInt,
      inherit,
      merge,
      noop,
      identity,
      valueFn,
      isUndefined,
      isDefined,
      isObject,
      isBlankObject,
      isString,
      isNumber,
      isNumberNaN,
      isDate,
      isArray,
      isFunction,
      isRegExp,
      isWindow,
      isScope,
      isFile,
      isFormData,
      isBlob,
      isBoolean,
      isPromiseLike,
      trim,
      escapeForRegexp,
      isElement,
      makeMap,
      includes,
      arrayRemove,
      copy,
      equals,
      csp,
      jq,
      concat,
      sliceArgs,
      bind,
      toJsonReplacer,
      toJson,
      fromJson,
      convertTimezoneToLocal,
      timezoneToOffset,
      startingTag,
      tryDecodeURIComponent,
      parseKeyValue,
      toKeyValue,
      encodeUriSegment,
      encodeUriQuery,
      angularInit,
      bootstrap,
      getTestability,
      snake_case,
      bindJQuery,
      assertArg,
      assertArgFn,
      assertNotHasOwnProperty,
      getter,
      getBlockNodes,
      hasOwnProperty,
      createMap,
      stringify,
    
      NODE_TYPE_ELEMENT,
      NODE_TYPE_ATTRIBUTE,
      NODE_TYPE_TEXT,
      NODE_TYPE_COMMENT,
      NODE_TYPE_DOCUMENT,
      NODE_TYPE_DOCUMENT_FRAGMENT
    */

    ////////////////////////////////////

    /**
     * @ngdoc module
     * @name ng
     * @module ng
     * @installation
     * @description
     *
     * # ng (core module)
     * The ng module is loaded by default when an AngularJS application is started. The module itself
     * contains the essential components for an AngularJS application to function. The table below
     * lists a high level breakdown of each of the services/factories, filters, directives and testing
     * components available within this core module.
     *
     * <div doc-module-components="ng"></div>
     */

    var REGEX_STRING_REGEXP = /^\/(.+)\/([a-z]*)$/;

    // The name of a form control's ValidityState property.
    // This is used so that it's possible for internal tests to create mock ValidityStates.
    var VALIDITY_STATE_PROPERTY = 'validity';

    var hasOwnProperty = Object.prototype.hasOwnProperty;

    var minErrConfig = {
      objectMaxDepth: 5
    };

    /**
    * @ngdoc function
    * @name angular.errorHandlingConfig
    * @module ng
    * @kind function
    *
    * @description
    * Configure several aspects of error handling in AngularJS if used as a setter or return the
    * current configuration if used as a getter. The following options are supported:
    *
    * - **objectMaxDepth**: The maximum depth to which objects are traversed when stringified for error messages.
    *
    * Omitted or undefined options will leave the corresponding configuration values unchanged.
    *
    * @param {Object=} config - The configuration object. May only contain the options that need to be
    *     updated. Supported keys:
    *
    * * `objectMaxDepth`  **{Number}** - The max depth for stringifying objects. Setting to a
    *   non-positive or non-numeric value, removes the max depth limit.
    *   Default: 5
    */
    function errorHandlingConfig(config) {
      if (isObject(config)) {
        if (isDefined(config.objectMaxDepth)) {
          minErrConfig.objectMaxDepth = isValidObjectMaxDepth(config.objectMaxDepth) ? config.objectMaxDepth : NaN;
        }
      } else {
        return minErrConfig;
      }
    }

    /**
     * @private
     * @param {Number} maxDepth
     * @return {boolean}
     */
    function isValidObjectMaxDepth(maxDepth) {
      return isNumber(maxDepth) && maxDepth > 0;
    }

    /**
     * @ngdoc function
     * @name angular.lowercase
     * @module ng
     * @kind function
     *
     * @deprecated
     * sinceVersion="1.5.0"
     * removeVersion="1.7.0"
     * Use [String.prototype.toLowerCase](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String/toLowerCase) instead.
     *
     * @description Converts the specified string to lowercase.
     * @param {string} string String to be converted to lowercase.
     * @returns {string} Lowercased string.
     */
    var lowercase = function (string) {
      return isString(string) ? string.toLowerCase() : string;
    };

    /**
     * @ngdoc function
     * @name angular.uppercase
     * @module ng
     * @kind function
     *
     * @deprecated
     * sinceVersion="1.5.0"
     * removeVersion="1.7.0"
     * Use [String.prototype.toUpperCase](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String/toUpperCase) instead.
     *
     * @description Converts the specified string to uppercase.
     * @param {string} string String to be converted to uppercase.
     * @returns {string} Uppercased string.
     */
    var uppercase = function (string) {
      return isString(string) ? string.toUpperCase() : string;
    };

    var manualLowercase = function (s) {
      /* eslint-disable no-bitwise */
      return isString(s) ? s.replace(/[A-Z]/g, function (ch) {
        return String.fromCharCode(ch.charCodeAt(0) | 32);
      }) : s;
      /* eslint-enable */
    };
    var manualUppercase = function (s) {
      /* eslint-disable no-bitwise */
      return isString(s) ? s.replace(/[a-z]/g, function (ch) {
        return String.fromCharCode(ch.charCodeAt(0) & ~32);
      }) : s;
      /* eslint-enable */
    };

    // String#toLowerCase and String#toUpperCase don't produce correct results in browsers with Turkish
    // locale, for this reason we need to detect this case and redefine lowercase/uppercase methods
    // with correct but slower alternatives. See https://github.com/angular/angular.js/issues/11387
    if ('i' !== 'I'.toLowerCase()) {
      lowercase = manualLowercase;
      uppercase = manualUppercase;
    }

    var msie,
        // holds major version number for IE, or NaN if UA is not IE.
    jqLite,
        // delay binding since jQuery could be loaded after us.
    jQuery,
        // delay binding
    slice = [].slice,
        splice = [].splice,
        push = [].push,
        toString = Object.prototype.toString,
        getPrototypeOf = Object.getPrototypeOf,
        ngMinErr = minErr('ng'),


    /** @name angular */
    angular = window.angular || (window.angular = {}),
        angularModule,
        uid = 0;

    // Support: IE 9-11 only
    /**
     * documentMode is an IE-only property
     * http://msdn.microsoft.com/en-us/library/ie/cc196988(v=vs.85).aspx
     */
    msie = window.document.documentMode;

    /**
     * @private
     * @param {*} obj
     * @return {boolean} Returns true if `obj` is an array or array-like object (NodeList, Arguments,
     *                   String ...)
     */
    function isArrayLike(obj) {

      // `null`, `undefined` and `window` are not array-like
      if (obj == null || isWindow(obj)) return false;

      // arrays, strings and jQuery/jqLite objects are array like
      // * jqLite is either the jQuery or jqLite constructor function
      // * we have to check the existence of jqLite first as this method is called
      //   via the forEach method when constructing the jqLite object in the first place
      if (isArray(obj) || isString(obj) || jqLite && obj instanceof jqLite) return true;

      // Support: iOS 8.2 (not reproducible in simulator)
      // "length" in obj used to prevent JIT error (gh-11508)
      var length = 'length' in Object(obj) && obj.length;

      // NodeList objects (with `item` method) and
      // other objects with suitable length characteristics are array-like
      return isNumber(length) && (length >= 0 && (length - 1 in obj || obj instanceof Array) || typeof obj.item === 'function');
    }

    /**
     * @ngdoc function
     * @name angular.forEach
     * @module ng
     * @kind function
     *
     * @description
     * Invokes the `iterator` function once for each item in `obj` collection, which can be either an
     * object or an array. The `iterator` function is invoked with `iterator(value, key, obj)`, where `value`
     * is the value of an object property or an array element, `key` is the object property key or
     * array element index and obj is the `obj` itself. Specifying a `context` for the function is optional.
     *
     * It is worth noting that `.forEach` does not iterate over inherited properties because it filters
     * using the `hasOwnProperty` method.
     *
     * Unlike ES262's
     * [Array.prototype.forEach](http://www.ecma-international.org/ecma-262/5.1/#sec-15.4.4.18),
     * providing 'undefined' or 'null' values for `obj` will not throw a TypeError, but rather just
     * return the value provided.
     *
       ```js
         var values = {name: 'misko', gender: 'male'};
         var log = [];
         angular.forEach(values, function(value, key) {
           this.push(key + ': ' + value);
         }, log);
         expect(log).toEqual(['name: misko', 'gender: male']);
       ```
     *
     * @param {Object|Array} obj Object to iterate over.
     * @param {Function} iterator Iterator function.
     * @param {Object=} context Object to become context (`this`) for the iterator function.
     * @returns {Object|Array} Reference to `obj`.
     */

    function forEach(obj, iterator, context) {
      var key, length;
      if (obj) {
        if (isFunction(obj)) {
          for (key in obj) {
            if (key !== 'prototype' && key !== 'length' && key !== 'name' && obj.hasOwnProperty(key)) {
              iterator.call(context, obj[key], key, obj);
            }
          }
        } else if (isArray(obj) || isArrayLike(obj)) {
          var isPrimitive = typeof obj !== 'object';
          for (key = 0, length = obj.length; key < length; key++) {
            if (isPrimitive || key in obj) {
              iterator.call(context, obj[key], key, obj);
            }
          }
        } else if (obj.forEach && obj.forEach !== forEach) {
          obj.forEach(iterator, context, obj);
        } else if (isBlankObject(obj)) {
          // createMap() fast path --- Safe to avoid hasOwnProperty check because prototype chain is empty
          for (key in obj) {
            iterator.call(context, obj[key], key, obj);
          }
        } else if (typeof obj.hasOwnProperty === 'function') {
          // Slow path for objects inheriting Object.prototype, hasOwnProperty check needed
          for (key in obj) {
            if (obj.hasOwnProperty(key)) {
              iterator.call(context, obj[key], key, obj);
            }
          }
        } else {
          // Slow path for objects which do not have a method `hasOwnProperty`
          for (key in obj) {
            if (hasOwnProperty.call(obj, key)) {
              iterator.call(context, obj[key], key, obj);
            }
          }
        }
      }
      return obj;
    }

    function forEachSorted(obj, iterator, context) {
      var keys = Object.keys(obj).sort();
      for (var i = 0; i < keys.length; i++) {
        iterator.call(context, obj[keys[i]], keys[i]);
      }
      return keys;
    }

    /**
     * when using forEach the params are value, key, but it is often useful to have key, value.
     * @param {function(string, *)} iteratorFn
     * @returns {function(*, string)}
     */
    function reverseParams(iteratorFn) {
      return function (value, key) {
        iteratorFn(key, value);
      };
    }

    /**
     * A consistent way of creating unique IDs in angular.
     *
     * Using simple numbers allows us to generate 28.6 million unique ids per second for 10 years before
     * we hit number precision issues in JavaScript.
     *
     * Math.pow(2,53) / 60 / 60 / 24 / 365 / 10 = 28.6M
     *
     * @returns {number} an unique alpha-numeric string
     */
    function nextUid() {
      return ++uid;
    }

    /**
     * Set or clear the hashkey for an object.
     * @param obj object
     * @param h the hashkey (!truthy to delete the hashkey)
     */
    function setHashKey(obj, h) {
      if (h) {
        obj.$$hashKey = h;
      } else {
        delete obj.$$hashKey;
      }
    }

    function baseExtend(dst, objs, deep) {
      var h = dst.$$hashKey;

      for (var i = 0, ii = objs.length; i < ii; ++i) {
        var obj = objs[i];
        if (!isObject(obj) && !isFunction(obj)) continue;
        var keys = Object.keys(obj);
        for (var j = 0, jj = keys.length; j < jj; j++) {
          var key = keys[j];
          var src = obj[key];

          if (deep && isObject(src)) {
            if (isDate(src)) {
              dst[key] = new Date(src.valueOf());
            } else if (isRegExp(src)) {
              dst[key] = new RegExp(src);
            } else if (src.nodeName) {
              dst[key] = src.cloneNode(true);
            } else if (isElement(src)) {
              dst[key] = src.clone();
            } else {
              if (!isObject(dst[key])) dst[key] = isArray(src) ? [] : {};
              baseExtend(dst[key], [src], true);
            }
          } else {
            dst[key] = src;
          }
        }
      }

      setHashKey(dst, h);
      return dst;
    }

    /**
     * @ngdoc function
     * @name angular.extend
     * @module ng
     * @kind function
     *
     * @description
     * Extends the destination object `dst` by copying own enumerable properties from the `src` object(s)
     * to `dst`. You can specify multiple `src` objects. If you want to preserve original objects, you can do so
     * by passing an empty object as the target: `var object = angular.extend({}, object1, object2)`.
     *
     * **Note:** Keep in mind that `angular.extend` does not support recursive merge (deep copy). Use
     * {@link angular.merge} for this.
     *
     * @param {Object} dst Destination object.
     * @param {...Object} src Source object(s).
     * @returns {Object} Reference to `dst`.
     */
    function extend(dst) {
      return baseExtend(dst, slice.call(arguments, 1), false);
    }

    /**
    * @ngdoc function
    * @name angular.merge
    * @module ng
    * @kind function
    *
    * @description
    * Deeply extends the destination object `dst` by copying own enumerable properties from the `src` object(s)
    * to `dst`. You can specify multiple `src` objects. If you want to preserve original objects, you can do so
    * by passing an empty object as the target: `var object = angular.merge({}, object1, object2)`.
    *
    * Unlike {@link angular.extend extend()}, `merge()` recursively descends into object properties of source
    * objects, performing a deep copy.
    *
    * @param {Object} dst Destination object.
    * @param {...Object} src Source object(s).
    * @returns {Object} Reference to `dst`.
    */
    function merge(dst) {
      return baseExtend(dst, slice.call(arguments, 1), true);
    }

    function toInt(str) {
      return parseInt(str, 10);
    }

    var isNumberNaN = Number.isNaN || function isNumberNaN(num) {
      // eslint-disable-next-line no-self-compare
      return num !== num;
    };

    function inherit(parent, extra) {
      return extend(Object.create(parent), extra);
    }

    /**
     * @ngdoc function
     * @name angular.noop
     * @module ng
     * @kind function
     *
     * @description
     * A function that performs no operations. This function can be useful when writing code in the
     * functional style.
       ```js
         function foo(callback) {
           var result = calculateResult();
           (callback || angular.noop)(result);
         }
       ```
     */
    function noop() {}
    noop.$inject = [];

    /**
     * @ngdoc function
     * @name angular.identity
     * @module ng
     * @kind function
     *
     * @description
     * A function that returns its first argument. This function is useful when writing code in the
     * functional style.
     *
       ```js
       function transformer(transformationFn, value) {
         return (transformationFn || angular.identity)(value);
       };
    
       // E.g.
       function getResult(fn, input) {
         return (fn || angular.identity)(input);
       };
    
       getResult(function(n) { return n * 2; }, 21);   // returns 42
       getResult(null, 21);                            // returns 21
       getResult(undefined, 21);                       // returns 21
       ```
     *
     * @param {*} value to be returned.
     * @returns {*} the value passed in.
     */
    function identity($) {
      return $;
    }
    identity.$inject = [];

    function valueFn(value) {
      return function valueRef() {
        return value;
      };
    }

    function hasCustomToString(obj) {
      return isFunction(obj.toString) && obj.toString !== toString;
    }

    /**
     * @ngdoc function
     * @name angular.isUndefined
     * @module ng
     * @kind function
     *
     * @description
     * Determines if a reference is undefined.
     *
     * @param {*} value Reference to check.
     * @returns {boolean} True if `value` is undefined.
     */
    function isUndefined(value) {
      return typeof value === 'undefined';
    }

    /**
     * @ngdoc function
     * @name angular.isDefined
     * @module ng
     * @kind function
     *
     * @description
     * Determines if a reference is defined.
     *
     * @param {*} value Reference to check.
     * @returns {boolean} True if `value` is defined.
     */
    function isDefined(value) {
      return typeof value !== 'undefined';
    }

    /**
     * @ngdoc function
     * @name angular.isObject
     * @module ng
     * @kind function
     *
     * @description
     * Determines if a reference is an `Object`. Unlike `typeof` in JavaScript, `null`s are not
     * considered to be objects. Note that JavaScript arrays are objects.
     *
     * @param {*} value Reference to check.
     * @returns {boolean} True if `value` is an `Object` but not `null`.
     */
    function isObject(value) {
      // http://jsperf.com/isobject4
      return value !== null && typeof value === 'object';
    }

    /**
     * Determine if a value is an object with a null prototype
     *
     * @returns {boolean} True if `value` is an `Object` with a null prototype
     */
    function isBlankObject(value) {
      return value !== null && typeof value === 'object' && !getPrototypeOf(value);
    }

    /**
     * @ngdoc function
     * @name angular.isString
     * @module ng
     * @kind function
     *
     * @description
     * Determines if a reference is a `String`.
     *
     * @param {*} value Reference to check.
     * @returns {boolean} True if `value` is a `String`.
     */
    function isString(value) {
      return typeof value === 'string';
    }

    /**
     * @ngdoc function
     * @name angular.isNumber
     * @module ng
     * @kind function
     *
     * @description
     * Determines if a reference is a `Number`.
     *
     * This includes the "special" numbers `NaN`, `+Infinity` and `-Infinity`.
     *
     * If you wish to exclude these then you can use the native
     * [`isFinite'](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/isFinite)
     * method.
     *
     * @param {*} value Reference to check.
     * @returns {boolean} True if `value` is a `Number`.
     */
    function isNumber(value) {
      return typeof value === 'number';
    }

    /**
     * @ngdoc function
     * @name angular.isDate
     * @module ng
     * @kind function
     *
     * @description
     * Determines if a value is a date.
     *
     * @param {*} value Reference to check.
     * @returns {boolean} True if `value` is a `Date`.
     */
    function isDate(value) {
      return toString.call(value) === '[object Date]';
    }

    /**
     * @ngdoc function
     * @name angular.isArray
     * @module ng
     * @kind function
     *
     * @description
     * Determines if a reference is an `Array`. Alias of Array.isArray.
     *
     * @param {*} value Reference to check.
     * @returns {boolean} True if `value` is an `Array`.
     */
    var isArray = Array.isArray;

    /**
     * @ngdoc function
     * @name angular.isFunction
     * @module ng
     * @kind function
     *
     * @description
     * Determines if a reference is a `Function`.
     *
     * @param {*} value Reference to check.
     * @returns {boolean} True if `value` is a `Function`.
     */
    function isFunction(value) {
      return typeof value === 'function';
    }

    /**
     * Determines if a value is a regular expression object.
     *
     * @private
     * @param {*} value Reference to check.
     * @returns {boolean} True if `value` is a `RegExp`.
     */
    function isRegExp(value) {
      return toString.call(value) === '[object RegExp]';
    }

    /**
     * Checks if `obj` is a window object.
     *
     * @private
     * @param {*} obj Object to check
     * @returns {boolean} True if `obj` is a window obj.
     */
    function isWindow(obj) {
      return obj && obj.window === obj;
    }

    function isScope(obj) {
      return obj && obj.$evalAsync && obj.$watch;
    }

    function isFile(obj) {
      return toString.call(obj) === '[object File]';
    }

    function isFormData(obj) {
      return toString.call(obj) === '[object FormData]';
    }

    function isBlob(obj) {
      return toString.call(obj) === '[object Blob]';
    }

    function isBoolean(value) {
      return typeof value === 'boolean';
    }

    function isPromiseLike(obj) {
      return obj && isFunction(obj.then);
    }

    var TYPED_ARRAY_REGEXP = /^\[object (?:Uint8|Uint8Clamped|Uint16|Uint32|Int8|Int16|Int32|Float32|Float64)Array]$/;
    function isTypedArray(value) {
      return value && isNumber(value.length) && TYPED_ARRAY_REGEXP.test(toString.call(value));
    }

    function isArrayBuffer(obj) {
      return toString.call(obj) === '[object ArrayBuffer]';
    }

    var trim = function (value) {
      return isString(value) ? value.trim() : value;
    };

    // Copied from:
    // http://docs.closure-library.googlecode.com/git/local_closure_goog_string_string.js.source.html#line1021
    // Prereq: s is a string.
    var escapeForRegexp = function (s) {
      return s.replace(/([-()[\]{}+?*.$^|,:#<!\\])/g, '\\$1')
      // eslint-disable-next-line no-control-regex
      .replace(/\x08/g, '\\x08');
    };

    /**
     * @ngdoc function
     * @name angular.isElement
     * @module ng
     * @kind function
     *
     * @description
     * Determines if a reference is a DOM element (or wrapped jQuery element).
     *
     * @param {*} value Reference to check.
     * @returns {boolean} True if `value` is a DOM element (or wrapped jQuery element).
     */
    function isElement(node) {
      return !!(node && (node.nodeName // We are a direct element.
      || node.prop && node.attr && node.find)); // We have an on and find method part of jQuery API.
    }

    /**
     * @param str 'key1,key2,...'
     * @returns {object} in the form of {key1:true, key2:true, ...}
     */
    function makeMap(str) {
      var obj = {},
          items = str.split(','),
          i;
      for (i = 0; i < items.length; i++) {
        obj[items[i]] = true;
      }
      return obj;
    }

    function nodeName_(element) {
      return lowercase(element.nodeName || element[0] && element[0].nodeName);
    }

    function includes(array, obj) {
      return Array.prototype.indexOf.call(array, obj) !== -1;
    }

    function arrayRemove(array, value) {
      var index = array.indexOf(value);
      if (index >= 0) {
        array.splice(index, 1);
      }
      return index;
    }

    /**
     * @ngdoc function
     * @name angular.copy
     * @module ng
     * @kind function
     *
     * @description
     * Creates a deep copy of `source`, which should be an object or an array.
     *
     * * If no destination is supplied, a copy of the object or array is created.
     * * If a destination is provided, all of its elements (for arrays) or properties (for objects)
     *   are deleted and then all elements/properties from the source are copied to it.
     * * If `source` is not an object or array (inc. `null` and `undefined`), `source` is returned.
     * * If `source` is identical to `destination` an exception will be thrown.
     *
     * <br />
     * <div class="alert alert-warning">
     *   Only enumerable properties are taken into account. Non-enumerable properties (both on `source`
     *   and on `destination`) will be ignored.
     * </div>
     *
     * @param {*} source The source that will be used to make a copy.
     *                   Can be any type, including primitives, `null`, and `undefined`.
     * @param {(Object|Array)=} destination Destination into which the source is copied. If
     *     provided, must be of the same type as `source`.
     * @returns {*} The copy or updated `destination`, if `destination` was specified.
     *
     * @example
      <example module="copyExample" name="angular-copy">
        <file name="index.html">
          <div ng-controller="ExampleController">
            <form novalidate class="simple-form">
              <label>Name: <input type="text" ng-model="user.name" /></label><br />
              <label>Age:  <input type="number" ng-model="user.age" /></label><br />
              Gender: <label><input type="radio" ng-model="user.gender" value="male" />male</label>
                      <label><input type="radio" ng-model="user.gender" value="female" />female</label><br />
              <button ng-click="reset()">RESET</button>
              <button ng-click="update(user)">SAVE</button>
            </form>
            <pre>form = {{user | json}}</pre>
            <pre>master = {{master | json}}</pre>
          </div>
        </file>
        <file name="script.js">
          // Module: copyExample
          angular.
            module('copyExample', []).
            controller('ExampleController', ['$scope', function($scope) {
              $scope.master = {};
    
              $scope.reset = function() {
                // Example with 1 argument
                $scope.user = angular.copy($scope.master);
              };
    
              $scope.update = function(user) {
                // Example with 2 arguments
                angular.copy(user, $scope.master);
              };
    
              $scope.reset();
            }]);
        </file>
      </example>
     */
    function copy(source, destination, maxDepth) {
      var stackSource = [];
      var stackDest = [];
      maxDepth = isValidObjectMaxDepth(maxDepth) ? maxDepth : NaN;

      if (destination) {
        if (isTypedArray(destination) || isArrayBuffer(destination)) {
          throw ngMinErr('cpta', 'Can\'t copy! TypedArray destination cannot be mutated.');
        }
        if (source === destination) {
          throw ngMinErr('cpi', 'Can\'t copy! Source and destination are identical.');
        }

        // Empty the destination object
        if (isArray(destination)) {
          destination.length = 0;
        } else {
          forEach(destination, function (value, key) {
            if (key !== '$$hashKey') {
              delete destination[key];
            }
          });
        }

        stackSource.push(source);
        stackDest.push(destination);
        return copyRecurse(source, destination, maxDepth);
      }

      return copyElement(source, maxDepth);

      function copyRecurse(source, destination, maxDepth) {
        maxDepth--;
        if (maxDepth < 0) {
          return '...';
        }
        var h = destination.$$hashKey;
        var key;
        if (isArray(source)) {
          for (var i = 0, ii = source.length; i < ii; i++) {
            destination.push(copyElement(source[i], maxDepth));
          }
        } else if (isBlankObject(source)) {
          // createMap() fast path --- Safe to avoid hasOwnProperty check because prototype chain is empty
          for (key in source) {
            destination[key] = copyElement(source[key], maxDepth);
          }
        } else if (source && typeof source.hasOwnProperty === 'function') {
          // Slow path, which must rely on hasOwnProperty
          for (key in source) {
            if (source.hasOwnProperty(key)) {
              destination[key] = copyElement(source[key], maxDepth);
            }
          }
        } else {
          // Slowest path --- hasOwnProperty can't be called as a method
          for (key in source) {
            if (hasOwnProperty.call(source, key)) {
              destination[key] = copyElement(source[key], maxDepth);
            }
          }
        }
        setHashKey(destination, h);
        return destination;
      }

      function copyElement(source, maxDepth) {
        // Simple values
        if (!isObject(source)) {
          return source;
        }

        // Already copied values
        var index = stackSource.indexOf(source);
        if (index !== -1) {
          return stackDest[index];
        }

        if (isWindow(source) || isScope(source)) {
          throw ngMinErr('cpws', 'Can\'t copy! Making copies of Window or Scope instances is not supported.');
        }

        var needsRecurse = false;
        var destination = copyType(source);

        if (destination === undefined) {
          destination = isArray(source) ? [] : Object.create(getPrototypeOf(source));
          needsRecurse = true;
        }

        stackSource.push(source);
        stackDest.push(destination);

        return needsRecurse ? copyRecurse(source, destination, maxDepth) : destination;
      }

      function copyType(source) {
        switch (toString.call(source)) {
          case '[object Int8Array]':
          case '[object Int16Array]':
          case '[object Int32Array]':
          case '[object Float32Array]':
          case '[object Float64Array]':
          case '[object Uint8Array]':
          case '[object Uint8ClampedArray]':
          case '[object Uint16Array]':
          case '[object Uint32Array]':
            return new source.constructor(copyElement(source.buffer), source.byteOffset, source.length);

          case '[object ArrayBuffer]':
            // Support: IE10
            if (!source.slice) {
              // If we're in this case we know the environment supports ArrayBuffer
              /* eslint-disable no-undef */
              var copied = new ArrayBuffer(source.byteLength);
              new Uint8Array(copied).set(new Uint8Array(source));
              /* eslint-enable */
              return copied;
            }
            return source.slice(0);

          case '[object Boolean]':
          case '[object Number]':
          case '[object String]':
          case '[object Date]':
            return new source.constructor(source.valueOf());

          case '[object RegExp]':
            var re = new RegExp(source.source, source.toString().match(/[^/]*$/)[0]);
            re.lastIndex = source.lastIndex;
            return re;

          case '[object Blob]':
            return new source.constructor([source], { type: source.type });
        }

        if (isFunction(source.cloneNode)) {
          return source.cloneNode(true);
        }
      }
    }

    /**
     * @ngdoc function
     * @name angular.equals
     * @module ng
     * @kind function
     *
     * @description
     * Determines if two objects or two values are equivalent. Supports value types, regular
     * expressions, arrays and objects.
     *
     * Two objects or values are considered equivalent if at least one of the following is true:
     *
     * * Both objects or values pass `===` comparison.
     * * Both objects or values are of the same type and all of their properties are equal by
     *   comparing them with `angular.equals`.
     * * Both values are NaN. (In JavaScript, NaN == NaN => false. But we consider two NaN as equal)
     * * Both values represent the same regular expression (In JavaScript,
     *   /abc/ == /abc/ => false. But we consider two regular expressions as equal when their textual
     *   representation matches).
     *
     * During a property comparison, properties of `function` type and properties with names
     * that begin with `$` are ignored.
     *
     * Scope and DOMWindow objects are being compared only by identify (`===`).
     *
     * @param {*} o1 Object or value to compare.
     * @param {*} o2 Object or value to compare.
     * @returns {boolean} True if arguments are equal.
     *
     * @example
       <example module="equalsExample" name="equalsExample">
         <file name="index.html">
          <div ng-controller="ExampleController">
            <form novalidate>
              <h3>User 1</h3>
              Name: <input type="text" ng-model="user1.name">
              Age: <input type="number" ng-model="user1.age">
    
              <h3>User 2</h3>
              Name: <input type="text" ng-model="user2.name">
              Age: <input type="number" ng-model="user2.age">
    
              <div>
                <br/>
                <input type="button" value="Compare" ng-click="compare()">
              </div>
              User 1: <pre>{{user1 | json}}</pre>
              User 2: <pre>{{user2 | json}}</pre>
              Equal: <pre>{{result}}</pre>
            </form>
          </div>
        </file>
        <file name="script.js">
            angular.module('equalsExample', []).controller('ExampleController', ['$scope', function($scope) {
              $scope.user1 = {};
              $scope.user2 = {};
              $scope.compare = function() {
                $scope.result = angular.equals($scope.user1, $scope.user2);
              };
            }]);
        </file>
      </example>
     */
    function equals(o1, o2) {
      if (o1 === o2) return true;
      if (o1 === null || o2 === null) return false;
      // eslint-disable-next-line no-self-compare
      if (o1 !== o1 && o2 !== o2) return true; // NaN === NaN
      var t1 = typeof o1,
          t2 = typeof o2,
          length,
          key,
          keySet;
      if (t1 === t2 && t1 === 'object') {
        if (isArray(o1)) {
          if (!isArray(o2)) return false;
          if ((length = o1.length) === o2.length) {
            for (key = 0; key < length; key++) {
              if (!equals(o1[key], o2[key])) return false;
            }
            return true;
          }
        } else if (isDate(o1)) {
          if (!isDate(o2)) return false;
          return equals(o1.getTime(), o2.getTime());
        } else if (isRegExp(o1)) {
          if (!isRegExp(o2)) return false;
          return o1.toString() === o2.toString();
        } else {
          if (isScope(o1) || isScope(o2) || isWindow(o1) || isWindow(o2) || isArray(o2) || isDate(o2) || isRegExp(o2)) return false;
          keySet = createMap();
          for (key in o1) {
            if (key.charAt(0) === '$' || isFunction(o1[key])) continue;
            if (!equals(o1[key], o2[key])) return false;
            keySet[key] = true;
          }
          for (key in o2) {
            if (!(key in keySet) && key.charAt(0) !== '$' && isDefined(o2[key]) && !isFunction(o2[key])) return false;
          }
          return true;
        }
      }
      return false;
    }

    var csp = function () {
      if (!isDefined(csp.rules)) {

        var ngCspElement = window.document.querySelector('[ng-csp]') || window.document.querySelector('[data-ng-csp]');

        if (ngCspElement) {
          var ngCspAttribute = ngCspElement.getAttribute('ng-csp') || ngCspElement.getAttribute('data-ng-csp');
          csp.rules = {
            noUnsafeEval: !ngCspAttribute || ngCspAttribute.indexOf('no-unsafe-eval') !== -1,
            noInlineStyle: !ngCspAttribute || ngCspAttribute.indexOf('no-inline-style') !== -1
          };
        } else {
          csp.rules = {
            noUnsafeEval: noUnsafeEval(),
            noInlineStyle: false
          };
        }
      }

      return csp.rules;

      function noUnsafeEval() {
        try {
          // eslint-disable-next-line no-new, no-new-func
          new Function('');
          return false;
        } catch (e) {
          return true;
        }
      }
    };

    /**
     * @ngdoc directive
     * @module ng
     * @name ngJq
     *
     * @element ANY
     * @param {string=} ngJq the name of the library available under `window`
     * to be used for angular.element
     * @description
     * Use this directive to force the angular.element library.  This should be
     * used to force either jqLite by leaving ng-jq blank or setting the name of
     * the jquery variable under window (eg. jQuery).
     *
     * Since angular looks for this directive when it is loaded (doesn't wait for the
     * DOMContentLoaded event), it must be placed on an element that comes before the script
     * which loads angular. Also, only the first instance of `ng-jq` will be used and all
     * others ignored.
     *
     * @example
     * This example shows how to force jqLite using the `ngJq` directive to the `html` tag.
     ```html
     <!doctype html>
     <html ng-app ng-jq>
     ...
     ...
     </html>
     ```
     * @example
     * This example shows how to use a jQuery based library of a different name.
     * The library name must be available at the top most 'window'.
     ```html
     <!doctype html>
     <html ng-app ng-jq="jQueryLib">
     ...
     ...
     </html>
     ```
     */
    var jq = function () {
      if (isDefined(jq.name_)) return jq.name_;
      var el;
      var i,
          ii = ngAttrPrefixes.length,
          prefix,
          name;
      for (i = 0; i < ii; ++i) {
        prefix = ngAttrPrefixes[i];
        el = window.document.querySelector('[' + prefix.replace(':', '\\:') + 'jq]');
        if (el) {
          name = el.getAttribute(prefix + 'jq');
          break;
        }
      }

      return jq.name_ = name;
    };

    function concat(array1, array2, index) {
      return array1.concat(slice.call(array2, index));
    }

    function sliceArgs(args, startIndex) {
      return slice.call(args, startIndex || 0);
    }

    /**
     * @ngdoc function
     * @name angular.bind
     * @module ng
     * @kind function
     *
     * @description
     * Returns a function which calls function `fn` bound to `self` (`self` becomes the `this` for
     * `fn`). You can supply optional `args` that are prebound to the function. This feature is also
     * known as [partial application](http://en.wikipedia.org/wiki/Partial_application), as
     * distinguished from [function currying](http://en.wikipedia.org/wiki/Currying#Contrast_with_partial_function_application).
     *
     * @param {Object} self Context which `fn` should be evaluated in.
     * @param {function()} fn Function to be bound.
     * @param {...*} args Optional arguments to be prebound to the `fn` function call.
     * @returns {function()} Function that wraps the `fn` with all the specified bindings.
     */
    function bind(self, fn) {
      var curryArgs = arguments.length > 2 ? sliceArgs(arguments, 2) : [];
      if (isFunction(fn) && !(fn instanceof RegExp)) {
        return curryArgs.length ? function () {
          return arguments.length ? fn.apply(self, concat(curryArgs, arguments, 0)) : fn.apply(self, curryArgs);
        } : function () {
          return arguments.length ? fn.apply(self, arguments) : fn.call(self);
        };
      } else {
        // In IE, native methods are not functions so they cannot be bound (note: they don't need to be).
        return fn;
      }
    }

    function toJsonReplacer(key, value) {
      var val = value;

      if (typeof key === 'string' && key.charAt(0) === '$' && key.charAt(1) === '$') {
        val = undefined;
      } else if (isWindow(value)) {
        val = '$WINDOW';
      } else if (value && window.document === value) {
        val = '$DOCUMENT';
      } else if (isScope(value)) {
        val = '$SCOPE';
      }

      return val;
    }

    /**
     * @ngdoc function
     * @name angular.toJson
     * @module ng
     * @kind function
     *
     * @description
     * Serializes input into a JSON-formatted string. Properties with leading $$ characters will be
     * stripped since angular uses this notation internally.
     *
     * @param {Object|Array|Date|string|number|boolean} obj Input to be serialized into JSON.
     * @param {boolean|number} [pretty=2] If set to true, the JSON output will contain newlines and whitespace.
     *    If set to an integer, the JSON output will contain that many spaces per indentation.
     * @returns {string|undefined} JSON-ified string representing `obj`.
     * @knownIssue
     *
     * The Safari browser throws a `RangeError` instead of returning `null` when it tries to stringify a `Date`
     * object with an invalid date value. The only reliable way to prevent this is to monkeypatch the
     * `Date.prototype.toJSON` method as follows:
     *
     * ```
     * var _DatetoJSON = Date.prototype.toJSON;
     * Date.prototype.toJSON = function() {
     *   try {
     *     return _DatetoJSON.call(this);
     *   } catch(e) {
     *     if (e instanceof RangeError) {
     *       return null;
     *     }
     *     throw e;
     *   }
     * };
     * ```
     *
     * See https://github.com/angular/angular.js/pull/14221 for more information.
     */
    function toJson(obj, pretty) {
      if (isUndefined(obj)) return undefined;
      if (!isNumber(pretty)) {
        pretty = pretty ? 2 : null;
      }
      return JSON.stringify(obj, toJsonReplacer, pretty);
    }

    /**
     * @ngdoc function
     * @name angular.fromJson
     * @module ng
     * @kind function
     *
     * @description
     * Deserializes a JSON string.
     *
     * @param {string} json JSON string to deserialize.
     * @returns {Object|Array|string|number} Deserialized JSON string.
     */
    function fromJson(json) {
      return isString(json) ? JSON.parse(json) : json;
    }

    var ALL_COLONS = /:/g;
    function timezoneToOffset(timezone, fallback) {
      // Support: IE 9-11 only, Edge 13-14+
      // IE/Edge do not "understand" colon (`:`) in timezone
      timezone = timezone.replace(ALL_COLONS, '');
      var requestedTimezoneOffset = Date.parse('Jan 01, 1970 00:00:00 ' + timezone) / 60000;
      return isNumberNaN(requestedTimezoneOffset) ? fallback : requestedTimezoneOffset;
    }

    function addDateMinutes(date, minutes) {
      date = new Date(date.getTime());
      date.setMinutes(date.getMinutes() + minutes);
      return date;
    }

    function convertTimezoneToLocal(date, timezone, reverse) {
      reverse = reverse ? -1 : 1;
      var dateTimezoneOffset = date.getTimezoneOffset();
      var timezoneOffset = timezoneToOffset(timezone, dateTimezoneOffset);
      return addDateMinutes(date, reverse * (timezoneOffset - dateTimezoneOffset));
    }

    /**
     * @returns {string} Returns the string representation of the element.
     */
    function startingTag(element) {
      element = jqLite(element).clone();
      try {
        // turns out IE does not let you set .html() on elements which
        // are not allowed to have children. So we just ignore it.
        element.empty();
      } catch (e) {/* empty */}
      var elemHtml = jqLite('<div>').append(element).html();
      try {
        return element[0].nodeType === NODE_TYPE_TEXT ? lowercase(elemHtml) : elemHtml.match(/^(<[^>]+>)/)[1].replace(/^<([\w-]+)/, function (match, nodeName) {
          return '<' + lowercase(nodeName);
        });
      } catch (e) {
        return lowercase(elemHtml);
      }
    }

    /////////////////////////////////////////////////

    /**
     * Tries to decode the URI component without throwing an exception.
     *
     * @private
     * @param str value potential URI component to check.
     * @returns {boolean} True if `value` can be decoded
     * with the decodeURIComponent function.
     */
    function tryDecodeURIComponent(value) {
      try {
        return decodeURIComponent(value);
      } catch (e) {
        // Ignore any invalid uri component.
      }
    }

    /**
     * Parses an escaped url query string into key-value pairs.
     * @returns {Object.<string,boolean|Array>}
     */
    function parseKeyValue( /**string*/keyValue) {
      var obj = {};
      forEach((keyValue || '').split('&'), function (keyValue) {
        var splitPoint, key, val;
        if (keyValue) {
          key = keyValue = keyValue.replace(/\+/g, '%20');
          splitPoint = keyValue.indexOf('=');
          if (splitPoint !== -1) {
            key = keyValue.substring(0, splitPoint);
            val = keyValue.substring(splitPoint + 1);
          }
          key = tryDecodeURIComponent(key);
          if (isDefined(key)) {
            val = isDefined(val) ? tryDecodeURIComponent(val) : true;
            if (!hasOwnProperty.call(obj, key)) {
              obj[key] = val;
            } else if (isArray(obj[key])) {
              obj[key].push(val);
            } else {
              obj[key] = [obj[key], val];
            }
          }
        }
      });
      return obj;
    }

    function toKeyValue(obj) {
      var parts = [];
      forEach(obj, function (value, key) {
        if (isArray(value)) {
          forEach(value, function (arrayValue) {
            parts.push(encodeUriQuery(key, true) + (arrayValue === true ? '' : '=' + encodeUriQuery(arrayValue, true)));
          });
        } else {
          parts.push(encodeUriQuery(key, true) + (value === true ? '' : '=' + encodeUriQuery(value, true)));
        }
      });
      return parts.length ? parts.join('&') : '';
    }

    /**
     * We need our custom method because encodeURIComponent is too aggressive and doesn't follow
     * http://www.ietf.org/rfc/rfc3986.txt with regards to the character set (pchar) allowed in path
     * segments:
     *    segment       = *pchar
     *    pchar         = unreserved / pct-encoded / sub-delims / ":" / "@"
     *    pct-encoded   = "%" HEXDIG HEXDIG
     *    unreserved    = ALPHA / DIGIT / "-" / "." / "_" / "~"
     *    sub-delims    = "!" / "$" / "&" / "'" / "(" / ")"
     *                     / "*" / "+" / "," / ";" / "="
     */
    function encodeUriSegment(val) {
      return encodeUriQuery(val, true).replace(/%26/gi, '&').replace(/%3D/gi, '=').replace(/%2B/gi, '+');
    }

    /**
     * This method is intended for encoding *key* or *value* parts of query component. We need a custom
     * method because encodeURIComponent is too aggressive and encodes stuff that doesn't have to be
     * encoded per http://tools.ietf.org/html/rfc3986:
     *    query         = *( pchar / "/" / "?" )
     *    pchar         = unreserved / pct-encoded / sub-delims / ":" / "@"
     *    unreserved    = ALPHA / DIGIT / "-" / "." / "_" / "~"
     *    pct-encoded   = "%" HEXDIG HEXDIG
     *    sub-delims    = "!" / "$" / "&" / "'" / "(" / ")"
     *                     / "*" / "+" / "," / ";" / "="
     */
    function encodeUriQuery(val, pctEncodeSpaces) {
      return encodeURIComponent(val).replace(/%40/gi, '@').replace(/%3A/gi, ':').replace(/%24/g, '$').replace(/%2C/gi, ',').replace(/%3B/gi, ';').replace(/%20/g, pctEncodeSpaces ? '%20' : '+');
    }

    var ngAttrPrefixes = ['ng-', 'data-ng-', 'ng:', 'x-ng-'];

    function getNgAttribute(element, ngAttr) {
      var attr,
          i,
          ii = ngAttrPrefixes.length;
      for (i = 0; i < ii; ++i) {
        attr = ngAttrPrefixes[i] + ngAttr;
        if (isString(attr = element.getAttribute(attr))) {
          return attr;
        }
      }
      return null;
    }

    function allowAutoBootstrap(document) {
      var script = document.currentScript;

      if (!script) {
        // IE does not have `document.currentScript`
        return true;
      }

      // If the `currentScript` property has been clobbered just return false, since this indicates a probable attack
      if (!(script instanceof window.HTMLScriptElement || script instanceof window.SVGScriptElement)) {
        return false;
      }

      var attributes = script.attributes;
      var srcs = [attributes.getNamedItem('src'), attributes.getNamedItem('href'), attributes.getNamedItem('xlink:href')];

      return srcs.every(function (src) {
        if (!src) {
          return true;
        }
        if (!src.value) {
          return false;
        }

        var link = document.createElement('a');
        link.href = src.value;

        if (document.location.origin === link.origin) {
          // Same-origin resources are always allowed, even for non-whitelisted schemes.
          return true;
        }
        // Disabled bootstrapping unless angular.js was loaded from a known scheme used on the web.
        // This is to prevent angular.js bundled with browser extensions from being used to bypass the
        // content security policy in web pages and other browser extensions.
        switch (link.protocol) {
          case 'http:':
          case 'https:':
          case 'ftp:':
          case 'blob:':
          case 'file:':
          case 'data:':
            return true;
          default:
            return false;
        }
      });
    }

    // Cached as it has to run during loading so that document.currentScript is available.
    var isAutoBootstrapAllowed = allowAutoBootstrap(window.document);

    /**
     * @ngdoc directive
     * @name ngApp
     * @module ng
     *
     * @element ANY
     * @param {angular.Module} ngApp an optional application
     *   {@link angular.module module} name to load.
     * @param {boolean=} ngStrictDi if this attribute is present on the app element, the injector will be
     *   created in "strict-di" mode. This means that the application will fail to invoke functions which
     *   do not use explicit function annotation (and are thus unsuitable for minification), as described
     *   in {@link guide/di the Dependency Injection guide}, and useful debugging info will assist in
     *   tracking down the root of these bugs.
     *
     * @description
     *
     * Use this directive to **auto-bootstrap** an AngularJS application. The `ngApp` directive
     * designates the **root element** of the application and is typically placed near the root element
     * of the page - e.g. on the `<body>` or `<html>` tags.
     *
     * There are a few things to keep in mind when using `ngApp`:
     * - only one AngularJS application can be auto-bootstrapped per HTML document. The first `ngApp`
     *   found in the document will be used to define the root element to auto-bootstrap as an
     *   application. To run multiple applications in an HTML document you must manually bootstrap them using
     *   {@link angular.bootstrap} instead.
     * - AngularJS applications cannot be nested within each other.
     * - Do not use a directive that uses {@link ng.$compile#transclusion transclusion} on the same element as `ngApp`.
     *   This includes directives such as {@link ng.ngIf `ngIf`}, {@link ng.ngInclude `ngInclude`} and
     *   {@link ngRoute.ngView `ngView`}.
     *   Doing this misplaces the app {@link ng.$rootElement `$rootElement`} and the app's {@link auto.$injector injector},
     *   causing animations to stop working and making the injector inaccessible from outside the app.
     *
     * You can specify an **AngularJS module** to be used as the root module for the application.  This
     * module will be loaded into the {@link auto.$injector} when the application is bootstrapped. It
     * should contain the application code needed or have dependencies on other modules that will
     * contain the code. See {@link angular.module} for more information.
     *
     * In the example below if the `ngApp` directive were not placed on the `html` element then the
     * document would not be compiled, the `AppController` would not be instantiated and the `{{ a+b }}`
     * would not be resolved to `3`.
     *
     * `ngApp` is the easiest, and most common way to bootstrap an application.
     *
     <example module="ngAppDemo" name="ng-app">
       <file name="index.html">
       <div ng-controller="ngAppDemoController">
         I can add: {{a}} + {{b}} =  {{ a+b }}
       </div>
       </file>
       <file name="script.js">
       angular.module('ngAppDemo', []).controller('ngAppDemoController', function($scope) {
         $scope.a = 1;
         $scope.b = 2;
       });
       </file>
     </example>
     *
     * Using `ngStrictDi`, you would see something like this:
     *
     <example ng-app-included="true" name="strict-di">
       <file name="index.html">
       <div ng-app="ngAppStrictDemo" ng-strict-di>
           <div ng-controller="GoodController1">
               I can add: {{a}} + {{b}} =  {{ a+b }}
    
               <p>This renders because the controller does not fail to
                  instantiate, by using explicit annotation style (see
                  script.js for details)
               </p>
           </div>
    
           <div ng-controller="GoodController2">
               Name: <input ng-model="name"><br />
               Hello, {{name}}!
    
               <p>This renders because the controller does not fail to
                  instantiate, by using explicit annotation style
                  (see script.js for details)
               </p>
           </div>
    
           <div ng-controller="BadController">
               I can add: {{a}} + {{b}} =  {{ a+b }}
    
               <p>The controller could not be instantiated, due to relying
                  on automatic function annotations (which are disabled in
                  strict mode). As such, the content of this section is not
                  interpolated, and there should be an error in your web console.
               </p>
           </div>
       </div>
       </file>
       <file name="script.js">
       angular.module('ngAppStrictDemo', [])
         // BadController will fail to instantiate, due to relying on automatic function annotation,
         // rather than an explicit annotation
         .controller('BadController', function($scope) {
           $scope.a = 1;
           $scope.b = 2;
         })
         // Unlike BadController, GoodController1 and GoodController2 will not fail to be instantiated,
         // due to using explicit annotations using the array style and $inject property, respectively.
         .controller('GoodController1', ['$scope', function($scope) {
           $scope.a = 1;
           $scope.b = 2;
         }])
         .controller('GoodController2', GoodController2);
         function GoodController2($scope) {
           $scope.name = 'World';
         }
         GoodController2.$inject = ['$scope'];
       </file>
       <file name="style.css">
       div[ng-controller] {
           margin-bottom: 1em;
           -webkit-border-radius: 4px;
           border-radius: 4px;
           border: 1px solid;
           padding: .5em;
       }
       div[ng-controller^=Good] {
           border-color: #d6e9c6;
           background-color: #dff0d8;
           color: #3c763d;
       }
       div[ng-controller^=Bad] {
           border-color: #ebccd1;
           background-color: #f2dede;
           color: #a94442;
           margin-bottom: 0;
       }
       </file>
     </example>
     */
    function angularInit(element, bootstrap) {
      var appElement,
          module,
          config = {};

      // The element `element` has priority over any other element.
      forEach(ngAttrPrefixes, function (prefix) {
        var name = prefix + 'app';

        if (!appElement && element.hasAttribute && element.hasAttribute(name)) {
          appElement = element;
          module = element.getAttribute(name);
        }
      });
      forEach(ngAttrPrefixes, function (prefix) {
        var name = prefix + 'app';
        var candidate;

        if (!appElement && (candidate = element.querySelector('[' + name.replace(':', '\\:') + ']'))) {
          appElement = candidate;
          module = candidate.getAttribute(name);
        }
      });
      if (appElement) {
        if (!isAutoBootstrapAllowed) {
          window.console.error('Angular: disabling automatic bootstrap. <script> protocol indicates ' + 'an extension, document.location.href does not match.');
          return;
        }
        config.strictDi = getNgAttribute(appElement, 'strict-di') !== null;
        bootstrap(appElement, module ? [module] : [], config);
      }
    }

    /**
     * @ngdoc function
     * @name angular.bootstrap
     * @module ng
     * @description
     * Use this function to manually start up angular application.
     *
     * For more information, see the {@link guide/bootstrap Bootstrap guide}.
     *
     * Angular will detect if it has been loaded into the browser more than once and only allow the
     * first loaded script to be bootstrapped and will report a warning to the browser console for
     * each of the subsequent scripts. This prevents strange results in applications, where otherwise
     * multiple instances of Angular try to work on the DOM.
     *
     * <div class="alert alert-warning">
     * **Note:** Protractor based end-to-end tests cannot use this function to bootstrap manually.
     * They must use {@link ng.directive:ngApp ngApp}.
     * </div>
     *
     * <div class="alert alert-warning">
     * **Note:** Do not bootstrap the app on an element with a directive that uses {@link ng.$compile#transclusion transclusion},
     * such as {@link ng.ngIf `ngIf`}, {@link ng.ngInclude `ngInclude`} and {@link ngRoute.ngView `ngView`}.
     * Doing this misplaces the app {@link ng.$rootElement `$rootElement`} and the app's {@link auto.$injector injector},
     * causing animations to stop working and making the injector inaccessible from outside the app.
     * </div>
     *
     * ```html
     * <!doctype html>
     * <html>
     * <body>
     * <div ng-controller="WelcomeController">
     *   {{greeting}}
     * </div>
     *
     * <script src="angular.js"></script>
     * <script>
     *   var app = angular.module('demo', [])
     *   .controller('WelcomeController', function($scope) {
     *       $scope.greeting = 'Welcome!';
     *   });
     *   angular.bootstrap(document, ['demo']);
     * </script>
     * </body>
     * </html>
     * ```
     *
     * @param {DOMElement} element DOM element which is the root of angular application.
     * @param {Array<String|Function|Array>=} modules an array of modules to load into the application.
     *     Each item in the array should be the name of a predefined module or a (DI annotated)
     *     function that will be invoked by the injector as a `config` block.
     *     See: {@link angular.module modules}
     * @param {Object=} config an object for defining configuration options for the application. The
     *     following keys are supported:
     *
     * * `strictDi` - disable automatic function annotation for the application. This is meant to
     *   assist in finding bugs which break minified code. Defaults to `false`.
     *
     * @returns {auto.$injector} Returns the newly created injector for this app.
     */
    function bootstrap(element, modules, config) {
      if (!isObject(config)) config = {};
      var defaultConfig = {
        strictDi: false
      };
      config = extend(defaultConfig, config);
      var doBootstrap = function () {
        element = jqLite(element);

        if (element.injector()) {
          var tag = element[0] === window.document ? 'document' : startingTag(element);
          // Encode angle brackets to prevent input from being sanitized to empty string #8683.
          throw ngMinErr('btstrpd', 'App already bootstrapped with this element \'{0}\'', tag.replace(/</, '&lt;').replace(/>/, '&gt;'));
        }

        modules = modules || [];
        modules.unshift(['$provide', function ($provide) {
          $provide.value('$rootElement', element);
        }]);

        if (config.debugInfoEnabled) {
          // Pushing so that this overrides `debugInfoEnabled` setting defined in user's `modules`.
          modules.push(['$compileProvider', function ($compileProvider) {
            $compileProvider.debugInfoEnabled(true);
          }]);
        }

        modules.unshift('ng');
        var injector = createInjector(modules, config.strictDi);
        injector.invoke(['$rootScope', '$rootElement', '$compile', '$injector', function bootstrapApply(scope, element, compile, injector) {
          scope.$apply(function () {
            element.data('$injector', injector);
            compile(element)(scope);
          });
        }]);
        return injector;
      };

      var NG_ENABLE_DEBUG_INFO = /^NG_ENABLE_DEBUG_INFO!/;
      var NG_DEFER_BOOTSTRAP = /^NG_DEFER_BOOTSTRAP!/;

      if (window && NG_ENABLE_DEBUG_INFO.test(window.name)) {
        config.debugInfoEnabled = true;
        window.name = window.name.replace(NG_ENABLE_DEBUG_INFO, '');
      }

      if (window && !NG_DEFER_BOOTSTRAP.test(window.name)) {
        return doBootstrap();
      }

      window.name = window.name.replace(NG_DEFER_BOOTSTRAP, '');
      angular.resumeBootstrap = function (extraModules) {
        forEach(extraModules, function (module) {
          modules.push(module);
        });
        return doBootstrap();
      };

      if (isFunction(angular.resumeDeferredBootstrap)) {
        angular.resumeDeferredBootstrap();
      }
    }

    /**
     * @ngdoc function
     * @name angular.reloadWithDebugInfo
     * @module ng
     * @description
     * Use this function to reload the current application with debug information turned on.
     * This takes precedence over a call to `$compileProvider.debugInfoEnabled(false)`.
     *
     * See {@link ng.$compileProvider#debugInfoEnabled} for more.
     */
    function reloadWithDebugInfo() {
      window.name = 'NG_ENABLE_DEBUG_INFO!' + window.name;
      window.location.reload();
    }

    /**
     * @name angular.getTestability
     * @module ng
     * @description
     * Get the testability service for the instance of Angular on the given
     * element.
     * @param {DOMElement} element DOM element which is the root of angular application.
     */
    function getTestability(rootElement) {
      var injector = angular.element(rootElement).injector();
      if (!injector) {
        throw ngMinErr('test', 'no injector found for element argument to getTestability');
      }
      return injector.get('$$testability');
    }

    var SNAKE_CASE_REGEXP = /[A-Z]/g;
    function snake_case(name, separator) {
      separator = separator || '_';
      return name.replace(SNAKE_CASE_REGEXP, function (letter, pos) {
        return (pos ? separator : '') + letter.toLowerCase();
      });
    }

    var bindJQueryFired = false;
    function bindJQuery() {
      var originalCleanData;

      if (bindJQueryFired) {
        return;
      }

      // bind to jQuery if present;
      var jqName = jq();
      jQuery = isUndefined(jqName) ? window.jQuery : // use jQuery (if present)
      !jqName ? undefined : // use jqLite
      window[jqName]; // use jQuery specified by `ngJq`

      // Use jQuery if it exists with proper functionality, otherwise default to us.
      // Angular 1.2+ requires jQuery 1.7+ for on()/off() support.
      // Angular 1.3+ technically requires at least jQuery 2.1+ but it may work with older
      // versions. It will not work for sure with jQuery <1.7, though.
      if (jQuery && jQuery.fn.on) {
        jqLite = jQuery;
        extend(jQuery.fn, {
          scope: JQLitePrototype.scope,
          isolateScope: JQLitePrototype.isolateScope,
          controller: /** @type {?} */JQLitePrototype.controller,
          injector: JQLitePrototype.injector,
          inheritedData: JQLitePrototype.inheritedData
        });

        // All nodes removed from the DOM via various jQuery APIs like .remove()
        // are passed through jQuery.cleanData. Monkey-patch this method to fire
        // the $destroy event on all removed nodes.
        originalCleanData = jQuery.cleanData;
        jQuery.cleanData = function (elems) {
          var events;
          for (var i = 0, elem; (elem = elems[i]) != null; i++) {
            events = jQuery._data(elem, 'events');
            if (events && events.$destroy) {
              jQuery(elem).triggerHandler('$destroy');
            }
          }
          originalCleanData(elems);
        };
      } else {
        jqLite = JQLite;
      }

      angular.element = jqLite;

      // Prevent double-proxying.
      bindJQueryFired = true;
    }

    /**
     * throw error if the argument is falsy.
     */
    function assertArg(arg, name, reason) {
      if (!arg) {
        throw ngMinErr('areq', 'Argument \'{0}\' is {1}', name || '?', reason || 'required');
      }
      return arg;
    }

    function assertArgFn(arg, name, acceptArrayAnnotation) {
      if (acceptArrayAnnotation && isArray(arg)) {
        arg = arg[arg.length - 1];
      }

      assertArg(isFunction(arg), name, 'not a function, got ' + (arg && typeof arg === 'object' ? arg.constructor.name || 'Object' : typeof arg));
      return arg;
    }

    /**
     * throw error if the name given is hasOwnProperty
     * @param  {String} name    the name to test
     * @param  {String} context the context in which the name is used, such as module or directive
     */
    function assertNotHasOwnProperty(name, context) {
      if (name === 'hasOwnProperty') {
        throw ngMinErr('badname', 'hasOwnProperty is not a valid {0} name', context);
      }
    }

    /**
     * Return the value accessible from the object by path. Any undefined traversals are ignored
     * @param {Object} obj starting object
     * @param {String} path path to traverse
     * @param {boolean} [bindFnToScope=true]
     * @returns {Object} value as accessible by path
     */
    //TODO(misko): this function needs to be removed
    function getter(obj, path, bindFnToScope) {
      if (!path) return obj;
      var keys = path.split('.');
      var key;
      var lastInstance = obj;
      var len = keys.length;

      for (var i = 0; i < len; i++) {
        key = keys[i];
        if (obj) {
          obj = (lastInstance = obj)[key];
        }
      }
      if (!bindFnToScope && isFunction(obj)) {
        return bind(lastInstance, obj);
      }
      return obj;
    }

    /**
     * Return the DOM siblings between the first and last node in the given array.
     * @param {Array} array like object
     * @returns {Array} the inputted object or a jqLite collection containing the nodes
     */
    function getBlockNodes(nodes) {
      // TODO(perf): update `nodes` instead of creating a new object?
      var node = nodes[0];
      var endNode = nodes[nodes.length - 1];
      var blockNodes;

      for (var i = 1; node !== endNode && (node = node.nextSibling); i++) {
        if (blockNodes || nodes[i] !== node) {
          if (!blockNodes) {
            blockNodes = jqLite(slice.call(nodes, 0, i));
          }
          blockNodes.push(node);
        }
      }

      return blockNodes || nodes;
    }

    /**
     * Creates a new object without a prototype. This object is useful for lookup without having to
     * guard against prototypically inherited properties via hasOwnProperty.
     *
     * Related micro-benchmarks:
     * - http://jsperf.com/object-create2
     * - http://jsperf.com/proto-map-lookup/2
     * - http://jsperf.com/for-in-vs-object-keys2
     *
     * @returns {Object}
     */
    function createMap() {
      return Object.create(null);
    }

    function stringify(value) {
      if (value == null) {
        // null || undefined
        return '';
      }
      switch (typeof value) {
        case 'string':
          break;
        case 'number':
          value = '' + value;
          break;
        default:
          if (hasCustomToString(value) && !isArray(value) && !isDate(value)) {
            value = value.toString();
          } else {
            value = toJson(value);
          }
      }

      return value;
    }

    var NODE_TYPE_ELEMENT = 1;
    var NODE_TYPE_ATTRIBUTE = 2;
    var NODE_TYPE_TEXT = 3;
    var NODE_TYPE_COMMENT = 8;
    var NODE_TYPE_DOCUMENT = 9;
    var NODE_TYPE_DOCUMENT_FRAGMENT = 11;

    /**
     * @ngdoc type
     * @name angular.Module
     * @module ng
     * @description
     *
     * Interface for configuring angular {@link angular.module modules}.
     */

    function setupModuleLoader(window) {

      var $injectorMinErr = minErr('$injector');
      var ngMinErr = minErr('ng');

      function ensure(obj, name, factory) {
        return obj[name] || (obj[name] = factory());
      }

      var angular = ensure(window, 'angular', Object);

      // We need to expose `angular.$$minErr` to modules such as `ngResource` that reference it during bootstrap
      angular.$$minErr = angular.$$minErr || minErr;

      return ensure(angular, 'module', function () {
        /** @type {Object.<string, angular.Module>} */
        var modules = {};

        /**
         * @ngdoc function
         * @name angular.module
         * @module ng
         * @description
         *
         * The `angular.module` is a global place for creating, registering and retrieving Angular
         * modules.
         * All modules (angular core or 3rd party) that should be available to an application must be
         * registered using this mechanism.
         *
         * Passing one argument retrieves an existing {@link angular.Module},
         * whereas passing more than one argument creates a new {@link angular.Module}
         *
         *
         * # Module
         *
         * A module is a collection of services, directives, controllers, filters, and configuration information.
         * `angular.module` is used to configure the {@link auto.$injector $injector}.
         *
         * ```js
         * // Create a new module
         * var myModule = angular.module('myModule', []);
         *
         * // register a new service
         * myModule.value('appName', 'MyCoolApp');
         *
         * // configure existing services inside initialization blocks.
         * myModule.config(['$locationProvider', function($locationProvider) {
         *   // Configure existing providers
         *   $locationProvider.hashPrefix('!');
         * }]);
         * ```
         *
         * Then you can create an injector and load your modules like this:
         *
         * ```js
         * var injector = angular.injector(['ng', 'myModule'])
         * ```
         *
         * However it's more likely that you'll just use
         * {@link ng.directive:ngApp ngApp} or
         * {@link angular.bootstrap} to simplify this process for you.
         *
         * @param {!string} name The name of the module to create or retrieve.
         * @param {!Array.<string>=} requires If specified then new module is being created. If
         *        unspecified then the module is being retrieved for further configuration.
         * @param {Function=} configFn Optional configuration function for the module. Same as
         *        {@link angular.Module#config Module#config()}.
         * @returns {angular.Module} new module with the {@link angular.Module} api.
         */
        return function module(name, requires, configFn) {

          var info = {};

          var assertNotHasOwnProperty = function (name, context) {
            if (name === 'hasOwnProperty') {
              throw ngMinErr('badname', 'hasOwnProperty is not a valid {0} name', context);
            }
          };

          assertNotHasOwnProperty(name, 'module');
          if (requires && modules.hasOwnProperty(name)) {
            modules[name] = null;
          }
          return ensure(modules, name, function () {
            if (!requires) {
              throw $injectorMinErr('nomod', 'Module \'{0}\' is not available! You either misspelled ' + 'the module name or forgot to load it. If registering a module ensure that you ' + 'specify the dependencies as the second argument.', name);
            }

            /** @type {!Array.<Array.<*>>} */
            var invokeQueue = [];

            /** @type {!Array.<Function>} */
            var configBlocks = [];

            /** @type {!Array.<Function>} */
            var runBlocks = [];

            var config = invokeLater('$injector', 'invoke', 'push', configBlocks);

            /** @type {angular.Module} */
            var moduleInstance = {
              // Private state
              _invokeQueue: invokeQueue,
              _configBlocks: configBlocks,
              _runBlocks: runBlocks,

              /**
               * @ngdoc method
               * @name angular.Module#info
               * @module ng
               *
               * @param {Object=} info Information about the module
               * @returns {Object|Module} The current info object for this module if called as a getter,
               *                          or `this` if called as a setter.
               *
               * @description
               * Read and write custom information about this module.
               * For example you could put the version of the module in here.
               *
               * ```js
               * angular.module('myModule', []).info({ version: '1.0.0' });
               * ```
               *
               * The version could then be read back out by accessing the module elsewhere:
               *
               * ```
               * var version = angular.module('myModule').info().version;
               * ```
               *
               * You can also retrieve this information during runtime via the
               * {@link $injector#modules `$injector.modules`} property:
               *
               * ```js
               * var version = $injector.modules['myModule'].info().version;
               * ```
               */
              info: function (value) {
                if (isDefined(value)) {
                  if (!isObject(value)) throw ngMinErr('aobj', 'Argument \'{0}\' must be an object', 'value');
                  info = value;
                  return this;
                }
                return info;
              },

              /**
               * @ngdoc property
               * @name angular.Module#requires
               * @module ng
               *
               * @description
               * Holds the list of modules which the injector will load before the current module is
               * loaded.
               */
              requires: requires,

              /**
               * @ngdoc property
               * @name angular.Module#name
               * @module ng
               *
               * @description
               * Name of the module.
               */
              name: name,

              /**
               * @ngdoc method
               * @name angular.Module#provider
               * @module ng
               * @param {string} name service name
               * @param {Function} providerType Construction function for creating new instance of the
               *                                service.
               * @description
               * See {@link auto.$provide#provider $provide.provider()}.
               */
              provider: invokeLaterAndSetModuleName('$provide', 'provider'),

              /**
               * @ngdoc method
               * @name angular.Module#factory
               * @module ng
               * @param {string} name service name
               * @param {Function} providerFunction Function for creating new instance of the service.
               * @description
               * See {@link auto.$provide#factory $provide.factory()}.
               */
              factory: invokeLaterAndSetModuleName('$provide', 'factory'),

              /**
               * @ngdoc method
               * @name angular.Module#service
               * @module ng
               * @param {string} name service name
               * @param {Function} constructor A constructor function that will be instantiated.
               * @description
               * See {@link auto.$provide#service $provide.service()}.
               */
              service: invokeLaterAndSetModuleName('$provide', 'service'),

              /**
               * @ngdoc method
               * @name angular.Module#value
               * @module ng
               * @param {string} name service name
               * @param {*} object Service instance object.
               * @description
               * See {@link auto.$provide#value $provide.value()}.
               */
              value: invokeLater('$provide', 'value'),

              /**
               * @ngdoc method
               * @name angular.Module#constant
               * @module ng
               * @param {string} name constant name
               * @param {*} object Constant value.
               * @description
               * Because the constants are fixed, they get applied before other provide methods.
               * See {@link auto.$provide#constant $provide.constant()}.
               */
              constant: invokeLater('$provide', 'constant', 'unshift'),

              /**
              * @ngdoc method
              * @name angular.Module#decorator
              * @module ng
              * @param {string} name The name of the service to decorate.
              * @param {Function} decorFn This function will be invoked when the service needs to be
              *                           instantiated and should return the decorated service instance.
              * @description
              * See {@link auto.$provide#decorator $provide.decorator()}.
              */
              decorator: invokeLaterAndSetModuleName('$provide', 'decorator', configBlocks),

              /**
               * @ngdoc method
               * @name angular.Module#animation
               * @module ng
               * @param {string} name animation name
               * @param {Function} animationFactory Factory function for creating new instance of an
               *                                    animation.
               * @description
               *
               * **NOTE**: animations take effect only if the **ngAnimate** module is loaded.
               *
               *
               * Defines an animation hook that can be later used with
               * {@link $animate $animate} service and directives that use this service.
               *
               * ```js
               * module.animation('.animation-name', function($inject1, $inject2) {
               *   return {
               *     eventName : function(element, done) {
               *       //code to run the animation
               *       //once complete, then run done()
               *       return function cancellationFunction(element) {
               *         //code to cancel the animation
               *       }
               *     }
               *   }
               * })
               * ```
               *
               * See {@link ng.$animateProvider#register $animateProvider.register()} and
               * {@link ngAnimate ngAnimate module} for more information.
               */
              animation: invokeLaterAndSetModuleName('$animateProvider', 'register'),

              /**
               * @ngdoc method
               * @name angular.Module#filter
               * @module ng
               * @param {string} name Filter name - this must be a valid angular expression identifier
               * @param {Function} filterFactory Factory function for creating new instance of filter.
               * @description
               * See {@link ng.$filterProvider#register $filterProvider.register()}.
               *
               * <div class="alert alert-warning">
               * **Note:** Filter names must be valid angular {@link expression} identifiers, such as `uppercase` or `orderBy`.
               * Names with special characters, such as hyphens and dots, are not allowed. If you wish to namespace
               * your filters, then you can use capitalization (`myappSubsectionFilterx`) or underscores
               * (`myapp_subsection_filterx`).
               * </div>
               */
              filter: invokeLaterAndSetModuleName('$filterProvider', 'register'),

              /**
               * @ngdoc method
               * @name angular.Module#controller
               * @module ng
               * @param {string|Object} name Controller name, or an object map of controllers where the
               *    keys are the names and the values are the constructors.
               * @param {Function} constructor Controller constructor function.
               * @description
               * See {@link ng.$controllerProvider#register $controllerProvider.register()}.
               */
              controller: invokeLaterAndSetModuleName('$controllerProvider', 'register'),

              /**
               * @ngdoc method
               * @name angular.Module#directive
               * @module ng
               * @param {string|Object} name Directive name, or an object map of directives where the
               *    keys are the names and the values are the factories.
               * @param {Function} directiveFactory Factory function for creating new instance of
               * directives.
               * @description
               * See {@link ng.$compileProvider#directive $compileProvider.directive()}.
               */
              directive: invokeLaterAndSetModuleName('$compileProvider', 'directive'),

              /**
               * @ngdoc method
               * @name angular.Module#component
               * @module ng
               * @param {string} name Name of the component in camel-case (i.e. myComp which will match as my-comp)
               * @param {Object} options Component definition object (a simplified
               *    {@link ng.$compile#directive-definition-object directive definition object})
               *
               * @description
               * See {@link ng.$compileProvider#component $compileProvider.component()}.
               */
              component: invokeLaterAndSetModuleName('$compileProvider', 'component'),

              /**
               * @ngdoc method
               * @name angular.Module#config
               * @module ng
               * @param {Function} configFn Execute this function on module load. Useful for service
               *    configuration.
               * @description
               * Use this method to register work which needs to be performed on module loading.
               * For more about how to configure services, see
               * {@link providers#provider-recipe Provider Recipe}.
               */
              config: config,

              /**
               * @ngdoc method
               * @name angular.Module#run
               * @module ng
               * @param {Function} initializationFn Execute this function after injector creation.
               *    Useful for application initialization.
               * @description
               * Use this method to register work which should be performed when the injector is done
               * loading all modules.
               */
              run: function (block) {
                runBlocks.push(block);
                return this;
              }
            };

            if (configFn) {
              config(configFn);
            }

            return moduleInstance;

            /**
             * @param {string} provider
             * @param {string} method
             * @param {String=} insertMethod
             * @returns {angular.Module}
             */
            function invokeLater(provider, method, insertMethod, queue) {
              if (!queue) queue = invokeQueue;
              return function () {
                queue[insertMethod || 'push']([provider, method, arguments]);
                return moduleInstance;
              };
            }

            /**
             * @param {string} provider
             * @param {string} method
             * @returns {angular.Module}
             */
            function invokeLaterAndSetModuleName(provider, method, queue) {
              if (!queue) queue = invokeQueue;
              return function (recipeName, factoryFunction) {
                if (factoryFunction && isFunction(factoryFunction)) factoryFunction.$$moduleName = name;
                queue.push([provider, method, arguments]);
                return moduleInstance;
              };
            }
          });
        };
      });
    }

    /* global shallowCopy: true */

    /**
     * Creates a shallow copy of an object, an array or a primitive.
     *
     * Assumes that there are no proto properties for objects.
     */
    function shallowCopy(src, dst) {
      if (isArray(src)) {
        dst = dst || [];

        for (var i = 0, ii = src.length; i < ii; i++) {
          dst[i] = src[i];
        }
      } else if (isObject(src)) {
        dst = dst || {};

        for (var key in src) {
          if (!(key.charAt(0) === '$' && key.charAt(1) === '$')) {
            dst[key] = src[key];
          }
        }
      }

      return dst || src;
    }

    /* global toDebugString: true */

    function serializeObject(obj, maxDepth) {
      var seen = [];

      // There is no direct way to stringify object until reaching a specific depth
      // and a very deep object can cause a performance issue, so we copy the object
      // based on this specific depth and then stringify it.
      if (isValidObjectMaxDepth(maxDepth)) {
        obj = copy(obj, null, maxDepth);
      }
      return JSON.stringify(obj, function (key, val) {
        val = toJsonReplacer(key, val);
        if (isObject(val)) {

          if (seen.indexOf(val) >= 0) return '...';

          seen.push(val);
        }
        return val;
      });
    }

    function toDebugString(obj, maxDepth) {
      if (typeof obj === 'function') {
        return obj.toString().replace(/ \{[\s\S]*$/, '');
      } else if (isUndefined(obj)) {
        return 'undefined';
      } else if (typeof obj !== 'string') {
        return serializeObject(obj, maxDepth);
      }
      return obj;
    }

    /* global angularModule: true,
      version: true,
    
      $CompileProvider,
    
      htmlAnchorDirective,
      inputDirective,
      inputDirective,
      formDirective,
      scriptDirective,
      selectDirective,
      optionDirective,
      ngBindDirective,
      ngBindHtmlDirective,
      ngBindTemplateDirective,
      ngClassDirective,
      ngClassEvenDirective,
      ngClassOddDirective,
      ngCloakDirective,
      ngControllerDirective,
      ngFormDirective,
      ngHideDirective,
      ngIfDirective,
      ngIncludeDirective,
      ngIncludeFillContentDirective,
      ngInitDirective,
      ngNonBindableDirective,
      ngPluralizeDirective,
      ngRepeatDirective,
      ngShowDirective,
      ngStyleDirective,
      ngSwitchDirective,
      ngSwitchWhenDirective,
      ngSwitchDefaultDirective,
      ngOptionsDirective,
      ngTranscludeDirective,
      ngModelDirective,
      ngListDirective,
      ngChangeDirective,
      patternDirective,
      patternDirective,
      requiredDirective,
      requiredDirective,
      minlengthDirective,
      minlengthDirective,
      maxlengthDirective,
      maxlengthDirective,
      ngValueDirective,
      ngModelOptionsDirective,
      ngAttributeAliasDirectives,
      ngEventDirectives,
    
      $AnchorScrollProvider,
      $AnimateProvider,
      $CoreAnimateCssProvider,
      $$CoreAnimateJsProvider,
      $$CoreAnimateQueueProvider,
      $$AnimateRunnerFactoryProvider,
      $$AnimateAsyncRunFactoryProvider,
      $BrowserProvider,
      $CacheFactoryProvider,
      $ControllerProvider,
      $DateProvider,
      $DocumentProvider,
      $$IsDocumentHiddenProvider,
      $ExceptionHandlerProvider,
      $FilterProvider,
      $$ForceReflowProvider,
      $InterpolateProvider,
      $IntervalProvider,
      $HttpProvider,
      $HttpParamSerializerProvider,
      $HttpParamSerializerJQLikeProvider,
      $HttpBackendProvider,
      $xhrFactoryProvider,
      $jsonpCallbacksProvider,
      $LocationProvider,
      $LogProvider,
      $$MapProvider,
      $ParseProvider,
      $RootScopeProvider,
      $QProvider,
      $$QProvider,
      $$SanitizeUriProvider,
      $SceProvider,
      $SceDelegateProvider,
      $SnifferProvider,
      $TemplateCacheProvider,
      $TemplateRequestProvider,
      $$TestabilityProvider,
      $TimeoutProvider,
      $$RAFProvider,
      $WindowProvider,
      $$jqLiteProvider,
      $$CookieReaderProvider
    */

    /**
     * @ngdoc object
     * @name angular.version
     * @module ng
     * @description
     * An object that contains information about the current AngularJS version.
     *
     * This object has the following properties:
     *
     * - `full` – `{string}` – Full version string, such as "0.9.18".
     * - `major` – `{number}` – Major version number, such as "0".
     * - `minor` – `{number}` – Minor version number, such as "9".
     * - `dot` – `{number}` – Dot version number, such as "18".
     * - `codeName` – `{string}` – Code name of the release, such as "jiggling-armfat".
     */
    var version = {
      // These placeholder strings will be replaced by grunt's `build` task.
      // They need to be double- or single-quoted.
      full: '1.6.3',
      major: 1,
      minor: 6,
      dot: 3,
      codeName: 'scriptalicious-bootstrapping'
    };

    function publishExternalAPI(angular) {
      extend(angular, {
        'errorHandlingConfig': errorHandlingConfig,
        'bootstrap': bootstrap,
        'copy': copy,
        'extend': extend,
        'merge': merge,
        'equals': equals,
        'element': jqLite,
        'forEach': forEach,
        'injector': createInjector,
        'noop': noop,
        'bind': bind,
        'toJson': toJson,
        'fromJson': fromJson,
        'identity': identity,
        'isUndefined': isUndefined,
        'isDefined': isDefined,
        'isString': isString,
        'isFunction': isFunction,
        'isObject': isObject,
        'isNumber': isNumber,
        'isElement': isElement,
        'isArray': isArray,
        'version': version,
        'isDate': isDate,
        'lowercase': lowercase,
        'uppercase': uppercase,
        'callbacks': { $$counter: 0 },
        'getTestability': getTestability,
        'reloadWithDebugInfo': reloadWithDebugInfo,
        '$$minErr': minErr,
        '$$csp': csp,
        '$$encodeUriSegment': encodeUriSegment,
        '$$encodeUriQuery': encodeUriQuery,
        '$$stringify': stringify
      });

      angularModule = setupModuleLoader(window);

      angularModule('ng', ['ngLocale'], ['$provide', function ngModule($provide) {
        // $$sanitizeUriProvider needs to be before $compileProvider as it is used by it.
        $provide.provider({
          $$sanitizeUri: $$SanitizeUriProvider
        });
        $provide.provider('$compile', $CompileProvider).directive({
          a: htmlAnchorDirective,
          input: inputDirective,
          textarea: inputDirective,
          form: formDirective,
          script: scriptDirective,
          select: selectDirective,
          option: optionDirective,
          ngBind: ngBindDirective,
          ngBindHtml: ngBindHtmlDirective,
          ngBindTemplate: ngBindTemplateDirective,
          ngClass: ngClassDirective,
          ngClassEven: ngClassEvenDirective,
          ngClassOdd: ngClassOddDirective,
          ngCloak: ngCloakDirective,
          ngController: ngControllerDirective,
          ngForm: ngFormDirective,
          ngHide: ngHideDirective,
          ngIf: ngIfDirective,
          ngInclude: ngIncludeDirective,
          ngInit: ngInitDirective,
          ngNonBindable: ngNonBindableDirective,
          ngPluralize: ngPluralizeDirective,
          ngRepeat: ngRepeatDirective,
          ngShow: ngShowDirective,
          ngStyle: ngStyleDirective,
          ngSwitch: ngSwitchDirective,
          ngSwitchWhen: ngSwitchWhenDirective,
          ngSwitchDefault: ngSwitchDefaultDirective,
          ngOptions: ngOptionsDirective,
          ngTransclude: ngTranscludeDirective,
          ngModel: ngModelDirective,
          ngList: ngListDirective,
          ngChange: ngChangeDirective,
          pattern: patternDirective,
          ngPattern: patternDirective,
          required: requiredDirective,
          ngRequired: requiredDirective,
          minlength: minlengthDirective,
          ngMinlength: minlengthDirective,
          maxlength: maxlengthDirective,
          ngMaxlength: maxlengthDirective,
          ngValue: ngValueDirective,
          ngModelOptions: ngModelOptionsDirective
        }).directive({
          ngInclude: ngIncludeFillContentDirective
        }).directive(ngAttributeAliasDirectives).directive(ngEventDirectives);
        $provide.provider({
          $anchorScroll: $AnchorScrollProvider,
          $animate: $AnimateProvider,
          $animateCss: $CoreAnimateCssProvider,
          $$animateJs: $$CoreAnimateJsProvider,
          $$animateQueue: $$CoreAnimateQueueProvider,
          $$AnimateRunner: $$AnimateRunnerFactoryProvider,
          $$animateAsyncRun: $$AnimateAsyncRunFactoryProvider,
          $browser: $BrowserProvider,
          $cacheFactory: $CacheFactoryProvider,
          $controller: $ControllerProvider,
          $document: $DocumentProvider,
          $$isDocumentHidden: $$IsDocumentHiddenProvider,
          $exceptionHandler: $ExceptionHandlerProvider,
          $filter: $FilterProvider,
          $$forceReflow: $$ForceReflowProvider,
          $interpolate: $InterpolateProvider,
          $interval: $IntervalProvider,
          $http: $HttpProvider,
          $httpParamSerializer: $HttpParamSerializerProvider,
          $httpParamSerializerJQLike: $HttpParamSerializerJQLikeProvider,
          $httpBackend: $HttpBackendProvider,
          $xhrFactory: $xhrFactoryProvider,
          $jsonpCallbacks: $jsonpCallbacksProvider,
          $location: $LocationProvider,
          $log: $LogProvider,
          $parse: $ParseProvider,
          $rootScope: $RootScopeProvider,
          $q: $QProvider,
          $$q: $$QProvider,
          $sce: $SceProvider,
          $sceDelegate: $SceDelegateProvider,
          $sniffer: $SnifferProvider,
          $templateCache: $TemplateCacheProvider,
          $templateRequest: $TemplateRequestProvider,
          $$testability: $$TestabilityProvider,
          $timeout: $TimeoutProvider,
          $window: $WindowProvider,
          $$rAF: $$RAFProvider,
          $$jqLite: $$jqLiteProvider,
          $$Map: $$MapProvider,
          $$cookieReader: $$CookieReaderProvider
        });
      }]).info({ angularVersion: '1.6.3' });
    }

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
     *     Any commits to this file should be reviewed with security in mind.  *
     *   Changes to this file can potentially create security vulnerabilities. *
     *          An approval from 2 Core members with history of modifying      *
     *                         this file is required.                          *
     *                                                                         *
     *  Does the change somehow allow for arbitrary javascript to be executed? *
     *    Or allows for someone to change the prototype of built-in objects?   *
     *     Or gives undesired access to variables likes document or window?    *
     * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

    /* global
      JQLitePrototype: true,
      BOOLEAN_ATTR: true,
      ALIASED_ATTR: true
    */

    //////////////////////////////////
    //JQLite
    //////////////////////////////////

    /**
     * @ngdoc function
     * @name angular.element
     * @module ng
     * @kind function
     *
     * @description
     * Wraps a raw DOM element or HTML string as a [jQuery](http://jquery.com) element.
     *
     * If jQuery is available, `angular.element` is an alias for the
     * [jQuery](http://api.jquery.com/jQuery/) function. If jQuery is not available, `angular.element`
     * delegates to Angular's built-in subset of jQuery, called "jQuery lite" or **jqLite**.
     *
     * jqLite is a tiny, API-compatible subset of jQuery that allows
     * Angular to manipulate the DOM in a cross-browser compatible way. jqLite implements only the most
     * commonly needed functionality with the goal of having a very small footprint.
     *
     * To use `jQuery`, simply ensure it is loaded before the `angular.js` file. You can also use the
     * {@link ngJq `ngJq`} directive to specify that jqlite should be used over jQuery, or to use a
     * specific version of jQuery if multiple versions exist on the page.
     *
     * <div class="alert alert-info">**Note:** All element references in Angular are always wrapped with jQuery or
     * jqLite (such as the element argument in a directive's compile / link function). They are never raw DOM references.</div>
     *
     * <div class="alert alert-warning">**Note:** Keep in mind that this function will not find elements
     * by tag name / CSS selector. For lookups by tag name, try instead `angular.element(document).find(...)`
     * or `$document.find()`, or use the standard DOM APIs, e.g. `document.querySelectorAll()`.</div>
     *
     * ## Angular's jqLite
     * jqLite provides only the following jQuery methods:
     *
     * - [`addClass()`](http://api.jquery.com/addClass/) - Does not support a function as first argument
     * - [`after()`](http://api.jquery.com/after/)
     * - [`append()`](http://api.jquery.com/append/)
     * - [`attr()`](http://api.jquery.com/attr/) - Does not support functions as parameters
     * - [`bind()`](http://api.jquery.com/bind/) (_deprecated_, use [`on()`](http://api.jquery.com/on/)) - Does not support namespaces, selectors or eventData
     * - [`children()`](http://api.jquery.com/children/) - Does not support selectors
     * - [`clone()`](http://api.jquery.com/clone/)
     * - [`contents()`](http://api.jquery.com/contents/)
     * - [`css()`](http://api.jquery.com/css/) - Only retrieves inline-styles, does not call `getComputedStyle()`.
     *   As a setter, does not convert numbers to strings or append 'px', and also does not have automatic property prefixing.
     * - [`data()`](http://api.jquery.com/data/)
     * - [`detach()`](http://api.jquery.com/detach/)
     * - [`empty()`](http://api.jquery.com/empty/)
     * - [`eq()`](http://api.jquery.com/eq/)
     * - [`find()`](http://api.jquery.com/find/) - Limited to lookups by tag name
     * - [`hasClass()`](http://api.jquery.com/hasClass/)
     * - [`html()`](http://api.jquery.com/html/)
     * - [`next()`](http://api.jquery.com/next/) - Does not support selectors
     * - [`on()`](http://api.jquery.com/on/) - Does not support namespaces, selectors or eventData
     * - [`off()`](http://api.jquery.com/off/) - Does not support namespaces, selectors or event object as parameter
     * - [`one()`](http://api.jquery.com/one/) - Does not support namespaces or selectors
     * - [`parent()`](http://api.jquery.com/parent/) - Does not support selectors
     * - [`prepend()`](http://api.jquery.com/prepend/)
     * - [`prop()`](http://api.jquery.com/prop/)
     * - [`ready()`](http://api.jquery.com/ready/) (_deprecated_, use `angular.element(callback)` instead of `angular.element(document).ready(callback)`)
     * - [`remove()`](http://api.jquery.com/remove/)
     * - [`removeAttr()`](http://api.jquery.com/removeAttr/) - Does not support multiple attributes
     * - [`removeClass()`](http://api.jquery.com/removeClass/) - Does not support a function as first argument
     * - [`removeData()`](http://api.jquery.com/removeData/)
     * - [`replaceWith()`](http://api.jquery.com/replaceWith/)
     * - [`text()`](http://api.jquery.com/text/)
     * - [`toggleClass()`](http://api.jquery.com/toggleClass/) - Does not support a function as first argument
     * - [`triggerHandler()`](http://api.jquery.com/triggerHandler/) - Passes a dummy event object to handlers
     * - [`unbind()`](http://api.jquery.com/unbind/) (_deprecated_, use [`off()`](http://api.jquery.com/off/)) - Does not support namespaces or event object as parameter
     * - [`val()`](http://api.jquery.com/val/)
     * - [`wrap()`](http://api.jquery.com/wrap/)
     *
     * ## jQuery/jqLite Extras
     * Angular also provides the following additional methods and events to both jQuery and jqLite:
     *
     * ### Events
     * - `$destroy` - AngularJS intercepts all jqLite/jQuery's DOM destruction apis and fires this event
     *    on all DOM nodes being removed.  This can be used to clean up any 3rd party bindings to the DOM
     *    element before it is removed.
     *
     * ### Methods
     * - `controller(name)` - retrieves the controller of the current element or its parent. By default
     *   retrieves controller associated with the `ngController` directive. If `name` is provided as
     *   camelCase directive name, then the controller for this directive will be retrieved (e.g.
     *   `'ngModel'`).
     * - `injector()` - retrieves the injector of the current element or its parent.
     * - `scope()` - retrieves the {@link ng.$rootScope.Scope scope} of the current
     *   element or its parent. Requires {@link guide/production#disabling-debug-data Debug Data} to
     *   be enabled.
     * - `isolateScope()` - retrieves an isolate {@link ng.$rootScope.Scope scope} if one is attached directly to the
     *   current element. This getter should be used only on elements that contain a directive which starts a new isolate
     *   scope. Calling `scope()` on this element always returns the original non-isolate scope.
     *   Requires {@link guide/production#disabling-debug-data Debug Data} to be enabled.
     * - `inheritedData()` - same as `data()`, but walks up the DOM until a value is found or the top
     *   parent element is reached.
     *
     * @knownIssue You cannot spy on `angular.element` if you are using Jasmine version 1.x. See
     * https://github.com/angular/angular.js/issues/14251 for more information.
     *
     * @param {string|DOMElement} element HTML string or DOMElement to be wrapped into jQuery.
     * @returns {Object} jQuery object.
     */

    JQLite.expando = 'ng339';

    var jqCache = JQLite.cache = {},
        jqId = 1;

    /*
     * !!! This is an undocumented "private" function !!!
     */
    JQLite._data = function (node) {
      //jQuery always returns an object on cache miss
      return this.cache[node[this.expando]] || {};
    };

    function jqNextId() {
      return ++jqId;
    }

    var DASH_LOWERCASE_REGEXP = /-([a-z])/g;
    var MS_HACK_REGEXP = /^-ms-/;
    var MOUSE_EVENT_MAP = { mouseleave: 'mouseout', mouseenter: 'mouseover' };
    var jqLiteMinErr = minErr('jqLite');

    /**
     * Converts kebab-case to camelCase.
     * There is also a special case for the ms prefix starting with a lowercase letter.
     * @param name Name to normalize
     */
    function cssKebabToCamel(name) {
      return kebabToCamel(name.replace(MS_HACK_REGEXP, 'ms-'));
    }

    function fnCamelCaseReplace(all, letter) {
      return letter.toUpperCase();
    }

    /**
     * Converts kebab-case to camelCase.
     * @param name Name to normalize
     */
    function kebabToCamel(name) {
      return name.replace(DASH_LOWERCASE_REGEXP, fnCamelCaseReplace);
    }

    var SINGLE_TAG_REGEXP = /^<([\w-]+)\s*\/?>(?:<\/\1>|)$/;
    var HTML_REGEXP = /<|&#?\w+;/;
    var TAG_NAME_REGEXP = /<([\w:-]+)/;
    var XHTML_TAG_REGEXP = /<(?!area|br|col|embed|hr|img|input|link|meta|param)(([\w:-]+)[^>]*)\/>/gi;

    var wrapMap = {
      'option': [1, '<select multiple="multiple">', '</select>'],

      'thead': [1, '<table>', '</table>'],
      'col': [2, '<table><colgroup>', '</colgroup></table>'],
      'tr': [2, '<table><tbody>', '</tbody></table>'],
      'td': [3, '<table><tbody><tr>', '</tr></tbody></table>'],
      '_default': [0, '', '']
    };

    wrapMap.optgroup = wrapMap.option;
    wrapMap.tbody = wrapMap.tfoot = wrapMap.colgroup = wrapMap.caption = wrapMap.thead;
    wrapMap.th = wrapMap.td;

    function jqLiteIsTextNode(html) {
      return !HTML_REGEXP.test(html);
    }

    function jqLiteAcceptsData(node) {
      // The window object can accept data but has no nodeType
      // Otherwise we are only interested in elements (1) and documents (9)
      var nodeType = node.nodeType;
      return nodeType === NODE_TYPE_ELEMENT || !nodeType || nodeType === NODE_TYPE_DOCUMENT;
    }

    function jqLiteHasData(node) {
      for (var key in jqCache[node.ng339]) {
        return true;
      }
      return false;
    }

    function jqLiteCleanData(nodes) {
      for (var i = 0, ii = nodes.length; i < ii; i++) {
        jqLiteRemoveData(nodes[i]);
      }
    }

    function jqLiteBuildFragment(html, context) {
      var tmp,
          tag,
          wrap,
          fragment = context.createDocumentFragment(),
          nodes = [],
          i;

      if (jqLiteIsTextNode(html)) {
        // Convert non-html into a text node
        nodes.push(context.createTextNode(html));
      } else {
        // Convert html into DOM nodes
        tmp = fragment.appendChild(context.createElement('div'));
        tag = (TAG_NAME_REGEXP.exec(html) || ['', ''])[1].toLowerCase();
        wrap = wrapMap[tag] || wrapMap._default;
        tmp.innerHTML = wrap[1] + html.replace(XHTML_TAG_REGEXP, '<$1></$2>') + wrap[2];

        // Descend through wrappers to the right content
        i = wrap[0];
        while (i--) {
          tmp = tmp.lastChild;
        }

        nodes = concat(nodes, tmp.childNodes);

        tmp = fragment.firstChild;
        tmp.textContent = '';
      }

      // Remove wrapper from fragment
      fragment.textContent = '';
      fragment.innerHTML = ''; // Clear inner HTML
      forEach(nodes, function (node) {
        fragment.appendChild(node);
      });

      return fragment;
    }

    function jqLiteParseHTML(html, context) {
      context = context || window.document;
      var parsed;

      if (parsed = SINGLE_TAG_REGEXP.exec(html)) {
        return [context.createElement(parsed[1])];
      }

      if (parsed = jqLiteBuildFragment(html, context)) {
        return parsed.childNodes;
      }

      return [];
    }

    function jqLiteWrapNode(node, wrapper) {
      var parent = node.parentNode;

      if (parent) {
        parent.replaceChild(wrapper, node);
      }

      wrapper.appendChild(node);
    }

    // IE9-11 has no method "contains" in SVG element and in Node.prototype. Bug #10259.
    var jqLiteContains = window.Node.prototype.contains || /** @this */function (arg) {
      // eslint-disable-next-line no-bitwise
      return !!(this.compareDocumentPosition(arg) & 16);
    };

    /////////////////////////////////////////////
    function JQLite(element) {
      if (element instanceof JQLite) {
        return element;
      }

      var argIsString;

      if (isString(element)) {
        element = trim(element);
        argIsString = true;
      }
      if (!(this instanceof JQLite)) {
        if (argIsString && element.charAt(0) !== '<') {
          throw jqLiteMinErr('nosel', 'Looking up elements via selectors is not supported by jqLite! See: http://docs.angularjs.org/api/angular.element');
        }
        return new JQLite(element);
      }

      if (argIsString) {
        jqLiteAddNodes(this, jqLiteParseHTML(element));
      } else if (isFunction(element)) {
        jqLiteReady(element);
      } else {
        jqLiteAddNodes(this, element);
      }
    }

    function jqLiteClone(element) {
      return element.cloneNode(true);
    }

    function jqLiteDealoc(element, onlyDescendants) {
      if (!onlyDescendants) jqLiteRemoveData(element);

      if (element.querySelectorAll) {
        var descendants = element.querySelectorAll('*');
        for (var i = 0, l = descendants.length; i < l; i++) {
          jqLiteRemoveData(descendants[i]);
        }
      }
    }

    function jqLiteOff(element, type, fn, unsupported) {
      if (isDefined(unsupported)) throw jqLiteMinErr('offargs', 'jqLite#off() does not support the `selector` argument');

      var expandoStore = jqLiteExpandoStore(element);
      var events = expandoStore && expandoStore.events;
      var handle = expandoStore && expandoStore.handle;

      if (!handle) return; //no listeners registered

      if (!type) {
        for (type in events) {
          if (type !== '$destroy') {
            element.removeEventListener(type, handle);
          }
          delete events[type];
        }
      } else {

        var removeHandler = function (type) {
          var listenerFns = events[type];
          if (isDefined(fn)) {
            arrayRemove(listenerFns || [], fn);
          }
          if (!(isDefined(fn) && listenerFns && listenerFns.length > 0)) {
            element.removeEventListener(type, handle);
            delete events[type];
          }
        };

        forEach(type.split(' '), function (type) {
          removeHandler(type);
          if (MOUSE_EVENT_MAP[type]) {
            removeHandler(MOUSE_EVENT_MAP[type]);
          }
        });
      }
    }

    function jqLiteRemoveData(element, name) {
      var expandoId = element.ng339;
      var expandoStore = expandoId && jqCache[expandoId];

      if (expandoStore) {
        if (name) {
          delete expandoStore.data[name];
          return;
        }

        if (expandoStore.handle) {
          if (expandoStore.events.$destroy) {
            expandoStore.handle({}, '$destroy');
          }
          jqLiteOff(element);
        }
        delete jqCache[expandoId];
        element.ng339 = undefined; // don't delete DOM expandos. IE and Chrome don't like it
      }
    }

    function jqLiteExpandoStore(element, createIfNecessary) {
      var expandoId = element.ng339,
          expandoStore = expandoId && jqCache[expandoId];

      if (createIfNecessary && !expandoStore) {
        element.ng339 = expandoId = jqNextId();
        expandoStore = jqCache[expandoId] = { events: {}, data: {}, handle: undefined };
      }

      return expandoStore;
    }

    function jqLiteData(element, key, value) {
      if (jqLiteAcceptsData(element)) {
        var prop;

        var isSimpleSetter = isDefined(value);
        var isSimpleGetter = !isSimpleSetter && key && !isObject(key);
        var massGetter = !key;
        var expandoStore = jqLiteExpandoStore(element, !isSimpleGetter);
        var data = expandoStore && expandoStore.data;

        if (isSimpleSetter) {
          // data('key', value)
          data[kebabToCamel(key)] = value;
        } else {
          if (massGetter) {
            // data()
            return data;
          } else {
            if (isSimpleGetter) {
              // data('key')
              // don't force creation of expandoStore if it doesn't exist yet
              return data && data[kebabToCamel(key)];
            } else {
              // mass-setter: data({key1: val1, key2: val2})
              for (prop in key) {
                data[kebabToCamel(prop)] = key[prop];
              }
            }
          }
        }
      }
    }

    function jqLiteHasClass(element, selector) {
      if (!element.getAttribute) return false;
      return (' ' + (element.getAttribute('class') || '') + ' ').replace(/[\n\t]/g, ' ').indexOf(' ' + selector + ' ') > -1;
    }

    function jqLiteRemoveClass(element, cssClasses) {
      if (cssClasses && element.setAttribute) {
        forEach(cssClasses.split(' '), function (cssClass) {
          element.setAttribute('class', trim((' ' + (element.getAttribute('class') || '') + ' ').replace(/[\n\t]/g, ' ').replace(' ' + trim(cssClass) + ' ', ' ')));
        });
      }
    }

    function jqLiteAddClass(element, cssClasses) {
      if (cssClasses && element.setAttribute) {
        var existingClasses = (' ' + (element.getAttribute('class') || '') + ' ').replace(/[\n\t]/g, ' ');

        forEach(cssClasses.split(' '), function (cssClass) {
          cssClass = trim(cssClass);
          if (existingClasses.indexOf(' ' + cssClass + ' ') === -1) {
            existingClasses += cssClass + ' ';
          }
        });

        element.setAttribute('class', trim(existingClasses));
      }
    }

    function jqLiteAddNodes(root, elements) {
      // THIS CODE IS VERY HOT. Don't make changes without benchmarking.

      if (elements) {

        // if a Node (the most common case)
        if (elements.nodeType) {
          root[root.length++] = elements;
        } else {
          var length = elements.length;

          // if an Array or NodeList and not a Window
          if (typeof length === 'number' && elements.window !== elements) {
            if (length) {
              for (var i = 0; i < length; i++) {
                root[root.length++] = elements[i];
              }
            }
          } else {
            root[root.length++] = elements;
          }
        }
      }
    }

    function jqLiteController(element, name) {
      return jqLiteInheritedData(element, '$' + (name || 'ngController') + 'Controller');
    }

    function jqLiteInheritedData(element, name, value) {
      // if element is the document object work with the html element instead
      // this makes $(document).scope() possible
      if (element.nodeType === NODE_TYPE_DOCUMENT) {
        element = element.documentElement;
      }
      var names = isArray(name) ? name : [name];

      while (element) {
        for (var i = 0, ii = names.length; i < ii; i++) {
          if (isDefined(value = jqLite.data(element, names[i]))) return value;
        }

        // If dealing with a document fragment node with a host element, and no parent, use the host
        // element as the parent. This enables directives within a Shadow DOM or polyfilled Shadow DOM
        // to lookup parent controllers.
        element = element.parentNode || element.nodeType === NODE_TYPE_DOCUMENT_FRAGMENT && element.host;
      }
    }

    function jqLiteEmpty(element) {
      jqLiteDealoc(element, true);
      while (element.firstChild) {
        element.removeChild(element.firstChild);
      }
    }

    function jqLiteRemove(element, keepData) {
      if (!keepData) jqLiteDealoc(element);
      var parent = element.parentNode;
      if (parent) parent.removeChild(element);
    }

    function jqLiteDocumentLoaded(action, win) {
      win = win || window;
      if (win.document.readyState === 'complete') {
        // Force the action to be run async for consistent behavior
        // from the action's point of view
        // i.e. it will definitely not be in a $apply
        win.setTimeout(action);
      } else {
        // No need to unbind this handler as load is only ever called once
        jqLite(win).on('load', action);
      }
    }

    function jqLiteReady(fn) {
      function trigger() {
        window.document.removeEventListener('DOMContentLoaded', trigger);
        window.removeEventListener('load', trigger);
        fn();
      }

      // check if document is already loaded
      if (window.document.readyState === 'complete') {
        window.setTimeout(fn);
      } else {
        // We can not use jqLite since we are not done loading and jQuery could be loaded later.

        // Works for modern browsers and IE9
        window.document.addEventListener('DOMContentLoaded', trigger);

        // Fallback to window.onload for others
        window.addEventListener('load', trigger);
      }
    }

    //////////////////////////////////////////
    // Functions which are declared directly.
    //////////////////////////////////////////
    var JQLitePrototype = JQLite.prototype = {
      ready: jqLiteReady,
      toString: function () {
        var value = [];
        forEach(this, function (e) {
          value.push('' + e);
        });
        return '[' + value.join(', ') + ']';
      },

      eq: function (index) {
        return index >= 0 ? jqLite(this[index]) : jqLite(this[this.length + index]);
      },

      length: 0,
      push: push,
      sort: [].sort,
      splice: [].splice
    };

    //////////////////////////////////////////
    // Functions iterating getter/setters.
    // these functions return self on setter and
    // value on get.
    //////////////////////////////////////////
    var BOOLEAN_ATTR = {};
    forEach('multiple,selected,checked,disabled,readOnly,required,open'.split(','), function (value) {
      BOOLEAN_ATTR[lowercase(value)] = value;
    });
    var BOOLEAN_ELEMENTS = {};
    forEach('input,select,option,textarea,button,form,details'.split(','), function (value) {
      BOOLEAN_ELEMENTS[value] = true;
    });
    var ALIASED_ATTR = {
      'ngMinlength': 'minlength',
      'ngMaxlength': 'maxlength',
      'ngMin': 'min',
      'ngMax': 'max',
      'ngPattern': 'pattern',
      'ngStep': 'step'
    };

    function getBooleanAttrName(element, name) {
      // check dom last since we will most likely fail on name
      var booleanAttr = BOOLEAN_ATTR[name.toLowerCase()];

      // booleanAttr is here twice to minimize DOM access
      return booleanAttr && BOOLEAN_ELEMENTS[nodeName_(element)] && booleanAttr;
    }

    function getAliasedAttrName(name) {
      return ALIASED_ATTR[name];
    }

    forEach({
      data: jqLiteData,
      removeData: jqLiteRemoveData,
      hasData: jqLiteHasData,
      cleanData: jqLiteCleanData
    }, function (fn, name) {
      JQLite[name] = fn;
    });

    forEach({
      data: jqLiteData,
      inheritedData: jqLiteInheritedData,

      scope: function (element) {
        // Can't use jqLiteData here directly so we stay compatible with jQuery!
        return jqLite.data(element, '$scope') || jqLiteInheritedData(element.parentNode || element, ['$isolateScope', '$scope']);
      },

      isolateScope: function (element) {
        // Can't use jqLiteData here directly so we stay compatible with jQuery!
        return jqLite.data(element, '$isolateScope') || jqLite.data(element, '$isolateScopeNoTemplate');
      },

      controller: jqLiteController,

      injector: function (element) {
        return jqLiteInheritedData(element, '$injector');
      },

      removeAttr: function (element, name) {
        element.removeAttribute(name);
      },

      hasClass: jqLiteHasClass,

      css: function (element, name, value) {
        name = cssKebabToCamel(name);

        if (isDefined(value)) {
          element.style[name] = value;
        } else {
          return element.style[name];
        }
      },

      attr: function (element, name, value) {
        var ret;
        var nodeType = element.nodeType;
        if (nodeType === NODE_TYPE_TEXT || nodeType === NODE_TYPE_ATTRIBUTE || nodeType === NODE_TYPE_COMMENT || !element.getAttribute) {
          return;
        }

        var lowercasedName = lowercase(name);
        var isBooleanAttr = BOOLEAN_ATTR[lowercasedName];

        if (isDefined(value)) {
          // setter

          if (value === null || value === false && isBooleanAttr) {
            element.removeAttribute(name);
          } else {
            element.setAttribute(name, isBooleanAttr ? lowercasedName : value);
          }
        } else {
          // getter

          ret = element.getAttribute(name);

          if (isBooleanAttr && ret !== null) {
            ret = lowercasedName;
          }
          // Normalize non-existing attributes to undefined (as jQuery).
          return ret === null ? undefined : ret;
        }
      },

      prop: function (element, name, value) {
        if (isDefined(value)) {
          element[name] = value;
        } else {
          return element[name];
        }
      },

      text: function () {
        getText.$dv = '';
        return getText;

        function getText(element, value) {
          if (isUndefined(value)) {
            var nodeType = element.nodeType;
            return nodeType === NODE_TYPE_ELEMENT || nodeType === NODE_TYPE_TEXT ? element.textContent : '';
          }
          element.textContent = value;
        }
      }(),

      val: function (element, value) {
        if (isUndefined(value)) {
          if (element.multiple && nodeName_(element) === 'select') {
            var result = [];
            forEach(element.options, function (option) {
              if (option.selected) {
                result.push(option.value || option.text);
              }
            });
            return result;
          }
          return element.value;
        }
        element.value = value;
      },

      html: function (element, value) {
        if (isUndefined(value)) {
          return element.innerHTML;
        }
        jqLiteDealoc(element, true);
        element.innerHTML = value;
      },

      empty: jqLiteEmpty
    }, function (fn, name) {
      /**
       * Properties: writes return selection, reads return first value
       */
      JQLite.prototype[name] = function (arg1, arg2) {
        var i, key;
        var nodeCount = this.length;

        // jqLiteHasClass has only two arguments, but is a getter-only fn, so we need to special-case it
        // in a way that survives minification.
        // jqLiteEmpty takes no arguments but is a setter.
        if (fn !== jqLiteEmpty && isUndefined(fn.length === 2 && fn !== jqLiteHasClass && fn !== jqLiteController ? arg1 : arg2)) {
          if (isObject(arg1)) {

            // we are a write, but the object properties are the key/values
            for (i = 0; i < nodeCount; i++) {
              if (fn === jqLiteData) {
                // data() takes the whole object in jQuery
                fn(this[i], arg1);
              } else {
                for (key in arg1) {
                  fn(this[i], key, arg1[key]);
                }
              }
            }
            // return self for chaining
            return this;
          } else {
            // we are a read, so read the first child.
            // TODO: do we still need this?
            var value = fn.$dv;
            // Only if we have $dv do we iterate over all, otherwise it is just the first element.
            var jj = isUndefined(value) ? Math.min(nodeCount, 1) : nodeCount;
            for (var j = 0; j < jj; j++) {
              var nodeValue = fn(this[j], arg1, arg2);
              value = value ? value + nodeValue : nodeValue;
            }
            return value;
          }
        } else {
          // we are a write, so apply to all children
          for (i = 0; i < nodeCount; i++) {
            fn(this[i], arg1, arg2);
          }
          // return self for chaining
          return this;
        }
      };
    });

    function createEventHandler(element, events) {
      var eventHandler = function (event, type) {
        // jQuery specific api
        event.isDefaultPrevented = function () {
          return event.defaultPrevented;
        };

        var eventFns = events[type || event.type];
        var eventFnsLength = eventFns ? eventFns.length : 0;

        if (!eventFnsLength) return;

        if (isUndefined(event.immediatePropagationStopped)) {
          var originalStopImmediatePropagation = event.stopImmediatePropagation;
          event.stopImmediatePropagation = function () {
            event.immediatePropagationStopped = true;

            if (event.stopPropagation) {
              event.stopPropagation();
            }

            if (originalStopImmediatePropagation) {
              originalStopImmediatePropagation.call(event);
            }
          };
        }

        event.isImmediatePropagationStopped = function () {
          return event.immediatePropagationStopped === true;
        };

        // Some events have special handlers that wrap the real handler
        var handlerWrapper = eventFns.specialHandlerWrapper || defaultHandlerWrapper;

        // Copy event handlers in case event handlers array is modified during execution.
        if (eventFnsLength > 1) {
          eventFns = shallowCopy(eventFns);
        }

        for (var i = 0; i < eventFnsLength; i++) {
          if (!event.isImmediatePropagationStopped()) {
            handlerWrapper(element, event, eventFns[i]);
          }
        }
      };

      // TODO: this is a hack for angularMocks/clearDataCache that makes it possible to deregister all
      //       events on `element`
      eventHandler.elem = element;
      return eventHandler;
    }

    function defaultHandlerWrapper(element, event, handler) {
      handler.call(element, event);
    }

    function specialMouseHandlerWrapper(target, event, handler) {
      // Refer to jQuery's implementation of mouseenter & mouseleave
      // Read about mouseenter and mouseleave:
      // http://www.quirksmode.org/js/events_mouse.html#link8
      var related = event.relatedTarget;
      // For mousenter/leave call the handler if related is outside the target.
      // NB: No relatedTarget if the mouse left/entered the browser window
      if (!related || related !== target && !jqLiteContains.call(target, related)) {
        handler.call(target, event);
      }
    }

    //////////////////////////////////////////
    // Functions iterating traversal.
    // These functions chain results into a single
    // selector.
    //////////////////////////////////////////
    forEach({
      removeData: jqLiteRemoveData,

      on: function jqLiteOn(element, type, fn, unsupported) {
        if (isDefined(unsupported)) throw jqLiteMinErr('onargs', 'jqLite#on() does not support the `selector` or `eventData` parameters');

        // Do not add event handlers to non-elements because they will not be cleaned up.
        if (!jqLiteAcceptsData(element)) {
          return;
        }

        var expandoStore = jqLiteExpandoStore(element, true);
        var events = expandoStore.events;
        var handle = expandoStore.handle;

        if (!handle) {
          handle = expandoStore.handle = createEventHandler(element, events);
        }

        // http://jsperf.com/string-indexof-vs-split
        var types = type.indexOf(' ') >= 0 ? type.split(' ') : [type];
        var i = types.length;

        var addHandler = function (type, specialHandlerWrapper, noEventListener) {
          var eventFns = events[type];

          if (!eventFns) {
            eventFns = events[type] = [];
            eventFns.specialHandlerWrapper = specialHandlerWrapper;
            if (type !== '$destroy' && !noEventListener) {
              element.addEventListener(type, handle);
            }
          }

          eventFns.push(fn);
        };

        while (i--) {
          type = types[i];
          if (MOUSE_EVENT_MAP[type]) {
            addHandler(MOUSE_EVENT_MAP[type], specialMouseHandlerWrapper);
            addHandler(type, undefined, true);
          } else {
            addHandler(type);
          }
        }
      },

      off: jqLiteOff,

      one: function (element, type, fn) {
        element = jqLite(element);

        //add the listener twice so that when it is called
        //you can remove the original function and still be
        //able to call element.off(ev, fn) normally
        element.on(type, function onFn() {
          element.off(type, fn);
          element.off(type, onFn);
        });
        element.on(type, fn);
      },

      replaceWith: function (element, replaceNode) {
        var index,
            parent = element.parentNode;
        jqLiteDealoc(element);
        forEach(new JQLite(replaceNode), function (node) {
          if (index) {
            parent.insertBefore(node, index.nextSibling);
          } else {
            parent.replaceChild(node, element);
          }
          index = node;
        });
      },

      children: function (element) {
        var children = [];
        forEach(element.childNodes, function (element) {
          if (element.nodeType === NODE_TYPE_ELEMENT) {
            children.push(element);
          }
        });
        return children;
      },

      contents: function (element) {
        return element.contentDocument || element.childNodes || [];
      },

      append: function (element, node) {
        var nodeType = element.nodeType;
        if (nodeType !== NODE_TYPE_ELEMENT && nodeType !== NODE_TYPE_DOCUMENT_FRAGMENT) return;

        node = new JQLite(node);

        for (var i = 0, ii = node.length; i < ii; i++) {
          var child = node[i];
          element.appendChild(child);
        }
      },

      prepend: function (element, node) {
        if (element.nodeType === NODE_TYPE_ELEMENT) {
          var index = element.firstChild;
          forEach(new JQLite(node), function (child) {
            element.insertBefore(child, index);
          });
        }
      },

      wrap: function (element, wrapNode) {
        jqLiteWrapNode(element, jqLite(wrapNode).eq(0).clone()[0]);
      },

      remove: jqLiteRemove,

      detach: function (element) {
        jqLiteRemove(element, true);
      },

      after: function (element, newElement) {
        var index = element,
            parent = element.parentNode;

        if (parent) {
          newElement = new JQLite(newElement);

          for (var i = 0, ii = newElement.length; i < ii; i++) {
            var node = newElement[i];
            parent.insertBefore(node, index.nextSibling);
            index = node;
          }
        }
      },

      addClass: jqLiteAddClass,
      removeClass: jqLiteRemoveClass,

      toggleClass: function (element, selector, condition) {
        if (selector) {
          forEach(selector.split(' '), function (className) {
            var classCondition = condition;
            if (isUndefined(classCondition)) {
              classCondition = !jqLiteHasClass(element, className);
            }
            (classCondition ? jqLiteAddClass : jqLiteRemoveClass)(element, className);
          });
        }
      },

      parent: function (element) {
        var parent = element.parentNode;
        return parent && parent.nodeType !== NODE_TYPE_DOCUMENT_FRAGMENT ? parent : null;
      },

      next: function (element) {
        return element.nextElementSibling;
      },

      find: function (element, selector) {
        if (element.getElementsByTagName) {
          return element.getElementsByTagName(selector);
        } else {
          return [];
        }
      },

      clone: jqLiteClone,

      triggerHandler: function (element, event, extraParameters) {

        var dummyEvent, eventFnsCopy, handlerArgs;
        var eventName = event.type || event;
        var expandoStore = jqLiteExpandoStore(element);
        var events = expandoStore && expandoStore.events;
        var eventFns = events && events[eventName];

        if (eventFns) {
          // Create a dummy event to pass to the handlers
          dummyEvent = {
            preventDefault: function () {
              this.defaultPrevented = true;
            },
            isDefaultPrevented: function () {
              return this.defaultPrevented === true;
            },
            stopImmediatePropagation: function () {
              this.immediatePropagationStopped = true;
            },
            isImmediatePropagationStopped: function () {
              return this.immediatePropagationStopped === true;
            },
            stopPropagation: noop,
            type: eventName,
            target: element
          };

          // If a custom event was provided then extend our dummy event with it
          if (event.type) {
            dummyEvent = extend(dummyEvent, event);
          }

          // Copy event handlers in case event handlers array is modified during execution.
          eventFnsCopy = shallowCopy(eventFns);
          handlerArgs = extraParameters ? [dummyEvent].concat(extraParameters) : [dummyEvent];

          forEach(eventFnsCopy, function (fn) {
            if (!dummyEvent.isImmediatePropagationStopped()) {
              fn.apply(element, handlerArgs);
            }
          });
        }
      }
    }, function (fn, name) {
      /**
       * chaining functions
       */
      JQLite.prototype[name] = function (arg1, arg2, arg3) {
        var value;

        for (var i = 0, ii = this.length; i < ii; i++) {
          if (isUndefined(value)) {
            value = fn(this[i], arg1, arg2, arg3);
            if (isDefined(value)) {
              // any function which returns a value needs to be wrapped
              value = jqLite(value);
            }
          } else {
            jqLiteAddNodes(value, fn(this[i], arg1, arg2, arg3));
          }
        }
        return isDefined(value) ? value : this;
      };
    });

    // bind legacy bind/unbind to on/off
    JQLite.prototype.bind = JQLite.prototype.on;
    JQLite.prototype.unbind = JQLite.prototype.off;

    // Provider for private $$jqLite service
    /** @this */
    function $$jqLiteProvider() {
      this.$get = function $$jqLite() {
        return extend(JQLite, {
          hasClass: function (node, classes) {
            if (node.attr) node = node[0];
            return jqLiteHasClass(node, classes);
          },
          addClass: function (node, classes) {
            if (node.attr) node = node[0];
            return jqLiteAddClass(node, classes);
          },
          removeClass: function (node, classes) {
            if (node.attr) node = node[0];
            return jqLiteRemoveClass(node, classes);
          }
        });
      };
    }

    /**
     * Computes a hash of an 'obj'.
     * Hash of a:
     *  string is string
     *  number is number as string
     *  object is either result of calling $$hashKey function on the object or uniquely generated id,
     *         that is also assigned to the $$hashKey property of the object.
     *
     * @param obj
     * @returns {string} hash string such that the same input will have the same hash string.
     *         The resulting string key is in 'type:hashKey' format.
     */
    function hashKey(obj, nextUidFn) {
      var key = obj && obj.$$hashKey;

      if (key) {
        if (typeof key === 'function') {
          key = obj.$$hashKey();
        }
        return key;
      }

      var objType = typeof obj;
      if (objType === 'function' || objType === 'object' && obj !== null) {
        key = obj.$$hashKey = objType + ':' + (nextUidFn || nextUid)();
      } else {
        key = objType + ':' + obj;
      }

      return key;
    }

    // A minimal ES2015 Map implementation.
    // Should be bug/feature equivalent to the native implementations of supported browsers
    // (for the features required in Angular).
    // See https://kangax.github.io/compat-table/es6/#test-Map
    var nanKey = Object.create(null);
    function NgMapShim() {
      this._keys = [];
      this._values = [];
      this._lastKey = NaN;
      this._lastIndex = -1;
    }
    NgMapShim.prototype = {
      _idx: function (key) {
        if (key === this._lastKey) {
          return this._lastIndex;
        }
        this._lastKey = key;
        this._lastIndex = this._keys.indexOf(key);
        return this._lastIndex;
      },
      _transformKey: function (key) {
        return isNumberNaN(key) ? nanKey : key;
      },
      get: function (key) {
        key = this._transformKey(key);
        var idx = this._idx(key);
        if (idx !== -1) {
          return this._values[idx];
        }
      },
      set: function (key, value) {
        key = this._transformKey(key);
        var idx = this._idx(key);
        if (idx === -1) {
          idx = this._lastIndex = this._keys.length;
        }
        this._keys[idx] = key;
        this._values[idx] = value;

        // Support: IE11
        // Do not `return this` to simulate the partial IE11 implementation
      },
      delete: function (key) {
        key = this._transformKey(key);
        var idx = this._idx(key);
        if (idx === -1) {
          return false;
        }
        this._keys.splice(idx, 1);
        this._values.splice(idx, 1);
        this._lastKey = NaN;
        this._lastIndex = -1;
        return true;
      }
    };

    // For now, always use `NgMapShim`, even if `window.Map` is available. Some native implementations
    // are still buggy (often in subtle ways) and can cause hard-to-debug failures. When native `Map`
    // implementations get more stable, we can reconsider switching to `window.Map` (when available).
    var NgMap = NgMapShim;

    var $$MapProvider = [/** @this */function () {
      this.$get = [function () {
        return NgMap;
      }];
    }];

    /**
     * @ngdoc function
     * @module ng
     * @name angular.injector
     * @kind function
     *
     * @description
     * Creates an injector object that can be used for retrieving services as well as for
     * dependency injection (see {@link guide/di dependency injection}).
     *
     * @param {Array.<string|Function>} modules A list of module functions or their aliases. See
     *     {@link angular.module}. The `ng` module must be explicitly added.
     * @param {boolean=} [strictDi=false] Whether the injector should be in strict mode, which
     *     disallows argument name annotation inference.
     * @returns {injector} Injector object. See {@link auto.$injector $injector}.
     *
     * @example
     * Typical usage
     * ```js
     *   // create an injector
     *   var $injector = angular.injector(['ng']);
     *
     *   // use the injector to kick off your application
     *   // use the type inference to auto inject arguments, or use implicit injection
     *   $injector.invoke(function($rootScope, $compile, $document) {
     *     $compile($document)($rootScope);
     *     $rootScope.$digest();
     *   });
     * ```
     *
     * Sometimes you want to get access to the injector of a currently running Angular app
     * from outside Angular. Perhaps, you want to inject and compile some markup after the
     * application has been bootstrapped. You can do this using the extra `injector()` added
     * to JQuery/jqLite elements. See {@link angular.element}.
     *
     * *This is fairly rare but could be the case if a third party library is injecting the
     * markup.*
     *
     * In the following example a new block of HTML containing a `ng-controller`
     * directive is added to the end of the document body by JQuery. We then compile and link
     * it into the current AngularJS scope.
     *
     * ```js
     * var $div = $('<div ng-controller="MyCtrl">{{content.label}}</div>');
     * $(document.body).append($div);
     *
     * angular.element(document).injector().invoke(function($compile) {
     *   var scope = angular.element($div).scope();
     *   $compile($div)(scope);
     * });
     * ```
     */

    /**
     * @ngdoc module
     * @name auto
     * @installation
     * @description
     *
     * Implicit module which gets automatically added to each {@link auto.$injector $injector}.
     */

    var ARROW_ARG = /^([^(]+?)=>/;
    var FN_ARGS = /^[^(]*\(\s*([^)]*)\)/m;
    var FN_ARG_SPLIT = /,/;
    var FN_ARG = /^\s*(_?)(\S+?)\1\s*$/;
    var STRIP_COMMENTS = /((\/\/.*$)|(\/\*[\s\S]*?\*\/))/mg;
    var $injectorMinErr = minErr('$injector');

    function stringifyFn(fn) {
      return Function.prototype.toString.call(fn);
    }

    function extractArgs(fn) {
      var fnText = stringifyFn(fn).replace(STRIP_COMMENTS, ''),
          args = fnText.match(ARROW_ARG) || fnText.match(FN_ARGS);
      return args;
    }

    function anonFn(fn) {
      // For anonymous functions, showing at the very least the function signature can help in
      // debugging.
      var args = extractArgs(fn);
      if (args) {
        return 'function(' + (args[1] || '').replace(/[\s\r\n]+/, ' ') + ')';
      }
      return 'fn';
    }

    function annotate(fn, strictDi, name) {
      var $inject, argDecl, last;

      if (typeof fn === 'function') {
        if (!($inject = fn.$inject)) {
          $inject = [];
          if (fn.length) {
            if (strictDi) {
              if (!isString(name) || !name) {
                name = fn.name || anonFn(fn);
              }
              throw $injectorMinErr('strictdi', '{0} is not using explicit annotation and cannot be invoked in strict mode', name);
            }
            argDecl = extractArgs(fn);
            forEach(argDecl[1].split(FN_ARG_SPLIT), function (arg) {
              arg.replace(FN_ARG, function (all, underscore, name) {
                $inject.push(name);
              });
            });
          }
          fn.$inject = $inject;
        }
      } else if (isArray(fn)) {
        last = fn.length - 1;
        assertArgFn(fn[last], 'fn');
        $inject = fn.slice(0, last);
      } else {
        assertArgFn(fn, 'fn', true);
      }
      return $inject;
    }

    ///////////////////////////////////////

    /**
     * @ngdoc service
     * @name $injector
     *
     * @description
     *
     * `$injector` is used to retrieve object instances as defined by
     * {@link auto.$provide provider}, instantiate types, invoke methods,
     * and load modules.
     *
     * The following always holds true:
     *
     * ```js
     *   var $injector = angular.injector();
     *   expect($injector.get('$injector')).toBe($injector);
     *   expect($injector.invoke(function($injector) {
     *     return $injector;
     *   })).toBe($injector);
     * ```
     *
     * # Injection Function Annotation
     *
     * JavaScript does not have annotations, and annotations are needed for dependency injection. The
     * following are all valid ways of annotating function with injection arguments and are equivalent.
     *
     * ```js
     *   // inferred (only works if code not minified/obfuscated)
     *   $injector.invoke(function(serviceA){});
     *
     *   // annotated
     *   function explicit(serviceA) {};
     *   explicit.$inject = ['serviceA'];
     *   $injector.invoke(explicit);
     *
     *   // inline
     *   $injector.invoke(['serviceA', function(serviceA){}]);
     * ```
     *
     * ## Inference
     *
     * In JavaScript calling `toString()` on a function returns the function definition. The definition
     * can then be parsed and the function arguments can be extracted. This method of discovering
     * annotations is disallowed when the injector is in strict mode.
     * *NOTE:* This does not work with minification, and obfuscation tools since these tools change the
     * argument names.
     *
     * ## `$inject` Annotation
     * By adding an `$inject` property onto a function the injection parameters can be specified.
     *
     * ## Inline
     * As an array of injection names, where the last item in the array is the function to call.
     */

    /**
     * @ngdoc property
     * @name $injector#modules
     * @type {Object}
     * @description
     * A hash containing all the modules that have been loaded into the
     * $injector.
     *
     * You can use this property to find out information about a module via the
     * {@link angular.Module#info `myModule.info(...)`} method.
     *
     * For example:
     *
     * ```
     * var info = $injector.modules['ngAnimate'].info();
     * ```
     *
     * **Do not use this property to attempt to modify the modules after the application
     * has been bootstrapped.**
     */

    /**
     * @ngdoc method
     * @name $injector#get
     *
     * @description
     * Return an instance of the service.
     *
     * @param {string} name The name of the instance to retrieve.
     * @param {string=} caller An optional string to provide the origin of the function call for error messages.
     * @return {*} The instance.
     */

    /**
     * @ngdoc method
     * @name $injector#invoke
     *
     * @description
     * Invoke the method and supply the method arguments from the `$injector`.
     *
     * @param {Function|Array.<string|Function>} fn The injectable function to invoke. Function parameters are
     *   injected according to the {@link guide/di $inject Annotation} rules.
     * @param {Object=} self The `this` for the invoked method.
     * @param {Object=} locals Optional object. If preset then any argument names are read from this
     *                         object first, before the `$injector` is consulted.
     * @returns {*} the value returned by the invoked `fn` function.
     */

    /**
     * @ngdoc method
     * @name $injector#has
     *
     * @description
     * Allows the user to query if the particular service exists.
     *
     * @param {string} name Name of the service to query.
     * @returns {boolean} `true` if injector has given service.
     */

    /**
     * @ngdoc method
     * @name $injector#instantiate
     * @description
     * Create a new instance of JS type. The method takes a constructor function, invokes the new
     * operator, and supplies all of the arguments to the constructor function as specified by the
     * constructor annotation.
     *
     * @param {Function} Type Annotated constructor function.
     * @param {Object=} locals Optional object. If preset then any argument names are read from this
     * object first, before the `$injector` is consulted.
     * @returns {Object} new instance of `Type`.
     */

    /**
     * @ngdoc method
     * @name $injector#annotate
     *
     * @description
     * Returns an array of service names which the function is requesting for injection. This API is
     * used by the injector to determine which services need to be injected into the function when the
     * function is invoked. There are three ways in which the function can be annotated with the needed
     * dependencies.
     *
     * # Argument names
     *
     * The simplest form is to extract the dependencies from the arguments of the function. This is done
     * by converting the function into a string using `toString()` method and extracting the argument
     * names.
     * ```js
     *   // Given
     *   function MyController($scope, $route) {
     *     // ...
     *   }
     *
     *   // Then
     *   expect(injector.annotate(MyController)).toEqual(['$scope', '$route']);
     * ```
     *
     * You can disallow this method by using strict injection mode.
     *
     * This method does not work with code minification / obfuscation. For this reason the following
     * annotation strategies are supported.
     *
     * # The `$inject` property
     *
     * If a function has an `$inject` property and its value is an array of strings, then the strings
     * represent names of services to be injected into the function.
     * ```js
     *   // Given
     *   var MyController = function(obfuscatedScope, obfuscatedRoute) {
     *     // ...
     *   }
     *   // Define function dependencies
     *   MyController['$inject'] = ['$scope', '$route'];
     *
     *   // Then
     *   expect(injector.annotate(MyController)).toEqual(['$scope', '$route']);
     * ```
     *
     * # The array notation
     *
     * It is often desirable to inline Injected functions and that's when setting the `$inject` property
     * is very inconvenient. In these situations using the array notation to specify the dependencies in
     * a way that survives minification is a better choice:
     *
     * ```js
     *   // We wish to write this (not minification / obfuscation safe)
     *   injector.invoke(function($compile, $rootScope) {
     *     // ...
     *   });
     *
     *   // We are forced to write break inlining
     *   var tmpFn = function(obfuscatedCompile, obfuscatedRootScope) {
     *     // ...
     *   };
     *   tmpFn.$inject = ['$compile', '$rootScope'];
     *   injector.invoke(tmpFn);
     *
     *   // To better support inline function the inline annotation is supported
     *   injector.invoke(['$compile', '$rootScope', function(obfCompile, obfRootScope) {
     *     // ...
     *   }]);
     *
     *   // Therefore
     *   expect(injector.annotate(
     *      ['$compile', '$rootScope', function(obfus_$compile, obfus_$rootScope) {}])
     *    ).toEqual(['$compile', '$rootScope']);
     * ```
     *
     * @param {Function|Array.<string|Function>} fn Function for which dependent service names need to
     * be retrieved as described above.
     *
     * @param {boolean=} [strictDi=false] Disallow argument name annotation inference.
     *
     * @returns {Array.<string>} The names of the services which the function requires.
     */

    /**
     * @ngdoc service
     * @name $provide
     *
     * @description
     *
     * The {@link auto.$provide $provide} service has a number of methods for registering components
     * with the {@link auto.$injector $injector}. Many of these functions are also exposed on
     * {@link angular.Module}.
     *
     * An Angular **service** is a singleton object created by a **service factory**.  These **service
     * factories** are functions which, in turn, are created by a **service provider**.
     * The **service providers** are constructor functions. When instantiated they must contain a
     * property called `$get`, which holds the **service factory** function.
     *
     * When you request a service, the {@link auto.$injector $injector} is responsible for finding the
     * correct **service provider**, instantiating it and then calling its `$get` **service factory**
     * function to get the instance of the **service**.
     *
     * Often services have no configuration options and there is no need to add methods to the service
     * provider.  The provider will be no more than a constructor function with a `$get` property. For
     * these cases the {@link auto.$provide $provide} service has additional helper methods to register
     * services without specifying a provider.
     *
     * * {@link auto.$provide#provider provider(name, provider)} - registers a **service provider** with the
     *     {@link auto.$injector $injector}
     * * {@link auto.$provide#constant constant(name, obj)} - registers a value/object that can be accessed by
     *     providers and services.
     * * {@link auto.$provide#value value(name, obj)} - registers a value/object that can only be accessed by
     *     services, not providers.
     * * {@link auto.$provide#factory factory(name, fn)} - registers a service **factory function**
     *     that will be wrapped in a **service provider** object, whose `$get` property will contain the
     *     given factory function.
     * * {@link auto.$provide#service service(name, Fn)} - registers a **constructor function**
     *     that will be wrapped in a **service provider** object, whose `$get` property will instantiate
     *      a new object using the given constructor function.
     * * {@link auto.$provide#decorator decorator(name, decorFn)} - registers a **decorator function** that
     *      will be able to modify or replace the implementation of another service.
     *
     * See the individual methods for more information and examples.
     */

    /**
     * @ngdoc method
     * @name $provide#provider
     * @description
     *
     * Register a **provider function** with the {@link auto.$injector $injector}. Provider functions
     * are constructor functions, whose instances are responsible for "providing" a factory for a
     * service.
     *
     * Service provider names start with the name of the service they provide followed by `Provider`.
     * For example, the {@link ng.$log $log} service has a provider called
     * {@link ng.$logProvider $logProvider}.
     *
     * Service provider objects can have additional methods which allow configuration of the provider
     * and its service. Importantly, you can configure what kind of service is created by the `$get`
     * method, or how that service will act. For example, the {@link ng.$logProvider $logProvider} has a
     * method {@link ng.$logProvider#debugEnabled debugEnabled}
     * which lets you specify whether the {@link ng.$log $log} service will log debug messages to the
     * console or not.
     *
     * @param {string} name The name of the instance. NOTE: the provider will be available under `name +
                            'Provider'` key.
     * @param {(Object|function())} provider If the provider is:
     *
     *   - `Object`: then it should have a `$get` method. The `$get` method will be invoked using
     *     {@link auto.$injector#invoke $injector.invoke()} when an instance needs to be created.
     *   - `Constructor`: a new instance of the provider will be created using
     *     {@link auto.$injector#instantiate $injector.instantiate()}, then treated as `object`.
     *
     * @returns {Object} registered provider instance
    
     * @example
     *
     * The following example shows how to create a simple event tracking service and register it using
     * {@link auto.$provide#provider $provide.provider()}.
     *
     * ```js
     *  // Define the eventTracker provider
     *  function EventTrackerProvider() {
     *    var trackingUrl = '/track';
     *
     *    // A provider method for configuring where the tracked events should been saved
     *    this.setTrackingUrl = function(url) {
     *      trackingUrl = url;
     *    };
     *
     *    // The service factory function
     *    this.$get = ['$http', function($http) {
     *      var trackedEvents = {};
     *      return {
     *        // Call this to track an event
     *        event: function(event) {
     *          var count = trackedEvents[event] || 0;
     *          count += 1;
     *          trackedEvents[event] = count;
     *          return count;
     *        },
     *        // Call this to save the tracked events to the trackingUrl
     *        save: function() {
     *          $http.post(trackingUrl, trackedEvents);
     *        }
     *      };
     *    }];
     *  }
     *
     *  describe('eventTracker', function() {
     *    var postSpy;
     *
     *    beforeEach(module(function($provide) {
     *      // Register the eventTracker provider
     *      $provide.provider('eventTracker', EventTrackerProvider);
     *    }));
     *
     *    beforeEach(module(function(eventTrackerProvider) {
     *      // Configure eventTracker provider
     *      eventTrackerProvider.setTrackingUrl('/custom-track');
     *    }));
     *
     *    it('tracks events', inject(function(eventTracker) {
     *      expect(eventTracker.event('login')).toEqual(1);
     *      expect(eventTracker.event('login')).toEqual(2);
     *    }));
     *
     *    it('saves to the tracking url', inject(function(eventTracker, $http) {
     *      postSpy = spyOn($http, 'post');
     *      eventTracker.event('login');
     *      eventTracker.save();
     *      expect(postSpy).toHaveBeenCalled();
     *      expect(postSpy.mostRecentCall.args[0]).not.toEqual('/track');
     *      expect(postSpy.mostRecentCall.args[0]).toEqual('/custom-track');
     *      expect(postSpy.mostRecentCall.args[1]).toEqual({ 'login': 1 });
     *    }));
     *  });
     * ```
     */

    /**
     * @ngdoc method
     * @name $provide#factory
     * @description
     *
     * Register a **service factory**, which will be called to return the service instance.
     * This is short for registering a service where its provider consists of only a `$get` property,
     * which is the given service factory function.
     * You should use {@link auto.$provide#factory $provide.factory(getFn)} if you do not need to
     * configure your service in a provider.
     *
     * @param {string} name The name of the instance.
     * @param {Function|Array.<string|Function>} $getFn The injectable $getFn for the instance creation.
     *                      Internally this is a short hand for `$provide.provider(name, {$get: $getFn})`.
     * @returns {Object} registered provider instance
     *
     * @example
     * Here is an example of registering a service
     * ```js
     *   $provide.factory('ping', ['$http', function($http) {
     *     return function ping() {
     *       return $http.send('/ping');
     *     };
     *   }]);
     * ```
     * You would then inject and use this service like this:
     * ```js
     *   someModule.controller('Ctrl', ['ping', function(ping) {
     *     ping();
     *   }]);
     * ```
     */

    /**
     * @ngdoc method
     * @name $provide#service
     * @description
     *
     * Register a **service constructor**, which will be invoked with `new` to create the service
     * instance.
     * This is short for registering a service where its provider's `$get` property is a factory
     * function that returns an instance instantiated by the injector from the service constructor
     * function.
     *
     * Internally it looks a bit like this:
     *
     * ```
     * {
     *   $get: function() {
     *     return $injector.instantiate(constructor);
     *   }
     * }
     * ```
     *
     *
     * You should use {@link auto.$provide#service $provide.service(class)} if you define your service
     * as a type/class.
     *
     * @param {string} name The name of the instance.
     * @param {Function|Array.<string|Function>} constructor An injectable class (constructor function)
     *     that will be instantiated.
     * @returns {Object} registered provider instance
     *
     * @example
     * Here is an example of registering a service using
     * {@link auto.$provide#service $provide.service(class)}.
     * ```js
     *   var Ping = function($http) {
     *     this.$http = $http;
     *   };
     *
     *   Ping.$inject = ['$http'];
     *
     *   Ping.prototype.send = function() {
     *     return this.$http.get('/ping');
     *   };
     *   $provide.service('ping', Ping);
     * ```
     * You would then inject and use this service like this:
     * ```js
     *   someModule.controller('Ctrl', ['ping', function(ping) {
     *     ping.send();
     *   }]);
     * ```
     */

    /**
     * @ngdoc method
     * @name $provide#value
     * @description
     *
     * Register a **value service** with the {@link auto.$injector $injector}, such as a string, a
     * number, an array, an object or a function. This is short for registering a service where its
     * provider's `$get` property is a factory function that takes no arguments and returns the **value
     * service**. That also means it is not possible to inject other services into a value service.
     *
     * Value services are similar to constant services, except that they cannot be injected into a
     * module configuration function (see {@link angular.Module#config}) but they can be overridden by
     * an Angular {@link auto.$provide#decorator decorator}.
     *
     * @param {string} name The name of the instance.
     * @param {*} value The value.
     * @returns {Object} registered provider instance
     *
     * @example
     * Here are some examples of creating value services.
     * ```js
     *   $provide.value('ADMIN_USER', 'admin');
     *
     *   $provide.value('RoleLookup', { admin: 0, writer: 1, reader: 2 });
     *
     *   $provide.value('halfOf', function(value) {
     *     return value / 2;
     *   });
     * ```
     */

    /**
     * @ngdoc method
     * @name $provide#constant
     * @description
     *
     * Register a **constant service** with the {@link auto.$injector $injector}, such as a string,
     * a number, an array, an object or a function. Like the {@link auto.$provide#value value}, it is not
     * possible to inject other services into a constant.
     *
     * But unlike {@link auto.$provide#value value}, a constant can be
     * injected into a module configuration function (see {@link angular.Module#config}) and it cannot
     * be overridden by an Angular {@link auto.$provide#decorator decorator}.
     *
     * @param {string} name The name of the constant.
     * @param {*} value The constant value.
     * @returns {Object} registered instance
     *
     * @example
     * Here a some examples of creating constants:
     * ```js
     *   $provide.constant('SHARD_HEIGHT', 306);
     *
     *   $provide.constant('MY_COLOURS', ['red', 'blue', 'grey']);
     *
     *   $provide.constant('double', function(value) {
     *     return value * 2;
     *   });
     * ```
     */

    /**
     * @ngdoc method
     * @name $provide#decorator
     * @description
     *
     * Register a **decorator function** with the {@link auto.$injector $injector}. A decorator function
     * intercepts the creation of a service, allowing it to override or modify the behavior of the
     * service. The return value of the decorator function may be the original service, or a new service
     * that replaces (or wraps and delegates to) the original service.
     *
     * You can find out more about using decorators in the {@link guide/decorators} guide.
     *
     * @param {string} name The name of the service to decorate.
     * @param {Function|Array.<string|Function>} decorator This function will be invoked when the service needs to be
     *    provided and should return the decorated service instance. The function is called using
     *    the {@link auto.$injector#invoke injector.invoke} method and is therefore fully injectable.
     *    Local injection arguments:
     *
     *    * `$delegate` - The original service instance, which can be replaced, monkey patched, configured,
     *      decorated or delegated to.
     *
     * @example
     * Here we decorate the {@link ng.$log $log} service to convert warnings to errors by intercepting
     * calls to {@link ng.$log#error $log.warn()}.
     * ```js
     *   $provide.decorator('$log', ['$delegate', function($delegate) {
     *     $delegate.warn = $delegate.error;
     *     return $delegate;
     *   }]);
     * ```
     */

    function createInjector(modulesToLoad, strictDi) {
      strictDi = strictDi === true;
      var INSTANTIATING = {},
          providerSuffix = 'Provider',
          path = [],
          loadedModules = new NgMap(),
          providerCache = {
        $provide: {
          provider: supportObject(provider),
          factory: supportObject(factory),
          service: supportObject(service),
          value: supportObject(value),
          constant: supportObject(constant),
          decorator: decorator
        }
      },
          providerInjector = providerCache.$injector = createInternalInjector(providerCache, function (serviceName, caller) {
        if (angular.isString(caller)) {
          path.push(caller);
        }
        throw $injectorMinErr('unpr', 'Unknown provider: {0}', path.join(' <- '));
      }),
          instanceCache = {},
          protoInstanceInjector = createInternalInjector(instanceCache, function (serviceName, caller) {
        var provider = providerInjector.get(serviceName + providerSuffix, caller);
        return instanceInjector.invoke(provider.$get, provider, undefined, serviceName);
      }),
          instanceInjector = protoInstanceInjector;

      providerCache['$injector' + providerSuffix] = { $get: valueFn(protoInstanceInjector) };
      instanceInjector.modules = providerInjector.modules = createMap();
      var runBlocks = loadModules(modulesToLoad);
      instanceInjector = protoInstanceInjector.get('$injector');
      instanceInjector.strictDi = strictDi;
      forEach(runBlocks, function (fn) {
        if (fn) instanceInjector.invoke(fn);
      });

      return instanceInjector;

      ////////////////////////////////////
      // $provider
      ////////////////////////////////////

      function supportObject(delegate) {
        return function (key, value) {
          if (isObject(key)) {
            forEach(key, reverseParams(delegate));
          } else {
            return delegate(key, value);
          }
        };
      }

      function provider(name, provider_) {
        assertNotHasOwnProperty(name, 'service');
        if (isFunction(provider_) || isArray(provider_)) {
          provider_ = providerInjector.instantiate(provider_);
        }
        if (!provider_.$get) {
          throw $injectorMinErr('pget', 'Provider \'{0}\' must define $get factory method.', name);
        }
        return providerCache[name + providerSuffix] = provider_;
      }

      function enforceReturnValue(name, factory) {
        return (/** @this */function enforcedReturnValue() {
            var result = instanceInjector.invoke(factory, this);
            if (isUndefined(result)) {
              throw $injectorMinErr('undef', 'Provider \'{0}\' must return a value from $get factory method.', name);
            }
            return result;
          }
        );
      }

      function factory(name, factoryFn, enforce) {
        return provider(name, {
          $get: enforce !== false ? enforceReturnValue(name, factoryFn) : factoryFn
        });
      }

      function service(name, constructor) {
        return factory(name, ['$injector', function ($injector) {
          return $injector.instantiate(constructor);
        }]);
      }

      function value(name, val) {
        return factory(name, valueFn(val), false);
      }

      function constant(name, value) {
        assertNotHasOwnProperty(name, 'constant');
        providerCache[name] = value;
        instanceCache[name] = value;
      }

      function decorator(serviceName, decorFn) {
        var origProvider = providerInjector.get(serviceName + providerSuffix),
            orig$get = origProvider.$get;

        origProvider.$get = function () {
          var origInstance = instanceInjector.invoke(orig$get, origProvider);
          return instanceInjector.invoke(decorFn, null, { $delegate: origInstance });
        };
      }

      ////////////////////////////////////
      // Module Loading
      ////////////////////////////////////
      function loadModules(modulesToLoad) {
        assertArg(isUndefined(modulesToLoad) || isArray(modulesToLoad), 'modulesToLoad', 'not an array');
        var runBlocks = [],
            moduleFn;
        forEach(modulesToLoad, function (module) {
          if (loadedModules.get(module)) return;
          loadedModules.set(module, true);

          function runInvokeQueue(queue) {
            var i, ii;
            for (i = 0, ii = queue.length; i < ii; i++) {
              var invokeArgs = queue[i],
                  provider = providerInjector.get(invokeArgs[0]);

              provider[invokeArgs[1]].apply(provider, invokeArgs[2]);
            }
          }

          try {
            if (isString(module)) {
              moduleFn = angularModule(module);
              instanceInjector.modules[module] = moduleFn;
              runBlocks = runBlocks.concat(loadModules(moduleFn.requires)).concat(moduleFn._runBlocks);
              runInvokeQueue(moduleFn._invokeQueue);
              runInvokeQueue(moduleFn._configBlocks);
            } else if (isFunction(module)) {
              runBlocks.push(providerInjector.invoke(module));
            } else if (isArray(module)) {
              runBlocks.push(providerInjector.invoke(module));
            } else {
              assertArgFn(module, 'module');
            }
          } catch (e) {
            if (isArray(module)) {
              module = module[module.length - 1];
            }
            if (e.message && e.stack && e.stack.indexOf(e.message) === -1) {
              // Safari & FF's stack traces don't contain error.message content
              // unlike those of Chrome and IE
              // So if stack doesn't contain message, we create a new string that contains both.
              // Since error.stack is read-only in Safari, I'm overriding e and not e.stack here.
              // eslint-disable-next-line no-ex-assign
              e = e.message + '\n' + e.stack;
            }
            throw $injectorMinErr('modulerr', 'Failed to instantiate module {0} due to:\n{1}', module, e.stack || e.message || e);
          }
        });
        return runBlocks;
      }

      ////////////////////////////////////
      // internal Injector
      ////////////////////////////////////

      function createInternalInjector(cache, factory) {

        function getService(serviceName, caller) {
          if (cache.hasOwnProperty(serviceName)) {
            if (cache[serviceName] === INSTANTIATING) {
              throw $injectorMinErr('cdep', 'Circular dependency found: {0}', serviceName + ' <- ' + path.join(' <- '));
            }
            return cache[serviceName];
          } else {
            try {
              path.unshift(serviceName);
              cache[serviceName] = INSTANTIATING;
              cache[serviceName] = factory(serviceName, caller);
              return cache[serviceName];
            } catch (err) {
              if (cache[serviceName] === INSTANTIATING) {
                delete cache[serviceName];
              }
              throw err;
            } finally {
              path.shift();
            }
          }
        }

        function injectionArgs(fn, locals, serviceName) {
          var args = [],
              $inject = createInjector.$$annotate(fn, strictDi, serviceName);

          for (var i = 0, length = $inject.length; i < length; i++) {
            var key = $inject[i];
            if (typeof key !== 'string') {
              throw $injectorMinErr('itkn', 'Incorrect injection token! Expected service name as string, got {0}', key);
            }
            args.push(locals && locals.hasOwnProperty(key) ? locals[key] : getService(key, serviceName));
          }
          return args;
        }

        function isClass(func) {
          // Support: IE 9-11 only
          // IE 9-11 do not support classes and IE9 leaks with the code below.
          if (msie || typeof func !== 'function') {
            return false;
          }
          var result = func.$$ngIsClass;
          if (!isBoolean(result)) {
            // Support: Edge 12-13 only
            // See: https://developer.microsoft.com/en-us/microsoft-edge/platform/issues/6156135/
            result = func.$$ngIsClass = /^(?:class\b|constructor\()/.test(stringifyFn(func));
          }
          return result;
        }

        function invoke(fn, self, locals, serviceName) {
          if (typeof locals === 'string') {
            serviceName = locals;
            locals = null;
          }

          var args = injectionArgs(fn, locals, serviceName);
          if (isArray(fn)) {
            fn = fn[fn.length - 1];
          }

          if (!isClass(fn)) {
            // http://jsperf.com/angularjs-invoke-apply-vs-switch
            // #5388
            return fn.apply(self, args);
          } else {
            args.unshift(null);
            return new (Function.prototype.bind.apply(fn, args))();
          }
        }

        function instantiate(Type, locals, serviceName) {
          // Check if Type is annotated and use just the given function at n-1 as parameter
          // e.g. someModule.factory('greeter', ['$window', function(renamed$window) {}]);
          var ctor = isArray(Type) ? Type[Type.length - 1] : Type;
          var args = injectionArgs(Type, locals, serviceName);
          // Empty object at position 0 is ignored for invocation with `new`, but required.
          args.unshift(null);
          return new (Function.prototype.bind.apply(ctor, args))();
        }

        return {
          invoke: invoke,
          instantiate: instantiate,
          get: getService,
          annotate: createInjector.$$annotate,
          has: function (name) {
            return providerCache.hasOwnProperty(name + providerSuffix) || cache.hasOwnProperty(name);
          }
        };
      }
    }

    createInjector.$$annotate = annotate;

    /**
     * @ngdoc provider
     * @name $anchorScrollProvider
     * @this
     *
     * @description
     * Use `$anchorScrollProvider` to disable automatic scrolling whenever
     * {@link ng.$location#hash $location.hash()} changes.
     */
    function $AnchorScrollProvider() {

      var autoScrollingEnabled = true;

      /**
       * @ngdoc method
       * @name $anchorScrollProvider#disableAutoScrolling
       *
       * @description
       * By default, {@link ng.$anchorScroll $anchorScroll()} will automatically detect changes to
       * {@link ng.$location#hash $location.hash()} and scroll to the element matching the new hash.<br />
       * Use this method to disable automatic scrolling.
       *
       * If automatic scrolling is disabled, one must explicitly call
       * {@link ng.$anchorScroll $anchorScroll()} in order to scroll to the element related to the
       * current hash.
       */
      this.disableAutoScrolling = function () {
        autoScrollingEnabled = false;
      };

      /**
       * @ngdoc service
       * @name $anchorScroll
       * @kind function
       * @requires $window
       * @requires $location
       * @requires $rootScope
       *
       * @description
       * When called, it scrolls to the element related to the specified `hash` or (if omitted) to the
       * current value of {@link ng.$location#hash $location.hash()}, according to the rules specified
       * in the
       * [HTML5 spec](http://www.w3.org/html/wg/drafts/html/master/browsers.html#an-indicated-part-of-the-document).
       *
       * It also watches the {@link ng.$location#hash $location.hash()} and automatically scrolls to
       * match any anchor whenever it changes. This can be disabled by calling
       * {@link ng.$anchorScrollProvider#disableAutoScrolling $anchorScrollProvider.disableAutoScrolling()}.
       *
       * Additionally, you can use its {@link ng.$anchorScroll#yOffset yOffset} property to specify a
       * vertical scroll-offset (either fixed or dynamic).
       *
       * @param {string=} hash The hash specifying the element to scroll to. If omitted, the value of
       *                       {@link ng.$location#hash $location.hash()} will be used.
       *
       * @property {(number|function|jqLite)} yOffset
       * If set, specifies a vertical scroll-offset. This is often useful when there are fixed
       * positioned elements at the top of the page, such as navbars, headers etc.
       *
       * `yOffset` can be specified in various ways:
       * - **number**: A fixed number of pixels to be used as offset.<br /><br />
       * - **function**: A getter function called everytime `$anchorScroll()` is executed. Must return
       *   a number representing the offset (in pixels).<br /><br />
       * - **jqLite**: A jqLite/jQuery element to be used for specifying the offset. The distance from
       *   the top of the page to the element's bottom will be used as offset.<br />
       *   **Note**: The element will be taken into account only as long as its `position` is set to
       *   `fixed`. This option is useful, when dealing with responsive navbars/headers that adjust
       *   their height and/or positioning according to the viewport's size.
       *
       * <br />
       * <div class="alert alert-warning">
       * In order for `yOffset` to work properly, scrolling should take place on the document's root and
       * not some child element.
       * </div>
       *
       * @example
         <example module="anchorScrollExample" name="anchor-scroll">
           <file name="index.html">
             <div id="scrollArea" ng-controller="ScrollController">
               <a ng-click="gotoBottom()">Go to bottom</a>
               <a id="bottom"></a> You're at the bottom!
             </div>
           </file>
           <file name="script.js">
             angular.module('anchorScrollExample', [])
               .controller('ScrollController', ['$scope', '$location', '$anchorScroll',
                 function($scope, $location, $anchorScroll) {
                   $scope.gotoBottom = function() {
                     // set the location.hash to the id of
                     // the element you wish to scroll to.
                     $location.hash('bottom');
                      // call $anchorScroll()
                     $anchorScroll();
                   };
                 }]);
           </file>
           <file name="style.css">
             #scrollArea {
               height: 280px;
               overflow: auto;
             }
              #bottom {
               display: block;
               margin-top: 2000px;
             }
           </file>
         </example>
       *
       * <hr />
       * The example below illustrates the use of a vertical scroll-offset (specified as a fixed value).
       * See {@link ng.$anchorScroll#yOffset $anchorScroll.yOffset} for more details.
       *
       * @example
         <example module="anchorScrollOffsetExample" name="anchor-scroll-offset">
           <file name="index.html">
             <div class="fixed-header" ng-controller="headerCtrl">
               <a href="" ng-click="gotoAnchor(x)" ng-repeat="x in [1,2,3,4,5]">
                 Go to anchor {{x}}
               </a>
             </div>
             <div id="anchor{{x}}" class="anchor" ng-repeat="x in [1,2,3,4,5]">
               Anchor {{x}} of 5
             </div>
           </file>
           <file name="script.js">
             angular.module('anchorScrollOffsetExample', [])
               .run(['$anchorScroll', function($anchorScroll) {
                 $anchorScroll.yOffset = 50;   // always scroll by 50 extra pixels
               }])
               .controller('headerCtrl', ['$anchorScroll', '$location', '$scope',
                 function($anchorScroll, $location, $scope) {
                   $scope.gotoAnchor = function(x) {
                     var newHash = 'anchor' + x;
                     if ($location.hash() !== newHash) {
                       // set the $location.hash to `newHash` and
                       // $anchorScroll will automatically scroll to it
                       $location.hash('anchor' + x);
                     } else {
                       // call $anchorScroll() explicitly,
                       // since $location.hash hasn't changed
                       $anchorScroll();
                     }
                   };
                 }
               ]);
           </file>
           <file name="style.css">
             body {
               padding-top: 50px;
             }
              .anchor {
               border: 2px dashed DarkOrchid;
               padding: 10px 10px 200px 10px;
             }
              .fixed-header {
               background-color: rgba(0, 0, 0, 0.2);
               height: 50px;
               position: fixed;
               top: 0; left: 0; right: 0;
             }
              .fixed-header > a {
               display: inline-block;
               margin: 5px 15px;
             }
           </file>
         </example>
       */
      this.$get = ['$window', '$location', '$rootScope', function ($window, $location, $rootScope) {
        var document = $window.document;

        // Helper function to get first anchor from a NodeList
        // (using `Array#some()` instead of `angular#forEach()` since it's more performant
        //  and working in all supported browsers.)
        function getFirstAnchor(list) {
          var result = null;
          Array.prototype.some.call(list, function (element) {
            if (nodeName_(element) === 'a') {
              result = element;
              return true;
            }
          });
          return result;
        }

        function getYOffset() {

          var offset = scroll.yOffset;

          if (isFunction(offset)) {
            offset = offset();
          } else if (isElement(offset)) {
            var elem = offset[0];
            var style = $window.getComputedStyle(elem);
            if (style.position !== 'fixed') {
              offset = 0;
            } else {
              offset = elem.getBoundingClientRect().bottom;
            }
          } else if (!isNumber(offset)) {
            offset = 0;
          }

          return offset;
        }

        function scrollTo(elem) {
          if (elem) {
            elem.scrollIntoView();

            var offset = getYOffset();

            if (offset) {
              // `offset` is the number of pixels we should scroll UP in order to align `elem` properly.
              // This is true ONLY if the call to `elem.scrollIntoView()` initially aligns `elem` at the
              // top of the viewport.
              //
              // IF the number of pixels from the top of `elem` to the end of the page's content is less
              // than the height of the viewport, then `elem.scrollIntoView()` will align the `elem` some
              // way down the page.
              //
              // This is often the case for elements near the bottom of the page.
              //
              // In such cases we do not need to scroll the whole `offset` up, just the difference between
              // the top of the element and the offset, which is enough to align the top of `elem` at the
              // desired position.
              var elemTop = elem.getBoundingClientRect().top;
              $window.scrollBy(0, elemTop - offset);
            }
          } else {
            $window.scrollTo(0, 0);
          }
        }

        function scroll(hash) {
          // Allow numeric hashes
          hash = isString(hash) ? hash : isNumber(hash) ? hash.toString() : $location.hash();
          var elm;

          // empty hash, scroll to the top of the page
          if (!hash) scrollTo(null);

          // element with given id
          else if (elm = document.getElementById(hash)) scrollTo(elm);

            // first anchor with given name :-D
            else if (elm = getFirstAnchor(document.getElementsByName(hash))) scrollTo(elm);

              // no element and hash === 'top', scroll to the top of the page
              else if (hash === 'top') scrollTo(null);
        }

        // does not scroll when user clicks on anchor link that is currently on
        // (no url change, no $location.hash() change), browser native does scroll
        if (autoScrollingEnabled) {
          $rootScope.$watch(function autoScrollWatch() {
            return $location.hash();
          }, function autoScrollWatchAction(newVal, oldVal) {
            // skip the initial scroll if $location.hash is empty
            if (newVal === oldVal && newVal === '') return;

            jqLiteDocumentLoaded(function () {
              $rootScope.$evalAsync(scroll);
            });
          });
        }

        return scroll;
      }];
    }

    var $animateMinErr = minErr('$animate');
    var ELEMENT_NODE = 1;
    var NG_ANIMATE_CLASSNAME = 'ng-animate';

    function mergeClasses(a, b) {
      if (!a && !b) return '';
      if (!a) return b;
      if (!b) return a;
      if (isArray(a)) a = a.join(' ');
      if (isArray(b)) b = b.join(' ');
      return a + ' ' + b;
    }

    function extractElementNode(element) {
      for (var i = 0; i < element.length; i++) {
        var elm = element[i];
        if (elm.nodeType === ELEMENT_NODE) {
          return elm;
        }
      }
    }

    function splitClasses(classes) {
      if (isString(classes)) {
        classes = classes.split(' ');
      }

      // Use createMap() to prevent class assumptions involving property names in
      // Object.prototype
      var obj = createMap();
      forEach(classes, function (klass) {
        // sometimes the split leaves empty string values
        // incase extra spaces were applied to the options
        if (klass.length) {
          obj[klass] = true;
        }
      });
      return obj;
    }

    // if any other type of options value besides an Object value is
    // passed into the $animate.method() animation then this helper code
    // will be run which will ignore it. While this patch is not the
    // greatest solution to this, a lot of existing plugins depend on
    // $animate to either call the callback (< 1.2) or return a promise
    // that can be changed. This helper function ensures that the options
    // are wiped clean incase a callback function is provided.
    function prepareAnimateOptions(options) {
      return isObject(options) ? options : {};
    }

    var $$CoreAnimateJsProvider = /** @this */function () {
      this.$get = noop;
    };

    // this is prefixed with Core since it conflicts with
    // the animateQueueProvider defined in ngAnimate/animateQueue.js
    var $$CoreAnimateQueueProvider = /** @this */function () {
      var postDigestQueue = new NgMap();
      var postDigestElements = [];

      this.$get = ['$$AnimateRunner', '$rootScope', function ($$AnimateRunner, $rootScope) {
        return {
          enabled: noop,
          on: noop,
          off: noop,
          pin: noop,

          push: function (element, event, options, domOperation) {
            if (domOperation) {
              domOperation();
            }

            options = options || {};
            if (options.from) {
              element.css(options.from);
            }
            if (options.to) {
              element.css(options.to);
            }

            if (options.addClass || options.removeClass) {
              addRemoveClassesPostDigest(element, options.addClass, options.removeClass);
            }

            var runner = new $$AnimateRunner();

            // since there are no animations to run the runner needs to be
            // notified that the animation call is complete.
            runner.complete();
            return runner;
          }
        };

        function updateData(data, classes, value) {
          var changed = false;
          if (classes) {
            classes = isString(classes) ? classes.split(' ') : isArray(classes) ? classes : [];
            forEach(classes, function (className) {
              if (className) {
                changed = true;
                data[className] = value;
              }
            });
          }
          return changed;
        }

        function handleCSSClassChanges() {
          forEach(postDigestElements, function (element) {
            var data = postDigestQueue.get(element);
            if (data) {
              var existing = splitClasses(element.attr('class'));
              var toAdd = '';
              var toRemove = '';
              forEach(data, function (status, className) {
                var hasClass = !!existing[className];
                if (status !== hasClass) {
                  if (status) {
                    toAdd += (toAdd.length ? ' ' : '') + className;
                  } else {
                    toRemove += (toRemove.length ? ' ' : '') + className;
                  }
                }
              });

              forEach(element, function (elm) {
                if (toAdd) {
                  jqLiteAddClass(elm, toAdd);
                }
                if (toRemove) {
                  jqLiteRemoveClass(elm, toRemove);
                }
              });
              postDigestQueue.delete(element);
            }
          });
          postDigestElements.length = 0;
        }

        function addRemoveClassesPostDigest(element, add, remove) {
          var data = postDigestQueue.get(element) || {};

          var classesAdded = updateData(data, add, true);
          var classesRemoved = updateData(data, remove, false);

          if (classesAdded || classesRemoved) {

            postDigestQueue.set(element, data);
            postDigestElements.push(element);

            if (postDigestElements.length === 1) {
              $rootScope.$$postDigest(handleCSSClassChanges);
            }
          }
        }
      }];
    };

    /**
     * @ngdoc provider
     * @name $animateProvider
     *
     * @description
     * Default implementation of $animate that doesn't perform any animations, instead just
     * synchronously performs DOM updates and resolves the returned runner promise.
     *
     * In order to enable animations the `ngAnimate` module has to be loaded.
     *
     * To see the functional implementation check out `src/ngAnimate/animate.js`.
     */
    var $AnimateProvider = ['$provide', /** @this */function ($provide) {
      var provider = this;
      var classNameFilter = null;

      this.$$registeredAnimations = Object.create(null);

      /**
      * @ngdoc method
      * @name $animateProvider#register
      *
      * @description
      * Registers a new injectable animation factory function. The factory function produces the
      * animation object which contains callback functions for each event that is expected to be
      * animated.
      *
      *   * `eventFn`: `function(element, ... , doneFunction, options)`
      *   The element to animate, the `doneFunction` and the options fed into the animation. Depending
      *   on the type of animation additional arguments will be injected into the animation function. The
      *   list below explains the function signatures for the different animation methods:
      *
      *   - setClass: function(element, addedClasses, removedClasses, doneFunction, options)
      *   - addClass: function(element, addedClasses, doneFunction, options)
      *   - removeClass: function(element, removedClasses, doneFunction, options)
      *   - enter, leave, move: function(element, doneFunction, options)
      *   - animate: function(element, fromStyles, toStyles, doneFunction, options)
      *
      *   Make sure to trigger the `doneFunction` once the animation is fully complete.
      *
      * ```js
      *   return {
      *     //enter, leave, move signature
      *     eventFn : function(element, done, options) {
      *       //code to run the animation
      *       //once complete, then run done()
      *       return function endFunction(wasCancelled) {
      *         //code to cancel the animation
      *       }
      *     }
      *   }
      * ```
      *
      * @param {string} name The name of the animation (this is what the class-based CSS value will be compared to).
      * @param {Function} factory The factory function that will be executed to return the animation
      *                           object.
      */
      this.register = function (name, factory) {
        if (name && name.charAt(0) !== '.') {
          throw $animateMinErr('notcsel', 'Expecting class selector starting with \'.\' got \'{0}\'.', name);
        }

        var key = name + '-animation';
        provider.$$registeredAnimations[name.substr(1)] = key;
        $provide.factory(key, factory);
      };

      /**
       * @ngdoc method
       * @name $animateProvider#classNameFilter
       *
       * @description
       * Sets and/or returns the CSS class regular expression that is checked when performing
       * an animation. Upon bootstrap the classNameFilter value is not set at all and will
       * therefore enable $animate to attempt to perform an animation on any element that is triggered.
       * When setting the `classNameFilter` value, animations will only be performed on elements
       * that successfully match the filter expression. This in turn can boost performance
       * for low-powered devices as well as applications containing a lot of structural operations.
       * @param {RegExp=} expression The className expression which will be checked against all animations
       * @return {RegExp} The current CSS className expression value. If null then there is no expression value
       */
      this.classNameFilter = function (expression) {
        if (arguments.length === 1) {
          classNameFilter = expression instanceof RegExp ? expression : null;
          if (classNameFilter) {
            var reservedRegex = new RegExp('[(\\s|\\/)]' + NG_ANIMATE_CLASSNAME + '[(\\s|\\/)]');
            if (reservedRegex.test(classNameFilter.toString())) {
              classNameFilter = null;
              throw $animateMinErr('nongcls', '$animateProvider.classNameFilter(regex) prohibits accepting a regex value which matches/contains the "{0}" CSS class.', NG_ANIMATE_CLASSNAME);
            }
          }
        }
        return classNameFilter;
      };

      this.$get = ['$$animateQueue', function ($$animateQueue) {
        function domInsert(element, parentElement, afterElement) {
          // if for some reason the previous element was removed
          // from the dom sometime before this code runs then let's
          // just stick to using the parent element as the anchor
          if (afterElement) {
            var afterNode = extractElementNode(afterElement);
            if (afterNode && !afterNode.parentNode && !afterNode.previousElementSibling) {
              afterElement = null;
            }
          }
          if (afterElement) {
            afterElement.after(element);
          } else {
            parentElement.prepend(element);
          }
        }

        /**
         * @ngdoc service
         * @name $animate
         * @description The $animate service exposes a series of DOM utility methods that provide support
         * for animation hooks. The default behavior is the application of DOM operations, however,
         * when an animation is detected (and animations are enabled), $animate will do the heavy lifting
         * to ensure that animation runs with the triggered DOM operation.
         *
         * By default $animate doesn't trigger any animations. This is because the `ngAnimate` module isn't
         * included and only when it is active then the animation hooks that `$animate` triggers will be
         * functional. Once active then all structural `ng-` directives will trigger animations as they perform
         * their DOM-related operations (enter, leave and move). Other directives such as `ngClass`,
         * `ngShow`, `ngHide` and `ngMessages` also provide support for animations.
         *
         * It is recommended that the`$animate` service is always used when executing DOM-related procedures within directives.
         *
         * To learn more about enabling animation support, click here to visit the
         * {@link ngAnimate ngAnimate module page}.
         */
        return {
          // we don't call it directly since non-existant arguments may
          // be interpreted as null within the sub enabled function

          /**
           *
           * @ngdoc method
           * @name $animate#on
           * @kind function
           * @description Sets up an event listener to fire whenever the animation event (enter, leave, move, etc...)
           *    has fired on the given element or among any of its children. Once the listener is fired, the provided callback
           *    is fired with the following params:
           *
           * ```js
           * $animate.on('enter', container,
           *    function callback(element, phase) {
           *      // cool we detected an enter animation within the container
           *    }
           * );
           * ```
           *
           * @param {string} event the animation event that will be captured (e.g. enter, leave, move, addClass, removeClass, etc...)
           * @param {DOMElement} container the container element that will capture each of the animation events that are fired on itself
           *     as well as among its children
           * @param {Function} callback the callback function that will be fired when the listener is triggered
           *
           * The arguments present in the callback function are:
           * * `element` - The captured DOM element that the animation was fired on.
           * * `phase` - The phase of the animation. The two possible phases are **start** (when the animation starts) and **close** (when it ends).
           */
          on: $$animateQueue.on,

          /**
           *
           * @ngdoc method
           * @name $animate#off
           * @kind function
           * @description Deregisters an event listener based on the event which has been associated with the provided element. This method
           * can be used in three different ways depending on the arguments:
           *
           * ```js
           * // remove all the animation event listeners listening for `enter`
           * $animate.off('enter');
           *
           * // remove listeners for all animation events from the container element
           * $animate.off(container);
           *
           * // remove all the animation event listeners listening for `enter` on the given element and its children
           * $animate.off('enter', container);
           *
           * // remove the event listener function provided by `callback` that is set
           * // to listen for `enter` on the given `container` as well as its children
           * $animate.off('enter', container, callback);
           * ```
           *
           * @param {string|DOMElement} event|container the animation event (e.g. enter, leave, move,
           * addClass, removeClass, etc...), or the container element. If it is the element, all other
           * arguments are ignored.
           * @param {DOMElement=} container the container element the event listener was placed on
           * @param {Function=} callback the callback function that was registered as the listener
           */
          off: $$animateQueue.off,

          /**
           * @ngdoc method
           * @name $animate#pin
           * @kind function
           * @description Associates the provided element with a host parent element to allow the element to be animated even if it exists
           *    outside of the DOM structure of the Angular application. By doing so, any animation triggered via `$animate` can be issued on the
           *    element despite being outside the realm of the application or within another application. Say for example if the application
           *    was bootstrapped on an element that is somewhere inside of the `<body>` tag, but we wanted to allow for an element to be situated
           *    as a direct child of `document.body`, then this can be achieved by pinning the element via `$animate.pin(element)`. Keep in mind
           *    that calling `$animate.pin(element, parentElement)` will not actually insert into the DOM anywhere; it will just create the association.
           *
           *    Note that this feature is only active when the `ngAnimate` module is used.
           *
           * @param {DOMElement} element the external element that will be pinned
           * @param {DOMElement} parentElement the host parent element that will be associated with the external element
           */
          pin: $$animateQueue.pin,

          /**
           *
           * @ngdoc method
           * @name $animate#enabled
           * @kind function
           * @description Used to get and set whether animations are enabled or not on the entire application or on an element and its children. This
           * function can be called in four ways:
           *
           * ```js
           * // returns true or false
           * $animate.enabled();
           *
           * // changes the enabled state for all animations
           * $animate.enabled(false);
           * $animate.enabled(true);
           *
           * // returns true or false if animations are enabled for an element
           * $animate.enabled(element);
           *
           * // changes the enabled state for an element and its children
           * $animate.enabled(element, true);
           * $animate.enabled(element, false);
           * ```
           *
           * @param {DOMElement=} element the element that will be considered for checking/setting the enabled state
           * @param {boolean=} enabled whether or not the animations will be enabled for the element
           *
           * @return {boolean} whether or not animations are enabled
           */
          enabled: $$animateQueue.enabled,

          /**
           * @ngdoc method
           * @name $animate#cancel
           * @kind function
           * @description Cancels the provided animation.
           *
           * @param {Promise} animationPromise The animation promise that is returned when an animation is started.
           */
          cancel: function (runner) {
            if (runner.end) {
              runner.end();
            }
          },

          /**
           *
           * @ngdoc method
           * @name $animate#enter
           * @kind function
           * @description Inserts the element into the DOM either after the `after` element (if provided) or
           *   as the first child within the `parent` element and then triggers an animation.
           *   A promise is returned that will be resolved during the next digest once the animation
           *   has completed.
           *
           * @param {DOMElement} element the element which will be inserted into the DOM
           * @param {DOMElement} parent the parent element which will append the element as
           *   a child (so long as the after element is not present)
           * @param {DOMElement=} after the sibling element after which the element will be appended
           * @param {object=} options an optional collection of options/styles that will be applied to the element.
           *   The object can have the following properties:
           *
           *   - **addClass** - `{string}` - space-separated CSS classes to add to element
           *   - **from** - `{Object}` - CSS properties & values at the beginning of animation. Must have matching `to`
           *   - **removeClass** - `{string}` - space-separated CSS classes to remove from element
           *   - **to** - `{Object}` - CSS properties & values at end of animation. Must have matching `from`
           *
           * @return {Promise} the animation callback promise
           */
          enter: function (element, parent, after, options) {
            parent = parent && jqLite(parent);
            after = after && jqLite(after);
            parent = parent || after.parent();
            domInsert(element, parent, after);
            return $$animateQueue.push(element, 'enter', prepareAnimateOptions(options));
          },

          /**
           *
           * @ngdoc method
           * @name $animate#move
           * @kind function
           * @description Inserts (moves) the element into its new position in the DOM either after
           *   the `after` element (if provided) or as the first child within the `parent` element
           *   and then triggers an animation. A promise is returned that will be resolved
           *   during the next digest once the animation has completed.
           *
           * @param {DOMElement} element the element which will be moved into the new DOM position
           * @param {DOMElement} parent the parent element which will append the element as
           *   a child (so long as the after element is not present)
           * @param {DOMElement=} after the sibling element after which the element will be appended
           * @param {object=} options an optional collection of options/styles that will be applied to the element.
           *   The object can have the following properties:
           *
           *   - **addClass** - `{string}` - space-separated CSS classes to add to element
           *   - **from** - `{Object}` - CSS properties & values at the beginning of animation. Must have matching `to`
           *   - **removeClass** - `{string}` - space-separated CSS classes to remove from element
           *   - **to** - `{Object}` - CSS properties & values at end of animation. Must have matching `from`
           *
           * @return {Promise} the animation callback promise
           */
          move: function (element, parent, after, options) {
            parent = parent && jqLite(parent);
            after = after && jqLite(after);
            parent = parent || after.parent();
            domInsert(element, parent, after);
            return $$animateQueue.push(element, 'move', prepareAnimateOptions(options));
          },

          /**
           * @ngdoc method
           * @name $animate#leave
           * @kind function
           * @description Triggers an animation and then removes the element from the DOM.
           * When the function is called a promise is returned that will be resolved during the next
           * digest once the animation has completed.
           *
           * @param {DOMElement} element the element which will be removed from the DOM
           * @param {object=} options an optional collection of options/styles that will be applied to the element.
           *   The object can have the following properties:
           *
           *   - **addClass** - `{string}` - space-separated CSS classes to add to element
           *   - **from** - `{Object}` - CSS properties & values at the beginning of animation. Must have matching `to`
           *   - **removeClass** - `{string}` - space-separated CSS classes to remove from element
           *   - **to** - `{Object}` - CSS properties & values at end of animation. Must have matching `from`
           *
           * @return {Promise} the animation callback promise
           */
          leave: function (element, options) {
            return $$animateQueue.push(element, 'leave', prepareAnimateOptions(options), function () {
              element.remove();
            });
          },

          /**
           * @ngdoc method
           * @name $animate#addClass
           * @kind function
           *
           * @description Triggers an addClass animation surrounding the addition of the provided CSS class(es). Upon
           *   execution, the addClass operation will only be handled after the next digest and it will not trigger an
           *   animation if element already contains the CSS class or if the class is removed at a later step.
           *   Note that class-based animations are treated differently compared to structural animations
           *   (like enter, move and leave) since the CSS classes may be added/removed at different points
           *   depending if CSS or JavaScript animations are used.
           *
           * @param {DOMElement} element the element which the CSS classes will be applied to
           * @param {string} className the CSS class(es) that will be added (multiple classes are separated via spaces)
           * @param {object=} options an optional collection of options/styles that will be applied to the element.
           *   The object can have the following properties:
           *
           *   - **addClass** - `{string}` - space-separated CSS classes to add to element
           *   - **from** - `{Object}` - CSS properties & values at the beginning of animation. Must have matching `to`
           *   - **removeClass** - `{string}` - space-separated CSS classes to remove from element
           *   - **to** - `{Object}` - CSS properties & values at end of animation. Must have matching `from`
           *
           * @return {Promise} the animation callback promise
           */
          addClass: function (element, className, options) {
            options = prepareAnimateOptions(options);
            options.addClass = mergeClasses(options.addclass, className);
            return $$animateQueue.push(element, 'addClass', options);
          },

          /**
           * @ngdoc method
           * @name $animate#removeClass
           * @kind function
           *
           * @description Triggers a removeClass animation surrounding the removal of the provided CSS class(es). Upon
           *   execution, the removeClass operation will only be handled after the next digest and it will not trigger an
           *   animation if element does not contain the CSS class or if the class is added at a later step.
           *   Note that class-based animations are treated differently compared to structural animations
           *   (like enter, move and leave) since the CSS classes may be added/removed at different points
           *   depending if CSS or JavaScript animations are used.
           *
           * @param {DOMElement} element the element which the CSS classes will be applied to
           * @param {string} className the CSS class(es) that will be removed (multiple classes are separated via spaces)
           * @param {object=} options an optional collection of options/styles that will be applied to the element.
           *   The object can have the following properties:
           *
           *   - **addClass** - `{string}` - space-separated CSS classes to add to element
           *   - **from** - `{Object}` - CSS properties & values at the beginning of animation. Must have matching `to`
           *   - **removeClass** - `{string}` - space-separated CSS classes to remove from element
           *   - **to** - `{Object}` - CSS properties & values at end of animation. Must have matching `from`
           *
           * @return {Promise} the animation callback promise
           */
          removeClass: function (element, className, options) {
            options = prepareAnimateOptions(options);
            options.removeClass = mergeClasses(options.removeClass, className);
            return $$animateQueue.push(element, 'removeClass', options);
          },

          /**
           * @ngdoc method
           * @name $animate#setClass
           * @kind function
           *
           * @description Performs both the addition and removal of a CSS classes on an element and (during the process)
           *    triggers an animation surrounding the class addition/removal. Much like `$animate.addClass` and
           *    `$animate.removeClass`, `setClass` will only evaluate the classes being added/removed once a digest has
           *    passed. Note that class-based animations are treated differently compared to structural animations
           *    (like enter, move and leave) since the CSS classes may be added/removed at different points
           *    depending if CSS or JavaScript animations are used.
           *
           * @param {DOMElement} element the element which the CSS classes will be applied to
           * @param {string} add the CSS class(es) that will be added (multiple classes are separated via spaces)
           * @param {string} remove the CSS class(es) that will be removed (multiple classes are separated via spaces)
           * @param {object=} options an optional collection of options/styles that will be applied to the element.
           *   The object can have the following properties:
           *
           *   - **addClass** - `{string}` - space-separated CSS classes to add to element
           *   - **from** - `{Object}` - CSS properties & values at the beginning of animation. Must have matching `to`
           *   - **removeClass** - `{string}` - space-separated CSS classes to remove from element
           *   - **to** - `{Object}` - CSS properties & values at end of animation. Must have matching `from`
           *
           * @return {Promise} the animation callback promise
           */
          setClass: function (element, add, remove, options) {
            options = prepareAnimateOptions(options);
            options.addClass = mergeClasses(options.addClass, add);
            options.removeClass = mergeClasses(options.removeClass, remove);
            return $$animateQueue.push(element, 'setClass', options);
          },

          /**
           * @ngdoc method
           * @name $animate#animate
           * @kind function
           *
           * @description Performs an inline animation on the element which applies the provided to and from CSS styles to the element.
           * If any detected CSS transition, keyframe or JavaScript matches the provided className value, then the animation will take
           * on the provided styles. For example, if a transition animation is set for the given className, then the provided `from` and
           * `to` styles will be applied alongside the given transition. If the CSS style provided in `from` does not have a corresponding
           * style in `to`, the style in `from` is applied immediately, and no animation is run.
           * If a JavaScript animation is detected then the provided styles will be given in as function parameters into the `animate`
           * method (or as part of the `options` parameter):
           *
           * ```js
           * ngModule.animation('.my-inline-animation', function() {
           *   return {
           *     animate : function(element, from, to, done, options) {
           *       //animation
           *       done();
           *     }
           *   }
           * });
           * ```
           *
           * @param {DOMElement} element the element which the CSS styles will be applied to
           * @param {object} from the from (starting) CSS styles that will be applied to the element and across the animation.
           * @param {object} to the to (destination) CSS styles that will be applied to the element and across the animation.
           * @param {string=} className an optional CSS class that will be applied to the element for the duration of the animation. If
           *    this value is left as empty then a CSS class of `ng-inline-animate` will be applied to the element.
           *    (Note that if no animation is detected then this value will not be applied to the element.)
           * @param {object=} options an optional collection of options/styles that will be applied to the element.
           *   The object can have the following properties:
           *
           *   - **addClass** - `{string}` - space-separated CSS classes to add to element
           *   - **from** - `{Object}` - CSS properties & values at the beginning of animation. Must have matching `to`
           *   - **removeClass** - `{string}` - space-separated CSS classes to remove from element
           *   - **to** - `{Object}` - CSS properties & values at end of animation. Must have matching `from`
           *
           * @return {Promise} the animation callback promise
           */
          animate: function (element, from, to, className, options) {
            options = prepareAnimateOptions(options);
            options.from = options.from ? extend(options.from, from) : from;
            options.to = options.to ? extend(options.to, to) : to;

            className = className || 'ng-inline-animate';
            options.tempClasses = mergeClasses(options.tempClasses, className);
            return $$animateQueue.push(element, 'animate', options);
          }
        };
      }];
    }];

    var $$AnimateAsyncRunFactoryProvider = /** @this */function () {
      this.$get = ['$$rAF', function ($$rAF) {
        var waitQueue = [];

        function waitForTick(fn) {
          waitQueue.push(fn);
          if (waitQueue.length > 1) return;
          $$rAF(function () {
            for (var i = 0; i < waitQueue.length; i++) {
              waitQueue[i]();
            }
            waitQueue = [];
          });
        }

        return function () {
          var passed = false;
          waitForTick(function () {
            passed = true;
          });
          return function (callback) {
            if (passed) {
              callback();
            } else {
              waitForTick(callback);
            }
          };
        };
      }];
    };

    var $$AnimateRunnerFactoryProvider = /** @this */function () {
      this.$get = ['$q', '$sniffer', '$$animateAsyncRun', '$$isDocumentHidden', '$timeout', function ($q, $sniffer, $$animateAsyncRun, $$isDocumentHidden, $timeout) {

        var INITIAL_STATE = 0;
        var DONE_PENDING_STATE = 1;
        var DONE_COMPLETE_STATE = 2;

        AnimateRunner.chain = function (chain, callback) {
          var index = 0;

          next();
          function next() {
            if (index === chain.length) {
              callback(true);
              return;
            }

            chain[index](function (response) {
              if (response === false) {
                callback(false);
                return;
              }
              index++;
              next();
            });
          }
        };

        AnimateRunner.all = function (runners, callback) {
          var count = 0;
          var status = true;
          forEach(runners, function (runner) {
            runner.done(onProgress);
          });

          function onProgress(response) {
            status = status && response;
            if (++count === runners.length) {
              callback(status);
            }
          }
        };

        function AnimateRunner(host) {
          this.setHost(host);

          var rafTick = $$animateAsyncRun();
          var timeoutTick = function (fn) {
            $timeout(fn, 0, false);
          };

          this._doneCallbacks = [];
          this._tick = function (fn) {
            if ($$isDocumentHidden()) {
              timeoutTick(fn);
            } else {
              rafTick(fn);
            }
          };
          this._state = 0;
        }

        AnimateRunner.prototype = {
          setHost: function (host) {
            this.host = host || {};
          },

          done: function (fn) {
            if (this._state === DONE_COMPLETE_STATE) {
              fn();
            } else {
              this._doneCallbacks.push(fn);
            }
          },

          progress: noop,

          getPromise: function () {
            if (!this.promise) {
              var self = this;
              this.promise = $q(function (resolve, reject) {
                self.done(function (status) {
                  if (status === false) {
                    reject();
                  } else {
                    resolve();
                  }
                });
              });
            }
            return this.promise;
          },

          then: function (resolveHandler, rejectHandler) {
            return this.getPromise().then(resolveHandler, rejectHandler);
          },

          'catch': function (handler) {
            return this.getPromise()['catch'](handler);
          },

          'finally': function (handler) {
            return this.getPromise()['finally'](handler);
          },

          pause: function () {
            if (this.host.pause) {
              this.host.pause();
            }
          },

          resume: function () {
            if (this.host.resume) {
              this.host.resume();
            }
          },

          end: function () {
            if (this.host.end) {
              this.host.end();
            }
            this._resolve(true);
          },

          cancel: function () {
            if (this.host.cancel) {
              this.host.cancel();
            }
            this._resolve(false);
          },

          complete: function (response) {
            var self = this;
            if (self._state === INITIAL_STATE) {
              self._state = DONE_PENDING_STATE;
              self._tick(function () {
                self._resolve(response);
              });
            }
          },

          _resolve: function (response) {
            if (this._state !== DONE_COMPLETE_STATE) {
              forEach(this._doneCallbacks, function (fn) {
                fn(response);
              });
              this._doneCallbacks.length = 0;
              this._state = DONE_COMPLETE_STATE;
            }
          }
        };

        return AnimateRunner;
      }];
    };

    /* exported $CoreAnimateCssProvider */

    /**
     * @ngdoc service
     * @name $animateCss
     * @kind object
     * @this
     *
     * @description
     * This is the core version of `$animateCss`. By default, only when the `ngAnimate` is included,
     * then the `$animateCss` service will actually perform animations.
     *
     * Click here {@link ngAnimate.$animateCss to read the documentation for $animateCss}.
     */
    var $CoreAnimateCssProvider = function () {
      this.$get = ['$$rAF', '$q', '$$AnimateRunner', function ($$rAF, $q, $$AnimateRunner) {

        return function (element, initialOptions) {
          // all of the animation functions should create
          // a copy of the options data, however, if a
          // parent service has already created a copy then
          // we should stick to using that
          var options = initialOptions || {};
          if (!options.$$prepared) {
            options = copy(options);
          }

          // there is no point in applying the styles since
          // there is no animation that goes on at all in
          // this version of $animateCss.
          if (options.cleanupStyles) {
            options.from = options.to = null;
          }

          if (options.from) {
            element.css(options.from);
            options.from = null;
          }

          var closed,
              runner = new $$AnimateRunner();
          return {
            start: run,
            end: run
          };

          function run() {
            $$rAF(function () {
              applyAnimationContents();
              if (!closed) {
                runner.complete();
              }
              closed = true;
            });
            return runner;
          }

          function applyAnimationContents() {
            if (options.addClass) {
              element.addClass(options.addClass);
              options.addClass = null;
            }
            if (options.removeClass) {
              element.removeClass(options.removeClass);
              options.removeClass = null;
            }
            if (options.to) {
              element.css(options.to);
              options.to = null;
            }
          }
        };
      }];
    };

    /* global stripHash: true */

    /**
     * ! This is a private undocumented service !
     *
     * @name $browser
     * @requires $log
     * @description
     * This object has two goals:
     *
     * - hide all the global state in the browser caused by the window object
     * - abstract away all the browser specific features and inconsistencies
     *
     * For tests we provide {@link ngMock.$browser mock implementation} of the `$browser`
     * service, which can be used for convenient testing of the application without the interaction with
     * the real browser apis.
     */
    /**
     * @param {object} window The global window object.
     * @param {object} document jQuery wrapped document.
     * @param {object} $log window.console or an object with the same interface.
     * @param {object} $sniffer $sniffer service
     */
    function Browser(window, document, $log, $sniffer) {
      var self = this,
          location = window.location,
          history = window.history,
          setTimeout = window.setTimeout,
          clearTimeout = window.clearTimeout,
          pendingDeferIds = {};

      self.isMock = false;

      var outstandingRequestCount = 0;
      var outstandingRequestCallbacks = [];

      // TODO(vojta): remove this temporary api
      self.$$completeOutstandingRequest = completeOutstandingRequest;
      self.$$incOutstandingRequestCount = function () {
        outstandingRequestCount++;
      };

      /**
       * Executes the `fn` function(supports currying) and decrements the `outstandingRequestCallbacks`
       * counter. If the counter reaches 0, all the `outstandingRequestCallbacks` are executed.
       */
      function completeOutstandingRequest(fn) {
        try {
          fn.apply(null, sliceArgs(arguments, 1));
        } finally {
          outstandingRequestCount--;
          if (outstandingRequestCount === 0) {
            while (outstandingRequestCallbacks.length) {
              try {
                outstandingRequestCallbacks.pop()();
              } catch (e) {
                $log.error(e);
              }
            }
          }
        }
      }

      function getHash(url) {
        var index = url.indexOf('#');
        return index === -1 ? '' : url.substr(index);
      }

      /**
       * @private
       * Note: this method is used only by scenario runner
       * TODO(vojta): prefix this method with $$ ?
       * @param {function()} callback Function that will be called when no outstanding request
       */
      self.notifyWhenNoOutstandingRequests = function (callback) {
        if (outstandingRequestCount === 0) {
          callback();
        } else {
          outstandingRequestCallbacks.push(callback);
        }
      };

      //////////////////////////////////////////////////////////////
      // URL API
      //////////////////////////////////////////////////////////////

      var cachedState,
          lastHistoryState,
          lastBrowserUrl = location.href,
          baseElement = document.find('base'),
          pendingLocation = null,
          getCurrentState = !$sniffer.history ? noop : function getCurrentState() {
        try {
          return history.state;
        } catch (e) {
          // MSIE can reportedly throw when there is no state (UNCONFIRMED).
        }
      };

      cacheState();

      /**
       * @name $browser#url
       *
       * @description
       * GETTER:
       * Without any argument, this method just returns current value of location.href.
       *
       * SETTER:
       * With at least one argument, this method sets url to new value.
       * If html5 history api supported, pushState/replaceState is used, otherwise
       * location.href/location.replace is used.
       * Returns its own instance to allow chaining
       *
       * NOTE: this api is intended for use only by the $location service. Please use the
       * {@link ng.$location $location service} to change url.
       *
       * @param {string} url New url (when used as setter)
       * @param {boolean=} replace Should new url replace current history record?
       * @param {object=} state object to use with pushState/replaceState
       */
      self.url = function (url, replace, state) {
        // In modern browsers `history.state` is `null` by default; treating it separately
        // from `undefined` would cause `$browser.url('/foo')` to change `history.state`
        // to undefined via `pushState`. Instead, let's change `undefined` to `null` here.
        if (isUndefined(state)) {
          state = null;
        }

        // Android Browser BFCache causes location, history reference to become stale.
        if (location !== window.location) location = window.location;
        if (history !== window.history) history = window.history;

        // setter
        if (url) {
          var sameState = lastHistoryState === state;

          // Don't change anything if previous and current URLs and states match. This also prevents
          // IE<10 from getting into redirect loop when in LocationHashbangInHtml5Url mode.
          // See https://github.com/angular/angular.js/commit/ffb2701
          if (lastBrowserUrl === url && (!$sniffer.history || sameState)) {
            return self;
          }
          var sameBase = lastBrowserUrl && stripHash(lastBrowserUrl) === stripHash(url);
          lastBrowserUrl = url;
          lastHistoryState = state;
          // Don't use history API if only the hash changed
          // due to a bug in IE10/IE11 which leads
          // to not firing a `hashchange` nor `popstate` event
          // in some cases (see #9143).
          if ($sniffer.history && (!sameBase || !sameState)) {
            history[replace ? 'replaceState' : 'pushState'](state, '', url);
            cacheState();
          } else {
            if (!sameBase) {
              pendingLocation = url;
            }
            if (replace) {
              location.replace(url);
            } else if (!sameBase) {
              location.href = url;
            } else {
              location.hash = getHash(url);
            }
            if (location.href !== url) {
              pendingLocation = url;
            }
          }
          if (pendingLocation) {
            pendingLocation = url;
          }
          return self;
          // getter
        } else {
          // - pendingLocation is needed as browsers don't allow to read out
          //   the new location.href if a reload happened or if there is a bug like in iOS 9 (see
          //   https://openradar.appspot.com/22186109).
          // - the replacement is a workaround for https://bugzilla.mozilla.org/show_bug.cgi?id=407172
          return pendingLocation || location.href.replace(/%27/g, '\'');
        }
      };

      /**
       * @name $browser#state
       *
       * @description
       * This method is a getter.
       *
       * Return history.state or null if history.state is undefined.
       *
       * @returns {object} state
       */
      self.state = function () {
        return cachedState;
      };

      var urlChangeListeners = [],
          urlChangeInit = false;

      function cacheStateAndFireUrlChange() {
        pendingLocation = null;
        fireStateOrUrlChange();
      }

      // This variable should be used *only* inside the cacheState function.
      var lastCachedState = null;
      function cacheState() {
        // This should be the only place in $browser where `history.state` is read.
        cachedState = getCurrentState();
        cachedState = isUndefined(cachedState) ? null : cachedState;

        // Prevent callbacks fo fire twice if both hashchange & popstate were fired.
        if (equals(cachedState, lastCachedState)) {
          cachedState = lastCachedState;
        }

        lastCachedState = cachedState;
        lastHistoryState = cachedState;
      }

      function fireStateOrUrlChange() {
        var prevLastHistoryState = lastHistoryState;
        cacheState();

        if (lastBrowserUrl === self.url() && prevLastHistoryState === cachedState) {
          return;
        }

        lastBrowserUrl = self.url();
        lastHistoryState = cachedState;
        forEach(urlChangeListeners, function (listener) {
          listener(self.url(), cachedState);
        });
      }

      /**
       * @name $browser#onUrlChange
       *
       * @description
       * Register callback function that will be called, when url changes.
       *
       * It's only called when the url is changed from outside of angular:
       * - user types different url into address bar
       * - user clicks on history (forward/back) button
       * - user clicks on a link
       *
       * It's not called when url is changed by $browser.url() method
       *
       * The listener gets called with new url as parameter.
       *
       * NOTE: this api is intended for use only by the $location service. Please use the
       * {@link ng.$location $location service} to monitor url changes in angular apps.
       *
       * @param {function(string)} listener Listener function to be called when url changes.
       * @return {function(string)} Returns the registered listener fn - handy if the fn is anonymous.
       */
      self.onUrlChange = function (callback) {
        // TODO(vojta): refactor to use node's syntax for events
        if (!urlChangeInit) {
          // We listen on both (hashchange/popstate) when available, as some browsers don't
          // fire popstate when user changes the address bar and don't fire hashchange when url
          // changed by push/replaceState

          // html5 history api - popstate event
          if ($sniffer.history) jqLite(window).on('popstate', cacheStateAndFireUrlChange);
          // hashchange event
          jqLite(window).on('hashchange', cacheStateAndFireUrlChange);

          urlChangeInit = true;
        }

        urlChangeListeners.push(callback);
        return callback;
      };

      /**
       * @private
       * Remove popstate and hashchange handler from window.
       *
       * NOTE: this api is intended for use only by $rootScope.
       */
      self.$$applicationDestroyed = function () {
        jqLite(window).off('hashchange popstate', cacheStateAndFireUrlChange);
      };

      /**
       * Checks whether the url has changed outside of Angular.
       * Needs to be exported to be able to check for changes that have been done in sync,
       * as hashchange/popstate events fire in async.
       */
      self.$$checkUrlChange = fireStateOrUrlChange;

      //////////////////////////////////////////////////////////////
      // Misc API
      //////////////////////////////////////////////////////////////

      /**
       * @name $browser#baseHref
       *
       * @description
       * Returns current <base href>
       * (always relative - without domain)
       *
       * @returns {string} The current base href
       */
      self.baseHref = function () {
        var href = baseElement.attr('href');
        return href ? href.replace(/^(https?:)?\/\/[^/]*/, '') : '';
      };

      /**
       * @name $browser#defer
       * @param {function()} fn A function, who's execution should be deferred.
       * @param {number=} [delay=0] of milliseconds to defer the function execution.
       * @returns {*} DeferId that can be used to cancel the task via `$browser.defer.cancel()`.
       *
       * @description
       * Executes a fn asynchronously via `setTimeout(fn, delay)`.
       *
       * Unlike when calling `setTimeout` directly, in test this function is mocked and instead of using
       * `setTimeout` in tests, the fns are queued in an array, which can be programmatically flushed
       * via `$browser.defer.flush()`.
       *
       */
      self.defer = function (fn, delay) {
        var timeoutId;
        outstandingRequestCount++;
        timeoutId = setTimeout(function () {
          delete pendingDeferIds[timeoutId];
          completeOutstandingRequest(fn);
        }, delay || 0);
        pendingDeferIds[timeoutId] = true;
        return timeoutId;
      };

      /**
       * @name $browser#defer.cancel
       *
       * @description
       * Cancels a deferred task identified with `deferId`.
       *
       * @param {*} deferId Token returned by the `$browser.defer` function.
       * @returns {boolean} Returns `true` if the task hasn't executed yet and was successfully
       *                    canceled.
       */
      self.defer.cancel = function (deferId) {
        if (pendingDeferIds[deferId]) {
          delete pendingDeferIds[deferId];
          clearTimeout(deferId);
          completeOutstandingRequest(noop);
          return true;
        }
        return false;
      };
    }

    /** @this */
    function $BrowserProvider() {
      this.$get = ['$window', '$log', '$sniffer', '$document', function ($window, $log, $sniffer, $document) {
        return new Browser($window, $document, $log, $sniffer);
      }];
    }

    /**
     * @ngdoc service
     * @name $cacheFactory
     * @this
     *
     * @description
     * Factory that constructs {@link $cacheFactory.Cache Cache} objects and gives access to
     * them.
     *
     * ```js
     *
     *  var cache = $cacheFactory('cacheId');
     *  expect($cacheFactory.get('cacheId')).toBe(cache);
     *  expect($cacheFactory.get('noSuchCacheId')).not.toBeDefined();
     *
     *  cache.put("key", "value");
     *  cache.put("another key", "another value");
     *
     *  // We've specified no options on creation
     *  expect(cache.info()).toEqual({id: 'cacheId', size: 2});
     *
     * ```
     *
     *
     * @param {string} cacheId Name or id of the newly created cache.
     * @param {object=} options Options object that specifies the cache behavior. Properties:
     *
     *   - `{number=}` `capacity` — turns the cache into LRU cache.
     *
     * @returns {object} Newly created cache object with the following set of methods:
     *
     * - `{object}` `info()` — Returns id, size, and options of cache.
     * - `{{*}}` `put({string} key, {*} value)` — Puts a new key-value pair into the cache and returns
     *   it.
     * - `{{*}}` `get({string} key)` — Returns cached value for `key` or undefined for cache miss.
     * - `{void}` `remove({string} key)` — Removes a key-value pair from the cache.
     * - `{void}` `removeAll()` — Removes all cached values.
     * - `{void}` `destroy()` — Removes references to this cache from $cacheFactory.
     *
     * @example
       <example module="cacheExampleApp" name="cache-factory">
         <file name="index.html">
           <div ng-controller="CacheController">
             <input ng-model="newCacheKey" placeholder="Key">
             <input ng-model="newCacheValue" placeholder="Value">
             <button ng-click="put(newCacheKey, newCacheValue)">Cache</button>
    
             <p ng-if="keys.length">Cached Values</p>
             <div ng-repeat="key in keys">
               <span ng-bind="key"></span>
               <span>: </span>
               <b ng-bind="cache.get(key)"></b>
             </div>
    
             <p>Cache Info</p>
             <div ng-repeat="(key, value) in cache.info()">
               <span ng-bind="key"></span>
               <span>: </span>
               <b ng-bind="value"></b>
             </div>
           </div>
         </file>
         <file name="script.js">
           angular.module('cacheExampleApp', []).
             controller('CacheController', ['$scope', '$cacheFactory', function($scope, $cacheFactory) {
               $scope.keys = [];
               $scope.cache = $cacheFactory('cacheId');
               $scope.put = function(key, value) {
                 if (angular.isUndefined($scope.cache.get(key))) {
                   $scope.keys.push(key);
                 }
                 $scope.cache.put(key, angular.isUndefined(value) ? null : value);
               };
             }]);
         </file>
         <file name="style.css">
           p {
             margin: 10px 0 3px;
           }
         </file>
       </example>
     */
    function $CacheFactoryProvider() {

      this.$get = function () {
        var caches = {};

        function cacheFactory(cacheId, options) {
          if (cacheId in caches) {
            throw minErr('$cacheFactory')('iid', 'CacheId \'{0}\' is already taken!', cacheId);
          }

          var size = 0,
              stats = extend({}, options, { id: cacheId }),
              data = createMap(),
              capacity = options && options.capacity || Number.MAX_VALUE,
              lruHash = createMap(),
              freshEnd = null,
              staleEnd = null;

          /**
           * @ngdoc type
           * @name $cacheFactory.Cache
           *
           * @description
           * A cache object used to store and retrieve data, primarily used by
           * {@link $http $http} and the {@link ng.directive:script script} directive to cache
           * templates and other data.
           *
           * ```js
           *  angular.module('superCache')
           *    .factory('superCache', ['$cacheFactory', function($cacheFactory) {
           *      return $cacheFactory('super-cache');
           *    }]);
           * ```
           *
           * Example test:
           *
           * ```js
           *  it('should behave like a cache', inject(function(superCache) {
           *    superCache.put('key', 'value');
           *    superCache.put('another key', 'another value');
           *
           *    expect(superCache.info()).toEqual({
           *      id: 'super-cache',
           *      size: 2
           *    });
           *
           *    superCache.remove('another key');
           *    expect(superCache.get('another key')).toBeUndefined();
           *
           *    superCache.removeAll();
           *    expect(superCache.info()).toEqual({
           *      id: 'super-cache',
           *      size: 0
           *    });
           *  }));
           * ```
           */
          return caches[cacheId] = {

            /**
             * @ngdoc method
             * @name $cacheFactory.Cache#put
             * @kind function
             *
             * @description
             * Inserts a named entry into the {@link $cacheFactory.Cache Cache} object to be
             * retrieved later, and incrementing the size of the cache if the key was not already
             * present in the cache. If behaving like an LRU cache, it will also remove stale
             * entries from the set.
             *
             * It will not insert undefined values into the cache.
             *
             * @param {string} key the key under which the cached data is stored.
             * @param {*} value the value to store alongside the key. If it is undefined, the key
             *    will not be stored.
             * @returns {*} the value stored.
             */
            put: function (key, value) {
              if (isUndefined(value)) return;
              if (capacity < Number.MAX_VALUE) {
                var lruEntry = lruHash[key] || (lruHash[key] = { key: key });

                refresh(lruEntry);
              }

              if (!(key in data)) size++;
              data[key] = value;

              if (size > capacity) {
                this.remove(staleEnd.key);
              }

              return value;
            },

            /**
             * @ngdoc method
             * @name $cacheFactory.Cache#get
             * @kind function
             *
             * @description
             * Retrieves named data stored in the {@link $cacheFactory.Cache Cache} object.
             *
             * @param {string} key the key of the data to be retrieved
             * @returns {*} the value stored.
             */
            get: function (key) {
              if (capacity < Number.MAX_VALUE) {
                var lruEntry = lruHash[key];

                if (!lruEntry) return;

                refresh(lruEntry);
              }

              return data[key];
            },

            /**
             * @ngdoc method
             * @name $cacheFactory.Cache#remove
             * @kind function
             *
             * @description
             * Removes an entry from the {@link $cacheFactory.Cache Cache} object.
             *
             * @param {string} key the key of the entry to be removed
             */
            remove: function (key) {
              if (capacity < Number.MAX_VALUE) {
                var lruEntry = lruHash[key];

                if (!lruEntry) return;

                if (lruEntry === freshEnd) freshEnd = lruEntry.p;
                if (lruEntry === staleEnd) staleEnd = lruEntry.n;
                link(lruEntry.n, lruEntry.p);

                delete lruHash[key];
              }

              if (!(key in data)) return;

              delete data[key];
              size--;
            },

            /**
             * @ngdoc method
             * @name $cacheFactory.Cache#removeAll
             * @kind function
             *
             * @description
             * Clears the cache object of any entries.
             */
            removeAll: function () {
              data = createMap();
              size = 0;
              lruHash = createMap();
              freshEnd = staleEnd = null;
            },

            /**
             * @ngdoc method
             * @name $cacheFactory.Cache#destroy
             * @kind function
             *
             * @description
             * Destroys the {@link $cacheFactory.Cache Cache} object entirely,
             * removing it from the {@link $cacheFactory $cacheFactory} set.
             */
            destroy: function () {
              data = null;
              stats = null;
              lruHash = null;
              delete caches[cacheId];
            },

            /**
             * @ngdoc method
             * @name $cacheFactory.Cache#info
             * @kind function
             *
             * @description
             * Retrieve information regarding a particular {@link $cacheFactory.Cache Cache}.
             *
             * @returns {object} an object with the following properties:
             *   <ul>
             *     <li>**id**: the id of the cache instance</li>
             *     <li>**size**: the number of entries kept in the cache instance</li>
             *     <li>**...**: any additional properties from the options object when creating the
             *       cache.</li>
             *   </ul>
             */
            info: function () {
              return extend({}, stats, { size: size });
            }
          };

          /**
           * makes the `entry` the freshEnd of the LRU linked list
           */
          function refresh(entry) {
            if (entry !== freshEnd) {
              if (!staleEnd) {
                staleEnd = entry;
              } else if (staleEnd === entry) {
                staleEnd = entry.n;
              }

              link(entry.n, entry.p);
              link(entry, freshEnd);
              freshEnd = entry;
              freshEnd.n = null;
            }
          }

          /**
           * bidirectionally links two entries of the LRU linked list
           */
          function link(nextEntry, prevEntry) {
            if (nextEntry !== prevEntry) {
              if (nextEntry) nextEntry.p = prevEntry; //p stands for previous, 'prev' didn't minify
              if (prevEntry) prevEntry.n = nextEntry; //n stands for next, 'next' didn't minify
            }
          }
        }

        /**
         * @ngdoc method
         * @name $cacheFactory#info
         *
         * @description
         * Get information about all the caches that have been created
         *
         * @returns {Object} - key-value map of `cacheId` to the result of calling `cache#info`
         */
        cacheFactory.info = function () {
          var info = {};
          forEach(caches, function (cache, cacheId) {
            info[cacheId] = cache.info();
          });
          return info;
        };

        /**
         * @ngdoc method
         * @name $cacheFactory#get
         *
         * @description
         * Get access to a cache object by the `cacheId` used when it was created.
         *
         * @param {string} cacheId Name or id of a cache to access.
         * @returns {object} Cache object identified by the cacheId or undefined if no such cache.
         */
        cacheFactory.get = function (cacheId) {
          return caches[cacheId];
        };

        return cacheFactory;
      };
    }

    /**
     * @ngdoc service
     * @name $templateCache
     * @this
     *
     * @description
     * The first time a template is used, it is loaded in the template cache for quick retrieval. You
     * can load templates directly into the cache in a `script` tag, or by consuming the
     * `$templateCache` service directly.
     *
     * Adding via the `script` tag:
     *
     * ```html
     *   <script type="text/ng-template" id="templateId.html">
     *     <p>This is the content of the template</p>
     *   </script>
     * ```
     *
     * **Note:** the `script` tag containing the template does not need to be included in the `head` of
     * the document, but it must be a descendent of the {@link ng.$rootElement $rootElement} (IE,
     * element with ng-app attribute), otherwise the template will be ignored.
     *
     * Adding via the `$templateCache` service:
     *
     * ```js
     * var myApp = angular.module('myApp', []);
     * myApp.run(function($templateCache) {
     *   $templateCache.put('templateId.html', 'This is the content of the template');
     * });
     * ```
     *
     * To retrieve the template later, simply use it in your component:
     * ```js
     * myApp.component('myComponent', {
     *    templateUrl: 'templateId.html'
     * });
     * ```
     *
     * or get it via the `$templateCache` service:
     * ```js
     * $templateCache.get('templateId.html')
     * ```
     *
     * See {@link ng.$cacheFactory $cacheFactory}.
     *
     */
    function $TemplateCacheProvider() {
      this.$get = ['$cacheFactory', function ($cacheFactory) {
        return $cacheFactory('templates');
      }];
    }

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
     *     Any commits to this file should be reviewed with security in mind.  *
     *   Changes to this file can potentially create security vulnerabilities. *
     *          An approval from 2 Core members with history of modifying      *
     *                         this file is required.                          *
     *                                                                         *
     *  Does the change somehow allow for arbitrary javascript to be executed? *
     *    Or allows for someone to change the prototype of built-in objects?   *
     *     Or gives undesired access to variables like document or window?    *
     * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

    /* ! VARIABLE/FUNCTION NAMING CONVENTIONS THAT APPLY TO THIS FILE!
     *
     * DOM-related variables:
     *
     * - "node" - DOM Node
     * - "element" - DOM Element or Node
     * - "$node" or "$element" - jqLite-wrapped node or element
     *
     *
     * Compiler related stuff:
     *
     * - "linkFn" - linking fn of a single directive
     * - "nodeLinkFn" - function that aggregates all linking fns for a particular node
     * - "childLinkFn" -  function that aggregates all linking fns for child nodes of a particular node
     * - "compositeLinkFn" - function that aggregates all linking fns for a compilation root (nodeList)
     */

    /**
     * @ngdoc service
     * @name $compile
     * @kind function
     *
     * @description
     * Compiles an HTML string or DOM into a template and produces a template function, which
     * can then be used to link {@link ng.$rootScope.Scope `scope`} and the template together.
     *
     * The compilation is a process of walking the DOM tree and matching DOM elements to
     * {@link ng.$compileProvider#directive directives}.
     *
     * <div class="alert alert-warning">
     * **Note:** This document is an in-depth reference of all directive options.
     * For a gentle introduction to directives with examples of common use cases,
     * see the {@link guide/directive directive guide}.
     * </div>
     *
     * ## Comprehensive Directive API
     *
     * There are many different options for a directive.
     *
     * The difference resides in the return value of the factory function.
     * You can either return a {@link $compile#directive-definition-object Directive Definition Object (see below)}
     * that defines the directive properties, or just the `postLink` function (all other properties will have
     * the default values).
     *
     * <div class="alert alert-success">
     * **Best Practice:** It's recommended to use the "directive definition object" form.
     * </div>
     *
     * Here's an example directive declared with a Directive Definition Object:
     *
     * ```js
     *   var myModule = angular.module(...);
     *
     *   myModule.directive('directiveName', function factory(injectables) {
     *     var directiveDefinitionObject = {
     *       {@link $compile#-priority- priority}: 0,
     *       {@link $compile#-template- template}: '<div></div>', // or // function(tElement, tAttrs) { ... },
     *       // or
     *       // {@link $compile#-templateurl- templateUrl}: 'directive.html', // or // function(tElement, tAttrs) { ... },
     *       {@link $compile#-transclude- transclude}: false,
     *       {@link $compile#-restrict- restrict}: 'A',
     *       {@link $compile#-templatenamespace- templateNamespace}: 'html',
     *       {@link $compile#-scope- scope}: false,
     *       {@link $compile#-controller- controller}: function($scope, $element, $attrs, $transclude, otherInjectables) { ... },
     *       {@link $compile#-controlleras- controllerAs}: 'stringIdentifier',
     *       {@link $compile#-bindtocontroller- bindToController}: false,
     *       {@link $compile#-require- require}: 'siblingDirectiveName', // or // ['^parentDirectiveName', '?optionalDirectiveName', '?^optionalParent'],
     *       {@link $compile#-multielement- multiElement}: false,
     *       {@link $compile#-compile- compile}: function compile(tElement, tAttrs, transclude) {
     *         return {
     *            {@link $compile#pre-linking-function pre}: function preLink(scope, iElement, iAttrs, controller) { ... },
     *            {@link $compile#post-linking-function post}: function postLink(scope, iElement, iAttrs, controller) { ... }
     *         }
     *         // or
     *         // return function postLink( ... ) { ... }
     *       },
     *       // or
     *       // {@link $compile#-link- link}: {
     *       //  {@link $compile#pre-linking-function pre}: function preLink(scope, iElement, iAttrs, controller) { ... },
     *       //  {@link $compile#post-linking-function post}: function postLink(scope, iElement, iAttrs, controller) { ... }
     *       // }
     *       // or
     *       // {@link $compile#-link- link}: function postLink( ... ) { ... }
     *     };
     *     return directiveDefinitionObject;
     *   });
     * ```
     *
     * <div class="alert alert-warning">
     * **Note:** Any unspecified options will use the default value. You can see the default values below.
     * </div>
     *
     * Therefore the above can be simplified as:
     *
     * ```js
     *   var myModule = angular.module(...);
     *
     *   myModule.directive('directiveName', function factory(injectables) {
     *     var directiveDefinitionObject = {
     *       link: function postLink(scope, iElement, iAttrs) { ... }
     *     };
     *     return directiveDefinitionObject;
     *     // or
     *     // return function postLink(scope, iElement, iAttrs) { ... }
     *   });
     * ```
     *
     * ### Life-cycle hooks
     * Directive controllers can provide the following methods that are called by Angular at points in the life-cycle of the
     * directive:
     * * `$onInit()` - Called on each controller after all the controllers on an element have been constructed and
     *   had their bindings initialized (and before the pre &amp; post linking functions for the directives on
     *   this element). This is a good place to put initialization code for your controller.
     * * `$onChanges(changesObj)` - Called whenever one-way (`<`) or interpolation (`@`) bindings are updated. The
     *   `changesObj` is a hash whose keys are the names of the bound properties that have changed, and the values are an
     *   object of the form `{ currentValue, previousValue, isFirstChange() }`. Use this hook to trigger updates within a
     *   component such as cloning the bound value to prevent accidental mutation of the outer value. Note that this will
     *   also be called when your bindings are initialized.
     * * `$doCheck()` - Called on each turn of the digest cycle. Provides an opportunity to detect and act on
     *   changes. Any actions that you wish to take in response to the changes that you detect must be
     *   invoked from this hook; implementing this has no effect on when `$onChanges` is called. For example, this hook
     *   could be useful if you wish to perform a deep equality check, or to check a Date object, changes to which would not
     *   be detected by Angular's change detector and thus not trigger `$onChanges`. This hook is invoked with no arguments;
     *   if detecting changes, you must store the previous value(s) for comparison to the current values.
     * * `$onDestroy()` - Called on a controller when its containing scope is destroyed. Use this hook for releasing
     *   external resources, watches and event handlers. Note that components have their `$onDestroy()` hooks called in
     *   the same order as the `$scope.$broadcast` events are triggered, which is top down. This means that parent
     *   components will have their `$onDestroy()` hook called before child components.
     * * `$postLink()` - Called after this controller's element and its children have been linked. Similar to the post-link
     *   function this hook can be used to set up DOM event handlers and do direct DOM manipulation.
     *   Note that child elements that contain `templateUrl` directives will not have been compiled and linked since
     *   they are waiting for their template to load asynchronously and their own compilation and linking has been
     *   suspended until that occurs.
     *
     * #### Comparison with Angular 2 life-cycle hooks
     * Angular 2 also uses life-cycle hooks for its components. While the Angular 1 life-cycle hooks are similar there are
     * some differences that you should be aware of, especially when it comes to moving your code from Angular 1 to Angular 2:
     *
     * * Angular 1 hooks are prefixed with `$`, such as `$onInit`. Angular 2 hooks are prefixed with `ng`, such as `ngOnInit`.
     * * Angular 1 hooks can be defined on the controller prototype or added to the controller inside its constructor.
     *   In Angular 2 you can only define hooks on the prototype of the Component class.
     * * Due to the differences in change-detection, you may get many more calls to `$doCheck` in Angular 1 than you would to
     *   `ngDoCheck` in Angular 2
     * * Changes to the model inside `$doCheck` will trigger new turns of the digest loop, which will cause the changes to be
     *   propagated throughout the application.
     *   Angular 2 does not allow the `ngDoCheck` hook to trigger a change outside of the component. It will either throw an
     *   error or do nothing depending upon the state of `enableProdMode()`.
     *
     * #### Life-cycle hook examples
     *
     * This example shows how you can check for mutations to a Date object even though the identity of the object
     * has not changed.
     *
     * <example name="doCheckDateExample" module="do-check-module">
     *   <file name="app.js">
     *     angular.module('do-check-module', [])
     *       .component('app', {
     *         template:
     *           'Month: <input ng-model="$ctrl.month" ng-change="$ctrl.updateDate()">' +
     *           'Date: {{ $ctrl.date }}' +
     *           '<test date="$ctrl.date"></test>',
     *         controller: function() {
     *           this.date = new Date();
     *           this.month = this.date.getMonth();
     *           this.updateDate = function() {
     *             this.date.setMonth(this.month);
     *           };
     *         }
     *       })
     *       .component('test', {
     *         bindings: { date: '<' },
     *         template:
     *           '<pre>{{ $ctrl.log | json }}</pre>',
     *         controller: function() {
     *           var previousValue;
     *           this.log = [];
     *           this.$doCheck = function() {
     *             var currentValue = this.date && this.date.valueOf();
     *             if (previousValue !== currentValue) {
     *               this.log.push('doCheck: date mutated: ' + this.date);
     *               previousValue = currentValue;
     *             }
     *           };
     *         }
     *       });
     *   </file>
     *   <file name="index.html">
     *     <app></app>
     *   </file>
     * </example>
     *
     * This example show how you might use `$doCheck` to trigger changes in your component's inputs even if the
     * actual identity of the component doesn't change. (Be aware that cloning and deep equality checks on large
     * arrays or objects can have a negative impact on your application performance)
     *
     * <example name="doCheckArrayExample" module="do-check-module">
     *   <file name="index.html">
     *     <div ng-init="items = []">
     *       <button ng-click="items.push(items.length)">Add Item</button>
     *       <button ng-click="items = []">Reset Items</button>
     *       <pre>{{ items }}</pre>
     *       <test items="items"></test>
     *     </div>
     *   </file>
     *   <file name="app.js">
     *      angular.module('do-check-module', [])
     *        .component('test', {
     *          bindings: { items: '<' },
     *          template:
     *            '<pre>{{ $ctrl.log | json }}</pre>',
     *          controller: function() {
     *            this.log = [];
     *
     *            this.$doCheck = function() {
     *              if (this.items_ref !== this.items) {
     *                this.log.push('doCheck: items changed');
     *                this.items_ref = this.items;
     *              }
     *              if (!angular.equals(this.items_clone, this.items)) {
     *                this.log.push('doCheck: items mutated');
     *                this.items_clone = angular.copy(this.items);
     *              }
     *            };
     *          }
     *        });
     *   </file>
     * </example>
     *
     *
     * ### Directive Definition Object
     *
     * The directive definition object provides instructions to the {@link ng.$compile
     * compiler}. The attributes are:
     *
     * #### `multiElement`
     * When this property is set to true (default is `false`), the HTML compiler will collect DOM nodes between
     * nodes with the attributes `directive-name-start` and `directive-name-end`, and group them
     * together as the directive elements. It is recommended that this feature be used on directives
     * which are not strictly behavioral (such as {@link ngClick}), and which
     * do not manipulate or replace child nodes (such as {@link ngInclude}).
     *
     * #### `priority`
     * When there are multiple directives defined on a single DOM element, sometimes it
     * is necessary to specify the order in which the directives are applied. The `priority` is used
     * to sort the directives before their `compile` functions get called. Priority is defined as a
     * number. Directives with greater numerical `priority` are compiled first. Pre-link functions
     * are also run in priority order, but post-link functions are run in reverse order. The order
     * of directives with the same priority is undefined. The default priority is `0`.
     *
     * #### `terminal`
     * If set to true then the current `priority` will be the last set of directives
     * which will execute (any directives at the current priority will still execute
     * as the order of execution on same `priority` is undefined). Note that expressions
     * and other directives used in the directive's template will also be excluded from execution.
     *
     * #### `scope`
     * The scope property can be `false`, `true`, or an object:
     *
     * * **`false` (default):** No scope will be created for the directive. The directive will use its
     * parent's scope.
     *
     * * **`true`:** A new child scope that prototypically inherits from its parent will be created for
     * the directive's element. If multiple directives on the same element request a new scope,
     * only one new scope is created.
     *
     * * **`{...}` (an object hash):** A new "isolate" scope is created for the directive's template.
     * The 'isolate' scope differs from normal scope in that it does not prototypically
     * inherit from its parent scope. This is useful when creating reusable components, which should not
     * accidentally read or modify data in the parent scope. Note that an isolate scope
     * directive without a `template` or `templateUrl` will not apply the isolate scope
     * to its children elements.
     *
     * The 'isolate' scope object hash defines a set of local scope properties derived from attributes on the
     * directive's element. These local properties are useful for aliasing values for templates. The keys in
     * the object hash map to the name of the property on the isolate scope; the values define how the property
     * is bound to the parent scope, via matching attributes on the directive's element:
     *
     * * `@` or `@attr` - bind a local scope property to the value of DOM attribute. The result is
     *   always a string since DOM attributes are strings. If no `attr` name is specified then the
     *   attribute name is assumed to be the same as the local name. Given `<my-component
     *   my-attr="hello {{name}}">` and the isolate scope definition `scope: { localName:'@myAttr' }`,
     *   the directive's scope property `localName` will reflect the interpolated value of `hello
     *   {{name}}`. As the `name` attribute changes so will the `localName` property on the directive's
     *   scope. The `name` is read from the parent scope (not the directive's scope).
     *
     * * `=` or `=attr` - set up a bidirectional binding between a local scope property and an expression
     *   passed via the attribute `attr`. The expression is evaluated in the context of the parent scope.
     *   If no `attr` name is specified then the attribute name is assumed to be the same as the local
     *   name. Given `<my-component my-attr="parentModel">` and the isolate scope definition `scope: {
     *   localModel: '=myAttr' }`, the property `localModel` on the directive's scope will reflect the
     *   value of `parentModel` on the parent scope. Changes to `parentModel` will be reflected in
     *   `localModel` and vice versa. Optional attributes should be marked as such with a question mark:
     *   `=?` or `=?attr`. If the binding expression is non-assignable, or if the attribute isn't
     *   optional and doesn't exist, an exception ({@link error/$compile/nonassign `$compile:nonassign`})
     *   will be thrown upon discovering changes to the local value, since it will be impossible to sync
     *   them back to the parent scope. By default, the {@link ng.$rootScope.Scope#$watch `$watch`}
     *   method is used for tracking changes, and the equality check is based on object identity.
     *   However, if an object literal or an array literal is passed as the binding expression, the
     *   equality check is done by value (using the {@link angular.equals} function). It's also possible
     *   to watch the evaluated value shallowly with {@link ng.$rootScope.Scope#$watchCollection
     *   `$watchCollection`}: use `=*` or `=*attr` (`=*?` or `=*?attr` if the attribute is optional).
     *
      * * `<` or `<attr` - set up a one-way (one-directional) binding between a local scope property and an
     *   expression passed via the attribute `attr`. The expression is evaluated in the context of the
     *   parent scope. If no `attr` name is specified then the attribute name is assumed to be the same as the
     *   local name. You can also make the binding optional by adding `?`: `<?` or `<?attr`.
     *
     *   For example, given `<my-component my-attr="parentModel">` and directive definition of
     *   `scope: { localModel:'<myAttr' }`, then the isolated scope property `localModel` will reflect the
     *   value of `parentModel` on the parent scope. Any changes to `parentModel` will be reflected
     *   in `localModel`, but changes in `localModel` will not reflect in `parentModel`. There are however
     *   two caveats:
     *     1. one-way binding does not copy the value from the parent to the isolate scope, it simply
     *     sets the same value. That means if your bound value is an object, changes to its properties
     *     in the isolated scope will be reflected in the parent scope (because both reference the same object).
     *     2. one-way binding watches changes to the **identity** of the parent value. That means the
     *     {@link ng.$rootScope.Scope#$watch `$watch`} on the parent value only fires if the reference
     *     to the value has changed. In most cases, this should not be of concern, but can be important
     *     to know if you one-way bind to an object, and then replace that object in the isolated scope.
     *     If you now change a property of the object in your parent scope, the change will not be
     *     propagated to the isolated scope, because the identity of the object on the parent scope
     *     has not changed. Instead you must assign a new object.
     *
     *   One-way binding is useful if you do not plan to propagate changes to your isolated scope bindings
     *   back to the parent. However, it does not make this completely impossible.
     *
     * * `&` or `&attr` - provides a way to execute an expression in the context of the parent scope. If
     *   no `attr` name is specified then the attribute name is assumed to be the same as the local name.
     *   Given `<my-component my-attr="count = count + value">` and the isolate scope definition `scope: {
     *   localFn:'&myAttr' }`, the isolate scope property `localFn` will point to a function wrapper for
     *   the `count = count + value` expression. Often it's desirable to pass data from the isolated scope
     *   via an expression to the parent scope. This can be done by passing a map of local variable names
     *   and values into the expression wrapper fn. For example, if the expression is `increment(amount)`
     *   then we can specify the amount value by calling the `localFn` as `localFn({amount: 22})`.
     *
     * In general it's possible to apply more than one directive to one element, but there might be limitations
     * depending on the type of scope required by the directives. The following points will help explain these limitations.
     * For simplicity only two directives are taken into account, but it is also applicable for several directives:
     *
     * * **no scope** + **no scope** => Two directives which don't require their own scope will use their parent's scope
     * * **child scope** + **no scope** =>  Both directives will share one single child scope
     * * **child scope** + **child scope** =>  Both directives will share one single child scope
     * * **isolated scope** + **no scope** =>  The isolated directive will use it's own created isolated scope. The other directive will use
     * its parent's scope
     * * **isolated scope** + **child scope** =>  **Won't work!** Only one scope can be related to one element. Therefore these directives cannot
     * be applied to the same element.
     * * **isolated scope** + **isolated scope**  =>  **Won't work!** Only one scope can be related to one element. Therefore these directives
     * cannot be applied to the same element.
     *
     *
     * #### `bindToController`
     * This property is used to bind scope properties directly to the controller. It can be either
     * `true` or an object hash with the same format as the `scope` property.
     *
     * When an isolate scope is used for a directive (see above), `bindToController: true` will
     * allow a component to have its properties bound to the controller, rather than to scope.
     *
     * After the controller is instantiated, the initial values of the isolate scope bindings will be bound to the controller
     * properties. You can access these bindings once they have been initialized by providing a controller method called
     * `$onInit`, which is called after all the controllers on an element have been constructed and had their bindings
     * initialized.
     *
     * <div class="alert alert-warning">
     * **Deprecation warning:** although bindings for non-ES6 class controllers are currently
     * bound to `this` before the controller constructor is called, this use is now deprecated. Please place initialization
     * code that relies upon bindings inside a `$onInit` method on the controller, instead.
     * </div>
     *
     * It is also possible to set `bindToController` to an object hash with the same format as the `scope` property.
     * This will set up the scope bindings to the controller directly. Note that `scope` can still be used
     * to define which kind of scope is created. By default, no scope is created. Use `scope: {}` to create an isolate
     * scope (useful for component directives).
     *
     * If both `bindToController` and `scope` are defined and have object hashes, `bindToController` overrides `scope`.
     *
     *
     * #### `controller`
     * Controller constructor function. The controller is instantiated before the
     * pre-linking phase and can be accessed by other directives (see
     * `require` attribute). This allows the directives to communicate with each other and augment
     * each other's behavior. The controller is injectable (and supports bracket notation) with the following locals:
     *
     * * `$scope` - Current scope associated with the element
     * * `$element` - Current element
     * * `$attrs` - Current attributes object for the element
     * * `$transclude` - A transclude linking function pre-bound to the correct transclusion scope:
     *   `function([scope], cloneLinkingFn, futureParentElement, slotName)`:
     *    * `scope`: (optional) override the scope.
     *    * `cloneLinkingFn`: (optional) argument to create clones of the original transcluded content.
     *    * `futureParentElement` (optional):
     *        * defines the parent to which the `cloneLinkingFn` will add the cloned elements.
     *        * default: `$element.parent()` resp. `$element` for `transclude:'element'` resp. `transclude:true`.
     *        * only needed for transcludes that are allowed to contain non html elements (e.g. SVG elements)
     *          and when the `cloneLinkingFn` is passed,
     *          as those elements need to created and cloned in a special way when they are defined outside their
     *          usual containers (e.g. like `<svg>`).
     *        * See also the `directive.templateNamespace` property.
     *    * `slotName`: (optional) the name of the slot to transclude. If falsy (e.g. `null`, `undefined` or `''`)
     *      then the default transclusion is provided.
     *    The `$transclude` function also has a method on it, `$transclude.isSlotFilled(slotName)`, which returns
     *    `true` if the specified slot contains content (i.e. one or more DOM nodes).
     *
     * #### `require`
     * Require another directive and inject its controller as the fourth argument to the linking function. The
     * `require` property can be a string, an array or an object:
     * * a **string** containing the name of the directive to pass to the linking function
     * * an **array** containing the names of directives to pass to the linking function. The argument passed to the
     * linking function will be an array of controllers in the same order as the names in the `require` property
     * * an **object** whose property values are the names of the directives to pass to the linking function. The argument
     * passed to the linking function will also be an object with matching keys, whose values will hold the corresponding
     * controllers.
     *
     * If the `require` property is an object and `bindToController` is truthy, then the required controllers are
     * bound to the controller using the keys of the `require` property. This binding occurs after all the controllers
     * have been constructed but before `$onInit` is called.
     * If the name of the required controller is the same as the local name (the key), the name can be
     * omitted. For example, `{parentDir: '^^'}` is equivalent to `{parentDir: '^^parentDir'}`.
     * See the {@link $compileProvider#component} helper for an example of how this can be used.
     * If no such required directive(s) can be found, or if the directive does not have a controller, then an error is
     * raised (unless no link function is specified and the required controllers are not being bound to the directive
     * controller, in which case error checking is skipped). The name can be prefixed with:
     *
     * * (no prefix) - Locate the required controller on the current element. Throw an error if not found.
     * * `?` - Attempt to locate the required controller or pass `null` to the `link` fn if not found.
     * * `^` - Locate the required controller by searching the element and its parents. Throw an error if not found.
     * * `^^` - Locate the required controller by searching the element's parents. Throw an error if not found.
     * * `?^` - Attempt to locate the required controller by searching the element and its parents or pass
     *   `null` to the `link` fn if not found.
     * * `?^^` - Attempt to locate the required controller by searching the element's parents, or pass
     *   `null` to the `link` fn if not found.
     *
     *
     * #### `controllerAs`
     * Identifier name for a reference to the controller in the directive's scope.
     * This allows the controller to be referenced from the directive template. This is especially
     * useful when a directive is used as component, i.e. with an `isolate` scope. It's also possible
     * to use it in a directive without an `isolate` / `new` scope, but you need to be aware that the
     * `controllerAs` reference might overwrite a property that already exists on the parent scope.
     *
     *
     * #### `restrict`
     * String of subset of `EACM` which restricts the directive to a specific directive
     * declaration style. If omitted, the defaults (elements and attributes) are used.
     *
     * * `E` - Element name (default): `<my-directive></my-directive>`
     * * `A` - Attribute (default): `<div my-directive="exp"></div>`
     * * `C` - Class: `<div class="my-directive: exp;"></div>`
     * * `M` - Comment: `<!-- directive: my-directive exp -->`
     *
     *
     * #### `templateNamespace`
     * String representing the document type used by the markup in the template.
     * AngularJS needs this information as those elements need to be created and cloned
     * in a special way when they are defined outside their usual containers like `<svg>` and `<math>`.
     *
     * * `html` - All root nodes in the template are HTML. Root nodes may also be
     *   top-level elements such as `<svg>` or `<math>`.
     * * `svg` - The root nodes in the template are SVG elements (excluding `<math>`).
     * * `math` - The root nodes in the template are MathML elements (excluding `<svg>`).
     *
     * If no `templateNamespace` is specified, then the namespace is considered to be `html`.
     *
     * #### `template`
     * HTML markup that may:
     * * Replace the contents of the directive's element (default).
     * * Replace the directive's element itself (if `replace` is true - DEPRECATED).
     * * Wrap the contents of the directive's element (if `transclude` is true).
     *
     * Value may be:
     *
     * * A string. For example `<div red-on-hover>{{delete_str}}</div>`.
     * * A function which takes two arguments `tElement` and `tAttrs` (described in the `compile`
     *   function api below) and returns a string value.
     *
     *
     * #### `templateUrl`
     * This is similar to `template` but the template is loaded from the specified URL, asynchronously.
     *
     * Because template loading is asynchronous the compiler will suspend compilation of directives on that element
     * for later when the template has been resolved.  In the meantime it will continue to compile and link
     * sibling and parent elements as though this element had not contained any directives.
     *
     * The compiler does not suspend the entire compilation to wait for templates to be loaded because this
     * would result in the whole app "stalling" until all templates are loaded asynchronously - even in the
     * case when only one deeply nested directive has `templateUrl`.
     *
     * Template loading is asynchronous even if the template has been preloaded into the {@link $templateCache}
     *
     * You can specify `templateUrl` as a string representing the URL or as a function which takes two
     * arguments `tElement` and `tAttrs` (described in the `compile` function api below) and returns
     * a string value representing the url.  In either case, the template URL is passed through {@link
     * $sce#getTrustedResourceUrl $sce.getTrustedResourceUrl}.
     *
     *
     * #### `replace` ([*DEPRECATED*!], will be removed in next major release - i.e. v2.0)
     * specify what the template should replace. Defaults to `false`.
     *
     * * `true` - the template will replace the directive's element.
     * * `false` - the template will replace the contents of the directive's element.
     *
     * The replacement process migrates all of the attributes / classes from the old element to the new
     * one. See the {@link guide/directive#template-expanding-directive
     * Directives Guide} for an example.
     *
     * There are very few scenarios where element replacement is required for the application function,
     * the main one being reusable custom components that are used within SVG contexts
     * (because SVG doesn't work with custom elements in the DOM tree).
     *
     * #### `transclude`
     * Extract the contents of the element where the directive appears and make it available to the directive.
     * The contents are compiled and provided to the directive as a **transclusion function**. See the
     * {@link $compile#transclusion Transclusion} section below.
     *
     *
     * #### `compile`
     *
     * ```js
     *   function compile(tElement, tAttrs, transclude) { ... }
     * ```
     *
     * The compile function deals with transforming the template DOM. Since most directives do not do
     * template transformation, it is not used often. The compile function takes the following arguments:
     *
     *   * `tElement` - template element - The element where the directive has been declared. It is
     *     safe to do template transformation on the element and child elements only.
     *
     *   * `tAttrs` - template attributes - Normalized list of attributes declared on this element shared
     *     between all directive compile functions.
     *
     *   * `transclude` -  [*DEPRECATED*!] A transclude linking function: `function(scope, cloneLinkingFn)`
     *
     * <div class="alert alert-warning">
     * **Note:** The template instance and the link instance may be different objects if the template has
     * been cloned. For this reason it is **not** safe to do anything other than DOM transformations that
     * apply to all cloned DOM nodes within the compile function. Specifically, DOM listener registration
     * should be done in a linking function rather than in a compile function.
     * </div>
    
     * <div class="alert alert-warning">
     * **Note:** The compile function cannot handle directives that recursively use themselves in their
     * own templates or compile functions. Compiling these directives results in an infinite loop and
     * stack overflow errors.
     *
     * This can be avoided by manually using $compile in the postLink function to imperatively compile
     * a directive's template instead of relying on automatic template compilation via `template` or
     * `templateUrl` declaration or manual compilation inside the compile function.
     * </div>
     *
     * <div class="alert alert-danger">
     * **Note:** The `transclude` function that is passed to the compile function is deprecated, as it
     *   e.g. does not know about the right outer scope. Please use the transclude function that is passed
     *   to the link function instead.
     * </div>
    
     * A compile function can have a return value which can be either a function or an object.
     *
     * * returning a (post-link) function - is equivalent to registering the linking function via the
     *   `link` property of the config object when the compile function is empty.
     *
     * * returning an object with function(s) registered via `pre` and `post` properties - allows you to
     *   control when a linking function should be called during the linking phase. See info about
     *   pre-linking and post-linking functions below.
     *
     *
     * #### `link`
     * This property is used only if the `compile` property is not defined.
     *
     * ```js
     *   function link(scope, iElement, iAttrs, controller, transcludeFn) { ... }
     * ```
     *
     * The link function is responsible for registering DOM listeners as well as updating the DOM. It is
     * executed after the template has been cloned. This is where most of the directive logic will be
     * put.
     *
     *   * `scope` - {@link ng.$rootScope.Scope Scope} - The scope to be used by the
     *     directive for registering {@link ng.$rootScope.Scope#$watch watches}.
     *
     *   * `iElement` - instance element - The element where the directive is to be used. It is safe to
     *     manipulate the children of the element only in `postLink` function since the children have
     *     already been linked.
     *
     *   * `iAttrs` - instance attributes - Normalized list of attributes declared on this element shared
     *     between all directive linking functions.
     *
     *   * `controller` - the directive's required controller instance(s) - Instances are shared
     *     among all directives, which allows the directives to use the controllers as a communication
     *     channel. The exact value depends on the directive's `require` property:
     *       * no controller(s) required: the directive's own controller, or `undefined` if it doesn't have one
     *       * `string`: the controller instance
     *       * `array`: array of controller instances
     *
     *     If a required controller cannot be found, and it is optional, the instance is `null`,
     *     otherwise the {@link error:$compile:ctreq Missing Required Controller} error is thrown.
     *
     *     Note that you can also require the directive's own controller - it will be made available like
     *     any other controller.
     *
     *   * `transcludeFn` - A transclude linking function pre-bound to the correct transclusion scope.
     *     This is the same as the `$transclude` parameter of directive controllers,
     *     see {@link ng.$compile#-controller- the controller section for details}.
     *     `function([scope], cloneLinkingFn, futureParentElement)`.
     *
     * #### Pre-linking function
     *
     * Executed before the child elements are linked. Not safe to do DOM transformation since the
     * compiler linking function will fail to locate the correct elements for linking.
     *
     * #### Post-linking function
     *
     * Executed after the child elements are linked.
     *
     * Note that child elements that contain `templateUrl` directives will not have been compiled
     * and linked since they are waiting for their template to load asynchronously and their own
     * compilation and linking has been suspended until that occurs.
     *
     * It is safe to do DOM transformation in the post-linking function on elements that are not waiting
     * for their async templates to be resolved.
     *
     *
     * ### Transclusion
     *
     * Transclusion is the process of extracting a collection of DOM elements from one part of the DOM and
     * copying them to another part of the DOM, while maintaining their connection to the original AngularJS
     * scope from where they were taken.
     *
     * Transclusion is used (often with {@link ngTransclude}) to insert the
     * original contents of a directive's element into a specified place in the template of the directive.
     * The benefit of transclusion, over simply moving the DOM elements manually, is that the transcluded
     * content has access to the properties on the scope from which it was taken, even if the directive
     * has isolated scope.
     * See the {@link guide/directive#creating-a-directive-that-wraps-other-elements Directives Guide}.
     *
     * This makes it possible for the widget to have private state for its template, while the transcluded
     * content has access to its originating scope.
     *
     * <div class="alert alert-warning">
     * **Note:** When testing an element transclude directive you must not place the directive at the root of the
     * DOM fragment that is being compiled. See {@link guide/unit-testing#testing-transclusion-directives
     * Testing Transclusion Directives}.
     * </div>
     *
     * There are three kinds of transclusion depending upon whether you want to transclude just the contents of the
     * directive's element, the entire element or multiple parts of the element contents:
     *
     * * `true` - transclude the content (i.e. the child nodes) of the directive's element.
     * * `'element'` - transclude the whole of the directive's element including any directives on this
     *   element that defined at a lower priority than this directive. When used, the `template`
     *   property is ignored.
     * * **`{...}` (an object hash):** - map elements of the content onto transclusion "slots" in the template.
     *
     * **Mult-slot transclusion** is declared by providing an object for the `transclude` property.
     *
     * This object is a map where the keys are the name of the slot to fill and the value is an element selector
     * used to match the HTML to the slot. The element selector should be in normalized form (e.g. `myElement`)
     * and will match the standard element variants (e.g. `my-element`, `my:element`, `data-my-element`, etc).
     *
     * For further information check out the guide on {@link guide/directive#matching-directives Matching Directives}
     *
     * If the element selector is prefixed with a `?` then that slot is optional.
     *
     * For example, the transclude object `{ slotA: '?myCustomElement' }` maps `<my-custom-element>` elements to
     * the `slotA` slot, which can be accessed via the `$transclude` function or via the {@link ngTransclude} directive.
     *
     * Slots that are not marked as optional (`?`) will trigger a compile time error if there are no matching elements
     * in the transclude content. If you wish to know if an optional slot was filled with content, then you can call
     * `$transclude.isSlotFilled(slotName)` on the transclude function passed to the directive's link function and
     * injectable into the directive's controller.
     *
     *
     * #### Transclusion Functions
     *
     * When a directive requests transclusion, the compiler extracts its contents and provides a **transclusion
     * function** to the directive's `link` function and `controller`. This transclusion function is a special
     * **linking function** that will return the compiled contents linked to a new transclusion scope.
     *
     * <div class="alert alert-info">
     * If you are just using {@link ngTransclude} then you don't need to worry about this function, since
     * ngTransclude will deal with it for us.
     * </div>
     *
     * If you want to manually control the insertion and removal of the transcluded content in your directive
     * then you must use this transclude function. When you call a transclude function it returns a a jqLite/JQuery
     * object that contains the compiled DOM, which is linked to the correct transclusion scope.
     *
     * When you call a transclusion function you can pass in a **clone attach function**. This function accepts
     * two parameters, `function(clone, scope) { ... }`, where the `clone` is a fresh compiled copy of your transcluded
     * content and the `scope` is the newly created transclusion scope, which the clone will be linked to.
     *
     * <div class="alert alert-info">
     * **Best Practice**: Always provide a `cloneFn` (clone attach function) when you call a transclude function
     * since you then get a fresh clone of the original DOM and also have access to the new transclusion scope.
     * </div>
     *
     * It is normal practice to attach your transcluded content (`clone`) to the DOM inside your **clone
     * attach function**:
     *
     * ```js
     * var transcludedContent, transclusionScope;
     *
     * $transclude(function(clone, scope) {
     *   element.append(clone);
     *   transcludedContent = clone;
     *   transclusionScope = scope;
     * });
     * ```
     *
     * Later, if you want to remove the transcluded content from your DOM then you should also destroy the
     * associated transclusion scope:
     *
     * ```js
     * transcludedContent.remove();
     * transclusionScope.$destroy();
     * ```
     *
     * <div class="alert alert-info">
     * **Best Practice**: if you intend to add and remove transcluded content manually in your directive
     * (by calling the transclude function to get the DOM and calling `element.remove()` to remove it),
     * then you are also responsible for calling `$destroy` on the transclusion scope.
     * </div>
     *
     * The built-in DOM manipulation directives, such as {@link ngIf}, {@link ngSwitch} and {@link ngRepeat}
     * automatically destroy their transcluded clones as necessary so you do not need to worry about this if
     * you are simply using {@link ngTransclude} to inject the transclusion into your directive.
     *
     *
     * #### Transclusion Scopes
     *
     * When you call a transclude function it returns a DOM fragment that is pre-bound to a **transclusion
     * scope**. This scope is special, in that it is a child of the directive's scope (and so gets destroyed
     * when the directive's scope gets destroyed) but it inherits the properties of the scope from which it
     * was taken.
     *
     * For example consider a directive that uses transclusion and isolated scope. The DOM hierarchy might look
     * like this:
     *
     * ```html
     * <div ng-app>
     *   <div isolate>
     *     <div transclusion>
     *     </div>
     *   </div>
     * </div>
     * ```
     *
     * The `$parent` scope hierarchy will look like this:
     *
       ```
       - $rootScope
         - isolate
           - transclusion
       ```
     *
     * but the scopes will inherit prototypically from different scopes to their `$parent`.
     *
       ```
       - $rootScope
         - transclusion
       - isolate
       ```
     *
     *
     * ### Attributes
     *
     * The {@link ng.$compile.directive.Attributes Attributes} object - passed as a parameter in the
     * `link()` or `compile()` functions. It has a variety of uses.
     *
     * * *Accessing normalized attribute names:* Directives like 'ngBind' can be expressed in many ways:
     *   'ng:bind', `data-ng-bind`, or 'x-ng-bind'. The attributes object allows for normalized access
     *   to the attributes.
     *
     * * *Directive inter-communication:* All directives share the same instance of the attributes
     *   object which allows the directives to use the attributes object as inter directive
     *   communication.
     *
     * * *Supports interpolation:* Interpolation attributes are assigned to the attribute object
     *   allowing other directives to read the interpolated value.
     *
     * * *Observing interpolated attributes:* Use `$observe` to observe the value changes of attributes
     *   that contain interpolation (e.g. `src="{{bar}}"`). Not only is this very efficient but it's also
     *   the only way to easily get the actual value because during the linking phase the interpolation
     *   hasn't been evaluated yet and so the value is at this time set to `undefined`.
     *
     * ```js
     * function linkingFn(scope, elm, attrs, ctrl) {
     *   // get the attribute value
     *   console.log(attrs.ngModel);
     *
     *   // change the attribute
     *   attrs.$set('ngModel', 'new value');
     *
     *   // observe changes to interpolated attribute
     *   attrs.$observe('ngModel', function(value) {
     *     console.log('ngModel has changed value to ' + value);
     *   });
     * }
     * ```
     *
     * ## Example
     *
     * <div class="alert alert-warning">
     * **Note**: Typically directives are registered with `module.directive`. The example below is
     * to illustrate how `$compile` works.
     * </div>
     *
     <example module="compileExample" name="compile">
       <file name="index.html">
        <script>
          angular.module('compileExample', [], function($compileProvider) {
            // configure new 'compile' directive by passing a directive
            // factory function. The factory function injects the '$compile'
            $compileProvider.directive('compile', function($compile) {
              // directive factory creates a link function
              return function(scope, element, attrs) {
                scope.$watch(
                  function(scope) {
                     // watch the 'compile' expression for changes
                    return scope.$eval(attrs.compile);
                  },
                  function(value) {
                    // when the 'compile' expression changes
                    // assign it into the current DOM
                    element.html(value);
    
                    // compile the new DOM and link it to the current
                    // scope.
                    // NOTE: we only compile .childNodes so that
                    // we don't get into infinite loop compiling ourselves
                    $compile(element.contents())(scope);
                  }
                );
              };
            });
          })
          .controller('GreeterController', ['$scope', function($scope) {
            $scope.name = 'Angular';
            $scope.html = 'Hello {{name}}';
          }]);
        </script>
        <div ng-controller="GreeterController">
          <input ng-model="name"> <br/>
          <textarea ng-model="html"></textarea> <br/>
          <div compile="html"></div>
        </div>
       </file>
       <file name="protractor.js" type="protractor">
         it('should auto compile', function() {
           var textarea = $('textarea');
           var output = $('div[compile]');
           // The initial state reads 'Hello Angular'.
           expect(output.getText()).toBe('Hello Angular');
           textarea.clear();
           textarea.sendKeys('{{name}}!');
           expect(output.getText()).toBe('Angular!');
         });
       </file>
     </example>
    
     *
     *
     * @param {string|DOMElement} element Element or HTML string to compile into a template function.
     * @param {function(angular.Scope, cloneAttachFn=)} transclude function available to directives - DEPRECATED.
     *
     * <div class="alert alert-danger">
     * **Note:** Passing a `transclude` function to the $compile function is deprecated, as it
     *   e.g. will not use the right outer scope. Please pass the transclude function as a
     *   `parentBoundTranscludeFn` to the link function instead.
     * </div>
     *
     * @param {number} maxPriority only apply directives lower than given priority (Only effects the
     *                 root element(s), not their children)
     * @returns {function(scope, cloneAttachFn=, options=)} a link function which is used to bind template
     * (a DOM element/tree) to a scope. Where:
     *
     *  * `scope` - A {@link ng.$rootScope.Scope Scope} to bind to.
     *  * `cloneAttachFn` - If `cloneAttachFn` is provided, then the link function will clone the
     *  `template` and call the `cloneAttachFn` function allowing the caller to attach the
     *  cloned elements to the DOM document at the appropriate place. The `cloneAttachFn` is
     *  called as: <br/> `cloneAttachFn(clonedElement, scope)` where:
     *
     *      * `clonedElement` - is a clone of the original `element` passed into the compiler.
     *      * `scope` - is the current scope with which the linking function is working with.
     *
     *  * `options` - An optional object hash with linking options. If `options` is provided, then the following
     *  keys may be used to control linking behavior:
     *
     *      * `parentBoundTranscludeFn` - the transclude function made available to
     *        directives; if given, it will be passed through to the link functions of
     *        directives found in `element` during compilation.
     *      * `transcludeControllers` - an object hash with keys that map controller names
     *        to a hash with the key `instance`, which maps to the controller instance;
     *        if given, it will make the controllers available to directives on the compileNode:
     *        ```
     *        {
     *          parent: {
     *            instance: parentControllerInstance
     *          }
     *        }
     *        ```
     *      * `futureParentElement` - defines the parent to which the `cloneAttachFn` will add
     *        the cloned elements; only needed for transcludes that are allowed to contain non html
     *        elements (e.g. SVG elements). See also the directive.controller property.
     *
     * Calling the linking function returns the element of the template. It is either the original
     * element passed in, or the clone of the element if the `cloneAttachFn` is provided.
     *
     * After linking the view is not updated until after a call to $digest which typically is done by
     * Angular automatically.
     *
     * If you need access to the bound view, there are two ways to do it:
     *
     * - If you are not asking the linking function to clone the template, create the DOM element(s)
     *   before you send them to the compiler and keep this reference around.
     *   ```js
     *     var element = $compile('<p>{{total}}</p>')(scope);
     *   ```
     *
     * - if on the other hand, you need the element to be cloned, the view reference from the original
     *   example would not point to the clone, but rather to the original template that was cloned. In
     *   this case, you can access the clone via the cloneAttachFn:
     *   ```js
     *     var templateElement = angular.element('<p>{{total}}</p>'),
     *         scope = ....;
     *
     *     var clonedElement = $compile(templateElement)(scope, function(clonedElement, scope) {
     *       //attach the clone to DOM document at the right place
     *     });
     *
     *     //now we have reference to the cloned DOM via `clonedElement`
     *   ```
     *
     *
     * For information on how the compiler works, see the
     * {@link guide/compiler Angular HTML Compiler} section of the Developer Guide.
     *
     * @knownIssue
     *
     * ### Double Compilation
     *
       Double compilation occurs when an already compiled part of the DOM gets
       compiled again. This is an undesired effect and can lead to misbehaving directives, performance issues,
       and memory leaks. Refer to the Compiler Guide {@link guide/compiler#double-compilation-and-how-to-avoid-it
       section on double compilation} for an in-depth explanation and ways to avoid it.
     *
     */

    var $compileMinErr = minErr('$compile');

    function UNINITIALIZED_VALUE() {}
    var _UNINITIALIZED_VALUE = new UNINITIALIZED_VALUE();

    /**
     * @ngdoc provider
     * @name $compileProvider
     *
     * @description
     */
    $CompileProvider.$inject = ['$provide', '$$sanitizeUriProvider'];
    /** @this */
    function $CompileProvider($provide, $$sanitizeUriProvider) {
      var hasDirectives = {},
          Suffix = 'Directive',
          COMMENT_DIRECTIVE_REGEXP = /^\s*directive:\s*([\w-]+)\s+(.*)$/,
          CLASS_DIRECTIVE_REGEXP = /(([\w-]+)(?::([^;]+))?;?)/,
          ALL_OR_NOTHING_ATTRS = makeMap('ngSrc,ngSrcset,src,srcset'),
          REQUIRE_PREFIX_REGEXP = /^(?:(\^\^?)?(\?)?(\^\^?)?)?/;

      // Ref: http://developers.whatwg.org/webappapis.html#event-handler-idl-attributes
      // The assumption is that future DOM event attribute names will begin with
      // 'on' and be composed of only English letters.
      var EVENT_HANDLER_ATTR_REGEXP = /^(on[a-z]+|formaction)$/;
      var bindingCache = createMap();

      function parseIsolateBindings(scope, directiveName, isController) {
        var LOCAL_REGEXP = /^\s*([@&<]|=(\*?))(\??)\s*([\w$]*)\s*$/;

        var bindings = createMap();

        forEach(scope, function (definition, scopeName) {
          if (definition in bindingCache) {
            bindings[scopeName] = bindingCache[definition];
            return;
          }
          var match = definition.match(LOCAL_REGEXP);

          if (!match) {
            throw $compileMinErr('iscp', 'Invalid {3} for directive \'{0}\'.' + ' Definition: {... {1}: \'{2}\' ...}', directiveName, scopeName, definition, isController ? 'controller bindings definition' : 'isolate scope definition');
          }

          bindings[scopeName] = {
            mode: match[1][0],
            collection: match[2] === '*',
            optional: match[3] === '?',
            attrName: match[4] || scopeName
          };
          if (match[4]) {
            bindingCache[definition] = bindings[scopeName];
          }
        });

        return bindings;
      }

      function parseDirectiveBindings(directive, directiveName) {
        var bindings = {
          isolateScope: null,
          bindToController: null
        };
        if (isObject(directive.scope)) {
          if (directive.bindToController === true) {
            bindings.bindToController = parseIsolateBindings(directive.scope, directiveName, true);
            bindings.isolateScope = {};
          } else {
            bindings.isolateScope = parseIsolateBindings(directive.scope, directiveName, false);
          }
        }
        if (isObject(directive.bindToController)) {
          bindings.bindToController = parseIsolateBindings(directive.bindToController, directiveName, true);
        }
        if (bindings.bindToController && !directive.controller) {
          // There is no controller
          throw $compileMinErr('noctrl', 'Cannot bind to controller without directive \'{0}\'s controller.', directiveName);
        }
        return bindings;
      }

      function assertValidDirectiveName(name) {
        var letter = name.charAt(0);
        if (!letter || letter !== lowercase(letter)) {
          throw $compileMinErr('baddir', 'Directive/Component name \'{0}\' is invalid. The first character must be a lowercase letter', name);
        }
        if (name !== name.trim()) {
          throw $compileMinErr('baddir', 'Directive/Component name \'{0}\' is invalid. The name should not contain leading or trailing whitespaces', name);
        }
      }

      function getDirectiveRequire(directive) {
        var require = directive.require || directive.controller && directive.name;

        if (!isArray(require) && isObject(require)) {
          forEach(require, function (value, key) {
            var match = value.match(REQUIRE_PREFIX_REGEXP);
            var name = value.substring(match[0].length);
            if (!name) require[key] = match[0] + key;
          });
        }

        return require;
      }

      function getDirectiveRestrict(restrict, name) {
        if (restrict && !(isString(restrict) && /[EACM]/.test(restrict))) {
          throw $compileMinErr('badrestrict', 'Restrict property \'{0}\' of directive \'{1}\' is invalid', restrict, name);
        }

        return restrict || 'EA';
      }

      /**
       * @ngdoc method
       * @name $compileProvider#directive
       * @kind function
       *
       * @description
       * Register a new directive with the compiler.
       *
       * @param {string|Object} name Name of the directive in camel-case (i.e. <code>ngBind</code> which
       *    will match as <code>ng-bind</code>), or an object map of directives where the keys are the
       *    names and the values are the factories.
       * @param {Function|Array} directiveFactory An injectable directive factory function. See the
       *    {@link guide/directive directive guide} and the {@link $compile compile API} for more info.
       * @returns {ng.$compileProvider} Self for chaining.
       */
      this.directive = function registerDirective(name, directiveFactory) {
        assertArg(name, 'name');
        assertNotHasOwnProperty(name, 'directive');
        if (isString(name)) {
          assertValidDirectiveName(name);
          assertArg(directiveFactory, 'directiveFactory');
          if (!hasDirectives.hasOwnProperty(name)) {
            hasDirectives[name] = [];
            $provide.factory(name + Suffix, ['$injector', '$exceptionHandler', function ($injector, $exceptionHandler) {
              var directives = [];
              forEach(hasDirectives[name], function (directiveFactory, index) {
                try {
                  var directive = $injector.invoke(directiveFactory);
                  if (isFunction(directive)) {
                    directive = { compile: valueFn(directive) };
                  } else if (!directive.compile && directive.link) {
                    directive.compile = valueFn(directive.link);
                  }
                  directive.priority = directive.priority || 0;
                  directive.index = index;
                  directive.name = directive.name || name;
                  directive.require = getDirectiveRequire(directive);
                  directive.restrict = getDirectiveRestrict(directive.restrict, name);
                  directive.$$moduleName = directiveFactory.$$moduleName;
                  directives.push(directive);
                } catch (e) {
                  $exceptionHandler(e);
                }
              });
              return directives;
            }]);
          }
          hasDirectives[name].push(directiveFactory);
        } else {
          forEach(name, reverseParams(registerDirective));
        }
        return this;
      };

      /**
       * @ngdoc method
       * @name $compileProvider#component
       * @module ng
       * @param {string} name Name of the component in camelCase (i.e. `myComp` which will match `<my-comp>`)
       * @param {Object} options Component definition object (a simplified
       *    {@link ng.$compile#directive-definition-object directive definition object}),
       *    with the following properties (all optional):
       *
       *    - `controller` – `{(string|function()=}` – controller constructor function that should be
       *      associated with newly created scope or the name of a {@link ng.$compile#-controller-
       *      registered controller} if passed as a string. An empty `noop` function by default.
       *    - `controllerAs` – `{string=}` – identifier name for to reference the controller in the component's scope.
       *      If present, the controller will be published to scope under the `controllerAs` name.
       *      If not present, this will default to be `$ctrl`.
       *    - `template` – `{string=|function()=}` – html template as a string or a function that
       *      returns an html template as a string which should be used as the contents of this component.
       *      Empty string by default.
       *
       *      If `template` is a function, then it is {@link auto.$injector#invoke injected} with
       *      the following locals:
       *
       *      - `$element` - Current element
       *      - `$attrs` - Current attributes object for the element
       *
       *    - `templateUrl` – `{string=|function()=}` – path or function that returns a path to an html
       *      template that should be used  as the contents of this component.
       *
       *      If `templateUrl` is a function, then it is {@link auto.$injector#invoke injected} with
       *      the following locals:
       *
       *      - `$element` - Current element
       *      - `$attrs` - Current attributes object for the element
       *
       *    - `bindings` – `{object=}` – defines bindings between DOM attributes and component properties.
       *      Component properties are always bound to the component controller and not to the scope.
       *      See {@link ng.$compile#-bindtocontroller- `bindToController`}.
       *    - `transclude` – `{boolean=}` – whether {@link $compile#transclusion content transclusion} is enabled.
       *      Disabled by default.
       *    - `require` - `{Object<string, string>=}` - requires the controllers of other directives and binds them to
       *      this component's controller. The object keys specify the property names under which the required
       *      controllers (object values) will be bound. See {@link ng.$compile#-require- `require`}.
       *    - `$...` – additional properties to attach to the directive factory function and the controller
       *      constructor function. (This is used by the component router to annotate)
       *
       * @returns {ng.$compileProvider} the compile provider itself, for chaining of function calls.
       * @description
       * Register a **component definition** with the compiler. This is a shorthand for registering a special
       * type of directive, which represents a self-contained UI component in your application. Such components
       * are always isolated (i.e. `scope: {}`) and are always restricted to elements (i.e. `restrict: 'E'`).
       *
       * Component definitions are very simple and do not require as much configuration as defining general
       * directives. Component definitions usually consist only of a template and a controller backing it.
       *
       * In order to make the definition easier, components enforce best practices like use of `controllerAs`,
       * `bindToController`. They always have **isolate scope** and are restricted to elements.
       *
       * Here are a few examples of how you would usually define components:
       *
       * ```js
       *   var myMod = angular.module(...);
       *   myMod.component('myComp', {
       *     template: '<div>My name is {{$ctrl.name}}</div>',
       *     controller: function() {
       *       this.name = 'shahar';
       *     }
       *   });
       *
       *   myMod.component('myComp', {
       *     template: '<div>My name is {{$ctrl.name}}</div>',
       *     bindings: {name: '@'}
       *   });
       *
       *   myMod.component('myComp', {
       *     templateUrl: 'views/my-comp.html',
       *     controller: 'MyCtrl',
       *     controllerAs: 'ctrl',
       *     bindings: {name: '@'}
       *   });
       *
       * ```
       * For more examples, and an in-depth guide, see the {@link guide/component component guide}.
       *
       * <br />
       * See also {@link ng.$compileProvider#directive $compileProvider.directive()}.
       */
      this.component = function registerComponent(name, options) {
        var controller = options.controller || function () {};

        function factory($injector) {
          function makeInjectable(fn) {
            if (isFunction(fn) || isArray(fn)) {
              return (/** @this */function (tElement, tAttrs) {
                  return $injector.invoke(fn, this, { $element: tElement, $attrs: tAttrs });
                }
              );
            } else {
              return fn;
            }
          }

          var template = !options.template && !options.templateUrl ? '' : options.template;
          var ddo = {
            controller: controller,
            controllerAs: identifierForController(options.controller) || options.controllerAs || '$ctrl',
            template: makeInjectable(template),
            templateUrl: makeInjectable(options.templateUrl),
            transclude: options.transclude,
            scope: {},
            bindToController: options.bindings || {},
            restrict: 'E',
            require: options.require
          };

          // Copy annotations (starting with $) over to the DDO
          forEach(options, function (val, key) {
            if (key.charAt(0) === '$') ddo[key] = val;
          });

          return ddo;
        }

        // TODO(pete) remove the following `forEach` before we release 1.6.0
        // The component-router@0.2.0 looks for the annotations on the controller constructor
        // Nothing in Angular looks for annotations on the factory function but we can't remove
        // it from 1.5.x yet.

        // Copy any annotation properties (starting with $) over to the factory and controller constructor functions
        // These could be used by libraries such as the new component router
        forEach(options, function (val, key) {
          if (key.charAt(0) === '$') {
            factory[key] = val;
            // Don't try to copy over annotations to named controller
            if (isFunction(controller)) controller[key] = val;
          }
        });

        factory.$inject = ['$injector'];

        return this.directive(name, factory);
      };

      /**
       * @ngdoc method
       * @name $compileProvider#aHrefSanitizationWhitelist
       * @kind function
       *
       * @description
       * Retrieves or overrides the default regular expression that is used for whitelisting of safe
       * urls during a[href] sanitization.
       *
       * The sanitization is a security measure aimed at preventing XSS attacks via html links.
       *
       * Any url about to be assigned to a[href] via data-binding is first normalized and turned into
       * an absolute url. Afterwards, the url is matched against the `aHrefSanitizationWhitelist`
       * regular expression. If a match is found, the original url is written into the dom. Otherwise,
       * the absolute url is prefixed with `'unsafe:'` string and only then is it written into the DOM.
       *
       * @param {RegExp=} regexp New regexp to whitelist urls with.
       * @returns {RegExp|ng.$compileProvider} Current RegExp if called without value or self for
       *    chaining otherwise.
       */
      this.aHrefSanitizationWhitelist = function (regexp) {
        if (isDefined(regexp)) {
          $$sanitizeUriProvider.aHrefSanitizationWhitelist(regexp);
          return this;
        } else {
          return $$sanitizeUriProvider.aHrefSanitizationWhitelist();
        }
      };

      /**
       * @ngdoc method
       * @name $compileProvider#imgSrcSanitizationWhitelist
       * @kind function
       *
       * @description
       * Retrieves or overrides the default regular expression that is used for whitelisting of safe
       * urls during img[src] sanitization.
       *
       * The sanitization is a security measure aimed at prevent XSS attacks via html links.
       *
       * Any url about to be assigned to img[src] via data-binding is first normalized and turned into
       * an absolute url. Afterwards, the url is matched against the `imgSrcSanitizationWhitelist`
       * regular expression. If a match is found, the original url is written into the dom. Otherwise,
       * the absolute url is prefixed with `'unsafe:'` string and only then is it written into the DOM.
       *
       * @param {RegExp=} regexp New regexp to whitelist urls with.
       * @returns {RegExp|ng.$compileProvider} Current RegExp if called without value or self for
       *    chaining otherwise.
       */
      this.imgSrcSanitizationWhitelist = function (regexp) {
        if (isDefined(regexp)) {
          $$sanitizeUriProvider.imgSrcSanitizationWhitelist(regexp);
          return this;
        } else {
          return $$sanitizeUriProvider.imgSrcSanitizationWhitelist();
        }
      };

      /**
       * @ngdoc method
       * @name  $compileProvider#debugInfoEnabled
       *
       * @param {boolean=} enabled update the debugInfoEnabled state if provided, otherwise just return the
       * current debugInfoEnabled state
       * @returns {*} current value if used as getter or itself (chaining) if used as setter
       *
       * @kind function
       *
       * @description
       * Call this method to enable/disable various debug runtime information in the compiler such as adding
       * binding information and a reference to the current scope on to DOM elements.
       * If enabled, the compiler will add the following to DOM elements that have been bound to the scope
       * * `ng-binding` CSS class
       * * `$binding` data property containing an array of the binding expressions
       *
       * You may want to disable this in production for a significant performance boost. See
       * {@link guide/production#disabling-debug-data Disabling Debug Data} for more.
       *
       * The default value is true.
       */
      var debugInfoEnabled = true;
      this.debugInfoEnabled = function (enabled) {
        if (isDefined(enabled)) {
          debugInfoEnabled = enabled;
          return this;
        }
        return debugInfoEnabled;
      };

      /**
       * @ngdoc method
       * @name  $compileProvider#preAssignBindingsEnabled
       *
       * @param {boolean=} enabled update the preAssignBindingsEnabled state if provided, otherwise just return the
       * current preAssignBindingsEnabled state
       * @returns {*} current value if used as getter or itself (chaining) if used as setter
       *
       * @kind function
       *
       * @description
       * Call this method to enable/disable whether directive controllers are assigned bindings before
       * calling the controller's constructor.
       * If enabled (true), the compiler assigns the value of each of the bindings to the
       * properties of the controller object before the constructor of this object is called.
       *
       * If disabled (false), the compiler calls the constructor first before assigning bindings.
       *
       * The default value is true in Angular 1.5.x but will switch to false in Angular 1.6.x.
       */
      var preAssignBindingsEnabled = false;
      this.preAssignBindingsEnabled = function (enabled) {
        if (isDefined(enabled)) {
          preAssignBindingsEnabled = enabled;
          return this;
        }
        return preAssignBindingsEnabled;
      };

      var TTL = 10;
      /**
       * @ngdoc method
       * @name $compileProvider#onChangesTtl
       * @description
       *
       * Sets the number of times `$onChanges` hooks can trigger new changes before giving up and
       * assuming that the model is unstable.
       *
       * The current default is 10 iterations.
       *
       * In complex applications it's possible that dependencies between `$onChanges` hooks and bindings will result
       * in several iterations of calls to these hooks. However if an application needs more than the default 10
       * iterations to stabilize then you should investigate what is causing the model to continuously change during
       * the `$onChanges` hook execution.
       *
       * Increasing the TTL could have performance implications, so you should not change it without proper justification.
       *
       * @param {number} limit The number of `$onChanges` hook iterations.
       * @returns {number|object} the current limit (or `this` if called as a setter for chaining)
       */
      this.onChangesTtl = function (value) {
        if (arguments.length) {
          TTL = value;
          return this;
        }
        return TTL;
      };

      var commentDirectivesEnabledConfig = true;
      /**
       * @ngdoc method
       * @name $compileProvider#commentDirectivesEnabled
       * @description
       *
       * It indicates to the compiler
       * whether or not directives on comments should be compiled.
       * Defaults to `true`.
       *
       * Calling this function with false disables the compilation of directives
       * on comments for the whole application.
       * This results in a compilation performance gain,
       * as the compiler doesn't have to check comments when looking for directives.
       * This should however only be used if you are sure that no comment directives are used in
       * the application (including any 3rd party directives).
       *
       * @param {boolean} enabled `false` if the compiler may ignore directives on comments
       * @returns {boolean|object} the current value (or `this` if called as a setter for chaining)
       */
      this.commentDirectivesEnabled = function (value) {
        if (arguments.length) {
          commentDirectivesEnabledConfig = value;
          return this;
        }
        return commentDirectivesEnabledConfig;
      };

      var cssClassDirectivesEnabledConfig = true;
      /**
       * @ngdoc method
       * @name $compileProvider#cssClassDirectivesEnabled
       * @description
       *
       * It indicates to the compiler
       * whether or not directives on element classes should be compiled.
       * Defaults to `true`.
       *
       * Calling this function with false disables the compilation of directives
       * on element classes for the whole application.
       * This results in a compilation performance gain,
       * as the compiler doesn't have to check element classes when looking for directives.
       * This should however only be used if you are sure that no class directives are used in
       * the application (including any 3rd party directives).
       *
       * @param {boolean} enabled `false` if the compiler may ignore directives on element classes
       * @returns {boolean|object} the current value (or `this` if called as a setter for chaining)
       */
      this.cssClassDirectivesEnabled = function (value) {
        if (arguments.length) {
          cssClassDirectivesEnabledConfig = value;
          return this;
        }
        return cssClassDirectivesEnabledConfig;
      };

      this.$get = ['$injector', '$interpolate', '$exceptionHandler', '$templateRequest', '$parse', '$controller', '$rootScope', '$sce', '$animate', '$$sanitizeUri', function ($injector, $interpolate, $exceptionHandler, $templateRequest, $parse, $controller, $rootScope, $sce, $animate, $$sanitizeUri) {

        var SIMPLE_ATTR_NAME = /^\w/;
        var specialAttrHolder = window.document.createElement('div');

        var commentDirectivesEnabled = commentDirectivesEnabledConfig;
        var cssClassDirectivesEnabled = cssClassDirectivesEnabledConfig;

        var onChangesTtl = TTL;
        // The onChanges hooks should all be run together in a single digest
        // When changes occur, the call to trigger their hooks will be added to this queue
        var onChangesQueue;

        // This function is called in a $$postDigest to trigger all the onChanges hooks in a single digest
        function flushOnChangesQueue() {
          try {
            if (! --onChangesTtl) {
              // We have hit the TTL limit so reset everything
              onChangesQueue = undefined;
              throw $compileMinErr('infchng', '{0} $onChanges() iterations reached. Aborting!\n', TTL);
            }
            // We must run this hook in an apply since the $$postDigest runs outside apply
            $rootScope.$apply(function () {
              var errors = [];
              for (var i = 0, ii = onChangesQueue.length; i < ii; ++i) {
                try {
                  onChangesQueue[i]();
                } catch (e) {
                  errors.push(e);
                }
              }
              // Reset the queue to trigger a new schedule next time there is a change
              onChangesQueue = undefined;
              if (errors.length) {
                throw errors;
              }
            });
          } finally {
            onChangesTtl++;
          }
        }

        function Attributes(element, attributesToCopy) {
          if (attributesToCopy) {
            var keys = Object.keys(attributesToCopy);
            var i, l, key;

            for (i = 0, l = keys.length; i < l; i++) {
              key = keys[i];
              this[key] = attributesToCopy[key];
            }
          } else {
            this.$attr = {};
          }

          this.$$element = element;
        }

        Attributes.prototype = {
          /**
           * @ngdoc method
           * @name $compile.directive.Attributes#$normalize
           * @kind function
           *
           * @description
           * Converts an attribute name (e.g. dash/colon/underscore-delimited string, optionally prefixed with `x-` or
           * `data-`) to its normalized, camelCase form.
           *
           * Also there is special case for Moz prefix starting with upper case letter.
           *
           * For further information check out the guide on {@link guide/directive#matching-directives Matching Directives}
           *
           * @param {string} name Name to normalize
           */
          $normalize: directiveNormalize,

          /**
           * @ngdoc method
           * @name $compile.directive.Attributes#$addClass
           * @kind function
           *
           * @description
           * Adds the CSS class value specified by the classVal parameter to the element. If animations
           * are enabled then an animation will be triggered for the class addition.
           *
           * @param {string} classVal The className value that will be added to the element
           */
          $addClass: function (classVal) {
            if (classVal && classVal.length > 0) {
              $animate.addClass(this.$$element, classVal);
            }
          },

          /**
           * @ngdoc method
           * @name $compile.directive.Attributes#$removeClass
           * @kind function
           *
           * @description
           * Removes the CSS class value specified by the classVal parameter from the element. If
           * animations are enabled then an animation will be triggered for the class removal.
           *
           * @param {string} classVal The className value that will be removed from the element
           */
          $removeClass: function (classVal) {
            if (classVal && classVal.length > 0) {
              $animate.removeClass(this.$$element, classVal);
            }
          },

          /**
           * @ngdoc method
           * @name $compile.directive.Attributes#$updateClass
           * @kind function
           *
           * @description
           * Adds and removes the appropriate CSS class values to the element based on the difference
           * between the new and old CSS class values (specified as newClasses and oldClasses).
           *
           * @param {string} newClasses The current CSS className value
           * @param {string} oldClasses The former CSS className value
           */
          $updateClass: function (newClasses, oldClasses) {
            var toAdd = tokenDifference(newClasses, oldClasses);
            if (toAdd && toAdd.length) {
              $animate.addClass(this.$$element, toAdd);
            }

            var toRemove = tokenDifference(oldClasses, newClasses);
            if (toRemove && toRemove.length) {
              $animate.removeClass(this.$$element, toRemove);
            }
          },

          /**
           * Set a normalized attribute on the element in a way such that all directives
           * can share the attribute. This function properly handles boolean attributes.
           * @param {string} key Normalized key. (ie ngAttribute)
           * @param {string|boolean} value The value to set. If `null` attribute will be deleted.
           * @param {boolean=} writeAttr If false, does not write the value to DOM element attribute.
           *     Defaults to true.
           * @param {string=} attrName Optional none normalized name. Defaults to key.
           */
          $set: function (key, value, writeAttr, attrName) {
            // TODO: decide whether or not to throw an error if "class"
            //is set through this function since it may cause $updateClass to
            //become unstable.

            var node = this.$$element[0],
                booleanKey = getBooleanAttrName(node, key),
                aliasedKey = getAliasedAttrName(key),
                observer = key,
                nodeName;

            if (booleanKey) {
              this.$$element.prop(key, value);
              attrName = booleanKey;
            } else if (aliasedKey) {
              this[aliasedKey] = value;
              observer = aliasedKey;
            }

            this[key] = value;

            // translate normalized key to actual key
            if (attrName) {
              this.$attr[key] = attrName;
            } else {
              attrName = this.$attr[key];
              if (!attrName) {
                this.$attr[key] = attrName = snake_case(key, '-');
              }
            }

            nodeName = nodeName_(this.$$element);

            if (nodeName === 'a' && (key === 'href' || key === 'xlinkHref') || nodeName === 'img' && key === 'src') {
              // sanitize a[href] and img[src] values
              this[key] = value = $$sanitizeUri(value, key === 'src');
            } else if (nodeName === 'img' && key === 'srcset' && isDefined(value)) {
              // sanitize img[srcset] values
              var result = '';

              // first check if there are spaces because it's not the same pattern
              var trimmedSrcset = trim(value);
              //                (   999x   ,|   999w   ,|   ,|,   )
              var srcPattern = /(\s+\d+x\s*,|\s+\d+w\s*,|\s+,|,\s+)/;
              var pattern = /\s/.test(trimmedSrcset) ? srcPattern : /(,)/;

              // split srcset into tuple of uri and descriptor except for the last item
              var rawUris = trimmedSrcset.split(pattern);

              // for each tuples
              var nbrUrisWith2parts = Math.floor(rawUris.length / 2);
              for (var i = 0; i < nbrUrisWith2parts; i++) {
                var innerIdx = i * 2;
                // sanitize the uri
                result += $$sanitizeUri(trim(rawUris[innerIdx]), true);
                // add the descriptor
                result += ' ' + trim(rawUris[innerIdx + 1]);
              }

              // split the last item into uri and descriptor
              var lastTuple = trim(rawUris[i * 2]).split(/\s/);

              // sanitize the last uri
              result += $$sanitizeUri(trim(lastTuple[0]), true);

              // and add the last descriptor if any
              if (lastTuple.length === 2) {
                result += ' ' + trim(lastTuple[1]);
              }
              this[key] = value = result;
            }

            if (writeAttr !== false) {
              if (value === null || isUndefined(value)) {
                this.$$element.removeAttr(attrName);
              } else {
                if (SIMPLE_ATTR_NAME.test(attrName)) {
                  this.$$element.attr(attrName, value);
                } else {
                  setSpecialAttr(this.$$element[0], attrName, value);
                }
              }
            }

            // fire observers
            var $$observers = this.$$observers;
            if ($$observers) {
              forEach($$observers[observer], function (fn) {
                try {
                  fn(value);
                } catch (e) {
                  $exceptionHandler(e);
                }
              });
            }
          },

          /**
           * @ngdoc method
           * @name $compile.directive.Attributes#$observe
           * @kind function
           *
           * @description
           * Observes an interpolated attribute.
           *
           * The observer function will be invoked once during the next `$digest` following
           * compilation. The observer is then invoked whenever the interpolated value
           * changes.
           *
           * @param {string} key Normalized key. (ie ngAttribute) .
           * @param {function(interpolatedValue)} fn Function that will be called whenever
                    the interpolated value of the attribute changes.
           *        See the {@link guide/interpolation#how-text-and-attribute-bindings-work Interpolation
           *        guide} for more info.
           * @returns {function()} Returns a deregistration function for this observer.
           */
          $observe: function (key, fn) {
            var attrs = this,
                $$observers = attrs.$$observers || (attrs.$$observers = createMap()),
                listeners = $$observers[key] || ($$observers[key] = []);

            listeners.push(fn);
            $rootScope.$evalAsync(function () {
              if (!listeners.$$inter && attrs.hasOwnProperty(key) && !isUndefined(attrs[key])) {
                // no one registered attribute interpolation function, so lets call it manually
                fn(attrs[key]);
              }
            });

            return function () {
              arrayRemove(listeners, fn);
            };
          }
        };

        function setSpecialAttr(element, attrName, value) {
          // Attributes names that do not start with letters (such as `(click)`) cannot be set using `setAttribute`
          // so we have to jump through some hoops to get such an attribute
          // https://github.com/angular/angular.js/pull/13318
          specialAttrHolder.innerHTML = '<span ' + attrName + '>';
          var attributes = specialAttrHolder.firstChild.attributes;
          var attribute = attributes[0];
          // We have to remove the attribute from its container element before we can add it to the destination element
          attributes.removeNamedItem(attribute.name);
          attribute.value = value;
          element.attributes.setNamedItem(attribute);
        }

        function safeAddClass($element, className) {
          try {
            $element.addClass(className);
          } catch (e) {
            // ignore, since it means that we are trying to set class on
            // SVG element, where class name is read-only.
          }
        }

        var startSymbol = $interpolate.startSymbol(),
            endSymbol = $interpolate.endSymbol(),
            denormalizeTemplate = startSymbol === '{{' && endSymbol === '}}' ? identity : function denormalizeTemplate(template) {
          return template.replace(/\{\{/g, startSymbol).replace(/}}/g, endSymbol);
        },
            NG_ATTR_BINDING = /^ngAttr[A-Z]/;
        var MULTI_ELEMENT_DIR_RE = /^(.+)Start$/;

        compile.$$addBindingInfo = debugInfoEnabled ? function $$addBindingInfo($element, binding) {
          var bindings = $element.data('$binding') || [];

          if (isArray(binding)) {
            bindings = bindings.concat(binding);
          } else {
            bindings.push(binding);
          }

          $element.data('$binding', bindings);
        } : noop;

        compile.$$addBindingClass = debugInfoEnabled ? function $$addBindingClass($element) {
          safeAddClass($element, 'ng-binding');
        } : noop;

        compile.$$addScopeInfo = debugInfoEnabled ? function $$addScopeInfo($element, scope, isolated, noTemplate) {
          var dataName = isolated ? noTemplate ? '$isolateScopeNoTemplate' : '$isolateScope' : '$scope';
          $element.data(dataName, scope);
        } : noop;

        compile.$$addScopeClass = debugInfoEnabled ? function $$addScopeClass($element, isolated) {
          safeAddClass($element, isolated ? 'ng-isolate-scope' : 'ng-scope');
        } : noop;

        compile.$$createComment = function (directiveName, comment) {
          var content = '';
          if (debugInfoEnabled) {
            content = ' ' + (directiveName || '') + ': ';
            if (comment) content += comment + ' ';
          }
          return window.document.createComment(content);
        };

        return compile;

        //================================

        function compile($compileNodes, transcludeFn, maxPriority, ignoreDirective, previousCompileContext) {
          if (!($compileNodes instanceof jqLite)) {
            // jquery always rewraps, whereas we need to preserve the original selector so that we can
            // modify it.
            $compileNodes = jqLite($compileNodes);
          }
          var compositeLinkFn = compileNodes($compileNodes, transcludeFn, $compileNodes, maxPriority, ignoreDirective, previousCompileContext);
          compile.$$addScopeClass($compileNodes);
          var namespace = null;
          return function publicLinkFn(scope, cloneConnectFn, options) {
            if (!$compileNodes) {
              throw $compileMinErr('multilink', 'This element has already been linked.');
            }
            assertArg(scope, 'scope');

            if (previousCompileContext && previousCompileContext.needsNewScope) {
              // A parent directive did a replace and a directive on this element asked
              // for transclusion, which caused us to lose a layer of element on which
              // we could hold the new transclusion scope, so we will create it manually
              // here.
              scope = scope.$parent.$new();
            }

            options = options || {};
            var parentBoundTranscludeFn = options.parentBoundTranscludeFn,
                transcludeControllers = options.transcludeControllers,
                futureParentElement = options.futureParentElement;

            // When `parentBoundTranscludeFn` is passed, it is a
            // `controllersBoundTransclude` function (it was previously passed
            // as `transclude` to directive.link) so we must unwrap it to get
            // its `boundTranscludeFn`
            if (parentBoundTranscludeFn && parentBoundTranscludeFn.$$boundTransclude) {
              parentBoundTranscludeFn = parentBoundTranscludeFn.$$boundTransclude;
            }

            if (!namespace) {
              namespace = detectNamespaceForChildElements(futureParentElement);
            }
            var $linkNode;
            if (namespace !== 'html') {
              // When using a directive with replace:true and templateUrl the $compileNodes
              // (or a child element inside of them)
              // might change, so we need to recreate the namespace adapted compileNodes
              // for call to the link function.
              // Note: This will already clone the nodes...
              $linkNode = jqLite(wrapTemplate(namespace, jqLite('<div>').append($compileNodes).html()));
            } else if (cloneConnectFn) {
              // important!!: we must call our jqLite.clone() since the jQuery one is trying to be smart
              // and sometimes changes the structure of the DOM.
              $linkNode = JQLitePrototype.clone.call($compileNodes);
            } else {
              $linkNode = $compileNodes;
            }

            if (transcludeControllers) {
              for (var controllerName in transcludeControllers) {
                $linkNode.data('$' + controllerName + 'Controller', transcludeControllers[controllerName].instance);
              }
            }

            compile.$$addScopeInfo($linkNode, scope);

            if (cloneConnectFn) cloneConnectFn($linkNode, scope);
            if (compositeLinkFn) compositeLinkFn(scope, $linkNode, $linkNode, parentBoundTranscludeFn);

            if (!cloneConnectFn) {
              $compileNodes = compositeLinkFn = null;
            }
            return $linkNode;
          };
        }

        function detectNamespaceForChildElements(parentElement) {
          // TODO: Make this detect MathML as well...
          var node = parentElement && parentElement[0];
          if (!node) {
            return 'html';
          } else {
            return nodeName_(node) !== 'foreignobject' && toString.call(node).match(/SVG/) ? 'svg' : 'html';
          }
        }

        /**
         * Compile function matches each node in nodeList against the directives. Once all directives
         * for a particular node are collected their compile functions are executed. The compile
         * functions return values - the linking functions - are combined into a composite linking
         * function, which is the a linking function for the node.
         *
         * @param {NodeList} nodeList an array of nodes or NodeList to compile
         * @param {function(angular.Scope, cloneAttachFn=)} transcludeFn A linking function, where the
         *        scope argument is auto-generated to the new child of the transcluded parent scope.
         * @param {DOMElement=} $rootElement If the nodeList is the root of the compilation tree then
         *        the rootElement must be set the jqLite collection of the compile root. This is
         *        needed so that the jqLite collection items can be replaced with widgets.
         * @param {number=} maxPriority Max directive priority.
         * @returns {Function} A composite linking function of all of the matched directives or null.
         */
        function compileNodes(nodeList, transcludeFn, $rootElement, maxPriority, ignoreDirective, previousCompileContext) {
          var linkFns = [],

          // `nodeList` can be either an element's `.childNodes` (live NodeList)
          // or a jqLite/jQuery collection or an array
          notLiveList = isArray(nodeList) || nodeList instanceof jqLite,
              attrs,
              directives,
              nodeLinkFn,
              childNodes,
              childLinkFn,
              linkFnFound,
              nodeLinkFnFound;

          for (var i = 0; i < nodeList.length; i++) {
            attrs = new Attributes();

            // Support: IE 11 only
            // Workaround for #11781 and #14924
            if (msie === 11) {
              mergeConsecutiveTextNodes(nodeList, i, notLiveList);
            }

            // We must always refer to `nodeList[i]` hereafter,
            // since the nodes can be replaced underneath us.
            directives = collectDirectives(nodeList[i], [], attrs, i === 0 ? maxPriority : undefined, ignoreDirective);

            nodeLinkFn = directives.length ? applyDirectivesToNode(directives, nodeList[i], attrs, transcludeFn, $rootElement, null, [], [], previousCompileContext) : null;

            if (nodeLinkFn && nodeLinkFn.scope) {
              compile.$$addScopeClass(attrs.$$element);
            }

            childLinkFn = nodeLinkFn && nodeLinkFn.terminal || !(childNodes = nodeList[i].childNodes) || !childNodes.length ? null : compileNodes(childNodes, nodeLinkFn ? (nodeLinkFn.transcludeOnThisElement || !nodeLinkFn.templateOnThisElement) && nodeLinkFn.transclude : transcludeFn);

            if (nodeLinkFn || childLinkFn) {
              linkFns.push(i, nodeLinkFn, childLinkFn);
              linkFnFound = true;
              nodeLinkFnFound = nodeLinkFnFound || nodeLinkFn;
            }

            //use the previous context only for the first element in the virtual group
            previousCompileContext = null;
          }

          // return a linking function if we have found anything, null otherwise
          return linkFnFound ? compositeLinkFn : null;

          function compositeLinkFn(scope, nodeList, $rootElement, parentBoundTranscludeFn) {
            var nodeLinkFn, childLinkFn, node, childScope, i, ii, idx, childBoundTranscludeFn;
            var stableNodeList;

            if (nodeLinkFnFound) {
              // copy nodeList so that if a nodeLinkFn removes or adds an element at this DOM level our
              // offsets don't get screwed up
              var nodeListLength = nodeList.length;
              stableNodeList = new Array(nodeListLength);

              // create a sparse array by only copying the elements which have a linkFn
              for (i = 0; i < linkFns.length; i += 3) {
                idx = linkFns[i];
                stableNodeList[idx] = nodeList[idx];
              }
            } else {
              stableNodeList = nodeList;
            }

            for (i = 0, ii = linkFns.length; i < ii;) {
              node = stableNodeList[linkFns[i++]];
              nodeLinkFn = linkFns[i++];
              childLinkFn = linkFns[i++];

              if (nodeLinkFn) {
                if (nodeLinkFn.scope) {
                  childScope = scope.$new();
                  compile.$$addScopeInfo(jqLite(node), childScope);
                } else {
                  childScope = scope;
                }

                if (nodeLinkFn.transcludeOnThisElement) {
                  childBoundTranscludeFn = createBoundTranscludeFn(scope, nodeLinkFn.transclude, parentBoundTranscludeFn);
                } else if (!nodeLinkFn.templateOnThisElement && parentBoundTranscludeFn) {
                  childBoundTranscludeFn = parentBoundTranscludeFn;
                } else if (!parentBoundTranscludeFn && transcludeFn) {
                  childBoundTranscludeFn = createBoundTranscludeFn(scope, transcludeFn);
                } else {
                  childBoundTranscludeFn = null;
                }

                nodeLinkFn(childLinkFn, childScope, node, $rootElement, childBoundTranscludeFn);
              } else if (childLinkFn) {
                childLinkFn(scope, node.childNodes, undefined, parentBoundTranscludeFn);
              }
            }
          }
        }

        function mergeConsecutiveTextNodes(nodeList, idx, notLiveList) {
          var node = nodeList[idx];
          var parent = node.parentNode;
          var sibling;

          if (node.nodeType !== NODE_TYPE_TEXT) {
            return;
          }

          while (true) {
            sibling = parent ? node.nextSibling : nodeList[idx + 1];
            if (!sibling || sibling.nodeType !== NODE_TYPE_TEXT) {
              break;
            }

            node.nodeValue = node.nodeValue + sibling.nodeValue;

            if (sibling.parentNode) {
              sibling.parentNode.removeChild(sibling);
            }
            if (notLiveList && sibling === nodeList[idx + 1]) {
              nodeList.splice(idx + 1, 1);
            }
          }
        }

        function createBoundTranscludeFn(scope, transcludeFn, previousBoundTranscludeFn) {
          function boundTranscludeFn(transcludedScope, cloneFn, controllers, futureParentElement, containingScope) {

            if (!transcludedScope) {
              transcludedScope = scope.$new(false, containingScope);
              transcludedScope.$$transcluded = true;
            }

            return transcludeFn(transcludedScope, cloneFn, {
              parentBoundTranscludeFn: previousBoundTranscludeFn,
              transcludeControllers: controllers,
              futureParentElement: futureParentElement
            });
          }

          // We need  to attach the transclusion slots onto the `boundTranscludeFn`
          // so that they are available inside the `controllersBoundTransclude` function
          var boundSlots = boundTranscludeFn.$$slots = createMap();
          for (var slotName in transcludeFn.$$slots) {
            if (transcludeFn.$$slots[slotName]) {
              boundSlots[slotName] = createBoundTranscludeFn(scope, transcludeFn.$$slots[slotName], previousBoundTranscludeFn);
            } else {
              boundSlots[slotName] = null;
            }
          }

          return boundTranscludeFn;
        }

        /**
         * Looks for directives on the given node and adds them to the directive collection which is
         * sorted.
         *
         * @param node Node to search.
         * @param directives An array to which the directives are added to. This array is sorted before
         *        the function returns.
         * @param attrs The shared attrs object which is used to populate the normalized attributes.
         * @param {number=} maxPriority Max directive priority.
         */
        function collectDirectives(node, directives, attrs, maxPriority, ignoreDirective) {
          var nodeType = node.nodeType,
              attrsMap = attrs.$attr,
              match,
              nodeName,
              className;

          switch (nodeType) {
            case NODE_TYPE_ELEMENT:
              /* Element */

              nodeName = nodeName_(node);

              // use the node name: <directive>
              addDirective(directives, directiveNormalize(nodeName), 'E', maxPriority, ignoreDirective);

              // iterate over the attributes
              for (var attr, name, nName, ngAttrName, value, isNgAttr, nAttrs = node.attributes, j = 0, jj = nAttrs && nAttrs.length; j < jj; j++) {
                var attrStartName = false;
                var attrEndName = false;

                attr = nAttrs[j];
                name = attr.name;
                value = attr.value;

                // support ngAttr attribute binding
                ngAttrName = directiveNormalize(name);
                isNgAttr = NG_ATTR_BINDING.test(ngAttrName);
                if (isNgAttr) {
                  name = name.replace(PREFIX_REGEXP, '').substr(8).replace(/_(.)/g, function (match, letter) {
                    return letter.toUpperCase();
                  });
                }

                var multiElementMatch = ngAttrName.match(MULTI_ELEMENT_DIR_RE);
                if (multiElementMatch && directiveIsMultiElement(multiElementMatch[1])) {
                  attrStartName = name;
                  attrEndName = name.substr(0, name.length - 5) + 'end';
                  name = name.substr(0, name.length - 6);
                }

                nName = directiveNormalize(name.toLowerCase());
                attrsMap[nName] = name;
                if (isNgAttr || !attrs.hasOwnProperty(nName)) {
                  attrs[nName] = value;
                  if (getBooleanAttrName(node, nName)) {
                    attrs[nName] = true; // presence means true
                  }
                }
                addAttrInterpolateDirective(node, directives, value, nName, isNgAttr);
                addDirective(directives, nName, 'A', maxPriority, ignoreDirective, attrStartName, attrEndName);
              }

              if (nodeName === 'input' && node.getAttribute('type') === 'hidden') {
                // Hidden input elements can have strange behaviour when navigating back to the page
                // This tells the browser not to try to cache and reinstate previous values
                node.setAttribute('autocomplete', 'off');
              }

              // use class as directive
              if (!cssClassDirectivesEnabled) break;
              className = node.className;
              if (isObject(className)) {
                // Maybe SVGAnimatedString
                className = className.animVal;
              }
              if (isString(className) && className !== '') {
                while (match = CLASS_DIRECTIVE_REGEXP.exec(className)) {
                  nName = directiveNormalize(match[2]);
                  if (addDirective(directives, nName, 'C', maxPriority, ignoreDirective)) {
                    attrs[nName] = trim(match[3]);
                  }
                  className = className.substr(match.index + match[0].length);
                }
              }
              break;
            case NODE_TYPE_TEXT:
              /* Text Node */
              addTextInterpolateDirective(directives, node.nodeValue);
              break;
            case NODE_TYPE_COMMENT:
              /* Comment */
              if (!commentDirectivesEnabled) break;
              collectCommentDirectives(node, directives, attrs, maxPriority, ignoreDirective);
              break;
          }

          directives.sort(byPriority);
          return directives;
        }

        function collectCommentDirectives(node, directives, attrs, maxPriority, ignoreDirective) {
          // function created because of performance, try/catch disables
          // the optimization of the whole function #14848
          try {
            var match = COMMENT_DIRECTIVE_REGEXP.exec(node.nodeValue);
            if (match) {
              var nName = directiveNormalize(match[1]);
              if (addDirective(directives, nName, 'M', maxPriority, ignoreDirective)) {
                attrs[nName] = trim(match[2]);
              }
            }
          } catch (e) {
            // turns out that under some circumstances IE9 throws errors when one attempts to read
            // comment's node value.
            // Just ignore it and continue. (Can't seem to reproduce in test case.)
          }
        }

        /**
         * Given a node with a directive-start it collects all of the siblings until it finds
         * directive-end.
         * @param node
         * @param attrStart
         * @param attrEnd
         * @returns {*}
         */
        function groupScan(node, attrStart, attrEnd) {
          var nodes = [];
          var depth = 0;
          if (attrStart && node.hasAttribute && node.hasAttribute(attrStart)) {
            do {
              if (!node) {
                throw $compileMinErr('uterdir', 'Unterminated attribute, found \'{0}\' but no matching \'{1}\' found.', attrStart, attrEnd);
              }
              if (node.nodeType === NODE_TYPE_ELEMENT) {
                if (node.hasAttribute(attrStart)) depth++;
                if (node.hasAttribute(attrEnd)) depth--;
              }
              nodes.push(node);
              node = node.nextSibling;
            } while (depth > 0);
          } else {
            nodes.push(node);
          }

          return jqLite(nodes);
        }

        /**
         * Wrapper for linking function which converts normal linking function into a grouped
         * linking function.
         * @param linkFn
         * @param attrStart
         * @param attrEnd
         * @returns {Function}
         */
        function groupElementsLinkFnWrapper(linkFn, attrStart, attrEnd) {
          return function groupedElementsLink(scope, element, attrs, controllers, transcludeFn) {
            element = groupScan(element[0], attrStart, attrEnd);
            return linkFn(scope, element, attrs, controllers, transcludeFn);
          };
        }

        /**
         * A function generator that is used to support both eager and lazy compilation
         * linking function.
         * @param eager
         * @param $compileNodes
         * @param transcludeFn
         * @param maxPriority
         * @param ignoreDirective
         * @param previousCompileContext
         * @returns {Function}
         */
        function compilationGenerator(eager, $compileNodes, transcludeFn, maxPriority, ignoreDirective, previousCompileContext) {
          var compiled;

          if (eager) {
            return compile($compileNodes, transcludeFn, maxPriority, ignoreDirective, previousCompileContext);
          }
          return (/** @this */function lazyCompilation() {
              if (!compiled) {
                compiled = compile($compileNodes, transcludeFn, maxPriority, ignoreDirective, previousCompileContext);

                // Null out all of these references in order to make them eligible for garbage collection
                // since this is a potentially long lived closure
                $compileNodes = transcludeFn = previousCompileContext = null;
              }
              return compiled.apply(this, arguments);
            }
          );
        }

        /**
         * Once the directives have been collected, their compile functions are executed. This method
         * is responsible for inlining directive templates as well as terminating the application
         * of the directives if the terminal directive has been reached.
         *
         * @param {Array} directives Array of collected directives to execute their compile function.
         *        this needs to be pre-sorted by priority order.
         * @param {Node} compileNode The raw DOM node to apply the compile functions to
         * @param {Object} templateAttrs The shared attribute function
         * @param {function(angular.Scope, cloneAttachFn=)} transcludeFn A linking function, where the
         *                                                  scope argument is auto-generated to the new
         *                                                  child of the transcluded parent scope.
         * @param {JQLite} jqCollection If we are working on the root of the compile tree then this
         *                              argument has the root jqLite array so that we can replace nodes
         *                              on it.
         * @param {Object=} originalReplaceDirective An optional directive that will be ignored when
         *                                           compiling the transclusion.
         * @param {Array.<Function>} preLinkFns
         * @param {Array.<Function>} postLinkFns
         * @param {Object} previousCompileContext Context used for previous compilation of the current
         *                                        node
         * @returns {Function} linkFn
         */
        function applyDirectivesToNode(directives, compileNode, templateAttrs, transcludeFn, jqCollection, originalReplaceDirective, preLinkFns, postLinkFns, previousCompileContext) {
          previousCompileContext = previousCompileContext || {};

          var terminalPriority = -Number.MAX_VALUE,
              newScopeDirective = previousCompileContext.newScopeDirective,
              controllerDirectives = previousCompileContext.controllerDirectives,
              newIsolateScopeDirective = previousCompileContext.newIsolateScopeDirective,
              templateDirective = previousCompileContext.templateDirective,
              nonTlbTranscludeDirective = previousCompileContext.nonTlbTranscludeDirective,
              hasTranscludeDirective = false,
              hasTemplate = false,
              hasElementTranscludeDirective = previousCompileContext.hasElementTranscludeDirective,
              $compileNode = templateAttrs.$$element = jqLite(compileNode),
              directive,
              directiveName,
              $template,
              replaceDirective = originalReplaceDirective,
              childTranscludeFn = transcludeFn,
              linkFn,
              didScanForMultipleTransclusion = false,
              mightHaveMultipleTransclusionError = false,
              directiveValue;

          // executes all directives on the current element
          for (var i = 0, ii = directives.length; i < ii; i++) {
            directive = directives[i];
            var attrStart = directive.$$start;
            var attrEnd = directive.$$end;

            // collect multiblock sections
            if (attrStart) {
              $compileNode = groupScan(compileNode, attrStart, attrEnd);
            }
            $template = undefined;

            if (terminalPriority > directive.priority) {
              break; // prevent further processing of directives
            }

            directiveValue = directive.scope;

            if (directiveValue) {

              // skip the check for directives with async templates, we'll check the derived sync
              // directive when the template arrives
              if (!directive.templateUrl) {
                if (isObject(directiveValue)) {
                  // This directive is trying to add an isolated scope.
                  // Check that there is no scope of any kind already
                  assertNoDuplicate('new/isolated scope', newIsolateScopeDirective || newScopeDirective, directive, $compileNode);
                  newIsolateScopeDirective = directive;
                } else {
                  // This directive is trying to add a child scope.
                  // Check that there is no isolated scope already
                  assertNoDuplicate('new/isolated scope', newIsolateScopeDirective, directive, $compileNode);
                }
              }

              newScopeDirective = newScopeDirective || directive;
            }

            directiveName = directive.name;

            // If we encounter a condition that can result in transclusion on the directive,
            // then scan ahead in the remaining directives for others that may cause a multiple
            // transclusion error to be thrown during the compilation process.  If a matching directive
            // is found, then we know that when we encounter a transcluded directive, we need to eagerly
            // compile the `transclude` function rather than doing it lazily in order to throw
            // exceptions at the correct time
            if (!didScanForMultipleTransclusion && (directive.replace && (directive.templateUrl || directive.template) || directive.transclude && !directive.$$tlb)) {
              var candidateDirective;

              for (var scanningIndex = i + 1; candidateDirective = directives[scanningIndex++];) {
                if (candidateDirective.transclude && !candidateDirective.$$tlb || candidateDirective.replace && (candidateDirective.templateUrl || candidateDirective.template)) {
                  mightHaveMultipleTransclusionError = true;
                  break;
                }
              }

              didScanForMultipleTransclusion = true;
            }

            if (!directive.templateUrl && directive.controller) {
              controllerDirectives = controllerDirectives || createMap();
              assertNoDuplicate('\'' + directiveName + '\' controller', controllerDirectives[directiveName], directive, $compileNode);
              controllerDirectives[directiveName] = directive;
            }

            directiveValue = directive.transclude;

            if (directiveValue) {
              hasTranscludeDirective = true;

              // Special case ngIf and ngRepeat so that we don't complain about duplicate transclusion.
              // This option should only be used by directives that know how to safely handle element transclusion,
              // where the transcluded nodes are added or replaced after linking.
              if (!directive.$$tlb) {
                assertNoDuplicate('transclusion', nonTlbTranscludeDirective, directive, $compileNode);
                nonTlbTranscludeDirective = directive;
              }

              if (directiveValue === 'element') {
                hasElementTranscludeDirective = true;
                terminalPriority = directive.priority;
                $template = $compileNode;
                $compileNode = templateAttrs.$$element = jqLite(compile.$$createComment(directiveName, templateAttrs[directiveName]));
                compileNode = $compileNode[0];
                replaceWith(jqCollection, sliceArgs($template), compileNode);

                // Support: Chrome < 50
                // https://github.com/angular/angular.js/issues/14041

                // In the versions of V8 prior to Chrome 50, the document fragment that is created
                // in the `replaceWith` function is improperly garbage collected despite still
                // being referenced by the `parentNode` property of all of the child nodes.  By adding
                // a reference to the fragment via a different property, we can avoid that incorrect
                // behavior.
                // TODO: remove this line after Chrome 50 has been released
                $template[0].$$parentNode = $template[0].parentNode;

                childTranscludeFn = compilationGenerator(mightHaveMultipleTransclusionError, $template, transcludeFn, terminalPriority, replaceDirective && replaceDirective.name, {
                  // Don't pass in:
                  // - controllerDirectives - otherwise we'll create duplicates controllers
                  // - newIsolateScopeDirective or templateDirective - combining templates with
                  //   element transclusion doesn't make sense.
                  //
                  // We need only nonTlbTranscludeDirective so that we prevent putting transclusion
                  // on the same element more than once.
                  nonTlbTranscludeDirective: nonTlbTranscludeDirective
                });
              } else {

                var slots = createMap();

                if (!isObject(directiveValue)) {
                  $template = jqLite(jqLiteClone(compileNode)).contents();
                } else {

                  // We have transclusion slots,
                  // collect them up, compile them and store their transclusion functions
                  $template = [];

                  var slotMap = createMap();
                  var filledSlots = createMap();

                  // Parse the element selectors
                  forEach(directiveValue, function (elementSelector, slotName) {
                    // If an element selector starts with a ? then it is optional
                    var optional = elementSelector.charAt(0) === '?';
                    elementSelector = optional ? elementSelector.substring(1) : elementSelector;

                    slotMap[elementSelector] = slotName;

                    // We explicitly assign `null` since this implies that a slot was defined but not filled.
                    // Later when calling boundTransclusion functions with a slot name we only error if the
                    // slot is `undefined`
                    slots[slotName] = null;

                    // filledSlots contains `true` for all slots that are either optional or have been
                    // filled. This is used to check that we have not missed any required slots
                    filledSlots[slotName] = optional;
                  });

                  // Add the matching elements into their slot
                  forEach($compileNode.contents(), function (node) {
                    var slotName = slotMap[directiveNormalize(nodeName_(node))];
                    if (slotName) {
                      filledSlots[slotName] = true;
                      slots[slotName] = slots[slotName] || [];
                      slots[slotName].push(node);
                    } else {
                      $template.push(node);
                    }
                  });

                  // Check for required slots that were not filled
                  forEach(filledSlots, function (filled, slotName) {
                    if (!filled) {
                      throw $compileMinErr('reqslot', 'Required transclusion slot `{0}` was not filled.', slotName);
                    }
                  });

                  for (var slotName in slots) {
                    if (slots[slotName]) {
                      // Only define a transclusion function if the slot was filled
                      slots[slotName] = compilationGenerator(mightHaveMultipleTransclusionError, slots[slotName], transcludeFn);
                    }
                  }
                }

                $compileNode.empty(); // clear contents
                childTranscludeFn = compilationGenerator(mightHaveMultipleTransclusionError, $template, transcludeFn, undefined, undefined, { needsNewScope: directive.$$isolateScope || directive.$$newScope });
                childTranscludeFn.$$slots = slots;
              }
            }

            if (directive.template) {
              hasTemplate = true;
              assertNoDuplicate('template', templateDirective, directive, $compileNode);
              templateDirective = directive;

              directiveValue = isFunction(directive.template) ? directive.template($compileNode, templateAttrs) : directive.template;

              directiveValue = denormalizeTemplate(directiveValue);

              if (directive.replace) {
                replaceDirective = directive;
                if (jqLiteIsTextNode(directiveValue)) {
                  $template = [];
                } else {
                  $template = removeComments(wrapTemplate(directive.templateNamespace, trim(directiveValue)));
                }
                compileNode = $template[0];

                if ($template.length !== 1 || compileNode.nodeType !== NODE_TYPE_ELEMENT) {
                  throw $compileMinErr('tplrt', 'Template for directive \'{0}\' must have exactly one root element. {1}', directiveName, '');
                }

                replaceWith(jqCollection, $compileNode, compileNode);

                var newTemplateAttrs = { $attr: {} };

                // combine directives from the original node and from the template:
                // - take the array of directives for this element
                // - split it into two parts, those that already applied (processed) and those that weren't (unprocessed)
                // - collect directives from the template and sort them by priority
                // - combine directives as: processed + template + unprocessed
                var templateDirectives = collectDirectives(compileNode, [], newTemplateAttrs);
                var unprocessedDirectives = directives.splice(i + 1, directives.length - (i + 1));

                if (newIsolateScopeDirective || newScopeDirective) {
                  // The original directive caused the current element to be replaced but this element
                  // also needs to have a new scope, so we need to tell the template directives
                  // that they would need to get their scope from further up, if they require transclusion
                  markDirectiveScope(templateDirectives, newIsolateScopeDirective, newScopeDirective);
                }
                directives = directives.concat(templateDirectives).concat(unprocessedDirectives);
                mergeTemplateAttributes(templateAttrs, newTemplateAttrs);

                ii = directives.length;
              } else {
                $compileNode.html(directiveValue);
              }
            }

            if (directive.templateUrl) {
              hasTemplate = true;
              assertNoDuplicate('template', templateDirective, directive, $compileNode);
              templateDirective = directive;

              if (directive.replace) {
                replaceDirective = directive;
              }

              // eslint-disable-next-line no-func-assign
              nodeLinkFn = compileTemplateUrl(directives.splice(i, directives.length - i), $compileNode, templateAttrs, jqCollection, hasTranscludeDirective && childTranscludeFn, preLinkFns, postLinkFns, {
                controllerDirectives: controllerDirectives,
                newScopeDirective: newScopeDirective !== directive && newScopeDirective,
                newIsolateScopeDirective: newIsolateScopeDirective,
                templateDirective: templateDirective,
                nonTlbTranscludeDirective: nonTlbTranscludeDirective
              });
              ii = directives.length;
            } else if (directive.compile) {
              try {
                linkFn = directive.compile($compileNode, templateAttrs, childTranscludeFn);
                var context = directive.$$originalDirective || directive;
                if (isFunction(linkFn)) {
                  addLinkFns(null, bind(context, linkFn), attrStart, attrEnd);
                } else if (linkFn) {
                  addLinkFns(bind(context, linkFn.pre), bind(context, linkFn.post), attrStart, attrEnd);
                }
              } catch (e) {
                $exceptionHandler(e, startingTag($compileNode));
              }
            }

            if (directive.terminal) {
              nodeLinkFn.terminal = true;
              terminalPriority = Math.max(terminalPriority, directive.priority);
            }
          }

          nodeLinkFn.scope = newScopeDirective && newScopeDirective.scope === true;
          nodeLinkFn.transcludeOnThisElement = hasTranscludeDirective;
          nodeLinkFn.templateOnThisElement = hasTemplate;
          nodeLinkFn.transclude = childTranscludeFn;

          previousCompileContext.hasElementTranscludeDirective = hasElementTranscludeDirective;

          // might be normal or delayed nodeLinkFn depending on if templateUrl is present
          return nodeLinkFn;

          ////////////////////

          function addLinkFns(pre, post, attrStart, attrEnd) {
            if (pre) {
              if (attrStart) pre = groupElementsLinkFnWrapper(pre, attrStart, attrEnd);
              pre.require = directive.require;
              pre.directiveName = directiveName;
              if (newIsolateScopeDirective === directive || directive.$$isolateScope) {
                pre = cloneAndAnnotateFn(pre, { isolateScope: true });
              }
              preLinkFns.push(pre);
            }
            if (post) {
              if (attrStart) post = groupElementsLinkFnWrapper(post, attrStart, attrEnd);
              post.require = directive.require;
              post.directiveName = directiveName;
              if (newIsolateScopeDirective === directive || directive.$$isolateScope) {
                post = cloneAndAnnotateFn(post, { isolateScope: true });
              }
              postLinkFns.push(post);
            }
          }

          function nodeLinkFn(childLinkFn, scope, linkNode, $rootElement, boundTranscludeFn) {
            var i, ii, linkFn, isolateScope, controllerScope, elementControllers, transcludeFn, $element, attrs, scopeBindingInfo;

            if (compileNode === linkNode) {
              attrs = templateAttrs;
              $element = templateAttrs.$$element;
            } else {
              $element = jqLite(linkNode);
              attrs = new Attributes($element, templateAttrs);
            }

            controllerScope = scope;
            if (newIsolateScopeDirective) {
              isolateScope = scope.$new(true);
            } else if (newScopeDirective) {
              controllerScope = scope.$parent;
            }

            if (boundTranscludeFn) {
              // track `boundTranscludeFn` so it can be unwrapped if `transcludeFn`
              // is later passed as `parentBoundTranscludeFn` to `publicLinkFn`
              transcludeFn = controllersBoundTransclude;
              transcludeFn.$$boundTransclude = boundTranscludeFn;
              // expose the slots on the `$transclude` function
              transcludeFn.isSlotFilled = function (slotName) {
                return !!boundTranscludeFn.$$slots[slotName];
              };
            }

            if (controllerDirectives) {
              elementControllers = setupControllers($element, attrs, transcludeFn, controllerDirectives, isolateScope, scope, newIsolateScopeDirective);
            }

            if (newIsolateScopeDirective) {
              // Initialize isolate scope bindings for new isolate scope directive.
              compile.$$addScopeInfo($element, isolateScope, true, !(templateDirective && (templateDirective === newIsolateScopeDirective || templateDirective === newIsolateScopeDirective.$$originalDirective)));
              compile.$$addScopeClass($element, true);
              isolateScope.$$isolateBindings = newIsolateScopeDirective.$$isolateBindings;
              scopeBindingInfo = initializeDirectiveBindings(scope, attrs, isolateScope, isolateScope.$$isolateBindings, newIsolateScopeDirective);
              if (scopeBindingInfo.removeWatches) {
                isolateScope.$on('$destroy', scopeBindingInfo.removeWatches);
              }
            }

            // Initialize bindToController bindings
            for (var name in elementControllers) {
              var controllerDirective = controllerDirectives[name];
              var controller = elementControllers[name];
              var bindings = controllerDirective.$$bindings.bindToController;

              if (preAssignBindingsEnabled) {
                if (bindings) {
                  controller.bindingInfo = initializeDirectiveBindings(controllerScope, attrs, controller.instance, bindings, controllerDirective);
                } else {
                  controller.bindingInfo = {};
                }

                var controllerResult = controller();
                if (controllerResult !== controller.instance) {
                  // If the controller constructor has a return value, overwrite the instance
                  // from setupControllers
                  controller.instance = controllerResult;
                  $element.data('$' + controllerDirective.name + 'Controller', controllerResult);
                  if (controller.bindingInfo.removeWatches) {
                    controller.bindingInfo.removeWatches();
                  }
                  controller.bindingInfo = initializeDirectiveBindings(controllerScope, attrs, controller.instance, bindings, controllerDirective);
                }
              } else {
                controller.instance = controller();
                $element.data('$' + controllerDirective.name + 'Controller', controller.instance);
                controller.bindingInfo = initializeDirectiveBindings(controllerScope, attrs, controller.instance, bindings, controllerDirective);
              }
            }

            // Bind the required controllers to the controller, if `require` is an object and `bindToController` is truthy
            forEach(controllerDirectives, function (controllerDirective, name) {
              var require = controllerDirective.require;
              if (controllerDirective.bindToController && !isArray(require) && isObject(require)) {
                extend(elementControllers[name].instance, getControllers(name, require, $element, elementControllers));
              }
            });

            // Handle the init and destroy lifecycle hooks on all controllers that have them
            forEach(elementControllers, function (controller) {
              var controllerInstance = controller.instance;
              if (isFunction(controllerInstance.$onChanges)) {
                try {
                  controllerInstance.$onChanges(controller.bindingInfo.initialChanges);
                } catch (e) {
                  $exceptionHandler(e);
                }
              }
              if (isFunction(controllerInstance.$onInit)) {
                try {
                  controllerInstance.$onInit();
                } catch (e) {
                  $exceptionHandler(e);
                }
              }
              if (isFunction(controllerInstance.$doCheck)) {
                controllerScope.$watch(function () {
                  controllerInstance.$doCheck();
                });
                controllerInstance.$doCheck();
              }
              if (isFunction(controllerInstance.$onDestroy)) {
                controllerScope.$on('$destroy', function callOnDestroyHook() {
                  controllerInstance.$onDestroy();
                });
              }
            });

            // PRELINKING
            for (i = 0, ii = preLinkFns.length; i < ii; i++) {
              linkFn = preLinkFns[i];
              invokeLinkFn(linkFn, linkFn.isolateScope ? isolateScope : scope, $element, attrs, linkFn.require && getControllers(linkFn.directiveName, linkFn.require, $element, elementControllers), transcludeFn);
            }

            // RECURSION
            // We only pass the isolate scope, if the isolate directive has a template,
            // otherwise the child elements do not belong to the isolate directive.
            var scopeToChild = scope;
            if (newIsolateScopeDirective && (newIsolateScopeDirective.template || newIsolateScopeDirective.templateUrl === null)) {
              scopeToChild = isolateScope;
            }
            if (childLinkFn) {
              childLinkFn(scopeToChild, linkNode.childNodes, undefined, boundTranscludeFn);
            }

            // POSTLINKING
            for (i = postLinkFns.length - 1; i >= 0; i--) {
              linkFn = postLinkFns[i];
              invokeLinkFn(linkFn, linkFn.isolateScope ? isolateScope : scope, $element, attrs, linkFn.require && getControllers(linkFn.directiveName, linkFn.require, $element, elementControllers), transcludeFn);
            }

            // Trigger $postLink lifecycle hooks
            forEach(elementControllers, function (controller) {
              var controllerInstance = controller.instance;
              if (isFunction(controllerInstance.$postLink)) {
                controllerInstance.$postLink();
              }
            });

            // This is the function that is injected as `$transclude`.
            // Note: all arguments are optional!
            function controllersBoundTransclude(scope, cloneAttachFn, futureParentElement, slotName) {
              var transcludeControllers;
              // No scope passed in:
              if (!isScope(scope)) {
                slotName = futureParentElement;
                futureParentElement = cloneAttachFn;
                cloneAttachFn = scope;
                scope = undefined;
              }

              if (hasElementTranscludeDirective) {
                transcludeControllers = elementControllers;
              }
              if (!futureParentElement) {
                futureParentElement = hasElementTranscludeDirective ? $element.parent() : $element;
              }
              if (slotName) {
                // slotTranscludeFn can be one of three things:
                //  * a transclude function - a filled slot
                //  * `null` - an optional slot that was not filled
                //  * `undefined` - a slot that was not declared (i.e. invalid)
                var slotTranscludeFn = boundTranscludeFn.$$slots[slotName];
                if (slotTranscludeFn) {
                  return slotTranscludeFn(scope, cloneAttachFn, transcludeControllers, futureParentElement, scopeToChild);
                } else if (isUndefined(slotTranscludeFn)) {
                  throw $compileMinErr('noslot', 'No parent directive that requires a transclusion with slot name "{0}". ' + 'Element: {1}', slotName, startingTag($element));
                }
              } else {
                return boundTranscludeFn(scope, cloneAttachFn, transcludeControllers, futureParentElement, scopeToChild);
              }
            }
          }
        }

        function getControllers(directiveName, require, $element, elementControllers) {
          var value;

          if (isString(require)) {
            var match = require.match(REQUIRE_PREFIX_REGEXP);
            var name = require.substring(match[0].length);
            var inheritType = match[1] || match[3];
            var optional = match[2] === '?';

            //If only parents then start at the parent element
            if (inheritType === '^^') {
              $element = $element.parent();
              //Otherwise attempt getting the controller from elementControllers in case
              //the element is transcluded (and has no data) and to avoid .data if possible
            } else {
              value = elementControllers && elementControllers[name];
              value = value && value.instance;
            }

            if (!value) {
              var dataName = '$' + name + 'Controller';
              value = inheritType ? $element.inheritedData(dataName) : $element.data(dataName);
            }

            if (!value && !optional) {
              throw $compileMinErr('ctreq', 'Controller \'{0}\', required by directive \'{1}\', can\'t be found!', name, directiveName);
            }
          } else if (isArray(require)) {
            value = [];
            for (var i = 0, ii = require.length; i < ii; i++) {
              value[i] = getControllers(directiveName, require[i], $element, elementControllers);
            }
          } else if (isObject(require)) {
            value = {};
            forEach(require, function (controller, property) {
              value[property] = getControllers(directiveName, controller, $element, elementControllers);
            });
          }

          return value || null;
        }

        function setupControllers($element, attrs, transcludeFn, controllerDirectives, isolateScope, scope, newIsolateScopeDirective) {
          var elementControllers = createMap();
          for (var controllerKey in controllerDirectives) {
            var directive = controllerDirectives[controllerKey];
            var locals = {
              $scope: directive === newIsolateScopeDirective || directive.$$isolateScope ? isolateScope : scope,
              $element: $element,
              $attrs: attrs,
              $transclude: transcludeFn
            };

            var controller = directive.controller;
            if (controller === '@') {
              controller = attrs[directive.name];
            }

            var controllerInstance = $controller(controller, locals, true, directive.controllerAs);

            // For directives with element transclusion the element is a comment.
            // In this case .data will not attach any data.
            // Instead, we save the controllers for the element in a local hash and attach to .data
            // later, once we have the actual element.
            elementControllers[directive.name] = controllerInstance;
            $element.data('$' + directive.name + 'Controller', controllerInstance.instance);
          }
          return elementControllers;
        }

        // Depending upon the context in which a directive finds itself it might need to have a new isolated
        // or child scope created. For instance:
        // * if the directive has been pulled into a template because another directive with a higher priority
        // asked for element transclusion
        // * if the directive itself asks for transclusion but it is at the root of a template and the original
        // element was replaced. See https://github.com/angular/angular.js/issues/12936
        function markDirectiveScope(directives, isolateScope, newScope) {
          for (var j = 0, jj = directives.length; j < jj; j++) {
            directives[j] = inherit(directives[j], { $$isolateScope: isolateScope, $$newScope: newScope });
          }
        }

        /**
         * looks up the directive and decorates it with exception handling and proper parameters. We
         * call this the boundDirective.
         *
         * @param {string} name name of the directive to look up.
         * @param {string} location The directive must be found in specific format.
         *   String containing any of theses characters:
         *
         *   * `E`: element name
         *   * `A': attribute
         *   * `C`: class
         *   * `M`: comment
         * @returns {boolean} true if directive was added.
         */
        function addDirective(tDirectives, name, location, maxPriority, ignoreDirective, startAttrName, endAttrName) {
          if (name === ignoreDirective) return null;
          var match = null;
          if (hasDirectives.hasOwnProperty(name)) {
            for (var directive, directives = $injector.get(name + Suffix), i = 0, ii = directives.length; i < ii; i++) {
              directive = directives[i];
              if ((isUndefined(maxPriority) || maxPriority > directive.priority) && directive.restrict.indexOf(location) !== -1) {
                if (startAttrName) {
                  directive = inherit(directive, { $$start: startAttrName, $$end: endAttrName });
                }
                if (!directive.$$bindings) {
                  var bindings = directive.$$bindings = parseDirectiveBindings(directive, directive.name);
                  if (isObject(bindings.isolateScope)) {
                    directive.$$isolateBindings = bindings.isolateScope;
                  }
                }
                tDirectives.push(directive);
                match = directive;
              }
            }
          }
          return match;
        }

        /**
         * looks up the directive and returns true if it is a multi-element directive,
         * and therefore requires DOM nodes between -start and -end markers to be grouped
         * together.
         *
         * @param {string} name name of the directive to look up.
         * @returns true if directive was registered as multi-element.
         */
        function directiveIsMultiElement(name) {
          if (hasDirectives.hasOwnProperty(name)) {
            for (var directive, directives = $injector.get(name + Suffix), i = 0, ii = directives.length; i < ii; i++) {
              directive = directives[i];
              if (directive.multiElement) {
                return true;
              }
            }
          }
          return false;
        }

        /**
         * When the element is replaced with HTML template then the new attributes
         * on the template need to be merged with the existing attributes in the DOM.
         * The desired effect is to have both of the attributes present.
         *
         * @param {object} dst destination attributes (original DOM)
         * @param {object} src source attributes (from the directive template)
         */
        function mergeTemplateAttributes(dst, src) {
          var srcAttr = src.$attr,
              dstAttr = dst.$attr;

          // reapply the old attributes to the new element
          forEach(dst, function (value, key) {
            if (key.charAt(0) !== '$') {
              if (src[key] && src[key] !== value) {
                if (value.length) {
                  value += (key === 'style' ? ';' : ' ') + src[key];
                } else {
                  value = src[key];
                }
              }
              dst.$set(key, value, true, srcAttr[key]);
            }
          });

          // copy the new attributes on the old attrs object
          forEach(src, function (value, key) {
            // Check if we already set this attribute in the loop above.
            // `dst` will never contain hasOwnProperty as DOM parser won't let it.
            // You will get an "InvalidCharacterError: DOM Exception 5" error if you
            // have an attribute like "has-own-property" or "data-has-own-property", etc.
            if (!dst.hasOwnProperty(key) && key.charAt(0) !== '$') {
              dst[key] = value;

              if (key !== 'class' && key !== 'style') {
                dstAttr[key] = srcAttr[key];
              }
            }
          });
        }

        function compileTemplateUrl(directives, $compileNode, tAttrs, $rootElement, childTranscludeFn, preLinkFns, postLinkFns, previousCompileContext) {
          var linkQueue = [],
              afterTemplateNodeLinkFn,
              afterTemplateChildLinkFn,
              beforeTemplateCompileNode = $compileNode[0],
              origAsyncDirective = directives.shift(),
              derivedSyncDirective = inherit(origAsyncDirective, {
            templateUrl: null, transclude: null, replace: null, $$originalDirective: origAsyncDirective
          }),
              templateUrl = isFunction(origAsyncDirective.templateUrl) ? origAsyncDirective.templateUrl($compileNode, tAttrs) : origAsyncDirective.templateUrl,
              templateNamespace = origAsyncDirective.templateNamespace;

          $compileNode.empty();

          $templateRequest(templateUrl).then(function (content) {
            var compileNode, tempTemplateAttrs, $template, childBoundTranscludeFn;

            content = denormalizeTemplate(content);

            if (origAsyncDirective.replace) {
              if (jqLiteIsTextNode(content)) {
                $template = [];
              } else {
                $template = removeComments(wrapTemplate(templateNamespace, trim(content)));
              }
              compileNode = $template[0];

              if ($template.length !== 1 || compileNode.nodeType !== NODE_TYPE_ELEMENT) {
                throw $compileMinErr('tplrt', 'Template for directive \'{0}\' must have exactly one root element. {1}', origAsyncDirective.name, templateUrl);
              }

              tempTemplateAttrs = { $attr: {} };
              replaceWith($rootElement, $compileNode, compileNode);
              var templateDirectives = collectDirectives(compileNode, [], tempTemplateAttrs);

              if (isObject(origAsyncDirective.scope)) {
                // the original directive that caused the template to be loaded async required
                // an isolate scope
                markDirectiveScope(templateDirectives, true);
              }
              directives = templateDirectives.concat(directives);
              mergeTemplateAttributes(tAttrs, tempTemplateAttrs);
            } else {
              compileNode = beforeTemplateCompileNode;
              $compileNode.html(content);
            }

            directives.unshift(derivedSyncDirective);

            afterTemplateNodeLinkFn = applyDirectivesToNode(directives, compileNode, tAttrs, childTranscludeFn, $compileNode, origAsyncDirective, preLinkFns, postLinkFns, previousCompileContext);
            forEach($rootElement, function (node, i) {
              if (node === compileNode) {
                $rootElement[i] = $compileNode[0];
              }
            });
            afterTemplateChildLinkFn = compileNodes($compileNode[0].childNodes, childTranscludeFn);

            while (linkQueue.length) {
              var scope = linkQueue.shift(),
                  beforeTemplateLinkNode = linkQueue.shift(),
                  linkRootElement = linkQueue.shift(),
                  boundTranscludeFn = linkQueue.shift(),
                  linkNode = $compileNode[0];

              if (scope.$$destroyed) continue;

              if (beforeTemplateLinkNode !== beforeTemplateCompileNode) {
                var oldClasses = beforeTemplateLinkNode.className;

                if (!(previousCompileContext.hasElementTranscludeDirective && origAsyncDirective.replace)) {
                  // it was cloned therefore we have to clone as well.
                  linkNode = jqLiteClone(compileNode);
                }
                replaceWith(linkRootElement, jqLite(beforeTemplateLinkNode), linkNode);

                // Copy in CSS classes from original node
                safeAddClass(jqLite(linkNode), oldClasses);
              }
              if (afterTemplateNodeLinkFn.transcludeOnThisElement) {
                childBoundTranscludeFn = createBoundTranscludeFn(scope, afterTemplateNodeLinkFn.transclude, boundTranscludeFn);
              } else {
                childBoundTranscludeFn = boundTranscludeFn;
              }
              afterTemplateNodeLinkFn(afterTemplateChildLinkFn, scope, linkNode, $rootElement, childBoundTranscludeFn);
            }
            linkQueue = null;
          }).catch(function (error) {
            if (error instanceof Error) {
              $exceptionHandler(error);
            }
          });

          return function delayedNodeLinkFn(ignoreChildLinkFn, scope, node, rootElement, boundTranscludeFn) {
            var childBoundTranscludeFn = boundTranscludeFn;
            if (scope.$$destroyed) return;
            if (linkQueue) {
              linkQueue.push(scope, node, rootElement, childBoundTranscludeFn);
            } else {
              if (afterTemplateNodeLinkFn.transcludeOnThisElement) {
                childBoundTranscludeFn = createBoundTranscludeFn(scope, afterTemplateNodeLinkFn.transclude, boundTranscludeFn);
              }
              afterTemplateNodeLinkFn(afterTemplateChildLinkFn, scope, node, rootElement, childBoundTranscludeFn);
            }
          };
        }

        /**
         * Sorting function for bound directives.
         */
        function byPriority(a, b) {
          var diff = b.priority - a.priority;
          if (diff !== 0) return diff;
          if (a.name !== b.name) return a.name < b.name ? -1 : 1;
          return a.index - b.index;
        }

        function assertNoDuplicate(what, previousDirective, directive, element) {

          function wrapModuleNameIfDefined(moduleName) {
            return moduleName ? ' (module: ' + moduleName + ')' : '';
          }

          if (previousDirective) {
            throw $compileMinErr('multidir', 'Multiple directives [{0}{1}, {2}{3}] asking for {4} on: {5}', previousDirective.name, wrapModuleNameIfDefined(previousDirective.$$moduleName), directive.name, wrapModuleNameIfDefined(directive.$$moduleName), what, startingTag(element));
          }
        }

        function addTextInterpolateDirective(directives, text) {
          var interpolateFn = $interpolate(text, true);
          if (interpolateFn) {
            directives.push({
              priority: 0,
              compile: function textInterpolateCompileFn(templateNode) {
                var templateNodeParent = templateNode.parent(),
                    hasCompileParent = !!templateNodeParent.length;

                // When transcluding a template that has bindings in the root
                // we don't have a parent and thus need to add the class during linking fn.
                if (hasCompileParent) compile.$$addBindingClass(templateNodeParent);

                return function textInterpolateLinkFn(scope, node) {
                  var parent = node.parent();
                  if (!hasCompileParent) compile.$$addBindingClass(parent);
                  compile.$$addBindingInfo(parent, interpolateFn.expressions);
                  scope.$watch(interpolateFn, function interpolateFnWatchAction(value) {
                    node[0].nodeValue = value;
                  });
                };
              }
            });
          }
        }

        function wrapTemplate(type, template) {
          type = lowercase(type || 'html');
          switch (type) {
            case 'svg':
            case 'math':
              var wrapper = window.document.createElement('div');
              wrapper.innerHTML = '<' + type + '>' + template + '</' + type + '>';
              return wrapper.childNodes[0].childNodes;
            default:
              return template;
          }
        }

        function getTrustedContext(node, attrNormalizedName) {
          if (attrNormalizedName === 'srcdoc') {
            return $sce.HTML;
          }
          var tag = nodeName_(node);
          // All tags with src attributes require a RESOURCE_URL value, except for
          // img and various html5 media tags.
          if (attrNormalizedName === 'src' || attrNormalizedName === 'ngSrc') {
            if (['img', 'video', 'audio', 'source', 'track'].indexOf(tag) === -1) {
              return $sce.RESOURCE_URL;
            }
            // maction[xlink:href] can source SVG.  It's not limited to <maction>.
          } else if (attrNormalizedName === 'xlinkHref' || tag === 'form' && attrNormalizedName === 'action' ||
          // links can be stylesheets or imports, which can run script in the current origin
          tag === 'link' && attrNormalizedName === 'href') {
            return $sce.RESOURCE_URL;
          }
        }

        function addAttrInterpolateDirective(node, directives, value, name, isNgAttr) {
          var trustedContext = getTrustedContext(node, name);
          var mustHaveExpression = !isNgAttr;
          var allOrNothing = ALL_OR_NOTHING_ATTRS[name] || isNgAttr;

          var interpolateFn = $interpolate(value, mustHaveExpression, trustedContext, allOrNothing);

          // no interpolation found -> ignore
          if (!interpolateFn) return;

          if (name === 'multiple' && nodeName_(node) === 'select') {
            throw $compileMinErr('selmulti', 'Binding to the \'multiple\' attribute is not supported. Element: {0}', startingTag(node));
          }

          if (EVENT_HANDLER_ATTR_REGEXP.test(name)) {
            throw $compileMinErr('nodomevents', 'Interpolations for HTML DOM event attributes are disallowed.  Please use the ' + 'ng- versions (such as ng-click instead of onclick) instead.');
          }

          directives.push({
            priority: 100,
            compile: function () {
              return {
                pre: function attrInterpolatePreLinkFn(scope, element, attr) {
                  var $$observers = attr.$$observers || (attr.$$observers = createMap());

                  // If the attribute has changed since last $interpolate()ed
                  var newValue = attr[name];
                  if (newValue !== value) {
                    // we need to interpolate again since the attribute value has been updated
                    // (e.g. by another directive's compile function)
                    // ensure unset/empty values make interpolateFn falsy
                    interpolateFn = newValue && $interpolate(newValue, true, trustedContext, allOrNothing);
                    value = newValue;
                  }

                  // if attribute was updated so that there is no interpolation going on we don't want to
                  // register any observers
                  if (!interpolateFn) return;

                  // initialize attr object so that it's ready in case we need the value for isolate
                  // scope initialization, otherwise the value would not be available from isolate
                  // directive's linking fn during linking phase
                  attr[name] = interpolateFn(scope);

                  ($$observers[name] || ($$observers[name] = [])).$$inter = true;
                  (attr.$$observers && attr.$$observers[name].$$scope || scope).$watch(interpolateFn, function interpolateFnWatchAction(newValue, oldValue) {
                    //special case for class attribute addition + removal
                    //so that class changes can tap into the animation
                    //hooks provided by the $animate service. Be sure to
                    //skip animations when the first digest occurs (when
                    //both the new and the old values are the same) since
                    //the CSS classes are the non-interpolated values
                    if (name === 'class' && newValue !== oldValue) {
                      attr.$updateClass(newValue, oldValue);
                    } else {
                      attr.$set(name, newValue);
                    }
                  });
                }
              };
            }
          });
        }

        /**
         * This is a special jqLite.replaceWith, which can replace items which
         * have no parents, provided that the containing jqLite collection is provided.
         *
         * @param {JqLite=} $rootElement The root of the compile tree. Used so that we can replace nodes
         *                               in the root of the tree.
         * @param {JqLite} elementsToRemove The jqLite element which we are going to replace. We keep
         *                                  the shell, but replace its DOM node reference.
         * @param {Node} newNode The new DOM node.
         */
        function replaceWith($rootElement, elementsToRemove, newNode) {
          var firstElementToRemove = elementsToRemove[0],
              removeCount = elementsToRemove.length,
              parent = firstElementToRemove.parentNode,
              i,
              ii;

          if ($rootElement) {
            for (i = 0, ii = $rootElement.length; i < ii; i++) {
              if ($rootElement[i] === firstElementToRemove) {
                $rootElement[i++] = newNode;
                for (var j = i, j2 = j + removeCount - 1, jj = $rootElement.length; j < jj; j++, j2++) {
                  if (j2 < jj) {
                    $rootElement[j] = $rootElement[j2];
                  } else {
                    delete $rootElement[j];
                  }
                }
                $rootElement.length -= removeCount - 1;

                // If the replaced element is also the jQuery .context then replace it
                // .context is a deprecated jQuery api, so we should set it only when jQuery set it
                // http://api.jquery.com/context/
                if ($rootElement.context === firstElementToRemove) {
                  $rootElement.context = newNode;
                }
                break;
              }
            }
          }

          if (parent) {
            parent.replaceChild(newNode, firstElementToRemove);
          }

          // Append all the `elementsToRemove` to a fragment. This will...
          // - remove them from the DOM
          // - allow them to still be traversed with .nextSibling
          // - allow a single fragment.qSA to fetch all elements being removed
          var fragment = window.document.createDocumentFragment();
          for (i = 0; i < removeCount; i++) {
            fragment.appendChild(elementsToRemove[i]);
          }

          if (jqLite.hasData(firstElementToRemove)) {
            // Copy over user data (that includes Angular's $scope etc.). Don't copy private
            // data here because there's no public interface in jQuery to do that and copying over
            // event listeners (which is the main use of private data) wouldn't work anyway.
            jqLite.data(newNode, jqLite.data(firstElementToRemove));

            // Remove $destroy event listeners from `firstElementToRemove`
            jqLite(firstElementToRemove).off('$destroy');
          }

          // Cleanup any data/listeners on the elements and children.
          // This includes invoking the $destroy event on any elements with listeners.
          jqLite.cleanData(fragment.querySelectorAll('*'));

          // Update the jqLite collection to only contain the `newNode`
          for (i = 1; i < removeCount; i++) {
            delete elementsToRemove[i];
          }
          elementsToRemove[0] = newNode;
          elementsToRemove.length = 1;
        }

        function cloneAndAnnotateFn(fn, annotation) {
          return extend(function () {
            return fn.apply(null, arguments);
          }, fn, annotation);
        }

        function invokeLinkFn(linkFn, scope, $element, attrs, controllers, transcludeFn) {
          try {
            linkFn(scope, $element, attrs, controllers, transcludeFn);
          } catch (e) {
            $exceptionHandler(e, startingTag($element));
          }
        }

        // Set up $watches for isolate scope and controller bindings.
        function initializeDirectiveBindings(scope, attrs, destination, bindings, directive) {
          var removeWatchCollection = [];
          var initialChanges = {};
          var changes;
          forEach(bindings, function initializeBinding(definition, scopeName) {
            var attrName = definition.attrName,
                optional = definition.optional,
                mode = definition.mode,
                // @, =, <, or &
            lastValue,
                parentGet,
                parentSet,
                compare,
                removeWatch;

            switch (mode) {

              case '@':
                if (!optional && !hasOwnProperty.call(attrs, attrName)) {
                  destination[scopeName] = attrs[attrName] = undefined;
                }
                removeWatch = attrs.$observe(attrName, function (value) {
                  if (isString(value) || isBoolean(value)) {
                    var oldValue = destination[scopeName];
                    recordChanges(scopeName, value, oldValue);
                    destination[scopeName] = value;
                  }
                });
                attrs.$$observers[attrName].$$scope = scope;
                lastValue = attrs[attrName];
                if (isString(lastValue)) {
                  // If the attribute has been provided then we trigger an interpolation to ensure
                  // the value is there for use in the link fn
                  destination[scopeName] = $interpolate(lastValue)(scope);
                } else if (isBoolean(lastValue)) {
                  // If the attributes is one of the BOOLEAN_ATTR then Angular will have converted
                  // the value to boolean rather than a string, so we special case this situation
                  destination[scopeName] = lastValue;
                }
                initialChanges[scopeName] = new SimpleChange(_UNINITIALIZED_VALUE, destination[scopeName]);
                removeWatchCollection.push(removeWatch);
                break;

              case '=':
                if (!hasOwnProperty.call(attrs, attrName)) {
                  if (optional) break;
                  attrs[attrName] = undefined;
                }
                if (optional && !attrs[attrName]) break;

                parentGet = $parse(attrs[attrName]);
                if (parentGet.literal) {
                  compare = equals;
                } else {
                  // eslint-disable-next-line no-self-compare
                  compare = function simpleCompare(a, b) {
                    return a === b || a !== a && b !== b;
                  };
                }
                parentSet = parentGet.assign || function () {
                  // reset the change, or we will throw this exception on every $digest
                  lastValue = destination[scopeName] = parentGet(scope);
                  throw $compileMinErr('nonassign', 'Expression \'{0}\' in attribute \'{1}\' used with directive \'{2}\' is non-assignable!', attrs[attrName], attrName, directive.name);
                };
                lastValue = destination[scopeName] = parentGet(scope);
                var parentValueWatch = function parentValueWatch(parentValue) {
                  if (!compare(parentValue, destination[scopeName])) {
                    // we are out of sync and need to copy
                    if (!compare(parentValue, lastValue)) {
                      // parent changed and it has precedence
                      destination[scopeName] = parentValue;
                    } else {
                      // if the parent can be assigned then do so
                      parentSet(scope, parentValue = destination[scopeName]);
                    }
                  }
                  lastValue = parentValue;
                  return lastValue;
                };
                parentValueWatch.$stateful = true;
                if (definition.collection) {
                  removeWatch = scope.$watchCollection(attrs[attrName], parentValueWatch);
                } else {
                  removeWatch = scope.$watch($parse(attrs[attrName], parentValueWatch), null, parentGet.literal);
                }
                removeWatchCollection.push(removeWatch);
                break;

              case '<':
                if (!hasOwnProperty.call(attrs, attrName)) {
                  if (optional) break;
                  attrs[attrName] = undefined;
                }
                if (optional && !attrs[attrName]) break;

                parentGet = $parse(attrs[attrName]);
                var deepWatch = parentGet.literal;

                var initialValue = destination[scopeName] = parentGet(scope);
                initialChanges[scopeName] = new SimpleChange(_UNINITIALIZED_VALUE, destination[scopeName]);

                removeWatch = scope.$watch(parentGet, function parentValueWatchAction(newValue, oldValue) {
                  if (oldValue === newValue) {
                    if (oldValue === initialValue || deepWatch && equals(oldValue, initialValue)) {
                      return;
                    }
                    oldValue = initialValue;
                  }
                  recordChanges(scopeName, newValue, oldValue);
                  destination[scopeName] = newValue;
                }, deepWatch);

                removeWatchCollection.push(removeWatch);
                break;

              case '&':
                // Don't assign Object.prototype method to scope
                parentGet = attrs.hasOwnProperty(attrName) ? $parse(attrs[attrName]) : noop;

                // Don't assign noop to destination if expression is not valid
                if (parentGet === noop && optional) break;

                destination[scopeName] = function (locals) {
                  return parentGet(scope, locals);
                };
                break;
            }
          });

          function recordChanges(key, currentValue, previousValue) {
            if (isFunction(destination.$onChanges) && currentValue !== previousValue && (
            // eslint-disable-next-line no-self-compare
            currentValue === currentValue || previousValue === previousValue)) {
              // If we have not already scheduled the top level onChangesQueue handler then do so now
              if (!onChangesQueue) {
                scope.$$postDigest(flushOnChangesQueue);
                onChangesQueue = [];
              }
              // If we have not already queued a trigger of onChanges for this controller then do so now
              if (!changes) {
                changes = {};
                onChangesQueue.push(triggerOnChangesHook);
              }
              // If the has been a change on this property already then we need to reuse the previous value
              if (changes[key]) {
                previousValue = changes[key].previousValue;
              }
              // Store this change
              changes[key] = new SimpleChange(previousValue, currentValue);
            }
          }

          function triggerOnChangesHook() {
            destination.$onChanges(changes);
            // Now clear the changes so that we schedule onChanges when more changes arrive
            changes = undefined;
          }

          return {
            initialChanges: initialChanges,
            removeWatches: removeWatchCollection.length && function removeWatches() {
              for (var i = 0, ii = removeWatchCollection.length; i < ii; ++i) {
                removeWatchCollection[i]();
              }
            }
          };
        }
      }];
    }

    function SimpleChange(previous, current) {
      this.previousValue = previous;
      this.currentValue = current;
    }
    SimpleChange.prototype.isFirstChange = function () {
      return this.previousValue === _UNINITIALIZED_VALUE;
    };

    var PREFIX_REGEXP = /^((?:x|data)[:\-_])/i;
    var SPECIAL_CHARS_REGEXP = /[:\-_]+(.)/g;

    /**
     * Converts all accepted directives format into proper directive name.
     * @param name Name to normalize
     */
    function directiveNormalize(name) {
      return name.replace(PREFIX_REGEXP, '').replace(SPECIAL_CHARS_REGEXP, fnCamelCaseReplace);
    }

    /**
     * @ngdoc type
     * @name $compile.directive.Attributes
     *
     * @description
     * A shared object between directive compile / linking functions which contains normalized DOM
     * element attributes. The values reflect current binding state `{{ }}`. The normalization is
     * needed since all of these are treated as equivalent in Angular:
     *
     * ```
     *    <span ng:bind="a" ng-bind="a" data-ng-bind="a" x-ng-bind="a">
     * ```
     */

    /**
     * @ngdoc property
     * @name $compile.directive.Attributes#$attr
     *
     * @description
     * A map of DOM element attribute names to the normalized name. This is
     * needed to do reverse lookup from normalized name back to actual name.
     */

    /**
     * @ngdoc method
     * @name $compile.directive.Attributes#$set
     * @kind function
     *
     * @description
     * Set DOM element attribute value.
     *
     *
     * @param {string} name Normalized element attribute name of the property to modify. The name is
     *          reverse-translated using the {@link ng.$compile.directive.Attributes#$attr $attr}
     *          property to the original name.
     * @param {string} value Value to set the attribute to. The value can be an interpolated string.
     */

    /**
     * Closure compiler type information
     */

    function nodesetLinkingFn(
    /* angular.Scope */scope,
    /* NodeList */nodeList,
    /* Element */rootElement,
    /* function(Function) */boundTranscludeFn) {}

    function directiveLinkingFn(
    /* nodesetLinkingFn */nodesetLinkingFn,
    /* angular.Scope */scope,
    /* Node */node,
    /* Element */rootElement,
    /* function(Function) */boundTranscludeFn) {}

    function tokenDifference(str1, str2) {
      var values = '',
          tokens1 = str1.split(/\s+/),
          tokens2 = str2.split(/\s+/);

      outer: for (var i = 0; i < tokens1.length; i++) {
        var token = tokens1[i];
        for (var j = 0; j < tokens2.length; j++) {
          if (token === tokens2[j]) continue outer;
        }
        values += (values.length > 0 ? ' ' : '') + token;
      }
      return values;
    }

    function removeComments(jqNodes) {
      jqNodes = jqLite(jqNodes);
      var i = jqNodes.length;

      if (i <= 1) {
        return jqNodes;
      }

      while (i--) {
        var node = jqNodes[i];
        if (node.nodeType === NODE_TYPE_COMMENT || node.nodeType === NODE_TYPE_TEXT && node.nodeValue.trim() === '') {
          splice.call(jqNodes, i, 1);
        }
      }
      return jqNodes;
    }

    var $controllerMinErr = minErr('$controller');

    var CNTRL_REG = /^(\S+)(\s+as\s+([\w$]+))?$/;
    function identifierForController(controller, ident) {
      if (ident && isString(ident)) return ident;
      if (isString(controller)) {
        var match = CNTRL_REG.exec(controller);
        if (match) return match[3];
      }
    }

    /**
     * @ngdoc provider
     * @name $controllerProvider
     * @this
     *
     * @description
     * The {@link ng.$controller $controller service} is used by Angular to create new
     * controllers.
     *
     * This provider allows controller registration via the
     * {@link ng.$controllerProvider#register register} method.
     */
    function $ControllerProvider() {
      var controllers = {},
          globals = false;

      /**
       * @ngdoc method
       * @name $controllerProvider#has
       * @param {string} name Controller name to check.
       */
      this.has = function (name) {
        return controllers.hasOwnProperty(name);
      };

      /**
       * @ngdoc method
       * @name $controllerProvider#register
       * @param {string|Object} name Controller name, or an object map of controllers where the keys are
       *    the names and the values are the constructors.
       * @param {Function|Array} constructor Controller constructor fn (optionally decorated with DI
       *    annotations in the array notation).
       */
      this.register = function (name, constructor) {
        assertNotHasOwnProperty(name, 'controller');
        if (isObject(name)) {
          extend(controllers, name);
        } else {
          controllers[name] = constructor;
        }
      };

      /**
       * @ngdoc method
       * @name $controllerProvider#allowGlobals
       * @description If called, allows `$controller` to find controller constructors on `window`
       *
       * @deprecated
       * sinceVersion="v1.3.0"
       * removeVersion="v1.7.0"
       * This method of finding controllers has been deprecated.
       */
      this.allowGlobals = function () {
        globals = true;
      };

      this.$get = ['$injector', '$window', function ($injector, $window) {

        /**
         * @ngdoc service
         * @name $controller
         * @requires $injector
         *
         * @param {Function|string} constructor If called with a function then it's considered to be the
         *    controller constructor function. Otherwise it's considered to be a string which is used
         *    to retrieve the controller constructor using the following steps:
         *
         *    * check if a controller with given name is registered via `$controllerProvider`
         *    * check if evaluating the string on the current scope returns a constructor
         *    * if $controllerProvider#allowGlobals, check `window[constructor]` on the global
         *      `window` object (deprecated, not recommended)
         *
         *    The string can use the `controller as property` syntax, where the controller instance is published
         *    as the specified property on the `scope`; the `scope` must be injected into `locals` param for this
         *    to work correctly.
         *
         * @param {Object} locals Injection locals for Controller.
         * @return {Object} Instance of given controller.
         *
         * @description
         * `$controller` service is responsible for instantiating controllers.
         *
         * It's just a simple call to {@link auto.$injector $injector}, but extracted into
         * a service, so that one can override this service with [BC version](https://gist.github.com/1649788).
         */
        return function $controller(expression, locals, later, ident) {
          // PRIVATE API:
          //   param `later` --- indicates that the controller's constructor is invoked at a later time.
          //                     If true, $controller will allocate the object with the correct
          //                     prototype chain, but will not invoke the controller until a returned
          //                     callback is invoked.
          //   param `ident` --- An optional label which overrides the label parsed from the controller
          //                     expression, if any.
          var instance, match, constructor, identifier;
          later = later === true;
          if (ident && isString(ident)) {
            identifier = ident;
          }

          if (isString(expression)) {
            match = expression.match(CNTRL_REG);
            if (!match) {
              throw $controllerMinErr('ctrlfmt', 'Badly formed controller string \'{0}\'. ' + 'Must match `__name__ as __id__` or `__name__`.', expression);
            }
            constructor = match[1];
            identifier = identifier || match[3];
            expression = controllers.hasOwnProperty(constructor) ? controllers[constructor] : getter(locals.$scope, constructor, true) || (globals ? getter($window, constructor, true) : undefined);

            if (!expression) {
              throw $controllerMinErr('ctrlreg', 'The controller with the name \'{0}\' is not registered.', constructor);
            }

            assertArgFn(expression, constructor, true);
          }

          if (later) {
            // Instantiate controller later:
            // This machinery is used to create an instance of the object before calling the
            // controller's constructor itself.
            //
            // This allows properties to be added to the controller before the constructor is
            // invoked. Primarily, this is used for isolate scope bindings in $compile.
            //
            // This feature is not intended for use by applications, and is thus not documented
            // publicly.
            // Object creation: http://jsperf.com/create-constructor/2
            var controllerPrototype = (isArray(expression) ? expression[expression.length - 1] : expression).prototype;
            instance = Object.create(controllerPrototype || null);

            if (identifier) {
              addIdentifier(locals, identifier, instance, constructor || expression.name);
            }

            return extend(function $controllerInit() {
              var result = $injector.invoke(expression, instance, locals, constructor);
              if (result !== instance && (isObject(result) || isFunction(result))) {
                instance = result;
                if (identifier) {
                  // If result changed, re-assign controllerAs value to scope.
                  addIdentifier(locals, identifier, instance, constructor || expression.name);
                }
              }
              return instance;
            }, {
              instance: instance,
              identifier: identifier
            });
          }

          instance = $injector.instantiate(expression, locals, constructor);

          if (identifier) {
            addIdentifier(locals, identifier, instance, constructor || expression.name);
          }

          return instance;
        };

        function addIdentifier(locals, identifier, instance, name) {
          if (!(locals && isObject(locals.$scope))) {
            throw minErr('$controller')('noscp', 'Cannot export controller \'{0}\' as \'{1}\'! No $scope object provided via `locals`.', name, identifier);
          }

          locals.$scope[identifier] = instance;
        }
      }];
    }

    /**
     * @ngdoc service
     * @name $document
     * @requires $window
     * @this
     *
     * @description
     * A {@link angular.element jQuery or jqLite} wrapper for the browser's `window.document` object.
     *
     * @example
       <example module="documentExample" name="document">
         <file name="index.html">
           <div ng-controller="ExampleController">
             <p>$document title: <b ng-bind="title"></b></p>
             <p>window.document title: <b ng-bind="windowTitle"></b></p>
           </div>
         </file>
         <file name="script.js">
           angular.module('documentExample', [])
             .controller('ExampleController', ['$scope', '$document', function($scope, $document) {
               $scope.title = $document[0].title;
               $scope.windowTitle = angular.element(window.document)[0].title;
             }]);
         </file>
       </example>
     */
    function $DocumentProvider() {
      this.$get = ['$window', function (window) {
        return jqLite(window.document);
      }];
    }

    /**
     * @private
     * @this
     * Listens for document visibility change and makes the current status accessible.
     */
    function $$IsDocumentHiddenProvider() {
      this.$get = ['$document', '$rootScope', function ($document, $rootScope) {
        var doc = $document[0];
        var hidden = doc && doc.hidden;

        $document.on('visibilitychange', changeListener);

        $rootScope.$on('$destroy', function () {
          $document.off('visibilitychange', changeListener);
        });

        function changeListener() {
          hidden = doc.hidden;
        }

        return function () {
          return hidden;
        };
      }];
    }

    /**
     * @ngdoc service
     * @name $exceptionHandler
     * @requires ng.$log
     * @this
     *
     * @description
     * Any uncaught exception in angular expressions is delegated to this service.
     * The default implementation simply delegates to `$log.error` which logs it into
     * the browser console.
     *
     * In unit tests, if `angular-mocks.js` is loaded, this service is overridden by
     * {@link ngMock.$exceptionHandler mock $exceptionHandler} which aids in testing.
     *
     * ## Example:
     *
     * The example below will overwrite the default `$exceptionHandler` in order to (a) log uncaught
     * errors to the backend for later inspection by the developers and (b) to use `$log.warn()` instead
     * of `$log.error()`.
     *
     * ```js
     *   angular.
     *     module('exceptionOverwrite', []).
     *     factory('$exceptionHandler', ['$log', 'logErrorsToBackend', function($log, logErrorsToBackend) {
     *       return function myExceptionHandler(exception, cause) {
     *         logErrorsToBackend(exception, cause);
     *         $log.warn(exception, cause);
     *       };
     *     }]);
     * ```
     *
     * <hr />
     * Note, that code executed in event-listeners (even those registered using jqLite's `on`/`bind`
     * methods) does not delegate exceptions to the {@link ng.$exceptionHandler $exceptionHandler}
     * (unless executed during a digest).
     *
     * If you wish, you can manually delegate exceptions, e.g.
     * `try { ... } catch(e) { $exceptionHandler(e); }`
     *
     * @param {Error} exception Exception associated with the error.
     * @param {string=} cause Optional information about the context in which
     *       the error was thrown.
     *
     */
    function $ExceptionHandlerProvider() {
      this.$get = ['$log', function ($log) {
        return function (exception, cause) {
          $log.error.apply($log, arguments);
        };
      }];
    }

    var $$ForceReflowProvider = /** @this */function () {
      this.$get = ['$document', function ($document) {
        return function (domNode) {
          //the line below will force the browser to perform a repaint so
          //that all the animated elements within the animation frame will
          //be properly updated and drawn on screen. This is required to
          //ensure that the preparation animation is properly flushed so that
          //the active state picks up from there. DO NOT REMOVE THIS LINE.
          //DO NOT OPTIMIZE THIS LINE. THE MINIFIER WILL REMOVE IT OTHERWISE WHICH
          //WILL RESULT IN AN UNPREDICTABLE BUG THAT IS VERY HARD TO TRACK DOWN AND
          //WILL TAKE YEARS AWAY FROM YOUR LIFE.
          if (domNode) {
            if (!domNode.nodeType && domNode instanceof jqLite) {
              domNode = domNode[0];
            }
          } else {
            domNode = $document[0].body;
          }
          return domNode.offsetWidth + 1;
        };
      }];
    };

    var APPLICATION_JSON = 'application/json';
    var CONTENT_TYPE_APPLICATION_JSON = { 'Content-Type': APPLICATION_JSON + ';charset=utf-8' };
    var JSON_START = /^\[|^\{(?!\{)/;
    var JSON_ENDS = {
      '[': /]$/,
      '{': /}$/
    };
    var JSON_PROTECTION_PREFIX = /^\)]\}',?\n/;
    var $httpMinErr = minErr('$http');

    function serializeValue(v) {
      if (isObject(v)) {
        return isDate(v) ? v.toISOString() : toJson(v);
      }
      return v;
    }

    /** @this */
    function $HttpParamSerializerProvider() {
      /**
       * @ngdoc service
       * @name $httpParamSerializer
       * @description
       *
       * Default {@link $http `$http`} params serializer that converts objects to strings
       * according to the following rules:
       *
       * * `{'foo': 'bar'}` results in `foo=bar`
       * * `{'foo': Date.now()}` results in `foo=2015-04-01T09%3A50%3A49.262Z` (`toISOString()` and encoded representation of a Date object)
       * * `{'foo': ['bar', 'baz']}` results in `foo=bar&foo=baz` (repeated key for each array element)
       * * `{'foo': {'bar':'baz'}}` results in `foo=%7B%22bar%22%3A%22baz%22%7D` (stringified and encoded representation of an object)
       *
       * Note that serializer will sort the request parameters alphabetically.
       * */

      this.$get = function () {
        return function ngParamSerializer(params) {
          if (!params) return '';
          var parts = [];
          forEachSorted(params, function (value, key) {
            if (value === null || isUndefined(value)) return;
            if (isArray(value)) {
              forEach(value, function (v) {
                parts.push(encodeUriQuery(key) + '=' + encodeUriQuery(serializeValue(v)));
              });
            } else {
              parts.push(encodeUriQuery(key) + '=' + encodeUriQuery(serializeValue(value)));
            }
          });

          return parts.join('&');
        };
      };
    }

    /** @this */
    function $HttpParamSerializerJQLikeProvider() {
      /**
       * @ngdoc service
       * @name $httpParamSerializerJQLike
       *
       * @description
       *
       * Alternative {@link $http `$http`} params serializer that follows
       * jQuery's [`param()`](http://api.jquery.com/jquery.param/) method logic.
       * The serializer will also sort the params alphabetically.
       *
       * To use it for serializing `$http` request parameters, set it as the `paramSerializer` property:
       *
       * ```js
       * $http({
       *   url: myUrl,
       *   method: 'GET',
       *   params: myParams,
       *   paramSerializer: '$httpParamSerializerJQLike'
       * });
       * ```
       *
       * It is also possible to set it as the default `paramSerializer` in the
       * {@link $httpProvider#defaults `$httpProvider`}.
       *
       * Additionally, you can inject the serializer and use it explicitly, for example to serialize
       * form data for submission:
       *
       * ```js
       * .controller(function($http, $httpParamSerializerJQLike) {
       *   //...
       *
       *   $http({
       *     url: myUrl,
       *     method: 'POST',
       *     data: $httpParamSerializerJQLike(myData),
       *     headers: {
       *       'Content-Type': 'application/x-www-form-urlencoded'
       *     }
       *   });
       *
       * });
       * ```
       *
       * */
      this.$get = function () {
        return function jQueryLikeParamSerializer(params) {
          if (!params) return '';
          var parts = [];
          serialize(params, '', true);
          return parts.join('&');

          function serialize(toSerialize, prefix, topLevel) {
            if (toSerialize === null || isUndefined(toSerialize)) return;
            if (isArray(toSerialize)) {
              forEach(toSerialize, function (value, index) {
                serialize(value, prefix + '[' + (isObject(value) ? index : '') + ']');
              });
            } else if (isObject(toSerialize) && !isDate(toSerialize)) {
              forEachSorted(toSerialize, function (value, key) {
                serialize(value, prefix + (topLevel ? '' : '[') + key + (topLevel ? '' : ']'));
              });
            } else {
              parts.push(encodeUriQuery(prefix) + '=' + encodeUriQuery(serializeValue(toSerialize)));
            }
          }
        };
      };
    }

    function defaultHttpResponseTransform(data, headers) {
      if (isString(data)) {
        // Strip json vulnerability protection prefix and trim whitespace
        var tempData = data.replace(JSON_PROTECTION_PREFIX, '').trim();

        if (tempData) {
          var contentType = headers('Content-Type');
          if (contentType && contentType.indexOf(APPLICATION_JSON) === 0 || isJsonLike(tempData)) {
            data = fromJson(tempData);
          }
        }
      }

      return data;
    }

    function isJsonLike(str) {
      var jsonStart = str.match(JSON_START);
      return jsonStart && JSON_ENDS[jsonStart[0]].test(str);
    }

    /**
     * Parse headers into key value object
     *
     * @param {string} headers Raw headers as a string
     * @returns {Object} Parsed headers as key value object
     */
    function parseHeaders(headers) {
      var parsed = createMap(),
          i;

      function fillInParsed(key, val) {
        if (key) {
          parsed[key] = parsed[key] ? parsed[key] + ', ' + val : val;
        }
      }

      if (isString(headers)) {
        forEach(headers.split('\n'), function (line) {
          i = line.indexOf(':');
          fillInParsed(lowercase(trim(line.substr(0, i))), trim(line.substr(i + 1)));
        });
      } else if (isObject(headers)) {
        forEach(headers, function (headerVal, headerKey) {
          fillInParsed(lowercase(headerKey), trim(headerVal));
        });
      }

      return parsed;
    }

    /**
     * Returns a function that provides access to parsed headers.
     *
     * Headers are lazy parsed when first requested.
     * @see parseHeaders
     *
     * @param {(string|Object)} headers Headers to provide access to.
     * @returns {function(string=)} Returns a getter function which if called with:
     *
     *   - if called with an argument returns a single header value or null
     *   - if called with no arguments returns an object containing all headers.
     */
    function headersGetter(headers) {
      var headersObj;

      return function (name) {
        if (!headersObj) headersObj = parseHeaders(headers);

        if (name) {
          var value = headersObj[lowercase(name)];
          if (value === undefined) {
            value = null;
          }
          return value;
        }

        return headersObj;
      };
    }

    /**
     * Chain all given functions
     *
     * This function is used for both request and response transforming
     *
     * @param {*} data Data to transform.
     * @param {function(string=)} headers HTTP headers getter fn.
     * @param {number} status HTTP status code of the response.
     * @param {(Function|Array.<Function>)} fns Function or an array of functions.
     * @returns {*} Transformed data.
     */
    function transformData(data, headers, status, fns) {
      if (isFunction(fns)) {
        return fns(data, headers, status);
      }

      forEach(fns, function (fn) {
        data = fn(data, headers, status);
      });

      return data;
    }

    function isSuccess(status) {
      return 200 <= status && status < 300;
    }

    /**
     * @ngdoc provider
     * @name $httpProvider
     * @this
     *
     * @description
     * Use `$httpProvider` to change the default behavior of the {@link ng.$http $http} service.
     * */
    function $HttpProvider() {
      /**
       * @ngdoc property
       * @name $httpProvider#defaults
       * @description
       *
       * Object containing default values for all {@link ng.$http $http} requests.
       *
       * - **`defaults.cache`** - {boolean|Object} - A boolean value or object created with
       * {@link ng.$cacheFactory `$cacheFactory`} to enable or disable caching of HTTP responses
       * by default. See {@link $http#caching $http Caching} for more information.
       *
       * - **`defaults.xsrfCookieName`** - {string} - Name of cookie containing the XSRF token.
       * Defaults value is `'XSRF-TOKEN'`.
       *
       * - **`defaults.xsrfHeaderName`** - {string} - Name of HTTP header to populate with the
       * XSRF token. Defaults value is `'X-XSRF-TOKEN'`.
       *
       * - **`defaults.headers`** - {Object} - Default headers for all $http requests.
       * Refer to {@link ng.$http#setting-http-headers $http} for documentation on
       * setting default headers.
       *     - **`defaults.headers.common`**
       *     - **`defaults.headers.post`**
       *     - **`defaults.headers.put`**
       *     - **`defaults.headers.patch`**
       *
       *
       * - **`defaults.paramSerializer`** - `{string|function(Object<string,string>):string}` - A function
       *  used to the prepare string representation of request parameters (specified as an object).
       *  If specified as string, it is interpreted as a function registered with the {@link auto.$injector $injector}.
       *  Defaults to {@link ng.$httpParamSerializer $httpParamSerializer}.
       *
       * - **`defaults.jsonpCallbackParam`** - `{string}` - the name of the query parameter that passes the name of the
       * callback in a JSONP request. The value of this parameter will be replaced with the expression generated by the
       * {@link $jsonpCallbacks} service. Defaults to `'callback'`.
       *
       **/
      var defaults = this.defaults = {
        // transform incoming response data
        transformResponse: [defaultHttpResponseTransform],

        // transform outgoing request data
        transformRequest: [function (d) {
          return isObject(d) && !isFile(d) && !isBlob(d) && !isFormData(d) ? toJson(d) : d;
        }],

        // default headers
        headers: {
          common: {
            'Accept': 'application/json, text/plain, */*'
          },
          post: shallowCopy(CONTENT_TYPE_APPLICATION_JSON),
          put: shallowCopy(CONTENT_TYPE_APPLICATION_JSON),
          patch: shallowCopy(CONTENT_TYPE_APPLICATION_JSON)
        },

        xsrfCookieName: 'XSRF-TOKEN',
        xsrfHeaderName: 'X-XSRF-TOKEN',

        paramSerializer: '$httpParamSerializer',

        jsonpCallbackParam: 'callback'
      };

      var useApplyAsync = false;
      /**
       * @ngdoc method
       * @name $httpProvider#useApplyAsync
       * @description
       *
       * Configure $http service to combine processing of multiple http responses received at around
       * the same time via {@link ng.$rootScope.Scope#$applyAsync $rootScope.$applyAsync}. This can result in
       * significant performance improvement for bigger applications that make many HTTP requests
       * concurrently (common during application bootstrap).
       *
       * Defaults to false. If no value is specified, returns the current configured value.
       *
       * @param {boolean=} value If true, when requests are loaded, they will schedule a deferred
       *    "apply" on the next tick, giving time for subsequent requests in a roughly ~10ms window
       *    to load and share the same digest cycle.
       *
       * @returns {boolean|Object} If a value is specified, returns the $httpProvider for chaining.
       *    otherwise, returns the current configured value.
       **/
      this.useApplyAsync = function (value) {
        if (isDefined(value)) {
          useApplyAsync = !!value;
          return this;
        }
        return useApplyAsync;
      };

      /**
       * @ngdoc property
       * @name $httpProvider#interceptors
       * @description
       *
       * Array containing service factories for all synchronous or asynchronous {@link ng.$http $http}
       * pre-processing of request or postprocessing of responses.
       *
       * These service factories are ordered by request, i.e. they are applied in the same order as the
       * array, on request, but reverse order, on response.
       *
       * {@link ng.$http#interceptors Interceptors detailed info}
       **/
      var interceptorFactories = this.interceptors = [];

      this.$get = ['$browser', '$httpBackend', '$$cookieReader', '$cacheFactory', '$rootScope', '$q', '$injector', '$sce', function ($browser, $httpBackend, $$cookieReader, $cacheFactory, $rootScope, $q, $injector, $sce) {

        var defaultCache = $cacheFactory('$http');

        /**
         * Make sure that default param serializer is exposed as a function
         */
        defaults.paramSerializer = isString(defaults.paramSerializer) ? $injector.get(defaults.paramSerializer) : defaults.paramSerializer;

        /**
         * Interceptors stored in reverse order. Inner interceptors before outer interceptors.
         * The reversal is needed so that we can build up the interception chain around the
         * server request.
         */
        var reversedInterceptors = [];

        forEach(interceptorFactories, function (interceptorFactory) {
          reversedInterceptors.unshift(isString(interceptorFactory) ? $injector.get(interceptorFactory) : $injector.invoke(interceptorFactory));
        });

        /**
         * @ngdoc service
         * @kind function
         * @name $http
         * @requires ng.$httpBackend
         * @requires $cacheFactory
         * @requires $rootScope
         * @requires $q
         * @requires $injector
         *
         * @description
         * The `$http` service is a core Angular service that facilitates communication with the remote
         * HTTP servers via the browser's [XMLHttpRequest](https://developer.mozilla.org/en/xmlhttprequest)
         * object or via [JSONP](http://en.wikipedia.org/wiki/JSONP).
         *
         * For unit testing applications that use `$http` service, see
         * {@link ngMock.$httpBackend $httpBackend mock}.
         *
         * For a higher level of abstraction, please check out the {@link ngResource.$resource
         * $resource} service.
         *
         * The $http API is based on the {@link ng.$q deferred/promise APIs} exposed by
         * the $q service. While for simple usage patterns this doesn't matter much, for advanced usage
         * it is important to familiarize yourself with these APIs and the guarantees they provide.
         *
         *
         * ## General usage
         * The `$http` service is a function which takes a single argument — a {@link $http#usage configuration object} —
         * that is used to generate an HTTP request and returns  a {@link ng.$q promise}.
         *
         * ```js
         *   // Simple GET request example:
         *   $http({
         *     method: 'GET',
         *     url: '/someUrl'
         *   }).then(function successCallback(response) {
         *       // this callback will be called asynchronously
         *       // when the response is available
         *     }, function errorCallback(response) {
         *       // called asynchronously if an error occurs
         *       // or server returns response with an error status.
         *     });
         * ```
         *
         * The response object has these properties:
         *
         *   - **data** – `{string|Object}` – The response body transformed with the transform
         *     functions.
         *   - **status** – `{number}` – HTTP status code of the response.
         *   - **headers** – `{function([headerName])}` – Header getter function.
         *   - **config** – `{Object}` – The configuration object that was used to generate the request.
         *   - **statusText** – `{string}` – HTTP status text of the response.
         *
         * A response status code between 200 and 299 is considered a success status and will result in
         * the success callback being called. Any response status code outside of that range is
         * considered an error status and will result in the error callback being called.
         * Also, status codes less than -1 are normalized to zero. -1 usually means the request was
         * aborted, e.g. using a `config.timeout`.
         * Note that if the response is a redirect, XMLHttpRequest will transparently follow it, meaning
         * that the outcome (success or error) will be determined by the final response status code.
         *
         *
         * ## Shortcut methods
         *
         * Shortcut methods are also available. All shortcut methods require passing in the URL, and
         * request data must be passed in for POST/PUT requests. An optional config can be passed as the
         * last argument.
         *
         * ```js
         *   $http.get('/someUrl', config).then(successCallback, errorCallback);
         *   $http.post('/someUrl', data, config).then(successCallback, errorCallback);
         * ```
         *
         * Complete list of shortcut methods:
         *
         * - {@link ng.$http#get $http.get}
         * - {@link ng.$http#head $http.head}
         * - {@link ng.$http#post $http.post}
         * - {@link ng.$http#put $http.put}
         * - {@link ng.$http#delete $http.delete}
         * - {@link ng.$http#jsonp $http.jsonp}
         * - {@link ng.$http#patch $http.patch}
         *
         *
         * ## Writing Unit Tests that use $http
         * When unit testing (using {@link ngMock ngMock}), it is necessary to call
         * {@link ngMock.$httpBackend#flush $httpBackend.flush()} to flush each pending
         * request using trained responses.
         *
         * ```
         * $httpBackend.expectGET(...);
         * $http.get(...);
         * $httpBackend.flush();
         * ```
         *
         * ## Setting HTTP Headers
         *
         * The $http service will automatically add certain HTTP headers to all requests. These defaults
         * can be fully configured by accessing the `$httpProvider.defaults.headers` configuration
         * object, which currently contains this default configuration:
         *
         * - `$httpProvider.defaults.headers.common` (headers that are common for all requests):
         *   - <code>Accept: application/json, text/plain, \*&#65279;/&#65279;\*</code>
         * - `$httpProvider.defaults.headers.post`: (header defaults for POST requests)
         *   - `Content-Type: application/json`
         * - `$httpProvider.defaults.headers.put` (header defaults for PUT requests)
         *   - `Content-Type: application/json`
         *
         * To add or overwrite these defaults, simply add or remove a property from these configuration
         * objects. To add headers for an HTTP method other than POST or PUT, simply add a new object
         * with the lowercased HTTP method name as the key, e.g.
         * `$httpProvider.defaults.headers.get = { 'My-Header' : 'value' }`.
         *
         * The defaults can also be set at runtime via the `$http.defaults` object in the same
         * fashion. For example:
         *
         * ```
         * module.run(function($http) {
         *   $http.defaults.headers.common.Authorization = 'Basic YmVlcDpib29w';
         * });
         * ```
         *
         * In addition, you can supply a `headers` property in the config object passed when
         * calling `$http(config)`, which overrides the defaults without changing them globally.
         *
         * To explicitly remove a header automatically added via $httpProvider.defaults.headers on a per request basis,
         * Use the `headers` property, setting the desired header to `undefined`. For example:
         *
         * ```js
         * var req = {
         *  method: 'POST',
         *  url: 'http://example.com',
         *  headers: {
         *    'Content-Type': undefined
         *  },
         *  data: { test: 'test' }
         * }
         *
         * $http(req).then(function(){...}, function(){...});
         * ```
         *
         * ## Transforming Requests and Responses
         *
         * Both requests and responses can be transformed using transformation functions: `transformRequest`
         * and `transformResponse`. These properties can be a single function that returns
         * the transformed value (`function(data, headersGetter, status)`) or an array of such transformation functions,
         * which allows you to `push` or `unshift` a new transformation function into the transformation chain.
         *
         * <div class="alert alert-warning">
         * **Note:** Angular does not make a copy of the `data` parameter before it is passed into the `transformRequest` pipeline.
         * That means changes to the properties of `data` are not local to the transform function (since Javascript passes objects by reference).
         * For example, when calling `$http.get(url, $scope.myObject)`, modifications to the object's properties in a transformRequest
         * function will be reflected on the scope and in any templates where the object is data-bound.
         * To prevent this, transform functions should have no side-effects.
         * If you need to modify properties, it is recommended to make a copy of the data, or create new object to return.
         * </div>
         *
         * ### Default Transformations
         *
         * The `$httpProvider` provider and `$http` service expose `defaults.transformRequest` and
         * `defaults.transformResponse` properties. If a request does not provide its own transformations
         * then these will be applied.
         *
         * You can augment or replace the default transformations by modifying these properties by adding to or
         * replacing the array.
         *
         * Angular provides the following default transformations:
         *
         * Request transformations (`$httpProvider.defaults.transformRequest` and `$http.defaults.transformRequest`):
         *
         * - If the `data` property of the request configuration object contains an object, serialize it
         *   into JSON format.
         *
         * Response transformations (`$httpProvider.defaults.transformResponse` and `$http.defaults.transformResponse`):
         *
         *  - If XSRF prefix is detected, strip it (see Security Considerations section below).
         *  - If JSON response is detected, deserialize it using a JSON parser.
         *
         *
         * ### Overriding the Default Transformations Per Request
         *
         * If you wish to override the request/response transformations only for a single request then provide
         * `transformRequest` and/or `transformResponse` properties on the configuration object passed
         * into `$http`.
         *
         * Note that if you provide these properties on the config object the default transformations will be
         * overwritten. If you wish to augment the default transformations then you must include them in your
         * local transformation array.
         *
         * The following code demonstrates adding a new response transformation to be run after the default response
         * transformations have been run.
         *
         * ```js
         * function appendTransform(defaults, transform) {
         *
         *   // We can't guarantee that the default transformation is an array
         *   defaults = angular.isArray(defaults) ? defaults : [defaults];
         *
         *   // Append the new transformation to the defaults
         *   return defaults.concat(transform);
         * }
         *
         * $http({
         *   url: '...',
         *   method: 'GET',
         *   transformResponse: appendTransform($http.defaults.transformResponse, function(value) {
         *     return doTransform(value);
         *   })
         * });
         * ```
         *
         *
         * ## Caching
         *
         * {@link ng.$http `$http`} responses are not cached by default. To enable caching, you must
         * set the config.cache value or the default cache value to TRUE or to a cache object (created
         * with {@link ng.$cacheFactory `$cacheFactory`}). If defined, the value of config.cache takes
         * precedence over the default cache value.
         *
         * In order to:
         *   * cache all responses - set the default cache value to TRUE or to a cache object
         *   * cache a specific response - set config.cache value to TRUE or to a cache object
         *
         * If caching is enabled, but neither the default cache nor config.cache are set to a cache object,
         * then the default `$cacheFactory("$http")` object is used.
         *
         * The default cache value can be set by updating the
         * {@link ng.$http#defaults `$http.defaults.cache`} property or the
         * {@link $httpProvider#defaults `$httpProvider.defaults.cache`} property.
         *
         * When caching is enabled, {@link ng.$http `$http`} stores the response from the server using
         * the relevant cache object. The next time the same request is made, the response is returned
         * from the cache without sending a request to the server.
         *
         * Take note that:
         *
         *   * Only GET and JSONP requests are cached.
         *   * The cache key is the request URL including search parameters; headers are not considered.
         *   * Cached responses are returned asynchronously, in the same way as responses from the server.
         *   * If multiple identical requests are made using the same cache, which is not yet populated,
         *     one request will be made to the server and remaining requests will return the same response.
         *   * A cache-control header on the response does not affect if or how responses are cached.
         *
         *
         * ## Interceptors
         *
         * Before you start creating interceptors, be sure to understand the
         * {@link ng.$q $q and deferred/promise APIs}.
         *
         * For purposes of global error handling, authentication, or any kind of synchronous or
         * asynchronous pre-processing of request or postprocessing of responses, it is desirable to be
         * able to intercept requests before they are handed to the server and
         * responses before they are handed over to the application code that
         * initiated these requests. The interceptors leverage the {@link ng.$q
         * promise APIs} to fulfill this need for both synchronous and asynchronous pre-processing.
         *
         * The interceptors are service factories that are registered with the `$httpProvider` by
         * adding them to the `$httpProvider.interceptors` array. The factory is called and
         * injected with dependencies (if specified) and returns the interceptor.
         *
         * There are two kinds of interceptors (and two kinds of rejection interceptors):
         *
         *   * `request`: interceptors get called with a http {@link $http#usage config} object. The function is free to
         *     modify the `config` object or create a new one. The function needs to return the `config`
         *     object directly, or a promise containing the `config` or a new `config` object.
         *   * `requestError`: interceptor gets called when a previous interceptor threw an error or
         *     resolved with a rejection.
         *   * `response`: interceptors get called with http `response` object. The function is free to
         *     modify the `response` object or create a new one. The function needs to return the `response`
         *     object directly, or as a promise containing the `response` or a new `response` object.
         *   * `responseError`: interceptor gets called when a previous interceptor threw an error or
         *     resolved with a rejection.
         *
         *
         * ```js
         *   // register the interceptor as a service
         *   $provide.factory('myHttpInterceptor', function($q, dependency1, dependency2) {
         *     return {
         *       // optional method
         *       'request': function(config) {
         *         // do something on success
         *         return config;
         *       },
         *
         *       // optional method
         *      'requestError': function(rejection) {
         *         // do something on error
         *         if (canRecover(rejection)) {
         *           return responseOrNewPromise
         *         }
         *         return $q.reject(rejection);
         *       },
         *
         *
         *
         *       // optional method
         *       'response': function(response) {
         *         // do something on success
         *         return response;
         *       },
         *
         *       // optional method
         *      'responseError': function(rejection) {
         *         // do something on error
         *         if (canRecover(rejection)) {
         *           return responseOrNewPromise
         *         }
         *         return $q.reject(rejection);
         *       }
         *     };
         *   });
         *
         *   $httpProvider.interceptors.push('myHttpInterceptor');
         *
         *
         *   // alternatively, register the interceptor via an anonymous factory
         *   $httpProvider.interceptors.push(function($q, dependency1, dependency2) {
         *     return {
         *      'request': function(config) {
         *          // same as above
         *       },
         *
         *       'response': function(response) {
         *          // same as above
         *       }
         *     };
         *   });
         * ```
         *
         * ## Security Considerations
         *
         * When designing web applications, consider security threats from:
         *
         * - [JSON vulnerability](http://haacked.com/archive/2008/11/20/anatomy-of-a-subtle-json-vulnerability.aspx)
         * - [XSRF](http://en.wikipedia.org/wiki/Cross-site_request_forgery)
         *
         * Both server and the client must cooperate in order to eliminate these threats. Angular comes
         * pre-configured with strategies that address these issues, but for this to work backend server
         * cooperation is required.
         *
         * ### JSON Vulnerability Protection
         *
         * A [JSON vulnerability](http://haacked.com/archive/2008/11/20/anatomy-of-a-subtle-json-vulnerability.aspx)
         * allows third party website to turn your JSON resource URL into
         * [JSONP](http://en.wikipedia.org/wiki/JSONP) request under some conditions. To
         * counter this your server can prefix all JSON requests with following string `")]}',\n"`.
         * Angular will automatically strip the prefix before processing it as JSON.
         *
         * For example if your server needs to return:
         * ```js
         * ['one','two']
         * ```
         *
         * which is vulnerable to attack, your server can return:
         * ```js
         * )]}',
         * ['one','two']
         * ```
         *
         * Angular will strip the prefix, before processing the JSON.
         *
         *
         * ### Cross Site Request Forgery (XSRF) Protection
         *
         * [XSRF](http://en.wikipedia.org/wiki/Cross-site_request_forgery) is an attack technique by
         * which the attacker can trick an authenticated user into unknowingly executing actions on your
         * website. Angular provides a mechanism to counter XSRF. When performing XHR requests, the
         * $http service reads a token from a cookie (by default, `XSRF-TOKEN`) and sets it as an HTTP
         * header (`X-XSRF-TOKEN`). Since only JavaScript that runs on your domain could read the
         * cookie, your server can be assured that the XHR came from JavaScript running on your domain.
         * The header will not be set for cross-domain requests.
         *
         * To take advantage of this, your server needs to set a token in a JavaScript readable session
         * cookie called `XSRF-TOKEN` on the first HTTP GET request. On subsequent XHR requests the
         * server can verify that the cookie matches `X-XSRF-TOKEN` HTTP header, and therefore be sure
         * that only JavaScript running on your domain could have sent the request. The token must be
         * unique for each user and must be verifiable by the server (to prevent the JavaScript from
         * making up its own tokens). We recommend that the token is a digest of your site's
         * authentication cookie with a [salt](https://en.wikipedia.org/wiki/Salt_(cryptography&#41;)
         * for added security.
         *
         * The name of the headers can be specified using the xsrfHeaderName and xsrfCookieName
         * properties of either $httpProvider.defaults at config-time, $http.defaults at run-time,
         * or the per-request config object.
         *
         * In order to prevent collisions in environments where multiple Angular apps share the
         * same domain or subdomain, we recommend that each application uses unique cookie name.
         *
         * @param {object} config Object describing the request to be made and how it should be
         *    processed. The object has following properties:
         *
         *    - **method** – `{string}` – HTTP method (e.g. 'GET', 'POST', etc)
         *    - **url** – `{string|TrustedObject}` – Absolute or relative URL of the resource that is being requested;
         *      or an object created by a call to `$sce.trustAsResourceUrl(url)`.
         *    - **params** – `{Object.<string|Object>}` – Map of strings or objects which will be serialized
         *      with the `paramSerializer` and appended as GET parameters.
         *    - **data** – `{string|Object}` – Data to be sent as the request message data.
         *    - **headers** – `{Object}` – Map of strings or functions which return strings representing
         *      HTTP headers to send to the server. If the return value of a function is null, the
         *      header will not be sent. Functions accept a config object as an argument.
         *    - **eventHandlers** - `{Object}` - Event listeners to be bound to the XMLHttpRequest object.
         *      To bind events to the XMLHttpRequest upload object, use `uploadEventHandlers`.
         *      The handler will be called in the context of a `$apply` block.
         *    - **uploadEventHandlers** - `{Object}` - Event listeners to be bound to the XMLHttpRequest upload
         *      object. To bind events to the XMLHttpRequest object, use `eventHandlers`.
         *      The handler will be called in the context of a `$apply` block.
         *    - **xsrfHeaderName** – `{string}` – Name of HTTP header to populate with the XSRF token.
         *    - **xsrfCookieName** – `{string}` – Name of cookie containing the XSRF token.
         *    - **transformRequest** –
         *      `{function(data, headersGetter)|Array.<function(data, headersGetter)>}` –
         *      transform function or an array of such functions. The transform function takes the http
         *      request body and headers and returns its transformed (typically serialized) version.
         *      See {@link ng.$http#overriding-the-default-transformations-per-request
         *      Overriding the Default Transformations}
         *    - **transformResponse** –
         *      `{function(data, headersGetter, status)|Array.<function(data, headersGetter, status)>}` –
         *      transform function or an array of such functions. The transform function takes the http
         *      response body, headers and status and returns its transformed (typically deserialized) version.
         *      See {@link ng.$http#overriding-the-default-transformations-per-request
         *      Overriding the Default Transformations}
         *    - **paramSerializer** - `{string|function(Object<string,string>):string}` - A function used to
         *      prepare the string representation of request parameters (specified as an object).
         *      If specified as string, it is interpreted as function registered with the
         *      {@link $injector $injector}, which means you can create your own serializer
         *      by registering it as a {@link auto.$provide#service service}.
         *      The default serializer is the {@link $httpParamSerializer $httpParamSerializer};
         *      alternatively, you can use the {@link $httpParamSerializerJQLike $httpParamSerializerJQLike}
         *    - **cache** – `{boolean|Object}` – A boolean value or object created with
         *      {@link ng.$cacheFactory `$cacheFactory`} to enable or disable caching of the HTTP response.
         *      See {@link $http#caching $http Caching} for more information.
         *    - **timeout** – `{number|Promise}` – timeout in milliseconds, or {@link ng.$q promise}
         *      that should abort the request when resolved.
         *    - **withCredentials** - `{boolean}` - whether to set the `withCredentials` flag on the
         *      XHR object. See [requests with credentials](https://developer.mozilla.org/docs/Web/HTTP/Access_control_CORS#Requests_with_credentials)
         *      for more information.
         *    - **responseType** - `{string}` - see
         *      [XMLHttpRequest.responseType](https://developer.mozilla.org/en-US/docs/Web/API/XMLHttpRequest#xmlhttprequest-responsetype).
         *
         * @returns {HttpPromise} Returns a {@link ng.$q `Promise}` that will be resolved to a response object
         *                        when the request succeeds or fails.
         *
         *
         * @property {Array.<Object>} pendingRequests Array of config objects for currently pending
         *   requests. This is primarily meant to be used for debugging purposes.
         *
         *
         * @example
        <example module="httpExample" name="http-service">
        <file name="index.html">
        <div ng-controller="FetchController">
        <select ng-model="method" aria-label="Request method">
          <option>GET</option>
          <option>JSONP</option>
        </select>
        <input type="text" ng-model="url" size="80" aria-label="URL" />
        <button id="fetchbtn" ng-click="fetch()">fetch</button><br>
        <button id="samplegetbtn" ng-click="updateModel('GET', 'http-hello.html')">Sample GET</button>
        <button id="samplejsonpbtn"
          ng-click="updateModel('JSONP',
                        'https://angularjs.org/greet.php?name=Super%20Hero')">
          Sample JSONP
        </button>
        <button id="invalidjsonpbtn"
          ng-click="updateModel('JSONP', 'https://angularjs.org/doesntexist')">
            Invalid JSONP
          </button>
        <pre>http status code: {{status}}</pre>
        <pre>http response data: {{data}}</pre>
        </div>
        </file>
        <file name="script.js">
        angular.module('httpExample', [])
        .config(['$sceDelegateProvider', function($sceDelegateProvider) {
          // We must whitelist the JSONP endpoint that we are using to show that we trust it
          $sceDelegateProvider.resourceUrlWhitelist([
            'self',
            'https://angularjs.org/**'
          ]);
        }])
        .controller('FetchController', ['$scope', '$http', '$templateCache',
          function($scope, $http, $templateCache) {
            $scope.method = 'GET';
            $scope.url = 'http-hello.html';
             $scope.fetch = function() {
              $scope.code = null;
              $scope.response = null;
               $http({method: $scope.method, url: $scope.url, cache: $templateCache}).
                then(function(response) {
                  $scope.status = response.status;
                  $scope.data = response.data;
                }, function(response) {
                  $scope.data = response.data || 'Request failed';
                  $scope.status = response.status;
              });
            };
             $scope.updateModel = function(method, url) {
              $scope.method = method;
              $scope.url = url;
            };
          }]);
        </file>
        <file name="http-hello.html">
        Hello, $http!
        </file>
        <file name="protractor.js" type="protractor">
        var status = element(by.binding('status'));
        var data = element(by.binding('data'));
        var fetchBtn = element(by.id('fetchbtn'));
        var sampleGetBtn = element(by.id('samplegetbtn'));
        var invalidJsonpBtn = element(by.id('invalidjsonpbtn'));
        it('should make an xhr GET request', function() {
        sampleGetBtn.click();
        fetchBtn.click();
        expect(status.getText()).toMatch('200');
        expect(data.getText()).toMatch(/Hello, \$http!/);
        });
        // Commented out due to flakes. See https://github.com/angular/angular.js/issues/9185
        // it('should make a JSONP request to angularjs.org', function() {
        //   var sampleJsonpBtn = element(by.id('samplejsonpbtn'));
        //   sampleJsonpBtn.click();
        //   fetchBtn.click();
        //   expect(status.getText()).toMatch('200');
        //   expect(data.getText()).toMatch(/Super Hero!/);
        // });
        it('should make JSONP request to invalid URL and invoke the error handler',
          function() {
        invalidJsonpBtn.click();
        fetchBtn.click();
        expect(status.getText()).toMatch('0');
        expect(data.getText()).toMatch('Request failed');
        });
        </file>
        </example>
         */
        function $http(requestConfig) {

          if (!isObject(requestConfig)) {
            throw minErr('$http')('badreq', 'Http request configuration must be an object.  Received: {0}', requestConfig);
          }

          if (!isString($sce.valueOf(requestConfig.url))) {
            throw minErr('$http')('badreq', 'Http request configuration url must be a string or a $sce trusted object.  Received: {0}', requestConfig.url);
          }

          var config = extend({
            method: 'get',
            transformRequest: defaults.transformRequest,
            transformResponse: defaults.transformResponse,
            paramSerializer: defaults.paramSerializer,
            jsonpCallbackParam: defaults.jsonpCallbackParam
          }, requestConfig);

          config.headers = mergeHeaders(requestConfig);
          config.method = uppercase(config.method);
          config.paramSerializer = isString(config.paramSerializer) ? $injector.get(config.paramSerializer) : config.paramSerializer;

          $browser.$$incOutstandingRequestCount();

          var requestInterceptors = [];
          var responseInterceptors = [];
          var promise = $q.resolve(config);

          // apply interceptors
          forEach(reversedInterceptors, function (interceptor) {
            if (interceptor.request || interceptor.requestError) {
              requestInterceptors.unshift(interceptor.request, interceptor.requestError);
            }
            if (interceptor.response || interceptor.responseError) {
              responseInterceptors.push(interceptor.response, interceptor.responseError);
            }
          });

          promise = chainInterceptors(promise, requestInterceptors);
          promise = promise.then(serverRequest);
          promise = chainInterceptors(promise, responseInterceptors);
          promise = promise.finally(completeOutstandingRequest);

          return promise;

          function chainInterceptors(promise, interceptors) {
            for (var i = 0, ii = interceptors.length; i < ii;) {
              var thenFn = interceptors[i++];
              var rejectFn = interceptors[i++];

              promise = promise.then(thenFn, rejectFn);
            }

            interceptors.length = 0;

            return promise;
          }

          function completeOutstandingRequest() {
            $browser.$$completeOutstandingRequest(noop);
          }

          function executeHeaderFns(headers, config) {
            var headerContent,
                processedHeaders = {};

            forEach(headers, function (headerFn, header) {
              if (isFunction(headerFn)) {
                headerContent = headerFn(config);
                if (headerContent != null) {
                  processedHeaders[header] = headerContent;
                }
              } else {
                processedHeaders[header] = headerFn;
              }
            });

            return processedHeaders;
          }

          function mergeHeaders(config) {
            var defHeaders = defaults.headers,
                reqHeaders = extend({}, config.headers),
                defHeaderName,
                lowercaseDefHeaderName,
                reqHeaderName;

            defHeaders = extend({}, defHeaders.common, defHeaders[lowercase(config.method)]);

            // using for-in instead of forEach to avoid unnecessary iteration after header has been found
            defaultHeadersIteration: for (defHeaderName in defHeaders) {
              lowercaseDefHeaderName = lowercase(defHeaderName);

              for (reqHeaderName in reqHeaders) {
                if (lowercase(reqHeaderName) === lowercaseDefHeaderName) {
                  continue defaultHeadersIteration;
                }
              }

              reqHeaders[defHeaderName] = defHeaders[defHeaderName];
            }

            // execute if header value is a function for merged headers
            return executeHeaderFns(reqHeaders, shallowCopy(config));
          }

          function serverRequest(config) {
            var headers = config.headers;
            var reqData = transformData(config.data, headersGetter(headers), undefined, config.transformRequest);

            // strip content-type if data is undefined
            if (isUndefined(reqData)) {
              forEach(headers, function (value, header) {
                if (lowercase(header) === 'content-type') {
                  delete headers[header];
                }
              });
            }

            if (isUndefined(config.withCredentials) && !isUndefined(defaults.withCredentials)) {
              config.withCredentials = defaults.withCredentials;
            }

            // send request
            return sendReq(config, reqData).then(transformResponse, transformResponse);
          }

          function transformResponse(response) {
            // make a copy since the response must be cacheable
            var resp = extend({}, response);
            resp.data = transformData(response.data, response.headers, response.status, config.transformResponse);
            return isSuccess(response.status) ? resp : $q.reject(resp);
          }
        }

        $http.pendingRequests = [];

        /**
         * @ngdoc method
         * @name $http#get
         *
         * @description
         * Shortcut method to perform `GET` request.
         *
         * @param {string|TrustedObject} url Absolute or relative URL of the resource that is being requested;
         *                                   or an object created by a call to `$sce.trustAsResourceUrl(url)`.
         * @param {Object=} config Optional configuration object
         * @returns {HttpPromise} Future object
         */

        /**
         * @ngdoc method
         * @name $http#delete
         *
         * @description
         * Shortcut method to perform `DELETE` request.
         *
         * @param {string|TrustedObject} url Absolute or relative URL of the resource that is being requested;
         *                                   or an object created by a call to `$sce.trustAsResourceUrl(url)`.
         * @param {Object=} config Optional configuration object
         * @returns {HttpPromise} Future object
         */

        /**
         * @ngdoc method
         * @name $http#head
         *
         * @description
         * Shortcut method to perform `HEAD` request.
         *
         * @param {string|TrustedObject} url Absolute or relative URL of the resource that is being requested;
         *                                   or an object created by a call to `$sce.trustAsResourceUrl(url)`.
         * @param {Object=} config Optional configuration object
         * @returns {HttpPromise} Future object
         */

        /**
         * @ngdoc method
         * @name $http#jsonp
         *
         * @description
         * Shortcut method to perform `JSONP` request.
         *
         * Note that, since JSONP requests are sensitive because the response is given full access to the browser,
         * the url must be declared, via {@link $sce} as a trusted resource URL.
         * You can trust a URL by adding it to the whitelist via
         * {@link $sceDelegateProvider#resourceUrlWhitelist  `$sceDelegateProvider.resourceUrlWhitelist`} or
         * by explicitly trusting the URL via {@link $sce#trustAsResourceUrl `$sce.trustAsResourceUrl(url)`}.
         *
         * JSONP requests must specify a callback to be used in the response from the server. This callback
         * is passed as a query parameter in the request. You must specify the name of this parameter by
         * setting the `jsonpCallbackParam` property on the request config object.
         *
         * ```
         * $http.jsonp('some/trusted/url', {jsonpCallbackParam: 'callback'})
         * ```
         *
         * You can also specify a default callback parameter name in `$http.defaults.jsonpCallbackParam`.
         * Initially this is set to `'callback'`.
         *
         * <div class="alert alert-danger">
         * You can no longer use the `JSON_CALLBACK` string as a placeholder for specifying where the callback
         * parameter value should go.
         * </div>
         *
         * If you would like to customise where and how the callbacks are stored then try overriding
         * or decorating the {@link $jsonpCallbacks} service.
         *
         * @param {string|TrustedObject} url Absolute or relative URL of the resource that is being requested;
         *                                   or an object created by a call to `$sce.trustAsResourceUrl(url)`.
         * @param {Object=} config Optional configuration object
         * @returns {HttpPromise} Future object
         */
        createShortMethods('get', 'delete', 'head', 'jsonp');

        /**
         * @ngdoc method
         * @name $http#post
         *
         * @description
         * Shortcut method to perform `POST` request.
         *
         * @param {string} url Relative or absolute URL specifying the destination of the request
         * @param {*} data Request content
         * @param {Object=} config Optional configuration object
         * @returns {HttpPromise} Future object
         */

        /**
         * @ngdoc method
         * @name $http#put
         *
         * @description
         * Shortcut method to perform `PUT` request.
         *
         * @param {string} url Relative or absolute URL specifying the destination of the request
         * @param {*} data Request content
         * @param {Object=} config Optional configuration object
         * @returns {HttpPromise} Future object
         */

        /**
         * @ngdoc method
         * @name $http#patch
         *
         * @description
         * Shortcut method to perform `PATCH` request.
         *
         * @param {string} url Relative or absolute URL specifying the destination of the request
         * @param {*} data Request content
         * @param {Object=} config Optional configuration object
         * @returns {HttpPromise} Future object
         */
        createShortMethodsWithData('post', 'put', 'patch');

        /**
         * @ngdoc property
         * @name $http#defaults
         *
         * @description
         * Runtime equivalent of the `$httpProvider.defaults` property. Allows configuration of
         * default headers, withCredentials as well as request and response transformations.
         *
         * See "Setting HTTP Headers" and "Transforming Requests and Responses" sections above.
         */
        $http.defaults = defaults;

        return $http;

        function createShortMethods(names) {
          forEach(arguments, function (name) {
            $http[name] = function (url, config) {
              return $http(extend({}, config || {}, {
                method: name,
                url: url
              }));
            };
          });
        }

        function createShortMethodsWithData(name) {
          forEach(arguments, function (name) {
            $http[name] = function (url, data, config) {
              return $http(extend({}, config || {}, {
                method: name,
                url: url,
                data: data
              }));
            };
          });
        }

        /**
         * Makes the request.
         *
         * !!! ACCESSES CLOSURE VARS:
         * $httpBackend, defaults, $log, $rootScope, defaultCache, $http.pendingRequests
         */
        function sendReq(config, reqData) {
          var deferred = $q.defer(),
              promise = deferred.promise,
              cache,
              cachedResp,
              reqHeaders = config.headers,
              isJsonp = lowercase(config.method) === 'jsonp',
              url = config.url;

          if (isJsonp) {
            // JSONP is a pretty sensitive operation where we're allowing a script to have full access to
            // our DOM and JS space.  So we require that the URL satisfies SCE.RESOURCE_URL.
            url = $sce.getTrustedResourceUrl(url);
          } else if (!isString(url)) {
            // If it is not a string then the URL must be a $sce trusted object
            url = $sce.valueOf(url);
          }

          url = buildUrl(url, config.paramSerializer(config.params));

          if (isJsonp) {
            // Check the url and add the JSONP callback placeholder
            url = sanitizeJsonpCallbackParam(url, config.jsonpCallbackParam);
          }

          $http.pendingRequests.push(config);
          promise.then(removePendingReq, removePendingReq);

          if ((config.cache || defaults.cache) && config.cache !== false && (config.method === 'GET' || config.method === 'JSONP')) {
            cache = isObject(config.cache) ? config.cache : isObject( /** @type {?} */defaults.cache) ? /** @type {?} */defaults.cache : defaultCache;
          }

          if (cache) {
            cachedResp = cache.get(url);
            if (isDefined(cachedResp)) {
              if (isPromiseLike(cachedResp)) {
                // cached request has already been sent, but there is no response yet
                cachedResp.then(resolvePromiseWithResult, resolvePromiseWithResult);
              } else {
                // serving from cache
                if (isArray(cachedResp)) {
                  resolvePromise(cachedResp[1], cachedResp[0], shallowCopy(cachedResp[2]), cachedResp[3]);
                } else {
                  resolvePromise(cachedResp, 200, {}, 'OK');
                }
              }
            } else {
              // put the promise for the non-transformed response into cache as a placeholder
              cache.put(url, promise);
            }
          }

          // if we won't have the response in cache, set the xsrf headers and
          // send the request to the backend
          if (isUndefined(cachedResp)) {
            var xsrfValue = urlIsSameOrigin(config.url) ? $$cookieReader()[config.xsrfCookieName || defaults.xsrfCookieName] : undefined;
            if (xsrfValue) {
              reqHeaders[config.xsrfHeaderName || defaults.xsrfHeaderName] = xsrfValue;
            }

            $httpBackend(config.method, url, reqData, done, reqHeaders, config.timeout, config.withCredentials, config.responseType, createApplyHandlers(config.eventHandlers), createApplyHandlers(config.uploadEventHandlers));
          }

          return promise;

          function createApplyHandlers(eventHandlers) {
            if (eventHandlers) {
              var applyHandlers = {};
              forEach(eventHandlers, function (eventHandler, key) {
                applyHandlers[key] = function (event) {
                  if (useApplyAsync) {
                    $rootScope.$applyAsync(callEventHandler);
                  } else if ($rootScope.$$phase) {
                    callEventHandler();
                  } else {
                    $rootScope.$apply(callEventHandler);
                  }

                  function callEventHandler() {
                    eventHandler(event);
                  }
                };
              });
              return applyHandlers;
            }
          }

          /**
           * Callback registered to $httpBackend():
           *  - caches the response if desired
           *  - resolves the raw $http promise
           *  - calls $apply
           */
          function done(status, response, headersString, statusText) {
            if (cache) {
              if (isSuccess(status)) {
                cache.put(url, [status, response, parseHeaders(headersString), statusText]);
              } else {
                // remove promise from the cache
                cache.remove(url);
              }
            }

            function resolveHttpPromise() {
              resolvePromise(response, status, headersString, statusText);
            }

            if (useApplyAsync) {
              $rootScope.$applyAsync(resolveHttpPromise);
            } else {
              resolveHttpPromise();
              if (!$rootScope.$$phase) $rootScope.$apply();
            }
          }

          /**
           * Resolves the raw $http promise.
           */
          function resolvePromise(response, status, headers, statusText) {
            //status: HTTP response status code, 0, -1 (aborted by timeout / promise)
            status = status >= -1 ? status : 0;

            (isSuccess(status) ? deferred.resolve : deferred.reject)({
              data: response,
              status: status,
              headers: headersGetter(headers),
              config: config,
              statusText: statusText
            });
          }

          function resolvePromiseWithResult(result) {
            resolvePromise(result.data, result.status, shallowCopy(result.headers()), result.statusText);
          }

          function removePendingReq() {
            var idx = $http.pendingRequests.indexOf(config);
            if (idx !== -1) $http.pendingRequests.splice(idx, 1);
          }
        }

        function buildUrl(url, serializedParams) {
          if (serializedParams.length > 0) {
            url += (url.indexOf('?') === -1 ? '?' : '&') + serializedParams;
          }
          return url;
        }

        function sanitizeJsonpCallbackParam(url, key) {
          if (/[&?][^=]+=JSON_CALLBACK/.test(url)) {
            // Throw if the url already contains a reference to JSON_CALLBACK
            throw $httpMinErr('badjsonp', 'Illegal use of JSON_CALLBACK in url, "{0}"', url);
          }

          var callbackParamRegex = new RegExp('[&?]' + key + '=');
          if (callbackParamRegex.test(url)) {
            // Throw if the callback param was already provided
            throw $httpMinErr('badjsonp', 'Illegal use of callback param, "{0}", in url, "{1}"', key, url);
          }

          // Add in the JSON_CALLBACK callback param value
          url += (url.indexOf('?') === -1 ? '?' : '&') + key + '=JSON_CALLBACK';

          return url;
        }
      }];
    }

    /**
     * @ngdoc service
     * @name $xhrFactory
     * @this
     *
     * @description
     * Factory function used to create XMLHttpRequest objects.
     *
     * Replace or decorate this service to create your own custom XMLHttpRequest objects.
     *
     * ```
     * angular.module('myApp', [])
     * .factory('$xhrFactory', function() {
     *   return function createXhr(method, url) {
     *     return new window.XMLHttpRequest({mozSystem: true});
     *   };
     * });
     * ```
     *
     * @param {string} method HTTP method of the request (GET, POST, PUT, ..)
     * @param {string} url URL of the request.
     */
    function $xhrFactoryProvider() {
      this.$get = function () {
        return function createXhr() {
          return new window.XMLHttpRequest();
        };
      };
    }

    /**
     * @ngdoc service
     * @name $httpBackend
     * @requires $jsonpCallbacks
     * @requires $document
     * @requires $xhrFactory
     * @this
     *
     * @description
     * HTTP backend used by the {@link ng.$http service} that delegates to
     * XMLHttpRequest object or JSONP and deals with browser incompatibilities.
     *
     * You should never need to use this service directly, instead use the higher-level abstractions:
     * {@link ng.$http $http} or {@link ngResource.$resource $resource}.
     *
     * During testing this implementation is swapped with {@link ngMock.$httpBackend mock
     * $httpBackend} which can be trained with responses.
     */
    function $HttpBackendProvider() {
      this.$get = ['$browser', '$jsonpCallbacks', '$document', '$xhrFactory', function ($browser, $jsonpCallbacks, $document, $xhrFactory) {
        return createHttpBackend($browser, $xhrFactory, $browser.defer, $jsonpCallbacks, $document[0]);
      }];
    }

    function createHttpBackend($browser, createXhr, $browserDefer, callbacks, rawDocument) {
      // TODO(vojta): fix the signature
      return function (method, url, post, callback, headers, timeout, withCredentials, responseType, eventHandlers, uploadEventHandlers) {
        url = url || $browser.url();

        if (lowercase(method) === 'jsonp') {
          var callbackPath = callbacks.createCallback(url);
          var jsonpDone = jsonpReq(url, callbackPath, function (status, text) {
            // jsonpReq only ever sets status to 200 (OK), 404 (ERROR) or -1 (WAITING)
            var response = status === 200 && callbacks.getResponse(callbackPath);
            completeRequest(callback, status, response, '', text);
            callbacks.removeCallback(callbackPath);
          });
        } else {

          var xhr = createXhr(method, url);

          xhr.open(method, url, true);
          forEach(headers, function (value, key) {
            if (isDefined(value)) {
              xhr.setRequestHeader(key, value);
            }
          });

          xhr.onload = function requestLoaded() {
            var statusText = xhr.statusText || '';

            // responseText is the old-school way of retrieving response (supported by IE9)
            // response/responseType properties were introduced in XHR Level2 spec (supported by IE10)
            var response = 'response' in xhr ? xhr.response : xhr.responseText;

            // normalize IE9 bug (http://bugs.jquery.com/ticket/1450)
            var status = xhr.status === 1223 ? 204 : xhr.status;

            // fix status code when it is 0 (0 status is undocumented).
            // Occurs when accessing file resources or on Android 4.1 stock browser
            // while retrieving files from application cache.
            if (status === 0) {
              status = response ? 200 : urlResolve(url).protocol === 'file' ? 404 : 0;
            }

            completeRequest(callback, status, response, xhr.getAllResponseHeaders(), statusText);
          };

          var requestError = function () {
            // The response is always empty
            // See https://xhr.spec.whatwg.org/#request-error-steps and https://fetch.spec.whatwg.org/#concept-network-error
            completeRequest(callback, -1, null, null, '');
          };

          xhr.onerror = requestError;
          xhr.onabort = requestError;
          xhr.ontimeout = requestError;

          forEach(eventHandlers, function (value, key) {
            xhr.addEventListener(key, value);
          });

          forEach(uploadEventHandlers, function (value, key) {
            xhr.upload.addEventListener(key, value);
          });

          if (withCredentials) {
            xhr.withCredentials = true;
          }

          if (responseType) {
            try {
              xhr.responseType = responseType;
            } catch (e) {
              // WebKit added support for the json responseType value on 09/03/2013
              // https://bugs.webkit.org/show_bug.cgi?id=73648. Versions of Safari prior to 7 are
              // known to throw when setting the value "json" as the response type. Other older
              // browsers implementing the responseType
              //
              // The json response type can be ignored if not supported, because JSON payloads are
              // parsed on the client-side regardless.
              if (responseType !== 'json') {
                throw e;
              }
            }
          }

          xhr.send(isUndefined(post) ? null : post);
        }

        if (timeout > 0) {
          var timeoutId = $browserDefer(timeoutRequest, timeout);
        } else if (isPromiseLike(timeout)) {
          timeout.then(timeoutRequest);
        }

        function timeoutRequest() {
          if (jsonpDone) {
            jsonpDone();
          }
          if (xhr) {
            xhr.abort();
          }
        }

        function completeRequest(callback, status, response, headersString, statusText) {
          // cancel timeout and subsequent timeout promise resolution
          if (isDefined(timeoutId)) {
            $browserDefer.cancel(timeoutId);
          }
          jsonpDone = xhr = null;

          callback(status, response, headersString, statusText);
        }
      };

      function jsonpReq(url, callbackPath, done) {
        url = url.replace('JSON_CALLBACK', callbackPath);
        // we can't use jQuery/jqLite here because jQuery does crazy stuff with script elements, e.g.:
        // - fetches local scripts via XHR and evals them
        // - adds and immediately removes script elements from the document
        var script = rawDocument.createElement('script'),
            callback = null;
        script.type = 'text/javascript';
        script.src = url;
        script.async = true;

        callback = function (event) {
          script.removeEventListener('load', callback);
          script.removeEventListener('error', callback);
          rawDocument.body.removeChild(script);
          script = null;
          var status = -1;
          var text = 'unknown';

          if (event) {
            if (event.type === 'load' && !callbacks.wasCalled(callbackPath)) {
              event = { type: 'error' };
            }
            text = event.type;
            status = event.type === 'error' ? 404 : 200;
          }

          if (done) {
            done(status, text);
          }
        };

        script.addEventListener('load', callback);
        script.addEventListener('error', callback);
        rawDocument.body.appendChild(script);
        return callback;
      }
    }

    var $interpolateMinErr = angular.$interpolateMinErr = minErr('$interpolate');
    $interpolateMinErr.throwNoconcat = function (text) {
      throw $interpolateMinErr('noconcat', 'Error while interpolating: {0}\nStrict Contextual Escaping disallows ' + 'interpolations that concatenate multiple expressions when a trusted value is ' + 'required.  See http://docs.angularjs.org/api/ng.$sce', text);
    };

    $interpolateMinErr.interr = function (text, err) {
      return $interpolateMinErr('interr', 'Can\'t interpolate: {0}\n{1}', text, err.toString());
    };

    /**
     * @ngdoc provider
     * @name $interpolateProvider
     * @this
     *
     * @description
     *
     * Used for configuring the interpolation markup. Defaults to `{{` and `}}`.
     *
     * <div class="alert alert-danger">
     * This feature is sometimes used to mix different markup languages, e.g. to wrap an Angular
     * template within a Python Jinja template (or any other template language). Mixing templating
     * languages is **very dangerous**. The embedding template language will not safely escape Angular
     * expressions, so any user-controlled values in the template will cause Cross Site Scripting (XSS)
     * security bugs!
     * </div>
     *
     * @example
    <example name="custom-interpolation-markup" module="customInterpolationApp">
    <file name="index.html">
    <script>
      var customInterpolationApp = angular.module('customInterpolationApp', []);
    
      customInterpolationApp.config(function($interpolateProvider) {
        $interpolateProvider.startSymbol('//');
        $interpolateProvider.endSymbol('//');
      });
    
    
      customInterpolationApp.controller('DemoController', function() {
          this.label = "This binding is brought you by // interpolation symbols.";
      });
    </script>
    <div ng-controller="DemoController as demo">
        //demo.label//
    </div>
    </file>
    <file name="protractor.js" type="protractor">
      it('should interpolate binding with custom symbols', function() {
        expect(element(by.binding('demo.label')).getText()).toBe('This binding is brought you by // interpolation symbols.');
      });
    </file>
    </example>
     */
    function $InterpolateProvider() {
      var startSymbol = '{{';
      var endSymbol = '}}';

      /**
       * @ngdoc method
       * @name $interpolateProvider#startSymbol
       * @description
       * Symbol to denote start of expression in the interpolated string. Defaults to `{{`.
       *
       * @param {string=} value new value to set the starting symbol to.
       * @returns {string|self} Returns the symbol when used as getter and self if used as setter.
       */
      this.startSymbol = function (value) {
        if (value) {
          startSymbol = value;
          return this;
        } else {
          return startSymbol;
        }
      };

      /**
       * @ngdoc method
       * @name $interpolateProvider#endSymbol
       * @description
       * Symbol to denote the end of expression in the interpolated string. Defaults to `}}`.
       *
       * @param {string=} value new value to set the ending symbol to.
       * @returns {string|self} Returns the symbol when used as getter and self if used as setter.
       */
      this.endSymbol = function (value) {
        if (value) {
          endSymbol = value;
          return this;
        } else {
          return endSymbol;
        }
      };

      this.$get = ['$parse', '$exceptionHandler', '$sce', function ($parse, $exceptionHandler, $sce) {
        var startSymbolLength = startSymbol.length,
            endSymbolLength = endSymbol.length,
            escapedStartRegexp = new RegExp(startSymbol.replace(/./g, escape), 'g'),
            escapedEndRegexp = new RegExp(endSymbol.replace(/./g, escape), 'g');

        function escape(ch) {
          return '\\\\\\' + ch;
        }

        function unescapeText(text) {
          return text.replace(escapedStartRegexp, startSymbol).replace(escapedEndRegexp, endSymbol);
        }

        // TODO: this is the same as the constantWatchDelegate in parse.js
        function constantWatchDelegate(scope, listener, objectEquality, constantInterp) {
          var unwatch = scope.$watch(function constantInterpolateWatch(scope) {
            unwatch();
            return constantInterp(scope);
          }, listener, objectEquality);
          return unwatch;
        }

        /**
         * @ngdoc service
         * @name $interpolate
         * @kind function
         *
         * @requires $parse
         * @requires $sce
         *
         * @description
         *
         * Compiles a string with markup into an interpolation function. This service is used by the
         * HTML {@link ng.$compile $compile} service for data binding. See
         * {@link ng.$interpolateProvider $interpolateProvider} for configuring the
         * interpolation markup.
         *
         *
         * ```js
         *   var $interpolate = ...; // injected
         *   var exp = $interpolate('Hello {{name | uppercase}}!');
         *   expect(exp({name:'Angular'})).toEqual('Hello ANGULAR!');
         * ```
         *
         * `$interpolate` takes an optional fourth argument, `allOrNothing`. If `allOrNothing` is
         * `true`, the interpolation function will return `undefined` unless all embedded expressions
         * evaluate to a value other than `undefined`.
         *
         * ```js
         *   var $interpolate = ...; // injected
         *   var context = {greeting: 'Hello', name: undefined };
         *
         *   // default "forgiving" mode
         *   var exp = $interpolate('{{greeting}} {{name}}!');
         *   expect(exp(context)).toEqual('Hello !');
         *
         *   // "allOrNothing" mode
         *   exp = $interpolate('{{greeting}} {{name}}!', false, null, true);
         *   expect(exp(context)).toBeUndefined();
         *   context.name = 'Angular';
         *   expect(exp(context)).toEqual('Hello Angular!');
         * ```
         *
         * `allOrNothing` is useful for interpolating URLs. `ngSrc` and `ngSrcset` use this behavior.
         *
         * #### Escaped Interpolation
         * $interpolate provides a mechanism for escaping interpolation markers. Start and end markers
         * can be escaped by preceding each of their characters with a REVERSE SOLIDUS U+005C (backslash).
         * It will be rendered as a regular start/end marker, and will not be interpreted as an expression
         * or binding.
         *
         * This enables web-servers to prevent script injection attacks and defacing attacks, to some
         * degree, while also enabling code examples to work without relying on the
         * {@link ng.directive:ngNonBindable ngNonBindable} directive.
         *
         * **For security purposes, it is strongly encouraged that web servers escape user-supplied data,
         * replacing angle brackets (&lt;, &gt;) with &amp;lt; and &amp;gt; respectively, and replacing all
         * interpolation start/end markers with their escaped counterparts.**
         *
         * Escaped interpolation markers are only replaced with the actual interpolation markers in rendered
         * output when the $interpolate service processes the text. So, for HTML elements interpolated
         * by {@link ng.$compile $compile}, or otherwise interpolated with the `mustHaveExpression` parameter
         * set to `true`, the interpolated text must contain an unescaped interpolation expression. As such,
         * this is typically useful only when user-data is used in rendering a template from the server, or
         * when otherwise untrusted data is used by a directive.
         *
         * <example name="interpolation">
         *  <file name="index.html">
         *    <div ng-init="username='A user'">
         *      <p ng-init="apptitle='Escaping demo'">{{apptitle}}: \{\{ username = "defaced value"; \}\}
         *        </p>
         *      <p><strong>{{username}}</strong> attempts to inject code which will deface the
         *        application, but fails to accomplish their task, because the server has correctly
         *        escaped the interpolation start/end markers with REVERSE SOLIDUS U+005C (backslash)
         *        characters.</p>
         *      <p>Instead, the result of the attempted script injection is visible, and can be removed
         *        from the database by an administrator.</p>
         *    </div>
         *  </file>
         * </example>
         *
         * @knownIssue
         * It is currently not possible for an interpolated expression to contain the interpolation end
         * symbol. For example, `{{ '}}' }}` will be incorrectly interpreted as `{{ ' }}` + `' }}`, i.e.
         * an interpolated expression consisting of a single-quote (`'`) and the `' }}` string.
         *
         * @knownIssue
         * All directives and components must use the standard `{{` `}}` interpolation symbols
         * in their templates. If you change the application interpolation symbols the {@link $compile}
         * service will attempt to denormalize the standard symbols to the custom symbols.
         * The denormalization process is not clever enough to know not to replace instances of the standard
         * symbols where they would not normally be treated as interpolation symbols. For example in the following
         * code snippet the closing braces of the literal object will get incorrectly denormalized:
         *
         * ```
         * <div data-context='{"context":{"id":3,"type":"page"}}">
         * ```
         *
         * The workaround is to ensure that such instances are separated by whitespace:
         * ```
         * <div data-context='{"context":{"id":3,"type":"page"} }">
         * ```
         *
         * See https://github.com/angular/angular.js/pull/14610#issuecomment-219401099 for more information.
         *
         * @param {string} text The text with markup to interpolate.
         * @param {boolean=} mustHaveExpression if set to true then the interpolation string must have
         *    embedded expression in order to return an interpolation function. Strings with no
         *    embedded expression will return null for the interpolation function.
         * @param {string=} trustedContext when provided, the returned function passes the interpolated
         *    result through {@link ng.$sce#getTrusted $sce.getTrusted(interpolatedResult,
         *    trustedContext)} before returning it.  Refer to the {@link ng.$sce $sce} service that
         *    provides Strict Contextual Escaping for details.
         * @param {boolean=} allOrNothing if `true`, then the returned function returns undefined
         *    unless all embedded expressions evaluate to a value other than `undefined`.
         * @returns {function(context)} an interpolation function which is used to compute the
         *    interpolated string. The function has these parameters:
         *
         * - `context`: evaluation context for all expressions embedded in the interpolated text
         */
        function $interpolate(text, mustHaveExpression, trustedContext, allOrNothing) {
          // Provide a quick exit and simplified result function for text with no interpolation
          if (!text.length || text.indexOf(startSymbol) === -1) {
            var constantInterp;
            if (!mustHaveExpression) {
              var unescapedText = unescapeText(text);
              constantInterp = valueFn(unescapedText);
              constantInterp.exp = text;
              constantInterp.expressions = [];
              constantInterp.$$watchDelegate = constantWatchDelegate;
            }
            return constantInterp;
          }

          allOrNothing = !!allOrNothing;
          var startIndex,
              endIndex,
              index = 0,
              expressions = [],
              parseFns = [],
              textLength = text.length,
              exp,
              concat = [],
              expressionPositions = [];

          while (index < textLength) {
            if ((startIndex = text.indexOf(startSymbol, index)) !== -1 && (endIndex = text.indexOf(endSymbol, startIndex + startSymbolLength)) !== -1) {
              if (index !== startIndex) {
                concat.push(unescapeText(text.substring(index, startIndex)));
              }
              exp = text.substring(startIndex + startSymbolLength, endIndex);
              expressions.push(exp);
              parseFns.push($parse(exp, parseStringifyInterceptor));
              index = endIndex + endSymbolLength;
              expressionPositions.push(concat.length);
              concat.push('');
            } else {
              // we did not find an interpolation, so we have to add the remainder to the separators array
              if (index !== textLength) {
                concat.push(unescapeText(text.substring(index)));
              }
              break;
            }
          }

          // Concatenating expressions makes it hard to reason about whether some combination of
          // concatenated values are unsafe to use and could easily lead to XSS.  By requiring that a
          // single expression be used for iframe[src], object[src], etc., we ensure that the value
          // that's used is assigned or constructed by some JS code somewhere that is more testable or
          // make it obvious that you bound the value to some user controlled value.  This helps reduce
          // the load when auditing for XSS issues.
          if (trustedContext && concat.length > 1) {
            $interpolateMinErr.throwNoconcat(text);
          }

          if (!mustHaveExpression || expressions.length) {
            var compute = function (values) {
              for (var i = 0, ii = expressions.length; i < ii; i++) {
                if (allOrNothing && isUndefined(values[i])) return;
                concat[expressionPositions[i]] = values[i];
              }
              return concat.join('');
            };

            var getValue = function (value) {
              return trustedContext ? $sce.getTrusted(trustedContext, value) : $sce.valueOf(value);
            };

            return extend(function interpolationFn(context) {
              var i = 0;
              var ii = expressions.length;
              var values = new Array(ii);

              try {
                for (; i < ii; i++) {
                  values[i] = parseFns[i](context);
                }

                return compute(values);
              } catch (err) {
                $exceptionHandler($interpolateMinErr.interr(text, err));
              }
            }, {
              // all of these properties are undocumented for now
              exp: text, //just for compatibility with regular watchers created via $watch
              expressions: expressions,
              $$watchDelegate: function (scope, listener) {
                var lastValue;
                return scope.$watchGroup(parseFns, /** @this */function interpolateFnWatcher(values, oldValues) {
                  var currValue = compute(values);
                  if (isFunction(listener)) {
                    listener.call(this, currValue, values !== oldValues ? lastValue : currValue, scope);
                  }
                  lastValue = currValue;
                });
              }
            });
          }

          function parseStringifyInterceptor(value) {
            try {
              value = getValue(value);
              return allOrNothing && !isDefined(value) ? value : stringify(value);
            } catch (err) {
              $exceptionHandler($interpolateMinErr.interr(text, err));
            }
          }
        }

        /**
         * @ngdoc method
         * @name $interpolate#startSymbol
         * @description
         * Symbol to denote the start of expression in the interpolated string. Defaults to `{{`.
         *
         * Use {@link ng.$interpolateProvider#startSymbol `$interpolateProvider.startSymbol`} to change
         * the symbol.
         *
         * @returns {string} start symbol.
         */
        $interpolate.startSymbol = function () {
          return startSymbol;
        };

        /**
         * @ngdoc method
         * @name $interpolate#endSymbol
         * @description
         * Symbol to denote the end of expression in the interpolated string. Defaults to `}}`.
         *
         * Use {@link ng.$interpolateProvider#endSymbol `$interpolateProvider.endSymbol`} to change
         * the symbol.
         *
         * @returns {string} end symbol.
         */
        $interpolate.endSymbol = function () {
          return endSymbol;
        };

        return $interpolate;
      }];
    }

    /** @this */
    function $IntervalProvider() {
      this.$get = ['$rootScope', '$window', '$q', '$$q', '$browser', function ($rootScope, $window, $q, $$q, $browser) {
        var intervals = {};

        /**
         * @ngdoc service
         * @name $interval
         *
         * @description
         * Angular's wrapper for `window.setInterval`. The `fn` function is executed every `delay`
         * milliseconds.
         *
         * The return value of registering an interval function is a promise. This promise will be
         * notified upon each tick of the interval, and will be resolved after `count` iterations, or
         * run indefinitely if `count` is not defined. The value of the notification will be the
         * number of iterations that have run.
         * To cancel an interval, call `$interval.cancel(promise)`.
         *
         * In tests you can use {@link ngMock.$interval#flush `$interval.flush(millis)`} to
         * move forward by `millis` milliseconds and trigger any functions scheduled to run in that
         * time.
         *
         * <div class="alert alert-warning">
         * **Note**: Intervals created by this service must be explicitly destroyed when you are finished
         * with them.  In particular they are not automatically destroyed when a controller's scope or a
         * directive's element are destroyed.
         * You should take this into consideration and make sure to always cancel the interval at the
         * appropriate moment.  See the example below for more details on how and when to do this.
         * </div>
         *
         * @param {function()} fn A function that should be called repeatedly. If no additional arguments
         *   are passed (see below), the function is called with the current iteration count.
         * @param {number} delay Number of milliseconds between each function call.
         * @param {number=} [count=0] Number of times to repeat. If not set, or 0, will repeat
         *   indefinitely.
         * @param {boolean=} [invokeApply=true] If set to `false` skips model dirty checking, otherwise
         *   will invoke `fn` within the {@link ng.$rootScope.Scope#$apply $apply} block.
         * @param {...*=} Pass additional parameters to the executed function.
         * @returns {promise} A promise which will be notified on each iteration.
         *
         * @example
         * <example module="intervalExample" name="interval-service">
         * <file name="index.html">
         *   <script>
         *     angular.module('intervalExample', [])
         *       .controller('ExampleController', ['$scope', '$interval',
         *         function($scope, $interval) {
         *           $scope.format = 'M/d/yy h:mm:ss a';
         *           $scope.blood_1 = 100;
         *           $scope.blood_2 = 120;
         *
         *           var stop;
         *           $scope.fight = function() {
         *             // Don't start a new fight if we are already fighting
         *             if ( angular.isDefined(stop) ) return;
         *
         *             stop = $interval(function() {
         *               if ($scope.blood_1 > 0 && $scope.blood_2 > 0) {
         *                 $scope.blood_1 = $scope.blood_1 - 3;
         *                 $scope.blood_2 = $scope.blood_2 - 4;
         *               } else {
         *                 $scope.stopFight();
         *               }
         *             }, 100);
         *           };
         *
         *           $scope.stopFight = function() {
         *             if (angular.isDefined(stop)) {
         *               $interval.cancel(stop);
         *               stop = undefined;
         *             }
         *           };
         *
         *           $scope.resetFight = function() {
         *             $scope.blood_1 = 100;
         *             $scope.blood_2 = 120;
         *           };
         *
         *           $scope.$on('$destroy', function() {
         *             // Make sure that the interval is destroyed too
         *             $scope.stopFight();
         *           });
         *         }])
         *       // Register the 'myCurrentTime' directive factory method.
         *       // We inject $interval and dateFilter service since the factory method is DI.
         *       .directive('myCurrentTime', ['$interval', 'dateFilter',
         *         function($interval, dateFilter) {
         *           // return the directive link function. (compile function not needed)
         *           return function(scope, element, attrs) {
         *             var format,  // date format
         *                 stopTime; // so that we can cancel the time updates
         *
         *             // used to update the UI
         *             function updateTime() {
         *               element.text(dateFilter(new Date(), format));
         *             }
         *
         *             // watch the expression, and update the UI on change.
         *             scope.$watch(attrs.myCurrentTime, function(value) {
         *               format = value;
         *               updateTime();
         *             });
         *
         *             stopTime = $interval(updateTime, 1000);
         *
         *             // listen on DOM destroy (removal) event, and cancel the next UI update
         *             // to prevent updating time after the DOM element was removed.
         *             element.on('$destroy', function() {
         *               $interval.cancel(stopTime);
         *             });
         *           }
         *         }]);
         *   </script>
         *
         *   <div>
         *     <div ng-controller="ExampleController">
         *       <label>Date format: <input ng-model="format"></label> <hr/>
         *       Current time is: <span my-current-time="format"></span>
         *       <hr/>
         *       Blood 1 : <font color='red'>{{blood_1}}</font>
         *       Blood 2 : <font color='red'>{{blood_2}}</font>
         *       <button type="button" data-ng-click="fight()">Fight</button>
         *       <button type="button" data-ng-click="stopFight()">StopFight</button>
         *       <button type="button" data-ng-click="resetFight()">resetFight</button>
         *     </div>
         *   </div>
         *
         * </file>
         * </example>
         */
        function interval(fn, delay, count, invokeApply) {
          var hasParams = arguments.length > 4,
              args = hasParams ? sliceArgs(arguments, 4) : [],
              setInterval = $window.setInterval,
              clearInterval = $window.clearInterval,
              iteration = 0,
              skipApply = isDefined(invokeApply) && !invokeApply,
              deferred = (skipApply ? $$q : $q).defer(),
              promise = deferred.promise;

          count = isDefined(count) ? count : 0;

          promise.$$intervalId = setInterval(function tick() {
            if (skipApply) {
              $browser.defer(callback);
            } else {
              $rootScope.$evalAsync(callback);
            }
            deferred.notify(iteration++);

            if (count > 0 && iteration >= count) {
              deferred.resolve(iteration);
              clearInterval(promise.$$intervalId);
              delete intervals[promise.$$intervalId];
            }

            if (!skipApply) $rootScope.$apply();
          }, delay);

          intervals[promise.$$intervalId] = deferred;

          return promise;

          function callback() {
            if (!hasParams) {
              fn(iteration);
            } else {
              fn.apply(null, args);
            }
          }
        }

        /**
         * @ngdoc method
         * @name $interval#cancel
         *
         * @description
         * Cancels a task associated with the `promise`.
         *
         * @param {Promise=} promise returned by the `$interval` function.
         * @returns {boolean} Returns `true` if the task was successfully canceled.
         */
        interval.cancel = function (promise) {
          if (promise && promise.$$intervalId in intervals) {
            // Interval cancels should not report as unhandled promise.
            intervals[promise.$$intervalId].promise.catch(noop);
            intervals[promise.$$intervalId].reject('canceled');
            $window.clearInterval(promise.$$intervalId);
            delete intervals[promise.$$intervalId];
            return true;
          }
          return false;
        };

        return interval;
      }];
    }

    /**
     * @ngdoc service
     * @name $jsonpCallbacks
     * @requires $window
     * @description
     * This service handles the lifecycle of callbacks to handle JSONP requests.
     * Override this service if you wish to customise where the callbacks are stored and
     * how they vary compared to the requested url.
     */
    var $jsonpCallbacksProvider = /** @this */function () {
      this.$get = function () {
        var callbacks = angular.callbacks;
        var callbackMap = {};

        function createCallback(callbackId) {
          var callback = function (data) {
            callback.data = data;
            callback.called = true;
          };
          callback.id = callbackId;
          return callback;
        }

        return {
          /**
           * @ngdoc method
           * @name $jsonpCallbacks#createCallback
           * @param {string} url the url of the JSONP request
           * @returns {string} the callback path to send to the server as part of the JSONP request
           * @description
           * {@link $httpBackend} calls this method to create a callback and get hold of the path to the callback
           * to pass to the server, which will be used to call the callback with its payload in the JSONP response.
           */
          createCallback: function (url) {
            var callbackId = '_' + (callbacks.$$counter++).toString(36);
            var callbackPath = 'angular.callbacks.' + callbackId;
            var callback = createCallback(callbackId);
            callbackMap[callbackPath] = callbacks[callbackId] = callback;
            return callbackPath;
          },
          /**
           * @ngdoc method
           * @name $jsonpCallbacks#wasCalled
           * @param {string} callbackPath the path to the callback that was sent in the JSONP request
           * @returns {boolean} whether the callback has been called, as a result of the JSONP response
           * @description
           * {@link $httpBackend} calls this method to find out whether the JSONP response actually called the
           * callback that was passed in the request.
           */
          wasCalled: function (callbackPath) {
            return callbackMap[callbackPath].called;
          },
          /**
           * @ngdoc method
           * @name $jsonpCallbacks#getResponse
           * @param {string} callbackPath the path to the callback that was sent in the JSONP request
           * @returns {*} the data received from the response via the registered callback
           * @description
           * {@link $httpBackend} calls this method to get hold of the data that was provided to the callback
           * in the JSONP response.
           */
          getResponse: function (callbackPath) {
            return callbackMap[callbackPath].data;
          },
          /**
           * @ngdoc method
           * @name $jsonpCallbacks#removeCallback
           * @param {string} callbackPath the path to the callback that was sent in the JSONP request
           * @description
           * {@link $httpBackend} calls this method to remove the callback after the JSONP request has
           * completed or timed-out.
           */
          removeCallback: function (callbackPath) {
            var callback = callbackMap[callbackPath];
            delete callbacks[callback.id];
            delete callbackMap[callbackPath];
          }
        };
      };
    };

    /**
     * @ngdoc service
     * @name $locale
     *
     * @description
     * $locale service provides localization rules for various Angular components. As of right now the
     * only public api is:
     *
     * * `id` – `{string}` – locale id formatted as `languageId-countryId` (e.g. `en-us`)
     */

    var PATH_MATCH = /^([^?#]*)(\?([^#]*))?(#(.*))?$/,
        DEFAULT_PORTS = { 'http': 80, 'https': 443, 'ftp': 21 };
    var $locationMinErr = minErr('$location');

    /**
     * Encode path using encodeUriSegment, ignoring forward slashes
     *
     * @param {string} path Path to encode
     * @returns {string}
     */
    function encodePath(path) {
      var segments = path.split('/'),
          i = segments.length;

      while (i--) {
        segments[i] = encodeUriSegment(segments[i]);
      }

      return segments.join('/');
    }

    function parseAbsoluteUrl(absoluteUrl, locationObj) {
      var parsedUrl = urlResolve(absoluteUrl);

      locationObj.$$protocol = parsedUrl.protocol;
      locationObj.$$host = parsedUrl.hostname;
      locationObj.$$port = toInt(parsedUrl.port) || DEFAULT_PORTS[parsedUrl.protocol] || null;
    }

    var DOUBLE_SLASH_REGEX = /^\s*[\\/]{2,}/;
    function parseAppUrl(url, locationObj) {

      if (DOUBLE_SLASH_REGEX.test(url)) {
        throw $locationMinErr('badpath', 'Invalid url "{0}".', url);
      }

      var prefixed = url.charAt(0) !== '/';
      if (prefixed) {
        url = '/' + url;
      }
      var match = urlResolve(url);
      locationObj.$$path = decodeURIComponent(prefixed && match.pathname.charAt(0) === '/' ? match.pathname.substring(1) : match.pathname);
      locationObj.$$search = parseKeyValue(match.search);
      locationObj.$$hash = decodeURIComponent(match.hash);

      // make sure path starts with '/';
      if (locationObj.$$path && locationObj.$$path.charAt(0) !== '/') {
        locationObj.$$path = '/' + locationObj.$$path;
      }
    }

    function startsWith(str, search) {
      return str.slice(0, search.length) === search;
    }

    /**
     *
     * @param {string} base
     * @param {string} url
     * @returns {string} returns text from `url` after `base` or `undefined` if it does not begin with
     *                   the expected string.
     */
    function stripBaseUrl(base, url) {
      if (startsWith(url, base)) {
        return url.substr(base.length);
      }
    }

    function stripHash(url) {
      var index = url.indexOf('#');
      return index === -1 ? url : url.substr(0, index);
    }

    function trimEmptyHash(url) {
      return url.replace(/(#.+)|#$/, '$1');
    }

    function stripFile(url) {
      return url.substr(0, stripHash(url).lastIndexOf('/') + 1);
    }

    /* return the server only (scheme://host:port) */
    function serverBase(url) {
      return url.substring(0, url.indexOf('/', url.indexOf('//') + 2));
    }

    /**
     * LocationHtml5Url represents a URL
     * This object is exposed as $location service when HTML5 mode is enabled and supported
     *
     * @constructor
     * @param {string} appBase application base URL
     * @param {string} appBaseNoFile application base URL stripped of any filename
     * @param {string} basePrefix URL path prefix
     */
    function LocationHtml5Url(appBase, appBaseNoFile, basePrefix) {
      this.$$html5 = true;
      basePrefix = basePrefix || '';
      parseAbsoluteUrl(appBase, this);

      /**
       * Parse given HTML5 (regular) URL string into properties
       * @param {string} url HTML5 URL
       * @private
       */
      this.$$parse = function (url) {
        var pathUrl = stripBaseUrl(appBaseNoFile, url);
        if (!isString(pathUrl)) {
          throw $locationMinErr('ipthprfx', 'Invalid url "{0}", missing path prefix "{1}".', url, appBaseNoFile);
        }

        parseAppUrl(pathUrl, this);

        if (!this.$$path) {
          this.$$path = '/';
        }

        this.$$compose();
      };

      /**
       * Compose url and update `absUrl` property
       * @private
       */
      this.$$compose = function () {
        var search = toKeyValue(this.$$search),
            hash = this.$$hash ? '#' + encodeUriSegment(this.$$hash) : '';

        this.$$url = encodePath(this.$$path) + (search ? '?' + search : '') + hash;
        this.$$absUrl = appBaseNoFile + this.$$url.substr(1); // first char is always '/'

        this.$$urlUpdatedByLocation = true;
      };

      this.$$parseLinkUrl = function (url, relHref) {
        if (relHref && relHref[0] === '#') {
          // special case for links to hash fragments:
          // keep the old url and only replace the hash fragment
          this.hash(relHref.slice(1));
          return true;
        }
        var appUrl, prevAppUrl;
        var rewrittenUrl;

        if (isDefined(appUrl = stripBaseUrl(appBase, url))) {
          prevAppUrl = appUrl;
          if (basePrefix && isDefined(appUrl = stripBaseUrl(basePrefix, appUrl))) {
            rewrittenUrl = appBaseNoFile + (stripBaseUrl('/', appUrl) || appUrl);
          } else {
            rewrittenUrl = appBase + prevAppUrl;
          }
        } else if (isDefined(appUrl = stripBaseUrl(appBaseNoFile, url))) {
          rewrittenUrl = appBaseNoFile + appUrl;
        } else if (appBaseNoFile === url + '/') {
          rewrittenUrl = appBaseNoFile;
        }
        if (rewrittenUrl) {
          this.$$parse(rewrittenUrl);
        }
        return !!rewrittenUrl;
      };
    }

    /**
     * LocationHashbangUrl represents URL
     * This object is exposed as $location service when developer doesn't opt into html5 mode.
     * It also serves as the base class for html5 mode fallback on legacy browsers.
     *
     * @constructor
     * @param {string} appBase application base URL
     * @param {string} appBaseNoFile application base URL stripped of any filename
     * @param {string} hashPrefix hashbang prefix
     */
    function LocationHashbangUrl(appBase, appBaseNoFile, hashPrefix) {

      parseAbsoluteUrl(appBase, this);

      /**
       * Parse given hashbang URL into properties
       * @param {string} url Hashbang URL
       * @private
       */
      this.$$parse = function (url) {
        var withoutBaseUrl = stripBaseUrl(appBase, url) || stripBaseUrl(appBaseNoFile, url);
        var withoutHashUrl;

        if (!isUndefined(withoutBaseUrl) && withoutBaseUrl.charAt(0) === '#') {

          // The rest of the URL starts with a hash so we have
          // got either a hashbang path or a plain hash fragment
          withoutHashUrl = stripBaseUrl(hashPrefix, withoutBaseUrl);
          if (isUndefined(withoutHashUrl)) {
            // There was no hashbang prefix so we just have a hash fragment
            withoutHashUrl = withoutBaseUrl;
          }
        } else {
          // There was no hashbang path nor hash fragment:
          // If we are in HTML5 mode we use what is left as the path;
          // Otherwise we ignore what is left
          if (this.$$html5) {
            withoutHashUrl = withoutBaseUrl;
          } else {
            withoutHashUrl = '';
            if (isUndefined(withoutBaseUrl)) {
              appBase = url;
              /** @type {?} */this.replace();
            }
          }
        }

        parseAppUrl(withoutHashUrl, this);

        this.$$path = removeWindowsDriveName(this.$$path, withoutHashUrl, appBase);

        this.$$compose();

        /*
         * In Windows, on an anchor node on documents loaded from
         * the filesystem, the browser will return a pathname
         * prefixed with the drive name ('/C:/path') when a
         * pathname without a drive is set:
         *  * a.setAttribute('href', '/foo')
         *   * a.pathname === '/C:/foo' //true
         *
         * Inside of Angular, we're always using pathnames that
         * do not include drive names for routing.
         */
        function removeWindowsDriveName(path, url, base) {
          /*
          Matches paths for file protocol on windows,
          such as /C:/foo/bar, and captures only /foo/bar.
          */
          var windowsFilePathExp = /^\/[A-Z]:(\/.*)/;

          var firstPathSegmentMatch;

          //Get the relative path from the input URL.
          if (startsWith(url, base)) {
            url = url.replace(base, '');
          }

          // The input URL intentionally contains a first path segment that ends with a colon.
          if (windowsFilePathExp.exec(url)) {
            return path;
          }

          firstPathSegmentMatch = windowsFilePathExp.exec(path);
          return firstPathSegmentMatch ? firstPathSegmentMatch[1] : path;
        }
      };

      /**
       * Compose hashbang URL and update `absUrl` property
       * @private
       */
      this.$$compose = function () {
        var search = toKeyValue(this.$$search),
            hash = this.$$hash ? '#' + encodeUriSegment(this.$$hash) : '';

        this.$$url = encodePath(this.$$path) + (search ? '?' + search : '') + hash;
        this.$$absUrl = appBase + (this.$$url ? hashPrefix + this.$$url : '');

        this.$$urlUpdatedByLocation = true;
      };

      this.$$parseLinkUrl = function (url, relHref) {
        if (stripHash(appBase) === stripHash(url)) {
          this.$$parse(url);
          return true;
        }
        return false;
      };
    }

    /**
     * LocationHashbangUrl represents URL
     * This object is exposed as $location service when html5 history api is enabled but the browser
     * does not support it.
     *
     * @constructor
     * @param {string} appBase application base URL
     * @param {string} appBaseNoFile application base URL stripped of any filename
     * @param {string} hashPrefix hashbang prefix
     */
    function LocationHashbangInHtml5Url(appBase, appBaseNoFile, hashPrefix) {
      this.$$html5 = true;
      LocationHashbangUrl.apply(this, arguments);

      this.$$parseLinkUrl = function (url, relHref) {
        if (relHref && relHref[0] === '#') {
          // special case for links to hash fragments:
          // keep the old url and only replace the hash fragment
          this.hash(relHref.slice(1));
          return true;
        }

        var rewrittenUrl;
        var appUrl;

        if (appBase === stripHash(url)) {
          rewrittenUrl = url;
        } else if (appUrl = stripBaseUrl(appBaseNoFile, url)) {
          rewrittenUrl = appBase + hashPrefix + appUrl;
        } else if (appBaseNoFile === url + '/') {
          rewrittenUrl = appBaseNoFile;
        }
        if (rewrittenUrl) {
          this.$$parse(rewrittenUrl);
        }
        return !!rewrittenUrl;
      };

      this.$$compose = function () {
        var search = toKeyValue(this.$$search),
            hash = this.$$hash ? '#' + encodeUriSegment(this.$$hash) : '';

        this.$$url = encodePath(this.$$path) + (search ? '?' + search : '') + hash;
        // include hashPrefix in $$absUrl when $$url is empty so IE9 does not reload page because of removal of '#'
        this.$$absUrl = appBase + hashPrefix + this.$$url;

        this.$$urlUpdatedByLocation = true;
      };
    }

    var locationPrototype = {

      /**
       * Ensure absolute URL is initialized.
       * @private
       */
      $$absUrl: '',

      /**
       * Are we in html5 mode?
       * @private
       */
      $$html5: false,

      /**
       * Has any change been replacing?
       * @private
       */
      $$replace: false,

      /**
       * @ngdoc method
       * @name $location#absUrl
       *
       * @description
       * This method is getter only.
       *
       * Return full URL representation with all segments encoded according to rules specified in
       * [RFC 3986](http://www.ietf.org/rfc/rfc3986.txt).
       *
       *
       * ```js
       * // given URL http://example.com/#/some/path?foo=bar&baz=xoxo
       * var absUrl = $location.absUrl();
       * // => "http://example.com/#/some/path?foo=bar&baz=xoxo"
       * ```
       *
       * @return {string} full URL
       */
      absUrl: locationGetter('$$absUrl'),

      /**
       * @ngdoc method
       * @name $location#url
       *
       * @description
       * This method is getter / setter.
       *
       * Return URL (e.g. `/path?a=b#hash`) when called without any parameter.
       *
       * Change path, search and hash, when called with parameter and return `$location`.
       *
       *
       * ```js
       * // given URL http://example.com/#/some/path?foo=bar&baz=xoxo
       * var url = $location.url();
       * // => "/some/path?foo=bar&baz=xoxo"
       * ```
       *
       * @param {string=} url New URL without base prefix (e.g. `/path?a=b#hash`)
       * @return {string} url
       */
      url: function (url) {
        if (isUndefined(url)) {
          return this.$$url;
        }

        var match = PATH_MATCH.exec(url);
        if (match[1] || url === '') this.path(decodeURIComponent(match[1]));
        if (match[2] || match[1] || url === '') this.search(match[3] || '');
        this.hash(match[5] || '');

        return this;
      },

      /**
       * @ngdoc method
       * @name $location#protocol
       *
       * @description
       * This method is getter only.
       *
       * Return protocol of current URL.
       *
       *
       * ```js
       * // given URL http://example.com/#/some/path?foo=bar&baz=xoxo
       * var protocol = $location.protocol();
       * // => "http"
       * ```
       *
       * @return {string} protocol of current URL
       */
      protocol: locationGetter('$$protocol'),

      /**
       * @ngdoc method
       * @name $location#host
       *
       * @description
       * This method is getter only.
       *
       * Return host of current URL.
       *
       * Note: compared to the non-angular version `location.host` which returns `hostname:port`, this returns the `hostname` portion only.
       *
       *
       * ```js
       * // given URL http://example.com/#/some/path?foo=bar&baz=xoxo
       * var host = $location.host();
       * // => "example.com"
       *
       * // given URL http://user:password@example.com:8080/#/some/path?foo=bar&baz=xoxo
       * host = $location.host();
       * // => "example.com"
       * host = location.host;
       * // => "example.com:8080"
       * ```
       *
       * @return {string} host of current URL.
       */
      host: locationGetter('$$host'),

      /**
       * @ngdoc method
       * @name $location#port
       *
       * @description
       * This method is getter only.
       *
       * Return port of current URL.
       *
       *
       * ```js
       * // given URL http://example.com/#/some/path?foo=bar&baz=xoxo
       * var port = $location.port();
       * // => 80
       * ```
       *
       * @return {Number} port
       */
      port: locationGetter('$$port'),

      /**
       * @ngdoc method
       * @name $location#path
       *
       * @description
       * This method is getter / setter.
       *
       * Return path of current URL when called without any parameter.
       *
       * Change path when called with parameter and return `$location`.
       *
       * Note: Path should always begin with forward slash (/), this method will add the forward slash
       * if it is missing.
       *
       *
       * ```js
       * // given URL http://example.com/#/some/path?foo=bar&baz=xoxo
       * var path = $location.path();
       * // => "/some/path"
       * ```
       *
       * @param {(string|number)=} path New path
       * @return {(string|object)} path if called with no parameters, or `$location` if called with a parameter
       */
      path: locationGetterSetter('$$path', function (path) {
        path = path !== null ? path.toString() : '';
        return path.charAt(0) === '/' ? path : '/' + path;
      }),

      /**
       * @ngdoc method
       * @name $location#search
       *
       * @description
       * This method is getter / setter.
       *
       * Return search part (as object) of current URL when called without any parameter.
       *
       * Change search part when called with parameter and return `$location`.
       *
       *
       * ```js
       * // given URL http://example.com/#/some/path?foo=bar&baz=xoxo
       * var searchObject = $location.search();
       * // => {foo: 'bar', baz: 'xoxo'}
       *
       * // set foo to 'yipee'
       * $location.search('foo', 'yipee');
       * // $location.search() => {foo: 'yipee', baz: 'xoxo'}
       * ```
       *
       * @param {string|Object.<string>|Object.<Array.<string>>} search New search params - string or
       * hash object.
       *
       * When called with a single argument the method acts as a setter, setting the `search` component
       * of `$location` to the specified value.
       *
       * If the argument is a hash object containing an array of values, these values will be encoded
       * as duplicate search parameters in the URL.
       *
       * @param {(string|Number|Array<string>|boolean)=} paramValue If `search` is a string or number, then `paramValue`
       * will override only a single search property.
       *
       * If `paramValue` is an array, it will override the property of the `search` component of
       * `$location` specified via the first argument.
       *
       * If `paramValue` is `null`, the property specified via the first argument will be deleted.
       *
       * If `paramValue` is `true`, the property specified via the first argument will be added with no
       * value nor trailing equal sign.
       *
       * @return {Object} If called with no arguments returns the parsed `search` object. If called with
       * one or more arguments returns `$location` object itself.
       */
      search: function (search, paramValue) {
        switch (arguments.length) {
          case 0:
            return this.$$search;
          case 1:
            if (isString(search) || isNumber(search)) {
              search = search.toString();
              this.$$search = parseKeyValue(search);
            } else if (isObject(search)) {
              search = copy(search, {});
              // remove object undefined or null properties
              forEach(search, function (value, key) {
                if (value == null) delete search[key];
              });

              this.$$search = search;
            } else {
              throw $locationMinErr('isrcharg', 'The first argument of the `$location#search()` call must be a string or an object.');
            }
            break;
          default:
            if (isUndefined(paramValue) || paramValue === null) {
              delete this.$$search[search];
            } else {
              this.$$search[search] = paramValue;
            }
        }

        this.$$compose();
        return this;
      },

      /**
       * @ngdoc method
       * @name $location#hash
       *
       * @description
       * This method is getter / setter.
       *
       * Returns the hash fragment when called without any parameters.
       *
       * Changes the hash fragment when called with a parameter and returns `$location`.
       *
       *
       * ```js
       * // given URL http://example.com/#/some/path?foo=bar&baz=xoxo#hashValue
       * var hash = $location.hash();
       * // => "hashValue"
       * ```
       *
       * @param {(string|number)=} hash New hash fragment
       * @return {string} hash
       */
      hash: locationGetterSetter('$$hash', function (hash) {
        return hash !== null ? hash.toString() : '';
      }),

      /**
       * @ngdoc method
       * @name $location#replace
       *
       * @description
       * If called, all changes to $location during the current `$digest` will replace the current history
       * record, instead of adding a new one.
       */
      replace: function () {
        this.$$replace = true;
        return this;
      }
    };

    forEach([LocationHashbangInHtml5Url, LocationHashbangUrl, LocationHtml5Url], function (Location) {
      Location.prototype = Object.create(locationPrototype);

      /**
       * @ngdoc method
       * @name $location#state
       *
       * @description
       * This method is getter / setter.
       *
       * Return the history state object when called without any parameter.
       *
       * Change the history state object when called with one parameter and return `$location`.
       * The state object is later passed to `pushState` or `replaceState`.
       *
       * NOTE: This method is supported only in HTML5 mode and only in browsers supporting
       * the HTML5 History API (i.e. methods `pushState` and `replaceState`). If you need to support
       * older browsers (like IE9 or Android < 4.0), don't use this method.
       *
       * @param {object=} state State object for pushState or replaceState
       * @return {object} state
       */
      Location.prototype.state = function (state) {
        if (!arguments.length) {
          return this.$$state;
        }

        if (Location !== LocationHtml5Url || !this.$$html5) {
          throw $locationMinErr('nostate', 'History API state support is available only ' + 'in HTML5 mode and only in browsers supporting HTML5 History API');
        }
        // The user might modify `stateObject` after invoking `$location.state(stateObject)`
        // but we're changing the $$state reference to $browser.state() during the $digest
        // so the modification window is narrow.
        this.$$state = isUndefined(state) ? null : state;
        this.$$urlUpdatedByLocation = true;

        return this;
      };
    });

    function locationGetter(property) {
      return (/** @this */function () {
          return this[property];
        }
      );
    }

    function locationGetterSetter(property, preprocess) {
      return (/** @this */function (value) {
          if (isUndefined(value)) {
            return this[property];
          }

          this[property] = preprocess(value);
          this.$$compose();

          return this;
        }
      );
    }

    /**
     * @ngdoc service
     * @name $location
     *
     * @requires $rootElement
     *
     * @description
     * The $location service parses the URL in the browser address bar (based on the
     * [window.location](https://developer.mozilla.org/en/window.location)) and makes the URL
     * available to your application. Changes to the URL in the address bar are reflected into
     * $location service and changes to $location are reflected into the browser address bar.
     *
     * **The $location service:**
     *
     * - Exposes the current URL in the browser address bar, so you can
     *   - Watch and observe the URL.
     *   - Change the URL.
     * - Synchronizes the URL with the browser when the user
     *   - Changes the address bar.
     *   - Clicks the back or forward button (or clicks a History link).
     *   - Clicks on a link.
     * - Represents the URL object as a set of methods (protocol, host, port, path, search, hash).
     *
     * For more information see {@link guide/$location Developer Guide: Using $location}
     */

    /**
     * @ngdoc provider
     * @name $locationProvider
     * @this
     *
     * @description
     * Use the `$locationProvider` to configure how the application deep linking paths are stored.
     */
    function $LocationProvider() {
      var hashPrefix = '!',
          html5Mode = {
        enabled: false,
        requireBase: true,
        rewriteLinks: true
      };

      /**
       * @ngdoc method
       * @name $locationProvider#hashPrefix
       * @description
       * The default value for the prefix is `'!'`.
       * @param {string=} prefix Prefix for hash part (containing path and search)
       * @returns {*} current value if used as getter or itself (chaining) if used as setter
       */
      this.hashPrefix = function (prefix) {
        if (isDefined(prefix)) {
          hashPrefix = prefix;
          return this;
        } else {
          return hashPrefix;
        }
      };

      /**
       * @ngdoc method
       * @name $locationProvider#html5Mode
       * @description
       * @param {(boolean|Object)=} mode If boolean, sets `html5Mode.enabled` to value.
       *   If object, sets `enabled`, `requireBase` and `rewriteLinks` to respective values. Supported
       *   properties:
       *   - **enabled** – `{boolean}` – (default: false) If true, will rely on `history.pushState` to
       *     change urls where supported. Will fall back to hash-prefixed paths in browsers that do not
       *     support `pushState`.
       *   - **requireBase** - `{boolean}` - (default: `true`) When html5Mode is enabled, specifies
       *     whether or not a <base> tag is required to be present. If `enabled` and `requireBase` are
       *     true, and a base tag is not present, an error will be thrown when `$location` is injected.
       *     See the {@link guide/$location $location guide for more information}
       *   - **rewriteLinks** - `{boolean|string}` - (default: `true`) When html5Mode is enabled,
       *     enables/disables URL rewriting for relative links. If set to a string, URL rewriting will
       *     only happen on links with an attribute that matches the given string. For example, if set
       *     to `'internal-link'`, then the URL will only be rewritten for `<a internal-link>` links.
       *     Note that [attribute name normalization](guide/directive#normalization) does not apply
       *     here, so `'internalLink'` will **not** match `'internal-link'`.
       *
       * @returns {Object} html5Mode object if used as getter or itself (chaining) if used as setter
       */
      this.html5Mode = function (mode) {
        if (isBoolean(mode)) {
          html5Mode.enabled = mode;
          return this;
        } else if (isObject(mode)) {

          if (isBoolean(mode.enabled)) {
            html5Mode.enabled = mode.enabled;
          }

          if (isBoolean(mode.requireBase)) {
            html5Mode.requireBase = mode.requireBase;
          }

          if (isBoolean(mode.rewriteLinks) || isString(mode.rewriteLinks)) {
            html5Mode.rewriteLinks = mode.rewriteLinks;
          }

          return this;
        } else {
          return html5Mode;
        }
      };

      /**
       * @ngdoc event
       * @name $location#$locationChangeStart
       * @eventType broadcast on root scope
       * @description
       * Broadcasted before a URL will change.
       *
       * This change can be prevented by calling
       * `preventDefault` method of the event. See {@link ng.$rootScope.Scope#$on} for more
       * details about event object. Upon successful change
       * {@link ng.$location#$locationChangeSuccess $locationChangeSuccess} is fired.
       *
       * The `newState` and `oldState` parameters may be defined only in HTML5 mode and when
       * the browser supports the HTML5 History API.
       *
       * @param {Object} angularEvent Synthetic event object.
       * @param {string} newUrl New URL
       * @param {string=} oldUrl URL that was before it was changed.
       * @param {string=} newState New history state object
       * @param {string=} oldState History state object that was before it was changed.
       */

      /**
       * @ngdoc event
       * @name $location#$locationChangeSuccess
       * @eventType broadcast on root scope
       * @description
       * Broadcasted after a URL was changed.
       *
       * The `newState` and `oldState` parameters may be defined only in HTML5 mode and when
       * the browser supports the HTML5 History API.
       *
       * @param {Object} angularEvent Synthetic event object.
       * @param {string} newUrl New URL
       * @param {string=} oldUrl URL that was before it was changed.
       * @param {string=} newState New history state object
       * @param {string=} oldState History state object that was before it was changed.
       */

      this.$get = ['$rootScope', '$browser', '$sniffer', '$rootElement', '$window', function ($rootScope, $browser, $sniffer, $rootElement, $window) {
        var $location,
            LocationMode,
            baseHref = $browser.baseHref(),
            // if base[href] is undefined, it defaults to ''
        initialUrl = $browser.url(),
            appBase;

        if (html5Mode.enabled) {
          if (!baseHref && html5Mode.requireBase) {
            throw $locationMinErr('nobase', '$location in HTML5 mode requires a <base> tag to be present!');
          }
          appBase = serverBase(initialUrl) + (baseHref || '/');
          LocationMode = $sniffer.history ? LocationHtml5Url : LocationHashbangInHtml5Url;
        } else {
          appBase = stripHash(initialUrl);
          LocationMode = LocationHashbangUrl;
        }
        var appBaseNoFile = stripFile(appBase);

        $location = new LocationMode(appBase, appBaseNoFile, '#' + hashPrefix);
        $location.$$parseLinkUrl(initialUrl, initialUrl);

        $location.$$state = $browser.state();

        var IGNORE_URI_REGEXP = /^\s*(javascript|mailto):/i;

        function setBrowserUrlWithFallback(url, replace, state) {
          var oldUrl = $location.url();
          var oldState = $location.$$state;
          try {
            $browser.url(url, replace, state);

            // Make sure $location.state() returns referentially identical (not just deeply equal)
            // state object; this makes possible quick checking if the state changed in the digest
            // loop. Checking deep equality would be too expensive.
            $location.$$state = $browser.state();
          } catch (e) {
            // Restore old values if pushState fails
            $location.url(oldUrl);
            $location.$$state = oldState;

            throw e;
          }
        }

        $rootElement.on('click', function (event) {
          var rewriteLinks = html5Mode.rewriteLinks;
          // TODO(vojta): rewrite link when opening in new tab/window (in legacy browser)
          // currently we open nice url link and redirect then

          if (!rewriteLinks || event.ctrlKey || event.metaKey || event.shiftKey || event.which === 2 || event.button === 2) return;

          var elm = jqLite(event.target);

          // traverse the DOM up to find first A tag
          while (nodeName_(elm[0]) !== 'a') {
            // ignore rewriting if no A tag (reached root element, or no parent - removed from document)
            if (elm[0] === $rootElement[0] || !(elm = elm.parent())[0]) return;
          }

          if (isString(rewriteLinks) && isUndefined(elm.attr(rewriteLinks))) return;

          var absHref = elm.prop('href');
          // get the actual href attribute - see
          // http://msdn.microsoft.com/en-us/library/ie/dd347148(v=vs.85).aspx
          var relHref = elm.attr('href') || elm.attr('xlink:href');

          if (isObject(absHref) && absHref.toString() === '[object SVGAnimatedString]') {
            // SVGAnimatedString.animVal should be identical to SVGAnimatedString.baseVal, unless during
            // an animation.
            absHref = urlResolve(absHref.animVal).href;
          }

          // Ignore when url is started with javascript: or mailto:
          if (IGNORE_URI_REGEXP.test(absHref)) return;

          if (absHref && !elm.attr('target') && !event.isDefaultPrevented()) {
            if ($location.$$parseLinkUrl(absHref, relHref)) {
              // We do a preventDefault for all urls that are part of the angular application,
              // in html5mode and also without, so that we are able to abort navigation without
              // getting double entries in the location history.
              event.preventDefault();
              // update location manually
              if ($location.absUrl() !== $browser.url()) {
                $rootScope.$apply();
                // hack to work around FF6 bug 684208 when scenario runner clicks on links
                $window.angular['ff-684208-preventDefault'] = true;
              }
            }
          }
        });

        // rewrite hashbang url <> html5 url
        if (trimEmptyHash($location.absUrl()) !== trimEmptyHash(initialUrl)) {
          $browser.url($location.absUrl(), true);
        }

        var initializing = true;

        // update $location when $browser url changes
        $browser.onUrlChange(function (newUrl, newState) {

          if (!startsWith(newUrl, appBaseNoFile)) {
            // If we are navigating outside of the app then force a reload
            $window.location.href = newUrl;
            return;
          }

          $rootScope.$evalAsync(function () {
            var oldUrl = $location.absUrl();
            var oldState = $location.$$state;
            var defaultPrevented;
            newUrl = trimEmptyHash(newUrl);
            $location.$$parse(newUrl);
            $location.$$state = newState;

            defaultPrevented = $rootScope.$broadcast('$locationChangeStart', newUrl, oldUrl, newState, oldState).defaultPrevented;

            // if the location was changed by a `$locationChangeStart` handler then stop
            // processing this location change
            if ($location.absUrl() !== newUrl) return;

            if (defaultPrevented) {
              $location.$$parse(oldUrl);
              $location.$$state = oldState;
              setBrowserUrlWithFallback(oldUrl, false, oldState);
            } else {
              initializing = false;
              afterLocationChange(oldUrl, oldState);
            }
          });
          if (!$rootScope.$$phase) $rootScope.$digest();
        });

        // update browser
        $rootScope.$watch(function $locationWatch() {
          if (initializing || $location.$$urlUpdatedByLocation) {
            $location.$$urlUpdatedByLocation = false;

            var oldUrl = trimEmptyHash($browser.url());
            var newUrl = trimEmptyHash($location.absUrl());
            var oldState = $browser.state();
            var currentReplace = $location.$$replace;
            var urlOrStateChanged = oldUrl !== newUrl || $location.$$html5 && $sniffer.history && oldState !== $location.$$state;

            if (initializing || urlOrStateChanged) {
              initializing = false;

              $rootScope.$evalAsync(function () {
                var newUrl = $location.absUrl();
                var defaultPrevented = $rootScope.$broadcast('$locationChangeStart', newUrl, oldUrl, $location.$$state, oldState).defaultPrevented;

                // if the location was changed by a `$locationChangeStart` handler then stop
                // processing this location change
                if ($location.absUrl() !== newUrl) return;

                if (defaultPrevented) {
                  $location.$$parse(oldUrl);
                  $location.$$state = oldState;
                } else {
                  if (urlOrStateChanged) {
                    setBrowserUrlWithFallback(newUrl, currentReplace, oldState === $location.$$state ? null : $location.$$state);
                  }
                  afterLocationChange(oldUrl, oldState);
                }
              });
            }
          }

          $location.$$replace = false;

          // we don't need to return anything because $evalAsync will make the digest loop dirty when
          // there is a change
        });

        return $location;

        function afterLocationChange(oldUrl, oldState) {
          $rootScope.$broadcast('$locationChangeSuccess', $location.absUrl(), oldUrl, $location.$$state, oldState);
        }
      }];
    }

    /**
     * @ngdoc service
     * @name $log
     * @requires $window
     *
     * @description
     * Simple service for logging. Default implementation safely writes the message
     * into the browser's console (if present).
     *
     * The main purpose of this service is to simplify debugging and troubleshooting.
     *
     * The default is to log `debug` messages. You can use
     * {@link ng.$logProvider ng.$logProvider#debugEnabled} to change this.
     *
     * @example
       <example module="logExample" name="log-service">
         <file name="script.js">
           angular.module('logExample', [])
             .controller('LogController', ['$scope', '$log', function($scope, $log) {
               $scope.$log = $log;
               $scope.message = 'Hello World!';
             }]);
         </file>
         <file name="index.html">
           <div ng-controller="LogController">
             <p>Reload this page with open console, enter text and hit the log button...</p>
             <label>Message:
             <input type="text" ng-model="message" /></label>
             <button ng-click="$log.log(message)">log</button>
             <button ng-click="$log.warn(message)">warn</button>
             <button ng-click="$log.info(message)">info</button>
             <button ng-click="$log.error(message)">error</button>
             <button ng-click="$log.debug(message)">debug</button>
           </div>
         </file>
       </example>
     */

    /**
     * @ngdoc provider
     * @name $logProvider
     * @this
     *
     * @description
     * Use the `$logProvider` to configure how the application logs messages
     */
    function $LogProvider() {
      var debug = true,
          self = this;

      /**
       * @ngdoc method
       * @name $logProvider#debugEnabled
       * @description
       * @param {boolean=} flag enable or disable debug level messages
       * @returns {*} current value if used as getter or itself (chaining) if used as setter
       */
      this.debugEnabled = function (flag) {
        if (isDefined(flag)) {
          debug = flag;
          return this;
        } else {
          return debug;
        }
      };

      this.$get = ['$window', function ($window) {
        // Support: IE 9-11, Edge 12-14+
        // IE/Edge display errors in such a way that it requires the user to click in 4 places
        // to see the stack trace. There is no way to feature-detect it so there's a chance
        // of the user agent sniffing to go wrong but since it's only about logging, this shouldn't
        // break apps. Other browsers display errors in a sensible way and some of them map stack
        // traces along source maps if available so it makes sense to let browsers display it
        // as they want.
        var formatStackTrace = msie || /\bEdge\//.test($window.navigator && $window.navigator.userAgent);

        return {
          /**
           * @ngdoc method
           * @name $log#log
           *
           * @description
           * Write a log message
           */
          log: consoleLog('log'),

          /**
           * @ngdoc method
           * @name $log#info
           *
           * @description
           * Write an information message
           */
          info: consoleLog('info'),

          /**
           * @ngdoc method
           * @name $log#warn
           *
           * @description
           * Write a warning message
           */
          warn: consoleLog('warn'),

          /**
           * @ngdoc method
           * @name $log#error
           *
           * @description
           * Write an error message
           */
          error: consoleLog('error'),

          /**
           * @ngdoc method
           * @name $log#debug
           *
           * @description
           * Write a debug message
           */
          debug: function () {
            var fn = consoleLog('debug');

            return function () {
              if (debug) {
                fn.apply(self, arguments);
              }
            };
          }()
        };

        function formatError(arg) {
          if (arg instanceof Error) {
            if (arg.stack && formatStackTrace) {
              arg = arg.message && arg.stack.indexOf(arg.message) === -1 ? 'Error: ' + arg.message + '\n' + arg.stack : arg.stack;
            } else if (arg.sourceURL) {
              arg = arg.message + '\n' + arg.sourceURL + ':' + arg.line;
            }
          }
          return arg;
        }

        function consoleLog(type) {
          var console = $window.console || {},
              logFn = console[type] || console.log || noop,
              hasApply = false;

          // Note: reading logFn.apply throws an error in IE11 in IE8 document mode.
          // The reason behind this is that console.log has type "object" in IE8...
          try {
            hasApply = !!logFn.apply;
          } catch (e) {/* empty */}

          if (hasApply) {
            return function () {
              var args = [];
              forEach(arguments, function (arg) {
                args.push(formatError(arg));
              });
              return logFn.apply(console, args);
            };
          }

          // we are IE which either doesn't have window.console => this is noop and we do nothing,
          // or we are IE where console.log doesn't have apply so we log at least first 2 args
          return function (arg1, arg2) {
            logFn(arg1, arg2 == null ? '' : arg2);
          };
        }
      }];
    }

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
     *     Any commits to this file should be reviewed with security in mind.  *
     *   Changes to this file can potentially create security vulnerabilities. *
     *          An approval from 2 Core members with history of modifying      *
     *                         this file is required.                          *
     *                                                                         *
     *  Does the change somehow allow for arbitrary javascript to be executed? *
     *    Or allows for someone to change the prototype of built-in objects?   *
     *     Or gives undesired access to variables likes document or window?    *
     * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

    var $parseMinErr = minErr('$parse');

    var objectValueOf = {}.constructor.prototype.valueOf;

    // Sandboxing Angular Expressions
    // ------------------------------
    // Angular expressions are no longer sandboxed. So it is now even easier to access arbitrary JS code by
    // various means such as obtaining a reference to native JS functions like the Function constructor.
    //
    // As an example, consider the following Angular expression:
    //
    //   {}.toString.constructor('alert("evil JS code")')
    //
    // It is important to realize that if you create an expression from a string that contains user provided
    // content then it is possible that your application contains a security vulnerability to an XSS style attack.
    //
    // See https://docs.angularjs.org/guide/security


    function getStringValue(name) {
      // Property names must be strings. This means that non-string objects cannot be used
      // as keys in an object. Any non-string object, including a number, is typecasted
      // into a string via the toString method.
      // -- MDN, https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Operators/Property_accessors#Property_names
      //
      // So, to ensure that we are checking the same `name` that JavaScript would use, we cast it
      // to a string. It's not always possible. If `name` is an object and its `toString` method is
      // 'broken' (doesn't return a string, isn't a function, etc.), an error will be thrown:
      //
      // TypeError: Cannot convert object to primitive value
      //
      // For performance reasons, we don't catch this error here and allow it to propagate up the call
      // stack. Note that you'll get the same error in JavaScript if you try to access a property using
      // such a 'broken' object as a key.
      return name + '';
    }

    var OPERATORS = createMap();
    forEach('+ - * / % === !== == != < > <= >= && || ! = |'.split(' '), function (operator) {
      OPERATORS[operator] = true;
    });
    var ESCAPE = { 'n': '\n', 'f': '\f', 'r': '\r', 't': '\t', 'v': '\v', '\'': '\'', '"': '"' };

    /////////////////////////////////////////


    /**
     * @constructor
     */
    var Lexer = function Lexer(options) {
      this.options = options;
    };

    Lexer.prototype = {
      constructor: Lexer,

      lex: function (text) {
        this.text = text;
        this.index = 0;
        this.tokens = [];

        while (this.index < this.text.length) {
          var ch = this.text.charAt(this.index);
          if (ch === '"' || ch === '\'') {
            this.readString(ch);
          } else if (this.isNumber(ch) || ch === '.' && this.isNumber(this.peek())) {
            this.readNumber();
          } else if (this.isIdentifierStart(this.peekMultichar())) {
            this.readIdent();
          } else if (this.is(ch, '(){}[].,;:?')) {
            this.tokens.push({ index: this.index, text: ch });
            this.index++;
          } else if (this.isWhitespace(ch)) {
            this.index++;
          } else {
            var ch2 = ch + this.peek();
            var ch3 = ch2 + this.peek(2);
            var op1 = OPERATORS[ch];
            var op2 = OPERATORS[ch2];
            var op3 = OPERATORS[ch3];
            if (op1 || op2 || op3) {
              var token = op3 ? ch3 : op2 ? ch2 : ch;
              this.tokens.push({ index: this.index, text: token, operator: true });
              this.index += token.length;
            } else {
              this.throwError('Unexpected next character ', this.index, this.index + 1);
            }
          }
        }
        return this.tokens;
      },

      is: function (ch, chars) {
        return chars.indexOf(ch) !== -1;
      },

      peek: function (i) {
        var num = i || 1;
        return this.index + num < this.text.length ? this.text.charAt(this.index + num) : false;
      },

      isNumber: function (ch) {
        return '0' <= ch && ch <= '9' && typeof ch === 'string';
      },

      isWhitespace: function (ch) {
        // IE treats non-breaking space as \u00A0
        return ch === ' ' || ch === '\r' || ch === '\t' || ch === '\n' || ch === '\v' || ch === '\u00A0';
      },

      isIdentifierStart: function (ch) {
        return this.options.isIdentifierStart ? this.options.isIdentifierStart(ch, this.codePointAt(ch)) : this.isValidIdentifierStart(ch);
      },

      isValidIdentifierStart: function (ch) {
        return 'a' <= ch && ch <= 'z' || 'A' <= ch && ch <= 'Z' || '_' === ch || ch === '$';
      },

      isIdentifierContinue: function (ch) {
        return this.options.isIdentifierContinue ? this.options.isIdentifierContinue(ch, this.codePointAt(ch)) : this.isValidIdentifierContinue(ch);
      },

      isValidIdentifierContinue: function (ch, cp) {
        return this.isValidIdentifierStart(ch, cp) || this.isNumber(ch);
      },

      codePointAt: function (ch) {
        if (ch.length === 1) return ch.charCodeAt(0);
        // eslint-disable-next-line no-bitwise
        return (ch.charCodeAt(0) << 10) + ch.charCodeAt(1) - 0x35FDC00;
      },

      peekMultichar: function () {
        var ch = this.text.charAt(this.index);
        var peek = this.peek();
        if (!peek) {
          return ch;
        }
        var cp1 = ch.charCodeAt(0);
        var cp2 = peek.charCodeAt(0);
        if (cp1 >= 0xD800 && cp1 <= 0xDBFF && cp2 >= 0xDC00 && cp2 <= 0xDFFF) {
          return ch + peek;
        }
        return ch;
      },

      isExpOperator: function (ch) {
        return ch === '-' || ch === '+' || this.isNumber(ch);
      },

      throwError: function (error, start, end) {
        end = end || this.index;
        var colStr = isDefined(start) ? 's ' + start + '-' + this.index + ' [' + this.text.substring(start, end) + ']' : ' ' + end;
        throw $parseMinErr('lexerr', 'Lexer Error: {0} at column{1} in expression [{2}].', error, colStr, this.text);
      },

      readNumber: function () {
        var number = '';
        var start = this.index;
        while (this.index < this.text.length) {
          var ch = lowercase(this.text.charAt(this.index));
          if (ch === '.' || this.isNumber(ch)) {
            number += ch;
          } else {
            var peekCh = this.peek();
            if (ch === 'e' && this.isExpOperator(peekCh)) {
              number += ch;
            } else if (this.isExpOperator(ch) && peekCh && this.isNumber(peekCh) && number.charAt(number.length - 1) === 'e') {
              number += ch;
            } else if (this.isExpOperator(ch) && (!peekCh || !this.isNumber(peekCh)) && number.charAt(number.length - 1) === 'e') {
              this.throwError('Invalid exponent');
            } else {
              break;
            }
          }
          this.index++;
        }
        this.tokens.push({
          index: start,
          text: number,
          constant: true,
          value: Number(number)
        });
      },

      readIdent: function () {
        var start = this.index;
        this.index += this.peekMultichar().length;
        while (this.index < this.text.length) {
          var ch = this.peekMultichar();
          if (!this.isIdentifierContinue(ch)) {
            break;
          }
          this.index += ch.length;
        }
        this.tokens.push({
          index: start,
          text: this.text.slice(start, this.index),
          identifier: true
        });
      },

      readString: function (quote) {
        var start = this.index;
        this.index++;
        var string = '';
        var rawString = quote;
        var escape = false;
        while (this.index < this.text.length) {
          var ch = this.text.charAt(this.index);
          rawString += ch;
          if (escape) {
            if (ch === 'u') {
              var hex = this.text.substring(this.index + 1, this.index + 5);
              if (!hex.match(/[\da-f]{4}/i)) {
                this.throwError('Invalid unicode escape [\\u' + hex + ']');
              }
              this.index += 4;
              string += String.fromCharCode(parseInt(hex, 16));
            } else {
              var rep = ESCAPE[ch];
              string = string + (rep || ch);
            }
            escape = false;
          } else if (ch === '\\') {
            escape = true;
          } else if (ch === quote) {
            this.index++;
            this.tokens.push({
              index: start,
              text: rawString,
              constant: true,
              value: string
            });
            return;
          } else {
            string += ch;
          }
          this.index++;
        }
        this.throwError('Unterminated quote', start);
      }
    };

    var AST = function AST(lexer, options) {
      this.lexer = lexer;
      this.options = options;
    };

    AST.Program = 'Program';
    AST.ExpressionStatement = 'ExpressionStatement';
    AST.AssignmentExpression = 'AssignmentExpression';
    AST.ConditionalExpression = 'ConditionalExpression';
    AST.LogicalExpression = 'LogicalExpression';
    AST.BinaryExpression = 'BinaryExpression';
    AST.UnaryExpression = 'UnaryExpression';
    AST.CallExpression = 'CallExpression';
    AST.MemberExpression = 'MemberExpression';
    AST.Identifier = 'Identifier';
    AST.Literal = 'Literal';
    AST.ArrayExpression = 'ArrayExpression';
    AST.Property = 'Property';
    AST.ObjectExpression = 'ObjectExpression';
    AST.ThisExpression = 'ThisExpression';
    AST.LocalsExpression = 'LocalsExpression';

    // Internal use only
    AST.NGValueParameter = 'NGValueParameter';

    AST.prototype = {
      ast: function (text) {
        this.text = text;
        this.tokens = this.lexer.lex(text);

        var value = this.program();

        if (this.tokens.length !== 0) {
          this.throwError('is an unexpected token', this.tokens[0]);
        }

        return value;
      },

      program: function () {
        var body = [];
        while (true) {
          if (this.tokens.length > 0 && !this.peek('}', ')', ';', ']')) body.push(this.expressionStatement());
          if (!this.expect(';')) {
            return { type: AST.Program, body: body };
          }
        }
      },

      expressionStatement: function () {
        return { type: AST.ExpressionStatement, expression: this.filterChain() };
      },

      filterChain: function () {
        var left = this.expression();
        while (this.expect('|')) {
          left = this.filter(left);
        }
        return left;
      },

      expression: function () {
        return this.assignment();
      },

      assignment: function () {
        var result = this.ternary();
        if (this.expect('=')) {
          if (!isAssignable(result)) {
            throw $parseMinErr('lval', 'Trying to assign a value to a non l-value');
          }

          result = { type: AST.AssignmentExpression, left: result, right: this.assignment(), operator: '=' };
        }
        return result;
      },

      ternary: function () {
        var test = this.logicalOR();
        var alternate;
        var consequent;
        if (this.expect('?')) {
          alternate = this.expression();
          if (this.consume(':')) {
            consequent = this.expression();
            return { type: AST.ConditionalExpression, test: test, alternate: alternate, consequent: consequent };
          }
        }
        return test;
      },

      logicalOR: function () {
        var left = this.logicalAND();
        while (this.expect('||')) {
          left = { type: AST.LogicalExpression, operator: '||', left: left, right: this.logicalAND() };
        }
        return left;
      },

      logicalAND: function () {
        var left = this.equality();
        while (this.expect('&&')) {
          left = { type: AST.LogicalExpression, operator: '&&', left: left, right: this.equality() };
        }
        return left;
      },

      equality: function () {
        var left = this.relational();
        var token;
        while (token = this.expect('==', '!=', '===', '!==')) {
          left = { type: AST.BinaryExpression, operator: token.text, left: left, right: this.relational() };
        }
        return left;
      },

      relational: function () {
        var left = this.additive();
        var token;
        while (token = this.expect('<', '>', '<=', '>=')) {
          left = { type: AST.BinaryExpression, operator: token.text, left: left, right: this.additive() };
        }
        return left;
      },

      additive: function () {
        var left = this.multiplicative();
        var token;
        while (token = this.expect('+', '-')) {
          left = { type: AST.BinaryExpression, operator: token.text, left: left, right: this.multiplicative() };
        }
        return left;
      },

      multiplicative: function () {
        var left = this.unary();
        var token;
        while (token = this.expect('*', '/', '%')) {
          left = { type: AST.BinaryExpression, operator: token.text, left: left, right: this.unary() };
        }
        return left;
      },

      unary: function () {
        var token;
        if (token = this.expect('+', '-', '!')) {
          return { type: AST.UnaryExpression, operator: token.text, prefix: true, argument: this.unary() };
        } else {
          return this.primary();
        }
      },

      primary: function () {
        var primary;
        if (this.expect('(')) {
          primary = this.filterChain();
          this.consume(')');
        } else if (this.expect('[')) {
          primary = this.arrayDeclaration();
        } else if (this.expect('{')) {
          primary = this.object();
        } else if (this.selfReferential.hasOwnProperty(this.peek().text)) {
          primary = copy(this.selfReferential[this.consume().text]);
        } else if (this.options.literals.hasOwnProperty(this.peek().text)) {
          primary = { type: AST.Literal, value: this.options.literals[this.consume().text] };
        } else if (this.peek().identifier) {
          primary = this.identifier();
        } else if (this.peek().constant) {
          primary = this.constant();
        } else {
          this.throwError('not a primary expression', this.peek());
        }

        var next;
        while (next = this.expect('(', '[', '.')) {
          if (next.text === '(') {
            primary = { type: AST.CallExpression, callee: primary, arguments: this.parseArguments() };
            this.consume(')');
          } else if (next.text === '[') {
            primary = { type: AST.MemberExpression, object: primary, property: this.expression(), computed: true };
            this.consume(']');
          } else if (next.text === '.') {
            primary = { type: AST.MemberExpression, object: primary, property: this.identifier(), computed: false };
          } else {
            this.throwError('IMPOSSIBLE');
          }
        }
        return primary;
      },

      filter: function (baseExpression) {
        var args = [baseExpression];
        var result = { type: AST.CallExpression, callee: this.identifier(), arguments: args, filter: true };

        while (this.expect(':')) {
          args.push(this.expression());
        }

        return result;
      },

      parseArguments: function () {
        var args = [];
        if (this.peekToken().text !== ')') {
          do {
            args.push(this.filterChain());
          } while (this.expect(','));
        }
        return args;
      },

      identifier: function () {
        var token = this.consume();
        if (!token.identifier) {
          this.throwError('is not a valid identifier', token);
        }
        return { type: AST.Identifier, name: token.text };
      },

      constant: function () {
        // TODO check that it is a constant
        return { type: AST.Literal, value: this.consume().value };
      },

      arrayDeclaration: function () {
        var elements = [];
        if (this.peekToken().text !== ']') {
          do {
            if (this.peek(']')) {
              // Support trailing commas per ES5.1.
              break;
            }
            elements.push(this.expression());
          } while (this.expect(','));
        }
        this.consume(']');

        return { type: AST.ArrayExpression, elements: elements };
      },

      object: function () {
        var properties = [],
            property;
        if (this.peekToken().text !== '}') {
          do {
            if (this.peek('}')) {
              // Support trailing commas per ES5.1.
              break;
            }
            property = { type: AST.Property, kind: 'init' };
            if (this.peek().constant) {
              property.key = this.constant();
              property.computed = false;
              this.consume(':');
              property.value = this.expression();
            } else if (this.peek().identifier) {
              property.key = this.identifier();
              property.computed = false;
              if (this.peek(':')) {
                this.consume(':');
                property.value = this.expression();
              } else {
                property.value = property.key;
              }
            } else if (this.peek('[')) {
              this.consume('[');
              property.key = this.expression();
              this.consume(']');
              property.computed = true;
              this.consume(':');
              property.value = this.expression();
            } else {
              this.throwError('invalid key', this.peek());
            }
            properties.push(property);
          } while (this.expect(','));
        }
        this.consume('}');

        return { type: AST.ObjectExpression, properties: properties };
      },

      throwError: function (msg, token) {
        throw $parseMinErr('syntax', 'Syntax Error: Token \'{0}\' {1} at column {2} of the expression [{3}] starting at [{4}].', token.text, msg, token.index + 1, this.text, this.text.substring(token.index));
      },

      consume: function (e1) {
        if (this.tokens.length === 0) {
          throw $parseMinErr('ueoe', 'Unexpected end of expression: {0}', this.text);
        }

        var token = this.expect(e1);
        if (!token) {
          this.throwError('is unexpected, expecting [' + e1 + ']', this.peek());
        }
        return token;
      },

      peekToken: function () {
        if (this.tokens.length === 0) {
          throw $parseMinErr('ueoe', 'Unexpected end of expression: {0}', this.text);
        }
        return this.tokens[0];
      },

      peek: function (e1, e2, e3, e4) {
        return this.peekAhead(0, e1, e2, e3, e4);
      },

      peekAhead: function (i, e1, e2, e3, e4) {
        if (this.tokens.length > i) {
          var token = this.tokens[i];
          var t = token.text;
          if (t === e1 || t === e2 || t === e3 || t === e4 || !e1 && !e2 && !e3 && !e4) {
            return token;
          }
        }
        return false;
      },

      expect: function (e1, e2, e3, e4) {
        var token = this.peek(e1, e2, e3, e4);
        if (token) {
          this.tokens.shift();
          return token;
        }
        return false;
      },

      selfReferential: {
        'this': { type: AST.ThisExpression },
        '$locals': { type: AST.LocalsExpression }
      }
    };

    function ifDefined(v, d) {
      return typeof v !== 'undefined' ? v : d;
    }

    function plusFn(l, r) {
      if (typeof l === 'undefined') return r;
      if (typeof r === 'undefined') return l;
      return l + r;
    }

    function isStateless($filter, filterName) {
      var fn = $filter(filterName);
      return !fn.$stateful;
    }

    function findConstantAndWatchExpressions(ast, $filter) {
      var allConstants;
      var argsToWatch;
      var isStatelessFilter;
      switch (ast.type) {
        case AST.Program:
          allConstants = true;
          forEach(ast.body, function (expr) {
            findConstantAndWatchExpressions(expr.expression, $filter);
            allConstants = allConstants && expr.expression.constant;
          });
          ast.constant = allConstants;
          break;
        case AST.Literal:
          ast.constant = true;
          ast.toWatch = [];
          break;
        case AST.UnaryExpression:
          findConstantAndWatchExpressions(ast.argument, $filter);
          ast.constant = ast.argument.constant;
          ast.toWatch = ast.argument.toWatch;
          break;
        case AST.BinaryExpression:
          findConstantAndWatchExpressions(ast.left, $filter);
          findConstantAndWatchExpressions(ast.right, $filter);
          ast.constant = ast.left.constant && ast.right.constant;
          ast.toWatch = ast.left.toWatch.concat(ast.right.toWatch);
          break;
        case AST.LogicalExpression:
          findConstantAndWatchExpressions(ast.left, $filter);
          findConstantAndWatchExpressions(ast.right, $filter);
          ast.constant = ast.left.constant && ast.right.constant;
          ast.toWatch = ast.constant ? [] : [ast];
          break;
        case AST.ConditionalExpression:
          findConstantAndWatchExpressions(ast.test, $filter);
          findConstantAndWatchExpressions(ast.alternate, $filter);
          findConstantAndWatchExpressions(ast.consequent, $filter);
          ast.constant = ast.test.constant && ast.alternate.constant && ast.consequent.constant;
          ast.toWatch = ast.constant ? [] : [ast];
          break;
        case AST.Identifier:
          ast.constant = false;
          ast.toWatch = [ast];
          break;
        case AST.MemberExpression:
          findConstantAndWatchExpressions(ast.object, $filter);
          if (ast.computed) {
            findConstantAndWatchExpressions(ast.property, $filter);
          }
          ast.constant = ast.object.constant && (!ast.computed || ast.property.constant);
          ast.toWatch = [ast];
          break;
        case AST.CallExpression:
          isStatelessFilter = ast.filter ? isStateless($filter, ast.callee.name) : false;
          allConstants = isStatelessFilter;
          argsToWatch = [];
          forEach(ast.arguments, function (expr) {
            findConstantAndWatchExpressions(expr, $filter);
            allConstants = allConstants && expr.constant;
            if (!expr.constant) {
              argsToWatch.push.apply(argsToWatch, expr.toWatch);
            }
          });
          ast.constant = allConstants;
          ast.toWatch = isStatelessFilter ? argsToWatch : [ast];
          break;
        case AST.AssignmentExpression:
          findConstantAndWatchExpressions(ast.left, $filter);
          findConstantAndWatchExpressions(ast.right, $filter);
          ast.constant = ast.left.constant && ast.right.constant;
          ast.toWatch = [ast];
          break;
        case AST.ArrayExpression:
          allConstants = true;
          argsToWatch = [];
          forEach(ast.elements, function (expr) {
            findConstantAndWatchExpressions(expr, $filter);
            allConstants = allConstants && expr.constant;
            if (!expr.constant) {
              argsToWatch.push.apply(argsToWatch, expr.toWatch);
            }
          });
          ast.constant = allConstants;
          ast.toWatch = argsToWatch;
          break;
        case AST.ObjectExpression:
          allConstants = true;
          argsToWatch = [];
          forEach(ast.properties, function (property) {
            findConstantAndWatchExpressions(property.value, $filter);
            allConstants = allConstants && property.value.constant && !property.computed;
            if (!property.value.constant) {
              argsToWatch.push.apply(argsToWatch, property.value.toWatch);
            }
            if (property.computed) {
              findConstantAndWatchExpressions(property.key, $filter);
              if (!property.key.constant) {
                argsToWatch.push.apply(argsToWatch, property.key.toWatch);
              }
            }
          });
          ast.constant = allConstants;
          ast.toWatch = argsToWatch;
          break;
        case AST.ThisExpression:
          ast.constant = false;
          ast.toWatch = [];
          break;
        case AST.LocalsExpression:
          ast.constant = false;
          ast.toWatch = [];
          break;
      }
    }

    function getInputs(body) {
      if (body.length !== 1) return;
      var lastExpression = body[0].expression;
      var candidate = lastExpression.toWatch;
      if (candidate.length !== 1) return candidate;
      return candidate[0] !== lastExpression ? candidate : undefined;
    }

    function isAssignable(ast) {
      return ast.type === AST.Identifier || ast.type === AST.MemberExpression;
    }

    function assignableAST(ast) {
      if (ast.body.length === 1 && isAssignable(ast.body[0].expression)) {
        return { type: AST.AssignmentExpression, left: ast.body[0].expression, right: { type: AST.NGValueParameter }, operator: '=' };
      }
    }

    function isLiteral(ast) {
      return ast.body.length === 0 || ast.body.length === 1 && (ast.body[0].expression.type === AST.Literal || ast.body[0].expression.type === AST.ArrayExpression || ast.body[0].expression.type === AST.ObjectExpression);
    }

    function isConstant(ast) {
      return ast.constant;
    }

    function ASTCompiler(astBuilder, $filter) {
      this.astBuilder = astBuilder;
      this.$filter = $filter;
    }

    ASTCompiler.prototype = {
      compile: function (expression) {
        var self = this;
        var ast = this.astBuilder.ast(expression);
        this.state = {
          nextId: 0,
          filters: {},
          fn: { vars: [], body: [], own: {} },
          assign: { vars: [], body: [], own: {} },
          inputs: []
        };
        findConstantAndWatchExpressions(ast, self.$filter);
        var extra = '';
        var assignable;
        this.stage = 'assign';
        if (assignable = assignableAST(ast)) {
          this.state.computing = 'assign';
          var result = this.nextId();
          this.recurse(assignable, result);
          this.return_(result);
          extra = 'fn.assign=' + this.generateFunction('assign', 's,v,l');
        }
        var toWatch = getInputs(ast.body);
        self.stage = 'inputs';
        forEach(toWatch, function (watch, key) {
          var fnKey = 'fn' + key;
          self.state[fnKey] = { vars: [], body: [], own: {} };
          self.state.computing = fnKey;
          var intoId = self.nextId();
          self.recurse(watch, intoId);
          self.return_(intoId);
          self.state.inputs.push(fnKey);
          watch.watchId = key;
        });
        this.state.computing = 'fn';
        this.stage = 'main';
        this.recurse(ast);
        var fnString =
        // The build and minification steps remove the string "use strict" from the code, but this is done using a regex.
        // This is a workaround for this until we do a better job at only removing the prefix only when we should.
        '"' + this.USE + ' ' + this.STRICT + '";\n' + this.filterPrefix() + 'var fn=' + this.generateFunction('fn', 's,l,a,i') + extra + this.watchFns() + 'return fn;';

        // eslint-disable-next-line no-new-func
        var fn = new Function('$filter', 'getStringValue', 'ifDefined', 'plus', fnString)(this.$filter, getStringValue, ifDefined, plusFn);
        this.state = this.stage = undefined;
        fn.literal = isLiteral(ast);
        fn.constant = isConstant(ast);
        return fn;
      },

      USE: 'use',

      STRICT: 'strict',

      watchFns: function () {
        var result = [];
        var fns = this.state.inputs;
        var self = this;
        forEach(fns, function (name) {
          result.push('var ' + name + '=' + self.generateFunction(name, 's'));
        });
        if (fns.length) {
          result.push('fn.inputs=[' + fns.join(',') + '];');
        }
        return result.join('');
      },

      generateFunction: function (name, params) {
        return 'function(' + params + '){' + this.varsPrefix(name) + this.body(name) + '};';
      },

      filterPrefix: function () {
        var parts = [];
        var self = this;
        forEach(this.state.filters, function (id, filter) {
          parts.push(id + '=$filter(' + self.escape(filter) + ')');
        });
        if (parts.length) return 'var ' + parts.join(',') + ';';
        return '';
      },

      varsPrefix: function (section) {
        return this.state[section].vars.length ? 'var ' + this.state[section].vars.join(',') + ';' : '';
      },

      body: function (section) {
        return this.state[section].body.join('');
      },

      recurse: function (ast, intoId, nameId, recursionFn, create, skipWatchIdCheck) {
        var left,
            right,
            self = this,
            args,
            expression,
            computed;
        recursionFn = recursionFn || noop;
        if (!skipWatchIdCheck && isDefined(ast.watchId)) {
          intoId = intoId || this.nextId();
          this.if_('i', this.lazyAssign(intoId, this.computedMember('i', ast.watchId)), this.lazyRecurse(ast, intoId, nameId, recursionFn, create, true));
          return;
        }
        switch (ast.type) {
          case AST.Program:
            forEach(ast.body, function (expression, pos) {
              self.recurse(expression.expression, undefined, undefined, function (expr) {
                right = expr;
              });
              if (pos !== ast.body.length - 1) {
                self.current().body.push(right, ';');
              } else {
                self.return_(right);
              }
            });
            break;
          case AST.Literal:
            expression = this.escape(ast.value);
            this.assign(intoId, expression);
            recursionFn(intoId || expression);
            break;
          case AST.UnaryExpression:
            this.recurse(ast.argument, undefined, undefined, function (expr) {
              right = expr;
            });
            expression = ast.operator + '(' + this.ifDefined(right, 0) + ')';
            this.assign(intoId, expression);
            recursionFn(expression);
            break;
          case AST.BinaryExpression:
            this.recurse(ast.left, undefined, undefined, function (expr) {
              left = expr;
            });
            this.recurse(ast.right, undefined, undefined, function (expr) {
              right = expr;
            });
            if (ast.operator === '+') {
              expression = this.plus(left, right);
            } else if (ast.operator === '-') {
              expression = this.ifDefined(left, 0) + ast.operator + this.ifDefined(right, 0);
            } else {
              expression = '(' + left + ')' + ast.operator + '(' + right + ')';
            }
            this.assign(intoId, expression);
            recursionFn(expression);
            break;
          case AST.LogicalExpression:
            intoId = intoId || this.nextId();
            self.recurse(ast.left, intoId);
            self.if_(ast.operator === '&&' ? intoId : self.not(intoId), self.lazyRecurse(ast.right, intoId));
            recursionFn(intoId);
            break;
          case AST.ConditionalExpression:
            intoId = intoId || this.nextId();
            self.recurse(ast.test, intoId);
            self.if_(intoId, self.lazyRecurse(ast.alternate, intoId), self.lazyRecurse(ast.consequent, intoId));
            recursionFn(intoId);
            break;
          case AST.Identifier:
            intoId = intoId || this.nextId();
            if (nameId) {
              nameId.context = self.stage === 'inputs' ? 's' : this.assign(this.nextId(), this.getHasOwnProperty('l', ast.name) + '?l:s');
              nameId.computed = false;
              nameId.name = ast.name;
            }
            self.if_(self.stage === 'inputs' || self.not(self.getHasOwnProperty('l', ast.name)), function () {
              self.if_(self.stage === 'inputs' || 's', function () {
                if (create && create !== 1) {
                  self.if_(self.isNull(self.nonComputedMember('s', ast.name)), self.lazyAssign(self.nonComputedMember('s', ast.name), '{}'));
                }
                self.assign(intoId, self.nonComputedMember('s', ast.name));
              });
            }, intoId && self.lazyAssign(intoId, self.nonComputedMember('l', ast.name)));
            recursionFn(intoId);
            break;
          case AST.MemberExpression:
            left = nameId && (nameId.context = this.nextId()) || this.nextId();
            intoId = intoId || this.nextId();
            self.recurse(ast.object, left, undefined, function () {
              self.if_(self.notNull(left), function () {
                if (ast.computed) {
                  right = self.nextId();
                  self.recurse(ast.property, right);
                  self.getStringValue(right);
                  if (create && create !== 1) {
                    self.if_(self.not(self.computedMember(left, right)), self.lazyAssign(self.computedMember(left, right), '{}'));
                  }
                  expression = self.computedMember(left, right);
                  self.assign(intoId, expression);
                  if (nameId) {
                    nameId.computed = true;
                    nameId.name = right;
                  }
                } else {
                  if (create && create !== 1) {
                    self.if_(self.isNull(self.nonComputedMember(left, ast.property.name)), self.lazyAssign(self.nonComputedMember(left, ast.property.name), '{}'));
                  }
                  expression = self.nonComputedMember(left, ast.property.name);
                  self.assign(intoId, expression);
                  if (nameId) {
                    nameId.computed = false;
                    nameId.name = ast.property.name;
                  }
                }
              }, function () {
                self.assign(intoId, 'undefined');
              });
              recursionFn(intoId);
            }, !!create);
            break;
          case AST.CallExpression:
            intoId = intoId || this.nextId();
            if (ast.filter) {
              right = self.filter(ast.callee.name);
              args = [];
              forEach(ast.arguments, function (expr) {
                var argument = self.nextId();
                self.recurse(expr, argument);
                args.push(argument);
              });
              expression = right + '(' + args.join(',') + ')';
              self.assign(intoId, expression);
              recursionFn(intoId);
            } else {
              right = self.nextId();
              left = {};
              args = [];
              self.recurse(ast.callee, right, left, function () {
                self.if_(self.notNull(right), function () {
                  forEach(ast.arguments, function (expr) {
                    self.recurse(expr, ast.constant ? undefined : self.nextId(), undefined, function (argument) {
                      args.push(argument);
                    });
                  });
                  if (left.name) {
                    expression = self.member(left.context, left.name, left.computed) + '(' + args.join(',') + ')';
                  } else {
                    expression = right + '(' + args.join(',') + ')';
                  }
                  self.assign(intoId, expression);
                }, function () {
                  self.assign(intoId, 'undefined');
                });
                recursionFn(intoId);
              });
            }
            break;
          case AST.AssignmentExpression:
            right = this.nextId();
            left = {};
            this.recurse(ast.left, undefined, left, function () {
              self.if_(self.notNull(left.context), function () {
                self.recurse(ast.right, right);
                expression = self.member(left.context, left.name, left.computed) + ast.operator + right;
                self.assign(intoId, expression);
                recursionFn(intoId || expression);
              });
            }, 1);
            break;
          case AST.ArrayExpression:
            args = [];
            forEach(ast.elements, function (expr) {
              self.recurse(expr, ast.constant ? undefined : self.nextId(), undefined, function (argument) {
                args.push(argument);
              });
            });
            expression = '[' + args.join(',') + ']';
            this.assign(intoId, expression);
            recursionFn(intoId || expression);
            break;
          case AST.ObjectExpression:
            args = [];
            computed = false;
            forEach(ast.properties, function (property) {
              if (property.computed) {
                computed = true;
              }
            });
            if (computed) {
              intoId = intoId || this.nextId();
              this.assign(intoId, '{}');
              forEach(ast.properties, function (property) {
                if (property.computed) {
                  left = self.nextId();
                  self.recurse(property.key, left);
                } else {
                  left = property.key.type === AST.Identifier ? property.key.name : '' + property.key.value;
                }
                right = self.nextId();
                self.recurse(property.value, right);
                self.assign(self.member(intoId, left, property.computed), right);
              });
            } else {
              forEach(ast.properties, function (property) {
                self.recurse(property.value, ast.constant ? undefined : self.nextId(), undefined, function (expr) {
                  args.push(self.escape(property.key.type === AST.Identifier ? property.key.name : '' + property.key.value) + ':' + expr);
                });
              });
              expression = '{' + args.join(',') + '}';
              this.assign(intoId, expression);
            }
            recursionFn(intoId || expression);
            break;
          case AST.ThisExpression:
            this.assign(intoId, 's');
            recursionFn(intoId || 's');
            break;
          case AST.LocalsExpression:
            this.assign(intoId, 'l');
            recursionFn(intoId || 'l');
            break;
          case AST.NGValueParameter:
            this.assign(intoId, 'v');
            recursionFn(intoId || 'v');
            break;
        }
      },

      getHasOwnProperty: function (element, property) {
        var key = element + '.' + property;
        var own = this.current().own;
        if (!own.hasOwnProperty(key)) {
          own[key] = this.nextId(false, element + '&&(' + this.escape(property) + ' in ' + element + ')');
        }
        return own[key];
      },

      assign: function (id, value) {
        if (!id) return;
        this.current().body.push(id, '=', value, ';');
        return id;
      },

      filter: function (filterName) {
        if (!this.state.filters.hasOwnProperty(filterName)) {
          this.state.filters[filterName] = this.nextId(true);
        }
        return this.state.filters[filterName];
      },

      ifDefined: function (id, defaultValue) {
        return 'ifDefined(' + id + ',' + this.escape(defaultValue) + ')';
      },

      plus: function (left, right) {
        return 'plus(' + left + ',' + right + ')';
      },

      return_: function (id) {
        this.current().body.push('return ', id, ';');
      },

      if_: function (test, alternate, consequent) {
        if (test === true) {
          alternate();
        } else {
          var body = this.current().body;
          body.push('if(', test, '){');
          alternate();
          body.push('}');
          if (consequent) {
            body.push('else{');
            consequent();
            body.push('}');
          }
        }
      },

      not: function (expression) {
        return '!(' + expression + ')';
      },

      isNull: function (expression) {
        return expression + '==null';
      },

      notNull: function (expression) {
        return expression + '!=null';
      },

      nonComputedMember: function (left, right) {
        var SAFE_IDENTIFIER = /^[$_a-zA-Z][$_a-zA-Z0-9]*$/;
        var UNSAFE_CHARACTERS = /[^$_a-zA-Z0-9]/g;
        if (SAFE_IDENTIFIER.test(right)) {
          return left + '.' + right;
        } else {
          return left + '["' + right.replace(UNSAFE_CHARACTERS, this.stringEscapeFn) + '"]';
        }
      },

      computedMember: function (left, right) {
        return left + '[' + right + ']';
      },

      member: function (left, right, computed) {
        if (computed) return this.computedMember(left, right);
        return this.nonComputedMember(left, right);
      },

      getStringValue: function (item) {
        this.assign(item, 'getStringValue(' + item + ')');
      },

      lazyRecurse: function (ast, intoId, nameId, recursionFn, create, skipWatchIdCheck) {
        var self = this;
        return function () {
          self.recurse(ast, intoId, nameId, recursionFn, create, skipWatchIdCheck);
        };
      },

      lazyAssign: function (id, value) {
        var self = this;
        return function () {
          self.assign(id, value);
        };
      },

      stringEscapeRegex: /[^ a-zA-Z0-9]/g,

      stringEscapeFn: function (c) {
        return '\\u' + ('0000' + c.charCodeAt(0).toString(16)).slice(-4);
      },

      escape: function (value) {
        if (isString(value)) return '\'' + value.replace(this.stringEscapeRegex, this.stringEscapeFn) + '\'';
        if (isNumber(value)) return value.toString();
        if (value === true) return 'true';
        if (value === false) return 'false';
        if (value === null) return 'null';
        if (typeof value === 'undefined') return 'undefined';

        throw $parseMinErr('esc', 'IMPOSSIBLE');
      },

      nextId: function (skip, init) {
        var id = 'v' + this.state.nextId++;
        if (!skip) {
          this.current().vars.push(id + (init ? '=' + init : ''));
        }
        return id;
      },

      current: function () {
        return this.state[this.state.computing];
      }
    };

    function ASTInterpreter(astBuilder, $filter) {
      this.astBuilder = astBuilder;
      this.$filter = $filter;
    }

    ASTInterpreter.prototype = {
      compile: function (expression) {
        var self = this;
        var ast = this.astBuilder.ast(expression);
        findConstantAndWatchExpressions(ast, self.$filter);
        var assignable;
        var assign;
        if (assignable = assignableAST(ast)) {
          assign = this.recurse(assignable);
        }
        var toWatch = getInputs(ast.body);
        var inputs;
        if (toWatch) {
          inputs = [];
          forEach(toWatch, function (watch, key) {
            var input = self.recurse(watch);
            watch.input = input;
            inputs.push(input);
            watch.watchId = key;
          });
        }
        var expressions = [];
        forEach(ast.body, function (expression) {
          expressions.push(self.recurse(expression.expression));
        });
        var fn = ast.body.length === 0 ? noop : ast.body.length === 1 ? expressions[0] : function (scope, locals) {
          var lastValue;
          forEach(expressions, function (exp) {
            lastValue = exp(scope, locals);
          });
          return lastValue;
        };
        if (assign) {
          fn.assign = function (scope, value, locals) {
            return assign(scope, locals, value);
          };
        }
        if (inputs) {
          fn.inputs = inputs;
        }
        fn.literal = isLiteral(ast);
        fn.constant = isConstant(ast);
        return fn;
      },

      recurse: function (ast, context, create) {
        var left,
            right,
            self = this,
            args;
        if (ast.input) {
          return this.inputs(ast.input, ast.watchId);
        }
        switch (ast.type) {
          case AST.Literal:
            return this.value(ast.value, context);
          case AST.UnaryExpression:
            right = this.recurse(ast.argument);
            return this['unary' + ast.operator](right, context);
          case AST.BinaryExpression:
            left = this.recurse(ast.left);
            right = this.recurse(ast.right);
            return this['binary' + ast.operator](left, right, context);
          case AST.LogicalExpression:
            left = this.recurse(ast.left);
            right = this.recurse(ast.right);
            return this['binary' + ast.operator](left, right, context);
          case AST.ConditionalExpression:
            return this['ternary?:'](this.recurse(ast.test), this.recurse(ast.alternate), this.recurse(ast.consequent), context);
          case AST.Identifier:
            return self.identifier(ast.name, context, create);
          case AST.MemberExpression:
            left = this.recurse(ast.object, false, !!create);
            if (!ast.computed) {
              right = ast.property.name;
            }
            if (ast.computed) right = this.recurse(ast.property);
            return ast.computed ? this.computedMember(left, right, context, create) : this.nonComputedMember(left, right, context, create);
          case AST.CallExpression:
            args = [];
            forEach(ast.arguments, function (expr) {
              args.push(self.recurse(expr));
            });
            if (ast.filter) right = this.$filter(ast.callee.name);
            if (!ast.filter) right = this.recurse(ast.callee, true);
            return ast.filter ? function (scope, locals, assign, inputs) {
              var values = [];
              for (var i = 0; i < args.length; ++i) {
                values.push(args[i](scope, locals, assign, inputs));
              }
              var value = right.apply(undefined, values, inputs);
              return context ? { context: undefined, name: undefined, value: value } : value;
            } : function (scope, locals, assign, inputs) {
              var rhs = right(scope, locals, assign, inputs);
              var value;
              if (rhs.value != null) {
                var values = [];
                for (var i = 0; i < args.length; ++i) {
                  values.push(args[i](scope, locals, assign, inputs));
                }
                value = rhs.value.apply(rhs.context, values);
              }
              return context ? { value: value } : value;
            };
          case AST.AssignmentExpression:
            left = this.recurse(ast.left, true, 1);
            right = this.recurse(ast.right);
            return function (scope, locals, assign, inputs) {
              var lhs = left(scope, locals, assign, inputs);
              var rhs = right(scope, locals, assign, inputs);
              lhs.context[lhs.name] = rhs;
              return context ? { value: rhs } : rhs;
            };
          case AST.ArrayExpression:
            args = [];
            forEach(ast.elements, function (expr) {
              args.push(self.recurse(expr));
            });
            return function (scope, locals, assign, inputs) {
              var value = [];
              for (var i = 0; i < args.length; ++i) {
                value.push(args[i](scope, locals, assign, inputs));
              }
              return context ? { value: value } : value;
            };
          case AST.ObjectExpression:
            args = [];
            forEach(ast.properties, function (property) {
              if (property.computed) {
                args.push({ key: self.recurse(property.key),
                  computed: true,
                  value: self.recurse(property.value)
                });
              } else {
                args.push({ key: property.key.type === AST.Identifier ? property.key.name : '' + property.key.value,
                  computed: false,
                  value: self.recurse(property.value)
                });
              }
            });
            return function (scope, locals, assign, inputs) {
              var value = {};
              for (var i = 0; i < args.length; ++i) {
                if (args[i].computed) {
                  value[args[i].key(scope, locals, assign, inputs)] = args[i].value(scope, locals, assign, inputs);
                } else {
                  value[args[i].key] = args[i].value(scope, locals, assign, inputs);
                }
              }
              return context ? { value: value } : value;
            };
          case AST.ThisExpression:
            return function (scope) {
              return context ? { value: scope } : scope;
            };
          case AST.LocalsExpression:
            return function (scope, locals) {
              return context ? { value: locals } : locals;
            };
          case AST.NGValueParameter:
            return function (scope, locals, assign) {
              return context ? { value: assign } : assign;
            };
        }
      },

      'unary+': function (argument, context) {
        return function (scope, locals, assign, inputs) {
          var arg = argument(scope, locals, assign, inputs);
          if (isDefined(arg)) {
            arg = +arg;
          } else {
            arg = 0;
          }
          return context ? { value: arg } : arg;
        };
      },
      'unary-': function (argument, context) {
        return function (scope, locals, assign, inputs) {
          var arg = argument(scope, locals, assign, inputs);
          if (isDefined(arg)) {
            arg = -arg;
          } else {
            arg = -0;
          }
          return context ? { value: arg } : arg;
        };
      },
      'unary!': function (argument, context) {
        return function (scope, locals, assign, inputs) {
          var arg = !argument(scope, locals, assign, inputs);
          return context ? { value: arg } : arg;
        };
      },
      'binary+': function (left, right, context) {
        return function (scope, locals, assign, inputs) {
          var lhs = left(scope, locals, assign, inputs);
          var rhs = right(scope, locals, assign, inputs);
          var arg = plusFn(lhs, rhs);
          return context ? { value: arg } : arg;
        };
      },
      'binary-': function (left, right, context) {
        return function (scope, locals, assign, inputs) {
          var lhs = left(scope, locals, assign, inputs);
          var rhs = right(scope, locals, assign, inputs);
          var arg = (isDefined(lhs) ? lhs : 0) - (isDefined(rhs) ? rhs : 0);
          return context ? { value: arg } : arg;
        };
      },
      'binary*': function (left, right, context) {
        return function (scope, locals, assign, inputs) {
          var arg = left(scope, locals, assign, inputs) * right(scope, locals, assign, inputs);
          return context ? { value: arg } : arg;
        };
      },
      'binary/': function (left, right, context) {
        return function (scope, locals, assign, inputs) {
          var arg = left(scope, locals, assign, inputs) / right(scope, locals, assign, inputs);
          return context ? { value: arg } : arg;
        };
      },
      'binary%': function (left, right, context) {
        return function (scope, locals, assign, inputs) {
          var arg = left(scope, locals, assign, inputs) % right(scope, locals, assign, inputs);
          return context ? { value: arg } : arg;
        };
      },
      'binary===': function (left, right, context) {
        return function (scope, locals, assign, inputs) {
          var arg = left(scope, locals, assign, inputs) === right(scope, locals, assign, inputs);
          return context ? { value: arg } : arg;
        };
      },
      'binary!==': function (left, right, context) {
        return function (scope, locals, assign, inputs) {
          var arg = left(scope, locals, assign, inputs) !== right(scope, locals, assign, inputs);
          return context ? { value: arg } : arg;
        };
      },
      'binary==': function (left, right, context) {
        return function (scope, locals, assign, inputs) {
          // eslint-disable-next-line eqeqeq
          var arg = left(scope, locals, assign, inputs) == right(scope, locals, assign, inputs);
          return context ? { value: arg } : arg;
        };
      },
      'binary!=': function (left, right, context) {
        return function (scope, locals, assign, inputs) {
          // eslint-disable-next-line eqeqeq
          var arg = left(scope, locals, assign, inputs) != right(scope, locals, assign, inputs);
          return context ? { value: arg } : arg;
        };
      },
      'binary<': function (left, right, context) {
        return function (scope, locals, assign, inputs) {
          var arg = left(scope, locals, assign, inputs) < right(scope, locals, assign, inputs);
          return context ? { value: arg } : arg;
        };
      },
      'binary>': function (left, right, context) {
        return function (scope, locals, assign, inputs) {
          var arg = left(scope, locals, assign, inputs) > right(scope, locals, assign, inputs);
          return context ? { value: arg } : arg;
        };
      },
      'binary<=': function (left, right, context) {
        return function (scope, locals, assign, inputs) {
          var arg = left(scope, locals, assign, inputs) <= right(scope, locals, assign, inputs);
          return context ? { value: arg } : arg;
        };
      },
      'binary>=': function (left, right, context) {
        return function (scope, locals, assign, inputs) {
          var arg = left(scope, locals, assign, inputs) >= right(scope, locals, assign, inputs);
          return context ? { value: arg } : arg;
        };
      },
      'binary&&': function (left, right, context) {
        return function (scope, locals, assign, inputs) {
          var arg = left(scope, locals, assign, inputs) && right(scope, locals, assign, inputs);
          return context ? { value: arg } : arg;
        };
      },
      'binary||': function (left, right, context) {
        return function (scope, locals, assign, inputs) {
          var arg = left(scope, locals, assign, inputs) || right(scope, locals, assign, inputs);
          return context ? { value: arg } : arg;
        };
      },
      'ternary?:': function (test, alternate, consequent, context) {
        return function (scope, locals, assign, inputs) {
          var arg = test(scope, locals, assign, inputs) ? alternate(scope, locals, assign, inputs) : consequent(scope, locals, assign, inputs);
          return context ? { value: arg } : arg;
        };
      },
      value: function (value, context) {
        return function () {
          return context ? { context: undefined, name: undefined, value: value } : value;
        };
      },
      identifier: function (name, context, create) {
        return function (scope, locals, assign, inputs) {
          var base = locals && name in locals ? locals : scope;
          if (create && create !== 1 && base && base[name] == null) {
            base[name] = {};
          }
          var value = base ? base[name] : undefined;
          if (context) {
            return { context: base, name: name, value: value };
          } else {
            return value;
          }
        };
      },
      computedMember: function (left, right, context, create) {
        return function (scope, locals, assign, inputs) {
          var lhs = left(scope, locals, assign, inputs);
          var rhs;
          var value;
          if (lhs != null) {
            rhs = right(scope, locals, assign, inputs);
            rhs = getStringValue(rhs);
            if (create && create !== 1) {
              if (lhs && !lhs[rhs]) {
                lhs[rhs] = {};
              }
            }
            value = lhs[rhs];
          }
          if (context) {
            return { context: lhs, name: rhs, value: value };
          } else {
            return value;
          }
        };
      },
      nonComputedMember: function (left, right, context, create) {
        return function (scope, locals, assign, inputs) {
          var lhs = left(scope, locals, assign, inputs);
          if (create && create !== 1) {
            if (lhs && lhs[right] == null) {
              lhs[right] = {};
            }
          }
          var value = lhs != null ? lhs[right] : undefined;
          if (context) {
            return { context: lhs, name: right, value: value };
          } else {
            return value;
          }
        };
      },
      inputs: function (input, watchId) {
        return function (scope, value, locals, inputs) {
          if (inputs) return inputs[watchId];
          return input(scope, value, locals);
        };
      }
    };

    /**
     * @constructor
     */
    var Parser = function Parser(lexer, $filter, options) {
      this.lexer = lexer;
      this.$filter = $filter;
      this.options = options;
      this.ast = new AST(lexer, options);
      this.astCompiler = options.csp ? new ASTInterpreter(this.ast, $filter) : new ASTCompiler(this.ast, $filter);
    };

    Parser.prototype = {
      constructor: Parser,

      parse: function (text) {
        return this.astCompiler.compile(text);
      }
    };

    function getValueOf(value) {
      return isFunction(value.valueOf) ? value.valueOf() : objectValueOf.call(value);
    }

    ///////////////////////////////////

    /**
     * @ngdoc service
     * @name $parse
     * @kind function
     *
     * @description
     *
     * Converts Angular {@link guide/expression expression} into a function.
     *
     * ```js
     *   var getter = $parse('user.name');
     *   var setter = getter.assign;
     *   var context = {user:{name:'angular'}};
     *   var locals = {user:{name:'local'}};
     *
     *   expect(getter(context)).toEqual('angular');
     *   setter(context, 'newValue');
     *   expect(context.user.name).toEqual('newValue');
     *   expect(getter(context, locals)).toEqual('local');
     * ```
     *
     *
     * @param {string} expression String expression to compile.
     * @returns {function(context, locals)} a function which represents the compiled expression:
     *
     *    * `context` – `{object}` – an object against which any expressions embedded in the strings
     *      are evaluated against (typically a scope object).
     *    * `locals` – `{object=}` – local variables context object, useful for overriding values in
     *      `context`.
     *
     *    The returned function also has the following properties:
     *      * `literal` – `{boolean}` – whether the expression's top-level node is a JavaScript
     *        literal.
     *      * `constant` – `{boolean}` – whether the expression is made entirely of JavaScript
     *        constant literals.
     *      * `assign` – `{?function(context, value)}` – if the expression is assignable, this will be
     *        set to a function to change its value on the given context.
     *
     */

    /**
     * @ngdoc provider
     * @name $parseProvider
     * @this
     *
     * @description
     * `$parseProvider` can be used for configuring the default behavior of the {@link ng.$parse $parse}
     *  service.
     */
    function $ParseProvider() {
      var cache = createMap();
      var literals = {
        'true': true,
        'false': false,
        'null': null,
        'undefined': undefined
      };
      var identStart, identContinue;

      /**
       * @ngdoc method
       * @name $parseProvider#addLiteral
       * @description
       *
       * Configure $parse service to add literal values that will be present as literal at expressions.
       *
       * @param {string} literalName Token for the literal value. The literal name value must be a valid literal name.
       * @param {*} literalValue Value for this literal. All literal values must be primitives or `undefined`.
       *
       **/
      this.addLiteral = function (literalName, literalValue) {
        literals[literalName] = literalValue;
      };

      /**
       * @ngdoc method
       * @name $parseProvider#setIdentifierFns
       *
       * @description
       *
       * Allows defining the set of characters that are allowed in Angular expressions. The function
       * `identifierStart` will get called to know if a given character is a valid character to be the
       * first character for an identifier. The function `identifierContinue` will get called to know if
       * a given character is a valid character to be a follow-up identifier character. The functions
       * `identifierStart` and `identifierContinue` will receive as arguments the single character to be
       * identifier and the character code point. These arguments will be `string` and `numeric`. Keep in
       * mind that the `string` parameter can be two characters long depending on the character
       * representation. It is expected for the function to return `true` or `false`, whether that
       * character is allowed or not.
       *
       * Since this function will be called extensively, keep the implementation of these functions fast,
       * as the performance of these functions have a direct impact on the expressions parsing speed.
       *
       * @param {function=} identifierStart The function that will decide whether the given character is
       *   a valid identifier start character.
       * @param {function=} identifierContinue The function that will decide whether the given character is
       *   a valid identifier continue character.
       */
      this.setIdentifierFns = function (identifierStart, identifierContinue) {
        identStart = identifierStart;
        identContinue = identifierContinue;
        return this;
      };

      this.$get = ['$filter', function ($filter) {
        var noUnsafeEval = csp().noUnsafeEval;
        var $parseOptions = {
          csp: noUnsafeEval,
          literals: copy(literals),
          isIdentifierStart: isFunction(identStart) && identStart,
          isIdentifierContinue: isFunction(identContinue) && identContinue
        };
        return $parse;

        function $parse(exp, interceptorFn) {
          var parsedExpression, oneTime, cacheKey;

          switch (typeof exp) {
            case 'string':
              exp = exp.trim();
              cacheKey = exp;

              parsedExpression = cache[cacheKey];

              if (!parsedExpression) {
                if (exp.charAt(0) === ':' && exp.charAt(1) === ':') {
                  oneTime = true;
                  exp = exp.substring(2);
                }
                var lexer = new Lexer($parseOptions);
                var parser = new Parser(lexer, $filter, $parseOptions);
                parsedExpression = parser.parse(exp);
                if (parsedExpression.constant) {
                  parsedExpression.$$watchDelegate = constantWatchDelegate;
                } else if (oneTime) {
                  parsedExpression.$$watchDelegate = parsedExpression.literal ? oneTimeLiteralWatchDelegate : oneTimeWatchDelegate;
                } else if (parsedExpression.inputs) {
                  parsedExpression.$$watchDelegate = inputsWatchDelegate;
                }
                cache[cacheKey] = parsedExpression;
              }
              return addInterceptor(parsedExpression, interceptorFn);

            case 'function':
              return addInterceptor(exp, interceptorFn);

            default:
              return addInterceptor(noop, interceptorFn);
          }
        }

        function expressionInputDirtyCheck(newValue, oldValueOfValue, compareObjectIdentity) {

          if (newValue == null || oldValueOfValue == null) {
            // null/undefined
            return newValue === oldValueOfValue;
          }

          if (typeof newValue === 'object' && !compareObjectIdentity) {

            // attempt to convert the value to a primitive type
            // TODO(docs): add a note to docs that by implementing valueOf even objects and arrays can
            //             be cheaply dirty-checked
            newValue = getValueOf(newValue);

            if (typeof newValue === 'object') {
              // objects/arrays are not supported - deep-watching them would be too expensive
              return false;
            }

            // fall-through to the primitive equality check
          }

          //Primitive or NaN
          // eslint-disable-next-line no-self-compare
          return newValue === oldValueOfValue || newValue !== newValue && oldValueOfValue !== oldValueOfValue;
        }

        function inputsWatchDelegate(scope, listener, objectEquality, parsedExpression, prettyPrintExpression) {
          var inputExpressions = parsedExpression.inputs;
          var lastResult;

          if (inputExpressions.length === 1) {
            var oldInputValueOf = expressionInputDirtyCheck; // init to something unique so that equals check fails
            inputExpressions = inputExpressions[0];
            return scope.$watch(function expressionInputWatch(scope) {
              var newInputValue = inputExpressions(scope);
              if (!expressionInputDirtyCheck(newInputValue, oldInputValueOf, parsedExpression.literal)) {
                lastResult = parsedExpression(scope, undefined, undefined, [newInputValue]);
                oldInputValueOf = newInputValue && getValueOf(newInputValue);
              }
              return lastResult;
            }, listener, objectEquality, prettyPrintExpression);
          }

          var oldInputValueOfValues = [];
          var oldInputValues = [];
          for (var i = 0, ii = inputExpressions.length; i < ii; i++) {
            oldInputValueOfValues[i] = expressionInputDirtyCheck; // init to something unique so that equals check fails
            oldInputValues[i] = null;
          }

          return scope.$watch(function expressionInputsWatch(scope) {
            var changed = false;

            for (var i = 0, ii = inputExpressions.length; i < ii; i++) {
              var newInputValue = inputExpressions[i](scope);
              if (changed || (changed = !expressionInputDirtyCheck(newInputValue, oldInputValueOfValues[i], parsedExpression.literal))) {
                oldInputValues[i] = newInputValue;
                oldInputValueOfValues[i] = newInputValue && getValueOf(newInputValue);
              }
            }

            if (changed) {
              lastResult = parsedExpression(scope, undefined, undefined, oldInputValues);
            }

            return lastResult;
          }, listener, objectEquality, prettyPrintExpression);
        }

        function oneTimeWatchDelegate(scope, listener, objectEquality, parsedExpression, prettyPrintExpression) {
          var unwatch, lastValue;
          if (parsedExpression.inputs) {
            unwatch = inputsWatchDelegate(scope, oneTimeListener, objectEquality, parsedExpression, prettyPrintExpression);
          } else {
            unwatch = scope.$watch(oneTimeWatch, oneTimeListener, objectEquality);
          }
          return unwatch;

          function oneTimeWatch(scope) {
            return parsedExpression(scope);
          }
          function oneTimeListener(value, old, scope) {
            lastValue = value;
            if (isFunction(listener)) {
              listener(value, old, scope);
            }
            if (isDefined(value)) {
              scope.$$postDigest(function () {
                if (isDefined(lastValue)) {
                  unwatch();
                }
              });
            }
          }
        }

        function oneTimeLiteralWatchDelegate(scope, listener, objectEquality, parsedExpression) {
          var unwatch, lastValue;
          unwatch = scope.$watch(function oneTimeWatch(scope) {
            return parsedExpression(scope);
          }, function oneTimeListener(value, old, scope) {
            lastValue = value;
            if (isFunction(listener)) {
              listener(value, old, scope);
            }
            if (isAllDefined(value)) {
              scope.$$postDigest(function () {
                if (isAllDefined(lastValue)) unwatch();
              });
            }
          }, objectEquality);

          return unwatch;

          function isAllDefined(value) {
            var allDefined = true;
            forEach(value, function (val) {
              if (!isDefined(val)) allDefined = false;
            });
            return allDefined;
          }
        }

        function constantWatchDelegate(scope, listener, objectEquality, parsedExpression) {
          var unwatch = scope.$watch(function constantWatch(scope) {
            unwatch();
            return parsedExpression(scope);
          }, listener, objectEquality);
          return unwatch;
        }

        function addInterceptor(parsedExpression, interceptorFn) {
          if (!interceptorFn) return parsedExpression;
          var watchDelegate = parsedExpression.$$watchDelegate;
          var useInputs = false;

          var regularWatch = watchDelegate !== oneTimeLiteralWatchDelegate && watchDelegate !== oneTimeWatchDelegate;

          var fn = regularWatch ? function regularInterceptedExpression(scope, locals, assign, inputs) {
            var value = useInputs && inputs ? inputs[0] : parsedExpression(scope, locals, assign, inputs);
            return interceptorFn(value, scope, locals);
          } : function oneTimeInterceptedExpression(scope, locals, assign, inputs) {
            var value = parsedExpression(scope, locals, assign, inputs);
            var result = interceptorFn(value, scope, locals);
            // we only return the interceptor's result if the
            // initial value is defined (for bind-once)
            return isDefined(value) ? result : value;
          };

          // Propagate $$watchDelegates other then inputsWatchDelegate
          useInputs = !parsedExpression.inputs;
          if (parsedExpression.$$watchDelegate && parsedExpression.$$watchDelegate !== inputsWatchDelegate) {
            fn.$$watchDelegate = parsedExpression.$$watchDelegate;
            fn.inputs = parsedExpression.inputs;
          } else if (!interceptorFn.$stateful) {
            // If there is an interceptor, but no watchDelegate then treat the interceptor like
            // we treat filters - it is assumed to be a pure function unless flagged with $stateful
            fn.$$watchDelegate = inputsWatchDelegate;
            fn.inputs = parsedExpression.inputs ? parsedExpression.inputs : [parsedExpression];
          }

          return fn;
        }
      }];
    }

    /**
     * @ngdoc service
     * @name $q
     * @requires $rootScope
     *
     * @description
     * A service that helps you run functions asynchronously, and use their return values (or exceptions)
     * when they are done processing.
     *
     * This is a [Promises/A+](https://promisesaplus.com/)-compliant implementation of promises/deferred
     * objects inspired by [Kris Kowal's Q](https://github.com/kriskowal/q).
     *
     * $q can be used in two fashions --- one which is more similar to Kris Kowal's Q or jQuery's Deferred
     * implementations, and the other which resembles ES6 (ES2015) promises to some degree.
     *
     * # $q constructor
     *
     * The streamlined ES6 style promise is essentially just using $q as a constructor which takes a `resolver`
     * function as the first argument. This is similar to the native Promise implementation from ES6,
     * see [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise).
     *
     * While the constructor-style use is supported, not all of the supporting methods from ES6 promises are
     * available yet.
     *
     * It can be used like so:
     *
     * ```js
     *   // for the purpose of this example let's assume that variables `$q` and `okToGreet`
     *   // are available in the current lexical scope (they could have been injected or passed in).
     *
     *   function asyncGreet(name) {
     *     // perform some asynchronous operation, resolve or reject the promise when appropriate.
     *     return $q(function(resolve, reject) {
     *       setTimeout(function() {
     *         if (okToGreet(name)) {
     *           resolve('Hello, ' + name + '!');
     *         } else {
     *           reject('Greeting ' + name + ' is not allowed.');
     *         }
     *       }, 1000);
     *     });
     *   }
     *
     *   var promise = asyncGreet('Robin Hood');
     *   promise.then(function(greeting) {
     *     alert('Success: ' + greeting);
     *   }, function(reason) {
     *     alert('Failed: ' + reason);
     *   });
     * ```
     *
     * Note: progress/notify callbacks are not currently supported via the ES6-style interface.
     *
     * Note: unlike ES6 behavior, an exception thrown in the constructor function will NOT implicitly reject the promise.
     *
     * However, the more traditional CommonJS-style usage is still available, and documented below.
     *
     * [The CommonJS Promise proposal](http://wiki.commonjs.org/wiki/Promises) describes a promise as an
     * interface for interacting with an object that represents the result of an action that is
     * performed asynchronously, and may or may not be finished at any given point in time.
     *
     * From the perspective of dealing with error handling, deferred and promise APIs are to
     * asynchronous programming what `try`, `catch` and `throw` keywords are to synchronous programming.
     *
     * ```js
     *   // for the purpose of this example let's assume that variables `$q` and `okToGreet`
     *   // are available in the current lexical scope (they could have been injected or passed in).
     *
     *   function asyncGreet(name) {
     *     var deferred = $q.defer();
     *
     *     setTimeout(function() {
     *       deferred.notify('About to greet ' + name + '.');
     *
     *       if (okToGreet(name)) {
     *         deferred.resolve('Hello, ' + name + '!');
     *       } else {
     *         deferred.reject('Greeting ' + name + ' is not allowed.');
     *       }
     *     }, 1000);
     *
     *     return deferred.promise;
     *   }
     *
     *   var promise = asyncGreet('Robin Hood');
     *   promise.then(function(greeting) {
     *     alert('Success: ' + greeting);
     *   }, function(reason) {
     *     alert('Failed: ' + reason);
     *   }, function(update) {
     *     alert('Got notification: ' + update);
     *   });
     * ```
     *
     * At first it might not be obvious why this extra complexity is worth the trouble. The payoff
     * comes in the way of guarantees that promise and deferred APIs make, see
     * https://github.com/kriskowal/uncommonjs/blob/master/promises/specification.md.
     *
     * Additionally the promise api allows for composition that is very hard to do with the
     * traditional callback ([CPS](http://en.wikipedia.org/wiki/Continuation-passing_style)) approach.
     * For more on this please see the [Q documentation](https://github.com/kriskowal/q) especially the
     * section on serial or parallel joining of promises.
     *
     * # The Deferred API
     *
     * A new instance of deferred is constructed by calling `$q.defer()`.
     *
     * The purpose of the deferred object is to expose the associated Promise instance as well as APIs
     * that can be used for signaling the successful or unsuccessful completion, as well as the status
     * of the task.
     *
     * **Methods**
     *
     * - `resolve(value)` – resolves the derived promise with the `value`. If the value is a rejection
     *   constructed via `$q.reject`, the promise will be rejected instead.
     * - `reject(reason)` – rejects the derived promise with the `reason`. This is equivalent to
     *   resolving it with a rejection constructed via `$q.reject`.
     * - `notify(value)` - provides updates on the status of the promise's execution. This may be called
     *   multiple times before the promise is either resolved or rejected.
     *
     * **Properties**
     *
     * - promise – `{Promise}` – promise object associated with this deferred.
     *
     *
     * # The Promise API
     *
     * A new promise instance is created when a deferred instance is created and can be retrieved by
     * calling `deferred.promise`.
     *
     * The purpose of the promise object is to allow for interested parties to get access to the result
     * of the deferred task when it completes.
     *
     * **Methods**
     *
     * - `then(successCallback, [errorCallback], [notifyCallback])` – regardless of when the promise was or
     *   will be resolved or rejected, `then` calls one of the success or error callbacks asynchronously
     *   as soon as the result is available. The callbacks are called with a single argument: the result
     *   or rejection reason. Additionally, the notify callback may be called zero or more times to
     *   provide a progress indication, before the promise is resolved or rejected.
     *
     *   This method *returns a new promise* which is resolved or rejected via the return value of the
     *   `successCallback`, `errorCallback` (unless that value is a promise, in which case it is resolved
     *   with the value which is resolved in that promise using
     *   [promise chaining](http://www.html5rocks.com/en/tutorials/es6/promises/#toc-promises-queues)).
     *   It also notifies via the return value of the `notifyCallback` method. The promise cannot be
     *   resolved or rejected from the notifyCallback method. The errorCallback and notifyCallback
     *   arguments are optional.
     *
     * - `catch(errorCallback)` – shorthand for `promise.then(null, errorCallback)`
     *
     * - `finally(callback, notifyCallback)` – allows you to observe either the fulfillment or rejection of a promise,
     *   but to do so without modifying the final value. This is useful to release resources or do some
     *   clean-up that needs to be done whether the promise was rejected or resolved. See the [full
     *   specification](https://github.com/kriskowal/q/wiki/API-Reference#promisefinallycallback) for
     *   more information.
     *
     * # Chaining promises
     *
     * Because calling the `then` method of a promise returns a new derived promise, it is easily
     * possible to create a chain of promises:
     *
     * ```js
     *   promiseB = promiseA.then(function(result) {
     *     return result + 1;
     *   });
     *
     *   // promiseB will be resolved immediately after promiseA is resolved and its value
     *   // will be the result of promiseA incremented by 1
     * ```
     *
     * It is possible to create chains of any length and since a promise can be resolved with another
     * promise (which will defer its resolution further), it is possible to pause/defer resolution of
     * the promises at any point in the chain. This makes it possible to implement powerful APIs like
     * $http's response interceptors.
     *
     *
     * # Differences between Kris Kowal's Q and $q
     *
     *  There are two main differences:
     *
     * - $q is integrated with the {@link ng.$rootScope.Scope} Scope model observation
     *   mechanism in angular, which means faster propagation of resolution or rejection into your
     *   models and avoiding unnecessary browser repaints, which would result in flickering UI.
     * - Q has many more features than $q, but that comes at a cost of bytes. $q is tiny, but contains
     *   all the important functionality needed for common async tasks.
     *
     * # Testing
     *
     *  ```js
     *    it('should simulate promise', inject(function($q, $rootScope) {
     *      var deferred = $q.defer();
     *      var promise = deferred.promise;
     *      var resolvedValue;
     *
     *      promise.then(function(value) { resolvedValue = value; });
     *      expect(resolvedValue).toBeUndefined();
     *
     *      // Simulate resolving of promise
     *      deferred.resolve(123);
     *      // Note that the 'then' function does not get called synchronously.
     *      // This is because we want the promise API to always be async, whether or not
     *      // it got called synchronously or asynchronously.
     *      expect(resolvedValue).toBeUndefined();
     *
     *      // Propagate promise resolution to 'then' functions using $apply().
     *      $rootScope.$apply();
     *      expect(resolvedValue).toEqual(123);
     *    }));
     *  ```
     *
     * @param {function(function, function)} resolver Function which is responsible for resolving or
     *   rejecting the newly created promise. The first parameter is a function which resolves the
     *   promise, the second parameter is a function which rejects the promise.
     *
     * @returns {Promise} The newly created promise.
     */
    /**
     * @ngdoc provider
     * @name $qProvider
     * @this
     *
     * @description
     */
    function $QProvider() {
      var errorOnUnhandledRejections = true;
      this.$get = ['$rootScope', '$exceptionHandler', function ($rootScope, $exceptionHandler) {
        return qFactory(function (callback) {
          $rootScope.$evalAsync(callback);
        }, $exceptionHandler, errorOnUnhandledRejections);
      }];

      /**
       * @ngdoc method
       * @name $qProvider#errorOnUnhandledRejections
       * @kind function
       *
       * @description
       * Retrieves or overrides whether to generate an error when a rejected promise is not handled.
       * This feature is enabled by default.
       *
       * @param {boolean=} value Whether to generate an error when a rejected promise is not handled.
       * @returns {boolean|ng.$qProvider} Current value when called without a new value or self for
       *    chaining otherwise.
       */
      this.errorOnUnhandledRejections = function (value) {
        if (isDefined(value)) {
          errorOnUnhandledRejections = value;
          return this;
        } else {
          return errorOnUnhandledRejections;
        }
      };
    }

    /** @this */
    function $$QProvider() {
      var errorOnUnhandledRejections = true;
      this.$get = ['$browser', '$exceptionHandler', function ($browser, $exceptionHandler) {
        return qFactory(function (callback) {
          $browser.defer(callback);
        }, $exceptionHandler, errorOnUnhandledRejections);
      }];

      this.errorOnUnhandledRejections = function (value) {
        if (isDefined(value)) {
          errorOnUnhandledRejections = value;
          return this;
        } else {
          return errorOnUnhandledRejections;
        }
      };
    }

    /**
     * Constructs a promise manager.
     *
     * @param {function(function)} nextTick Function for executing functions in the next turn.
     * @param {function(...*)} exceptionHandler Function into which unexpected exceptions are passed for
     *     debugging purposes.
     @ param {=boolean} errorOnUnhandledRejections Whether an error should be generated on unhandled
     *     promises rejections.
     * @returns {object} Promise manager.
     */
    function qFactory(nextTick, exceptionHandler, errorOnUnhandledRejections) {
      var $qMinErr = minErr('$q', TypeError);
      var queueSize = 0;
      var checkQueue = [];

      /**
       * @ngdoc method
       * @name ng.$q#defer
       * @kind function
       *
       * @description
       * Creates a `Deferred` object which represents a task which will finish in the future.
       *
       * @returns {Deferred} Returns a new instance of deferred.
       */
      function defer() {
        return new Deferred();
      }

      function Deferred() {
        var promise = this.promise = new Promise();
        //Non prototype methods necessary to support unbound execution :/
        this.resolve = function (val) {
          resolvePromise(promise, val);
        };
        this.reject = function (reason) {
          rejectPromise(promise, reason);
        };
        this.notify = function (progress) {
          notifyPromise(promise, progress);
        };
      }

      function Promise() {
        this.$$state = { status: 0 };
      }

      extend(Promise.prototype, {
        then: function (onFulfilled, onRejected, progressBack) {
          if (isUndefined(onFulfilled) && isUndefined(onRejected) && isUndefined(progressBack)) {
            return this;
          }
          var result = new Promise();

          this.$$state.pending = this.$$state.pending || [];
          this.$$state.pending.push([result, onFulfilled, onRejected, progressBack]);
          if (this.$$state.status > 0) scheduleProcessQueue(this.$$state);

          return result;
        },

        'catch': function (callback) {
          return this.then(null, callback);
        },

        'finally': function (callback, progressBack) {
          return this.then(function (value) {
            return handleCallback(value, resolve, callback);
          }, function (error) {
            return handleCallback(error, reject, callback);
          }, progressBack);
        }
      });

      function processQueue(state) {
        var fn, promise, pending;

        pending = state.pending;
        state.processScheduled = false;
        state.pending = undefined;
        try {
          for (var i = 0, ii = pending.length; i < ii; ++i) {
            state.pur = true;
            promise = pending[i][0];
            fn = pending[i][state.status];
            try {
              if (isFunction(fn)) {
                resolvePromise(promise, fn(state.value));
              } else if (state.status === 1) {
                resolvePromise(promise, state.value);
              } else {
                rejectPromise(promise, state.value);
              }
            } catch (e) {
              rejectPromise(promise, e);
            }
          }
        } finally {
          --queueSize;
          if (errorOnUnhandledRejections && queueSize === 0) {
            nextTick(processChecks);
          }
        }
      }

      function processChecks() {
        // eslint-disable-next-line no-unmodified-loop-condition
        while (!queueSize && checkQueue.length) {
          var toCheck = checkQueue.shift();
          if (!toCheck.pur) {
            toCheck.pur = true;
            var errorMessage = 'Possibly unhandled rejection: ' + toDebugString(toCheck.value);
            if (toCheck.value instanceof Error) {
              exceptionHandler(toCheck.value, errorMessage);
            } else {
              exceptionHandler(errorMessage);
            }
          }
        }
      }

      function scheduleProcessQueue(state) {
        if (errorOnUnhandledRejections && !state.pending && state.status === 2 && !state.pur) {
          if (queueSize === 0 && checkQueue.length === 0) {
            nextTick(processChecks);
          }
          checkQueue.push(state);
        }
        if (state.processScheduled || !state.pending) return;
        state.processScheduled = true;
        ++queueSize;
        nextTick(function () {
          processQueue(state);
        });
      }

      function resolvePromise(promise, val) {
        if (promise.$$state.status) return;
        if (val === promise) {
          $$reject(promise, $qMinErr('qcycle', 'Expected promise to be resolved with value other than itself \'{0}\'', val));
        } else {
          $$resolve(promise, val);
        }
      }

      function $$resolve(promise, val) {
        var then;
        var done = false;
        try {
          if (isObject(val) || isFunction(val)) then = val.then;
          if (isFunction(then)) {
            promise.$$state.status = -1;
            then.call(val, doResolve, doReject, doNotify);
          } else {
            promise.$$state.value = val;
            promise.$$state.status = 1;
            scheduleProcessQueue(promise.$$state);
          }
        } catch (e) {
          doReject(e);
        }

        function doResolve(val) {
          if (done) return;
          done = true;
          $$resolve(promise, val);
        }
        function doReject(val) {
          if (done) return;
          done = true;
          $$reject(promise, val);
        }
        function doNotify(progress) {
          notifyPromise(promise, progress);
        }
      }

      function rejectPromise(promise, reason) {
        if (promise.$$state.status) return;
        $$reject(promise, reason);
      }

      function $$reject(promise, reason) {
        promise.$$state.value = reason;
        promise.$$state.status = 2;
        scheduleProcessQueue(promise.$$state);
      }

      function notifyPromise(promise, progress) {
        var callbacks = promise.$$state.pending;

        if (promise.$$state.status <= 0 && callbacks && callbacks.length) {
          nextTick(function () {
            var callback, result;
            for (var i = 0, ii = callbacks.length; i < ii; i++) {
              result = callbacks[i][0];
              callback = callbacks[i][3];
              try {
                notifyPromise(result, isFunction(callback) ? callback(progress) : progress);
              } catch (e) {
                exceptionHandler(e);
              }
            }
          });
        }
      }

      /**
       * @ngdoc method
       * @name $q#reject
       * @kind function
       *
       * @description
       * Creates a promise that is resolved as rejected with the specified `reason`. This api should be
       * used to forward rejection in a chain of promises. If you are dealing with the last promise in
       * a promise chain, you don't need to worry about it.
       *
       * When comparing deferreds/promises to the familiar behavior of try/catch/throw, think of
       * `reject` as the `throw` keyword in JavaScript. This also means that if you "catch" an error via
       * a promise error callback and you want to forward the error to the promise derived from the
       * current promise, you have to "rethrow" the error by returning a rejection constructed via
       * `reject`.
       *
       * ```js
       *   promiseB = promiseA.then(function(result) {
       *     // success: do something and resolve promiseB
       *     //          with the old or a new result
       *     return result;
       *   }, function(reason) {
       *     // error: handle the error if possible and
       *     //        resolve promiseB with newPromiseOrValue,
       *     //        otherwise forward the rejection to promiseB
       *     if (canHandle(reason)) {
       *      // handle the error and recover
       *      return newPromiseOrValue;
       *     }
       *     return $q.reject(reason);
       *   });
       * ```
       *
       * @param {*} reason Constant, message, exception or an object representing the rejection reason.
       * @returns {Promise} Returns a promise that was already resolved as rejected with the `reason`.
       */
      function reject(reason) {
        var result = new Promise();
        rejectPromise(result, reason);
        return result;
      }

      function handleCallback(value, resolver, callback) {
        var callbackOutput = null;
        try {
          if (isFunction(callback)) callbackOutput = callback();
        } catch (e) {
          return reject(e);
        }
        if (isPromiseLike(callbackOutput)) {
          return callbackOutput.then(function () {
            return resolver(value);
          }, reject);
        } else {
          return resolver(value);
        }
      }

      /**
       * @ngdoc method
       * @name $q#when
       * @kind function
       *
       * @description
       * Wraps an object that might be a value or a (3rd party) then-able promise into a $q promise.
       * This is useful when you are dealing with an object that might or might not be a promise, or if
       * the promise comes from a source that can't be trusted.
       *
       * @param {*} value Value or a promise
       * @param {Function=} successCallback
       * @param {Function=} errorCallback
       * @param {Function=} progressCallback
       * @returns {Promise} Returns a promise of the passed value or promise
       */

      function when(value, callback, errback, progressBack) {
        var result = new Promise();
        resolvePromise(result, value);
        return result.then(callback, errback, progressBack);
      }

      /**
       * @ngdoc method
       * @name $q#resolve
       * @kind function
       *
       * @description
       * Alias of {@link ng.$q#when when} to maintain naming consistency with ES6.
       *
       * @param {*} value Value or a promise
       * @param {Function=} successCallback
       * @param {Function=} errorCallback
       * @param {Function=} progressCallback
       * @returns {Promise} Returns a promise of the passed value or promise
       */
      var resolve = when;

      /**
       * @ngdoc method
       * @name $q#all
       * @kind function
       *
       * @description
       * Combines multiple promises into a single promise that is resolved when all of the input
       * promises are resolved.
       *
       * @param {Array.<Promise>|Object.<Promise>} promises An array or hash of promises.
       * @returns {Promise} Returns a single promise that will be resolved with an array/hash of values,
       *   each value corresponding to the promise at the same index/key in the `promises` array/hash.
       *   If any of the promises is resolved with a rejection, this resulting promise will be rejected
       *   with the same rejection value.
       */

      function all(promises) {
        var result = new Promise(),
            counter = 0,
            results = isArray(promises) ? [] : {};

        forEach(promises, function (promise, key) {
          counter++;
          when(promise).then(function (value) {
            results[key] = value;
            if (! --counter) resolvePromise(result, results);
          }, function (reason) {
            rejectPromise(result, reason);
          });
        });

        if (counter === 0) {
          resolvePromise(result, results);
        }

        return result;
      }

      /**
       * @ngdoc method
       * @name $q#race
       * @kind function
       *
       * @description
       * Returns a promise that resolves or rejects as soon as one of those promises
       * resolves or rejects, with the value or reason from that promise.
       *
       * @param {Array.<Promise>|Object.<Promise>} promises An array or hash of promises.
       * @returns {Promise} a promise that resolves or rejects as soon as one of the `promises`
       * resolves or rejects, with the value or reason from that promise.
       */

      function race(promises) {
        var deferred = defer();

        forEach(promises, function (promise) {
          when(promise).then(deferred.resolve, deferred.reject);
        });

        return deferred.promise;
      }

      function $Q(resolver) {
        if (!isFunction(resolver)) {
          throw $qMinErr('norslvr', 'Expected resolverFn, got \'{0}\'', resolver);
        }

        var promise = new Promise();

        function resolveFn(value) {
          resolvePromise(promise, value);
        }

        function rejectFn(reason) {
          rejectPromise(promise, reason);
        }

        resolver(resolveFn, rejectFn);

        return promise;
      }

      // Let's make the instanceof operator work for promises, so that
      // `new $q(fn) instanceof $q` would evaluate to true.
      $Q.prototype = Promise.prototype;

      $Q.defer = defer;
      $Q.reject = reject;
      $Q.when = when;
      $Q.resolve = resolve;
      $Q.all = all;
      $Q.race = race;

      return $Q;
    }

    /** @this */
    function $$RAFProvider() {
      //rAF
      this.$get = ['$window', '$timeout', function ($window, $timeout) {
        var requestAnimationFrame = $window.requestAnimationFrame || $window.webkitRequestAnimationFrame;

        var cancelAnimationFrame = $window.cancelAnimationFrame || $window.webkitCancelAnimationFrame || $window.webkitCancelRequestAnimationFrame;

        var rafSupported = !!requestAnimationFrame;
        var raf = rafSupported ? function (fn) {
          var id = requestAnimationFrame(fn);
          return function () {
            cancelAnimationFrame(id);
          };
        } : function (fn) {
          var timer = $timeout(fn, 16.66, false); // 1000 / 60 = 16.666
          return function () {
            $timeout.cancel(timer);
          };
        };

        raf.supported = rafSupported;

        return raf;
      }];
    }

    /**
     * DESIGN NOTES
     *
     * The design decisions behind the scope are heavily favored for speed and memory consumption.
     *
     * The typical use of scope is to watch the expressions, which most of the time return the same
     * value as last time so we optimize the operation.
     *
     * Closures construction is expensive in terms of speed as well as memory:
     *   - No closures, instead use prototypical inheritance for API
     *   - Internal state needs to be stored on scope directly, which means that private state is
     *     exposed as $$____ properties
     *
     * Loop operations are optimized by using while(count--) { ... }
     *   - This means that in order to keep the same order of execution as addition we have to add
     *     items to the array at the beginning (unshift) instead of at the end (push)
     *
     * Child scopes are created and removed often
     *   - Using an array would be slow since inserts in the middle are expensive; so we use linked lists
     *
     * There are fewer watches than observers. This is why you don't want the observer to be implemented
     * in the same way as watch. Watch requires return of the initialization function which is expensive
     * to construct.
     */

    /**
     * @ngdoc provider
     * @name $rootScopeProvider
     * @description
     *
     * Provider for the $rootScope service.
     */

    /**
     * @ngdoc method
     * @name $rootScopeProvider#digestTtl
     * @description
     *
     * Sets the number of `$digest` iterations the scope should attempt to execute before giving up and
     * assuming that the model is unstable.
     *
     * The current default is 10 iterations.
     *
     * In complex applications it's possible that the dependencies between `$watch`s will result in
     * several digest iterations. However if an application needs more than the default 10 digest
     * iterations for its model to stabilize then you should investigate what is causing the model to
     * continuously change during the digest.
     *
     * Increasing the TTL could have performance implications, so you should not change it without
     * proper justification.
     *
     * @param {number} limit The number of digest iterations.
     */

    /**
     * @ngdoc service
     * @name $rootScope
     * @this
     *
     * @description
     *
     * Every application has a single root {@link ng.$rootScope.Scope scope}.
     * All other scopes are descendant scopes of the root scope. Scopes provide separation
     * between the model and the view, via a mechanism for watching the model for changes.
     * They also provide event emission/broadcast and subscription facility. See the
     * {@link guide/scope developer guide on scopes}.
     */
    function $RootScopeProvider() {
      var TTL = 10;
      var $rootScopeMinErr = minErr('$rootScope');
      var lastDirtyWatch = null;
      var applyAsyncId = null;

      this.digestTtl = function (value) {
        if (arguments.length) {
          TTL = value;
        }
        return TTL;
      };

      function createChildScopeClass(parent) {
        function ChildScope() {
          this.$$watchers = this.$$nextSibling = this.$$childHead = this.$$childTail = null;
          this.$$listeners = {};
          this.$$listenerCount = {};
          this.$$watchersCount = 0;
          this.$id = nextUid();
          this.$$ChildScope = null;
        }
        ChildScope.prototype = parent;
        return ChildScope;
      }

      this.$get = ['$exceptionHandler', '$parse', '$browser', function ($exceptionHandler, $parse, $browser) {

        function destroyChildScope($event) {
          $event.currentScope.$$destroyed = true;
        }

        function cleanUpScope($scope) {

          // Support: IE 9 only
          if (msie === 9) {
            // There is a memory leak in IE9 if all child scopes are not disconnected
            // completely when a scope is destroyed. So this code will recurse up through
            // all this scopes children
            //
            // See issue https://github.com/angular/angular.js/issues/10706
            if ($scope.$$childHead) {
              cleanUpScope($scope.$$childHead);
            }
            if ($scope.$$nextSibling) {
              cleanUpScope($scope.$$nextSibling);
            }
          }

          // The code below works around IE9 and V8's memory leaks
          //
          // See:
          // - https://code.google.com/p/v8/issues/detail?id=2073#c26
          // - https://github.com/angular/angular.js/issues/6794#issuecomment-38648909
          // - https://github.com/angular/angular.js/issues/1313#issuecomment-10378451

          $scope.$parent = $scope.$$nextSibling = $scope.$$prevSibling = $scope.$$childHead = $scope.$$childTail = $scope.$root = $scope.$$watchers = null;
        }

        /**
         * @ngdoc type
         * @name $rootScope.Scope
         *
         * @description
         * A root scope can be retrieved using the {@link ng.$rootScope $rootScope} key from the
         * {@link auto.$injector $injector}. Child scopes are created using the
         * {@link ng.$rootScope.Scope#$new $new()} method. (Most scopes are created automatically when
         * compiled HTML template is executed.) See also the {@link guide/scope Scopes guide} for
         * an in-depth introduction and usage examples.
         *
         *
         * # Inheritance
         * A scope can inherit from a parent scope, as in this example:
         * ```js
             var parent = $rootScope;
             var child = parent.$new();
              parent.salutation = "Hello";
             expect(child.salutation).toEqual('Hello');
              child.salutation = "Welcome";
             expect(child.salutation).toEqual('Welcome');
             expect(parent.salutation).toEqual('Hello');
         * ```
         *
         * When interacting with `Scope` in tests, additional helper methods are available on the
         * instances of `Scope` type. See {@link ngMock.$rootScope.Scope ngMock Scope} for additional
         * details.
         *
         *
         * @param {Object.<string, function()>=} providers Map of service factory which need to be
         *                                       provided for the current scope. Defaults to {@link ng}.
         * @param {Object.<string, *>=} instanceCache Provides pre-instantiated services which should
         *                              append/override services provided by `providers`. This is handy
         *                              when unit-testing and having the need to override a default
         *                              service.
         * @returns {Object} Newly created scope.
         *
         */
        function Scope() {
          this.$id = nextUid();
          this.$$phase = this.$parent = this.$$watchers = this.$$nextSibling = this.$$prevSibling = this.$$childHead = this.$$childTail = null;
          this.$root = this;
          this.$$destroyed = false;
          this.$$listeners = {};
          this.$$listenerCount = {};
          this.$$watchersCount = 0;
          this.$$isolateBindings = null;
        }

        /**
         * @ngdoc property
         * @name $rootScope.Scope#$id
         *
         * @description
         * Unique scope ID (monotonically increasing) useful for debugging.
         */

        /**
         * @ngdoc property
         * @name $rootScope.Scope#$parent
         *
         * @description
         * Reference to the parent scope.
         */

        /**
         * @ngdoc property
         * @name $rootScope.Scope#$root
         *
         * @description
         * Reference to the root scope.
         */

        Scope.prototype = {
          constructor: Scope,
          /**
           * @ngdoc method
           * @name $rootScope.Scope#$new
           * @kind function
           *
           * @description
           * Creates a new child {@link ng.$rootScope.Scope scope}.
           *
           * The parent scope will propagate the {@link ng.$rootScope.Scope#$digest $digest()} event.
           * The scope can be removed from the scope hierarchy using {@link ng.$rootScope.Scope#$destroy $destroy()}.
           *
           * {@link ng.$rootScope.Scope#$destroy $destroy()} must be called on a scope when it is
           * desired for the scope and its child scopes to be permanently detached from the parent and
           * thus stop participating in model change detection and listener notification by invoking.
           *
           * @param {boolean} isolate If true, then the scope does not prototypically inherit from the
           *         parent scope. The scope is isolated, as it can not see parent scope properties.
           *         When creating widgets, it is useful for the widget to not accidentally read parent
           *         state.
           *
           * @param {Scope} [parent=this] The {@link ng.$rootScope.Scope `Scope`} that will be the `$parent`
           *                              of the newly created scope. Defaults to `this` scope if not provided.
           *                              This is used when creating a transclude scope to correctly place it
           *                              in the scope hierarchy while maintaining the correct prototypical
           *                              inheritance.
           *
           * @returns {Object} The newly created child scope.
           *
           */
          $new: function (isolate, parent) {
            var child;

            parent = parent || this;

            if (isolate) {
              child = new Scope();
              child.$root = this.$root;
            } else {
              // Only create a child scope class if somebody asks for one,
              // but cache it to allow the VM to optimize lookups.
              if (!this.$$ChildScope) {
                this.$$ChildScope = createChildScopeClass(this);
              }
              child = new this.$$ChildScope();
            }
            child.$parent = parent;
            child.$$prevSibling = parent.$$childTail;
            if (parent.$$childHead) {
              parent.$$childTail.$$nextSibling = child;
              parent.$$childTail = child;
            } else {
              parent.$$childHead = parent.$$childTail = child;
            }

            // When the new scope is not isolated or we inherit from `this`, and
            // the parent scope is destroyed, the property `$$destroyed` is inherited
            // prototypically. In all other cases, this property needs to be set
            // when the parent scope is destroyed.
            // The listener needs to be added after the parent is set
            if (isolate || parent !== this) child.$on('$destroy', destroyChildScope);

            return child;
          },

          /**
           * @ngdoc method
           * @name $rootScope.Scope#$watch
           * @kind function
           *
           * @description
           * Registers a `listener` callback to be executed whenever the `watchExpression` changes.
           *
           * - The `watchExpression` is called on every call to {@link ng.$rootScope.Scope#$digest
           *   $digest()} and should return the value that will be watched. (`watchExpression` should not change
           *   its value when executed multiple times with the same input because it may be executed multiple
           *   times by {@link ng.$rootScope.Scope#$digest $digest()}. That is, `watchExpression` should be
           *   [idempotent](http://en.wikipedia.org/wiki/Idempotence).)
           * - The `listener` is called only when the value from the current `watchExpression` and the
           *   previous call to `watchExpression` are not equal (with the exception of the initial run,
           *   see below). Inequality is determined according to reference inequality,
           *   [strict comparison](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/Comparison_Operators)
           *    via the `!==` Javascript operator, unless `objectEquality == true`
           *   (see next point)
           * - When `objectEquality == true`, inequality of the `watchExpression` is determined
           *   according to the {@link angular.equals} function. To save the value of the object for
           *   later comparison, the {@link angular.copy} function is used. This therefore means that
           *   watching complex objects will have adverse memory and performance implications.
           * - This should not be used to watch for changes in objects that are
           *   or contain [File](https://developer.mozilla.org/docs/Web/API/File) objects due to limitations with {@link angular.copy `angular.copy`}.
           * - The watch `listener` may change the model, which may trigger other `listener`s to fire.
           *   This is achieved by rerunning the watchers until no changes are detected. The rerun
           *   iteration limit is 10 to prevent an infinite loop deadlock.
           *
           *
           * If you want to be notified whenever {@link ng.$rootScope.Scope#$digest $digest} is called,
           * you can register a `watchExpression` function with no `listener`. (Be prepared for
           * multiple calls to your `watchExpression` because it will execute multiple times in a
           * single {@link ng.$rootScope.Scope#$digest $digest} cycle if a change is detected.)
           *
           * After a watcher is registered with the scope, the `listener` fn is called asynchronously
           * (via {@link ng.$rootScope.Scope#$evalAsync $evalAsync}) to initialize the
           * watcher. In rare cases, this is undesirable because the listener is called when the result
           * of `watchExpression` didn't change. To detect this scenario within the `listener` fn, you
           * can compare the `newVal` and `oldVal`. If these two values are identical (`===`) then the
           * listener was called due to initialization.
           *
           *
           *
           * # Example
           * ```js
               // let's assume that scope was dependency injected as the $rootScope
               var scope = $rootScope;
               scope.name = 'misko';
               scope.counter = 0;
                expect(scope.counter).toEqual(0);
               scope.$watch('name', function(newValue, oldValue) {
                 scope.counter = scope.counter + 1;
               });
               expect(scope.counter).toEqual(0);
                scope.$digest();
               // the listener is always called during the first $digest loop after it was registered
               expect(scope.counter).toEqual(1);
                scope.$digest();
               // but now it will not be called unless the value changes
               expect(scope.counter).toEqual(1);
                scope.name = 'adam';
               scope.$digest();
               expect(scope.counter).toEqual(2);
                  // Using a function as a watchExpression
               var food;
               scope.foodCounter = 0;
               expect(scope.foodCounter).toEqual(0);
               scope.$watch(
                 // This function returns the value being watched. It is called for each turn of the $digest loop
                 function() { return food; },
                 // This is the change listener, called when the value returned from the above function changes
                 function(newValue, oldValue) {
                   if ( newValue !== oldValue ) {
                     // Only increment the counter if the value changed
                     scope.foodCounter = scope.foodCounter + 1;
                   }
                 }
               );
               // No digest has been run so the counter will be zero
               expect(scope.foodCounter).toEqual(0);
                // Run the digest but since food has not changed count will still be zero
               scope.$digest();
               expect(scope.foodCounter).toEqual(0);
                // Update food and run digest.  Now the counter will increment
               food = 'cheeseburger';
               scope.$digest();
               expect(scope.foodCounter).toEqual(1);
            * ```
           *
           *
           *
           * @param {(function()|string)} watchExpression Expression that is evaluated on each
           *    {@link ng.$rootScope.Scope#$digest $digest} cycle. A change in the return value triggers
           *    a call to the `listener`.
           *
           *    - `string`: Evaluated as {@link guide/expression expression}
           *    - `function(scope)`: called with current `scope` as a parameter.
           * @param {function(newVal, oldVal, scope)} listener Callback called whenever the value
           *    of `watchExpression` changes.
           *
           *    - `newVal` contains the current value of the `watchExpression`
           *    - `oldVal` contains the previous value of the `watchExpression`
           *    - `scope` refers to the current scope
           * @param {boolean=} [objectEquality=false] Compare for object equality using {@link angular.equals} instead of
           *     comparing for reference equality.
           * @returns {function()} Returns a deregistration function for this listener.
           */
          $watch: function (watchExp, listener, objectEquality, prettyPrintExpression) {
            var get = $parse(watchExp);

            if (get.$$watchDelegate) {
              return get.$$watchDelegate(this, listener, objectEquality, get, watchExp);
            }
            var scope = this,
                array = scope.$$watchers,
                watcher = {
              fn: listener,
              last: initWatchVal,
              get: get,
              exp: prettyPrintExpression || watchExp,
              eq: !!objectEquality
            };

            lastDirtyWatch = null;

            if (!isFunction(listener)) {
              watcher.fn = noop;
            }

            if (!array) {
              array = scope.$$watchers = [];
              array.$$digestWatchIndex = -1;
            }
            // we use unshift since we use a while loop in $digest for speed.
            // the while loop reads in reverse order.
            array.unshift(watcher);
            array.$$digestWatchIndex++;
            incrementWatchersCount(this, 1);

            return function deregisterWatch() {
              var index = arrayRemove(array, watcher);
              if (index >= 0) {
                incrementWatchersCount(scope, -1);
                if (index < array.$$digestWatchIndex) {
                  array.$$digestWatchIndex--;
                }
              }
              lastDirtyWatch = null;
            };
          },

          /**
           * @ngdoc method
           * @name $rootScope.Scope#$watchGroup
           * @kind function
           *
           * @description
           * A variant of {@link ng.$rootScope.Scope#$watch $watch()} where it watches an array of `watchExpressions`.
           * If any one expression in the collection changes the `listener` is executed.
           *
           * - The items in the `watchExpressions` array are observed via the standard `$watch` operation. Their return
           *   values are examined for changes on every call to `$digest`.
           * - The `listener` is called whenever any expression in the `watchExpressions` array changes.
           *
           * @param {Array.<string|Function(scope)>} watchExpressions Array of expressions that will be individually
           * watched using {@link ng.$rootScope.Scope#$watch $watch()}
           *
           * @param {function(newValues, oldValues, scope)} listener Callback called whenever the return value of any
           *    expression in `watchExpressions` changes
           *    The `newValues` array contains the current values of the `watchExpressions`, with the indexes matching
           *    those of `watchExpression`
           *    and the `oldValues` array contains the previous values of the `watchExpressions`, with the indexes matching
           *    those of `watchExpression`
           *    The `scope` refers to the current scope.
           * @returns {function()} Returns a de-registration function for all listeners.
           */
          $watchGroup: function (watchExpressions, listener) {
            var oldValues = new Array(watchExpressions.length);
            var newValues = new Array(watchExpressions.length);
            var deregisterFns = [];
            var self = this;
            var changeReactionScheduled = false;
            var firstRun = true;

            if (!watchExpressions.length) {
              // No expressions means we call the listener ASAP
              var shouldCall = true;
              self.$evalAsync(function () {
                if (shouldCall) listener(newValues, newValues, self);
              });
              return function deregisterWatchGroup() {
                shouldCall = false;
              };
            }

            if (watchExpressions.length === 1) {
              // Special case size of one
              return this.$watch(watchExpressions[0], function watchGroupAction(value, oldValue, scope) {
                newValues[0] = value;
                oldValues[0] = oldValue;
                listener(newValues, value === oldValue ? newValues : oldValues, scope);
              });
            }

            forEach(watchExpressions, function (expr, i) {
              var unwatchFn = self.$watch(expr, function watchGroupSubAction(value, oldValue) {
                newValues[i] = value;
                oldValues[i] = oldValue;
                if (!changeReactionScheduled) {
                  changeReactionScheduled = true;
                  self.$evalAsync(watchGroupAction);
                }
              });
              deregisterFns.push(unwatchFn);
            });

            function watchGroupAction() {
              changeReactionScheduled = false;

              if (firstRun) {
                firstRun = false;
                listener(newValues, newValues, self);
              } else {
                listener(newValues, oldValues, self);
              }
            }

            return function deregisterWatchGroup() {
              while (deregisterFns.length) {
                deregisterFns.shift()();
              }
            };
          },

          /**
           * @ngdoc method
           * @name $rootScope.Scope#$watchCollection
           * @kind function
           *
           * @description
           * Shallow watches the properties of an object and fires whenever any of the properties change
           * (for arrays, this implies watching the array items; for object maps, this implies watching
           * the properties). If a change is detected, the `listener` callback is fired.
           *
           * - The `obj` collection is observed via standard $watch operation and is examined on every
           *   call to $digest() to see if any items have been added, removed, or moved.
           * - The `listener` is called whenever anything within the `obj` has changed. Examples include
           *   adding, removing, and moving items belonging to an object or array.
           *
           *
           * # Example
           * ```js
              $scope.names = ['igor', 'matias', 'misko', 'james'];
              $scope.dataCount = 4;
               $scope.$watchCollection('names', function(newNames, oldNames) {
                $scope.dataCount = newNames.length;
              });
               expect($scope.dataCount).toEqual(4);
              $scope.$digest();
               //still at 4 ... no changes
              expect($scope.dataCount).toEqual(4);
               $scope.names.pop();
              $scope.$digest();
               //now there's been a change
              expect($scope.dataCount).toEqual(3);
           * ```
           *
           *
           * @param {string|function(scope)} obj Evaluated as {@link guide/expression expression}. The
           *    expression value should evaluate to an object or an array which is observed on each
           *    {@link ng.$rootScope.Scope#$digest $digest} cycle. Any shallow change within the
           *    collection will trigger a call to the `listener`.
           *
           * @param {function(newCollection, oldCollection, scope)} listener a callback function called
           *    when a change is detected.
           *    - The `newCollection` object is the newly modified data obtained from the `obj` expression
           *    - The `oldCollection` object is a copy of the former collection data.
           *      Due to performance considerations, the`oldCollection` value is computed only if the
           *      `listener` function declares two or more arguments.
           *    - The `scope` argument refers to the current scope.
           *
           * @returns {function()} Returns a de-registration function for this listener. When the
           *    de-registration function is executed, the internal watch operation is terminated.
           */
          $watchCollection: function (obj, listener) {
            $watchCollectionInterceptor.$stateful = true;

            var self = this;
            // the current value, updated on each dirty-check run
            var newValue;
            // a shallow copy of the newValue from the last dirty-check run,
            // updated to match newValue during dirty-check run
            var oldValue;
            // a shallow copy of the newValue from when the last change happened
            var veryOldValue;
            // only track veryOldValue if the listener is asking for it
            var trackVeryOldValue = listener.length > 1;
            var changeDetected = 0;
            var changeDetector = $parse(obj, $watchCollectionInterceptor);
            var internalArray = [];
            var internalObject = {};
            var initRun = true;
            var oldLength = 0;

            function $watchCollectionInterceptor(_value) {
              newValue = _value;
              var newLength, key, bothNaN, newItem, oldItem;

              // If the new value is undefined, then return undefined as the watch may be a one-time watch
              if (isUndefined(newValue)) return;

              if (!isObject(newValue)) {
                // if primitive
                if (oldValue !== newValue) {
                  oldValue = newValue;
                  changeDetected++;
                }
              } else if (isArrayLike(newValue)) {
                if (oldValue !== internalArray) {
                  // we are transitioning from something which was not an array into array.
                  oldValue = internalArray;
                  oldLength = oldValue.length = 0;
                  changeDetected++;
                }

                newLength = newValue.length;

                if (oldLength !== newLength) {
                  // if lengths do not match we need to trigger change notification
                  changeDetected++;
                  oldValue.length = oldLength = newLength;
                }
                // copy the items to oldValue and look for changes.
                for (var i = 0; i < newLength; i++) {
                  oldItem = oldValue[i];
                  newItem = newValue[i];

                  // eslint-disable-next-line no-self-compare
                  bothNaN = oldItem !== oldItem && newItem !== newItem;
                  if (!bothNaN && oldItem !== newItem) {
                    changeDetected++;
                    oldValue[i] = newItem;
                  }
                }
              } else {
                if (oldValue !== internalObject) {
                  // we are transitioning from something which was not an object into object.
                  oldValue = internalObject = {};
                  oldLength = 0;
                  changeDetected++;
                }
                // copy the items to oldValue and look for changes.
                newLength = 0;
                for (key in newValue) {
                  if (hasOwnProperty.call(newValue, key)) {
                    newLength++;
                    newItem = newValue[key];
                    oldItem = oldValue[key];

                    if (key in oldValue) {
                      // eslint-disable-next-line no-self-compare
                      bothNaN = oldItem !== oldItem && newItem !== newItem;
                      if (!bothNaN && oldItem !== newItem) {
                        changeDetected++;
                        oldValue[key] = newItem;
                      }
                    } else {
                      oldLength++;
                      oldValue[key] = newItem;
                      changeDetected++;
                    }
                  }
                }
                if (oldLength > newLength) {
                  // we used to have more keys, need to find them and destroy them.
                  changeDetected++;
                  for (key in oldValue) {
                    if (!hasOwnProperty.call(newValue, key)) {
                      oldLength--;
                      delete oldValue[key];
                    }
                  }
                }
              }
              return changeDetected;
            }

            function $watchCollectionAction() {
              if (initRun) {
                initRun = false;
                listener(newValue, newValue, self);
              } else {
                listener(newValue, veryOldValue, self);
              }

              // make a copy for the next time a collection is changed
              if (trackVeryOldValue) {
                if (!isObject(newValue)) {
                  //primitive
                  veryOldValue = newValue;
                } else if (isArrayLike(newValue)) {
                  veryOldValue = new Array(newValue.length);
                  for (var i = 0; i < newValue.length; i++) {
                    veryOldValue[i] = newValue[i];
                  }
                } else {
                  // if object
                  veryOldValue = {};
                  for (var key in newValue) {
                    if (hasOwnProperty.call(newValue, key)) {
                      veryOldValue[key] = newValue[key];
                    }
                  }
                }
              }
            }

            return this.$watch(changeDetector, $watchCollectionAction);
          },

          /**
           * @ngdoc method
           * @name $rootScope.Scope#$digest
           * @kind function
           *
           * @description
           * Processes all of the {@link ng.$rootScope.Scope#$watch watchers} of the current scope and
           * its children. Because a {@link ng.$rootScope.Scope#$watch watcher}'s listener can change
           * the model, the `$digest()` keeps calling the {@link ng.$rootScope.Scope#$watch watchers}
           * until no more listeners are firing. This means that it is possible to get into an infinite
           * loop. This function will throw `'Maximum iteration limit exceeded.'` if the number of
           * iterations exceeds 10.
           *
           * Usually, you don't call `$digest()` directly in
           * {@link ng.directive:ngController controllers} or in
           * {@link ng.$compileProvider#directive directives}.
           * Instead, you should call {@link ng.$rootScope.Scope#$apply $apply()} (typically from within
           * a {@link ng.$compileProvider#directive directive}), which will force a `$digest()`.
           *
           * If you want to be notified whenever `$digest()` is called,
           * you can register a `watchExpression` function with
           * {@link ng.$rootScope.Scope#$watch $watch()} with no `listener`.
           *
           * In unit tests, you may need to call `$digest()` to simulate the scope life cycle.
           *
           * # Example
           * ```js
               var scope = ...;
               scope.name = 'misko';
               scope.counter = 0;
                expect(scope.counter).toEqual(0);
               scope.$watch('name', function(newValue, oldValue) {
                 scope.counter = scope.counter + 1;
               });
               expect(scope.counter).toEqual(0);
                scope.$digest();
               // the listener is always called during the first $digest loop after it was registered
               expect(scope.counter).toEqual(1);
                scope.$digest();
               // but now it will not be called unless the value changes
               expect(scope.counter).toEqual(1);
                scope.name = 'adam';
               scope.$digest();
               expect(scope.counter).toEqual(2);
           * ```
           *
           */
          $digest: function () {
            var watch,
                value,
                last,
                fn,
                get,
                watchers,
                dirty,
                ttl = TTL,
                next,
                current,
                target = this,
                watchLog = [],
                logIdx,
                asyncTask;

            beginPhase('$digest');
            // Check for changes to browser url that happened in sync before the call to $digest
            $browser.$$checkUrlChange();

            if (this === $rootScope && applyAsyncId !== null) {
              // If this is the root scope, and $applyAsync has scheduled a deferred $apply(), then
              // cancel the scheduled $apply and flush the queue of expressions to be evaluated.
              $browser.defer.cancel(applyAsyncId);
              flushApplyAsync();
            }

            lastDirtyWatch = null;

            do {
              // "while dirty" loop
              dirty = false;
              current = target;

              // It's safe for asyncQueuePosition to be a local variable here because this loop can't
              // be reentered recursively. Calling $digest from a function passed to $evalAsync would
              // lead to a '$digest already in progress' error.
              for (var asyncQueuePosition = 0; asyncQueuePosition < asyncQueue.length; asyncQueuePosition++) {
                try {
                  asyncTask = asyncQueue[asyncQueuePosition];
                  fn = asyncTask.fn;
                  fn(asyncTask.scope, asyncTask.locals);
                } catch (e) {
                  $exceptionHandler(e);
                }
                lastDirtyWatch = null;
              }
              asyncQueue.length = 0;

              traverseScopesLoop: do {
                // "traverse the scopes" loop
                if (watchers = current.$$watchers) {
                  // process our watches
                  watchers.$$digestWatchIndex = watchers.length;
                  while (watchers.$$digestWatchIndex--) {
                    try {
                      watch = watchers[watchers.$$digestWatchIndex];
                      // Most common watches are on primitives, in which case we can short
                      // circuit it with === operator, only when === fails do we use .equals
                      if (watch) {
                        get = watch.get;
                        if ((value = get(current)) !== (last = watch.last) && !(watch.eq ? equals(value, last) : isNumberNaN(value) && isNumberNaN(last))) {
                          dirty = true;
                          lastDirtyWatch = watch;
                          watch.last = watch.eq ? copy(value, null) : value;
                          fn = watch.fn;
                          fn(value, last === initWatchVal ? value : last, current);
                          if (ttl < 5) {
                            logIdx = 4 - ttl;
                            if (!watchLog[logIdx]) watchLog[logIdx] = [];
                            watchLog[logIdx].push({
                              msg: isFunction(watch.exp) ? 'fn: ' + (watch.exp.name || watch.exp.toString()) : watch.exp,
                              newVal: value,
                              oldVal: last
                            });
                          }
                        } else if (watch === lastDirtyWatch) {
                          // If the most recently dirty watcher is now clean, short circuit since the remaining watchers
                          // have already been tested.
                          dirty = false;
                          break traverseScopesLoop;
                        }
                      }
                    } catch (e) {
                      $exceptionHandler(e);
                    }
                  }
                }

                // Insanity Warning: scope depth-first traversal
                // yes, this code is a bit crazy, but it works and we have tests to prove it!
                // this piece should be kept in sync with the traversal in $broadcast
                if (!(next = current.$$watchersCount && current.$$childHead || current !== target && current.$$nextSibling)) {
                  while (current !== target && !(next = current.$$nextSibling)) {
                    current = current.$parent;
                  }
                }
              } while (current = next);

              // `break traverseScopesLoop;` takes us to here

              if ((dirty || asyncQueue.length) && !ttl--) {
                clearPhase();
                throw $rootScopeMinErr('infdig', '{0} $digest() iterations reached. Aborting!\n' + 'Watchers fired in the last 5 iterations: {1}', TTL, watchLog);
              }
            } while (dirty || asyncQueue.length);

            clearPhase();

            // postDigestQueuePosition isn't local here because this loop can be reentered recursively.
            while (postDigestQueuePosition < postDigestQueue.length) {
              try {
                postDigestQueue[postDigestQueuePosition++]();
              } catch (e) {
                $exceptionHandler(e);
              }
            }
            postDigestQueue.length = postDigestQueuePosition = 0;

            // Check for changes to browser url that happened during the $digest
            // (for which no event is fired; e.g. via `history.pushState()`)
            $browser.$$checkUrlChange();
          },

          /**
           * @ngdoc event
           * @name $rootScope.Scope#$destroy
           * @eventType broadcast on scope being destroyed
           *
           * @description
           * Broadcasted when a scope and its children are being destroyed.
           *
           * Note that, in AngularJS, there is also a `$destroy` jQuery event, which can be used to
           * clean up DOM bindings before an element is removed from the DOM.
           */

          /**
           * @ngdoc method
           * @name $rootScope.Scope#$destroy
           * @kind function
           *
           * @description
           * Removes the current scope (and all of its children) from the parent scope. Removal implies
           * that calls to {@link ng.$rootScope.Scope#$digest $digest()} will no longer
           * propagate to the current scope and its children. Removal also implies that the current
           * scope is eligible for garbage collection.
           *
           * The `$destroy()` is usually used by directives such as
           * {@link ng.directive:ngRepeat ngRepeat} for managing the
           * unrolling of the loop.
           *
           * Just before a scope is destroyed, a `$destroy` event is broadcasted on this scope.
           * Application code can register a `$destroy` event handler that will give it a chance to
           * perform any necessary cleanup.
           *
           * Note that, in AngularJS, there is also a `$destroy` jQuery event, which can be used to
           * clean up DOM bindings before an element is removed from the DOM.
           */
          $destroy: function () {
            // We can't destroy a scope that has been already destroyed.
            if (this.$$destroyed) return;
            var parent = this.$parent;

            this.$broadcast('$destroy');
            this.$$destroyed = true;

            if (this === $rootScope) {
              //Remove handlers attached to window when $rootScope is removed
              $browser.$$applicationDestroyed();
            }

            incrementWatchersCount(this, -this.$$watchersCount);
            for (var eventName in this.$$listenerCount) {
              decrementListenerCount(this, this.$$listenerCount[eventName], eventName);
            }

            // sever all the references to parent scopes (after this cleanup, the current scope should
            // not be retained by any of our references and should be eligible for garbage collection)
            if (parent && parent.$$childHead === this) parent.$$childHead = this.$$nextSibling;
            if (parent && parent.$$childTail === this) parent.$$childTail = this.$$prevSibling;
            if (this.$$prevSibling) this.$$prevSibling.$$nextSibling = this.$$nextSibling;
            if (this.$$nextSibling) this.$$nextSibling.$$prevSibling = this.$$prevSibling;

            // Disable listeners, watchers and apply/digest methods
            this.$destroy = this.$digest = this.$apply = this.$evalAsync = this.$applyAsync = noop;
            this.$on = this.$watch = this.$watchGroup = function () {
              return noop;
            };
            this.$$listeners = {};

            // Disconnect the next sibling to prevent `cleanUpScope` destroying those too
            this.$$nextSibling = null;
            cleanUpScope(this);
          },

          /**
           * @ngdoc method
           * @name $rootScope.Scope#$eval
           * @kind function
           *
           * @description
           * Executes the `expression` on the current scope and returns the result. Any exceptions in
           * the expression are propagated (uncaught). This is useful when evaluating Angular
           * expressions.
           *
           * # Example
           * ```js
               var scope = ng.$rootScope.Scope();
               scope.a = 1;
               scope.b = 2;
                expect(scope.$eval('a+b')).toEqual(3);
               expect(scope.$eval(function(scope){ return scope.a + scope.b; })).toEqual(3);
           * ```
           *
           * @param {(string|function())=} expression An angular expression to be executed.
           *
           *    - `string`: execute using the rules as defined in  {@link guide/expression expression}.
           *    - `function(scope)`: execute the function with the current `scope` parameter.
           *
           * @param {(object)=} locals Local variables object, useful for overriding values in scope.
           * @returns {*} The result of evaluating the expression.
           */
          $eval: function (expr, locals) {
            return $parse(expr)(this, locals);
          },

          /**
           * @ngdoc method
           * @name $rootScope.Scope#$evalAsync
           * @kind function
           *
           * @description
           * Executes the expression on the current scope at a later point in time.
           *
           * The `$evalAsync` makes no guarantees as to when the `expression` will be executed, only
           * that:
           *
           *   - it will execute after the function that scheduled the evaluation (preferably before DOM
           *     rendering).
           *   - at least one {@link ng.$rootScope.Scope#$digest $digest cycle} will be performed after
           *     `expression` execution.
           *
           * Any exceptions from the execution of the expression are forwarded to the
           * {@link ng.$exceptionHandler $exceptionHandler} service.
           *
           * __Note:__ if this function is called outside of a `$digest` cycle, a new `$digest` cycle
           * will be scheduled. However, it is encouraged to always call code that changes the model
           * from within an `$apply` call. That includes code evaluated via `$evalAsync`.
           *
           * @param {(string|function())=} expression An angular expression to be executed.
           *
           *    - `string`: execute using the rules as defined in {@link guide/expression expression}.
           *    - `function(scope)`: execute the function with the current `scope` parameter.
           *
           * @param {(object)=} locals Local variables object, useful for overriding values in scope.
           */
          $evalAsync: function (expr, locals) {
            // if we are outside of an $digest loop and this is the first time we are scheduling async
            // task also schedule async auto-flush
            if (!$rootScope.$$phase && !asyncQueue.length) {
              $browser.defer(function () {
                if (asyncQueue.length) {
                  $rootScope.$digest();
                }
              });
            }

            asyncQueue.push({ scope: this, fn: $parse(expr), locals: locals });
          },

          $$postDigest: function (fn) {
            postDigestQueue.push(fn);
          },

          /**
           * @ngdoc method
           * @name $rootScope.Scope#$apply
           * @kind function
           *
           * @description
           * `$apply()` is used to execute an expression in angular from outside of the angular
           * framework. (For example from browser DOM events, setTimeout, XHR or third party libraries).
           * Because we are calling into the angular framework we need to perform proper scope life
           * cycle of {@link ng.$exceptionHandler exception handling},
           * {@link ng.$rootScope.Scope#$digest executing watches}.
           *
           * ## Life cycle
           *
           * # Pseudo-Code of `$apply()`
           * ```js
               function $apply(expr) {
                 try {
                   return $eval(expr);
                 } catch (e) {
                   $exceptionHandler(e);
                 } finally {
                   $root.$digest();
                 }
               }
           * ```
           *
           *
           * Scope's `$apply()` method transitions through the following stages:
           *
           * 1. The {@link guide/expression expression} is executed using the
           *    {@link ng.$rootScope.Scope#$eval $eval()} method.
           * 2. Any exceptions from the execution of the expression are forwarded to the
           *    {@link ng.$exceptionHandler $exceptionHandler} service.
           * 3. The {@link ng.$rootScope.Scope#$watch watch} listeners are fired immediately after the
           *    expression was executed using the {@link ng.$rootScope.Scope#$digest $digest()} method.
           *
           *
           * @param {(string|function())=} exp An angular expression to be executed.
           *
           *    - `string`: execute using the rules as defined in {@link guide/expression expression}.
           *    - `function(scope)`: execute the function with current `scope` parameter.
           *
           * @returns {*} The result of evaluating the expression.
           */
          $apply: function (expr) {
            try {
              beginPhase('$apply');
              try {
                return this.$eval(expr);
              } finally {
                clearPhase();
              }
            } catch (e) {
              $exceptionHandler(e);
            } finally {
              try {
                $rootScope.$digest();
              } catch (e) {
                $exceptionHandler(e);
                // eslint-disable-next-line no-unsafe-finally
                throw e;
              }
            }
          },

          /**
           * @ngdoc method
           * @name $rootScope.Scope#$applyAsync
           * @kind function
           *
           * @description
           * Schedule the invocation of $apply to occur at a later time. The actual time difference
           * varies across browsers, but is typically around ~10 milliseconds.
           *
           * This can be used to queue up multiple expressions which need to be evaluated in the same
           * digest.
           *
           * @param {(string|function())=} exp An angular expression to be executed.
           *
           *    - `string`: execute using the rules as defined in {@link guide/expression expression}.
           *    - `function(scope)`: execute the function with current `scope` parameter.
           */
          $applyAsync: function (expr) {
            var scope = this;
            if (expr) {
              applyAsyncQueue.push($applyAsyncExpression);
            }
            expr = $parse(expr);
            scheduleApplyAsync();

            function $applyAsyncExpression() {
              scope.$eval(expr);
            }
          },

          /**
           * @ngdoc method
           * @name $rootScope.Scope#$on
           * @kind function
           *
           * @description
           * Listens on events of a given type. See {@link ng.$rootScope.Scope#$emit $emit} for
           * discussion of event life cycle.
           *
           * The event listener function format is: `function(event, args...)`. The `event` object
           * passed into the listener has the following attributes:
           *
           *   - `targetScope` - `{Scope}`: the scope on which the event was `$emit`-ed or
           *     `$broadcast`-ed.
           *   - `currentScope` - `{Scope}`: the scope that is currently handling the event. Once the
           *     event propagates through the scope hierarchy, this property is set to null.
           *   - `name` - `{string}`: name of the event.
           *   - `stopPropagation` - `{function=}`: calling `stopPropagation` function will cancel
           *     further event propagation (available only for events that were `$emit`-ed).
           *   - `preventDefault` - `{function}`: calling `preventDefault` sets `defaultPrevented` flag
           *     to true.
           *   - `defaultPrevented` - `{boolean}`: true if `preventDefault` was called.
           *
           * @param {string} name Event name to listen on.
           * @param {function(event, ...args)} listener Function to call when the event is emitted.
           * @returns {function()} Returns a deregistration function for this listener.
           */
          $on: function (name, listener) {
            var namedListeners = this.$$listeners[name];
            if (!namedListeners) {
              this.$$listeners[name] = namedListeners = [];
            }
            namedListeners.push(listener);

            var current = this;
            do {
              if (!current.$$listenerCount[name]) {
                current.$$listenerCount[name] = 0;
              }
              current.$$listenerCount[name]++;
            } while (current = current.$parent);

            var self = this;
            return function () {
              var indexOfListener = namedListeners.indexOf(listener);
              if (indexOfListener !== -1) {
                namedListeners[indexOfListener] = null;
                decrementListenerCount(self, 1, name);
              }
            };
          },

          /**
           * @ngdoc method
           * @name $rootScope.Scope#$emit
           * @kind function
           *
           * @description
           * Dispatches an event `name` upwards through the scope hierarchy notifying the
           * registered {@link ng.$rootScope.Scope#$on} listeners.
           *
           * The event life cycle starts at the scope on which `$emit` was called. All
           * {@link ng.$rootScope.Scope#$on listeners} listening for `name` event on this scope get
           * notified. Afterwards, the event traverses upwards toward the root scope and calls all
           * registered listeners along the way. The event will stop propagating if one of the listeners
           * cancels it.
           *
           * Any exception emitted from the {@link ng.$rootScope.Scope#$on listeners} will be passed
           * onto the {@link ng.$exceptionHandler $exceptionHandler} service.
           *
           * @param {string} name Event name to emit.
           * @param {...*} args Optional one or more arguments which will be passed onto the event listeners.
           * @return {Object} Event object (see {@link ng.$rootScope.Scope#$on}).
           */
          $emit: function (name, args) {
            var empty = [],
                namedListeners,
                scope = this,
                stopPropagation = false,
                event = {
              name: name,
              targetScope: scope,
              stopPropagation: function () {
                stopPropagation = true;
              },
              preventDefault: function () {
                event.defaultPrevented = true;
              },
              defaultPrevented: false
            },
                listenerArgs = concat([event], arguments, 1),
                i,
                length;

            do {
              namedListeners = scope.$$listeners[name] || empty;
              event.currentScope = scope;
              for (i = 0, length = namedListeners.length; i < length; i++) {

                // if listeners were deregistered, defragment the array
                if (!namedListeners[i]) {
                  namedListeners.splice(i, 1);
                  i--;
                  length--;
                  continue;
                }
                try {
                  //allow all listeners attached to the current scope to run
                  namedListeners[i].apply(null, listenerArgs);
                } catch (e) {
                  $exceptionHandler(e);
                }
              }
              //if any listener on the current scope stops propagation, prevent bubbling
              if (stopPropagation) {
                event.currentScope = null;
                return event;
              }
              //traverse upwards
              scope = scope.$parent;
            } while (scope);

            event.currentScope = null;

            return event;
          },

          /**
           * @ngdoc method
           * @name $rootScope.Scope#$broadcast
           * @kind function
           *
           * @description
           * Dispatches an event `name` downwards to all child scopes (and their children) notifying the
           * registered {@link ng.$rootScope.Scope#$on} listeners.
           *
           * The event life cycle starts at the scope on which `$broadcast` was called. All
           * {@link ng.$rootScope.Scope#$on listeners} listening for `name` event on this scope get
           * notified. Afterwards, the event propagates to all direct and indirect scopes of the current
           * scope and calls all registered listeners along the way. The event cannot be canceled.
           *
           * Any exception emitted from the {@link ng.$rootScope.Scope#$on listeners} will be passed
           * onto the {@link ng.$exceptionHandler $exceptionHandler} service.
           *
           * @param {string} name Event name to broadcast.
           * @param {...*} args Optional one or more arguments which will be passed onto the event listeners.
           * @return {Object} Event object, see {@link ng.$rootScope.Scope#$on}
           */
          $broadcast: function (name, args) {
            var target = this,
                current = target,
                next = target,
                event = {
              name: name,
              targetScope: target,
              preventDefault: function () {
                event.defaultPrevented = true;
              },
              defaultPrevented: false
            };

            if (!target.$$listenerCount[name]) return event;

            var listenerArgs = concat([event], arguments, 1),
                listeners,
                i,
                length;

            //down while you can, then up and next sibling or up and next sibling until back at root
            while (current = next) {
              event.currentScope = current;
              listeners = current.$$listeners[name] || [];
              for (i = 0, length = listeners.length; i < length; i++) {
                // if listeners were deregistered, defragment the array
                if (!listeners[i]) {
                  listeners.splice(i, 1);
                  i--;
                  length--;
                  continue;
                }

                try {
                  listeners[i].apply(null, listenerArgs);
                } catch (e) {
                  $exceptionHandler(e);
                }
              }

              // Insanity Warning: scope depth-first traversal
              // yes, this code is a bit crazy, but it works and we have tests to prove it!
              // this piece should be kept in sync with the traversal in $digest
              // (though it differs due to having the extra check for $$listenerCount)
              if (!(next = current.$$listenerCount[name] && current.$$childHead || current !== target && current.$$nextSibling)) {
                while (current !== target && !(next = current.$$nextSibling)) {
                  current = current.$parent;
                }
              }
            }

            event.currentScope = null;
            return event;
          }
        };

        var $rootScope = new Scope();

        //The internal queues. Expose them on the $rootScope for debugging/testing purposes.
        var asyncQueue = $rootScope.$$asyncQueue = [];
        var postDigestQueue = $rootScope.$$postDigestQueue = [];
        var applyAsyncQueue = $rootScope.$$applyAsyncQueue = [];

        var postDigestQueuePosition = 0;

        return $rootScope;

        function beginPhase(phase) {
          if ($rootScope.$$phase) {
            throw $rootScopeMinErr('inprog', '{0} already in progress', $rootScope.$$phase);
          }

          $rootScope.$$phase = phase;
        }

        function clearPhase() {
          $rootScope.$$phase = null;
        }

        function incrementWatchersCount(current, count) {
          do {
            current.$$watchersCount += count;
          } while (current = current.$parent);
        }

        function decrementListenerCount(current, count, name) {
          do {
            current.$$listenerCount[name] -= count;

            if (current.$$listenerCount[name] === 0) {
              delete current.$$listenerCount[name];
            }
          } while (current = current.$parent);
        }

        /**
         * function used as an initial value for watchers.
         * because it's unique we can easily tell it apart from other values
         */
        function initWatchVal() {}

        function flushApplyAsync() {
          while (applyAsyncQueue.length) {
            try {
              applyAsyncQueue.shift()();
            } catch (e) {
              $exceptionHandler(e);
            }
          }
          applyAsyncId = null;
        }

        function scheduleApplyAsync() {
          if (applyAsyncId === null) {
            applyAsyncId = $browser.defer(function () {
              $rootScope.$apply(flushApplyAsync);
            });
          }
        }
      }];
    }

    /**
     * @ngdoc service
     * @name $rootElement
     *
     * @description
     * The root element of Angular application. This is either the element where {@link
     * ng.directive:ngApp ngApp} was declared or the element passed into
     * {@link angular.bootstrap}. The element represents the root element of application. It is also the
     * location where the application's {@link auto.$injector $injector} service gets
     * published, and can be retrieved using `$rootElement.injector()`.
     */

    // the implementation is in angular.bootstrap

    /**
     * @this
     * @description
     * Private service to sanitize uris for links and images. Used by $compile and $sanitize.
     */
    function $$SanitizeUriProvider() {
      var aHrefSanitizationWhitelist = /^\s*(https?|ftp|mailto|tel|file):/,
          imgSrcSanitizationWhitelist = /^\s*((https?|ftp|file|blob):|data:image\/)/;

      /**
       * @description
       * Retrieves or overrides the default regular expression that is used for whitelisting of safe
       * urls during a[href] sanitization.
       *
       * The sanitization is a security measure aimed at prevent XSS attacks via html links.
       *
       * Any url about to be assigned to a[href] via data-binding is first normalized and turned into
       * an absolute url. Afterwards, the url is matched against the `aHrefSanitizationWhitelist`
       * regular expression. If a match is found, the original url is written into the dom. Otherwise,
       * the absolute url is prefixed with `'unsafe:'` string and only then is it written into the DOM.
       *
       * @param {RegExp=} regexp New regexp to whitelist urls with.
       * @returns {RegExp|ng.$compileProvider} Current RegExp if called without value or self for
       *    chaining otherwise.
       */
      this.aHrefSanitizationWhitelist = function (regexp) {
        if (isDefined(regexp)) {
          aHrefSanitizationWhitelist = regexp;
          return this;
        }
        return aHrefSanitizationWhitelist;
      };

      /**
       * @description
       * Retrieves or overrides the default regular expression that is used for whitelisting of safe
       * urls during img[src] sanitization.
       *
       * The sanitization is a security measure aimed at prevent XSS attacks via html links.
       *
       * Any url about to be assigned to img[src] via data-binding is first normalized and turned into
       * an absolute url. Afterwards, the url is matched against the `imgSrcSanitizationWhitelist`
       * regular expression. If a match is found, the original url is written into the dom. Otherwise,
       * the absolute url is prefixed with `'unsafe:'` string and only then is it written into the DOM.
       *
       * @param {RegExp=} regexp New regexp to whitelist urls with.
       * @returns {RegExp|ng.$compileProvider} Current RegExp if called without value or self for
       *    chaining otherwise.
       */
      this.imgSrcSanitizationWhitelist = function (regexp) {
        if (isDefined(regexp)) {
          imgSrcSanitizationWhitelist = regexp;
          return this;
        }
        return imgSrcSanitizationWhitelist;
      };

      this.$get = function () {
        return function sanitizeUri(uri, isImage) {
          var regex = isImage ? imgSrcSanitizationWhitelist : aHrefSanitizationWhitelist;
          var normalizedVal;
          normalizedVal = urlResolve(uri).href;
          if (normalizedVal !== '' && !normalizedVal.match(regex)) {
            return 'unsafe:' + normalizedVal;
          }
          return uri;
        };
      };
    }

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
     *     Any commits to this file should be reviewed with security in mind.  *
     *   Changes to this file can potentially create security vulnerabilities. *
     *          An approval from 2 Core members with history of modifying      *
     *                         this file is required.                          *
     *                                                                         *
     *  Does the change somehow allow for arbitrary javascript to be executed? *
     *    Or allows for someone to change the prototype of built-in objects?   *
     *     Or gives undesired access to variables likes document or window?    *
     * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

    /* exported $SceProvider, $SceDelegateProvider */

    var $sceMinErr = minErr('$sce');

    var SCE_CONTEXTS = {
      HTML: 'html',
      CSS: 'css',
      URL: 'url',
      // RESOURCE_URL is a subtype of URL used in contexts where a privileged resource is sourced from a
      // url.  (e.g. ng-include, script src, templateUrl)
      RESOURCE_URL: 'resourceUrl',
      JS: 'js'
    };

    // Helper functions follow.

    var UNDERSCORE_LOWERCASE_REGEXP = /_([a-z])/g;

    function snakeToCamel(name) {
      return name.replace(UNDERSCORE_LOWERCASE_REGEXP, fnCamelCaseReplace);
    }

    function adjustMatcher(matcher) {
      if (matcher === 'self') {
        return matcher;
      } else if (isString(matcher)) {
        // Strings match exactly except for 2 wildcards - '*' and '**'.
        // '*' matches any character except those from the set ':/.?&'.
        // '**' matches any character (like .* in a RegExp).
        // More than 2 *'s raises an error as it's ill defined.
        if (matcher.indexOf('***') > -1) {
          throw $sceMinErr('iwcard', 'Illegal sequence *** in string matcher.  String: {0}', matcher);
        }
        matcher = escapeForRegexp(matcher).replace(/\\\*\\\*/g, '.*').replace(/\\\*/g, '[^:/.?&;]*');
        return new RegExp('^' + matcher + '$');
      } else if (isRegExp(matcher)) {
        // The only other type of matcher allowed is a Regexp.
        // Match entire URL / disallow partial matches.
        // Flags are reset (i.e. no global, ignoreCase or multiline)
        return new RegExp('^' + matcher.source + '$');
      } else {
        throw $sceMinErr('imatcher', 'Matchers may only be "self", string patterns or RegExp objects');
      }
    }

    function adjustMatchers(matchers) {
      var adjustedMatchers = [];
      if (isDefined(matchers)) {
        forEach(matchers, function (matcher) {
          adjustedMatchers.push(adjustMatcher(matcher));
        });
      }
      return adjustedMatchers;
    }

    /**
     * @ngdoc service
     * @name $sceDelegate
     * @kind function
     *
     * @description
     *
     * `$sceDelegate` is a service that is used by the `$sce` service to provide {@link ng.$sce Strict
     * Contextual Escaping (SCE)} services to AngularJS.
     *
     * Typically, you would configure or override the {@link ng.$sceDelegate $sceDelegate} instead of
     * the `$sce` service to customize the way Strict Contextual Escaping works in AngularJS.  This is
     * because, while the `$sce` provides numerous shorthand methods, etc., you really only need to
     * override 3 core functions (`trustAs`, `getTrusted` and `valueOf`) to replace the way things
     * work because `$sce` delegates to `$sceDelegate` for these operations.
     *
     * Refer {@link ng.$sceDelegateProvider $sceDelegateProvider} to configure this service.
     *
     * The default instance of `$sceDelegate` should work out of the box with little pain.  While you
     * can override it completely to change the behavior of `$sce`, the common case would
     * involve configuring the {@link ng.$sceDelegateProvider $sceDelegateProvider} instead by setting
     * your own whitelists and blacklists for trusting URLs used for loading AngularJS resources such as
     * templates.  Refer {@link ng.$sceDelegateProvider#resourceUrlWhitelist
     * $sceDelegateProvider.resourceUrlWhitelist} and {@link
     * ng.$sceDelegateProvider#resourceUrlBlacklist $sceDelegateProvider.resourceUrlBlacklist}
     */

    /**
     * @ngdoc provider
     * @name $sceDelegateProvider
     * @this
     *
     * @description
     *
     * The `$sceDelegateProvider` provider allows developers to configure the {@link ng.$sceDelegate
     * $sceDelegate} service.  This allows one to get/set the whitelists and blacklists used to ensure
     * that the URLs used for sourcing Angular templates are safe.  Refer {@link
     * ng.$sceDelegateProvider#resourceUrlWhitelist $sceDelegateProvider.resourceUrlWhitelist} and
     * {@link ng.$sceDelegateProvider#resourceUrlBlacklist $sceDelegateProvider.resourceUrlBlacklist}
     *
     * For the general details about this service in Angular, read the main page for {@link ng.$sce
     * Strict Contextual Escaping (SCE)}.
     *
     * **Example**:  Consider the following case. <a name="example"></a>
     *
     * - your app is hosted at url `http://myapp.example.com/`
     * - but some of your templates are hosted on other domains you control such as
     *   `http://srv01.assets.example.com/`, `http://srv02.assets.example.com/`, etc.
     * - and you have an open redirect at `http://myapp.example.com/clickThru?...`.
     *
     * Here is what a secure configuration for this scenario might look like:
     *
     * ```
     *  angular.module('myApp', []).config(function($sceDelegateProvider) {
     *    $sceDelegateProvider.resourceUrlWhitelist([
     *      // Allow same origin resource loads.
     *      'self',
     *      // Allow loading from our assets domain.  Notice the difference between * and **.
     *      'http://srv*.assets.example.com/**'
     *    ]);
     *
     *    // The blacklist overrides the whitelist so the open redirect here is blocked.
     *    $sceDelegateProvider.resourceUrlBlacklist([
     *      'http://myapp.example.com/clickThru**'
     *    ]);
     *  });
     * ```
     */

    function $SceDelegateProvider() {
      this.SCE_CONTEXTS = SCE_CONTEXTS;

      // Resource URLs can also be trusted by policy.
      var resourceUrlWhitelist = ['self'],
          resourceUrlBlacklist = [];

      /**
       * @ngdoc method
       * @name $sceDelegateProvider#resourceUrlWhitelist
       * @kind function
       *
       * @param {Array=} whitelist When provided, replaces the resourceUrlWhitelist with the value
       *    provided.  This must be an array or null.  A snapshot of this array is used so further
       *    changes to the array are ignored.
       *
       *    Follow {@link ng.$sce#resourceUrlPatternItem this link} for a description of the items
       *    allowed in this array.
       *
       *    <div class="alert alert-warning">
       *    **Note:** an empty whitelist array will block all URLs!
       *    </div>
       *
       * @return {Array} the currently set whitelist array.
       *
       * The **default value** when no whitelist has been explicitly set is `['self']` allowing only
       * same origin resource requests.
       *
       * @description
       * Sets/Gets the whitelist of trusted resource URLs.
       */
      this.resourceUrlWhitelist = function (value) {
        if (arguments.length) {
          resourceUrlWhitelist = adjustMatchers(value);
        }
        return resourceUrlWhitelist;
      };

      /**
       * @ngdoc method
       * @name $sceDelegateProvider#resourceUrlBlacklist
       * @kind function
       *
       * @param {Array=} blacklist When provided, replaces the resourceUrlBlacklist with the value
       *    provided.  This must be an array or null.  A snapshot of this array is used so further
       *    changes to the array are ignored.
       *
       *    Follow {@link ng.$sce#resourceUrlPatternItem this link} for a description of the items
       *    allowed in this array.
       *
       *    The typical usage for the blacklist is to **block
       *    [open redirects](http://cwe.mitre.org/data/definitions/601.html)** served by your domain as
       *    these would otherwise be trusted but actually return content from the redirected domain.
       *
       *    Finally, **the blacklist overrides the whitelist** and has the final say.
       *
       * @return {Array} the currently set blacklist array.
       *
       * The **default value** when no whitelist has been explicitly set is the empty array (i.e. there
       * is no blacklist.)
       *
       * @description
       * Sets/Gets the blacklist of trusted resource URLs.
       */

      this.resourceUrlBlacklist = function (value) {
        if (arguments.length) {
          resourceUrlBlacklist = adjustMatchers(value);
        }
        return resourceUrlBlacklist;
      };

      this.$get = ['$injector', function ($injector) {

        var htmlSanitizer = function htmlSanitizer(html) {
          throw $sceMinErr('unsafe', 'Attempting to use an unsafe value in a safe context.');
        };

        if ($injector.has('$sanitize')) {
          htmlSanitizer = $injector.get('$sanitize');
        }

        function matchUrl(matcher, parsedUrl) {
          if (matcher === 'self') {
            return urlIsSameOrigin(parsedUrl);
          } else {
            // definitely a regex.  See adjustMatchers()
            return !!matcher.exec(parsedUrl.href);
          }
        }

        function isResourceUrlAllowedByPolicy(url) {
          var parsedUrl = urlResolve(url.toString());
          var i,
              n,
              allowed = false;
          // Ensure that at least one item from the whitelist allows this url.
          for (i = 0, n = resourceUrlWhitelist.length; i < n; i++) {
            if (matchUrl(resourceUrlWhitelist[i], parsedUrl)) {
              allowed = true;
              break;
            }
          }
          if (allowed) {
            // Ensure that no item from the blacklist blocked this url.
            for (i = 0, n = resourceUrlBlacklist.length; i < n; i++) {
              if (matchUrl(resourceUrlBlacklist[i], parsedUrl)) {
                allowed = false;
                break;
              }
            }
          }
          return allowed;
        }

        function generateHolderType(Base) {
          var holderType = function TrustedValueHolderType(trustedValue) {
            this.$$unwrapTrustedValue = function () {
              return trustedValue;
            };
          };
          if (Base) {
            holderType.prototype = new Base();
          }
          holderType.prototype.valueOf = function sceValueOf() {
            return this.$$unwrapTrustedValue();
          };
          holderType.prototype.toString = function sceToString() {
            return this.$$unwrapTrustedValue().toString();
          };
          return holderType;
        }

        var trustedValueHolderBase = generateHolderType(),
            byType = {};

        byType[SCE_CONTEXTS.HTML] = generateHolderType(trustedValueHolderBase);
        byType[SCE_CONTEXTS.CSS] = generateHolderType(trustedValueHolderBase);
        byType[SCE_CONTEXTS.URL] = generateHolderType(trustedValueHolderBase);
        byType[SCE_CONTEXTS.JS] = generateHolderType(trustedValueHolderBase);
        byType[SCE_CONTEXTS.RESOURCE_URL] = generateHolderType(byType[SCE_CONTEXTS.URL]);

        /**
         * @ngdoc method
         * @name $sceDelegate#trustAs
         *
         * @description
         * Returns an object that is trusted by angular for use in specified strict
         * contextual escaping contexts (such as ng-bind-html, ng-include, any src
         * attribute interpolation, any dom event binding attribute interpolation
         * such as for onclick,  etc.) that uses the provided value.
         * See {@link ng.$sce $sce} for enabling strict contextual escaping.
         *
         * @param {string} type The kind of context in which this value is safe for use.  e.g. url,
         *   resourceUrl, html, js and css.
         * @param {*} value The value that that should be considered trusted/safe.
         * @returns {*} A value that can be used to stand in for the provided `value` in places
         * where Angular expects a $sce.trustAs() return value.
         */
        function trustAs(type, trustedValue) {
          var Constructor = byType.hasOwnProperty(type) ? byType[type] : null;
          if (!Constructor) {
            throw $sceMinErr('icontext', 'Attempted to trust a value in invalid context. Context: {0}; Value: {1}', type, trustedValue);
          }
          if (trustedValue === null || isUndefined(trustedValue) || trustedValue === '') {
            return trustedValue;
          }
          // All the current contexts in SCE_CONTEXTS happen to be strings.  In order to avoid trusting
          // mutable objects, we ensure here that the value passed in is actually a string.
          if (typeof trustedValue !== 'string') {
            throw $sceMinErr('itype', 'Attempted to trust a non-string value in a content requiring a string: Context: {0}', type);
          }
          return new Constructor(trustedValue);
        }

        /**
         * @ngdoc method
         * @name $sceDelegate#valueOf
         *
         * @description
         * If the passed parameter had been returned by a prior call to {@link ng.$sceDelegate#trustAs
         * `$sceDelegate.trustAs`}, returns the value that had been passed to {@link
         * ng.$sceDelegate#trustAs `$sceDelegate.trustAs`}.
         *
         * If the passed parameter is not a value that had been returned by {@link
         * ng.$sceDelegate#trustAs `$sceDelegate.trustAs`}, returns it as-is.
         *
         * @param {*} value The result of a prior {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs`}
         *      call or anything else.
         * @returns {*} The `value` that was originally provided to {@link ng.$sceDelegate#trustAs
         *     `$sceDelegate.trustAs`} if `value` is the result of such a call.  Otherwise, returns
         *     `value` unchanged.
         */
        function valueOf(maybeTrusted) {
          if (maybeTrusted instanceof trustedValueHolderBase) {
            return maybeTrusted.$$unwrapTrustedValue();
          } else {
            return maybeTrusted;
          }
        }

        /**
         * @ngdoc method
         * @name $sceDelegate#getTrusted
         *
         * @description
         * Takes the result of a {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs`} call and
         * returns the originally supplied value if the queried context type is a supertype of the
         * created type.  If this condition isn't satisfied, throws an exception.
         *
         * <div class="alert alert-danger">
         * Disabling auto-escaping is extremely dangerous, it usually creates a Cross Site Scripting
         * (XSS) vulnerability in your application.
         * </div>
         *
         * @param {string} type The kind of context in which this value is to be used.
         * @param {*} maybeTrusted The result of a prior {@link ng.$sceDelegate#trustAs
         *     `$sceDelegate.trustAs`} call.
         * @returns {*} The value the was originally provided to {@link ng.$sceDelegate#trustAs
         *     `$sceDelegate.trustAs`} if valid in this context.  Otherwise, throws an exception.
         */
        function getTrusted(type, maybeTrusted) {
          if (maybeTrusted === null || isUndefined(maybeTrusted) || maybeTrusted === '') {
            return maybeTrusted;
          }
          var constructor = byType.hasOwnProperty(type) ? byType[type] : null;
          if (constructor && maybeTrusted instanceof constructor) {
            return maybeTrusted.$$unwrapTrustedValue();
          }
          // If we get here, then we may only take one of two actions.
          // 1. sanitize the value for the requested type, or
          // 2. throw an exception.
          if (type === SCE_CONTEXTS.RESOURCE_URL) {
            if (isResourceUrlAllowedByPolicy(maybeTrusted)) {
              return maybeTrusted;
            } else {
              throw $sceMinErr('insecurl', 'Blocked loading resource from url not allowed by $sceDelegate policy.  URL: {0}', maybeTrusted.toString());
            }
          } else if (type === SCE_CONTEXTS.HTML) {
            return htmlSanitizer(maybeTrusted);
          }
          throw $sceMinErr('unsafe', 'Attempting to use an unsafe value in a safe context.');
        }

        return { trustAs: trustAs,
          getTrusted: getTrusted,
          valueOf: valueOf };
      }];
    }

    /**
     * @ngdoc provider
     * @name $sceProvider
     * @this
     *
     * @description
     *
     * The $sceProvider provider allows developers to configure the {@link ng.$sce $sce} service.
     * -   enable/disable Strict Contextual Escaping (SCE) in a module
     * -   override the default implementation with a custom delegate
     *
     * Read more about {@link ng.$sce Strict Contextual Escaping (SCE)}.
     */

    /**
     * @ngdoc service
     * @name $sce
     * @kind function
     *
     * @description
     *
     * `$sce` is a service that provides Strict Contextual Escaping services to AngularJS.
     *
     * # Strict Contextual Escaping
     *
     * Strict Contextual Escaping (SCE) is a mode in which AngularJS requires bindings in certain
     * contexts to result in a value that is marked as safe to use for that context.  One example of
     * such a context is binding arbitrary html controlled by the user via `ng-bind-html`.  We refer
     * to these contexts as privileged or SCE contexts.
     *
     * As of version 1.2, Angular ships with SCE enabled by default.
     *
     * Note:  When enabled (the default), IE<11 in quirks mode is not supported.  In this mode, IE<11 allow
     * one to execute arbitrary javascript by the use of the expression() syntax.  Refer
     * <http://blogs.msdn.com/b/ie/archive/2008/10/16/ending-expressions.aspx> to learn more about them.
     * You can ensure your document is in standards mode and not quirks mode by adding `<!doctype html>`
     * to the top of your HTML document.
     *
     * SCE assists in writing code in a way that (a) is secure by default and (b) makes auditing for
     * security vulnerabilities such as XSS, clickjacking, etc. a lot easier.
     *
     * Here's an example of a binding in a privileged context:
     *
     * ```
     * <input ng-model="userHtml" aria-label="User input">
     * <div ng-bind-html="userHtml"></div>
     * ```
     *
     * Notice that `ng-bind-html` is bound to `userHtml` controlled by the user.  With SCE
     * disabled, this application allows the user to render arbitrary HTML into the DIV.
     * In a more realistic example, one may be rendering user comments, blog articles, etc. via
     * bindings.  (HTML is just one example of a context where rendering user controlled input creates
     * security vulnerabilities.)
     *
     * For the case of HTML, you might use a library, either on the client side, or on the server side,
     * to sanitize unsafe HTML before binding to the value and rendering it in the document.
     *
     * How would you ensure that every place that used these types of bindings was bound to a value that
     * was sanitized by your library (or returned as safe for rendering by your server?)  How can you
     * ensure that you didn't accidentally delete the line that sanitized the value, or renamed some
     * properties/fields and forgot to update the binding to the sanitized value?
     *
     * To be secure by default, you want to ensure that any such bindings are disallowed unless you can
     * determine that something explicitly says it's safe to use a value for binding in that
     * context.  You can then audit your code (a simple grep would do) to ensure that this is only done
     * for those values that you can easily tell are safe - because they were received from your server,
     * sanitized by your library, etc.  You can organize your codebase to help with this - perhaps
     * allowing only the files in a specific directory to do this.  Ensuring that the internal API
     * exposed by that code doesn't markup arbitrary values as safe then becomes a more manageable task.
     *
     * In the case of AngularJS' SCE service, one uses {@link ng.$sce#trustAs $sce.trustAs}
     * (and shorthand methods such as {@link ng.$sce#trustAsHtml $sce.trustAsHtml}, etc.) to
     * obtain values that will be accepted by SCE / privileged contexts.
     *
     *
     * ## How does it work?
     *
     * In privileged contexts, directives and code will bind to the result of {@link ng.$sce#getTrusted
     * $sce.getTrusted(context, value)} rather than to the value directly.  Directives use {@link
     * ng.$sce#parseAs $sce.parseAs} rather than `$parse` to watch attribute bindings, which performs the
     * {@link ng.$sce#getTrusted $sce.getTrusted} behind the scenes on non-constant literals.
     *
     * As an example, {@link ng.directive:ngBindHtml ngBindHtml} uses {@link
     * ng.$sce#parseAsHtml $sce.parseAsHtml(binding expression)}.  Here's the actual code (slightly
     * simplified):
     *
     * ```
     * var ngBindHtmlDirective = ['$sce', function($sce) {
     *   return function(scope, element, attr) {
     *     scope.$watch($sce.parseAsHtml(attr.ngBindHtml), function(value) {
     *       element.html(value || '');
     *     });
     *   };
     * }];
     * ```
     *
     * ## Impact on loading templates
     *
     * This applies both to the {@link ng.directive:ngInclude `ng-include`} directive as well as
     * `templateUrl`'s specified by {@link guide/directive directives}.
     *
     * By default, Angular only loads templates from the same domain and protocol as the application
     * document.  This is done by calling {@link ng.$sce#getTrustedResourceUrl
     * $sce.getTrustedResourceUrl} on the template URL.  To load templates from other domains and/or
     * protocols, you may either {@link ng.$sceDelegateProvider#resourceUrlWhitelist whitelist
     * them} or {@link ng.$sce#trustAsResourceUrl wrap it} into a trusted value.
     *
     * *Please note*:
     * The browser's
     * [Same Origin Policy](https://code.google.com/p/browsersec/wiki/Part2#Same-origin_policy_for_XMLHttpRequest)
     * and [Cross-Origin Resource Sharing (CORS)](http://www.w3.org/TR/cors/)
     * policy apply in addition to this and may further restrict whether the template is successfully
     * loaded.  This means that without the right CORS policy, loading templates from a different domain
     * won't work on all browsers.  Also, loading templates from `file://` URL does not work on some
     * browsers.
     *
     * ## This feels like too much overhead
     *
     * It's important to remember that SCE only applies to interpolation expressions.
     *
     * If your expressions are constant literals, they're automatically trusted and you don't need to
     * call `$sce.trustAs` on them (remember to include the `ngSanitize` module) (e.g.
     * `<div ng-bind-html="'<b>implicitly trusted</b>'"></div>`) just works.
     *
     * Additionally, `a[href]` and `img[src]` automatically sanitize their URLs and do not pass them
     * through {@link ng.$sce#getTrusted $sce.getTrusted}.  SCE doesn't play a role here.
     *
     * The included {@link ng.$sceDelegate $sceDelegate} comes with sane defaults to allow you to load
     * templates in `ng-include` from your application's domain without having to even know about SCE.
     * It blocks loading templates from other domains or loading templates over http from an https
     * served document.  You can change these by setting your own custom {@link
     * ng.$sceDelegateProvider#resourceUrlWhitelist whitelists} and {@link
     * ng.$sceDelegateProvider#resourceUrlBlacklist blacklists} for matching such URLs.
     *
     * This significantly reduces the overhead.  It is far easier to pay the small overhead and have an
     * application that's secure and can be audited to verify that with much more ease than bolting
     * security onto an application later.
     *
     * <a name="contexts"></a>
     * ## What trusted context types are supported?
     *
     * | Context             | Notes          |
     * |---------------------|----------------|
     * | `$sce.HTML`         | For HTML that's safe to source into the application.  The {@link ng.directive:ngBindHtml ngBindHtml} directive uses this context for bindings. If an unsafe value is encountered and the {@link ngSanitize $sanitize} module is present this will sanitize the value instead of throwing an error. |
     * | `$sce.CSS`          | For CSS that's safe to source into the application.  Currently unused.  Feel free to use it in your own directives. |
     * | `$sce.URL`          | For URLs that are safe to follow as links.  Currently unused (`<a href=` and `<img src=` sanitize their urls and don't constitute an SCE context. |
     * | `$sce.RESOURCE_URL` | For URLs that are not only safe to follow as links, but whose contents are also safe to include in your application.  Examples include `ng-include`, `src` / `ngSrc` bindings for tags other than `IMG`, `VIDEO`, `AUDIO`, `SOURCE`, and `TRACK` (e.g. `IFRAME`, `OBJECT`, etc.)  <br><br>Note that `$sce.RESOURCE_URL` makes a stronger statement about the URL than `$sce.URL` does and therefore contexts requiring values trusted for `$sce.RESOURCE_URL` can be used anywhere that values trusted for `$sce.URL` are required. |
     * | `$sce.JS`           | For JavaScript that is safe to execute in your application's context.  Currently unused.  Feel free to use it in your own directives. |
     *
     * ## Format of items in {@link ng.$sceDelegateProvider#resourceUrlWhitelist resourceUrlWhitelist}/{@link ng.$sceDelegateProvider#resourceUrlBlacklist Blacklist} <a name="resourceUrlPatternItem"></a>
     *
     *  Each element in these arrays must be one of the following:
     *
     *  - **'self'**
     *    - The special **string**, `'self'`, can be used to match against all URLs of the **same
     *      domain** as the application document using the **same protocol**.
     *  - **String** (except the special value `'self'`)
     *    - The string is matched against the full *normalized / absolute URL* of the resource
     *      being tested (substring matches are not good enough.)
     *    - There are exactly **two wildcard sequences** - `*` and `**`.  All other characters
     *      match themselves.
     *    - `*`: matches zero or more occurrences of any character other than one of the following 6
     *      characters: '`:`', '`/`', '`.`', '`?`', '`&`' and '`;`'.  It's a useful wildcard for use
     *      in a whitelist.
     *    - `**`: matches zero or more occurrences of *any* character.  As such, it's not
     *      appropriate for use in a scheme, domain, etc. as it would match too much.  (e.g.
     *      http://**.example.com/ would match http://evil.com/?ignore=.example.com/ and that might
     *      not have been the intention.)  Its usage at the very end of the path is ok.  (e.g.
     *      http://foo.example.com/templates/**).
     *  - **RegExp** (*see caveat below*)
     *    - *Caveat*:  While regular expressions are powerful and offer great flexibility,  their syntax
     *      (and all the inevitable escaping) makes them *harder to maintain*.  It's easy to
     *      accidentally introduce a bug when one updates a complex expression (imho, all regexes should
     *      have good test coverage).  For instance, the use of `.` in the regex is correct only in a
     *      small number of cases.  A `.` character in the regex used when matching the scheme or a
     *      subdomain could be matched against a `:` or literal `.` that was likely not intended.   It
     *      is highly recommended to use the string patterns and only fall back to regular expressions
     *      as a last resort.
     *    - The regular expression must be an instance of RegExp (i.e. not a string.)  It is
     *      matched against the **entire** *normalized / absolute URL* of the resource being tested
     *      (even when the RegExp did not have the `^` and `$` codes.)  In addition, any flags
     *      present on the RegExp (such as multiline, global, ignoreCase) are ignored.
     *    - If you are generating your JavaScript from some other templating engine (not
     *      recommended, e.g. in issue [#4006](https://github.com/angular/angular.js/issues/4006)),
     *      remember to escape your regular expression (and be aware that you might need more than
     *      one level of escaping depending on your templating engine and the way you interpolated
     *      the value.)  Do make use of your platform's escaping mechanism as it might be good
     *      enough before coding your own.  E.g. Ruby has
     *      [Regexp.escape(str)](http://www.ruby-doc.org/core-2.0.0/Regexp.html#method-c-escape)
     *      and Python has [re.escape](http://docs.python.org/library/re.html#re.escape).
     *      Javascript lacks a similar built in function for escaping.  Take a look at Google
     *      Closure library's [goog.string.regExpEscape(s)](
     *      http://docs.closure-library.googlecode.com/git/closure_goog_string_string.js.source.html#line962).
     *
     * Refer {@link ng.$sceDelegateProvider $sceDelegateProvider} for an example.
     *
     * ## Show me an example using SCE.
     *
     * <example module="mySceApp" deps="angular-sanitize.js" name="sce-service">
     * <file name="index.html">
     *   <div ng-controller="AppController as myCtrl">
     *     <i ng-bind-html="myCtrl.explicitlyTrustedHtml" id="explicitlyTrustedHtml"></i><br><br>
     *     <b>User comments</b><br>
     *     By default, HTML that isn't explicitly trusted (e.g. Alice's comment) is sanitized when
     *     $sanitize is available.  If $sanitize isn't available, this results in an error instead of an
     *     exploit.
     *     <div class="well">
     *       <div ng-repeat="userComment in myCtrl.userComments">
     *         <b>{{userComment.name}}</b>:
     *         <span ng-bind-html="userComment.htmlComment" class="htmlComment"></span>
     *         <br>
     *       </div>
     *     </div>
     *   </div>
     * </file>
     *
     * <file name="script.js">
     *   angular.module('mySceApp', ['ngSanitize'])
     *     .controller('AppController', ['$http', '$templateCache', '$sce',
     *       function AppController($http, $templateCache, $sce) {
     *         var self = this;
     *         $http.get('test_data.json', {cache: $templateCache}).then(function(response) {
     *           self.userComments = response.data;
     *         });
     *         self.explicitlyTrustedHtml = $sce.trustAsHtml(
     *             '<span onmouseover="this.textContent=&quot;Explicitly trusted HTML bypasses ' +
     *             'sanitization.&quot;">Hover over this text.</span>');
     *       }]);
     * </file>
     *
     * <file name="test_data.json">
     * [
     *   { "name": "Alice",
     *     "htmlComment":
     *         "<span onmouseover='this.textContent=\"PWN3D!\"'>Is <i>anyone</i> reading this?</span>"
     *   },
     *   { "name": "Bob",
     *     "htmlComment": "<i>Yes!</i>  Am I the only other one?"
     *   }
     * ]
     * </file>
     *
     * <file name="protractor.js" type="protractor">
     *   describe('SCE doc demo', function() {
     *     it('should sanitize untrusted values', function() {
     *       expect(element.all(by.css('.htmlComment')).first().getAttribute('innerHTML'))
     *           .toBe('<span>Is <i>anyone</i> reading this?</span>');
     *     });
     *
     *     it('should NOT sanitize explicitly trusted values', function() {
     *       expect(element(by.id('explicitlyTrustedHtml')).getAttribute('innerHTML')).toBe(
     *           '<span onmouseover="this.textContent=&quot;Explicitly trusted HTML bypasses ' +
     *           'sanitization.&quot;">Hover over this text.</span>');
     *     });
     *   });
     * </file>
     * </example>
     *
     *
     *
     * ## Can I disable SCE completely?
     *
     * Yes, you can.  However, this is strongly discouraged.  SCE gives you a lot of security benefits
     * for little coding overhead.  It will be much harder to take an SCE disabled application and
     * either secure it on your own or enable SCE at a later stage.  It might make sense to disable SCE
     * for cases where you have a lot of existing code that was written before SCE was introduced and
     * you're migrating them a module at a time.
     *
     * That said, here's how you can completely disable SCE:
     *
     * ```
     * angular.module('myAppWithSceDisabledmyApp', []).config(function($sceProvider) {
     *   // Completely disable SCE.  For demonstration purposes only!
     *   // Do not use in new projects.
     *   $sceProvider.enabled(false);
     * });
     * ```
     *
     */

    function $SceProvider() {
      var enabled = true;

      /**
       * @ngdoc method
       * @name $sceProvider#enabled
       * @kind function
       *
       * @param {boolean=} value If provided, then enables/disables SCE.
       * @return {boolean} true if SCE is enabled, false otherwise.
       *
       * @description
       * Enables/disables SCE and returns the current value.
       */
      this.enabled = function (value) {
        if (arguments.length) {
          enabled = !!value;
        }
        return enabled;
      };

      /* Design notes on the default implementation for SCE.
       *
       * The API contract for the SCE delegate
       * -------------------------------------
       * The SCE delegate object must provide the following 3 methods:
       *
       * - trustAs(contextEnum, value)
       *     This method is used to tell the SCE service that the provided value is OK to use in the
       *     contexts specified by contextEnum.  It must return an object that will be accepted by
       *     getTrusted() for a compatible contextEnum and return this value.
       *
       * - valueOf(value)
       *     For values that were not produced by trustAs(), return them as is.  For values that were
       *     produced by trustAs(), return the corresponding input value to trustAs.  Basically, if
       *     trustAs is wrapping the given values into some type, this operation unwraps it when given
       *     such a value.
       *
       * - getTrusted(contextEnum, value)
       *     This function should return the a value that is safe to use in the context specified by
       *     contextEnum or throw and exception otherwise.
       *
       * NOTE: This contract deliberately does NOT state that values returned by trustAs() must be
       * opaque or wrapped in some holder object.  That happens to be an implementation detail.  For
       * instance, an implementation could maintain a registry of all trusted objects by context.  In
       * such a case, trustAs() would return the same object that was passed in.  getTrusted() would
       * return the same object passed in if it was found in the registry under a compatible context or
       * throw an exception otherwise.  An implementation might only wrap values some of the time based
       * on some criteria.  getTrusted() might return a value and not throw an exception for special
       * constants or objects even if not wrapped.  All such implementations fulfill this contract.
       *
       *
       * A note on the inheritance model for SCE contexts
       * ------------------------------------------------
       * I've used inheritance and made RESOURCE_URL wrapped types a subtype of URL wrapped types.  This
       * is purely an implementation details.
       *
       * The contract is simply this:
       *
       *     getTrusted($sce.RESOURCE_URL, value) succeeding implies that getTrusted($sce.URL, value)
       *     will also succeed.
       *
       * Inheritance happens to capture this in a natural way.  In some future, we
       * may not use inheritance anymore.  That is OK because no code outside of
       * sce.js and sceSpecs.js would need to be aware of this detail.
       */

      this.$get = ['$parse', '$sceDelegate', function ($parse, $sceDelegate) {
        // Support: IE 9-11 only
        // Prereq: Ensure that we're not running in IE<11 quirks mode.  In that mode, IE < 11 allow
        // the "expression(javascript expression)" syntax which is insecure.
        if (enabled && msie < 8) {
          throw $sceMinErr('iequirks', 'Strict Contextual Escaping does not support Internet Explorer version < 11 in quirks ' + 'mode.  You can fix this by adding the text <!doctype html> to the top of your HTML ' + 'document.  See http://docs.angularjs.org/api/ng.$sce for more information.');
        }

        var sce = shallowCopy(SCE_CONTEXTS);

        /**
         * @ngdoc method
         * @name $sce#isEnabled
         * @kind function
         *
         * @return {Boolean} true if SCE is enabled, false otherwise.  If you want to set the value, you
         * have to do it at module config time on {@link ng.$sceProvider $sceProvider}.
         *
         * @description
         * Returns a boolean indicating if SCE is enabled.
         */
        sce.isEnabled = function () {
          return enabled;
        };
        sce.trustAs = $sceDelegate.trustAs;
        sce.getTrusted = $sceDelegate.getTrusted;
        sce.valueOf = $sceDelegate.valueOf;

        if (!enabled) {
          sce.trustAs = sce.getTrusted = function (type, value) {
            return value;
          };
          sce.valueOf = identity;
        }

        /**
         * @ngdoc method
         * @name $sce#parseAs
         *
         * @description
         * Converts Angular {@link guide/expression expression} into a function.  This is like {@link
         * ng.$parse $parse} and is identical when the expression is a literal constant.  Otherwise, it
         * wraps the expression in a call to {@link ng.$sce#getTrusted $sce.getTrusted(*type*,
         * *result*)}
         *
         * @param {string} type The kind of SCE context in which this result will be used.
         * @param {string} expression String expression to compile.
         * @returns {function(context, locals)} a function which represents the compiled expression:
         *
         *    * `context` – `{object}` – an object against which any expressions embedded in the strings
         *      are evaluated against (typically a scope object).
         *    * `locals` – `{object=}` – local variables context object, useful for overriding values in
         *      `context`.
         */
        sce.parseAs = function sceParseAs(type, expr) {
          var parsed = $parse(expr);
          if (parsed.literal && parsed.constant) {
            return parsed;
          } else {
            return $parse(expr, function (value) {
              return sce.getTrusted(type, value);
            });
          }
        };

        /**
         * @ngdoc method
         * @name $sce#trustAs
         *
         * @description
         * Delegates to {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs`}.  As such,
         * returns an object that is trusted by angular for use in specified strict contextual
         * escaping contexts (such as ng-bind-html, ng-include, any src attribute
         * interpolation, any dom event binding attribute interpolation such as for onclick,  etc.)
         * that uses the provided value.  See * {@link ng.$sce $sce} for enabling strict contextual
         * escaping.
         *
         * @param {string} type The kind of context in which this value is safe for use.  e.g. url,
         *   resourceUrl, html, js and css.
         * @param {*} value The value that that should be considered trusted/safe.
         * @returns {*} A value that can be used to stand in for the provided `value` in places
         * where Angular expects a $sce.trustAs() return value.
         */

        /**
         * @ngdoc method
         * @name $sce#trustAsHtml
         *
         * @description
         * Shorthand method.  `$sce.trustAsHtml(value)` →
         *     {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs($sce.HTML, value)`}
         *
         * @param {*} value The value to trustAs.
         * @returns {*} An object that can be passed to {@link ng.$sce#getTrustedHtml
         *     $sce.getTrustedHtml(value)} to obtain the original value.  (privileged directives
         *     only accept expressions that are either literal constants or are the
         *     return value of {@link ng.$sce#trustAs $sce.trustAs}.)
         */

        /**
         * @ngdoc method
         * @name $sce#trustAsUrl
         *
         * @description
         * Shorthand method.  `$sce.trustAsUrl(value)` →
         *     {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs($sce.URL, value)`}
         *
         * @param {*} value The value to trustAs.
         * @returns {*} An object that can be passed to {@link ng.$sce#getTrustedUrl
         *     $sce.getTrustedUrl(value)} to obtain the original value.  (privileged directives
         *     only accept expressions that are either literal constants or are the
         *     return value of {@link ng.$sce#trustAs $sce.trustAs}.)
         */

        /**
         * @ngdoc method
         * @name $sce#trustAsResourceUrl
         *
         * @description
         * Shorthand method.  `$sce.trustAsResourceUrl(value)` →
         *     {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs($sce.RESOURCE_URL, value)`}
         *
         * @param {*} value The value to trustAs.
         * @returns {*} An object that can be passed to {@link ng.$sce#getTrustedResourceUrl
         *     $sce.getTrustedResourceUrl(value)} to obtain the original value.  (privileged directives
         *     only accept expressions that are either literal constants or are the return
         *     value of {@link ng.$sce#trustAs $sce.trustAs}.)
         */

        /**
         * @ngdoc method
         * @name $sce#trustAsJs
         *
         * @description
         * Shorthand method.  `$sce.trustAsJs(value)` →
         *     {@link ng.$sceDelegate#trustAs `$sceDelegate.trustAs($sce.JS, value)`}
         *
         * @param {*} value The value to trustAs.
         * @returns {*} An object that can be passed to {@link ng.$sce#getTrustedJs
         *     $sce.getTrustedJs(value)} to obtain the original value.  (privileged directives
         *     only accept expressions that are either literal constants or are the
         *     return value of {@link ng.$sce#trustAs $sce.trustAs}.)
         */

        /**
         * @ngdoc method
         * @name $sce#getTrusted
         *
         * @description
         * Delegates to {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted`}.  As such,
         * takes the result of a {@link ng.$sce#trustAs `$sce.trustAs`}() call and returns the
         * originally supplied value if the queried context type is a supertype of the created type.
         * If this condition isn't satisfied, throws an exception.
         *
         * @param {string} type The kind of context in which this value is to be used.
         * @param {*} maybeTrusted The result of a prior {@link ng.$sce#trustAs `$sce.trustAs`}
         *                         call.
         * @returns {*} The value the was originally provided to
         *              {@link ng.$sce#trustAs `$sce.trustAs`} if valid in this context.
         *              Otherwise, throws an exception.
         */

        /**
         * @ngdoc method
         * @name $sce#getTrustedHtml
         *
         * @description
         * Shorthand method.  `$sce.getTrustedHtml(value)` →
         *     {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted($sce.HTML, value)`}
         *
         * @param {*} value The value to pass to `$sce.getTrusted`.
         * @returns {*} The return value of `$sce.getTrusted($sce.HTML, value)`
         */

        /**
         * @ngdoc method
         * @name $sce#getTrustedCss
         *
         * @description
         * Shorthand method.  `$sce.getTrustedCss(value)` →
         *     {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted($sce.CSS, value)`}
         *
         * @param {*} value The value to pass to `$sce.getTrusted`.
         * @returns {*} The return value of `$sce.getTrusted($sce.CSS, value)`
         */

        /**
         * @ngdoc method
         * @name $sce#getTrustedUrl
         *
         * @description
         * Shorthand method.  `$sce.getTrustedUrl(value)` →
         *     {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted($sce.URL, value)`}
         *
         * @param {*} value The value to pass to `$sce.getTrusted`.
         * @returns {*} The return value of `$sce.getTrusted($sce.URL, value)`
         */

        /**
         * @ngdoc method
         * @name $sce#getTrustedResourceUrl
         *
         * @description
         * Shorthand method.  `$sce.getTrustedResourceUrl(value)` →
         *     {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted($sce.RESOURCE_URL, value)`}
         *
         * @param {*} value The value to pass to `$sceDelegate.getTrusted`.
         * @returns {*} The return value of `$sce.getTrusted($sce.RESOURCE_URL, value)`
         */

        /**
         * @ngdoc method
         * @name $sce#getTrustedJs
         *
         * @description
         * Shorthand method.  `$sce.getTrustedJs(value)` →
         *     {@link ng.$sceDelegate#getTrusted `$sceDelegate.getTrusted($sce.JS, value)`}
         *
         * @param {*} value The value to pass to `$sce.getTrusted`.
         * @returns {*} The return value of `$sce.getTrusted($sce.JS, value)`
         */

        /**
         * @ngdoc method
         * @name $sce#parseAsHtml
         *
         * @description
         * Shorthand method.  `$sce.parseAsHtml(expression string)` →
         *     {@link ng.$sce#parseAs `$sce.parseAs($sce.HTML, value)`}
         *
         * @param {string} expression String expression to compile.
         * @returns {function(context, locals)} a function which represents the compiled expression:
         *
         *    * `context` – `{object}` – an object against which any expressions embedded in the strings
         *      are evaluated against (typically a scope object).
         *    * `locals` – `{object=}` – local variables context object, useful for overriding values in
         *      `context`.
         */

        /**
         * @ngdoc method
         * @name $sce#parseAsCss
         *
         * @description
         * Shorthand method.  `$sce.parseAsCss(value)` →
         *     {@link ng.$sce#parseAs `$sce.parseAs($sce.CSS, value)`}
         *
         * @param {string} expression String expression to compile.
         * @returns {function(context, locals)} a function which represents the compiled expression:
         *
         *    * `context` – `{object}` – an object against which any expressions embedded in the strings
         *      are evaluated against (typically a scope object).
         *    * `locals` – `{object=}` – local variables context object, useful for overriding values in
         *      `context`.
         */

        /**
         * @ngdoc method
         * @name $sce#parseAsUrl
         *
         * @description
         * Shorthand method.  `$sce.parseAsUrl(value)` →
         *     {@link ng.$sce#parseAs `$sce.parseAs($sce.URL, value)`}
         *
         * @param {string} expression String expression to compile.
         * @returns {function(context, locals)} a function which represents the compiled expression:
         *
         *    * `context` – `{object}` – an object against which any expressions embedded in the strings
         *      are evaluated against (typically a scope object).
         *    * `locals` – `{object=}` – local variables context object, useful for overriding values in
         *      `context`.
         */

        /**
         * @ngdoc method
         * @name $sce#parseAsResourceUrl
         *
         * @description
         * Shorthand method.  `$sce.parseAsResourceUrl(value)` →
         *     {@link ng.$sce#parseAs `$sce.parseAs($sce.RESOURCE_URL, value)`}
         *
         * @param {string} expression String expression to compile.
         * @returns {function(context, locals)} a function which represents the compiled expression:
         *
         *    * `context` – `{object}` – an object against which any expressions embedded in the strings
         *      are evaluated against (typically a scope object).
         *    * `locals` – `{object=}` – local variables context object, useful for overriding values in
         *      `context`.
         */

        /**
         * @ngdoc method
         * @name $sce#parseAsJs
         *
         * @description
         * Shorthand method.  `$sce.parseAsJs(value)` →
         *     {@link ng.$sce#parseAs `$sce.parseAs($sce.JS, value)`}
         *
         * @param {string} expression String expression to compile.
         * @returns {function(context, locals)} a function which represents the compiled expression:
         *
         *    * `context` – `{object}` – an object against which any expressions embedded in the strings
         *      are evaluated against (typically a scope object).
         *    * `locals` – `{object=}` – local variables context object, useful for overriding values in
         *      `context`.
         */

        // Shorthand delegations.
        var parse = sce.parseAs,
            getTrusted = sce.getTrusted,
            trustAs = sce.trustAs;

        forEach(SCE_CONTEXTS, function (enumValue, name) {
          var lName = lowercase(name);
          sce[snakeToCamel('parse_as_' + lName)] = function (expr) {
            return parse(enumValue, expr);
          };
          sce[snakeToCamel('get_trusted_' + lName)] = function (value) {
            return getTrusted(enumValue, value);
          };
          sce[snakeToCamel('trust_as_' + lName)] = function (value) {
            return trustAs(enumValue, value);
          };
        });

        return sce;
      }];
    }

    /* exported $SnifferProvider */

    /**
     * !!! This is an undocumented "private" service !!!
     *
     * @name $sniffer
     * @requires $window
     * @requires $document
     * @this
     *
     * @property {boolean} history Does the browser support html5 history api ?
     * @property {boolean} transitions Does the browser support CSS transition events ?
     * @property {boolean} animations Does the browser support CSS animation events ?
     *
     * @description
     * This is very simple implementation of testing browser's features.
     */
    function $SnifferProvider() {
      this.$get = ['$window', '$document', function ($window, $document) {
        var eventSupport = {},

        // Chrome Packaged Apps are not allowed to access `history.pushState`.
        // If not sandboxed, they can be detected by the presence of `chrome.app.runtime`
        // (see https://developer.chrome.com/apps/api_index). If sandboxed, they can be detected by
        // the presence of an extension runtime ID and the absence of other Chrome runtime APIs
        // (see https://developer.chrome.com/apps/manifest/sandbox).
        // (NW.js apps have access to Chrome APIs, but do support `history`.)
        isNw = $window.nw && $window.nw.process,
            isChromePackagedApp = !isNw && $window.chrome && ($window.chrome.app && $window.chrome.app.runtime || !$window.chrome.app && $window.chrome.runtime && $window.chrome.runtime.id),
            hasHistoryPushState = !isChromePackagedApp && $window.history && $window.history.pushState,
            android = toInt((/android (\d+)/.exec(lowercase(($window.navigator || {}).userAgent)) || [])[1]),
            boxee = /Boxee/i.test(($window.navigator || {}).userAgent),
            document = $document[0] || {},
            bodyStyle = document.body && document.body.style,
            transitions = false,
            animations = false;

        if (bodyStyle) {
          // Support: Android <5, Blackberry Browser 10, default Chrome in Android 4.4.x
          // Mentioned browsers need a -webkit- prefix for transitions & animations.
          transitions = !!('transition' in bodyStyle || 'webkitTransition' in bodyStyle);
          animations = !!('animation' in bodyStyle || 'webkitAnimation' in bodyStyle);
        }

        return {
          // Android has history.pushState, but it does not update location correctly
          // so let's not use the history API at all.
          // http://code.google.com/p/android/issues/detail?id=17471
          // https://github.com/angular/angular.js/issues/904

          // older webkit browser (533.9) on Boxee box has exactly the same problem as Android has
          // so let's not use the history API also
          // We are purposefully using `!(android < 4)` to cover the case when `android` is undefined
          history: !!(hasHistoryPushState && !(android < 4) && !boxee),
          hasEvent: function (event) {
            // Support: IE 9-11 only
            // IE9 implements 'input' event it's so fubared that we rather pretend that it doesn't have
            // it. In particular the event is not fired when backspace or delete key are pressed or
            // when cut operation is performed.
            // IE10+ implements 'input' event but it erroneously fires under various situations,
            // e.g. when placeholder changes, or a form is focused.
            if (event === 'input' && msie) return false;

            if (isUndefined(eventSupport[event])) {
              var divElm = document.createElement('div');
              eventSupport[event] = 'on' + event in divElm;
            }

            return eventSupport[event];
          },
          csp: csp(),
          transitions: transitions,
          animations: animations,
          android: android
        };
      }];
    }

    var $templateRequestMinErr = minErr('$compile');

    /**
     * @ngdoc provider
     * @name $templateRequestProvider
     * @this
     *
     * @description
     * Used to configure the options passed to the {@link $http} service when making a template request.
     *
     * For example, it can be used for specifying the "Accept" header that is sent to the server, when
     * requesting a template.
     */
    function $TemplateRequestProvider() {

      var httpOptions;

      /**
       * @ngdoc method
       * @name $templateRequestProvider#httpOptions
       * @description
       * The options to be passed to the {@link $http} service when making the request.
       * You can use this to override options such as the "Accept" header for template requests.
       *
       * The {@link $templateRequest} will set the `cache` and the `transformResponse` properties of the
       * options if not overridden here.
       *
       * @param {string=} value new value for the {@link $http} options.
       * @returns {string|self} Returns the {@link $http} options when used as getter and self if used as setter.
       */
      this.httpOptions = function (val) {
        if (val) {
          httpOptions = val;
          return this;
        }
        return httpOptions;
      };

      /**
       * @ngdoc service
       * @name $templateRequest
       *
       * @description
       * The `$templateRequest` service runs security checks then downloads the provided template using
       * `$http` and, upon success, stores the contents inside of `$templateCache`. If the HTTP request
       * fails or the response data of the HTTP request is empty, a `$compile` error will be thrown (the
       * exception can be thwarted by setting the 2nd parameter of the function to true). Note that the
       * contents of `$templateCache` are trusted, so the call to `$sce.getTrustedUrl(tpl)` is omitted
       * when `tpl` is of type string and `$templateCache` has the matching entry.
       *
       * If you want to pass custom options to the `$http` service, such as setting the Accept header you
       * can configure this via {@link $templateRequestProvider#httpOptions}.
       *
       * @param {string|TrustedResourceUrl} tpl The HTTP request template URL
       * @param {boolean=} ignoreRequestError Whether or not to ignore the exception when the request fails or the template is empty
       *
       * @return {Promise} a promise for the HTTP response data of the given URL.
       *
       * @property {number} totalPendingRequests total amount of pending template requests being downloaded.
       */
      this.$get = ['$exceptionHandler', '$templateCache', '$http', '$q', '$sce', function ($exceptionHandler, $templateCache, $http, $q, $sce) {

        function handleRequestFn(tpl, ignoreRequestError) {
          handleRequestFn.totalPendingRequests++;

          // We consider the template cache holds only trusted templates, so
          // there's no need to go through whitelisting again for keys that already
          // are included in there. This also makes Angular accept any script
          // directive, no matter its name. However, we still need to unwrap trusted
          // types.
          if (!isString(tpl) || isUndefined($templateCache.get(tpl))) {
            tpl = $sce.getTrustedResourceUrl(tpl);
          }

          var transformResponse = $http.defaults && $http.defaults.transformResponse;

          if (isArray(transformResponse)) {
            transformResponse = transformResponse.filter(function (transformer) {
              return transformer !== defaultHttpResponseTransform;
            });
          } else if (transformResponse === defaultHttpResponseTransform) {
            transformResponse = null;
          }

          return $http.get(tpl, extend({
            cache: $templateCache,
            transformResponse: transformResponse
          }, httpOptions)).finally(function () {
            handleRequestFn.totalPendingRequests--;
          }).then(function (response) {
            $templateCache.put(tpl, response.data);
            return response.data;
          }, handleError);

          function handleError(resp) {
            if (!ignoreRequestError) {
              resp = $templateRequestMinErr('tpload', 'Failed to load template: {0} (HTTP status: {1} {2})', tpl, resp.status, resp.statusText);

              $exceptionHandler(resp);
            }

            return $q.reject(resp);
          }
        }

        handleRequestFn.totalPendingRequests = 0;

        return handleRequestFn;
      }];
    }

    /** @this */
    function $$TestabilityProvider() {
      this.$get = ['$rootScope', '$browser', '$location', function ($rootScope, $browser, $location) {

        /**
         * @name $testability
         *
         * @description
         * The private $$testability service provides a collection of methods for use when debugging
         * or by automated test and debugging tools.
         */
        var testability = {};

        /**
         * @name $$testability#findBindings
         *
         * @description
         * Returns an array of elements that are bound (via ng-bind or {{}})
         * to expressions matching the input.
         *
         * @param {Element} element The element root to search from.
         * @param {string} expression The binding expression to match.
         * @param {boolean} opt_exactMatch If true, only returns exact matches
         *     for the expression. Filters and whitespace are ignored.
         */
        testability.findBindings = function (element, expression, opt_exactMatch) {
          var bindings = element.getElementsByClassName('ng-binding');
          var matches = [];
          forEach(bindings, function (binding) {
            var dataBinding = angular.element(binding).data('$binding');
            if (dataBinding) {
              forEach(dataBinding, function (bindingName) {
                if (opt_exactMatch) {
                  var matcher = new RegExp('(^|\\s)' + escapeForRegexp(expression) + '(\\s|\\||$)');
                  if (matcher.test(bindingName)) {
                    matches.push(binding);
                  }
                } else {
                  if (bindingName.indexOf(expression) !== -1) {
                    matches.push(binding);
                  }
                }
              });
            }
          });
          return matches;
        };

        /**
         * @name $$testability#findModels
         *
         * @description
         * Returns an array of elements that are two-way found via ng-model to
         * expressions matching the input.
         *
         * @param {Element} element The element root to search from.
         * @param {string} expression The model expression to match.
         * @param {boolean} opt_exactMatch If true, only returns exact matches
         *     for the expression.
         */
        testability.findModels = function (element, expression, opt_exactMatch) {
          var prefixes = ['ng-', 'data-ng-', 'ng\\:'];
          for (var p = 0; p < prefixes.length; ++p) {
            var attributeEquals = opt_exactMatch ? '=' : '*=';
            var selector = '[' + prefixes[p] + 'model' + attributeEquals + '"' + expression + '"]';
            var elements = element.querySelectorAll(selector);
            if (elements.length) {
              return elements;
            }
          }
        };

        /**
         * @name $$testability#getLocation
         *
         * @description
         * Shortcut for getting the location in a browser agnostic way. Returns
         *     the path, search, and hash. (e.g. /path?a=b#hash)
         */
        testability.getLocation = function () {
          return $location.url();
        };

        /**
         * @name $$testability#setLocation
         *
         * @description
         * Shortcut for navigating to a location without doing a full page reload.
         *
         * @param {string} url The location url (path, search and hash,
         *     e.g. /path?a=b#hash) to go to.
         */
        testability.setLocation = function (url) {
          if (url !== $location.url()) {
            $location.url(url);
            $rootScope.$digest();
          }
        };

        /**
         * @name $$testability#whenStable
         *
         * @description
         * Calls the callback when $timeout and $http requests are completed.
         *
         * @param {function} callback
         */
        testability.whenStable = function (callback) {
          $browser.notifyWhenNoOutstandingRequests(callback);
        };

        return testability;
      }];
    }

    /** @this */
    function $TimeoutProvider() {
      this.$get = ['$rootScope', '$browser', '$q', '$$q', '$exceptionHandler', function ($rootScope, $browser, $q, $$q, $exceptionHandler) {

        var deferreds = {};

        /**
         * @ngdoc service
         * @name $timeout
         *
         * @description
         * Angular's wrapper for `window.setTimeout`. The `fn` function is wrapped into a try/catch
         * block and delegates any exceptions to
         * {@link ng.$exceptionHandler $exceptionHandler} service.
         *
         * The return value of calling `$timeout` is a promise, which will be resolved when
         * the delay has passed and the timeout function, if provided, is executed.
         *
         * To cancel a timeout request, call `$timeout.cancel(promise)`.
         *
         * In tests you can use {@link ngMock.$timeout `$timeout.flush()`} to
         * synchronously flush the queue of deferred functions.
         *
         * If you only want a promise that will be resolved after some specified delay
         * then you can call `$timeout` without the `fn` function.
         *
         * @param {function()=} fn A function, whose execution should be delayed.
         * @param {number=} [delay=0] Delay in milliseconds.
         * @param {boolean=} [invokeApply=true] If set to `false` skips model dirty checking, otherwise
         *   will invoke `fn` within the {@link ng.$rootScope.Scope#$apply $apply} block.
         * @param {...*=} Pass additional parameters to the executed function.
         * @returns {Promise} Promise that will be resolved when the timeout is reached. The promise
         *   will be resolved with the return value of the `fn` function.
         *
         */
        function timeout(fn, delay, invokeApply) {
          if (!isFunction(fn)) {
            invokeApply = delay;
            delay = fn;
            fn = noop;
          }

          var args = sliceArgs(arguments, 3),
              skipApply = isDefined(invokeApply) && !invokeApply,
              deferred = (skipApply ? $$q : $q).defer(),
              promise = deferred.promise,
              timeoutId;

          timeoutId = $browser.defer(function () {
            try {
              deferred.resolve(fn.apply(null, args));
            } catch (e) {
              deferred.reject(e);
              $exceptionHandler(e);
            } finally {
              delete deferreds[promise.$$timeoutId];
            }

            if (!skipApply) $rootScope.$apply();
          }, delay);

          promise.$$timeoutId = timeoutId;
          deferreds[timeoutId] = deferred;

          return promise;
        }

        /**
         * @ngdoc method
         * @name $timeout#cancel
         *
         * @description
         * Cancels a task associated with the `promise`. As a result of this, the promise will be
         * resolved with a rejection.
         *
         * @param {Promise=} promise Promise returned by the `$timeout` function.
         * @returns {boolean} Returns `true` if the task hasn't executed yet and was successfully
         *   canceled.
         */
        timeout.cancel = function (promise) {
          if (promise && promise.$$timeoutId in deferreds) {
            // Timeout cancels should not report an unhandled promise.
            deferreds[promise.$$timeoutId].promise.catch(noop);
            deferreds[promise.$$timeoutId].reject('canceled');
            delete deferreds[promise.$$timeoutId];
            return $browser.defer.cancel(promise.$$timeoutId);
          }
          return false;
        };

        return timeout;
      }];
    }

    // NOTE:  The usage of window and document instead of $window and $document here is
    // deliberate.  This service depends on the specific behavior of anchor nodes created by the
    // browser (resolving and parsing URLs) that is unlikely to be provided by mock objects and
    // cause us to break tests.  In addition, when the browser resolves a URL for XHR, it
    // doesn't know about mocked locations and resolves URLs to the real document - which is
    // exactly the behavior needed here.  There is little value is mocking these out for this
    // service.
    var urlParsingNode = window.document.createElement('a');
    var originUrl = urlResolve(window.location.href);

    /**
     *
     * Implementation Notes for non-IE browsers
     * ----------------------------------------
     * Assigning a URL to the href property of an anchor DOM node, even one attached to the DOM,
     * results both in the normalizing and parsing of the URL.  Normalizing means that a relative
     * URL will be resolved into an absolute URL in the context of the application document.
     * Parsing means that the anchor node's host, hostname, protocol, port, pathname and related
     * properties are all populated to reflect the normalized URL.  This approach has wide
     * compatibility - Safari 1+, Mozilla 1+ etc.  See
     * http://www.aptana.com/reference/html/api/HTMLAnchorElement.html
     *
     * Implementation Notes for IE
     * ---------------------------
     * IE <= 10 normalizes the URL when assigned to the anchor node similar to the other
     * browsers.  However, the parsed components will not be set if the URL assigned did not specify
     * them.  (e.g. if you assign a.href = "foo", then a.protocol, a.host, etc. will be empty.)  We
     * work around that by performing the parsing in a 2nd step by taking a previously normalized
     * URL (e.g. by assigning to a.href) and assigning it a.href again.  This correctly populates the
     * properties such as protocol, hostname, port, etc.
     *
     * References:
     *   http://developer.mozilla.org/en-US/docs/Web/API/HTMLAnchorElement
     *   http://www.aptana.com/reference/html/api/HTMLAnchorElement.html
     *   http://url.spec.whatwg.org/#urlutils
     *   https://github.com/angular/angular.js/pull/2902
     *   http://james.padolsey.com/javascript/parsing-urls-with-the-dom/
     *
     * @kind function
     * @param {string} url The URL to be parsed.
     * @description Normalizes and parses a URL.
     * @returns {object} Returns the normalized URL as a dictionary.
     *
     *   | member name   | Description    |
     *   |---------------|----------------|
     *   | href          | A normalized version of the provided URL if it was not an absolute URL |
     *   | protocol      | The protocol including the trailing colon                              |
     *   | host          | The host and port (if the port is non-default) of the normalizedUrl    |
     *   | search        | The search params, minus the question mark                             |
     *   | hash          | The hash string, minus the hash symbol
     *   | hostname      | The hostname
     *   | port          | The port, without ":"
     *   | pathname      | The pathname, beginning with "/"
     *
     */
    function urlResolve(url) {
      var href = url;

      // Support: IE 9-11 only
      if (msie) {
        // Normalize before parse.  Refer Implementation Notes on why this is
        // done in two steps on IE.
        urlParsingNode.setAttribute('href', href);
        href = urlParsingNode.href;
      }

      urlParsingNode.setAttribute('href', href);

      // urlParsingNode provides the UrlUtils interface - http://url.spec.whatwg.org/#urlutils
      return {
        href: urlParsingNode.href,
        protocol: urlParsingNode.protocol ? urlParsingNode.protocol.replace(/:$/, '') : '',
        host: urlParsingNode.host,
        search: urlParsingNode.search ? urlParsingNode.search.replace(/^\?/, '') : '',
        hash: urlParsingNode.hash ? urlParsingNode.hash.replace(/^#/, '') : '',
        hostname: urlParsingNode.hostname,
        port: urlParsingNode.port,
        pathname: urlParsingNode.pathname.charAt(0) === '/' ? urlParsingNode.pathname : '/' + urlParsingNode.pathname
      };
    }

    /**
     * Parse a request URL and determine whether this is a same-origin request as the application document.
     *
     * @param {string|object} requestUrl The url of the request as a string that will be resolved
     * or a parsed URL object.
     * @returns {boolean} Whether the request is for the same origin as the application document.
     */
    function urlIsSameOrigin(requestUrl) {
      var parsed = isString(requestUrl) ? urlResolve(requestUrl) : requestUrl;
      return parsed.protocol === originUrl.protocol && parsed.host === originUrl.host;
    }

    /**
     * @ngdoc service
     * @name $window
     * @this
     *
     * @description
     * A reference to the browser's `window` object. While `window`
     * is globally available in JavaScript, it causes testability problems, because
     * it is a global variable. In angular we always refer to it through the
     * `$window` service, so it may be overridden, removed or mocked for testing.
     *
     * Expressions, like the one defined for the `ngClick` directive in the example
     * below, are evaluated with respect to the current scope.  Therefore, there is
     * no risk of inadvertently coding in a dependency on a global value in such an
     * expression.
     *
     * @example
       <example module="windowExample" name="window-service">
         <file name="index.html">
           <script>
             angular.module('windowExample', [])
               .controller('ExampleController', ['$scope', '$window', function($scope, $window) {
                 $scope.greeting = 'Hello, World!';
                 $scope.doGreeting = function(greeting) {
                   $window.alert(greeting);
                 };
               }]);
           </script>
           <div ng-controller="ExampleController">
             <input type="text" ng-model="greeting" aria-label="greeting" />
             <button ng-click="doGreeting(greeting)">ALERT</button>
           </div>
         </file>
         <file name="protractor.js" type="protractor">
          it('should display the greeting in the input box', function() {
           element(by.model('greeting')).sendKeys('Hello, E2E Tests');
           // If we click the button it will block the test runner
           // element(':button').click();
          });
         </file>
       </example>
     */
    function $WindowProvider() {
      this.$get = valueFn(window);
    }

    /**
     * @name $$cookieReader
     * @requires $document
     *
     * @description
     * This is a private service for reading cookies used by $http and ngCookies
     *
     * @return {Object} a key/value map of the current cookies
     */
    function $$CookieReader($document) {
      var rawDocument = $document[0] || {};
      var lastCookies = {};
      var lastCookieString = '';

      function safeGetCookie(rawDocument) {
        try {
          return rawDocument.cookie || '';
        } catch (e) {
          return '';
        }
      }

      function safeDecodeURIComponent(str) {
        try {
          return decodeURIComponent(str);
        } catch (e) {
          return str;
        }
      }

      return function () {
        var cookieArray, cookie, i, index, name;
        var currentCookieString = safeGetCookie(rawDocument);

        if (currentCookieString !== lastCookieString) {
          lastCookieString = currentCookieString;
          cookieArray = lastCookieString.split('; ');
          lastCookies = {};

          for (i = 0; i < cookieArray.length; i++) {
            cookie = cookieArray[i];
            index = cookie.indexOf('=');
            if (index > 0) {
              //ignore nameless cookies
              name = safeDecodeURIComponent(cookie.substring(0, index));
              // the first value that is seen for a cookie is the most
              // specific one.  values for the same cookie name that
              // follow are for less specific paths.
              if (isUndefined(lastCookies[name])) {
                lastCookies[name] = safeDecodeURIComponent(cookie.substring(index + 1));
              }
            }
          }
        }
        return lastCookies;
      };
    }

    $$CookieReader.$inject = ['$document'];

    /** @this */
    function $$CookieReaderProvider() {
      this.$get = $$CookieReader;
    }

    /* global currencyFilter: true,
     dateFilter: true,
     filterFilter: true,
     jsonFilter: true,
     limitToFilter: true,
     lowercaseFilter: true,
     numberFilter: true,
     orderByFilter: true,
     uppercaseFilter: true,
     */

    /**
     * @ngdoc provider
     * @name $filterProvider
     * @description
     *
     * Filters are just functions which transform input to an output. However filters need to be
     * Dependency Injected. To achieve this a filter definition consists of a factory function which is
     * annotated with dependencies and is responsible for creating a filter function.
     *
     * <div class="alert alert-warning">
     * **Note:** Filter names must be valid angular {@link expression} identifiers, such as `uppercase` or `orderBy`.
     * Names with special characters, such as hyphens and dots, are not allowed. If you wish to namespace
     * your filters, then you can use capitalization (`myappSubsectionFilterx`) or underscores
     * (`myapp_subsection_filterx`).
     * </div>
     *
     * ```js
     *   // Filter registration
     *   function MyModule($provide, $filterProvider) {
     *     // create a service to demonstrate injection (not always needed)
     *     $provide.value('greet', function(name){
     *       return 'Hello ' + name + '!';
     *     });
     *
     *     // register a filter factory which uses the
     *     // greet service to demonstrate DI.
     *     $filterProvider.register('greet', function(greet){
     *       // return the filter function which uses the greet service
     *       // to generate salutation
     *       return function(text) {
     *         // filters need to be forgiving so check input validity
     *         return text && greet(text) || text;
     *       };
     *     });
     *   }
     * ```
     *
     * The filter function is registered with the `$injector` under the filter name suffix with
     * `Filter`.
     *
     * ```js
     *   it('should be the same instance', inject(
     *     function($filterProvider) {
     *       $filterProvider.register('reverse', function(){
     *         return ...;
     *       });
     *     },
     *     function($filter, reverseFilter) {
     *       expect($filter('reverse')).toBe(reverseFilter);
     *     });
     * ```
     *
     *
     * For more information about how angular filters work, and how to create your own filters, see
     * {@link guide/filter Filters} in the Angular Developer Guide.
     */

    /**
     * @ngdoc service
     * @name $filter
     * @kind function
     * @description
     * Filters are used for formatting data displayed to the user.
     *
     * They can be used in view templates, controllers or services.Angular comes
     * with a collection of [built-in filters](api/ng/filter), but it is easy to
     * define your own as well.
     *
     * The general syntax in templates is as follows:
     *
     * ```html
     * {{ expression [| filter_name[:parameter_value] ... ] }}
     * ```
     *
     * @param {String} name Name of the filter function to retrieve
     * @return {Function} the filter function
     * @example
       <example name="$filter" module="filterExample">
         <file name="index.html">
           <div ng-controller="MainCtrl">
            <h3>{{ originalText }}</h3>
            <h3>{{ filteredText }}</h3>
           </div>
         </file>
    
         <file name="script.js">
          angular.module('filterExample', [])
          .controller('MainCtrl', function($scope, $filter) {
            $scope.originalText = 'hello';
            $scope.filteredText = $filter('uppercase')($scope.originalText);
          });
         </file>
       </example>
      */
    $FilterProvider.$inject = ['$provide'];
    /** @this */
    function $FilterProvider($provide) {
      var suffix = 'Filter';

      /**
       * @ngdoc method
       * @name $filterProvider#register
       * @param {string|Object} name Name of the filter function, or an object map of filters where
       *    the keys are the filter names and the values are the filter factories.
       *
       *    <div class="alert alert-warning">
       *    **Note:** Filter names must be valid angular {@link expression} identifiers, such as `uppercase` or `orderBy`.
       *    Names with special characters, such as hyphens and dots, are not allowed. If you wish to namespace
       *    your filters, then you can use capitalization (`myappSubsectionFilterx`) or underscores
       *    (`myapp_subsection_filterx`).
       *    </div>
        * @param {Function} factory If the first argument was a string, a factory function for the filter to be registered.
       * @returns {Object} Registered filter instance, or if a map of filters was provided then a map
       *    of the registered filter instances.
       */
      function register(name, factory) {
        if (isObject(name)) {
          var filters = {};
          forEach(name, function (filter, key) {
            filters[key] = register(key, filter);
          });
          return filters;
        } else {
          return $provide.factory(name + suffix, factory);
        }
      }
      this.register = register;

      this.$get = ['$injector', function ($injector) {
        return function (name) {
          return $injector.get(name + suffix);
        };
      }];

      ////////////////////////////////////////

      /* global
        currencyFilter: false,
        dateFilter: false,
        filterFilter: false,
        jsonFilter: false,
        limitToFilter: false,
        lowercaseFilter: false,
        numberFilter: false,
        orderByFilter: false,
        uppercaseFilter: false
      */

      register('currency', currencyFilter);
      register('date', dateFilter);
      register('filter', filterFilter);
      register('json', jsonFilter);
      register('limitTo', limitToFilter);
      register('lowercase', lowercaseFilter);
      register('number', numberFilter);
      register('orderBy', orderByFilter);
      register('uppercase', uppercaseFilter);
    }

    /**
     * @ngdoc filter
     * @name filter
     * @kind function
     *
     * @description
     * Selects a subset of items from `array` and returns it as a new array.
     *
     * @param {Array} array The source array.
     * <div class="alert alert-info">
     *   **Note**: If the array contains objects that reference themselves, filtering is not possible.
     * </div>
     * @param {string|Object|function()} expression The predicate to be used for selecting items from
     *   `array`.
     *
     *   Can be one of:
     *
     *   - `string`: The string is used for matching against the contents of the `array`. All strings or
     *     objects with string properties in `array` that match this string will be returned. This also
     *     applies to nested object properties.
     *     The predicate can be negated by prefixing the string with `!`.
     *
     *   - `Object`: A pattern object can be used to filter specific properties on objects contained
     *     by `array`. For example `{name:"M", phone:"1"}` predicate will return an array of items
     *     which have property `name` containing "M" and property `phone` containing "1". A special
     *     property name (`$` by default) can be used (e.g. as in `{$: "text"}`) to accept a match
     *     against any property of the object or its nested object properties. That's equivalent to the
     *     simple substring match with a `string` as described above. The special property name can be
     *     overwritten, using the `anyPropertyKey` parameter.
     *     The predicate can be negated by prefixing the string with `!`.
     *     For example `{name: "!M"}` predicate will return an array of items which have property `name`
     *     not containing "M".
     *
     *     Note that a named property will match properties on the same level only, while the special
     *     `$` property will match properties on the same level or deeper. E.g. an array item like
     *     `{name: {first: 'John', last: 'Doe'}}` will **not** be matched by `{name: 'John'}`, but
     *     **will** be matched by `{$: 'John'}`.
     *
     *   - `function(value, index, array)`: A predicate function can be used to write arbitrary filters.
     *     The function is called for each element of the array, with the element, its index, and
     *     the entire array itself as arguments.
     *
     *     The final result is an array of those elements that the predicate returned true for.
     *
     * @param {function(actual, expected)|true|false} [comparator] Comparator which is used in
     *     determining if the expected value (from the filter expression) and actual value (from
     *     the object in the array) should be considered a match.
     *
     *   Can be one of:
     *
     *   - `function(actual, expected)`:
     *     The function will be given the object value and the predicate value to compare and
     *     should return true if both values should be considered equal.
     *
     *   - `true`: A shorthand for `function(actual, expected) { return angular.equals(actual, expected)}`.
     *     This is essentially strict comparison of expected and actual.
     *
     *   - `false`: A short hand for a function which will look for a substring match in a case
     *     insensitive way. Primitive values are converted to strings. Objects are not compared against
     *     primitives, unless they have a custom `toString` method (e.g. `Date` objects).
     *
     *
     *   Defaults to `false`.
     *
     * @param {string} [anyPropertyKey] The special property name that matches against any property.
     *     By default `$`.
     *
     * @example
       <example name="filter-filter">
         <file name="index.html">
           <div ng-init="friends = [{name:'John', phone:'555-1276'},
                                    {name:'Mary', phone:'800-BIG-MARY'},
                                    {name:'Mike', phone:'555-4321'},
                                    {name:'Adam', phone:'555-5678'},
                                    {name:'Julie', phone:'555-8765'},
                                    {name:'Juliette', phone:'555-5678'}]"></div>
    
           <label>Search: <input ng-model="searchText"></label>
           <table id="searchTextResults">
             <tr><th>Name</th><th>Phone</th></tr>
             <tr ng-repeat="friend in friends | filter:searchText">
               <td>{{friend.name}}</td>
               <td>{{friend.phone}}</td>
             </tr>
           </table>
           <hr>
           <label>Any: <input ng-model="search.$"></label> <br>
           <label>Name only <input ng-model="search.name"></label><br>
           <label>Phone only <input ng-model="search.phone"></label><br>
           <label>Equality <input type="checkbox" ng-model="strict"></label><br>
           <table id="searchObjResults">
             <tr><th>Name</th><th>Phone</th></tr>
             <tr ng-repeat="friendObj in friends | filter:search:strict">
               <td>{{friendObj.name}}</td>
               <td>{{friendObj.phone}}</td>
             </tr>
           </table>
         </file>
         <file name="protractor.js" type="protractor">
           var expectFriendNames = function(expectedNames, key) {
             element.all(by.repeater(key + ' in friends').column(key + '.name')).then(function(arr) {
               arr.forEach(function(wd, i) {
                 expect(wd.getText()).toMatch(expectedNames[i]);
               });
             });
           };
    
           it('should search across all fields when filtering with a string', function() {
             var searchText = element(by.model('searchText'));
             searchText.clear();
             searchText.sendKeys('m');
             expectFriendNames(['Mary', 'Mike', 'Adam'], 'friend');
    
             searchText.clear();
             searchText.sendKeys('76');
             expectFriendNames(['John', 'Julie'], 'friend');
           });
    
           it('should search in specific fields when filtering with a predicate object', function() {
             var searchAny = element(by.model('search.$'));
             searchAny.clear();
             searchAny.sendKeys('i');
             expectFriendNames(['Mary', 'Mike', 'Julie', 'Juliette'], 'friendObj');
           });
           it('should use a equal comparison when comparator is true', function() {
             var searchName = element(by.model('search.name'));
             var strict = element(by.model('strict'));
             searchName.clear();
             searchName.sendKeys('Julie');
             strict.click();
             expectFriendNames(['Julie'], 'friendObj');
           });
         </file>
       </example>
     */

    function filterFilter() {
      return function (array, expression, comparator, anyPropertyKey) {
        if (!isArrayLike(array)) {
          if (array == null) {
            return array;
          } else {
            throw minErr('filter')('notarray', 'Expected array but received: {0}', array);
          }
        }

        anyPropertyKey = anyPropertyKey || '$';
        var expressionType = getTypeForFilter(expression);
        var predicateFn;
        var matchAgainstAnyProp;

        switch (expressionType) {
          case 'function':
            predicateFn = expression;
            break;
          case 'boolean':
          case 'null':
          case 'number':
          case 'string':
            matchAgainstAnyProp = true;
          // falls through
          case 'object':
            predicateFn = createPredicateFn(expression, comparator, anyPropertyKey, matchAgainstAnyProp);
            break;
          default:
            return array;
        }

        return Array.prototype.filter.call(array, predicateFn);
      };
    }

    // Helper functions for `filterFilter`
    function createPredicateFn(expression, comparator, anyPropertyKey, matchAgainstAnyProp) {
      var shouldMatchPrimitives = isObject(expression) && anyPropertyKey in expression;
      var predicateFn;

      if (comparator === true) {
        comparator = equals;
      } else if (!isFunction(comparator)) {
        comparator = function (actual, expected) {
          if (isUndefined(actual)) {
            // No substring matching against `undefined`
            return false;
          }
          if (actual === null || expected === null) {
            // No substring matching against `null`; only match against `null`
            return actual === expected;
          }
          if (isObject(expected) || isObject(actual) && !hasCustomToString(actual)) {
            // Should not compare primitives against objects, unless they have custom `toString` method
            return false;
          }

          actual = lowercase('' + actual);
          expected = lowercase('' + expected);
          return actual.indexOf(expected) !== -1;
        };
      }

      predicateFn = function (item) {
        if (shouldMatchPrimitives && !isObject(item)) {
          return deepCompare(item, expression[anyPropertyKey], comparator, anyPropertyKey, false);
        }
        return deepCompare(item, expression, comparator, anyPropertyKey, matchAgainstAnyProp);
      };

      return predicateFn;
    }

    function deepCompare(actual, expected, comparator, anyPropertyKey, matchAgainstAnyProp, dontMatchWholeObject) {
      var actualType = getTypeForFilter(actual);
      var expectedType = getTypeForFilter(expected);

      if (expectedType === 'string' && expected.charAt(0) === '!') {
        return !deepCompare(actual, expected.substring(1), comparator, anyPropertyKey, matchAgainstAnyProp);
      } else if (isArray(actual)) {
        // In case `actual` is an array, consider it a match
        // if ANY of it's items matches `expected`
        return actual.some(function (item) {
          return deepCompare(item, expected, comparator, anyPropertyKey, matchAgainstAnyProp);
        });
      }

      switch (actualType) {
        case 'object':
          var key;
          if (matchAgainstAnyProp) {
            for (key in actual) {
              // Under certain, rare, circumstances, key may not be a string and `charAt` will be undefined
              // See: https://github.com/angular/angular.js/issues/15644
              if (key.charAt && key.charAt(0) !== '$' && deepCompare(actual[key], expected, comparator, anyPropertyKey, true)) {
                return true;
              }
            }
            return dontMatchWholeObject ? false : deepCompare(actual, expected, comparator, anyPropertyKey, false);
          } else if (expectedType === 'object') {
            for (key in expected) {
              var expectedVal = expected[key];
              if (isFunction(expectedVal) || isUndefined(expectedVal)) {
                continue;
              }

              var matchAnyProperty = key === anyPropertyKey;
              var actualVal = matchAnyProperty ? actual : actual[key];
              if (!deepCompare(actualVal, expectedVal, comparator, anyPropertyKey, matchAnyProperty, matchAnyProperty)) {
                return false;
              }
            }
            return true;
          } else {
            return comparator(actual, expected);
          }
        case 'function':
          return false;
        default:
          return comparator(actual, expected);
      }
    }

    // Used for easily differentiating between `null` and actual `object`
    function getTypeForFilter(val) {
      return val === null ? 'null' : typeof val;
    }

    var MAX_DIGITS = 22;
    var DECIMAL_SEP = '.';
    var ZERO_CHAR = '0';

    /**
     * @ngdoc filter
     * @name currency
     * @kind function
     *
     * @description
     * Formats a number as a currency (ie $1,234.56). When no currency symbol is provided, default
     * symbol for current locale is used.
     *
     * @param {number} amount Input to filter.
     * @param {string=} symbol Currency symbol or identifier to be displayed.
     * @param {number=} fractionSize Number of decimal places to round the amount to, defaults to default max fraction size for current locale
     * @returns {string} Formatted number.
     *
     *
     * @example
       <example module="currencyExample" name="currency-filter">
         <file name="index.html">
           <script>
             angular.module('currencyExample', [])
               .controller('ExampleController', ['$scope', function($scope) {
                 $scope.amount = 1234.56;
               }]);
           </script>
           <div ng-controller="ExampleController">
             <input type="number" ng-model="amount" aria-label="amount"> <br>
             default currency symbol ($): <span id="currency-default">{{amount | currency}}</span><br>
             custom currency identifier (USD$): <span id="currency-custom">{{amount | currency:"USD$"}}</span><br>
             no fractions (0): <span id="currency-no-fractions">{{amount | currency:"USD$":0}}</span>
           </div>
         </file>
         <file name="protractor.js" type="protractor">
           it('should init with 1234.56', function() {
             expect(element(by.id('currency-default')).getText()).toBe('$1,234.56');
             expect(element(by.id('currency-custom')).getText()).toBe('USD$1,234.56');
             expect(element(by.id('currency-no-fractions')).getText()).toBe('USD$1,235');
           });
           it('should update', function() {
             if (browser.params.browser === 'safari') {
               // Safari does not understand the minus key. See
               // https://github.com/angular/protractor/issues/481
               return;
             }
             element(by.model('amount')).clear();
             element(by.model('amount')).sendKeys('-1234');
             expect(element(by.id('currency-default')).getText()).toBe('-$1,234.00');
             expect(element(by.id('currency-custom')).getText()).toBe('-USD$1,234.00');
             expect(element(by.id('currency-no-fractions')).getText()).toBe('-USD$1,234');
           });
         </file>
       </example>
     */
    currencyFilter.$inject = ['$locale'];
    function currencyFilter($locale) {
      var formats = $locale.NUMBER_FORMATS;
      return function (amount, currencySymbol, fractionSize) {
        if (isUndefined(currencySymbol)) {
          currencySymbol = formats.CURRENCY_SYM;
        }

        if (isUndefined(fractionSize)) {
          fractionSize = formats.PATTERNS[1].maxFrac;
        }

        // if null or undefined pass it through
        return amount == null ? amount : formatNumber(amount, formats.PATTERNS[1], formats.GROUP_SEP, formats.DECIMAL_SEP, fractionSize).replace(/\u00A4/g, currencySymbol);
      };
    }

    /**
     * @ngdoc filter
     * @name number
     * @kind function
     *
     * @description
     * Formats a number as text.
     *
     * If the input is null or undefined, it will just be returned.
     * If the input is infinite (Infinity or -Infinity), the Infinity symbol '∞' or '-∞' is returned, respectively.
     * If the input is not a number an empty string is returned.
     *
     *
     * @param {number|string} number Number to format.
     * @param {(number|string)=} fractionSize Number of decimal places to round the number to.
     * If this is not provided then the fraction size is computed from the current locale's number
     * formatting pattern. In the case of the default locale, it will be 3.
     * @returns {string} Number rounded to `fractionSize` appropriately formatted based on the current
     *                   locale (e.g., in the en_US locale it will have "." as the decimal separator and
     *                   include "," group separators after each third digit).
     *
     * @example
       <example module="numberFilterExample" name="number-filter">
         <file name="index.html">
           <script>
             angular.module('numberFilterExample', [])
               .controller('ExampleController', ['$scope', function($scope) {
                 $scope.val = 1234.56789;
               }]);
           </script>
           <div ng-controller="ExampleController">
             <label>Enter number: <input ng-model='val'></label><br>
             Default formatting: <span id='number-default'>{{val | number}}</span><br>
             No fractions: <span>{{val | number:0}}</span><br>
             Negative number: <span>{{-val | number:4}}</span>
           </div>
         </file>
         <file name="protractor.js" type="protractor">
           it('should format numbers', function() {
             expect(element(by.id('number-default')).getText()).toBe('1,234.568');
             expect(element(by.binding('val | number:0')).getText()).toBe('1,235');
             expect(element(by.binding('-val | number:4')).getText()).toBe('-1,234.5679');
           });
    
           it('should update', function() {
             element(by.model('val')).clear();
             element(by.model('val')).sendKeys('3374.333');
             expect(element(by.id('number-default')).getText()).toBe('3,374.333');
             expect(element(by.binding('val | number:0')).getText()).toBe('3,374');
             expect(element(by.binding('-val | number:4')).getText()).toBe('-3,374.3330');
          });
         </file>
       </example>
     */
    numberFilter.$inject = ['$locale'];
    function numberFilter($locale) {
      var formats = $locale.NUMBER_FORMATS;
      return function (number, fractionSize) {

        // if null or undefined pass it through
        return number == null ? number : formatNumber(number, formats.PATTERNS[0], formats.GROUP_SEP, formats.DECIMAL_SEP, fractionSize);
      };
    }

    /**
     * Parse a number (as a string) into three components that can be used
     * for formatting the number.
     *
     * (Significant bits of this parse algorithm came from https://github.com/MikeMcl/big.js/)
     *
     * @param  {string} numStr The number to parse
     * @return {object} An object describing this number, containing the following keys:
     *  - d : an array of digits containing leading zeros as necessary
     *  - i : the number of the digits in `d` that are to the left of the decimal point
     *  - e : the exponent for numbers that would need more than `MAX_DIGITS` digits in `d`
     *
     */
    function parse(numStr) {
      var exponent = 0,
          digits,
          numberOfIntegerDigits;
      var i, j, zeros;

      // Decimal point?
      if ((numberOfIntegerDigits = numStr.indexOf(DECIMAL_SEP)) > -1) {
        numStr = numStr.replace(DECIMAL_SEP, '');
      }

      // Exponential form?
      if ((i = numStr.search(/e/i)) > 0) {
        // Work out the exponent.
        if (numberOfIntegerDigits < 0) numberOfIntegerDigits = i;
        numberOfIntegerDigits += +numStr.slice(i + 1);
        numStr = numStr.substring(0, i);
      } else if (numberOfIntegerDigits < 0) {
        // There was no decimal point or exponent so it is an integer.
        numberOfIntegerDigits = numStr.length;
      }

      // Count the number of leading zeros.
      for (i = 0; numStr.charAt(i) === ZERO_CHAR; i++) {/* empty */}

      if (i === (zeros = numStr.length)) {
        // The digits are all zero.
        digits = [0];
        numberOfIntegerDigits = 1;
      } else {
        // Count the number of trailing zeros
        zeros--;
        while (numStr.charAt(zeros) === ZERO_CHAR) zeros--;

        // Trailing zeros are insignificant so ignore them
        numberOfIntegerDigits -= i;
        digits = [];
        // Convert string to array of digits without leading/trailing zeros.
        for (j = 0; i <= zeros; i++, j++) {
          digits[j] = +numStr.charAt(i);
        }
      }

      // If the number overflows the maximum allowed digits then use an exponent.
      if (numberOfIntegerDigits > MAX_DIGITS) {
        digits = digits.splice(0, MAX_DIGITS - 1);
        exponent = numberOfIntegerDigits - 1;
        numberOfIntegerDigits = 1;
      }

      return { d: digits, e: exponent, i: numberOfIntegerDigits };
    }

    /**
     * Round the parsed number to the specified number of decimal places
     * This function changed the parsedNumber in-place
     */
    function roundNumber(parsedNumber, fractionSize, minFrac, maxFrac) {
      var digits = parsedNumber.d;
      var fractionLen = digits.length - parsedNumber.i;

      // determine fractionSize if it is not specified; `+fractionSize` converts it to a number
      fractionSize = isUndefined(fractionSize) ? Math.min(Math.max(minFrac, fractionLen), maxFrac) : +fractionSize;

      // The index of the digit to where rounding is to occur
      var roundAt = fractionSize + parsedNumber.i;
      var digit = digits[roundAt];

      if (roundAt > 0) {
        // Drop fractional digits beyond `roundAt`
        digits.splice(Math.max(parsedNumber.i, roundAt));

        // Set non-fractional digits beyond `roundAt` to 0
        for (var j = roundAt; j < digits.length; j++) {
          digits[j] = 0;
        }
      } else {
        // We rounded to zero so reset the parsedNumber
        fractionLen = Math.max(0, fractionLen);
        parsedNumber.i = 1;
        digits.length = Math.max(1, roundAt = fractionSize + 1);
        digits[0] = 0;
        for (var i = 1; i < roundAt; i++) digits[i] = 0;
      }

      if (digit >= 5) {
        if (roundAt - 1 < 0) {
          for (var k = 0; k > roundAt; k--) {
            digits.unshift(0);
            parsedNumber.i++;
          }
          digits.unshift(1);
          parsedNumber.i++;
        } else {
          digits[roundAt - 1]++;
        }
      }

      // Pad out with zeros to get the required fraction length
      for (; fractionLen < Math.max(0, fractionSize); fractionLen++) digits.push(0);

      // Do any carrying, e.g. a digit was rounded up to 10
      var carry = digits.reduceRight(function (carry, d, i, digits) {
        d = d + carry;
        digits[i] = d % 10;
        return Math.floor(d / 10);
      }, 0);
      if (carry) {
        digits.unshift(carry);
        parsedNumber.i++;
      }
    }

    /**
     * Format a number into a string
     * @param  {number} number       The number to format
     * @param  {{
     *           minFrac, // the minimum number of digits required in the fraction part of the number
     *           maxFrac, // the maximum number of digits required in the fraction part of the number
     *           gSize,   // number of digits in each group of separated digits
     *           lgSize,  // number of digits in the last group of digits before the decimal separator
     *           negPre,  // the string to go in front of a negative number (e.g. `-` or `(`))
     *           posPre,  // the string to go in front of a positive number
     *           negSuf,  // the string to go after a negative number (e.g. `)`)
     *           posSuf   // the string to go after a positive number
     *         }} pattern
     * @param  {string} groupSep     The string to separate groups of number (e.g. `,`)
     * @param  {string} decimalSep   The string to act as the decimal separator (e.g. `.`)
     * @param  {[type]} fractionSize The size of the fractional part of the number
     * @return {string}              The number formatted as a string
     */
    function formatNumber(number, pattern, groupSep, decimalSep, fractionSize) {

      if (!(isString(number) || isNumber(number)) || isNaN(number)) return '';

      var isInfinity = !isFinite(number);
      var isZero = false;
      var numStr = Math.abs(number) + '',
          formattedText = '',
          parsedNumber;

      if (isInfinity) {
        formattedText = '\u221e';
      } else {
        parsedNumber = parse(numStr);

        roundNumber(parsedNumber, fractionSize, pattern.minFrac, pattern.maxFrac);

        var digits = parsedNumber.d;
        var integerLen = parsedNumber.i;
        var exponent = parsedNumber.e;
        var decimals = [];
        isZero = digits.reduce(function (isZero, d) {
          return isZero && !d;
        }, true);

        // pad zeros for small numbers
        while (integerLen < 0) {
          digits.unshift(0);
          integerLen++;
        }

        // extract decimals digits
        if (integerLen > 0) {
          decimals = digits.splice(integerLen, digits.length);
        } else {
          decimals = digits;
          digits = [0];
        }

        // format the integer digits with grouping separators
        var groups = [];
        if (digits.length >= pattern.lgSize) {
          groups.unshift(digits.splice(-pattern.lgSize, digits.length).join(''));
        }
        while (digits.length > pattern.gSize) {
          groups.unshift(digits.splice(-pattern.gSize, digits.length).join(''));
        }
        if (digits.length) {
          groups.unshift(digits.join(''));
        }
        formattedText = groups.join(groupSep);

        // append the decimal digits
        if (decimals.length) {
          formattedText += decimalSep + decimals.join('');
        }

        if (exponent) {
          formattedText += 'e+' + exponent;
        }
      }
      if (number < 0 && !isZero) {
        return pattern.negPre + formattedText + pattern.negSuf;
      } else {
        return pattern.posPre + formattedText + pattern.posSuf;
      }
    }

    function padNumber(num, digits, trim, negWrap) {
      var neg = '';
      if (num < 0 || negWrap && num <= 0) {
        if (negWrap) {
          num = -num + 1;
        } else {
          num = -num;
          neg = '-';
        }
      }
      num = '' + num;
      while (num.length < digits) num = ZERO_CHAR + num;
      if (trim) {
        num = num.substr(num.length - digits);
      }
      return neg + num;
    }

    function dateGetter(name, size, offset, trim, negWrap) {
      offset = offset || 0;
      return function (date) {
        var value = date['get' + name]();
        if (offset > 0 || value > -offset) {
          value += offset;
        }
        if (value === 0 && offset === -12) value = 12;
        return padNumber(value, size, trim, negWrap);
      };
    }

    function dateStrGetter(name, shortForm, standAlone) {
      return function (date, formats) {
        var value = date['get' + name]();
        var propPrefix = (standAlone ? 'STANDALONE' : '') + (shortForm ? 'SHORT' : '');
        var get = uppercase(propPrefix + name);

        return formats[get][value];
      };
    }

    function timeZoneGetter(date, formats, offset) {
      var zone = -1 * offset;
      var paddedZone = zone >= 0 ? '+' : '';

      paddedZone += padNumber(Math[zone > 0 ? 'floor' : 'ceil'](zone / 60), 2) + padNumber(Math.abs(zone % 60), 2);

      return paddedZone;
    }

    function getFirstThursdayOfYear(year) {
      // 0 = index of January
      var dayOfWeekOnFirst = new Date(year, 0, 1).getDay();
      // 4 = index of Thursday (+1 to account for 1st = 5)
      // 11 = index of *next* Thursday (+1 account for 1st = 12)
      return new Date(year, 0, (dayOfWeekOnFirst <= 4 ? 5 : 12) - dayOfWeekOnFirst);
    }

    function getThursdayThisWeek(datetime) {
      return new Date(datetime.getFullYear(), datetime.getMonth(),
      // 4 = index of Thursday
      datetime.getDate() + (4 - datetime.getDay()));
    }

    function weekGetter(size) {
      return function (date) {
        var firstThurs = getFirstThursdayOfYear(date.getFullYear()),
            thisThurs = getThursdayThisWeek(date);

        var diff = +thisThurs - +firstThurs,
            result = 1 + Math.round(diff / 6.048e8); // 6.048e8 ms per week

        return padNumber(result, size);
      };
    }

    function ampmGetter(date, formats) {
      return date.getHours() < 12 ? formats.AMPMS[0] : formats.AMPMS[1];
    }

    function eraGetter(date, formats) {
      return date.getFullYear() <= 0 ? formats.ERAS[0] : formats.ERAS[1];
    }

    function longEraGetter(date, formats) {
      return date.getFullYear() <= 0 ? formats.ERANAMES[0] : formats.ERANAMES[1];
    }

    var DATE_FORMATS = {
      yyyy: dateGetter('FullYear', 4, 0, false, true),
      yy: dateGetter('FullYear', 2, 0, true, true),
      y: dateGetter('FullYear', 1, 0, false, true),
      MMMM: dateStrGetter('Month'),
      MMM: dateStrGetter('Month', true),
      MM: dateGetter('Month', 2, 1),
      M: dateGetter('Month', 1, 1),
      LLLL: dateStrGetter('Month', false, true),
      dd: dateGetter('Date', 2),
      d: dateGetter('Date', 1),
      HH: dateGetter('Hours', 2),
      H: dateGetter('Hours', 1),
      hh: dateGetter('Hours', 2, -12),
      h: dateGetter('Hours', 1, -12),
      mm: dateGetter('Minutes', 2),
      m: dateGetter('Minutes', 1),
      ss: dateGetter('Seconds', 2),
      s: dateGetter('Seconds', 1),
      // while ISO 8601 requires fractions to be prefixed with `.` or `,`
      // we can be just safely rely on using `sss` since we currently don't support single or two digit fractions
      sss: dateGetter('Milliseconds', 3),
      EEEE: dateStrGetter('Day'),
      EEE: dateStrGetter('Day', true),
      a: ampmGetter,
      Z: timeZoneGetter,
      ww: weekGetter(2),
      w: weekGetter(1),
      G: eraGetter,
      GG: eraGetter,
      GGG: eraGetter,
      GGGG: longEraGetter
    };

    var DATE_FORMATS_SPLIT = /((?:[^yMLdHhmsaZEwG']+)|(?:'(?:[^']|'')*')|(?:E+|y+|M+|L+|d+|H+|h+|m+|s+|a|Z|G+|w+))(.*)/,
        NUMBER_STRING = /^-?\d+$/;

    /**
     * @ngdoc filter
     * @name date
     * @kind function
     *
     * @description
     *   Formats `date` to a string based on the requested `format`.
     *
     *   `format` string can be composed of the following elements:
     *
     *   * `'yyyy'`: 4 digit representation of year (e.g. AD 1 => 0001, AD 2010 => 2010)
     *   * `'yy'`: 2 digit representation of year, padded (00-99). (e.g. AD 2001 => 01, AD 2010 => 10)
     *   * `'y'`: 1 digit representation of year, e.g. (AD 1 => 1, AD 199 => 199)
     *   * `'MMMM'`: Month in year (January-December)
     *   * `'MMM'`: Month in year (Jan-Dec)
     *   * `'MM'`: Month in year, padded (01-12)
     *   * `'M'`: Month in year (1-12)
     *   * `'LLLL'`: Stand-alone month in year (January-December)
     *   * `'dd'`: Day in month, padded (01-31)
     *   * `'d'`: Day in month (1-31)
     *   * `'EEEE'`: Day in Week,(Sunday-Saturday)
     *   * `'EEE'`: Day in Week, (Sun-Sat)
     *   * `'HH'`: Hour in day, padded (00-23)
     *   * `'H'`: Hour in day (0-23)
     *   * `'hh'`: Hour in AM/PM, padded (01-12)
     *   * `'h'`: Hour in AM/PM, (1-12)
     *   * `'mm'`: Minute in hour, padded (00-59)
     *   * `'m'`: Minute in hour (0-59)
     *   * `'ss'`: Second in minute, padded (00-59)
     *   * `'s'`: Second in minute (0-59)
     *   * `'sss'`: Millisecond in second, padded (000-999)
     *   * `'a'`: AM/PM marker
     *   * `'Z'`: 4 digit (+sign) representation of the timezone offset (-1200-+1200)
     *   * `'ww'`: Week of year, padded (00-53). Week 01 is the week with the first Thursday of the year
     *   * `'w'`: Week of year (0-53). Week 1 is the week with the first Thursday of the year
     *   * `'G'`, `'GG'`, `'GGG'`: The abbreviated form of the era string (e.g. 'AD')
     *   * `'GGGG'`: The long form of the era string (e.g. 'Anno Domini')
     *
     *   `format` string can also be one of the following predefined
     *   {@link guide/i18n localizable formats}:
     *
     *   * `'medium'`: equivalent to `'MMM d, y h:mm:ss a'` for en_US locale
     *     (e.g. Sep 3, 2010 12:05:08 PM)
     *   * `'short'`: equivalent to `'M/d/yy h:mm a'` for en_US  locale (e.g. 9/3/10 12:05 PM)
     *   * `'fullDate'`: equivalent to `'EEEE, MMMM d, y'` for en_US  locale
     *     (e.g. Friday, September 3, 2010)
     *   * `'longDate'`: equivalent to `'MMMM d, y'` for en_US  locale (e.g. September 3, 2010)
     *   * `'mediumDate'`: equivalent to `'MMM d, y'` for en_US  locale (e.g. Sep 3, 2010)
     *   * `'shortDate'`: equivalent to `'M/d/yy'` for en_US locale (e.g. 9/3/10)
     *   * `'mediumTime'`: equivalent to `'h:mm:ss a'` for en_US locale (e.g. 12:05:08 PM)
     *   * `'shortTime'`: equivalent to `'h:mm a'` for en_US locale (e.g. 12:05 PM)
     *
     *   `format` string can contain literal values. These need to be escaped by surrounding with single quotes (e.g.
     *   `"h 'in the morning'"`). In order to output a single quote, escape it - i.e., two single quotes in a sequence
     *   (e.g. `"h 'o''clock'"`).
     *
     * @param {(Date|number|string)} date Date to format either as Date object, milliseconds (string or
     *    number) or various ISO 8601 datetime string formats (e.g. yyyy-MM-ddTHH:mm:ss.sssZ and its
     *    shorter versions like yyyy-MM-ddTHH:mmZ, yyyy-MM-dd or yyyyMMddTHHmmssZ). If no timezone is
     *    specified in the string input, the time is considered to be in the local timezone.
     * @param {string=} format Formatting rules (see Description). If not specified,
     *    `mediumDate` is used.
     * @param {string=} timezone Timezone to be used for formatting. It understands UTC/GMT and the
     *    continental US time zone abbreviations, but for general use, use a time zone offset, for
     *    example, `'+0430'` (4 hours, 30 minutes east of the Greenwich meridian)
     *    If not specified, the timezone of the browser will be used.
     * @returns {string} Formatted string or the input if input is not recognized as date/millis.
     *
     * @example
       <example name="filter-date">
         <file name="index.html">
           <span ng-non-bindable>{{1288323623006 | date:'medium'}}</span>:
               <span>{{1288323623006 | date:'medium'}}</span><br>
           <span ng-non-bindable>{{1288323623006 | date:'yyyy-MM-dd HH:mm:ss Z'}}</span>:
              <span>{{1288323623006 | date:'yyyy-MM-dd HH:mm:ss Z'}}</span><br>
           <span ng-non-bindable>{{1288323623006 | date:'MM/dd/yyyy @ h:mma'}}</span>:
              <span>{{'1288323623006' | date:'MM/dd/yyyy @ h:mma'}}</span><br>
           <span ng-non-bindable>{{1288323623006 | date:"MM/dd/yyyy 'at' h:mma"}}</span>:
              <span>{{'1288323623006' | date:"MM/dd/yyyy 'at' h:mma"}}</span><br>
         </file>
         <file name="protractor.js" type="protractor">
           it('should format date', function() {
             expect(element(by.binding("1288323623006 | date:'medium'")).getText()).
                toMatch(/Oct 2\d, 2010 \d{1,2}:\d{2}:\d{2} (AM|PM)/);
             expect(element(by.binding("1288323623006 | date:'yyyy-MM-dd HH:mm:ss Z'")).getText()).
                toMatch(/2010-10-2\d \d{2}:\d{2}:\d{2} (-|\+)?\d{4}/);
             expect(element(by.binding("'1288323623006' | date:'MM/dd/yyyy @ h:mma'")).getText()).
                toMatch(/10\/2\d\/2010 @ \d{1,2}:\d{2}(AM|PM)/);
             expect(element(by.binding("'1288323623006' | date:\"MM/dd/yyyy 'at' h:mma\"")).getText()).
                toMatch(/10\/2\d\/2010 at \d{1,2}:\d{2}(AM|PM)/);
           });
         </file>
       </example>
     */
    dateFilter.$inject = ['$locale'];
    function dateFilter($locale) {

      var R_ISO8601_STR = /^(\d{4})-?(\d\d)-?(\d\d)(?:T(\d\d)(?::?(\d\d)(?::?(\d\d)(?:\.(\d+))?)?)?(Z|([+-])(\d\d):?(\d\d))?)?$/;
      // 1        2       3         4          5          6          7          8  9     10      11
      function jsonStringToDate(string) {
        var match;
        if (match = string.match(R_ISO8601_STR)) {
          var date = new Date(0),
              tzHour = 0,
              tzMin = 0,
              dateSetter = match[8] ? date.setUTCFullYear : date.setFullYear,
              timeSetter = match[8] ? date.setUTCHours : date.setHours;

          if (match[9]) {
            tzHour = toInt(match[9] + match[10]);
            tzMin = toInt(match[9] + match[11]);
          }
          dateSetter.call(date, toInt(match[1]), toInt(match[2]) - 1, toInt(match[3]));
          var h = toInt(match[4] || 0) - tzHour;
          var m = toInt(match[5] || 0) - tzMin;
          var s = toInt(match[6] || 0);
          var ms = Math.round(parseFloat('0.' + (match[7] || 0)) * 1000);
          timeSetter.call(date, h, m, s, ms);
          return date;
        }
        return string;
      }

      return function (date, format, timezone) {
        var text = '',
            parts = [],
            fn,
            match;

        format = format || 'mediumDate';
        format = $locale.DATETIME_FORMATS[format] || format;
        if (isString(date)) {
          date = NUMBER_STRING.test(date) ? toInt(date) : jsonStringToDate(date);
        }

        if (isNumber(date)) {
          date = new Date(date);
        }

        if (!isDate(date) || !isFinite(date.getTime())) {
          return date;
        }

        while (format) {
          match = DATE_FORMATS_SPLIT.exec(format);
          if (match) {
            parts = concat(parts, match, 1);
            format = parts.pop();
          } else {
            parts.push(format);
            format = null;
          }
        }

        var dateTimezoneOffset = date.getTimezoneOffset();
        if (timezone) {
          dateTimezoneOffset = timezoneToOffset(timezone, dateTimezoneOffset);
          date = convertTimezoneToLocal(date, timezone, true);
        }
        forEach(parts, function (value) {
          fn = DATE_FORMATS[value];
          text += fn ? fn(date, $locale.DATETIME_FORMATS, dateTimezoneOffset) : value === '\'\'' ? '\'' : value.replace(/(^'|'$)/g, '').replace(/''/g, '\'');
        });

        return text;
      };
    }

    /**
     * @ngdoc filter
     * @name json
     * @kind function
     *
     * @description
     *   Allows you to convert a JavaScript object into JSON string.
     *
     *   This filter is mostly useful for debugging. When using the double curly {{value}} notation
     *   the binding is automatically converted to JSON.
     *
     * @param {*} object Any JavaScript object (including arrays and primitive types) to filter.
     * @param {number=} spacing The number of spaces to use per indentation, defaults to 2.
     * @returns {string} JSON string.
     *
     *
     * @example
       <example name="filter-json">
         <file name="index.html">
           <pre id="default-spacing">{{ {'name':'value'} | json }}</pre>
           <pre id="custom-spacing">{{ {'name':'value'} | json:4 }}</pre>
         </file>
         <file name="protractor.js" type="protractor">
           it('should jsonify filtered objects', function() {
             expect(element(by.id('default-spacing')).getText()).toMatch(/\{\n {2}"name": ?"value"\n}/);
             expect(element(by.id('custom-spacing')).getText()).toMatch(/\{\n {4}"name": ?"value"\n}/);
           });
         </file>
       </example>
     *
     */
    function jsonFilter() {
      return function (object, spacing) {
        if (isUndefined(spacing)) {
          spacing = 2;
        }
        return toJson(object, spacing);
      };
    }

    /**
     * @ngdoc filter
     * @name lowercase
     * @kind function
     * @description
     * Converts string to lowercase.
     * @see angular.lowercase
     */
    var lowercaseFilter = valueFn(lowercase);

    /**
     * @ngdoc filter
     * @name uppercase
     * @kind function
     * @description
     * Converts string to uppercase.
     * @see angular.uppercase
     */
    var uppercaseFilter = valueFn(uppercase);

    /**
     * @ngdoc filter
     * @name limitTo
     * @kind function
     *
     * @description
     * Creates a new array or string containing only a specified number of elements. The elements are
     * taken from either the beginning or the end of the source array, string or number, as specified by
     * the value and sign (positive or negative) of `limit`. Other array-like objects are also supported
     * (e.g. array subclasses, NodeLists, jqLite/jQuery collections etc). If a number is used as input,
     * it is converted to a string.
     *
     * @param {Array|ArrayLike|string|number} input - Array/array-like, string or number to be limited.
     * @param {string|number} limit - The length of the returned array or string. If the `limit` number
     *     is positive, `limit` number of items from the beginning of the source array/string are copied.
     *     If the number is negative, `limit` number  of items from the end of the source array/string
     *     are copied. The `limit` will be trimmed if it exceeds `array.length`. If `limit` is undefined,
     *     the input will be returned unchanged.
     * @param {(string|number)=} begin - Index at which to begin limitation. As a negative index,
     *     `begin` indicates an offset from the end of `input`. Defaults to `0`.
     * @returns {Array|string} A new sub-array or substring of length `limit` or less if the input had
     *     less than `limit` elements.
     *
     * @example
       <example module="limitToExample" name="limit-to-filter">
         <file name="index.html">
           <script>
             angular.module('limitToExample', [])
               .controller('ExampleController', ['$scope', function($scope) {
                 $scope.numbers = [1,2,3,4,5,6,7,8,9];
                 $scope.letters = "abcdefghi";
                 $scope.longNumber = 2345432342;
                 $scope.numLimit = 3;
                 $scope.letterLimit = 3;
                 $scope.longNumberLimit = 3;
               }]);
           </script>
           <div ng-controller="ExampleController">
             <label>
                Limit {{numbers}} to:
                <input type="number" step="1" ng-model="numLimit">
             </label>
             <p>Output numbers: {{ numbers | limitTo:numLimit }}</p>
             <label>
                Limit {{letters}} to:
                <input type="number" step="1" ng-model="letterLimit">
             </label>
             <p>Output letters: {{ letters | limitTo:letterLimit }}</p>
             <label>
                Limit {{longNumber}} to:
                <input type="number" step="1" ng-model="longNumberLimit">
             </label>
             <p>Output long number: {{ longNumber | limitTo:longNumberLimit }}</p>
           </div>
         </file>
         <file name="protractor.js" type="protractor">
           var numLimitInput = element(by.model('numLimit'));
           var letterLimitInput = element(by.model('letterLimit'));
           var longNumberLimitInput = element(by.model('longNumberLimit'));
           var limitedNumbers = element(by.binding('numbers | limitTo:numLimit'));
           var limitedLetters = element(by.binding('letters | limitTo:letterLimit'));
           var limitedLongNumber = element(by.binding('longNumber | limitTo:longNumberLimit'));
    
           it('should limit the number array to first three items', function() {
             expect(numLimitInput.getAttribute('value')).toBe('3');
             expect(letterLimitInput.getAttribute('value')).toBe('3');
             expect(longNumberLimitInput.getAttribute('value')).toBe('3');
             expect(limitedNumbers.getText()).toEqual('Output numbers: [1,2,3]');
             expect(limitedLetters.getText()).toEqual('Output letters: abc');
             expect(limitedLongNumber.getText()).toEqual('Output long number: 234');
           });
    
           // There is a bug in safari and protractor that doesn't like the minus key
           // it('should update the output when -3 is entered', function() {
           //   numLimitInput.clear();
           //   numLimitInput.sendKeys('-3');
           //   letterLimitInput.clear();
           //   letterLimitInput.sendKeys('-3');
           //   longNumberLimitInput.clear();
           //   longNumberLimitInput.sendKeys('-3');
           //   expect(limitedNumbers.getText()).toEqual('Output numbers: [7,8,9]');
           //   expect(limitedLetters.getText()).toEqual('Output letters: ghi');
           //   expect(limitedLongNumber.getText()).toEqual('Output long number: 342');
           // });
    
           it('should not exceed the maximum size of input array', function() {
             numLimitInput.clear();
             numLimitInput.sendKeys('100');
             letterLimitInput.clear();
             letterLimitInput.sendKeys('100');
             longNumberLimitInput.clear();
             longNumberLimitInput.sendKeys('100');
             expect(limitedNumbers.getText()).toEqual('Output numbers: [1,2,3,4,5,6,7,8,9]');
             expect(limitedLetters.getText()).toEqual('Output letters: abcdefghi');
             expect(limitedLongNumber.getText()).toEqual('Output long number: 2345432342');
           });
         </file>
       </example>
    */
    function limitToFilter() {
      return function (input, limit, begin) {
        if (Math.abs(Number(limit)) === Infinity) {
          limit = Number(limit);
        } else {
          limit = toInt(limit);
        }
        if (isNumberNaN(limit)) return input;

        if (isNumber(input)) input = input.toString();
        if (!isArrayLike(input)) return input;

        begin = !begin || isNaN(begin) ? 0 : toInt(begin);
        begin = begin < 0 ? Math.max(0, input.length + begin) : begin;

        if (limit >= 0) {
          return sliceFn(input, begin, begin + limit);
        } else {
          if (begin === 0) {
            return sliceFn(input, limit, input.length);
          } else {
            return sliceFn(input, Math.max(0, begin + limit), begin);
          }
        }
      };
    }

    function sliceFn(input, begin, end) {
      if (isString(input)) return input.slice(begin, end);

      return slice.call(input, begin, end);
    }

    /**
     * @ngdoc filter
     * @name orderBy
     * @kind function
     *
     * @description
     * Returns an array containing the items from the specified `collection`, ordered by a `comparator`
     * function based on the values computed using the `expression` predicate.
     *
     * For example, `[{id: 'foo'}, {id: 'bar'}] | orderBy:'id'` would result in
     * `[{id: 'bar'}, {id: 'foo'}]`.
     *
     * The `collection` can be an Array or array-like object (e.g. NodeList, jQuery object, TypedArray,
     * String, etc).
     *
     * The `expression` can be a single predicate, or a list of predicates each serving as a tie-breaker
     * for the preceding one. The `expression` is evaluated against each item and the output is used
     * for comparing with other items.
     *
     * You can change the sorting order by setting `reverse` to `true`. By default, items are sorted in
     * ascending order.
     *
     * The comparison is done using the `comparator` function. If none is specified, a default, built-in
     * comparator is used (see below for details - in a nutshell, it compares numbers numerically and
     * strings alphabetically).
     *
     * ### Under the hood
     *
     * Ordering the specified `collection` happens in two phases:
     *
     * 1. All items are passed through the predicate (or predicates), and the returned values are saved
     *    along with their type (`string`, `number` etc). For example, an item `{label: 'foo'}`, passed
     *    through a predicate that extracts the value of the `label` property, would be transformed to:
     *    ```
     *    {
     *      value: 'foo',
     *      type: 'string',
     *      index: ...
     *    }
     *    ```
     * 2. The comparator function is used to sort the items, based on the derived values, types and
     *    indices.
     *
     * If you use a custom comparator, it will be called with pairs of objects of the form
     * `{value: ..., type: '...', index: ...}` and is expected to return `0` if the objects are equal
     * (as far as the comparator is concerned), `-1` if the 1st one should be ranked higher than the
     * second, or `1` otherwise.
     *
     * In order to ensure that the sorting will be deterministic across platforms, if none of the
     * specified predicates can distinguish between two items, `orderBy` will automatically introduce a
     * dummy predicate that returns the item's index as `value`.
     * (If you are using a custom comparator, make sure it can handle this predicate as well.)
     *
     * Finally, in an attempt to simplify things, if a predicate returns an object as the extracted
     * value for an item, `orderBy` will try to convert that object to a primitive value, before passing
     * it to the comparator. The following rules govern the conversion:
     *
     * 1. If the object has a `valueOf()` method that returns a primitive, its return value will be
     *    used instead.<br />
     *    (If the object has a `valueOf()` method that returns another object, then the returned object
     *    will be used in subsequent steps.)
     * 2. If the object has a custom `toString()` method (i.e. not the one inherited from `Object`) that
     *    returns a primitive, its return value will be used instead.<br />
     *    (If the object has a `toString()` method that returns another object, then the returned object
     *    will be used in subsequent steps.)
     * 3. No conversion; the object itself is used.
     *
     * ### The default comparator
     *
     * The default, built-in comparator should be sufficient for most usecases. In short, it compares
     * numbers numerically, strings alphabetically (and case-insensitively), for objects falls back to
     * using their index in the original collection, and sorts values of different types by type.
     *
     * More specifically, it follows these steps to determine the relative order of items:
     *
     * 1. If the compared values are of different types, compare the types themselves alphabetically.
     * 2. If both values are of type `string`, compare them alphabetically in a case- and
     *    locale-insensitive way.
     * 3. If both values are objects, compare their indices instead.
     * 4. Otherwise, return:
     *    -  `0`, if the values are equal (by strict equality comparison, i.e. using `===`).
     *    - `-1`, if the 1st value is "less than" the 2nd value (compared using the `<` operator).
     *    -  `1`, otherwise.
     *
     * **Note:** If you notice numbers not being sorted as expected, make sure they are actually being
     *           saved as numbers and not strings.
     * **Note:** For the purpose of sorting, `null` values are treated as the string `'null'` (i.e.
     *           `type: 'string'`, `value: 'null'`). This may cause unexpected sort order relative to
     *           other values.
     *
     * @param {Array|ArrayLike} collection - The collection (array or array-like object) to sort.
     * @param {(Function|string|Array.<Function|string>)=} expression - A predicate (or list of
     *    predicates) to be used by the comparator to determine the order of elements.
     *
     *    Can be one of:
     *
     *    - `Function`: A getter function. This function will be called with each item as argument and
     *      the return value will be used for sorting.
     *    - `string`: An Angular expression. This expression will be evaluated against each item and the
     *      result will be used for sorting. For example, use `'label'` to sort by a property called
     *      `label` or `'label.substring(0, 3)'` to sort by the first 3 characters of the `label`
     *      property.<br />
     *      (The result of a constant expression is interpreted as a property name to be used for
     *      comparison. For example, use `'"special name"'` (note the extra pair of quotes) to sort by a
     *      property called `special name`.)<br />
     *      An expression can be optionally prefixed with `+` or `-` to control the sorting direction,
     *      ascending or descending. For example, `'+label'` or `'-label'`. If no property is provided,
     *      (e.g. `'+'` or `'-'`), the collection element itself is used in comparisons.
     *    - `Array`: An array of function and/or string predicates. If a predicate cannot determine the
     *      relative order of two items, the next predicate is used as a tie-breaker.
     *
     * **Note:** If the predicate is missing or empty then it defaults to `'+'`.
     *
     * @param {boolean=} reverse - If `true`, reverse the sorting order.
     * @param {(Function)=} comparator - The comparator function used to determine the relative order of
     *    value pairs. If omitted, the built-in comparator will be used.
     *
     * @returns {Array} - The sorted array.
     *
     *
     * @example
     * ### Ordering a table with `ngRepeat`
     *
     * The example below demonstrates a simple {@link ngRepeat ngRepeat}, where the data is sorted by
     * age in descending order (expression is set to `'-age'`). The `comparator` is not set, which means
     * it defaults to the built-in comparator.
     *
       <example name="orderBy-static" module="orderByExample1">
         <file name="index.html">
           <div ng-controller="ExampleController">
             <table class="friends">
               <tr>
                 <th>Name</th>
                 <th>Phone Number</th>
                 <th>Age</th>
               </tr>
               <tr ng-repeat="friend in friends | orderBy:'-age'">
                 <td>{{friend.name}}</td>
                 <td>{{friend.phone}}</td>
                 <td>{{friend.age}}</td>
               </tr>
             </table>
           </div>
         </file>
         <file name="script.js">
           angular.module('orderByExample1', [])
             .controller('ExampleController', ['$scope', function($scope) {
               $scope.friends = [
                 {name: 'John',   phone: '555-1212',  age: 10},
                 {name: 'Mary',   phone: '555-9876',  age: 19},
                 {name: 'Mike',   phone: '555-4321',  age: 21},
                 {name: 'Adam',   phone: '555-5678',  age: 35},
                 {name: 'Julie',  phone: '555-8765',  age: 29}
               ];
             }]);
         </file>
         <file name="style.css">
           .friends {
             border-collapse: collapse;
           }
    
           .friends th {
             border-bottom: 1px solid;
           }
           .friends td, .friends th {
             border-left: 1px solid;
             padding: 5px 10px;
           }
           .friends td:first-child, .friends th:first-child {
             border-left: none;
           }
         </file>
         <file name="protractor.js" type="protractor">
           // Element locators
           var names = element.all(by.repeater('friends').column('friend.name'));
    
           it('should sort friends by age in reverse order', function() {
             expect(names.get(0).getText()).toBe('Adam');
             expect(names.get(1).getText()).toBe('Julie');
             expect(names.get(2).getText()).toBe('Mike');
             expect(names.get(3).getText()).toBe('Mary');
             expect(names.get(4).getText()).toBe('John');
           });
         </file>
       </example>
     * <hr />
     *
     * @example
     * ### Changing parameters dynamically
     *
     * All parameters can be changed dynamically. The next example shows how you can make the columns of
     * a table sortable, by binding the `expression` and `reverse` parameters to scope properties.
     *
       <example name="orderBy-dynamic" module="orderByExample2">
         <file name="index.html">
           <div ng-controller="ExampleController">
             <pre>Sort by = {{propertyName}}; reverse = {{reverse}}</pre>
             <hr/>
             <button ng-click="propertyName = null; reverse = false">Set to unsorted</button>
             <hr/>
             <table class="friends">
               <tr>
                 <th>
                   <button ng-click="sortBy('name')">Name</button>
                   <span class="sortorder" ng-show="propertyName === 'name'" ng-class="{reverse: reverse}"></span>
                 </th>
                 <th>
                   <button ng-click="sortBy('phone')">Phone Number</button>
                   <span class="sortorder" ng-show="propertyName === 'phone'" ng-class="{reverse: reverse}"></span>
                 </th>
                 <th>
                   <button ng-click="sortBy('age')">Age</button>
                   <span class="sortorder" ng-show="propertyName === 'age'" ng-class="{reverse: reverse}"></span>
                 </th>
               </tr>
               <tr ng-repeat="friend in friends | orderBy:propertyName:reverse">
                 <td>{{friend.name}}</td>
                 <td>{{friend.phone}}</td>
                 <td>{{friend.age}}</td>
               </tr>
             </table>
           </div>
         </file>
         <file name="script.js">
           angular.module('orderByExample2', [])
             .controller('ExampleController', ['$scope', function($scope) {
               var friends = [
                 {name: 'John',   phone: '555-1212',  age: 10},
                 {name: 'Mary',   phone: '555-9876',  age: 19},
                 {name: 'Mike',   phone: '555-4321',  age: 21},
                 {name: 'Adam',   phone: '555-5678',  age: 35},
                 {name: 'Julie',  phone: '555-8765',  age: 29}
               ];
    
               $scope.propertyName = 'age';
               $scope.reverse = true;
               $scope.friends = friends;
    
               $scope.sortBy = function(propertyName) {
                 $scope.reverse = ($scope.propertyName === propertyName) ? !$scope.reverse : false;
                 $scope.propertyName = propertyName;
               };
             }]);
         </file>
         <file name="style.css">
           .friends {
             border-collapse: collapse;
           }
    
           .friends th {
             border-bottom: 1px solid;
           }
           .friends td, .friends th {
             border-left: 1px solid;
             padding: 5px 10px;
           }
           .friends td:first-child, .friends th:first-child {
             border-left: none;
           }
    
           .sortorder:after {
             content: '\25b2';   // BLACK UP-POINTING TRIANGLE
           }
           .sortorder.reverse:after {
             content: '\25bc';   // BLACK DOWN-POINTING TRIANGLE
           }
         </file>
         <file name="protractor.js" type="protractor">
           // Element locators
           var unsortButton = element(by.partialButtonText('unsorted'));
           var nameHeader = element(by.partialButtonText('Name'));
           var phoneHeader = element(by.partialButtonText('Phone'));
           var ageHeader = element(by.partialButtonText('Age'));
           var firstName = element(by.repeater('friends').column('friend.name').row(0));
           var lastName = element(by.repeater('friends').column('friend.name').row(4));
    
           it('should sort friends by some property, when clicking on the column header', function() {
             expect(firstName.getText()).toBe('Adam');
             expect(lastName.getText()).toBe('John');
    
             phoneHeader.click();
             expect(firstName.getText()).toBe('John');
             expect(lastName.getText()).toBe('Mary');
    
             nameHeader.click();
             expect(firstName.getText()).toBe('Adam');
             expect(lastName.getText()).toBe('Mike');
    
             ageHeader.click();
             expect(firstName.getText()).toBe('John');
             expect(lastName.getText()).toBe('Adam');
           });
    
           it('should sort friends in reverse order, when clicking on the same column', function() {
             expect(firstName.getText()).toBe('Adam');
             expect(lastName.getText()).toBe('John');
    
             ageHeader.click();
             expect(firstName.getText()).toBe('John');
             expect(lastName.getText()).toBe('Adam');
    
             ageHeader.click();
             expect(firstName.getText()).toBe('Adam');
             expect(lastName.getText()).toBe('John');
           });
    
           it('should restore the original order, when clicking "Set to unsorted"', function() {
             expect(firstName.getText()).toBe('Adam');
             expect(lastName.getText()).toBe('John');
    
             unsortButton.click();
             expect(firstName.getText()).toBe('John');
             expect(lastName.getText()).toBe('Julie');
           });
         </file>
       </example>
     * <hr />
     *
     * @example
     * ### Using `orderBy` inside a controller
     *
     * It is also possible to call the `orderBy` filter manually, by injecting `orderByFilter`, and
     * calling it with the desired parameters. (Alternatively, you could inject the `$filter` factory
     * and retrieve the `orderBy` filter with `$filter('orderBy')`.)
     *
       <example name="orderBy-call-manually" module="orderByExample3">
         <file name="index.html">
           <div ng-controller="ExampleController">
             <pre>Sort by = {{propertyName}}; reverse = {{reverse}}</pre>
             <hr/>
             <button ng-click="sortBy(null)">Set to unsorted</button>
             <hr/>
             <table class="friends">
               <tr>
                 <th>
                   <button ng-click="sortBy('name')">Name</button>
                   <span class="sortorder" ng-show="propertyName === 'name'" ng-class="{reverse: reverse}"></span>
                 </th>
                 <th>
                   <button ng-click="sortBy('phone')">Phone Number</button>
                   <span class="sortorder" ng-show="propertyName === 'phone'" ng-class="{reverse: reverse}"></span>
                 </th>
                 <th>
                   <button ng-click="sortBy('age')">Age</button>
                   <span class="sortorder" ng-show="propertyName === 'age'" ng-class="{reverse: reverse}"></span>
                 </th>
               </tr>
               <tr ng-repeat="friend in friends">
                 <td>{{friend.name}}</td>
                 <td>{{friend.phone}}</td>
                 <td>{{friend.age}}</td>
               </tr>
             </table>
           </div>
         </file>
         <file name="script.js">
           angular.module('orderByExample3', [])
             .controller('ExampleController', ['$scope', 'orderByFilter', function($scope, orderBy) {
               var friends = [
                 {name: 'John',   phone: '555-1212',  age: 10},
                 {name: 'Mary',   phone: '555-9876',  age: 19},
                 {name: 'Mike',   phone: '555-4321',  age: 21},
                 {name: 'Adam',   phone: '555-5678',  age: 35},
                 {name: 'Julie',  phone: '555-8765',  age: 29}
               ];
    
               $scope.propertyName = 'age';
               $scope.reverse = true;
               $scope.friends = orderBy(friends, $scope.propertyName, $scope.reverse);
    
               $scope.sortBy = function(propertyName) {
                 $scope.reverse = (propertyName !== null && $scope.propertyName === propertyName)
                     ? !$scope.reverse : false;
                 $scope.propertyName = propertyName;
                 $scope.friends = orderBy(friends, $scope.propertyName, $scope.reverse);
               };
             }]);
         </file>
         <file name="style.css">
           .friends {
             border-collapse: collapse;
           }
    
           .friends th {
             border-bottom: 1px solid;
           }
           .friends td, .friends th {
             border-left: 1px solid;
             padding: 5px 10px;
           }
           .friends td:first-child, .friends th:first-child {
             border-left: none;
           }
    
           .sortorder:after {
             content: '\25b2';   // BLACK UP-POINTING TRIANGLE
           }
           .sortorder.reverse:after {
             content: '\25bc';   // BLACK DOWN-POINTING TRIANGLE
           }
         </file>
         <file name="protractor.js" type="protractor">
           // Element locators
           var unsortButton = element(by.partialButtonText('unsorted'));
           var nameHeader = element(by.partialButtonText('Name'));
           var phoneHeader = element(by.partialButtonText('Phone'));
           var ageHeader = element(by.partialButtonText('Age'));
           var firstName = element(by.repeater('friends').column('friend.name').row(0));
           var lastName = element(by.repeater('friends').column('friend.name').row(4));
    
           it('should sort friends by some property, when clicking on the column header', function() {
             expect(firstName.getText()).toBe('Adam');
             expect(lastName.getText()).toBe('John');
    
             phoneHeader.click();
             expect(firstName.getText()).toBe('John');
             expect(lastName.getText()).toBe('Mary');
    
             nameHeader.click();
             expect(firstName.getText()).toBe('Adam');
             expect(lastName.getText()).toBe('Mike');
    
             ageHeader.click();
             expect(firstName.getText()).toBe('John');
             expect(lastName.getText()).toBe('Adam');
           });
    
           it('should sort friends in reverse order, when clicking on the same column', function() {
             expect(firstName.getText()).toBe('Adam');
             expect(lastName.getText()).toBe('John');
    
             ageHeader.click();
             expect(firstName.getText()).toBe('John');
             expect(lastName.getText()).toBe('Adam');
    
             ageHeader.click();
             expect(firstName.getText()).toBe('Adam');
             expect(lastName.getText()).toBe('John');
           });
    
           it('should restore the original order, when clicking "Set to unsorted"', function() {
             expect(firstName.getText()).toBe('Adam');
             expect(lastName.getText()).toBe('John');
    
             unsortButton.click();
             expect(firstName.getText()).toBe('John');
             expect(lastName.getText()).toBe('Julie');
           });
         </file>
       </example>
     * <hr />
     *
     * @example
     * ### Using a custom comparator
     *
     * If you have very specific requirements about the way items are sorted, you can pass your own
     * comparator function. For example, you might need to compare some strings in a locale-sensitive
     * way. (When specifying a custom comparator, you also need to pass a value for the `reverse`
     * argument - passing `false` retains the default sorting order, i.e. ascending.)
     *
       <example name="orderBy-custom-comparator" module="orderByExample4">
         <file name="index.html">
           <div ng-controller="ExampleController">
             <div class="friends-container custom-comparator">
               <h3>Locale-sensitive Comparator</h3>
               <table class="friends">
                 <tr>
                   <th>Name</th>
                   <th>Favorite Letter</th>
                 </tr>
                 <tr ng-repeat="friend in friends | orderBy:'favoriteLetter':false:localeSensitiveComparator">
                   <td>{{friend.name}}</td>
                   <td>{{friend.favoriteLetter}}</td>
                 </tr>
               </table>
             </div>
             <div class="friends-container default-comparator">
               <h3>Default Comparator</h3>
               <table class="friends">
                 <tr>
                   <th>Name</th>
                   <th>Favorite Letter</th>
                 </tr>
                 <tr ng-repeat="friend in friends | orderBy:'favoriteLetter'">
                   <td>{{friend.name}}</td>
                   <td>{{friend.favoriteLetter}}</td>
                 </tr>
               </table>
             </div>
           </div>
         </file>
         <file name="script.js">
           angular.module('orderByExample4', [])
             .controller('ExampleController', ['$scope', function($scope) {
               $scope.friends = [
                 {name: 'John',   favoriteLetter: 'Ä'},
                 {name: 'Mary',   favoriteLetter: 'Ü'},
                 {name: 'Mike',   favoriteLetter: 'Ö'},
                 {name: 'Adam',   favoriteLetter: 'H'},
                 {name: 'Julie',  favoriteLetter: 'Z'}
               ];
    
               $scope.localeSensitiveComparator = function(v1, v2) {
                 // If we don't get strings, just compare by index
                 if (v1.type !== 'string' || v2.type !== 'string') {
                   return (v1.index < v2.index) ? -1 : 1;
                 }
    
                 // Compare strings alphabetically, taking locale into account
                 return v1.value.localeCompare(v2.value);
               };
             }]);
         </file>
         <file name="style.css">
           .friends-container {
             display: inline-block;
             margin: 0 30px;
           }
    
           .friends {
             border-collapse: collapse;
           }
    
           .friends th {
             border-bottom: 1px solid;
           }
           .friends td, .friends th {
             border-left: 1px solid;
             padding: 5px 10px;
           }
           .friends td:first-child, .friends th:first-child {
             border-left: none;
           }
         </file>
         <file name="protractor.js" type="protractor">
           // Element locators
           var container = element(by.css('.custom-comparator'));
           var names = container.all(by.repeater('friends').column('friend.name'));
    
           it('should sort friends by favorite letter (in correct alphabetical order)', function() {
             expect(names.get(0).getText()).toBe('John');
             expect(names.get(1).getText()).toBe('Adam');
             expect(names.get(2).getText()).toBe('Mike');
             expect(names.get(3).getText()).toBe('Mary');
             expect(names.get(4).getText()).toBe('Julie');
           });
         </file>
       </example>
     *
     */
    orderByFilter.$inject = ['$parse'];
    function orderByFilter($parse) {
      return function (array, sortPredicate, reverseOrder, compareFn) {

        if (array == null) return array;
        if (!isArrayLike(array)) {
          throw minErr('orderBy')('notarray', 'Expected array but received: {0}', array);
        }

        if (!isArray(sortPredicate)) {
          sortPredicate = [sortPredicate];
        }
        if (sortPredicate.length === 0) {
          sortPredicate = ['+'];
        }

        var predicates = processPredicates(sortPredicate);

        var descending = reverseOrder ? -1 : 1;

        // Define the `compare()` function. Use a default comparator if none is specified.
        var compare = isFunction(compareFn) ? compareFn : defaultCompare;

        // The next three lines are a version of a Swartzian Transform idiom from Perl
        // (sometimes called the Decorate-Sort-Undecorate idiom)
        // See https://en.wikipedia.org/wiki/Schwartzian_transform
        var compareValues = Array.prototype.map.call(array, getComparisonObject);
        compareValues.sort(doComparison);
        array = compareValues.map(function (item) {
          return item.value;
        });

        return array;

        function getComparisonObject(value, index) {
          // NOTE: We are adding an extra `tieBreaker` value based on the element's index.
          // This will be used to keep the sort stable when none of the input predicates can
          // distinguish between two elements.
          return {
            value: value,
            tieBreaker: { value: index, type: 'number', index: index },
            predicateValues: predicates.map(function (predicate) {
              return getPredicateValue(predicate.get(value), index);
            })
          };
        }

        function doComparison(v1, v2) {
          for (var i = 0, ii = predicates.length; i < ii; i++) {
            var result = compare(v1.predicateValues[i], v2.predicateValues[i]);
            if (result) {
              return result * predicates[i].descending * descending;
            }
          }

          return compare(v1.tieBreaker, v2.tieBreaker) * descending;
        }
      };

      function processPredicates(sortPredicates) {
        return sortPredicates.map(function (predicate) {
          var descending = 1,
              get = identity;

          if (isFunction(predicate)) {
            get = predicate;
          } else if (isString(predicate)) {
            if (predicate.charAt(0) === '+' || predicate.charAt(0) === '-') {
              descending = predicate.charAt(0) === '-' ? -1 : 1;
              predicate = predicate.substring(1);
            }
            if (predicate !== '') {
              get = $parse(predicate);
              if (get.constant) {
                var key = get();
                get = function (value) {
                  return value[key];
                };
              }
            }
          }
          return { get: get, descending: descending };
        });
      }

      function isPrimitive(value) {
        switch (typeof value) {
          case 'number': /* falls through */
          case 'boolean': /* falls through */
          case 'string':
            return true;
          default:
            return false;
        }
      }

      function objectValue(value) {
        // If `valueOf` is a valid function use that
        if (isFunction(value.valueOf)) {
          value = value.valueOf();
          if (isPrimitive(value)) return value;
        }
        // If `toString` is a valid function and not the one from `Object.prototype` use that
        if (hasCustomToString(value)) {
          value = value.toString();
          if (isPrimitive(value)) return value;
        }

        return value;
      }

      function getPredicateValue(value, index) {
        var type = typeof value;
        if (value === null) {
          type = 'string';
          value = 'null';
        } else if (type === 'object') {
          value = objectValue(value);
        }
        return { value: value, type: type, index: index };
      }

      function defaultCompare(v1, v2) {
        var result = 0;
        var type1 = v1.type;
        var type2 = v2.type;

        if (type1 === type2) {
          var value1 = v1.value;
          var value2 = v2.value;

          if (type1 === 'string') {
            // Compare strings case-insensitively
            value1 = value1.toLowerCase();
            value2 = value2.toLowerCase();
          } else if (type1 === 'object') {
            // For basic objects, use the position of the object
            // in the collection instead of the value
            if (isObject(value1)) value1 = v1.index;
            if (isObject(value2)) value2 = v2.index;
          }

          if (value1 !== value2) {
            result = value1 < value2 ? -1 : 1;
          }
        } else {
          result = type1 < type2 ? -1 : 1;
        }

        return result;
      }
    }

    function ngDirective(directive) {
      if (isFunction(directive)) {
        directive = {
          link: directive
        };
      }
      directive.restrict = directive.restrict || 'AC';
      return valueFn(directive);
    }

    /**
     * @ngdoc directive
     * @name a
     * @restrict E
     *
     * @description
     * Modifies the default behavior of the html a tag so that the default action is prevented when
     * the href attribute is empty.
     *
     * For dynamically creating `href` attributes for a tags, see the {@link ng.ngHref `ngHref`} directive.
     */
    var htmlAnchorDirective = valueFn({
      restrict: 'E',
      compile: function (element, attr) {
        if (!attr.href && !attr.xlinkHref) {
          return function (scope, element) {
            // If the linked element is not an anchor tag anymore, do nothing
            if (element[0].nodeName.toLowerCase() !== 'a') return;

            // SVGAElement does not use the href attribute, but rather the 'xlinkHref' attribute.
            var href = toString.call(element.prop('href')) === '[object SVGAnimatedString]' ? 'xlink:href' : 'href';
            element.on('click', function (event) {
              // if we have no href url, then don't navigate anywhere.
              if (!element.attr(href)) {
                event.preventDefault();
              }
            });
          };
        }
      }
    });

    /**
     * @ngdoc directive
     * @name ngHref
     * @restrict A
     * @priority 99
     *
     * @description
     * Using Angular markup like `{{hash}}` in an href attribute will
     * make the link go to the wrong URL if the user clicks it before
     * Angular has a chance to replace the `{{hash}}` markup with its
     * value. Until Angular replaces the markup the link will be broken
     * and will most likely return a 404 error. The `ngHref` directive
     * solves this problem.
     *
     * The wrong way to write it:
     * ```html
     * <a href="http://www.gravatar.com/avatar/{{hash}}">link1</a>
     * ```
     *
     * The correct way to write it:
     * ```html
     * <a ng-href="http://www.gravatar.com/avatar/{{hash}}">link1</a>
     * ```
     *
     * @element A
     * @param {template} ngHref any string which can contain `{{}}` markup.
     *
     * @example
     * This example shows various combinations of `href`, `ng-href` and `ng-click` attributes
     * in links and their different behaviors:
        <example name="ng-href">
          <file name="index.html">
            <input ng-model="value" /><br />
            <a id="link-1" href ng-click="value = 1">link 1</a> (link, don't reload)<br />
            <a id="link-2" href="" ng-click="value = 2">link 2</a> (link, don't reload)<br />
            <a id="link-3" ng-href="/{{'123'}}">link 3</a> (link, reload!)<br />
            <a id="link-4" href="" name="xx" ng-click="value = 4">anchor</a> (link, don't reload)<br />
            <a id="link-5" name="xxx" ng-click="value = 5">anchor</a> (no link)<br />
            <a id="link-6" ng-href="{{value}}">link</a> (link, change location)
          </file>
          <file name="protractor.js" type="protractor">
            it('should execute ng-click but not reload when href without value', function() {
              element(by.id('link-1')).click();
              expect(element(by.model('value')).getAttribute('value')).toEqual('1');
              expect(element(by.id('link-1')).getAttribute('href')).toBe('');
            });
    
            it('should execute ng-click but not reload when href empty string', function() {
              element(by.id('link-2')).click();
              expect(element(by.model('value')).getAttribute('value')).toEqual('2');
              expect(element(by.id('link-2')).getAttribute('href')).toBe('');
            });
    
            it('should execute ng-click and change url when ng-href specified', function() {
              expect(element(by.id('link-3')).getAttribute('href')).toMatch(/\/123$/);
    
              element(by.id('link-3')).click();
    
              // At this point, we navigate away from an Angular page, so we need
              // to use browser.driver to get the base webdriver.
    
              browser.wait(function() {
                return browser.driver.getCurrentUrl().then(function(url) {
                  return url.match(/\/123$/);
                });
              }, 5000, 'page should navigate to /123');
            });
    
            it('should execute ng-click but not reload when href empty string and name specified', function() {
              element(by.id('link-4')).click();
              expect(element(by.model('value')).getAttribute('value')).toEqual('4');
              expect(element(by.id('link-4')).getAttribute('href')).toBe('');
            });
    
            it('should execute ng-click but not reload when no href but name specified', function() {
              element(by.id('link-5')).click();
              expect(element(by.model('value')).getAttribute('value')).toEqual('5');
              expect(element(by.id('link-5')).getAttribute('href')).toBe(null);
            });
    
            it('should only change url when only ng-href', function() {
              element(by.model('value')).clear();
              element(by.model('value')).sendKeys('6');
              expect(element(by.id('link-6')).getAttribute('href')).toMatch(/\/6$/);
    
              element(by.id('link-6')).click();
    
              // At this point, we navigate away from an Angular page, so we need
              // to use browser.driver to get the base webdriver.
              browser.wait(function() {
                return browser.driver.getCurrentUrl().then(function(url) {
                  return url.match(/\/6$/);
                });
              }, 5000, 'page should navigate to /6');
            });
          </file>
        </example>
     */

    /**
     * @ngdoc directive
     * @name ngSrc
     * @restrict A
     * @priority 99
     *
     * @description
     * Using Angular markup like `{{hash}}` in a `src` attribute doesn't
     * work right: The browser will fetch from the URL with the literal
     * text `{{hash}}` until Angular replaces the expression inside
     * `{{hash}}`. The `ngSrc` directive solves this problem.
     *
     * The buggy way to write it:
     * ```html
     * <img src="http://www.gravatar.com/avatar/{{hash}}" alt="Description"/>
     * ```
     *
     * The correct way to write it:
     * ```html
     * <img ng-src="http://www.gravatar.com/avatar/{{hash}}" alt="Description" />
     * ```
     *
     * @element IMG
     * @param {template} ngSrc any string which can contain `{{}}` markup.
     */

    /**
     * @ngdoc directive
     * @name ngSrcset
     * @restrict A
     * @priority 99
     *
     * @description
     * Using Angular markup like `{{hash}}` in a `srcset` attribute doesn't
     * work right: The browser will fetch from the URL with the literal
     * text `{{hash}}` until Angular replaces the expression inside
     * `{{hash}}`. The `ngSrcset` directive solves this problem.
     *
     * The buggy way to write it:
     * ```html
     * <img srcset="http://www.gravatar.com/avatar/{{hash}} 2x" alt="Description"/>
     * ```
     *
     * The correct way to write it:
     * ```html
     * <img ng-srcset="http://www.gravatar.com/avatar/{{hash}} 2x" alt="Description" />
     * ```
     *
     * @element IMG
     * @param {template} ngSrcset any string which can contain `{{}}` markup.
     */

    /**
     * @ngdoc directive
     * @name ngDisabled
     * @restrict A
     * @priority 100
     *
     * @description
     *
     * This directive sets the `disabled` attribute on the element (typically a form control,
     * e.g. `input`, `button`, `select` etc.) if the
     * {@link guide/expression expression} inside `ngDisabled` evaluates to truthy.
     *
     * A special directive is necessary because we cannot use interpolation inside the `disabled`
     * attribute. See the {@link guide/interpolation interpolation guide} for more info.
     *
     * @example
        <example name="ng-disabled">
          <file name="index.html">
            <label>Click me to toggle: <input type="checkbox" ng-model="checked"></label><br/>
            <button ng-model="button" ng-disabled="checked">Button</button>
          </file>
          <file name="protractor.js" type="protractor">
            it('should toggle button', function() {
              expect(element(by.css('button')).getAttribute('disabled')).toBeFalsy();
              element(by.model('checked')).click();
              expect(element(by.css('button')).getAttribute('disabled')).toBeTruthy();
            });
          </file>
        </example>
     *
     * @element INPUT
     * @param {expression} ngDisabled If the {@link guide/expression expression} is truthy,
     *     then the `disabled` attribute will be set on the element
     */

    /**
     * @ngdoc directive
     * @name ngChecked
     * @restrict A
     * @priority 100
     *
     * @description
     * Sets the `checked` attribute on the element, if the expression inside `ngChecked` is truthy.
     *
     * Note that this directive should not be used together with {@link ngModel `ngModel`},
     * as this can lead to unexpected behavior.
     *
     * A special directive is necessary because we cannot use interpolation inside the `checked`
     * attribute. See the {@link guide/interpolation interpolation guide} for more info.
     *
     * @example
        <example name="ng-checked">
          <file name="index.html">
            <label>Check me to check both: <input type="checkbox" ng-model="master"></label><br/>
            <input id="checkSlave" type="checkbox" ng-checked="master" aria-label="Slave input">
          </file>
          <file name="protractor.js" type="protractor">
            it('should check both checkBoxes', function() {
              expect(element(by.id('checkSlave')).getAttribute('checked')).toBeFalsy();
              element(by.model('master')).click();
              expect(element(by.id('checkSlave')).getAttribute('checked')).toBeTruthy();
            });
          </file>
        </example>
     *
     * @element INPUT
     * @param {expression} ngChecked If the {@link guide/expression expression} is truthy,
     *     then the `checked` attribute will be set on the element
     */

    /**
     * @ngdoc directive
     * @name ngReadonly
     * @restrict A
     * @priority 100
     *
     * @description
     *
     * Sets the `readonly` attribute on the element, if the expression inside `ngReadonly` is truthy.
     * Note that `readonly` applies only to `input` elements with specific types. [See the input docs on
     * MDN](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#attr-readonly) for more information.
     *
     * A special directive is necessary because we cannot use interpolation inside the `readonly`
     * attribute. See the {@link guide/interpolation interpolation guide} for more info.
     *
     * @example
        <example name="ng-readonly">
          <file name="index.html">
            <label>Check me to make text readonly: <input type="checkbox" ng-model="checked"></label><br/>
            <input type="text" ng-readonly="checked" value="I'm Angular" aria-label="Readonly field" />
          </file>
          <file name="protractor.js" type="protractor">
            it('should toggle readonly attr', function() {
              expect(element(by.css('[type="text"]')).getAttribute('readonly')).toBeFalsy();
              element(by.model('checked')).click();
              expect(element(by.css('[type="text"]')).getAttribute('readonly')).toBeTruthy();
            });
          </file>
        </example>
     *
     * @element INPUT
     * @param {expression} ngReadonly If the {@link guide/expression expression} is truthy,
     *     then special attribute "readonly" will be set on the element
     */

    /**
     * @ngdoc directive
     * @name ngSelected
     * @restrict A
     * @priority 100
     *
     * @description
     *
     * Sets the `selected` attribute on the element, if the expression inside `ngSelected` is truthy.
     *
     * A special directive is necessary because we cannot use interpolation inside the `selected`
     * attribute. See the {@link guide/interpolation interpolation guide} for more info.
     *
     * <div class="alert alert-warning">
     *   **Note:** `ngSelected` does not interact with the `select` and `ngModel` directives, it only
     *   sets the `selected` attribute on the element. If you are using `ngModel` on the select, you
     *   should not use `ngSelected` on the options, as `ngModel` will set the select value and
     *   selected options.
     * </div>
     *
     * @example
        <example name="ng-selected">
          <file name="index.html">
            <label>Check me to select: <input type="checkbox" ng-model="selected"></label><br/>
            <select aria-label="ngSelected demo">
              <option>Hello!</option>
              <option id="greet" ng-selected="selected">Greetings!</option>
            </select>
          </file>
          <file name="protractor.js" type="protractor">
            it('should select Greetings!', function() {
              expect(element(by.id('greet')).getAttribute('selected')).toBeFalsy();
              element(by.model('selected')).click();
              expect(element(by.id('greet')).getAttribute('selected')).toBeTruthy();
            });
          </file>
        </example>
     *
     * @element OPTION
     * @param {expression} ngSelected If the {@link guide/expression expression} is truthy,
     *     then special attribute "selected" will be set on the element
     */

    /**
     * @ngdoc directive
     * @name ngOpen
     * @restrict A
     * @priority 100
     *
     * @description
     *
     * Sets the `open` attribute on the element, if the expression inside `ngOpen` is truthy.
     *
     * A special directive is necessary because we cannot use interpolation inside the `open`
     * attribute. See the {@link guide/interpolation interpolation guide} for more info.
     *
     * ## A note about browser compatibility
     *
     * Edge, Firefox, and Internet Explorer do not support the `details` element, it is
     * recommended to use {@link ng.ngShow} and {@link ng.ngHide} instead.
     *
     * @example
         <example name="ng-open">
           <file name="index.html">
             <label>Check me check multiple: <input type="checkbox" ng-model="open"></label><br/>
             <details id="details" ng-open="open">
                <summary>Show/Hide me</summary>
             </details>
           </file>
           <file name="protractor.js" type="protractor">
             it('should toggle open', function() {
               expect(element(by.id('details')).getAttribute('open')).toBeFalsy();
               element(by.model('open')).click();
               expect(element(by.id('details')).getAttribute('open')).toBeTruthy();
             });
           </file>
         </example>
     *
     * @element DETAILS
     * @param {expression} ngOpen If the {@link guide/expression expression} is truthy,
     *     then special attribute "open" will be set on the element
     */

    var ngAttributeAliasDirectives = {};

    // boolean attrs are evaluated
    forEach(BOOLEAN_ATTR, function (propName, attrName) {
      // binding to multiple is not supported
      if (propName === 'multiple') return;

      function defaultLinkFn(scope, element, attr) {
        scope.$watch(attr[normalized], function ngBooleanAttrWatchAction(value) {
          attr.$set(attrName, !!value);
        });
      }

      var normalized = directiveNormalize('ng-' + attrName);
      var linkFn = defaultLinkFn;

      if (propName === 'checked') {
        linkFn = function (scope, element, attr) {
          // ensuring ngChecked doesn't interfere with ngModel when both are set on the same input
          if (attr.ngModel !== attr[normalized]) {
            defaultLinkFn(scope, element, attr);
          }
        };
      }

      ngAttributeAliasDirectives[normalized] = function () {
        return {
          restrict: 'A',
          priority: 100,
          link: linkFn
        };
      };
    });

    // aliased input attrs are evaluated
    forEach(ALIASED_ATTR, function (htmlAttr, ngAttr) {
      ngAttributeAliasDirectives[ngAttr] = function () {
        return {
          priority: 100,
          link: function (scope, element, attr) {
            //special case ngPattern when a literal regular expression value
            //is used as the expression (this way we don't have to watch anything).
            if (ngAttr === 'ngPattern' && attr.ngPattern.charAt(0) === '/') {
              var match = attr.ngPattern.match(REGEX_STRING_REGEXP);
              if (match) {
                attr.$set('ngPattern', new RegExp(match[1], match[2]));
                return;
              }
            }

            scope.$watch(attr[ngAttr], function ngAttrAliasWatchAction(value) {
              attr.$set(ngAttr, value);
            });
          }
        };
      };
    });

    // ng-src, ng-srcset, ng-href are interpolated
    forEach(['src', 'srcset', 'href'], function (attrName) {
      var normalized = directiveNormalize('ng-' + attrName);
      ngAttributeAliasDirectives[normalized] = function () {
        return {
          priority: 99, // it needs to run after the attributes are interpolated
          link: function (scope, element, attr) {
            var propName = attrName,
                name = attrName;

            if (attrName === 'href' && toString.call(element.prop('href')) === '[object SVGAnimatedString]') {
              name = 'xlinkHref';
              attr.$attr[name] = 'xlink:href';
              propName = null;
            }

            attr.$observe(normalized, function (value) {
              if (!value) {
                if (attrName === 'href') {
                  attr.$set(name, null);
                }
                return;
              }

              attr.$set(name, value);

              // Support: IE 9-11 only
              // On IE, if "ng:src" directive declaration is used and "src" attribute doesn't exist
              // then calling element.setAttribute('src', 'foo') doesn't do anything, so we need
              // to set the property as well to achieve the desired effect.
              // We use attr[attrName] value since $set can sanitize the url.
              if (msie && propName) element.prop(propName, attr[name]);
            });
          }
        };
      };
    });

    /* global -nullFormCtrl, -PENDING_CLASS, -SUBMITTED_CLASS
     */
    var nullFormCtrl = {
      $addControl: noop,
      $$renameControl: nullFormRenameControl,
      $removeControl: noop,
      $setValidity: noop,
      $setDirty: noop,
      $setPristine: noop,
      $setSubmitted: noop
    },
        PENDING_CLASS = 'ng-pending',
        SUBMITTED_CLASS = 'ng-submitted';

    function nullFormRenameControl(control, name) {
      control.$name = name;
    }

    /**
     * @ngdoc type
     * @name form.FormController
     *
     * @property {boolean} $pristine True if user has not interacted with the form yet.
     * @property {boolean} $dirty True if user has already interacted with the form.
     * @property {boolean} $valid True if all of the containing forms and controls are valid.
     * @property {boolean} $invalid True if at least one containing control or form is invalid.
     * @property {boolean} $pending True if at least one containing control or form is pending.
     * @property {boolean} $submitted True if user has submitted the form even if its invalid.
     *
     * @property {Object} $error Is an object hash, containing references to controls or
     *  forms with failing validators, where:
     *
     *  - keys are validation tokens (error names),
     *  - values are arrays of controls or forms that have a failing validator for given error name.
     *
     *  Built-in validation tokens:
     *
     *  - `email`
     *  - `max`
     *  - `maxlength`
     *  - `min`
     *  - `minlength`
     *  - `number`
     *  - `pattern`
     *  - `required`
     *  - `url`
     *  - `date`
     *  - `datetimelocal`
     *  - `time`
     *  - `week`
     *  - `month`
     *
     * @description
     * `FormController` keeps track of all its controls and nested forms as well as the state of them,
     * such as being valid/invalid or dirty/pristine.
     *
     * Each {@link ng.directive:form form} directive creates an instance
     * of `FormController`.
     *
     */
    //asks for $scope to fool the BC controller module
    FormController.$inject = ['$element', '$attrs', '$scope', '$animate', '$interpolate'];
    function FormController($element, $attrs, $scope, $animate, $interpolate) {
      this.$$controls = [];

      // init state
      this.$error = {};
      this.$$success = {};
      this.$pending = undefined;
      this.$name = $interpolate($attrs.name || $attrs.ngForm || '')($scope);
      this.$dirty = false;
      this.$pristine = true;
      this.$valid = true;
      this.$invalid = false;
      this.$submitted = false;
      this.$$parentForm = nullFormCtrl;

      this.$$element = $element;
      this.$$animate = $animate;

      setupValidity(this);
    }

    FormController.prototype = {
      /**
       * @ngdoc method
       * @name form.FormController#$rollbackViewValue
       *
       * @description
       * Rollback all form controls pending updates to the `$modelValue`.
       *
       * Updates may be pending by a debounced event or because the input is waiting for a some future
       * event defined in `ng-model-options`. This method is typically needed by the reset button of
       * a form that uses `ng-model-options` to pend updates.
       */
      $rollbackViewValue: function () {
        forEach(this.$$controls, function (control) {
          control.$rollbackViewValue();
        });
      },

      /**
       * @ngdoc method
       * @name form.FormController#$commitViewValue
       *
       * @description
       * Commit all form controls pending updates to the `$modelValue`.
       *
       * Updates may be pending by a debounced event or because the input is waiting for a some future
       * event defined in `ng-model-options`. This method is rarely needed as `NgModelController`
       * usually handles calling this in response to input events.
       */
      $commitViewValue: function () {
        forEach(this.$$controls, function (control) {
          control.$commitViewValue();
        });
      },

      /**
       * @ngdoc method
       * @name form.FormController#$addControl
       * @param {object} control control object, either a {@link form.FormController} or an
       * {@link ngModel.NgModelController}
       *
       * @description
       * Register a control with the form. Input elements using ngModelController do this automatically
       * when they are linked.
       *
       * Note that the current state of the control will not be reflected on the new parent form. This
       * is not an issue with normal use, as freshly compiled and linked controls are in a `$pristine`
       * state.
       *
       * However, if the method is used programmatically, for example by adding dynamically created controls,
       * or controls that have been previously removed without destroying their corresponding DOM element,
       * it's the developers responsibility to make sure the current state propagates to the parent form.
       *
       * For example, if an input control is added that is already `$dirty` and has `$error` properties,
       * calling `$setDirty()` and `$validate()` afterwards will propagate the state to the parent form.
       */
      $addControl: function (control) {
        // Breaking change - before, inputs whose name was "hasOwnProperty" were quietly ignored
        // and not added to the scope.  Now we throw an error.
        assertNotHasOwnProperty(control.$name, 'input');
        this.$$controls.push(control);

        if (control.$name) {
          this[control.$name] = control;
        }

        control.$$parentForm = this;
      },

      // Private API: rename a form control
      $$renameControl: function (control, newName) {
        var oldName = control.$name;

        if (this[oldName] === control) {
          delete this[oldName];
        }
        this[newName] = control;
        control.$name = newName;
      },

      /**
       * @ngdoc method
       * @name form.FormController#$removeControl
       * @param {object} control control object, either a {@link form.FormController} or an
       * {@link ngModel.NgModelController}
       *
       * @description
       * Deregister a control from the form.
       *
       * Input elements using ngModelController do this automatically when they are destroyed.
       *
       * Note that only the removed control's validation state (`$errors`etc.) will be removed from the
       * form. `$dirty`, `$submitted` states will not be changed, because the expected behavior can be
       * different from case to case. For example, removing the only `$dirty` control from a form may or
       * may not mean that the form is still `$dirty`.
       */
      $removeControl: function (control) {
        if (control.$name && this[control.$name] === control) {
          delete this[control.$name];
        }
        forEach(this.$pending, function (value, name) {
          // eslint-disable-next-line no-invalid-this
          this.$setValidity(name, null, control);
        }, this);
        forEach(this.$error, function (value, name) {
          // eslint-disable-next-line no-invalid-this
          this.$setValidity(name, null, control);
        }, this);
        forEach(this.$$success, function (value, name) {
          // eslint-disable-next-line no-invalid-this
          this.$setValidity(name, null, control);
        }, this);

        arrayRemove(this.$$controls, control);
        control.$$parentForm = nullFormCtrl;
      },

      /**
       * @ngdoc method
       * @name form.FormController#$setDirty
       *
       * @description
       * Sets the form to a dirty state.
       *
       * This method can be called to add the 'ng-dirty' class and set the form to a dirty
       * state (ng-dirty class). This method will also propagate to parent forms.
       */
      $setDirty: function () {
        this.$$animate.removeClass(this.$$element, PRISTINE_CLASS);
        this.$$animate.addClass(this.$$element, DIRTY_CLASS);
        this.$dirty = true;
        this.$pristine = false;
        this.$$parentForm.$setDirty();
      },

      /**
       * @ngdoc method
       * @name form.FormController#$setPristine
       *
       * @description
       * Sets the form to its pristine state.
       *
       * This method sets the form's `$pristine` state to true, the `$dirty` state to false, removes
       * the `ng-dirty` class and adds the `ng-pristine` class. Additionally, it sets the `$submitted`
       * state to false.
       *
       * This method will also propagate to all the controls contained in this form.
       *
       * Setting a form back to a pristine state is often useful when we want to 'reuse' a form after
       * saving or resetting it.
       */
      $setPristine: function () {
        this.$$animate.setClass(this.$$element, PRISTINE_CLASS, DIRTY_CLASS + ' ' + SUBMITTED_CLASS);
        this.$dirty = false;
        this.$pristine = true;
        this.$submitted = false;
        forEach(this.$$controls, function (control) {
          control.$setPristine();
        });
      },

      /**
       * @ngdoc method
       * @name form.FormController#$setUntouched
       *
       * @description
       * Sets the form to its untouched state.
       *
       * This method can be called to remove the 'ng-touched' class and set the form controls to their
       * untouched state (ng-untouched class).
       *
       * Setting a form controls back to their untouched state is often useful when setting the form
       * back to its pristine state.
       */
      $setUntouched: function () {
        forEach(this.$$controls, function (control) {
          control.$setUntouched();
        });
      },

      /**
       * @ngdoc method
       * @name form.FormController#$setSubmitted
       *
       * @description
       * Sets the form to its submitted state.
       */
      $setSubmitted: function () {
        this.$$animate.addClass(this.$$element, SUBMITTED_CLASS);
        this.$submitted = true;
        this.$$parentForm.$setSubmitted();
      }
    };

    /**
     * @ngdoc method
     * @name form.FormController#$setValidity
     *
     * @description
     * Sets the validity of a form control.
     *
     * This method will also propagate to parent forms.
     */
    addSetValidityMethod({
      clazz: FormController,
      set: function (object, property, controller) {
        var list = object[property];
        if (!list) {
          object[property] = [controller];
        } else {
          var index = list.indexOf(controller);
          if (index === -1) {
            list.push(controller);
          }
        }
      },
      unset: function (object, property, controller) {
        var list = object[property];
        if (!list) {
          return;
        }
        arrayRemove(list, controller);
        if (list.length === 0) {
          delete object[property];
        }
      }
    });

    /**
     * @ngdoc directive
     * @name ngForm
     * @restrict EAC
     *
     * @description
     * Nestable alias of {@link ng.directive:form `form`} directive. HTML
     * does not allow nesting of form elements. It is useful to nest forms, for example if the validity of a
     * sub-group of controls needs to be determined.
     *
     * Note: the purpose of `ngForm` is to group controls,
     * but not to be a replacement for the `<form>` tag with all of its capabilities
     * (e.g. posting to the server, ...).
     *
     * @param {string=} ngForm|name Name of the form. If specified, the form controller will be published into
     *                       related scope, under this name.
     *
     */

    /**
    * @ngdoc directive
    * @name form
    * @restrict E
    *
    * @description
    * Directive that instantiates
    * {@link form.FormController FormController}.
    *
    * If the `name` attribute is specified, the form controller is published onto the current scope under
    * this name.
    *
    * # Alias: {@link ng.directive:ngForm `ngForm`}
    *
    * In Angular, forms can be nested. This means that the outer form is valid when all of the child
    * forms are valid as well. However, browsers do not allow nesting of `<form>` elements, so
    * Angular provides the {@link ng.directive:ngForm `ngForm`} directive, which behaves identically to
    * `form` but can be nested. Nested forms can be useful, for example, if the validity of a sub-group
    * of controls needs to be determined.
    *
    * # CSS classes
    *  - `ng-valid` is set if the form is valid.
    *  - `ng-invalid` is set if the form is invalid.
    *  - `ng-pending` is set if the form is pending.
    *  - `ng-pristine` is set if the form is pristine.
    *  - `ng-dirty` is set if the form is dirty.
    *  - `ng-submitted` is set if the form was submitted.
    *
    * Keep in mind that ngAnimate can detect each of these classes when added and removed.
    *
    *
    * # Submitting a form and preventing the default action
    *
    * Since the role of forms in client-side Angular applications is different than in classical
    * roundtrip apps, it is desirable for the browser not to translate the form submission into a full
    * page reload that sends the data to the server. Instead some javascript logic should be triggered
    * to handle the form submission in an application-specific way.
    *
    * For this reason, Angular prevents the default action (form submission to the server) unless the
    * `<form>` element has an `action` attribute specified.
    *
    * You can use one of the following two ways to specify what javascript method should be called when
    * a form is submitted:
    *
    * - {@link ng.directive:ngSubmit ngSubmit} directive on the form element
    * - {@link ng.directive:ngClick ngClick} directive on the first
     *  button or input field of type submit (input[type=submit])
    *
    * To prevent double execution of the handler, use only one of the {@link ng.directive:ngSubmit ngSubmit}
    * or {@link ng.directive:ngClick ngClick} directives.
    * This is because of the following form submission rules in the HTML specification:
    *
    * - If a form has only one input field then hitting enter in this field triggers form submit
    * (`ngSubmit`)
    * - if a form has 2+ input fields and no buttons or input[type=submit] then hitting enter
    * doesn't trigger submit
    * - if a form has one or more input fields and one or more buttons or input[type=submit] then
    * hitting enter in any of the input fields will trigger the click handler on the *first* button or
    * input[type=submit] (`ngClick`) *and* a submit handler on the enclosing form (`ngSubmit`)
    *
    * Any pending `ngModelOptions` changes will take place immediately when an enclosing form is
    * submitted. Note that `ngClick` events will occur before the model is updated. Use `ngSubmit`
    * to have access to the updated model.
    *
    * ## Animation Hooks
    *
    * Animations in ngForm are triggered when any of the associated CSS classes are added and removed.
    * These classes are: `.ng-pristine`, `.ng-dirty`, `.ng-invalid` and `.ng-valid` as well as any
    * other validations that are performed within the form. Animations in ngForm are similar to how
    * they work in ngClass and animations can be hooked into using CSS transitions, keyframes as well
    * as JS animations.
    *
    * The following example shows a simple way to utilize CSS transitions to style a form element
    * that has been rendered as invalid after it has been validated:
    *
    * <pre>
    * //be sure to include ngAnimate as a module to hook into more
    * //advanced animations
    * .my-form {
    *   transition:0.5s linear all;
    *   background: white;
    * }
    * .my-form.ng-invalid {
    *   background: red;
    *   color:white;
    * }
    * </pre>
    *
    * @example
       <example name="ng-form" deps="angular-animate.js" animations="true" fixBase="true" module="formExample">
         <file name="index.html">
          <script>
            angular.module('formExample', [])
              .controller('FormController', ['$scope', function($scope) {
                $scope.userType = 'guest';
              }]);
          </script>
          <style>
           .my-form {
             transition:all linear 0.5s;
             background: transparent;
           }
           .my-form.ng-invalid {
             background: red;
           }
          </style>
          <form name="myForm" ng-controller="FormController" class="my-form">
            userType: <input name="input" ng-model="userType" required>
            <span class="error" ng-show="myForm.input.$error.required">Required!</span><br>
            <code>userType = {{userType}}</code><br>
            <code>myForm.input.$valid = {{myForm.input.$valid}}</code><br>
            <code>myForm.input.$error = {{myForm.input.$error}}</code><br>
            <code>myForm.$valid = {{myForm.$valid}}</code><br>
            <code>myForm.$error.required = {{!!myForm.$error.required}}</code><br>
           </form>
         </file>
         <file name="protractor.js" type="protractor">
           it('should initialize to model', function() {
             var userType = element(by.binding('userType'));
             var valid = element(by.binding('myForm.input.$valid'));
              expect(userType.getText()).toContain('guest');
             expect(valid.getText()).toContain('true');
           });
            it('should be invalid if empty', function() {
             var userType = element(by.binding('userType'));
             var valid = element(by.binding('myForm.input.$valid'));
             var userInput = element(by.model('userType'));
              userInput.clear();
             userInput.sendKeys('');
              expect(userType.getText()).toEqual('userType =');
             expect(valid.getText()).toContain('false');
           });
         </file>
       </example>
    *
    * @param {string=} name Name of the form. If specified, the form controller will be published into
    *                       related scope, under this name.
    */
    var formDirectiveFactory = function (isNgForm) {
      return ['$timeout', '$parse', function ($timeout, $parse) {
        var formDirective = {
          name: 'form',
          restrict: isNgForm ? 'EAC' : 'E',
          require: ['form', '^^?form'], //first is the form's own ctrl, second is an optional parent form
          controller: FormController,
          compile: function ngFormCompile(formElement, attr) {
            // Setup initial state of the control
            formElement.addClass(PRISTINE_CLASS).addClass(VALID_CLASS);

            var nameAttr = attr.name ? 'name' : isNgForm && attr.ngForm ? 'ngForm' : false;

            return {
              pre: function ngFormPreLink(scope, formElement, attr, ctrls) {
                var controller = ctrls[0];

                // if `action` attr is not present on the form, prevent the default action (submission)
                if (!('action' in attr)) {
                  // we can't use jq events because if a form is destroyed during submission the default
                  // action is not prevented. see #1238
                  //
                  // IE 9 is not affected because it doesn't fire a submit event and try to do a full
                  // page reload if the form was destroyed by submission of the form via a click handler
                  // on a button in the form. Looks like an IE9 specific bug.
                  var handleFormSubmission = function (event) {
                    scope.$apply(function () {
                      controller.$commitViewValue();
                      controller.$setSubmitted();
                    });

                    event.preventDefault();
                  };

                  formElement[0].addEventListener('submit', handleFormSubmission);

                  // unregister the preventDefault listener so that we don't not leak memory but in a
                  // way that will achieve the prevention of the default action.
                  formElement.on('$destroy', function () {
                    $timeout(function () {
                      formElement[0].removeEventListener('submit', handleFormSubmission);
                    }, 0, false);
                  });
                }

                var parentFormCtrl = ctrls[1] || controller.$$parentForm;
                parentFormCtrl.$addControl(controller);

                var setter = nameAttr ? getSetter(controller.$name) : noop;

                if (nameAttr) {
                  setter(scope, controller);
                  attr.$observe(nameAttr, function (newValue) {
                    if (controller.$name === newValue) return;
                    setter(scope, undefined);
                    controller.$$parentForm.$$renameControl(controller, newValue);
                    setter = getSetter(controller.$name);
                    setter(scope, controller);
                  });
                }
                formElement.on('$destroy', function () {
                  controller.$$parentForm.$removeControl(controller);
                  setter(scope, undefined);
                  extend(controller, nullFormCtrl); //stop propagating child destruction handlers upwards
                });
              }
            };
          }
        };

        return formDirective;

        function getSetter(expression) {
          if (expression === '') {
            //create an assignable expression, so forms with an empty name can be renamed later
            return $parse('this[""]').assign;
          }
          return $parse(expression).assign || noop;
        }
      }];
    };

    var formDirective = formDirectiveFactory();
    var ngFormDirective = formDirectiveFactory(true);

    // helper methods
    function setupValidity(instance) {
      instance.$$classCache = {};
      instance.$$classCache[INVALID_CLASS] = !(instance.$$classCache[VALID_CLASS] = instance.$$element.hasClass(VALID_CLASS));
    }
    function addSetValidityMethod(context) {
      var clazz = context.clazz,
          set = context.set,
          unset = context.unset;

      clazz.prototype.$setValidity = function (validationErrorKey, state, controller) {
        if (isUndefined(state)) {
          createAndSet(this, '$pending', validationErrorKey, controller);
        } else {
          unsetAndCleanup(this, '$pending', validationErrorKey, controller);
        }
        if (!isBoolean(state)) {
          unset(this.$error, validationErrorKey, controller);
          unset(this.$$success, validationErrorKey, controller);
        } else {
          if (state) {
            unset(this.$error, validationErrorKey, controller);
            set(this.$$success, validationErrorKey, controller);
          } else {
            set(this.$error, validationErrorKey, controller);
            unset(this.$$success, validationErrorKey, controller);
          }
        }
        if (this.$pending) {
          cachedToggleClass(this, PENDING_CLASS, true);
          this.$valid = this.$invalid = undefined;
          toggleValidationCss(this, '', null);
        } else {
          cachedToggleClass(this, PENDING_CLASS, false);
          this.$valid = isObjectEmpty(this.$error);
          this.$invalid = !this.$valid;
          toggleValidationCss(this, '', this.$valid);
        }

        // re-read the state as the set/unset methods could have
        // combined state in this.$error[validationError] (used for forms),
        // where setting/unsetting only increments/decrements the value,
        // and does not replace it.
        var combinedState;
        if (this.$pending && this.$pending[validationErrorKey]) {
          combinedState = undefined;
        } else if (this.$error[validationErrorKey]) {
          combinedState = false;
        } else if (this.$$success[validationErrorKey]) {
          combinedState = true;
        } else {
          combinedState = null;
        }

        toggleValidationCss(this, validationErrorKey, combinedState);
        this.$$parentForm.$setValidity(validationErrorKey, combinedState, this);
      };

      function createAndSet(ctrl, name, value, controller) {
        if (!ctrl[name]) {
          ctrl[name] = {};
        }
        set(ctrl[name], value, controller);
      }

      function unsetAndCleanup(ctrl, name, value, controller) {
        if (ctrl[name]) {
          unset(ctrl[name], value, controller);
        }
        if (isObjectEmpty(ctrl[name])) {
          ctrl[name] = undefined;
        }
      }

      function cachedToggleClass(ctrl, className, switchValue) {
        if (switchValue && !ctrl.$$classCache[className]) {
          ctrl.$$animate.addClass(ctrl.$$element, className);
          ctrl.$$classCache[className] = true;
        } else if (!switchValue && ctrl.$$classCache[className]) {
          ctrl.$$animate.removeClass(ctrl.$$element, className);
          ctrl.$$classCache[className] = false;
        }
      }

      function toggleValidationCss(ctrl, validationErrorKey, isValid) {
        validationErrorKey = validationErrorKey ? '-' + snake_case(validationErrorKey, '-') : '';

        cachedToggleClass(ctrl, VALID_CLASS + validationErrorKey, isValid === true);
        cachedToggleClass(ctrl, INVALID_CLASS + validationErrorKey, isValid === false);
      }
    }

    function isObjectEmpty(obj) {
      if (obj) {
        for (var prop in obj) {
          if (obj.hasOwnProperty(prop)) {
            return false;
          }
        }
      }
      return true;
    }

    /* global
      VALID_CLASS: false,
      INVALID_CLASS: false,
      PRISTINE_CLASS: false,
      DIRTY_CLASS: false,
      ngModelMinErr: false
    */

    // Regex code was initially obtained from SO prior to modification: https://stackoverflow.com/questions/3143070/javascript-regex-iso-datetime#answer-3143231
    var ISO_DATE_REGEXP = /^\d{4,}-[01]\d-[0-3]\dT[0-2]\d:[0-5]\d:[0-5]\d\.\d+(?:[+-][0-2]\d:[0-5]\d|Z)$/;
    // See valid URLs in RFC3987 (http://tools.ietf.org/html/rfc3987)
    // Note: We are being more lenient, because browsers are too.
    //   1. Scheme
    //   2. Slashes
    //   3. Username
    //   4. Password
    //   5. Hostname
    //   6. Port
    //   7. Path
    //   8. Query
    //   9. Fragment
    //                 1111111111111111 222   333333    44444        55555555555555555555555     666     77777777     8888888     999
    var URL_REGEXP = /^[a-z][a-z\d.+-]*:\/*(?:[^:@]+(?::[^@]+)?@)?(?:[^\s:/?#]+|\[[a-f\d:]+])(?::\d+)?(?:\/[^?#]*)?(?:\?[^#]*)?(?:#.*)?$/i;
    // eslint-disable-next-line max-len
    var EMAIL_REGEXP = /^(?=.{1,254}$)(?=.{1,64}@)[-!#$%&'*+/0-9=?A-Z^_`a-z{|}~]+(\.[-!#$%&'*+/0-9=?A-Z^_`a-z{|}~]+)*@[A-Za-z0-9]([A-Za-z0-9-]{0,61}[A-Za-z0-9])?(\.[A-Za-z0-9]([A-Za-z0-9-]{0,61}[A-Za-z0-9])?)*$/;
    var NUMBER_REGEXP = /^\s*(-|\+)?(\d+|(\d*(\.\d*)))([eE][+-]?\d+)?\s*$/;
    var DATE_REGEXP = /^(\d{4,})-(\d{2})-(\d{2})$/;
    var DATETIMELOCAL_REGEXP = /^(\d{4,})-(\d\d)-(\d\d)T(\d\d):(\d\d)(?::(\d\d)(\.\d{1,3})?)?$/;
    var WEEK_REGEXP = /^(\d{4,})-W(\d\d)$/;
    var MONTH_REGEXP = /^(\d{4,})-(\d\d)$/;
    var TIME_REGEXP = /^(\d\d):(\d\d)(?::(\d\d)(\.\d{1,3})?)?$/;

    var PARTIAL_VALIDATION_EVENTS = 'keydown wheel mousedown';
    var PARTIAL_VALIDATION_TYPES = createMap();
    forEach('date,datetime-local,month,time,week'.split(','), function (type) {
      PARTIAL_VALIDATION_TYPES[type] = true;
    });

    var inputType = {

      /**
       * @ngdoc input
       * @name input[text]
       *
       * @description
       * Standard HTML text input with angular data binding, inherited by most of the `input` elements.
       *
       *
       * @param {string} ngModel Assignable angular expression to data-bind to.
       * @param {string=} name Property name of the form under which the control is published.
       * @param {string=} required Adds `required` validation error key if the value is not entered.
       * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
       *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
       *    `required` when you want to data-bind to the `required` attribute.
       * @param {number=} ngMinlength Sets `minlength` validation error key if the value is shorter than
       *    minlength.
       * @param {number=} ngMaxlength Sets `maxlength` validation error key if the value is longer than
       *    maxlength. Setting the attribute to a negative or non-numeric value, allows view values of
       *    any length.
       * @param {string=} pattern Similar to `ngPattern` except that the attribute value is the actual string
       *    that contains the regular expression body that will be converted to a regular expression
       *    as in the ngPattern directive.
       * @param {string=} ngPattern Sets `pattern` validation error key if the ngModel {@link ngModel.NgModelController#$viewValue $viewValue}
       *    does not match a RegExp found by evaluating the Angular expression given in the attribute value.
       *    If the expression evaluates to a RegExp object, then this is used directly.
       *    If the expression evaluates to a string, then it will be converted to a RegExp
       *    after wrapping it in `^` and `$` characters. For instance, `"abc"` will be converted to
       *    `new RegExp('^abc$')`.<br />
       *    **Note:** Avoid using the `g` flag on the RegExp, as it will cause each successive search to
       *    start at the index of the last search's match, thus not taking the whole input value into
       *    account.
       * @param {string=} ngChange Angular expression to be executed when input changes due to user
       *    interaction with the input element.
       * @param {boolean=} [ngTrim=true] If set to false Angular will not automatically trim the input.
       *    This parameter is ignored for input[type=password] controls, which will never trim the
       *    input.
       *
       * @example
          <example name="text-input-directive" module="textInputExample">
            <file name="index.html">
             <script>
               angular.module('textInputExample', [])
                 .controller('ExampleController', ['$scope', function($scope) {
                   $scope.example = {
                     text: 'guest',
                     word: /^\s*\w*\s*$/
                   };
                 }]);
             </script>
             <form name="myForm" ng-controller="ExampleController">
               <label>Single word:
                 <input type="text" name="input" ng-model="example.text"
                        ng-pattern="example.word" required ng-trim="false">
               </label>
               <div role="alert">
                 <span class="error" ng-show="myForm.input.$error.required">
                   Required!</span>
                 <span class="error" ng-show="myForm.input.$error.pattern">
                   Single word only!</span>
               </div>
               <code>text = {{example.text}}</code><br/>
               <code>myForm.input.$valid = {{myForm.input.$valid}}</code><br/>
               <code>myForm.input.$error = {{myForm.input.$error}}</code><br/>
               <code>myForm.$valid = {{myForm.$valid}}</code><br/>
               <code>myForm.$error.required = {{!!myForm.$error.required}}</code><br/>
              </form>
            </file>
            <file name="protractor.js" type="protractor">
              var text = element(by.binding('example.text'));
              var valid = element(by.binding('myForm.input.$valid'));
              var input = element(by.model('example.text'));
               it('should initialize to model', function() {
                expect(text.getText()).toContain('guest');
                expect(valid.getText()).toContain('true');
              });
               it('should be invalid if empty', function() {
                input.clear();
                input.sendKeys('');
                 expect(text.getText()).toEqual('text =');
                expect(valid.getText()).toContain('false');
              });
               it('should be invalid if multi word', function() {
                input.clear();
                input.sendKeys('hello world');
                 expect(valid.getText()).toContain('false');
              });
            </file>
          </example>
       */
      'text': textInputType,

      /**
       * @ngdoc input
       * @name input[date]
       *
       * @description
       * Input with date validation and transformation. In browsers that do not yet support
       * the HTML5 date input, a text element will be used. In that case, text must be entered in a valid ISO-8601
       * date format (yyyy-MM-dd), for example: `2009-01-06`. Since many
       * modern browsers do not yet support this input type, it is important to provide cues to users on the
       * expected input format via a placeholder or label.
       *
       * The model must always be a Date object, otherwise Angular will throw an error.
       * Invalid `Date` objects (dates whose `getTime()` is `NaN`) will be rendered as an empty string.
       *
       * The timezone to be used to read/write the `Date` instance in the model can be defined using
       * {@link ng.directive:ngModelOptions ngModelOptions}. By default, this is the timezone of the browser.
       *
       * @param {string} ngModel Assignable angular expression to data-bind to.
       * @param {string=} name Property name of the form under which the control is published.
       * @param {string=} min Sets the `min` validation error key if the value entered is less than `min`. This must be a
       *   valid ISO date string (yyyy-MM-dd). You can also use interpolation inside this attribute
       *   (e.g. `min="{{minDate | date:'yyyy-MM-dd'}}"`). Note that `min` will also add native HTML5
       *   constraint validation.
       * @param {string=} max Sets the `max` validation error key if the value entered is greater than `max`. This must be
       *   a valid ISO date string (yyyy-MM-dd). You can also use interpolation inside this attribute
       *   (e.g. `max="{{maxDate | date:'yyyy-MM-dd'}}"`). Note that `max` will also add native HTML5
       *   constraint validation.
       * @param {(date|string)=} ngMin Sets the `min` validation constraint to the Date / ISO date string
       *   the `ngMin` expression evaluates to. Note that it does not set the `min` attribute.
       * @param {(date|string)=} ngMax Sets the `max` validation constraint to the Date / ISO date string
       *   the `ngMax` expression evaluates to. Note that it does not set the `max` attribute.
       * @param {string=} required Sets `required` validation error key if the value is not entered.
       * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
       *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
       *    `required` when you want to data-bind to the `required` attribute.
       * @param {string=} ngChange Angular expression to be executed when input changes due to user
       *    interaction with the input element.
       *
       * @example
       <example name="date-input-directive" module="dateInputExample">
       <file name="index.html">
         <script>
            angular.module('dateInputExample', [])
              .controller('DateController', ['$scope', function($scope) {
                $scope.example = {
                  value: new Date(2013, 9, 22)
                };
              }]);
         </script>
         <form name="myForm" ng-controller="DateController as dateCtrl">
            <label for="exampleInput">Pick a date in 2013:</label>
            <input type="date" id="exampleInput" name="input" ng-model="example.value"
                placeholder="yyyy-MM-dd" min="2013-01-01" max="2013-12-31" required />
            <div role="alert">
              <span class="error" ng-show="myForm.input.$error.required">
                  Required!</span>
              <span class="error" ng-show="myForm.input.$error.date">
                  Not a valid date!</span>
             </div>
             <tt>value = {{example.value | date: "yyyy-MM-dd"}}</tt><br/>
             <tt>myForm.input.$valid = {{myForm.input.$valid}}</tt><br/>
             <tt>myForm.input.$error = {{myForm.input.$error}}</tt><br/>
             <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
             <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
         </form>
       </file>
       <file name="protractor.js" type="protractor">
          var value = element(by.binding('example.value | date: "yyyy-MM-dd"'));
          var valid = element(by.binding('myForm.input.$valid'));
           // currently protractor/webdriver does not support
          // sending keys to all known HTML5 input controls
          // for various browsers (see https://github.com/angular/protractor/issues/562).
          function setInput(val) {
            // set the value of the element and force validation.
            var scr = "var ipt = document.getElementById('exampleInput'); " +
            "ipt.value = '" + val + "';" +
            "angular.element(ipt).scope().$apply(function(s) { s.myForm[ipt.name].$setViewValue('" + val + "'); });";
            browser.executeScript(scr);
          }
           it('should initialize to model', function() {
            expect(value.getText()).toContain('2013-10-22');
            expect(valid.getText()).toContain('myForm.input.$valid = true');
          });
           it('should be invalid if empty', function() {
            setInput('');
            expect(value.getText()).toEqual('value =');
            expect(valid.getText()).toContain('myForm.input.$valid = false');
          });
           it('should be invalid if over max', function() {
            setInput('2015-01-01');
            expect(value.getText()).toContain('');
            expect(valid.getText()).toContain('myForm.input.$valid = false');
          });
       </file>
       </example>
       */
      'date': createDateInputType('date', DATE_REGEXP, createDateParser(DATE_REGEXP, ['yyyy', 'MM', 'dd']), 'yyyy-MM-dd'),

      /**
       * @ngdoc input
       * @name input[datetime-local]
       *
       * @description
       * Input with datetime validation and transformation. In browsers that do not yet support
       * the HTML5 date input, a text element will be used. In that case, the text must be entered in a valid ISO-8601
       * local datetime format (yyyy-MM-ddTHH:mm:ss), for example: `2010-12-28T14:57:00`.
       *
       * The model must always be a Date object, otherwise Angular will throw an error.
       * Invalid `Date` objects (dates whose `getTime()` is `NaN`) will be rendered as an empty string.
       *
       * The timezone to be used to read/write the `Date` instance in the model can be defined using
       * {@link ng.directive:ngModelOptions ngModelOptions}. By default, this is the timezone of the browser.
       *
       * @param {string} ngModel Assignable angular expression to data-bind to.
       * @param {string=} name Property name of the form under which the control is published.
       * @param {string=} min Sets the `min` validation error key if the value entered is less than `min`.
       *   This must be a valid ISO datetime format (yyyy-MM-ddTHH:mm:ss). You can also use interpolation
       *   inside this attribute (e.g. `min="{{minDatetimeLocal | date:'yyyy-MM-ddTHH:mm:ss'}}"`).
       *   Note that `min` will also add native HTML5 constraint validation.
       * @param {string=} max Sets the `max` validation error key if the value entered is greater than `max`.
       *   This must be a valid ISO datetime format (yyyy-MM-ddTHH:mm:ss). You can also use interpolation
       *   inside this attribute (e.g. `max="{{maxDatetimeLocal | date:'yyyy-MM-ddTHH:mm:ss'}}"`).
       *   Note that `max` will also add native HTML5 constraint validation.
       * @param {(date|string)=} ngMin Sets the `min` validation error key to the Date / ISO datetime string
       *   the `ngMin` expression evaluates to. Note that it does not set the `min` attribute.
       * @param {(date|string)=} ngMax Sets the `max` validation error key to the Date / ISO datetime string
       *   the `ngMax` expression evaluates to. Note that it does not set the `max` attribute.
       * @param {string=} required Sets `required` validation error key if the value is not entered.
       * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
       *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
       *    `required` when you want to data-bind to the `required` attribute.
       * @param {string=} ngChange Angular expression to be executed when input changes due to user
       *    interaction with the input element.
       *
       * @example
       <example name="datetimelocal-input-directive" module="dateExample">
       <file name="index.html">
         <script>
           angular.module('dateExample', [])
             .controller('DateController', ['$scope', function($scope) {
               $scope.example = {
                 value: new Date(2010, 11, 28, 14, 57)
               };
             }]);
         </script>
         <form name="myForm" ng-controller="DateController as dateCtrl">
           <label for="exampleInput">Pick a date between in 2013:</label>
           <input type="datetime-local" id="exampleInput" name="input" ng-model="example.value"
               placeholder="yyyy-MM-ddTHH:mm:ss" min="2001-01-01T00:00:00" max="2013-12-31T00:00:00" required />
           <div role="alert">
             <span class="error" ng-show="myForm.input.$error.required">
                 Required!</span>
             <span class="error" ng-show="myForm.input.$error.datetimelocal">
                 Not a valid date!</span>
           </div>
           <tt>value = {{example.value | date: "yyyy-MM-ddTHH:mm:ss"}}</tt><br/>
           <tt>myForm.input.$valid = {{myForm.input.$valid}}</tt><br/>
           <tt>myForm.input.$error = {{myForm.input.$error}}</tt><br/>
           <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
           <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
         </form>
       </file>
       <file name="protractor.js" type="protractor">
         var value = element(by.binding('example.value | date: "yyyy-MM-ddTHH:mm:ss"'));
         var valid = element(by.binding('myForm.input.$valid'));
          // currently protractor/webdriver does not support
         // sending keys to all known HTML5 input controls
         // for various browsers (https://github.com/angular/protractor/issues/562).
         function setInput(val) {
           // set the value of the element and force validation.
           var scr = "var ipt = document.getElementById('exampleInput'); " +
           "ipt.value = '" + val + "';" +
           "angular.element(ipt).scope().$apply(function(s) { s.myForm[ipt.name].$setViewValue('" + val + "'); });";
           browser.executeScript(scr);
         }
          it('should initialize to model', function() {
           expect(value.getText()).toContain('2010-12-28T14:57:00');
           expect(valid.getText()).toContain('myForm.input.$valid = true');
         });
          it('should be invalid if empty', function() {
           setInput('');
           expect(value.getText()).toEqual('value =');
           expect(valid.getText()).toContain('myForm.input.$valid = false');
         });
          it('should be invalid if over max', function() {
           setInput('2015-01-01T23:59:00');
           expect(value.getText()).toContain('');
           expect(valid.getText()).toContain('myForm.input.$valid = false');
         });
       </file>
       </example>
       */
      'datetime-local': createDateInputType('datetimelocal', DATETIMELOCAL_REGEXP, createDateParser(DATETIMELOCAL_REGEXP, ['yyyy', 'MM', 'dd', 'HH', 'mm', 'ss', 'sss']), 'yyyy-MM-ddTHH:mm:ss.sss'),

      /**
       * @ngdoc input
       * @name input[time]
       *
       * @description
       * Input with time validation and transformation. In browsers that do not yet support
       * the HTML5 time input, a text element will be used. In that case, the text must be entered in a valid ISO-8601
       * local time format (HH:mm:ss), for example: `14:57:00`. Model must be a Date object. This binding will always output a
       * Date object to the model of January 1, 1970, or local date `new Date(1970, 0, 1, HH, mm, ss)`.
       *
       * The model must always be a Date object, otherwise Angular will throw an error.
       * Invalid `Date` objects (dates whose `getTime()` is `NaN`) will be rendered as an empty string.
       *
       * The timezone to be used to read/write the `Date` instance in the model can be defined using
       * {@link ng.directive:ngModelOptions ngModelOptions}. By default, this is the timezone of the browser.
       *
       * @param {string} ngModel Assignable angular expression to data-bind to.
       * @param {string=} name Property name of the form under which the control is published.
       * @param {string=} min Sets the `min` validation error key if the value entered is less than `min`.
       *   This must be a valid ISO time format (HH:mm:ss). You can also use interpolation inside this
       *   attribute (e.g. `min="{{minTime | date:'HH:mm:ss'}}"`). Note that `min` will also add
       *   native HTML5 constraint validation.
       * @param {string=} max Sets the `max` validation error key if the value entered is greater than `max`.
       *   This must be a valid ISO time format (HH:mm:ss). You can also use interpolation inside this
       *   attribute (e.g. `max="{{maxTime | date:'HH:mm:ss'}}"`). Note that `max` will also add
       *   native HTML5 constraint validation.
       * @param {(date|string)=} ngMin Sets the `min` validation constraint to the Date / ISO time string the
       *   `ngMin` expression evaluates to. Note that it does not set the `min` attribute.
       * @param {(date|string)=} ngMax Sets the `max` validation constraint to the Date / ISO time string the
       *   `ngMax` expression evaluates to. Note that it does not set the `max` attribute.
       * @param {string=} required Sets `required` validation error key if the value is not entered.
       * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
       *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
       *    `required` when you want to data-bind to the `required` attribute.
       * @param {string=} ngChange Angular expression to be executed when input changes due to user
       *    interaction with the input element.
       *
       * @example
       <example name="time-input-directive" module="timeExample">
       <file name="index.html">
         <script>
          angular.module('timeExample', [])
            .controller('DateController', ['$scope', function($scope) {
              $scope.example = {
                value: new Date(1970, 0, 1, 14, 57, 0)
              };
            }]);
         </script>
         <form name="myForm" ng-controller="DateController as dateCtrl">
            <label for="exampleInput">Pick a time between 8am and 5pm:</label>
            <input type="time" id="exampleInput" name="input" ng-model="example.value"
                placeholder="HH:mm:ss" min="08:00:00" max="17:00:00" required />
            <div role="alert">
              <span class="error" ng-show="myForm.input.$error.required">
                  Required!</span>
              <span class="error" ng-show="myForm.input.$error.time">
                  Not a valid date!</span>
            </div>
            <tt>value = {{example.value | date: "HH:mm:ss"}}</tt><br/>
            <tt>myForm.input.$valid = {{myForm.input.$valid}}</tt><br/>
            <tt>myForm.input.$error = {{myForm.input.$error}}</tt><br/>
            <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
            <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
         </form>
       </file>
       <file name="protractor.js" type="protractor">
          var value = element(by.binding('example.value | date: "HH:mm:ss"'));
          var valid = element(by.binding('myForm.input.$valid'));
           // currently protractor/webdriver does not support
          // sending keys to all known HTML5 input controls
          // for various browsers (https://github.com/angular/protractor/issues/562).
          function setInput(val) {
            // set the value of the element and force validation.
            var scr = "var ipt = document.getElementById('exampleInput'); " +
            "ipt.value = '" + val + "';" +
            "angular.element(ipt).scope().$apply(function(s) { s.myForm[ipt.name].$setViewValue('" + val + "'); });";
            browser.executeScript(scr);
          }
           it('should initialize to model', function() {
            expect(value.getText()).toContain('14:57:00');
            expect(valid.getText()).toContain('myForm.input.$valid = true');
          });
           it('should be invalid if empty', function() {
            setInput('');
            expect(value.getText()).toEqual('value =');
            expect(valid.getText()).toContain('myForm.input.$valid = false');
          });
           it('should be invalid if over max', function() {
            setInput('23:59:00');
            expect(value.getText()).toContain('');
            expect(valid.getText()).toContain('myForm.input.$valid = false');
          });
       </file>
       </example>
       */
      'time': createDateInputType('time', TIME_REGEXP, createDateParser(TIME_REGEXP, ['HH', 'mm', 'ss', 'sss']), 'HH:mm:ss.sss'),

      /**
       * @ngdoc input
       * @name input[week]
       *
       * @description
       * Input with week-of-the-year validation and transformation to Date. In browsers that do not yet support
       * the HTML5 week input, a text element will be used. In that case, the text must be entered in a valid ISO-8601
       * week format (yyyy-W##), for example: `2013-W02`.
       *
       * The model must always be a Date object, otherwise Angular will throw an error.
       * Invalid `Date` objects (dates whose `getTime()` is `NaN`) will be rendered as an empty string.
       *
       * The timezone to be used to read/write the `Date` instance in the model can be defined using
       * {@link ng.directive:ngModelOptions ngModelOptions}. By default, this is the timezone of the browser.
       *
       * @param {string} ngModel Assignable angular expression to data-bind to.
       * @param {string=} name Property name of the form under which the control is published.
       * @param {string=} min Sets the `min` validation error key if the value entered is less than `min`.
       *   This must be a valid ISO week format (yyyy-W##). You can also use interpolation inside this
       *   attribute (e.g. `min="{{minWeek | date:'yyyy-Www'}}"`). Note that `min` will also add
       *   native HTML5 constraint validation.
       * @param {string=} max Sets the `max` validation error key if the value entered is greater than `max`.
       *   This must be a valid ISO week format (yyyy-W##). You can also use interpolation inside this
       *   attribute (e.g. `max="{{maxWeek | date:'yyyy-Www'}}"`). Note that `max` will also add
       *   native HTML5 constraint validation.
       * @param {(date|string)=} ngMin Sets the `min` validation constraint to the Date / ISO week string
       *   the `ngMin` expression evaluates to. Note that it does not set the `min` attribute.
       * @param {(date|string)=} ngMax Sets the `max` validation constraint to the Date / ISO week string
       *   the `ngMax` expression evaluates to. Note that it does not set the `max` attribute.
       * @param {string=} required Sets `required` validation error key if the value is not entered.
       * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
       *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
       *    `required` when you want to data-bind to the `required` attribute.
       * @param {string=} ngChange Angular expression to be executed when input changes due to user
       *    interaction with the input element.
       *
       * @example
       <example name="week-input-directive" module="weekExample">
       <file name="index.html">
         <script>
         angular.module('weekExample', [])
           .controller('DateController', ['$scope', function($scope) {
             $scope.example = {
               value: new Date(2013, 0, 3)
             };
           }]);
         </script>
         <form name="myForm" ng-controller="DateController as dateCtrl">
           <label>Pick a date between in 2013:
             <input id="exampleInput" type="week" name="input" ng-model="example.value"
                    placeholder="YYYY-W##" min="2012-W32"
                    max="2013-W52" required />
           </label>
           <div role="alert">
             <span class="error" ng-show="myForm.input.$error.required">
                 Required!</span>
             <span class="error" ng-show="myForm.input.$error.week">
                 Not a valid date!</span>
           </div>
           <tt>value = {{example.value | date: "yyyy-Www"}}</tt><br/>
           <tt>myForm.input.$valid = {{myForm.input.$valid}}</tt><br/>
           <tt>myForm.input.$error = {{myForm.input.$error}}</tt><br/>
           <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
           <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
         </form>
       </file>
       <file name="protractor.js" type="protractor">
         var value = element(by.binding('example.value | date: "yyyy-Www"'));
         var valid = element(by.binding('myForm.input.$valid'));
          // currently protractor/webdriver does not support
         // sending keys to all known HTML5 input controls
         // for various browsers (https://github.com/angular/protractor/issues/562).
         function setInput(val) {
           // set the value of the element and force validation.
           var scr = "var ipt = document.getElementById('exampleInput'); " +
           "ipt.value = '" + val + "';" +
           "angular.element(ipt).scope().$apply(function(s) { s.myForm[ipt.name].$setViewValue('" + val + "'); });";
           browser.executeScript(scr);
         }
          it('should initialize to model', function() {
           expect(value.getText()).toContain('2013-W01');
           expect(valid.getText()).toContain('myForm.input.$valid = true');
         });
          it('should be invalid if empty', function() {
           setInput('');
           expect(value.getText()).toEqual('value =');
           expect(valid.getText()).toContain('myForm.input.$valid = false');
         });
          it('should be invalid if over max', function() {
           setInput('2015-W01');
           expect(value.getText()).toContain('');
           expect(valid.getText()).toContain('myForm.input.$valid = false');
         });
       </file>
       </example>
       */
      'week': createDateInputType('week', WEEK_REGEXP, weekParser, 'yyyy-Www'),

      /**
       * @ngdoc input
       * @name input[month]
       *
       * @description
       * Input with month validation and transformation. In browsers that do not yet support
       * the HTML5 month input, a text element will be used. In that case, the text must be entered in a valid ISO-8601
       * month format (yyyy-MM), for example: `2009-01`.
       *
       * The model must always be a Date object, otherwise Angular will throw an error.
       * Invalid `Date` objects (dates whose `getTime()` is `NaN`) will be rendered as an empty string.
       * If the model is not set to the first of the month, the next view to model update will set it
       * to the first of the month.
       *
       * The timezone to be used to read/write the `Date` instance in the model can be defined using
       * {@link ng.directive:ngModelOptions ngModelOptions}. By default, this is the timezone of the browser.
       *
       * @param {string} ngModel Assignable angular expression to data-bind to.
       * @param {string=} name Property name of the form under which the control is published.
       * @param {string=} min Sets the `min` validation error key if the value entered is less than `min`.
       *   This must be a valid ISO month format (yyyy-MM). You can also use interpolation inside this
       *   attribute (e.g. `min="{{minMonth | date:'yyyy-MM'}}"`). Note that `min` will also add
       *   native HTML5 constraint validation.
       * @param {string=} max Sets the `max` validation error key if the value entered is greater than `max`.
       *   This must be a valid ISO month format (yyyy-MM). You can also use interpolation inside this
       *   attribute (e.g. `max="{{maxMonth | date:'yyyy-MM'}}"`). Note that `max` will also add
       *   native HTML5 constraint validation.
       * @param {(date|string)=} ngMin Sets the `min` validation constraint to the Date / ISO week string
       *   the `ngMin` expression evaluates to. Note that it does not set the `min` attribute.
       * @param {(date|string)=} ngMax Sets the `max` validation constraint to the Date / ISO week string
       *   the `ngMax` expression evaluates to. Note that it does not set the `max` attribute.
        * @param {string=} required Sets `required` validation error key if the value is not entered.
       * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
       *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
       *    `required` when you want to data-bind to the `required` attribute.
       * @param {string=} ngChange Angular expression to be executed when input changes due to user
       *    interaction with the input element.
       *
       * @example
       <example name="month-input-directive" module="monthExample">
       <file name="index.html">
         <script>
          angular.module('monthExample', [])
            .controller('DateController', ['$scope', function($scope) {
              $scope.example = {
                value: new Date(2013, 9, 1)
              };
            }]);
         </script>
         <form name="myForm" ng-controller="DateController as dateCtrl">
           <label for="exampleInput">Pick a month in 2013:</label>
           <input id="exampleInput" type="month" name="input" ng-model="example.value"
              placeholder="yyyy-MM" min="2013-01" max="2013-12" required />
           <div role="alert">
             <span class="error" ng-show="myForm.input.$error.required">
                Required!</span>
             <span class="error" ng-show="myForm.input.$error.month">
                Not a valid month!</span>
           </div>
           <tt>value = {{example.value | date: "yyyy-MM"}}</tt><br/>
           <tt>myForm.input.$valid = {{myForm.input.$valid}}</tt><br/>
           <tt>myForm.input.$error = {{myForm.input.$error}}</tt><br/>
           <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
           <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
         </form>
       </file>
       <file name="protractor.js" type="protractor">
          var value = element(by.binding('example.value | date: "yyyy-MM"'));
          var valid = element(by.binding('myForm.input.$valid'));
           // currently protractor/webdriver does not support
          // sending keys to all known HTML5 input controls
          // for various browsers (https://github.com/angular/protractor/issues/562).
          function setInput(val) {
            // set the value of the element and force validation.
            var scr = "var ipt = document.getElementById('exampleInput'); " +
            "ipt.value = '" + val + "';" +
            "angular.element(ipt).scope().$apply(function(s) { s.myForm[ipt.name].$setViewValue('" + val + "'); });";
            browser.executeScript(scr);
          }
           it('should initialize to model', function() {
            expect(value.getText()).toContain('2013-10');
            expect(valid.getText()).toContain('myForm.input.$valid = true');
          });
           it('should be invalid if empty', function() {
            setInput('');
            expect(value.getText()).toEqual('value =');
            expect(valid.getText()).toContain('myForm.input.$valid = false');
          });
           it('should be invalid if over max', function() {
            setInput('2015-01');
            expect(value.getText()).toContain('');
            expect(valid.getText()).toContain('myForm.input.$valid = false');
          });
       </file>
       </example>
       */
      'month': createDateInputType('month', MONTH_REGEXP, createDateParser(MONTH_REGEXP, ['yyyy', 'MM']), 'yyyy-MM'),

      /**
       * @ngdoc input
       * @name input[number]
       *
       * @description
       * Text input with number validation and transformation. Sets the `number` validation
       * error if not a valid number.
       *
       * <div class="alert alert-warning">
       * The model must always be of type `number` otherwise Angular will throw an error.
       * Be aware that a string containing a number is not enough. See the {@link ngModel:numfmt}
       * error docs for more information and an example of how to convert your model if necessary.
       * </div>
       *
       * ## Issues with HTML5 constraint validation
       *
       * In browsers that follow the
       * [HTML5 specification](https://html.spec.whatwg.org/multipage/forms.html#number-state-%28type=number%29),
       * `input[number]` does not work as expected with {@link ngModelOptions `ngModelOptions.allowInvalid`}.
       * If a non-number is entered in the input, the browser will report the value as an empty string,
       * which means the view / model values in `ngModel` and subsequently the scope value
       * will also be an empty string.
       *
       *
       * @param {string} ngModel Assignable angular expression to data-bind to.
       * @param {string=} name Property name of the form under which the control is published.
       * @param {string=} min Sets the `min` validation error key if the value entered is less than `min`.
       *    Can be interpolated.
       * @param {string=} max Sets the `max` validation error key if the value entered is greater than `max`.
       *    Can be interpolated.
       * @param {string=} ngMin Like `min`, sets the `min` validation error key if the value entered is less than `ngMin`,
       *    but does not trigger HTML5 native validation. Takes an expression.
       * @param {string=} ngMax Like `max`, sets the `max` validation error key if the value entered is greater than `ngMax`,
       *    but does not trigger HTML5 native validation. Takes an expression.
       * @param {string=} step Sets the `step` validation error key if the value entered does not fit the `step` constraint.
       *    Can be interpolated.
       * @param {string=} ngStep Like `step`, sets the `step` validation error key if the value entered does not fit the `ngStep` constraint,
       *    but does not trigger HTML5 native validation. Takes an expression.
       * @param {string=} required Sets `required` validation error key if the value is not entered.
       * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
       *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
       *    `required` when you want to data-bind to the `required` attribute.
       * @param {number=} ngMinlength Sets `minlength` validation error key if the value is shorter than
       *    minlength.
       * @param {number=} ngMaxlength Sets `maxlength` validation error key if the value is longer than
       *    maxlength. Setting the attribute to a negative or non-numeric value, allows view values of
       *    any length.
       * @param {string=} pattern Similar to `ngPattern` except that the attribute value is the actual string
       *    that contains the regular expression body that will be converted to a regular expression
       *    as in the ngPattern directive.
       * @param {string=} ngPattern Sets `pattern` validation error key if the ngModel {@link ngModel.NgModelController#$viewValue $viewValue}
       *    does not match a RegExp found by evaluating the Angular expression given in the attribute value.
       *    If the expression evaluates to a RegExp object, then this is used directly.
       *    If the expression evaluates to a string, then it will be converted to a RegExp
       *    after wrapping it in `^` and `$` characters. For instance, `"abc"` will be converted to
       *    `new RegExp('^abc$')`.<br />
       *    **Note:** Avoid using the `g` flag on the RegExp, as it will cause each successive search to
       *    start at the index of the last search's match, thus not taking the whole input value into
       *    account.
       * @param {string=} ngChange Angular expression to be executed when input changes due to user
       *    interaction with the input element.
       *
       * @example
          <example name="number-input-directive" module="numberExample">
            <file name="index.html">
             <script>
               angular.module('numberExample', [])
                 .controller('ExampleController', ['$scope', function($scope) {
                   $scope.example = {
                     value: 12
                   };
                 }]);
             </script>
             <form name="myForm" ng-controller="ExampleController">
               <label>Number:
                 <input type="number" name="input" ng-model="example.value"
                        min="0" max="99" required>
              </label>
               <div role="alert">
                 <span class="error" ng-show="myForm.input.$error.required">
                   Required!</span>
                 <span class="error" ng-show="myForm.input.$error.number">
                   Not valid number!</span>
               </div>
               <tt>value = {{example.value}}</tt><br/>
               <tt>myForm.input.$valid = {{myForm.input.$valid}}</tt><br/>
               <tt>myForm.input.$error = {{myForm.input.$error}}</tt><br/>
               <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
               <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
              </form>
            </file>
            <file name="protractor.js" type="protractor">
              var value = element(by.binding('example.value'));
              var valid = element(by.binding('myForm.input.$valid'));
              var input = element(by.model('example.value'));
               it('should initialize to model', function() {
                expect(value.getText()).toContain('12');
                expect(valid.getText()).toContain('true');
              });
               it('should be invalid if empty', function() {
                input.clear();
                input.sendKeys('');
                expect(value.getText()).toEqual('value =');
                expect(valid.getText()).toContain('false');
              });
               it('should be invalid if over max', function() {
                input.clear();
                input.sendKeys('123');
                expect(value.getText()).toEqual('value =');
                expect(valid.getText()).toContain('false');
              });
            </file>
          </example>
       */
      'number': numberInputType,

      /**
       * @ngdoc input
       * @name input[url]
       *
       * @description
       * Text input with URL validation. Sets the `url` validation error key if the content is not a
       * valid URL.
       *
       * <div class="alert alert-warning">
       * **Note:** `input[url]` uses a regex to validate urls that is derived from the regex
       * used in Chromium. If you need stricter validation, you can use `ng-pattern` or modify
       * the built-in validators (see the {@link guide/forms Forms guide})
       * </div>
       *
       * @param {string} ngModel Assignable angular expression to data-bind to.
       * @param {string=} name Property name of the form under which the control is published.
       * @param {string=} required Sets `required` validation error key if the value is not entered.
       * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
       *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
       *    `required` when you want to data-bind to the `required` attribute.
       * @param {number=} ngMinlength Sets `minlength` validation error key if the value is shorter than
       *    minlength.
       * @param {number=} ngMaxlength Sets `maxlength` validation error key if the value is longer than
       *    maxlength. Setting the attribute to a negative or non-numeric value, allows view values of
       *    any length.
       * @param {string=} pattern Similar to `ngPattern` except that the attribute value is the actual string
       *    that contains the regular expression body that will be converted to a regular expression
       *    as in the ngPattern directive.
       * @param {string=} ngPattern Sets `pattern` validation error key if the ngModel {@link ngModel.NgModelController#$viewValue $viewValue}
       *    does not match a RegExp found by evaluating the Angular expression given in the attribute value.
       *    If the expression evaluates to a RegExp object, then this is used directly.
       *    If the expression evaluates to a string, then it will be converted to a RegExp
       *    after wrapping it in `^` and `$` characters. For instance, `"abc"` will be converted to
       *    `new RegExp('^abc$')`.<br />
       *    **Note:** Avoid using the `g` flag on the RegExp, as it will cause each successive search to
       *    start at the index of the last search's match, thus not taking the whole input value into
       *    account.
       * @param {string=} ngChange Angular expression to be executed when input changes due to user
       *    interaction with the input element.
       *
       * @example
          <example name="url-input-directive" module="urlExample">
            <file name="index.html">
             <script>
               angular.module('urlExample', [])
                 .controller('ExampleController', ['$scope', function($scope) {
                   $scope.url = {
                     text: 'http://google.com'
                   };
                 }]);
             </script>
             <form name="myForm" ng-controller="ExampleController">
               <label>URL:
                 <input type="url" name="input" ng-model="url.text" required>
               <label>
               <div role="alert">
                 <span class="error" ng-show="myForm.input.$error.required">
                   Required!</span>
                 <span class="error" ng-show="myForm.input.$error.url">
                   Not valid url!</span>
               </div>
               <tt>text = {{url.text}}</tt><br/>
               <tt>myForm.input.$valid = {{myForm.input.$valid}}</tt><br/>
               <tt>myForm.input.$error = {{myForm.input.$error}}</tt><br/>
               <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
               <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
               <tt>myForm.$error.url = {{!!myForm.$error.url}}</tt><br/>
              </form>
            </file>
            <file name="protractor.js" type="protractor">
              var text = element(by.binding('url.text'));
              var valid = element(by.binding('myForm.input.$valid'));
              var input = element(by.model('url.text'));
               it('should initialize to model', function() {
                expect(text.getText()).toContain('http://google.com');
                expect(valid.getText()).toContain('true');
              });
               it('should be invalid if empty', function() {
                input.clear();
                input.sendKeys('');
                 expect(text.getText()).toEqual('text =');
                expect(valid.getText()).toContain('false');
              });
               it('should be invalid if not url', function() {
                input.clear();
                input.sendKeys('box');
                 expect(valid.getText()).toContain('false');
              });
            </file>
          </example>
       */
      'url': urlInputType,

      /**
       * @ngdoc input
       * @name input[email]
       *
       * @description
       * Text input with email validation. Sets the `email` validation error key if not a valid email
       * address.
       *
       * <div class="alert alert-warning">
       * **Note:** `input[email]` uses a regex to validate email addresses that is derived from the regex
       * used in Chromium. If you need stricter validation (e.g. requiring a top-level domain), you can
       * use `ng-pattern` or modify the built-in validators (see the {@link guide/forms Forms guide})
       * </div>
       *
       * @param {string} ngModel Assignable angular expression to data-bind to.
       * @param {string=} name Property name of the form under which the control is published.
       * @param {string=} required Sets `required` validation error key if the value is not entered.
       * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
       *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
       *    `required` when you want to data-bind to the `required` attribute.
       * @param {number=} ngMinlength Sets `minlength` validation error key if the value is shorter than
       *    minlength.
       * @param {number=} ngMaxlength Sets `maxlength` validation error key if the value is longer than
       *    maxlength. Setting the attribute to a negative or non-numeric value, allows view values of
       *    any length.
       * @param {string=} pattern Similar to `ngPattern` except that the attribute value is the actual string
       *    that contains the regular expression body that will be converted to a regular expression
       *    as in the ngPattern directive.
       * @param {string=} ngPattern Sets `pattern` validation error key if the ngModel {@link ngModel.NgModelController#$viewValue $viewValue}
       *    does not match a RegExp found by evaluating the Angular expression given in the attribute value.
       *    If the expression evaluates to a RegExp object, then this is used directly.
       *    If the expression evaluates to a string, then it will be converted to a RegExp
       *    after wrapping it in `^` and `$` characters. For instance, `"abc"` will be converted to
       *    `new RegExp('^abc$')`.<br />
       *    **Note:** Avoid using the `g` flag on the RegExp, as it will cause each successive search to
       *    start at the index of the last search's match, thus not taking the whole input value into
       *    account.
       * @param {string=} ngChange Angular expression to be executed when input changes due to user
       *    interaction with the input element.
       *
       * @example
          <example name="email-input-directive" module="emailExample">
            <file name="index.html">
             <script>
               angular.module('emailExample', [])
                 .controller('ExampleController', ['$scope', function($scope) {
                   $scope.email = {
                     text: 'me@example.com'
                   };
                 }]);
             </script>
               <form name="myForm" ng-controller="ExampleController">
                 <label>Email:
                   <input type="email" name="input" ng-model="email.text" required>
                 </label>
                 <div role="alert">
                   <span class="error" ng-show="myForm.input.$error.required">
                     Required!</span>
                   <span class="error" ng-show="myForm.input.$error.email">
                     Not valid email!</span>
                 </div>
                 <tt>text = {{email.text}}</tt><br/>
                 <tt>myForm.input.$valid = {{myForm.input.$valid}}</tt><br/>
                 <tt>myForm.input.$error = {{myForm.input.$error}}</tt><br/>
                 <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
                 <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
                 <tt>myForm.$error.email = {{!!myForm.$error.email}}</tt><br/>
               </form>
             </file>
            <file name="protractor.js" type="protractor">
              var text = element(by.binding('email.text'));
              var valid = element(by.binding('myForm.input.$valid'));
              var input = element(by.model('email.text'));
               it('should initialize to model', function() {
                expect(text.getText()).toContain('me@example.com');
                expect(valid.getText()).toContain('true');
              });
               it('should be invalid if empty', function() {
                input.clear();
                input.sendKeys('');
                expect(text.getText()).toEqual('text =');
                expect(valid.getText()).toContain('false');
              });
               it('should be invalid if not email', function() {
                input.clear();
                input.sendKeys('xxx');
                 expect(valid.getText()).toContain('false');
              });
            </file>
          </example>
       */
      'email': emailInputType,

      /**
       * @ngdoc input
       * @name input[radio]
       *
       * @description
       * HTML radio button.
       *
       * @param {string} ngModel Assignable angular expression to data-bind to.
       * @param {string} value The value to which the `ngModel` expression should be set when selected.
       *    Note that `value` only supports `string` values, i.e. the scope model needs to be a string,
       *    too. Use `ngValue` if you need complex models (`number`, `object`, ...).
       * @param {string=} name Property name of the form under which the control is published.
       * @param {string=} ngChange Angular expression to be executed when input changes due to user
       *    interaction with the input element.
       * @param {string} ngValue Angular expression to which `ngModel` will be be set when the radio
       *    is selected. Should be used instead of the `value` attribute if you need
       *    a non-string `ngModel` (`boolean`, `array`, ...).
       *
       * @example
          <example name="radio-input-directive" module="radioExample">
            <file name="index.html">
             <script>
               angular.module('radioExample', [])
                 .controller('ExampleController', ['$scope', function($scope) {
                   $scope.color = {
                     name: 'blue'
                   };
                   $scope.specialValue = {
                     "id": "12345",
                     "value": "green"
                   };
                 }]);
             </script>
             <form name="myForm" ng-controller="ExampleController">
               <label>
                 <input type="radio" ng-model="color.name" value="red">
                 Red
               </label><br/>
               <label>
                 <input type="radio" ng-model="color.name" ng-value="specialValue">
                 Green
               </label><br/>
               <label>
                 <input type="radio" ng-model="color.name" value="blue">
                 Blue
               </label><br/>
               <tt>color = {{color.name | json}}</tt><br/>
              </form>
              Note that `ng-value="specialValue"` sets radio item's value to be the value of `$scope.specialValue`.
            </file>
            <file name="protractor.js" type="protractor">
              it('should change state', function() {
                var inputs = element.all(by.model('color.name'));
                var color = element(by.binding('color.name'));
                 expect(color.getText()).toContain('blue');
                 inputs.get(0).click();
                expect(color.getText()).toContain('red');
                 inputs.get(1).click();
                expect(color.getText()).toContain('green');
              });
            </file>
          </example>
       */
      'radio': radioInputType,

      /**
       * @ngdoc input
       * @name input[range]
       *
       * @description
       * Native range input with validation and transformation.
       *
       * The model for the range input must always be a `Number`.
       *
       * IE9 and other browsers that do not support the `range` type fall back
       * to a text input without any default values for `min`, `max` and `step`. Model binding,
       * validation and number parsing are nevertheless supported.
       *
       * Browsers that support range (latest Chrome, Safari, Firefox, Edge) treat `input[range]`
       * in a way that never allows the input to hold an invalid value. That means:
       * - any non-numerical value is set to `(max + min) / 2`.
       * - any numerical value that is less than the current min val, or greater than the current max val
       * is set to the min / max val respectively.
       * - additionally, the current `step` is respected, so the nearest value that satisfies a step
       * is used.
       *
       * See the [HTML Spec on input[type=range]](https://www.w3.org/TR/html5/forms.html#range-state-(type=range))
       * for more info.
       *
       * This has the following consequences for Angular:
       *
       * Since the element value should always reflect the current model value, a range input
       * will set the bound ngModel expression to the value that the browser has set for the
       * input element. For example, in the following input `<input type="range" ng-model="model.value">`,
       * if the application sets `model.value = null`, the browser will set the input to `'50'`.
       * Angular will then set the model to `50`, to prevent input and model value being out of sync.
       *
       * That means the model for range will immediately be set to `50` after `ngModel` has been
       * initialized. It also means a range input can never have the required error.
       *
       * This does not only affect changes to the model value, but also to the values of the `min`,
       * `max`, and `step` attributes. When these change in a way that will cause the browser to modify
       * the input value, Angular will also update the model value.
       *
       * Automatic value adjustment also means that a range input element can never have the `required`,
       * `min`, or `max` errors.
       *
       * However, `step` is currently only fully implemented by Firefox. Other browsers have problems
       * when the step value changes dynamically - they do not adjust the element value correctly, but
       * instead may set the `stepMismatch` error. If that's the case, the Angular will set the `step`
       * error on the input, and set the model to `undefined`.
       *
       * Note that `input[range]` is not compatible with`ngMax`, `ngMin`, and `ngStep`, because they do
       * not set the `min` and `max` attributes, which means that the browser won't automatically adjust
       * the input value based on their values, and will always assume min = 0, max = 100, and step = 1.
       *
       * @param {string}  ngModel Assignable angular expression to data-bind to.
       * @param {string=} name Property name of the form under which the control is published.
       * @param {string=} min Sets the `min` validation to ensure that the value entered is greater
       *                  than `min`. Can be interpolated.
       * @param {string=} max Sets the `max` validation to ensure that the value entered is less than `max`.
       *                  Can be interpolated.
       * @param {string=} step Sets the `step` validation to ensure that the value entered matches the `step`
       *                  Can be interpolated.
       * @param {string=} ngChange Angular expression to be executed when the ngModel value changes due
       *                  to user interaction with the input element.
       * @param {expression=} ngChecked If the expression is truthy, then the `checked` attribute will be set on the
       *                      element. **Note** : `ngChecked` should not be used alongside `ngModel`.
       *                      Checkout {@link ng.directive:ngChecked ngChecked} for usage.
       *
       * @example
          <example name="range-input-directive" module="rangeExample">
            <file name="index.html">
              <script>
                angular.module('rangeExample', [])
                  .controller('ExampleController', ['$scope', function($scope) {
                    $scope.value = 75;
                    $scope.min = 10;
                    $scope.max = 90;
                  }]);
              </script>
              <form name="myForm" ng-controller="ExampleController">
                 Model as range: <input type="range" name="range" ng-model="value" min="{{min}}"  max="{{max}}">
                <hr>
                Model as number: <input type="number" ng-model="value"><br>
                Min: <input type="number" ng-model="min"><br>
                Max: <input type="number" ng-model="max"><br>
                value = <code>{{value}}</code><br/>
                myForm.range.$valid = <code>{{myForm.range.$valid}}</code><br/>
                myForm.range.$error = <code>{{myForm.range.$error}}</code>
              </form>
            </file>
          </example>
        * ## Range Input with ngMin & ngMax attributes
        * @example
          <example name="range-input-directive-ng" module="rangeExample">
            <file name="index.html">
              <script>
                angular.module('rangeExample', [])
                  .controller('ExampleController', ['$scope', function($scope) {
                    $scope.value = 75;
                    $scope.min = 10;
                    $scope.max = 90;
                  }]);
              </script>
              <form name="myForm" ng-controller="ExampleController">
                Model as range: <input type="range" name="range" ng-model="value" ng-min="min" ng-max="max">
                <hr>
                Model as number: <input type="number" ng-model="value"><br>
                Min: <input type="number" ng-model="min"><br>
                Max: <input type="number" ng-model="max"><br>
                value = <code>{{value}}</code><br/>
                myForm.range.$valid = <code>{{myForm.range.$valid}}</code><br/>
                myForm.range.$error = <code>{{myForm.range.$error}}</code>
              </form>
            </file>
          </example>
        */
      'range': rangeInputType,

      /**
       * @ngdoc input
       * @name input[checkbox]
       *
       * @description
       * HTML checkbox.
       *
       * @param {string} ngModel Assignable angular expression to data-bind to.
       * @param {string=} name Property name of the form under which the control is published.
       * @param {expression=} ngTrueValue The value to which the expression should be set when selected.
       * @param {expression=} ngFalseValue The value to which the expression should be set when not selected.
       * @param {string=} ngChange Angular expression to be executed when input changes due to user
       *    interaction with the input element.
       *
       * @example
          <example name="checkbox-input-directive" module="checkboxExample">
            <file name="index.html">
             <script>
               angular.module('checkboxExample', [])
                 .controller('ExampleController', ['$scope', function($scope) {
                   $scope.checkboxModel = {
                    value1 : true,
                    value2 : 'YES'
                  };
                 }]);
             </script>
             <form name="myForm" ng-controller="ExampleController">
               <label>Value1:
                 <input type="checkbox" ng-model="checkboxModel.value1">
               </label><br/>
               <label>Value2:
                 <input type="checkbox" ng-model="checkboxModel.value2"
                        ng-true-value="'YES'" ng-false-value="'NO'">
                </label><br/>
               <tt>value1 = {{checkboxModel.value1}}</tt><br/>
               <tt>value2 = {{checkboxModel.value2}}</tt><br/>
              </form>
            </file>
            <file name="protractor.js" type="protractor">
              it('should change state', function() {
                var value1 = element(by.binding('checkboxModel.value1'));
                var value2 = element(by.binding('checkboxModel.value2'));
                 expect(value1.getText()).toContain('true');
                expect(value2.getText()).toContain('YES');
                 element(by.model('checkboxModel.value1')).click();
                element(by.model('checkboxModel.value2')).click();
                 expect(value1.getText()).toContain('false');
                expect(value2.getText()).toContain('NO');
              });
            </file>
          </example>
       */
      'checkbox': checkboxInputType,

      'hidden': noop,
      'button': noop,
      'submit': noop,
      'reset': noop,
      'file': noop
    };

    function stringBasedInputType(ctrl) {
      ctrl.$formatters.push(function (value) {
        return ctrl.$isEmpty(value) ? value : value.toString();
      });
    }

    function textInputType(scope, element, attr, ctrl, $sniffer, $browser) {
      baseInputType(scope, element, attr, ctrl, $sniffer, $browser);
      stringBasedInputType(ctrl);
    }

    function baseInputType(scope, element, attr, ctrl, $sniffer, $browser) {
      var type = lowercase(element[0].type);

      // In composition mode, users are still inputting intermediate text buffer,
      // hold the listener until composition is done.
      // More about composition events: https://developer.mozilla.org/en-US/docs/Web/API/CompositionEvent
      if (!$sniffer.android) {
        var composing = false;

        element.on('compositionstart', function () {
          composing = true;
        });

        element.on('compositionend', function () {
          composing = false;
          listener();
        });
      }

      var timeout;

      var listener = function (ev) {
        if (timeout) {
          $browser.defer.cancel(timeout);
          timeout = null;
        }
        if (composing) return;
        var value = element.val(),
            event = ev && ev.type;

        // By default we will trim the value
        // If the attribute ng-trim exists we will avoid trimming
        // If input type is 'password', the value is never trimmed
        if (type !== 'password' && (!attr.ngTrim || attr.ngTrim !== 'false')) {
          value = trim(value);
        }

        // If a control is suffering from bad input (due to native validators), browsers discard its
        // value, so it may be necessary to revalidate (by calling $setViewValue again) even if the
        // control's value is the same empty value twice in a row.
        if (ctrl.$viewValue !== value || value === '' && ctrl.$$hasNativeValidators) {
          ctrl.$setViewValue(value, event);
        }
      };

      // if the browser does support "input" event, we are fine - except on IE9 which doesn't fire the
      // input event on backspace, delete or cut
      if ($sniffer.hasEvent('input')) {
        element.on('input', listener);
      } else {
        var deferListener = function (ev, input, origValue) {
          if (!timeout) {
            timeout = $browser.defer(function () {
              timeout = null;
              if (!input || input.value !== origValue) {
                listener(ev);
              }
            });
          }
        };

        element.on('keydown', /** @this */function (event) {
          var key = event.keyCode;

          // ignore
          //    command            modifiers                   arrows
          if (key === 91 || 15 < key && key < 19 || 37 <= key && key <= 40) return;

          deferListener(event, this, this.value);
        });

        // if user modifies input value using context menu in IE, we need "paste" and "cut" events to catch it
        if ($sniffer.hasEvent('paste')) {
          element.on('paste cut', deferListener);
        }
      }

      // if user paste into input using mouse on older browser
      // or form autocomplete on newer browser, we need "change" event to catch it
      element.on('change', listener);

      // Some native input types (date-family) have the ability to change validity without
      // firing any input/change events.
      // For these event types, when native validators are present and the browser supports the type,
      // check for validity changes on various DOM events.
      if (PARTIAL_VALIDATION_TYPES[type] && ctrl.$$hasNativeValidators && type === attr.type) {
        element.on(PARTIAL_VALIDATION_EVENTS, /** @this */function (ev) {
          if (!timeout) {
            var validity = this[VALIDITY_STATE_PROPERTY];
            var origBadInput = validity.badInput;
            var origTypeMismatch = validity.typeMismatch;
            timeout = $browser.defer(function () {
              timeout = null;
              if (validity.badInput !== origBadInput || validity.typeMismatch !== origTypeMismatch) {
                listener(ev);
              }
            });
          }
        });
      }

      ctrl.$render = function () {
        // Workaround for Firefox validation #12102.
        var value = ctrl.$isEmpty(ctrl.$viewValue) ? '' : ctrl.$viewValue;
        if (element.val() !== value) {
          element.val(value);
        }
      };
    }

    function weekParser(isoWeek, existingDate) {
      if (isDate(isoWeek)) {
        return isoWeek;
      }

      if (isString(isoWeek)) {
        WEEK_REGEXP.lastIndex = 0;
        var parts = WEEK_REGEXP.exec(isoWeek);
        if (parts) {
          var year = +parts[1],
              week = +parts[2],
              hours = 0,
              minutes = 0,
              seconds = 0,
              milliseconds = 0,
              firstThurs = getFirstThursdayOfYear(year),
              addDays = (week - 1) * 7;

          if (existingDate) {
            hours = existingDate.getHours();
            minutes = existingDate.getMinutes();
            seconds = existingDate.getSeconds();
            milliseconds = existingDate.getMilliseconds();
          }

          return new Date(year, 0, firstThurs.getDate() + addDays, hours, minutes, seconds, milliseconds);
        }
      }

      return NaN;
    }

    function createDateParser(regexp, mapping) {
      return function (iso, date) {
        var parts, map;

        if (isDate(iso)) {
          return iso;
        }

        if (isString(iso)) {
          // When a date is JSON'ified to wraps itself inside of an extra
          // set of double quotes. This makes the date parsing code unable
          // to match the date string and parse it as a date.
          if (iso.charAt(0) === '"' && iso.charAt(iso.length - 1) === '"') {
            iso = iso.substring(1, iso.length - 1);
          }
          if (ISO_DATE_REGEXP.test(iso)) {
            return new Date(iso);
          }
          regexp.lastIndex = 0;
          parts = regexp.exec(iso);

          if (parts) {
            parts.shift();
            if (date) {
              map = {
                yyyy: date.getFullYear(),
                MM: date.getMonth() + 1,
                dd: date.getDate(),
                HH: date.getHours(),
                mm: date.getMinutes(),
                ss: date.getSeconds(),
                sss: date.getMilliseconds() / 1000
              };
            } else {
              map = { yyyy: 1970, MM: 1, dd: 1, HH: 0, mm: 0, ss: 0, sss: 0 };
            }

            forEach(parts, function (part, index) {
              if (index < mapping.length) {
                map[mapping[index]] = +part;
              }
            });
            return new Date(map.yyyy, map.MM - 1, map.dd, map.HH, map.mm, map.ss || 0, map.sss * 1000 || 0);
          }
        }

        return NaN;
      };
    }

    function createDateInputType(type, regexp, parseDate, format) {
      return function dynamicDateInputType(scope, element, attr, ctrl, $sniffer, $browser, $filter) {
        badInputChecker(scope, element, attr, ctrl);
        baseInputType(scope, element, attr, ctrl, $sniffer, $browser);
        var timezone = ctrl && ctrl.$options.getOption('timezone');
        var previousDate;

        ctrl.$$parserName = type;
        ctrl.$parsers.push(function (value) {
          if (ctrl.$isEmpty(value)) return null;
          if (regexp.test(value)) {
            // Note: We cannot read ctrl.$modelValue, as there might be a different
            // parser/formatter in the processing chain so that the model
            // contains some different data format!
            var parsedDate = parseDate(value, previousDate);
            if (timezone) {
              parsedDate = convertTimezoneToLocal(parsedDate, timezone);
            }
            return parsedDate;
          }
          return undefined;
        });

        ctrl.$formatters.push(function (value) {
          if (value && !isDate(value)) {
            throw ngModelMinErr('datefmt', 'Expected `{0}` to be a date', value);
          }
          if (isValidDate(value)) {
            previousDate = value;
            if (previousDate && timezone) {
              previousDate = convertTimezoneToLocal(previousDate, timezone, true);
            }
            return $filter('date')(value, format, timezone);
          } else {
            previousDate = null;
            return '';
          }
        });

        if (isDefined(attr.min) || attr.ngMin) {
          var minVal;
          ctrl.$validators.min = function (value) {
            return !isValidDate(value) || isUndefined(minVal) || parseDate(value) >= minVal;
          };
          attr.$observe('min', function (val) {
            minVal = parseObservedDateValue(val);
            ctrl.$validate();
          });
        }

        if (isDefined(attr.max) || attr.ngMax) {
          var maxVal;
          ctrl.$validators.max = function (value) {
            return !isValidDate(value) || isUndefined(maxVal) || parseDate(value) <= maxVal;
          };
          attr.$observe('max', function (val) {
            maxVal = parseObservedDateValue(val);
            ctrl.$validate();
          });
        }

        function isValidDate(value) {
          // Invalid Date: getTime() returns NaN
          return value && !(value.getTime && value.getTime() !== value.getTime());
        }

        function parseObservedDateValue(val) {
          return isDefined(val) && !isDate(val) ? parseDate(val) || undefined : val;
        }
      };
    }

    function badInputChecker(scope, element, attr, ctrl) {
      var node = element[0];
      var nativeValidation = ctrl.$$hasNativeValidators = isObject(node.validity);
      if (nativeValidation) {
        ctrl.$parsers.push(function (value) {
          var validity = element.prop(VALIDITY_STATE_PROPERTY) || {};
          return validity.badInput || validity.typeMismatch ? undefined : value;
        });
      }
    }

    function numberFormatterParser(ctrl) {
      ctrl.$$parserName = 'number';
      ctrl.$parsers.push(function (value) {
        if (ctrl.$isEmpty(value)) return null;
        if (NUMBER_REGEXP.test(value)) return parseFloat(value);
        return undefined;
      });

      ctrl.$formatters.push(function (value) {
        if (!ctrl.$isEmpty(value)) {
          if (!isNumber(value)) {
            throw ngModelMinErr('numfmt', 'Expected `{0}` to be a number', value);
          }
          value = value.toString();
        }
        return value;
      });
    }

    function parseNumberAttrVal(val) {
      if (isDefined(val) && !isNumber(val)) {
        val = parseFloat(val);
      }
      return !isNumberNaN(val) ? val : undefined;
    }

    function isNumberInteger(num) {
      // See http://stackoverflow.com/questions/14636536/how-to-check-if-a-variable-is-an-integer-in-javascript#14794066
      // (minus the assumption that `num` is a number)

      // eslint-disable-next-line no-bitwise
      return (num | 0) === num;
    }

    function countDecimals(num) {
      var numString = num.toString();
      var decimalSymbolIndex = numString.indexOf('.');

      if (decimalSymbolIndex === -1) {
        if (-1 < num && num < 1) {
          // It may be in the exponential notation format (`1e-X`)
          var match = /e-(\d+)$/.exec(numString);

          if (match) {
            return Number(match[1]);
          }
        }

        return 0;
      }

      return numString.length - decimalSymbolIndex - 1;
    }

    function isValidForStep(viewValue, stepBase, step) {
      // At this point `stepBase` and `step` are expected to be non-NaN values
      // and `viewValue` is expected to be a valid stringified number.
      var value = Number(viewValue);

      var isNonIntegerValue = !isNumberInteger(value);
      var isNonIntegerStepBase = !isNumberInteger(stepBase);
      var isNonIntegerStep = !isNumberInteger(step);

      // Due to limitations in Floating Point Arithmetic (e.g. `0.3 - 0.2 !== 0.1` or
      // `0.5 % 0.1 !== 0`), we need to convert all numbers to integers.
      if (isNonIntegerValue || isNonIntegerStepBase || isNonIntegerStep) {
        var valueDecimals = isNonIntegerValue ? countDecimals(value) : 0;
        var stepBaseDecimals = isNonIntegerStepBase ? countDecimals(stepBase) : 0;
        var stepDecimals = isNonIntegerStep ? countDecimals(step) : 0;

        var decimalCount = Math.max(valueDecimals, stepBaseDecimals, stepDecimals);
        var multiplier = Math.pow(10, decimalCount);

        value = value * multiplier;
        stepBase = stepBase * multiplier;
        step = step * multiplier;

        if (isNonIntegerValue) value = Math.round(value);
        if (isNonIntegerStepBase) stepBase = Math.round(stepBase);
        if (isNonIntegerStep) step = Math.round(step);
      }

      return (value - stepBase) % step === 0;
    }

    function numberInputType(scope, element, attr, ctrl, $sniffer, $browser) {
      badInputChecker(scope, element, attr, ctrl);
      numberFormatterParser(ctrl);
      baseInputType(scope, element, attr, ctrl, $sniffer, $browser);

      var minVal;
      var maxVal;

      if (isDefined(attr.min) || attr.ngMin) {
        ctrl.$validators.min = function (value) {
          return ctrl.$isEmpty(value) || isUndefined(minVal) || value >= minVal;
        };

        attr.$observe('min', function (val) {
          minVal = parseNumberAttrVal(val);
          // TODO(matsko): implement validateLater to reduce number of validations
          ctrl.$validate();
        });
      }

      if (isDefined(attr.max) || attr.ngMax) {
        ctrl.$validators.max = function (value) {
          return ctrl.$isEmpty(value) || isUndefined(maxVal) || value <= maxVal;
        };

        attr.$observe('max', function (val) {
          maxVal = parseNumberAttrVal(val);
          // TODO(matsko): implement validateLater to reduce number of validations
          ctrl.$validate();
        });
      }

      if (isDefined(attr.step) || attr.ngStep) {
        var stepVal;
        ctrl.$validators.step = function (modelValue, viewValue) {
          return ctrl.$isEmpty(viewValue) || isUndefined(stepVal) || isValidForStep(viewValue, minVal || 0, stepVal);
        };

        attr.$observe('step', function (val) {
          stepVal = parseNumberAttrVal(val);
          // TODO(matsko): implement validateLater to reduce number of validations
          ctrl.$validate();
        });
      }
    }

    function rangeInputType(scope, element, attr, ctrl, $sniffer, $browser) {
      badInputChecker(scope, element, attr, ctrl);
      numberFormatterParser(ctrl);
      baseInputType(scope, element, attr, ctrl, $sniffer, $browser);

      var supportsRange = ctrl.$$hasNativeValidators && element[0].type === 'range',
          minVal = supportsRange ? 0 : undefined,
          maxVal = supportsRange ? 100 : undefined,
          stepVal = supportsRange ? 1 : undefined,
          validity = element[0].validity,
          hasMinAttr = isDefined(attr.min),
          hasMaxAttr = isDefined(attr.max),
          hasStepAttr = isDefined(attr.step);

      var originalRender = ctrl.$render;

      ctrl.$render = supportsRange && isDefined(validity.rangeUnderflow) && isDefined(validity.rangeOverflow) ?
      //Browsers that implement range will set these values automatically, but reading the adjusted values after
      //$render would cause the min / max validators to be applied with the wrong value
      function rangeRender() {
        originalRender();
        ctrl.$setViewValue(element.val());
      } : originalRender;

      if (hasMinAttr) {
        ctrl.$validators.min = supportsRange ?
        // Since all browsers set the input to a valid value, we don't need to check validity
        function noopMinValidator() {
          return true;
        } :
        // non-support browsers validate the min val
        function minValidator(modelValue, viewValue) {
          return ctrl.$isEmpty(viewValue) || isUndefined(minVal) || viewValue >= minVal;
        };

        setInitialValueAndObserver('min', minChange);
      }

      if (hasMaxAttr) {
        ctrl.$validators.max = supportsRange ?
        // Since all browsers set the input to a valid value, we don't need to check validity
        function noopMaxValidator() {
          return true;
        } :
        // non-support browsers validate the max val
        function maxValidator(modelValue, viewValue) {
          return ctrl.$isEmpty(viewValue) || isUndefined(maxVal) || viewValue <= maxVal;
        };

        setInitialValueAndObserver('max', maxChange);
      }

      if (hasStepAttr) {
        ctrl.$validators.step = supportsRange ? function nativeStepValidator() {
          // Currently, only FF implements the spec on step change correctly (i.e. adjusting the
          // input element value to a valid value). It's possible that other browsers set the stepMismatch
          // validity error instead, so we can at least report an error in that case.
          return !validity.stepMismatch;
        } :
        // ngStep doesn't set the setp attr, so the browser doesn't adjust the input value as setting step would
        function stepValidator(modelValue, viewValue) {
          return ctrl.$isEmpty(viewValue) || isUndefined(stepVal) || isValidForStep(viewValue, minVal || 0, stepVal);
        };

        setInitialValueAndObserver('step', stepChange);
      }

      function setInitialValueAndObserver(htmlAttrName, changeFn) {
        // interpolated attributes set the attribute value only after a digest, but we need the
        // attribute value when the input is first rendered, so that the browser can adjust the
        // input value based on the min/max value
        element.attr(htmlAttrName, attr[htmlAttrName]);
        attr.$observe(htmlAttrName, changeFn);
      }

      function minChange(val) {
        minVal = parseNumberAttrVal(val);
        // ignore changes before model is initialized
        if (isNumberNaN(ctrl.$modelValue)) {
          return;
        }

        if (supportsRange) {
          var elVal = element.val();
          // IE11 doesn't set the el val correctly if the minVal is greater than the element value
          if (minVal > elVal) {
            elVal = minVal;
            element.val(elVal);
          }
          ctrl.$setViewValue(elVal);
        } else {
          // TODO(matsko): implement validateLater to reduce number of validations
          ctrl.$validate();
        }
      }

      function maxChange(val) {
        maxVal = parseNumberAttrVal(val);
        // ignore changes before model is initialized
        if (isNumberNaN(ctrl.$modelValue)) {
          return;
        }

        if (supportsRange) {
          var elVal = element.val();
          // IE11 doesn't set the el val correctly if the maxVal is less than the element value
          if (maxVal < elVal) {
            element.val(maxVal);
            // IE11 and Chrome don't set the value to the minVal when max < min
            elVal = maxVal < minVal ? minVal : maxVal;
          }
          ctrl.$setViewValue(elVal);
        } else {
          // TODO(matsko): implement validateLater to reduce number of validations
          ctrl.$validate();
        }
      }

      function stepChange(val) {
        stepVal = parseNumberAttrVal(val);
        // ignore changes before model is initialized
        if (isNumberNaN(ctrl.$modelValue)) {
          return;
        }

        // Some browsers don't adjust the input value correctly, but set the stepMismatch error
        if (supportsRange && ctrl.$viewValue !== element.val()) {
          ctrl.$setViewValue(element.val());
        } else {
          // TODO(matsko): implement validateLater to reduce number of validations
          ctrl.$validate();
        }
      }
    }

    function urlInputType(scope, element, attr, ctrl, $sniffer, $browser) {
      // Note: no badInputChecker here by purpose as `url` is only a validation
      // in browsers, i.e. we can always read out input.value even if it is not valid!
      baseInputType(scope, element, attr, ctrl, $sniffer, $browser);
      stringBasedInputType(ctrl);

      ctrl.$$parserName = 'url';
      ctrl.$validators.url = function (modelValue, viewValue) {
        var value = modelValue || viewValue;
        return ctrl.$isEmpty(value) || URL_REGEXP.test(value);
      };
    }

    function emailInputType(scope, element, attr, ctrl, $sniffer, $browser) {
      // Note: no badInputChecker here by purpose as `url` is only a validation
      // in browsers, i.e. we can always read out input.value even if it is not valid!
      baseInputType(scope, element, attr, ctrl, $sniffer, $browser);
      stringBasedInputType(ctrl);

      ctrl.$$parserName = 'email';
      ctrl.$validators.email = function (modelValue, viewValue) {
        var value = modelValue || viewValue;
        return ctrl.$isEmpty(value) || EMAIL_REGEXP.test(value);
      };
    }

    function radioInputType(scope, element, attr, ctrl) {
      var doTrim = !attr.ngTrim || trim(attr.ngTrim) !== 'false';
      // make the name unique, if not defined
      if (isUndefined(attr.name)) {
        element.attr('name', nextUid());
      }

      var listener = function (ev) {
        var value;
        if (element[0].checked) {
          value = attr.value;
          if (doTrim) {
            value = trim(value);
          }
          ctrl.$setViewValue(value, ev && ev.type);
        }
      };

      element.on('click', listener);

      ctrl.$render = function () {
        var value = attr.value;
        if (doTrim) {
          value = trim(value);
        }
        element[0].checked = value === ctrl.$viewValue;
      };

      attr.$observe('value', ctrl.$render);
    }

    function parseConstantExpr($parse, context, name, expression, fallback) {
      var parseFn;
      if (isDefined(expression)) {
        parseFn = $parse(expression);
        if (!parseFn.constant) {
          throw ngModelMinErr('constexpr', 'Expected constant expression for `{0}`, but saw ' + '`{1}`.', name, expression);
        }
        return parseFn(context);
      }
      return fallback;
    }

    function checkboxInputType(scope, element, attr, ctrl, $sniffer, $browser, $filter, $parse) {
      var trueValue = parseConstantExpr($parse, scope, 'ngTrueValue', attr.ngTrueValue, true);
      var falseValue = parseConstantExpr($parse, scope, 'ngFalseValue', attr.ngFalseValue, false);

      var listener = function (ev) {
        ctrl.$setViewValue(element[0].checked, ev && ev.type);
      };

      element.on('click', listener);

      ctrl.$render = function () {
        element[0].checked = ctrl.$viewValue;
      };

      // Override the standard `$isEmpty` because the $viewValue of an empty checkbox is always set to `false`
      // This is because of the parser below, which compares the `$modelValue` with `trueValue` to convert
      // it to a boolean.
      ctrl.$isEmpty = function (value) {
        return value === false;
      };

      ctrl.$formatters.push(function (value) {
        return equals(value, trueValue);
      });

      ctrl.$parsers.push(function (value) {
        return value ? trueValue : falseValue;
      });
    }

    /**
     * @ngdoc directive
     * @name textarea
     * @restrict E
     *
     * @description
     * HTML textarea element control with angular data-binding. The data-binding and validation
     * properties of this element are exactly the same as those of the
     * {@link ng.directive:input input element}.
     *
     * @param {string} ngModel Assignable angular expression to data-bind to.
     * @param {string=} name Property name of the form under which the control is published.
     * @param {string=} required Sets `required` validation error key if the value is not entered.
     * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
     *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
     *    `required` when you want to data-bind to the `required` attribute.
     * @param {number=} ngMinlength Sets `minlength` validation error key if the value is shorter than
     *    minlength.
     * @param {number=} ngMaxlength Sets `maxlength` validation error key if the value is longer than
     *    maxlength. Setting the attribute to a negative or non-numeric value, allows view values of any
     *    length.
     * @param {string=} ngPattern Sets `pattern` validation error key if the ngModel {@link ngModel.NgModelController#$viewValue $viewValue}
     *    does not match a RegExp found by evaluating the Angular expression given in the attribute value.
     *    If the expression evaluates to a RegExp object, then this is used directly.
     *    If the expression evaluates to a string, then it will be converted to a RegExp
     *    after wrapping it in `^` and `$` characters. For instance, `"abc"` will be converted to
     *    `new RegExp('^abc$')`.<br />
     *    **Note:** Avoid using the `g` flag on the RegExp, as it will cause each successive search to
     *    start at the index of the last search's match, thus not taking the whole input value into
     *    account.
     * @param {string=} ngChange Angular expression to be executed when input changes due to user
     *    interaction with the input element.
     * @param {boolean=} [ngTrim=true] If set to false Angular will not automatically trim the input.
     *
     * @knownIssue
     *
     * When specifying the `placeholder` attribute of `<textarea>`, Internet Explorer will temporarily
     * insert the placeholder value as the textarea's content. If the placeholder value contains
     * interpolation (`{{ ... }}`), an error will be logged in the console when Angular tries to update
     * the value of the by-then-removed text node. This doesn't affect the functionality of the
     * textarea, but can be undesirable.
     *
     * You can work around this Internet Explorer issue by using `ng-attr-placeholder` instead of
     * `placeholder` on textareas, whenever you need interpolation in the placeholder value. You can
     * find more details on `ngAttr` in the
     * [Interpolation](guide/interpolation#-ngattr-for-binding-to-arbitrary-attributes) section of the
     * Developer Guide.
     */

    /**
     * @ngdoc directive
     * @name input
     * @restrict E
     *
     * @description
     * HTML input element control. When used together with {@link ngModel `ngModel`}, it provides data-binding,
     * input state control, and validation.
     * Input control follows HTML5 input types and polyfills the HTML5 validation behavior for older browsers.
     *
     * <div class="alert alert-warning">
     * **Note:** Not every feature offered is available for all input types.
     * Specifically, data binding and event handling via `ng-model` is unsupported for `input[file]`.
     * </div>
     *
     * @param {string} ngModel Assignable angular expression to data-bind to.
     * @param {string=} name Property name of the form under which the control is published.
     * @param {string=} required Sets `required` validation error key if the value is not entered.
     * @param {boolean=} ngRequired Sets `required` attribute if set to true
     * @param {number=} ngMinlength Sets `minlength` validation error key if the value is shorter than
     *    minlength.
     * @param {number=} ngMaxlength Sets `maxlength` validation error key if the value is longer than
     *    maxlength. Setting the attribute to a negative or non-numeric value, allows view values of any
     *    length.
     * @param {string=} ngPattern Sets `pattern` validation error key if the ngModel {@link ngModel.NgModelController#$viewValue $viewValue}
     *    value does not match a RegExp found by evaluating the Angular expression given in the attribute value.
     *    If the expression evaluates to a RegExp object, then this is used directly.
     *    If the expression evaluates to a string, then it will be converted to a RegExp
     *    after wrapping it in `^` and `$` characters. For instance, `"abc"` will be converted to
     *    `new RegExp('^abc$')`.<br />
     *    **Note:** Avoid using the `g` flag on the RegExp, as it will cause each successive search to
     *    start at the index of the last search's match, thus not taking the whole input value into
     *    account.
     * @param {string=} ngChange Angular expression to be executed when input changes due to user
     *    interaction with the input element.
     * @param {boolean=} [ngTrim=true] If set to false Angular will not automatically trim the input.
     *    This parameter is ignored for input[type=password] controls, which will never trim the
     *    input.
     *
     * @example
        <example name="input-directive" module="inputExample">
          <file name="index.html">
           <script>
              angular.module('inputExample', [])
                .controller('ExampleController', ['$scope', function($scope) {
                  $scope.user = {name: 'guest', last: 'visitor'};
                }]);
           </script>
           <div ng-controller="ExampleController">
             <form name="myForm">
               <label>
                  User name:
                  <input type="text" name="userName" ng-model="user.name" required>
               </label>
               <div role="alert">
                 <span class="error" ng-show="myForm.userName.$error.required">
                  Required!</span>
               </div>
               <label>
                  Last name:
                  <input type="text" name="lastName" ng-model="user.last"
                  ng-minlength="3" ng-maxlength="10">
               </label>
               <div role="alert">
                 <span class="error" ng-show="myForm.lastName.$error.minlength">
                   Too short!</span>
                 <span class="error" ng-show="myForm.lastName.$error.maxlength">
                   Too long!</span>
               </div>
             </form>
             <hr>
             <tt>user = {{user}}</tt><br/>
             <tt>myForm.userName.$valid = {{myForm.userName.$valid}}</tt><br/>
             <tt>myForm.userName.$error = {{myForm.userName.$error}}</tt><br/>
             <tt>myForm.lastName.$valid = {{myForm.lastName.$valid}}</tt><br/>
             <tt>myForm.lastName.$error = {{myForm.lastName.$error}}</tt><br/>
             <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
             <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
             <tt>myForm.$error.minlength = {{!!myForm.$error.minlength}}</tt><br/>
             <tt>myForm.$error.maxlength = {{!!myForm.$error.maxlength}}</tt><br/>
           </div>
          </file>
          <file name="protractor.js" type="protractor">
            var user = element(by.exactBinding('user'));
            var userNameValid = element(by.binding('myForm.userName.$valid'));
            var lastNameValid = element(by.binding('myForm.lastName.$valid'));
            var lastNameError = element(by.binding('myForm.lastName.$error'));
            var formValid = element(by.binding('myForm.$valid'));
            var userNameInput = element(by.model('user.name'));
            var userLastInput = element(by.model('user.last'));
    
            it('should initialize to model', function() {
              expect(user.getText()).toContain('{"name":"guest","last":"visitor"}');
              expect(userNameValid.getText()).toContain('true');
              expect(formValid.getText()).toContain('true');
            });
    
            it('should be invalid if empty when required', function() {
              userNameInput.clear();
              userNameInput.sendKeys('');
    
              expect(user.getText()).toContain('{"last":"visitor"}');
              expect(userNameValid.getText()).toContain('false');
              expect(formValid.getText()).toContain('false');
            });
    
            it('should be valid if empty when min length is set', function() {
              userLastInput.clear();
              userLastInput.sendKeys('');
    
              expect(user.getText()).toContain('{"name":"guest","last":""}');
              expect(lastNameValid.getText()).toContain('true');
              expect(formValid.getText()).toContain('true');
            });
    
            it('should be invalid if less than required min length', function() {
              userLastInput.clear();
              userLastInput.sendKeys('xx');
    
              expect(user.getText()).toContain('{"name":"guest"}');
              expect(lastNameValid.getText()).toContain('false');
              expect(lastNameError.getText()).toContain('minlength');
              expect(formValid.getText()).toContain('false');
            });
    
            it('should be invalid if longer than max length', function() {
              userLastInput.clear();
              userLastInput.sendKeys('some ridiculously long name');
    
              expect(user.getText()).toContain('{"name":"guest"}');
              expect(lastNameValid.getText()).toContain('false');
              expect(lastNameError.getText()).toContain('maxlength');
              expect(formValid.getText()).toContain('false');
            });
          </file>
        </example>
     */
    var inputDirective = ['$browser', '$sniffer', '$filter', '$parse', function ($browser, $sniffer, $filter, $parse) {
      return {
        restrict: 'E',
        require: ['?ngModel'],
        link: {
          pre: function (scope, element, attr, ctrls) {
            if (ctrls[0]) {
              (inputType[lowercase(attr.type)] || inputType.text)(scope, element, attr, ctrls[0], $sniffer, $browser, $filter, $parse);
            }
          }
        }
      };
    }];

    var CONSTANT_VALUE_REGEXP = /^(true|false|\d+)$/;
    /**
     * @ngdoc directive
     * @name ngValue
     *
     * @description
     * Binds the given expression to the value of the element.
     *
     * It is mainly used on {@link input[radio] `input[radio]`} and option elements,
     * so that when the element is selected, the {@link ngModel `ngModel`} of that element (or its
     * {@link select `select`} parent element) is set to the bound value. It is especially useful
     * for dynamically generated lists using {@link ngRepeat `ngRepeat`}, as shown below.
     *
     * It can also be used to achieve one-way binding of a given expression to an input element
     * such as an `input[text]` or a `textarea`, when that element does not use ngModel.
     *
     * @element input
     * @param {string=} ngValue angular expression, whose value will be bound to the `value` attribute
     * and `value` property of the element.
     *
     * @example
        <example name="ngValue-directive" module="valueExample">
          <file name="index.html">
           <script>
              angular.module('valueExample', [])
                .controller('ExampleController', ['$scope', function($scope) {
                  $scope.names = ['pizza', 'unicorns', 'robots'];
                  $scope.my = { favorite: 'unicorns' };
                }]);
           </script>
            <form ng-controller="ExampleController">
              <h2>Which is your favorite?</h2>
                <label ng-repeat="name in names" for="{{name}}">
                  {{name}}
                  <input type="radio"
                         ng-model="my.favorite"
                         ng-value="name"
                         id="{{name}}"
                         name="favorite">
                </label>
              <div>You chose {{my.favorite}}</div>
            </form>
          </file>
          <file name="protractor.js" type="protractor">
            var favorite = element(by.binding('my.favorite'));
    
            it('should initialize to model', function() {
              expect(favorite.getText()).toContain('unicorns');
            });
            it('should bind the values to the inputs', function() {
              element.all(by.model('my.favorite')).get(0).click();
              expect(favorite.getText()).toContain('pizza');
            });
          </file>
        </example>
     */
    var ngValueDirective = function () {
      /**
       *  inputs use the value attribute as their default value if the value property is not set.
       *  Once the value property has been set (by adding input), it will not react to changes to
       *  the value attribute anymore. Setting both attribute and property fixes this behavior, and
       *  makes it possible to use ngValue as a sort of one-way bind.
       */
      function updateElementValue(element, attr, value) {
        // Support: IE9 only
        // In IE9 values are converted to string (e.g. `input.value = null` results in `input.value === 'null'`).
        var propValue = isDefined(value) ? value : msie === 9 ? '' : null;
        element.prop('value', propValue);
        attr.$set('value', value);
      }

      return {
        restrict: 'A',
        priority: 100,
        compile: function (tpl, tplAttr) {
          if (CONSTANT_VALUE_REGEXP.test(tplAttr.ngValue)) {
            return function ngValueConstantLink(scope, elm, attr) {
              var value = scope.$eval(attr.ngValue);
              updateElementValue(elm, attr, value);
            };
          } else {
            return function ngValueLink(scope, elm, attr) {
              scope.$watch(attr.ngValue, function valueWatchAction(value) {
                updateElementValue(elm, attr, value);
              });
            };
          }
        }
      };
    };

    /**
     * @ngdoc directive
     * @name ngBind
     * @restrict AC
     *
     * @description
     * The `ngBind` attribute tells Angular to replace the text content of the specified HTML element
     * with the value of a given expression, and to update the text content when the value of that
     * expression changes.
     *
     * Typically, you don't use `ngBind` directly, but instead you use the double curly markup like
     * `{{ expression }}` which is similar but less verbose.
     *
     * It is preferable to use `ngBind` instead of `{{ expression }}` if a template is momentarily
     * displayed by the browser in its raw state before Angular compiles it. Since `ngBind` is an
     * element attribute, it makes the bindings invisible to the user while the page is loading.
     *
     * An alternative solution to this problem would be using the
     * {@link ng.directive:ngCloak ngCloak} directive.
     *
     *
     * @element ANY
     * @param {expression} ngBind {@link guide/expression Expression} to evaluate.
     *
     * @example
     * Enter a name in the Live Preview text box; the greeting below the text box changes instantly.
       <example module="bindExample" name="ng-bind">
         <file name="index.html">
           <script>
             angular.module('bindExample', [])
               .controller('ExampleController', ['$scope', function($scope) {
                 $scope.name = 'Whirled';
               }]);
           </script>
           <div ng-controller="ExampleController">
             <label>Enter name: <input type="text" ng-model="name"></label><br>
             Hello <span ng-bind="name"></span>!
           </div>
         </file>
         <file name="protractor.js" type="protractor">
           it('should check ng-bind', function() {
             var nameInput = element(by.model('name'));
    
             expect(element(by.binding('name')).getText()).toBe('Whirled');
             nameInput.clear();
             nameInput.sendKeys('world');
             expect(element(by.binding('name')).getText()).toBe('world');
           });
         </file>
       </example>
     */
    var ngBindDirective = ['$compile', function ($compile) {
      return {
        restrict: 'AC',
        compile: function ngBindCompile(templateElement) {
          $compile.$$addBindingClass(templateElement);
          return function ngBindLink(scope, element, attr) {
            $compile.$$addBindingInfo(element, attr.ngBind);
            element = element[0];
            scope.$watch(attr.ngBind, function ngBindWatchAction(value) {
              element.textContent = stringify(value);
            });
          };
        }
      };
    }];

    /**
     * @ngdoc directive
     * @name ngBindTemplate
     *
     * @description
     * The `ngBindTemplate` directive specifies that the element
     * text content should be replaced with the interpolation of the template
     * in the `ngBindTemplate` attribute.
     * Unlike `ngBind`, the `ngBindTemplate` can contain multiple `{{` `}}`
     * expressions. This directive is needed since some HTML elements
     * (such as TITLE and OPTION) cannot contain SPAN elements.
     *
     * @element ANY
     * @param {string} ngBindTemplate template of form
     *   <tt>{{</tt> <tt>expression</tt> <tt>}}</tt> to eval.
     *
     * @example
     * Try it here: enter text in text box and watch the greeting change.
       <example module="bindExample" name="ng-bind-template">
         <file name="index.html">
           <script>
             angular.module('bindExample', [])
               .controller('ExampleController', ['$scope', function($scope) {
                 $scope.salutation = 'Hello';
                 $scope.name = 'World';
               }]);
           </script>
           <div ng-controller="ExampleController">
            <label>Salutation: <input type="text" ng-model="salutation"></label><br>
            <label>Name: <input type="text" ng-model="name"></label><br>
            <pre ng-bind-template="{{salutation}} {{name}}!"></pre>
           </div>
         </file>
         <file name="protractor.js" type="protractor">
           it('should check ng-bind', function() {
             var salutationElem = element(by.binding('salutation'));
             var salutationInput = element(by.model('salutation'));
             var nameInput = element(by.model('name'));
    
             expect(salutationElem.getText()).toBe('Hello World!');
    
             salutationInput.clear();
             salutationInput.sendKeys('Greetings');
             nameInput.clear();
             nameInput.sendKeys('user');
    
             expect(salutationElem.getText()).toBe('Greetings user!');
           });
         </file>
       </example>
     */
    var ngBindTemplateDirective = ['$interpolate', '$compile', function ($interpolate, $compile) {
      return {
        compile: function ngBindTemplateCompile(templateElement) {
          $compile.$$addBindingClass(templateElement);
          return function ngBindTemplateLink(scope, element, attr) {
            var interpolateFn = $interpolate(element.attr(attr.$attr.ngBindTemplate));
            $compile.$$addBindingInfo(element, interpolateFn.expressions);
            element = element[0];
            attr.$observe('ngBindTemplate', function (value) {
              element.textContent = isUndefined(value) ? '' : value;
            });
          };
        }
      };
    }];

    /**
     * @ngdoc directive
     * @name ngBindHtml
     *
     * @description
     * Evaluates the expression and inserts the resulting HTML into the element in a secure way. By default,
     * the resulting HTML content will be sanitized using the {@link ngSanitize.$sanitize $sanitize} service.
     * To utilize this functionality, ensure that `$sanitize` is available, for example, by including {@link
     * ngSanitize} in your module's dependencies (not in core Angular). In order to use {@link ngSanitize}
     * in your module's dependencies, you need to include "angular-sanitize.js" in your application.
     *
     * You may also bypass sanitization for values you know are safe. To do so, bind to
     * an explicitly trusted value via {@link ng.$sce#trustAsHtml $sce.trustAsHtml}.  See the example
     * under {@link ng.$sce#show-me-an-example-using-sce- Strict Contextual Escaping (SCE)}.
     *
     * Note: If a `$sanitize` service is unavailable and the bound value isn't explicitly trusted, you
     * will have an exception (instead of an exploit.)
     *
     * @element ANY
     * @param {expression} ngBindHtml {@link guide/expression Expression} to evaluate.
     *
     * @example
    
       <example module="bindHtmlExample" deps="angular-sanitize.js" name="ng-bind-html">
         <file name="index.html">
           <div ng-controller="ExampleController">
            <p ng-bind-html="myHTML"></p>
           </div>
         </file>
    
         <file name="script.js">
           angular.module('bindHtmlExample', ['ngSanitize'])
             .controller('ExampleController', ['$scope', function($scope) {
               $scope.myHTML =
                  'I am an <code>HTML</code>string with ' +
                  '<a href="#">links!</a> and other <em>stuff</em>';
             }]);
         </file>
    
         <file name="protractor.js" type="protractor">
           it('should check ng-bind-html', function() {
             expect(element(by.binding('myHTML')).getText()).toBe(
                 'I am an HTMLstring with links! and other stuff');
           });
         </file>
       </example>
     */
    var ngBindHtmlDirective = ['$sce', '$parse', '$compile', function ($sce, $parse, $compile) {
      return {
        restrict: 'A',
        compile: function ngBindHtmlCompile(tElement, tAttrs) {
          var ngBindHtmlGetter = $parse(tAttrs.ngBindHtml);
          var ngBindHtmlWatch = $parse(tAttrs.ngBindHtml, function sceValueOf(val) {
            // Unwrap the value to compare the actual inner safe value, not the wrapper object.
            return $sce.valueOf(val);
          });
          $compile.$$addBindingClass(tElement);

          return function ngBindHtmlLink(scope, element, attr) {
            $compile.$$addBindingInfo(element, attr.ngBindHtml);

            scope.$watch(ngBindHtmlWatch, function ngBindHtmlWatchAction() {
              // The watched value is the unwrapped value. To avoid re-escaping, use the direct getter.
              var value = ngBindHtmlGetter(scope);
              element.html($sce.getTrustedHtml(value) || '');
            });
          };
        }
      };
    }];

    /**
     * @ngdoc directive
     * @name ngChange
     *
     * @description
     * Evaluate the given expression when the user changes the input.
     * The expression is evaluated immediately, unlike the JavaScript onchange event
     * which only triggers at the end of a change (usually, when the user leaves the
     * form element or presses the return key).
     *
     * The `ngChange` expression is only evaluated when a change in the input value causes
     * a new value to be committed to the model.
     *
     * It will not be evaluated:
     * * if the value returned from the `$parsers` transformation pipeline has not changed
     * * if the input has continued to be invalid since the model will stay `null`
     * * if the model is changed programmatically and not by a change to the input value
     *
     *
     * Note, this directive requires `ngModel` to be present.
     *
     * @element input
     * @param {expression} ngChange {@link guide/expression Expression} to evaluate upon change
     * in input value.
     *
     * @example
     * <example name="ngChange-directive" module="changeExample">
     *   <file name="index.html">
     *     <script>
     *       angular.module('changeExample', [])
     *         .controller('ExampleController', ['$scope', function($scope) {
     *           $scope.counter = 0;
     *           $scope.change = function() {
     *             $scope.counter++;
     *           };
     *         }]);
     *     </script>
     *     <div ng-controller="ExampleController">
     *       <input type="checkbox" ng-model="confirmed" ng-change="change()" id="ng-change-example1" />
     *       <input type="checkbox" ng-model="confirmed" id="ng-change-example2" />
     *       <label for="ng-change-example2">Confirmed</label><br />
     *       <tt>debug = {{confirmed}}</tt><br/>
     *       <tt>counter = {{counter}}</tt><br/>
     *     </div>
     *   </file>
     *   <file name="protractor.js" type="protractor">
     *     var counter = element(by.binding('counter'));
     *     var debug = element(by.binding('confirmed'));
     *
     *     it('should evaluate the expression if changing from view', function() {
     *       expect(counter.getText()).toContain('0');
     *
     *       element(by.id('ng-change-example1')).click();
     *
     *       expect(counter.getText()).toContain('1');
     *       expect(debug.getText()).toContain('true');
     *     });
     *
     *     it('should not evaluate the expression if changing from model', function() {
     *       element(by.id('ng-change-example2')).click();
    
     *       expect(counter.getText()).toContain('0');
     *       expect(debug.getText()).toContain('true');
     *     });
     *   </file>
     * </example>
     */
    var ngChangeDirective = valueFn({
      restrict: 'A',
      require: 'ngModel',
      link: function (scope, element, attr, ctrl) {
        ctrl.$viewChangeListeners.push(function () {
          scope.$eval(attr.ngChange);
        });
      }
    });

    /* exported
      ngClassDirective,
      ngClassEvenDirective,
      ngClassOddDirective
    */

    function classDirective(name, selector) {
      name = 'ngClass' + name;
      var indexWatchExpression;

      return ['$parse', function ($parse) {
        return {
          restrict: 'AC',
          link: function (scope, element, attr) {
            var expression = attr[name].trim();
            var isOneTime = expression.charAt(0) === ':' && expression.charAt(1) === ':';

            var watchInterceptor = isOneTime ? toFlatValue : toClassString;
            var watchExpression = $parse(expression, watchInterceptor);
            var watchAction = isOneTime ? ngClassOneTimeWatchAction : ngClassWatchAction;

            var classCounts = element.data('$classCounts');
            var oldModulo = true;
            var oldClassString;

            if (!classCounts) {
              // Use createMap() to prevent class assumptions involving property
              // names in Object.prototype
              classCounts = createMap();
              element.data('$classCounts', classCounts);
            }

            if (name !== 'ngClass') {
              if (!indexWatchExpression) {
                indexWatchExpression = $parse('$index', function moduloTwo($index) {
                  // eslint-disable-next-line no-bitwise
                  return $index & 1;
                });
              }

              scope.$watch(indexWatchExpression, ngClassIndexWatchAction);
            }

            scope.$watch(watchExpression, watchAction, isOneTime);

            function addClasses(classString) {
              classString = digestClassCounts(split(classString), 1);
              attr.$addClass(classString);
            }

            function removeClasses(classString) {
              classString = digestClassCounts(split(classString), -1);
              attr.$removeClass(classString);
            }

            function updateClasses(oldClassString, newClassString) {
              var oldClassArray = split(oldClassString);
              var newClassArray = split(newClassString);

              var toRemoveArray = arrayDifference(oldClassArray, newClassArray);
              var toAddArray = arrayDifference(newClassArray, oldClassArray);

              var toRemoveString = digestClassCounts(toRemoveArray, -1);
              var toAddString = digestClassCounts(toAddArray, 1);

              attr.$addClass(toAddString);
              attr.$removeClass(toRemoveString);
            }

            function digestClassCounts(classArray, count) {
              var classesToUpdate = [];

              forEach(classArray, function (className) {
                if (count > 0 || classCounts[className]) {
                  classCounts[className] = (classCounts[className] || 0) + count;
                  if (classCounts[className] === +(count > 0)) {
                    classesToUpdate.push(className);
                  }
                }
              });

              return classesToUpdate.join(' ');
            }

            function ngClassIndexWatchAction(newModulo) {
              // This watch-action should run before the `ngClass[OneTime]WatchAction()`, thus it
              // adds/removes `oldClassString`. If the `ngClass` expression has changed as well, the
              // `ngClass[OneTime]WatchAction()` will update the classes.
              if (newModulo === selector) {
                addClasses(oldClassString);
              } else {
                removeClasses(oldClassString);
              }

              oldModulo = newModulo;
            }

            function ngClassOneTimeWatchAction(newClassValue) {
              var newClassString = toClassString(newClassValue);

              if (newClassString !== oldClassString) {
                ngClassWatchAction(newClassString);
              }
            }

            function ngClassWatchAction(newClassString) {
              if (oldModulo === selector) {
                updateClasses(oldClassString, newClassString);
              }

              oldClassString = newClassString;
            }
          }
        };
      }];

      // Helpers
      function arrayDifference(tokens1, tokens2) {
        if (!tokens1 || !tokens1.length) return [];
        if (!tokens2 || !tokens2.length) return tokens1;

        var values = [];

        outer: for (var i = 0; i < tokens1.length; i++) {
          var token = tokens1[i];
          for (var j = 0; j < tokens2.length; j++) {
            if (token === tokens2[j]) continue outer;
          }
          values.push(token);
        }

        return values;
      }

      function split(classString) {
        return classString && classString.split(' ');
      }

      function toClassString(classValue) {
        var classString = classValue;

        if (isArray(classValue)) {
          classString = classValue.map(toClassString).join(' ');
        } else if (isObject(classValue)) {
          classString = Object.keys(classValue).filter(function (key) {
            return classValue[key];
          }).join(' ');
        }

        return classString;
      }

      function toFlatValue(classValue) {
        var flatValue = classValue;

        if (isArray(classValue)) {
          flatValue = classValue.map(toFlatValue);
        } else if (isObject(classValue)) {
          var hasUndefined = false;

          flatValue = Object.keys(classValue).filter(function (key) {
            var value = classValue[key];

            if (!hasUndefined && isUndefined(value)) {
              hasUndefined = true;
            }

            return value;
          });

          if (hasUndefined) {
            // Prevent the `oneTimeLiteralWatchInterceptor` from unregistering
            // the watcher, by including at least one `undefined` value.
            flatValue.push(undefined);
          }
        }

        return flatValue;
      }
    }

    /**
     * @ngdoc directive
     * @name ngClass
     * @restrict AC
     *
     * @description
     * The `ngClass` directive allows you to dynamically set CSS classes on an HTML element by databinding
     * an expression that represents all classes to be added.
     *
     * The directive operates in three different ways, depending on which of three types the expression
     * evaluates to:
     *
     * 1. If the expression evaluates to a string, the string should be one or more space-delimited class
     * names.
     *
     * 2. If the expression evaluates to an object, then for each key-value pair of the
     * object with a truthy value the corresponding key is used as a class name.
     *
     * 3. If the expression evaluates to an array, each element of the array should either be a string as in
     * type 1 or an object as in type 2. This means that you can mix strings and objects together in an array
     * to give you more control over what CSS classes appear. See the code below for an example of this.
     *
     *
     * The directive won't add duplicate classes if a particular class was already set.
     *
     * When the expression changes, the previously added classes are removed and only then are the
     * new classes added.
     *
     * @knownIssue
     * You should not use {@link guide/interpolation interpolation} in the value of the `class`
     * attribute, when using the `ngClass` directive on the same element.
     * See {@link guide/interpolation#known-issues here} for more info.
     *
     * @animations
     * | Animation                        | Occurs                              |
     * |----------------------------------|-------------------------------------|
     * | {@link ng.$animate#addClass addClass}       | just before the class is applied to the element   |
     * | {@link ng.$animate#removeClass removeClass} | just before the class is removed from the element |
     *
     * @element ANY
     * @param {expression} ngClass {@link guide/expression Expression} to eval. The result
     *   of the evaluation can be a string representing space delimited class
     *   names, an array, or a map of class names to boolean values. In the case of a map, the
     *   names of the properties whose values are truthy will be added as css classes to the
     *   element.
     *
     * @example Example that demonstrates basic bindings via ngClass directive.
       <example name="ng-class">
         <file name="index.html">
           <p ng-class="{strike: deleted, bold: important, 'has-error': error}">Map Syntax Example</p>
           <label>
              <input type="checkbox" ng-model="deleted">
              deleted (apply "strike" class)
           </label><br>
           <label>
              <input type="checkbox" ng-model="important">
              important (apply "bold" class)
           </label><br>
           <label>
              <input type="checkbox" ng-model="error">
              error (apply "has-error" class)
           </label>
           <hr>
           <p ng-class="style">Using String Syntax</p>
           <input type="text" ng-model="style"
                  placeholder="Type: bold strike red" aria-label="Type: bold strike red">
           <hr>
           <p ng-class="[style1, style2, style3]">Using Array Syntax</p>
           <input ng-model="style1"
                  placeholder="Type: bold, strike or red" aria-label="Type: bold, strike or red"><br>
           <input ng-model="style2"
                  placeholder="Type: bold, strike or red" aria-label="Type: bold, strike or red 2"><br>
           <input ng-model="style3"
                  placeholder="Type: bold, strike or red" aria-label="Type: bold, strike or red 3"><br>
           <hr>
           <p ng-class="[style4, {orange: warning}]">Using Array and Map Syntax</p>
           <input ng-model="style4" placeholder="Type: bold, strike" aria-label="Type: bold, strike"><br>
           <label><input type="checkbox" ng-model="warning"> warning (apply "orange" class)</label>
         </file>
         <file name="style.css">
           .strike {
               text-decoration: line-through;
           }
           .bold {
               font-weight: bold;
           }
           .red {
               color: red;
           }
           .has-error {
               color: red;
               background-color: yellow;
           }
           .orange {
               color: orange;
           }
         </file>
         <file name="protractor.js" type="protractor">
           var ps = element.all(by.css('p'));
    
           it('should let you toggle the class', function() {
    
             expect(ps.first().getAttribute('class')).not.toMatch(/bold/);
             expect(ps.first().getAttribute('class')).not.toMatch(/has-error/);
    
             element(by.model('important')).click();
             expect(ps.first().getAttribute('class')).toMatch(/bold/);
    
             element(by.model('error')).click();
             expect(ps.first().getAttribute('class')).toMatch(/has-error/);
           });
    
           it('should let you toggle string example', function() {
             expect(ps.get(1).getAttribute('class')).toBe('');
             element(by.model('style')).clear();
             element(by.model('style')).sendKeys('red');
             expect(ps.get(1).getAttribute('class')).toBe('red');
           });
    
           it('array example should have 3 classes', function() {
             expect(ps.get(2).getAttribute('class')).toBe('');
             element(by.model('style1')).sendKeys('bold');
             element(by.model('style2')).sendKeys('strike');
             element(by.model('style3')).sendKeys('red');
             expect(ps.get(2).getAttribute('class')).toBe('bold strike red');
           });
    
           it('array with map example should have 2 classes', function() {
             expect(ps.last().getAttribute('class')).toBe('');
             element(by.model('style4')).sendKeys('bold');
             element(by.model('warning')).click();
             expect(ps.last().getAttribute('class')).toBe('bold orange');
           });
         </file>
       </example>
    
       ## Animations
    
       The example below demonstrates how to perform animations using ngClass.
    
       <example module="ngAnimate" deps="angular-animate.js" animations="true" name="ng-class">
         <file name="index.html">
          <input id="setbtn" type="button" value="set" ng-click="myVar='my-class'">
          <input id="clearbtn" type="button" value="clear" ng-click="myVar=''">
          <br>
          <span class="base-class" ng-class="myVar">Sample Text</span>
         </file>
         <file name="style.css">
           .base-class {
             transition:all cubic-bezier(0.250, 0.460, 0.450, 0.940) 0.5s;
           }
    
           .base-class.my-class {
             color: red;
             font-size:3em;
           }
         </file>
         <file name="protractor.js" type="protractor">
           it('should check ng-class', function() {
             expect(element(by.css('.base-class')).getAttribute('class')).not.
               toMatch(/my-class/);
    
             element(by.id('setbtn')).click();
    
             expect(element(by.css('.base-class')).getAttribute('class')).
               toMatch(/my-class/);
    
             element(by.id('clearbtn')).click();
    
             expect(element(by.css('.base-class')).getAttribute('class')).not.
               toMatch(/my-class/);
           });
         </file>
       </example>
    
    
       ## ngClass and pre-existing CSS3 Transitions/Animations
       The ngClass directive still supports CSS3 Transitions/Animations even if they do not follow the ngAnimate CSS naming structure.
       Upon animation ngAnimate will apply supplementary CSS classes to track the start and end of an animation, but this will not hinder
       any pre-existing CSS transitions already on the element. To get an idea of what happens during a class-based animation, be sure
       to view the step by step details of {@link $animate#addClass $animate.addClass} and
       {@link $animate#removeClass $animate.removeClass}.
     */
    var ngClassDirective = classDirective('', true);

    /**
     * @ngdoc directive
     * @name ngClassOdd
     * @restrict AC
     *
     * @description
     * The `ngClassOdd` and `ngClassEven` directives work exactly as
     * {@link ng.directive:ngClass ngClass}, except they work in
     * conjunction with `ngRepeat` and take effect only on odd (even) rows.
     *
     * This directive can be applied only within the scope of an
     * {@link ng.directive:ngRepeat ngRepeat}.
     *
     * @element ANY
     * @param {expression} ngClassOdd {@link guide/expression Expression} to eval. The result
     *   of the evaluation can be a string representing space delimited class names or an array.
     *
     * @example
       <example name="ng-class-odd">
         <file name="index.html">
            <ol ng-init="names=['John', 'Mary', 'Cate', 'Suz']">
              <li ng-repeat="name in names">
               <span ng-class-odd="'odd'" ng-class-even="'even'">
                 {{name}}
               </span>
              </li>
            </ol>
         </file>
         <file name="style.css">
           .odd {
             color: red;
           }
           .even {
             color: blue;
           }
         </file>
         <file name="protractor.js" type="protractor">
           it('should check ng-class-odd and ng-class-even', function() {
             expect(element(by.repeater('name in names').row(0).column('name')).getAttribute('class')).
               toMatch(/odd/);
             expect(element(by.repeater('name in names').row(1).column('name')).getAttribute('class')).
               toMatch(/even/);
           });
         </file>
       </example>
     */
    var ngClassOddDirective = classDirective('Odd', 0);

    /**
     * @ngdoc directive
     * @name ngClassEven
     * @restrict AC
     *
     * @description
     * The `ngClassOdd` and `ngClassEven` directives work exactly as
     * {@link ng.directive:ngClass ngClass}, except they work in
     * conjunction with `ngRepeat` and take effect only on odd (even) rows.
     *
     * This directive can be applied only within the scope of an
     * {@link ng.directive:ngRepeat ngRepeat}.
     *
     * @element ANY
     * @param {expression} ngClassEven {@link guide/expression Expression} to eval. The
     *   result of the evaluation can be a string representing space delimited class names or an array.
     *
     * @example
       <example name="ng-class-even">
         <file name="index.html">
            <ol ng-init="names=['John', 'Mary', 'Cate', 'Suz']">
              <li ng-repeat="name in names">
               <span ng-class-odd="'odd'" ng-class-even="'even'">
                 {{name}} &nbsp; &nbsp; &nbsp;
               </span>
              </li>
            </ol>
         </file>
         <file name="style.css">
           .odd {
             color: red;
           }
           .even {
             color: blue;
           }
         </file>
         <file name="protractor.js" type="protractor">
           it('should check ng-class-odd and ng-class-even', function() {
             expect(element(by.repeater('name in names').row(0).column('name')).getAttribute('class')).
               toMatch(/odd/);
             expect(element(by.repeater('name in names').row(1).column('name')).getAttribute('class')).
               toMatch(/even/);
           });
         </file>
       </example>
     */
    var ngClassEvenDirective = classDirective('Even', 1);

    /**
     * @ngdoc directive
     * @name ngCloak
     * @restrict AC
     *
     * @description
     * The `ngCloak` directive is used to prevent the Angular html template from being briefly
     * displayed by the browser in its raw (uncompiled) form while your application is loading. Use this
     * directive to avoid the undesirable flicker effect caused by the html template display.
     *
     * The directive can be applied to the `<body>` element, but the preferred usage is to apply
     * multiple `ngCloak` directives to small portions of the page to permit progressive rendering
     * of the browser view.
     *
     * `ngCloak` works in cooperation with the following css rule embedded within `angular.js` and
     * `angular.min.js`.
     * For CSP mode please add `angular-csp.css` to your html file (see {@link ng.directive:ngCsp ngCsp}).
     *
     * ```css
     * [ng\:cloak], [ng-cloak], [data-ng-cloak], [x-ng-cloak], .ng-cloak, .x-ng-cloak {
     *   display: none !important;
     * }
     * ```
     *
     * When this css rule is loaded by the browser, all html elements (including their children) that
     * are tagged with the `ngCloak` directive are hidden. When Angular encounters this directive
     * during the compilation of the template it deletes the `ngCloak` element attribute, making
     * the compiled element visible.
     *
     * For the best result, the `angular.js` script must be loaded in the head section of the html
     * document; alternatively, the css rule above must be included in the external stylesheet of the
     * application.
     *
     * @element ANY
     *
     * @example
       <example name="ng-cloak">
         <file name="index.html">
            <div id="template1" ng-cloak>{{ 'hello' }}</div>
            <div id="template2" class="ng-cloak">{{ 'world' }}</div>
         </file>
         <file name="protractor.js" type="protractor">
           it('should remove the template directive and css class', function() {
             expect($('#template1').getAttribute('ng-cloak')).
               toBeNull();
             expect($('#template2').getAttribute('ng-cloak')).
               toBeNull();
           });
         </file>
       </example>
     *
     */
    var ngCloakDirective = ngDirective({
      compile: function (element, attr) {
        attr.$set('ngCloak', undefined);
        element.removeClass('ng-cloak');
      }
    });

    /**
     * @ngdoc directive
     * @name ngController
     *
     * @description
     * The `ngController` directive attaches a controller class to the view. This is a key aspect of how angular
     * supports the principles behind the Model-View-Controller design pattern.
     *
     * MVC components in angular:
     *
     * * Model — Models are the properties of a scope; scopes are attached to the DOM where scope properties
     *   are accessed through bindings.
     * * View — The template (HTML with data bindings) that is rendered into the View.
     * * Controller — The `ngController` directive specifies a Controller class; the class contains business
     *   logic behind the application to decorate the scope with functions and values
     *
     * Note that you can also attach controllers to the DOM by declaring it in a route definition
     * via the {@link ngRoute.$route $route} service. A common mistake is to declare the controller
     * again using `ng-controller` in the template itself.  This will cause the controller to be attached
     * and executed twice.
     *
     * @element ANY
     * @scope
     * @priority 500
     * @param {expression} ngController Name of a constructor function registered with the current
     * {@link ng.$controllerProvider $controllerProvider} or an {@link guide/expression expression}
     * that on the current scope evaluates to a constructor function.
     *
     * The controller instance can be published into a scope property by specifying
     * `ng-controller="as propertyName"`.
     *
     * If the current `$controllerProvider` is configured to use globals (via
     * {@link ng.$controllerProvider#allowGlobals `$controllerProvider.allowGlobals()` }), this may
     * also be the name of a globally accessible constructor function (deprecated, not recommended).
     *
     * @example
     * Here is a simple form for editing user contact information. Adding, removing, clearing, and
     * greeting are methods declared on the controller (see source tab). These methods can
     * easily be called from the angular markup. Any changes to the data are automatically reflected
     * in the View without the need for a manual update.
     *
     * Two different declaration styles are included below:
     *
     * * one binds methods and properties directly onto the controller using `this`:
     * `ng-controller="SettingsController1 as settings"`
     * * one injects `$scope` into the controller:
     * `ng-controller="SettingsController2"`
     *
     * The second option is more common in the Angular community, and is generally used in boilerplates
     * and in this guide. However, there are advantages to binding properties directly to the controller
     * and avoiding scope.
     *
     * * Using `controller as` makes it obvious which controller you are accessing in the template when
     * multiple controllers apply to an element.
     * * If you are writing your controllers as classes you have easier access to the properties and
     * methods, which will appear on the scope, from inside the controller code.
     * * Since there is always a `.` in the bindings, you don't have to worry about prototypal
     * inheritance masking primitives.
     *
     * This example demonstrates the `controller as` syntax.
     *
     * <example name="ngControllerAs" module="controllerAsExample">
     *   <file name="index.html">
     *    <div id="ctrl-as-exmpl" ng-controller="SettingsController1 as settings">
     *      <label>Name: <input type="text" ng-model="settings.name"/></label>
     *      <button ng-click="settings.greet()">greet</button><br/>
     *      Contact:
     *      <ul>
     *        <li ng-repeat="contact in settings.contacts">
     *          <select ng-model="contact.type" aria-label="Contact method" id="select_{{$index}}">
     *             <option>phone</option>
     *             <option>email</option>
     *          </select>
     *          <input type="text" ng-model="contact.value" aria-labelledby="select_{{$index}}" />
     *          <button ng-click="settings.clearContact(contact)">clear</button>
     *          <button ng-click="settings.removeContact(contact)" aria-label="Remove">X</button>
     *        </li>
     *        <li><button ng-click="settings.addContact()">add</button></li>
     *     </ul>
     *    </div>
     *   </file>
     *   <file name="app.js">
     *    angular.module('controllerAsExample', [])
     *      .controller('SettingsController1', SettingsController1);
     *
     *    function SettingsController1() {
     *      this.name = 'John Smith';
     *      this.contacts = [
     *        {type: 'phone', value: '408 555 1212'},
     *        {type: 'email', value: 'john.smith@example.org'}
     *      ];
     *    }
     *
     *    SettingsController1.prototype.greet = function() {
     *      alert(this.name);
     *    };
     *
     *    SettingsController1.prototype.addContact = function() {
     *      this.contacts.push({type: 'email', value: 'yourname@example.org'});
     *    };
     *
     *    SettingsController1.prototype.removeContact = function(contactToRemove) {
     *     var index = this.contacts.indexOf(contactToRemove);
     *      this.contacts.splice(index, 1);
     *    };
     *
     *    SettingsController1.prototype.clearContact = function(contact) {
     *      contact.type = 'phone';
     *      contact.value = '';
     *    };
     *   </file>
     *   <file name="protractor.js" type="protractor">
     *     it('should check controller as', function() {
     *       var container = element(by.id('ctrl-as-exmpl'));
     *         expect(container.element(by.model('settings.name'))
     *           .getAttribute('value')).toBe('John Smith');
     *
     *       var firstRepeat =
     *           container.element(by.repeater('contact in settings.contacts').row(0));
     *       var secondRepeat =
     *           container.element(by.repeater('contact in settings.contacts').row(1));
     *
     *       expect(firstRepeat.element(by.model('contact.value')).getAttribute('value'))
     *           .toBe('408 555 1212');
     *
     *       expect(secondRepeat.element(by.model('contact.value')).getAttribute('value'))
     *           .toBe('john.smith@example.org');
     *
     *       firstRepeat.element(by.buttonText('clear')).click();
     *
     *       expect(firstRepeat.element(by.model('contact.value')).getAttribute('value'))
     *           .toBe('');
     *
     *       container.element(by.buttonText('add')).click();
     *
     *       expect(container.element(by.repeater('contact in settings.contacts').row(2))
     *           .element(by.model('contact.value'))
     *           .getAttribute('value'))
     *           .toBe('yourname@example.org');
     *     });
     *   </file>
     * </example>
     *
     * This example demonstrates the "attach to `$scope`" style of controller.
     *
     * <example name="ngController" module="controllerExample">
     *  <file name="index.html">
     *   <div id="ctrl-exmpl" ng-controller="SettingsController2">
     *     <label>Name: <input type="text" ng-model="name"/></label>
     *     <button ng-click="greet()">greet</button><br/>
     *     Contact:
     *     <ul>
     *       <li ng-repeat="contact in contacts">
     *         <select ng-model="contact.type" id="select_{{$index}}">
     *            <option>phone</option>
     *            <option>email</option>
     *         </select>
     *         <input type="text" ng-model="contact.value" aria-labelledby="select_{{$index}}" />
     *         <button ng-click="clearContact(contact)">clear</button>
     *         <button ng-click="removeContact(contact)">X</button>
     *       </li>
     *       <li>[ <button ng-click="addContact()">add</button> ]</li>
     *    </ul>
     *   </div>
     *  </file>
     *  <file name="app.js">
     *   angular.module('controllerExample', [])
     *     .controller('SettingsController2', ['$scope', SettingsController2]);
     *
     *   function SettingsController2($scope) {
     *     $scope.name = 'John Smith';
     *     $scope.contacts = [
     *       {type:'phone', value:'408 555 1212'},
     *       {type:'email', value:'john.smith@example.org'}
     *     ];
     *
     *     $scope.greet = function() {
     *       alert($scope.name);
     *     };
     *
     *     $scope.addContact = function() {
     *       $scope.contacts.push({type:'email', value:'yourname@example.org'});
     *     };
     *
     *     $scope.removeContact = function(contactToRemove) {
     *       var index = $scope.contacts.indexOf(contactToRemove);
     *       $scope.contacts.splice(index, 1);
     *     };
     *
     *     $scope.clearContact = function(contact) {
     *       contact.type = 'phone';
     *       contact.value = '';
     *     };
     *   }
     *  </file>
     *  <file name="protractor.js" type="protractor">
     *    it('should check controller', function() {
     *      var container = element(by.id('ctrl-exmpl'));
     *
     *      expect(container.element(by.model('name'))
     *          .getAttribute('value')).toBe('John Smith');
     *
     *      var firstRepeat =
     *          container.element(by.repeater('contact in contacts').row(0));
     *      var secondRepeat =
     *          container.element(by.repeater('contact in contacts').row(1));
     *
     *      expect(firstRepeat.element(by.model('contact.value')).getAttribute('value'))
     *          .toBe('408 555 1212');
     *      expect(secondRepeat.element(by.model('contact.value')).getAttribute('value'))
     *          .toBe('john.smith@example.org');
     *
     *      firstRepeat.element(by.buttonText('clear')).click();
     *
     *      expect(firstRepeat.element(by.model('contact.value')).getAttribute('value'))
     *          .toBe('');
     *
     *      container.element(by.buttonText('add')).click();
     *
     *      expect(container.element(by.repeater('contact in contacts').row(2))
     *          .element(by.model('contact.value'))
     *          .getAttribute('value'))
     *          .toBe('yourname@example.org');
     *    });
     *  </file>
     *</example>
    
     */
    var ngControllerDirective = [function () {
      return {
        restrict: 'A',
        scope: true,
        controller: '@',
        priority: 500
      };
    }];

    /**
     * @ngdoc directive
     * @name ngCsp
     *
     * @restrict A
     * @element ANY
     * @description
     *
     * Angular has some features that can conflict with certain restrictions that are applied when using
     * [CSP (Content Security Policy)](https://developer.mozilla.org/en/Security/CSP) rules.
     *
     * If you intend to implement CSP with these rules then you must tell Angular not to use these
     * features.
     *
     * This is necessary when developing things like Google Chrome Extensions or Universal Windows Apps.
     *
     *
     * The following default rules in CSP affect Angular:
     *
     * * The use of `eval()`, `Function(string)` and similar functions to dynamically create and execute
     * code from strings is forbidden. Angular makes use of this in the {@link $parse} service to
     * provide a 30% increase in the speed of evaluating Angular expressions. (This CSP rule can be
     * disabled with the CSP keyword `unsafe-eval`, but it is generally not recommended as it would
     * weaken the protections offered by CSP.)
     *
     * * The use of inline resources, such as inline `<script>` and `<style>` elements, are forbidden.
     * This prevents apps from injecting custom styles directly into the document. Angular makes use of
     * this to include some CSS rules (e.g. {@link ngCloak} and {@link ngHide}). To make these
     * directives work when a CSP rule is blocking inline styles, you must link to the `angular-csp.css`
     * in your HTML manually. (This CSP rule can be disabled with the CSP keyword `unsafe-inline`, but
     * it is generally not recommended as it would weaken the protections offered by CSP.)
     *
     * If you do not provide `ngCsp` then Angular tries to autodetect if CSP is blocking dynamic code
     * creation from strings (e.g., `unsafe-eval` not specified in CSP header) and automatically
     * deactivates this feature in the {@link $parse} service. This autodetection, however, triggers a
     * CSP error to be logged in the console:
     *
     * ```
     * Refused to evaluate a string as JavaScript because 'unsafe-eval' is not an allowed source of
     * script in the following Content Security Policy directive: "default-src 'self'". Note that
     * 'script-src' was not explicitly set, so 'default-src' is used as a fallback.
     * ```
     *
     * This error is harmless but annoying. To prevent the error from showing up, put the `ngCsp`
     * directive on an element of the HTML document that appears before the `<script>` tag that loads
     * the `angular.js` file.
     *
     * *Note: This directive is only available in the `ng-csp` and `data-ng-csp` attribute form.*
     *
     * You can specify which of the CSP related Angular features should be deactivated by providing
     * a value for the `ng-csp` attribute. The options are as follows:
     *
     * * no-inline-style: this stops Angular from injecting CSS styles into the DOM
     *
     * * no-unsafe-eval: this stops Angular from optimizing $parse with unsafe eval of strings
     *
     * You can use these values in the following combinations:
     *
     *
     * * No declaration means that Angular will assume that you can do inline styles, but it will do
     * a runtime check for unsafe-eval. E.g. `<body>`. This is backwardly compatible with previous
     * versions of Angular.
     *
     * * A simple `ng-csp` (or `data-ng-csp`) attribute will tell Angular to deactivate both inline
     * styles and unsafe eval. E.g. `<body ng-csp>`. This is backwardly compatible with previous
     * versions of Angular.
     *
     * * Specifying only `no-unsafe-eval` tells Angular that we must not use eval, but that we can
     * inject inline styles. E.g. `<body ng-csp="no-unsafe-eval">`.
     *
     * * Specifying only `no-inline-style` tells Angular that we must not inject styles, but that we can
     * run eval - no automatic check for unsafe eval will occur. E.g. `<body ng-csp="no-inline-style">`
     *
     * * Specifying both `no-unsafe-eval` and `no-inline-style` tells Angular that we must not inject
     * styles nor use eval, which is the same as an empty: ng-csp.
     * E.g.`<body ng-csp="no-inline-style;no-unsafe-eval">`
     *
     * @example
     * This example shows how to apply the `ngCsp` directive to the `html` tag.
       ```html
         <!doctype html>
         <html ng-app ng-csp>
         ...
         ...
         </html>
       ```
      * @example
          <!-- Note: the `.csp` suffix in the example name triggers CSP mode in our http server! -->
          <example name="example.csp" module="cspExample" ng-csp="true">
            <file name="index.html">
              <div ng-controller="MainController as ctrl">
                <div>
                  <button ng-click="ctrl.inc()" id="inc">Increment</button>
                  <span id="counter">
                    {{ctrl.counter}}
                  </span>
                </div>
    
                <div>
                  <button ng-click="ctrl.evil()" id="evil">Evil</button>
                  <span id="evilError">
                    {{ctrl.evilError}}
                  </span>
                </div>
              </div>
            </file>
            <file name="script.js">
               angular.module('cspExample', [])
                 .controller('MainController', function MainController() {
                    this.counter = 0;
                    this.inc = function() {
                      this.counter++;
                    };
                    this.evil = function() {
                      try {
                        eval('1+2'); // eslint-disable-line no-eval
                      } catch (e) {
                        this.evilError = e.message;
                      }
                    };
                  });
            </file>
            <file name="protractor.js" type="protractor">
              var util, webdriver;
    
              var incBtn = element(by.id('inc'));
              var counter = element(by.id('counter'));
              var evilBtn = element(by.id('evil'));
              var evilError = element(by.id('evilError'));
    
              function getAndClearSevereErrors() {
                return browser.manage().logs().get('browser').then(function(browserLog) {
                  return browserLog.filter(function(logEntry) {
                    return logEntry.level.value > webdriver.logging.Level.WARNING.value;
                  });
                });
              }
    
              function clearErrors() {
                getAndClearSevereErrors();
              }
    
              function expectNoErrors() {
                getAndClearSevereErrors().then(function(filteredLog) {
                  expect(filteredLog.length).toEqual(0);
                  if (filteredLog.length) {
                    console.log('browser console errors: ' + util.inspect(filteredLog));
                  }
                });
              }
    
              function expectError(regex) {
                getAndClearSevereErrors().then(function(filteredLog) {
                  var found = false;
                  filteredLog.forEach(function(log) {
                    if (log.message.match(regex)) {
                      found = true;
                    }
                  });
                  if (!found) {
                    throw new Error('expected an error that matches ' + regex);
                  }
                });
              }
    
              beforeEach(function() {
                util = require('util');
                webdriver = require('selenium-webdriver');
              });
    
              // For now, we only test on Chrome,
              // as Safari does not load the page with Protractor's injected scripts,
              // and Firefox webdriver always disables content security policy (#6358)
              if (browser.params.browser !== 'chrome') {
                return;
              }
    
              it('should not report errors when the page is loaded', function() {
                // clear errors so we are not dependent on previous tests
                clearErrors();
                // Need to reload the page as the page is already loaded when
                // we come here
                browser.driver.getCurrentUrl().then(function(url) {
                  browser.get(url);
                });
                expectNoErrors();
              });
    
              it('should evaluate expressions', function() {
                expect(counter.getText()).toEqual('0');
                incBtn.click();
                expect(counter.getText()).toEqual('1');
                expectNoErrors();
              });
    
              it('should throw and report an error when using "eval"', function() {
                evilBtn.click();
                expect(evilError.getText()).toMatch(/Content Security Policy/);
                expectError(/Content Security Policy/);
              });
            </file>
          </example>
      */

    // `ngCsp` is not implemented as a proper directive any more, because we need it be processed while
    // we bootstrap the app (before `$parse` is instantiated). For this reason, we just have the `csp()`
    // fn that looks for the `ng-csp` attribute anywhere in the current doc.

    /**
     * @ngdoc directive
     * @name ngClick
     *
     * @description
     * The ngClick directive allows you to specify custom behavior when
     * an element is clicked.
     *
     * @element ANY
     * @priority 0
     * @param {expression} ngClick {@link guide/expression Expression} to evaluate upon
     * click. ({@link guide/expression#-event- Event object is available as `$event`})
     *
     * @example
       <example name="ng-click">
         <file name="index.html">
          <button ng-click="count = count + 1" ng-init="count=0">
            Increment
          </button>
          <span>
            count: {{count}}
          </span>
         </file>
         <file name="protractor.js" type="protractor">
           it('should check ng-click', function() {
             expect(element(by.binding('count')).getText()).toMatch('0');
             element(by.css('button')).click();
             expect(element(by.binding('count')).getText()).toMatch('1');
           });
         </file>
       </example>
     */
    /*
     * A collection of directives that allows creation of custom event handlers that are defined as
     * angular expressions and are compiled and executed within the current scope.
     */
    var ngEventDirectives = {};

    // For events that might fire synchronously during DOM manipulation
    // we need to execute their event handlers asynchronously using $evalAsync,
    // so that they are not executed in an inconsistent state.
    var forceAsyncEvents = {
      'blur': true,
      'focus': true
    };
    forEach('click dblclick mousedown mouseup mouseover mouseout mousemove mouseenter mouseleave keydown keyup keypress submit focus blur copy cut paste'.split(' '), function (eventName) {
      var directiveName = directiveNormalize('ng-' + eventName);
      ngEventDirectives[directiveName] = ['$parse', '$rootScope', function ($parse, $rootScope) {
        return {
          restrict: 'A',
          compile: function ($element, attr) {
            // NOTE:
            // We expose the powerful `$event` object on the scope that provides access to the Window,
            // etc. This is OK, because expressions are not sandboxed any more (and the expression
            // sandbox was never meant to be a security feature anyway).
            var fn = $parse(attr[directiveName]);
            return function ngEventHandler(scope, element) {
              element.on(eventName, function (event) {
                var callback = function () {
                  fn(scope, { $event: event });
                };
                if (forceAsyncEvents[eventName] && $rootScope.$$phase) {
                  scope.$evalAsync(callback);
                } else {
                  scope.$apply(callback);
                }
              });
            };
          }
        };
      }];
    });

    /**
     * @ngdoc directive
     * @name ngDblclick
     *
     * @description
     * The `ngDblclick` directive allows you to specify custom behavior on a dblclick event.
     *
     * @element ANY
     * @priority 0
     * @param {expression} ngDblclick {@link guide/expression Expression} to evaluate upon
     * a dblclick. (The Event object is available as `$event`)
     *
     * @example
       <example name="ng-dblclick">
         <file name="index.html">
          <button ng-dblclick="count = count + 1" ng-init="count=0">
            Increment (on double click)
          </button>
          count: {{count}}
         </file>
       </example>
     */

    /**
     * @ngdoc directive
     * @name ngMousedown
     *
     * @description
     * The ngMousedown directive allows you to specify custom behavior on mousedown event.
     *
     * @element ANY
     * @priority 0
     * @param {expression} ngMousedown {@link guide/expression Expression} to evaluate upon
     * mousedown. ({@link guide/expression#-event- Event object is available as `$event`})
     *
     * @example
       <example name="ng-mousedown">
         <file name="index.html">
          <button ng-mousedown="count = count + 1" ng-init="count=0">
            Increment (on mouse down)
          </button>
          count: {{count}}
         </file>
       </example>
     */

    /**
     * @ngdoc directive
     * @name ngMouseup
     *
     * @description
     * Specify custom behavior on mouseup event.
     *
     * @element ANY
     * @priority 0
     * @param {expression} ngMouseup {@link guide/expression Expression} to evaluate upon
     * mouseup. ({@link guide/expression#-event- Event object is available as `$event`})
     *
     * @example
       <example name="ng-mouseup">
         <file name="index.html">
          <button ng-mouseup="count = count + 1" ng-init="count=0">
            Increment (on mouse up)
          </button>
          count: {{count}}
         </file>
       </example>
     */

    /**
     * @ngdoc directive
     * @name ngMouseover
     *
     * @description
     * Specify custom behavior on mouseover event.
     *
     * @element ANY
     * @priority 0
     * @param {expression} ngMouseover {@link guide/expression Expression} to evaluate upon
     * mouseover. ({@link guide/expression#-event- Event object is available as `$event`})
     *
     * @example
       <example name="ng-mouseover">
         <file name="index.html">
          <button ng-mouseover="count = count + 1" ng-init="count=0">
            Increment (when mouse is over)
          </button>
          count: {{count}}
         </file>
       </example>
     */

    /**
     * @ngdoc directive
     * @name ngMouseenter
     *
     * @description
     * Specify custom behavior on mouseenter event.
     *
     * @element ANY
     * @priority 0
     * @param {expression} ngMouseenter {@link guide/expression Expression} to evaluate upon
     * mouseenter. ({@link guide/expression#-event- Event object is available as `$event`})
     *
     * @example
       <example name="ng-mouseenter">
         <file name="index.html">
          <button ng-mouseenter="count = count + 1" ng-init="count=0">
            Increment (when mouse enters)
          </button>
          count: {{count}}
         </file>
       </example>
     */

    /**
     * @ngdoc directive
     * @name ngMouseleave
     *
     * @description
     * Specify custom behavior on mouseleave event.
     *
     * @element ANY
     * @priority 0
     * @param {expression} ngMouseleave {@link guide/expression Expression} to evaluate upon
     * mouseleave. ({@link guide/expression#-event- Event object is available as `$event`})
     *
     * @example
       <example name="ng-mouseleave">
         <file name="index.html">
          <button ng-mouseleave="count = count + 1" ng-init="count=0">
            Increment (when mouse leaves)
          </button>
          count: {{count}}
         </file>
       </example>
     */

    /**
     * @ngdoc directive
     * @name ngMousemove
     *
     * @description
     * Specify custom behavior on mousemove event.
     *
     * @element ANY
     * @priority 0
     * @param {expression} ngMousemove {@link guide/expression Expression} to evaluate upon
     * mousemove. ({@link guide/expression#-event- Event object is available as `$event`})
     *
     * @example
       <example name="ng-mousemove">
         <file name="index.html">
          <button ng-mousemove="count = count + 1" ng-init="count=0">
            Increment (when mouse moves)
          </button>
          count: {{count}}
         </file>
       </example>
     */

    /**
     * @ngdoc directive
     * @name ngKeydown
     *
     * @description
     * Specify custom behavior on keydown event.
     *
     * @element ANY
     * @priority 0
     * @param {expression} ngKeydown {@link guide/expression Expression} to evaluate upon
     * keydown. (Event object is available as `$event` and can be interrogated for keyCode, altKey, etc.)
     *
     * @example
       <example name="ng-keydown">
         <file name="index.html">
          <input ng-keydown="count = count + 1" ng-init="count=0">
          key down count: {{count}}
         </file>
       </example>
     */

    /**
     * @ngdoc directive
     * @name ngKeyup
     *
     * @description
     * Specify custom behavior on keyup event.
     *
     * @element ANY
     * @priority 0
     * @param {expression} ngKeyup {@link guide/expression Expression} to evaluate upon
     * keyup. (Event object is available as `$event` and can be interrogated for keyCode, altKey, etc.)
     *
     * @example
       <example name="ng-keyup">
         <file name="index.html">
           <p>Typing in the input box below updates the key count</p>
           <input ng-keyup="count = count + 1" ng-init="count=0"> key up count: {{count}}
    
           <p>Typing in the input box below updates the keycode</p>
           <input ng-keyup="event=$event">
           <p>event keyCode: {{ event.keyCode }}</p>
           <p>event altKey: {{ event.altKey }}</p>
         </file>
       </example>
     */

    /**
     * @ngdoc directive
     * @name ngKeypress
     *
     * @description
     * Specify custom behavior on keypress event.
     *
     * @element ANY
     * @param {expression} ngKeypress {@link guide/expression Expression} to evaluate upon
     * keypress. ({@link guide/expression#-event- Event object is available as `$event`}
     * and can be interrogated for keyCode, altKey, etc.)
     *
     * @example
       <example name="ng-keypress">
         <file name="index.html">
          <input ng-keypress="count = count + 1" ng-init="count=0">
          key press count: {{count}}
         </file>
       </example>
     */

    /**
     * @ngdoc directive
     * @name ngSubmit
     *
     * @description
     * Enables binding angular expressions to onsubmit events.
     *
     * Additionally it prevents the default action (which for form means sending the request to the
     * server and reloading the current page), but only if the form does not contain `action`,
     * `data-action`, or `x-action` attributes.
     *
     * <div class="alert alert-warning">
     * **Warning:** Be careful not to cause "double-submission" by using both the `ngClick` and
     * `ngSubmit` handlers together. See the
     * {@link form#submitting-a-form-and-preventing-the-default-action `form` directive documentation}
     * for a detailed discussion of when `ngSubmit` may be triggered.
     * </div>
     *
     * @element form
     * @priority 0
     * @param {expression} ngSubmit {@link guide/expression Expression} to eval.
     * ({@link guide/expression#-event- Event object is available as `$event`})
     *
     * @example
       <example module="submitExample" name="ng-submit">
         <file name="index.html">
          <script>
            angular.module('submitExample', [])
              .controller('ExampleController', ['$scope', function($scope) {
                $scope.list = [];
                $scope.text = 'hello';
                $scope.submit = function() {
                  if ($scope.text) {
                    $scope.list.push(this.text);
                    $scope.text = '';
                  }
                };
              }]);
          </script>
          <form ng-submit="submit()" ng-controller="ExampleController">
            Enter text and hit enter:
            <input type="text" ng-model="text" name="text" />
            <input type="submit" id="submit" value="Submit" />
            <pre>list={{list}}</pre>
          </form>
         </file>
         <file name="protractor.js" type="protractor">
           it('should check ng-submit', function() {
             expect(element(by.binding('list')).getText()).toBe('list=[]');
             element(by.css('#submit')).click();
             expect(element(by.binding('list')).getText()).toContain('hello');
             expect(element(by.model('text')).getAttribute('value')).toBe('');
           });
           it('should ignore empty strings', function() {
             expect(element(by.binding('list')).getText()).toBe('list=[]');
             element(by.css('#submit')).click();
             element(by.css('#submit')).click();
             expect(element(by.binding('list')).getText()).toContain('hello');
            });
         </file>
       </example>
     */

    /**
     * @ngdoc directive
     * @name ngFocus
     *
     * @description
     * Specify custom behavior on focus event.
     *
     * Note: As the `focus` event is executed synchronously when calling `input.focus()`
     * AngularJS executes the expression using `scope.$evalAsync` if the event is fired
     * during an `$apply` to ensure a consistent state.
     *
     * @element window, input, select, textarea, a
     * @priority 0
     * @param {expression} ngFocus {@link guide/expression Expression} to evaluate upon
     * focus. ({@link guide/expression#-event- Event object is available as `$event`})
     *
     * @example
     * See {@link ng.directive:ngClick ngClick}
     */

    /**
     * @ngdoc directive
     * @name ngBlur
     *
     * @description
     * Specify custom behavior on blur event.
     *
     * A [blur event](https://developer.mozilla.org/en-US/docs/Web/Events/blur) fires when
     * an element has lost focus.
     *
     * Note: As the `blur` event is executed synchronously also during DOM manipulations
     * (e.g. removing a focussed input),
     * AngularJS executes the expression using `scope.$evalAsync` if the event is fired
     * during an `$apply` to ensure a consistent state.
     *
     * @element window, input, select, textarea, a
     * @priority 0
     * @param {expression} ngBlur {@link guide/expression Expression} to evaluate upon
     * blur. ({@link guide/expression#-event- Event object is available as `$event`})
     *
     * @example
     * See {@link ng.directive:ngClick ngClick}
     */

    /**
     * @ngdoc directive
     * @name ngCopy
     *
     * @description
     * Specify custom behavior on copy event.
     *
     * @element window, input, select, textarea, a
     * @priority 0
     * @param {expression} ngCopy {@link guide/expression Expression} to evaluate upon
     * copy. ({@link guide/expression#-event- Event object is available as `$event`})
     *
     * @example
       <example name="ng-copy">
         <file name="index.html">
          <input ng-copy="copied=true" ng-init="copied=false; value='copy me'" ng-model="value">
          copied: {{copied}}
         </file>
       </example>
     */

    /**
     * @ngdoc directive
     * @name ngCut
     *
     * @description
     * Specify custom behavior on cut event.
     *
     * @element window, input, select, textarea, a
     * @priority 0
     * @param {expression} ngCut {@link guide/expression Expression} to evaluate upon
     * cut. ({@link guide/expression#-event- Event object is available as `$event`})
     *
     * @example
       <example name="ng-cut">
         <file name="index.html">
          <input ng-cut="cut=true" ng-init="cut=false; value='cut me'" ng-model="value">
          cut: {{cut}}
         </file>
       </example>
     */

    /**
     * @ngdoc directive
     * @name ngPaste
     *
     * @description
     * Specify custom behavior on paste event.
     *
     * @element window, input, select, textarea, a
     * @priority 0
     * @param {expression} ngPaste {@link guide/expression Expression} to evaluate upon
     * paste. ({@link guide/expression#-event- Event object is available as `$event`})
     *
     * @example
       <example name="ng-paste">
         <file name="index.html">
          <input ng-paste="paste=true" ng-init="paste=false" placeholder='paste here'>
          pasted: {{paste}}
         </file>
       </example>
     */

    /**
     * @ngdoc directive
     * @name ngIf
     * @restrict A
     * @multiElement
     *
     * @description
     * The `ngIf` directive removes or recreates a portion of the DOM tree based on an
     * {expression}. If the expression assigned to `ngIf` evaluates to a false
     * value then the element is removed from the DOM, otherwise a clone of the
     * element is reinserted into the DOM.
     *
     * `ngIf` differs from `ngShow` and `ngHide` in that `ngIf` completely removes and recreates the
     * element in the DOM rather than changing its visibility via the `display` css property.  A common
     * case when this difference is significant is when using css selectors that rely on an element's
     * position within the DOM, such as the `:first-child` or `:last-child` pseudo-classes.
     *
     * Note that when an element is removed using `ngIf` its scope is destroyed and a new scope
     * is created when the element is restored.  The scope created within `ngIf` inherits from
     * its parent scope using
     * [prototypal inheritance](https://github.com/angular/angular.js/wiki/Understanding-Scopes#javascript-prototypal-inheritance).
     * An important implication of this is if `ngModel` is used within `ngIf` to bind to
     * a javascript primitive defined in the parent scope. In this case any modifications made to the
     * variable within the child scope will override (hide) the value in the parent scope.
     *
     * Also, `ngIf` recreates elements using their compiled state. An example of this behavior
     * is if an element's class attribute is directly modified after it's compiled, using something like
     * jQuery's `.addClass()` method, and the element is later removed. When `ngIf` recreates the element
     * the added class will be lost because the original compiled state is used to regenerate the element.
     *
     * Additionally, you can provide animations via the `ngAnimate` module to animate the `enter`
     * and `leave` effects.
     *
     * @animations
     * | Animation                        | Occurs                               |
     * |----------------------------------|-------------------------------------|
     * | {@link ng.$animate#enter enter}  | just after the `ngIf` contents change and a new DOM element is created and injected into the `ngIf` container |
     * | {@link ng.$animate#leave leave}  | just before the `ngIf` contents are removed from the DOM |
     *
     * @element ANY
     * @scope
     * @priority 600
     * @param {expression} ngIf If the {@link guide/expression expression} is falsy then
     *     the element is removed from the DOM tree. If it is truthy a copy of the compiled
     *     element is added to the DOM tree.
     *
     * @example
      <example module="ngAnimate" deps="angular-animate.js" animations="true" name="ng-if">
        <file name="index.html">
          <label>Click me: <input type="checkbox" ng-model="checked" ng-init="checked=true" /></label><br/>
          Show when checked:
          <span ng-if="checked" class="animate-if">
            This is removed when the checkbox is unchecked.
          </span>
        </file>
        <file name="animations.css">
          .animate-if {
            background:white;
            border:1px solid black;
            padding:10px;
          }
    
          .animate-if.ng-enter, .animate-if.ng-leave {
            transition:all cubic-bezier(0.250, 0.460, 0.450, 0.940) 0.5s;
          }
    
          .animate-if.ng-enter,
          .animate-if.ng-leave.ng-leave-active {
            opacity:0;
          }
    
          .animate-if.ng-leave,
          .animate-if.ng-enter.ng-enter-active {
            opacity:1;
          }
        </file>
      </example>
     */
    var ngIfDirective = ['$animate', '$compile', function ($animate, $compile) {
      return {
        multiElement: true,
        transclude: 'element',
        priority: 600,
        terminal: true,
        restrict: 'A',
        $$tlb: true,
        link: function ($scope, $element, $attr, ctrl, $transclude) {
          var block, childScope, previousElements;
          $scope.$watch($attr.ngIf, function ngIfWatchAction(value) {

            if (value) {
              if (!childScope) {
                $transclude(function (clone, newScope) {
                  childScope = newScope;
                  clone[clone.length++] = $compile.$$createComment('end ngIf', $attr.ngIf);
                  // Note: We only need the first/last node of the cloned nodes.
                  // However, we need to keep the reference to the jqlite wrapper as it might be changed later
                  // by a directive with templateUrl when its template arrives.
                  block = {
                    clone: clone
                  };
                  $animate.enter(clone, $element.parent(), $element);
                });
              }
            } else {
              if (previousElements) {
                previousElements.remove();
                previousElements = null;
              }
              if (childScope) {
                childScope.$destroy();
                childScope = null;
              }
              if (block) {
                previousElements = getBlockNodes(block.clone);
                $animate.leave(previousElements).done(function (response) {
                  if (response !== false) previousElements = null;
                });
                block = null;
              }
            }
          });
        }
      };
    }];

    /**
     * @ngdoc directive
     * @name ngInclude
     * @restrict ECA
     *
     * @description
     * Fetches, compiles and includes an external HTML fragment.
     *
     * By default, the template URL is restricted to the same domain and protocol as the
     * application document. This is done by calling {@link $sce#getTrustedResourceUrl
     * $sce.getTrustedResourceUrl} on it. To load templates from other domains or protocols
     * you may either {@link ng.$sceDelegateProvider#resourceUrlWhitelist whitelist them} or
     * {@link $sce#trustAsResourceUrl wrap them} as trusted values. Refer to Angular's {@link
     * ng.$sce Strict Contextual Escaping}.
     *
     * In addition, the browser's
     * [Same Origin Policy](https://code.google.com/p/browsersec/wiki/Part2#Same-origin_policy_for_XMLHttpRequest)
     * and [Cross-Origin Resource Sharing (CORS)](http://www.w3.org/TR/cors/)
     * policy may further restrict whether the template is successfully loaded.
     * For example, `ngInclude` won't work for cross-domain requests on all browsers and for `file://`
     * access on some browsers.
     *
     * @animations
     * | Animation                        | Occurs                              |
     * |----------------------------------|-------------------------------------|
     * | {@link ng.$animate#enter enter}  | when the expression changes, on the new include |
     * | {@link ng.$animate#leave leave}  | when the expression changes, on the old include |
     *
     * The enter and leave animation occur concurrently.
     *
     * @scope
     * @priority 400
     *
     * @param {string} ngInclude|src angular expression evaluating to URL. If the source is a string constant,
     *                 make sure you wrap it in **single** quotes, e.g. `src="'myPartialTemplate.html'"`.
     * @param {string=} onload Expression to evaluate when a new partial is loaded.
     *                  <div class="alert alert-warning">
     *                  **Note:** When using onload on SVG elements in IE11, the browser will try to call
     *                  a function with the name on the window element, which will usually throw a
     *                  "function is undefined" error. To fix this, you can instead use `data-onload` or a
     *                  different form that {@link guide/directive#normalization matches} `onload`.
     *                  </div>
       *
     * @param {string=} autoscroll Whether `ngInclude` should call {@link ng.$anchorScroll
     *                  $anchorScroll} to scroll the viewport after the content is loaded.
     *
     *                  - If the attribute is not set, disable scrolling.
     *                  - If the attribute is set without value, enable scrolling.
     *                  - Otherwise enable scrolling only if the expression evaluates to truthy value.
     *
     * @example
      <example module="includeExample" deps="angular-animate.js" animations="true" name="ng-include">
        <file name="index.html">
         <div ng-controller="ExampleController">
           <select ng-model="template" ng-options="t.name for t in templates">
            <option value="">(blank)</option>
           </select>
           url of the template: <code>{{template.url}}</code>
           <hr/>
           <div class="slide-animate-container">
             <div class="slide-animate" ng-include="template.url"></div>
           </div>
         </div>
        </file>
        <file name="script.js">
          angular.module('includeExample', ['ngAnimate'])
            .controller('ExampleController', ['$scope', function($scope) {
              $scope.templates =
                [{ name: 'template1.html', url: 'template1.html'},
                 { name: 'template2.html', url: 'template2.html'}];
              $scope.template = $scope.templates[0];
            }]);
         </file>
        <file name="template1.html">
          Content of template1.html
        </file>
        <file name="template2.html">
          Content of template2.html
        </file>
        <file name="animations.css">
          .slide-animate-container {
            position:relative;
            background:white;
            border:1px solid black;
            height:40px;
            overflow:hidden;
          }
    
          .slide-animate {
            padding:10px;
          }
    
          .slide-animate.ng-enter, .slide-animate.ng-leave {
            transition:all cubic-bezier(0.250, 0.460, 0.450, 0.940) 0.5s;
    
            position:absolute;
            top:0;
            left:0;
            right:0;
            bottom:0;
            display:block;
            padding:10px;
          }
    
          .slide-animate.ng-enter {
            top:-50px;
          }
          .slide-animate.ng-enter.ng-enter-active {
            top:0;
          }
    
          .slide-animate.ng-leave {
            top:0;
          }
          .slide-animate.ng-leave.ng-leave-active {
            top:50px;
          }
        </file>
        <file name="protractor.js" type="protractor">
          var templateSelect = element(by.model('template'));
          var includeElem = element(by.css('[ng-include]'));
    
          it('should load template1.html', function() {
            expect(includeElem.getText()).toMatch(/Content of template1.html/);
          });
    
          it('should load template2.html', function() {
            if (browser.params.browser === 'firefox') {
              // Firefox can't handle using selects
              // See https://github.com/angular/protractor/issues/480
              return;
            }
            templateSelect.click();
            templateSelect.all(by.css('option')).get(2).click();
            expect(includeElem.getText()).toMatch(/Content of template2.html/);
          });
    
          it('should change to blank', function() {
            if (browser.params.browser === 'firefox') {
              // Firefox can't handle using selects
              return;
            }
            templateSelect.click();
            templateSelect.all(by.css('option')).get(0).click();
            expect(includeElem.isPresent()).toBe(false);
          });
        </file>
      </example>
     */

    /**
     * @ngdoc event
     * @name ngInclude#$includeContentRequested
     * @eventType emit on the scope ngInclude was declared in
     * @description
     * Emitted every time the ngInclude content is requested.
     *
     * @param {Object} angularEvent Synthetic event object.
     * @param {String} src URL of content to load.
     */

    /**
     * @ngdoc event
     * @name ngInclude#$includeContentLoaded
     * @eventType emit on the current ngInclude scope
     * @description
     * Emitted every time the ngInclude content is reloaded.
     *
     * @param {Object} angularEvent Synthetic event object.
     * @param {String} src URL of content to load.
     */

    /**
     * @ngdoc event
     * @name ngInclude#$includeContentError
     * @eventType emit on the scope ngInclude was declared in
     * @description
     * Emitted when a template HTTP request yields an erroneous response (status < 200 || status > 299)
     *
     * @param {Object} angularEvent Synthetic event object.
     * @param {String} src URL of content to load.
     */
    var ngIncludeDirective = ['$templateRequest', '$anchorScroll', '$animate', function ($templateRequest, $anchorScroll, $animate) {
      return {
        restrict: 'ECA',
        priority: 400,
        terminal: true,
        transclude: 'element',
        controller: angular.noop,
        compile: function (element, attr) {
          var srcExp = attr.ngInclude || attr.src,
              onloadExp = attr.onload || '',
              autoScrollExp = attr.autoscroll;

          return function (scope, $element, $attr, ctrl, $transclude) {
            var changeCounter = 0,
                currentScope,
                previousElement,
                currentElement;

            var cleanupLastIncludeContent = function () {
              if (previousElement) {
                previousElement.remove();
                previousElement = null;
              }
              if (currentScope) {
                currentScope.$destroy();
                currentScope = null;
              }
              if (currentElement) {
                $animate.leave(currentElement).done(function (response) {
                  if (response !== false) previousElement = null;
                });
                previousElement = currentElement;
                currentElement = null;
              }
            };

            scope.$watch(srcExp, function ngIncludeWatchAction(src) {
              var afterAnimation = function (response) {
                if (response !== false && isDefined(autoScrollExp) && (!autoScrollExp || scope.$eval(autoScrollExp))) {
                  $anchorScroll();
                }
              };
              var thisChangeId = ++changeCounter;

              if (src) {
                //set the 2nd param to true to ignore the template request error so that the inner
                //contents and scope can be cleaned up.
                $templateRequest(src, true).then(function (response) {
                  if (scope.$$destroyed) return;

                  if (thisChangeId !== changeCounter) return;
                  var newScope = scope.$new();
                  ctrl.template = response;

                  // Note: This will also link all children of ng-include that were contained in the original
                  // html. If that content contains controllers, ... they could pollute/change the scope.
                  // However, using ng-include on an element with additional content does not make sense...
                  // Note: We can't remove them in the cloneAttchFn of $transclude as that
                  // function is called before linking the content, which would apply child
                  // directives to non existing elements.
                  var clone = $transclude(newScope, function (clone) {
                    cleanupLastIncludeContent();
                    $animate.enter(clone, null, $element).done(afterAnimation);
                  });

                  currentScope = newScope;
                  currentElement = clone;

                  currentScope.$emit('$includeContentLoaded', src);
                  scope.$eval(onloadExp);
                }, function () {
                  if (scope.$$destroyed) return;

                  if (thisChangeId === changeCounter) {
                    cleanupLastIncludeContent();
                    scope.$emit('$includeContentError', src);
                  }
                });
                scope.$emit('$includeContentRequested', src);
              } else {
                cleanupLastIncludeContent();
                ctrl.template = null;
              }
            });
          };
        }
      };
    }];

    // This directive is called during the $transclude call of the first `ngInclude` directive.
    // It will replace and compile the content of the element with the loaded template.
    // We need this directive so that the element content is already filled when
    // the link function of another directive on the same element as ngInclude
    // is called.
    var ngIncludeFillContentDirective = ['$compile', function ($compile) {
      return {
        restrict: 'ECA',
        priority: -400,
        require: 'ngInclude',
        link: function (scope, $element, $attr, ctrl) {
          if (toString.call($element[0]).match(/SVG/)) {
            // WebKit: https://bugs.webkit.org/show_bug.cgi?id=135698 --- SVG elements do not
            // support innerHTML, so detect this here and try to generate the contents
            // specially.
            $element.empty();
            $compile(jqLiteBuildFragment(ctrl.template, window.document).childNodes)(scope, function namespaceAdaptedClone(clone) {
              $element.append(clone);
            }, { futureParentElement: $element });
            return;
          }

          $element.html(ctrl.template);
          $compile($element.contents())(scope);
        }
      };
    }];

    /**
     * @ngdoc directive
     * @name ngInit
     * @restrict AC
     *
     * @description
     * The `ngInit` directive allows you to evaluate an expression in the
     * current scope.
     *
     * <div class="alert alert-danger">
     * This directive can be abused to add unnecessary amounts of logic into your templates.
     * There are only a few appropriate uses of `ngInit`, such as for aliasing special properties of
     * {@link ng.directive:ngRepeat `ngRepeat`}, as seen in the demo below; and for injecting data via
     * server side scripting. Besides these few cases, you should use {@link guide/controller controllers}
     * rather than `ngInit` to initialize values on a scope.
     * </div>
     *
     * <div class="alert alert-warning">
     * **Note**: If you have assignment in `ngInit` along with a {@link ng.$filter `filter`}, make
     * sure you have parentheses to ensure correct operator precedence:
     * <pre class="prettyprint">
     * `<div ng-init="test1 = ($index | toString)"></div>`
     * </pre>
     * </div>
     *
     * @priority 450
     *
     * @element ANY
     * @param {expression} ngInit {@link guide/expression Expression} to eval.
     *
     * @example
       <example module="initExample" name="ng-init">
         <file name="index.html">
       <script>
         angular.module('initExample', [])
           .controller('ExampleController', ['$scope', function($scope) {
             $scope.list = [['a', 'b'], ['c', 'd']];
           }]);
       </script>
       <div ng-controller="ExampleController">
         <div ng-repeat="innerList in list" ng-init="outerIndex = $index">
           <div ng-repeat="value in innerList" ng-init="innerIndex = $index">
              <span class="example-init">list[ {{outerIndex}} ][ {{innerIndex}} ] = {{value}};</span>
           </div>
         </div>
       </div>
         </file>
         <file name="protractor.js" type="protractor">
           it('should alias index positions', function() {
             var elements = element.all(by.css('.example-init'));
             expect(elements.get(0).getText()).toBe('list[ 0 ][ 0 ] = a;');
             expect(elements.get(1).getText()).toBe('list[ 0 ][ 1 ] = b;');
             expect(elements.get(2).getText()).toBe('list[ 1 ][ 0 ] = c;');
             expect(elements.get(3).getText()).toBe('list[ 1 ][ 1 ] = d;');
           });
         </file>
       </example>
     */
    var ngInitDirective = ngDirective({
      priority: 450,
      compile: function () {
        return {
          pre: function (scope, element, attrs) {
            scope.$eval(attrs.ngInit);
          }
        };
      }
    });

    /**
     * @ngdoc directive
     * @name ngList
     *
     * @description
     * Text input that converts between a delimited string and an array of strings. The default
     * delimiter is a comma followed by a space - equivalent to `ng-list=", "`. You can specify a custom
     * delimiter as the value of the `ngList` attribute - for example, `ng-list=" | "`.
     *
     * The behaviour of the directive is affected by the use of the `ngTrim` attribute.
     * * If `ngTrim` is set to `"false"` then whitespace around both the separator and each
     *   list item is respected. This implies that the user of the directive is responsible for
     *   dealing with whitespace but also allows you to use whitespace as a delimiter, such as a
     *   tab or newline character.
     * * Otherwise whitespace around the delimiter is ignored when splitting (although it is respected
     *   when joining the list items back together) and whitespace around each list item is stripped
     *   before it is added to the model.
     *
     * ### Example with Validation
     *
     * <example name="ngList-directive" module="listExample">
     *   <file name="app.js">
     *      angular.module('listExample', [])
     *        .controller('ExampleController', ['$scope', function($scope) {
     *          $scope.names = ['morpheus', 'neo', 'trinity'];
     *        }]);
     *   </file>
     *   <file name="index.html">
     *    <form name="myForm" ng-controller="ExampleController">
     *      <label>List: <input name="namesInput" ng-model="names" ng-list required></label>
     *      <span role="alert">
     *        <span class="error" ng-show="myForm.namesInput.$error.required">
     *        Required!</span>
     *      </span>
     *      <br>
     *      <tt>names = {{names}}</tt><br/>
     *      <tt>myForm.namesInput.$valid = {{myForm.namesInput.$valid}}</tt><br/>
     *      <tt>myForm.namesInput.$error = {{myForm.namesInput.$error}}</tt><br/>
     *      <tt>myForm.$valid = {{myForm.$valid}}</tt><br/>
     *      <tt>myForm.$error.required = {{!!myForm.$error.required}}</tt><br/>
     *     </form>
     *   </file>
     *   <file name="protractor.js" type="protractor">
     *     var listInput = element(by.model('names'));
     *     var names = element(by.exactBinding('names'));
     *     var valid = element(by.binding('myForm.namesInput.$valid'));
     *     var error = element(by.css('span.error'));
     *
     *     it('should initialize to model', function() {
     *       expect(names.getText()).toContain('["morpheus","neo","trinity"]');
     *       expect(valid.getText()).toContain('true');
     *       expect(error.getCssValue('display')).toBe('none');
     *     });
     *
     *     it('should be invalid if empty', function() {
     *       listInput.clear();
     *       listInput.sendKeys('');
     *
     *       expect(names.getText()).toContain('');
     *       expect(valid.getText()).toContain('false');
     *       expect(error.getCssValue('display')).not.toBe('none');
     *     });
     *   </file>
     * </example>
     *
     * ### Example - splitting on newline
     * <example name="ngList-directive-newlines">
     *   <file name="index.html">
     *    <textarea ng-model="list" ng-list="&#10;" ng-trim="false"></textarea>
     *    <pre>{{ list | json }}</pre>
     *   </file>
     *   <file name="protractor.js" type="protractor">
     *     it("should split the text by newlines", function() {
     *       var listInput = element(by.model('list'));
     *       var output = element(by.binding('list | json'));
     *       listInput.sendKeys('abc\ndef\nghi');
     *       expect(output.getText()).toContain('[\n  "abc",\n  "def",\n  "ghi"\n]');
     *     });
     *   </file>
     * </example>
     *
     * @element input
     * @param {string=} ngList optional delimiter that should be used to split the value.
     */
    var ngListDirective = function () {
      return {
        restrict: 'A',
        priority: 100,
        require: 'ngModel',
        link: function (scope, element, attr, ctrl) {
          var ngList = attr.ngList || ', ';
          var trimValues = attr.ngTrim !== 'false';
          var separator = trimValues ? trim(ngList) : ngList;

          var parse = function (viewValue) {
            // If the viewValue is invalid (say required but empty) it will be `undefined`
            if (isUndefined(viewValue)) return;

            var list = [];

            if (viewValue) {
              forEach(viewValue.split(separator), function (value) {
                if (value) list.push(trimValues ? trim(value) : value);
              });
            }

            return list;
          };

          ctrl.$parsers.push(parse);
          ctrl.$formatters.push(function (value) {
            if (isArray(value)) {
              return value.join(ngList);
            }

            return undefined;
          });

          // Override the standard $isEmpty because an empty array means the input is empty.
          ctrl.$isEmpty = function (value) {
            return !value || !value.length;
          };
        }
      };
    };

    /* global VALID_CLASS: true,
      INVALID_CLASS: true,
      PRISTINE_CLASS: true,
      DIRTY_CLASS: true,
      UNTOUCHED_CLASS: true,
      TOUCHED_CLASS: true,
      PENDING_CLASS: true,
      addSetValidityMethod: true,
      setupValidity: true,
      defaultModelOptions: false
    */

    var VALID_CLASS = 'ng-valid',
        INVALID_CLASS = 'ng-invalid',
        PRISTINE_CLASS = 'ng-pristine',
        DIRTY_CLASS = 'ng-dirty',
        UNTOUCHED_CLASS = 'ng-untouched',
        TOUCHED_CLASS = 'ng-touched',
        EMPTY_CLASS = 'ng-empty',
        NOT_EMPTY_CLASS = 'ng-not-empty';

    var ngModelMinErr = minErr('ngModel');

    /**
     * @ngdoc type
     * @name ngModel.NgModelController
     *
     * @property {*} $viewValue The actual value from the control's view. For `input` elements, this is a
     * String. See {@link ngModel.NgModelController#$setViewValue} for information about when the $viewValue
     * is set.
     *
     * @property {*} $modelValue The value in the model that the control is bound to.
     *
     * @property {Array.<Function>} $parsers Array of functions to execute, as a pipeline, whenever
     *  the control updates the ngModelController with a new {@link ngModel.NgModelController#$viewValue
        `$viewValue`} from the DOM, usually via user input.
        See {@link ngModel.NgModelController#$setViewValue `$setViewValue()`} for a detailed lifecycle explanation.
        Note that the `$parsers` are not called when the bound ngModel expression changes programmatically.
    
      The functions are called in array order, each passing
        its return value through to the next. The last return value is forwarded to the
        {@link ngModel.NgModelController#$validators `$validators`} collection.
    
      Parsers are used to sanitize / convert the {@link ngModel.NgModelController#$viewValue
        `$viewValue`}.
    
      Returning `undefined` from a parser means a parse error occurred. In that case,
        no {@link ngModel.NgModelController#$validators `$validators`} will run and the `ngModel`
        will be set to `undefined` unless {@link ngModelOptions `ngModelOptions.allowInvalid`}
        is set to `true`. The parse error is stored in `ngModel.$error.parse`.
    
      This simple example shows a parser that would convert text input value to lowercase:
     * ```js
     * function parse(value) {
     *   if (value) {
     *     return value.toLowerCase();
     *   }
     * }
     * ngModelController.$parsers.push(parse);
     * ```
    
     *
     * @property {Array.<Function>} $formatters Array of functions to execute, as a pipeline, whenever
        the bound ngModel expression changes programmatically. The `$formatters` are not called when the
        value of the control is changed by user interaction.
    
      Formatters are used to format / convert the {@link ngModel.NgModelController#$modelValue
        `$modelValue`} for display in the control.
    
      The functions are called in reverse array order, each passing the value through to the
        next. The last return value is used as the actual DOM value.
    
      This simple example shows a formatter that would convert the model value to uppercase:
    
     * ```js
     * function format(value) {
     *   if (value) {
     *     return value.toUpperCase();
     *   }
     * }
     * ngModel.$formatters.push(format);
     * ```
     *
     * @property {Object.<string, function>} $validators A collection of validators that are applied
     *      whenever the model value changes. The key value within the object refers to the name of the
     *      validator while the function refers to the validation operation. The validation operation is
     *      provided with the model value as an argument and must return a true or false value depending
     *      on the response of that validation.
     *
     * ```js
     * ngModel.$validators.validCharacters = function(modelValue, viewValue) {
     *   var value = modelValue || viewValue;
     *   return /[0-9]+/.test(value) &&
     *          /[a-z]+/.test(value) &&
     *          /[A-Z]+/.test(value) &&
     *          /\W+/.test(value);
     * };
     * ```
     *
     * @property {Object.<string, function>} $asyncValidators A collection of validations that are expected to
     *      perform an asynchronous validation (e.g. a HTTP request). The validation function that is provided
     *      is expected to return a promise when it is run during the model validation process. Once the promise
     *      is delivered then the validation status will be set to true when fulfilled and false when rejected.
     *      When the asynchronous validators are triggered, each of the validators will run in parallel and the model
     *      value will only be updated once all validators have been fulfilled. As long as an asynchronous validator
     *      is unfulfilled, its key will be added to the controllers `$pending` property. Also, all asynchronous validators
     *      will only run once all synchronous validators have passed.
     *
     * Please note that if $http is used then it is important that the server returns a success HTTP response code
     * in order to fulfill the validation and a status level of `4xx` in order to reject the validation.
     *
     * ```js
     * ngModel.$asyncValidators.uniqueUsername = function(modelValue, viewValue) {
     *   var value = modelValue || viewValue;
     *
     *   // Lookup user by username
     *   return $http.get('/api/users/' + value).
     *      then(function resolved() {
     *        //username exists, this means validation fails
     *        return $q.reject('exists');
     *      }, function rejected() {
     *        //username does not exist, therefore this validation passes
     *        return true;
     *      });
     * };
     * ```
     *
     * @property {Array.<Function>} $viewChangeListeners Array of functions to execute whenever the
     *     view value has changed. It is called with no arguments, and its return value is ignored.
     *     This can be used in place of additional $watches against the model value.
     *
     * @property {Object} $error An object hash with all failing validator ids as keys.
     * @property {Object} $pending An object hash with all pending validator ids as keys.
     *
     * @property {boolean} $untouched True if control has not lost focus yet.
     * @property {boolean} $touched True if control has lost focus.
     * @property {boolean} $pristine True if user has not interacted with the control yet.
     * @property {boolean} $dirty True if user has already interacted with the control.
     * @property {boolean} $valid True if there is no error.
     * @property {boolean} $invalid True if at least one error on the control.
     * @property {string} $name The name attribute of the control.
     *
     * @description
     *
     * `NgModelController` provides API for the {@link ngModel `ngModel`} directive.
     * The controller contains services for data-binding, validation, CSS updates, and value formatting
     * and parsing. It purposefully does not contain any logic which deals with DOM rendering or
     * listening to DOM events.
     * Such DOM related logic should be provided by other directives which make use of
     * `NgModelController` for data-binding to control elements.
     * Angular provides this DOM logic for most {@link input `input`} elements.
     * At the end of this page you can find a {@link ngModel.NgModelController#custom-control-example
     * custom control example} that uses `ngModelController` to bind to `contenteditable` elements.
     *
     * @example
     * ### Custom Control Example
     * This example shows how to use `NgModelController` with a custom control to achieve
     * data-binding. Notice how different directives (`contenteditable`, `ng-model`, and `required`)
     * collaborate together to achieve the desired result.
     *
     * `contenteditable` is an HTML5 attribute, which tells the browser to let the element
     * contents be edited in place by the user.
     *
     * We are using the {@link ng.service:$sce $sce} service here and include the {@link ngSanitize $sanitize}
     * module to automatically remove "bad" content like inline event listener (e.g. `<span onclick="...">`).
     * However, as we are using `$sce` the model can still decide to provide unsafe content if it marks
     * that content using the `$sce` service.
     *
     * <example name="NgModelController" module="customControl" deps="angular-sanitize.js">
        <file name="style.css">
          [contenteditable] {
            border: 1px solid black;
            background-color: white;
            min-height: 20px;
          }
    
          .ng-invalid {
            border: 1px solid red;
          }
    
        </file>
        <file name="script.js">
          angular.module('customControl', ['ngSanitize']).
            directive('contenteditable', ['$sce', function($sce) {
              return {
                restrict: 'A', // only activate on element attribute
                require: '?ngModel', // get a hold of NgModelController
                link: function(scope, element, attrs, ngModel) {
                  if (!ngModel) return; // do nothing if no ng-model
    
                  // Specify how UI should be updated
                  ngModel.$render = function() {
                    element.html($sce.getTrustedHtml(ngModel.$viewValue || ''));
                  };
    
                  // Listen for change events to enable binding
                  element.on('blur keyup change', function() {
                    scope.$evalAsync(read);
                  });
                  read(); // initialize
    
                  // Write data to the model
                  function read() {
                    var html = element.html();
                    // When we clear the content editable the browser leaves a <br> behind
                    // If strip-br attribute is provided then we strip this out
                    if (attrs.stripBr && html === '<br>') {
                      html = '';
                    }
                    ngModel.$setViewValue(html);
                  }
                }
              };
            }]);
        </file>
        <file name="index.html">
          <form name="myForm">
           <div contenteditable
                name="myWidget" ng-model="userContent"
                strip-br="true"
                required>Change me!</div>
            <span ng-show="myForm.myWidget.$error.required">Required!</span>
           <hr>
           <textarea ng-model="userContent" aria-label="Dynamic textarea"></textarea>
          </form>
        </file>
        <file name="protractor.js" type="protractor">
        it('should data-bind and become invalid', function() {
          if (browser.params.browser === 'safari' || browser.params.browser === 'firefox') {
            // SafariDriver can't handle contenteditable
            // and Firefox driver can't clear contenteditables very well
            return;
          }
          var contentEditable = element(by.css('[contenteditable]'));
          var content = 'Change me!';
    
          expect(contentEditable.getText()).toEqual(content);
    
          contentEditable.clear();
          contentEditable.sendKeys(protractor.Key.BACK_SPACE);
          expect(contentEditable.getText()).toEqual('');
          expect(contentEditable.getAttribute('class')).toMatch(/ng-invalid-required/);
        });
        </file>
     * </example>
     *
     *
     */
    NgModelController.$inject = ['$scope', '$exceptionHandler', '$attrs', '$element', '$parse', '$animate', '$timeout', '$q', '$interpolate'];
    function NgModelController($scope, $exceptionHandler, $attr, $element, $parse, $animate, $timeout, $q, $interpolate) {
      this.$viewValue = Number.NaN;
      this.$modelValue = Number.NaN;
      this.$$rawModelValue = undefined; // stores the parsed modelValue / model set from scope regardless of validity.
      this.$validators = {};
      this.$asyncValidators = {};
      this.$parsers = [];
      this.$formatters = [];
      this.$viewChangeListeners = [];
      this.$untouched = true;
      this.$touched = false;
      this.$pristine = true;
      this.$dirty = false;
      this.$valid = true;
      this.$invalid = false;
      this.$error = {}; // keep invalid keys here
      this.$$success = {}; // keep valid keys here
      this.$pending = undefined; // keep pending keys here
      this.$name = $interpolate($attr.name || '', false)($scope);
      this.$$parentForm = nullFormCtrl;
      this.$options = defaultModelOptions;

      this.$$parsedNgModel = $parse($attr.ngModel);
      this.$$parsedNgModelAssign = this.$$parsedNgModel.assign;
      this.$$ngModelGet = this.$$parsedNgModel;
      this.$$ngModelSet = this.$$parsedNgModelAssign;
      this.$$pendingDebounce = null;
      this.$$parserValid = undefined;

      this.$$currentValidationRunId = 0;

      this.$$scope = $scope;
      this.$$attr = $attr;
      this.$$element = $element;
      this.$$animate = $animate;
      this.$$timeout = $timeout;
      this.$$parse = $parse;
      this.$$q = $q;
      this.$$exceptionHandler = $exceptionHandler;

      setupValidity(this);
      setupModelWatcher(this);
    }

    NgModelController.prototype = {
      $$initGetterSetters: function () {
        if (this.$options.getOption('getterSetter')) {
          var invokeModelGetter = this.$$parse(this.$$attr.ngModel + '()'),
              invokeModelSetter = this.$$parse(this.$$attr.ngModel + '($$$p)');

          this.$$ngModelGet = function ($scope) {
            var modelValue = this.$$parsedNgModel($scope);
            if (isFunction(modelValue)) {
              modelValue = invokeModelGetter($scope);
            }
            return modelValue;
          };
          this.$$ngModelSet = function ($scope, newValue) {
            if (isFunction(this.$$parsedNgModel($scope))) {
              invokeModelSetter($scope, { $$$p: newValue });
            } else {
              this.$$parsedNgModelAssign($scope, newValue);
            }
          };
        } else if (!this.$$parsedNgModel.assign) {
          throw ngModelMinErr('nonassign', 'Expression \'{0}\' is non-assignable. Element: {1}', this.$$attr.ngModel, startingTag(this.$$element));
        }
      },

      /**
       * @ngdoc method
       * @name ngModel.NgModelController#$render
       *
       * @description
       * Called when the view needs to be updated. It is expected that the user of the ng-model
       * directive will implement this method.
       *
       * The `$render()` method is invoked in the following situations:
       *
       * * `$rollbackViewValue()` is called.  If we are rolling back the view value to the last
       *   committed value then `$render()` is called to update the input control.
       * * The value referenced by `ng-model` is changed programmatically and both the `$modelValue` and
       *   the `$viewValue` are different from last time.
       *
       * Since `ng-model` does not do a deep watch, `$render()` is only invoked if the values of
       * `$modelValue` and `$viewValue` are actually different from their previous values. If `$modelValue`
       * or `$viewValue` are objects (rather than a string or number) then `$render()` will not be
       * invoked if you only change a property on the objects.
       */
      $render: noop,

      /**
       * @ngdoc method
       * @name ngModel.NgModelController#$isEmpty
       *
       * @description
       * This is called when we need to determine if the value of an input is empty.
       *
       * For instance, the required directive does this to work out if the input has data or not.
       *
       * The default `$isEmpty` function checks whether the value is `undefined`, `''`, `null` or `NaN`.
       *
       * You can override this for input directives whose concept of being empty is different from the
       * default. The `checkboxInputType` directive does this because in its case a value of `false`
       * implies empty.
       *
       * @param {*} value The value of the input to check for emptiness.
       * @returns {boolean} True if `value` is "empty".
       */
      $isEmpty: function (value) {
        // eslint-disable-next-line no-self-compare
        return isUndefined(value) || value === '' || value === null || value !== value;
      },

      $$updateEmptyClasses: function (value) {
        if (this.$isEmpty(value)) {
          this.$$animate.removeClass(this.$$element, NOT_EMPTY_CLASS);
          this.$$animate.addClass(this.$$element, EMPTY_CLASS);
        } else {
          this.$$animate.removeClass(this.$$element, EMPTY_CLASS);
          this.$$animate.addClass(this.$$element, NOT_EMPTY_CLASS);
        }
      },

      /**
       * @ngdoc method
       * @name ngModel.NgModelController#$setPristine
       *
       * @description
       * Sets the control to its pristine state.
       *
       * This method can be called to remove the `ng-dirty` class and set the control to its pristine
       * state (`ng-pristine` class). A model is considered to be pristine when the control
       * has not been changed from when first compiled.
       */
      $setPristine: function () {
        this.$dirty = false;
        this.$pristine = true;
        this.$$animate.removeClass(this.$$element, DIRTY_CLASS);
        this.$$animate.addClass(this.$$element, PRISTINE_CLASS);
      },

      /**
       * @ngdoc method
       * @name ngModel.NgModelController#$setDirty
       *
       * @description
       * Sets the control to its dirty state.
       *
       * This method can be called to remove the `ng-pristine` class and set the control to its dirty
       * state (`ng-dirty` class). A model is considered to be dirty when the control has been changed
       * from when first compiled.
       */
      $setDirty: function () {
        this.$dirty = true;
        this.$pristine = false;
        this.$$animate.removeClass(this.$$element, PRISTINE_CLASS);
        this.$$animate.addClass(this.$$element, DIRTY_CLASS);
        this.$$parentForm.$setDirty();
      },

      /**
       * @ngdoc method
       * @name ngModel.NgModelController#$setUntouched
       *
       * @description
       * Sets the control to its untouched state.
       *
       * This method can be called to remove the `ng-touched` class and set the control to its
       * untouched state (`ng-untouched` class). Upon compilation, a model is set as untouched
       * by default, however this function can be used to restore that state if the model has
       * already been touched by the user.
       */
      $setUntouched: function () {
        this.$touched = false;
        this.$untouched = true;
        this.$$animate.setClass(this.$$element, UNTOUCHED_CLASS, TOUCHED_CLASS);
      },

      /**
       * @ngdoc method
       * @name ngModel.NgModelController#$setTouched
       *
       * @description
       * Sets the control to its touched state.
       *
       * This method can be called to remove the `ng-untouched` class and set the control to its
       * touched state (`ng-touched` class). A model is considered to be touched when the user has
       * first focused the control element and then shifted focus away from the control (blur event).
       */
      $setTouched: function () {
        this.$touched = true;
        this.$untouched = false;
        this.$$animate.setClass(this.$$element, TOUCHED_CLASS, UNTOUCHED_CLASS);
      },

      /**
       * @ngdoc method
       * @name ngModel.NgModelController#$rollbackViewValue
       *
       * @description
       * Cancel an update and reset the input element's value to prevent an update to the `$modelValue`,
       * which may be caused by a pending debounced event or because the input is waiting for some
       * future event.
       *
       * If you have an input that uses `ng-model-options` to set up debounced updates or updates that
       * depend on special events such as `blur`, there can be a period when the `$viewValue` is out of
       * sync with the ngModel's `$modelValue`.
       *
       * In this case, you can use `$rollbackViewValue()` to manually cancel the debounced / future update
       * and reset the input to the last committed view value.
       *
       * It is also possible that you run into difficulties if you try to update the ngModel's `$modelValue`
       * programmatically before these debounced/future events have resolved/occurred, because Angular's
       * dirty checking mechanism is not able to tell whether the model has actually changed or not.
       *
       * The `$rollbackViewValue()` method should be called before programmatically changing the model of an
       * input which may have such events pending. This is important in order to make sure that the
       * input field will be updated with the new model value and any pending operations are cancelled.
       *
       * <example name="ng-model-cancel-update" module="cancel-update-example">
       *   <file name="app.js">
       *     angular.module('cancel-update-example', [])
       *
       *     .controller('CancelUpdateController', ['$scope', function($scope) {
       *       $scope.model = {value1: '', value2: ''};
       *
       *       $scope.setEmpty = function(e, value, rollback) {
       *         if (e.keyCode === 27) {
       *           e.preventDefault();
       *           if (rollback) {
       *             $scope.myForm[value].$rollbackViewValue();
       *           }
       *           $scope.model[value] = '';
       *         }
       *       };
       *     }]);
       *   </file>
       *   <file name="index.html">
       *     <div ng-controller="CancelUpdateController">
       *       <p>Both of these inputs are only updated if they are blurred. Hitting escape should
       *       empty them. Follow these steps and observe the difference:</p>
       *       <ol>
       *         <li>Type something in the input. You will see that the model is not yet updated</li>
       *         <li>Press the Escape key.
       *           <ol>
       *             <li> In the first example, nothing happens, because the model is already '', and no
       *             update is detected. If you blur the input, the model will be set to the current view.
       *             </li>
       *             <li> In the second example, the pending update is cancelled, and the input is set back
       *             to the last committed view value (''). Blurring the input does nothing.
       *             </li>
       *           </ol>
       *         </li>
       *       </ol>
       *
       *       <form name="myForm" ng-model-options="{ updateOn: 'blur' }">
       *         <div>
       *           <p id="inputDescription1">Without $rollbackViewValue():</p>
       *           <input name="value1" aria-describedby="inputDescription1" ng-model="model.value1"
       *                  ng-keydown="setEmpty($event, 'value1')">
       *           value1: "{{ model.value1 }}"
       *         </div>
       *
       *         <div>
       *           <p id="inputDescription2">With $rollbackViewValue():</p>
       *           <input name="value2" aria-describedby="inputDescription2" ng-model="model.value2"
       *                  ng-keydown="setEmpty($event, 'value2', true)">
       *           value2: "{{ model.value2 }}"
       *         </div>
       *       </form>
       *     </div>
       *   </file>
           <file name="style.css">
              div {
                display: table-cell;
              }
              div:nth-child(1) {
                padding-right: 30px;
              }
             </file>
       * </example>
       */
      $rollbackViewValue: function () {
        this.$$timeout.cancel(this.$$pendingDebounce);
        this.$viewValue = this.$$lastCommittedViewValue;
        this.$render();
      },

      /**
       * @ngdoc method
       * @name ngModel.NgModelController#$validate
       *
       * @description
       * Runs each of the registered validators (first synchronous validators and then
       * asynchronous validators).
       * If the validity changes to invalid, the model will be set to `undefined`,
       * unless {@link ngModelOptions `ngModelOptions.allowInvalid`} is `true`.
       * If the validity changes to valid, it will set the model to the last available valid
       * `$modelValue`, i.e. either the last parsed value or the last value set from the scope.
       */
      $validate: function () {
        // ignore $validate before model is initialized
        if (isNumberNaN(this.$modelValue)) {
          return;
        }

        var viewValue = this.$$lastCommittedViewValue;
        // Note: we use the $$rawModelValue as $modelValue might have been
        // set to undefined during a view -> model update that found validation
        // errors. We can't parse the view here, since that could change
        // the model although neither viewValue nor the model on the scope changed
        var modelValue = this.$$rawModelValue;

        var prevValid = this.$valid;
        var prevModelValue = this.$modelValue;

        var allowInvalid = this.$options.getOption('allowInvalid');

        var that = this;
        this.$$runValidators(modelValue, viewValue, function (allValid) {
          // If there was no change in validity, don't update the model
          // This prevents changing an invalid modelValue to undefined
          if (!allowInvalid && prevValid !== allValid) {
            // Note: Don't check this.$valid here, as we could have
            // external validators (e.g. calculated on the server),
            // that just call $setValidity and need the model value
            // to calculate their validity.
            that.$modelValue = allValid ? modelValue : undefined;

            if (that.$modelValue !== prevModelValue) {
              that.$$writeModelToScope();
            }
          }
        });
      },

      $$runValidators: function (modelValue, viewValue, doneCallback) {
        this.$$currentValidationRunId++;
        var localValidationRunId = this.$$currentValidationRunId;
        var that = this;

        // check parser error
        if (!processParseErrors()) {
          validationDone(false);
          return;
        }
        if (!processSyncValidators()) {
          validationDone(false);
          return;
        }
        processAsyncValidators();

        function processParseErrors() {
          var errorKey = that.$$parserName || 'parse';
          if (isUndefined(that.$$parserValid)) {
            setValidity(errorKey, null);
          } else {
            if (!that.$$parserValid) {
              forEach(that.$validators, function (v, name) {
                setValidity(name, null);
              });
              forEach(that.$asyncValidators, function (v, name) {
                setValidity(name, null);
              });
            }
            // Set the parse error last, to prevent unsetting it, should a $validators key == parserName
            setValidity(errorKey, that.$$parserValid);
            return that.$$parserValid;
          }
          return true;
        }

        function processSyncValidators() {
          var syncValidatorsValid = true;
          forEach(that.$validators, function (validator, name) {
            var result = Boolean(validator(modelValue, viewValue));
            syncValidatorsValid = syncValidatorsValid && result;
            setValidity(name, result);
          });
          if (!syncValidatorsValid) {
            forEach(that.$asyncValidators, function (v, name) {
              setValidity(name, null);
            });
            return false;
          }
          return true;
        }

        function processAsyncValidators() {
          var validatorPromises = [];
          var allValid = true;
          forEach(that.$asyncValidators, function (validator, name) {
            var promise = validator(modelValue, viewValue);
            if (!isPromiseLike(promise)) {
              throw ngModelMinErr('nopromise', 'Expected asynchronous validator to return a promise but got \'{0}\' instead.', promise);
            }
            setValidity(name, undefined);
            validatorPromises.push(promise.then(function () {
              setValidity(name, true);
            }, function () {
              allValid = false;
              setValidity(name, false);
            }));
          });
          if (!validatorPromises.length) {
            validationDone(true);
          } else {
            that.$$q.all(validatorPromises).then(function () {
              validationDone(allValid);
            }, noop);
          }
        }

        function setValidity(name, isValid) {
          if (localValidationRunId === that.$$currentValidationRunId) {
            that.$setValidity(name, isValid);
          }
        }

        function validationDone(allValid) {
          if (localValidationRunId === that.$$currentValidationRunId) {

            doneCallback(allValid);
          }
        }
      },

      /**
       * @ngdoc method
       * @name ngModel.NgModelController#$commitViewValue
       *
       * @description
       * Commit a pending update to the `$modelValue`.
       *
       * Updates may be pending by a debounced event or because the input is waiting for a some future
       * event defined in `ng-model-options`. this method is rarely needed as `NgModelController`
       * usually handles calling this in response to input events.
       */
      $commitViewValue: function () {
        var viewValue = this.$viewValue;

        this.$$timeout.cancel(this.$$pendingDebounce);

        // If the view value has not changed then we should just exit, except in the case where there is
        // a native validator on the element. In this case the validation state may have changed even though
        // the viewValue has stayed empty.
        if (this.$$lastCommittedViewValue === viewValue && (viewValue !== '' || !this.$$hasNativeValidators)) {
          return;
        }
        this.$$updateEmptyClasses(viewValue);
        this.$$lastCommittedViewValue = viewValue;

        // change to dirty
        if (this.$pristine) {
          this.$setDirty();
        }
        this.$$parseAndValidate();
      },

      $$parseAndValidate: function () {
        var viewValue = this.$$lastCommittedViewValue;
        var modelValue = viewValue;
        var that = this;

        this.$$parserValid = isUndefined(modelValue) ? undefined : true;

        if (this.$$parserValid) {
          for (var i = 0; i < this.$parsers.length; i++) {
            modelValue = this.$parsers[i](modelValue);
            if (isUndefined(modelValue)) {
              this.$$parserValid = false;
              break;
            }
          }
        }
        if (isNumberNaN(this.$modelValue)) {
          // this.$modelValue has not been touched yet...
          this.$modelValue = this.$$ngModelGet(this.$$scope);
        }
        var prevModelValue = this.$modelValue;
        var allowInvalid = this.$options.getOption('allowInvalid');
        this.$$rawModelValue = modelValue;

        if (allowInvalid) {
          this.$modelValue = modelValue;
          writeToModelIfNeeded();
        }

        // Pass the $$lastCommittedViewValue here, because the cached viewValue might be out of date.
        // This can happen if e.g. $setViewValue is called from inside a parser
        this.$$runValidators(modelValue, this.$$lastCommittedViewValue, function (allValid) {
          if (!allowInvalid) {
            // Note: Don't check this.$valid here, as we could have
            // external validators (e.g. calculated on the server),
            // that just call $setValidity and need the model value
            // to calculate their validity.
            that.$modelValue = allValid ? modelValue : undefined;
            writeToModelIfNeeded();
          }
        });

        function writeToModelIfNeeded() {
          if (that.$modelValue !== prevModelValue) {
            that.$$writeModelToScope();
          }
        }
      },

      $$writeModelToScope: function () {
        this.$$ngModelSet(this.$$scope, this.$modelValue);
        forEach(this.$viewChangeListeners, function (listener) {
          try {
            listener();
          } catch (e) {
            // eslint-disable-next-line no-invalid-this
            this.$$exceptionHandler(e);
          }
        }, this);
      },

      /**
       * @ngdoc method
       * @name ngModel.NgModelController#$setViewValue
       *
       * @description
       * Update the view value.
       *
       * This method should be called when a control wants to change the view value; typically,
       * this is done from within a DOM event handler. For example, the {@link ng.directive:input input}
       * directive calls it when the value of the input changes and {@link ng.directive:select select}
       * calls it when an option is selected.
       *
       * When `$setViewValue` is called, the new `value` will be staged for committing through the `$parsers`
       * and `$validators` pipelines. If there are no special {@link ngModelOptions} specified then the staged
       * value is sent directly for processing through the `$parsers` pipeline. After this, the `$validators` and
       * `$asyncValidators` are called and the value is applied to `$modelValue`.
       * Finally, the value is set to the **expression** specified in the `ng-model` attribute and
       * all the registered change listeners, in the `$viewChangeListeners` list are called.
       *
       * In case the {@link ng.directive:ngModelOptions ngModelOptions} directive is used with `updateOn`
       * and the `default` trigger is not listed, all those actions will remain pending until one of the
       * `updateOn` events is triggered on the DOM element.
       * All these actions will be debounced if the {@link ng.directive:ngModelOptions ngModelOptions}
       * directive is used with a custom debounce for this particular event.
       * Note that a `$digest` is only triggered once the `updateOn` events are fired, or if `debounce`
       * is specified, once the timer runs out.
       *
       * When used with standard inputs, the view value will always be a string (which is in some cases
       * parsed into another type, such as a `Date` object for `input[date]`.)
       * However, custom controls might also pass objects to this method. In this case, we should make
       * a copy of the object before passing it to `$setViewValue`. This is because `ngModel` does not
       * perform a deep watch of objects, it only looks for a change of identity. If you only change
       * the property of the object then ngModel will not realize that the object has changed and
       * will not invoke the `$parsers` and `$validators` pipelines. For this reason, you should
       * not change properties of the copy once it has been passed to `$setViewValue`.
       * Otherwise you may cause the model value on the scope to change incorrectly.
       *
       * <div class="alert alert-info">
       * In any case, the value passed to the method should always reflect the current value
       * of the control. For example, if you are calling `$setViewValue` for an input element,
       * you should pass the input DOM value. Otherwise, the control and the scope model become
       * out of sync. It's also important to note that `$setViewValue` does not call `$render` or change
       * the control's DOM value in any way. If we want to change the control's DOM value
       * programmatically, we should update the `ngModel` scope expression. Its new value will be
       * picked up by the model controller, which will run it through the `$formatters`, `$render` it
       * to update the DOM, and finally call `$validate` on it.
       * </div>
       *
       * @param {*} value value from the view.
       * @param {string} trigger Event that triggered the update.
       */
      $setViewValue: function (value, trigger) {
        this.$viewValue = value;
        if (this.$options.getOption('updateOnDefault')) {
          this.$$debounceViewValueCommit(trigger);
        }
      },

      $$debounceViewValueCommit: function (trigger) {
        var debounceDelay = this.$options.getOption('debounce');

        if (isNumber(debounceDelay[trigger])) {
          debounceDelay = debounceDelay[trigger];
        } else if (isNumber(debounceDelay['default'])) {
          debounceDelay = debounceDelay['default'];
        }

        this.$$timeout.cancel(this.$$pendingDebounce);
        var that = this;
        if (debounceDelay > 0) {
          // this fails if debounceDelay is an object
          this.$$pendingDebounce = this.$$timeout(function () {
            that.$commitViewValue();
          }, debounceDelay);
        } else if (this.$$scope.$root.$$phase) {
          this.$commitViewValue();
        } else {
          this.$$scope.$apply(function () {
            that.$commitViewValue();
          });
        }
      },

      /**
       * @ngdoc method
       *
       * @name ngModel.NgModelController#$overrideModelOptions
       *
       * @description
       *
       * Override the current model options settings programmatically.
       *
       * The previous `ModelOptions` value will not be modified. Instead, a
       * new `ModelOptions` object will inherit from the previous one overriding
       * or inheriting settings that are defined in the given parameter.
       *
       * See {@link ngModelOptions} for information about what options can be specified
       * and how model option inheritance works.
       *
       * @param {Object} options a hash of settings to override the previous options
       *
       */
      $overrideModelOptions: function (options) {
        this.$options = this.$options.createChild(options);
      }
    };

    function setupModelWatcher(ctrl) {
      // model -> value
      // Note: we cannot use a normal scope.$watch as we want to detect the following:
      // 1. scope value is 'a'
      // 2. user enters 'b'
      // 3. ng-change kicks in and reverts scope value to 'a'
      //    -> scope value did not change since the last digest as
      //       ng-change executes in apply phase
      // 4. view should be changed back to 'a'
      ctrl.$$scope.$watch(function ngModelWatch() {
        var modelValue = ctrl.$$ngModelGet(ctrl.$$scope);

        // if scope model value and ngModel value are out of sync
        // TODO(perf): why not move this to the action fn?
        if (modelValue !== ctrl.$modelValue && (
        // checks for NaN is needed to allow setting the model to NaN when there's an asyncValidator
        // eslint-disable-next-line no-self-compare
        ctrl.$modelValue === ctrl.$modelValue || modelValue === modelValue)) {
          ctrl.$modelValue = ctrl.$$rawModelValue = modelValue;
          ctrl.$$parserValid = undefined;

          var formatters = ctrl.$formatters,
              idx = formatters.length;

          var viewValue = modelValue;
          while (idx--) {
            viewValue = formatters[idx](viewValue);
          }
          if (ctrl.$viewValue !== viewValue) {
            ctrl.$$updateEmptyClasses(viewValue);
            ctrl.$viewValue = ctrl.$$lastCommittedViewValue = viewValue;
            ctrl.$render();

            // It is possible that model and view value have been updated during render
            ctrl.$$runValidators(ctrl.$modelValue, ctrl.$viewValue, noop);
          }
        }

        return modelValue;
      });
    }

    /**
     * @ngdoc method
     * @name ngModel.NgModelController#$setValidity
     *
     * @description
     * Change the validity state, and notify the form.
     *
     * This method can be called within $parsers/$formatters or a custom validation implementation.
     * However, in most cases it should be sufficient to use the `ngModel.$validators` and
     * `ngModel.$asyncValidators` collections which will call `$setValidity` automatically.
     *
     * @param {string} validationErrorKey Name of the validator. The `validationErrorKey` will be assigned
     *        to either `$error[validationErrorKey]` or `$pending[validationErrorKey]`
     *        (for unfulfilled `$asyncValidators`), so that it is available for data-binding.
     *        The `validationErrorKey` should be in camelCase and will get converted into dash-case
     *        for class name. Example: `myError` will result in `ng-valid-my-error` and `ng-invalid-my-error`
     *        class and can be bound to as  `{{someForm.someControl.$error.myError}}` .
     * @param {boolean} isValid Whether the current state is valid (true), invalid (false), pending (undefined),
     *                          or skipped (null). Pending is used for unfulfilled `$asyncValidators`.
     *                          Skipped is used by Angular when validators do not run because of parse errors and
     *                          when `$asyncValidators` do not run because any of the `$validators` failed.
     */
    addSetValidityMethod({
      clazz: NgModelController,
      set: function (object, property) {
        object[property] = true;
      },
      unset: function (object, property) {
        delete object[property];
      }
    });

    /**
     * @ngdoc directive
     * @name ngModel
     *
     * @element input
     * @priority 1
     *
     * @description
     * The `ngModel` directive binds an `input`,`select`, `textarea` (or custom form control) to a
     * property on the scope using {@link ngModel.NgModelController NgModelController},
     * which is created and exposed by this directive.
     *
     * `ngModel` is responsible for:
     *
     * - Binding the view into the model, which other directives such as `input`, `textarea` or `select`
     *   require.
     * - Providing validation behavior (i.e. required, number, email, url).
     * - Keeping the state of the control (valid/invalid, dirty/pristine, touched/untouched, validation errors).
     * - Setting related css classes on the element (`ng-valid`, `ng-invalid`, `ng-dirty`, `ng-pristine`, `ng-touched`,
     *   `ng-untouched`, `ng-empty`, `ng-not-empty`) including animations.
     * - Registering the control with its parent {@link ng.directive:form form}.
     *
     * Note: `ngModel` will try to bind to the property given by evaluating the expression on the
     * current scope. If the property doesn't already exist on this scope, it will be created
     * implicitly and added to the scope.
     *
     * For best practices on using `ngModel`, see:
     *
     *  - [Understanding Scopes](https://github.com/angular/angular.js/wiki/Understanding-Scopes)
     *
     * For basic examples, how to use `ngModel`, see:
     *
     *  - {@link ng.directive:input input}
     *    - {@link input[text] text}
     *    - {@link input[checkbox] checkbox}
     *    - {@link input[radio] radio}
     *    - {@link input[number] number}
     *    - {@link input[email] email}
     *    - {@link input[url] url}
     *    - {@link input[date] date}
     *    - {@link input[datetime-local] datetime-local}
     *    - {@link input[time] time}
     *    - {@link input[month] month}
     *    - {@link input[week] week}
     *  - {@link ng.directive:select select}
     *  - {@link ng.directive:textarea textarea}
     *
     * # Complex Models (objects or collections)
     *
     * By default, `ngModel` watches the model by reference, not value. This is important to know when
     * binding inputs to models that are objects (e.g. `Date`) or collections (e.g. arrays). If only properties of the
     * object or collection change, `ngModel` will not be notified and so the input will not be  re-rendered.
     *
     * The model must be assigned an entirely new object or collection before a re-rendering will occur.
     *
     * Some directives have options that will cause them to use a custom `$watchCollection` on the model expression
     * - for example, `ngOptions` will do so when a `track by` clause is included in the comprehension expression or
     * if the select is given the `multiple` attribute.
     *
     * The `$watchCollection()` method only does a shallow comparison, meaning that changing properties deeper than the
     * first level of the object (or only changing the properties of an item in the collection if it's an array) will still
     * not trigger a re-rendering of the model.
     *
     * # CSS classes
     * The following CSS classes are added and removed on the associated input/select/textarea element
     * depending on the validity of the model.
     *
     *  - `ng-valid`: the model is valid
     *  - `ng-invalid`: the model is invalid
     *  - `ng-valid-[key]`: for each valid key added by `$setValidity`
     *  - `ng-invalid-[key]`: for each invalid key added by `$setValidity`
     *  - `ng-pristine`: the control hasn't been interacted with yet
     *  - `ng-dirty`: the control has been interacted with
     *  - `ng-touched`: the control has been blurred
     *  - `ng-untouched`: the control hasn't been blurred
     *  - `ng-pending`: any `$asyncValidators` are unfulfilled
     *  - `ng-empty`: the view does not contain a value or the value is deemed "empty", as defined
     *     by the {@link ngModel.NgModelController#$isEmpty} method
     *  - `ng-not-empty`: the view contains a non-empty value
     *
     * Keep in mind that ngAnimate can detect each of these classes when added and removed.
     *
     * ## Animation Hooks
     *
     * Animations within models are triggered when any of the associated CSS classes are added and removed
     * on the input element which is attached to the model. These classes include: `.ng-pristine`, `.ng-dirty`,
     * `.ng-invalid` and `.ng-valid` as well as any other validations that are performed on the model itself.
     * The animations that are triggered within ngModel are similar to how they work in ngClass and
     * animations can be hooked into using CSS transitions, keyframes as well as JS animations.
     *
     * The following example shows a simple way to utilize CSS transitions to style an input element
     * that has been rendered as invalid after it has been validated:
     *
     * <pre>
     * //be sure to include ngAnimate as a module to hook into more
     * //advanced animations
     * .my-input {
     *   transition:0.5s linear all;
     *   background: white;
     * }
     * .my-input.ng-invalid {
     *   background: red;
     *   color:white;
     * }
     * </pre>
     *
     * @example
     * <example deps="angular-animate.js" animations="true" fixBase="true" module="inputExample" name="ng-model">
         <file name="index.html">
           <script>
            angular.module('inputExample', [])
              .controller('ExampleController', ['$scope', function($scope) {
                $scope.val = '1';
              }]);
           </script>
           <style>
             .my-input {
               transition:all linear 0.5s;
               background: transparent;
             }
             .my-input.ng-invalid {
               color:white;
               background: red;
             }
           </style>
           <p id="inputDescription">
            Update input to see transitions when valid/invalid.
            Integer is a valid value.
           </p>
           <form name="testForm" ng-controller="ExampleController">
             <input ng-model="val" ng-pattern="/^\d+$/" name="anim" class="my-input"
                    aria-describedby="inputDescription" />
           </form>
         </file>
     * </example>
     *
     * ## Binding to a getter/setter
     *
     * Sometimes it's helpful to bind `ngModel` to a getter/setter function.  A getter/setter is a
     * function that returns a representation of the model when called with zero arguments, and sets
     * the internal state of a model when called with an argument. It's sometimes useful to use this
     * for models that have an internal representation that's different from what the model exposes
     * to the view.
     *
     * <div class="alert alert-success">
     * **Best Practice:** It's best to keep getters fast because Angular is likely to call them more
     * frequently than other parts of your code.
     * </div>
     *
     * You use this behavior by adding `ng-model-options="{ getterSetter: true }"` to an element that
     * has `ng-model` attached to it. You can also add `ng-model-options="{ getterSetter: true }"` to
     * a `<form>`, which will enable this behavior for all `<input>`s within it. See
     * {@link ng.directive:ngModelOptions `ngModelOptions`} for more.
     *
     * The following example shows how to use `ngModel` with a getter/setter:
     *
     * @example
     * <example name="ngModel-getter-setter" module="getterSetterExample">
         <file name="index.html">
           <div ng-controller="ExampleController">
             <form name="userForm">
               <label>Name:
                 <input type="text" name="userName"
                        ng-model="user.name"
                        ng-model-options="{ getterSetter: true }" />
               </label>
             </form>
             <pre>user.name = <span ng-bind="user.name()"></span></pre>
           </div>
         </file>
         <file name="app.js">
           angular.module('getterSetterExample', [])
             .controller('ExampleController', ['$scope', function($scope) {
               var _name = 'Brian';
               $scope.user = {
                 name: function(newName) {
                  // Note that newName can be undefined for two reasons:
                  // 1. Because it is called as a getter and thus called with no arguments
                  // 2. Because the property should actually be set to undefined. This happens e.g. if the
                  //    input is invalid
                  return arguments.length ? (_name = newName) : _name;
                 }
               };
             }]);
         </file>
     * </example>
     */
    var ngModelDirective = ['$rootScope', function ($rootScope) {
      return {
        restrict: 'A',
        require: ['ngModel', '^?form', '^?ngModelOptions'],
        controller: NgModelController,
        // Prelink needs to run before any input directive
        // so that we can set the NgModelOptions in NgModelController
        // before anyone else uses it.
        priority: 1,
        compile: function ngModelCompile(element) {
          // Setup initial state of the control
          element.addClass(PRISTINE_CLASS).addClass(UNTOUCHED_CLASS).addClass(VALID_CLASS);

          return {
            pre: function ngModelPreLink(scope, element, attr, ctrls) {
              var modelCtrl = ctrls[0],
                  formCtrl = ctrls[1] || modelCtrl.$$parentForm,
                  optionsCtrl = ctrls[2];

              if (optionsCtrl) {
                modelCtrl.$options = optionsCtrl.$options;
              }

              modelCtrl.$$initGetterSetters();

              // notify others, especially parent forms
              formCtrl.$addControl(modelCtrl);

              attr.$observe('name', function (newValue) {
                if (modelCtrl.$name !== newValue) {
                  modelCtrl.$$parentForm.$$renameControl(modelCtrl, newValue);
                }
              });

              scope.$on('$destroy', function () {
                modelCtrl.$$parentForm.$removeControl(modelCtrl);
              });
            },
            post: function ngModelPostLink(scope, element, attr, ctrls) {
              var modelCtrl = ctrls[0];
              if (modelCtrl.$options.getOption('updateOn')) {
                element.on(modelCtrl.$options.getOption('updateOn'), function (ev) {
                  modelCtrl.$$debounceViewValueCommit(ev && ev.type);
                });
              }

              function setTouched() {
                modelCtrl.$setTouched();
              }

              element.on('blur', function () {
                if (modelCtrl.$touched) return;

                if ($rootScope.$$phase) {
                  scope.$evalAsync(setTouched);
                } else {
                  scope.$apply(setTouched);
                }
              });
            }
          };
        }
      };
    }];

    /* exported defaultModelOptions */
    var defaultModelOptions;
    var DEFAULT_REGEXP = /(\s+|^)default(\s+|$)/;

    /**
     * @ngdoc type
     * @name ModelOptions
     * @description
     * A container for the options set by the {@link ngModelOptions} directive
     */
    function ModelOptions(options) {
      this.$$options = options;
    }

    ModelOptions.prototype = {

      /**
       * @ngdoc method
       * @name ModelOptions#getOption
       * @param {string} name the name of the option to retrieve
       * @returns {*} the value of the option
       * @description
       * Returns the value of the given option
       */
      getOption: function (name) {
        return this.$$options[name];
      },

      /**
       * @ngdoc method
       * @name ModelOptions#createChild
       * @param {Object} options a hash of options for the new child that will override the parent's options
       * @return {ModelOptions} a new `ModelOptions` object initialized with the given options.
       */
      createChild: function (options) {
        var inheritAll = false;

        // make a shallow copy
        options = extend({}, options);

        // Inherit options from the parent if specified by the value `"$inherit"`
        forEach(options, /* @this */function (option, key) {
          if (option === '$inherit') {
            if (key === '*') {
              inheritAll = true;
            } else {
              options[key] = this.$$options[key];
              // `updateOn` is special so we must also inherit the `updateOnDefault` option
              if (key === 'updateOn') {
                options.updateOnDefault = this.$$options.updateOnDefault;
              }
            }
          } else {
            if (key === 'updateOn') {
              // If the `updateOn` property contains the `default` event then we have to remove
              // it from the event list and set the `updateOnDefault` flag.
              options.updateOnDefault = false;
              options[key] = trim(option.replace(DEFAULT_REGEXP, function () {
                options.updateOnDefault = true;
                return ' ';
              }));
            }
          }
        }, this);

        if (inheritAll) {
          // We have a property of the form: `"*": "$inherit"`
          delete options['*'];
          defaults(options, this.$$options);
        }

        // Finally add in any missing defaults
        defaults(options, defaultModelOptions.$$options);

        return new ModelOptions(options);
      }
    };

    defaultModelOptions = new ModelOptions({
      updateOn: '',
      updateOnDefault: true,
      debounce: 0,
      getterSetter: false,
      allowInvalid: false,
      timezone: null
    });

    /**
     * @ngdoc directive
     * @name ngModelOptions
     *
     * @description
     * This directive allows you to modify the behaviour of {@link ngModel} directives within your
     * application. You can specify an `ngModelOptions` directive on any element. All {@link ngModel}
     * directives will use the options of their nearest `ngModelOptions` ancestor.
     *
     * The `ngModelOptions` settings are found by evaluating the value of the attribute directive as
     * an Angular expression. This expression should evaluate to an object, whose properties contain
     * the settings. For example: `<div "ng-model-options"="{ debounce: 100 }"`.
     *
     * ## Inheriting Options
     *
     * You can specify that an `ngModelOptions` setting should be inherited from a parent `ngModelOptions`
     * directive by giving it the value of `"$inherit"`.
     * Then it will inherit that setting from the first `ngModelOptions` directive found by traversing up the
     * DOM tree. If there is no ancestor element containing an `ngModelOptions` directive then default settings
     * will be used.
     *
     * For example given the following fragment of HTML
     *
     *
     * ```html
     * <div ng-model-options="{ allowInvalid: true, debounce: 200 }">
     *   <form ng-model-options="{ updateOn: 'blur', allowInvalid: '$inherit' }">
     *     <input ng-model-options="{ updateOn: 'default', allowInvalid: '$inherit' }" />
     *   </form>
     * </div>
     * ```
     *
     * the `input` element will have the following settings
     *
     * ```js
     * { allowInvalid: true, updateOn: 'default', debounce: 0 }
     * ```
     *
     * Notice that the `debounce` setting was not inherited and used the default value instead.
     *
     * You can specify that all undefined settings are automatically inherited from an ancestor by
     * including a property with key of `"*"` and value of `"$inherit"`.
     *
     * For example given the following fragment of HTML
     *
     *
     * ```html
     * <div ng-model-options="{ allowInvalid: true, debounce: 200 }">
     *   <form ng-model-options="{ updateOn: 'blur', "*": '$inherit' }">
     *     <input ng-model-options="{ updateOn: 'default', "*": '$inherit' }" />
     *   </form>
     * </div>
     * ```
     *
     * the `input` element will have the following settings
     *
     * ```js
     * { allowInvalid: true, updateOn: 'default', debounce: 200 }
     * ```
     *
     * Notice that the `debounce` setting now inherits the value from the outer `<div>` element.
     *
     * If you are creating a reusable component then you should be careful when using `"*": "$inherit"`
     * since you may inadvertently inherit a setting in the future that changes the behavior of your component.
     *
     *
     * ## Triggering and debouncing model updates
     *
     * The `updateOn` and `debounce` properties allow you to specify a custom list of events that will
     * trigger a model update and/or a debouncing delay so that the actual update only takes place when
     * a timer expires; this timer will be reset after another change takes place.
     *
     * Given the nature of `ngModelOptions`, the value displayed inside input fields in the view might
     * be different from the value in the actual model. This means that if you update the model you
     * should also invoke {@link ngModel.NgModelController#$rollbackViewValue} on the relevant input field in
     * order to make sure it is synchronized with the model and that any debounced action is canceled.
     *
     * The easiest way to reference the control's {@link ngModel.NgModelController#$rollbackViewValue}
     * method is by making sure the input is placed inside a form that has a `name` attribute. This is
     * important because `form` controllers are published to the related scope under the name in their
     * `name` attribute.
     *
     * Any pending changes will take place immediately when an enclosing form is submitted via the
     * `submit` event. Note that `ngClick` events will occur before the model is updated. Use `ngSubmit`
     * to have access to the updated model.
     *
     * The following example shows how to override immediate updates. Changes on the inputs within the
     * form will update the model only when the control loses focus (blur event). If `escape` key is
     * pressed while the input field is focused, the value is reset to the value in the current model.
     *
     * <example name="ngModelOptions-directive-blur" module="optionsExample">
     *   <file name="index.html">
     *     <div ng-controller="ExampleController">
     *       <form name="userForm">
     *         <label>
     *           Name:
     *           <input type="text" name="userName"
     *                  ng-model="user.name"
     *                  ng-model-options="{ updateOn: 'blur' }"
     *                  ng-keyup="cancel($event)" />
     *         </label><br />
     *         <label>
     *           Other data:
     *           <input type="text" ng-model="user.data" />
     *         </label><br />
     *       </form>
     *       <pre>user.name = <span ng-bind="user.name"></span></pre>
     *     </div>
     *   </file>
     *   <file name="app.js">
     *     angular.module('optionsExample', [])
     *       .controller('ExampleController', ['$scope', function($scope) {
     *         $scope.user = { name: 'say', data: '' };
     *
     *         $scope.cancel = function(e) {
     *           if (e.keyCode === 27) {
     *             $scope.userForm.userName.$rollbackViewValue();
     *           }
     *         };
     *       }]);
     *   </file>
     *   <file name="protractor.js" type="protractor">
     *     var model = element(by.binding('user.name'));
     *     var input = element(by.model('user.name'));
     *     var other = element(by.model('user.data'));
     *
     *     it('should allow custom events', function() {
     *       input.sendKeys(' hello');
     *       input.click();
     *       expect(model.getText()).toEqual('say');
     *       other.click();
     *       expect(model.getText()).toEqual('say hello');
     *     });
     *
     *     it('should $rollbackViewValue when model changes', function() {
     *       input.sendKeys(' hello');
     *       expect(input.getAttribute('value')).toEqual('say hello');
     *       input.sendKeys(protractor.Key.ESCAPE);
     *       expect(input.getAttribute('value')).toEqual('say');
     *       other.click();
     *       expect(model.getText()).toEqual('say');
     *     });
     *   </file>
     * </example>
     *
     * The next example shows how to debounce model changes. Model will be updated only 1 sec after last change.
     * If the `Clear` button is pressed, any debounced action is canceled and the value becomes empty.
     *
     * <example name="ngModelOptions-directive-debounce" module="optionsExample">
     *   <file name="index.html">
     *     <div ng-controller="ExampleController">
     *       <form name="userForm">
     *         Name:
     *         <input type="text" name="userName"
     *                ng-model="user.name"
     *                ng-model-options="{ debounce: 1000 }" />
     *         <button ng-click="userForm.userName.$rollbackViewValue(); user.name=''">Clear</button><br />
     *       </form>
     *       <pre>user.name = <span ng-bind="user.name"></span></pre>
     *     </div>
     *   </file>
     *   <file name="app.js">
     *     angular.module('optionsExample', [])
     *       .controller('ExampleController', ['$scope', function($scope) {
     *         $scope.user = { name: 'say' };
     *       }]);
     *   </file>
     * </example>
     *
     * ## Model updates and validation
     *
     * The default behaviour in `ngModel` is that the model value is set to `undefined` when the
     * validation determines that the value is invalid. By setting the `allowInvalid` property to true,
     * the model will still be updated even if the value is invalid.
     *
     *
     * ## Connecting to the scope
     *
     * By setting the `getterSetter` property to true you are telling ngModel that the `ngModel` expression
     * on the scope refers to a "getter/setter" function rather than the value itself.
     *
     * The following example shows how to bind to getter/setters:
     *
     * <example name="ngModelOptions-directive-getter-setter" module="getterSetterExample">
     *   <file name="index.html">
     *     <div ng-controller="ExampleController">
     *       <form name="userForm">
     *         <label>
     *           Name:
     *           <input type="text" name="userName"
     *                  ng-model="user.name"
     *                  ng-model-options="{ getterSetter: true }" />
     *         </label>
     *       </form>
     *       <pre>user.name = <span ng-bind="user.name()"></span></pre>
     *     </div>
     *   </file>
     *   <file name="app.js">
     *     angular.module('getterSetterExample', [])
     *       .controller('ExampleController', ['$scope', function($scope) {
     *         var _name = 'Brian';
     *         $scope.user = {
     *           name: function(newName) {
     *             return angular.isDefined(newName) ? (_name = newName) : _name;
     *           }
     *         };
     *       }]);
     *   </file>
     * </example>
     *
     *
     * ## Specifying timezones
     *
     * You can specify the timezone that date/time input directives expect by providing its name in the
     * `timezone` property.
     *
     * @param {Object} ngModelOptions options to apply to {@link ngModel} directives on this element and
     *   and its descendents. Valid keys are:
     *   - `updateOn`: string specifying which event should the input be bound to. You can set several
     *     events using an space delimited list. There is a special event called `default` that
     *     matches the default events belonging to the control.
     *   - `debounce`: integer value which contains the debounce model update value in milliseconds. A
     *     value of 0 triggers an immediate update. If an object is supplied instead, you can specify a
     *     custom value for each event. For example:
     *     ```
     *     ng-model-options="{
     *       updateOn: 'default blur',
     *       debounce: { 'default': 500, 'blur': 0 }
     *     }"
     *     ```
     *   - `allowInvalid`: boolean value which indicates that the model can be set with values that did
     *     not validate correctly instead of the default behavior of setting the model to undefined.
     *   - `getterSetter`: boolean value which determines whether or not to treat functions bound to
     *     `ngModel` as getters/setters.
     *   - `timezone`: Defines the timezone to be used to read/write the `Date` instance in the model for
     *     `<input type="date" />`, `<input type="time" />`, ... . It understands UTC/GMT and the
     *     continental US time zone abbreviations, but for general use, use a time zone offset, for
     *     example, `'+0430'` (4 hours, 30 minutes east of the Greenwich meridian)
     *     If not specified, the timezone of the browser will be used.
     *
     */
    var ngModelOptionsDirective = function () {
      NgModelOptionsController.$inject = ['$attrs', '$scope'];
      function NgModelOptionsController($attrs, $scope) {
        this.$$attrs = $attrs;
        this.$$scope = $scope;
      }
      NgModelOptionsController.prototype = {
        $onInit: function () {
          var parentOptions = this.parentCtrl ? this.parentCtrl.$options : defaultModelOptions;
          var modelOptionsDefinition = this.$$scope.$eval(this.$$attrs.ngModelOptions);

          this.$options = parentOptions.createChild(modelOptionsDefinition);
        }
      };

      return {
        restrict: 'A',
        // ngModelOptions needs to run before ngModel and input directives
        priority: 10,
        require: { parentCtrl: '?^^ngModelOptions' },
        bindToController: true,
        controller: NgModelOptionsController
      };
    };

    // shallow copy over values from `src` that are not already specified on `dst`
    function defaults(dst, src) {
      forEach(src, function (value, key) {
        if (!isDefined(dst[key])) {
          dst[key] = value;
        }
      });
    }

    /**
     * @ngdoc directive
     * @name ngNonBindable
     * @restrict AC
     * @priority 1000
     *
     * @description
     * The `ngNonBindable` directive tells Angular not to compile or bind the contents of the current
     * DOM element. This is useful if the element contains what appears to be Angular directives and
     * bindings but which should be ignored by Angular. This could be the case if you have a site that
     * displays snippets of code, for instance.
     *
     * @element ANY
     *
     * @example
     * In this example there are two locations where a simple interpolation binding (`{{}}`) is present,
     * but the one wrapped in `ngNonBindable` is left alone.
     *
     * @example
        <example name="ng-non-bindable">
          <file name="index.html">
            <div>Normal: {{1 + 2}}</div>
            <div ng-non-bindable>Ignored: {{1 + 2}}</div>
          </file>
          <file name="protractor.js" type="protractor">
           it('should check ng-non-bindable', function() {
             expect(element(by.binding('1 + 2')).getText()).toContain('3');
             expect(element.all(by.css('div')).last().getText()).toMatch(/1 \+ 2/);
           });
          </file>
        </example>
     */
    var ngNonBindableDirective = ngDirective({ terminal: true, priority: 1000 });

    /* exported ngOptionsDirective */

    /* global jqLiteRemove */

    var ngOptionsMinErr = minErr('ngOptions');

    /**
     * @ngdoc directive
     * @name ngOptions
     * @restrict A
     *
     * @description
     *
     * The `ngOptions` attribute can be used to dynamically generate a list of `<option>`
     * elements for the `<select>` element using the array or object obtained by evaluating the
     * `ngOptions` comprehension expression.
     *
     * In many cases, {@link ng.directive:ngRepeat ngRepeat} can be used on `<option>` elements instead of
     * `ngOptions` to achieve a similar result. However, `ngOptions` provides some benefits:
     * - more flexibility in how the `<select>`'s model is assigned via the `select` **`as`** part of the
     * comprehension expression
     * - reduced memory consumption by not creating a new scope for each repeated instance
     * - increased render speed by creating the options in a documentFragment instead of individually
     *
     * When an item in the `<select>` menu is selected, the array element or object property
     * represented by the selected option will be bound to the model identified by the `ngModel`
     * directive.
     *
     * Optionally, a single hard-coded `<option>` element, with the value set to an empty string, can
     * be nested into the `<select>` element. This element will then represent the `null` or "not selected"
     * option. See example below for demonstration.
     *
     * ## Complex Models (objects or collections)
     *
     * By default, `ngModel` watches the model by reference, not value. This is important to know when
     * binding the select to a model that is an object or a collection.
     *
     * One issue occurs if you want to preselect an option. For example, if you set
     * the model to an object that is equal to an object in your collection, `ngOptions` won't be able to set the selection,
     * because the objects are not identical. So by default, you should always reference the item in your collection
     * for preselections, e.g.: `$scope.selected = $scope.collection[3]`.
     *
     * Another solution is to use a `track by` clause, because then `ngOptions` will track the identity
     * of the item not by reference, but by the result of the `track by` expression. For example, if your
     * collection items have an id property, you would `track by item.id`.
     *
     * A different issue with objects or collections is that ngModel won't detect if an object property or
     * a collection item changes. For that reason, `ngOptions` additionally watches the model using
     * `$watchCollection`, when the expression contains a `track by` clause or the the select has the `multiple` attribute.
     * This allows ngOptions to trigger a re-rendering of the options even if the actual object/collection
     * has not changed identity, but only a property on the object or an item in the collection changes.
     *
     * Note that `$watchCollection` does a shallow comparison of the properties of the object (or the items in the collection
     * if the model is an array). This means that changing a property deeper than the first level inside the
     * object/collection will not trigger a re-rendering.
     *
     * ## `select` **`as`**
     *
     * Using `select` **`as`** will bind the result of the `select` expression to the model, but
     * the value of the `<select>` and `<option>` html elements will be either the index (for array data sources)
     * or property name (for object data sources) of the value within the collection. If a **`track by`** expression
     * is used, the result of that expression will be set as the value of the `option` and `select` elements.
     *
     *
     * ### `select` **`as`** and **`track by`**
     *
     * <div class="alert alert-warning">
     * Be careful when using `select` **`as`** and **`track by`** in the same expression.
     * </div>
     *
     * Given this array of items on the $scope:
     *
     * ```js
     * $scope.items = [{
     *   id: 1,
     *   label: 'aLabel',
     *   subItem: { name: 'aSubItem' }
     * }, {
     *   id: 2,
     *   label: 'bLabel',
     *   subItem: { name: 'bSubItem' }
     * }];
     * ```
     *
     * This will work:
     *
     * ```html
     * <select ng-options="item as item.label for item in items track by item.id" ng-model="selected"></select>
     * ```
     * ```js
     * $scope.selected = $scope.items[0];
     * ```
     *
     * but this will not work:
     *
     * ```html
     * <select ng-options="item.subItem as item.label for item in items track by item.id" ng-model="selected"></select>
     * ```
     * ```js
     * $scope.selected = $scope.items[0].subItem;
     * ```
     *
     * In both examples, the **`track by`** expression is applied successfully to each `item` in the
     * `items` array. Because the selected option has been set programmatically in the controller, the
     * **`track by`** expression is also applied to the `ngModel` value. In the first example, the
     * `ngModel` value is `items[0]` and the **`track by`** expression evaluates to `items[0].id` with
     * no issue. In the second example, the `ngModel` value is `items[0].subItem` and the **`track by`**
     * expression evaluates to `items[0].subItem.id` (which is undefined). As a result, the model value
     * is not matched against any `<option>` and the `<select>` appears as having no selected value.
     *
     *
     * @param {string} ngModel Assignable AngularJS expression to data-bind to.
     * @param {comprehension_expression} ngOptions in one of the following forms:
     *
     *   * for array data sources:
     *     * `label` **`for`** `value` **`in`** `array`
     *     * `select` **`as`** `label` **`for`** `value` **`in`** `array`
     *     * `label` **`group by`** `group` **`for`** `value` **`in`** `array`
     *     * `label` **`disable when`** `disable` **`for`** `value` **`in`** `array`
     *     * `label` **`group by`** `group` **`for`** `value` **`in`** `array` **`track by`** `trackexpr`
     *     * `label` **`disable when`** `disable` **`for`** `value` **`in`** `array` **`track by`** `trackexpr`
     *     * `label` **`for`** `value` **`in`** `array` | orderBy:`orderexpr` **`track by`** `trackexpr`
     *        (for including a filter with `track by`)
     *   * for object data sources:
     *     * `label` **`for (`**`key` **`,`** `value`**`) in`** `object`
     *     * `select` **`as`** `label` **`for (`**`key` **`,`** `value`**`) in`** `object`
     *     * `label` **`group by`** `group` **`for (`**`key`**`,`** `value`**`) in`** `object`
     *     * `label` **`disable when`** `disable` **`for (`**`key`**`,`** `value`**`) in`** `object`
     *     * `select` **`as`** `label` **`group by`** `group`
     *         **`for` `(`**`key`**`,`** `value`**`) in`** `object`
     *     * `select` **`as`** `label` **`disable when`** `disable`
     *         **`for` `(`**`key`**`,`** `value`**`) in`** `object`
     *
     * Where:
     *
     *   * `array` / `object`: an expression which evaluates to an array / object to iterate over.
     *   * `value`: local variable which will refer to each item in the `array` or each property value
     *      of `object` during iteration.
     *   * `key`: local variable which will refer to a property name in `object` during iteration.
     *   * `label`: The result of this expression will be the label for `<option>` element. The
     *     `expression` will most likely refer to the `value` variable (e.g. `value.propertyName`).
     *   * `select`: The result of this expression will be bound to the model of the parent `<select>`
     *      element. If not specified, `select` expression will default to `value`.
     *   * `group`: The result of this expression will be used to group options using the `<optgroup>`
     *      DOM element.
     *   * `disable`: The result of this expression will be used to disable the rendered `<option>`
     *      element. Return `true` to disable.
     *   * `trackexpr`: Used when working with an array of objects. The result of this expression will be
     *      used to identify the objects in the array. The `trackexpr` will most likely refer to the
     *     `value` variable (e.g. `value.propertyName`). With this the selection is preserved
     *      even when the options are recreated (e.g. reloaded from the server).
     * @param {string=} name Property name of the form under which the control is published.
     * @param {string=} required The control is considered valid only if value is entered.
     * @param {string=} ngRequired Adds `required` attribute and `required` validation constraint to
     *    the element when the ngRequired expression evaluates to true. Use `ngRequired` instead of
     *    `required` when you want to data-bind to the `required` attribute.
     * @param {string=} ngAttrSize sets the size of the select element dynamically. Uses the
     * {@link guide/interpolation#-ngattr-for-binding-to-arbitrary-attributes ngAttr} directive.
     *
     * @example
        <example module="selectExample" name="select">
          <file name="index.html">
            <script>
            angular.module('selectExample', [])
              .controller('ExampleController', ['$scope', function($scope) {
                $scope.colors = [
                  {name:'black', shade:'dark'},
                  {name:'white', shade:'light', notAnOption: true},
                  {name:'red', shade:'dark'},
                  {name:'blue', shade:'dark', notAnOption: true},
                  {name:'yellow', shade:'light', notAnOption: false}
                ];
                $scope.myColor = $scope.colors[2]; // red
              }]);
            </script>
            <div ng-controller="ExampleController">
              <ul>
                <li ng-repeat="color in colors">
                  <label>Name: <input ng-model="color.name"></label>
                  <label><input type="checkbox" ng-model="color.notAnOption"> Disabled?</label>
                  <button ng-click="colors.splice($index, 1)" aria-label="Remove">X</button>
                </li>
                <li>
                  <button ng-click="colors.push({})">add</button>
                </li>
              </ul>
              <hr/>
              <label>Color (null not allowed):
                <select ng-model="myColor" ng-options="color.name for color in colors"></select>
              </label><br/>
              <label>Color (null allowed):
              <span  class="nullable">
                <select ng-model="myColor" ng-options="color.name for color in colors">
                  <option value="">-- choose color --</option>
                </select>
              </span></label><br/>
    
              <label>Color grouped by shade:
                <select ng-model="myColor" ng-options="color.name group by color.shade for color in colors">
                </select>
              </label><br/>
    
              <label>Color grouped by shade, with some disabled:
                <select ng-model="myColor"
                      ng-options="color.name group by color.shade disable when color.notAnOption for color in colors">
                </select>
              </label><br/>
    
    
    
              Select <button ng-click="myColor = { name:'not in list', shade: 'other' }">bogus</button>.
              <br/>
              <hr/>
              Currently selected: {{ {selected_color:myColor} }}
              <div style="border:solid 1px black; height:20px"
                   ng-style="{'background-color':myColor.name}">
              </div>
            </div>
          </file>
          <file name="protractor.js" type="protractor">
             it('should check ng-options', function() {
               expect(element(by.binding('{selected_color:myColor}')).getText()).toMatch('red');
               element.all(by.model('myColor')).first().click();
               element.all(by.css('select[ng-model="myColor"] option')).first().click();
               expect(element(by.binding('{selected_color:myColor}')).getText()).toMatch('black');
               element(by.css('.nullable select[ng-model="myColor"]')).click();
               element.all(by.css('.nullable select[ng-model="myColor"] option')).first().click();
               expect(element(by.binding('{selected_color:myColor}')).getText()).toMatch('null');
             });
          </file>
        </example>
     */

    /* eslint-disable max-len */
    //                     //00001111111111000000000002222222222000000000000000000000333333333300000000000000000000000004444444444400000000000005555555555555000000000666666666666600000007777777777777000000000000000888888888800000000000000000009999999999
    var NG_OPTIONS_REGEXP = /^\s*([\s\S]+?)(?:\s+as\s+([\s\S]+?))?(?:\s+group\s+by\s+([\s\S]+?))?(?:\s+disable\s+when\s+([\s\S]+?))?\s+for\s+(?:([$\w][$\w]*)|(?:\(\s*([$\w][$\w]*)\s*,\s*([$\w][$\w]*)\s*\)))\s+in\s+([\s\S]+?)(?:\s+track\s+by\s+([\s\S]+?))?$/;
    // 1: value expression (valueFn)
    // 2: label expression (displayFn)
    // 3: group by expression (groupByFn)
    // 4: disable when expression (disableWhenFn)
    // 5: array item variable name
    // 6: object item key variable name
    // 7: object item value variable name
    // 8: collection expression
    // 9: track by expression
    /* eslint-enable */

    var ngOptionsDirective = ['$compile', '$document', '$parse', function ($compile, $document, $parse) {

      function parseOptionsExpression(optionsExp, selectElement, scope) {

        var match = optionsExp.match(NG_OPTIONS_REGEXP);
        if (!match) {
          throw ngOptionsMinErr('iexp', 'Expected expression in form of ' + '\'_select_ (as _label_)? for (_key_,)?_value_ in _collection_\'' + ' but got \'{0}\'. Element: {1}', optionsExp, startingTag(selectElement));
        }

        // Extract the parts from the ngOptions expression

        // The variable name for the value of the item in the collection
        var valueName = match[5] || match[7];
        // The variable name for the key of the item in the collection
        var keyName = match[6];

        // An expression that generates the viewValue for an option if there is a label expression
        var selectAs = / as /.test(match[0]) && match[1];
        // An expression that is used to track the id of each object in the options collection
        var trackBy = match[9];
        // An expression that generates the viewValue for an option if there is no label expression
        var valueFn = $parse(match[2] ? match[1] : valueName);
        var selectAsFn = selectAs && $parse(selectAs);
        var viewValueFn = selectAsFn || valueFn;
        var trackByFn = trackBy && $parse(trackBy);

        // Get the value by which we are going to track the option
        // if we have a trackFn then use that (passing scope and locals)
        // otherwise just hash the given viewValue
        var getTrackByValueFn = trackBy ? function (value, locals) {
          return trackByFn(scope, locals);
        } : function getHashOfValue(value) {
          return hashKey(value);
        };
        var getTrackByValue = function (value, key) {
          return getTrackByValueFn(value, getLocals(value, key));
        };

        var displayFn = $parse(match[2] || match[1]);
        var groupByFn = $parse(match[3] || '');
        var disableWhenFn = $parse(match[4] || '');
        var valuesFn = $parse(match[8]);

        var locals = {};
        var getLocals = keyName ? function (value, key) {
          locals[keyName] = key;
          locals[valueName] = value;
          return locals;
        } : function (value) {
          locals[valueName] = value;
          return locals;
        };

        function Option(selectValue, viewValue, label, group, disabled) {
          this.selectValue = selectValue;
          this.viewValue = viewValue;
          this.label = label;
          this.group = group;
          this.disabled = disabled;
        }

        function getOptionValuesKeys(optionValues) {
          var optionValuesKeys;

          if (!keyName && isArrayLike(optionValues)) {
            optionValuesKeys = optionValues;
          } else {
            // if object, extract keys, in enumeration order, unsorted
            optionValuesKeys = [];
            for (var itemKey in optionValues) {
              if (optionValues.hasOwnProperty(itemKey) && itemKey.charAt(0) !== '$') {
                optionValuesKeys.push(itemKey);
              }
            }
          }
          return optionValuesKeys;
        }

        return {
          trackBy: trackBy,
          getTrackByValue: getTrackByValue,
          getWatchables: $parse(valuesFn, function (optionValues) {
            // Create a collection of things that we would like to watch (watchedArray)
            // so that they can all be watched using a single $watchCollection
            // that only runs the handler once if anything changes
            var watchedArray = [];
            optionValues = optionValues || [];

            var optionValuesKeys = getOptionValuesKeys(optionValues);
            var optionValuesLength = optionValuesKeys.length;
            for (var index = 0; index < optionValuesLength; index++) {
              var key = optionValues === optionValuesKeys ? index : optionValuesKeys[index];
              var value = optionValues[key];

              var locals = getLocals(value, key);
              var selectValue = getTrackByValueFn(value, locals);
              watchedArray.push(selectValue);

              // Only need to watch the displayFn if there is a specific label expression
              if (match[2] || match[1]) {
                var label = displayFn(scope, locals);
                watchedArray.push(label);
              }

              // Only need to watch the disableWhenFn if there is a specific disable expression
              if (match[4]) {
                var disableWhen = disableWhenFn(scope, locals);
                watchedArray.push(disableWhen);
              }
            }
            return watchedArray;
          }),

          getOptions: function () {

            var optionItems = [];
            var selectValueMap = {};

            // The option values were already computed in the `getWatchables` fn,
            // which must have been called to trigger `getOptions`
            var optionValues = valuesFn(scope) || [];
            var optionValuesKeys = getOptionValuesKeys(optionValues);
            var optionValuesLength = optionValuesKeys.length;

            for (var index = 0; index < optionValuesLength; index++) {
              var key = optionValues === optionValuesKeys ? index : optionValuesKeys[index];
              var value = optionValues[key];
              var locals = getLocals(value, key);
              var viewValue = viewValueFn(scope, locals);
              var selectValue = getTrackByValueFn(viewValue, locals);
              var label = displayFn(scope, locals);
              var group = groupByFn(scope, locals);
              var disabled = disableWhenFn(scope, locals);
              var optionItem = new Option(selectValue, viewValue, label, group, disabled);

              optionItems.push(optionItem);
              selectValueMap[selectValue] = optionItem;
            }

            return {
              items: optionItems,
              selectValueMap: selectValueMap,
              getOptionFromViewValue: function (value) {
                return selectValueMap[getTrackByValue(value)];
              },
              getViewValueFromOption: function (option) {
                // If the viewValue could be an object that may be mutated by the application,
                // we need to make a copy and not return the reference to the value on the option.
                return trackBy ? copy(option.viewValue) : option.viewValue;
              }
            };
          }
        };
      }

      // we can't just jqLite('<option>') since jqLite is not smart enough
      // to create it in <select> and IE barfs otherwise.
      var optionTemplate = window.document.createElement('option'),
          optGroupTemplate = window.document.createElement('optgroup');

      function ngOptionsPostLink(scope, selectElement, attr, ctrls) {

        var selectCtrl = ctrls[0];
        var ngModelCtrl = ctrls[1];
        var multiple = attr.multiple;

        // The emptyOption allows the application developer to provide their own custom "empty"
        // option when the viewValue does not match any of the option values.
        for (var i = 0, children = selectElement.children(), ii = children.length; i < ii; i++) {
          if (children[i].value === '') {
            selectCtrl.hasEmptyOption = true;
            selectCtrl.emptyOption = children.eq(i);
            break;
          }
        }

        var providedEmptyOption = !!selectCtrl.emptyOption;

        var unknownOption = jqLite(optionTemplate.cloneNode(false));
        unknownOption.val('?');

        var options;
        var ngOptions = parseOptionsExpression(attr.ngOptions, selectElement, scope);
        // This stores the newly created options before they are appended to the select.
        // Since the contents are removed from the fragment when it is appended,
        // we only need to create it once.
        var listFragment = $document[0].createDocumentFragment();

        // Overwrite the implementation. ngOptions doesn't use hashes
        selectCtrl.generateUnknownOptionValue = function (val) {
          return '?';
        };

        // Update the controller methods for multiple selectable options
        if (!multiple) {

          selectCtrl.writeValue = function writeNgOptionsValue(value) {
            var selectedOption = options.selectValueMap[selectElement.val()];
            var option = options.getOptionFromViewValue(value);

            // Make sure to remove the selected attribute from the previously selected option
            // Otherwise, screen readers might get confused
            if (selectedOption) selectedOption.element.removeAttribute('selected');

            if (option) {
              // Don't update the option when it is already selected.
              // For example, the browser will select the first option by default. In that case,
              // most properties are set automatically - except the `selected` attribute, which we
              // set always

              if (selectElement[0].value !== option.selectValue) {
                selectCtrl.removeUnknownOption();
                selectCtrl.unselectEmptyOption();

                selectElement[0].value = option.selectValue;
                option.element.selected = true;
              }

              option.element.setAttribute('selected', 'selected');
            } else {

              if (providedEmptyOption) {
                selectCtrl.selectEmptyOption();
              } else if (selectCtrl.unknownOption.parent().length) {
                selectCtrl.updateUnknownOption(value);
              } else {
                selectCtrl.renderUnknownOption(value);
              }
            }
          };

          selectCtrl.readValue = function readNgOptionsValue() {

            var selectedOption = options.selectValueMap[selectElement.val()];

            if (selectedOption && !selectedOption.disabled) {
              selectCtrl.unselectEmptyOption();
              selectCtrl.removeUnknownOption();
              return options.getViewValueFromOption(selectedOption);
            }
            return null;
          };

          // If we are using `track by` then we must watch the tracked value on the model
          // since ngModel only watches for object identity change
          // FIXME: When a user selects an option, this watch will fire needlessly
          if (ngOptions.trackBy) {
            scope.$watch(function () {
              return ngOptions.getTrackByValue(ngModelCtrl.$viewValue);
            }, function () {
              ngModelCtrl.$render();
            });
          }
        } else {

          selectCtrl.writeValue = function writeNgOptionsMultiple(values) {
            // Only set `<option>.selected` if necessary, in order to prevent some browsers from
            // scrolling to `<option>` elements that are outside the `<select>` element's viewport.

            var selectedOptions = values && values.map(getAndUpdateSelectedOption) || [];

            options.items.forEach(function (option) {
              if (option.element.selected && !includes(selectedOptions, option)) {
                option.element.selected = false;
              }
            });
          };

          selectCtrl.readValue = function readNgOptionsMultiple() {
            var selectedValues = selectElement.val() || [],
                selections = [];

            forEach(selectedValues, function (value) {
              var option = options.selectValueMap[value];
              if (option && !option.disabled) selections.push(options.getViewValueFromOption(option));
            });

            return selections;
          };

          // If we are using `track by` then we must watch these tracked values on the model
          // since ngModel only watches for object identity change
          if (ngOptions.trackBy) {

            scope.$watchCollection(function () {
              if (isArray(ngModelCtrl.$viewValue)) {
                return ngModelCtrl.$viewValue.map(function (value) {
                  return ngOptions.getTrackByValue(value);
                });
              }
            }, function () {
              ngModelCtrl.$render();
            });
          }
        }

        if (providedEmptyOption) {

          // we need to remove it before calling selectElement.empty() because otherwise IE will
          // remove the label from the element. wtf?
          selectCtrl.emptyOption.remove();

          // compile the element since there might be bindings in it
          $compile(selectCtrl.emptyOption)(scope);

          if (selectCtrl.emptyOption[0].nodeType === NODE_TYPE_COMMENT) {
            // This means the empty option has currently no actual DOM node, probably because
            // it has been modified by a transclusion directive.
            selectCtrl.hasEmptyOption = false;

            // Redefine the registerOption function, which will catch
            // options that are added by ngIf etc. (rendering of the node is async because of
            // lazy transclusion)
            selectCtrl.registerOption = function (optionScope, optionEl) {
              if (optionEl.val() === '') {
                selectCtrl.hasEmptyOption = true;
                selectCtrl.emptyOption = optionEl;
                selectCtrl.emptyOption.removeClass('ng-scope');
                // This ensures the new empty option is selected if previously no option was selected
                ngModelCtrl.$render();

                optionEl.on('$destroy', function () {
                  selectCtrl.hasEmptyOption = false;
                  selectCtrl.emptyOption = undefined;
                });
              }
            };
          } else {
            // remove the class, which is added automatically because we recompile the element and it
            // becomes the compilation root
            selectCtrl.emptyOption.removeClass('ng-scope');
          }
        }

        selectElement.empty();

        // We need to do this here to ensure that the options object is defined
        // when we first hit it in writeNgOptionsValue
        updateOptions();

        // We will re-render the option elements if the option values or labels change
        scope.$watchCollection(ngOptions.getWatchables, updateOptions);

        // ------------------------------------------------------------------ //

        function addOptionElement(option, parent) {
          var optionElement = optionTemplate.cloneNode(false);
          parent.appendChild(optionElement);
          updateOptionElement(option, optionElement);
        }

        function getAndUpdateSelectedOption(viewValue) {
          var option = options.getOptionFromViewValue(viewValue);
          var element = option && option.element;

          if (element && !element.selected) element.selected = true;

          return option;
        }

        function updateOptionElement(option, element) {
          option.element = element;
          element.disabled = option.disabled;
          // NOTE: The label must be set before the value, otherwise IE10/11/EDGE create unresponsive
          // selects in certain circumstances when multiple selects are next to each other and display
          // the option list in listbox style, i.e. the select is [multiple], or specifies a [size].
          // See https://github.com/angular/angular.js/issues/11314 for more info.
          // This is unfortunately untestable with unit / e2e tests
          if (option.label !== element.label) {
            element.label = option.label;
            element.textContent = option.label;
          }
          element.value = option.selectValue;
        }

        function updateOptions() {
          var previousValue = options && selectCtrl.readValue();

          // We must remove all current options, but cannot simply set innerHTML = null
          // since the providedEmptyOption might have an ngIf on it that inserts comments which we
          // must preserve.
          // Instead, iterate over the current option elements and remove them or their optgroup
          // parents
          if (options) {

            for (var i = options.items.length - 1; i >= 0; i--) {
              var option = options.items[i];
              if (isDefined(option.group)) {
                jqLiteRemove(option.element.parentNode);
              } else {
                jqLiteRemove(option.element);
              }
            }
          }

          options = ngOptions.getOptions();

          var groupElementMap = {};

          // Ensure that the empty option is always there if it was explicitly provided
          if (providedEmptyOption) {
            selectElement.prepend(selectCtrl.emptyOption);
          }

          options.items.forEach(function addOption(option) {
            var groupElement;

            if (isDefined(option.group)) {

              // This option is to live in a group
              // See if we have already created this group
              groupElement = groupElementMap[option.group];

              if (!groupElement) {

                groupElement = optGroupTemplate.cloneNode(false);
                listFragment.appendChild(groupElement);

                // Update the label on the group element
                // "null" is special cased because of Safari
                groupElement.label = option.group === null ? 'null' : option.group;

                // Store it for use later
                groupElementMap[option.group] = groupElement;
              }

              addOptionElement(option, groupElement);
            } else {

              // This option is not in a group
              addOptionElement(option, listFragment);
            }
          });

          selectElement[0].appendChild(listFragment);

          ngModelCtrl.$render();

          // Check to see if the value has changed due to the update to the options
          if (!ngModelCtrl.$isEmpty(previousValue)) {
            var nextValue = selectCtrl.readValue();
            var isNotPrimitive = ngOptions.trackBy || multiple;
            if (isNotPrimitive ? !equals(previousValue, nextValue) : previousValue !== nextValue) {
              ngModelCtrl.$setViewValue(nextValue);
              ngModelCtrl.$render();
            }
          }
        }
      }

      return {
        restrict: 'A',
        terminal: true,
        require: ['select', 'ngModel'],
        link: {
          pre: function ngOptionsPreLink(scope, selectElement, attr, ctrls) {
            // Deactivate the SelectController.register method to prevent
            // option directives from accidentally registering themselves
            // (and unwanted $destroy handlers etc.)
            ctrls[0].registerOption = noop;
          },
          post: ngOptionsPostLink
        }
      };
    }];

    /**
     * @ngdoc directive
     * @name ngPluralize
     * @restrict EA
     *
     * @description
     * `ngPluralize` is a directive that displays messages according to en-US localization rules.
     * These rules are bundled with angular.js, but can be overridden
     * (see {@link guide/i18n Angular i18n} dev guide). You configure ngPluralize directive
     * by specifying the mappings between
     * [plural categories](http://unicode.org/repos/cldr-tmp/trunk/diff/supplemental/language_plural_rules.html)
     * and the strings to be displayed.
     *
     * # Plural categories and explicit number rules
     * There are two
     * [plural categories](http://unicode.org/repos/cldr-tmp/trunk/diff/supplemental/language_plural_rules.html)
     * in Angular's default en-US locale: "one" and "other".
     *
     * While a plural category may match many numbers (for example, in en-US locale, "other" can match
     * any number that is not 1), an explicit number rule can only match one number. For example, the
     * explicit number rule for "3" matches the number 3. There are examples of plural categories
     * and explicit number rules throughout the rest of this documentation.
     *
     * # Configuring ngPluralize
     * You configure ngPluralize by providing 2 attributes: `count` and `when`.
     * You can also provide an optional attribute, `offset`.
     *
     * The value of the `count` attribute can be either a string or an {@link guide/expression
     * Angular expression}; these are evaluated on the current scope for its bound value.
     *
     * The `when` attribute specifies the mappings between plural categories and the actual
     * string to be displayed. The value of the attribute should be a JSON object.
     *
     * The following example shows how to configure ngPluralize:
     *
     * ```html
     * <ng-pluralize count="personCount"
                     when="{'0': 'Nobody is viewing.',
     *                      'one': '1 person is viewing.',
     *                      'other': '{} people are viewing.'}">
     * </ng-pluralize>
     *```
     *
     * In the example, `"0: Nobody is viewing."` is an explicit number rule. If you did not
     * specify this rule, 0 would be matched to the "other" category and "0 people are viewing"
     * would be shown instead of "Nobody is viewing". You can specify an explicit number rule for
     * other numbers, for example 12, so that instead of showing "12 people are viewing", you can
     * show "a dozen people are viewing".
     *
     * You can use a set of closed braces (`{}`) as a placeholder for the number that you want substituted
     * into pluralized strings. In the previous example, Angular will replace `{}` with
     * <span ng-non-bindable>`{{personCount}}`</span>. The closed braces `{}` is a placeholder
     * for <span ng-non-bindable>{{numberExpression}}</span>.
     *
     * If no rule is defined for a category, then an empty string is displayed and a warning is generated.
     * Note that some locales define more categories than `one` and `other`. For example, fr-fr defines `few` and `many`.
     *
     * # Configuring ngPluralize with offset
     * The `offset` attribute allows further customization of pluralized text, which can result in
     * a better user experience. For example, instead of the message "4 people are viewing this document",
     * you might display "John, Kate and 2 others are viewing this document".
     * The offset attribute allows you to offset a number by any desired value.
     * Let's take a look at an example:
     *
     * ```html
     * <ng-pluralize count="personCount" offset=2
     *               when="{'0': 'Nobody is viewing.',
     *                      '1': '{{person1}} is viewing.',
     *                      '2': '{{person1}} and {{person2}} are viewing.',
     *                      'one': '{{person1}}, {{person2}} and one other person are viewing.',
     *                      'other': '{{person1}}, {{person2}} and {} other people are viewing.'}">
     * </ng-pluralize>
     * ```
     *
     * Notice that we are still using two plural categories(one, other), but we added
     * three explicit number rules 0, 1 and 2.
     * When one person, perhaps John, views the document, "John is viewing" will be shown.
     * When three people view the document, no explicit number rule is found, so
     * an offset of 2 is taken off 3, and Angular uses 1 to decide the plural category.
     * In this case, plural category 'one' is matched and "John, Mary and one other person are viewing"
     * is shown.
     *
     * Note that when you specify offsets, you must provide explicit number rules for
     * numbers from 0 up to and including the offset. If you use an offset of 3, for example,
     * you must provide explicit number rules for 0, 1, 2 and 3. You must also provide plural strings for
     * plural categories "one" and "other".
     *
     * @param {string|expression} count The variable to be bound to.
     * @param {string} when The mapping between plural category to its corresponding strings.
     * @param {number=} offset Offset to deduct from the total number.
     *
     * @example
        <example module="pluralizeExample" name="ng-pluralize">
          <file name="index.html">
            <script>
              angular.module('pluralizeExample', [])
                .controller('ExampleController', ['$scope', function($scope) {
                  $scope.person1 = 'Igor';
                  $scope.person2 = 'Misko';
                  $scope.personCount = 1;
                }]);
            </script>
            <div ng-controller="ExampleController">
              <label>Person 1:<input type="text" ng-model="person1" value="Igor" /></label><br/>
              <label>Person 2:<input type="text" ng-model="person2" value="Misko" /></label><br/>
              <label>Number of People:<input type="text" ng-model="personCount" value="1" /></label><br/>
    
              <!--- Example with simple pluralization rules for en locale --->
              Without Offset:
              <ng-pluralize count="personCount"
                            when="{'0': 'Nobody is viewing.',
                                   'one': '1 person is viewing.',
                                   'other': '{} people are viewing.'}">
              </ng-pluralize><br>
    
              <!--- Example with offset --->
              With Offset(2):
              <ng-pluralize count="personCount" offset=2
                            when="{'0': 'Nobody is viewing.',
                                   '1': '{{person1}} is viewing.',
                                   '2': '{{person1}} and {{person2}} are viewing.',
                                   'one': '{{person1}}, {{person2}} and one other person are viewing.',
                                   'other': '{{person1}}, {{person2}} and {} other people are viewing.'}">
              </ng-pluralize>
            </div>
          </file>
          <file name="protractor.js" type="protractor">
            it('should show correct pluralized string', function() {
              var withoutOffset = element.all(by.css('ng-pluralize')).get(0);
              var withOffset = element.all(by.css('ng-pluralize')).get(1);
              var countInput = element(by.model('personCount'));
    
              expect(withoutOffset.getText()).toEqual('1 person is viewing.');
              expect(withOffset.getText()).toEqual('Igor is viewing.');
    
              countInput.clear();
              countInput.sendKeys('0');
    
              expect(withoutOffset.getText()).toEqual('Nobody is viewing.');
              expect(withOffset.getText()).toEqual('Nobody is viewing.');
    
              countInput.clear();
              countInput.sendKeys('2');
    
              expect(withoutOffset.getText()).toEqual('2 people are viewing.');
              expect(withOffset.getText()).toEqual('Igor and Misko are viewing.');
    
              countInput.clear();
              countInput.sendKeys('3');
    
              expect(withoutOffset.getText()).toEqual('3 people are viewing.');
              expect(withOffset.getText()).toEqual('Igor, Misko and one other person are viewing.');
    
              countInput.clear();
              countInput.sendKeys('4');
    
              expect(withoutOffset.getText()).toEqual('4 people are viewing.');
              expect(withOffset.getText()).toEqual('Igor, Misko and 2 other people are viewing.');
            });
            it('should show data-bound names', function() {
              var withOffset = element.all(by.css('ng-pluralize')).get(1);
              var personCount = element(by.model('personCount'));
              var person1 = element(by.model('person1'));
              var person2 = element(by.model('person2'));
              personCount.clear();
              personCount.sendKeys('4');
              person1.clear();
              person1.sendKeys('Di');
              person2.clear();
              person2.sendKeys('Vojta');
              expect(withOffset.getText()).toEqual('Di, Vojta and 2 other people are viewing.');
            });
          </file>
        </example>
     */
    var ngPluralizeDirective = ['$locale', '$interpolate', '$log', function ($locale, $interpolate, $log) {
      var BRACE = /{}/g,
          IS_WHEN = /^when(Minus)?(.+)$/;

      return {
        link: function (scope, element, attr) {
          var numberExp = attr.count,
              whenExp = attr.$attr.when && element.attr(attr.$attr.when),
              // we have {{}} in attrs
          offset = attr.offset || 0,
              whens = scope.$eval(whenExp) || {},
              whensExpFns = {},
              startSymbol = $interpolate.startSymbol(),
              endSymbol = $interpolate.endSymbol(),
              braceReplacement = startSymbol + numberExp + '-' + offset + endSymbol,
              watchRemover = angular.noop,
              lastCount;

          forEach(attr, function (expression, attributeName) {
            var tmpMatch = IS_WHEN.exec(attributeName);
            if (tmpMatch) {
              var whenKey = (tmpMatch[1] ? '-' : '') + lowercase(tmpMatch[2]);
              whens[whenKey] = element.attr(attr.$attr[attributeName]);
            }
          });
          forEach(whens, function (expression, key) {
            whensExpFns[key] = $interpolate(expression.replace(BRACE, braceReplacement));
          });

          scope.$watch(numberExp, function ngPluralizeWatchAction(newVal) {
            var count = parseFloat(newVal);
            var countIsNaN = isNumberNaN(count);

            if (!countIsNaN && !(count in whens)) {
              // If an explicit number rule such as 1, 2, 3... is defined, just use it.
              // Otherwise, check it against pluralization rules in $locale service.
              count = $locale.pluralCat(count - offset);
            }

            // If both `count` and `lastCount` are NaN, we don't need to re-register a watch.
            // In JS `NaN !== NaN`, so we have to explicitly check.
            if (count !== lastCount && !(countIsNaN && isNumberNaN(lastCount))) {
              watchRemover();
              var whenExpFn = whensExpFns[count];
              if (isUndefined(whenExpFn)) {
                if (newVal != null) {
                  $log.debug('ngPluralize: no rule defined for \'' + count + '\' in ' + whenExp);
                }
                watchRemover = noop;
                updateElementText();
              } else {
                watchRemover = scope.$watch(whenExpFn, updateElementText);
              }
              lastCount = count;
            }
          });

          function updateElementText(newText) {
            element.text(newText || '');
          }
        }
      };
    }];

    /* exported ngRepeatDirective */

    /**
     * @ngdoc directive
     * @name ngRepeat
     * @multiElement
     * @restrict A
     *
     * @description
     * The `ngRepeat` directive instantiates a template once per item from a collection. Each template
     * instance gets its own scope, where the given loop variable is set to the current collection item,
     * and `$index` is set to the item index or key.
     *
     * Special properties are exposed on the local scope of each template instance, including:
     *
     * | Variable  | Type            | Details                                                                     |
     * |-----------|-----------------|-----------------------------------------------------------------------------|
     * | `$index`  | {@type number}  | iterator offset of the repeated element (0..length-1)                       |
     * | `$first`  | {@type boolean} | true if the repeated element is first in the iterator.                      |
     * | `$middle` | {@type boolean} | true if the repeated element is between the first and last in the iterator. |
     * | `$last`   | {@type boolean} | true if the repeated element is last in the iterator.                       |
     * | `$even`   | {@type boolean} | true if the iterator position `$index` is even (otherwise false).           |
     * | `$odd`    | {@type boolean} | true if the iterator position `$index` is odd (otherwise false).            |
     *
     * <div class="alert alert-info">
     *   Creating aliases for these properties is possible with {@link ng.directive:ngInit `ngInit`}.
     *   This may be useful when, for instance, nesting ngRepeats.
     * </div>
     *
     *
     * # Iterating over object properties
     *
     * It is possible to get `ngRepeat` to iterate over the properties of an object using the following
     * syntax:
     *
     * ```js
     * <div ng-repeat="(key, value) in myObj"> ... </div>
     * ```
     *
     * However, there are a few limitations compared to array iteration:
     *
     * - The JavaScript specification does not define the order of keys
     *   returned for an object, so Angular relies on the order returned by the browser
     *   when running `for key in myObj`. Browsers generally follow the strategy of providing
     *   keys in the order in which they were defined, although there are exceptions when keys are deleted
     *   and reinstated. See the
     *   [MDN page on `delete` for more info](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/delete#Cross-browser_notes).
     *
     * - `ngRepeat` will silently *ignore* object keys starting with `$`, because
     *   it's a prefix used by Angular for public (`$`) and private (`$$`) properties.
     *
     * - The built-in filters {@link ng.orderBy orderBy} and {@link ng.filter filter} do not work with
     *   objects, and will throw an error if used with one.
     *
     * If you are hitting any of these limitations, the recommended workaround is to convert your object into an array
     * that is sorted into the order that you prefer before providing it to `ngRepeat`. You could
     * do this with a filter such as [toArrayFilter](http://ngmodules.org/modules/angular-toArrayFilter)
     * or implement a `$watch` on the object yourself.
     *
     *
     * # Tracking and Duplicates
     *
     * `ngRepeat` uses {@link $rootScope.Scope#$watchCollection $watchCollection} to detect changes in
     * the collection. When a change happens, `ngRepeat` then makes the corresponding changes to the DOM:
     *
     * * When an item is added, a new instance of the template is added to the DOM.
     * * When an item is removed, its template instance is removed from the DOM.
     * * When items are reordered, their respective templates are reordered in the DOM.
     *
     * To minimize creation of DOM elements, `ngRepeat` uses a function
     * to "keep track" of all items in the collection and their corresponding DOM elements.
     * For example, if an item is added to the collection, `ngRepeat` will know that all other items
     * already have DOM elements, and will not re-render them.
     *
     * The default tracking function (which tracks items by their identity) does not allow
     * duplicate items in arrays. This is because when there are duplicates, it is not possible
     * to maintain a one-to-one mapping between collection items and DOM elements.
     *
     * If you do need to repeat duplicate items, you can substitute the default tracking behavior
     * with your own using the `track by` expression.
     *
     * For example, you may track items by the index of each item in the collection, using the
     * special scope property `$index`:
     * ```html
     *    <div ng-repeat="n in [42, 42, 43, 43] track by $index">
     *      {{n}}
     *    </div>
     * ```
     *
     * You may also use arbitrary expressions in `track by`, including references to custom functions
     * on the scope:
     * ```html
     *    <div ng-repeat="n in [42, 42, 43, 43] track by myTrackingFunction(n)">
     *      {{n}}
     *    </div>
     * ```
     *
     * <div class="alert alert-success">
     * If you are working with objects that have a unique identifier property, you should track
     * by this identifier instead of the object instance. Should you reload your data later, `ngRepeat`
     * will not have to rebuild the DOM elements for items it has already rendered, even if the
     * JavaScript objects in the collection have been substituted for new ones. For large collections,
     * this significantly improves rendering performance. If you don't have a unique identifier,
     * `track by $index` can also provide a performance boost.
     * </div>
     *
     * ```html
     *    <div ng-repeat="model in collection track by model.id">
     *      {{model.name}}
     *    </div>
     * ```
     *
     * <br />
     * <div class="alert alert-warning">
     * Avoid using `track by $index` when the repeated template contains
     * {@link guide/expression#one-time-binding one-time bindings}. In such cases, the `nth` DOM
     * element will always be matched with the `nth` item of the array, so the bindings on that element
     * will not be updated even when the corresponding item changes, essentially causing the view to get
     * out-of-sync with the underlying data.
     * </div>
     *
     * When no `track by` expression is provided, it is equivalent to tracking by the built-in
     * `$id` function, which tracks items by their identity:
     * ```html
     *    <div ng-repeat="obj in collection track by $id(obj)">
     *      {{obj.prop}}
     *    </div>
     * ```
     *
     * <br />
     * <div class="alert alert-warning">
     * **Note:** `track by` must always be the last expression:
     * </div>
     * ```
     *    <div ng-repeat="model in collection | orderBy: 'id' as filtered_result track by model.id">
     *      {{model.name}}
     *    </div>
     * ```
     *
     *
     * # Special repeat start and end points
     * To repeat a series of elements instead of just one parent element, ngRepeat (as well as other ng directives) supports extending
     * the range of the repeater by defining explicit start and end points by using **ng-repeat-start** and **ng-repeat-end** respectively.
     * The **ng-repeat-start** directive works the same as **ng-repeat**, but will repeat all the HTML code (including the tag it's defined on)
     * up to and including the ending HTML tag where **ng-repeat-end** is placed.
     *
     * The example below makes use of this feature:
     * ```html
     *   <header ng-repeat-start="item in items">
     *     Header {{ item }}
     *   </header>
     *   <div class="body">
     *     Body {{ item }}
     *   </div>
     *   <footer ng-repeat-end>
     *     Footer {{ item }}
     *   </footer>
     * ```
     *
     * And with an input of {@type ['A','B']} for the items variable in the example above, the output will evaluate to:
     * ```html
     *   <header>
     *     Header A
     *   </header>
     *   <div class="body">
     *     Body A
     *   </div>
     *   <footer>
     *     Footer A
     *   </footer>
     *   <header>
     *     Header B
     *   </header>
     *   <div class="body">
     *     Body B
     *   </div>
     *   <footer>
     *     Footer B
     *   </footer>
     * ```
     *
     * The custom start and end points for ngRepeat also support all other HTML directive syntax flavors provided in AngularJS (such
     * as **data-ng-repeat-start**, **x-ng-repeat-start** and **ng:repeat-start**).
     *
     * @animations
     * | Animation                        | Occurs                              |
     * |----------------------------------|-------------------------------------|
     * | {@link ng.$animate#enter enter} | when a new item is added to the list or when an item is revealed after a filter |
     * | {@link ng.$animate#leave leave} | when an item is removed from the list or when an item is filtered out |
     * | {@link ng.$animate#move move } | when an adjacent item is filtered out causing a reorder or when the item contents are reordered |
     *
     * See the example below for defining CSS animations with ngRepeat.
     *
     * @element ANY
     * @scope
     * @priority 1000
     * @param {repeat_expression} ngRepeat The expression indicating how to enumerate a collection. These
     *   formats are currently supported:
     *
     *   * `variable in expression` – where variable is the user defined loop variable and `expression`
     *     is a scope expression giving the collection to enumerate.
     *
     *     For example: `album in artist.albums`.
     *
     *   * `(key, value) in expression` – where `key` and `value` can be any user defined identifiers,
     *     and `expression` is the scope expression giving the collection to enumerate.
     *
     *     For example: `(name, age) in {'adam':10, 'amalie':12}`.
     *
     *   * `variable in expression track by tracking_expression` – You can also provide an optional tracking expression
     *     which can be used to associate the objects in the collection with the DOM elements. If no tracking expression
     *     is specified, ng-repeat associates elements by identity. It is an error to have
     *     more than one tracking expression value resolve to the same key. (This would mean that two distinct objects are
     *     mapped to the same DOM element, which is not possible.)
     *
     *     Note that the tracking expression must come last, after any filters, and the alias expression.
     *
     *     For example: `item in items` is equivalent to `item in items track by $id(item)`. This implies that the DOM elements
     *     will be associated by item identity in the array.
     *
     *     For example: `item in items track by $id(item)`. A built in `$id()` function can be used to assign a unique
     *     `$$hashKey` property to each item in the array. This property is then used as a key to associated DOM elements
     *     with the corresponding item in the array by identity. Moving the same object in array would move the DOM
     *     element in the same way in the DOM.
     *
     *     For example: `item in items track by item.id` is a typical pattern when the items come from the database. In this
     *     case the object identity does not matter. Two objects are considered equivalent as long as their `id`
     *     property is same.
     *
     *     For example: `item in items | filter:searchText track by item.id` is a pattern that might be used to apply a filter
     *     to items in conjunction with a tracking expression.
     *
     *   * `variable in expression as alias_expression` – You can also provide an optional alias expression which will then store the
     *     intermediate results of the repeater after the filters have been applied. Typically this is used to render a special message
     *     when a filter is active on the repeater, but the filtered result set is empty.
     *
     *     For example: `item in items | filter:x as results` will store the fragment of the repeated items as `results`, but only after
     *     the items have been processed through the filter.
     *
     *     Please note that `as [variable name] is not an operator but rather a part of ngRepeat micro-syntax so it can be used only at the end
     *     (and not as operator, inside an expression).
     *
     *     For example: `item in items | filter : x | orderBy : order | limitTo : limit as results` .
     *
     * @example
     * This example uses `ngRepeat` to display a list of people. A filter is used to restrict the displayed
     * results by name or by age. New (entering) and removed (leaving) items are animated.
      <example module="ngRepeat" name="ngRepeat" deps="angular-animate.js" animations="true" name="ng-repeat">
        <file name="index.html">
          <div ng-controller="repeatController">
            I have {{friends.length}} friends. They are:
            <input type="search" ng-model="q" placeholder="filter friends..." aria-label="filter friends" />
            <ul class="example-animate-container">
              <li class="animate-repeat" ng-repeat="friend in friends | filter:q as results">
                [{{$index + 1}}] {{friend.name}} who is {{friend.age}} years old.
              </li>
              <li class="animate-repeat" ng-if="results.length === 0">
                <strong>No results found...</strong>
              </li>
            </ul>
          </div>
        </file>
        <file name="script.js">
          angular.module('ngRepeat', ['ngAnimate']).controller('repeatController', function($scope) {
            $scope.friends = [
              {name:'John', age:25, gender:'boy'},
              {name:'Jessie', age:30, gender:'girl'},
              {name:'Johanna', age:28, gender:'girl'},
              {name:'Joy', age:15, gender:'girl'},
              {name:'Mary', age:28, gender:'girl'},
              {name:'Peter', age:95, gender:'boy'},
              {name:'Sebastian', age:50, gender:'boy'},
              {name:'Erika', age:27, gender:'girl'},
              {name:'Patrick', age:40, gender:'boy'},
              {name:'Samantha', age:60, gender:'girl'}
            ];
          });
        </file>
        <file name="animations.css">
          .example-animate-container {
            background:white;
            border:1px solid black;
            list-style:none;
            margin:0;
            padding:0 10px;
          }
    
          .animate-repeat {
            line-height:30px;
            list-style:none;
            box-sizing:border-box;
          }
    
          .animate-repeat.ng-move,
          .animate-repeat.ng-enter,
          .animate-repeat.ng-leave {
            transition:all linear 0.5s;
          }
    
          .animate-repeat.ng-leave.ng-leave-active,
          .animate-repeat.ng-move,
          .animate-repeat.ng-enter {
            opacity:0;
            max-height:0;
          }
    
          .animate-repeat.ng-leave,
          .animate-repeat.ng-move.ng-move-active,
          .animate-repeat.ng-enter.ng-enter-active {
            opacity:1;
            max-height:30px;
          }
        </file>
        <file name="protractor.js" type="protractor">
          var friends = element.all(by.repeater('friend in friends'));
    
          it('should render initial data set', function() {
            expect(friends.count()).toBe(10);
            expect(friends.get(0).getText()).toEqual('[1] John who is 25 years old.');
            expect(friends.get(1).getText()).toEqual('[2] Jessie who is 30 years old.');
            expect(friends.last().getText()).toEqual('[10] Samantha who is 60 years old.');
            expect(element(by.binding('friends.length')).getText())
                .toMatch("I have 10 friends. They are:");
          });
    
           it('should update repeater when filter predicate changes', function() {
             expect(friends.count()).toBe(10);
    
             element(by.model('q')).sendKeys('ma');
    
             expect(friends.count()).toBe(2);
             expect(friends.get(0).getText()).toEqual('[1] Mary who is 28 years old.');
             expect(friends.last().getText()).toEqual('[2] Samantha who is 60 years old.');
           });
          </file>
        </example>
     */
    var ngRepeatDirective = ['$parse', '$animate', '$compile', function ($parse, $animate, $compile) {
      var NG_REMOVED = '$$NG_REMOVED';
      var ngRepeatMinErr = minErr('ngRepeat');

      var updateScope = function (scope, index, valueIdentifier, value, keyIdentifier, key, arrayLength) {
        // TODO(perf): generate setters to shave off ~40ms or 1-1.5%
        scope[valueIdentifier] = value;
        if (keyIdentifier) scope[keyIdentifier] = key;
        scope.$index = index;
        scope.$first = index === 0;
        scope.$last = index === arrayLength - 1;
        scope.$middle = !(scope.$first || scope.$last);
        // eslint-disable-next-line no-bitwise
        scope.$odd = !(scope.$even = (index & 1) === 0);
      };

      var getBlockStart = function (block) {
        return block.clone[0];
      };

      var getBlockEnd = function (block) {
        return block.clone[block.clone.length - 1];
      };

      return {
        restrict: 'A',
        multiElement: true,
        transclude: 'element',
        priority: 1000,
        terminal: true,
        $$tlb: true,
        compile: function ngRepeatCompile($element, $attr) {
          var expression = $attr.ngRepeat;
          var ngRepeatEndComment = $compile.$$createComment('end ngRepeat', expression);

          var match = expression.match(/^\s*([\s\S]+?)\s+in\s+([\s\S]+?)(?:\s+as\s+([\s\S]+?))?(?:\s+track\s+by\s+([\s\S]+?))?\s*$/);

          if (!match) {
            throw ngRepeatMinErr('iexp', 'Expected expression in form of \'_item_ in _collection_[ track by _id_]\' but got \'{0}\'.', expression);
          }

          var lhs = match[1];
          var rhs = match[2];
          var aliasAs = match[3];
          var trackByExp = match[4];

          match = lhs.match(/^(?:(\s*[$\w]+)|\(\s*([$\w]+)\s*,\s*([$\w]+)\s*\))$/);

          if (!match) {
            throw ngRepeatMinErr('iidexp', '\'_item_\' in \'_item_ in _collection_\' should be an identifier or \'(_key_, _value_)\' expression, but got \'{0}\'.', lhs);
          }
          var valueIdentifier = match[3] || match[1];
          var keyIdentifier = match[2];

          if (aliasAs && (!/^[$a-zA-Z_][$a-zA-Z0-9_]*$/.test(aliasAs) || /^(null|undefined|this|\$index|\$first|\$middle|\$last|\$even|\$odd|\$parent|\$root|\$id)$/.test(aliasAs))) {
            throw ngRepeatMinErr('badident', 'alias \'{0}\' is invalid --- must be a valid JS identifier which is not a reserved name.', aliasAs);
          }

          var trackByExpGetter, trackByIdExpFn, trackByIdArrayFn, trackByIdObjFn;
          var hashFnLocals = { $id: hashKey };

          if (trackByExp) {
            trackByExpGetter = $parse(trackByExp);
          } else {
            trackByIdArrayFn = function (key, value) {
              return hashKey(value);
            };
            trackByIdObjFn = function (key) {
              return key;
            };
          }

          return function ngRepeatLink($scope, $element, $attr, ctrl, $transclude) {

            if (trackByExpGetter) {
              trackByIdExpFn = function (key, value, index) {
                // assign key, value, and $index to the locals so that they can be used in hash functions
                if (keyIdentifier) hashFnLocals[keyIdentifier] = key;
                hashFnLocals[valueIdentifier] = value;
                hashFnLocals.$index = index;
                return trackByExpGetter($scope, hashFnLocals);
              };
            }

            // Store a list of elements from previous run. This is a hash where key is the item from the
            // iterator, and the value is objects with following properties.
            //   - scope: bound scope
            //   - element: previous element.
            //   - index: position
            //
            // We are using no-proto object so that we don't need to guard against inherited props via
            // hasOwnProperty.
            var lastBlockMap = createMap();

            //watch props
            $scope.$watchCollection(rhs, function ngRepeatAction(collection) {
              var index,
                  length,
                  previousNode = $element[0],
                  // node that cloned nodes should be inserted after
              // initialized to the comment node anchor
              nextNode,

              // Same as lastBlockMap but it has the current state. It will become the
              // lastBlockMap on the next iteration.
              nextBlockMap = createMap(),
                  collectionLength,
                  key,
                  value,
                  // key/value of iteration
              trackById,
                  trackByIdFn,
                  collectionKeys,
                  block,
                  // last object information {scope, element, id}
              nextBlockOrder,
                  elementsToRemove;

              if (aliasAs) {
                $scope[aliasAs] = collection;
              }

              if (isArrayLike(collection)) {
                collectionKeys = collection;
                trackByIdFn = trackByIdExpFn || trackByIdArrayFn;
              } else {
                trackByIdFn = trackByIdExpFn || trackByIdObjFn;
                // if object, extract keys, in enumeration order, unsorted
                collectionKeys = [];
                for (var itemKey in collection) {
                  if (hasOwnProperty.call(collection, itemKey) && itemKey.charAt(0) !== '$') {
                    collectionKeys.push(itemKey);
                  }
                }
              }

              collectionLength = collectionKeys.length;
              nextBlockOrder = new Array(collectionLength);

              // locate existing items
              for (index = 0; index < collectionLength; index++) {
                key = collection === collectionKeys ? index : collectionKeys[index];
                value = collection[key];
                trackById = trackByIdFn(key, value, index);
                if (lastBlockMap[trackById]) {
                  // found previously seen block
                  block = lastBlockMap[trackById];
                  delete lastBlockMap[trackById];
                  nextBlockMap[trackById] = block;
                  nextBlockOrder[index] = block;
                } else if (nextBlockMap[trackById]) {
                  // if collision detected. restore lastBlockMap and throw an error
                  forEach(nextBlockOrder, function (block) {
                    if (block && block.scope) lastBlockMap[block.id] = block;
                  });
                  throw ngRepeatMinErr('dupes', 'Duplicates in a repeater are not allowed. Use \'track by\' expression to specify unique keys. Repeater: {0}, Duplicate key: {1}, Duplicate value: {2}', expression, trackById, value);
                } else {
                  // new never before seen block
                  nextBlockOrder[index] = { id: trackById, scope: undefined, clone: undefined };
                  nextBlockMap[trackById] = true;
                }
              }

              // remove leftover items
              for (var blockKey in lastBlockMap) {
                block = lastBlockMap[blockKey];
                elementsToRemove = getBlockNodes(block.clone);
                $animate.leave(elementsToRemove);
                if (elementsToRemove[0].parentNode) {
                  // if the element was not removed yet because of pending animation, mark it as deleted
                  // so that we can ignore it later
                  for (index = 0, length = elementsToRemove.length; index < length; index++) {
                    elementsToRemove[index][NG_REMOVED] = true;
                  }
                }
                block.scope.$destroy();
              }

              // we are not using forEach for perf reasons (trying to avoid #call)
              for (index = 0; index < collectionLength; index++) {
                key = collection === collectionKeys ? index : collectionKeys[index];
                value = collection[key];
                block = nextBlockOrder[index];

                if (block.scope) {
                  // if we have already seen this object, then we need to reuse the
                  // associated scope/element

                  nextNode = previousNode;

                  // skip nodes that are already pending removal via leave animation
                  do {
                    nextNode = nextNode.nextSibling;
                  } while (nextNode && nextNode[NG_REMOVED]);

                  if (getBlockStart(block) !== nextNode) {
                    // existing item which got moved
                    $animate.move(getBlockNodes(block.clone), null, previousNode);
                  }
                  previousNode = getBlockEnd(block);
                  updateScope(block.scope, index, valueIdentifier, value, keyIdentifier, key, collectionLength);
                } else {
                  // new item which we don't know about
                  $transclude(function ngRepeatTransclude(clone, scope) {
                    block.scope = scope;
                    // http://jsperf.com/clone-vs-createcomment
                    var endNode = ngRepeatEndComment.cloneNode(false);
                    clone[clone.length++] = endNode;

                    $animate.enter(clone, null, previousNode);
                    previousNode = endNode;
                    // Note: We only need the first/last node of the cloned nodes.
                    // However, we need to keep the reference to the jqlite wrapper as it might be changed later
                    // by a directive with templateUrl when its template arrives.
                    block.clone = clone;
                    nextBlockMap[block.id] = block;
                    updateScope(block.scope, index, valueIdentifier, value, keyIdentifier, key, collectionLength);
                  });
                }
              }
              lastBlockMap = nextBlockMap;
            });
          };
        }
      };
    }];

    var NG_HIDE_CLASS = 'ng-hide';
    var NG_HIDE_IN_PROGRESS_CLASS = 'ng-hide-animate';
    /**
     * @ngdoc directive
     * @name ngShow
     * @multiElement
     *
     * @description
     * The `ngShow` directive shows or hides the given HTML element based on the expression provided to
     * the `ngShow` attribute.
     *
     * The element is shown or hidden by removing or adding the `.ng-hide` CSS class onto the element.
     * The `.ng-hide` CSS class is predefined in AngularJS and sets the display style to none (using an
     * `!important` flag). For CSP mode please add `angular-csp.css` to your HTML file (see
     * {@link ng.directive:ngCsp ngCsp}).
     *
     * ```html
     * <!-- when $scope.myValue is truthy (element is visible) -->
     * <div ng-show="myValue"></div>
     *
     * <!-- when $scope.myValue is falsy (element is hidden) -->
     * <div ng-show="myValue" class="ng-hide"></div>
     * ```
     *
     * When the `ngShow` expression evaluates to a falsy value then the `.ng-hide` CSS class is added
     * to the class attribute on the element causing it to become hidden. When truthy, the `.ng-hide`
     * CSS class is removed from the element causing the element not to appear hidden.
     *
     * ## Why is `!important` used?
     *
     * You may be wondering why `!important` is used for the `.ng-hide` CSS class. This is because the
     * `.ng-hide` selector can be easily overridden by heavier selectors. For example, something as
     * simple as changing the display style on a HTML list item would make hidden elements appear
     * visible. This also becomes a bigger issue when dealing with CSS frameworks.
     *
     * By using `!important`, the show and hide behavior will work as expected despite any clash between
     * CSS selector specificity (when `!important` isn't used with any conflicting styles). If a
     * developer chooses to override the styling to change how to hide an element then it is just a
     * matter of using `!important` in their own CSS code.
     *
     * ### Overriding `.ng-hide`
     *
     * By default, the `.ng-hide` class will style the element with `display: none !important`. If you
     * wish to change the hide behavior with `ngShow`/`ngHide`, you can simply overwrite the styles for
     * the `.ng-hide` CSS class. Note that the selector that needs to be used is actually
     * `.ng-hide:not(.ng-hide-animate)` to cope with extra animation classes that can be added.
     *
     * ```css
     * .ng-hide:not(.ng-hide-animate) {
     *   /&#42; These are just alternative ways of hiding an element &#42;/
     *   display: block!important;
     *   position: absolute;
     *   top: -9999px;
     *   left: -9999px;
     * }
     * ```
     *
     * By default you don't need to override anything in CSS and the animations will work around the
     * display style.
     *
     * ## A note about animations with `ngShow`
     *
     * Animations in `ngShow`/`ngHide` work with the show and hide events that are triggered when the
     * directive expression is true and false. This system works like the animation system present with
     * `ngClass` except that you must also include the `!important` flag to override the display
     * property so that the elements are not actually hidden during the animation.
     *
     * ```css
     * /&#42; A working example can be found at the bottom of this page. &#42;/
     * .my-element.ng-hide-add, .my-element.ng-hide-remove {
     *   transition: all 0.5s linear;
     * }
     *
     * .my-element.ng-hide-add { ... }
     * .my-element.ng-hide-add.ng-hide-add-active { ... }
     * .my-element.ng-hide-remove { ... }
     * .my-element.ng-hide-remove.ng-hide-remove-active { ... }
     * ```
     *
     * Keep in mind that, as of AngularJS version 1.3, there is no need to change the display property
     * to block during animation states - ngAnimate will automatically handle the style toggling for you.
     *
     * @animations
     * | Animation                                           | Occurs                                                                                                        |
     * |-----------------------------------------------------|---------------------------------------------------------------------------------------------------------------|
     * | {@link $animate#addClass addClass} `.ng-hide`       | After the `ngShow` expression evaluates to a non truthy value and just before the contents are set to hidden. |
     * | {@link $animate#removeClass removeClass} `.ng-hide` | After the `ngShow` expression evaluates to a truthy value and just before contents are set to visible.        |
     *
     * @element ANY
     * @param {expression} ngShow If the {@link guide/expression expression} is truthy/falsy then the
     *                            element is shown/hidden respectively.
     *
     * @example
     * A simple example, animating the element's opacity:
     *
      <example module="ngAnimate" deps="angular-animate.js" animations="true" name="ng-show-simple">
        <file name="index.html">
          Show: <input type="checkbox" ng-model="checked" aria-label="Toggle ngShow"><br />
          <div class="check-element animate-show-hide" ng-show="checked">
            I show up when your checkbox is checked.
          </div>
        </file>
        <file name="animations.css">
          .animate-show-hide.ng-hide {
            opacity: 0;
          }
    
          .animate-show-hide.ng-hide-add,
          .animate-show-hide.ng-hide-remove {
            transition: all linear 0.5s;
          }
    
          .check-element {
            border: 1px solid black;
            opacity: 1;
            padding: 10px;
          }
        </file>
        <file name="protractor.js" type="protractor">
          it('should check ngShow', function() {
            var checkbox = element(by.model('checked'));
            var checkElem = element(by.css('.check-element'));
    
            expect(checkElem.isDisplayed()).toBe(false);
            checkbox.click();
            expect(checkElem.isDisplayed()).toBe(true);
          });
        </file>
      </example>
     *
     * <hr />
     * @example
     * A more complex example, featuring different show/hide animations:
     *
      <example module="ngAnimate" deps="angular-animate.js" animations="true" name="ng-show-complex">
        <file name="index.html">
          Show: <input type="checkbox" ng-model="checked" aria-label="Toggle ngShow"><br />
          <div class="check-element funky-show-hide" ng-show="checked">
            I show up when your checkbox is checked.
          </div>
        </file>
        <file name="animations.css">
          body {
            overflow: hidden;
            perspective: 1000px;
          }
    
          .funky-show-hide.ng-hide-add {
            transform: rotateZ(0);
            transform-origin: right;
            transition: all 0.5s ease-in-out;
          }
    
          .funky-show-hide.ng-hide-add.ng-hide-add-active {
            transform: rotateZ(-135deg);
          }
    
          .funky-show-hide.ng-hide-remove {
            transform: rotateY(90deg);
            transform-origin: left;
            transition: all 0.5s ease;
          }
    
          .funky-show-hide.ng-hide-remove.ng-hide-remove-active {
            transform: rotateY(0);
          }
    
          .check-element {
            border: 1px solid black;
            opacity: 1;
            padding: 10px;
          }
        </file>
        <file name="protractor.js" type="protractor">
          it('should check ngShow', function() {
            var checkbox = element(by.model('checked'));
            var checkElem = element(by.css('.check-element'));
    
            expect(checkElem.isDisplayed()).toBe(false);
            checkbox.click();
            expect(checkElem.isDisplayed()).toBe(true);
          });
        </file>
      </example>
     */
    var ngShowDirective = ['$animate', function ($animate) {
      return {
        restrict: 'A',
        multiElement: true,
        link: function (scope, element, attr) {
          scope.$watch(attr.ngShow, function ngShowWatchAction(value) {
            // we're adding a temporary, animation-specific class for ng-hide since this way
            // we can control when the element is actually displayed on screen without having
            // to have a global/greedy CSS selector that breaks when other animations are run.
            // Read: https://github.com/angular/angular.js/issues/9103#issuecomment-58335845
            $animate[value ? 'removeClass' : 'addClass'](element, NG_HIDE_CLASS, {
              tempClasses: NG_HIDE_IN_PROGRESS_CLASS
            });
          });
        }
      };
    }];

    /**
     * @ngdoc directive
     * @name ngHide
     * @multiElement
     *
     * @description
     * The `ngHide` directive shows or hides the given HTML element based on the expression provided to
     * the `ngHide` attribute.
     *
     * The element is shown or hidden by removing or adding the `.ng-hide` CSS class onto the element.
     * The `.ng-hide` CSS class is predefined in AngularJS and sets the display style to none (using an
     * `!important` flag). For CSP mode please add `angular-csp.css` to your HTML file (see
     * {@link ng.directive:ngCsp ngCsp}).
     *
     * ```html
     * <!-- when $scope.myValue is truthy (element is hidden) -->
     * <div ng-hide="myValue" class="ng-hide"></div>
     *
     * <!-- when $scope.myValue is falsy (element is visible) -->
     * <div ng-hide="myValue"></div>
     * ```
     *
     * When the `ngHide` expression evaluates to a truthy value then the `.ng-hide` CSS class is added
     * to the class attribute on the element causing it to become hidden. When falsy, the `.ng-hide`
     * CSS class is removed from the element causing the element not to appear hidden.
     *
     * ## Why is `!important` used?
     *
     * You may be wondering why `!important` is used for the `.ng-hide` CSS class. This is because the
     * `.ng-hide` selector can be easily overridden by heavier selectors. For example, something as
     * simple as changing the display style on a HTML list item would make hidden elements appear
     * visible. This also becomes a bigger issue when dealing with CSS frameworks.
     *
     * By using `!important`, the show and hide behavior will work as expected despite any clash between
     * CSS selector specificity (when `!important` isn't used with any conflicting styles). If a
     * developer chooses to override the styling to change how to hide an element then it is just a
     * matter of using `!important` in their own CSS code.
     *
     * ### Overriding `.ng-hide`
     *
     * By default, the `.ng-hide` class will style the element with `display: none !important`. If you
     * wish to change the hide behavior with `ngShow`/`ngHide`, you can simply overwrite the styles for
     * the `.ng-hide` CSS class. Note that the selector that needs to be used is actually
     * `.ng-hide:not(.ng-hide-animate)` to cope with extra animation classes that can be added.
     *
     * ```css
     * .ng-hide:not(.ng-hide-animate) {
     *   /&#42; These are just alternative ways of hiding an element &#42;/
     *   display: block!important;
     *   position: absolute;
     *   top: -9999px;
     *   left: -9999px;
     * }
     * ```
     *
     * By default you don't need to override in CSS anything and the animations will work around the
     * display style.
     *
     * ## A note about animations with `ngHide`
     *
     * Animations in `ngShow`/`ngHide` work with the show and hide events that are triggered when the
     * directive expression is true and false. This system works like the animation system present with
     * `ngClass` except that you must also include the `!important` flag to override the display
     * property so that the elements are not actually hidden during the animation.
     *
     * ```css
     * /&#42; A working example can be found at the bottom of this page. &#42;/
     * .my-element.ng-hide-add, .my-element.ng-hide-remove {
     *   transition: all 0.5s linear;
     * }
     *
     * .my-element.ng-hide-add { ... }
     * .my-element.ng-hide-add.ng-hide-add-active { ... }
     * .my-element.ng-hide-remove { ... }
     * .my-element.ng-hide-remove.ng-hide-remove-active { ... }
     * ```
     *
     * Keep in mind that, as of AngularJS version 1.3, there is no need to change the display property
     * to block during animation states - ngAnimate will automatically handle the style toggling for you.
     *
     * @animations
     * | Animation                                           | Occurs                                                                                                     |
     * |-----------------------------------------------------|------------------------------------------------------------------------------------------------------------|
     * | {@link $animate#addClass addClass} `.ng-hide`       | After the `ngHide` expression evaluates to a truthy value and just before the contents are set to hidden.  |
     * | {@link $animate#removeClass removeClass} `.ng-hide` | After the `ngHide` expression evaluates to a non truthy value and just before contents are set to visible. |
     *
     *
     * @element ANY
     * @param {expression} ngHide If the {@link guide/expression expression} is truthy/falsy then the
     *                            element is hidden/shown respectively.
     *
     * @example
     * A simple example, animating the element's opacity:
     *
      <example module="ngAnimate" deps="angular-animate.js" animations="true" name="ng-hide-simple">
        <file name="index.html">
          Hide: <input type="checkbox" ng-model="checked" aria-label="Toggle ngHide"><br />
          <div class="check-element animate-show-hide" ng-hide="checked">
            I hide when your checkbox is checked.
          </div>
        </file>
        <file name="animations.css">
          .animate-show-hide.ng-hide {
            opacity: 0;
          }
    
          .animate-show-hide.ng-hide-add,
          .animate-show-hide.ng-hide-remove {
            transition: all linear 0.5s;
          }
    
          .check-element {
            border: 1px solid black;
            opacity: 1;
            padding: 10px;
          }
        </file>
        <file name="protractor.js" type="protractor">
          it('should check ngHide', function() {
            var checkbox = element(by.model('checked'));
            var checkElem = element(by.css('.check-element'));
    
            expect(checkElem.isDisplayed()).toBe(true);
            checkbox.click();
            expect(checkElem.isDisplayed()).toBe(false);
          });
        </file>
      </example>
     *
     * <hr />
     * @example
     * A more complex example, featuring different show/hide animations:
     *
      <example module="ngAnimate" deps="angular-animate.js" animations="true" name="ng-hide-complex">
        <file name="index.html">
          Hide: <input type="checkbox" ng-model="checked" aria-label="Toggle ngHide"><br />
          <div class="check-element funky-show-hide" ng-hide="checked">
            I hide when your checkbox is checked.
          </div>
        </file>
        <file name="animations.css">
          body {
            overflow: hidden;
            perspective: 1000px;
          }
    
          .funky-show-hide.ng-hide-add {
            transform: rotateZ(0);
            transform-origin: right;
            transition: all 0.5s ease-in-out;
          }
    
          .funky-show-hide.ng-hide-add.ng-hide-add-active {
            transform: rotateZ(-135deg);
          }
    
          .funky-show-hide.ng-hide-remove {
            transform: rotateY(90deg);
            transform-origin: left;
            transition: all 0.5s ease;
          }
    
          .funky-show-hide.ng-hide-remove.ng-hide-remove-active {
            transform: rotateY(0);
          }
    
          .check-element {
            border: 1px solid black;
            opacity: 1;
            padding: 10px;
          }
        </file>
        <file name="protractor.js" type="protractor">
          it('should check ngHide', function() {
            var checkbox = element(by.model('checked'));
            var checkElem = element(by.css('.check-element'));
    
            expect(checkElem.isDisplayed()).toBe(true);
            checkbox.click();
            expect(checkElem.isDisplayed()).toBe(false);
          });
        </file>
      </example>
     */
    var ngHideDirective = ['$animate', function ($animate) {
      return {
        restrict: 'A',
        multiElement: true,
        link: function (scope, element, attr) {
          scope.$watch(attr.ngHide, function ngHideWatchAction(value) {
            // The comment inside of the ngShowDirective explains why we add and
            // remove a temporary class for the show/hide animation
            $animate[value ? 'addClass' : 'removeClass'](element, NG_HIDE_CLASS, {
              tempClasses: NG_HIDE_IN_PROGRESS_CLASS
            });
          });
        }
      };
    }];

    /**
     * @ngdoc directive
     * @name ngStyle
     * @restrict AC
     *
     * @description
     * The `ngStyle` directive allows you to set CSS style on an HTML element conditionally.
     *
     * @knownIssue
     * You should not use {@link guide/interpolation interpolation} in the value of the `style`
     * attribute, when using the `ngStyle` directive on the same element.
     * See {@link guide/interpolation#known-issues here} for more info.
     *
     * @element ANY
     * @param {expression} ngStyle
     *
     * {@link guide/expression Expression} which evals to an
     * object whose keys are CSS style names and values are corresponding values for those CSS
     * keys.
     *
     * Since some CSS style names are not valid keys for an object, they must be quoted.
     * See the 'background-color' style in the example below.
     *
     * @example
       <example name="ng-style">
         <file name="index.html">
            <input type="button" value="set color" ng-click="myStyle={color:'red'}">
            <input type="button" value="set background" ng-click="myStyle={'background-color':'blue'}">
            <input type="button" value="clear" ng-click="myStyle={}">
            <br/>
            <span ng-style="myStyle">Sample Text</span>
            <pre>myStyle={{myStyle}}</pre>
         </file>
         <file name="style.css">
           span {
             color: black;
           }
         </file>
         <file name="protractor.js" type="protractor">
           var colorSpan = element(by.css('span'));
    
           it('should check ng-style', function() {
             expect(colorSpan.getCssValue('color')).toBe('rgba(0, 0, 0, 1)');
             element(by.css('input[value=\'set color\']')).click();
             expect(colorSpan.getCssValue('color')).toBe('rgba(255, 0, 0, 1)');
             element(by.css('input[value=clear]')).click();
             expect(colorSpan.getCssValue('color')).toBe('rgba(0, 0, 0, 1)');
           });
         </file>
       </example>
     */
    var ngStyleDirective = ngDirective(function (scope, element, attr) {
      scope.$watch(attr.ngStyle, function ngStyleWatchAction(newStyles, oldStyles) {
        if (oldStyles && newStyles !== oldStyles) {
          forEach(oldStyles, function (val, style) {
            element.css(style, '');
          });
        }
        if (newStyles) element.css(newStyles);
      }, true);
    });

    /**
     * @ngdoc directive
     * @name ngSwitch
     * @restrict EA
     *
     * @description
     * The `ngSwitch` directive is used to conditionally swap DOM structure on your template based on a scope expression.
     * Elements within `ngSwitch` but without `ngSwitchWhen` or `ngSwitchDefault` directives will be preserved at the location
     * as specified in the template.
     *
     * The directive itself works similar to ngInclude, however, instead of downloading template code (or loading it
     * from the template cache), `ngSwitch` simply chooses one of the nested elements and makes it visible based on which element
     * matches the value obtained from the evaluated expression. In other words, you define a container element
     * (where you place the directive), place an expression on the **`on="..."` attribute**
     * (or the **`ng-switch="..."` attribute**), define any inner elements inside of the directive and place
     * a when attribute per element. The when attribute is used to inform ngSwitch which element to display when the on
     * expression is evaluated. If a matching expression is not found via a when attribute then an element with the default
     * attribute is displayed.
     *
     * <div class="alert alert-info">
     * Be aware that the attribute values to match against cannot be expressions. They are interpreted
     * as literal string values to match against.
     * For example, **`ng-switch-when="someVal"`** will match against the string `"someVal"` not against the
     * value of the expression `$scope.someVal`.
     * </div>
    
     * @animations
     * | Animation                        | Occurs                              |
     * |----------------------------------|-------------------------------------|
     * | {@link ng.$animate#enter enter}  | after the ngSwitch contents change and the matched child element is placed inside the container |
     * | {@link ng.$animate#leave leave}  | after the ngSwitch contents change and just before the former contents are removed from the DOM |
     *
     * @usage
     *
     * ```
     * <ANY ng-switch="expression">
     *   <ANY ng-switch-when="matchValue1">...</ANY>
     *   <ANY ng-switch-when="matchValue2">...</ANY>
     *   <ANY ng-switch-default>...</ANY>
     * </ANY>
     * ```
     *
     *
     * @scope
     * @priority 1200
     * @param {*} ngSwitch|on expression to match against <code>ng-switch-when</code>.
     * On child elements add:
     *
     * * `ngSwitchWhen`: the case statement to match against. If match then this
     *   case will be displayed. If the same match appears multiple times, all the
     *   elements will be displayed. It is possible to associate multiple values to
     *   the same `ngSwitchWhen` by defining the optional attribute
     *   `ngSwitchWhenSeparator`. The separator will be used to split the value of
     *   the `ngSwitchWhen` attribute into multiple tokens, and the element will show
     *   if any of the `ngSwitch` evaluates to any of these tokens.
     * * `ngSwitchDefault`: the default case when no other case match. If there
     *   are multiple default cases, all of them will be displayed when no other
     *   case match.
     *
     *
     * @example
      <example module="switchExample" deps="angular-animate.js" animations="true" name="ng-switch">
        <file name="index.html">
          <div ng-controller="ExampleController">
            <select ng-model="selection" ng-options="item for item in items">
            </select>
            <code>selection={{selection}}</code>
            <hr/>
            <div class="animate-switch-container"
              ng-switch on="selection">
                <div class="animate-switch" ng-switch-when="settings|options" ng-switch-when-separator="|">Settings Div</div>
                <div class="animate-switch" ng-switch-when="home">Home Span</div>
                <div class="animate-switch" ng-switch-default>default</div>
            </div>
          </div>
        </file>
        <file name="script.js">
          angular.module('switchExample', ['ngAnimate'])
            .controller('ExampleController', ['$scope', function($scope) {
              $scope.items = ['settings', 'home', 'options', 'other'];
              $scope.selection = $scope.items[0];
            }]);
        </file>
        <file name="animations.css">
          .animate-switch-container {
            position:relative;
            background:white;
            border:1px solid black;
            height:40px;
            overflow:hidden;
          }
    
          .animate-switch {
            padding:10px;
          }
    
          .animate-switch.ng-animate {
            transition:all cubic-bezier(0.250, 0.460, 0.450, 0.940) 0.5s;
    
            position:absolute;
            top:0;
            left:0;
            right:0;
            bottom:0;
          }
    
          .animate-switch.ng-leave.ng-leave-active,
          .animate-switch.ng-enter {
            top:-50px;
          }
          .animate-switch.ng-leave,
          .animate-switch.ng-enter.ng-enter-active {
            top:0;
          }
        </file>
        <file name="protractor.js" type="protractor">
          var switchElem = element(by.css('[ng-switch]'));
          var select = element(by.model('selection'));
    
          it('should start in settings', function() {
            expect(switchElem.getText()).toMatch(/Settings Div/);
          });
          it('should change to home', function() {
            select.all(by.css('option')).get(1).click();
            expect(switchElem.getText()).toMatch(/Home Span/);
          });
          it('should change to settings via "options"', function() {
            select.all(by.css('option')).get(2).click();
            expect(switchElem.getText()).toMatch(/Settings Div/);
          });
          it('should select default', function() {
            select.all(by.css('option')).get(3).click();
            expect(switchElem.getText()).toMatch(/default/);
          });
        </file>
      </example>
     */
    var ngSwitchDirective = ['$animate', '$compile', function ($animate, $compile) {
      return {
        require: 'ngSwitch',

        // asks for $scope to fool the BC controller module
        controller: ['$scope', function NgSwitchController() {
          this.cases = {};
        }],
        link: function (scope, element, attr, ngSwitchController) {
          var watchExpr = attr.ngSwitch || attr.on,
              selectedTranscludes = [],
              selectedElements = [],
              previousLeaveAnimations = [],
              selectedScopes = [];

          var spliceFactory = function (array, index) {
            return function (response) {
              if (response !== false) array.splice(index, 1);
            };
          };

          scope.$watch(watchExpr, function ngSwitchWatchAction(value) {
            var i, ii;

            // Start with the last, in case the array is modified during the loop
            while (previousLeaveAnimations.length) {
              $animate.cancel(previousLeaveAnimations.pop());
            }

            for (i = 0, ii = selectedScopes.length; i < ii; ++i) {
              var selected = getBlockNodes(selectedElements[i].clone);
              selectedScopes[i].$destroy();
              var runner = previousLeaveAnimations[i] = $animate.leave(selected);
              runner.done(spliceFactory(previousLeaveAnimations, i));
            }

            selectedElements.length = 0;
            selectedScopes.length = 0;

            if (selectedTranscludes = ngSwitchController.cases['!' + value] || ngSwitchController.cases['?']) {
              forEach(selectedTranscludes, function (selectedTransclude) {
                selectedTransclude.transclude(function (caseElement, selectedScope) {
                  selectedScopes.push(selectedScope);
                  var anchor = selectedTransclude.element;
                  caseElement[caseElement.length++] = $compile.$$createComment('end ngSwitchWhen');
                  var block = { clone: caseElement };

                  selectedElements.push(block);
                  $animate.enter(caseElement, anchor.parent(), anchor);
                });
              });
            }
          });
        }
      };
    }];

    var ngSwitchWhenDirective = ngDirective({
      transclude: 'element',
      priority: 1200,
      require: '^ngSwitch',
      multiElement: true,
      link: function (scope, element, attrs, ctrl, $transclude) {

        var cases = attrs.ngSwitchWhen.split(attrs.ngSwitchWhenSeparator).sort().filter(
        // Filter duplicate cases
        function (element, index, array) {
          return array[index - 1] !== element;
        });

        forEach(cases, function (whenCase) {
          ctrl.cases['!' + whenCase] = ctrl.cases['!' + whenCase] || [];
          ctrl.cases['!' + whenCase].push({ transclude: $transclude, element: element });
        });
      }
    });

    var ngSwitchDefaultDirective = ngDirective({
      transclude: 'element',
      priority: 1200,
      require: '^ngSwitch',
      multiElement: true,
      link: function (scope, element, attr, ctrl, $transclude) {
        ctrl.cases['?'] = ctrl.cases['?'] || [];
        ctrl.cases['?'].push({ transclude: $transclude, element: element });
      }
    });

    /**
     * @ngdoc directive
     * @name ngTransclude
     * @restrict EAC
     *
     * @description
     * Directive that marks the insertion point for the transcluded DOM of the nearest parent directive that uses transclusion.
     *
     * You can specify that you want to insert a named transclusion slot, instead of the default slot, by providing the slot name
     * as the value of the `ng-transclude` or `ng-transclude-slot` attribute.
     *
     * If the transcluded content is not empty (i.e. contains one or more DOM nodes, including whitespace text nodes), any existing
     * content of this element will be removed before the transcluded content is inserted.
     * If the transcluded content is empty (or only whitespace), the existing content is left intact. This lets you provide fallback
     * content in the case that no transcluded content is provided.
     *
     * @element ANY
     *
     * @param {string} ngTransclude|ngTranscludeSlot the name of the slot to insert at this point. If this is not provided, is empty
     *                                               or its value is the same as the name of the attribute then the default slot is used.
     *
     * @example
     * ### Basic transclusion
     * This example demonstrates basic transclusion of content into a component directive.
     * <example name="simpleTranscludeExample" module="transcludeExample">
     *   <file name="index.html">
     *     <script>
     *       angular.module('transcludeExample', [])
     *        .directive('pane', function(){
     *           return {
     *             restrict: 'E',
     *             transclude: true,
     *             scope: { title:'@' },
     *             template: '<div style="border: 1px solid black;">' +
     *                         '<div style="background-color: gray">{{title}}</div>' +
     *                         '<ng-transclude></ng-transclude>' +
     *                       '</div>'
     *           };
     *       })
     *       .controller('ExampleController', ['$scope', function($scope) {
     *         $scope.title = 'Lorem Ipsum';
     *         $scope.text = 'Neque porro quisquam est qui dolorem ipsum quia dolor...';
     *       }]);
     *     </script>
     *     <div ng-controller="ExampleController">
     *       <input ng-model="title" aria-label="title"> <br/>
     *       <textarea ng-model="text" aria-label="text"></textarea> <br/>
     *       <pane title="{{title}}"><span>{{text}}</span></pane>
     *     </div>
     *   </file>
     *   <file name="protractor.js" type="protractor">
     *      it('should have transcluded', function() {
     *        var titleElement = element(by.model('title'));
     *        titleElement.clear();
     *        titleElement.sendKeys('TITLE');
     *        var textElement = element(by.model('text'));
     *        textElement.clear();
     *        textElement.sendKeys('TEXT');
     *        expect(element(by.binding('title')).getText()).toEqual('TITLE');
     *        expect(element(by.binding('text')).getText()).toEqual('TEXT');
     *      });
     *   </file>
     * </example>
     *
     * @example
     * ### Transclude fallback content
     * This example shows how to use `NgTransclude` with fallback content, that
     * is displayed if no transcluded content is provided.
     *
     * <example module="transcludeFallbackContentExample" name="ng-transclude">
     * <file name="index.html">
     * <script>
     * angular.module('transcludeFallbackContentExample', [])
     * .directive('myButton', function(){
     *             return {
     *               restrict: 'E',
     *               transclude: true,
     *               scope: true,
     *               template: '<button style="cursor: pointer;">' +
     *                           '<ng-transclude>' +
     *                             '<b style="color: red;">Button1</b>' +
     *                           '</ng-transclude>' +
     *                         '</button>'
     *             };
     *         });
     * </script>
     * <!-- fallback button content -->
     * <my-button id="fallback"></my-button>
     * <!-- modified button content -->
     * <my-button id="modified">
     *   <i style="color: green;">Button2</i>
     * </my-button>
     * </file>
     * <file name="protractor.js" type="protractor">
     * it('should have different transclude element content', function() {
     *          expect(element(by.id('fallback')).getText()).toBe('Button1');
     *          expect(element(by.id('modified')).getText()).toBe('Button2');
     *        });
     * </file>
     * </example>
     *
     * @example
     * ### Multi-slot transclusion
     * This example demonstrates using multi-slot transclusion in a component directive.
     * <example name="multiSlotTranscludeExample" module="multiSlotTranscludeExample">
     *   <file name="index.html">
     *    <style>
     *      .title, .footer {
     *        background-color: gray
     *      }
     *    </style>
     *    <div ng-controller="ExampleController">
     *      <input ng-model="title" aria-label="title"> <br/>
     *      <textarea ng-model="text" aria-label="text"></textarea> <br/>
     *      <pane>
     *        <pane-title><a ng-href="{{link}}">{{title}}</a></pane-title>
     *        <pane-body><p>{{text}}</p></pane-body>
     *      </pane>
     *    </div>
     *   </file>
     *   <file name="app.js">
     *    angular.module('multiSlotTranscludeExample', [])
     *     .directive('pane', function() {
     *        return {
     *          restrict: 'E',
     *          transclude: {
     *            'title': '?paneTitle',
     *            'body': 'paneBody',
     *            'footer': '?paneFooter'
     *          },
     *          template: '<div style="border: 1px solid black;">' +
     *                      '<div class="title" ng-transclude="title">Fallback Title</div>' +
     *                      '<div ng-transclude="body"></div>' +
     *                      '<div class="footer" ng-transclude="footer">Fallback Footer</div>' +
     *                    '</div>'
     *        };
     *    })
     *    .controller('ExampleController', ['$scope', function($scope) {
     *      $scope.title = 'Lorem Ipsum';
     *      $scope.link = 'https://google.com';
     *      $scope.text = 'Neque porro quisquam est qui dolorem ipsum quia dolor...';
     *    }]);
     *   </file>
     *   <file name="protractor.js" type="protractor">
     *      it('should have transcluded the title and the body', function() {
     *        var titleElement = element(by.model('title'));
     *        titleElement.clear();
     *        titleElement.sendKeys('TITLE');
     *        var textElement = element(by.model('text'));
     *        textElement.clear();
     *        textElement.sendKeys('TEXT');
     *        expect(element(by.css('.title')).getText()).toEqual('TITLE');
     *        expect(element(by.binding('text')).getText()).toEqual('TEXT');
     *        expect(element(by.css('.footer')).getText()).toEqual('Fallback Footer');
     *      });
     *   </file>
     * </example>
     */
    var ngTranscludeMinErr = minErr('ngTransclude');
    var ngTranscludeDirective = ['$compile', function ($compile) {
      return {
        restrict: 'EAC',
        terminal: true,
        compile: function ngTranscludeCompile(tElement) {

          // Remove and cache any original content to act as a fallback
          var fallbackLinkFn = $compile(tElement.contents());
          tElement.empty();

          return function ngTranscludePostLink($scope, $element, $attrs, controller, $transclude) {

            if (!$transclude) {
              throw ngTranscludeMinErr('orphan', 'Illegal use of ngTransclude directive in the template! ' + 'No parent directive that requires a transclusion found. ' + 'Element: {0}', startingTag($element));
            }

            // If the attribute is of the form: `ng-transclude="ng-transclude"` then treat it like the default
            if ($attrs.ngTransclude === $attrs.$attr.ngTransclude) {
              $attrs.ngTransclude = '';
            }
            var slotName = $attrs.ngTransclude || $attrs.ngTranscludeSlot;

            // If the slot is required and no transclusion content is provided then this call will throw an error
            $transclude(ngTranscludeCloneAttachFn, null, slotName);

            // If the slot is optional and no transclusion content is provided then use the fallback content
            if (slotName && !$transclude.isSlotFilled(slotName)) {
              useFallbackContent();
            }

            function ngTranscludeCloneAttachFn(clone, transcludedScope) {
              if (clone.length && notWhitespace(clone)) {
                $element.append(clone);
              } else {
                useFallbackContent();
                // There is nothing linked against the transcluded scope since no content was available,
                // so it should be safe to clean up the generated scope.
                transcludedScope.$destroy();
              }
            }

            function useFallbackContent() {
              // Since this is the fallback content rather than the transcluded content,
              // we link against the scope of this directive rather than the transcluded scope
              fallbackLinkFn($scope, function (clone) {
                $element.append(clone);
              });
            }

            function notWhitespace(nodes) {
              for (var i = 0, ii = nodes.length; i < ii; i++) {
                var node = nodes[i];
                if (node.nodeType !== NODE_TYPE_TEXT || node.nodeValue.trim()) {
                  return true;
                }
              }
            }
          };
        }
      };
    }];

    /**
     * @ngdoc directive
     * @name script
     * @restrict E
     *
     * @description
     * Load the content of a `<script>` element into {@link ng.$templateCache `$templateCache`}, so that the
     * template can be used by {@link ng.directive:ngInclude `ngInclude`},
     * {@link ngRoute.directive:ngView `ngView`}, or {@link guide/directive directives}. The type of the
     * `<script>` element must be specified as `text/ng-template`, and a cache name for the template must be
     * assigned through the element's `id`, which can then be used as a directive's `templateUrl`.
     *
     * @param {string} type Must be set to `'text/ng-template'`.
     * @param {string} id Cache name of the template.
     *
     * @example
      <example  name="script-tag">
        <file name="index.html">
          <script type="text/ng-template" id="/tpl.html">
            Content of the template.
          </script>
    
          <a ng-click="currentTpl='/tpl.html'" id="tpl-link">Load inlined template</a>
          <div id="tpl-content" ng-include src="currentTpl"></div>
        </file>
        <file name="protractor.js" type="protractor">
          it('should load template defined inside script tag', function() {
            element(by.css('#tpl-link')).click();
            expect(element(by.css('#tpl-content')).getText()).toMatch(/Content of the template/);
          });
        </file>
      </example>
     */
    var scriptDirective = ['$templateCache', function ($templateCache) {
      return {
        restrict: 'E',
        terminal: true,
        compile: function (element, attr) {
          if (attr.type === 'text/ng-template') {
            var templateUrl = attr.id,
                text = element[0].text;

            $templateCache.put(templateUrl, text);
          }
        }
      };
    }];

    /* exported selectDirective, optionDirective */

    var noopNgModelController = { $setViewValue: noop, $render: noop };

    function setOptionSelectedStatus(optionEl, value) {
      optionEl.prop('selected', value); // needed for IE
      /**
       * When unselecting an option, setting the property to null / false should be enough
       * However, screenreaders might react to the selected attribute instead, see
       * https://github.com/angular/angular.js/issues/14419
       * Note: "selected" is a boolean attr and will be removed when the "value" arg in attr() is false
       * or null
       */
      optionEl.attr('selected', value);
    }

    /**
     * @ngdoc type
     * @name  select.SelectController
     * @description
     * The controller for the `<select>` directive. This provides support for reading
     * and writing the selected value(s) of the control and also coordinates dynamically
     * added `<option>` elements, perhaps by an `ngRepeat` directive.
     */
    var SelectController = ['$element', '$scope', /** @this */function ($element, $scope) {

      var self = this,
          optionsMap = new NgMap();

      self.selectValueMap = {}; // Keys are the hashed values, values the original values

      // If the ngModel doesn't get provided then provide a dummy noop version to prevent errors
      self.ngModelCtrl = noopNgModelController;
      self.multiple = false;

      // The "unknown" option is one that is prepended to the list if the viewValue
      // does not match any of the options. When it is rendered the value of the unknown
      // option is '? XXX ?' where XXX is the hashKey of the value that is not known.
      //
      // We can't just jqLite('<option>') since jqLite is not smart enough
      // to create it in <select> and IE barfs otherwise.
      self.unknownOption = jqLite(window.document.createElement('option'));

      // The empty option is an option with the value '' that te application developer can
      // provide inside the select. When the model changes to a value that doesn't match an option,
      // it is selected - so if an empty option is provided, no unknown option is generated.
      // However, the empty option is not removed when the model matches an option. It is always selectable
      // and indicates that a "null" selection has been made.
      self.hasEmptyOption = false;
      self.emptyOption = undefined;

      self.renderUnknownOption = function (val) {
        var unknownVal = self.generateUnknownOptionValue(val);
        self.unknownOption.val(unknownVal);
        $element.prepend(self.unknownOption);
        setOptionSelectedStatus(self.unknownOption, true);
        $element.val(unknownVal);
      };

      self.updateUnknownOption = function (val) {
        var unknownVal = self.generateUnknownOptionValue(val);
        self.unknownOption.val(unknownVal);
        setOptionSelectedStatus(self.unknownOption, true);
        $element.val(unknownVal);
      };

      self.generateUnknownOptionValue = function (val) {
        return '? ' + hashKey(val) + ' ?';
      };

      self.removeUnknownOption = function () {
        if (self.unknownOption.parent()) self.unknownOption.remove();
      };

      self.selectEmptyOption = function () {
        if (self.emptyOption) {
          $element.val('');
          setOptionSelectedStatus(self.emptyOption, true);
        }
      };

      self.unselectEmptyOption = function () {
        if (self.hasEmptyOption) {
          self.emptyOption.removeAttr('selected');
        }
      };

      $scope.$on('$destroy', function () {
        // disable unknown option so that we don't do work when the whole select is being destroyed
        self.renderUnknownOption = noop;
      });

      // Read the value of the select control, the implementation of this changes depending
      // upon whether the select can have multiple values and whether ngOptions is at work.
      self.readValue = function readSingleValue() {
        var val = $element.val();
        // ngValue added option values are stored in the selectValueMap, normal interpolations are not
        var realVal = val in self.selectValueMap ? self.selectValueMap[val] : val;

        if (self.hasOption(realVal)) {
          return realVal;
        }

        return null;
      };

      // Write the value to the select control, the implementation of this changes depending
      // upon whether the select can have multiple values and whether ngOptions is at work.
      self.writeValue = function writeSingleValue(value) {
        // Make sure to remove the selected attribute from the previously selected option
        // Otherwise, screen readers might get confused
        var currentlySelectedOption = $element[0].options[$element[0].selectedIndex];
        if (currentlySelectedOption) setOptionSelectedStatus(jqLite(currentlySelectedOption), false);

        if (self.hasOption(value)) {
          self.removeUnknownOption();

          var hashedVal = hashKey(value);
          $element.val(hashedVal in self.selectValueMap ? hashedVal : value);

          // Set selected attribute and property on selected option for screen readers
          var selectedOption = $element[0].options[$element[0].selectedIndex];
          setOptionSelectedStatus(jqLite(selectedOption), true);
        } else {
          if (value == null && self.emptyOption) {
            self.removeUnknownOption();
            self.selectEmptyOption();
          } else if (self.unknownOption.parent().length) {
            self.updateUnknownOption(value);
          } else {
            self.renderUnknownOption(value);
          }
        }
      };

      // Tell the select control that an option, with the given value, has been added
      self.addOption = function (value, element) {
        // Skip comment nodes, as they only pollute the `optionsMap`
        if (element[0].nodeType === NODE_TYPE_COMMENT) return;

        assertNotHasOwnProperty(value, '"option value"');
        if (value === '') {
          self.hasEmptyOption = true;
          self.emptyOption = element;
        }
        var count = optionsMap.get(value) || 0;
        optionsMap.set(value, count + 1);
        // Only render at the end of a digest. This improves render performance when many options
        // are added during a digest and ensures all relevant options are correctly marked as selected
        scheduleRender();
      };

      // Tell the select control that an option, with the given value, has been removed
      self.removeOption = function (value) {
        var count = optionsMap.get(value);
        if (count) {
          if (count === 1) {
            optionsMap.delete(value);
            if (value === '') {
              self.hasEmptyOption = false;
              self.emptyOption = undefined;
            }
          } else {
            optionsMap.set(value, count - 1);
          }
        }
      };

      // Check whether the select control has an option matching the given value
      self.hasOption = function (value) {
        return !!optionsMap.get(value);
      };

      var renderScheduled = false;
      function scheduleRender() {
        if (renderScheduled) return;
        renderScheduled = true;
        $scope.$$postDigest(function () {
          renderScheduled = false;
          self.ngModelCtrl.$render();
        });
      }

      var updateScheduled = false;
      function scheduleViewValueUpdate(renderAfter) {
        if (updateScheduled) return;

        updateScheduled = true;

        $scope.$$postDigest(function () {
          if ($scope.$$destroyed) return;

          updateScheduled = false;
          self.ngModelCtrl.$setViewValue(self.readValue());
          if (renderAfter) self.ngModelCtrl.$render();
        });
      }

      self.registerOption = function (optionScope, optionElement, optionAttrs, interpolateValueFn, interpolateTextFn) {

        if (optionAttrs.$attr.ngValue) {
          // The value attribute is set by ngValue
          var oldVal,
              hashedVal = NaN;
          optionAttrs.$observe('value', function valueAttributeObserveAction(newVal) {

            var removal;
            var previouslySelected = optionElement.prop('selected');

            if (isDefined(hashedVal)) {
              self.removeOption(oldVal);
              delete self.selectValueMap[hashedVal];
              removal = true;
            }

            hashedVal = hashKey(newVal);
            oldVal = newVal;
            self.selectValueMap[hashedVal] = newVal;
            self.addOption(newVal, optionElement);
            // Set the attribute directly instead of using optionAttrs.$set - this stops the observer
            // from firing a second time. Other $observers on value will also get the result of the
            // ngValue expression, not the hashed value
            optionElement.attr('value', hashedVal);

            if (removal && previouslySelected) {
              scheduleViewValueUpdate();
            }
          });
        } else if (interpolateValueFn) {
          // The value attribute is interpolated
          optionAttrs.$observe('value', function valueAttributeObserveAction(newVal) {
            // This method is overwritten in ngOptions and has side-effects!
            self.readValue();

            var removal;
            var previouslySelected = optionElement.prop('selected');

            if (isDefined(oldVal)) {
              self.removeOption(oldVal);
              removal = true;
            }
            oldVal = newVal;
            self.addOption(newVal, optionElement);

            if (removal && previouslySelected) {
              scheduleViewValueUpdate();
            }
          });
        } else if (interpolateTextFn) {
          // The text content is interpolated
          optionScope.$watch(interpolateTextFn, function interpolateWatchAction(newVal, oldVal) {
            optionAttrs.$set('value', newVal);
            var previouslySelected = optionElement.prop('selected');
            if (oldVal !== newVal) {
              self.removeOption(oldVal);
            }
            self.addOption(newVal, optionElement);

            if (oldVal && previouslySelected) {
              scheduleViewValueUpdate();
            }
          });
        } else {
          // The value attribute is static
          self.addOption(optionAttrs.value, optionElement);
        }

        optionAttrs.$observe('disabled', function (newVal) {

          // Since model updates will also select disabled options (like ngOptions),
          // we only have to handle options becoming disabled, not enabled

          if (newVal === 'true' || newVal && optionElement.prop('selected')) {
            if (self.multiple) {
              scheduleViewValueUpdate(true);
            } else {
              self.ngModelCtrl.$setViewValue(null);
              self.ngModelCtrl.$render();
            }
          }
        });

        optionElement.on('$destroy', function () {
          var currentValue = self.readValue();
          var removeValue = optionAttrs.value;

          self.removeOption(removeValue);
          scheduleRender();

          if (self.multiple && currentValue && currentValue.indexOf(removeValue) !== -1 || currentValue === removeValue) {
            // When multiple (selected) options are destroyed at the same time, we don't want
            // to run a model update for each of them. Instead, run a single update in the $$postDigest
            scheduleViewValueUpdate(true);
          }
        });
      };
    }];

    /**
     * @ngdoc directive
     * @name select
     * @restrict E
     *
     * @description
     * HTML `select` element with angular data-binding.
     *
     * The `select` directive is used together with {@link ngModel `ngModel`} to provide data-binding
     * between the scope and the `<select>` control (including setting default values).
     * It also handles dynamic `<option>` elements, which can be added using the {@link ngRepeat `ngRepeat}` or
     * {@link ngOptions `ngOptions`} directives.
     *
     * When an item in the `<select>` menu is selected, the value of the selected option will be bound
     * to the model identified by the `ngModel` directive. With static or repeated options, this is
     * the content of the `value` attribute or the textContent of the `<option>`, if the value attribute is missing.
     * Value and textContent can be interpolated.
     *
     * ## Matching model and option values
     *
     * In general, the match between the model and an option is evaluated by strictly comparing the model
     * value against the value of the available options.
     *
     * If you are setting the option value with the option's `value` attribute, or textContent, the
     * value will always be a `string` which means that the model value must also be a string.
     * Otherwise the `select` directive cannot match them correctly.
     *
     * To bind the model to a non-string value, you can use one of the following strategies:
     * - the {@link ng.ngOptions `ngOptions`} directive
     *   ({@link ng.select#using-select-with-ngoptions-and-setting-a-default-value})
     * - the {@link ng.ngValue `ngValue`} directive, which allows arbitrary expressions to be
     *   option values ({@link ng.select#using-ngvalue-to-bind-the-model-to-an-array-of-objects Example})
     * - model $parsers / $formatters to convert the string value
     *   ({@link ng.select#binding-select-to-a-non-string-value-via-ngmodel-parsing-formatting Example})
     *
     * If the viewValue of `ngModel` does not match any of the options, then the control
     * will automatically add an "unknown" option, which it then removes when the mismatch is resolved.
     *
     * Optionally, a single hard-coded `<option>` element, with the value set to an empty string, can
     * be nested into the `<select>` element. This element will then represent the `null` or "not selected"
     * option. See example below for demonstration.
     *
     * ## Choosing between `ngRepeat` and `ngOptions`
     *
     * In many cases, `ngRepeat` can be used on `<option>` elements instead of {@link ng.directive:ngOptions
     * ngOptions} to achieve a similar result. However, `ngOptions` provides some benefits:
     * - more flexibility in how the `<select>`'s model is assigned via the `select` **`as`** part of the
     * comprehension expression
     * - reduced memory consumption by not creating a new scope for each repeated instance
     * - increased render speed by creating the options in a documentFragment instead of individually
     *
     * Specifically, select with repeated options slows down significantly starting at 2000 options in
     * Chrome and Internet Explorer / Edge.
     *
     *
     * @param {string} ngModel Assignable angular expression to data-bind to.
     * @param {string=} name Property name of the form under which the control is published.
     * @param {string=} multiple Allows multiple options to be selected. The selected values will be
     *     bound to the model as an array.
     * @param {string=} required Sets `required` validation error key if the value is not entered.
     * @param {string=} ngRequired Adds required attribute and required validation constraint to
     * the element when the ngRequired expression evaluates to true. Use ngRequired instead of required
     * when you want to data-bind to the required attribute.
     * @param {string=} ngChange Angular expression to be executed when selected option(s) changes due to user
     *    interaction with the select element.
     * @param {string=} ngOptions sets the options that the select is populated with and defines what is
     * set on the model on selection. See {@link ngOptions `ngOptions`}.
     * @param {string=} ngAttrSize sets the size of the select element dynamically. Uses the
     * {@link guide/interpolation#-ngattr-for-binding-to-arbitrary-attributes ngAttr} directive.
     *
     * @example
     * ### Simple `select` elements with static options
     *
     * <example name="static-select" module="staticSelect">
     * <file name="index.html">
     * <div ng-controller="ExampleController">
     *   <form name="myForm">
     *     <label for="singleSelect"> Single select: </label><br>
     *     <select name="singleSelect" ng-model="data.singleSelect">
     *       <option value="option-1">Option 1</option>
     *       <option value="option-2">Option 2</option>
     *     </select><br>
     *
     *     <label for="singleSelect"> Single select with "not selected" option and dynamic option values: </label><br>
     *     <select name="singleSelect" id="singleSelect" ng-model="data.singleSelect">
     *       <option value="">---Please select---</option> <!-- not selected / blank option -->
     *       <option value="{{data.option1}}">Option 1</option> <!-- interpolation -->
     *       <option value="option-2">Option 2</option>
     *     </select><br>
     *     <button ng-click="forceUnknownOption()">Force unknown option</button><br>
     *     <tt>singleSelect = {{data.singleSelect}}</tt>
     *
     *     <hr>
     *     <label for="multipleSelect"> Multiple select: </label><br>
     *     <select name="multipleSelect" id="multipleSelect" ng-model="data.multipleSelect" multiple>
     *       <option value="option-1">Option 1</option>
     *       <option value="option-2">Option 2</option>
     *       <option value="option-3">Option 3</option>
     *     </select><br>
     *     <tt>multipleSelect = {{data.multipleSelect}}</tt><br/>
     *   </form>
     * </div>
     * </file>
     * <file name="app.js">
     *  angular.module('staticSelect', [])
     *    .controller('ExampleController', ['$scope', function($scope) {
     *      $scope.data = {
     *       singleSelect: null,
     *       multipleSelect: [],
     *       option1: 'option-1'
     *      };
     *
     *      $scope.forceUnknownOption = function() {
     *        $scope.data.singleSelect = 'nonsense';
     *      };
     *   }]);
     * </file>
     *</example>
     *
     * ### Using `ngRepeat` to generate `select` options
     * <example name="select-ngrepeat" module="ngrepeatSelect">
     * <file name="index.html">
     * <div ng-controller="ExampleController">
     *   <form name="myForm">
     *     <label for="repeatSelect"> Repeat select: </label>
     *     <select name="repeatSelect" id="repeatSelect" ng-model="data.model">
     *       <option ng-repeat="option in data.availableOptions" value="{{option.id}}">{{option.name}}</option>
     *     </select>
     *   </form>
     *   <hr>
     *   <tt>model = {{data.model}}</tt><br/>
     * </div>
     * </file>
     * <file name="app.js">
     *  angular.module('ngrepeatSelect', [])
     *    .controller('ExampleController', ['$scope', function($scope) {
     *      $scope.data = {
     *       model: null,
     *       availableOptions: [
     *         {id: '1', name: 'Option A'},
     *         {id: '2', name: 'Option B'},
     *         {id: '3', name: 'Option C'}
     *       ]
     *      };
     *   }]);
     * </file>
     *</example>
     *
     * ### Using `ngValue` to bind the model to an array of objects
     * <example name="select-ngvalue" module="ngvalueSelect">
     * <file name="index.html">
     * <div ng-controller="ExampleController">
     *   <form name="myForm">
     *     <label for="ngvalueselect"> ngvalue select: </label>
     *     <select size="6" name="ngvalueselect" ng-model="data.model" multiple>
     *       <option ng-repeat="option in data.availableOptions" ng-value="option.value">{{option.name}}</option>
     *     </select>
     *   </form>
     *   <hr>
     *   <pre>model = {{data.model | json}}</pre><br/>
     * </div>
     * </file>
     * <file name="app.js">
     *  angular.module('ngvalueSelect', [])
     *    .controller('ExampleController', ['$scope', function($scope) {
     *      $scope.data = {
     *       model: null,
     *       availableOptions: [
               {value: 'myString', name: 'string'},
               {value: 1, name: 'integer'},
               {value: true, name: 'boolean'},
               {value: null, name: 'null'},
               {value: {prop: 'value'}, name: 'object'},
               {value: ['a'], name: 'array'}
     *       ]
     *      };
     *   }]);
     * </file>
     *</example>
     *
     * ### Using `select` with `ngOptions` and setting a default value
     * See the {@link ngOptions ngOptions documentation} for more `ngOptions` usage examples.
     *
     * <example name="select-with-default-values" module="defaultValueSelect">
     * <file name="index.html">
     * <div ng-controller="ExampleController">
     *   <form name="myForm">
     *     <label for="mySelect">Make a choice:</label>
     *     <select name="mySelect" id="mySelect"
     *       ng-options="option.name for option in data.availableOptions track by option.id"
     *       ng-model="data.selectedOption"></select>
     *   </form>
     *   <hr>
     *   <tt>option = {{data.selectedOption}}</tt><br/>
     * </div>
     * </file>
     * <file name="app.js">
     *  angular.module('defaultValueSelect', [])
     *    .controller('ExampleController', ['$scope', function($scope) {
     *      $scope.data = {
     *       availableOptions: [
     *         {id: '1', name: 'Option A'},
     *         {id: '2', name: 'Option B'},
     *         {id: '3', name: 'Option C'}
     *       ],
     *       selectedOption: {id: '3', name: 'Option C'} //This sets the default value of the select in the ui
     *       };
     *   }]);
     * </file>
     *</example>
     *
     *
     * ### Binding `select` to a non-string value via `ngModel` parsing / formatting
     *
     * <example name="select-with-non-string-options" module="nonStringSelect">
     *   <file name="index.html">
     *     <select ng-model="model.id" convert-to-number>
     *       <option value="0">Zero</option>
     *       <option value="1">One</option>
     *       <option value="2">Two</option>
     *     </select>
     *     {{ model }}
     *   </file>
     *   <file name="app.js">
     *     angular.module('nonStringSelect', [])
     *       .run(function($rootScope) {
     *         $rootScope.model = { id: 2 };
     *       })
     *       .directive('convertToNumber', function() {
     *         return {
     *           require: 'ngModel',
     *           link: function(scope, element, attrs, ngModel) {
     *             ngModel.$parsers.push(function(val) {
     *               return parseInt(val, 10);
     *             });
     *             ngModel.$formatters.push(function(val) {
     *               return '' + val;
     *             });
     *           }
     *         };
     *       });
     *   </file>
     *   <file name="protractor.js" type="protractor">
     *     it('should initialize to model', function() {
     *       expect(element(by.model('model.id')).$('option:checked').getText()).toEqual('Two');
     *     });
     *   </file>
     * </example>
     *
     */
    var selectDirective = function () {

      return {
        restrict: 'E',
        require: ['select', '?ngModel'],
        controller: SelectController,
        priority: 1,
        link: {
          pre: selectPreLink,
          post: selectPostLink
        }
      };

      function selectPreLink(scope, element, attr, ctrls) {

        var selectCtrl = ctrls[0];
        var ngModelCtrl = ctrls[1];

        // if ngModel is not defined, we don't need to do anything but set the registerOption
        // function to noop, so options don't get added internally
        if (!ngModelCtrl) {
          selectCtrl.registerOption = noop;
          return;
        }

        selectCtrl.ngModelCtrl = ngModelCtrl;

        // When the selected item(s) changes we delegate getting the value of the select control
        // to the `readValue` method, which can be changed if the select can have multiple
        // selected values or if the options are being generated by `ngOptions`
        element.on('change', function () {
          selectCtrl.removeUnknownOption();
          scope.$apply(function () {
            ngModelCtrl.$setViewValue(selectCtrl.readValue());
          });
        });

        // If the select allows multiple values then we need to modify how we read and write
        // values from and to the control; also what it means for the value to be empty and
        // we have to add an extra watch since ngModel doesn't work well with arrays - it
        // doesn't trigger rendering if only an item in the array changes.
        if (attr.multiple) {
          selectCtrl.multiple = true;

          // Read value now needs to check each option to see if it is selected
          selectCtrl.readValue = function readMultipleValue() {
            var array = [];
            forEach(element.find('option'), function (option) {
              if (option.selected && !option.disabled) {
                var val = option.value;
                array.push(val in selectCtrl.selectValueMap ? selectCtrl.selectValueMap[val] : val);
              }
            });
            return array;
          };

          // Write value now needs to set the selected property of each matching option
          selectCtrl.writeValue = function writeMultipleValue(value) {
            forEach(element.find('option'), function (option) {
              var shouldBeSelected = !!value && (includes(value, option.value) || includes(value, selectCtrl.selectValueMap[option.value]));
              var currentlySelected = option.selected;

              // IE and Edge, adding options to the selection via shift+click/UP/DOWN,
              // will de-select already selected options if "selected" on those options was set
              // more than once (i.e. when the options were already selected)
              // So we only modify the selected property if neccessary.
              // Note: this behavior cannot be replicated via unit tests because it only shows in the
              // actual user interface.
              if (shouldBeSelected !== currentlySelected) {
                setOptionSelectedStatus(jqLite(option), shouldBeSelected);
              }
            });
          };

          // we have to do it on each watch since ngModel watches reference, but
          // we need to work of an array, so we need to see if anything was inserted/removed
          var lastView,
              lastViewRef = NaN;
          scope.$watch(function selectMultipleWatch() {
            if (lastViewRef === ngModelCtrl.$viewValue && !equals(lastView, ngModelCtrl.$viewValue)) {
              lastView = shallowCopy(ngModelCtrl.$viewValue);
              ngModelCtrl.$render();
            }
            lastViewRef = ngModelCtrl.$viewValue;
          });

          // If we are a multiple select then value is now a collection
          // so the meaning of $isEmpty changes
          ngModelCtrl.$isEmpty = function (value) {
            return !value || value.length === 0;
          };
        }
      }

      function selectPostLink(scope, element, attrs, ctrls) {
        // if ngModel is not defined, we don't need to do anything
        var ngModelCtrl = ctrls[1];
        if (!ngModelCtrl) return;

        var selectCtrl = ctrls[0];

        // We delegate rendering to the `writeValue` method, which can be changed
        // if the select can have multiple selected values or if the options are being
        // generated by `ngOptions`.
        // This must be done in the postLink fn to prevent $render to be called before
        // all nodes have been linked correctly.
        ngModelCtrl.$render = function () {
          selectCtrl.writeValue(ngModelCtrl.$viewValue);
        };
      }
    };

    // The option directive is purely designed to communicate the existence (or lack of)
    // of dynamically created (and destroyed) option elements to their containing select
    // directive via its controller.
    var optionDirective = ['$interpolate', function ($interpolate) {
      return {
        restrict: 'E',
        priority: 100,
        compile: function (element, attr) {
          var interpolateValueFn, interpolateTextFn;

          if (isDefined(attr.ngValue)) {
            // Will be handled by registerOption
          } else if (isDefined(attr.value)) {
            // If the value attribute is defined, check if it contains an interpolation
            interpolateValueFn = $interpolate(attr.value, true);
          } else {
            // If the value attribute is not defined then we fall back to the
            // text content of the option element, which may be interpolated
            interpolateTextFn = $interpolate(element.text(), true);
            if (!interpolateTextFn) {
              attr.$set('value', element.text());
            }
          }

          return function (scope, element, attr) {
            // This is an optimization over using ^^ since we don't want to have to search
            // all the way to the root of the DOM for every single option element
            var selectCtrlName = '$selectController',
                parent = element.parent(),
                selectCtrl = parent.data(selectCtrlName) || parent.parent().data(selectCtrlName); // in case we are in optgroup

            if (selectCtrl) {
              selectCtrl.registerOption(scope, element, attr, interpolateValueFn, interpolateTextFn);
            }
          };
        }
      };
    }];

    /**
     * @ngdoc directive
     * @name ngRequired
     * @restrict A
     *
     * @description
     *
     * ngRequired adds the required {@link ngModel.NgModelController#$validators `validator`} to {@link ngModel `ngModel`}.
     * It is most often used for {@link input `input`} and {@link select `select`} controls, but can also be
     * applied to custom controls.
     *
     * The directive sets the `required` attribute on the element if the Angular expression inside
     * `ngRequired` evaluates to true. A special directive for setting `required` is necessary because we
     * cannot use interpolation inside `required`. See the {@link guide/interpolation interpolation guide}
     * for more info.
     *
     * The validator will set the `required` error key to true if the `required` attribute is set and
     * calling {@link ngModel.NgModelController#$isEmpty `NgModelController.$isEmpty`} with the
     * {@link ngModel.NgModelController#$viewValue `ngModel.$viewValue`} returns `true`. For example, the
     * `$isEmpty()` implementation for `input[text]` checks the length of the `$viewValue`. When developing
     * custom controls, `$isEmpty()` can be overwritten to account for a $viewValue that is not string-based.
     *
     * @example
     * <example name="ngRequiredDirective" module="ngRequiredExample">
     *   <file name="index.html">
     *     <script>
     *       angular.module('ngRequiredExample', [])
     *         .controller('ExampleController', ['$scope', function($scope) {
     *           $scope.required = true;
     *         }]);
     *     </script>
     *     <div ng-controller="ExampleController">
     *       <form name="form">
     *         <label for="required">Toggle required: </label>
     *         <input type="checkbox" ng-model="required" id="required" />
     *         <br>
     *         <label for="input">This input must be filled if `required` is true: </label>
     *         <input type="text" ng-model="model" id="input" name="input" ng-required="required" /><br>
     *         <hr>
     *         required error set? = <code>{{form.input.$error.required}}</code><br>
     *         model = <code>{{model}}</code>
     *       </form>
     *     </div>
     *   </file>
     *   <file name="protractor.js" type="protractor">
           var required = element(by.binding('form.input.$error.required'));
           var model = element(by.binding('model'));
           var input = element(by.id('input'));
    
           it('should set the required error', function() {
             expect(required.getText()).toContain('true');
    
             input.sendKeys('123');
             expect(required.getText()).not.toContain('true');
             expect(model.getText()).toContain('123');
           });
     *   </file>
     * </example>
     */
    var requiredDirective = function () {
      return {
        restrict: 'A',
        require: '?ngModel',
        link: function (scope, elm, attr, ctrl) {
          if (!ctrl) return;
          attr.required = true; // force truthy in case we are on non input element

          ctrl.$validators.required = function (modelValue, viewValue) {
            return !attr.required || !ctrl.$isEmpty(viewValue);
          };

          attr.$observe('required', function () {
            ctrl.$validate();
          });
        }
      };
    };

    /**
     * @ngdoc directive
     * @name ngPattern
     *
     * @description
     *
     * ngPattern adds the pattern {@link ngModel.NgModelController#$validators `validator`} to {@link ngModel `ngModel`}.
     * It is most often used for text-based {@link input `input`} controls, but can also be applied to custom text-based controls.
     *
     * The validator sets the `pattern` error key if the {@link ngModel.NgModelController#$viewValue `ngModel.$viewValue`}
     * does not match a RegExp which is obtained by evaluating the Angular expression given in the
     * `ngPattern` attribute value:
     * * If the expression evaluates to a RegExp object, then this is used directly.
     * * If the expression evaluates to a string, then it will be converted to a RegExp after wrapping it
     * in `^` and `$` characters. For instance, `"abc"` will be converted to `new RegExp('^abc$')`.
     *
     * <div class="alert alert-info">
     * **Note:** Avoid using the `g` flag on the RegExp, as it will cause each successive search to
     * start at the index of the last search's match, thus not taking the whole input value into
     * account.
     * </div>
     *
     * <div class="alert alert-info">
     * **Note:** This directive is also added when the plain `pattern` attribute is used, with two
     * differences:
     * <ol>
     *   <li>
     *     `ngPattern` does not set the `pattern` attribute and therefore HTML5 constraint validation is
     *     not available.
     *   </li>
     *   <li>
     *     The `ngPattern` attribute must be an expression, while the `pattern` value must be
     *     interpolated.
     *   </li>
     * </ol>
     * </div>
     *
     * @example
     * <example name="ngPatternDirective" module="ngPatternExample">
     *   <file name="index.html">
     *     <script>
     *       angular.module('ngPatternExample', [])
     *         .controller('ExampleController', ['$scope', function($scope) {
     *           $scope.regex = '\\d+';
     *         }]);
     *     </script>
     *     <div ng-controller="ExampleController">
     *       <form name="form">
     *         <label for="regex">Set a pattern (regex string): </label>
     *         <input type="text" ng-model="regex" id="regex" />
     *         <br>
     *         <label for="input">This input is restricted by the current pattern: </label>
     *         <input type="text" ng-model="model" id="input" name="input" ng-pattern="regex" /><br>
     *         <hr>
     *         input valid? = <code>{{form.input.$valid}}</code><br>
     *         model = <code>{{model}}</code>
     *       </form>
     *     </div>
     *   </file>
     *   <file name="protractor.js" type="protractor">
           var model = element(by.binding('model'));
           var input = element(by.id('input'));
    
           it('should validate the input with the default pattern', function() {
             input.sendKeys('aaa');
             expect(model.getText()).not.toContain('aaa');
    
             input.clear().then(function() {
               input.sendKeys('123');
               expect(model.getText()).toContain('123');
             });
           });
     *   </file>
     * </example>
     */
    var patternDirective = function () {
      return {
        restrict: 'A',
        require: '?ngModel',
        link: function (scope, elm, attr, ctrl) {
          if (!ctrl) return;

          var regexp,
              patternExp = attr.ngPattern || attr.pattern;
          attr.$observe('pattern', function (regex) {
            if (isString(regex) && regex.length > 0) {
              regex = new RegExp('^' + regex + '$');
            }

            if (regex && !regex.test) {
              throw minErr('ngPattern')('noregexp', 'Expected {0} to be a RegExp but was {1}. Element: {2}', patternExp, regex, startingTag(elm));
            }

            regexp = regex || undefined;
            ctrl.$validate();
          });

          ctrl.$validators.pattern = function (modelValue, viewValue) {
            // HTML5 pattern constraint validates the input value, so we validate the viewValue
            return ctrl.$isEmpty(viewValue) || isUndefined(regexp) || regexp.test(viewValue);
          };
        }
      };
    };

    /**
     * @ngdoc directive
     * @name ngMaxlength
     *
     * @description
     *
     * ngMaxlength adds the maxlength {@link ngModel.NgModelController#$validators `validator`} to {@link ngModel `ngModel`}.
     * It is most often used for text-based {@link input `input`} controls, but can also be applied to custom text-based controls.
     *
     * The validator sets the `maxlength` error key if the {@link ngModel.NgModelController#$viewValue `ngModel.$viewValue`}
     * is longer than the integer obtained by evaluating the Angular expression given in the
     * `ngMaxlength` attribute value.
     *
     * <div class="alert alert-info">
     * **Note:** This directive is also added when the plain `maxlength` attribute is used, with two
     * differences:
     * <ol>
     *   <li>
     *     `ngMaxlength` does not set the `maxlength` attribute and therefore HTML5 constraint
     *     validation is not available.
     *   </li>
     *   <li>
     *     The `ngMaxlength` attribute must be an expression, while the `maxlength` value must be
     *     interpolated.
     *   </li>
     * </ol>
     * </div>
     *
     * @example
     * <example name="ngMaxlengthDirective" module="ngMaxlengthExample">
     *   <file name="index.html">
     *     <script>
     *       angular.module('ngMaxlengthExample', [])
     *         .controller('ExampleController', ['$scope', function($scope) {
     *           $scope.maxlength = 5;
     *         }]);
     *     </script>
     *     <div ng-controller="ExampleController">
     *       <form name="form">
     *         <label for="maxlength">Set a maxlength: </label>
     *         <input type="number" ng-model="maxlength" id="maxlength" />
     *         <br>
     *         <label for="input">This input is restricted by the current maxlength: </label>
     *         <input type="text" ng-model="model" id="input" name="input" ng-maxlength="maxlength" /><br>
     *         <hr>
     *         input valid? = <code>{{form.input.$valid}}</code><br>
     *         model = <code>{{model}}</code>
     *       </form>
     *     </div>
     *   </file>
     *   <file name="protractor.js" type="protractor">
           var model = element(by.binding('model'));
           var input = element(by.id('input'));
    
           it('should validate the input with the default maxlength', function() {
             input.sendKeys('abcdef');
             expect(model.getText()).not.toContain('abcdef');
    
             input.clear().then(function() {
               input.sendKeys('abcde');
               expect(model.getText()).toContain('abcde');
             });
           });
     *   </file>
     * </example>
     */
    var maxlengthDirective = function () {
      return {
        restrict: 'A',
        require: '?ngModel',
        link: function (scope, elm, attr, ctrl) {
          if (!ctrl) return;

          var maxlength = -1;
          attr.$observe('maxlength', function (value) {
            var intVal = toInt(value);
            maxlength = isNumberNaN(intVal) ? -1 : intVal;
            ctrl.$validate();
          });
          ctrl.$validators.maxlength = function (modelValue, viewValue) {
            return maxlength < 0 || ctrl.$isEmpty(viewValue) || viewValue.length <= maxlength;
          };
        }
      };
    };

    /**
     * @ngdoc directive
     * @name ngMinlength
     *
     * @description
     *
     * ngMinlength adds the minlength {@link ngModel.NgModelController#$validators `validator`} to {@link ngModel `ngModel`}.
     * It is most often used for text-based {@link input `input`} controls, but can also be applied to custom text-based controls.
     *
     * The validator sets the `minlength` error key if the {@link ngModel.NgModelController#$viewValue `ngModel.$viewValue`}
     * is shorter than the integer obtained by evaluating the Angular expression given in the
     * `ngMinlength` attribute value.
     *
     * <div class="alert alert-info">
     * **Note:** This directive is also added when the plain `minlength` attribute is used, with two
     * differences:
     * <ol>
     *   <li>
     *     `ngMinlength` does not set the `minlength` attribute and therefore HTML5 constraint
     *     validation is not available.
     *   </li>
     *   <li>
     *     The `ngMinlength` value must be an expression, while the `minlength` value must be
     *     interpolated.
     *   </li>
     * </ol>
     * </div>
     *
     * @example
     * <example name="ngMinlengthDirective" module="ngMinlengthExample">
     *   <file name="index.html">
     *     <script>
     *       angular.module('ngMinlengthExample', [])
     *         .controller('ExampleController', ['$scope', function($scope) {
     *           $scope.minlength = 3;
     *         }]);
     *     </script>
     *     <div ng-controller="ExampleController">
     *       <form name="form">
     *         <label for="minlength">Set a minlength: </label>
     *         <input type="number" ng-model="minlength" id="minlength" />
     *         <br>
     *         <label for="input">This input is restricted by the current minlength: </label>
     *         <input type="text" ng-model="model" id="input" name="input" ng-minlength="minlength" /><br>
     *         <hr>
     *         input valid? = <code>{{form.input.$valid}}</code><br>
     *         model = <code>{{model}}</code>
     *       </form>
     *     </div>
     *   </file>
     *   <file name="protractor.js" type="protractor">
           var model = element(by.binding('model'));
           var input = element(by.id('input'));
    
           it('should validate the input with the default minlength', function() {
             input.sendKeys('ab');
             expect(model.getText()).not.toContain('ab');
    
             input.sendKeys('abc');
             expect(model.getText()).toContain('abc');
           });
     *   </file>
     * </example>
     */
    var minlengthDirective = function () {
      return {
        restrict: 'A',
        require: '?ngModel',
        link: function (scope, elm, attr, ctrl) {
          if (!ctrl) return;

          var minlength = 0;
          attr.$observe('minlength', function (value) {
            minlength = toInt(value) || 0;
            ctrl.$validate();
          });
          ctrl.$validators.minlength = function (modelValue, viewValue) {
            return ctrl.$isEmpty(viewValue) || viewValue.length >= minlength;
          };
        }
      };
    };

    if (window.angular.bootstrap) {
      // AngularJS is already loaded, so we can return here...
      if (window.console) {
        console.log('WARNING: Tried to load angular more than once.');
      }
      return;
    }

    // try to bind to jquery now so that one can write jqLite(fn)
    // but we will rebind on bootstrap again.
    bindJQuery();

    publishExternalAPI(angular);

    angular.module("ngLocale", [], ["$provide", function ($provide) {
      var PLURAL_CATEGORY = { ZERO: "zero", ONE: "one", TWO: "two", FEW: "few", MANY: "many", OTHER: "other" };
      function getDecimals(n) {
        n = n + '';
        var i = n.indexOf('.');
        return i == -1 ? 0 : n.length - i - 1;
      }

      function getVF(n, opt_precision) {
        var v = opt_precision;

        if (undefined === v) {
          v = Math.min(getDecimals(n), 3);
        }

        var base = Math.pow(10, v);
        var f = (n * base | 0) % base;
        return { v: v, f: f };
      }

      $provide.value("$locale", {
        "DATETIME_FORMATS": {
          "AMPMS": ["AM", "PM"],
          "DAY": ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
          "ERANAMES": ["Before Christ", "Anno Domini"],
          "ERAS": ["BC", "AD"],
          "FIRSTDAYOFWEEK": 6,
          "MONTH": ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"],
          "SHORTDAY": ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"],
          "SHORTMONTH": ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"],
          "STANDALONEMONTH": ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"],
          "WEEKENDRANGE": [5, 6],
          "fullDate": "EEEE, MMMM d, y",
          "longDate": "MMMM d, y",
          "medium": "MMM d, y h:mm:ss a",
          "mediumDate": "MMM d, y",
          "mediumTime": "h:mm:ss a",
          "short": "M/d/yy h:mm a",
          "shortDate": "M/d/yy",
          "shortTime": "h:mm a"
        },
        "NUMBER_FORMATS": {
          "CURRENCY_SYM": "$",
          "DECIMAL_SEP": ".",
          "GROUP_SEP": ",",
          "PATTERNS": [{
            "gSize": 3,
            "lgSize": 3,
            "maxFrac": 3,
            "minFrac": 0,
            "minInt": 1,
            "negPre": "-",
            "negSuf": "",
            "posPre": "",
            "posSuf": ""
          }, {
            "gSize": 3,
            "lgSize": 3,
            "maxFrac": 2,
            "minFrac": 2,
            "minInt": 1,
            "negPre": "-\u00a4",
            "negSuf": "",
            "posPre": "\u00a4",
            "posSuf": ""
          }]
        },
        "id": "en-us",
        "localeID": "en_US",
        "pluralCat": function (n, opt_precision) {
          var i = n | 0;var vf = getVF(n, opt_precision);if (i == 1 && vf.v == 0) {
            return PLURAL_CATEGORY.ONE;
          }return PLURAL_CATEGORY.OTHER;
        }
      });
    }]);

    jqLite(function () {
      angularInit(window.document, bootstrap);
    });
  })(window);

  !window.angular.$$csp().noInlineStyle && window.angular.element(document.head).prepend('<style type="text/css">@charset "UTF-8";[ng\\:cloak],[ng-cloak],[data-ng-cloak],[x-ng-cloak],.ng-cloak,.x-ng-cloak,.ng-hide:not(.ng-hide-animate){display:none !important;}ng\\:form{display:block;}.ng-animate-shim{visibility:hidden;}.ng-anchor{position:absolute;}</style>');
});
System.registerDynamic('node_modules/angular/index.js', ['./angular'], true, function ($__require, exports, module) {
  var global = this || self,
      GLOBAL = global;
  $__require('./angular');
  module.exports = angular;
});
System.registerDynamic("node_modules/angular2-uuid/index.js", [], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var UUID = function () {
        function UUID() {
            // no-op
        }
        UUID.UUID = function () {
            if (typeof window !== "undefined" && typeof window.crypto !== "undefined" && typeof window.crypto.getRandomValues !== "undefined") {
                // If we have a cryptographically secure PRNG, use that
                // http://stackoverflow.com/questions/6906916/collisions-when-generating-uuids-in-javascript
                var buf = new Uint16Array(8);
                window.crypto.getRandomValues(buf);
                return this.pad4(buf[0]) + this.pad4(buf[1]) + "-" + this.pad4(buf[2]) + "-" + this.pad4(buf[3]) + "-" + this.pad4(buf[4]) + "-" + this.pad4(buf[5]) + this.pad4(buf[6]) + this.pad4(buf[7]);
            } else {
                // Otherwise, just use Math.random
                // https://stackoverflow.com/questions/105034/create-guid-uuid-in-javascript
                // https://stackoverflow.com/questions/11605068/why-does-jshint-argue-against-bitwise-operators-how-should-i-express-this-code
                return this.random4() + this.random4() + "-" + this.random4() + "-" + this.random4() + "-" + this.random4() + "-" + this.random4() + this.random4() + this.random4();
            }
        };
        UUID.pad4 = function (num) {
            var ret = num.toString(16);
            while (ret.length < 4) {
                ret = "0" + ret;
            }
            return ret;
        };
        UUID.random4 = function () {
            return Math.floor((1 + Math.random()) * 0x10000).toString(16).substring(1);
        };
        return UUID;
    }();
    exports.UUID = UUID;
    
});
(function() {
var define = System.amdDefine;
;
(function() {
  var undefined;
  var VERSION = '4.5.1';
  var BIND_FLAG = 1,
      BIND_KEY_FLAG = 2,
      CURRY_BOUND_FLAG = 4,
      CURRY_FLAG = 8,
      CURRY_RIGHT_FLAG = 16,
      PARTIAL_FLAG = 32,
      PARTIAL_RIGHT_FLAG = 64,
      ARY_FLAG = 128,
      REARG_FLAG = 256,
      FLIP_FLAG = 512;
  var UNORDERED_COMPARE_FLAG = 1,
      PARTIAL_COMPARE_FLAG = 2;
  var DEFAULT_TRUNC_LENGTH = 30,
      DEFAULT_TRUNC_OMISSION = '...';
  var HOT_COUNT = 150,
      HOT_SPAN = 16;
  var LARGE_ARRAY_SIZE = 200;
  var LAZY_FILTER_FLAG = 1,
      LAZY_MAP_FLAG = 2,
      LAZY_WHILE_FLAG = 3;
  var FUNC_ERROR_TEXT = 'Expected a function';
  var HASH_UNDEFINED = '__lodash_hash_undefined__';
  var INFINITY = 1 / 0,
      MAX_SAFE_INTEGER = 9007199254740991,
      MAX_INTEGER = 1.7976931348623157e+308,
      NAN = 0 / 0;
  var MAX_ARRAY_LENGTH = 4294967295,
      MAX_ARRAY_INDEX = MAX_ARRAY_LENGTH - 1,
      HALF_MAX_ARRAY_LENGTH = MAX_ARRAY_LENGTH >>> 1;
  var PLACEHOLDER = '__lodash_placeholder__';
  var argsTag = '[object Arguments]',
      arrayTag = '[object Array]',
      boolTag = '[object Boolean]',
      dateTag = '[object Date]',
      errorTag = '[object Error]',
      funcTag = '[object Function]',
      genTag = '[object GeneratorFunction]',
      mapTag = '[object Map]',
      numberTag = '[object Number]',
      objectTag = '[object Object]',
      regexpTag = '[object RegExp]',
      setTag = '[object Set]',
      stringTag = '[object String]',
      symbolTag = '[object Symbol]',
      weakMapTag = '[object WeakMap]',
      weakSetTag = '[object WeakSet]';
  var arrayBufferTag = '[object ArrayBuffer]',
      float32Tag = '[object Float32Array]',
      float64Tag = '[object Float64Array]',
      int8Tag = '[object Int8Array]',
      int16Tag = '[object Int16Array]',
      int32Tag = '[object Int32Array]',
      uint8Tag = '[object Uint8Array]',
      uint8ClampedTag = '[object Uint8ClampedArray]',
      uint16Tag = '[object Uint16Array]',
      uint32Tag = '[object Uint32Array]';
  var reEmptyStringLeading = /\b__p \+= '';/g,
      reEmptyStringMiddle = /\b(__p \+=) '' \+/g,
      reEmptyStringTrailing = /(__e\(.*?\)|\b__t\)) \+\n'';/g;
  var reEscapedHtml = /&(?:amp|lt|gt|quot|#39|#96);/g,
      reUnescapedHtml = /[&<>"'`]/g,
      reHasEscapedHtml = RegExp(reEscapedHtml.source),
      reHasUnescapedHtml = RegExp(reUnescapedHtml.source);
  var reEscape = /<%-([\s\S]+?)%>/g,
      reEvaluate = /<%([\s\S]+?)%>/g,
      reInterpolate = /<%=([\s\S]+?)%>/g;
  var reIsDeepProp = /\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,
      reIsPlainProp = /^\w*$/,
      rePropName = /[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]/g;
  var reRegExpChar = /[\\^$.*+?()[\]{}|]/g,
      reHasRegExpChar = RegExp(reRegExpChar.source);
  var reTrim = /^\s+|\s+$/g,
      reTrimStart = /^\s+/,
      reTrimEnd = /\s+$/;
  var reEscapeChar = /\\(\\)?/g;
  var reEsTemplate = /\$\{([^\\}]*(?:\\.[^\\}]*)*)\}/g;
  var reFlags = /\w*$/;
  var reHasHexPrefix = /^0x/i;
  var reIsBadHex = /^[-+]0x[0-9a-f]+$/i;
  var reIsBinary = /^0b[01]+$/i;
  var reIsHostCtor = /^\[object .+?Constructor\]$/;
  var reIsOctal = /^0o[0-7]+$/i;
  var reIsUint = /^(?:0|[1-9]\d*)$/;
  var reLatin1 = /[\xc0-\xd6\xd8-\xde\xdf-\xf6\xf8-\xff]/g;
  var reNoMatch = /($^)/;
  var reUnescapedString = /['\n\r\u2028\u2029\\]/g;
  var rsAstralRange = '\\ud800-\\udfff',
      rsComboMarksRange = '\\u0300-\\u036f\\ufe20-\\ufe23',
      rsComboSymbolsRange = '\\u20d0-\\u20f0',
      rsDingbatRange = '\\u2700-\\u27bf',
      rsLowerRange = 'a-z\\xdf-\\xf6\\xf8-\\xff',
      rsMathOpRange = '\\xac\\xb1\\xd7\\xf7',
      rsNonCharRange = '\\x00-\\x2f\\x3a-\\x40\\x5b-\\x60\\x7b-\\xbf',
      rsQuoteRange = '\\u2018\\u2019\\u201c\\u201d',
      rsSpaceRange = ' \\t\\x0b\\f\\xa0\\ufeff\\n\\r\\u2028\\u2029\\u1680\\u180e\\u2000\\u2001\\u2002\\u2003\\u2004\\u2005\\u2006\\u2007\\u2008\\u2009\\u200a\\u202f\\u205f\\u3000',
      rsUpperRange = 'A-Z\\xc0-\\xd6\\xd8-\\xde',
      rsVarRange = '\\ufe0e\\ufe0f',
      rsBreakRange = rsMathOpRange + rsNonCharRange + rsQuoteRange + rsSpaceRange;
  var rsAstral = '[' + rsAstralRange + ']',
      rsBreak = '[' + rsBreakRange + ']',
      rsCombo = '[' + rsComboMarksRange + rsComboSymbolsRange + ']',
      rsDigits = '\\d+',
      rsDingbat = '[' + rsDingbatRange + ']',
      rsLower = '[' + rsLowerRange + ']',
      rsMisc = '[^' + rsAstralRange + rsBreakRange + rsDigits + rsDingbatRange + rsLowerRange + rsUpperRange + ']',
      rsFitz = '\\ud83c[\\udffb-\\udfff]',
      rsModifier = '(?:' + rsCombo + '|' + rsFitz + ')',
      rsNonAstral = '[^' + rsAstralRange + ']',
      rsRegional = '(?:\\ud83c[\\udde6-\\uddff]){2}',
      rsSurrPair = '[\\ud800-\\udbff][\\udc00-\\udfff]',
      rsUpper = '[' + rsUpperRange + ']',
      rsZWJ = '\\u200d';
  var rsLowerMisc = '(?:' + rsLower + '|' + rsMisc + ')',
      rsUpperMisc = '(?:' + rsUpper + '|' + rsMisc + ')',
      reOptMod = rsModifier + '?',
      rsOptVar = '[' + rsVarRange + ']?',
      rsOptJoin = '(?:' + rsZWJ + '(?:' + [rsNonAstral, rsRegional, rsSurrPair].join('|') + ')' + rsOptVar + reOptMod + ')*',
      rsSeq = rsOptVar + reOptMod + rsOptJoin,
      rsEmoji = '(?:' + [rsDingbat, rsRegional, rsSurrPair].join('|') + ')' + rsSeq,
      rsSymbol = '(?:' + [rsNonAstral + rsCombo + '?', rsCombo, rsRegional, rsSurrPair, rsAstral].join('|') + ')';
  var reComboMark = RegExp(rsCombo, 'g');
  var reComplexSymbol = RegExp(rsFitz + '(?=' + rsFitz + ')|' + rsSymbol + rsSeq, 'g');
  var reHasComplexSymbol = RegExp('[' + rsZWJ + rsAstralRange + rsComboMarksRange + rsComboSymbolsRange + rsVarRange + ']');
  var reBasicWord = /[a-zA-Z0-9]+/g;
  var reComplexWord = RegExp([rsUpper + '?' + rsLower + '+(?=' + [rsBreak, rsUpper, '$'].join('|') + ')', rsUpperMisc + '+(?=' + [rsBreak, rsUpper + rsLowerMisc, '$'].join('|') + ')', rsUpper + '?' + rsLowerMisc + '+', rsUpper + '+', rsDigits, rsEmoji].join('|'), 'g');
  var reHasComplexWord = /[a-z][A-Z]|[0-9][a-zA-Z]|[a-zA-Z][0-9]|[^a-zA-Z0-9 ]/;
  var contextProps = ['Array', 'Buffer', 'Date', 'Error', 'Float32Array', 'Float64Array', 'Function', 'Int8Array', 'Int16Array', 'Int32Array', 'Map', 'Math', 'Object', 'Reflect', 'RegExp', 'Set', 'String', 'Symbol', 'TypeError', 'Uint8Array', 'Uint8ClampedArray', 'Uint16Array', 'Uint32Array', 'WeakMap', '_', 'clearTimeout', 'isFinite', 'parseInt', 'setTimeout'];
  var templateCounter = -1;
  var typedArrayTags = {};
  typedArrayTags[float32Tag] = typedArrayTags[float64Tag] = typedArrayTags[int8Tag] = typedArrayTags[int16Tag] = typedArrayTags[int32Tag] = typedArrayTags[uint8Tag] = typedArrayTags[uint8ClampedTag] = typedArrayTags[uint16Tag] = typedArrayTags[uint32Tag] = true;
  typedArrayTags[argsTag] = typedArrayTags[arrayTag] = typedArrayTags[arrayBufferTag] = typedArrayTags[boolTag] = typedArrayTags[dateTag] = typedArrayTags[errorTag] = typedArrayTags[funcTag] = typedArrayTags[mapTag] = typedArrayTags[numberTag] = typedArrayTags[objectTag] = typedArrayTags[regexpTag] = typedArrayTags[setTag] = typedArrayTags[stringTag] = typedArrayTags[weakMapTag] = false;
  var cloneableTags = {};
  cloneableTags[argsTag] = cloneableTags[arrayTag] = cloneableTags[arrayBufferTag] = cloneableTags[boolTag] = cloneableTags[dateTag] = cloneableTags[float32Tag] = cloneableTags[float64Tag] = cloneableTags[int8Tag] = cloneableTags[int16Tag] = cloneableTags[int32Tag] = cloneableTags[mapTag] = cloneableTags[numberTag] = cloneableTags[objectTag] = cloneableTags[regexpTag] = cloneableTags[setTag] = cloneableTags[stringTag] = cloneableTags[symbolTag] = cloneableTags[uint8Tag] = cloneableTags[uint8ClampedTag] = cloneableTags[uint16Tag] = cloneableTags[uint32Tag] = true;
  cloneableTags[errorTag] = cloneableTags[funcTag] = cloneableTags[weakMapTag] = false;
  var deburredLetters = {
    '\xc0': 'A',
    '\xc1': 'A',
    '\xc2': 'A',
    '\xc3': 'A',
    '\xc4': 'A',
    '\xc5': 'A',
    '\xe0': 'a',
    '\xe1': 'a',
    '\xe2': 'a',
    '\xe3': 'a',
    '\xe4': 'a',
    '\xe5': 'a',
    '\xc7': 'C',
    '\xe7': 'c',
    '\xd0': 'D',
    '\xf0': 'd',
    '\xc8': 'E',
    '\xc9': 'E',
    '\xca': 'E',
    '\xcb': 'E',
    '\xe8': 'e',
    '\xe9': 'e',
    '\xea': 'e',
    '\xeb': 'e',
    '\xcC': 'I',
    '\xcd': 'I',
    '\xce': 'I',
    '\xcf': 'I',
    '\xeC': 'i',
    '\xed': 'i',
    '\xee': 'i',
    '\xef': 'i',
    '\xd1': 'N',
    '\xf1': 'n',
    '\xd2': 'O',
    '\xd3': 'O',
    '\xd4': 'O',
    '\xd5': 'O',
    '\xd6': 'O',
    '\xd8': 'O',
    '\xf2': 'o',
    '\xf3': 'o',
    '\xf4': 'o',
    '\xf5': 'o',
    '\xf6': 'o',
    '\xf8': 'o',
    '\xd9': 'U',
    '\xda': 'U',
    '\xdb': 'U',
    '\xdc': 'U',
    '\xf9': 'u',
    '\xfa': 'u',
    '\xfb': 'u',
    '\xfc': 'u',
    '\xdd': 'Y',
    '\xfd': 'y',
    '\xff': 'y',
    '\xc6': 'Ae',
    '\xe6': 'ae',
    '\xde': 'Th',
    '\xfe': 'th',
    '\xdf': 'ss'
  };
  var htmlEscapes = {
    '&': '&amp;',
    '<': '&lt;',
    '>': '&gt;',
    '"': '&quot;',
    "'": '&#39;',
    '`': '&#96;'
  };
  var htmlUnescapes = {
    '&amp;': '&',
    '&lt;': '<',
    '&gt;': '>',
    '&quot;': '"',
    '&#39;': "'",
    '&#96;': '`'
  };
  var objectTypes = {
    'function': true,
    'object': true
  };
  var stringEscapes = {
    '\\': '\\',
    "'": "'",
    '\n': 'n',
    '\r': 'r',
    '\u2028': 'u2028',
    '\u2029': 'u2029'
  };
  var freeParseFloat = parseFloat,
      freeParseInt = parseInt;
  var freeExports = (objectTypes[typeof exports] && exports && !exports.nodeType) ? exports : undefined;
  var freeModule = (objectTypes[typeof module] && module && !module.nodeType) ? module : undefined;
  var moduleExports = (freeModule && freeModule.exports === freeExports) ? freeExports : undefined;
  var freeGlobal = checkGlobal(freeExports && freeModule && typeof global == 'object' && global);
  var freeSelf = checkGlobal(objectTypes[typeof self] && self);
  var freeWindow = checkGlobal(objectTypes[typeof window] && window);
  var thisGlobal = checkGlobal(objectTypes[typeof this] && this);
  var root = freeGlobal || ((freeWindow !== (thisGlobal && thisGlobal.window)) && freeWindow) || freeSelf || thisGlobal || Function('return this')();
  function addMapEntry(map, pair) {
    map.set(pair[0], pair[1]);
    return map;
  }
  function addSetEntry(set, value) {
    set.add(value);
    return set;
  }
  function apply(func, thisArg, args) {
    var length = args.length;
    switch (length) {
      case 0:
        return func.call(thisArg);
      case 1:
        return func.call(thisArg, args[0]);
      case 2:
        return func.call(thisArg, args[0], args[1]);
      case 3:
        return func.call(thisArg, args[0], args[1], args[2]);
    }
    return func.apply(thisArg, args);
  }
  function arrayAggregator(array, setter, iteratee, accumulator) {
    var index = -1,
        length = array.length;
    while (++index < length) {
      var value = array[index];
      setter(accumulator, value, iteratee(value), array);
    }
    return accumulator;
  }
  function arrayConcat(array, other) {
    var index = -1,
        length = array.length,
        othIndex = -1,
        othLength = other.length,
        result = Array(length + othLength);
    while (++index < length) {
      result[index] = array[index];
    }
    while (++othIndex < othLength) {
      result[index++] = other[othIndex];
    }
    return result;
  }
  function arrayEach(array, iteratee) {
    var index = -1,
        length = array.length;
    while (++index < length) {
      if (iteratee(array[index], index, array) === false) {
        break;
      }
    }
    return array;
  }
  function arrayEachRight(array, iteratee) {
    var length = array.length;
    while (length--) {
      if (iteratee(array[length], length, array) === false) {
        break;
      }
    }
    return array;
  }
  function arrayEvery(array, predicate) {
    var index = -1,
        length = array.length;
    while (++index < length) {
      if (!predicate(array[index], index, array)) {
        return false;
      }
    }
    return true;
  }
  function arrayFilter(array, predicate) {
    var index = -1,
        length = array.length,
        resIndex = -1,
        result = [];
    while (++index < length) {
      var value = array[index];
      if (predicate(value, index, array)) {
        result[++resIndex] = value;
      }
    }
    return result;
  }
  function arrayIncludes(array, value) {
    return !!array.length && baseIndexOf(array, value, 0) > -1;
  }
  function arrayIncludesWith(array, value, comparator) {
    var index = -1,
        length = array.length;
    while (++index < length) {
      if (comparator(value, array[index])) {
        return true;
      }
    }
    return false;
  }
  function arrayMap(array, iteratee) {
    var index = -1,
        length = array.length,
        result = Array(length);
    while (++index < length) {
      result[index] = iteratee(array[index], index, array);
    }
    return result;
  }
  function arrayPush(array, values) {
    var index = -1,
        length = values.length,
        offset = array.length;
    while (++index < length) {
      array[offset + index] = values[index];
    }
    return array;
  }
  function arrayReduce(array, iteratee, accumulator, initAccum) {
    var index = -1,
        length = array.length;
    if (initAccum && length) {
      accumulator = array[++index];
    }
    while (++index < length) {
      accumulator = iteratee(accumulator, array[index], index, array);
    }
    return accumulator;
  }
  function arrayReduceRight(array, iteratee, accumulator, initAccum) {
    var length = array.length;
    if (initAccum && length) {
      accumulator = array[--length];
    }
    while (length--) {
      accumulator = iteratee(accumulator, array[length], length, array);
    }
    return accumulator;
  }
  function arraySome(array, predicate) {
    var index = -1,
        length = array.length;
    while (++index < length) {
      if (predicate(array[index], index, array)) {
        return true;
      }
    }
    return false;
  }
  function baseExtremum(array, iteratee, comparator) {
    var index = -1,
        length = array.length;
    while (++index < length) {
      var value = array[index],
          current = iteratee(value);
      if (current != null && (computed === undefined ? current === current : comparator(current, computed))) {
        var computed = current,
            result = value;
      }
    }
    return result;
  }
  function baseFind(collection, predicate, eachFunc, retKey) {
    var result;
    eachFunc(collection, function(value, key, collection) {
      if (predicate(value, key, collection)) {
        result = retKey ? key : value;
        return false;
      }
    });
    return result;
  }
  function baseFindIndex(array, predicate, fromRight) {
    var length = array.length,
        index = fromRight ? length : -1;
    while ((fromRight ? index-- : ++index < length)) {
      if (predicate(array[index], index, array)) {
        return index;
      }
    }
    return -1;
  }
  function baseIndexOf(array, value, fromIndex) {
    if (value !== value) {
      return indexOfNaN(array, fromIndex);
    }
    var index = fromIndex - 1,
        length = array.length;
    while (++index < length) {
      if (array[index] === value) {
        return index;
      }
    }
    return -1;
  }
  function baseReduce(collection, iteratee, accumulator, initAccum, eachFunc) {
    eachFunc(collection, function(value, index, collection) {
      accumulator = initAccum ? (initAccum = false, value) : iteratee(accumulator, value, index, collection);
    });
    return accumulator;
  }
  function baseSortBy(array, comparer) {
    var length = array.length;
    array.sort(comparer);
    while (length--) {
      array[length] = array[length].value;
    }
    return array;
  }
  function baseSum(array, iteratee) {
    var result,
        index = -1,
        length = array.length;
    while (++index < length) {
      var current = iteratee(array[index]);
      if (current !== undefined) {
        result = result === undefined ? current : (result + current);
      }
    }
    return result;
  }
  function baseTimes(n, iteratee) {
    var index = -1,
        result = Array(n);
    while (++index < n) {
      result[index] = iteratee(index);
    }
    return result;
  }
  function baseToPairs(object, props) {
    return arrayMap(props, function(key) {
      return [key, object[key]];
    });
  }
  function baseUnary(func) {
    return function(value) {
      return func(value);
    };
  }
  function baseValues(object, props) {
    return arrayMap(props, function(key) {
      return object[key];
    });
  }
  function charsStartIndex(strSymbols, chrSymbols) {
    var index = -1,
        length = strSymbols.length;
    while (++index < length && baseIndexOf(chrSymbols, strSymbols[index], 0) > -1) {}
    return index;
  }
  function charsEndIndex(strSymbols, chrSymbols) {
    var index = strSymbols.length;
    while (index-- && baseIndexOf(chrSymbols, strSymbols[index], 0) > -1) {}
    return index;
  }
  function checkGlobal(value) {
    return (value && value.Object === Object) ? value : null;
  }
  function compareAscending(value, other) {
    if (value !== other) {
      var valIsNull = value === null,
          valIsUndef = value === undefined,
          valIsReflexive = value === value;
      var othIsNull = other === null,
          othIsUndef = other === undefined,
          othIsReflexive = other === other;
      if ((value > other && !othIsNull) || !valIsReflexive || (valIsNull && !othIsUndef && othIsReflexive) || (valIsUndef && othIsReflexive)) {
        return 1;
      }
      if ((value < other && !valIsNull) || !othIsReflexive || (othIsNull && !valIsUndef && valIsReflexive) || (othIsUndef && valIsReflexive)) {
        return -1;
      }
    }
    return 0;
  }
  function compareMultiple(object, other, orders) {
    var index = -1,
        objCriteria = object.criteria,
        othCriteria = other.criteria,
        length = objCriteria.length,
        ordersLength = orders.length;
    while (++index < length) {
      var result = compareAscending(objCriteria[index], othCriteria[index]);
      if (result) {
        if (index >= ordersLength) {
          return result;
        }
        var order = orders[index];
        return result * (order == 'desc' ? -1 : 1);
      }
    }
    return object.index - other.index;
  }
  function countHolders(array, placeholder) {
    var length = array.length,
        result = 0;
    while (length--) {
      if (array[length] === placeholder) {
        result++;
      }
    }
    return result;
  }
  function deburrLetter(letter) {
    return deburredLetters[letter];
  }
  function escapeHtmlChar(chr) {
    return htmlEscapes[chr];
  }
  function escapeStringChar(chr) {
    return '\\' + stringEscapes[chr];
  }
  function indexOfNaN(array, fromIndex, fromRight) {
    var length = array.length,
        index = fromIndex + (fromRight ? 0 : -1);
    while ((fromRight ? index-- : ++index < length)) {
      var other = array[index];
      if (other !== other) {
        return index;
      }
    }
    return -1;
  }
  function isHostObject(value) {
    var result = false;
    if (value != null && typeof value.toString != 'function') {
      try {
        result = !!(value + '');
      } catch (e) {}
    }
    return result;
  }
  function isIndex(value, length) {
    value = (typeof value == 'number' || reIsUint.test(value)) ? +value : -1;
    length = length == null ? MAX_SAFE_INTEGER : length;
    return value > -1 && value % 1 == 0 && value < length;
  }
  function iteratorToArray(iterator) {
    var data,
        result = [];
    while (!(data = iterator.next()).done) {
      result.push(data.value);
    }
    return result;
  }
  function mapToArray(map) {
    var index = -1,
        result = Array(map.size);
    map.forEach(function(value, key) {
      result[++index] = [key, value];
    });
    return result;
  }
  function replaceHolders(array, placeholder) {
    var index = -1,
        length = array.length,
        resIndex = -1,
        result = [];
    while (++index < length) {
      var value = array[index];
      if (value === placeholder || value === PLACEHOLDER) {
        array[index] = PLACEHOLDER;
        result[++resIndex] = index;
      }
    }
    return result;
  }
  function setToArray(set) {
    var index = -1,
        result = Array(set.size);
    set.forEach(function(value) {
      result[++index] = value;
    });
    return result;
  }
  function stringSize(string) {
    if (!(string && reHasComplexSymbol.test(string))) {
      return string.length;
    }
    var result = reComplexSymbol.lastIndex = 0;
    while (reComplexSymbol.test(string)) {
      result++;
    }
    return result;
  }
  function stringToArray(string) {
    return string.match(reComplexSymbol);
  }
  function unescapeHtmlChar(chr) {
    return htmlUnescapes[chr];
  }
  function runInContext(context) {
    context = context ? _.defaults({}, context, _.pick(root, contextProps)) : root;
    var Date = context.Date,
        Error = context.Error,
        Math = context.Math,
        RegExp = context.RegExp,
        TypeError = context.TypeError;
    var arrayProto = context.Array.prototype,
        objectProto = context.Object.prototype;
    var funcToString = context.Function.prototype.toString;
    var hasOwnProperty = objectProto.hasOwnProperty;
    var idCounter = 0;
    var objectCtorString = funcToString.call(Object);
    var objectToString = objectProto.toString;
    var oldDash = root._;
    var reIsNative = RegExp('^' + funcToString.call(hasOwnProperty).replace(reRegExpChar, '\\$&').replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g, '$1.*?') + '$');
    var Buffer = moduleExports ? context.Buffer : undefined,
        Reflect = context.Reflect,
        Symbol = context.Symbol,
        Uint8Array = context.Uint8Array,
        clearTimeout = context.clearTimeout,
        enumerate = Reflect ? Reflect.enumerate : undefined,
        getPrototypeOf = Object.getPrototypeOf,
        getOwnPropertySymbols = Object.getOwnPropertySymbols,
        iteratorSymbol = typeof(iteratorSymbol = Symbol && Symbol.iterator) == 'symbol' ? iteratorSymbol : undefined,
        objectCreate = Object.create,
        propertyIsEnumerable = objectProto.propertyIsEnumerable,
        setTimeout = context.setTimeout,
        splice = arrayProto.splice;
    var nativeCeil = Math.ceil,
        nativeFloor = Math.floor,
        nativeIsFinite = context.isFinite,
        nativeJoin = arrayProto.join,
        nativeKeys = Object.keys,
        nativeMax = Math.max,
        nativeMin = Math.min,
        nativeParseInt = context.parseInt,
        nativeRandom = Math.random,
        nativeReverse = arrayProto.reverse;
    var Map = getNative(context, 'Map'),
        Set = getNative(context, 'Set'),
        WeakMap = getNative(context, 'WeakMap'),
        nativeCreate = getNative(Object, 'create');
    var metaMap = WeakMap && new WeakMap;
    var mapCtorString = Map ? funcToString.call(Map) : '',
        setCtorString = Set ? funcToString.call(Set) : '',
        weakMapCtorString = WeakMap ? funcToString.call(WeakMap) : '';
    var symbolProto = Symbol ? Symbol.prototype : undefined,
        symbolValueOf = Symbol ? symbolProto.valueOf : undefined,
        symbolToString = Symbol ? symbolProto.toString : undefined;
    var realNames = {};
    function lodash(value) {
      if (isObjectLike(value) && !isArray(value) && !(value instanceof LazyWrapper)) {
        if (value instanceof LodashWrapper) {
          return value;
        }
        if (hasOwnProperty.call(value, '__wrapped__')) {
          return wrapperClone(value);
        }
      }
      return new LodashWrapper(value);
    }
    function baseLodash() {}
    function LodashWrapper(value, chainAll) {
      this.__wrapped__ = value;
      this.__actions__ = [];
      this.__chain__ = !!chainAll;
      this.__index__ = 0;
      this.__values__ = undefined;
    }
    lodash.templateSettings = {
      'escape': reEscape,
      'evaluate': reEvaluate,
      'interpolate': reInterpolate,
      'variable': '',
      'imports': {'_': lodash}
    };
    function LazyWrapper(value) {
      this.__wrapped__ = value;
      this.__actions__ = [];
      this.__dir__ = 1;
      this.__filtered__ = false;
      this.__iteratees__ = [];
      this.__takeCount__ = MAX_ARRAY_LENGTH;
      this.__views__ = [];
    }
    function lazyClone() {
      var result = new LazyWrapper(this.__wrapped__);
      result.__actions__ = copyArray(this.__actions__);
      result.__dir__ = this.__dir__;
      result.__filtered__ = this.__filtered__;
      result.__iteratees__ = copyArray(this.__iteratees__);
      result.__takeCount__ = this.__takeCount__;
      result.__views__ = copyArray(this.__views__);
      return result;
    }
    function lazyReverse() {
      if (this.__filtered__) {
        var result = new LazyWrapper(this);
        result.__dir__ = -1;
        result.__filtered__ = true;
      } else {
        result = this.clone();
        result.__dir__ *= -1;
      }
      return result;
    }
    function lazyValue() {
      var array = this.__wrapped__.value(),
          dir = this.__dir__,
          isArr = isArray(array),
          isRight = dir < 0,
          arrLength = isArr ? array.length : 0,
          view = getView(0, arrLength, this.__views__),
          start = view.start,
          end = view.end,
          length = end - start,
          index = isRight ? end : (start - 1),
          iteratees = this.__iteratees__,
          iterLength = iteratees.length,
          resIndex = 0,
          takeCount = nativeMin(length, this.__takeCount__);
      if (!isArr || arrLength < LARGE_ARRAY_SIZE || (arrLength == length && takeCount == length)) {
        return baseWrapperValue(array, this.__actions__);
      }
      var result = [];
      outer: while (length-- && resIndex < takeCount) {
        index += dir;
        var iterIndex = -1,
            value = array[index];
        while (++iterIndex < iterLength) {
          var data = iteratees[iterIndex],
              iteratee = data.iteratee,
              type = data.type,
              computed = iteratee(value);
          if (type == LAZY_MAP_FLAG) {
            value = computed;
          } else if (!computed) {
            if (type == LAZY_FILTER_FLAG) {
              continue outer;
            } else {
              break outer;
            }
          }
        }
        result[resIndex++] = value;
      }
      return result;
    }
    function Hash() {}
    function hashDelete(hash, key) {
      return hashHas(hash, key) && delete hash[key];
    }
    function hashGet(hash, key) {
      if (nativeCreate) {
        var result = hash[key];
        return result === HASH_UNDEFINED ? undefined : result;
      }
      return hasOwnProperty.call(hash, key) ? hash[key] : undefined;
    }
    function hashHas(hash, key) {
      return nativeCreate ? hash[key] !== undefined : hasOwnProperty.call(hash, key);
    }
    function hashSet(hash, key, value) {
      hash[key] = (nativeCreate && value === undefined) ? HASH_UNDEFINED : value;
    }
    function MapCache(values) {
      var index = -1,
          length = values ? values.length : 0;
      this.clear();
      while (++index < length) {
        var entry = values[index];
        this.set(entry[0], entry[1]);
      }
    }
    function mapClear() {
      this.__data__ = {
        'hash': new Hash,
        'map': Map ? new Map : [],
        'string': new Hash
      };
    }
    function mapDelete(key) {
      var data = this.__data__;
      if (isKeyable(key)) {
        return hashDelete(typeof key == 'string' ? data.string : data.hash, key);
      }
      return Map ? data.map['delete'](key) : assocDelete(data.map, key);
    }
    function mapGet(key) {
      var data = this.__data__;
      if (isKeyable(key)) {
        return hashGet(typeof key == 'string' ? data.string : data.hash, key);
      }
      return Map ? data.map.get(key) : assocGet(data.map, key);
    }
    function mapHas(key) {
      var data = this.__data__;
      if (isKeyable(key)) {
        return hashHas(typeof key == 'string' ? data.string : data.hash, key);
      }
      return Map ? data.map.has(key) : assocHas(data.map, key);
    }
    function mapSet(key, value) {
      var data = this.__data__;
      if (isKeyable(key)) {
        hashSet(typeof key == 'string' ? data.string : data.hash, key, value);
      } else if (Map) {
        data.map.set(key, value);
      } else {
        assocSet(data.map, key, value);
      }
      return this;
    }
    function SetCache(values) {
      var index = -1,
          length = values ? values.length : 0;
      this.__data__ = new MapCache;
      while (++index < length) {
        this.push(values[index]);
      }
    }
    function cacheHas(cache, value) {
      var map = cache.__data__;
      if (isKeyable(value)) {
        var data = map.__data__,
            hash = typeof value == 'string' ? data.string : data.hash;
        return hash[value] === HASH_UNDEFINED;
      }
      return map.has(value);
    }
    function cachePush(value) {
      var map = this.__data__;
      if (isKeyable(value)) {
        var data = map.__data__,
            hash = typeof value == 'string' ? data.string : data.hash;
        hash[value] = HASH_UNDEFINED;
      } else {
        map.set(value, HASH_UNDEFINED);
      }
    }
    function Stack(values) {
      var index = -1,
          length = values ? values.length : 0;
      this.clear();
      while (++index < length) {
        var entry = values[index];
        this.set(entry[0], entry[1]);
      }
    }
    function stackClear() {
      this.__data__ = {
        'array': [],
        'map': null
      };
    }
    function stackDelete(key) {
      var data = this.__data__,
          array = data.array;
      return array ? assocDelete(array, key) : data.map['delete'](key);
    }
    function stackGet(key) {
      var data = this.__data__,
          array = data.array;
      return array ? assocGet(array, key) : data.map.get(key);
    }
    function stackHas(key) {
      var data = this.__data__,
          array = data.array;
      return array ? assocHas(array, key) : data.map.has(key);
    }
    function stackSet(key, value) {
      var data = this.__data__,
          array = data.array;
      if (array) {
        if (array.length < (LARGE_ARRAY_SIZE - 1)) {
          assocSet(array, key, value);
        } else {
          data.array = null;
          data.map = new MapCache(array);
        }
      }
      var map = data.map;
      if (map) {
        map.set(key, value);
      }
      return this;
    }
    function assocDelete(array, key) {
      var index = assocIndexOf(array, key);
      if (index < 0) {
        return false;
      }
      var lastIndex = array.length - 1;
      if (index == lastIndex) {
        array.pop();
      } else {
        splice.call(array, index, 1);
      }
      return true;
    }
    function assocGet(array, key) {
      var index = assocIndexOf(array, key);
      return index < 0 ? undefined : array[index][1];
    }
    function assocHas(array, key) {
      return assocIndexOf(array, key) > -1;
    }
    function assocIndexOf(array, key) {
      var length = array.length;
      while (length--) {
        if (eq(array[length][0], key)) {
          return length;
        }
      }
      return -1;
    }
    function assocSet(array, key, value) {
      var index = assocIndexOf(array, key);
      if (index < 0) {
        array.push([key, value]);
      } else {
        array[index][1] = value;
      }
    }
    function assignInDefaults(objValue, srcValue, key, object) {
      if (objValue === undefined || (eq(objValue, objectProto[key]) && !hasOwnProperty.call(object, key))) {
        return srcValue;
      }
      return objValue;
    }
    function assignMergeValue(object, key, value) {
      if ((value !== undefined && !eq(object[key], value)) || (typeof key == 'number' && value === undefined && !(key in object))) {
        object[key] = value;
      }
    }
    function assignValue(object, key, value) {
      var objValue = object[key];
      if (!(hasOwnProperty.call(object, key) && eq(objValue, value)) || (value === undefined && !(key in object))) {
        object[key] = value;
      }
    }
    function baseAggregator(collection, setter, iteratee, accumulator) {
      baseEach(collection, function(value, key, collection) {
        setter(accumulator, value, iteratee(value), collection);
      });
      return accumulator;
    }
    function baseAssign(object, source) {
      return object && copyObject(source, keys(source), object);
    }
    function baseAt(object, paths) {
      var index = -1,
          isNil = object == null,
          length = paths.length,
          result = Array(length);
      while (++index < length) {
        result[index] = isNil ? undefined : get(object, paths[index]);
      }
      return result;
    }
    function baseCastArrayLikeObject(value) {
      return isArrayLikeObject(value) ? value : [];
    }
    function baseCastFunction(value) {
      return typeof value == 'function' ? value : identity;
    }
    function baseCastPath(value) {
      return isArray(value) ? value : stringToPath(value);
    }
    function baseClamp(number, lower, upper) {
      if (number === number) {
        if (upper !== undefined) {
          number = number <= upper ? number : upper;
        }
        if (lower !== undefined) {
          number = number >= lower ? number : lower;
        }
      }
      return number;
    }
    function baseClone(value, isDeep, customizer, key, object, stack) {
      var result;
      if (customizer) {
        result = object ? customizer(value, key, object, stack) : customizer(value);
      }
      if (result !== undefined) {
        return result;
      }
      if (!isObject(value)) {
        return value;
      }
      var isArr = isArray(value);
      if (isArr) {
        result = initCloneArray(value);
        if (!isDeep) {
          return copyArray(value, result);
        }
      } else {
        var tag = getTag(value),
            isFunc = tag == funcTag || tag == genTag;
        if (isBuffer(value)) {
          return cloneBuffer(value, isDeep);
        }
        if (tag == objectTag || tag == argsTag || (isFunc && !object)) {
          if (isHostObject(value)) {
            return object ? value : {};
          }
          result = initCloneObject(isFunc ? {} : value);
          if (!isDeep) {
            return copySymbols(value, baseAssign(result, value));
          }
        } else {
          if (!cloneableTags[tag]) {
            return object ? value : {};
          }
          result = initCloneByTag(value, tag, isDeep);
        }
      }
      stack || (stack = new Stack);
      var stacked = stack.get(value);
      if (stacked) {
        return stacked;
      }
      stack.set(value, result);
      (isArr ? arrayEach : baseForOwn)(value, function(subValue, key) {
        assignValue(result, key, baseClone(subValue, isDeep, customizer, key, value, stack));
      });
      return isArr ? result : copySymbols(value, result);
    }
    function baseConforms(source) {
      var props = keys(source),
          length = props.length;
      return function(object) {
        if (object == null) {
          return !length;
        }
        var index = length;
        while (index--) {
          var key = props[index],
              predicate = source[key],
              value = object[key];
          if ((value === undefined && !(key in Object(object))) || !predicate(value)) {
            return false;
          }
        }
        return true;
      };
    }
    function baseCreate(proto) {
      return isObject(proto) ? objectCreate(proto) : {};
    }
    function baseDelay(func, wait, args) {
      if (typeof func != 'function') {
        throw new TypeError(FUNC_ERROR_TEXT);
      }
      return setTimeout(function() {
        func.apply(undefined, args);
      }, wait);
    }
    function baseDifference(array, values, iteratee, comparator) {
      var index = -1,
          includes = arrayIncludes,
          isCommon = true,
          length = array.length,
          result = [],
          valuesLength = values.length;
      if (!length) {
        return result;
      }
      if (iteratee) {
        values = arrayMap(values, baseUnary(iteratee));
      }
      if (comparator) {
        includes = arrayIncludesWith;
        isCommon = false;
      } else if (values.length >= LARGE_ARRAY_SIZE) {
        includes = cacheHas;
        isCommon = false;
        values = new SetCache(values);
      }
      outer: while (++index < length) {
        var value = array[index],
            computed = iteratee ? iteratee(value) : value;
        if (isCommon && computed === computed) {
          var valuesIndex = valuesLength;
          while (valuesIndex--) {
            if (values[valuesIndex] === computed) {
              continue outer;
            }
          }
          result.push(value);
        } else if (!includes(values, computed, comparator)) {
          result.push(value);
        }
      }
      return result;
    }
    var baseEach = createBaseEach(baseForOwn);
    var baseEachRight = createBaseEach(baseForOwnRight, true);
    function baseEvery(collection, predicate) {
      var result = true;
      baseEach(collection, function(value, index, collection) {
        result = !!predicate(value, index, collection);
        return result;
      });
      return result;
    }
    function baseFill(array, value, start, end) {
      var length = array.length;
      start = toInteger(start);
      if (start < 0) {
        start = -start > length ? 0 : (length + start);
      }
      end = (end === undefined || end > length) ? length : toInteger(end);
      if (end < 0) {
        end += length;
      }
      end = start > end ? 0 : toLength(end);
      while (start < end) {
        array[start++] = value;
      }
      return array;
    }
    function baseFilter(collection, predicate) {
      var result = [];
      baseEach(collection, function(value, index, collection) {
        if (predicate(value, index, collection)) {
          result.push(value);
        }
      });
      return result;
    }
    function baseFlatten(array, depth, isStrict, result) {
      result || (result = []);
      var index = -1,
          length = array.length;
      while (++index < length) {
        var value = array[index];
        if (depth > 0 && isArrayLikeObject(value) && (isStrict || isArray(value) || isArguments(value))) {
          if (depth > 1) {
            baseFlatten(value, depth - 1, isStrict, result);
          } else {
            arrayPush(result, value);
          }
        } else if (!isStrict) {
          result[result.length] = value;
        }
      }
      return result;
    }
    var baseFor = createBaseFor();
    var baseForRight = createBaseFor(true);
    function baseForIn(object, iteratee) {
      return object == null ? object : baseFor(object, iteratee, keysIn);
    }
    function baseForOwn(object, iteratee) {
      return object && baseFor(object, iteratee, keys);
    }
    function baseForOwnRight(object, iteratee) {
      return object && baseForRight(object, iteratee, keys);
    }
    function baseFunctions(object, props) {
      return arrayFilter(props, function(key) {
        return isFunction(object[key]);
      });
    }
    function baseGet(object, path) {
      path = isKey(path, object) ? [path + ''] : baseCastPath(path);
      var index = 0,
          length = path.length;
      while (object != null && index < length) {
        object = object[path[index++]];
      }
      return (index && index == length) ? object : undefined;
    }
    function baseHas(object, key) {
      return hasOwnProperty.call(object, key) || (typeof object == 'object' && key in object && getPrototypeOf(object) === null);
    }
    function baseHasIn(object, key) {
      return key in Object(object);
    }
    function baseInRange(number, start, end) {
      return number >= nativeMin(start, end) && number < nativeMax(start, end);
    }
    function baseIntersection(arrays, iteratee, comparator) {
      var includes = comparator ? arrayIncludesWith : arrayIncludes,
          othLength = arrays.length,
          othIndex = othLength,
          caches = Array(othLength),
          result = [];
      while (othIndex--) {
        var array = arrays[othIndex];
        if (othIndex && iteratee) {
          array = arrayMap(array, baseUnary(iteratee));
        }
        caches[othIndex] = !comparator && (iteratee || array.length >= 120) ? new SetCache(othIndex && array) : undefined;
      }
      array = arrays[0];
      var index = -1,
          length = array.length,
          seen = caches[0];
      outer: while (++index < length) {
        var value = array[index],
            computed = iteratee ? iteratee(value) : value;
        if (!(seen ? cacheHas(seen, computed) : includes(result, computed, comparator))) {
          var othIndex = othLength;
          while (--othIndex) {
            var cache = caches[othIndex];
            if (!(cache ? cacheHas(cache, computed) : includes(arrays[othIndex], computed, comparator))) {
              continue outer;
            }
          }
          if (seen) {
            seen.push(computed);
          }
          result.push(value);
        }
      }
      return result;
    }
    function baseInverter(object, setter, iteratee, accumulator) {
      baseForOwn(object, function(value, key, object) {
        setter(accumulator, iteratee(value), key, object);
      });
      return accumulator;
    }
    function baseInvoke(object, path, args) {
      if (!isKey(path, object)) {
        path = baseCastPath(path);
        object = parent(object, path);
        path = last(path);
      }
      var func = object == null ? object : object[path];
      return func == null ? undefined : apply(func, object, args);
    }
    function baseIsEqual(value, other, customizer, bitmask, stack) {
      if (value === other) {
        return true;
      }
      if (value == null || other == null || (!isObject(value) && !isObjectLike(other))) {
        return value !== value && other !== other;
      }
      return baseIsEqualDeep(value, other, baseIsEqual, customizer, bitmask, stack);
    }
    function baseIsEqualDeep(object, other, equalFunc, customizer, bitmask, stack) {
      var objIsArr = isArray(object),
          othIsArr = isArray(other),
          objTag = arrayTag,
          othTag = arrayTag;
      if (!objIsArr) {
        objTag = getTag(object);
        if (objTag == argsTag) {
          objTag = objectTag;
        } else if (objTag != objectTag) {
          objIsArr = isTypedArray(object);
        }
      }
      if (!othIsArr) {
        othTag = getTag(other);
        if (othTag == argsTag) {
          othTag = objectTag;
        } else if (othTag != objectTag) {
          othIsArr = isTypedArray(other);
        }
      }
      var objIsObj = objTag == objectTag && !isHostObject(object),
          othIsObj = othTag == objectTag && !isHostObject(other),
          isSameTag = objTag == othTag;
      if (isSameTag && !(objIsArr || objIsObj)) {
        return equalByTag(object, other, objTag, equalFunc, customizer, bitmask);
      }
      var isPartial = bitmask & PARTIAL_COMPARE_FLAG;
      if (!isPartial) {
        var objIsWrapped = objIsObj && hasOwnProperty.call(object, '__wrapped__'),
            othIsWrapped = othIsObj && hasOwnProperty.call(other, '__wrapped__');
        if (objIsWrapped || othIsWrapped) {
          return equalFunc(objIsWrapped ? object.value() : object, othIsWrapped ? other.value() : other, customizer, bitmask, stack);
        }
      }
      if (!isSameTag) {
        return false;
      }
      stack || (stack = new Stack);
      return (objIsArr ? equalArrays : equalObjects)(object, other, equalFunc, customizer, bitmask, stack);
    }
    function baseIsMatch(object, source, matchData, customizer) {
      var index = matchData.length,
          length = index,
          noCustomizer = !customizer;
      if (object == null) {
        return !length;
      }
      object = Object(object);
      while (index--) {
        var data = matchData[index];
        if ((noCustomizer && data[2]) ? data[1] !== object[data[0]] : !(data[0] in object)) {
          return false;
        }
      }
      while (++index < length) {
        data = matchData[index];
        var key = data[0],
            objValue = object[key],
            srcValue = data[1];
        if (noCustomizer && data[2]) {
          if (objValue === undefined && !(key in object)) {
            return false;
          }
        } else {
          var stack = new Stack,
              result = customizer ? customizer(objValue, srcValue, key, object, source, stack) : undefined;
          if (!(result === undefined ? baseIsEqual(srcValue, objValue, customizer, UNORDERED_COMPARE_FLAG | PARTIAL_COMPARE_FLAG, stack) : result)) {
            return false;
          }
        }
      }
      return true;
    }
    function baseIteratee(value) {
      var type = typeof value;
      if (type == 'function') {
        return value;
      }
      if (value == null) {
        return identity;
      }
      if (type == 'object') {
        return isArray(value) ? baseMatchesProperty(value[0], value[1]) : baseMatches(value);
      }
      return property(value);
    }
    function baseKeys(object) {
      return nativeKeys(Object(object));
    }
    function baseKeysIn(object) {
      object = object == null ? object : Object(object);
      var result = [];
      for (var key in object) {
        result.push(key);
      }
      return result;
    }
    if (enumerate && !propertyIsEnumerable.call({'valueOf': 1}, 'valueOf')) {
      baseKeysIn = function(object) {
        return iteratorToArray(enumerate(object));
      };
    }
    function baseMap(collection, iteratee) {
      var index = -1,
          result = isArrayLike(collection) ? Array(collection.length) : [];
      baseEach(collection, function(value, key, collection) {
        result[++index] = iteratee(value, key, collection);
      });
      return result;
    }
    function baseMatches(source) {
      var matchData = getMatchData(source);
      if (matchData.length == 1 && matchData[0][2]) {
        var key = matchData[0][0],
            value = matchData[0][1];
        return function(object) {
          if (object == null) {
            return false;
          }
          return object[key] === value && (value !== undefined || (key in Object(object)));
        };
      }
      return function(object) {
        return object === source || baseIsMatch(object, source, matchData);
      };
    }
    function baseMatchesProperty(path, srcValue) {
      return function(object) {
        var objValue = get(object, path);
        return (objValue === undefined && objValue === srcValue) ? hasIn(object, path) : baseIsEqual(srcValue, objValue, undefined, UNORDERED_COMPARE_FLAG | PARTIAL_COMPARE_FLAG);
      };
    }
    function baseMerge(object, source, srcIndex, customizer, stack) {
      if (object === source) {
        return;
      }
      var props = (isArray(source) || isTypedArray(source)) ? undefined : keysIn(source);
      arrayEach(props || source, function(srcValue, key) {
        if (props) {
          key = srcValue;
          srcValue = source[key];
        }
        if (isObject(srcValue)) {
          stack || (stack = new Stack);
          baseMergeDeep(object, source, key, srcIndex, baseMerge, customizer, stack);
        } else {
          var newValue = customizer ? customizer(object[key], srcValue, (key + ''), object, source, stack) : undefined;
          if (newValue === undefined) {
            newValue = srcValue;
          }
          assignMergeValue(object, key, newValue);
        }
      });
    }
    function baseMergeDeep(object, source, key, srcIndex, mergeFunc, customizer, stack) {
      var objValue = object[key],
          srcValue = source[key],
          stacked = stack.get(srcValue);
      if (stacked) {
        assignMergeValue(object, key, stacked);
        return;
      }
      var newValue = customizer ? customizer(objValue, srcValue, (key + ''), object, source, stack) : undefined;
      var isCommon = newValue === undefined;
      if (isCommon) {
        newValue = srcValue;
        if (isArray(srcValue) || isTypedArray(srcValue)) {
          if (isArray(objValue)) {
            newValue = objValue;
          } else if (isArrayLikeObject(objValue)) {
            newValue = copyArray(objValue);
          } else {
            isCommon = false;
            newValue = baseClone(srcValue, true);
          }
        } else if (isPlainObject(srcValue) || isArguments(srcValue)) {
          if (isArguments(objValue)) {
            newValue = toPlainObject(objValue);
          } else if (!isObject(objValue) || (srcIndex && isFunction(objValue))) {
            isCommon = false;
            newValue = baseClone(srcValue, true);
          } else {
            newValue = objValue;
          }
        } else {
          isCommon = false;
        }
      }
      stack.set(srcValue, newValue);
      if (isCommon) {
        mergeFunc(newValue, srcValue, srcIndex, customizer, stack);
      }
      assignMergeValue(object, key, newValue);
    }
    function baseOrderBy(collection, iteratees, orders) {
      var index = -1,
          toIteratee = getIteratee();
      iteratees = arrayMap(iteratees.length ? iteratees : Array(1), function(iteratee) {
        return toIteratee(iteratee);
      });
      var result = baseMap(collection, function(value, key, collection) {
        var criteria = arrayMap(iteratees, function(iteratee) {
          return iteratee(value);
        });
        return {
          'criteria': criteria,
          'index': ++index,
          'value': value
        };
      });
      return baseSortBy(result, function(object, other) {
        return compareMultiple(object, other, orders);
      });
    }
    function basePick(object, props) {
      object = Object(object);
      return arrayReduce(props, function(result, key) {
        if (key in object) {
          result[key] = object[key];
        }
        return result;
      }, {});
    }
    function basePickBy(object, predicate) {
      var result = {};
      baseForIn(object, function(value, key) {
        if (predicate(value, key)) {
          result[key] = value;
        }
      });
      return result;
    }
    function baseProperty(key) {
      return function(object) {
        return object == null ? undefined : object[key];
      };
    }
    function basePropertyDeep(path) {
      return function(object) {
        return baseGet(object, path);
      };
    }
    function basePullAll(array, values) {
      return basePullAllBy(array, values);
    }
    function basePullAllBy(array, values, iteratee) {
      var index = -1,
          length = values.length,
          seen = array;
      if (iteratee) {
        seen = arrayMap(array, function(value) {
          return iteratee(value);
        });
      }
      while (++index < length) {
        var fromIndex = 0,
            value = values[index],
            computed = iteratee ? iteratee(value) : value;
        while ((fromIndex = baseIndexOf(seen, computed, fromIndex)) > -1) {
          if (seen !== array) {
            splice.call(seen, fromIndex, 1);
          }
          splice.call(array, fromIndex, 1);
        }
      }
      return array;
    }
    function basePullAt(array, indexes) {
      var length = array ? indexes.length : 0,
          lastIndex = length - 1;
      while (length--) {
        var index = indexes[length];
        if (lastIndex == length || index != previous) {
          var previous = index;
          if (isIndex(index)) {
            splice.call(array, index, 1);
          } else if (!isKey(index, array)) {
            var path = baseCastPath(index),
                object = parent(array, path);
            if (object != null) {
              delete object[last(path)];
            }
          } else {
            delete array[index];
          }
        }
      }
      return array;
    }
    function baseRandom(lower, upper) {
      return lower + nativeFloor(nativeRandom() * (upper - lower + 1));
    }
    function baseRange(start, end, step, fromRight) {
      var index = -1,
          length = nativeMax(nativeCeil((end - start) / (step || 1)), 0),
          result = Array(length);
      while (length--) {
        result[fromRight ? length : ++index] = start;
        start += step;
      }
      return result;
    }
    function baseSet(object, path, value, customizer) {
      path = isKey(path, object) ? [path + ''] : baseCastPath(path);
      var index = -1,
          length = path.length,
          lastIndex = length - 1,
          nested = object;
      while (nested != null && ++index < length) {
        var key = path[index];
        if (isObject(nested)) {
          var newValue = value;
          if (index != lastIndex) {
            var objValue = nested[key];
            newValue = customizer ? customizer(objValue, key, nested) : undefined;
            if (newValue === undefined) {
              newValue = objValue == null ? (isIndex(path[index + 1]) ? [] : {}) : objValue;
            }
          }
          assignValue(nested, key, newValue);
        }
        nested = nested[key];
      }
      return object;
    }
    var baseSetData = !metaMap ? identity : function(func, data) {
      metaMap.set(func, data);
      return func;
    };
    function baseSlice(array, start, end) {
      var index = -1,
          length = array.length;
      if (start < 0) {
        start = -start > length ? 0 : (length + start);
      }
      end = end > length ? length : end;
      if (end < 0) {
        end += length;
      }
      length = start > end ? 0 : ((end - start) >>> 0);
      start >>>= 0;
      var result = Array(length);
      while (++index < length) {
        result[index] = array[index + start];
      }
      return result;
    }
    function baseSome(collection, predicate) {
      var result;
      baseEach(collection, function(value, index, collection) {
        result = predicate(value, index, collection);
        return !result;
      });
      return !!result;
    }
    function baseSortedIndex(array, value, retHighest) {
      var low = 0,
          high = array ? array.length : low;
      if (typeof value == 'number' && value === value && high <= HALF_MAX_ARRAY_LENGTH) {
        while (low < high) {
          var mid = (low + high) >>> 1,
              computed = array[mid];
          if ((retHighest ? (computed <= value) : (computed < value)) && computed !== null) {
            low = mid + 1;
          } else {
            high = mid;
          }
        }
        return high;
      }
      return baseSortedIndexBy(array, value, identity, retHighest);
    }
    function baseSortedIndexBy(array, value, iteratee, retHighest) {
      value = iteratee(value);
      var low = 0,
          high = array ? array.length : 0,
          valIsNaN = value !== value,
          valIsNull = value === null,
          valIsUndef = value === undefined;
      while (low < high) {
        var mid = nativeFloor((low + high) / 2),
            computed = iteratee(array[mid]),
            isDef = computed !== undefined,
            isReflexive = computed === computed;
        if (valIsNaN) {
          var setLow = isReflexive || retHighest;
        } else if (valIsNull) {
          setLow = isReflexive && isDef && (retHighest || computed != null);
        } else if (valIsUndef) {
          setLow = isReflexive && (retHighest || isDef);
        } else if (computed == null) {
          setLow = false;
        } else {
          setLow = retHighest ? (computed <= value) : (computed < value);
        }
        if (setLow) {
          low = mid + 1;
        } else {
          high = mid;
        }
      }
      return nativeMin(high, MAX_ARRAY_INDEX);
    }
    function baseSortedUniq(array) {
      return baseSortedUniqBy(array);
    }
    function baseSortedUniqBy(array, iteratee) {
      var index = 0,
          length = array.length,
          value = array[0],
          computed = iteratee ? iteratee(value) : value,
          seen = computed,
          resIndex = 0,
          result = [value];
      while (++index < length) {
        value = array[index], computed = iteratee ? iteratee(value) : value;
        if (!eq(computed, seen)) {
          seen = computed;
          result[++resIndex] = value;
        }
      }
      return result;
    }
    function baseUniq(array, iteratee, comparator) {
      var index = -1,
          includes = arrayIncludes,
          length = array.length,
          isCommon = true,
          result = [],
          seen = result;
      if (comparator) {
        isCommon = false;
        includes = arrayIncludesWith;
      } else if (length >= LARGE_ARRAY_SIZE) {
        var set = iteratee ? null : createSet(array);
        if (set) {
          return setToArray(set);
        }
        isCommon = false;
        includes = cacheHas;
        seen = new SetCache;
      } else {
        seen = iteratee ? [] : result;
      }
      outer: while (++index < length) {
        var value = array[index],
            computed = iteratee ? iteratee(value) : value;
        if (isCommon && computed === computed) {
          var seenIndex = seen.length;
          while (seenIndex--) {
            if (seen[seenIndex] === computed) {
              continue outer;
            }
          }
          if (iteratee) {
            seen.push(computed);
          }
          result.push(value);
        } else if (!includes(seen, computed, comparator)) {
          if (seen !== result) {
            seen.push(computed);
          }
          result.push(value);
        }
      }
      return result;
    }
    function baseUnset(object, path) {
      path = isKey(path, object) ? [path + ''] : baseCastPath(path);
      object = parent(object, path);
      var key = last(path);
      return (object != null && has(object, key)) ? delete object[key] : true;
    }
    function baseWhile(array, predicate, isDrop, fromRight) {
      var length = array.length,
          index = fromRight ? length : -1;
      while ((fromRight ? index-- : ++index < length) && predicate(array[index], index, array)) {}
      return isDrop ? baseSlice(array, (fromRight ? 0 : index), (fromRight ? index + 1 : length)) : baseSlice(array, (fromRight ? index + 1 : 0), (fromRight ? length : index));
    }
    function baseWrapperValue(value, actions) {
      var result = value;
      if (result instanceof LazyWrapper) {
        result = result.value();
      }
      return arrayReduce(actions, function(result, action) {
        return action.func.apply(action.thisArg, arrayPush([result], action.args));
      }, result);
    }
    function baseXor(arrays, iteratee, comparator) {
      var index = -1,
          length = arrays.length;
      while (++index < length) {
        var result = result ? arrayPush(baseDifference(result, arrays[index], iteratee, comparator), baseDifference(arrays[index], result, iteratee, comparator)) : arrays[index];
      }
      return (result && result.length) ? baseUniq(result, iteratee, comparator) : [];
    }
    function baseZipObject(props, values, assignFunc) {
      var index = -1,
          length = props.length,
          valsLength = values.length,
          result = {};
      while (++index < length) {
        assignFunc(result, props[index], index < valsLength ? values[index] : undefined);
      }
      return result;
    }
    function cloneBuffer(buffer, isDeep) {
      if (isDeep) {
        return buffer.slice();
      }
      var Ctor = buffer.constructor,
          result = new Ctor(buffer.length);
      buffer.copy(result);
      return result;
    }
    function cloneArrayBuffer(arrayBuffer) {
      var Ctor = arrayBuffer.constructor,
          result = new Ctor(arrayBuffer.byteLength),
          view = new Uint8Array(result);
      view.set(new Uint8Array(arrayBuffer));
      return result;
    }
    function cloneMap(map) {
      var Ctor = map.constructor;
      return arrayReduce(mapToArray(map), addMapEntry, new Ctor);
    }
    function cloneRegExp(regexp) {
      var Ctor = regexp.constructor,
          result = new Ctor(regexp.source, reFlags.exec(regexp));
      result.lastIndex = regexp.lastIndex;
      return result;
    }
    function cloneSet(set) {
      var Ctor = set.constructor;
      return arrayReduce(setToArray(set), addSetEntry, new Ctor);
    }
    function cloneSymbol(symbol) {
      return Symbol ? Object(symbolValueOf.call(symbol)) : {};
    }
    function cloneTypedArray(typedArray, isDeep) {
      var arrayBuffer = typedArray.buffer,
          buffer = isDeep ? cloneArrayBuffer(arrayBuffer) : arrayBuffer,
          Ctor = typedArray.constructor;
      return new Ctor(buffer, typedArray.byteOffset, typedArray.length);
    }
    function composeArgs(args, partials, holders, isCurried) {
      var argsIndex = -1,
          argsLength = args.length,
          holdersLength = holders.length,
          leftIndex = -1,
          leftLength = partials.length,
          rangeLength = nativeMax(argsLength - holdersLength, 0),
          result = Array(leftLength + rangeLength),
          isUncurried = !isCurried;
      while (++leftIndex < leftLength) {
        result[leftIndex] = partials[leftIndex];
      }
      while (++argsIndex < holdersLength) {
        if (isUncurried || argsIndex < argsLength) {
          result[holders[argsIndex]] = args[argsIndex];
        }
      }
      while (rangeLength--) {
        result[leftIndex++] = args[argsIndex++];
      }
      return result;
    }
    function composeArgsRight(args, partials, holders, isCurried) {
      var argsIndex = -1,
          argsLength = args.length,
          holdersIndex = -1,
          holdersLength = holders.length,
          rightIndex = -1,
          rightLength = partials.length,
          rangeLength = nativeMax(argsLength - holdersLength, 0),
          result = Array(rangeLength + rightLength),
          isUncurried = !isCurried;
      while (++argsIndex < rangeLength) {
        result[argsIndex] = args[argsIndex];
      }
      var offset = argsIndex;
      while (++rightIndex < rightLength) {
        result[offset + rightIndex] = partials[rightIndex];
      }
      while (++holdersIndex < holdersLength) {
        if (isUncurried || argsIndex < argsLength) {
          result[offset + holders[holdersIndex]] = args[argsIndex++];
        }
      }
      return result;
    }
    function copyArray(source, array) {
      var index = -1,
          length = source.length;
      array || (array = Array(length));
      while (++index < length) {
        array[index] = source[index];
      }
      return array;
    }
    function copyObject(source, props, object) {
      return copyObjectWith(source, props, object);
    }
    function copyObjectWith(source, props, object, customizer) {
      object || (object = {});
      var index = -1,
          length = props.length;
      while (++index < length) {
        var key = props[index];
        var newValue = customizer ? customizer(object[key], source[key], key, object, source) : source[key];
        assignValue(object, key, newValue);
      }
      return object;
    }
    function copySymbols(source, object) {
      return copyObject(source, getSymbols(source), object);
    }
    function createAggregator(setter, initializer) {
      return function(collection, iteratee) {
        var func = isArray(collection) ? arrayAggregator : baseAggregator,
            accumulator = initializer ? initializer() : {};
        return func(collection, setter, getIteratee(iteratee), accumulator);
      };
    }
    function createAssigner(assigner) {
      return rest(function(object, sources) {
        var index = -1,
            length = sources.length,
            customizer = length > 1 ? sources[length - 1] : undefined,
            guard = length > 2 ? sources[2] : undefined;
        customizer = typeof customizer == 'function' ? (length--, customizer) : undefined;
        if (guard && isIterateeCall(sources[0], sources[1], guard)) {
          customizer = length < 3 ? undefined : customizer;
          length = 1;
        }
        object = Object(object);
        while (++index < length) {
          var source = sources[index];
          if (source) {
            assigner(object, source, index, customizer);
          }
        }
        return object;
      });
    }
    function createBaseEach(eachFunc, fromRight) {
      return function(collection, iteratee) {
        if (collection == null) {
          return collection;
        }
        if (!isArrayLike(collection)) {
          return eachFunc(collection, iteratee);
        }
        var length = collection.length,
            index = fromRight ? length : -1,
            iterable = Object(collection);
        while ((fromRight ? index-- : ++index < length)) {
          if (iteratee(iterable[index], index, iterable) === false) {
            break;
          }
        }
        return collection;
      };
    }
    function createBaseFor(fromRight) {
      return function(object, iteratee, keysFunc) {
        var index = -1,
            iterable = Object(object),
            props = keysFunc(object),
            length = props.length;
        while (length--) {
          var key = props[fromRight ? length : ++index];
          if (iteratee(iterable[key], key, iterable) === false) {
            break;
          }
        }
        return object;
      };
    }
    function createBaseWrapper(func, bitmask, thisArg) {
      var isBind = bitmask & BIND_FLAG,
          Ctor = createCtorWrapper(func);
      function wrapper() {
        var fn = (this && this !== root && this instanceof wrapper) ? Ctor : func;
        return fn.apply(isBind ? thisArg : this, arguments);
      }
      return wrapper;
    }
    function createCaseFirst(methodName) {
      return function(string) {
        string = toString(string);
        var strSymbols = reHasComplexSymbol.test(string) ? stringToArray(string) : undefined;
        var chr = strSymbols ? strSymbols[0] : string.charAt(0),
            trailing = strSymbols ? strSymbols.slice(1).join('') : string.slice(1);
        return chr[methodName]() + trailing;
      };
    }
    function createCompounder(callback) {
      return function(string) {
        return arrayReduce(words(deburr(string)), callback, '');
      };
    }
    function createCtorWrapper(Ctor) {
      return function() {
        var args = arguments;
        switch (args.length) {
          case 0:
            return new Ctor;
          case 1:
            return new Ctor(args[0]);
          case 2:
            return new Ctor(args[0], args[1]);
          case 3:
            return new Ctor(args[0], args[1], args[2]);
          case 4:
            return new Ctor(args[0], args[1], args[2], args[3]);
          case 5:
            return new Ctor(args[0], args[1], args[2], args[3], args[4]);
          case 6:
            return new Ctor(args[0], args[1], args[2], args[3], args[4], args[5]);
          case 7:
            return new Ctor(args[0], args[1], args[2], args[3], args[4], args[5], args[6]);
        }
        var thisBinding = baseCreate(Ctor.prototype),
            result = Ctor.apply(thisBinding, args);
        return isObject(result) ? result : thisBinding;
      };
    }
    function createCurryWrapper(func, bitmask, arity) {
      var Ctor = createCtorWrapper(func);
      function wrapper() {
        var length = arguments.length,
            args = Array(length),
            index = length,
            placeholder = getPlaceholder(wrapper);
        while (index--) {
          args[index] = arguments[index];
        }
        var holders = (length < 3 && args[0] !== placeholder && args[length - 1] !== placeholder) ? [] : replaceHolders(args, placeholder);
        length -= holders.length;
        if (length < arity) {
          return createRecurryWrapper(func, bitmask, createHybridWrapper, wrapper.placeholder, undefined, args, holders, undefined, undefined, arity - length);
        }
        var fn = (this && this !== root && this instanceof wrapper) ? Ctor : func;
        return apply(fn, this, args);
      }
      return wrapper;
    }
    function createFlow(fromRight) {
      return rest(function(funcs) {
        funcs = baseFlatten(funcs, 1);
        var length = funcs.length,
            index = length,
            prereq = LodashWrapper.prototype.thru;
        if (fromRight) {
          funcs.reverse();
        }
        while (index--) {
          var func = funcs[index];
          if (typeof func != 'function') {
            throw new TypeError(FUNC_ERROR_TEXT);
          }
          if (prereq && !wrapper && getFuncName(func) == 'wrapper') {
            var wrapper = new LodashWrapper([], true);
          }
        }
        index = wrapper ? index : length;
        while (++index < length) {
          func = funcs[index];
          var funcName = getFuncName(func),
              data = funcName == 'wrapper' ? getData(func) : undefined;
          if (data && isLaziable(data[0]) && data[1] == (ARY_FLAG | CURRY_FLAG | PARTIAL_FLAG | REARG_FLAG) && !data[4].length && data[9] == 1) {
            wrapper = wrapper[getFuncName(data[0])].apply(wrapper, data[3]);
          } else {
            wrapper = (func.length == 1 && isLaziable(func)) ? wrapper[funcName]() : wrapper.thru(func);
          }
        }
        return function() {
          var args = arguments,
              value = args[0];
          if (wrapper && args.length == 1 && isArray(value) && value.length >= LARGE_ARRAY_SIZE) {
            return wrapper.plant(value).value();
          }
          var index = 0,
              result = length ? funcs[index].apply(this, args) : value;
          while (++index < length) {
            result = funcs[index].call(this, result);
          }
          return result;
        };
      });
    }
    function createHybridWrapper(func, bitmask, thisArg, partials, holders, partialsRight, holdersRight, argPos, ary, arity) {
      var isAry = bitmask & ARY_FLAG,
          isBind = bitmask & BIND_FLAG,
          isBindKey = bitmask & BIND_KEY_FLAG,
          isCurried = bitmask & (CURRY_FLAG | CURRY_RIGHT_FLAG),
          isFlip = bitmask & FLIP_FLAG,
          Ctor = isBindKey ? undefined : createCtorWrapper(func);
      function wrapper() {
        var length = arguments.length,
            index = length,
            args = Array(length);
        while (index--) {
          args[index] = arguments[index];
        }
        if (isCurried) {
          var placeholder = getPlaceholder(wrapper),
              holdersCount = countHolders(args, placeholder);
        }
        if (partials) {
          args = composeArgs(args, partials, holders, isCurried);
        }
        if (partialsRight) {
          args = composeArgsRight(args, partialsRight, holdersRight, isCurried);
        }
        length -= holdersCount;
        if (isCurried && length < arity) {
          var newHolders = replaceHolders(args, placeholder);
          return createRecurryWrapper(func, bitmask, createHybridWrapper, wrapper.placeholder, thisArg, args, newHolders, argPos, ary, arity - length);
        }
        var thisBinding = isBind ? thisArg : this,
            fn = isBindKey ? thisBinding[func] : func;
        length = args.length;
        if (argPos) {
          args = reorder(args, argPos);
        } else if (isFlip && length > 1) {
          args.reverse();
        }
        if (isAry && ary < length) {
          args.length = ary;
        }
        if (this && this !== root && this instanceof wrapper) {
          fn = Ctor || createCtorWrapper(fn);
        }
        return fn.apply(thisBinding, args);
      }
      return wrapper;
    }
    function createInverter(setter, toIteratee) {
      return function(object, iteratee) {
        return baseInverter(object, setter, toIteratee(iteratee), {});
      };
    }
    function createOver(arrayFunc) {
      return rest(function(iteratees) {
        iteratees = arrayMap(baseFlatten(iteratees, 1), getIteratee());
        return rest(function(args) {
          var thisArg = this;
          return arrayFunc(iteratees, function(iteratee) {
            return apply(iteratee, thisArg, args);
          });
        });
      });
    }
    function createPadding(string, length, chars) {
      length = toInteger(length);
      var strLength = stringSize(string);
      if (!length || strLength >= length) {
        return '';
      }
      var padLength = length - strLength;
      chars = chars === undefined ? ' ' : (chars + '');
      var result = repeat(chars, nativeCeil(padLength / stringSize(chars)));
      return reHasComplexSymbol.test(chars) ? stringToArray(result).slice(0, padLength).join('') : result.slice(0, padLength);
    }
    function createPartialWrapper(func, bitmask, thisArg, partials) {
      var isBind = bitmask & BIND_FLAG,
          Ctor = createCtorWrapper(func);
      function wrapper() {
        var argsIndex = -1,
            argsLength = arguments.length,
            leftIndex = -1,
            leftLength = partials.length,
            args = Array(leftLength + argsLength),
            fn = (this && this !== root && this instanceof wrapper) ? Ctor : func;
        while (++leftIndex < leftLength) {
          args[leftIndex] = partials[leftIndex];
        }
        while (argsLength--) {
          args[leftIndex++] = arguments[++argsIndex];
        }
        return apply(fn, isBind ? thisArg : this, args);
      }
      return wrapper;
    }
    function createRange(fromRight) {
      return function(start, end, step) {
        if (step && typeof step != 'number' && isIterateeCall(start, end, step)) {
          end = step = undefined;
        }
        start = toNumber(start);
        start = start === start ? start : 0;
        if (end === undefined) {
          end = start;
          start = 0;
        } else {
          end = toNumber(end) || 0;
        }
        step = step === undefined ? (start < end ? 1 : -1) : (toNumber(step) || 0);
        return baseRange(start, end, step, fromRight);
      };
    }
    function createRecurryWrapper(func, bitmask, wrapFunc, placeholder, thisArg, partials, holders, argPos, ary, arity) {
      var isCurry = bitmask & CURRY_FLAG,
          newArgPos = argPos ? copyArray(argPos) : undefined,
          newHolders = isCurry ? holders : undefined,
          newHoldersRight = isCurry ? undefined : holders,
          newPartials = isCurry ? partials : undefined,
          newPartialsRight = isCurry ? undefined : partials;
      bitmask |= (isCurry ? PARTIAL_FLAG : PARTIAL_RIGHT_FLAG);
      bitmask &= ~(isCurry ? PARTIAL_RIGHT_FLAG : PARTIAL_FLAG);
      if (!(bitmask & CURRY_BOUND_FLAG)) {
        bitmask &= ~(BIND_FLAG | BIND_KEY_FLAG);
      }
      var newData = [func, bitmask, thisArg, newPartials, newHolders, newPartialsRight, newHoldersRight, newArgPos, ary, arity];
      var result = wrapFunc.apply(undefined, newData);
      if (isLaziable(func)) {
        setData(result, newData);
      }
      result.placeholder = placeholder;
      return result;
    }
    function createRound(methodName) {
      var func = Math[methodName];
      return function(number, precision) {
        number = toNumber(number);
        precision = toInteger(precision);
        if (precision) {
          var pair = (toString(number) + 'e').split('e'),
              value = func(pair[0] + 'e' + (+pair[1] + precision));
          pair = (toString(value) + 'e').split('e');
          return +(pair[0] + 'e' + (+pair[1] - precision));
        }
        return func(number);
      };
    }
    var createSet = !(Set && new Set([1, 2]).size === 2) ? noop : function(values) {
      return new Set(values);
    };
    function createWrapper(func, bitmask, thisArg, partials, holders, argPos, ary, arity) {
      var isBindKey = bitmask & BIND_KEY_FLAG;
      if (!isBindKey && typeof func != 'function') {
        throw new TypeError(FUNC_ERROR_TEXT);
      }
      var length = partials ? partials.length : 0;
      if (!length) {
        bitmask &= ~(PARTIAL_FLAG | PARTIAL_RIGHT_FLAG);
        partials = holders = undefined;
      }
      ary = ary === undefined ? ary : nativeMax(toInteger(ary), 0);
      arity = arity === undefined ? arity : toInteger(arity);
      length -= holders ? holders.length : 0;
      if (bitmask & PARTIAL_RIGHT_FLAG) {
        var partialsRight = partials,
            holdersRight = holders;
        partials = holders = undefined;
      }
      var data = isBindKey ? undefined : getData(func);
      var newData = [func, bitmask, thisArg, partials, holders, partialsRight, holdersRight, argPos, ary, arity];
      if (data) {
        mergeData(newData, data);
      }
      func = newData[0];
      bitmask = newData[1];
      thisArg = newData[2];
      partials = newData[3];
      holders = newData[4];
      arity = newData[9] = newData[9] == null ? (isBindKey ? 0 : func.length) : nativeMax(newData[9] - length, 0);
      if (!arity && bitmask & (CURRY_FLAG | CURRY_RIGHT_FLAG)) {
        bitmask &= ~(CURRY_FLAG | CURRY_RIGHT_FLAG);
      }
      if (!bitmask || bitmask == BIND_FLAG) {
        var result = createBaseWrapper(func, bitmask, thisArg);
      } else if (bitmask == CURRY_FLAG || bitmask == CURRY_RIGHT_FLAG) {
        result = createCurryWrapper(func, bitmask, arity);
      } else if ((bitmask == PARTIAL_FLAG || bitmask == (BIND_FLAG | PARTIAL_FLAG)) && !holders.length) {
        result = createPartialWrapper(func, bitmask, thisArg, partials);
      } else {
        result = createHybridWrapper.apply(undefined, newData);
      }
      var setter = data ? baseSetData : setData;
      return setter(result, newData);
    }
    function equalArrays(array, other, equalFunc, customizer, bitmask, stack) {
      var index = -1,
          isPartial = bitmask & PARTIAL_COMPARE_FLAG,
          isUnordered = bitmask & UNORDERED_COMPARE_FLAG,
          arrLength = array.length,
          othLength = other.length;
      if (arrLength != othLength && !(isPartial && othLength > arrLength)) {
        return false;
      }
      var stacked = stack.get(array);
      if (stacked) {
        return stacked == other;
      }
      var result = true;
      stack.set(array, other);
      while (++index < arrLength) {
        var arrValue = array[index],
            othValue = other[index];
        if (customizer) {
          var compared = isPartial ? customizer(othValue, arrValue, index, other, array, stack) : customizer(arrValue, othValue, index, array, other, stack);
        }
        if (compared !== undefined) {
          if (compared) {
            continue;
          }
          result = false;
          break;
        }
        if (isUnordered) {
          if (!arraySome(other, function(othValue) {
            return arrValue === othValue || equalFunc(arrValue, othValue, customizer, bitmask, stack);
          })) {
            result = false;
            break;
          }
        } else if (!(arrValue === othValue || equalFunc(arrValue, othValue, customizer, bitmask, stack))) {
          result = false;
          break;
        }
      }
      stack['delete'](array);
      return result;
    }
    function equalByTag(object, other, tag, equalFunc, customizer, bitmask) {
      switch (tag) {
        case arrayBufferTag:
          if ((object.byteLength != other.byteLength) || !equalFunc(new Uint8Array(object), new Uint8Array(other))) {
            return false;
          }
          return true;
        case boolTag:
        case dateTag:
          return +object == +other;
        case errorTag:
          return object.name == other.name && object.message == other.message;
        case numberTag:
          return (object != +object) ? other != +other : object == +other;
        case regexpTag:
        case stringTag:
          return object == (other + '');
        case mapTag:
          var convert = mapToArray;
        case setTag:
          var isPartial = bitmask & PARTIAL_COMPARE_FLAG;
          convert || (convert = setToArray);
          return (isPartial || object.size == other.size) && equalFunc(convert(object), convert(other), customizer, bitmask | UNORDERED_COMPARE_FLAG);
        case symbolTag:
          return !!Symbol && (symbolValueOf.call(object) == symbolValueOf.call(other));
      }
      return false;
    }
    function equalObjects(object, other, equalFunc, customizer, bitmask, stack) {
      var isPartial = bitmask & PARTIAL_COMPARE_FLAG,
          objProps = keys(object),
          objLength = objProps.length,
          othProps = keys(other),
          othLength = othProps.length;
      if (objLength != othLength && !isPartial) {
        return false;
      }
      var index = objLength;
      while (index--) {
        var key = objProps[index];
        if (!(isPartial ? key in other : baseHas(other, key))) {
          return false;
        }
      }
      var stacked = stack.get(object);
      if (stacked) {
        return stacked == other;
      }
      var result = true;
      stack.set(object, other);
      var skipCtor = isPartial;
      while (++index < objLength) {
        key = objProps[index];
        var objValue = object[key],
            othValue = other[key];
        if (customizer) {
          var compared = isPartial ? customizer(othValue, objValue, key, other, object, stack) : customizer(objValue, othValue, key, object, other, stack);
        }
        if (!(compared === undefined ? (objValue === othValue || equalFunc(objValue, othValue, customizer, bitmask, stack)) : compared)) {
          result = false;
          break;
        }
        skipCtor || (skipCtor = key == 'constructor');
      }
      if (result && !skipCtor) {
        var objCtor = object.constructor,
            othCtor = other.constructor;
        if (objCtor != othCtor && ('constructor' in object && 'constructor' in other) && !(typeof objCtor == 'function' && objCtor instanceof objCtor && typeof othCtor == 'function' && othCtor instanceof othCtor)) {
          result = false;
        }
      }
      stack['delete'](object);
      return result;
    }
    var getData = !metaMap ? noop : function(func) {
      return metaMap.get(func);
    };
    function getFuncName(func) {
      var result = (func.name + ''),
          array = realNames[result],
          length = hasOwnProperty.call(realNames, result) ? array.length : 0;
      while (length--) {
        var data = array[length],
            otherFunc = data.func;
        if (otherFunc == null || otherFunc == func) {
          return data.name;
        }
      }
      return result;
    }
    function getIteratee() {
      var result = lodash.iteratee || iteratee;
      result = result === iteratee ? baseIteratee : result;
      return arguments.length ? result(arguments[0], arguments[1]) : result;
    }
    var getLength = baseProperty('length');
    function getMatchData(object) {
      var result = toPairs(object),
          length = result.length;
      while (length--) {
        result[length][2] = isStrictComparable(result[length][1]);
      }
      return result;
    }
    function getNative(object, key) {
      var value = object == null ? undefined : object[key];
      return isNative(value) ? value : undefined;
    }
    function getPlaceholder(func) {
      var object = hasOwnProperty.call(lodash, 'placeholder') ? lodash : func;
      return object.placeholder;
    }
    var getSymbols = getOwnPropertySymbols || function() {
      return [];
    };
    function getTag(value) {
      return objectToString.call(value);
    }
    if ((Map && getTag(new Map) != mapTag) || (Set && getTag(new Set) != setTag) || (WeakMap && getTag(new WeakMap) != weakMapTag)) {
      getTag = function(value) {
        var result = objectToString.call(value),
            Ctor = result == objectTag ? value.constructor : null,
            ctorString = typeof Ctor == 'function' ? funcToString.call(Ctor) : '';
        if (ctorString) {
          switch (ctorString) {
            case mapCtorString:
              return mapTag;
            case setCtorString:
              return setTag;
            case weakMapCtorString:
              return weakMapTag;
          }
        }
        return result;
      };
    }
    function getView(start, end, transforms) {
      var index = -1,
          length = transforms.length;
      while (++index < length) {
        var data = transforms[index],
            size = data.size;
        switch (data.type) {
          case 'drop':
            start += size;
            break;
          case 'dropRight':
            end -= size;
            break;
          case 'take':
            end = nativeMin(end, start + size);
            break;
          case 'takeRight':
            start = nativeMax(start, end - size);
            break;
        }
      }
      return {
        'start': start,
        'end': end
      };
    }
    function hasPath(object, path, hasFunc) {
      if (object == null) {
        return false;
      }
      var result = hasFunc(object, path);
      if (!result && !isKey(path)) {
        path = baseCastPath(path);
        object = parent(object, path);
        if (object != null) {
          path = last(path);
          result = hasFunc(object, path);
        }
      }
      var length = object ? object.length : undefined;
      return result || (!!length && isLength(length) && isIndex(path, length) && (isArray(object) || isString(object) || isArguments(object)));
    }
    function initCloneArray(array) {
      var length = array.length,
          result = array.constructor(length);
      if (length && typeof array[0] == 'string' && hasOwnProperty.call(array, 'index')) {
        result.index = array.index;
        result.input = array.input;
      }
      return result;
    }
    function initCloneObject(object) {
      return (isFunction(object.constructor) && !isPrototype(object)) ? baseCreate(getPrototypeOf(object)) : {};
    }
    function initCloneByTag(object, tag, isDeep) {
      var Ctor = object.constructor;
      switch (tag) {
        case arrayBufferTag:
          return cloneArrayBuffer(object);
        case boolTag:
        case dateTag:
          return new Ctor(+object);
        case float32Tag:
        case float64Tag:
        case int8Tag:
        case int16Tag:
        case int32Tag:
        case uint8Tag:
        case uint8ClampedTag:
        case uint16Tag:
        case uint32Tag:
          return cloneTypedArray(object, isDeep);
        case mapTag:
          return cloneMap(object);
        case numberTag:
        case stringTag:
          return new Ctor(object);
        case regexpTag:
          return cloneRegExp(object);
        case setTag:
          return cloneSet(object);
        case symbolTag:
          return cloneSymbol(object);
      }
    }
    function indexKeys(object) {
      var length = object ? object.length : undefined;
      if (isLength(length) && (isArray(object) || isString(object) || isArguments(object))) {
        return baseTimes(length, String);
      }
      return null;
    }
    function isIterateeCall(value, index, object) {
      if (!isObject(object)) {
        return false;
      }
      var type = typeof index;
      if (type == 'number' ? (isArrayLike(object) && isIndex(index, object.length)) : (type == 'string' && index in object)) {
        return eq(object[index], value);
      }
      return false;
    }
    function isKey(value, object) {
      if (typeof value == 'number') {
        return true;
      }
      return !isArray(value) && (reIsPlainProp.test(value) || !reIsDeepProp.test(value) || (object != null && value in Object(object)));
    }
    function isKeyable(value) {
      var type = typeof value;
      return type == 'number' || type == 'boolean' || (type == 'string' && value != '__proto__') || value == null;
    }
    function isLaziable(func) {
      var funcName = getFuncName(func),
          other = lodash[funcName];
      if (typeof other != 'function' || !(funcName in LazyWrapper.prototype)) {
        return false;
      }
      if (func === other) {
        return true;
      }
      var data = getData(other);
      return !!data && func === data[0];
    }
    function isPrototype(value) {
      var Ctor = value && value.constructor,
          proto = (isFunction(Ctor) && Ctor.prototype) || objectProto;
      return value === proto;
    }
    function isStrictComparable(value) {
      return value === value && !isObject(value);
    }
    function mergeData(data, source) {
      var bitmask = data[1],
          srcBitmask = source[1],
          newBitmask = bitmask | srcBitmask,
          isCommon = newBitmask < (BIND_FLAG | BIND_KEY_FLAG | ARY_FLAG);
      var isCombo = ((srcBitmask == ARY_FLAG) && (bitmask == CURRY_FLAG)) || ((srcBitmask == ARY_FLAG) && (bitmask == REARG_FLAG) && (data[7].length <= source[8])) || ((srcBitmask == (ARY_FLAG | REARG_FLAG)) && (source[7].length <= source[8]) && (bitmask == CURRY_FLAG));
      if (!(isCommon || isCombo)) {
        return data;
      }
      if (srcBitmask & BIND_FLAG) {
        data[2] = source[2];
        newBitmask |= bitmask & BIND_FLAG ? 0 : CURRY_BOUND_FLAG;
      }
      var value = source[3];
      if (value) {
        var partials = data[3];
        data[3] = partials ? composeArgs(partials, value, source[4]) : copyArray(value);
        data[4] = partials ? replaceHolders(data[3], PLACEHOLDER) : copyArray(source[4]);
      }
      value = source[5];
      if (value) {
        partials = data[5];
        data[5] = partials ? composeArgsRight(partials, value, source[6]) : copyArray(value);
        data[6] = partials ? replaceHolders(data[5], PLACEHOLDER) : copyArray(source[6]);
      }
      value = source[7];
      if (value) {
        data[7] = copyArray(value);
      }
      if (srcBitmask & ARY_FLAG) {
        data[8] = data[8] == null ? source[8] : nativeMin(data[8], source[8]);
      }
      if (data[9] == null) {
        data[9] = source[9];
      }
      data[0] = source[0];
      data[1] = newBitmask;
      return data;
    }
    function mergeDefaults(objValue, srcValue, key, object, source, stack) {
      if (isObject(objValue) && isObject(srcValue)) {
        stack.set(srcValue, objValue);
        baseMerge(objValue, srcValue, undefined, mergeDefaults, stack);
      }
      return objValue;
    }
    function parent(object, path) {
      return path.length == 1 ? object : get(object, baseSlice(path, 0, -1));
    }
    function reorder(array, indexes) {
      var arrLength = array.length,
          length = nativeMin(indexes.length, arrLength),
          oldArray = copyArray(array);
      while (length--) {
        var index = indexes[length];
        array[length] = isIndex(index, arrLength) ? oldArray[index] : undefined;
      }
      return array;
    }
    var setData = (function() {
      var count = 0,
          lastCalled = 0;
      return function(key, value) {
        var stamp = now(),
            remaining = HOT_SPAN - (stamp - lastCalled);
        lastCalled = stamp;
        if (remaining > 0) {
          if (++count >= HOT_COUNT) {
            return key;
          }
        } else {
          count = 0;
        }
        return baseSetData(key, value);
      };
    }());
    function stringToPath(string) {
      var result = [];
      toString(string).replace(rePropName, function(match, number, quote, string) {
        result.push(quote ? string.replace(reEscapeChar, '$1') : (number || match));
      });
      return result;
    }
    function wrapperClone(wrapper) {
      if (wrapper instanceof LazyWrapper) {
        return wrapper.clone();
      }
      var result = new LodashWrapper(wrapper.__wrapped__, wrapper.__chain__);
      result.__actions__ = copyArray(wrapper.__actions__);
      result.__index__ = wrapper.__index__;
      result.__values__ = wrapper.__values__;
      return result;
    }
    function chunk(array, size) {
      size = nativeMax(toInteger(size), 0);
      var length = array ? array.length : 0;
      if (!length || size < 1) {
        return [];
      }
      var index = 0,
          resIndex = -1,
          result = Array(nativeCeil(length / size));
      while (index < length) {
        result[++resIndex] = baseSlice(array, index, (index += size));
      }
      return result;
    }
    function compact(array) {
      var index = -1,
          length = array ? array.length : 0,
          resIndex = -1,
          result = [];
      while (++index < length) {
        var value = array[index];
        if (value) {
          result[++resIndex] = value;
        }
      }
      return result;
    }
    var concat = rest(function(array, values) {
      if (!isArray(array)) {
        array = array == null ? [] : [Object(array)];
      }
      values = baseFlatten(values, 1);
      return arrayConcat(array, values);
    });
    var difference = rest(function(array, values) {
      return isArrayLikeObject(array) ? baseDifference(array, baseFlatten(values, 1, true)) : [];
    });
    var differenceBy = rest(function(array, values) {
      var iteratee = last(values);
      if (isArrayLikeObject(iteratee)) {
        iteratee = undefined;
      }
      return isArrayLikeObject(array) ? baseDifference(array, baseFlatten(values, 1, true), getIteratee(iteratee)) : [];
    });
    var differenceWith = rest(function(array, values) {
      var comparator = last(values);
      if (isArrayLikeObject(comparator)) {
        comparator = undefined;
      }
      return isArrayLikeObject(array) ? baseDifference(array, baseFlatten(values, 1, true), undefined, comparator) : [];
    });
    function drop(array, n, guard) {
      var length = array ? array.length : 0;
      if (!length) {
        return [];
      }
      n = (guard || n === undefined) ? 1 : toInteger(n);
      return baseSlice(array, n < 0 ? 0 : n, length);
    }
    function dropRight(array, n, guard) {
      var length = array ? array.length : 0;
      if (!length) {
        return [];
      }
      n = (guard || n === undefined) ? 1 : toInteger(n);
      n = length - n;
      return baseSlice(array, 0, n < 0 ? 0 : n);
    }
    function dropRightWhile(array, predicate) {
      return (array && array.length) ? baseWhile(array, getIteratee(predicate, 3), true, true) : [];
    }
    function dropWhile(array, predicate) {
      return (array && array.length) ? baseWhile(array, getIteratee(predicate, 3), true) : [];
    }
    function fill(array, value, start, end) {
      var length = array ? array.length : 0;
      if (!length) {
        return [];
      }
      if (start && typeof start != 'number' && isIterateeCall(array, value, start)) {
        start = 0;
        end = length;
      }
      return baseFill(array, value, start, end);
    }
    function findIndex(array, predicate) {
      return (array && array.length) ? baseFindIndex(array, getIteratee(predicate, 3)) : -1;
    }
    function findLastIndex(array, predicate) {
      return (array && array.length) ? baseFindIndex(array, getIteratee(predicate, 3), true) : -1;
    }
    function flatten(array) {
      var length = array ? array.length : 0;
      return length ? baseFlatten(array, 1) : [];
    }
    function flattenDeep(array) {
      var length = array ? array.length : 0;
      return length ? baseFlatten(array, INFINITY) : [];
    }
    function flattenDepth(array, depth) {
      var length = array ? array.length : 0;
      if (!length) {
        return [];
      }
      depth = depth === undefined ? 1 : toInteger(depth);
      return baseFlatten(array, depth);
    }
    function fromPairs(pairs) {
      var index = -1,
          length = pairs ? pairs.length : 0,
          result = {};
      while (++index < length) {
        var pair = pairs[index];
        result[pair[0]] = pair[1];
      }
      return result;
    }
    function head(array) {
      return array ? array[0] : undefined;
    }
    function indexOf(array, value, fromIndex) {
      var length = array ? array.length : 0;
      if (!length) {
        return -1;
      }
      fromIndex = toInteger(fromIndex);
      if (fromIndex < 0) {
        fromIndex = nativeMax(length + fromIndex, 0);
      }
      return baseIndexOf(array, value, fromIndex);
    }
    function initial(array) {
      return dropRight(array, 1);
    }
    var intersection = rest(function(arrays) {
      var mapped = arrayMap(arrays, baseCastArrayLikeObject);
      return (mapped.length && mapped[0] === arrays[0]) ? baseIntersection(mapped) : [];
    });
    var intersectionBy = rest(function(arrays) {
      var iteratee = last(arrays),
          mapped = arrayMap(arrays, baseCastArrayLikeObject);
      if (iteratee === last(mapped)) {
        iteratee = undefined;
      } else {
        mapped.pop();
      }
      return (mapped.length && mapped[0] === arrays[0]) ? baseIntersection(mapped, getIteratee(iteratee)) : [];
    });
    var intersectionWith = rest(function(arrays) {
      var comparator = last(arrays),
          mapped = arrayMap(arrays, baseCastArrayLikeObject);
      if (comparator === last(mapped)) {
        comparator = undefined;
      } else {
        mapped.pop();
      }
      return (mapped.length && mapped[0] === arrays[0]) ? baseIntersection(mapped, undefined, comparator) : [];
    });
    function join(array, separator) {
      return array ? nativeJoin.call(array, separator) : '';
    }
    function last(array) {
      var length = array ? array.length : 0;
      return length ? array[length - 1] : undefined;
    }
    function lastIndexOf(array, value, fromIndex) {
      var length = array ? array.length : 0;
      if (!length) {
        return -1;
      }
      var index = length;
      if (fromIndex !== undefined) {
        index = toInteger(fromIndex);
        index = (index < 0 ? nativeMax(length + index, 0) : nativeMin(index, length - 1)) + 1;
      }
      if (value !== value) {
        return indexOfNaN(array, index, true);
      }
      while (index--) {
        if (array[index] === value) {
          return index;
        }
      }
      return -1;
    }
    var pull = rest(pullAll);
    function pullAll(array, values) {
      return (array && array.length && values && values.length) ? basePullAll(array, values) : array;
    }
    function pullAllBy(array, values, iteratee) {
      return (array && array.length && values && values.length) ? basePullAllBy(array, values, getIteratee(iteratee)) : array;
    }
    var pullAt = rest(function(array, indexes) {
      indexes = arrayMap(baseFlatten(indexes, 1), String);
      var result = baseAt(array, indexes);
      basePullAt(array, indexes.sort(compareAscending));
      return result;
    });
    function remove(array, predicate) {
      var result = [];
      if (!(array && array.length)) {
        return result;
      }
      var index = -1,
          indexes = [],
          length = array.length;
      predicate = getIteratee(predicate, 3);
      while (++index < length) {
        var value = array[index];
        if (predicate(value, index, array)) {
          result.push(value);
          indexes.push(index);
        }
      }
      basePullAt(array, indexes);
      return result;
    }
    function reverse(array) {
      return array ? nativeReverse.call(array) : array;
    }
    function slice(array, start, end) {
      var length = array ? array.length : 0;
      if (!length) {
        return [];
      }
      if (end && typeof end != 'number' && isIterateeCall(array, start, end)) {
        start = 0;
        end = length;
      } else {
        start = start == null ? 0 : toInteger(start);
        end = end === undefined ? length : toInteger(end);
      }
      return baseSlice(array, start, end);
    }
    function sortedIndex(array, value) {
      return baseSortedIndex(array, value);
    }
    function sortedIndexBy(array, value, iteratee) {
      return baseSortedIndexBy(array, value, getIteratee(iteratee));
    }
    function sortedIndexOf(array, value) {
      var length = array ? array.length : 0;
      if (length) {
        var index = baseSortedIndex(array, value);
        if (index < length && eq(array[index], value)) {
          return index;
        }
      }
      return -1;
    }
    function sortedLastIndex(array, value) {
      return baseSortedIndex(array, value, true);
    }
    function sortedLastIndexBy(array, value, iteratee) {
      return baseSortedIndexBy(array, value, getIteratee(iteratee), true);
    }
    function sortedLastIndexOf(array, value) {
      var length = array ? array.length : 0;
      if (length) {
        var index = baseSortedIndex(array, value, true) - 1;
        if (eq(array[index], value)) {
          return index;
        }
      }
      return -1;
    }
    function sortedUniq(array) {
      return (array && array.length) ? baseSortedUniq(array) : [];
    }
    function sortedUniqBy(array, iteratee) {
      return (array && array.length) ? baseSortedUniqBy(array, getIteratee(iteratee)) : [];
    }
    function tail(array) {
      return drop(array, 1);
    }
    function take(array, n, guard) {
      if (!(array && array.length)) {
        return [];
      }
      n = (guard || n === undefined) ? 1 : toInteger(n);
      return baseSlice(array, 0, n < 0 ? 0 : n);
    }
    function takeRight(array, n, guard) {
      var length = array ? array.length : 0;
      if (!length) {
        return [];
      }
      n = (guard || n === undefined) ? 1 : toInteger(n);
      n = length - n;
      return baseSlice(array, n < 0 ? 0 : n, length);
    }
    function takeRightWhile(array, predicate) {
      return (array && array.length) ? baseWhile(array, getIteratee(predicate, 3), false, true) : [];
    }
    function takeWhile(array, predicate) {
      return (array && array.length) ? baseWhile(array, getIteratee(predicate, 3)) : [];
    }
    var union = rest(function(arrays) {
      return baseUniq(baseFlatten(arrays, 1, true));
    });
    var unionBy = rest(function(arrays) {
      var iteratee = last(arrays);
      if (isArrayLikeObject(iteratee)) {
        iteratee = undefined;
      }
      return baseUniq(baseFlatten(arrays, 1, true), getIteratee(iteratee));
    });
    var unionWith = rest(function(arrays) {
      var comparator = last(arrays);
      if (isArrayLikeObject(comparator)) {
        comparator = undefined;
      }
      return baseUniq(baseFlatten(arrays, 1, true), undefined, comparator);
    });
    function uniq(array) {
      return (array && array.length) ? baseUniq(array) : [];
    }
    function uniqBy(array, iteratee) {
      return (array && array.length) ? baseUniq(array, getIteratee(iteratee)) : [];
    }
    function uniqWith(array, comparator) {
      return (array && array.length) ? baseUniq(array, undefined, comparator) : [];
    }
    function unzip(array) {
      if (!(array && array.length)) {
        return [];
      }
      var length = 0;
      array = arrayFilter(array, function(group) {
        if (isArrayLikeObject(group)) {
          length = nativeMax(group.length, length);
          return true;
        }
      });
      return baseTimes(length, function(index) {
        return arrayMap(array, baseProperty(index));
      });
    }
    function unzipWith(array, iteratee) {
      if (!(array && array.length)) {
        return [];
      }
      var result = unzip(array);
      if (iteratee == null) {
        return result;
      }
      return arrayMap(result, function(group) {
        return apply(iteratee, undefined, group);
      });
    }
    var without = rest(function(array, values) {
      return isArrayLikeObject(array) ? baseDifference(array, values) : [];
    });
    var xor = rest(function(arrays) {
      return baseXor(arrayFilter(arrays, isArrayLikeObject));
    });
    var xorBy = rest(function(arrays) {
      var iteratee = last(arrays);
      if (isArrayLikeObject(iteratee)) {
        iteratee = undefined;
      }
      return baseXor(arrayFilter(arrays, isArrayLikeObject), getIteratee(iteratee));
    });
    var xorWith = rest(function(arrays) {
      var comparator = last(arrays);
      if (isArrayLikeObject(comparator)) {
        comparator = undefined;
      }
      return baseXor(arrayFilter(arrays, isArrayLikeObject), undefined, comparator);
    });
    var zip = rest(unzip);
    function zipObject(props, values) {
      return baseZipObject(props || [], values || [], assignValue);
    }
    function zipObjectDeep(props, values) {
      return baseZipObject(props || [], values || [], baseSet);
    }
    var zipWith = rest(function(arrays) {
      var length = arrays.length,
          iteratee = length > 1 ? arrays[length - 1] : undefined;
      iteratee = typeof iteratee == 'function' ? (arrays.pop(), iteratee) : undefined;
      return unzipWith(arrays, iteratee);
    });
    function chain(value) {
      var result = lodash(value);
      result.__chain__ = true;
      return result;
    }
    function tap(value, interceptor) {
      interceptor(value);
      return value;
    }
    function thru(value, interceptor) {
      return interceptor(value);
    }
    var wrapperAt = rest(function(paths) {
      paths = baseFlatten(paths, 1);
      var length = paths.length,
          start = length ? paths[0] : 0,
          value = this.__wrapped__,
          interceptor = function(object) {
            return baseAt(object, paths);
          };
      if (length > 1 || this.__actions__.length || !(value instanceof LazyWrapper) || !isIndex(start)) {
        return this.thru(interceptor);
      }
      value = value.slice(start, +start + (length ? 1 : 0));
      value.__actions__.push({
        'func': thru,
        'args': [interceptor],
        'thisArg': undefined
      });
      return new LodashWrapper(value, this.__chain__).thru(function(array) {
        if (length && !array.length) {
          array.push(undefined);
        }
        return array;
      });
    });
    function wrapperChain() {
      return chain(this);
    }
    function wrapperCommit() {
      return new LodashWrapper(this.value(), this.__chain__);
    }
    function wrapperFlatMap(iteratee) {
      return this.map(iteratee).flatten();
    }
    function wrapperNext() {
      if (this.__values__ === undefined) {
        this.__values__ = toArray(this.value());
      }
      var done = this.__index__ >= this.__values__.length,
          value = done ? undefined : this.__values__[this.__index__++];
      return {
        'done': done,
        'value': value
      };
    }
    function wrapperToIterator() {
      return this;
    }
    function wrapperPlant(value) {
      var result,
          parent = this;
      while (parent instanceof baseLodash) {
        var clone = wrapperClone(parent);
        clone.__index__ = 0;
        clone.__values__ = undefined;
        if (result) {
          previous.__wrapped__ = clone;
        } else {
          result = clone;
        }
        var previous = clone;
        parent = parent.__wrapped__;
      }
      previous.__wrapped__ = value;
      return result;
    }
    function wrapperReverse() {
      var value = this.__wrapped__;
      if (value instanceof LazyWrapper) {
        var wrapped = value;
        if (this.__actions__.length) {
          wrapped = new LazyWrapper(this);
        }
        wrapped = wrapped.reverse();
        wrapped.__actions__.push({
          'func': thru,
          'args': [reverse],
          'thisArg': undefined
        });
        return new LodashWrapper(wrapped, this.__chain__);
      }
      return this.thru(reverse);
    }
    function wrapperValue() {
      return baseWrapperValue(this.__wrapped__, this.__actions__);
    }
    var countBy = createAggregator(function(result, value, key) {
      hasOwnProperty.call(result, key) ? ++result[key] : (result[key] = 1);
    });
    function every(collection, predicate, guard) {
      var func = isArray(collection) ? arrayEvery : baseEvery;
      if (guard && isIterateeCall(collection, predicate, guard)) {
        predicate = undefined;
      }
      return func(collection, getIteratee(predicate, 3));
    }
    function filter(collection, predicate) {
      var func = isArray(collection) ? arrayFilter : baseFilter;
      return func(collection, getIteratee(predicate, 3));
    }
    function find(collection, predicate) {
      predicate = getIteratee(predicate, 3);
      if (isArray(collection)) {
        var index = baseFindIndex(collection, predicate);
        return index > -1 ? collection[index] : undefined;
      }
      return baseFind(collection, predicate, baseEach);
    }
    function findLast(collection, predicate) {
      predicate = getIteratee(predicate, 3);
      if (isArray(collection)) {
        var index = baseFindIndex(collection, predicate, true);
        return index > -1 ? collection[index] : undefined;
      }
      return baseFind(collection, predicate, baseEachRight);
    }
    function flatMap(collection, iteratee) {
      return baseFlatten(map(collection, iteratee), 1);
    }
    function forEach(collection, iteratee) {
      return (typeof iteratee == 'function' && isArray(collection)) ? arrayEach(collection, iteratee) : baseEach(collection, baseCastFunction(iteratee));
    }
    function forEachRight(collection, iteratee) {
      return (typeof iteratee == 'function' && isArray(collection)) ? arrayEachRight(collection, iteratee) : baseEachRight(collection, baseCastFunction(iteratee));
    }
    var groupBy = createAggregator(function(result, value, key) {
      if (hasOwnProperty.call(result, key)) {
        result[key].push(value);
      } else {
        result[key] = [value];
      }
    });
    function includes(collection, value, fromIndex, guard) {
      collection = isArrayLike(collection) ? collection : values(collection);
      fromIndex = (fromIndex && !guard) ? toInteger(fromIndex) : 0;
      var length = collection.length;
      if (fromIndex < 0) {
        fromIndex = nativeMax(length + fromIndex, 0);
      }
      return isString(collection) ? (fromIndex <= length && collection.indexOf(value, fromIndex) > -1) : (!!length && baseIndexOf(collection, value, fromIndex) > -1);
    }
    var invokeMap = rest(function(collection, path, args) {
      var index = -1,
          isFunc = typeof path == 'function',
          isProp = isKey(path),
          result = isArrayLike(collection) ? Array(collection.length) : [];
      baseEach(collection, function(value) {
        var func = isFunc ? path : ((isProp && value != null) ? value[path] : undefined);
        result[++index] = func ? apply(func, value, args) : baseInvoke(value, path, args);
      });
      return result;
    });
    var keyBy = createAggregator(function(result, value, key) {
      result[key] = value;
    });
    function map(collection, iteratee) {
      var func = isArray(collection) ? arrayMap : baseMap;
      return func(collection, getIteratee(iteratee, 3));
    }
    function orderBy(collection, iteratees, orders, guard) {
      if (collection == null) {
        return [];
      }
      if (!isArray(iteratees)) {
        iteratees = iteratees == null ? [] : [iteratees];
      }
      orders = guard ? undefined : orders;
      if (!isArray(orders)) {
        orders = orders == null ? [] : [orders];
      }
      return baseOrderBy(collection, iteratees, orders);
    }
    var partition = createAggregator(function(result, value, key) {
      result[key ? 0 : 1].push(value);
    }, function() {
      return [[], []];
    });
    function reduce(collection, iteratee, accumulator) {
      var func = isArray(collection) ? arrayReduce : baseReduce,
          initAccum = arguments.length < 3;
      return func(collection, getIteratee(iteratee, 4), accumulator, initAccum, baseEach);
    }
    function reduceRight(collection, iteratee, accumulator) {
      var func = isArray(collection) ? arrayReduceRight : baseReduce,
          initAccum = arguments.length < 3;
      return func(collection, getIteratee(iteratee, 4), accumulator, initAccum, baseEachRight);
    }
    function reject(collection, predicate) {
      var func = isArray(collection) ? arrayFilter : baseFilter;
      predicate = getIteratee(predicate, 3);
      return func(collection, function(value, index, collection) {
        return !predicate(value, index, collection);
      });
    }
    function sample(collection) {
      var array = isArrayLike(collection) ? collection : values(collection),
          length = array.length;
      return length > 0 ? array[baseRandom(0, length - 1)] : undefined;
    }
    function sampleSize(collection, n) {
      var index = -1,
          result = toArray(collection),
          length = result.length,
          lastIndex = length - 1;
      n = baseClamp(toInteger(n), 0, length);
      while (++index < n) {
        var rand = baseRandom(index, lastIndex),
            value = result[rand];
        result[rand] = result[index];
        result[index] = value;
      }
      result.length = n;
      return result;
    }
    function shuffle(collection) {
      return sampleSize(collection, MAX_ARRAY_LENGTH);
    }
    function size(collection) {
      if (collection == null) {
        return 0;
      }
      if (isArrayLike(collection)) {
        var result = collection.length;
        return (result && isString(collection)) ? stringSize(collection) : result;
      }
      return keys(collection).length;
    }
    function some(collection, predicate, guard) {
      var func = isArray(collection) ? arraySome : baseSome;
      if (guard && isIterateeCall(collection, predicate, guard)) {
        predicate = undefined;
      }
      return func(collection, getIteratee(predicate, 3));
    }
    var sortBy = rest(function(collection, iteratees) {
      if (collection == null) {
        return [];
      }
      var length = iteratees.length;
      if (length > 1 && isIterateeCall(collection, iteratees[0], iteratees[1])) {
        iteratees = [];
      } else if (length > 2 && isIterateeCall(iteratees[0], iteratees[1], iteratees[2])) {
        iteratees.length = 1;
      }
      return baseOrderBy(collection, baseFlatten(iteratees, 1), []);
    });
    var now = Date.now;
    function after(n, func) {
      if (typeof func != 'function') {
        throw new TypeError(FUNC_ERROR_TEXT);
      }
      n = toInteger(n);
      return function() {
        if (--n < 1) {
          return func.apply(this, arguments);
        }
      };
    }
    function ary(func, n, guard) {
      n = guard ? undefined : n;
      n = (func && n == null) ? func.length : n;
      return createWrapper(func, ARY_FLAG, undefined, undefined, undefined, undefined, n);
    }
    function before(n, func) {
      var result;
      if (typeof func != 'function') {
        throw new TypeError(FUNC_ERROR_TEXT);
      }
      n = toInteger(n);
      return function() {
        if (--n > 0) {
          result = func.apply(this, arguments);
        }
        if (n <= 1) {
          func = undefined;
        }
        return result;
      };
    }
    var bind = rest(function(func, thisArg, partials) {
      var bitmask = BIND_FLAG;
      if (partials.length) {
        var holders = replaceHolders(partials, getPlaceholder(bind));
        bitmask |= PARTIAL_FLAG;
      }
      return createWrapper(func, bitmask, thisArg, partials, holders);
    });
    var bindKey = rest(function(object, key, partials) {
      var bitmask = BIND_FLAG | BIND_KEY_FLAG;
      if (partials.length) {
        var holders = replaceHolders(partials, getPlaceholder(bindKey));
        bitmask |= PARTIAL_FLAG;
      }
      return createWrapper(key, bitmask, object, partials, holders);
    });
    function curry(func, arity, guard) {
      arity = guard ? undefined : arity;
      var result = createWrapper(func, CURRY_FLAG, undefined, undefined, undefined, undefined, undefined, arity);
      result.placeholder = curry.placeholder;
      return result;
    }
    function curryRight(func, arity, guard) {
      arity = guard ? undefined : arity;
      var result = createWrapper(func, CURRY_RIGHT_FLAG, undefined, undefined, undefined, undefined, undefined, arity);
      result.placeholder = curryRight.placeholder;
      return result;
    }
    function debounce(func, wait, options) {
      var args,
          maxTimeoutId,
          result,
          stamp,
          thisArg,
          timeoutId,
          trailingCall,
          lastCalled = 0,
          leading = false,
          maxWait = false,
          trailing = true;
      if (typeof func != 'function') {
        throw new TypeError(FUNC_ERROR_TEXT);
      }
      wait = toNumber(wait) || 0;
      if (isObject(options)) {
        leading = !!options.leading;
        maxWait = 'maxWait' in options && nativeMax(toNumber(options.maxWait) || 0, wait);
        trailing = 'trailing' in options ? !!options.trailing : trailing;
      }
      function cancel() {
        if (timeoutId) {
          clearTimeout(timeoutId);
        }
        if (maxTimeoutId) {
          clearTimeout(maxTimeoutId);
        }
        lastCalled = 0;
        args = maxTimeoutId = thisArg = timeoutId = trailingCall = undefined;
      }
      function complete(isCalled, id) {
        if (id) {
          clearTimeout(id);
        }
        maxTimeoutId = timeoutId = trailingCall = undefined;
        if (isCalled) {
          lastCalled = now();
          result = func.apply(thisArg, args);
          if (!timeoutId && !maxTimeoutId) {
            args = thisArg = undefined;
          }
        }
      }
      function delayed() {
        var remaining = wait - (now() - stamp);
        if (remaining <= 0 || remaining > wait) {
          complete(trailingCall, maxTimeoutId);
        } else {
          timeoutId = setTimeout(delayed, remaining);
        }
      }
      function flush() {
        if ((timeoutId && trailingCall) || (maxTimeoutId && trailing)) {
          result = func.apply(thisArg, args);
        }
        cancel();
        return result;
      }
      function maxDelayed() {
        complete(trailing, timeoutId);
      }
      function debounced() {
        args = arguments;
        stamp = now();
        thisArg = this;
        trailingCall = trailing && (timeoutId || !leading);
        if (maxWait === false) {
          var leadingCall = leading && !timeoutId;
        } else {
          if (!lastCalled && !maxTimeoutId && !leading) {
            lastCalled = stamp;
          }
          var remaining = maxWait - (stamp - lastCalled);
          var isCalled = (remaining <= 0 || remaining > maxWait) && (leading || maxTimeoutId);
          if (isCalled) {
            if (maxTimeoutId) {
              maxTimeoutId = clearTimeout(maxTimeoutId);
            }
            lastCalled = stamp;
            result = func.apply(thisArg, args);
          } else if (!maxTimeoutId) {
            maxTimeoutId = setTimeout(maxDelayed, remaining);
          }
        }
        if (isCalled && timeoutId) {
          timeoutId = clearTimeout(timeoutId);
        } else if (!timeoutId && wait !== maxWait) {
          timeoutId = setTimeout(delayed, wait);
        }
        if (leadingCall) {
          isCalled = true;
          result = func.apply(thisArg, args);
        }
        if (isCalled && !timeoutId && !maxTimeoutId) {
          args = thisArg = undefined;
        }
        return result;
      }
      debounced.cancel = cancel;
      debounced.flush = flush;
      return debounced;
    }
    var defer = rest(function(func, args) {
      return baseDelay(func, 1, args);
    });
    var delay = rest(function(func, wait, args) {
      return baseDelay(func, toNumber(wait) || 0, args);
    });
    function flip(func) {
      return createWrapper(func, FLIP_FLAG);
    }
    function memoize(func, resolver) {
      if (typeof func != 'function' || (resolver && typeof resolver != 'function')) {
        throw new TypeError(FUNC_ERROR_TEXT);
      }
      var memoized = function() {
        var args = arguments,
            key = resolver ? resolver.apply(this, args) : args[0],
            cache = memoized.cache;
        if (cache.has(key)) {
          return cache.get(key);
        }
        var result = func.apply(this, args);
        memoized.cache = cache.set(key, result);
        return result;
      };
      memoized.cache = new memoize.Cache;
      return memoized;
    }
    function negate(predicate) {
      if (typeof predicate != 'function') {
        throw new TypeError(FUNC_ERROR_TEXT);
      }
      return function() {
        return !predicate.apply(this, arguments);
      };
    }
    function once(func) {
      return before(2, func);
    }
    var overArgs = rest(function(func, transforms) {
      transforms = arrayMap(baseFlatten(transforms, 1), getIteratee());
      var funcsLength = transforms.length;
      return rest(function(args) {
        var index = -1,
            length = nativeMin(args.length, funcsLength);
        while (++index < length) {
          args[index] = transforms[index].call(this, args[index]);
        }
        return apply(func, this, args);
      });
    });
    var partial = rest(function(func, partials) {
      var holders = replaceHolders(partials, getPlaceholder(partial));
      return createWrapper(func, PARTIAL_FLAG, undefined, partials, holders);
    });
    var partialRight = rest(function(func, partials) {
      var holders = replaceHolders(partials, getPlaceholder(partialRight));
      return createWrapper(func, PARTIAL_RIGHT_FLAG, undefined, partials, holders);
    });
    var rearg = rest(function(func, indexes) {
      return createWrapper(func, REARG_FLAG, undefined, undefined, undefined, baseFlatten(indexes, 1));
    });
    function rest(func, start) {
      if (typeof func != 'function') {
        throw new TypeError(FUNC_ERROR_TEXT);
      }
      start = nativeMax(start === undefined ? (func.length - 1) : toInteger(start), 0);
      return function() {
        var args = arguments,
            index = -1,
            length = nativeMax(args.length - start, 0),
            array = Array(length);
        while (++index < length) {
          array[index] = args[start + index];
        }
        switch (start) {
          case 0:
            return func.call(this, array);
          case 1:
            return func.call(this, args[0], array);
          case 2:
            return func.call(this, args[0], args[1], array);
        }
        var otherArgs = Array(start + 1);
        index = -1;
        while (++index < start) {
          otherArgs[index] = args[index];
        }
        otherArgs[start] = array;
        return apply(func, this, otherArgs);
      };
    }
    function spread(func, start) {
      if (typeof func != 'function') {
        throw new TypeError(FUNC_ERROR_TEXT);
      }
      start = start === undefined ? 0 : nativeMax(toInteger(start), 0);
      return rest(function(args) {
        var array = args[start],
            otherArgs = args.slice(0, start);
        if (array) {
          arrayPush(otherArgs, array);
        }
        return apply(func, this, otherArgs);
      });
    }
    function throttle(func, wait, options) {
      var leading = true,
          trailing = true;
      if (typeof func != 'function') {
        throw new TypeError(FUNC_ERROR_TEXT);
      }
      if (isObject(options)) {
        leading = 'leading' in options ? !!options.leading : leading;
        trailing = 'trailing' in options ? !!options.trailing : trailing;
      }
      return debounce(func, wait, {
        'leading': leading,
        'maxWait': wait,
        'trailing': trailing
      });
    }
    function unary(func) {
      return ary(func, 1);
    }
    function wrap(value, wrapper) {
      wrapper = wrapper == null ? identity : wrapper;
      return partial(wrapper, value);
    }
    function castArray() {
      if (!arguments.length) {
        return [];
      }
      var value = arguments[0];
      return isArray(value) ? value : [value];
    }
    function clone(value) {
      return baseClone(value);
    }
    function cloneWith(value, customizer) {
      return baseClone(value, false, customizer);
    }
    function cloneDeep(value) {
      return baseClone(value, true);
    }
    function cloneDeepWith(value, customizer) {
      return baseClone(value, true, customizer);
    }
    function eq(value, other) {
      return value === other || (value !== value && other !== other);
    }
    function gt(value, other) {
      return value > other;
    }
    function gte(value, other) {
      return value >= other;
    }
    function isArguments(value) {
      return isArrayLikeObject(value) && hasOwnProperty.call(value, 'callee') && (!propertyIsEnumerable.call(value, 'callee') || objectToString.call(value) == argsTag);
    }
    var isArray = Array.isArray;
    function isArrayBuffer(value) {
      return isObjectLike(value) && objectToString.call(value) == arrayBufferTag;
    }
    function isArrayLike(value) {
      return value != null && !(typeof value == 'function' && isFunction(value)) && isLength(getLength(value));
    }
    function isArrayLikeObject(value) {
      return isObjectLike(value) && isArrayLike(value);
    }
    function isBoolean(value) {
      return value === true || value === false || (isObjectLike(value) && objectToString.call(value) == boolTag);
    }
    var isBuffer = !Buffer ? constant(false) : function(value) {
      return value instanceof Buffer;
    };
    function isDate(value) {
      return isObjectLike(value) && objectToString.call(value) == dateTag;
    }
    function isElement(value) {
      return !!value && value.nodeType === 1 && isObjectLike(value) && !isPlainObject(value);
    }
    function isEmpty(value) {
      if (isArrayLike(value) && (isArray(value) || isString(value) || isFunction(value.splice) || isArguments(value))) {
        return !value.length;
      }
      for (var key in value) {
        if (hasOwnProperty.call(value, key)) {
          return false;
        }
      }
      return true;
    }
    function isEqual(value, other) {
      return baseIsEqual(value, other);
    }
    function isEqualWith(value, other, customizer) {
      customizer = typeof customizer == 'function' ? customizer : undefined;
      var result = customizer ? customizer(value, other) : undefined;
      return result === undefined ? baseIsEqual(value, other, customizer) : !!result;
    }
    function isError(value) {
      if (!isObjectLike(value)) {
        return false;
      }
      return (objectToString.call(value) == errorTag) || (typeof value.message == 'string' && typeof value.name == 'string');
    }
    function isFinite(value) {
      return typeof value == 'number' && nativeIsFinite(value);
    }
    function isFunction(value) {
      var tag = isObject(value) ? objectToString.call(value) : '';
      return tag == funcTag || tag == genTag;
    }
    function isInteger(value) {
      return typeof value == 'number' && value == toInteger(value);
    }
    function isLength(value) {
      return typeof value == 'number' && value > -1 && value % 1 == 0 && value <= MAX_SAFE_INTEGER;
    }
    function isObject(value) {
      var type = typeof value;
      return !!value && (type == 'object' || type == 'function');
    }
    function isObjectLike(value) {
      return !!value && typeof value == 'object';
    }
    function isMap(value) {
      return isObjectLike(value) && getTag(value) == mapTag;
    }
    function isMatch(object, source) {
      return object === source || baseIsMatch(object, source, getMatchData(source));
    }
    function isMatchWith(object, source, customizer) {
      customizer = typeof customizer == 'function' ? customizer : undefined;
      return baseIsMatch(object, source, getMatchData(source), customizer);
    }
    function isNaN(value) {
      return isNumber(value) && value != +value;
    }
    function isNative(value) {
      if (value == null) {
        return false;
      }
      if (isFunction(value)) {
        return reIsNative.test(funcToString.call(value));
      }
      return isObjectLike(value) && (isHostObject(value) ? reIsNative : reIsHostCtor).test(value);
    }
    function isNull(value) {
      return value === null;
    }
    function isNil(value) {
      return value == null;
    }
    function isNumber(value) {
      return typeof value == 'number' || (isObjectLike(value) && objectToString.call(value) == numberTag);
    }
    function isPlainObject(value) {
      if (!isObjectLike(value) || objectToString.call(value) != objectTag || isHostObject(value)) {
        return false;
      }
      var proto = getPrototypeOf(value);
      if (proto === null) {
        return true;
      }
      var Ctor = proto.constructor;
      return (typeof Ctor == 'function' && Ctor instanceof Ctor && funcToString.call(Ctor) == objectCtorString);
    }
    function isRegExp(value) {
      return isObject(value) && objectToString.call(value) == regexpTag;
    }
    function isSafeInteger(value) {
      return isInteger(value) && value >= -MAX_SAFE_INTEGER && value <= MAX_SAFE_INTEGER;
    }
    function isSet(value) {
      return isObjectLike(value) && getTag(value) == setTag;
    }
    function isString(value) {
      return typeof value == 'string' || (!isArray(value) && isObjectLike(value) && objectToString.call(value) == stringTag);
    }
    function isSymbol(value) {
      return typeof value == 'symbol' || (isObjectLike(value) && objectToString.call(value) == symbolTag);
    }
    function isTypedArray(value) {
      return isObjectLike(value) && isLength(value.length) && !!typedArrayTags[objectToString.call(value)];
    }
    function isUndefined(value) {
      return value === undefined;
    }
    function isWeakMap(value) {
      return isObjectLike(value) && getTag(value) == weakMapTag;
    }
    function isWeakSet(value) {
      return isObjectLike(value) && objectToString.call(value) == weakSetTag;
    }
    function lt(value, other) {
      return value < other;
    }
    function lte(value, other) {
      return value <= other;
    }
    function toArray(value) {
      if (!value) {
        return [];
      }
      if (isArrayLike(value)) {
        return isString(value) ? stringToArray(value) : copyArray(value);
      }
      if (iteratorSymbol && value[iteratorSymbol]) {
        return iteratorToArray(value[iteratorSymbol]());
      }
      var tag = getTag(value),
          func = tag == mapTag ? mapToArray : (tag == setTag ? setToArray : values);
      return func(value);
    }
    function toInteger(value) {
      if (!value) {
        return value === 0 ? value : 0;
      }
      value = toNumber(value);
      if (value === INFINITY || value === -INFINITY) {
        var sign = (value < 0 ? -1 : 1);
        return sign * MAX_INTEGER;
      }
      var remainder = value % 1;
      return value === value ? (remainder ? value - remainder : value) : 0;
    }
    function toLength(value) {
      return value ? baseClamp(toInteger(value), 0, MAX_ARRAY_LENGTH) : 0;
    }
    function toNumber(value) {
      if (isObject(value)) {
        var other = isFunction(value.valueOf) ? value.valueOf() : value;
        value = isObject(other) ? (other + '') : other;
      }
      if (typeof value != 'string') {
        return value === 0 ? value : +value;
      }
      value = value.replace(reTrim, '');
      var isBinary = reIsBinary.test(value);
      return (isBinary || reIsOctal.test(value)) ? freeParseInt(value.slice(2), isBinary ? 2 : 8) : (reIsBadHex.test(value) ? NAN : +value);
    }
    function toPlainObject(value) {
      return copyObject(value, keysIn(value));
    }
    function toSafeInteger(value) {
      return baseClamp(toInteger(value), -MAX_SAFE_INTEGER, MAX_SAFE_INTEGER);
    }
    function toString(value) {
      if (typeof value == 'string') {
        return value;
      }
      if (value == null) {
        return '';
      }
      if (isSymbol(value)) {
        return Symbol ? symbolToString.call(value) : '';
      }
      var result = (value + '');
      return (result == '0' && (1 / value) == -INFINITY) ? '-0' : result;
    }
    var assign = createAssigner(function(object, source) {
      copyObject(source, keys(source), object);
    });
    var assignIn = createAssigner(function(object, source) {
      copyObject(source, keysIn(source), object);
    });
    var assignInWith = createAssigner(function(object, source, srcIndex, customizer) {
      copyObjectWith(source, keysIn(source), object, customizer);
    });
    var assignWith = createAssigner(function(object, source, srcIndex, customizer) {
      copyObjectWith(source, keys(source), object, customizer);
    });
    var at = rest(function(object, paths) {
      return baseAt(object, baseFlatten(paths, 1));
    });
    function create(prototype, properties) {
      var result = baseCreate(prototype);
      return properties ? baseAssign(result, properties) : result;
    }
    var defaults = rest(function(args) {
      args.push(undefined, assignInDefaults);
      return apply(assignInWith, undefined, args);
    });
    var defaultsDeep = rest(function(args) {
      args.push(undefined, mergeDefaults);
      return apply(mergeWith, undefined, args);
    });
    function findKey(object, predicate) {
      return baseFind(object, getIteratee(predicate, 3), baseForOwn, true);
    }
    function findLastKey(object, predicate) {
      return baseFind(object, getIteratee(predicate, 3), baseForOwnRight, true);
    }
    function forIn(object, iteratee) {
      return object == null ? object : baseFor(object, baseCastFunction(iteratee), keysIn);
    }
    function forInRight(object, iteratee) {
      return object == null ? object : baseForRight(object, baseCastFunction(iteratee), keysIn);
    }
    function forOwn(object, iteratee) {
      return object && baseForOwn(object, baseCastFunction(iteratee));
    }
    function forOwnRight(object, iteratee) {
      return object && baseForOwnRight(object, baseCastFunction(iteratee));
    }
    function functions(object) {
      return object == null ? [] : baseFunctions(object, keys(object));
    }
    function functionsIn(object) {
      return object == null ? [] : baseFunctions(object, keysIn(object));
    }
    function get(object, path, defaultValue) {
      var result = object == null ? undefined : baseGet(object, path);
      return result === undefined ? defaultValue : result;
    }
    function has(object, path) {
      return hasPath(object, path, baseHas);
    }
    function hasIn(object, path) {
      return hasPath(object, path, baseHasIn);
    }
    var invert = createInverter(function(result, value, key) {
      result[value] = key;
    }, constant(identity));
    var invertBy = createInverter(function(result, value, key) {
      if (hasOwnProperty.call(result, value)) {
        result[value].push(key);
      } else {
        result[value] = [key];
      }
    }, getIteratee);
    var invoke = rest(baseInvoke);
    function keys(object) {
      var isProto = isPrototype(object);
      if (!(isProto || isArrayLike(object))) {
        return baseKeys(object);
      }
      var indexes = indexKeys(object),
          skipIndexes = !!indexes,
          result = indexes || [],
          length = result.length;
      for (var key in object) {
        if (baseHas(object, key) && !(skipIndexes && (key == 'length' || isIndex(key, length))) && !(isProto && key == 'constructor')) {
          result.push(key);
        }
      }
      return result;
    }
    function keysIn(object) {
      var index = -1,
          isProto = isPrototype(object),
          props = baseKeysIn(object),
          propsLength = props.length,
          indexes = indexKeys(object),
          skipIndexes = !!indexes,
          result = indexes || [],
          length = result.length;
      while (++index < propsLength) {
        var key = props[index];
        if (!(skipIndexes && (key == 'length' || isIndex(key, length))) && !(key == 'constructor' && (isProto || !hasOwnProperty.call(object, key)))) {
          result.push(key);
        }
      }
      return result;
    }
    function mapKeys(object, iteratee) {
      var result = {};
      iteratee = getIteratee(iteratee, 3);
      baseForOwn(object, function(value, key, object) {
        result[iteratee(value, key, object)] = value;
      });
      return result;
    }
    function mapValues(object, iteratee) {
      var result = {};
      iteratee = getIteratee(iteratee, 3);
      baseForOwn(object, function(value, key, object) {
        result[key] = iteratee(value, key, object);
      });
      return result;
    }
    var merge = createAssigner(function(object, source, srcIndex) {
      baseMerge(object, source, srcIndex);
    });
    var mergeWith = createAssigner(function(object, source, srcIndex, customizer) {
      baseMerge(object, source, srcIndex, customizer);
    });
    var omit = rest(function(object, props) {
      if (object == null) {
        return {};
      }
      props = arrayMap(baseFlatten(props, 1), String);
      return basePick(object, baseDifference(keysIn(object), props));
    });
    function omitBy(object, predicate) {
      predicate = getIteratee(predicate);
      return basePickBy(object, function(value, key) {
        return !predicate(value, key);
      });
    }
    var pick = rest(function(object, props) {
      return object == null ? {} : basePick(object, baseFlatten(props, 1));
    });
    function pickBy(object, predicate) {
      return object == null ? {} : basePickBy(object, getIteratee(predicate));
    }
    function result(object, path, defaultValue) {
      if (!isKey(path, object)) {
        path = baseCastPath(path);
        var result = get(object, path);
        object = parent(object, path);
      } else {
        result = object == null ? undefined : object[path];
      }
      if (result === undefined) {
        result = defaultValue;
      }
      return isFunction(result) ? result.call(object) : result;
    }
    function set(object, path, value) {
      return object == null ? object : baseSet(object, path, value);
    }
    function setWith(object, path, value, customizer) {
      customizer = typeof customizer == 'function' ? customizer : undefined;
      return object == null ? object : baseSet(object, path, value, customizer);
    }
    function toPairs(object) {
      return baseToPairs(object, keys(object));
    }
    function toPairsIn(object) {
      return baseToPairs(object, keysIn(object));
    }
    function transform(object, iteratee, accumulator) {
      var isArr = isArray(object) || isTypedArray(object);
      iteratee = getIteratee(iteratee, 4);
      if (accumulator == null) {
        if (isArr || isObject(object)) {
          var Ctor = object.constructor;
          if (isArr) {
            accumulator = isArray(object) ? new Ctor : [];
          } else {
            accumulator = isFunction(Ctor) ? baseCreate(getPrototypeOf(object)) : {};
          }
        } else {
          accumulator = {};
        }
      }
      (isArr ? arrayEach : baseForOwn)(object, function(value, index, object) {
        return iteratee(accumulator, value, index, object);
      });
      return accumulator;
    }
    function unset(object, path) {
      return object == null ? true : baseUnset(object, path);
    }
    function values(object) {
      return object ? baseValues(object, keys(object)) : [];
    }
    function valuesIn(object) {
      return object == null ? [] : baseValues(object, keysIn(object));
    }
    function clamp(number, lower, upper) {
      if (upper === undefined) {
        upper = lower;
        lower = undefined;
      }
      if (upper !== undefined) {
        upper = toNumber(upper);
        upper = upper === upper ? upper : 0;
      }
      if (lower !== undefined) {
        lower = toNumber(lower);
        lower = lower === lower ? lower : 0;
      }
      return baseClamp(toNumber(number), lower, upper);
    }
    function inRange(number, start, end) {
      start = toNumber(start) || 0;
      if (end === undefined) {
        end = start;
        start = 0;
      } else {
        end = toNumber(end) || 0;
      }
      number = toNumber(number);
      return baseInRange(number, start, end);
    }
    function random(lower, upper, floating) {
      if (floating && typeof floating != 'boolean' && isIterateeCall(lower, upper, floating)) {
        upper = floating = undefined;
      }
      if (floating === undefined) {
        if (typeof upper == 'boolean') {
          floating = upper;
          upper = undefined;
        } else if (typeof lower == 'boolean') {
          floating = lower;
          lower = undefined;
        }
      }
      if (lower === undefined && upper === undefined) {
        lower = 0;
        upper = 1;
      } else {
        lower = toNumber(lower) || 0;
        if (upper === undefined) {
          upper = lower;
          lower = 0;
        } else {
          upper = toNumber(upper) || 0;
        }
      }
      if (lower > upper) {
        var temp = lower;
        lower = upper;
        upper = temp;
      }
      if (floating || lower % 1 || upper % 1) {
        var rand = nativeRandom();
        return nativeMin(lower + (rand * (upper - lower + freeParseFloat('1e-' + ((rand + '').length - 1)))), upper);
      }
      return baseRandom(lower, upper);
    }
    var camelCase = createCompounder(function(result, word, index) {
      word = word.toLowerCase();
      return result + (index ? capitalize(word) : word);
    });
    function capitalize(string) {
      return upperFirst(toString(string).toLowerCase());
    }
    function deburr(string) {
      string = toString(string);
      return string && string.replace(reLatin1, deburrLetter).replace(reComboMark, '');
    }
    function endsWith(string, target, position) {
      string = toString(string);
      target = typeof target == 'string' ? target : (target + '');
      var length = string.length;
      position = position === undefined ? length : baseClamp(toInteger(position), 0, length);
      position -= target.length;
      return position >= 0 && string.indexOf(target, position) == position;
    }
    function escape(string) {
      string = toString(string);
      return (string && reHasUnescapedHtml.test(string)) ? string.replace(reUnescapedHtml, escapeHtmlChar) : string;
    }
    function escapeRegExp(string) {
      string = toString(string);
      return (string && reHasRegExpChar.test(string)) ? string.replace(reRegExpChar, '\\$&') : string;
    }
    var kebabCase = createCompounder(function(result, word, index) {
      return result + (index ? '-' : '') + word.toLowerCase();
    });
    var lowerCase = createCompounder(function(result, word, index) {
      return result + (index ? ' ' : '') + word.toLowerCase();
    });
    var lowerFirst = createCaseFirst('toLowerCase');
    var upperFirst = createCaseFirst('toUpperCase');
    function pad(string, length, chars) {
      string = toString(string);
      length = toInteger(length);
      var strLength = stringSize(string);
      if (!length || strLength >= length) {
        return string;
      }
      var mid = (length - strLength) / 2,
          leftLength = nativeFloor(mid),
          rightLength = nativeCeil(mid);
      return createPadding('', leftLength, chars) + string + createPadding('', rightLength, chars);
    }
    function padEnd(string, length, chars) {
      string = toString(string);
      return string + createPadding(string, length, chars);
    }
    function padStart(string, length, chars) {
      string = toString(string);
      return createPadding(string, length, chars) + string;
    }
    function parseInt(string, radix, guard) {
      if (guard || radix == null) {
        radix = 0;
      } else if (radix) {
        radix = +radix;
      }
      string = toString(string).replace(reTrim, '');
      return nativeParseInt(string, radix || (reHasHexPrefix.test(string) ? 16 : 10));
    }
    function repeat(string, n) {
      string = toString(string);
      n = toInteger(n);
      var result = '';
      if (!string || n < 1 || n > MAX_SAFE_INTEGER) {
        return result;
      }
      do {
        if (n % 2) {
          result += string;
        }
        n = nativeFloor(n / 2);
        string += string;
      } while (n);
      return result;
    }
    function replace() {
      var args = arguments,
          string = toString(args[0]);
      return args.length < 3 ? string : string.replace(args[1], args[2]);
    }
    var snakeCase = createCompounder(function(result, word, index) {
      return result + (index ? '_' : '') + word.toLowerCase();
    });
    function split(string, separator, limit) {
      return toString(string).split(separator, limit);
    }
    var startCase = createCompounder(function(result, word, index) {
      return result + (index ? ' ' : '') + capitalize(word);
    });
    function startsWith(string, target, position) {
      string = toString(string);
      position = baseClamp(toInteger(position), 0, string.length);
      return string.lastIndexOf(target, position) == position;
    }
    function template(string, options, guard) {
      var settings = lodash.templateSettings;
      if (guard && isIterateeCall(string, options, guard)) {
        options = undefined;
      }
      string = toString(string);
      options = assignInWith({}, options, settings, assignInDefaults);
      var imports = assignInWith({}, options.imports, settings.imports, assignInDefaults),
          importsKeys = keys(imports),
          importsValues = baseValues(imports, importsKeys);
      var isEscaping,
          isEvaluating,
          index = 0,
          interpolate = options.interpolate || reNoMatch,
          source = "__p += '";
      var reDelimiters = RegExp((options.escape || reNoMatch).source + '|' + interpolate.source + '|' + (interpolate === reInterpolate ? reEsTemplate : reNoMatch).source + '|' + (options.evaluate || reNoMatch).source + '|$', 'g');
      var sourceURL = '//# sourceURL=' + ('sourceURL' in options ? options.sourceURL : ('lodash.templateSources[' + (++templateCounter) + ']')) + '\n';
      string.replace(reDelimiters, function(match, escapeValue, interpolateValue, esTemplateValue, evaluateValue, offset) {
        interpolateValue || (interpolateValue = esTemplateValue);
        source += string.slice(index, offset).replace(reUnescapedString, escapeStringChar);
        if (escapeValue) {
          isEscaping = true;
          source += "' +\n__e(" + escapeValue + ") +\n'";
        }
        if (evaluateValue) {
          isEvaluating = true;
          source += "';\n" + evaluateValue + ";\n__p += '";
        }
        if (interpolateValue) {
          source += "' +\n((__t = (" + interpolateValue + ")) == null ? '' : __t) +\n'";
        }
        index = offset + match.length;
        return match;
      });
      source += "';\n";
      var variable = options.variable;
      if (!variable) {
        source = 'with (obj) {\n' + source + '\n}\n';
      }
      source = (isEvaluating ? source.replace(reEmptyStringLeading, '') : source).replace(reEmptyStringMiddle, '$1').replace(reEmptyStringTrailing, '$1;');
      source = 'function(' + (variable || 'obj') + ') {\n' + (variable ? '' : 'obj || (obj = {});\n') + "var __t, __p = ''" + (isEscaping ? ', __e = _.escape' : '') + (isEvaluating ? ', __j = Array.prototype.join;\n' + "function print() { __p += __j.call(arguments, '') }\n" : ';\n') + source + 'return __p\n}';
      var result = attempt(function() {
        return Function(importsKeys, sourceURL + 'return ' + source).apply(undefined, importsValues);
      });
      result.source = source;
      if (isError(result)) {
        throw result;
      }
      return result;
    }
    function toLower(value) {
      return toString(value).toLowerCase();
    }
    function toUpper(value) {
      return toString(value).toUpperCase();
    }
    function trim(string, chars, guard) {
      string = toString(string);
      if (!string) {
        return string;
      }
      if (guard || chars === undefined) {
        return string.replace(reTrim, '');
      }
      chars = (chars + '');
      if (!chars) {
        return string;
      }
      var strSymbols = stringToArray(string),
          chrSymbols = stringToArray(chars);
      return strSymbols.slice(charsStartIndex(strSymbols, chrSymbols), charsEndIndex(strSymbols, chrSymbols) + 1).join('');
    }
    function trimEnd(string, chars, guard) {
      string = toString(string);
      if (!string) {
        return string;
      }
      if (guard || chars === undefined) {
        return string.replace(reTrimEnd, '');
      }
      chars = (chars + '');
      if (!chars) {
        return string;
      }
      var strSymbols = stringToArray(string);
      return strSymbols.slice(0, charsEndIndex(strSymbols, stringToArray(chars)) + 1).join('');
    }
    function trimStart(string, chars, guard) {
      string = toString(string);
      if (!string) {
        return string;
      }
      if (guard || chars === undefined) {
        return string.replace(reTrimStart, '');
      }
      chars = (chars + '');
      if (!chars) {
        return string;
      }
      var strSymbols = stringToArray(string);
      return strSymbols.slice(charsStartIndex(strSymbols, stringToArray(chars))).join('');
    }
    function truncate(string, options) {
      var length = DEFAULT_TRUNC_LENGTH,
          omission = DEFAULT_TRUNC_OMISSION;
      if (isObject(options)) {
        var separator = 'separator' in options ? options.separator : separator;
        length = 'length' in options ? toInteger(options.length) : length;
        omission = 'omission' in options ? toString(options.omission) : omission;
      }
      string = toString(string);
      var strLength = string.length;
      if (reHasComplexSymbol.test(string)) {
        var strSymbols = stringToArray(string);
        strLength = strSymbols.length;
      }
      if (length >= strLength) {
        return string;
      }
      var end = length - stringSize(omission);
      if (end < 1) {
        return omission;
      }
      var result = strSymbols ? strSymbols.slice(0, end).join('') : string.slice(0, end);
      if (separator === undefined) {
        return result + omission;
      }
      if (strSymbols) {
        end += (result.length - end);
      }
      if (isRegExp(separator)) {
        if (string.slice(end).search(separator)) {
          var match,
              substring = result;
          if (!separator.global) {
            separator = RegExp(separator.source, toString(reFlags.exec(separator)) + 'g');
          }
          separator.lastIndex = 0;
          while ((match = separator.exec(substring))) {
            var newEnd = match.index;
          }
          result = result.slice(0, newEnd === undefined ? end : newEnd);
        }
      } else if (string.indexOf(separator, end) != end) {
        var index = result.lastIndexOf(separator);
        if (index > -1) {
          result = result.slice(0, index);
        }
      }
      return result + omission;
    }
    function unescape(string) {
      string = toString(string);
      return (string && reHasEscapedHtml.test(string)) ? string.replace(reEscapedHtml, unescapeHtmlChar) : string;
    }
    var upperCase = createCompounder(function(result, word, index) {
      return result + (index ? ' ' : '') + word.toUpperCase();
    });
    function words(string, pattern, guard) {
      string = toString(string);
      pattern = guard ? undefined : pattern;
      if (pattern === undefined) {
        pattern = reHasComplexWord.test(string) ? reComplexWord : reBasicWord;
      }
      return string.match(pattern) || [];
    }
    var attempt = rest(function(func, args) {
      try {
        return apply(func, undefined, args);
      } catch (e) {
        return isError(e) ? e : new Error(e);
      }
    });
    var bindAll = rest(function(object, methodNames) {
      arrayEach(baseFlatten(methodNames, 1), function(key) {
        object[key] = bind(object[key], object);
      });
      return object;
    });
    function cond(pairs) {
      var length = pairs ? pairs.length : 0,
          toIteratee = getIteratee();
      pairs = !length ? [] : arrayMap(pairs, function(pair) {
        if (typeof pair[1] != 'function') {
          throw new TypeError(FUNC_ERROR_TEXT);
        }
        return [toIteratee(pair[0]), pair[1]];
      });
      return rest(function(args) {
        var index = -1;
        while (++index < length) {
          var pair = pairs[index];
          if (apply(pair[0], this, args)) {
            return apply(pair[1], this, args);
          }
        }
      });
    }
    function conforms(source) {
      return baseConforms(baseClone(source, true));
    }
    function constant(value) {
      return function() {
        return value;
      };
    }
    var flow = createFlow();
    var flowRight = createFlow(true);
    function identity(value) {
      return value;
    }
    function iteratee(func) {
      return baseIteratee(typeof func == 'function' ? func : baseClone(func, true));
    }
    function matches(source) {
      return baseMatches(baseClone(source, true));
    }
    function matchesProperty(path, srcValue) {
      return baseMatchesProperty(path, baseClone(srcValue, true));
    }
    var method = rest(function(path, args) {
      return function(object) {
        return baseInvoke(object, path, args);
      };
    });
    var methodOf = rest(function(object, args) {
      return function(path) {
        return baseInvoke(object, path, args);
      };
    });
    function mixin(object, source, options) {
      var props = keys(source),
          methodNames = baseFunctions(source, props);
      if (options == null && !(isObject(source) && (methodNames.length || !props.length))) {
        options = source;
        source = object;
        object = this;
        methodNames = baseFunctions(source, keys(source));
      }
      var chain = (isObject(options) && 'chain' in options) ? options.chain : true,
          isFunc = isFunction(object);
      arrayEach(methodNames, function(methodName) {
        var func = source[methodName];
        object[methodName] = func;
        if (isFunc) {
          object.prototype[methodName] = function() {
            var chainAll = this.__chain__;
            if (chain || chainAll) {
              var result = object(this.__wrapped__),
                  actions = result.__actions__ = copyArray(this.__actions__);
              actions.push({
                'func': func,
                'args': arguments,
                'thisArg': object
              });
              result.__chain__ = chainAll;
              return result;
            }
            return func.apply(object, arrayPush([this.value()], arguments));
          };
        }
      });
      return object;
    }
    function noConflict() {
      if (root._ === this) {
        root._ = oldDash;
      }
      return this;
    }
    function noop() {}
    function nthArg(n) {
      n = toInteger(n);
      return function() {
        return arguments[n];
      };
    }
    var over = createOver(arrayMap);
    var overEvery = createOver(arrayEvery);
    var overSome = createOver(arraySome);
    function property(path) {
      return isKey(path) ? baseProperty(path) : basePropertyDeep(path);
    }
    function propertyOf(object) {
      return function(path) {
        return object == null ? undefined : baseGet(object, path);
      };
    }
    var range = createRange();
    var rangeRight = createRange(true);
    function times(n, iteratee) {
      n = toInteger(n);
      if (n < 1 || n > MAX_SAFE_INTEGER) {
        return [];
      }
      var index = MAX_ARRAY_LENGTH,
          length = nativeMin(n, MAX_ARRAY_LENGTH);
      iteratee = baseCastFunction(iteratee);
      n -= MAX_ARRAY_LENGTH;
      var result = baseTimes(length, iteratee);
      while (++index < n) {
        iteratee(index);
      }
      return result;
    }
    function toPath(value) {
      return isArray(value) ? arrayMap(value, String) : stringToPath(value);
    }
    function uniqueId(prefix) {
      var id = ++idCounter;
      return toString(prefix) + id;
    }
    function add(augend, addend) {
      var result;
      if (augend === undefined && addend === undefined) {
        return 0;
      }
      if (augend !== undefined) {
        result = augend;
      }
      if (addend !== undefined) {
        result = result === undefined ? addend : (result + addend);
      }
      return result;
    }
    var ceil = createRound('ceil');
    var floor = createRound('floor');
    function max(array) {
      return (array && array.length) ? baseExtremum(array, identity, gt) : undefined;
    }
    function maxBy(array, iteratee) {
      return (array && array.length) ? baseExtremum(array, getIteratee(iteratee), gt) : undefined;
    }
    function mean(array) {
      return sum(array) / (array ? array.length : 0);
    }
    function min(array) {
      return (array && array.length) ? baseExtremum(array, identity, lt) : undefined;
    }
    function minBy(array, iteratee) {
      return (array && array.length) ? baseExtremum(array, getIteratee(iteratee), lt) : undefined;
    }
    var round = createRound('round');
    function subtract(minuend, subtrahend) {
      var result;
      if (minuend === undefined && subtrahend === undefined) {
        return 0;
      }
      if (minuend !== undefined) {
        result = minuend;
      }
      if (subtrahend !== undefined) {
        result = result === undefined ? subtrahend : (result - subtrahend);
      }
      return result;
    }
    function sum(array) {
      return (array && array.length) ? baseSum(array, identity) : 0;
    }
    function sumBy(array, iteratee) {
      return (array && array.length) ? baseSum(array, getIteratee(iteratee)) : 0;
    }
    lodash.prototype = baseLodash.prototype;
    LodashWrapper.prototype = baseCreate(baseLodash.prototype);
    LodashWrapper.prototype.constructor = LodashWrapper;
    LazyWrapper.prototype = baseCreate(baseLodash.prototype);
    LazyWrapper.prototype.constructor = LazyWrapper;
    Hash.prototype = nativeCreate ? nativeCreate(null) : objectProto;
    MapCache.prototype.clear = mapClear;
    MapCache.prototype['delete'] = mapDelete;
    MapCache.prototype.get = mapGet;
    MapCache.prototype.has = mapHas;
    MapCache.prototype.set = mapSet;
    SetCache.prototype.push = cachePush;
    Stack.prototype.clear = stackClear;
    Stack.prototype['delete'] = stackDelete;
    Stack.prototype.get = stackGet;
    Stack.prototype.has = stackHas;
    Stack.prototype.set = stackSet;
    memoize.Cache = MapCache;
    lodash.after = after;
    lodash.ary = ary;
    lodash.assign = assign;
    lodash.assignIn = assignIn;
    lodash.assignInWith = assignInWith;
    lodash.assignWith = assignWith;
    lodash.at = at;
    lodash.before = before;
    lodash.bind = bind;
    lodash.bindAll = bindAll;
    lodash.bindKey = bindKey;
    lodash.castArray = castArray;
    lodash.chain = chain;
    lodash.chunk = chunk;
    lodash.compact = compact;
    lodash.concat = concat;
    lodash.cond = cond;
    lodash.conforms = conforms;
    lodash.constant = constant;
    lodash.countBy = countBy;
    lodash.create = create;
    lodash.curry = curry;
    lodash.curryRight = curryRight;
    lodash.debounce = debounce;
    lodash.defaults = defaults;
    lodash.defaultsDeep = defaultsDeep;
    lodash.defer = defer;
    lodash.delay = delay;
    lodash.difference = difference;
    lodash.differenceBy = differenceBy;
    lodash.differenceWith = differenceWith;
    lodash.drop = drop;
    lodash.dropRight = dropRight;
    lodash.dropRightWhile = dropRightWhile;
    lodash.dropWhile = dropWhile;
    lodash.fill = fill;
    lodash.filter = filter;
    lodash.flatMap = flatMap;
    lodash.flatten = flatten;
    lodash.flattenDeep = flattenDeep;
    lodash.flattenDepth = flattenDepth;
    lodash.flip = flip;
    lodash.flow = flow;
    lodash.flowRight = flowRight;
    lodash.fromPairs = fromPairs;
    lodash.functions = functions;
    lodash.functionsIn = functionsIn;
    lodash.groupBy = groupBy;
    lodash.initial = initial;
    lodash.intersection = intersection;
    lodash.intersectionBy = intersectionBy;
    lodash.intersectionWith = intersectionWith;
    lodash.invert = invert;
    lodash.invertBy = invertBy;
    lodash.invokeMap = invokeMap;
    lodash.iteratee = iteratee;
    lodash.keyBy = keyBy;
    lodash.keys = keys;
    lodash.keysIn = keysIn;
    lodash.map = map;
    lodash.mapKeys = mapKeys;
    lodash.mapValues = mapValues;
    lodash.matches = matches;
    lodash.matchesProperty = matchesProperty;
    lodash.memoize = memoize;
    lodash.merge = merge;
    lodash.mergeWith = mergeWith;
    lodash.method = method;
    lodash.methodOf = methodOf;
    lodash.mixin = mixin;
    lodash.negate = negate;
    lodash.nthArg = nthArg;
    lodash.omit = omit;
    lodash.omitBy = omitBy;
    lodash.once = once;
    lodash.orderBy = orderBy;
    lodash.over = over;
    lodash.overArgs = overArgs;
    lodash.overEvery = overEvery;
    lodash.overSome = overSome;
    lodash.partial = partial;
    lodash.partialRight = partialRight;
    lodash.partition = partition;
    lodash.pick = pick;
    lodash.pickBy = pickBy;
    lodash.property = property;
    lodash.propertyOf = propertyOf;
    lodash.pull = pull;
    lodash.pullAll = pullAll;
    lodash.pullAllBy = pullAllBy;
    lodash.pullAt = pullAt;
    lodash.range = range;
    lodash.rangeRight = rangeRight;
    lodash.rearg = rearg;
    lodash.reject = reject;
    lodash.remove = remove;
    lodash.rest = rest;
    lodash.reverse = reverse;
    lodash.sampleSize = sampleSize;
    lodash.set = set;
    lodash.setWith = setWith;
    lodash.shuffle = shuffle;
    lodash.slice = slice;
    lodash.sortBy = sortBy;
    lodash.sortedUniq = sortedUniq;
    lodash.sortedUniqBy = sortedUniqBy;
    lodash.split = split;
    lodash.spread = spread;
    lodash.tail = tail;
    lodash.take = take;
    lodash.takeRight = takeRight;
    lodash.takeRightWhile = takeRightWhile;
    lodash.takeWhile = takeWhile;
    lodash.tap = tap;
    lodash.throttle = throttle;
    lodash.thru = thru;
    lodash.toArray = toArray;
    lodash.toPairs = toPairs;
    lodash.toPairsIn = toPairsIn;
    lodash.toPath = toPath;
    lodash.toPlainObject = toPlainObject;
    lodash.transform = transform;
    lodash.unary = unary;
    lodash.union = union;
    lodash.unionBy = unionBy;
    lodash.unionWith = unionWith;
    lodash.uniq = uniq;
    lodash.uniqBy = uniqBy;
    lodash.uniqWith = uniqWith;
    lodash.unset = unset;
    lodash.unzip = unzip;
    lodash.unzipWith = unzipWith;
    lodash.values = values;
    lodash.valuesIn = valuesIn;
    lodash.without = without;
    lodash.words = words;
    lodash.wrap = wrap;
    lodash.xor = xor;
    lodash.xorBy = xorBy;
    lodash.xorWith = xorWith;
    lodash.zip = zip;
    lodash.zipObject = zipObject;
    lodash.zipObjectDeep = zipObjectDeep;
    lodash.zipWith = zipWith;
    lodash.extend = assignIn;
    lodash.extendWith = assignInWith;
    mixin(lodash, lodash);
    lodash.add = add;
    lodash.attempt = attempt;
    lodash.camelCase = camelCase;
    lodash.capitalize = capitalize;
    lodash.ceil = ceil;
    lodash.clamp = clamp;
    lodash.clone = clone;
    lodash.cloneDeep = cloneDeep;
    lodash.cloneDeepWith = cloneDeepWith;
    lodash.cloneWith = cloneWith;
    lodash.deburr = deburr;
    lodash.endsWith = endsWith;
    lodash.eq = eq;
    lodash.escape = escape;
    lodash.escapeRegExp = escapeRegExp;
    lodash.every = every;
    lodash.find = find;
    lodash.findIndex = findIndex;
    lodash.findKey = findKey;
    lodash.findLast = findLast;
    lodash.findLastIndex = findLastIndex;
    lodash.findLastKey = findLastKey;
    lodash.floor = floor;
    lodash.forEach = forEach;
    lodash.forEachRight = forEachRight;
    lodash.forIn = forIn;
    lodash.forInRight = forInRight;
    lodash.forOwn = forOwn;
    lodash.forOwnRight = forOwnRight;
    lodash.get = get;
    lodash.gt = gt;
    lodash.gte = gte;
    lodash.has = has;
    lodash.hasIn = hasIn;
    lodash.head = head;
    lodash.identity = identity;
    lodash.includes = includes;
    lodash.indexOf = indexOf;
    lodash.inRange = inRange;
    lodash.invoke = invoke;
    lodash.isArguments = isArguments;
    lodash.isArray = isArray;
    lodash.isArrayBuffer = isArrayBuffer;
    lodash.isArrayLike = isArrayLike;
    lodash.isArrayLikeObject = isArrayLikeObject;
    lodash.isBoolean = isBoolean;
    lodash.isBuffer = isBuffer;
    lodash.isDate = isDate;
    lodash.isElement = isElement;
    lodash.isEmpty = isEmpty;
    lodash.isEqual = isEqual;
    lodash.isEqualWith = isEqualWith;
    lodash.isError = isError;
    lodash.isFinite = isFinite;
    lodash.isFunction = isFunction;
    lodash.isInteger = isInteger;
    lodash.isLength = isLength;
    lodash.isMap = isMap;
    lodash.isMatch = isMatch;
    lodash.isMatchWith = isMatchWith;
    lodash.isNaN = isNaN;
    lodash.isNative = isNative;
    lodash.isNil = isNil;
    lodash.isNull = isNull;
    lodash.isNumber = isNumber;
    lodash.isObject = isObject;
    lodash.isObjectLike = isObjectLike;
    lodash.isPlainObject = isPlainObject;
    lodash.isRegExp = isRegExp;
    lodash.isSafeInteger = isSafeInteger;
    lodash.isSet = isSet;
    lodash.isString = isString;
    lodash.isSymbol = isSymbol;
    lodash.isTypedArray = isTypedArray;
    lodash.isUndefined = isUndefined;
    lodash.isWeakMap = isWeakMap;
    lodash.isWeakSet = isWeakSet;
    lodash.join = join;
    lodash.kebabCase = kebabCase;
    lodash.last = last;
    lodash.lastIndexOf = lastIndexOf;
    lodash.lowerCase = lowerCase;
    lodash.lowerFirst = lowerFirst;
    lodash.lt = lt;
    lodash.lte = lte;
    lodash.max = max;
    lodash.maxBy = maxBy;
    lodash.mean = mean;
    lodash.min = min;
    lodash.minBy = minBy;
    lodash.noConflict = noConflict;
    lodash.noop = noop;
    lodash.now = now;
    lodash.pad = pad;
    lodash.padEnd = padEnd;
    lodash.padStart = padStart;
    lodash.parseInt = parseInt;
    lodash.random = random;
    lodash.reduce = reduce;
    lodash.reduceRight = reduceRight;
    lodash.repeat = repeat;
    lodash.replace = replace;
    lodash.result = result;
    lodash.round = round;
    lodash.runInContext = runInContext;
    lodash.sample = sample;
    lodash.size = size;
    lodash.snakeCase = snakeCase;
    lodash.some = some;
    lodash.sortedIndex = sortedIndex;
    lodash.sortedIndexBy = sortedIndexBy;
    lodash.sortedIndexOf = sortedIndexOf;
    lodash.sortedLastIndex = sortedLastIndex;
    lodash.sortedLastIndexBy = sortedLastIndexBy;
    lodash.sortedLastIndexOf = sortedLastIndexOf;
    lodash.startCase = startCase;
    lodash.startsWith = startsWith;
    lodash.subtract = subtract;
    lodash.sum = sum;
    lodash.sumBy = sumBy;
    lodash.template = template;
    lodash.times = times;
    lodash.toInteger = toInteger;
    lodash.toLength = toLength;
    lodash.toLower = toLower;
    lodash.toNumber = toNumber;
    lodash.toSafeInteger = toSafeInteger;
    lodash.toString = toString;
    lodash.toUpper = toUpper;
    lodash.trim = trim;
    lodash.trimEnd = trimEnd;
    lodash.trimStart = trimStart;
    lodash.truncate = truncate;
    lodash.unescape = unescape;
    lodash.uniqueId = uniqueId;
    lodash.upperCase = upperCase;
    lodash.upperFirst = upperFirst;
    lodash.each = forEach;
    lodash.eachRight = forEachRight;
    lodash.first = head;
    mixin(lodash, (function() {
      var source = {};
      baseForOwn(lodash, function(func, methodName) {
        if (!hasOwnProperty.call(lodash.prototype, methodName)) {
          source[methodName] = func;
        }
      });
      return source;
    }()), {'chain': false});
    lodash.VERSION = VERSION;
    arrayEach(['bind', 'bindKey', 'curry', 'curryRight', 'partial', 'partialRight'], function(methodName) {
      lodash[methodName].placeholder = lodash;
    });
    arrayEach(['drop', 'take'], function(methodName, index) {
      LazyWrapper.prototype[methodName] = function(n) {
        var filtered = this.__filtered__;
        if (filtered && !index) {
          return new LazyWrapper(this);
        }
        n = n === undefined ? 1 : nativeMax(toInteger(n), 0);
        var result = this.clone();
        if (filtered) {
          result.__takeCount__ = nativeMin(n, result.__takeCount__);
        } else {
          result.__views__.push({
            'size': nativeMin(n, MAX_ARRAY_LENGTH),
            'type': methodName + (result.__dir__ < 0 ? 'Right' : '')
          });
        }
        return result;
      };
      LazyWrapper.prototype[methodName + 'Right'] = function(n) {
        return this.reverse()[methodName](n).reverse();
      };
    });
    arrayEach(['filter', 'map', 'takeWhile'], function(methodName, index) {
      var type = index + 1,
          isFilter = type == LAZY_FILTER_FLAG || type == LAZY_WHILE_FLAG;
      LazyWrapper.prototype[methodName] = function(iteratee) {
        var result = this.clone();
        result.__iteratees__.push({
          'iteratee': getIteratee(iteratee, 3),
          'type': type
        });
        result.__filtered__ = result.__filtered__ || isFilter;
        return result;
      };
    });
    arrayEach(['head', 'last'], function(methodName, index) {
      var takeName = 'take' + (index ? 'Right' : '');
      LazyWrapper.prototype[methodName] = function() {
        return this[takeName](1).value()[0];
      };
    });
    arrayEach(['initial', 'tail'], function(methodName, index) {
      var dropName = 'drop' + (index ? '' : 'Right');
      LazyWrapper.prototype[methodName] = function() {
        return this.__filtered__ ? new LazyWrapper(this) : this[dropName](1);
      };
    });
    LazyWrapper.prototype.compact = function() {
      return this.filter(identity);
    };
    LazyWrapper.prototype.find = function(predicate) {
      return this.filter(predicate).head();
    };
    LazyWrapper.prototype.findLast = function(predicate) {
      return this.reverse().find(predicate);
    };
    LazyWrapper.prototype.invokeMap = rest(function(path, args) {
      if (typeof path == 'function') {
        return new LazyWrapper(this);
      }
      return this.map(function(value) {
        return baseInvoke(value, path, args);
      });
    });
    LazyWrapper.prototype.reject = function(predicate) {
      predicate = getIteratee(predicate, 3);
      return this.filter(function(value) {
        return !predicate(value);
      });
    };
    LazyWrapper.prototype.slice = function(start, end) {
      start = toInteger(start);
      var result = this;
      if (result.__filtered__ && (start > 0 || end < 0)) {
        return new LazyWrapper(result);
      }
      if (start < 0) {
        result = result.takeRight(-start);
      } else if (start) {
        result = result.drop(start);
      }
      if (end !== undefined) {
        end = toInteger(end);
        result = end < 0 ? result.dropRight(-end) : result.take(end - start);
      }
      return result;
    };
    LazyWrapper.prototype.takeRightWhile = function(predicate) {
      return this.reverse().takeWhile(predicate).reverse();
    };
    LazyWrapper.prototype.toArray = function() {
      return this.take(MAX_ARRAY_LENGTH);
    };
    baseForOwn(LazyWrapper.prototype, function(func, methodName) {
      var checkIteratee = /^(?:filter|find|map|reject)|While$/.test(methodName),
          isTaker = /^(?:head|last)$/.test(methodName),
          lodashFunc = lodash[isTaker ? ('take' + (methodName == 'last' ? 'Right' : '')) : methodName],
          retUnwrapped = isTaker || /^find/.test(methodName);
      if (!lodashFunc) {
        return;
      }
      lodash.prototype[methodName] = function() {
        var value = this.__wrapped__,
            args = isTaker ? [1] : arguments,
            isLazy = value instanceof LazyWrapper,
            iteratee = args[0],
            useLazy = isLazy || isArray(value);
        var interceptor = function(value) {
          var result = lodashFunc.apply(lodash, arrayPush([value], args));
          return (isTaker && chainAll) ? result[0] : result;
        };
        if (useLazy && checkIteratee && typeof iteratee == 'function' && iteratee.length != 1) {
          isLazy = useLazy = false;
        }
        var chainAll = this.__chain__,
            isHybrid = !!this.__actions__.length,
            isUnwrapped = retUnwrapped && !chainAll,
            onlyLazy = isLazy && !isHybrid;
        if (!retUnwrapped && useLazy) {
          value = onlyLazy ? value : new LazyWrapper(this);
          var result = func.apply(value, args);
          result.__actions__.push({
            'func': thru,
            'args': [interceptor],
            'thisArg': undefined
          });
          return new LodashWrapper(result, chainAll);
        }
        if (isUnwrapped && onlyLazy) {
          return func.apply(this, args);
        }
        result = this.thru(interceptor);
        return isUnwrapped ? (isTaker ? result.value()[0] : result.value()) : result;
      };
    });
    arrayEach(['pop', 'push', 'shift', 'sort', 'splice', 'unshift'], function(methodName) {
      var func = arrayProto[methodName],
          chainName = /^(?:push|sort|unshift)$/.test(methodName) ? 'tap' : 'thru',
          retUnwrapped = /^(?:pop|shift)$/.test(methodName);
      lodash.prototype[methodName] = function() {
        var args = arguments;
        if (retUnwrapped && !this.__chain__) {
          return func.apply(this.value(), args);
        }
        return this[chainName](function(value) {
          return func.apply(value, args);
        });
      };
    });
    baseForOwn(LazyWrapper.prototype, function(func, methodName) {
      var lodashFunc = lodash[methodName];
      if (lodashFunc) {
        var key = (lodashFunc.name + ''),
            names = realNames[key] || (realNames[key] = []);
        names.push({
          'name': methodName,
          'func': lodashFunc
        });
      }
    });
    realNames[createHybridWrapper(undefined, BIND_KEY_FLAG).name] = [{
      'name': 'wrapper',
      'func': undefined
    }];
    LazyWrapper.prototype.clone = lazyClone;
    LazyWrapper.prototype.reverse = lazyReverse;
    LazyWrapper.prototype.value = lazyValue;
    lodash.prototype.at = wrapperAt;
    lodash.prototype.chain = wrapperChain;
    lodash.prototype.commit = wrapperCommit;
    lodash.prototype.flatMap = wrapperFlatMap;
    lodash.prototype.next = wrapperNext;
    lodash.prototype.plant = wrapperPlant;
    lodash.prototype.reverse = wrapperReverse;
    lodash.prototype.toJSON = lodash.prototype.valueOf = lodash.prototype.value = wrapperValue;
    if (iteratorSymbol) {
      lodash.prototype[iteratorSymbol] = wrapperToIterator;
    }
    return lodash;
  }
  var _ = runInContext();
  (freeWindow || freeSelf || {})._ = _;
  if (typeof define == 'function' && typeof define.amd == 'object' && define.amd) {
    define("node_modules/lodash/lodash.js", [], function() {
      return _;
    });
  } else if (freeExports && freeModule) {
    if (moduleExports) {
      (freeModule.exports = _)._ = _;
    }
    freeExports._ = _;
  } else {
    root._ = _;
  }
}.call(this));

})();
System.registerDynamic('node_modules/lodash/index.js', ['./lodash'], true, function ($__require, exports, module) {
  var global = this || self,
      GLOBAL = global;
  module.exports = $__require('./lodash');
});
(function() {
var define = System.amdDefine;
;
(function(global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() : typeof define === 'function' && define.amd ? define("node_modules/moment/moment.js", [], factory) : global.moment = factory();
}(this, function() {
  'use strict';
  var hookCallback;
  function utils_hooks__hooks() {
    return hookCallback.apply(null, arguments);
  }
  function setHookCallback(callback) {
    hookCallback = callback;
  }
  function isArray(input) {
    return input instanceof Array || Object.prototype.toString.call(input) === '[object Array]';
  }
  function isObject(input) {
    return Object.prototype.toString.call(input) === '[object Object]';
  }
  function isObjectEmpty(obj) {
    var k;
    for (k in obj) {
      return false;
    }
    return true;
  }
  function isDate(input) {
    return input instanceof Date || Object.prototype.toString.call(input) === '[object Date]';
  }
  function map(arr, fn) {
    var res = [],
        i;
    for (i = 0; i < arr.length; ++i) {
      res.push(fn(arr[i], i));
    }
    return res;
  }
  function hasOwnProp(a, b) {
    return Object.prototype.hasOwnProperty.call(a, b);
  }
  function extend(a, b) {
    for (var i in b) {
      if (hasOwnProp(b, i)) {
        a[i] = b[i];
      }
    }
    if (hasOwnProp(b, 'toString')) {
      a.toString = b.toString;
    }
    if (hasOwnProp(b, 'valueOf')) {
      a.valueOf = b.valueOf;
    }
    return a;
  }
  function create_utc__createUTC(input, format, locale, strict) {
    return createLocalOrUTC(input, format, locale, strict, true).utc();
  }
  function defaultParsingFlags() {
    return {
      empty: false,
      unusedTokens: [],
      unusedInput: [],
      overflow: -2,
      charsLeftOver: 0,
      nullInput: false,
      invalidMonth: null,
      invalidFormat: false,
      userInvalidated: false,
      iso: false,
      parsedDateParts: [],
      meridiem: null
    };
  }
  function getParsingFlags(m) {
    if (m._pf == null) {
      m._pf = defaultParsingFlags();
    }
    return m._pf;
  }
  var some;
  if (Array.prototype.some) {
    some = Array.prototype.some;
  } else {
    some = function(fun) {
      var t = Object(this);
      var len = t.length >>> 0;
      for (var i = 0; i < len; i++) {
        if (i in t && fun.call(this, t[i], i, t)) {
          return true;
        }
      }
      return false;
    };
  }
  function valid__isValid(m) {
    if (m._isValid == null) {
      var flags = getParsingFlags(m);
      var parsedParts = some.call(flags.parsedDateParts, function(i) {
        return i != null;
      });
      m._isValid = !isNaN(m._d.getTime()) && flags.overflow < 0 && !flags.empty && !flags.invalidMonth && !flags.invalidWeekday && !flags.nullInput && !flags.invalidFormat && !flags.userInvalidated && (!flags.meridiem || (flags.meridiem && parsedParts));
      if (m._strict) {
        m._isValid = m._isValid && flags.charsLeftOver === 0 && flags.unusedTokens.length === 0 && flags.bigHour === undefined;
      }
    }
    return m._isValid;
  }
  function valid__createInvalid(flags) {
    var m = create_utc__createUTC(NaN);
    if (flags != null) {
      extend(getParsingFlags(m), flags);
    } else {
      getParsingFlags(m).userInvalidated = true;
    }
    return m;
  }
  function isUndefined(input) {
    return input === void 0;
  }
  var momentProperties = utils_hooks__hooks.momentProperties = [];
  function copyConfig(to, from) {
    var i,
        prop,
        val;
    if (!isUndefined(from._isAMomentObject)) {
      to._isAMomentObject = from._isAMomentObject;
    }
    if (!isUndefined(from._i)) {
      to._i = from._i;
    }
    if (!isUndefined(from._f)) {
      to._f = from._f;
    }
    if (!isUndefined(from._l)) {
      to._l = from._l;
    }
    if (!isUndefined(from._strict)) {
      to._strict = from._strict;
    }
    if (!isUndefined(from._tzm)) {
      to._tzm = from._tzm;
    }
    if (!isUndefined(from._isUTC)) {
      to._isUTC = from._isUTC;
    }
    if (!isUndefined(from._offset)) {
      to._offset = from._offset;
    }
    if (!isUndefined(from._pf)) {
      to._pf = getParsingFlags(from);
    }
    if (!isUndefined(from._locale)) {
      to._locale = from._locale;
    }
    if (momentProperties.length > 0) {
      for (i in momentProperties) {
        prop = momentProperties[i];
        val = from[prop];
        if (!isUndefined(val)) {
          to[prop] = val;
        }
      }
    }
    return to;
  }
  var updateInProgress = false;
  function Moment(config) {
    copyConfig(this, config);
    this._d = new Date(config._d != null ? config._d.getTime() : NaN);
    if (updateInProgress === false) {
      updateInProgress = true;
      utils_hooks__hooks.updateOffset(this);
      updateInProgress = false;
    }
  }
  function isMoment(obj) {
    return obj instanceof Moment || (obj != null && obj._isAMomentObject != null);
  }
  function absFloor(number) {
    if (number < 0) {
      return Math.ceil(number) || 0;
    } else {
      return Math.floor(number);
    }
  }
  function toInt(argumentForCoercion) {
    var coercedNumber = +argumentForCoercion,
        value = 0;
    if (coercedNumber !== 0 && isFinite(coercedNumber)) {
      value = absFloor(coercedNumber);
    }
    return value;
  }
  function compareArrays(array1, array2, dontConvert) {
    var len = Math.min(array1.length, array2.length),
        lengthDiff = Math.abs(array1.length - array2.length),
        diffs = 0,
        i;
    for (i = 0; i < len; i++) {
      if ((dontConvert && array1[i] !== array2[i]) || (!dontConvert && toInt(array1[i]) !== toInt(array2[i]))) {
        diffs++;
      }
    }
    return diffs + lengthDiff;
  }
  function warn(msg) {
    if (utils_hooks__hooks.suppressDeprecationWarnings === false && (typeof console !== 'undefined') && console.warn) {
      console.warn('Deprecation warning: ' + msg);
    }
  }
  function deprecate(msg, fn) {
    var firstTime = true;
    return extend(function() {
      if (utils_hooks__hooks.deprecationHandler != null) {
        utils_hooks__hooks.deprecationHandler(null, msg);
      }
      if (firstTime) {
        warn(msg + '\nArguments: ' + Array.prototype.slice.call(arguments).join(', ') + '\n' + (new Error()).stack);
        firstTime = false;
      }
      return fn.apply(this, arguments);
    }, fn);
  }
  var deprecations = {};
  function deprecateSimple(name, msg) {
    if (utils_hooks__hooks.deprecationHandler != null) {
      utils_hooks__hooks.deprecationHandler(name, msg);
    }
    if (!deprecations[name]) {
      warn(msg);
      deprecations[name] = true;
    }
  }
  utils_hooks__hooks.suppressDeprecationWarnings = false;
  utils_hooks__hooks.deprecationHandler = null;
  function isFunction(input) {
    return input instanceof Function || Object.prototype.toString.call(input) === '[object Function]';
  }
  function locale_set__set(config) {
    var prop,
        i;
    for (i in config) {
      prop = config[i];
      if (isFunction(prop)) {
        this[i] = prop;
      } else {
        this['_' + i] = prop;
      }
    }
    this._config = config;
    this._ordinalParseLenient = new RegExp(this._ordinalParse.source + '|' + (/\d{1,2}/).source);
  }
  function mergeConfigs(parentConfig, childConfig) {
    var res = extend({}, parentConfig),
        prop;
    for (prop in childConfig) {
      if (hasOwnProp(childConfig, prop)) {
        if (isObject(parentConfig[prop]) && isObject(childConfig[prop])) {
          res[prop] = {};
          extend(res[prop], parentConfig[prop]);
          extend(res[prop], childConfig[prop]);
        } else if (childConfig[prop] != null) {
          res[prop] = childConfig[prop];
        } else {
          delete res[prop];
        }
      }
    }
    for (prop in parentConfig) {
      if (hasOwnProp(parentConfig, prop) && !hasOwnProp(childConfig, prop) && isObject(parentConfig[prop])) {
        res[prop] = extend({}, res[prop]);
      }
    }
    return res;
  }
  function Locale(config) {
    if (config != null) {
      this.set(config);
    }
  }
  var keys;
  if (Object.keys) {
    keys = Object.keys;
  } else {
    keys = function(obj) {
      var i,
          res = [];
      for (i in obj) {
        if (hasOwnProp(obj, i)) {
          res.push(i);
        }
      }
      return res;
    };
  }
  var defaultCalendar = {
    sameDay: '[Today at] LT',
    nextDay: '[Tomorrow at] LT',
    nextWeek: 'dddd [at] LT',
    lastDay: '[Yesterday at] LT',
    lastWeek: '[Last] dddd [at] LT',
    sameElse: 'L'
  };
  function locale_calendar__calendar(key, mom, now) {
    var output = this._calendar[key] || this._calendar['sameElse'];
    return isFunction(output) ? output.call(mom, now) : output;
  }
  var defaultLongDateFormat = {
    LTS: 'h:mm:ss A',
    LT: 'h:mm A',
    L: 'MM/DD/YYYY',
    LL: 'MMMM D, YYYY',
    LLL: 'MMMM D, YYYY h:mm A',
    LLLL: 'dddd, MMMM D, YYYY h:mm A'
  };
  function longDateFormat(key) {
    var format = this._longDateFormat[key],
        formatUpper = this._longDateFormat[key.toUpperCase()];
    if (format || !formatUpper) {
      return format;
    }
    this._longDateFormat[key] = formatUpper.replace(/MMMM|MM|DD|dddd/g, function(val) {
      return val.slice(1);
    });
    return this._longDateFormat[key];
  }
  var defaultInvalidDate = 'Invalid date';
  function invalidDate() {
    return this._invalidDate;
  }
  var defaultOrdinal = '%d';
  var defaultOrdinalParse = /\d{1,2}/;
  function ordinal(number) {
    return this._ordinal.replace('%d', number);
  }
  var defaultRelativeTime = {
    future: 'in %s',
    past: '%s ago',
    s: 'a few seconds',
    m: 'a minute',
    mm: '%d minutes',
    h: 'an hour',
    hh: '%d hours',
    d: 'a day',
    dd: '%d days',
    M: 'a month',
    MM: '%d months',
    y: 'a year',
    yy: '%d years'
  };
  function relative__relativeTime(number, withoutSuffix, string, isFuture) {
    var output = this._relativeTime[string];
    return (isFunction(output)) ? output(number, withoutSuffix, string, isFuture) : output.replace(/%d/i, number);
  }
  function pastFuture(diff, output) {
    var format = this._relativeTime[diff > 0 ? 'future' : 'past'];
    return isFunction(format) ? format(output) : format.replace(/%s/i, output);
  }
  var aliases = {};
  function addUnitAlias(unit, shorthand) {
    var lowerCase = unit.toLowerCase();
    aliases[lowerCase] = aliases[lowerCase + 's'] = aliases[shorthand] = unit;
  }
  function normalizeUnits(units) {
    return typeof units === 'string' ? aliases[units] || aliases[units.toLowerCase()] : undefined;
  }
  function normalizeObjectUnits(inputObject) {
    var normalizedInput = {},
        normalizedProp,
        prop;
    for (prop in inputObject) {
      if (hasOwnProp(inputObject, prop)) {
        normalizedProp = normalizeUnits(prop);
        if (normalizedProp) {
          normalizedInput[normalizedProp] = inputObject[prop];
        }
      }
    }
    return normalizedInput;
  }
  var priorities = {};
  function addUnitPriority(unit, priority) {
    priorities[unit] = priority;
  }
  function getPrioritizedUnits(unitsObj) {
    var units = [];
    for (var u in unitsObj) {
      units.push({
        unit: u,
        priority: priorities[u]
      });
    }
    units.sort(function(a, b) {
      return a.priority - b.priority;
    });
    return units;
  }
  function makeGetSet(unit, keepTime) {
    return function(value) {
      if (value != null) {
        get_set__set(this, unit, value);
        utils_hooks__hooks.updateOffset(this, keepTime);
        return this;
      } else {
        return get_set__get(this, unit);
      }
    };
  }
  function get_set__get(mom, unit) {
    return mom.isValid() ? mom._d['get' + (mom._isUTC ? 'UTC' : '') + unit]() : NaN;
  }
  function get_set__set(mom, unit, value) {
    if (mom.isValid()) {
      mom._d['set' + (mom._isUTC ? 'UTC' : '') + unit](value);
    }
  }
  function stringGet(units) {
    units = normalizeUnits(units);
    if (isFunction(this[units])) {
      return this[units]();
    }
    return this;
  }
  function stringSet(units, value) {
    if (typeof units === 'object') {
      units = normalizeObjectUnits(units);
      var prioritized = getPrioritizedUnits(units);
      for (var i = 0; i < prioritized.length; i++) {
        this[prioritized[i].unit](units[prioritized[i].unit]);
      }
    } else {
      units = normalizeUnits(units);
      if (isFunction(this[units])) {
        return this[units](value);
      }
    }
    return this;
  }
  function zeroFill(number, targetLength, forceSign) {
    var absNumber = '' + Math.abs(number),
        zerosToFill = targetLength - absNumber.length,
        sign = number >= 0;
    return (sign ? (forceSign ? '+' : '') : '-') + Math.pow(10, Math.max(0, zerosToFill)).toString().substr(1) + absNumber;
  }
  var formattingTokens = /(\[[^\[]*\])|(\\)?([Hh]mm(ss)?|Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Qo?|YYYYYY|YYYYY|YYYY|YY|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|kk?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g;
  var localFormattingTokens = /(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g;
  var formatFunctions = {};
  var formatTokenFunctions = {};
  function addFormatToken(token, padded, ordinal, callback) {
    var func = callback;
    if (typeof callback === 'string') {
      func = function() {
        return this[callback]();
      };
    }
    if (token) {
      formatTokenFunctions[token] = func;
    }
    if (padded) {
      formatTokenFunctions[padded[0]] = function() {
        return zeroFill(func.apply(this, arguments), padded[1], padded[2]);
      };
    }
    if (ordinal) {
      formatTokenFunctions[ordinal] = function() {
        return this.localeData().ordinal(func.apply(this, arguments), token);
      };
    }
  }
  function removeFormattingTokens(input) {
    if (input.match(/\[[\s\S]/)) {
      return input.replace(/^\[|\]$/g, '');
    }
    return input.replace(/\\/g, '');
  }
  function makeFormatFunction(format) {
    var array = format.match(formattingTokens),
        i,
        length;
    for (i = 0, length = array.length; i < length; i++) {
      if (formatTokenFunctions[array[i]]) {
        array[i] = formatTokenFunctions[array[i]];
      } else {
        array[i] = removeFormattingTokens(array[i]);
      }
    }
    return function(mom) {
      var output = '',
          i;
      for (i = 0; i < length; i++) {
        output += array[i] instanceof Function ? array[i].call(mom, format) : array[i];
      }
      return output;
    };
  }
  function formatMoment(m, format) {
    if (!m.isValid()) {
      return m.localeData().invalidDate();
    }
    format = expandFormat(format, m.localeData());
    formatFunctions[format] = formatFunctions[format] || makeFormatFunction(format);
    return formatFunctions[format](m);
  }
  function expandFormat(format, locale) {
    var i = 5;
    function replaceLongDateFormatTokens(input) {
      return locale.longDateFormat(input) || input;
    }
    localFormattingTokens.lastIndex = 0;
    while (i >= 0 && localFormattingTokens.test(format)) {
      format = format.replace(localFormattingTokens, replaceLongDateFormatTokens);
      localFormattingTokens.lastIndex = 0;
      i -= 1;
    }
    return format;
  }
  var match1 = /\d/;
  var match2 = /\d\d/;
  var match3 = /\d{3}/;
  var match4 = /\d{4}/;
  var match6 = /[+-]?\d{6}/;
  var match1to2 = /\d\d?/;
  var match3to4 = /\d\d\d\d?/;
  var match5to6 = /\d\d\d\d\d\d?/;
  var match1to3 = /\d{1,3}/;
  var match1to4 = /\d{1,4}/;
  var match1to6 = /[+-]?\d{1,6}/;
  var matchUnsigned = /\d+/;
  var matchSigned = /[+-]?\d+/;
  var matchOffset = /Z|[+-]\d\d:?\d\d/gi;
  var matchShortOffset = /Z|[+-]\d\d(?::?\d\d)?/gi;
  var matchTimestamp = /[+-]?\d+(\.\d{1,3})?/;
  var matchWord = /[0-9]*['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]+|[\u0600-\u06FF\/]+(\s*?[\u0600-\u06FF]+){1,2}/i;
  var regexes = {};
  function addRegexToken(token, regex, strictRegex) {
    regexes[token] = isFunction(regex) ? regex : function(isStrict, localeData) {
      return (isStrict && strictRegex) ? strictRegex : regex;
    };
  }
  function getParseRegexForToken(token, config) {
    if (!hasOwnProp(regexes, token)) {
      return new RegExp(unescapeFormat(token));
    }
    return regexes[token](config._strict, config._locale);
  }
  function unescapeFormat(s) {
    return regexEscape(s.replace('\\', '').replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g, function(matched, p1, p2, p3, p4) {
      return p1 || p2 || p3 || p4;
    }));
  }
  function regexEscape(s) {
    return s.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
  }
  var tokens = {};
  function addParseToken(token, callback) {
    var i,
        func = callback;
    if (typeof token === 'string') {
      token = [token];
    }
    if (typeof callback === 'number') {
      func = function(input, array) {
        array[callback] = toInt(input);
      };
    }
    for (i = 0; i < token.length; i++) {
      tokens[token[i]] = func;
    }
  }
  function addWeekParseToken(token, callback) {
    addParseToken(token, function(input, array, config, token) {
      config._w = config._w || {};
      callback(input, config._w, config, token);
    });
  }
  function addTimeToArrayFromToken(token, input, config) {
    if (input != null && hasOwnProp(tokens, token)) {
      tokens[token](input, config._a, config, token);
    }
  }
  var YEAR = 0;
  var MONTH = 1;
  var DATE = 2;
  var HOUR = 3;
  var MINUTE = 4;
  var SECOND = 5;
  var MILLISECOND = 6;
  var WEEK = 7;
  var WEEKDAY = 8;
  var indexOf;
  if (Array.prototype.indexOf) {
    indexOf = Array.prototype.indexOf;
  } else {
    indexOf = function(o) {
      var i;
      for (i = 0; i < this.length; ++i) {
        if (this[i] === o) {
          return i;
        }
      }
      return -1;
    };
  }
  function daysInMonth(year, month) {
    return new Date(Date.UTC(year, month + 1, 0)).getUTCDate();
  }
  addFormatToken('M', ['MM', 2], 'Mo', function() {
    return this.month() + 1;
  });
  addFormatToken('MMM', 0, 0, function(format) {
    return this.localeData().monthsShort(this, format);
  });
  addFormatToken('MMMM', 0, 0, function(format) {
    return this.localeData().months(this, format);
  });
  addUnitAlias('month', 'M');
  addUnitPriority('month', 8);
  addRegexToken('M', match1to2);
  addRegexToken('MM', match1to2, match2);
  addRegexToken('MMM', function(isStrict, locale) {
    return locale.monthsShortRegex(isStrict);
  });
  addRegexToken('MMMM', function(isStrict, locale) {
    return locale.monthsRegex(isStrict);
  });
  addParseToken(['M', 'MM'], function(input, array) {
    array[MONTH] = toInt(input) - 1;
  });
  addParseToken(['MMM', 'MMMM'], function(input, array, config, token) {
    var month = config._locale.monthsParse(input, token, config._strict);
    if (month != null) {
      array[MONTH] = month;
    } else {
      getParsingFlags(config).invalidMonth = input;
    }
  });
  var MONTHS_IN_FORMAT = /D[oD]?(\[[^\[\]]*\]|\s+)+MMMM?/;
  var defaultLocaleMonths = 'January_February_March_April_May_June_July_August_September_October_November_December'.split('_');
  function localeMonths(m, format) {
    return isArray(this._months) ? this._months[m.month()] : this._months[(this._months.isFormat || MONTHS_IN_FORMAT).test(format) ? 'format' : 'standalone'][m.month()];
  }
  var defaultLocaleMonthsShort = 'Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec'.split('_');
  function localeMonthsShort(m, format) {
    return isArray(this._monthsShort) ? this._monthsShort[m.month()] : this._monthsShort[MONTHS_IN_FORMAT.test(format) ? 'format' : 'standalone'][m.month()];
  }
  function units_month__handleStrictParse(monthName, format, strict) {
    var i,
        ii,
        mom,
        llc = monthName.toLocaleLowerCase();
    if (!this._monthsParse) {
      this._monthsParse = [];
      this._longMonthsParse = [];
      this._shortMonthsParse = [];
      for (i = 0; i < 12; ++i) {
        mom = create_utc__createUTC([2000, i]);
        this._shortMonthsParse[i] = this.monthsShort(mom, '').toLocaleLowerCase();
        this._longMonthsParse[i] = this.months(mom, '').toLocaleLowerCase();
      }
    }
    if (strict) {
      if (format === 'MMM') {
        ii = indexOf.call(this._shortMonthsParse, llc);
        return ii !== -1 ? ii : null;
      } else {
        ii = indexOf.call(this._longMonthsParse, llc);
        return ii !== -1 ? ii : null;
      }
    } else {
      if (format === 'MMM') {
        ii = indexOf.call(this._shortMonthsParse, llc);
        if (ii !== -1) {
          return ii;
        }
        ii = indexOf.call(this._longMonthsParse, llc);
        return ii !== -1 ? ii : null;
      } else {
        ii = indexOf.call(this._longMonthsParse, llc);
        if (ii !== -1) {
          return ii;
        }
        ii = indexOf.call(this._shortMonthsParse, llc);
        return ii !== -1 ? ii : null;
      }
    }
  }
  function localeMonthsParse(monthName, format, strict) {
    var i,
        mom,
        regex;
    if (this._monthsParseExact) {
      return units_month__handleStrictParse.call(this, monthName, format, strict);
    }
    if (!this._monthsParse) {
      this._monthsParse = [];
      this._longMonthsParse = [];
      this._shortMonthsParse = [];
    }
    for (i = 0; i < 12; i++) {
      mom = create_utc__createUTC([2000, i]);
      if (strict && !this._longMonthsParse[i]) {
        this._longMonthsParse[i] = new RegExp('^' + this.months(mom, '').replace('.', '') + '$', 'i');
        this._shortMonthsParse[i] = new RegExp('^' + this.monthsShort(mom, '').replace('.', '') + '$', 'i');
      }
      if (!strict && !this._monthsParse[i]) {
        regex = '^' + this.months(mom, '') + '|^' + this.monthsShort(mom, '');
        this._monthsParse[i] = new RegExp(regex.replace('.', ''), 'i');
      }
      if (strict && format === 'MMMM' && this._longMonthsParse[i].test(monthName)) {
        return i;
      } else if (strict && format === 'MMM' && this._shortMonthsParse[i].test(monthName)) {
        return i;
      } else if (!strict && this._monthsParse[i].test(monthName)) {
        return i;
      }
    }
  }
  function setMonth(mom, value) {
    var dayOfMonth;
    if (!mom.isValid()) {
      return mom;
    }
    if (typeof value === 'string') {
      if (/^\d+$/.test(value)) {
        value = toInt(value);
      } else {
        value = mom.localeData().monthsParse(value);
        if (typeof value !== 'number') {
          return mom;
        }
      }
    }
    dayOfMonth = Math.min(mom.date(), daysInMonth(mom.year(), value));
    mom._d['set' + (mom._isUTC ? 'UTC' : '') + 'Month'](value, dayOfMonth);
    return mom;
  }
  function getSetMonth(value) {
    if (value != null) {
      setMonth(this, value);
      utils_hooks__hooks.updateOffset(this, true);
      return this;
    } else {
      return get_set__get(this, 'Month');
    }
  }
  function getDaysInMonth() {
    return daysInMonth(this.year(), this.month());
  }
  var defaultMonthsShortRegex = matchWord;
  function monthsShortRegex(isStrict) {
    if (this._monthsParseExact) {
      if (!hasOwnProp(this, '_monthsRegex')) {
        computeMonthsParse.call(this);
      }
      if (isStrict) {
        return this._monthsShortStrictRegex;
      } else {
        return this._monthsShortRegex;
      }
    } else {
      if (!hasOwnProp(this, '_monthsShortRegex')) {
        this._monthsShortRegex = defaultMonthsShortRegex;
      }
      return this._monthsShortStrictRegex && isStrict ? this._monthsShortStrictRegex : this._monthsShortRegex;
    }
  }
  var defaultMonthsRegex = matchWord;
  function monthsRegex(isStrict) {
    if (this._monthsParseExact) {
      if (!hasOwnProp(this, '_monthsRegex')) {
        computeMonthsParse.call(this);
      }
      if (isStrict) {
        return this._monthsStrictRegex;
      } else {
        return this._monthsRegex;
      }
    } else {
      if (!hasOwnProp(this, '_monthsRegex')) {
        this._monthsRegex = defaultMonthsRegex;
      }
      return this._monthsStrictRegex && isStrict ? this._monthsStrictRegex : this._monthsRegex;
    }
  }
  function computeMonthsParse() {
    function cmpLenRev(a, b) {
      return b.length - a.length;
    }
    var shortPieces = [],
        longPieces = [],
        mixedPieces = [],
        i,
        mom;
    for (i = 0; i < 12; i++) {
      mom = create_utc__createUTC([2000, i]);
      shortPieces.push(this.monthsShort(mom, ''));
      longPieces.push(this.months(mom, ''));
      mixedPieces.push(this.months(mom, ''));
      mixedPieces.push(this.monthsShort(mom, ''));
    }
    shortPieces.sort(cmpLenRev);
    longPieces.sort(cmpLenRev);
    mixedPieces.sort(cmpLenRev);
    for (i = 0; i < 12; i++) {
      shortPieces[i] = regexEscape(shortPieces[i]);
      longPieces[i] = regexEscape(longPieces[i]);
    }
    for (i = 0; i < 24; i++) {
      mixedPieces[i] = regexEscape(mixedPieces[i]);
    }
    this._monthsRegex = new RegExp('^(' + mixedPieces.join('|') + ')', 'i');
    this._monthsShortRegex = this._monthsRegex;
    this._monthsStrictRegex = new RegExp('^(' + longPieces.join('|') + ')', 'i');
    this._monthsShortStrictRegex = new RegExp('^(' + shortPieces.join('|') + ')', 'i');
  }
  addFormatToken('Y', 0, 0, function() {
    var y = this.year();
    return y <= 9999 ? '' + y : '+' + y;
  });
  addFormatToken(0, ['YY', 2], 0, function() {
    return this.year() % 100;
  });
  addFormatToken(0, ['YYYY', 4], 0, 'year');
  addFormatToken(0, ['YYYYY', 5], 0, 'year');
  addFormatToken(0, ['YYYYYY', 6, true], 0, 'year');
  addUnitAlias('year', 'y');
  addUnitPriority('year', 1);
  addRegexToken('Y', matchSigned);
  addRegexToken('YY', match1to2, match2);
  addRegexToken('YYYY', match1to4, match4);
  addRegexToken('YYYYY', match1to6, match6);
  addRegexToken('YYYYYY', match1to6, match6);
  addParseToken(['YYYYY', 'YYYYYY'], YEAR);
  addParseToken('YYYY', function(input, array) {
    array[YEAR] = input.length === 2 ? utils_hooks__hooks.parseTwoDigitYear(input) : toInt(input);
  });
  addParseToken('YY', function(input, array) {
    array[YEAR] = utils_hooks__hooks.parseTwoDigitYear(input);
  });
  addParseToken('Y', function(input, array) {
    array[YEAR] = parseInt(input, 10);
  });
  function daysInYear(year) {
    return isLeapYear(year) ? 366 : 365;
  }
  function isLeapYear(year) {
    return (year % 4 === 0 && year % 100 !== 0) || year % 400 === 0;
  }
  utils_hooks__hooks.parseTwoDigitYear = function(input) {
    return toInt(input) + (toInt(input) > 68 ? 1900 : 2000);
  };
  var getSetYear = makeGetSet('FullYear', true);
  function getIsLeapYear() {
    return isLeapYear(this.year());
  }
  function createDate(y, m, d, h, M, s, ms) {
    var date = new Date(y, m, d, h, M, s, ms);
    if (y < 100 && y >= 0 && isFinite(date.getFullYear())) {
      date.setFullYear(y);
    }
    return date;
  }
  function createUTCDate(y) {
    var date = new Date(Date.UTC.apply(null, arguments));
    if (y < 100 && y >= 0 && isFinite(date.getUTCFullYear())) {
      date.setUTCFullYear(y);
    }
    return date;
  }
  function firstWeekOffset(year, dow, doy) {
    var fwd = 7 + dow - doy,
        fwdlw = (7 + createUTCDate(year, 0, fwd).getUTCDay() - dow) % 7;
    return -fwdlw + fwd - 1;
  }
  function dayOfYearFromWeeks(year, week, weekday, dow, doy) {
    var localWeekday = (7 + weekday - dow) % 7,
        weekOffset = firstWeekOffset(year, dow, doy),
        dayOfYear = 1 + 7 * (week - 1) + localWeekday + weekOffset,
        resYear,
        resDayOfYear;
    if (dayOfYear <= 0) {
      resYear = year - 1;
      resDayOfYear = daysInYear(resYear) + dayOfYear;
    } else if (dayOfYear > daysInYear(year)) {
      resYear = year + 1;
      resDayOfYear = dayOfYear - daysInYear(year);
    } else {
      resYear = year;
      resDayOfYear = dayOfYear;
    }
    return {
      year: resYear,
      dayOfYear: resDayOfYear
    };
  }
  function weekOfYear(mom, dow, doy) {
    var weekOffset = firstWeekOffset(mom.year(), dow, doy),
        week = Math.floor((mom.dayOfYear() - weekOffset - 1) / 7) + 1,
        resWeek,
        resYear;
    if (week < 1) {
      resYear = mom.year() - 1;
      resWeek = week + weeksInYear(resYear, dow, doy);
    } else if (week > weeksInYear(mom.year(), dow, doy)) {
      resWeek = week - weeksInYear(mom.year(), dow, doy);
      resYear = mom.year() + 1;
    } else {
      resYear = mom.year();
      resWeek = week;
    }
    return {
      week: resWeek,
      year: resYear
    };
  }
  function weeksInYear(year, dow, doy) {
    var weekOffset = firstWeekOffset(year, dow, doy),
        weekOffsetNext = firstWeekOffset(year + 1, dow, doy);
    return (daysInYear(year) - weekOffset + weekOffsetNext) / 7;
  }
  addFormatToken('w', ['ww', 2], 'wo', 'week');
  addFormatToken('W', ['WW', 2], 'Wo', 'isoWeek');
  addUnitAlias('week', 'w');
  addUnitAlias('isoWeek', 'W');
  addUnitPriority('week', 5);
  addUnitPriority('isoWeek', 5);
  addRegexToken('w', match1to2);
  addRegexToken('ww', match1to2, match2);
  addRegexToken('W', match1to2);
  addRegexToken('WW', match1to2, match2);
  addWeekParseToken(['w', 'ww', 'W', 'WW'], function(input, week, config, token) {
    week[token.substr(0, 1)] = toInt(input);
  });
  function localeWeek(mom) {
    return weekOfYear(mom, this._week.dow, this._week.doy).week;
  }
  var defaultLocaleWeek = {
    dow: 0,
    doy: 6
  };
  function localeFirstDayOfWeek() {
    return this._week.dow;
  }
  function localeFirstDayOfYear() {
    return this._week.doy;
  }
  function getSetWeek(input) {
    var week = this.localeData().week(this);
    return input == null ? week : this.add((input - week) * 7, 'd');
  }
  function getSetISOWeek(input) {
    var week = weekOfYear(this, 1, 4).week;
    return input == null ? week : this.add((input - week) * 7, 'd');
  }
  addFormatToken('d', 0, 'do', 'day');
  addFormatToken('dd', 0, 0, function(format) {
    return this.localeData().weekdaysMin(this, format);
  });
  addFormatToken('ddd', 0, 0, function(format) {
    return this.localeData().weekdaysShort(this, format);
  });
  addFormatToken('dddd', 0, 0, function(format) {
    return this.localeData().weekdays(this, format);
  });
  addFormatToken('e', 0, 0, 'weekday');
  addFormatToken('E', 0, 0, 'isoWeekday');
  addUnitAlias('day', 'd');
  addUnitAlias('weekday', 'e');
  addUnitAlias('isoWeekday', 'E');
  addUnitPriority('day', 11);
  addUnitPriority('weekday', 11);
  addUnitPriority('isoWeekday', 11);
  addRegexToken('d', match1to2);
  addRegexToken('e', match1to2);
  addRegexToken('E', match1to2);
  addRegexToken('dd', function(isStrict, locale) {
    return locale.weekdaysMinRegex(isStrict);
  });
  addRegexToken('ddd', function(isStrict, locale) {
    return locale.weekdaysShortRegex(isStrict);
  });
  addRegexToken('dddd', function(isStrict, locale) {
    return locale.weekdaysRegex(isStrict);
  });
  addWeekParseToken(['dd', 'ddd', 'dddd'], function(input, week, config, token) {
    var weekday = config._locale.weekdaysParse(input, token, config._strict);
    if (weekday != null) {
      week.d = weekday;
    } else {
      getParsingFlags(config).invalidWeekday = input;
    }
  });
  addWeekParseToken(['d', 'e', 'E'], function(input, week, config, token) {
    week[token] = toInt(input);
  });
  function parseWeekday(input, locale) {
    if (typeof input !== 'string') {
      return input;
    }
    if (!isNaN(input)) {
      return parseInt(input, 10);
    }
    input = locale.weekdaysParse(input);
    if (typeof input === 'number') {
      return input;
    }
    return null;
  }
  function parseIsoWeekday(input, locale) {
    if (typeof input === 'string') {
      return locale.weekdaysParse(input) % 7 || 7;
    }
    return isNaN(input) ? null : input;
  }
  var defaultLocaleWeekdays = 'Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday'.split('_');
  function localeWeekdays(m, format) {
    return isArray(this._weekdays) ? this._weekdays[m.day()] : this._weekdays[this._weekdays.isFormat.test(format) ? 'format' : 'standalone'][m.day()];
  }
  var defaultLocaleWeekdaysShort = 'Sun_Mon_Tue_Wed_Thu_Fri_Sat'.split('_');
  function localeWeekdaysShort(m) {
    return this._weekdaysShort[m.day()];
  }
  var defaultLocaleWeekdaysMin = 'Su_Mo_Tu_We_Th_Fr_Sa'.split('_');
  function localeWeekdaysMin(m) {
    return this._weekdaysMin[m.day()];
  }
  function day_of_week__handleStrictParse(weekdayName, format, strict) {
    var i,
        ii,
        mom,
        llc = weekdayName.toLocaleLowerCase();
    if (!this._weekdaysParse) {
      this._weekdaysParse = [];
      this._shortWeekdaysParse = [];
      this._minWeekdaysParse = [];
      for (i = 0; i < 7; ++i) {
        mom = create_utc__createUTC([2000, 1]).day(i);
        this._minWeekdaysParse[i] = this.weekdaysMin(mom, '').toLocaleLowerCase();
        this._shortWeekdaysParse[i] = this.weekdaysShort(mom, '').toLocaleLowerCase();
        this._weekdaysParse[i] = this.weekdays(mom, '').toLocaleLowerCase();
      }
    }
    if (strict) {
      if (format === 'dddd') {
        ii = indexOf.call(this._weekdaysParse, llc);
        return ii !== -1 ? ii : null;
      } else if (format === 'ddd') {
        ii = indexOf.call(this._shortWeekdaysParse, llc);
        return ii !== -1 ? ii : null;
      } else {
        ii = indexOf.call(this._minWeekdaysParse, llc);
        return ii !== -1 ? ii : null;
      }
    } else {
      if (format === 'dddd') {
        ii = indexOf.call(this._weekdaysParse, llc);
        if (ii !== -1) {
          return ii;
        }
        ii = indexOf.call(this._shortWeekdaysParse, llc);
        if (ii !== -1) {
          return ii;
        }
        ii = indexOf.call(this._minWeekdaysParse, llc);
        return ii !== -1 ? ii : null;
      } else if (format === 'ddd') {
        ii = indexOf.call(this._shortWeekdaysParse, llc);
        if (ii !== -1) {
          return ii;
        }
        ii = indexOf.call(this._weekdaysParse, llc);
        if (ii !== -1) {
          return ii;
        }
        ii = indexOf.call(this._minWeekdaysParse, llc);
        return ii !== -1 ? ii : null;
      } else {
        ii = indexOf.call(this._minWeekdaysParse, llc);
        if (ii !== -1) {
          return ii;
        }
        ii = indexOf.call(this._weekdaysParse, llc);
        if (ii !== -1) {
          return ii;
        }
        ii = indexOf.call(this._shortWeekdaysParse, llc);
        return ii !== -1 ? ii : null;
      }
    }
  }
  function localeWeekdaysParse(weekdayName, format, strict) {
    var i,
        mom,
        regex;
    if (this._weekdaysParseExact) {
      return day_of_week__handleStrictParse.call(this, weekdayName, format, strict);
    }
    if (!this._weekdaysParse) {
      this._weekdaysParse = [];
      this._minWeekdaysParse = [];
      this._shortWeekdaysParse = [];
      this._fullWeekdaysParse = [];
    }
    for (i = 0; i < 7; i++) {
      mom = create_utc__createUTC([2000, 1]).day(i);
      if (strict && !this._fullWeekdaysParse[i]) {
        this._fullWeekdaysParse[i] = new RegExp('^' + this.weekdays(mom, '').replace('.', '\.?') + '$', 'i');
        this._shortWeekdaysParse[i] = new RegExp('^' + this.weekdaysShort(mom, '').replace('.', '\.?') + '$', 'i');
        this._minWeekdaysParse[i] = new RegExp('^' + this.weekdaysMin(mom, '').replace('.', '\.?') + '$', 'i');
      }
      if (!this._weekdaysParse[i]) {
        regex = '^' + this.weekdays(mom, '') + '|^' + this.weekdaysShort(mom, '') + '|^' + this.weekdaysMin(mom, '');
        this._weekdaysParse[i] = new RegExp(regex.replace('.', ''), 'i');
      }
      if (strict && format === 'dddd' && this._fullWeekdaysParse[i].test(weekdayName)) {
        return i;
      } else if (strict && format === 'ddd' && this._shortWeekdaysParse[i].test(weekdayName)) {
        return i;
      } else if (strict && format === 'dd' && this._minWeekdaysParse[i].test(weekdayName)) {
        return i;
      } else if (!strict && this._weekdaysParse[i].test(weekdayName)) {
        return i;
      }
    }
  }
  function getSetDayOfWeek(input) {
    if (!this.isValid()) {
      return input != null ? this : NaN;
    }
    var day = this._isUTC ? this._d.getUTCDay() : this._d.getDay();
    if (input != null) {
      input = parseWeekday(input, this.localeData());
      return this.add(input - day, 'd');
    } else {
      return day;
    }
  }
  function getSetLocaleDayOfWeek(input) {
    if (!this.isValid()) {
      return input != null ? this : NaN;
    }
    var weekday = (this.day() + 7 - this.localeData()._week.dow) % 7;
    return input == null ? weekday : this.add(input - weekday, 'd');
  }
  function getSetISODayOfWeek(input) {
    if (!this.isValid()) {
      return input != null ? this : NaN;
    }
    if (input != null) {
      var weekday = parseIsoWeekday(input, this.localeData());
      return this.day(this.day() % 7 ? weekday : weekday - 7);
    } else {
      return this.day() || 7;
    }
  }
  var defaultWeekdaysRegex = matchWord;
  function weekdaysRegex(isStrict) {
    if (this._weekdaysParseExact) {
      if (!hasOwnProp(this, '_weekdaysRegex')) {
        computeWeekdaysParse.call(this);
      }
      if (isStrict) {
        return this._weekdaysStrictRegex;
      } else {
        return this._weekdaysRegex;
      }
    } else {
      if (!hasOwnProp(this, '_weekdaysRegex')) {
        this._weekdaysRegex = defaultWeekdaysRegex;
      }
      return this._weekdaysStrictRegex && isStrict ? this._weekdaysStrictRegex : this._weekdaysRegex;
    }
  }
  var defaultWeekdaysShortRegex = matchWord;
  function weekdaysShortRegex(isStrict) {
    if (this._weekdaysParseExact) {
      if (!hasOwnProp(this, '_weekdaysRegex')) {
        computeWeekdaysParse.call(this);
      }
      if (isStrict) {
        return this._weekdaysShortStrictRegex;
      } else {
        return this._weekdaysShortRegex;
      }
    } else {
      if (!hasOwnProp(this, '_weekdaysShortRegex')) {
        this._weekdaysShortRegex = defaultWeekdaysShortRegex;
      }
      return this._weekdaysShortStrictRegex && isStrict ? this._weekdaysShortStrictRegex : this._weekdaysShortRegex;
    }
  }
  var defaultWeekdaysMinRegex = matchWord;
  function weekdaysMinRegex(isStrict) {
    if (this._weekdaysParseExact) {
      if (!hasOwnProp(this, '_weekdaysRegex')) {
        computeWeekdaysParse.call(this);
      }
      if (isStrict) {
        return this._weekdaysMinStrictRegex;
      } else {
        return this._weekdaysMinRegex;
      }
    } else {
      if (!hasOwnProp(this, '_weekdaysMinRegex')) {
        this._weekdaysMinRegex = defaultWeekdaysMinRegex;
      }
      return this._weekdaysMinStrictRegex && isStrict ? this._weekdaysMinStrictRegex : this._weekdaysMinRegex;
    }
  }
  function computeWeekdaysParse() {
    function cmpLenRev(a, b) {
      return b.length - a.length;
    }
    var minPieces = [],
        shortPieces = [],
        longPieces = [],
        mixedPieces = [],
        i,
        mom,
        minp,
        shortp,
        longp;
    for (i = 0; i < 7; i++) {
      mom = create_utc__createUTC([2000, 1]).day(i);
      minp = this.weekdaysMin(mom, '');
      shortp = this.weekdaysShort(mom, '');
      longp = this.weekdays(mom, '');
      minPieces.push(minp);
      shortPieces.push(shortp);
      longPieces.push(longp);
      mixedPieces.push(minp);
      mixedPieces.push(shortp);
      mixedPieces.push(longp);
    }
    minPieces.sort(cmpLenRev);
    shortPieces.sort(cmpLenRev);
    longPieces.sort(cmpLenRev);
    mixedPieces.sort(cmpLenRev);
    for (i = 0; i < 7; i++) {
      shortPieces[i] = regexEscape(shortPieces[i]);
      longPieces[i] = regexEscape(longPieces[i]);
      mixedPieces[i] = regexEscape(mixedPieces[i]);
    }
    this._weekdaysRegex = new RegExp('^(' + mixedPieces.join('|') + ')', 'i');
    this._weekdaysShortRegex = this._weekdaysRegex;
    this._weekdaysMinRegex = this._weekdaysRegex;
    this._weekdaysStrictRegex = new RegExp('^(' + longPieces.join('|') + ')', 'i');
    this._weekdaysShortStrictRegex = new RegExp('^(' + shortPieces.join('|') + ')', 'i');
    this._weekdaysMinStrictRegex = new RegExp('^(' + minPieces.join('|') + ')', 'i');
  }
  function hFormat() {
    return this.hours() % 12 || 12;
  }
  function kFormat() {
    return this.hours() || 24;
  }
  addFormatToken('H', ['HH', 2], 0, 'hour');
  addFormatToken('h', ['hh', 2], 0, hFormat);
  addFormatToken('k', ['kk', 2], 0, kFormat);
  addFormatToken('hmm', 0, 0, function() {
    return '' + hFormat.apply(this) + zeroFill(this.minutes(), 2);
  });
  addFormatToken('hmmss', 0, 0, function() {
    return '' + hFormat.apply(this) + zeroFill(this.minutes(), 2) + zeroFill(this.seconds(), 2);
  });
  addFormatToken('Hmm', 0, 0, function() {
    return '' + this.hours() + zeroFill(this.minutes(), 2);
  });
  addFormatToken('Hmmss', 0, 0, function() {
    return '' + this.hours() + zeroFill(this.minutes(), 2) + zeroFill(this.seconds(), 2);
  });
  function meridiem(token, lowercase) {
    addFormatToken(token, 0, 0, function() {
      return this.localeData().meridiem(this.hours(), this.minutes(), lowercase);
    });
  }
  meridiem('a', true);
  meridiem('A', false);
  addUnitAlias('hour', 'h');
  addUnitPriority('hour', 13);
  function matchMeridiem(isStrict, locale) {
    return locale._meridiemParse;
  }
  addRegexToken('a', matchMeridiem);
  addRegexToken('A', matchMeridiem);
  addRegexToken('H', match1to2);
  addRegexToken('h', match1to2);
  addRegexToken('HH', match1to2, match2);
  addRegexToken('hh', match1to2, match2);
  addRegexToken('hmm', match3to4);
  addRegexToken('hmmss', match5to6);
  addRegexToken('Hmm', match3to4);
  addRegexToken('Hmmss', match5to6);
  addParseToken(['H', 'HH'], HOUR);
  addParseToken(['a', 'A'], function(input, array, config) {
    config._isPm = config._locale.isPM(input);
    config._meridiem = input;
  });
  addParseToken(['h', 'hh'], function(input, array, config) {
    array[HOUR] = toInt(input);
    getParsingFlags(config).bigHour = true;
  });
  addParseToken('hmm', function(input, array, config) {
    var pos = input.length - 2;
    array[HOUR] = toInt(input.substr(0, pos));
    array[MINUTE] = toInt(input.substr(pos));
    getParsingFlags(config).bigHour = true;
  });
  addParseToken('hmmss', function(input, array, config) {
    var pos1 = input.length - 4;
    var pos2 = input.length - 2;
    array[HOUR] = toInt(input.substr(0, pos1));
    array[MINUTE] = toInt(input.substr(pos1, 2));
    array[SECOND] = toInt(input.substr(pos2));
    getParsingFlags(config).bigHour = true;
  });
  addParseToken('Hmm', function(input, array, config) {
    var pos = input.length - 2;
    array[HOUR] = toInt(input.substr(0, pos));
    array[MINUTE] = toInt(input.substr(pos));
  });
  addParseToken('Hmmss', function(input, array, config) {
    var pos1 = input.length - 4;
    var pos2 = input.length - 2;
    array[HOUR] = toInt(input.substr(0, pos1));
    array[MINUTE] = toInt(input.substr(pos1, 2));
    array[SECOND] = toInt(input.substr(pos2));
  });
  function localeIsPM(input) {
    return ((input + '').toLowerCase().charAt(0) === 'p');
  }
  var defaultLocaleMeridiemParse = /[ap]\.?m?\.?/i;
  function localeMeridiem(hours, minutes, isLower) {
    if (hours > 11) {
      return isLower ? 'pm' : 'PM';
    } else {
      return isLower ? 'am' : 'AM';
    }
  }
  var getSetHour = makeGetSet('Hours', true);
  var baseConfig = {
    calendar: defaultCalendar,
    longDateFormat: defaultLongDateFormat,
    invalidDate: defaultInvalidDate,
    ordinal: defaultOrdinal,
    ordinalParse: defaultOrdinalParse,
    relativeTime: defaultRelativeTime,
    months: defaultLocaleMonths,
    monthsShort: defaultLocaleMonthsShort,
    week: defaultLocaleWeek,
    weekdays: defaultLocaleWeekdays,
    weekdaysMin: defaultLocaleWeekdaysMin,
    weekdaysShort: defaultLocaleWeekdaysShort,
    meridiemParse: defaultLocaleMeridiemParse
  };
  var locales = {};
  var globalLocale;
  function normalizeLocale(key) {
    return key ? key.toLowerCase().replace('_', '-') : key;
  }
  function chooseLocale(names) {
    var i = 0,
        j,
        next,
        locale,
        split;
    while (i < names.length) {
      split = normalizeLocale(names[i]).split('-');
      j = split.length;
      next = normalizeLocale(names[i + 1]);
      next = next ? next.split('-') : null;
      while (j > 0) {
        locale = loadLocale(split.slice(0, j).join('-'));
        if (locale) {
          return locale;
        }
        if (next && next.length >= j && compareArrays(split, next, true) >= j - 1) {
          break;
        }
        j--;
      }
      i++;
    }
    return null;
  }
  function loadLocale(name) {
    var oldLocale = null;
    if (!locales[name] && (typeof module !== 'undefined') && module && module.exports) {
      try {
        oldLocale = globalLocale._abbr;
        require('./locale/' + name);
        locale_locales__getSetGlobalLocale(oldLocale);
      } catch (e) {}
    }
    return locales[name];
  }
  function locale_locales__getSetGlobalLocale(key, values) {
    var data;
    if (key) {
      if (isUndefined(values)) {
        data = locale_locales__getLocale(key);
      } else {
        data = defineLocale(key, values);
      }
      if (data) {
        globalLocale = data;
      }
    }
    return globalLocale._abbr;
  }
  function defineLocale(name, config) {
    if (config !== null) {
      var parentConfig = baseConfig;
      config.abbr = name;
      if (locales[name] != null) {
        deprecateSimple('defineLocaleOverride', 'use moment.updateLocale(localeName, config) to change ' + 'an existing locale. moment.defineLocale(localeName, ' + 'config) should only be used for creating a new locale ' + 'See http://momentjs.com/guides/#/warnings/define-locale/ for more info.');
        parentConfig = locales[name]._config;
      } else if (config.parentLocale != null) {
        if (locales[config.parentLocale] != null) {
          parentConfig = locales[config.parentLocale]._config;
        } else {
          deprecateSimple('parentLocaleUndefined', 'specified parentLocale is not defined yet. See http://momentjs.com/guides/#/warnings/parent-locale/');
        }
      }
      locales[name] = new Locale(mergeConfigs(parentConfig, config));
      locale_locales__getSetGlobalLocale(name);
      return locales[name];
    } else {
      delete locales[name];
      return null;
    }
  }
  function updateLocale(name, config) {
    if (config != null) {
      var locale,
          parentConfig = baseConfig;
      if (locales[name] != null) {
        parentConfig = locales[name]._config;
      }
      config = mergeConfigs(parentConfig, config);
      locale = new Locale(config);
      locale.parentLocale = locales[name];
      locales[name] = locale;
      locale_locales__getSetGlobalLocale(name);
    } else {
      if (locales[name] != null) {
        if (locales[name].parentLocale != null) {
          locales[name] = locales[name].parentLocale;
        } else if (locales[name] != null) {
          delete locales[name];
        }
      }
    }
    return locales[name];
  }
  function locale_locales__getLocale(key) {
    var locale;
    if (key && key._locale && key._locale._abbr) {
      key = key._locale._abbr;
    }
    if (!key) {
      return globalLocale;
    }
    if (!isArray(key)) {
      locale = loadLocale(key);
      if (locale) {
        return locale;
      }
      key = [key];
    }
    return chooseLocale(key);
  }
  function locale_locales__listLocales() {
    return keys(locales);
  }
  function checkOverflow(m) {
    var overflow;
    var a = m._a;
    if (a && getParsingFlags(m).overflow === -2) {
      overflow = a[MONTH] < 0 || a[MONTH] > 11 ? MONTH : a[DATE] < 1 || a[DATE] > daysInMonth(a[YEAR], a[MONTH]) ? DATE : a[HOUR] < 0 || a[HOUR] > 24 || (a[HOUR] === 24 && (a[MINUTE] !== 0 || a[SECOND] !== 0 || a[MILLISECOND] !== 0)) ? HOUR : a[MINUTE] < 0 || a[MINUTE] > 59 ? MINUTE : a[SECOND] < 0 || a[SECOND] > 59 ? SECOND : a[MILLISECOND] < 0 || a[MILLISECOND] > 999 ? MILLISECOND : -1;
      if (getParsingFlags(m)._overflowDayOfYear && (overflow < YEAR || overflow > DATE)) {
        overflow = DATE;
      }
      if (getParsingFlags(m)._overflowWeeks && overflow === -1) {
        overflow = WEEK;
      }
      if (getParsingFlags(m)._overflowWeekday && overflow === -1) {
        overflow = WEEKDAY;
      }
      getParsingFlags(m).overflow = overflow;
    }
    return m;
  }
  var extendedIsoRegex = /^\s*((?:[+-]\d{6}|\d{4})-(?:\d\d-\d\d|W\d\d-\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?::\d\d(?::\d\d(?:[.,]\d+)?)?)?)([\+\-]\d\d(?::?\d\d)?|\s*Z)?)?/;
  var basicIsoRegex = /^\s*((?:[+-]\d{6}|\d{4})(?:\d\d\d\d|W\d\d\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?:\d\d(?:\d\d(?:[.,]\d+)?)?)?)([\+\-]\d\d(?::?\d\d)?|\s*Z)?)?/;
  var tzRegex = /Z|[+-]\d\d(?::?\d\d)?/;
  var isoDates = [['YYYYYY-MM-DD', /[+-]\d{6}-\d\d-\d\d/], ['YYYY-MM-DD', /\d{4}-\d\d-\d\d/], ['GGGG-[W]WW-E', /\d{4}-W\d\d-\d/], ['GGGG-[W]WW', /\d{4}-W\d\d/, false], ['YYYY-DDD', /\d{4}-\d{3}/], ['YYYY-MM', /\d{4}-\d\d/, false], ['YYYYYYMMDD', /[+-]\d{10}/], ['YYYYMMDD', /\d{8}/], ['GGGG[W]WWE', /\d{4}W\d{3}/], ['GGGG[W]WW', /\d{4}W\d{2}/, false], ['YYYYDDD', /\d{7}/]];
  var isoTimes = [['HH:mm:ss.SSSS', /\d\d:\d\d:\d\d\.\d+/], ['HH:mm:ss,SSSS', /\d\d:\d\d:\d\d,\d+/], ['HH:mm:ss', /\d\d:\d\d:\d\d/], ['HH:mm', /\d\d:\d\d/], ['HHmmss.SSSS', /\d\d\d\d\d\d\.\d+/], ['HHmmss,SSSS', /\d\d\d\d\d\d,\d+/], ['HHmmss', /\d\d\d\d\d\d/], ['HHmm', /\d\d\d\d/], ['HH', /\d\d/]];
  var aspNetJsonRegex = /^\/?Date\((\-?\d+)/i;
  function configFromISO(config) {
    var i,
        l,
        string = config._i,
        match = extendedIsoRegex.exec(string) || basicIsoRegex.exec(string),
        allowTime,
        dateFormat,
        timeFormat,
        tzFormat;
    if (match) {
      getParsingFlags(config).iso = true;
      for (i = 0, l = isoDates.length; i < l; i++) {
        if (isoDates[i][1].exec(match[1])) {
          dateFormat = isoDates[i][0];
          allowTime = isoDates[i][2] !== false;
          break;
        }
      }
      if (dateFormat == null) {
        config._isValid = false;
        return;
      }
      if (match[3]) {
        for (i = 0, l = isoTimes.length; i < l; i++) {
          if (isoTimes[i][1].exec(match[3])) {
            timeFormat = (match[2] || ' ') + isoTimes[i][0];
            break;
          }
        }
        if (timeFormat == null) {
          config._isValid = false;
          return;
        }
      }
      if (!allowTime && timeFormat != null) {
        config._isValid = false;
        return;
      }
      if (match[4]) {
        if (tzRegex.exec(match[4])) {
          tzFormat = 'Z';
        } else {
          config._isValid = false;
          return;
        }
      }
      config._f = dateFormat + (timeFormat || '') + (tzFormat || '');
      configFromStringAndFormat(config);
    } else {
      config._isValid = false;
    }
  }
  function configFromString(config) {
    var matched = aspNetJsonRegex.exec(config._i);
    if (matched !== null) {
      config._d = new Date(+matched[1]);
      return;
    }
    configFromISO(config);
    if (config._isValid === false) {
      delete config._isValid;
      utils_hooks__hooks.createFromInputFallback(config);
    }
  }
  utils_hooks__hooks.createFromInputFallback = deprecate('moment construction falls back to js Date. This is ' + 'discouraged and will be removed in upcoming major ' + 'release. Please refer to ' + 'http://momentjs.com/guides/#/warnings/js-date/ for more info.', function(config) {
    config._d = new Date(config._i + (config._useUTC ? ' UTC' : ''));
  });
  function defaults(a, b, c) {
    if (a != null) {
      return a;
    }
    if (b != null) {
      return b;
    }
    return c;
  }
  function currentDateArray(config) {
    var nowValue = new Date(utils_hooks__hooks.now());
    if (config._useUTC) {
      return [nowValue.getUTCFullYear(), nowValue.getUTCMonth(), nowValue.getUTCDate()];
    }
    return [nowValue.getFullYear(), nowValue.getMonth(), nowValue.getDate()];
  }
  function configFromArray(config) {
    var i,
        date,
        input = [],
        currentDate,
        yearToUse;
    if (config._d) {
      return;
    }
    currentDate = currentDateArray(config);
    if (config._w && config._a[DATE] == null && config._a[MONTH] == null) {
      dayOfYearFromWeekInfo(config);
    }
    if (config._dayOfYear) {
      yearToUse = defaults(config._a[YEAR], currentDate[YEAR]);
      if (config._dayOfYear > daysInYear(yearToUse)) {
        getParsingFlags(config)._overflowDayOfYear = true;
      }
      date = createUTCDate(yearToUse, 0, config._dayOfYear);
      config._a[MONTH] = date.getUTCMonth();
      config._a[DATE] = date.getUTCDate();
    }
    for (i = 0; i < 3 && config._a[i] == null; ++i) {
      config._a[i] = input[i] = currentDate[i];
    }
    for (; i < 7; i++) {
      config._a[i] = input[i] = (config._a[i] == null) ? (i === 2 ? 1 : 0) : config._a[i];
    }
    if (config._a[HOUR] === 24 && config._a[MINUTE] === 0 && config._a[SECOND] === 0 && config._a[MILLISECOND] === 0) {
      config._nextDay = true;
      config._a[HOUR] = 0;
    }
    config._d = (config._useUTC ? createUTCDate : createDate).apply(null, input);
    if (config._tzm != null) {
      config._d.setUTCMinutes(config._d.getUTCMinutes() - config._tzm);
    }
    if (config._nextDay) {
      config._a[HOUR] = 24;
    }
  }
  function dayOfYearFromWeekInfo(config) {
    var w,
        weekYear,
        week,
        weekday,
        dow,
        doy,
        temp,
        weekdayOverflow;
    w = config._w;
    if (w.GG != null || w.W != null || w.E != null) {
      dow = 1;
      doy = 4;
      weekYear = defaults(w.GG, config._a[YEAR], weekOfYear(local__createLocal(), 1, 4).year);
      week = defaults(w.W, 1);
      weekday = defaults(w.E, 1);
      if (weekday < 1 || weekday > 7) {
        weekdayOverflow = true;
      }
    } else {
      dow = config._locale._week.dow;
      doy = config._locale._week.doy;
      weekYear = defaults(w.gg, config._a[YEAR], weekOfYear(local__createLocal(), dow, doy).year);
      week = defaults(w.w, 1);
      if (w.d != null) {
        weekday = w.d;
        if (weekday < 0 || weekday > 6) {
          weekdayOverflow = true;
        }
      } else if (w.e != null) {
        weekday = w.e + dow;
        if (w.e < 0 || w.e > 6) {
          weekdayOverflow = true;
        }
      } else {
        weekday = dow;
      }
    }
    if (week < 1 || week > weeksInYear(weekYear, dow, doy)) {
      getParsingFlags(config)._overflowWeeks = true;
    } else if (weekdayOverflow != null) {
      getParsingFlags(config)._overflowWeekday = true;
    } else {
      temp = dayOfYearFromWeeks(weekYear, week, weekday, dow, doy);
      config._a[YEAR] = temp.year;
      config._dayOfYear = temp.dayOfYear;
    }
  }
  utils_hooks__hooks.ISO_8601 = function() {};
  function configFromStringAndFormat(config) {
    if (config._f === utils_hooks__hooks.ISO_8601) {
      configFromISO(config);
      return;
    }
    config._a = [];
    getParsingFlags(config).empty = true;
    var string = '' + config._i,
        i,
        parsedInput,
        tokens,
        token,
        skipped,
        stringLength = string.length,
        totalParsedInputLength = 0;
    tokens = expandFormat(config._f, config._locale).match(formattingTokens) || [];
    for (i = 0; i < tokens.length; i++) {
      token = tokens[i];
      parsedInput = (string.match(getParseRegexForToken(token, config)) || [])[0];
      if (parsedInput) {
        skipped = string.substr(0, string.indexOf(parsedInput));
        if (skipped.length > 0) {
          getParsingFlags(config).unusedInput.push(skipped);
        }
        string = string.slice(string.indexOf(parsedInput) + parsedInput.length);
        totalParsedInputLength += parsedInput.length;
      }
      if (formatTokenFunctions[token]) {
        if (parsedInput) {
          getParsingFlags(config).empty = false;
        } else {
          getParsingFlags(config).unusedTokens.push(token);
        }
        addTimeToArrayFromToken(token, parsedInput, config);
      } else if (config._strict && !parsedInput) {
        getParsingFlags(config).unusedTokens.push(token);
      }
    }
    getParsingFlags(config).charsLeftOver = stringLength - totalParsedInputLength;
    if (string.length > 0) {
      getParsingFlags(config).unusedInput.push(string);
    }
    if (config._a[HOUR] <= 12 && getParsingFlags(config).bigHour === true && config._a[HOUR] > 0) {
      getParsingFlags(config).bigHour = undefined;
    }
    getParsingFlags(config).parsedDateParts = config._a.slice(0);
    getParsingFlags(config).meridiem = config._meridiem;
    config._a[HOUR] = meridiemFixWrap(config._locale, config._a[HOUR], config._meridiem);
    configFromArray(config);
    checkOverflow(config);
  }
  function meridiemFixWrap(locale, hour, meridiem) {
    var isPm;
    if (meridiem == null) {
      return hour;
    }
    if (locale.meridiemHour != null) {
      return locale.meridiemHour(hour, meridiem);
    } else if (locale.isPM != null) {
      isPm = locale.isPM(meridiem);
      if (isPm && hour < 12) {
        hour += 12;
      }
      if (!isPm && hour === 12) {
        hour = 0;
      }
      return hour;
    } else {
      return hour;
    }
  }
  function configFromStringAndArray(config) {
    var tempConfig,
        bestMoment,
        scoreToBeat,
        i,
        currentScore;
    if (config._f.length === 0) {
      getParsingFlags(config).invalidFormat = true;
      config._d = new Date(NaN);
      return;
    }
    for (i = 0; i < config._f.length; i++) {
      currentScore = 0;
      tempConfig = copyConfig({}, config);
      if (config._useUTC != null) {
        tempConfig._useUTC = config._useUTC;
      }
      tempConfig._f = config._f[i];
      configFromStringAndFormat(tempConfig);
      if (!valid__isValid(tempConfig)) {
        continue;
      }
      currentScore += getParsingFlags(tempConfig).charsLeftOver;
      currentScore += getParsingFlags(tempConfig).unusedTokens.length * 10;
      getParsingFlags(tempConfig).score = currentScore;
      if (scoreToBeat == null || currentScore < scoreToBeat) {
        scoreToBeat = currentScore;
        bestMoment = tempConfig;
      }
    }
    extend(config, bestMoment || tempConfig);
  }
  function configFromObject(config) {
    if (config._d) {
      return;
    }
    var i = normalizeObjectUnits(config._i);
    config._a = map([i.year, i.month, i.day || i.date, i.hour, i.minute, i.second, i.millisecond], function(obj) {
      return obj && parseInt(obj, 10);
    });
    configFromArray(config);
  }
  function createFromConfig(config) {
    var res = new Moment(checkOverflow(prepareConfig(config)));
    if (res._nextDay) {
      res.add(1, 'd');
      res._nextDay = undefined;
    }
    return res;
  }
  function prepareConfig(config) {
    var input = config._i,
        format = config._f;
    config._locale = config._locale || locale_locales__getLocale(config._l);
    if (input === null || (format === undefined && input === '')) {
      return valid__createInvalid({nullInput: true});
    }
    if (typeof input === 'string') {
      config._i = input = config._locale.preparse(input);
    }
    if (isMoment(input)) {
      return new Moment(checkOverflow(input));
    } else if (isArray(format)) {
      configFromStringAndArray(config);
    } else if (isDate(input)) {
      config._d = input;
    } else if (format) {
      configFromStringAndFormat(config);
    } else {
      configFromInput(config);
    }
    if (!valid__isValid(config)) {
      config._d = null;
    }
    return config;
  }
  function configFromInput(config) {
    var input = config._i;
    if (input === undefined) {
      config._d = new Date(utils_hooks__hooks.now());
    } else if (isDate(input)) {
      config._d = new Date(input.valueOf());
    } else if (typeof input === 'string') {
      configFromString(config);
    } else if (isArray(input)) {
      config._a = map(input.slice(0), function(obj) {
        return parseInt(obj, 10);
      });
      configFromArray(config);
    } else if (typeof(input) === 'object') {
      configFromObject(config);
    } else if (typeof(input) === 'number') {
      config._d = new Date(input);
    } else {
      utils_hooks__hooks.createFromInputFallback(config);
    }
  }
  function createLocalOrUTC(input, format, locale, strict, isUTC) {
    var c = {};
    if (typeof(locale) === 'boolean') {
      strict = locale;
      locale = undefined;
    }
    if ((isObject(input) && isObjectEmpty(input)) || (isArray(input) && input.length === 0)) {
      input = undefined;
    }
    c._isAMomentObject = true;
    c._useUTC = c._isUTC = isUTC;
    c._l = locale;
    c._i = input;
    c._f = format;
    c._strict = strict;
    return createFromConfig(c);
  }
  function local__createLocal(input, format, locale, strict) {
    return createLocalOrUTC(input, format, locale, strict, false);
  }
  var prototypeMin = deprecate('moment().min is deprecated, use moment.max instead. http://momentjs.com/guides/#/warnings/min-max/', function() {
    var other = local__createLocal.apply(null, arguments);
    if (this.isValid() && other.isValid()) {
      return other < this ? this : other;
    } else {
      return valid__createInvalid();
    }
  });
  var prototypeMax = deprecate('moment().max is deprecated, use moment.min instead. http://momentjs.com/guides/#/warnings/min-max/', function() {
    var other = local__createLocal.apply(null, arguments);
    if (this.isValid() && other.isValid()) {
      return other > this ? this : other;
    } else {
      return valid__createInvalid();
    }
  });
  function pickBy(fn, moments) {
    var res,
        i;
    if (moments.length === 1 && isArray(moments[0])) {
      moments = moments[0];
    }
    if (!moments.length) {
      return local__createLocal();
    }
    res = moments[0];
    for (i = 1; i < moments.length; ++i) {
      if (!moments[i].isValid() || moments[i][fn](res)) {
        res = moments[i];
      }
    }
    return res;
  }
  function min() {
    var args = [].slice.call(arguments, 0);
    return pickBy('isBefore', args);
  }
  function max() {
    var args = [].slice.call(arguments, 0);
    return pickBy('isAfter', args);
  }
  var now = function() {
    return Date.now ? Date.now() : +(new Date());
  };
  function Duration(duration) {
    var normalizedInput = normalizeObjectUnits(duration),
        years = normalizedInput.year || 0,
        quarters = normalizedInput.quarter || 0,
        months = normalizedInput.month || 0,
        weeks = normalizedInput.week || 0,
        days = normalizedInput.day || 0,
        hours = normalizedInput.hour || 0,
        minutes = normalizedInput.minute || 0,
        seconds = normalizedInput.second || 0,
        milliseconds = normalizedInput.millisecond || 0;
    this._milliseconds = +milliseconds + seconds * 1e3 + minutes * 6e4 + hours * 1000 * 60 * 60;
    this._days = +days + weeks * 7;
    this._months = +months + quarters * 3 + years * 12;
    this._data = {};
    this._locale = locale_locales__getLocale();
    this._bubble();
  }
  function isDuration(obj) {
    return obj instanceof Duration;
  }
  function offset(token, separator) {
    addFormatToken(token, 0, 0, function() {
      var offset = this.utcOffset();
      var sign = '+';
      if (offset < 0) {
        offset = -offset;
        sign = '-';
      }
      return sign + zeroFill(~~(offset / 60), 2) + separator + zeroFill(~~(offset) % 60, 2);
    });
  }
  offset('Z', ':');
  offset('ZZ', '');
  addRegexToken('Z', matchShortOffset);
  addRegexToken('ZZ', matchShortOffset);
  addParseToken(['Z', 'ZZ'], function(input, array, config) {
    config._useUTC = true;
    config._tzm = offsetFromString(matchShortOffset, input);
  });
  var chunkOffset = /([\+\-]|\d\d)/gi;
  function offsetFromString(matcher, string) {
    var matches = ((string || '').match(matcher) || []);
    var chunk = matches[matches.length - 1] || [];
    var parts = (chunk + '').match(chunkOffset) || ['-', 0, 0];
    var minutes = +(parts[1] * 60) + toInt(parts[2]);
    return parts[0] === '+' ? minutes : -minutes;
  }
  function cloneWithOffset(input, model) {
    var res,
        diff;
    if (model._isUTC) {
      res = model.clone();
      diff = (isMoment(input) || isDate(input) ? input.valueOf() : local__createLocal(input).valueOf()) - res.valueOf();
      res._d.setTime(res._d.valueOf() + diff);
      utils_hooks__hooks.updateOffset(res, false);
      return res;
    } else {
      return local__createLocal(input).local();
    }
  }
  function getDateOffset(m) {
    return -Math.round(m._d.getTimezoneOffset() / 15) * 15;
  }
  utils_hooks__hooks.updateOffset = function() {};
  function getSetOffset(input, keepLocalTime) {
    var offset = this._offset || 0,
        localAdjust;
    if (!this.isValid()) {
      return input != null ? this : NaN;
    }
    if (input != null) {
      if (typeof input === 'string') {
        input = offsetFromString(matchShortOffset, input);
      } else if (Math.abs(input) < 16) {
        input = input * 60;
      }
      if (!this._isUTC && keepLocalTime) {
        localAdjust = getDateOffset(this);
      }
      this._offset = input;
      this._isUTC = true;
      if (localAdjust != null) {
        this.add(localAdjust, 'm');
      }
      if (offset !== input) {
        if (!keepLocalTime || this._changeInProgress) {
          add_subtract__addSubtract(this, create__createDuration(input - offset, 'm'), 1, false);
        } else if (!this._changeInProgress) {
          this._changeInProgress = true;
          utils_hooks__hooks.updateOffset(this, true);
          this._changeInProgress = null;
        }
      }
      return this;
    } else {
      return this._isUTC ? offset : getDateOffset(this);
    }
  }
  function getSetZone(input, keepLocalTime) {
    if (input != null) {
      if (typeof input !== 'string') {
        input = -input;
      }
      this.utcOffset(input, keepLocalTime);
      return this;
    } else {
      return -this.utcOffset();
    }
  }
  function setOffsetToUTC(keepLocalTime) {
    return this.utcOffset(0, keepLocalTime);
  }
  function setOffsetToLocal(keepLocalTime) {
    if (this._isUTC) {
      this.utcOffset(0, keepLocalTime);
      this._isUTC = false;
      if (keepLocalTime) {
        this.subtract(getDateOffset(this), 'm');
      }
    }
    return this;
  }
  function setOffsetToParsedOffset() {
    if (this._tzm) {
      this.utcOffset(this._tzm);
    } else if (typeof this._i === 'string') {
      this.utcOffset(offsetFromString(matchOffset, this._i));
    }
    return this;
  }
  function hasAlignedHourOffset(input) {
    if (!this.isValid()) {
      return false;
    }
    input = input ? local__createLocal(input).utcOffset() : 0;
    return (this.utcOffset() - input) % 60 === 0;
  }
  function isDaylightSavingTime() {
    return (this.utcOffset() > this.clone().month(0).utcOffset() || this.utcOffset() > this.clone().month(5).utcOffset());
  }
  function isDaylightSavingTimeShifted() {
    if (!isUndefined(this._isDSTShifted)) {
      return this._isDSTShifted;
    }
    var c = {};
    copyConfig(c, this);
    c = prepareConfig(c);
    if (c._a) {
      var other = c._isUTC ? create_utc__createUTC(c._a) : local__createLocal(c._a);
      this._isDSTShifted = this.isValid() && compareArrays(c._a, other.toArray()) > 0;
    } else {
      this._isDSTShifted = false;
    }
    return this._isDSTShifted;
  }
  function isLocal() {
    return this.isValid() ? !this._isUTC : false;
  }
  function isUtcOffset() {
    return this.isValid() ? this._isUTC : false;
  }
  function isUtc() {
    return this.isValid() ? this._isUTC && this._offset === 0 : false;
  }
  var aspNetRegex = /^(\-)?(?:(\d*)[. ])?(\d+)\:(\d+)(?:\:(\d+)\.?(\d{3})?\d*)?$/;
  var isoRegex = /^(-)?P(?:(-?[0-9,.]*)Y)?(?:(-?[0-9,.]*)M)?(?:(-?[0-9,.]*)W)?(?:(-?[0-9,.]*)D)?(?:T(?:(-?[0-9,.]*)H)?(?:(-?[0-9,.]*)M)?(?:(-?[0-9,.]*)S)?)?$/;
  function create__createDuration(input, key) {
    var duration = input,
        match = null,
        sign,
        ret,
        diffRes;
    if (isDuration(input)) {
      duration = {
        ms: input._milliseconds,
        d: input._days,
        M: input._months
      };
    } else if (typeof input === 'number') {
      duration = {};
      if (key) {
        duration[key] = input;
      } else {
        duration.milliseconds = input;
      }
    } else if (!!(match = aspNetRegex.exec(input))) {
      sign = (match[1] === '-') ? -1 : 1;
      duration = {
        y: 0,
        d: toInt(match[DATE]) * sign,
        h: toInt(match[HOUR]) * sign,
        m: toInt(match[MINUTE]) * sign,
        s: toInt(match[SECOND]) * sign,
        ms: toInt(match[MILLISECOND]) * sign
      };
    } else if (!!(match = isoRegex.exec(input))) {
      sign = (match[1] === '-') ? -1 : 1;
      duration = {
        y: parseIso(match[2], sign),
        M: parseIso(match[3], sign),
        w: parseIso(match[4], sign),
        d: parseIso(match[5], sign),
        h: parseIso(match[6], sign),
        m: parseIso(match[7], sign),
        s: parseIso(match[8], sign)
      };
    } else if (duration == null) {
      duration = {};
    } else if (typeof duration === 'object' && ('from' in duration || 'to' in duration)) {
      diffRes = momentsDifference(local__createLocal(duration.from), local__createLocal(duration.to));
      duration = {};
      duration.ms = diffRes.milliseconds;
      duration.M = diffRes.months;
    }
    ret = new Duration(duration);
    if (isDuration(input) && hasOwnProp(input, '_locale')) {
      ret._locale = input._locale;
    }
    return ret;
  }
  create__createDuration.fn = Duration.prototype;
  function parseIso(inp, sign) {
    var res = inp && parseFloat(inp.replace(',', '.'));
    return (isNaN(res) ? 0 : res) * sign;
  }
  function positiveMomentsDifference(base, other) {
    var res = {
      milliseconds: 0,
      months: 0
    };
    res.months = other.month() - base.month() + (other.year() - base.year()) * 12;
    if (base.clone().add(res.months, 'M').isAfter(other)) {
      --res.months;
    }
    res.milliseconds = +other - +(base.clone().add(res.months, 'M'));
    return res;
  }
  function momentsDifference(base, other) {
    var res;
    if (!(base.isValid() && other.isValid())) {
      return {
        milliseconds: 0,
        months: 0
      };
    }
    other = cloneWithOffset(other, base);
    if (base.isBefore(other)) {
      res = positiveMomentsDifference(base, other);
    } else {
      res = positiveMomentsDifference(other, base);
      res.milliseconds = -res.milliseconds;
      res.months = -res.months;
    }
    return res;
  }
  function absRound(number) {
    if (number < 0) {
      return Math.round(-1 * number) * -1;
    } else {
      return Math.round(number);
    }
  }
  function createAdder(direction, name) {
    return function(val, period) {
      var dur,
          tmp;
      if (period !== null && !isNaN(+period)) {
        deprecateSimple(name, 'moment().' + name + '(period, number) is deprecated. Please use moment().' + name + '(number, period). ' + 'See http://momentjs.com/guides/#/warnings/add-inverted-param/ for more info.');
        tmp = val;
        val = period;
        period = tmp;
      }
      val = typeof val === 'string' ? +val : val;
      dur = create__createDuration(val, period);
      add_subtract__addSubtract(this, dur, direction);
      return this;
    };
  }
  function add_subtract__addSubtract(mom, duration, isAdding, updateOffset) {
    var milliseconds = duration._milliseconds,
        days = absRound(duration._days),
        months = absRound(duration._months);
    if (!mom.isValid()) {
      return;
    }
    updateOffset = updateOffset == null ? true : updateOffset;
    if (milliseconds) {
      mom._d.setTime(mom._d.valueOf() + milliseconds * isAdding);
    }
    if (days) {
      get_set__set(mom, 'Date', get_set__get(mom, 'Date') + days * isAdding);
    }
    if (months) {
      setMonth(mom, get_set__get(mom, 'Month') + months * isAdding);
    }
    if (updateOffset) {
      utils_hooks__hooks.updateOffset(mom, days || months);
    }
  }
  var add_subtract__add = createAdder(1, 'add');
  var add_subtract__subtract = createAdder(-1, 'subtract');
  function getCalendarFormat(myMoment, now) {
    var diff = myMoment.diff(now, 'days', true);
    return diff < -6 ? 'sameElse' : diff < -1 ? 'lastWeek' : diff < 0 ? 'lastDay' : diff < 1 ? 'sameDay' : diff < 2 ? 'nextDay' : diff < 7 ? 'nextWeek' : 'sameElse';
  }
  function moment_calendar__calendar(time, formats) {
    var now = time || local__createLocal(),
        sod = cloneWithOffset(now, this).startOf('day'),
        format = utils_hooks__hooks.calendarFormat(this, sod) || 'sameElse';
    var output = formats && (isFunction(formats[format]) ? formats[format].call(this, now) : formats[format]);
    return this.format(output || this.localeData().calendar(format, this, local__createLocal(now)));
  }
  function clone() {
    return new Moment(this);
  }
  function isAfter(input, units) {
    var localInput = isMoment(input) ? input : local__createLocal(input);
    if (!(this.isValid() && localInput.isValid())) {
      return false;
    }
    units = normalizeUnits(!isUndefined(units) ? units : 'millisecond');
    if (units === 'millisecond') {
      return this.valueOf() > localInput.valueOf();
    } else {
      return localInput.valueOf() < this.clone().startOf(units).valueOf();
    }
  }
  function isBefore(input, units) {
    var localInput = isMoment(input) ? input : local__createLocal(input);
    if (!(this.isValid() && localInput.isValid())) {
      return false;
    }
    units = normalizeUnits(!isUndefined(units) ? units : 'millisecond');
    if (units === 'millisecond') {
      return this.valueOf() < localInput.valueOf();
    } else {
      return this.clone().endOf(units).valueOf() < localInput.valueOf();
    }
  }
  function isBetween(from, to, units, inclusivity) {
    inclusivity = inclusivity || '()';
    return (inclusivity[0] === '(' ? this.isAfter(from, units) : !this.isBefore(from, units)) && (inclusivity[1] === ')' ? this.isBefore(to, units) : !this.isAfter(to, units));
  }
  function isSame(input, units) {
    var localInput = isMoment(input) ? input : local__createLocal(input),
        inputMs;
    if (!(this.isValid() && localInput.isValid())) {
      return false;
    }
    units = normalizeUnits(units || 'millisecond');
    if (units === 'millisecond') {
      return this.valueOf() === localInput.valueOf();
    } else {
      inputMs = localInput.valueOf();
      return this.clone().startOf(units).valueOf() <= inputMs && inputMs <= this.clone().endOf(units).valueOf();
    }
  }
  function isSameOrAfter(input, units) {
    return this.isSame(input, units) || this.isAfter(input, units);
  }
  function isSameOrBefore(input, units) {
    return this.isSame(input, units) || this.isBefore(input, units);
  }
  function diff(input, units, asFloat) {
    var that,
        zoneDelta,
        delta,
        output;
    if (!this.isValid()) {
      return NaN;
    }
    that = cloneWithOffset(input, this);
    if (!that.isValid()) {
      return NaN;
    }
    zoneDelta = (that.utcOffset() - this.utcOffset()) * 6e4;
    units = normalizeUnits(units);
    if (units === 'year' || units === 'month' || units === 'quarter') {
      output = monthDiff(this, that);
      if (units === 'quarter') {
        output = output / 3;
      } else if (units === 'year') {
        output = output / 12;
      }
    } else {
      delta = this - that;
      output = units === 'second' ? delta / 1e3 : units === 'minute' ? delta / 6e4 : units === 'hour' ? delta / 36e5 : units === 'day' ? (delta - zoneDelta) / 864e5 : units === 'week' ? (delta - zoneDelta) / 6048e5 : delta;
    }
    return asFloat ? output : absFloor(output);
  }
  function monthDiff(a, b) {
    var wholeMonthDiff = ((b.year() - a.year()) * 12) + (b.month() - a.month()),
        anchor = a.clone().add(wholeMonthDiff, 'months'),
        anchor2,
        adjust;
    if (b - anchor < 0) {
      anchor2 = a.clone().add(wholeMonthDiff - 1, 'months');
      adjust = (b - anchor) / (anchor - anchor2);
    } else {
      anchor2 = a.clone().add(wholeMonthDiff + 1, 'months');
      adjust = (b - anchor) / (anchor2 - anchor);
    }
    return -(wholeMonthDiff + adjust) || 0;
  }
  utils_hooks__hooks.defaultFormat = 'YYYY-MM-DDTHH:mm:ssZ';
  utils_hooks__hooks.defaultFormatUtc = 'YYYY-MM-DDTHH:mm:ss[Z]';
  function toString() {
    return this.clone().locale('en').format('ddd MMM DD YYYY HH:mm:ss [GMT]ZZ');
  }
  function moment_format__toISOString() {
    var m = this.clone().utc();
    if (0 < m.year() && m.year() <= 9999) {
      if (isFunction(Date.prototype.toISOString)) {
        return this.toDate().toISOString();
      } else {
        return formatMoment(m, 'YYYY-MM-DD[T]HH:mm:ss.SSS[Z]');
      }
    } else {
      return formatMoment(m, 'YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]');
    }
  }
  function format(inputString) {
    if (!inputString) {
      inputString = this.isUtc() ? utils_hooks__hooks.defaultFormatUtc : utils_hooks__hooks.defaultFormat;
    }
    var output = formatMoment(this, inputString);
    return this.localeData().postformat(output);
  }
  function from(time, withoutSuffix) {
    if (this.isValid() && ((isMoment(time) && time.isValid()) || local__createLocal(time).isValid())) {
      return create__createDuration({
        to: this,
        from: time
      }).locale(this.locale()).humanize(!withoutSuffix);
    } else {
      return this.localeData().invalidDate();
    }
  }
  function fromNow(withoutSuffix) {
    return this.from(local__createLocal(), withoutSuffix);
  }
  function to(time, withoutSuffix) {
    if (this.isValid() && ((isMoment(time) && time.isValid()) || local__createLocal(time).isValid())) {
      return create__createDuration({
        from: this,
        to: time
      }).locale(this.locale()).humanize(!withoutSuffix);
    } else {
      return this.localeData().invalidDate();
    }
  }
  function toNow(withoutSuffix) {
    return this.to(local__createLocal(), withoutSuffix);
  }
  function locale(key) {
    var newLocaleData;
    if (key === undefined) {
      return this._locale._abbr;
    } else {
      newLocaleData = locale_locales__getLocale(key);
      if (newLocaleData != null) {
        this._locale = newLocaleData;
      }
      return this;
    }
  }
  var lang = deprecate('moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.', function(key) {
    if (key === undefined) {
      return this.localeData();
    } else {
      return this.locale(key);
    }
  });
  function localeData() {
    return this._locale;
  }
  function startOf(units) {
    units = normalizeUnits(units);
    switch (units) {
      case 'year':
        this.month(0);
      case 'quarter':
      case 'month':
        this.date(1);
      case 'week':
      case 'isoWeek':
      case 'day':
      case 'date':
        this.hours(0);
      case 'hour':
        this.minutes(0);
      case 'minute':
        this.seconds(0);
      case 'second':
        this.milliseconds(0);
    }
    if (units === 'week') {
      this.weekday(0);
    }
    if (units === 'isoWeek') {
      this.isoWeekday(1);
    }
    if (units === 'quarter') {
      this.month(Math.floor(this.month() / 3) * 3);
    }
    return this;
  }
  function endOf(units) {
    units = normalizeUnits(units);
    if (units === undefined || units === 'millisecond') {
      return this;
    }
    if (units === 'date') {
      units = 'day';
    }
    return this.startOf(units).add(1, (units === 'isoWeek' ? 'week' : units)).subtract(1, 'ms');
  }
  function to_type__valueOf() {
    return this._d.valueOf() - ((this._offset || 0) * 60000);
  }
  function unix() {
    return Math.floor(this.valueOf() / 1000);
  }
  function toDate() {
    return new Date(this.valueOf());
  }
  function toArray() {
    var m = this;
    return [m.year(), m.month(), m.date(), m.hour(), m.minute(), m.second(), m.millisecond()];
  }
  function toObject() {
    var m = this;
    return {
      years: m.year(),
      months: m.month(),
      date: m.date(),
      hours: m.hours(),
      minutes: m.minutes(),
      seconds: m.seconds(),
      milliseconds: m.milliseconds()
    };
  }
  function toJSON() {
    return this.isValid() ? this.toISOString() : null;
  }
  function moment_valid__isValid() {
    return valid__isValid(this);
  }
  function parsingFlags() {
    return extend({}, getParsingFlags(this));
  }
  function invalidAt() {
    return getParsingFlags(this).overflow;
  }
  function creationData() {
    return {
      input: this._i,
      format: this._f,
      locale: this._locale,
      isUTC: this._isUTC,
      strict: this._strict
    };
  }
  addFormatToken(0, ['gg', 2], 0, function() {
    return this.weekYear() % 100;
  });
  addFormatToken(0, ['GG', 2], 0, function() {
    return this.isoWeekYear() % 100;
  });
  function addWeekYearFormatToken(token, getter) {
    addFormatToken(0, [token, token.length], 0, getter);
  }
  addWeekYearFormatToken('gggg', 'weekYear');
  addWeekYearFormatToken('ggggg', 'weekYear');
  addWeekYearFormatToken('GGGG', 'isoWeekYear');
  addWeekYearFormatToken('GGGGG', 'isoWeekYear');
  addUnitAlias('weekYear', 'gg');
  addUnitAlias('isoWeekYear', 'GG');
  addUnitPriority('weekYear', 1);
  addUnitPriority('isoWeekYear', 1);
  addRegexToken('G', matchSigned);
  addRegexToken('g', matchSigned);
  addRegexToken('GG', match1to2, match2);
  addRegexToken('gg', match1to2, match2);
  addRegexToken('GGGG', match1to4, match4);
  addRegexToken('gggg', match1to4, match4);
  addRegexToken('GGGGG', match1to6, match6);
  addRegexToken('ggggg', match1to6, match6);
  addWeekParseToken(['gggg', 'ggggg', 'GGGG', 'GGGGG'], function(input, week, config, token) {
    week[token.substr(0, 2)] = toInt(input);
  });
  addWeekParseToken(['gg', 'GG'], function(input, week, config, token) {
    week[token] = utils_hooks__hooks.parseTwoDigitYear(input);
  });
  function getSetWeekYear(input) {
    return getSetWeekYearHelper.call(this, input, this.week(), this.weekday(), this.localeData()._week.dow, this.localeData()._week.doy);
  }
  function getSetISOWeekYear(input) {
    return getSetWeekYearHelper.call(this, input, this.isoWeek(), this.isoWeekday(), 1, 4);
  }
  function getISOWeeksInYear() {
    return weeksInYear(this.year(), 1, 4);
  }
  function getWeeksInYear() {
    var weekInfo = this.localeData()._week;
    return weeksInYear(this.year(), weekInfo.dow, weekInfo.doy);
  }
  function getSetWeekYearHelper(input, week, weekday, dow, doy) {
    var weeksTarget;
    if (input == null) {
      return weekOfYear(this, dow, doy).year;
    } else {
      weeksTarget = weeksInYear(input, dow, doy);
      if (week > weeksTarget) {
        week = weeksTarget;
      }
      return setWeekAll.call(this, input, week, weekday, dow, doy);
    }
  }
  function setWeekAll(weekYear, week, weekday, dow, doy) {
    var dayOfYearData = dayOfYearFromWeeks(weekYear, week, weekday, dow, doy),
        date = createUTCDate(dayOfYearData.year, 0, dayOfYearData.dayOfYear);
    this.year(date.getUTCFullYear());
    this.month(date.getUTCMonth());
    this.date(date.getUTCDate());
    return this;
  }
  addFormatToken('Q', 0, 'Qo', 'quarter');
  addUnitAlias('quarter', 'Q');
  addUnitPriority('quarter', 7);
  addRegexToken('Q', match1);
  addParseToken('Q', function(input, array) {
    array[MONTH] = (toInt(input) - 1) * 3;
  });
  function getSetQuarter(input) {
    return input == null ? Math.ceil((this.month() + 1) / 3) : this.month((input - 1) * 3 + this.month() % 3);
  }
  addFormatToken('D', ['DD', 2], 'Do', 'date');
  addUnitAlias('date', 'D');
  addUnitPriority('date', 9);
  addRegexToken('D', match1to2);
  addRegexToken('DD', match1to2, match2);
  addRegexToken('Do', function(isStrict, locale) {
    return isStrict ? locale._ordinalParse : locale._ordinalParseLenient;
  });
  addParseToken(['D', 'DD'], DATE);
  addParseToken('Do', function(input, array) {
    array[DATE] = toInt(input.match(match1to2)[0], 10);
  });
  var getSetDayOfMonth = makeGetSet('Date', true);
  addFormatToken('DDD', ['DDDD', 3], 'DDDo', 'dayOfYear');
  addUnitAlias('dayOfYear', 'DDD');
  addUnitPriority('dayOfYear', 4);
  addRegexToken('DDD', match1to3);
  addRegexToken('DDDD', match3);
  addParseToken(['DDD', 'DDDD'], function(input, array, config) {
    config._dayOfYear = toInt(input);
  });
  function getSetDayOfYear(input) {
    var dayOfYear = Math.round((this.clone().startOf('day') - this.clone().startOf('year')) / 864e5) + 1;
    return input == null ? dayOfYear : this.add((input - dayOfYear), 'd');
  }
  addFormatToken('m', ['mm', 2], 0, 'minute');
  addUnitAlias('minute', 'm');
  addUnitPriority('minute', 14);
  addRegexToken('m', match1to2);
  addRegexToken('mm', match1to2, match2);
  addParseToken(['m', 'mm'], MINUTE);
  var getSetMinute = makeGetSet('Minutes', false);
  addFormatToken('s', ['ss', 2], 0, 'second');
  addUnitAlias('second', 's');
  addUnitPriority('second', 15);
  addRegexToken('s', match1to2);
  addRegexToken('ss', match1to2, match2);
  addParseToken(['s', 'ss'], SECOND);
  var getSetSecond = makeGetSet('Seconds', false);
  addFormatToken('S', 0, 0, function() {
    return ~~(this.millisecond() / 100);
  });
  addFormatToken(0, ['SS', 2], 0, function() {
    return ~~(this.millisecond() / 10);
  });
  addFormatToken(0, ['SSS', 3], 0, 'millisecond');
  addFormatToken(0, ['SSSS', 4], 0, function() {
    return this.millisecond() * 10;
  });
  addFormatToken(0, ['SSSSS', 5], 0, function() {
    return this.millisecond() * 100;
  });
  addFormatToken(0, ['SSSSSS', 6], 0, function() {
    return this.millisecond() * 1000;
  });
  addFormatToken(0, ['SSSSSSS', 7], 0, function() {
    return this.millisecond() * 10000;
  });
  addFormatToken(0, ['SSSSSSSS', 8], 0, function() {
    return this.millisecond() * 100000;
  });
  addFormatToken(0, ['SSSSSSSSS', 9], 0, function() {
    return this.millisecond() * 1000000;
  });
  addUnitAlias('millisecond', 'ms');
  addUnitPriority('millisecond', 16);
  addRegexToken('S', match1to3, match1);
  addRegexToken('SS', match1to3, match2);
  addRegexToken('SSS', match1to3, match3);
  var token;
  for (token = 'SSSS'; token.length <= 9; token += 'S') {
    addRegexToken(token, matchUnsigned);
  }
  function parseMs(input, array) {
    array[MILLISECOND] = toInt(('0.' + input) * 1000);
  }
  for (token = 'S'; token.length <= 9; token += 'S') {
    addParseToken(token, parseMs);
  }
  var getSetMillisecond = makeGetSet('Milliseconds', false);
  addFormatToken('z', 0, 0, 'zoneAbbr');
  addFormatToken('zz', 0, 0, 'zoneName');
  function getZoneAbbr() {
    return this._isUTC ? 'UTC' : '';
  }
  function getZoneName() {
    return this._isUTC ? 'Coordinated Universal Time' : '';
  }
  var momentPrototype__proto = Moment.prototype;
  momentPrototype__proto.add = add_subtract__add;
  momentPrototype__proto.calendar = moment_calendar__calendar;
  momentPrototype__proto.clone = clone;
  momentPrototype__proto.diff = diff;
  momentPrototype__proto.endOf = endOf;
  momentPrototype__proto.format = format;
  momentPrototype__proto.from = from;
  momentPrototype__proto.fromNow = fromNow;
  momentPrototype__proto.to = to;
  momentPrototype__proto.toNow = toNow;
  momentPrototype__proto.get = stringGet;
  momentPrototype__proto.invalidAt = invalidAt;
  momentPrototype__proto.isAfter = isAfter;
  momentPrototype__proto.isBefore = isBefore;
  momentPrototype__proto.isBetween = isBetween;
  momentPrototype__proto.isSame = isSame;
  momentPrototype__proto.isSameOrAfter = isSameOrAfter;
  momentPrototype__proto.isSameOrBefore = isSameOrBefore;
  momentPrototype__proto.isValid = moment_valid__isValid;
  momentPrototype__proto.lang = lang;
  momentPrototype__proto.locale = locale;
  momentPrototype__proto.localeData = localeData;
  momentPrototype__proto.max = prototypeMax;
  momentPrototype__proto.min = prototypeMin;
  momentPrototype__proto.parsingFlags = parsingFlags;
  momentPrototype__proto.set = stringSet;
  momentPrototype__proto.startOf = startOf;
  momentPrototype__proto.subtract = add_subtract__subtract;
  momentPrototype__proto.toArray = toArray;
  momentPrototype__proto.toObject = toObject;
  momentPrototype__proto.toDate = toDate;
  momentPrototype__proto.toISOString = moment_format__toISOString;
  momentPrototype__proto.toJSON = toJSON;
  momentPrototype__proto.toString = toString;
  momentPrototype__proto.unix = unix;
  momentPrototype__proto.valueOf = to_type__valueOf;
  momentPrototype__proto.creationData = creationData;
  momentPrototype__proto.year = getSetYear;
  momentPrototype__proto.isLeapYear = getIsLeapYear;
  momentPrototype__proto.weekYear = getSetWeekYear;
  momentPrototype__proto.isoWeekYear = getSetISOWeekYear;
  momentPrototype__proto.quarter = momentPrototype__proto.quarters = getSetQuarter;
  momentPrototype__proto.month = getSetMonth;
  momentPrototype__proto.daysInMonth = getDaysInMonth;
  momentPrototype__proto.week = momentPrototype__proto.weeks = getSetWeek;
  momentPrototype__proto.isoWeek = momentPrototype__proto.isoWeeks = getSetISOWeek;
  momentPrototype__proto.weeksInYear = getWeeksInYear;
  momentPrototype__proto.isoWeeksInYear = getISOWeeksInYear;
  momentPrototype__proto.date = getSetDayOfMonth;
  momentPrototype__proto.day = momentPrototype__proto.days = getSetDayOfWeek;
  momentPrototype__proto.weekday = getSetLocaleDayOfWeek;
  momentPrototype__proto.isoWeekday = getSetISODayOfWeek;
  momentPrototype__proto.dayOfYear = getSetDayOfYear;
  momentPrototype__proto.hour = momentPrototype__proto.hours = getSetHour;
  momentPrototype__proto.minute = momentPrototype__proto.minutes = getSetMinute;
  momentPrototype__proto.second = momentPrototype__proto.seconds = getSetSecond;
  momentPrototype__proto.millisecond = momentPrototype__proto.milliseconds = getSetMillisecond;
  momentPrototype__proto.utcOffset = getSetOffset;
  momentPrototype__proto.utc = setOffsetToUTC;
  momentPrototype__proto.local = setOffsetToLocal;
  momentPrototype__proto.parseZone = setOffsetToParsedOffset;
  momentPrototype__proto.hasAlignedHourOffset = hasAlignedHourOffset;
  momentPrototype__proto.isDST = isDaylightSavingTime;
  momentPrototype__proto.isLocal = isLocal;
  momentPrototype__proto.isUtcOffset = isUtcOffset;
  momentPrototype__proto.isUtc = isUtc;
  momentPrototype__proto.isUTC = isUtc;
  momentPrototype__proto.zoneAbbr = getZoneAbbr;
  momentPrototype__proto.zoneName = getZoneName;
  momentPrototype__proto.dates = deprecate('dates accessor is deprecated. Use date instead.', getSetDayOfMonth);
  momentPrototype__proto.months = deprecate('months accessor is deprecated. Use month instead', getSetMonth);
  momentPrototype__proto.years = deprecate('years accessor is deprecated. Use year instead', getSetYear);
  momentPrototype__proto.zone = deprecate('moment().zone is deprecated, use moment().utcOffset instead. http://momentjs.com/guides/#/warnings/zone/', getSetZone);
  momentPrototype__proto.isDSTShifted = deprecate('isDSTShifted is deprecated. See http://momentjs.com/guides/#/warnings/dst-shifted/ for more information', isDaylightSavingTimeShifted);
  var momentPrototype = momentPrototype__proto;
  function moment__createUnix(input) {
    return local__createLocal(input * 1000);
  }
  function moment__createInZone() {
    return local__createLocal.apply(null, arguments).parseZone();
  }
  function preParsePostFormat(string) {
    return string;
  }
  var prototype__proto = Locale.prototype;
  prototype__proto.calendar = locale_calendar__calendar;
  prototype__proto.longDateFormat = longDateFormat;
  prototype__proto.invalidDate = invalidDate;
  prototype__proto.ordinal = ordinal;
  prototype__proto.preparse = preParsePostFormat;
  prototype__proto.postformat = preParsePostFormat;
  prototype__proto.relativeTime = relative__relativeTime;
  prototype__proto.pastFuture = pastFuture;
  prototype__proto.set = locale_set__set;
  prototype__proto.months = localeMonths;
  prototype__proto.monthsShort = localeMonthsShort;
  prototype__proto.monthsParse = localeMonthsParse;
  prototype__proto.monthsRegex = monthsRegex;
  prototype__proto.monthsShortRegex = monthsShortRegex;
  prototype__proto.week = localeWeek;
  prototype__proto.firstDayOfYear = localeFirstDayOfYear;
  prototype__proto.firstDayOfWeek = localeFirstDayOfWeek;
  prototype__proto.weekdays = localeWeekdays;
  prototype__proto.weekdaysMin = localeWeekdaysMin;
  prototype__proto.weekdaysShort = localeWeekdaysShort;
  prototype__proto.weekdaysParse = localeWeekdaysParse;
  prototype__proto.weekdaysRegex = weekdaysRegex;
  prototype__proto.weekdaysShortRegex = weekdaysShortRegex;
  prototype__proto.weekdaysMinRegex = weekdaysMinRegex;
  prototype__proto.isPM = localeIsPM;
  prototype__proto.meridiem = localeMeridiem;
  function lists__get(format, index, field, setter) {
    var locale = locale_locales__getLocale();
    var utc = create_utc__createUTC().set(setter, index);
    return locale[field](utc, format);
  }
  function listMonthsImpl(format, index, field) {
    if (typeof format === 'number') {
      index = format;
      format = undefined;
    }
    format = format || '';
    if (index != null) {
      return lists__get(format, index, field, 'month');
    }
    var i;
    var out = [];
    for (i = 0; i < 12; i++) {
      out[i] = lists__get(format, i, field, 'month');
    }
    return out;
  }
  function listWeekdaysImpl(localeSorted, format, index, field) {
    if (typeof localeSorted === 'boolean') {
      if (typeof format === 'number') {
        index = format;
        format = undefined;
      }
      format = format || '';
    } else {
      format = localeSorted;
      index = format;
      localeSorted = false;
      if (typeof format === 'number') {
        index = format;
        format = undefined;
      }
      format = format || '';
    }
    var locale = locale_locales__getLocale(),
        shift = localeSorted ? locale._week.dow : 0;
    if (index != null) {
      return lists__get(format, (index + shift) % 7, field, 'day');
    }
    var i;
    var out = [];
    for (i = 0; i < 7; i++) {
      out[i] = lists__get(format, (i + shift) % 7, field, 'day');
    }
    return out;
  }
  function lists__listMonths(format, index) {
    return listMonthsImpl(format, index, 'months');
  }
  function lists__listMonthsShort(format, index) {
    return listMonthsImpl(format, index, 'monthsShort');
  }
  function lists__listWeekdays(localeSorted, format, index) {
    return listWeekdaysImpl(localeSorted, format, index, 'weekdays');
  }
  function lists__listWeekdaysShort(localeSorted, format, index) {
    return listWeekdaysImpl(localeSorted, format, index, 'weekdaysShort');
  }
  function lists__listWeekdaysMin(localeSorted, format, index) {
    return listWeekdaysImpl(localeSorted, format, index, 'weekdaysMin');
  }
  locale_locales__getSetGlobalLocale('en', {
    ordinalParse: /\d{1,2}(th|st|nd|rd)/,
    ordinal: function(number) {
      var b = number % 10,
          output = (toInt(number % 100 / 10) === 1) ? 'th' : (b === 1) ? 'st' : (b === 2) ? 'nd' : (b === 3) ? 'rd' : 'th';
      return number + output;
    }
  });
  utils_hooks__hooks.lang = deprecate('moment.lang is deprecated. Use moment.locale instead.', locale_locales__getSetGlobalLocale);
  utils_hooks__hooks.langData = deprecate('moment.langData is deprecated. Use moment.localeData instead.', locale_locales__getLocale);
  var mathAbs = Math.abs;
  function duration_abs__abs() {
    var data = this._data;
    this._milliseconds = mathAbs(this._milliseconds);
    this._days = mathAbs(this._days);
    this._months = mathAbs(this._months);
    data.milliseconds = mathAbs(data.milliseconds);
    data.seconds = mathAbs(data.seconds);
    data.minutes = mathAbs(data.minutes);
    data.hours = mathAbs(data.hours);
    data.months = mathAbs(data.months);
    data.years = mathAbs(data.years);
    return this;
  }
  function duration_add_subtract__addSubtract(duration, input, value, direction) {
    var other = create__createDuration(input, value);
    duration._milliseconds += direction * other._milliseconds;
    duration._days += direction * other._days;
    duration._months += direction * other._months;
    return duration._bubble();
  }
  function duration_add_subtract__add(input, value) {
    return duration_add_subtract__addSubtract(this, input, value, 1);
  }
  function duration_add_subtract__subtract(input, value) {
    return duration_add_subtract__addSubtract(this, input, value, -1);
  }
  function absCeil(number) {
    if (number < 0) {
      return Math.floor(number);
    } else {
      return Math.ceil(number);
    }
  }
  function bubble() {
    var milliseconds = this._milliseconds;
    var days = this._days;
    var months = this._months;
    var data = this._data;
    var seconds,
        minutes,
        hours,
        years,
        monthsFromDays;
    if (!((milliseconds >= 0 && days >= 0 && months >= 0) || (milliseconds <= 0 && days <= 0 && months <= 0))) {
      milliseconds += absCeil(monthsToDays(months) + days) * 864e5;
      days = 0;
      months = 0;
    }
    data.milliseconds = milliseconds % 1000;
    seconds = absFloor(milliseconds / 1000);
    data.seconds = seconds % 60;
    minutes = absFloor(seconds / 60);
    data.minutes = minutes % 60;
    hours = absFloor(minutes / 60);
    data.hours = hours % 24;
    days += absFloor(hours / 24);
    monthsFromDays = absFloor(daysToMonths(days));
    months += monthsFromDays;
    days -= absCeil(monthsToDays(monthsFromDays));
    years = absFloor(months / 12);
    months %= 12;
    data.days = days;
    data.months = months;
    data.years = years;
    return this;
  }
  function daysToMonths(days) {
    return days * 4800 / 146097;
  }
  function monthsToDays(months) {
    return months * 146097 / 4800;
  }
  function as(units) {
    var days;
    var months;
    var milliseconds = this._milliseconds;
    units = normalizeUnits(units);
    if (units === 'month' || units === 'year') {
      days = this._days + milliseconds / 864e5;
      months = this._months + daysToMonths(days);
      return units === 'month' ? months : months / 12;
    } else {
      days = this._days + Math.round(monthsToDays(this._months));
      switch (units) {
        case 'week':
          return days / 7 + milliseconds / 6048e5;
        case 'day':
          return days + milliseconds / 864e5;
        case 'hour':
          return days * 24 + milliseconds / 36e5;
        case 'minute':
          return days * 1440 + milliseconds / 6e4;
        case 'second':
          return days * 86400 + milliseconds / 1000;
        case 'millisecond':
          return Math.floor(days * 864e5) + milliseconds;
        default:
          throw new Error('Unknown unit ' + units);
      }
    }
  }
  function duration_as__valueOf() {
    return (this._milliseconds + this._days * 864e5 + (this._months % 12) * 2592e6 + toInt(this._months / 12) * 31536e6);
  }
  function makeAs(alias) {
    return function() {
      return this.as(alias);
    };
  }
  var asMilliseconds = makeAs('ms');
  var asSeconds = makeAs('s');
  var asMinutes = makeAs('m');
  var asHours = makeAs('h');
  var asDays = makeAs('d');
  var asWeeks = makeAs('w');
  var asMonths = makeAs('M');
  var asYears = makeAs('y');
  function duration_get__get(units) {
    units = normalizeUnits(units);
    return this[units + 's']();
  }
  function makeGetter(name) {
    return function() {
      return this._data[name];
    };
  }
  var milliseconds = makeGetter('milliseconds');
  var seconds = makeGetter('seconds');
  var minutes = makeGetter('minutes');
  var hours = makeGetter('hours');
  var days = makeGetter('days');
  var months = makeGetter('months');
  var years = makeGetter('years');
  function weeks() {
    return absFloor(this.days() / 7);
  }
  var round = Math.round;
  var thresholds = {
    s: 45,
    m: 45,
    h: 22,
    d: 26,
    M: 11
  };
  function substituteTimeAgo(string, number, withoutSuffix, isFuture, locale) {
    return locale.relativeTime(number || 1, !!withoutSuffix, string, isFuture);
  }
  function duration_humanize__relativeTime(posNegDuration, withoutSuffix, locale) {
    var duration = create__createDuration(posNegDuration).abs();
    var seconds = round(duration.as('s'));
    var minutes = round(duration.as('m'));
    var hours = round(duration.as('h'));
    var days = round(duration.as('d'));
    var months = round(duration.as('M'));
    var years = round(duration.as('y'));
    var a = seconds < thresholds.s && ['s', seconds] || minutes <= 1 && ['m'] || minutes < thresholds.m && ['mm', minutes] || hours <= 1 && ['h'] || hours < thresholds.h && ['hh', hours] || days <= 1 && ['d'] || days < thresholds.d && ['dd', days] || months <= 1 && ['M'] || months < thresholds.M && ['MM', months] || years <= 1 && ['y'] || ['yy', years];
    a[2] = withoutSuffix;
    a[3] = +posNegDuration > 0;
    a[4] = locale;
    return substituteTimeAgo.apply(null, a);
  }
  function duration_humanize__getSetRelativeTimeRounding(roundingFunction) {
    if (roundingFunction === undefined) {
      return round;
    }
    if (typeof(roundingFunction) === 'function') {
      round = roundingFunction;
      return true;
    }
    return false;
  }
  function duration_humanize__getSetRelativeTimeThreshold(threshold, limit) {
    if (thresholds[threshold] === undefined) {
      return false;
    }
    if (limit === undefined) {
      return thresholds[threshold];
    }
    thresholds[threshold] = limit;
    return true;
  }
  function humanize(withSuffix) {
    var locale = this.localeData();
    var output = duration_humanize__relativeTime(this, !withSuffix, locale);
    if (withSuffix) {
      output = locale.pastFuture(+this, output);
    }
    return locale.postformat(output);
  }
  var iso_string__abs = Math.abs;
  function iso_string__toISOString() {
    var seconds = iso_string__abs(this._milliseconds) / 1000;
    var days = iso_string__abs(this._days);
    var months = iso_string__abs(this._months);
    var minutes,
        hours,
        years;
    minutes = absFloor(seconds / 60);
    hours = absFloor(minutes / 60);
    seconds %= 60;
    minutes %= 60;
    years = absFloor(months / 12);
    months %= 12;
    var Y = years;
    var M = months;
    var D = days;
    var h = hours;
    var m = minutes;
    var s = seconds;
    var total = this.asSeconds();
    if (!total) {
      return 'P0D';
    }
    return (total < 0 ? '-' : '') + 'P' + (Y ? Y + 'Y' : '') + (M ? M + 'M' : '') + (D ? D + 'D' : '') + ((h || m || s) ? 'T' : '') + (h ? h + 'H' : '') + (m ? m + 'M' : '') + (s ? s + 'S' : '');
  }
  var duration_prototype__proto = Duration.prototype;
  duration_prototype__proto.abs = duration_abs__abs;
  duration_prototype__proto.add = duration_add_subtract__add;
  duration_prototype__proto.subtract = duration_add_subtract__subtract;
  duration_prototype__proto.as = as;
  duration_prototype__proto.asMilliseconds = asMilliseconds;
  duration_prototype__proto.asSeconds = asSeconds;
  duration_prototype__proto.asMinutes = asMinutes;
  duration_prototype__proto.asHours = asHours;
  duration_prototype__proto.asDays = asDays;
  duration_prototype__proto.asWeeks = asWeeks;
  duration_prototype__proto.asMonths = asMonths;
  duration_prototype__proto.asYears = asYears;
  duration_prototype__proto.valueOf = duration_as__valueOf;
  duration_prototype__proto._bubble = bubble;
  duration_prototype__proto.get = duration_get__get;
  duration_prototype__proto.milliseconds = milliseconds;
  duration_prototype__proto.seconds = seconds;
  duration_prototype__proto.minutes = minutes;
  duration_prototype__proto.hours = hours;
  duration_prototype__proto.days = days;
  duration_prototype__proto.weeks = weeks;
  duration_prototype__proto.months = months;
  duration_prototype__proto.years = years;
  duration_prototype__proto.humanize = humanize;
  duration_prototype__proto.toISOString = iso_string__toISOString;
  duration_prototype__proto.toString = iso_string__toISOString;
  duration_prototype__proto.toJSON = iso_string__toISOString;
  duration_prototype__proto.locale = locale;
  duration_prototype__proto.localeData = localeData;
  duration_prototype__proto.toIsoString = deprecate('toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)', iso_string__toISOString);
  duration_prototype__proto.lang = lang;
  addFormatToken('X', 0, 0, 'unix');
  addFormatToken('x', 0, 0, 'valueOf');
  addRegexToken('x', matchSigned);
  addRegexToken('X', matchTimestamp);
  addParseToken('X', function(input, array, config) {
    config._d = new Date(parseFloat(input, 10) * 1000);
  });
  addParseToken('x', function(input, array, config) {
    config._d = new Date(toInt(input));
  });
  utils_hooks__hooks.version = '2.14.1';
  setHookCallback(local__createLocal);
  utils_hooks__hooks.fn = momentPrototype;
  utils_hooks__hooks.min = min;
  utils_hooks__hooks.max = max;
  utils_hooks__hooks.now = now;
  utils_hooks__hooks.utc = create_utc__createUTC;
  utils_hooks__hooks.unix = moment__createUnix;
  utils_hooks__hooks.months = lists__listMonths;
  utils_hooks__hooks.isDate = isDate;
  utils_hooks__hooks.locale = locale_locales__getSetGlobalLocale;
  utils_hooks__hooks.invalid = valid__createInvalid;
  utils_hooks__hooks.duration = create__createDuration;
  utils_hooks__hooks.isMoment = isMoment;
  utils_hooks__hooks.weekdays = lists__listWeekdays;
  utils_hooks__hooks.parseZone = moment__createInZone;
  utils_hooks__hooks.localeData = locale_locales__getLocale;
  utils_hooks__hooks.isDuration = isDuration;
  utils_hooks__hooks.monthsShort = lists__listMonthsShort;
  utils_hooks__hooks.weekdaysMin = lists__listWeekdaysMin;
  utils_hooks__hooks.defineLocale = defineLocale;
  utils_hooks__hooks.updateLocale = updateLocale;
  utils_hooks__hooks.locales = locale_locales__listLocales;
  utils_hooks__hooks.weekdaysShort = lists__listWeekdaysShort;
  utils_hooks__hooks.normalizeUnits = normalizeUnits;
  utils_hooks__hooks.relativeTimeRounding = duration_humanize__getSetRelativeTimeRounding;
  utils_hooks__hooks.relativeTimeThreshold = duration_humanize__getSetRelativeTimeThreshold;
  utils_hooks__hooks.calendarFormat = getCalendarFormat;
  utils_hooks__hooks.prototype = momentPrototype;
  var _moment = utils_hooks__hooks;
  return _moment;
}));

})();
(function() {
var define = System.amdDefine;
!function(a, b) {
  "use strict";
  "function" == typeof define && define.amd ? define("node_modules/moment-timezone/builds/moment-timezone-with-data.min.js", ["moment"], b) : "object" == typeof module && module.exports ? module.exports = b(require("moment")) : b(a.moment);
}(this, function(a) {
  "use strict";
  function b(a) {
    return a > 96 ? a - 87 : a > 64 ? a - 29 : a - 48;
  }
  function c(a) {
    var c,
        d = 0,
        e = a.split("."),
        f = e[0],
        g = e[1] || "",
        h = 1,
        i = 0,
        j = 1;
    for (45 === a.charCodeAt(0) && (d = 1, j = -1), d; d < f.length; d++)
      c = b(f.charCodeAt(d)), i = 60 * i + c;
    for (d = 0; d < g.length; d++)
      h /= 60, c = b(g.charCodeAt(d)), i += c * h;
    return i * j;
  }
  function d(a) {
    for (var b = 0; b < a.length; b++)
      a[b] = c(a[b]);
  }
  function e(a, b) {
    for (var c = 0; c < b; c++)
      a[c] = Math.round((a[c - 1] || 0) + 6e4 * a[c]);
    a[b - 1] = 1 / 0;
  }
  function f(a, b) {
    var c,
        d = [];
    for (c = 0; c < b.length; c++)
      d[c] = a[b[c]];
    return d;
  }
  function g(a) {
    var b = a.split("|"),
        c = b[2].split(" "),
        g = b[3].split(""),
        h = b[4].split(" ");
    return d(c), d(g), d(h), e(h, g.length), {
      name: b[0],
      abbrs: f(b[1].split(" "), g),
      offsets: f(c, g),
      untils: h,
      population: 0 | b[5]
    };
  }
  function h(a) {
    a && this._set(g(a));
  }
  function i(a) {
    var b = a.toTimeString(),
        c = b.match(/\([a-z ]+\)/i);
    c && c[0] ? (c = c[0].match(/[A-Z]/g), c = c ? c.join("") : void 0) : (c = b.match(/[A-Z]{3,5}/g), c = c ? c[0] : void 0), "GMT" === c && (c = void 0), this.at = +a, this.abbr = c, this.offset = a.getTimezoneOffset();
  }
  function j(a) {
    this.zone = a, this.offsetScore = 0, this.abbrScore = 0;
  }
  function k(a, b) {
    for (var c,
        d; d = 6e4 * ((b.at - a.at) / 12e4 | 0); )
      c = new i(new Date(a.at + d)), c.offset === a.offset ? a = c : b = c;
    return a;
  }
  function l() {
    var a,
        b,
        c,
        d = (new Date).getFullYear() - 2,
        e = new i(new Date(d, 0, 1)),
        f = [e];
    for (c = 1; c < 48; c++)
      b = new i(new Date(d, c, 1)), b.offset !== e.offset && (a = k(e, b), f.push(a), f.push(new i(new Date(a.at + 6e4)))), e = b;
    for (c = 0; c < 4; c++)
      f.push(new i(new Date(d + c, 0, 1))), f.push(new i(new Date(d + c, 6, 1)));
    return f;
  }
  function m(a, b) {
    return a.offsetScore !== b.offsetScore ? a.offsetScore - b.offsetScore : a.abbrScore !== b.abbrScore ? a.abbrScore - b.abbrScore : b.zone.population - a.zone.population;
  }
  function n(a, b) {
    var c,
        e;
    for (d(b), c = 0; c < b.length; c++)
      e = b[c], I[e] = I[e] || {}, I[e][a] = !0;
  }
  function o(a) {
    var b,
        c,
        d,
        e = a.length,
        f = {},
        g = [];
    for (b = 0; b < e; b++) {
      d = I[a[b].offset] || {};
      for (c in d)
        d.hasOwnProperty(c) && (f[c] = !0);
    }
    for (b in f)
      f.hasOwnProperty(b) && g.push(H[b]);
    return g;
  }
  function p() {
    try {
      var a = Intl.DateTimeFormat().resolvedOptions().timeZone;
      if (a) {
        var b = H[r(a)];
        if (b)
          return b;
        z("Moment Timezone found " + a + " from the Intl api, but did not have that data loaded.");
      }
    } catch (c) {}
    var d,
        e,
        f,
        g = l(),
        h = g.length,
        i = o(g),
        k = [];
    for (e = 0; e < i.length; e++) {
      for (d = new j(t(i[e]), h), f = 0; f < h; f++)
        d.scoreOffsetAt(g[f]);
      k.push(d);
    }
    return k.sort(m), k.length > 0 ? k[0].zone.name : void 0;
  }
  function q(a) {
    return D && !a || (D = p()), D;
  }
  function r(a) {
    return (a || "").toLowerCase().replace(/\//g, "_");
  }
  function s(a) {
    var b,
        c,
        d,
        e;
    for ("string" == typeof a && (a = [a]), b = 0; b < a.length; b++)
      d = a[b].split("|"), c = d[0], e = r(c), F[e] = a[b], H[e] = c, d[5] && n(e, d[2].split(" "));
  }
  function t(a, b) {
    a = r(a);
    var c,
        d = F[a];
    return d instanceof h ? d : "string" == typeof d ? (d = new h(d), F[a] = d, d) : G[a] && b !== t && (c = t(G[a], t)) ? (d = F[a] = new h, d._set(c), d.name = H[a], d) : null;
  }
  function u() {
    var a,
        b = [];
    for (a in H)
      H.hasOwnProperty(a) && (F[a] || F[G[a]]) && H[a] && b.push(H[a]);
    return b.sort();
  }
  function v(a) {
    var b,
        c,
        d,
        e;
    for ("string" == typeof a && (a = [a]), b = 0; b < a.length; b++)
      c = a[b].split("|"), d = r(c[0]), e = r(c[1]), G[d] = e, H[d] = c[0], G[e] = d, H[e] = c[1];
  }
  function w(a) {
    s(a.zones), v(a.links), A.dataVersion = a.version;
  }
  function x(a) {
    return x.didShowError || (x.didShowError = !0, z("moment.tz.zoneExists('" + a + "') has been deprecated in favor of !moment.tz.zone('" + a + "')")), !!t(a);
  }
  function y(a) {
    return !(!a._a || void 0 !== a._tzm);
  }
  function z(a) {
    "undefined" != typeof console && "function" == typeof console.error && console.error(a);
  }
  function A(b) {
    var c = Array.prototype.slice.call(arguments, 0, -1),
        d = arguments[arguments.length - 1],
        e = t(d),
        f = a.utc.apply(null, c);
    return e && !a.isMoment(b) && y(f) && f.add(e.parse(f), "minutes"), f.tz(d), f;
  }
  function B(a) {
    return function() {
      return this._z ? this._z.abbr(this) : a.call(this);
    };
  }
  function C(a) {
    return function() {
      return this._z = null, a.apply(this, arguments);
    };
  }
  var D,
      E = "0.5.11",
      F = {},
      G = {},
      H = {},
      I = {},
      J = a.version.split("."),
      K = +J[0],
      L = +J[1];
  (K < 2 || 2 === K && L < 6) && z("Moment Timezone requires Moment.js >= 2.6.0. You are using Moment.js " + a.version + ". See momentjs.com"), h.prototype = {
    _set: function(a) {
      this.name = a.name, this.abbrs = a.abbrs, this.untils = a.untils, this.offsets = a.offsets, this.population = a.population;
    },
    _index: function(a) {
      var b,
          c = +a,
          d = this.untils;
      for (b = 0; b < d.length; b++)
        if (c < d[b])
          return b;
    },
    parse: function(a) {
      var b,
          c,
          d,
          e,
          f = +a,
          g = this.offsets,
          h = this.untils,
          i = h.length - 1;
      for (e = 0; e < i; e++)
        if (b = g[e], c = g[e + 1], d = g[e ? e - 1 : e], b < c && A.moveAmbiguousForward ? b = c : b > d && A.moveInvalidForward && (b = d), f < h[e] - 6e4 * b)
          return g[e];
      return g[i];
    },
    abbr: function(a) {
      return this.abbrs[this._index(a)];
    },
    offset: function(a) {
      return this.offsets[this._index(a)];
    }
  }, j.prototype.scoreOffsetAt = function(a) {
    this.offsetScore += Math.abs(this.zone.offset(a.at) - a.offset), this.zone.abbr(a.at).replace(/[^A-Z]/g, "") !== a.abbr && this.abbrScore++;
  }, A.version = E, A.dataVersion = "", A._zones = F, A._links = G, A._names = H, A.add = s, A.link = v, A.load = w, A.zone = t, A.zoneExists = x, A.guess = q, A.names = u, A.Zone = h, A.unpack = g, A.unpackBase60 = c, A.needsOffset = y, A.moveInvalidForward = !0, A.moveAmbiguousForward = !1;
  var M = a.fn;
  a.tz = A, a.defaultZone = null, a.updateOffset = function(b, c) {
    var d,
        e = a.defaultZone;
    void 0 === b._z && (e && y(b) && !b._isUTC && (b._d = a.utc(b._a)._d, b.utc().add(e.parse(b), "minutes")), b._z = e), b._z && (d = b._z.offset(b), Math.abs(d) < 16 && (d /= 60), void 0 !== b.utcOffset ? b.utcOffset(-d, c) : b.zone(d, c));
  }, M.tz = function(b) {
    return b ? (this._z = t(b), this._z ? a.updateOffset(this) : z("Moment Timezone has no data for " + b + ". See http://momentjs.com/timezone/docs/#/data-loading/."), this) : this._z ? this._z.name : void 0;
  }, M.zoneName = B(M.zoneName), M.zoneAbbr = B(M.zoneAbbr), M.utc = C(M.utc), a.tz.setDefault = function(b) {
    return (K < 2 || 2 === K && L < 9) && z("Moment Timezone setDefault() requires Moment.js >= 2.9.0. You are using Moment.js " + a.version + "."), a.defaultZone = b ? t(b) : null, a;
  };
  var N = a.momentProperties;
  return "[object Array]" === Object.prototype.toString.call(N) ? (N.push("_z"), N.push("_a")) : N && (N._z = null), w({
    version: "2016j",
    zones: ["Africa/Abidjan|LMT GMT|g.8 0|01|-2ldXH.Q|48e5", "Africa/Accra|LMT GMT GHST|.Q 0 -k|012121212121212121212121212121212121212121212121|-26BbX.8 6tzX.8 MnE 1BAk MnE 1BAk MnE 1BAk MnE 1C0k MnE 1BAk MnE 1BAk MnE 1BAk MnE 1C0k MnE 1BAk MnE 1BAk MnE 1BAk MnE 1C0k MnE 1BAk MnE 1BAk MnE 1BAk MnE 1C0k MnE 1BAk MnE 1BAk MnE 1BAk MnE 1C0k MnE 1BAk MnE 1BAk MnE|41e5", "Africa/Nairobi|LMT EAT BEAT BEAUT|-2r.g -30 -2u -2J|01231|-1F3Cr.g 3Dzr.g okMu MFXJ|47e5", "Africa/Algiers|PMT WET WEST CET CEST|-9.l 0 -10 -10 -20|0121212121212121343431312123431213|-2nco9.l cNb9.l HA0 19A0 1iM0 11c0 1oo0 Wo0 1rc0 QM0 1EM0 UM0 DA0 Imo0 rd0 De0 9Xz0 1fb0 1ap0 16K0 2yo0 mEp0 hwL0 jxA0 11A0 dDd0 17b0 11B0 1cN0 2Dy0 1cN0 1fB0 1cL0|26e5", "Africa/Lagos|LMT WAT|-d.A -10|01|-22y0d.A|17e6", "Africa/Bissau|LMT WAT GMT|12.k 10 0|012|-2ldWV.E 2xonV.E|39e4", "Africa/Maputo|LMT CAT|-2a.k -20|01|-2GJea.k|26e5", "Africa/Cairo|EET EEST|-20 -30|0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-1bIO0 vb0 1ip0 11z0 1iN0 1nz0 12p0 1pz0 10N0 1pz0 16p0 1jz0 s3d0 Vz0 1oN0 11b0 1oO0 10N0 1pz0 10N0 1pb0 10N0 1pb0 10N0 1pb0 10N0 1pz0 10N0 1pb0 10N0 1pb0 11d0 1oL0 11d0 1pb0 11d0 1oL0 11d0 1oL0 11d0 1oL0 11d0 1pb0 11d0 1oL0 11d0 1oL0 11d0 1oL0 11d0 1pb0 11d0 1oL0 11d0 1oL0 11d0 1oL0 11d0 1pb0 11d0 1oL0 11d0 1WL0 rd0 1Rz0 wp0 1pb0 11d0 1oL0 11d0 1oL0 11d0 1oL0 11d0 1pb0 11d0 1qL0 Xd0 1oL0 11d0 1oL0 11d0 1pb0 11d0 1oL0 11d0 1oL0 11d0 1ny0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 11z0 1o10 11z0 1o10 WL0 1qN0 Rb0 1wp0 On0 1zd0 Lz0 1EN0 Fb0 c10 8n0 8Nd0 gL0 e10 mn0|15e6", "Africa/Casablanca|LMT WET WEST CET|u.k 0 -10 -10|0121212121212121213121212121212121212121212121212121212121212121212121212121212121212121212121212121|-2gMnt.E 130Lt.E rb0 Dd0 dVb0 b6p0 TX0 EoB0 LL0 gnd0 rz0 43d0 AL0 1Nd0 XX0 1Cp0 pz0 dEp0 4mn0 SyN0 AL0 1Nd0 wn0 1FB0 Db0 1zd0 Lz0 1Nf0 wM0 co0 go0 1o00 s00 dA0 vc0 11A0 A00 e00 y00 11A0 uM0 e00 Dc0 11A0 s00 e00 IM0 WM0 mo0 gM0 LA0 WM0 jA0 e00 Rc0 11A0 e00 e00 U00 11A0 8o0 e00 11A0 11A0 5A0 e00 17c0 1fA0 1a00 1a00 1fA0 17c0 1io0 14o0 1lc0 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1lc0 14o0 1fA0|32e5", "Africa/Ceuta|WET WEST CET CEST|0 -10 -10 -20|010101010101010101010232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232|-25KN0 11z0 drd0 18o0 3I00 17c0 1fA0 1a00 1io0 1a00 1y7p0 LL0 gnd0 rz0 43d0 AL0 1Nd0 XX0 1Cp0 pz0 dEp0 4VB0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|85e3", "Africa/El_Aaiun|LMT WAT WET WEST|Q.M 10 0 -10|01232323232323232323232323232323232323232323232323232323232323232323232323232323232323232|-1rDz7.c 1GVA7.c 6L0 AL0 1Nd0 XX0 1Cp0 pz0 1cBB0 AL0 1Nd0 wn0 1FB0 Db0 1zd0 Lz0 1Nf0 wM0 co0 go0 1o00 s00 dA0 vc0 11A0 A00 e00 y00 11A0 uM0 e00 Dc0 11A0 s00 e00 IM0 WM0 mo0 gM0 LA0 WM0 jA0 e00 Rc0 11A0 e00 e00 U00 11A0 8o0 e00 11A0 11A0 5A0 e00 17c0 1fA0 1a00 1a00 1fA0 17c0 1io0 14o0 1lc0 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1lc0 14o0 1fA0|20e4", "Africa/Johannesburg|SAST SAST SAST|-1u -20 -30|012121|-2GJdu 1Ajdu 1cL0 1cN0 1cL0|84e5", "Africa/Khartoum|LMT CAT CAST EAT|-2a.8 -20 -30 -30|01212121212121212121212121212121213|-1yW2a.8 1zK0a.8 16L0 1iN0 17b0 1jd0 17b0 1ip0 17z0 1i10 17X0 1hB0 18n0 1hd0 19b0 1gp0 19z0 1iN0 17b0 1ip0 17z0 1i10 18n0 1hd0 18L0 1gN0 19b0 1gp0 19z0 1iN0 17z0 1i10 17X0 yGd0|51e5", "Africa/Monrovia|MMT LRT GMT|H.8 I.u 0|012|-23Lzg.Q 29s01.m|11e5", "Africa/Ndjamena|LMT WAT WAST|-10.c -10 -20|0121|-2le10.c 2J3c0.c Wn0|13e5", "Africa/Tripoli|LMT CET CEST EET|-Q.I -10 -20 -20|012121213121212121212121213123123|-21JcQ.I 1hnBQ.I vx0 4iP0 xx0 4eN0 Bb0 7ip0 U0n0 A10 1db0 1cN0 1db0 1dd0 1db0 1eN0 1bb0 1e10 1cL0 1c10 1db0 1dd0 1db0 1cN0 1db0 1q10 fAn0 1ep0 1db0 AKq0 TA0 1o00|11e5", "Africa/Tunis|PMT CET CEST|-9.l -10 -20|0121212121212121212121212121212121|-2nco9.l 18pa9.l 1qM0 DA0 3Tc0 11B0 1ze0 WM0 7z0 3d0 14L0 1cN0 1f90 1ar0 16J0 1gXB0 WM0 1rA0 11c0 nwo0 Ko0 1cM0 1cM0 1rA0 10M0 zuM0 10N0 1aN0 1qM0 WM0 1qM0 11A0 1o00|20e5", "Africa/Windhoek|SWAT SAST SAST CAT WAT WAST|-1u -20 -30 -20 -10 -20|012134545454545454545454545454545454545454545454545454545454545454545454545454545454545454545|-2GJdu 1Ajdu 1cL0 1SqL0 9NA0 11D0 1nX0 11B0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1qL0 11B0 1nX0 11B0|32e4", "America/Adak|NST NWT NPT BST BDT AHST HST HDT|b0 a0 a0 b0 a0 a0 a0 90|012034343434343434343434343434343456767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676|-17SX0 8wW0 iB0 Qlb0 52O0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 cm0 10q0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|326", "America/Anchorage|CAT CAWT CAPT AHST AHDT YST AKST AKDT|a0 90 90 a0 90 90 90 80|012034343434343434343434343434343456767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676|-17T00 8wX0 iA0 Qlb0 52O0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 cm0 10q0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|30e4", "America/Port_of_Spain|LMT AST|46.4 40|01|-2kNvR.U|43e3", "America/Araguaina|LMT BRT BRST|3c.M 30 20|0121212121212121212121212121212121212121212121212121|-2glwL.c HdKL.c 1cc0 1e10 1bX0 Ezd0 So0 1vA0 Mn0 1BB0 ML0 1BB0 zX0 qe10 xb0 2ep0 nz0 1C10 zX0 1C10 LX0 1C10 Mn0 H210 Rb0 1tB0 IL0 1Fd0 FX0 1EN0 FX0 1HB0 Lz0 dMN0 Lz0 1zd0 Rb0 1wN0 Wn0 1tB0 Rb0 1tB0 WL0 1tB0 Rb0 1zd0 On0 1HB0 FX0 ny10 Lz0|14e4", "America/Argentina/Buenos_Aires|CMT ART ARST ART ARST|4g.M 40 30 30 20|0121212121212121212121212121212121212121213434343434343234343|-20UHH.c pKnH.c Mn0 1iN0 Tb0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 MN0 2jz0 MN0 4lX0 u10 5Lb0 1pB0 Fnz0 u10 uL0 1vd0 SL0 1vd0 SL0 1vd0 17z0 1cN0 1fz0 1cN0 1cL0 1cN0 asn0 Db0 zvd0 Bz0 1tB0 TX0 1wp0 Rb0 1wp0 Rb0 1wp0 TX0 g0p0 10M0 j3c0 uL0 1qN0 WL0", "America/Argentina/Catamarca|CMT ART ARST ART ARST WART|4g.M 40 30 30 20 40|0121212121212121212121212121212121212121213434343454343235343|-20UHH.c pKnH.c Mn0 1iN0 Tb0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 MN0 2jz0 MN0 4lX0 u10 5Lb0 1pB0 Fnz0 u10 uL0 1vd0 SL0 1vd0 SL0 1vd0 17z0 1cN0 1fz0 1cN0 1cL0 1cN0 asn0 Db0 zvd0 Bz0 1tB0 TX0 1wp0 Rb0 1wq0 Ra0 1wp0 TX0 g0p0 10M0 ako0 7B0 8zb0 uL0", "America/Argentina/Cordoba|CMT ART ARST ART ARST WART|4g.M 40 30 30 20 40|0121212121212121212121212121212121212121213434343454343234343|-20UHH.c pKnH.c Mn0 1iN0 Tb0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 MN0 2jz0 MN0 4lX0 u10 5Lb0 1pB0 Fnz0 u10 uL0 1vd0 SL0 1vd0 SL0 1vd0 17z0 1cN0 1fz0 1cN0 1cL0 1cN0 asn0 Db0 zvd0 Bz0 1tB0 TX0 1wp0 Rb0 1wq0 Ra0 1wp0 TX0 g0p0 10M0 j3c0 uL0 1qN0 WL0", "America/Argentina/Jujuy|CMT ART ARST ART ARST WART WARST|4g.M 40 30 30 20 40 30|01212121212121212121212121212121212121212134343456543432343|-20UHH.c pKnH.c Mn0 1iN0 Tb0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 MN0 2jz0 MN0 4lX0 u10 5Lb0 1pB0 Fnz0 u10 uL0 1vd0 SL0 1vd0 SL0 1vd0 17z0 1cN0 1fz0 1cN0 1cL0 1cN0 asn0 Db0 zvd0 Bz0 1tB0 TX0 1ze0 TX0 1ld0 WK0 1wp0 TX0 g0p0 10M0 j3c0 uL0", "America/Argentina/La_Rioja|CMT ART ARST ART ARST WART|4g.M 40 30 30 20 40|01212121212121212121212121212121212121212134343434534343235343|-20UHH.c pKnH.c Mn0 1iN0 Tb0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 MN0 2jz0 MN0 4lX0 u10 5Lb0 1pB0 Fnz0 u10 uL0 1vd0 SL0 1vd0 SL0 1vd0 17z0 1cN0 1fz0 1cN0 1cL0 1cN0 asn0 Db0 zvd0 Bz0 1tB0 TX0 1wp0 Qn0 qO0 16n0 Rb0 1wp0 TX0 g0p0 10M0 ako0 7B0 8zb0 uL0", "America/Argentina/Mendoza|CMT ART ARST ART ARST WART WARST|4g.M 40 30 30 20 40 30|0121212121212121212121212121212121212121213434345656543235343|-20UHH.c pKnH.c Mn0 1iN0 Tb0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 MN0 2jz0 MN0 4lX0 u10 5Lb0 1pB0 Fnz0 u10 uL0 1vd0 SL0 1vd0 SL0 1vd0 17z0 1cN0 1fz0 1cN0 1cL0 1cN0 asn0 Db0 zvd0 Bz0 1tB0 TX0 1u20 SL0 1vd0 Tb0 1wp0 TW0 g0p0 10M0 agM0 Op0 7TX0 uL0", "America/Argentina/Rio_Gallegos|CMT ART ARST ART ARST WART|4g.M 40 30 30 20 40|0121212121212121212121212121212121212121213434343434343235343|-20UHH.c pKnH.c Mn0 1iN0 Tb0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 MN0 2jz0 MN0 4lX0 u10 5Lb0 1pB0 Fnz0 u10 uL0 1vd0 SL0 1vd0 SL0 1vd0 17z0 1cN0 1fz0 1cN0 1cL0 1cN0 asn0 Db0 zvd0 Bz0 1tB0 TX0 1wp0 Rb0 1wp0 Rb0 1wp0 TX0 g0p0 10M0 ako0 7B0 8zb0 uL0", "America/Argentina/Salta|CMT ART ARST ART ARST WART|4g.M 40 30 30 20 40|01212121212121212121212121212121212121212134343434543432343|-20UHH.c pKnH.c Mn0 1iN0 Tb0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 MN0 2jz0 MN0 4lX0 u10 5Lb0 1pB0 Fnz0 u10 uL0 1vd0 SL0 1vd0 SL0 1vd0 17z0 1cN0 1fz0 1cN0 1cL0 1cN0 asn0 Db0 zvd0 Bz0 1tB0 TX0 1wp0 Rb0 1wq0 Ra0 1wp0 TX0 g0p0 10M0 j3c0 uL0", "America/Argentina/San_Juan|CMT ART ARST ART ARST WART|4g.M 40 30 30 20 40|01212121212121212121212121212121212121212134343434534343235343|-20UHH.c pKnH.c Mn0 1iN0 Tb0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 MN0 2jz0 MN0 4lX0 u10 5Lb0 1pB0 Fnz0 u10 uL0 1vd0 SL0 1vd0 SL0 1vd0 17z0 1cN0 1fz0 1cN0 1cL0 1cN0 asn0 Db0 zvd0 Bz0 1tB0 TX0 1wp0 Qn0 qO0 16n0 Rb0 1wp0 TX0 g0p0 10M0 ak00 m10 8lb0 uL0", "America/Argentina/San_Luis|CMT ART ARST ART ARST WART WARST|4g.M 40 30 30 20 40 30|01212121212121212121212121212121212121212134343456536353465653|-20UHH.c pKnH.c Mn0 1iN0 Tb0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 MN0 2jz0 MN0 4lX0 u10 5Lb0 1pB0 Fnz0 u10 uL0 1vd0 SL0 1vd0 SL0 1vd0 17z0 1cN0 1fz0 1cN0 1cL0 1cN0 asn0 Db0 zvd0 Bz0 1tB0 XX0 1q20 SL0 AN0 kin0 10M0 ak00 m10 8lb0 8L0 jd0 1qN0 WL0 1qN0", "America/Argentina/Tucuman|CMT ART ARST ART ARST WART|4g.M 40 30 30 20 40|012121212121212121212121212121212121212121343434345434323534343|-20UHH.c pKnH.c Mn0 1iN0 Tb0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 MN0 2jz0 MN0 4lX0 u10 5Lb0 1pB0 Fnz0 u10 uL0 1vd0 SL0 1vd0 SL0 1vd0 17z0 1cN0 1fz0 1cN0 1cL0 1cN0 asn0 Db0 zvd0 Bz0 1tB0 TX0 1wp0 Rb0 1wq0 Ra0 1wp0 TX0 g0p0 10M0 ako0 4N0 8BX0 uL0 1qN0 WL0", "America/Argentina/Ushuaia|CMT ART ARST ART ARST WART|4g.M 40 30 30 20 40|0121212121212121212121212121212121212121213434343434343235343|-20UHH.c pKnH.c Mn0 1iN0 Tb0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 1C10 LX0 1C10 LX0 1C10 LX0 1C10 Mn0 MN0 2jz0 MN0 4lX0 u10 5Lb0 1pB0 Fnz0 u10 uL0 1vd0 SL0 1vd0 SL0 1vd0 17z0 1cN0 1fz0 1cN0 1cL0 1cN0 asn0 Db0 zvd0 Bz0 1tB0 TX0 1wp0 Rb0 1wp0 Rb0 1wp0 TX0 g0p0 10M0 ajA0 8p0 8zb0 uL0", "America/Curacao|LMT ANT AST|4z.L 4u 40|012|-2kV7o.d 28KLS.d|15e4", "America/Asuncion|AMT PYT PYT PYST|3O.E 40 30 30|012131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313|-1x589.k 1DKM9.k 3CL0 3Dd0 10L0 1pB0 10n0 1pB0 10n0 1pB0 1cL0 1dd0 1db0 1dd0 1cL0 1dd0 1cL0 1dd0 1cL0 1dd0 1db0 1dd0 1cL0 1dd0 1cL0 1dd0 1cL0 1dd0 1db0 1dd0 1cL0 1lB0 14n0 1dd0 1cL0 1fd0 WL0 1rd0 1aL0 1dB0 Xz0 1qp0 Xb0 1qN0 10L0 1rB0 TX0 1tB0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1qN0 1cL0 WN0 1qL0 11B0 1nX0 1ip0 WL0 1qN0 WL0 1qN0 WL0 1tB0 TX0 1tB0 TX0 1tB0 19X0 1a10 1fz0 1a10 1fz0 1cN0 17b0 1ip0 17b0 1ip0 17b0 1ip0 19X0 1fB0 19X0 1fB0 19X0 1ip0 17b0 1ip0 17b0 1ip0 19X0 1fB0 19X0 1fB0 19X0 1fB0 19X0 1ip0 17b0 1ip0 17b0 1ip0 19X0 1fB0 19X0 1fB0 19X0 1ip0 17b0 1ip0 17b0 1ip0 19X0 1fB0 19X0 1fB0 19X0 1fB0 19X0 1ip0 17b0 1ip0 17b0 1ip0|28e5", "America/Atikokan|CST CDT CWT CPT EST|60 50 50 50 50|0101234|-25TQ0 1in0 Rnb0 3je0 8x30 iw0|28e2", "America/Bahia|LMT BRT BRST|2y.4 30 20|01212121212121212121212121212121212121212121212121212121212121|-2glxp.U HdLp.U 1cc0 1e10 1bX0 Ezd0 So0 1vA0 Mn0 1BB0 ML0 1BB0 zX0 qe10 xb0 2ep0 nz0 1C10 zX0 1C10 LX0 1C10 Mn0 H210 Rb0 1tB0 IL0 1Fd0 FX0 1EN0 FX0 1HB0 Lz0 1EN0 Lz0 1C10 IL0 1HB0 Db0 1HB0 On0 1zd0 On0 1zd0 Lz0 1zd0 Rb0 1wN0 Wn0 1tB0 Rb0 1tB0 WL0 1tB0 Rb0 1zd0 On0 1HB0 FX0 l5B0 Rb0|27e5", "America/Bahia_Banderas|LMT MST CST PST MDT CDT|71 70 60 80 60 50|0121212131414141414141414141414141414152525252525252525252525252525252525252525252525252525252|-1UQF0 deL0 8lc0 17c0 10M0 1dd0 otX0 gmN0 P2N0 13Vd0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 1fB0 WL0 1fB0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nW0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0|84e3", "America/Barbados|LMT BMT AST ADT|3W.t 3W.t 40 30|01232323232|-1Q0I1.v jsM0 1ODC1.v IL0 1ip0 17b0 1ip0 17b0 1ld0 13b0|28e4", "America/Belem|LMT BRT BRST|3d.U 30 20|012121212121212121212121212121|-2glwK.4 HdKK.4 1cc0 1e10 1bX0 Ezd0 So0 1vA0 Mn0 1BB0 ML0 1BB0 zX0 qe10 xb0 2ep0 nz0 1C10 zX0 1C10 LX0 1C10 Mn0 H210 Rb0 1tB0 IL0 1Fd0 FX0|20e5", "America/Belize|LMT CST CHDT CDT|5Q.M 60 5u 50|01212121212121212121212121212121212121212121212121213131|-2kBu7.c fPA7.c Onu 1zcu Rbu 1wou Rbu 1wou Rbu 1zcu Onu 1zcu Onu 1zcu Rbu 1wou Rbu 1wou Rbu 1wou Rbu 1zcu Onu 1zcu Onu 1zcu Rbu 1wou Rbu 1wou Rbu 1zcu Onu 1zcu Onu 1zcu Onu 1zcu Rbu 1wou Rbu 1wou Rbu 1zcu Onu 1zcu Onu 1zcu Rbu 1wou Rbu 1f0Mu qn0 lxB0 mn0|57e3", "America/Blanc-Sablon|AST ADT AWT APT|40 30 30 30|010230|-25TS0 1in0 UGp0 8x50 iu0|11e2", "America/Boa_Vista|LMT AMT AMST|42.E 40 30|0121212121212121212121212121212121|-2glvV.k HdKV.k 1cc0 1e10 1bX0 Ezd0 So0 1vA0 Mn0 1BB0 ML0 1BB0 zX0 qe10 xb0 2ep0 nz0 1C10 zX0 1C10 LX0 1C10 Mn0 H210 Rb0 1tB0 IL0 1Fd0 FX0 smp0 WL0 1tB0 2L0|62e2", "America/Bogota|BMT COT COST|4U.g 50 40|0121|-2eb73.I 38yo3.I 2en0|90e5", "America/Boise|PST PDT MST MWT MPT MDT|80 70 70 60 60 60|0101023425252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252|-261q0 1nX0 11B0 1nX0 8C10 JCL0 8x20 ix0 QwN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 Dd0 1Kn0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|21e4", "America/Cambridge_Bay|-00 MST MWT MPT MDDT MDT CST CDT EST|0 70 60 60 50 60 60 50 50|0123141515151515151515151515151515151515151515678651515151515151515151515151515151515151515151515151515151515151515151515151|-21Jc0 RO90 8x20 ix0 LCL0 1fA0 zgO0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11A0 1nX0 2K0 WQ0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|15e2", "America/Campo_Grande|LMT AMT AMST|3C.s 40 30|012121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212|-2glwl.w HdLl.w 1cc0 1e10 1bX0 Ezd0 So0 1vA0 Mn0 1BB0 ML0 1BB0 zX0 qe10 xb0 2ep0 nz0 1C10 zX0 1C10 LX0 1C10 Mn0 H210 Rb0 1tB0 IL0 1Fd0 FX0 1EN0 FX0 1HB0 Lz0 1EN0 Lz0 1C10 IL0 1HB0 Db0 1HB0 On0 1zd0 On0 1zd0 Lz0 1zd0 Rb0 1wN0 Wn0 1tB0 Rb0 1tB0 WL0 1tB0 Rb0 1zd0 On0 1HB0 FX0 1C10 Lz0 1Ip0 HX0 1zd0 On0 1HB0 IL0 1wp0 On0 1C10 Lz0 1C10 On0 1zd0 On0 1zd0 Rb0 1zd0 Lz0 1C10 Lz0 1C10 On0 1zd0 On0 1zd0 On0 1zd0 On0 1C10 Lz0 1C10 Lz0 1C10 On0 1zd0 On0 1zd0 Rb0 1wp0 On0 1C10 Lz0 1C10 On0 1zd0 On0 1zd0 On0 1zd0 On0 1C10 Lz0 1C10 Lz0 1C10 Lz0 1C10 On0 1zd0 Rb0 1wp0 On0 1C10 Lz0 1C10 On0 1zd0|77e4", "America/Cancun|LMT CST EST EDT CDT|5L.4 60 50 40 50|0123232341414141414141414141414141414141412|-1UQG0 2q2o0 yLB0 1lb0 14p0 1lb0 14p0 Lz0 xB0 14p0 1nX0 11B0 1nX0 1fB0 WL0 1fB0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 Dd0|63e4", "America/Caracas|CMT VET VET|4r.E 4u 40|01212|-2kV7w.k 28KM2.k 1IwOu kqo0|29e5", "America/Cayenne|LMT GFT GFT|3t.k 40 30|012|-2mrwu.E 2gWou.E|58e3", "America/Panama|CMT EST|5j.A 50|01|-2uduE.o|15e5", "America/Chicago|CST CDT EST CWT CPT|60 50 50 50 50|01010101010101010101010101010101010102010101010103401010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-261s0 1nX0 11B0 1nX0 1wp0 TX0 WN0 1qL0 1cN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 11z0 1o10 11z0 11B0 1Hz0 14p0 11z0 1o10 11z0 1qN0 WL0 1qN0 11z0 1o10 11z0 RB0 8x30 iw0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|92e5", "America/Chihuahua|LMT MST CST CDT MDT|74.k 70 60 50 60|0121212323241414141414141414141414141414141414141414141414141414141414141414141414141414141|-1UQF0 deL0 8lc0 17c0 10M0 1dd0 2zQN0 1lb0 14p0 1lb0 14q0 1lb0 14p0 1nX0 11B0 1nX0 1fB0 WL0 1fB0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0|81e4", "America/Costa_Rica|SJMT CST CDT|5A.d 60 50|0121212121|-1Xd6n.L 2lu0n.L Db0 1Kp0 Db0 pRB0 15b0 1kp0 mL0|12e5", "America/Creston|MST PST|70 80|010|-29DR0 43B0|53e2", "America/Cuiaba|LMT AMT AMST|3I.k 40 30|0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212|-2glwf.E HdLf.E 1cc0 1e10 1bX0 Ezd0 So0 1vA0 Mn0 1BB0 ML0 1BB0 zX0 qe10 xb0 2ep0 nz0 1C10 zX0 1C10 LX0 1C10 Mn0 H210 Rb0 1tB0 IL0 1Fd0 FX0 1EN0 FX0 1HB0 Lz0 1EN0 Lz0 1C10 IL0 1HB0 Db0 1HB0 On0 1zd0 On0 1zd0 Lz0 1zd0 Rb0 1wN0 Wn0 1tB0 Rb0 1tB0 WL0 1tB0 Rb0 1zd0 On0 1HB0 FX0 4a10 HX0 1zd0 On0 1HB0 IL0 1wp0 On0 1C10 Lz0 1C10 On0 1zd0 On0 1zd0 Rb0 1zd0 Lz0 1C10 Lz0 1C10 On0 1zd0 On0 1zd0 On0 1zd0 On0 1C10 Lz0 1C10 Lz0 1C10 On0 1zd0 On0 1zd0 Rb0 1wp0 On0 1C10 Lz0 1C10 On0 1zd0 On0 1zd0 On0 1zd0 On0 1C10 Lz0 1C10 Lz0 1C10 Lz0 1C10 On0 1zd0 Rb0 1wp0 On0 1C10 Lz0 1C10 On0 1zd0|54e4", "America/Danmarkshavn|LMT WGT WGST GMT|1e.E 30 20 0|01212121212121212121212121212121213|-2a5WJ.k 2z5fJ.k 19U0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 DC0|8", "America/Dawson|YST YDT YWT YPT YDDT PST PDT|90 80 80 80 70 80 70|0101023040565656565656565656565656565656565656565656565656565656565656565656565656565656565656565656565656565656565656565656565|-25TN0 1in0 1o10 13V0 Ser0 8x00 iz0 LCL0 1fA0 jrA0 fNd0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|13e2", "America/Dawson_Creek|PST PDT PWT PPT MST|80 70 70 70 70|0102301010101010101010101010101010101010101010101010101014|-25TO0 1in0 UGp0 8x10 iy0 3NB0 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 ML0|12e3", "America/Denver|MST MDT MWT MPT|70 60 60 60|01010101023010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-261r0 1nX0 11B0 1nX0 11B0 1qL0 WN0 mn0 Ord0 8x20 ix0 LCN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|26e5", "America/Detroit|LMT CST EST EWT EPT EDT|5w.b 60 50 40 40 40|01234252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252|-2Cgir.N peqr.N 156L0 8x40 iv0 6fd0 11z0 Jy10 SL0 dnB0 1cL0 s10 1Vz0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|37e5", "America/Edmonton|LMT MST MDT MWT MPT|7x.Q 70 60 60 60|01212121212121341212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-2yd4q.8 shdq.8 1in0 17d0 hz0 2dB0 1fz0 1a10 11z0 1qN0 WL0 1qN0 11z0 IGN0 8x20 ix0 3NB0 11z0 LFB0 1cL0 3Cp0 1cL0 66N0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|10e5", "America/Eirunepe|LMT ACT ACST AMT|4D.s 50 40 40|0121212121212121212121212121212131|-2glvk.w HdLk.w 1cc0 1e10 1bX0 Ezd0 So0 1vA0 Mn0 1BB0 ML0 1BB0 zX0 qe10 xb0 2ep0 nz0 1C10 zX0 1C10 LX0 1C10 Mn0 H210 Rb0 1tB0 IL0 1Fd0 FX0 dPB0 On0 yTd0 d5X0|31e3", "America/El_Salvador|LMT CST CDT|5U.M 60 50|012121|-1XiG3.c 2Fvc3.c WL0 1qN0 WL0|11e5", "America/Tijuana|LMT MST PST PDT PWT PPT|7M.4 70 80 70 70 70|012123245232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232|-1UQE0 4PX0 8mM0 8lc0 SN0 1cL0 pHB0 83r0 zI0 5O10 1Rz0 cOO0 11A0 1o00 11A0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1o00 11A0 BUp0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 U10 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|20e5", "America/Fort_Nelson|PST PDT PWT PPT MST|80 70 70 70 70|01023010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010104|-25TO0 1in0 UGp0 8x10 iy0 3NB0 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0|39e2", "America/Fort_Wayne|CST CDT CWT CPT EST EDT|60 50 50 50 50 40|010101023010101010101010101040454545454545454545454545454545454545454545454545454545454545454545454|-261s0 1nX0 11B0 1nX0 QI10 Db0 RB0 8x30 iw0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 5Tz0 1o10 qLb0 1cL0 1cN0 1cL0 1qhd0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0", "America/Fortaleza|LMT BRT BRST|2y 30 20|0121212121212121212121212121212121212121|-2glxq HdLq 1cc0 1e10 1bX0 Ezd0 So0 1vA0 Mn0 1BB0 ML0 1BB0 zX0 qe10 xb0 2ep0 nz0 1C10 zX0 1C10 LX0 1C10 Mn0 H210 Rb0 1tB0 IL0 1Fd0 FX0 1EN0 FX0 1HB0 Lz0 nsp0 WL0 1tB0 5z0 2mN0 On0|34e5", "America/Glace_Bay|LMT AST ADT AWT APT|3X.M 40 30 30 30|012134121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-2IsI0.c CwO0.c 1in0 UGp0 8x50 iu0 iq10 11z0 Jg10 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|19e3", "America/Godthab|LMT WGT WGST|3q.U 30 20|0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-2a5Ux.4 2z5dx.4 19U0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|17e3", "America/Goose_Bay|NST NDT NST NDT NWT NPT AST ADT ADDT|3u.Q 2u.Q 3u 2u 2u 2u 40 30 20|010232323232323245232323232323232323232323232323232323232326767676767676767676767676767676767676767676768676767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676|-25TSt.8 1in0 DXb0 2HbX.8 WL0 1qN0 WL0 1qN0 WL0 1tB0 TX0 1tB0 WL0 1qN0 WL0 1qN0 7UHu itu 1tB0 WL0 1qN0 WL0 1qN0 WL0 1qN0 WL0 1tB0 WL0 1ld0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 S10 g0u 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14n1 1lb0 14p0 1nW0 11C0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zcX Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|76e2", "America/Grand_Turk|KMT EST EDT AST|57.b 50 40 40|0121212121212121212121212121212121212121212121212121212121212121212121212123|-2l1uQ.N 2HHBQ.N 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|37e2", "America/Guatemala|LMT CST CDT|62.4 60 50|0121212121|-24KhV.U 2efXV.U An0 mtd0 Nz0 ifB0 17b0 zDB0 11z0|13e5", "America/Guayaquil|QMT ECT|5e 50|01|-1yVSK|27e5", "America/Guyana|LMT GBGT GYT GYT GYT|3Q.E 3J 3J 30 40|01234|-2dvU7.k 24JzQ.k mlc0 Bxbf|80e4", "America/Halifax|LMT AST ADT AWT APT|4e.o 40 30 30 30|0121212121212121212121212121212121212121212121212134121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-2IsHJ.A xzzJ.A 1db0 3I30 1in0 3HX0 IL0 1E10 ML0 1yN0 Pb0 1Bd0 Mn0 1Bd0 Rz0 1w10 Xb0 1w10 LX0 1w10 Xb0 1w10 Lz0 1C10 Jz0 1E10 OL0 1yN0 Un0 1qp0 Xb0 1qp0 11X0 1w10 Lz0 1HB0 LX0 1C10 FX0 1w10 Xb0 1qp0 Xb0 1BB0 LX0 1td0 Xb0 1qp0 Xb0 Rf0 8x50 iu0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 3Qp0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 3Qp0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 6i10 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|39e4", "America/Havana|HMT CST CDT|5t.A 50 40|012121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-1Meuu.o 72zu.o ML0 sld0 An0 1Nd0 Db0 1Nd0 An0 6Ep0 An0 1Nd0 An0 JDd0 Mn0 1Ap0 On0 1fd0 11X0 1qN0 WL0 1wp0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 14n0 1ld0 14L0 1kN0 15b0 1kp0 1cL0 1cN0 1fz0 1a10 1fz0 1fB0 11z0 14p0 1nX0 11B0 1nX0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 14n0 1ld0 14n0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 1a10 1in0 1a10 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1cM0 1cM0 1fA0 17c0 1o00 11A0 1qM0 11A0 1o00 11A0 1o00 14o0 1lc0 14o0 1lc0 11A0 6i00 Rc0 1wo0 U00 1tA0 Rc0 1wo0 U00 1wo0 U00 1zc0 U00 1qM0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0|21e5", "America/Hermosillo|LMT MST CST PST MDT|7n.Q 70 60 80 60|0121212131414141|-1UQF0 deL0 8lc0 17c0 10M0 1dd0 otX0 gmN0 P2N0 13Vd0 1lb0 14p0 1lb0 14p0 1lb0|64e4", "America/Indiana/Knox|CST CDT CWT CPT EST|60 50 50 50 50|0101023010101010101010101010101010101040101010101010101010101010101010101010101010101010141010101010101010101010101010101010101010101010101010101010101010|-261s0 1nX0 11B0 1nX0 SgN0 8x30 iw0 3NB0 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 1fz0 1cN0 1cL0 1cN0 11z0 1o10 11z0 1o10 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 3Cn0 8wp0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 z8o0 1o00 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0", "America/Indiana/Marengo|CST CDT CWT CPT EST EDT|60 50 50 50 50 40|0101023010101010101010104545454545414545454545454545454545454545454545454545454545454545454545454545454|-261s0 1nX0 11B0 1nX0 SgN0 8x30 iw0 dyN0 11z0 6fd0 11z0 1o10 11z0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 jrz0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1VA0 LA0 1BX0 1e6p0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0", "America/Indiana/Petersburg|CST CDT CWT CPT EST EDT|60 50 50 50 50 40|01010230101010101010101010104010101010101010101010141014545454545454545454545454545454545454545454545454545454545454|-261s0 1nX0 11B0 1nX0 SgN0 8x30 iw0 njX0 WN0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 3Fb0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 19co0 1o00 Rd0 1zb0 Oo0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0", "America/Indiana/Tell_City|CST CDT CWT CPT EST EDT|60 50 50 50 50 40|01010230101010101010101010101010454541010101010101010101010101010101010101010101010101010101010101010|-261s0 1nX0 11B0 1nX0 SgN0 8x30 iw0 1o10 11z0 g0p0 11z0 1o10 11z0 1qL0 WN0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 1fz0 1cN0 WL0 1qN0 1cL0 1cN0 1cL0 1cN0 caL0 1cL0 1cN0 1cL0 1qhd0 1o00 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0", "America/Indiana/Vevay|CST CDT CWT CPT EST EDT|60 50 50 50 50 40|010102304545454545454545454545454545454545454545454545454545454545454545454545454|-261s0 1nX0 11B0 1nX0 SgN0 8x30 iw0 kPB0 Awn0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1lnd0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0", "America/Indiana/Vincennes|CST CDT CWT CPT EST EDT|60 50 50 50 50 40|01010230101010101010101010101010454541014545454545454545454545454545454545454545454545454545454545454|-261s0 1nX0 11B0 1nX0 SgN0 8x30 iw0 1o10 11z0 g0p0 11z0 1o10 11z0 1qL0 WN0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 1fz0 1cN0 WL0 1qN0 1cL0 1cN0 1cL0 1cN0 caL0 1cL0 1cN0 1cL0 1qhd0 1o00 Rd0 1zb0 Oo0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0", "America/Indiana/Winamac|CST CDT CWT CPT EST EDT|60 50 50 50 50 40|01010230101010101010101010101010101010454541054545454545454545454545454545454545454545454545454545454545454|-261s0 1nX0 11B0 1nX0 SgN0 8x30 iw0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 1fz0 1cN0 1cL0 1cN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 jrz0 1cL0 1cN0 1cL0 1qhd0 1o00 Rd0 1za0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0", "America/Inuvik|-00 PST PDDT MST MDT|0 80 60 70 60|0121343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343|-FnA0 tWU0 1fA0 wPe0 2pz0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|35e2", "America/Iqaluit|-00 EWT EPT EST EDDT EDT CST CDT|0 40 40 50 30 40 60 50|01234353535353535353535353535353535353535353567353535353535353535353535353535353535353535353535353535353535353535353535353|-16K00 7nX0 iv0 LCL0 1fA0 zgO0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11C0 1nX0 11A0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|67e2", "America/Jamaica|KMT EST EDT|57.b 50 40|0121212121212121212121|-2l1uQ.N 2uM1Q.N 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0|94e4", "America/Juneau|PST PWT PPT PDT YDT YST AKST AKDT|80 70 70 70 80 90 90 80|01203030303030303030303030403030356767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676|-17T20 8x10 iy0 Vo10 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cM0 1cM0 1cL0 1cN0 1fz0 1a10 1fz0 co0 10q0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|33e3", "America/Kentucky/Louisville|CST CDT CWT CPT EST EDT|60 50 50 50 50 40|0101010102301010101010101010101010101454545454545414545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454|-261s0 1nX0 11B0 1nX0 3Fd0 Nb0 LPd0 11z0 RB0 8x30 iw0 Bb0 10N0 2bB0 8in0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 xz0 gso0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1VA0 LA0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0", "America/Kentucky/Monticello|CST CDT CWT CPT EST EDT|60 50 50 50 50 40|0101023010101010101010101010101010101010101010101010101010101010101010101454545454545454545454545454545454545454545454545454545454545454545454545454|-261s0 1nX0 11B0 1nX0 SgN0 8x30 iw0 SWp0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11A0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0", "America/La_Paz|CMT BOST BOT|4w.A 3w.A 40|012|-1x37r.o 13b0|19e5", "America/Lima|LMT PET PEST|58.A 50 40|0121212121212121|-2tyGP.o 1bDzP.o zX0 1aN0 1cL0 1cN0 1cL0 1PrB0 zX0 1O10 zX0 6Gp0 zX0 98p0 zX0|11e6", "America/Los_Angeles|PST PDT PWT PPT|80 70 70 70|010102301010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-261q0 1nX0 11B0 1nX0 SgN0 8x10 iy0 5Wp1 1VaX 3dA0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1a00 1fA0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|15e6", "America/Maceio|LMT BRT BRST|2m.Q 30 20|012121212121212121212121212121212121212121|-2glxB.8 HdLB.8 1cc0 1e10 1bX0 Ezd0 So0 1vA0 Mn0 1BB0 ML0 1BB0 zX0 qe10 xb0 2ep0 nz0 1C10 zX0 1C10 LX0 1C10 Mn0 H210 Rb0 1tB0 IL0 1Fd0 FX0 1EN0 FX0 1HB0 Lz0 dMN0 Lz0 8Q10 WL0 1tB0 5z0 2mN0 On0|93e4", "America/Managua|MMT CST EST CDT|5J.c 60 50 50|0121313121213131|-1quie.M 1yAMe.M 4mn0 9Up0 Dz0 1K10 Dz0 s3F0 1KH0 DB0 9In0 k8p0 19X0 1o30 11y0|22e5", "America/Manaus|LMT AMT AMST|40.4 40 30|01212121212121212121212121212121|-2glvX.U HdKX.U 1cc0 1e10 1bX0 Ezd0 So0 1vA0 Mn0 1BB0 ML0 1BB0 zX0 qe10 xb0 2ep0 nz0 1C10 zX0 1C10 LX0 1C10 Mn0 H210 Rb0 1tB0 IL0 1Fd0 FX0 dPB0 On0|19e5", "America/Martinique|FFMT AST ADT|44.k 40 30|0121|-2mPTT.E 2LPbT.E 19X0|39e4", "America/Matamoros|LMT CST CDT|6E 60 50|0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-1UQG0 2FjC0 1nX0 i6p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 1fB0 WL0 1fB0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 U10 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|45e4", "America/Mazatlan|LMT MST CST PST MDT|75.E 70 60 80 60|0121212131414141414141414141414141414141414141414141414141414141414141414141414141414141414141|-1UQF0 deL0 8lc0 17c0 10M0 1dd0 otX0 gmN0 P2N0 13Vd0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 1fB0 WL0 1fB0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0|44e4", "America/Menominee|CST CDT CWT CPT EST|60 50 50 50 50|01010230101041010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-261s0 1nX0 11B0 1nX0 SgN0 8x30 iw0 1o10 11z0 LCN0 1fz0 6410 9Jb0 1cM0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|85e2", "America/Merida|LMT CST EST CDT|5W.s 60 50 50|0121313131313131313131313131313131313131313131313131313131313131313131313131313131313131|-1UQG0 2q2o0 2hz0 wu30 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 1fB0 WL0 1fB0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0|11e5", "America/Metlakatla|PST PWT PPT PDT AKST AKDT|80 70 70 70 90 80|0120303030303030303030303030303030454545454545454545454545454545454545454545454|-17T20 8x10 iy0 Vo10 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1hU10 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|14e2", "America/Mexico_City|LMT MST CST CDT CWT|6A.A 70 60 50 50|012121232324232323232323232323232323232323232323232323232323232323232323232323232323232323232323232|-1UQF0 deL0 8lc0 17c0 10M0 1dd0 gEn0 TX0 3xd0 Jb0 6zB0 SL0 e5d0 17b0 1Pff0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 1fB0 WL0 1fB0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0|20e6", "America/Miquelon|LMT AST PMST PMDT|3I.E 40 30 20|012323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232|-2mKkf.k 2LTAf.k gQ10 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|61e2", "America/Moncton|EST AST ADT AWT APT|50 40 30 30 30|012121212121212121212134121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-2IsH0 CwN0 1in0 zAo0 An0 1Nd0 An0 1Nd0 An0 1Nd0 An0 1Nd0 An0 1Nd0 An0 1K10 Lz0 1zB0 NX0 1u10 Wn0 S20 8x50 iu0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 11z0 1o10 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 3Cp0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14n1 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 ReX 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|64e3", "America/Monterrey|LMT CST CDT|6F.g 60 50|0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-1UQG0 2FjC0 1nX0 i6p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 1fB0 WL0 1fB0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0|41e5", "America/Montevideo|MMT UYT UYHST UYST UYT UYHST|3I.I 3u 30 20 30 2u|012121212121212121212121213434343434345454543453434343434343434343434343434343434343434|-20UIf.g 8jzJ.g 1cLu 1dcu 1cLu 1dcu 1cLu ircu 11zu 1o0u 11zu 1o0u 11zu 1qMu WLu 1qMu WLu 1qMu WLu 1qMu 11zu 1o0u 11zu NAu 11bu 2iMu zWu Dq10 19X0 pd0 jz0 cm10 19X0 1fB0 1on0 11d0 1oL0 1nB0 1fzu 1aou 1fzu 1aou 1fzu 3nAu Jb0 3MN0 1SLu 4jzu 2PB0 Lb0 3Dd0 1pb0 ixd0 An0 1MN0 An0 1wp0 On0 1wp0 Rb0 1zd0 On0 1wp0 Rb0 s8p0 1fB0 1ip0 11z0 1ld0 14n0 1o10 11z0 1o10 11z0 1o10 14n0 1ld0 14n0 1ld0 14n0 1o10 11z0 1o10 11z0 1o10 11z0|17e5", "America/Toronto|EST EDT EWT EPT|50 40 40 40|01010101010101010101010101010101010101010101012301010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-25TR0 1in0 11Wu 1nzu 1fD0 WJ0 1wr0 Nb0 1Ap0 On0 1zd0 On0 1wp0 TX0 1tB0 TX0 1tB0 TX0 1tB0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 4kM0 8x40 iv0 1o10 11z0 1nX0 11z0 1o10 11z0 1o10 1qL0 11D0 1nX0 11B0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 11z0 1o10 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|65e5", "America/Nassau|LMT EST EDT|59.u 50 40|012121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-2kNuO.u 26XdO.u 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|24e4", "America/New_York|EST EDT EWT EPT|50 40 40 40|01010101010101010101010101010101010101010101010102301010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-261t0 1nX0 11B0 1nX0 11B0 1qL0 1a10 11z0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 11z0 1o10 11z0 RB0 8x40 iv0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|21e6", "America/Nipigon|EST EDT EWT EPT|50 40 40 40|010123010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-25TR0 1in0 Rnb0 3je0 8x40 iv0 19yN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|16e2", "America/Nome|NST NWT NPT BST BDT YST AKST AKDT|b0 a0 a0 b0 a0 90 90 80|012034343434343434343434343434343456767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676767676|-17SX0 8wW0 iB0 Qlb0 52O0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 cl0 10q0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|38e2", "America/Noronha|LMT FNT FNST|29.E 20 10|0121212121212121212121212121212121212121|-2glxO.k HdKO.k 1cc0 1e10 1bX0 Ezd0 So0 1vA0 Mn0 1BB0 ML0 1BB0 zX0 qe10 xb0 2ep0 nz0 1C10 zX0 1C10 LX0 1C10 Mn0 H210 Rb0 1tB0 IL0 1Fd0 FX0 1EN0 FX0 1HB0 Lz0 nsp0 WL0 1tB0 2L0 2pB0 On0|30e2", "America/North_Dakota/Beulah|MST MDT MWT MPT CST CDT|70 60 60 60 60 50|010102301010101010101010101010101010101010101010101010101010101010101010101010101010101010101014545454545454545454545454545454545454545454545454545454|-261r0 1nX0 11B0 1nX0 SgN0 8x20 ix0 QwN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Oo0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0", "America/North_Dakota/Center|MST MDT MWT MPT CST CDT|70 60 60 60 60 50|010102301010101010101010101010101010101010101010101010101014545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454|-261r0 1nX0 11B0 1nX0 SgN0 8x20 ix0 QwN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14o0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0", "America/North_Dakota/New_Salem|MST MDT MWT MPT CST CDT|70 60 60 60 60 50|010102301010101010101010101010101010101010101010101010101010101010101010101010101454545454545454545454545454545454545454545454545454545454545454545454|-261r0 1nX0 11B0 1nX0 SgN0 8x20 ix0 QwN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14o0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0", "America/Ojinaga|LMT MST CST CDT MDT|6V.E 70 60 50 60|0121212323241414141414141414141414141414141414141414141414141414141414141414141414141414141|-1UQF0 deL0 8lc0 17c0 10M0 1dd0 2zQN0 1lb0 14p0 1lb0 14q0 1lb0 14p0 1nX0 11B0 1nX0 1fB0 WL0 1fB0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 U10 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|23e3", "America/Pangnirtung|-00 AST AWT APT ADDT ADT EDT EST CST CDT|0 40 30 30 20 30 40 50 60 50|012314151515151515151515151515151515167676767689767676767676767676767676767676767676767676767676767676767676767676767676767|-1XiM0 PnG0 8x50 iu0 LCL0 1fA0 zgO0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1o00 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11C0 1nX0 11A0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|14e2", "America/Paramaribo|LMT PMT PMT NEGT SRT SRT|3E.E 3E.Q 3E.A 3u 3u 30|012345|-2nDUj.k Wqo0.c qanX.I 1dmLN.o lzc0|24e4", "America/Phoenix|MST MDT MWT|70 60 60|01010202010|-261r0 1nX0 11B0 1nX0 SgN0 4Al1 Ap0 1db0 SWqX 1cL0|42e5", "America/Port-au-Prince|PPMT EST EDT|4N 50 40|01212121212121212121212121212121212121212121|-28RHb 2FnMb 19X0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14q0 1o00 11A0 1o00 11A0 1o00 14o0 1lc0 14o0 1lc0 14o0 1o00 11A0 1o00 11A0 1o00 14o0 1lc0 14o0 1lc0 i6n0 1nX0 11B0 1nX0 d430 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|23e5", "America/Rio_Branco|LMT ACT ACST AMT|4v.c 50 40 40|01212121212121212121212121212131|-2glvs.M HdLs.M 1cc0 1e10 1bX0 Ezd0 So0 1vA0 Mn0 1BB0 ML0 1BB0 zX0 qe10 xb0 2ep0 nz0 1C10 zX0 1C10 LX0 1C10 Mn0 H210 Rb0 1tB0 IL0 1Fd0 FX0 NBd0 d5X0|31e4", "America/Porto_Velho|LMT AMT AMST|4f.A 40 30|012121212121212121212121212121|-2glvI.o HdKI.o 1cc0 1e10 1bX0 Ezd0 So0 1vA0 Mn0 1BB0 ML0 1BB0 zX0 qe10 xb0 2ep0 nz0 1C10 zX0 1C10 LX0 1C10 Mn0 H210 Rb0 1tB0 IL0 1Fd0 FX0|37e4", "America/Puerto_Rico|AST AWT APT|40 30 30|0120|-17lU0 7XT0 iu0|24e5", "America/Rainy_River|CST CDT CWT CPT|60 50 50 50|010123010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-25TQ0 1in0 Rnb0 3je0 8x30 iw0 19yN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|842", "America/Rankin_Inlet|-00 CST CDDT CDT EST|0 60 40 50 50|012131313131313131313131313131313131313131313431313131313131313131313131313131313131313131313131313131313131313131313131|-vDc0 keu0 1fA0 zgO0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|26e2", "America/Recife|LMT BRT BRST|2j.A 30 20|0121212121212121212121212121212121212121|-2glxE.o HdLE.o 1cc0 1e10 1bX0 Ezd0 So0 1vA0 Mn0 1BB0 ML0 1BB0 zX0 qe10 xb0 2ep0 nz0 1C10 zX0 1C10 LX0 1C10 Mn0 H210 Rb0 1tB0 IL0 1Fd0 FX0 1EN0 FX0 1HB0 Lz0 nsp0 WL0 1tB0 2L0 2pB0 On0|33e5", "America/Regina|LMT MST MDT MWT MPT CST|6W.A 70 60 60 60 60|012121212121212121212121341212121212121212121212121215|-2AD51.o uHe1.o 1in0 s2L0 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 11z0 66N0 1cL0 1cN0 19X0 1fB0 1cL0 1fB0 1cL0 1cN0 1cL0 M30 8x20 ix0 1ip0 1cL0 1ip0 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 11z0 1o10 11z0 3NB0 1cL0 1cN0|19e4", "America/Resolute|-00 CST CDDT CDT EST|0 60 40 50 50|012131313131313131313131313131313131313131313431313131313431313131313131313131313131313131313131313131313131313131313131|-SnA0 GWS0 1fA0 zgO0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|229", "America/Santarem|LMT AMT AMST BRT|3C.M 40 30 30|0121212121212121212121212121213|-2glwl.c HdLl.c 1cc0 1e10 1bX0 Ezd0 So0 1vA0 Mn0 1BB0 ML0 1BB0 zX0 qe10 xb0 2ep0 nz0 1C10 zX0 1C10 LX0 1C10 Mn0 H210 Rb0 1tB0 IL0 1Fd0 FX0 NBd0|21e4", "America/Santiago|SMT CLT CLT CLST CLST|4G.K 50 40 40 30|010203131313131212421242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424|-2q2jh.e fJAh.e 5knG.K 1Vzh.e jRAG.K 1pbh.e 11d0 1oL0 11d0 1oL0 11d0 1oL0 11d0 1pb0 11d0 nHX0 op0 9Bz0 jb0 1oN0 ko0 Qeo0 WL0 1zd0 On0 1ip0 11z0 1o10 11z0 1qN0 WL0 1ld0 14n0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 WL0 1qN0 1cL0 1cN0 11z0 1o10 11z0 1qN0 WL0 1fB0 19X0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 17b0 1ip0 11z0 1ip0 1fz0 1fB0 11z0 1qN0 WL0 1qN0 WL0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 17b0 1ip0 11z0 1o10 19X0 1fB0 1nX0 G10 1EL0 Op0 1zb0 Rd0 1wn0 Rd0 46n0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Dd0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Dd0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Dd0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0|62e5", "America/Santo_Domingo|SDMT EST EDT EHDT AST|4E 50 40 4u 40|01213131313131414|-1ttjk 1lJMk Mn0 6sp0 Lbu 1Cou yLu 1RAu wLu 1QMu xzu 1Q0u xXu 1PAu 13jB0 e00|29e5", "America/Sao_Paulo|LMT BRT BRST|36.s 30 20|012121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212|-2glwR.w HdKR.w 1cc0 1e10 1bX0 Ezd0 So0 1vA0 Mn0 1BB0 ML0 1BB0 zX0 pTd0 PX0 2ep0 nz0 1C10 zX0 1C10 LX0 1C10 Mn0 H210 Rb0 1tB0 IL0 1Fd0 FX0 1EN0 FX0 1HB0 Lz0 1EN0 Lz0 1C10 IL0 1HB0 Db0 1HB0 On0 1zd0 On0 1zd0 Lz0 1zd0 Rb0 1wN0 Wn0 1tB0 Rb0 1tB0 WL0 1tB0 Rb0 1zd0 On0 1HB0 FX0 1C10 Lz0 1Ip0 HX0 1zd0 On0 1HB0 IL0 1wp0 On0 1C10 Lz0 1C10 On0 1zd0 On0 1zd0 Rb0 1zd0 Lz0 1C10 Lz0 1C10 On0 1zd0 On0 1zd0 On0 1zd0 On0 1C10 Lz0 1C10 Lz0 1C10 On0 1zd0 On0 1zd0 Rb0 1wp0 On0 1C10 Lz0 1C10 On0 1zd0 On0 1zd0 On0 1zd0 On0 1C10 Lz0 1C10 Lz0 1C10 Lz0 1C10 On0 1zd0 Rb0 1wp0 On0 1C10 Lz0 1C10 On0 1zd0|20e6", "America/Scoresbysund|LMT CGT CGST EGST EGT|1r.Q 20 10 0 10|0121343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434|-2a5Ww.8 2z5ew.8 1a00 1cK0 1cL0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|452", "America/Sitka|PST PWT PPT PDT YST AKST AKDT|80 70 70 70 90 90 80|01203030303030303030303030303030345656565656565656565656565656565656565656565656565656565656565656565656565656565656565656565656565656565656565|-17T20 8x10 iy0 Vo10 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 co0 10q0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|90e2", "America/St_Johns|NST NDT NST NDT NWT NPT NDDT|3u.Q 2u.Q 3u 2u 2u 2u 1u|01010101010101010101010101010101010102323232323232324523232323232323232323232323232323232323232323232323232323232323232323232323232323232326232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232|-28oit.8 14L0 1nB0 1in0 1gm0 Dz0 1JB0 1cL0 1cN0 1cL0 1fB0 19X0 1fB0 19X0 1fB0 19X0 1fB0 19X0 1fB0 1cL0 1cN0 1cL0 1fB0 19X0 1fB0 19X0 1fB0 19X0 1fB0 19X0 1fB0 1cL0 1fB0 19X0 1fB0 19X0 10O0 eKX.8 19X0 1iq0 WL0 1qN0 WL0 1qN0 WL0 1tB0 TX0 1tB0 WL0 1qN0 WL0 1qN0 7UHu itu 1tB0 WL0 1qN0 WL0 1qN0 WL0 1qN0 WL0 1tB0 WL0 1ld0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14n1 1lb0 14p0 1nW0 11C0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zcX Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|11e4", "America/Swift_Current|LMT MST MDT MWT MPT CST|7b.k 70 60 60 60 60|012134121212121212121215|-2AD4M.E uHdM.E 1in0 UGp0 8x20 ix0 1o10 17b0 1ip0 11z0 1o10 11z0 1o10 11z0 isN0 1cL0 3Cp0 1cL0 1cN0 11z0 1qN0 WL0 pMp0|16e3", "America/Tegucigalpa|LMT CST CDT|5M.Q 60 50|01212121|-1WGGb.8 2ETcb.8 WL0 1qN0 WL0 GRd0 AL0|11e5", "America/Thule|LMT AST ADT|4z.8 40 30|012121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-2a5To.Q 31NBo.Q 1cL0 1cN0 1cL0 1fB0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|656", "America/Thunder_Bay|CST EST EWT EPT EDT|60 50 40 40 40|0123141414141414141414141414141414141414141414141414141414141414141414141414141414141414141414141414141414141414141414141414141414141414141|-2q5S0 1iaN0 8x40 iv0 XNB0 1cL0 1cN0 1fz0 1cN0 1cL0 3Cp0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|11e4", "America/Vancouver|PST PDT PWT PPT|80 70 70 70|0102301010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-25TO0 1in0 UGp0 8x10 iy0 1o10 17b0 1ip0 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|23e5", "America/Whitehorse|YST YDT YWT YPT YDDT PST PDT|90 80 80 80 70 80 70|0101023040565656565656565656565656565656565656565656565656565656565656565656565656565656565656565656565656565656565656565656565|-25TN0 1in0 1o10 13V0 Ser0 8x00 iz0 LCL0 1fA0 3NA0 vrd0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|23e3", "America/Winnipeg|CST CDT CWT CPT|60 50 50 50|010101023010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-2aIi0 WL0 3ND0 1in0 Jap0 Rb0 aCN0 8x30 iw0 1tB0 11z0 1ip0 11z0 1o10 11z0 1o10 11z0 1rd0 10L0 1op0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 1cL0 1cN0 11z0 6i10 WL0 6i10 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1a00 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1a00 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 14o0 1lc0 14o0 1o00 11A0 1o00 11A0 1o00 14o0 1lc0 14o0 1lc0 14o0 1o00 11A0 1o00 11A0 1o00 14o0 1lc0 14o0 1lc0 14o0 1lc0 14o0 1o00 11A0 1o00 11A0 1o00 14o0 1lc0 14o0 1lc0 14o0 1o00 11A0 1o00 11A0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|66e4", "America/Yakutat|YST YWT YPT YDT AKST AKDT|90 80 80 80 90 80|01203030303030303030303030303030304545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454|-17T10 8x00 iz0 Vo10 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 cn0 10q0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|642", "America/Yellowknife|-00 MST MWT MPT MDDT MDT|0 70 60 60 50 60|012314151515151515151515151515151515151515151515151515151515151515151515151515151515151515151515151515151515151515151515151|-1pdA0 hix0 8x20 ix0 LCL0 1fA0 zgO0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|19e3", "Antarctica/Casey|-00 +08 +11|0 -80 -b0|0121212|-2q00 1DjS0 T90 40P0 KL0 blz0|10", "Antarctica/Davis|-00 +07 +05|0 -70 -50|01012121|-vyo0 iXt0 alj0 1D7v0 VB0 3Wn0 KN0|70", "Antarctica/DumontDUrville|-00 +10|0 -a0|0101|-U0o0 cfq0 bFm0|80", "Antarctica/Macquarie|AEST AEDT -00 MIST|-a0 -b0 0 -b0|0102010101010101010101010101010101010101010101010101010101010101010101010101010101010101013|-29E80 19X0 4SL0 1ayy0 Lvs0 1cM0 1o00 Rc0 1wo0 Rc0 1wo0 U00 1wo0 LA0 1C00 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 11A0 1qM0 WM0 1qM0 Oo0 1zc0 Oo0 1zc0 Oo0 1wo0 WM0 1tA0 WM0 1tA0 U00 1tA0 U00 1tA0 11A0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 11A0 1o00 1io0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1cM0 1a00 1io0 1cM0 1cM0 1cM0 1cM0 1cM0|1", "Antarctica/Mawson|-00 +06 +05|0 -60 -50|012|-CEo0 2fyk0|60", "Pacific/Auckland|NZMT NZST NZST NZDT|-bu -cu -c0 -d0|01020202020202020202020202023232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323|-1GCVu Lz0 1tB0 11zu 1o0u 11zu 1o0u 11zu 1o0u 14nu 1lcu 14nu 1lcu 1lbu 11Au 1nXu 11Au 1nXu 11Au 1nXu 11Au 1nXu 11Au 1qLu WMu 1qLu 11Au 1n1bu IM0 1C00 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1qM0 14o0 1lc0 14o0 1lc0 14o0 1lc0 17c0 1io0 17c0 1io0 17c0 1io0 17c0 1lc0 14o0 1lc0 14o0 1lc0 17c0 1io0 17c0 1io0 17c0 1lc0 14o0 1lc0 14o0 1lc0 17c0 1io0 17c0 1io0 17c0 1io0 17c0 1io0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1io0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1fA0 1a00 1fA0 1a00|14e5", "Antarctica/Palmer|-00 ARST ART ART ARST CLT CLST|0 30 40 30 20 40 30|0121212121234356565656565656565656565656565656565656565656565656565656565656565656565656565656565656565656565656565656565656|-cao0 nD0 1vd0 SL0 1vd0 17z0 1cN0 1fz0 1cN0 1cL0 1cN0 asn0 Db0 jsN0 14N0 11z0 1o10 11z0 1qN0 WL0 1qN0 WL0 1qN0 1cL0 1cN0 11z0 1o10 11z0 1qN0 WL0 1fB0 19X0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 17b0 1ip0 11z0 1ip0 1fz0 1fB0 11z0 1qN0 WL0 1qN0 WL0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 17b0 1ip0 11z0 1o10 19X0 1fB0 1nX0 G10 1EL0 Op0 1zb0 Rd0 1wn0 Rd0 46n0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Dd0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Dd0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Dd0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0|40", "Antarctica/Rothera|-00 -03|0 30|01|gOo0|130", "Antarctica/Syowa|-00 +03|0 -30|01|-vs00|20", "Antarctica/Troll|-00 +00 +02|0 0 -20|01212121212121212121212121212121212121212121212121212121212121212121|1puo0 hd0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|40", "Antarctica/Vostok|-00 +06|0 -60|01|-tjA0|25", "Europe/Oslo|CET CEST|-10 -20|010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-2awM0 Qm0 W6o0 5pf0 WM0 1fA0 1cM0 1cM0 1cM0 1cM0 wJc0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1qM0 WM0 zpc0 1a00 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|62e4", "Asia/Riyadh|LMT AST|-36.Q -30|01|-TvD6.Q|57e5", "Asia/Almaty|LMT +05 +06 +07|-57.M -50 -60 -70|012323232323232323232321232323232323232323232323232|-1Pc57.M eUo7.M 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0|15e5", "Asia/Amman|LMT EET EEST|-2n.I -20 -30|0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-1yW2n.I 1HiMn.I KL0 1oN0 11b0 1oN0 11b0 1pd0 1dz0 1cp0 11b0 1op0 11b0 fO10 1db0 1e10 1cL0 1cN0 1cL0 1cN0 1fz0 1pd0 10n0 1ld0 14n0 1hB0 15b0 1ip0 19X0 1cN0 1cL0 1cN0 17b0 1ld0 14o0 1lc0 17c0 1io0 17c0 1io0 17c0 1So0 y00 1fc0 1dc0 1co0 1dc0 1cM0 1cM0 1cM0 1o00 11A0 1lc0 17c0 1cM0 1cM0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 4bX0 Dd0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0|25e5", "Asia/Anadyr|LMT +12 +13 +14 +11|-bN.U -c0 -d0 -e0 -b0|01232121212121212121214121212121212121212121212121212121212141|-1PcbN.U eUnN.U 23CL0 1db0 2q10 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 2sp0 WM0|13e3", "Asia/Aqtau|LMT +04 +05 +06|-3l.4 -40 -50 -60|012323232323232323232123232312121212121212121212|-1Pc3l.4 eUnl.4 24PX0 2pX0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cN0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0|15e4", "Asia/Aqtobe|LMT +04 +05 +06|-3M.E -40 -50 -60|0123232323232323232321232323232323232323232323232|-1Pc3M.E eUnM.E 23CL0 3Db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0|27e4", "Asia/Ashgabat|LMT +04 +05 +06|-3R.w -40 -50 -60|0123232323232323232323212|-1Pc3R.w eUnR.w 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0|41e4", "Asia/Atyrau|LMT +04 +05 +06|-3r.I -40 -50 -60|01232323232323232323212323232323232321212121212|-1Pc3r.I eUnr.I 24PX0 2pX0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 2sp0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0", "Asia/Baghdad|BMT AST ADT|-2V.A -30 -40|012121212121212121212121212121212121212121212121212121|-26BeV.A 2ACnV.A 11b0 1cp0 1dz0 1dd0 1db0 1cN0 1cp0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1de0 1dc0 1dc0 1dc0 1cM0 1dc0 1cM0 1dc0 1cM0 1dc0 1dc0 1dc0 1cM0 1dc0 1cM0 1dc0 1cM0 1dc0 1dc0 1dc0 1cM0 1dc0 1cM0 1dc0 1cM0 1dc0 1dc0 1dc0 1cM0 1dc0 1cM0 1dc0 1cM0 1dc0|66e5", "Asia/Qatar|LMT GST AST|-3q.8 -40 -30|012|-21Jfq.8 27BXq.8|96e4", "Asia/Baku|LMT +03 +04 +05|-3j.o -30 -40 -50|01232323232323232323232123232323232323232323232323232323232323232|-1Pc3j.o 1jUoj.o WCL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 1cM0 9Je0 1o00 11z0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00|27e5", "Asia/Bangkok|BMT ICT|-6G.4 -70|01|-218SG.4|15e6", "Asia/Barnaul|LMT +06 +07 +08|-5z -60 -70 -80|0123232323232323232323212323232321212121212121212121212121212121212|-21S5z pCnz 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 p90 LE0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0 3rd0", "Asia/Beirut|EET EEST|-20 -30|010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-21aq0 1on0 1410 1db0 19B0 1in0 1ip0 WL0 1lQp0 11b0 1oN0 11b0 1oN0 11b0 1pd0 11b0 1oN0 11b0 q6N0 En0 1oN0 11b0 1oN0 11b0 1oN0 11b0 1pd0 11b0 1oN0 11b0 1op0 11b0 dA10 17b0 1iN0 17b0 1iN0 17b0 1iN0 17b0 1vB0 SL0 1mp0 13z0 1iN0 17b0 1iN0 17b0 1jd0 12n0 1a10 1cL0 1cN0 1cL0 1cN0 1cL0 1fB0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0|22e5", "Asia/Bishkek|LMT +05 +06 +07|-4W.o -50 -60 -70|012323232323232323232321212121212121212121212121212|-1Pc4W.o eUnW.o 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2e00 1tX0 17b0 1ip0 17b0 1ip0 17b0 1ip0 17b0 1ip0 19X0 1cPu 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0|87e4", "Asia/Brunei|LMT BNT BNT|-7D.E -7u -80|012|-1KITD.E gDc9.E|42e4", "Asia/Kolkata|HMT BURT IST IST|-5R.k -6u -5u -6u|01232|-18LFR.k 1unn.k HB0 7zX0|15e6", "Asia/Chita|LMT +08 +09 +10|-7x.Q -80 -90 -a0|012323232323232323232321232323232323232323232323232323232323232312|-21Q7x.Q pAnx.Q 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0 3re0|33e4", "Asia/Choibalsan|LMT ULAT ULAT CHOST CHOT CHOT CHOST|-7C -70 -80 -a0 -90 -80 -90|0123434343434343434343434343434343434343434343456565656565656565656565656565656565656565656565|-2APHC 2UkoC cKn0 1da0 1dd0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1fB0 1cL0 1cN0 1cL0 1cN0 1cL0 6hD0 11z0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 3Db0 h1f0 1cJ0 1cP0 1cJ0 1cP0 1fx0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1fx0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1fx0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1fx0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0|38e3", "Asia/Shanghai|CST CDT|-80 -90|01010101010101010|-1c1I0 LX0 16p0 1jz0 1Myp0 Rb0 1o10 11z0 1o10 11z0 1qN0 11z0 1o10 11z0 1o10 11z0|23e6", "Asia/Colombo|MMT +0530 +06 +0630|-5j.w -5u -60 -6u|01231321|-2zOtj.w 1rFbN.w 1zzu 7Apu 23dz0 11zu n3cu|22e5", "Asia/Dhaka|HMT BURT IST DACT BDT BDST|-5R.k -6u -5u -60 -60 -70|01213454|-18LFR.k 1unn.k HB0 m6n0 LqMu 1x6n0 1i00|16e6", "Asia/Damascus|LMT EET EEST|-2p.c -20 -30|01212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-21Jep.c Hep.c 17b0 1ip0 17b0 1ip0 17b0 1ip0 19X0 1xRB0 11X0 1oN0 10L0 1pB0 11b0 1oN0 10L0 1mp0 13X0 1oN0 11b0 1pd0 11b0 1oN0 11b0 1oN0 11b0 1oN0 11b0 1pd0 11b0 1oN0 11b0 1oN0 11b0 1oN0 11b0 1pd0 11b0 1oN0 Nb0 1AN0 Nb0 bcp0 19X0 1gp0 19X0 3ld0 1xX0 Vd0 1Bz0 Sp0 1vX0 10p0 1dz0 1cN0 1cL0 1db0 1db0 1g10 1an0 1ap0 1db0 1fd0 1db0 1cN0 1db0 1dd0 1db0 1cp0 1dz0 1c10 1dX0 1cN0 1db0 1dd0 1db0 1cN0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1db0 1cN0 1db0 1cN0 19z0 1fB0 1qL0 11B0 1on0 Wp0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0|26e5", "Asia/Dili|LMT TLT JST TLT WITA|-8m.k -80 -90 -90 -80|012343|-2le8m.k 1dnXm.k 8HA0 1ew00 Xld0|19e4", "Asia/Dubai|LMT GST|-3F.c -40|01|-21JfF.c|39e5", "Asia/Dushanbe|LMT +05 +06 +07|-4z.c -50 -60 -70|012323232323232323232321|-1Pc4z.c eUnz.c 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2hB0|76e4", "Asia/Famagusta|LMT EET EEST +03|-2f.M -20 -30 -30|01212121212121212121212121212121212121212121212121212121212121212121212121212121212123|-1Vc2f.M 2a3cf.M 1cL0 1qp0 Xz0 19B0 19X0 1fB0 1db0 1cp0 1cL0 1fB0 19X0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1fB0 1cL0 1cN0 1cL0 1cN0 1o30 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 15U0", "Asia/Gaza|EET EEST IST IDT|-20 -30 -20 -30|010101010101010101010101010101012323232323232323232323232320101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-1c2q0 5Rb0 10r0 1px0 10N0 1pz0 16p0 1jB0 16p0 1jx0 pBd0 Vz0 1oN0 11b0 1oO0 10N0 1pz0 10N0 1pb0 10N0 1pb0 10N0 1pb0 10N0 1pz0 10N0 1pb0 10N0 1pb0 11d0 1oL0 dW0 hfB0 Db0 1fB0 Rb0 npB0 11z0 1C10 IL0 1s10 10n0 1o10 WL0 1zd0 On0 1ld0 11z0 1o10 14n0 1o10 14n0 1nd0 12n0 1nd0 Xz0 1q10 12n0 M10 C00 17c0 1io0 17c0 1io0 17c0 1o00 1cL0 1fB0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 17c0 1io0 18N0 1bz0 19z0 1gp0 1610 1iL0 11z0 1o10 14o0 1lA1 SKX 1xd1 MKX 1AN0 1a00 1fA0 1cL0 1cN0 1nX0 1210 1nz0 1220 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1qL0|18e5", "Asia/Hebron|EET EEST IST IDT|-20 -30 -20 -30|01010101010101010101010101010101232323232323232323232323232010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-1c2q0 5Rb0 10r0 1px0 10N0 1pz0 16p0 1jB0 16p0 1jx0 pBd0 Vz0 1oN0 11b0 1oO0 10N0 1pz0 10N0 1pb0 10N0 1pb0 10N0 1pb0 10N0 1pz0 10N0 1pb0 10N0 1pb0 11d0 1oL0 dW0 hfB0 Db0 1fB0 Rb0 npB0 11z0 1C10 IL0 1s10 10n0 1o10 WL0 1zd0 On0 1ld0 11z0 1o10 14n0 1o10 14n0 1nd0 12n0 1nd0 Xz0 1q10 12n0 M10 C00 17c0 1io0 17c0 1io0 17c0 1o00 1cL0 1fB0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 17c0 1io0 18N0 1bz0 19z0 1gp0 1610 1iL0 12L0 1mN0 14o0 1lc0 Tb0 1xd1 MKX bB0 cn0 1cN0 1a00 1fA0 1cL0 1cN0 1nX0 1210 1nz0 1220 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 1qL0|25e4", "Asia/Ho_Chi_Minh|LMT PLMT ICT IDT JST|-76.E -76.u -70 -80 -90|0123423232|-2yC76.E bK00.a 1h7b6.u 5lz0 18o0 3Oq0 k5b0 aW00 BAM0|90e5", "Asia/Hong_Kong|LMT HKT HKST JST|-7A.G -80 -90 -90|0121312121212121212121212121212121212121212121212121212121212121212121|-2CFHA.G 1sEP6.G 1cL0 ylu 93X0 1qQu 1tX0 Rd0 1In0 NB0 1cL0 11B0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1kL0 14N0 1nX0 U10 1tz0 U10 1wn0 Rd0 1wn0 U10 1tz0 U10 1tz0 U10 1tz0 U10 1wn0 Rd0 1wn0 Rd0 1wn0 U10 1tz0 U10 1tz0 17d0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 s10 1Vz0 1cN0 1cL0 1cN0 1cL0 6fd0 14n0|73e5", "Asia/Hovd|LMT HOVT HOVT HOVST|-66.A -60 -70 -80|012323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232|-2APG6.A 2Uko6.A cKn0 1db0 1dd0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1fB0 1cL0 1cN0 1cL0 1cN0 1cL0 6hD0 11z0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 kEp0 1cJ0 1cP0 1cJ0 1cP0 1fx0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1fx0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1fx0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1fx0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0|81e3", "Asia/Irkutsk|IMT +07 +08 +09|-6V.5 -70 -80 -90|01232323232323232323232123232323232323232323232323232323232323232|-21zGV.5 pjXV.5 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0|60e4", "Europe/Istanbul|IMT EET EEST +04 +03|-1U.U -20 -30 -40 -30|012121212121212121212121212121212121212121212121212121234343434342121212121212121212121212121212121212121212121212121212121212124|-2ogNU.U dzzU.U 11b0 8tB0 1on0 1410 1db0 19B0 1in0 3Rd0 Un0 1oN0 11b0 zSp0 CL0 mN0 1Vz0 1gN0 1pz0 5Rd0 1fz0 1yp0 ML0 1kp0 17b0 1ip0 17b0 1fB0 19X0 1jB0 18L0 1ip0 17z0 qdd0 xX0 3S10 Tz0 dA10 11z0 1o10 11z0 1qN0 11z0 1ze0 11B0 WM0 1qO0 WI0 1nX0 1rB0 10L0 11B0 1in0 17d0 1in0 2pX0 19E0 1fU0 16Q0 1iI0 16Q0 1iI0 1Vd0 pb0 3Kp0 14o0 1de0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1a00 1fA0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WO0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 Xc0 1qo0 WM0 1qM0 11A0 1o00 1200 1nA0 11A0 1tA0 U00 15w0|13e6", "Asia/Jakarta|BMT JAVT WIB JST WIB WIB|-77.c -7k -7u -90 -80 -70|01232425|-1Q0Tk luM0 mPzO 8vWu 6kpu 4PXu xhcu|31e6", "Asia/Jayapura|LMT WIT ACST|-9m.M -90 -9u|0121|-1uu9m.M sMMm.M L4nu|26e4", "Asia/Jerusalem|JMT IST IDT IDDT|-2k.E -20 -30 -40|01212121212132121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-26Bek.E SyMk.E 5Rb0 10r0 1px0 10N0 1pz0 16p0 1jB0 16p0 1jx0 3LB0 Em0 or0 1cn0 1dB0 16n0 10O0 1ja0 1tC0 14o0 1cM0 1a00 11A0 1Na0 An0 1MP0 AJ0 1Kp0 LC0 1oo0 Wl0 EQN0 Db0 1fB0 Rb0 npB0 11z0 1C10 IL0 1s10 10n0 1o10 WL0 1zd0 On0 1ld0 11z0 1o10 14n0 1o10 14n0 1nd0 12n0 1nd0 Xz0 1q10 12n0 1hB0 1dX0 1ep0 1aL0 1eN0 17X0 1nf0 11z0 1tB0 19W0 1e10 17b0 1ep0 1gL0 18N0 1fz0 1eN0 17b0 1gq0 1gn0 19d0 1dz0 1c10 17X0 1hB0 1gn0 19d0 1dz0 1c10 17X0 1kp0 1dz0 1c10 1aL0 1eN0 1oL0 10N0 1oL0 10N0 1oL0 10N0 1rz0 W10 1rz0 W10 1rz0 10N0 1oL0 10N0 1oL0 10N0 1rz0 W10 1rz0 W10 1rz0 10N0 1oL0 10N0 1oL0 10N0 1oL0 10N0 1rz0 W10 1rz0 W10 1rz0 10N0 1oL0 10N0 1oL0 10N0 1rz0 W10 1rz0 W10 1rz0 W10 1rz0 10N0 1oL0 10N0 1oL0|81e4", "Asia/Kabul|AFT AFT|-40 -4u|01|-10Qs0|46e5", "Asia/Kamchatka|LMT +11 +12 +13|-ay.A -b0 -c0 -d0|012323232323232323232321232323232323232323232323232323232323212|-1SLKy.A ivXy.A 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 2sp0 WM0|18e4", "Asia/Karachi|LMT IST IST KART PKT PKST|-4s.c -5u -6u -50 -50 -60|012134545454|-2xoss.c 1qOKW.c 7zX0 eup0 LqMu 1fy00 1cL0 dK10 11b0 1610 1jX0|24e6", "Asia/Urumqi|LMT XJT|-5O.k -60|01|-1GgtO.k|32e5", "Asia/Kathmandu|LMT IST NPT|-5F.g -5u -5J|012|-21JhF.g 2EGMb.g|12e5", "Asia/Khandyga|LMT +08 +09 +10 +11|-92.d -80 -90 -a0 -b0|0123232323232323232323212323232323232323232323232343434343434343432|-21Q92.d pAp2.d 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 qK0 yN0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 17V0 7zD0|66e2", "Asia/Krasnoyarsk|LMT +06 +07 +08|-6b.q -60 -70 -80|01232323232323232323232123232323232323232323232323232323232323232|-21Hib.q prAb.q 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0|10e5", "Asia/Kuala_Lumpur|SMT MALT MALST MALT MALT JST MYT|-6T.p -70 -7k -7k -7u -90 -80|01234546|-2Bg6T.p 17anT.p 7hXE dM00 17bO 8Fyu 1so1u|71e5", "Asia/Kuching|LMT BORT BORT BORTST JST MYT|-7l.k -7u -80 -8k -90 -80|01232323232323232425|-1KITl.k gDbP.k 6ynu AnE 1O0k AnE 1NAk AnE 1NAk AnE 1NAk AnE 1O0k AnE 1NAk AnE pAk 8Fz0 1so10|13e4", "Asia/Macau|LMT MOT MOST CST|-7y.k -80 -90 -80|0121212121212121212121212121212121212121213|-2le7y.k 1XO34.k 1wn0 Rd0 1wn0 R9u 1wqu U10 1tz0 TVu 1tz0 17gu 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cJu 1cL0 1cN0 1fz0 1cN0 1cOu 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cJu 1cL0 1cN0 1fz0 1cN0 1cL0 KEp0|57e4", "Asia/Magadan|LMT +10 +11 +12|-a3.c -a0 -b0 -c0|012323232323232323232321232323232323232323232323232323232323232312|-1Pca3.c eUo3.c 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0 3Cq0|95e3", "Asia/Makassar|LMT MMT WITA JST|-7V.A -7V.A -80 -90|01232|-21JjV.A vfc0 myLV.A 8ML0|15e5", "Asia/Manila|PHT PHST JST|-80 -90 -90|010201010|-1kJI0 AL0 cK10 65X0 mXB0 vX0 VK10 1db0|24e6", "Asia/Nicosia|LMT EET EEST|-2d.s -20 -30|01212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-1Vc2d.s 2a3cd.s 1cL0 1qp0 Xz0 19B0 19X0 1fB0 1db0 1cp0 1cL0 1fB0 19X0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1fB0 1cL0 1cN0 1cL0 1cN0 1o30 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|32e4", "Asia/Novokuznetsk|LMT +06 +07 +08|-5M.M -60 -70 -80|012323232323232323232321232323232323232323232323232323232323212|-1PctM.M eULM.M 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 2sp0 WM0|55e4", "Asia/Novosibirsk|LMT +06 +07 +08|-5v.E -60 -70 -80|0123232323232323232323212323212121212121212121212121212121212121212|-21Qnv.E pAFv.E 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 ml0 Os0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0 4eN0|15e5", "Asia/Omsk|LMT +05 +06 +07|-4R.u -50 -60 -70|01232323232323232323232123232323232323232323232323232323232323232|-224sR.u pMLR.u 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0|12e5", "Asia/Oral|LMT +04 +05 +06|-3p.o -40 -50 -60|01232323232323232121212121212121212121212121212|-1Pc3p.o eUnp.o 23CL0 3Db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 2pB0 1cM0 1fA0 1cM0 1cM0 IM0 1EM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0|27e4", "Asia/Pontianak|LMT PMT WIB JST WIB WITA WIB|-7h.k -7h.k -7u -90 -80 -80 -70|012324256|-2ua7h.k XE00 munL.k 8Rau 6kpu 4PXu xhcu Wqnu|23e4", "Asia/Pyongyang|LMT KST JCST JST KST|-8n -8u -90 -90 -90|012341|-2um8n 97XR 12FXu jdA0 2Onc0|29e5", "Asia/Qyzylorda|LMT +04 +05 +06|-4l.Q -40 -50 -60|0123232323232323232323232323232323232323232323|-1Pc4l.Q eUol.Q 23CL0 3Db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 3ao0 1EM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0|73e4", "Asia/Rangoon|RMT BURT JST MMT|-6o.E -6u -90 -6u|0123|-21Jio.E SmnS.E 7j9u|48e5", "Asia/Sakhalin|LMT +09 +11 +12 +10|-9u.M -90 -b0 -c0 -a0|01232323232323232323232423232323232424242424242424242424242424242|-2AGVu.M 1BoMu.M 1qFa0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 2pB0 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0 3rd0|58e4", "Asia/Samarkand|LMT +04 +05 +06|-4r.R -40 -50 -60|01232323232323232323232|-1Pc4r.R eUor.R 23CL0 3Db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0|36e4", "Asia/Seoul|LMT KST JCST JST KST KDT KDT|-8r.Q -8u -90 -90 -90 -9u -a0|01234151515151515146464|-2um8r.Q 97XV.Q 12FXu jjA0 kKo0 2I0u OL0 1FB0 Rb0 1qN0 TX0 1tB0 TX0 1tB0 TX0 1tB0 TX0 2ap0 12FBu 11A0 1o00 11A0|23e6", "Asia/Singapore|SMT MALT MALST MALT MALT JST SGT SGT|-6T.p -70 -7k -7k -7u -90 -7u -80|012345467|-2Bg6T.p 17anT.p 7hXE dM00 17bO 8Fyu Mspu DTA0|56e5", "Asia/Srednekolymsk|LMT +10 +11 +12|-ae.Q -a0 -b0 -c0|01232323232323232323232123232323232323232323232323232323232323232|-1Pcae.Q eUoe.Q 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0|35e2", "Asia/Taipei|JWST JST CST CDT|-80 -90 -80 -90|01232323232323232323232323232323232323232|-1iw80 joM0 1yo0 Tz0 1ip0 1jX0 1cN0 11b0 1oN0 11b0 1oN0 11b0 1oN0 11b0 10N0 1BX0 10p0 1pz0 10p0 1pz0 10p0 1db0 1dd0 1db0 1cN0 1db0 1cN0 1db0 1cN0 1db0 1BB0 ML0 1Bd0 ML0 uq10 1db0 1cN0 1db0 97B0 AL0|74e5", "Asia/Tashkent|LMT +05 +06 +07|-4B.b -50 -60 -70|012323232323232323232321|-1Pc4B.b eUnB.b 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0|23e5", "Asia/Tbilisi|TBMT +03 +04 +05|-2X.b -30 -40 -50|0123232323232323232323212121232323232323232323212|-1Pc2X.b 1jUnX.b WCL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 1cK0 1cL0 1cN0 1cL0 1cN0 2pz0 1cL0 1fB0 3Nz0 11B0 1nX0 11B0 1qL0 WN0 1qL0 WN0 1qL0 11B0 1nX0 11B0 1nX0 11B0 An0 Os0 WM0|11e5", "Asia/Tehran|LMT TMT IRST IRST IRDT IRDT|-3p.I -3p.I -3u -40 -50 -4u|01234325252525252525252525252525252525252525252525252525252525252525252525252525252525252525252525252|-2btDp.I 1d3c0 1huLT.I TXu 1pz0 sN0 vAu 1cL0 1dB0 1en0 pNB0 UL0 1cN0 1dz0 1cp0 1dz0 1cp0 1dz0 1cp0 1dz0 1cp0 1dz0 1cN0 1dz0 1cp0 1dz0 1cp0 1dz0 1cp0 1dz0 1cN0 1dz0 1cp0 1dz0 1cp0 1dz0 1cp0 1dz0 1cN0 1dz0 64p0 1dz0 1cN0 1dz0 1cp0 1dz0 1cp0 1dz0 1cp0 1dz0 1cN0 1dz0 1cp0 1dz0 1cp0 1dz0 1cp0 1dz0 1cN0 1dz0 1cp0 1dz0 1cp0 1dz0 1cp0 1dz0 1cN0 1dz0 1cp0 1dz0 1cp0 1dz0 1cp0 1dz0 1cN0 1dz0 1cp0 1dz0 1cp0 1dz0 1cp0 1dz0 1cp0 1dz0 1cN0 1dz0 1cp0 1dz0 1cp0 1dz0 1cp0 1dz0 1cN0 1dz0 1cp0 1dz0 1cp0 1dz0 1cp0 1dz0|14e6", "Asia/Thimphu|LMT IST BTT|-5W.A -5u -60|012|-Su5W.A 1BGMs.A|79e3", "Asia/Tokyo|JCST JST JDT|-90 -90 -a0|0121212121|-1iw90 pKq0 QL0 1lB0 13X0 1zB0 NX0 1zB0 NX0|38e6", "Asia/Tomsk|LMT +06 +07 +08|-5D.P -60 -70 -80|0123232323232323232323212323232323232323232323212121212121212121212|-21NhD.P pxzD.P 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 co0 1bB0 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0 3Qp0|10e5", "Asia/Ulaanbaatar|LMT ULAT ULAT ULAST|-77.w -70 -80 -90|012323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232|-2APH7.w 2Uko7.w cKn0 1db0 1dd0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1fB0 1cL0 1cN0 1cL0 1cN0 1cL0 6hD0 11z0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 kEp0 1cJ0 1cP0 1cJ0 1cP0 1fx0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1fx0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1fx0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1fx0 1cP0 1cJ0 1cP0 1cJ0 1cP0 1cJ0|12e5", "Asia/Ust-Nera|LMT +08 +09 +12 +11 +10|-9w.S -80 -90 -c0 -b0 -a0|012343434343434343434345434343434343434343434343434343434343434345|-21Q9w.S pApw.S 23CL0 1d90 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 17V0 7zD0|65e2", "Asia/Vladivostok|LMT +09 +10 +11|-8L.v -90 -a0 -b0|01232323232323232323232123232323232323232323232323232323232323232|-1SJIL.v itXL.v 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0|60e4", "Asia/Yakutsk|LMT +08 +09 +10|-8C.W -80 -90 -a0|01232323232323232323232123232323232323232323232323232323232323232|-21Q8C.W pAoC.W 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0|28e4", "Asia/Yekaterinburg|LMT PMT +04 +05 +06|-42.x -3J.5 -40 -50 -60|012343434343434343434343234343434343434343434343434343434343434343|-2ag42.x 7mQh.s qBvJ.5 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0|14e5", "Asia/Yerevan|LMT +03 +04 +05|-2W -30 -40 -50|0123232323232323232323212121212323232323232323232323232323232|-1Pc2W 1jUnW WCL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 2pB0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 4RX0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0|13e5", "Atlantic/Azores|HMT AZOT AZOST AZOMT AZOT AZOST WET|1S.w 20 10 0 10 0 0|01212121212121212121212121212121212121212121232123212321232121212121212121212121212121212121212121454545454545454545454545454545456545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454|-2ldW5.s aPX5.s Sp0 LX0 1vc0 Tc0 1uM0 SM0 1vc0 Tc0 1vc0 SM0 1vc0 6600 1co0 3E00 17c0 1fA0 1a00 1io0 1a00 1io0 17c0 3I00 17c0 1cM0 1cM0 3Fc0 1cM0 1a00 1fA0 1io0 17c0 1cM0 1cM0 1a00 1fA0 1io0 1qM0 Dc0 1tA0 1cM0 1dc0 1400 gL0 IM0 s10 U00 dX0 Rc0 pd0 Rc0 gL0 Oo0 pd0 Rc0 gL0 Oo0 pd0 14o0 1cM0 1cP0 1cM0 1cM0 1cM0 1cM0 1cM0 3Co0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 qIl0 1cM0 1fA0 1cM0 1cM0 1cN0 1cL0 1cN0 1cM0 1cM0 1cM0 1cM0 1cN0 1cL0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cL0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|25e4", "Atlantic/Bermuda|LMT AST ADT|4j.i 40 30|0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-1BnRE.G 1LTbE.G 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0|65e3", "Atlantic/Canary|LMT CANT WET WEST|11.A 10 0 -10|01232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232|-1UtaW.o XPAW.o 1lAK0 1a10 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|54e4", "Atlantic/Cape_Verde|LMT CVT CVST CVT|1y.4 20 10 10|01213|-2xomp.U 1qOMp.U 7zX0 1djf0|50e4", "Atlantic/Faroe|LMT WET WEST|r.4 0 -10|01212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-2uSnw.U 2Wgow.U 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|49e3", "Atlantic/Madeira|FMT MADT MADST MADMT WET WEST|17.A 10 0 -10 0 -10|01212121212121212121212121212121212121212121232123212321232121212121212121212121212121212121212121454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454|-2ldWQ.o aPWQ.o Sp0 LX0 1vc0 Tc0 1uM0 SM0 1vc0 Tc0 1vc0 SM0 1vc0 6600 1co0 3E00 17c0 1fA0 1a00 1io0 1a00 1io0 17c0 3I00 17c0 1cM0 1cM0 3Fc0 1cM0 1a00 1fA0 1io0 17c0 1cM0 1cM0 1a00 1fA0 1io0 1qM0 Dc0 1tA0 1cM0 1dc0 1400 gL0 IM0 s10 U00 dX0 Rc0 pd0 Rc0 gL0 Oo0 pd0 Rc0 gL0 Oo0 pd0 14o0 1cM0 1cP0 1cM0 1cM0 1cM0 1cM0 1cM0 3Co0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 qIl0 1cM0 1fA0 1cM0 1cM0 1cN0 1cL0 1cN0 1cM0 1cM0 1cM0 1cM0 1cN0 1cL0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|27e4", "Atlantic/Reykjavik|LMT IST ISST GMT|1s 10 0 0|012121212121212121212121212121212121212121212121212121212121212121213|-2uWmw mfaw 1Bd0 ML0 1LB0 Cn0 1LB0 3fX0 C10 HrX0 1cO0 LB0 1EL0 LA0 1C00 Oo0 1wo0 Rc0 1wo0 Rc0 1wo0 Rc0 1zc0 Oo0 1zc0 14o0 1lc0 14o0 1lc0 14o0 1o00 11A0 1lc0 14o0 1o00 14o0 1lc0 14o0 1lc0 14o0 1lc0 14o0 1lc0 14o0 1o00 14o0 1lc0 14o0 1lc0 14o0 1lc0 14o0 1lc0 14o0 1lc0 14o0 1o00 14o0 1lc0 14o0 1lc0 14o0 1lc0 14o0 1lc0 14o0 1o00 14o0|12e4", "Atlantic/South_Georgia|GST|20|0||30", "Atlantic/Stanley|SMT FKT FKST FKT FKST|3P.o 40 30 30 20|0121212121212134343212121212121212121212121212121212121212121212121212|-2kJw8.A 12bA8.A 19X0 1fB0 19X0 1ip0 19X0 1fB0 19X0 1fB0 19X0 1fB0 Cn0 1Cc10 WL0 1qL0 U10 1tz0 U10 1qM0 WN0 1qL0 WN0 1qL0 WN0 1qL0 WN0 1tz0 U10 1tz0 WN0 1qL0 WN0 1qL0 WN0 1qL0 WN0 1qL0 WN0 1tz0 WN0 1qL0 WN0 1qL0 WN0 1qL0 WN0 1qL0 WN0 1qN0 U10 1wn0 Rd0 1wn0 U10 1tz0 U10 1tz0 U10 1tz0 U10 1tz0 U10 1wn0 U10 1tz0 U10 1tz0 U10|21e2", "Australia/Sydney|AEST AEDT|-a0 -b0|0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101|-293lX xcX 10jd0 yL0 1cN0 1cL0 1fB0 19X0 17c10 LA0 1C00 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 14o0 1o00 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 U00 1qM0 WM0 1tA0 WM0 1tA0 U00 1tA0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 11A0 1o00 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 11A0 1o00 WM0 1qM0 14o0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0|40e5", "Australia/Adelaide|ACST ACDT|-9u -au|0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101|-293lt xcX 10jd0 yL0 1cN0 1cL0 1fB0 19X0 17c10 LA0 1C00 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 U00 1qM0 WM0 1tA0 WM0 1tA0 U00 1tA0 U00 1tA0 Oo0 1zc0 WM0 1qM0 Rc0 1zc0 U00 1tA0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 11A0 1o00 WM0 1qM0 14o0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0|11e5", "Australia/Brisbane|AEST AEDT|-a0 -b0|01010101010101010|-293lX xcX 10jd0 yL0 1cN0 1cL0 1fB0 19X0 17c10 LA0 H1A0 Oo0 1zc0 Oo0 1zc0 Oo0|20e5", "Australia/Broken_Hill|ACST ACDT|-9u -au|0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101|-293lt xcX 10jd0 yL0 1cN0 1cL0 1fB0 19X0 17c10 LA0 1C00 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 14o0 1o00 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 U00 1qM0 WM0 1tA0 WM0 1tA0 U00 1tA0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 11A0 1o00 WM0 1qM0 14o0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0|18e3", "Australia/Currie|AEST AEDT|-a0 -b0|0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101|-29E80 19X0 10jd0 yL0 1cN0 1cL0 1fB0 19X0 17c10 LA0 1C00 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 11A0 1qM0 WM0 1qM0 Oo0 1zc0 Oo0 1zc0 Oo0 1wo0 WM0 1tA0 WM0 1tA0 U00 1tA0 U00 1tA0 11A0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 11A0 1o00 1io0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1cM0 1a00 1io0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0|746", "Australia/Darwin|ACST ACDT|-9u -au|010101010|-293lt xcX 10jd0 yL0 1cN0 1cL0 1fB0 19X0|12e4", "Australia/Eucla|ACWST ACWDT|-8J -9J|0101010101010101010|-293kI xcX 10jd0 yL0 1cN0 1cL0 1gSp0 Oo0 l5A0 Oo0 iJA0 G00 zU00 IM0 1qM0 11A0 1o00 11A0|368", "Australia/Hobart|AEST AEDT|-a0 -b0|010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101|-29E80 19X0 10jd0 yL0 1cN0 1cL0 1fB0 19X0 VfB0 1cM0 1o00 Rc0 1wo0 Rc0 1wo0 U00 1wo0 LA0 1C00 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 11A0 1qM0 WM0 1qM0 Oo0 1zc0 Oo0 1zc0 Oo0 1wo0 WM0 1tA0 WM0 1tA0 U00 1tA0 U00 1tA0 11A0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 11A0 1o00 1io0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1cM0 1a00 1io0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0|21e4", "Australia/Lord_Howe|AEST LHST LHDT LHDT|-a0 -au -bu -b0|0121212121313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313131313|raC0 1zdu Rb0 1zd0 On0 1zd0 On0 1zd0 On0 1zd0 TXu 1qMu WLu 1tAu WLu 1tAu TXu 1tAu Onu 1zcu Onu 1zcu Onu 1zcu Rbu 1zcu Onu 1zcu Onu 1zcu 11zu 1o0u 11zu 1o0u 11zu 1o0u 11zu 1qMu WLu 11Au 1nXu 1qMu 11zu 1o0u 11zu 1o0u 11zu 1qMu WLu 1qMu 11zu 1o0u WLu 1qMu 14nu 1cMu 1cLu 1cMu 1cLu 1cMu 1cLu 1cMu 1cLu 1fAu 1cLu 1cMu 1cLu 1cMu 1cLu 1cMu 1cLu 1cMu 1cLu 1cMu 1cLu 1fAu 1cLu 1cMu 1cLu 1cMu 1cLu 1cMu 1cLu 1cMu 1cLu 1cMu 1fzu 1cMu 1cLu 1cMu 1cLu 1cMu 1cLu 1cMu 1cLu 1cMu 1cLu 1fAu 1cLu 1cMu 1cLu 1cMu 1cLu 1cMu 1cLu 1cMu 1cLu 1cMu 1cLu 1fAu 1cLu 1cMu 1cLu 1cMu|347", "Australia/Lindeman|AEST AEDT|-a0 -b0|010101010101010101010|-293lX xcX 10jd0 yL0 1cN0 1cL0 1fB0 19X0 17c10 LA0 H1A0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0|10", "Australia/Melbourne|AEST AEDT|-a0 -b0|0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101|-293lX xcX 10jd0 yL0 1cN0 1cL0 1fB0 19X0 17c10 LA0 1C00 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 U00 1qM0 WM0 1qM0 11A0 1tA0 U00 1tA0 U00 1tA0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 11A0 1o00 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 11A0 1o00 WM0 1qM0 14o0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0|39e5", "Australia/Perth|AWST AWDT|-80 -90|0101010101010101010|-293jX xcX 10jd0 yL0 1cN0 1cL0 1gSp0 Oo0 l5A0 Oo0 iJA0 G00 zU00 IM0 1qM0 11A0 1o00 11A0|18e5", "CET|CET CEST|-10 -20|01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-2aFe0 11d0 1iO0 11A0 1o00 11A0 Qrc0 6i00 WM0 1fA0 1cM0 1cM0 1cM0 16M0 1gMM0 1a00 1fA0 1cM0 1cM0 1cM0 1fA0 1a00 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00", "CST6CDT|CST CDT CWT CPT|60 50 50 50|010102301010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-261s0 1nX0 11B0 1nX0 SgN0 8x30 iw0 QwN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0", "Pacific/Easter|EMT EAST EASST EAST EASST|7h.s 70 60 60 50|0121212121212121212121212121234343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434|-1uSgG.w 1s4IG.w WL0 1zd0 On0 1ip0 11z0 1o10 11z0 1qN0 WL0 1ld0 14n0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 WL0 1qN0 1cL0 1cN0 11z0 1o10 11z0 1qN0 WL0 1fB0 19X0 1qN0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 17b0 1ip0 11z0 1ip0 1fz0 1fB0 11z0 1qN0 WL0 1qN0 WL0 1qN0 WL0 1qN0 11z0 1o10 11z0 1o10 11z0 1qN0 WL0 1qN0 17b0 1ip0 11z0 1o10 19X0 1fB0 1nX0 G10 1EL0 Op0 1zb0 Rd0 1wn0 Rd0 46n0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Dd0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Dd0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Dd0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0 1Nb0 Ap0|30e2", "EET|EET EEST|-20 -30|010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|hDB0 1a00 1fA0 1cM0 1cM0 1cM0 1fA0 1a00 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00", "EST|EST|50|0|", "EST5EDT|EST EDT EWT EPT|50 40 40 40|010102301010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-261t0 1nX0 11B0 1nX0 SgN0 8x40 iv0 QwN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0", "Europe/Dublin|DMT IST GMT BST IST|p.l -y.D 0 -10 -10|01232323232324242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242424242|-2ax9y.D Rc0 1fzy.D 14M0 1fc0 1g00 1co0 1dc0 1co0 1oo0 1400 1dc0 19A0 1io0 1io0 WM0 1o00 14o0 1o00 17c0 1io0 17c0 1fA0 1a00 1lc0 17c0 1io0 17c0 1fA0 1a00 1io0 17c0 1io0 17c0 1fA0 1cM0 1io0 17c0 1fA0 1a00 1io0 17c0 1io0 17c0 1fA0 1a00 1io0 1qM0 Dc0 g5X0 14p0 1wn0 17d0 1io0 11A0 1o00 17c0 1fA0 1a00 1fA0 1cM0 1fA0 1a00 17c0 1fA0 1a00 1io0 17c0 1lc0 17c0 1fA0 1a00 1io0 17c0 1io0 17c0 1fA0 1a00 1a00 1qM0 WM0 1qM0 11A0 1o00 WM0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1tA0 IM0 90o0 U00 1tA0 U00 1tA0 U00 1tA0 U00 1tA0 WM0 1qM0 WM0 1qM0 WM0 1tA0 U00 1tA0 U00 1tA0 11z0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1o00 14o0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|12e5", "Etc/GMT+0|GMT|0|0|", "Etc/GMT+1|-01|10|0|", "Etc/GMT+10|-10|a0|0|", "Etc/GMT+11|-11|b0|0|", "Etc/GMT+12|-12|c0|0|", "Etc/GMT+2|-02|20|0|", "Etc/GMT+3|-03|30|0|", "Etc/GMT+4|-04|40|0|", "Etc/GMT+5|-05|50|0|", "Etc/GMT+6|-06|60|0|", "Etc/GMT+7|-07|70|0|", "Etc/GMT+8|-08|80|0|", "Etc/GMT+9|-09|90|0|", "Etc/GMT-1|+01|-10|0|", "Etc/GMT-10|+10|-a0|0|", "Etc/GMT-11|+11|-b0|0|", "Etc/GMT-12|+12|-c0|0|", "Etc/GMT-13|+13|-d0|0|", "Etc/GMT-14|+14|-e0|0|", "Etc/GMT-2|+02|-20|0|", "Etc/GMT-3|+03|-30|0|", "Etc/GMT-4|+04|-40|0|", "Etc/GMT-5|+05|-50|0|", "Etc/GMT-6|+06|-60|0|", "Etc/GMT-7|+07|-70|0|", "Etc/GMT-8|+08|-80|0|", "Etc/GMT-9|+09|-90|0|", "Etc/UCT|UCT|0|0|", "Etc/UTC|UTC|0|0|", "Europe/Amsterdam|AMT NST NEST NET CEST CET|-j.w -1j.w -1k -k -20 -10|010101010101010101010101010101010101010101012323234545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545|-2aFcj.w 11b0 1iP0 11A0 1io0 1cM0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1co0 1io0 1yo0 Pc0 1a00 1fA0 1Bc0 Mo0 1tc0 Uo0 1tA0 U00 1uo0 W00 1s00 VA0 1so0 Vc0 1sM0 UM0 1wo0 Rc0 1u00 Wo0 1rA0 W00 1s00 VA0 1sM0 UM0 1w00 fV0 BCX.w 1tA0 U00 1u00 Wo0 1sm0 601k WM0 1fA0 1cM0 1cM0 1cM0 16M0 1gMM0 1a00 1fA0 1cM0 1cM0 1cM0 1fA0 1a00 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|16e5", "Europe/Andorra|WET CET CEST|0 -10 -20|012121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-UBA0 1xIN0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|79e3", "Europe/Astrakhan|LMT +03 +04 +05|-3c.c -30 -40 -50|012323232323232323212121212121212121212121212121212121212121212|-1Pcrc.c eUMc.c 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 2pB0 1cM0 1fA0 1cM0 3Co0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0 3rd0", "Europe/Athens|AMT EET EEST CEST CET|-1y.Q -20 -30 -20 -10|012123434121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-2a61x.Q CNbx.Q mn0 kU10 9b0 3Es0 Xa0 1fb0 1dd0 k3X0 Nz0 SCp0 1vc0 SO0 1cM0 1a00 1ao0 1fc0 1a10 1fG0 1cg0 1dX0 1bX0 1cQ0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|35e5", "Europe/London|GMT BST BDST|0 -10 -20|0101010101010101010101010101010101010101010101010121212121210101210101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-2axa0 Rc0 1fA0 14M0 1fc0 1g00 1co0 1dc0 1co0 1oo0 1400 1dc0 19A0 1io0 1io0 WM0 1o00 14o0 1o00 17c0 1io0 17c0 1fA0 1a00 1lc0 17c0 1io0 17c0 1fA0 1a00 1io0 17c0 1io0 17c0 1fA0 1cM0 1io0 17c0 1fA0 1a00 1io0 17c0 1io0 17c0 1fA0 1a00 1io0 1qM0 Dc0 2Rz0 Dc0 1zc0 Oo0 1zc0 Rc0 1wo0 17c0 1iM0 FA0 xB0 1fA0 1a00 14o0 bb0 LA0 xB0 Rc0 1wo0 11A0 1o00 17c0 1fA0 1a00 1fA0 1cM0 1fA0 1a00 17c0 1fA0 1a00 1io0 17c0 1lc0 17c0 1fA0 1a00 1io0 17c0 1io0 17c0 1fA0 1a00 1a00 1qM0 WM0 1qM0 11A0 1o00 WM0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1tA0 IM0 90o0 U00 1tA0 U00 1tA0 U00 1tA0 U00 1tA0 WM0 1qM0 WM0 1qM0 WM0 1tA0 U00 1tA0 U00 1tA0 11z0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1o00 14o0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|10e6", "Europe/Belgrade|CET CEST|-10 -20|01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-19RC0 3IP0 WM0 1fA0 1cM0 1cM0 1rc0 Qo0 1vmo0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|12e5", "Europe/Berlin|CET CEST CEMT|-10 -20 -30|01010101010101210101210101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-2aFe0 11d0 1iO0 11A0 1o00 11A0 Qrc0 6i00 WM0 1fA0 1cM0 1cM0 1cM0 kL0 Nc0 m10 WM0 1ao0 1cp0 dX0 jz0 Dd0 1io0 17c0 1fA0 1a00 1ehA0 1a00 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|41e5", "Europe/Prague|CET CEST|-10 -20|010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-2aFe0 11d0 1iO0 11A0 1o00 11A0 Qrc0 6i00 WM0 1fA0 1cM0 16M0 1lc0 1tA0 17A0 11c0 1io0 17c0 1io0 17c0 1fc0 1ao0 1bNc0 1cM0 1fA0 1a00 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|13e5", "Europe/Brussels|WET CET CEST WEST|0 -10 -20 -10|0121212103030303030303030303030303030303030303030303212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-2ehc0 3zX0 11c0 1iO0 11A0 1o00 11A0 my0 Ic0 1qM0 Rc0 1EM0 UM0 1u00 10o0 1io0 1io0 17c0 1a00 1fA0 1cM0 1cM0 1io0 17c0 1fA0 1a00 1io0 1a30 1io0 17c0 1fA0 1a00 1io0 17c0 1cM0 1cM0 1a00 1io0 1cM0 1cM0 1a00 1fA0 1io0 17c0 1cM0 1cM0 1a00 1fA0 1io0 1qM0 Dc0 y00 5Wn0 WM0 1fA0 1cM0 16M0 1iM0 16M0 1C00 Uo0 1eeo0 1a00 1fA0 1cM0 1cM0 1cM0 1fA0 1a00 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|21e5", "Europe/Bucharest|BMT EET EEST|-1I.o -20 -30|0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-1xApI.o 20LI.o RA0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1Axc0 On0 1fA0 1a10 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cK0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cL0 1cN0 1cL0 1fB0 1nX0 11E0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|19e5", "Europe/Budapest|CET CEST|-10 -20|0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-2aFe0 11d0 1iO0 11A0 1ip0 17b0 1op0 1tb0 Q2m0 3Ne0 WM0 1fA0 1cM0 1cM0 1oJ0 1dc0 1030 1fA0 1cM0 1cM0 1cM0 1cM0 1fA0 1a00 1iM0 1fA0 8Ha0 Rb0 1wN0 Rb0 1BB0 Lz0 1C20 LB0 SNX0 1a10 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|17e5", "Europe/Zurich|CET CEST|-10 -20|01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-19Lc0 11A0 1o00 11A0 1xG10 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|38e4", "Europe/Chisinau|CMT BMT EET EEST CEST CET MSK MSD|-1T -1I.o -20 -30 -20 -10 -30 -40|012323232323232323234545467676767676767676767323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232323232|-26jdT wGMa.A 20LI.o RA0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 27A0 2en0 39g0 WM0 1fA0 1cM0 V90 1t7z0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 gL0 WO0 1cM0 1cM0 1cK0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1fB0 1nX0 11D0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|67e4", "Europe/Copenhagen|CET CEST|-10 -20|0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-2azC0 Tz0 VuO0 60q0 WM0 1fA0 1cM0 1cM0 1cM0 S00 1HA0 Nc0 1C00 Dc0 1Nc0 Ao0 1h5A0 1a00 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|12e5", "Europe/Gibraltar|GMT BST BDST CET CEST|0 -10 -20 -10 -20|010101010101010101010101010101010101010101010101012121212121010121010101010101010101034343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343|-2axa0 Rc0 1fA0 14M0 1fc0 1g00 1co0 1dc0 1co0 1oo0 1400 1dc0 19A0 1io0 1io0 WM0 1o00 14o0 1o00 17c0 1io0 17c0 1fA0 1a00 1lc0 17c0 1io0 17c0 1fA0 1a00 1io0 17c0 1io0 17c0 1fA0 1cM0 1io0 17c0 1fA0 1a00 1io0 17c0 1io0 17c0 1fA0 1a00 1io0 1qM0 Dc0 2Rz0 Dc0 1zc0 Oo0 1zc0 Rc0 1wo0 17c0 1iM0 FA0 xB0 1fA0 1a00 14o0 bb0 LA0 xB0 Rc0 1wo0 11A0 1o00 17c0 1fA0 1a00 1fA0 1cM0 1fA0 1a00 17c0 1fA0 1a00 1io0 17c0 1lc0 17c0 1fA0 10Jz0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|30e3", "Europe/Helsinki|HMT EET EEST|-1D.N -20 -30|0121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-1WuND.N OULD.N 1dA0 1xGq0 1cM0 1cM0 1cM0 1cN0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|12e5", "Europe/Kaliningrad|CET CEST CET CEST MSK MSD EEST EET +03|-10 -20 -20 -30 -30 -40 -30 -20 -30|0101010101010232454545454545454546767676767676767676767676767676767676767676787|-2aFe0 11d0 1iO0 11A0 1o00 11A0 Qrc0 6i00 WM0 1fA0 1cM0 1cM0 Am0 Lb0 1en0 op0 1pNz0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cN0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0|44e4", "Europe/Kiev|KMT EET MSK CEST CET MSD EEST|-22.4 -20 -30 -20 -10 -40 -30|0123434252525252525252525256161616161616161616161616161616161616161616161616161616161616161616161616161616161616161616161|-1Pc22.4 eUo2.4 rnz0 2Hg0 WM0 1fA0 da0 1v4m0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 Db0 3220 1cK0 1cL0 1cN0 1cL0 1cN0 1cL0 1cQ0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|34e5", "Europe/Kirov|LMT +03 +04 +05|-3i.M -30 -40 -50|01232323232323232321212121212121212121212121212121212121212121|-22WM0 qH90 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 2pB0 1cM0 1fA0 1cM0 3Co0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0|48e4", "Europe/Lisbon|LMT WET WEST WEMT CET CEST|A.J 0 -10 -20 -10 -20|012121212121212121212121212121212121212121212321232123212321212121212121212121212121212121212121214121212121212121212121212121212124545454212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-2ldXn.f aPWn.f Sp0 LX0 1vc0 Tc0 1uM0 SM0 1vc0 Tc0 1vc0 SM0 1vc0 6600 1co0 3E00 17c0 1fA0 1a00 1io0 1a00 1io0 17c0 3I00 17c0 1cM0 1cM0 3Fc0 1cM0 1a00 1fA0 1io0 17c0 1cM0 1cM0 1a00 1fA0 1io0 1qM0 Dc0 1tA0 1cM0 1dc0 1400 gL0 IM0 s10 U00 dX0 Rc0 pd0 Rc0 gL0 Oo0 pd0 Rc0 gL0 Oo0 pd0 14o0 1cM0 1cP0 1cM0 1cM0 1cM0 1cM0 1cM0 3Co0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 pvy0 1cM0 1cM0 1fA0 1cM0 1cM0 1cN0 1cL0 1cN0 1cM0 1cM0 1cM0 1cM0 1cN0 1cL0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|27e5", "Europe/Luxembourg|LMT CET CEST WET WEST WEST WET|-o.A -10 -20 0 -10 -20 -10|0121212134343434343434343434343434343434343434343434565651212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-2DG0o.A t6mo.A TB0 1nX0 Up0 1o20 11A0 rW0 CM0 1qP0 R90 1EO0 UK0 1u20 10m0 1ip0 1in0 17e0 19W0 1fB0 1db0 1cp0 1in0 17d0 1fz0 1a10 1in0 1a10 1in0 17f0 1fA0 1a00 1io0 17c0 1cM0 1cM0 1a00 1io0 1cM0 1cM0 1a00 1fA0 1io0 17c0 1cM0 1cM0 1a00 1fA0 1io0 1qM0 Dc0 vA0 60L0 WM0 1fA0 1cM0 17c0 1io0 16M0 1C00 Uo0 1eeo0 1a00 1fA0 1cM0 1cM0 1cM0 1fA0 1a00 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|54e4", "Europe/Madrid|WET WEST WEMT CET CEST|0 -10 -20 -10 -20|01010101010101010101010121212121234343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343|-28dd0 11A0 1go0 19A0 1co0 1dA0 b1A0 18o0 3I00 17c0 1fA0 1a00 1io0 1a00 1io0 17c0 iyo0 Rc0 18o0 1hc0 1io0 1a00 14o0 5aL0 MM0 1vc0 17A0 1i00 1bc0 1eo0 17d0 1in0 17A0 6hA0 10N0 XIL0 1a10 1in0 17d0 19X0 1cN0 1fz0 1a10 1fX0 1cp0 1cO0 1cM0 1fA0 1a00 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|62e5", "Europe/Malta|CET CEST|-10 -20|0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-2arB0 Lz0 1cN0 1db0 1410 1on0 Wp0 1qL0 17d0 1cL0 M3B0 5M20 WM0 1fA0 1co0 17c0 1iM0 16m0 1de0 1lc0 14m0 1lc0 WO0 1qM0 GTW0 On0 1C10 LA0 1C00 LA0 1EM0 LA0 1C00 LA0 1zc0 Oo0 1C00 Oo0 1co0 1cM0 1lA0 Xc0 1qq0 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1o10 11z0 1iN0 19z0 1fB0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|42e4", "Europe/Minsk|MMT EET MSK CEST CET MSD EEST +03|-1O -20 -30 -20 -10 -40 -30 -30|01234343252525252525252525261616161616161616161616161616161616161617|-1Pc1O eUnO qNX0 3gQ0 WM0 1fA0 1cM0 Al0 1tsn0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 3Fc0 1cN0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0|19e5", "Europe/Monaco|PMT WET WEST WEMT CET CEST|-9.l 0 -10 -20 -10 -20|01212121212121212121212121212121212121212121212121232323232345454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454|-2nco9.l cNb9.l HA0 19A0 1iM0 11c0 1oo0 Wo0 1rc0 QM0 1EM0 UM0 1u00 10o0 1io0 1wo0 Rc0 1a00 1fA0 1cM0 1cM0 1io0 17c0 1fA0 1a00 1io0 1a00 1io0 17c0 1fA0 1a00 1io0 17c0 1cM0 1cM0 1a00 1io0 1cM0 1cM0 1a00 1fA0 1io0 17c0 1cM0 1cM0 1a00 1fA0 1io0 1qM0 Df0 2RV0 11z0 11B0 1ze0 WM0 1fA0 1cM0 1fa0 1aq0 16M0 1ekn0 1cL0 1fC0 1a00 1fA0 1cM0 1cM0 1cM0 1fA0 1a00 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|38e3", "Europe/Moscow|MMT MMT MST MDST MSD MSK +05 EET EEST MSK|-2u.h -2v.j -3v.j -4v.j -40 -30 -50 -20 -30 -40|012132345464575454545454545454545458754545454545454545454545454545454545454595|-2ag2u.h 2pyW.W 1bA0 11X0 GN0 1Hb0 c4v.j ik0 3DA0 dz0 15A0 c10 2q10 iM10 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cN0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0|16e6", "Europe/Paris|PMT WET WEST CEST CET WEMT|-9.l 0 -10 -20 -10 -20|0121212121212121212121212121212121212121212121212123434352543434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434343434|-2nco8.l cNb8.l HA0 19A0 1iM0 11c0 1oo0 Wo0 1rc0 QM0 1EM0 UM0 1u00 10o0 1io0 1wo0 Rc0 1a00 1fA0 1cM0 1cM0 1io0 17c0 1fA0 1a00 1io0 1a00 1io0 17c0 1fA0 1a00 1io0 17c0 1cM0 1cM0 1a00 1io0 1cM0 1cM0 1a00 1fA0 1io0 17c0 1cM0 1cM0 1a00 1fA0 1io0 1qM0 Df0 Ik0 5M30 WM0 1fA0 1cM0 Vx0 hB0 1aq0 16M0 1ekn0 1cL0 1fC0 1a00 1fA0 1cM0 1cM0 1cM0 1fA0 1a00 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|11e6", "Europe/Riga|RMT LST EET MSK CEST CET MSD EEST|-1A.y -2A.y -20 -30 -20 -10 -40 -30|010102345454536363636363636363727272727272727272727272727272727272727272727272727272727272727272727272727272727272727272727272|-25TzA.y 11A0 1iM0 ko0 gWm0 yDXA.y 2bX0 3fE0 WM0 1fA0 1cM0 1cM0 4m0 1sLy0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cN0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cN0 1o00 11A0 1o00 11A0 1qM0 3oo0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|64e4", "Europe/Rome|CET CEST|-10 -20|0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-2arB0 Lz0 1cN0 1db0 1410 1on0 Wp0 1qL0 17d0 1cL0 M3B0 5M20 WM0 1fA0 1cM0 16M0 1iM0 16m0 1de0 1lc0 14m0 1lc0 WO0 1qM0 GTW0 On0 1C10 LA0 1C00 LA0 1EM0 LA0 1C00 LA0 1zc0 Oo0 1C00 Oo0 1C00 LA0 1zc0 Oo0 1C00 LA0 1C00 LA0 1zc0 Oo0 1C00 Oo0 1zc0 Oo0 1fC0 1a00 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|39e5", "Europe/Samara|LMT +03 +04 +05|-3k.k -30 -40 -50|0123232323232323232121232323232323232323232323232323232323212|-22WM0 qH90 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 2pB0 1cM0 1fA0 2y10 14m0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 2sp0 WM0|12e5", "Europe/Saratov|LMT +03 +04 +05|-34.i -30 -40 -50|012323232323232321212121212121212121212121212121212121212121212|-22WM0 qH90 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 2pB0 1cM0 1cM0 1cM0 1fA0 1cM0 3Co0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0 5810", "Europe/Simferopol|SMT EET MSK CEST CET MSD EEST MSK|-2g -20 -30 -20 -10 -40 -30 -40|012343432525252525252525252161616525252616161616161616161616161616161616172|-1Pc2g eUog rEn0 2qs0 WM0 1fA0 1cM0 3V0 1u0L0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1Q00 4eL0 1cL0 1cN0 1cL0 1cN0 dX0 WL0 1cN0 1cL0 1fB0 1o30 11B0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11z0 1nW0|33e4", "Europe/Sofia|EET CET CEST EEST|-20 -10 -20 -30|01212103030303030303030303030303030303030303030303030303030303030303030303030303030303030303030303030303030303030303030303030|-168L0 WM0 1fA0 1cM0 1cM0 1cN0 1mKH0 1dd0 1fb0 1ap0 1fb0 1a20 1fy0 1a30 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cK0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1fB0 1nX0 11E0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|12e5", "Europe/Stockholm|CET CEST|-10 -20|01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-2azC0 TB0 2yDe0 1a00 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|15e5", "Europe/Tallinn|TMT CET CEST EET MSK MSD EEST|-1D -10 -20 -20 -30 -40 -30|012103421212454545454545454546363636363636363636363636363636363636363636363636363636363636363636363636363636363636363636363|-26oND teD 11A0 1Ta0 4rXl KSLD 2FX0 2Jg0 WM0 1fA0 1cM0 18J0 1sTX0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cN0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o10 11A0 1qM0 5QM0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|41e4", "Europe/Tirane|LMT CET CEST|-1j.k -10 -20|01212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-2glBj.k 14pcj.k 5LC0 WM0 4M0 1fCK0 10n0 1op0 11z0 1pd0 11z0 1qN0 WL0 1qp0 Xb0 1qp0 Xb0 1qp0 11z0 1lB0 11z0 1qN0 11z0 1iN0 16n0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|42e4", "Europe/Ulyanovsk|LMT +03 +04 +05 +02|-3d.A -30 -40 -50 -20|01232323232323232321214121212121212121212121212121212121212121212|-22WM0 qH90 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 2pB0 1cM0 1fA0 2pB0 IM0 rX0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0 3rd0", "Europe/Uzhgorod|CET CEST MSK MSD EET EEST|-10 -20 -30 -40 -20 -30|010101023232323232323232320454545454545454545454545454545454545454545454545454545454545454545454545454545454545454545454|-1cqL0 6i00 WM0 1fA0 1cM0 1ml0 1Cp0 1r3W0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1Q00 1Nf0 2pw0 1cL0 1cN0 1cL0 1cN0 1cL0 1cQ0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|11e4", "Europe/Vienna|CET CEST|-10 -20|0101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-2aFe0 11d0 1iO0 11A0 1o00 11A0 3KM0 14o0 LA00 6i00 WM0 1fA0 1cM0 1cM0 1cM0 400 2qM0 1a00 1cM0 1cM0 1io0 17c0 1gHa0 19X0 1cP0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|18e5", "Europe/Vilnius|WMT KMT CET EET MSK CEST MSD EEST|-1o -1z.A -10 -20 -30 -20 -40 -30|012324525254646464646464646473737373737373737352537373737373737373737373737373737373737373737373737373737373737373737373|-293do 6ILM.o 1Ooz.A zz0 Mfd0 29W0 3is0 WM0 1fA0 1cM0 LV0 1tgL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cN0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11B0 1o00 11A0 1qM0 8io0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|54e4", "Europe/Volgograd|LMT +03 +04 +05|-2V.E -30 -40 -50|01232323232323232121212121212121212121212121212121212121212121|-21IqV.E psLV.E 23CL0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 2pB0 1cM0 1cM0 1cM0 1fA0 1cM0 3Co0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 8Hz0|10e5", "Europe/Warsaw|WMT CET CEST EET EEST|-1o -10 -20 -20 -30|012121234312121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121|-2ctdo 1LXo 11d0 1iO0 11A0 1o00 11A0 1on0 11A0 6zy0 HWP0 5IM0 WM0 1fA0 1cM0 1dz0 1mL0 1en0 15B0 1aq0 1nA0 11A0 1io0 17c0 1fA0 1a00 iDX0 LA0 1cM0 1cM0 1C00 Oo0 1cM0 1cM0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1C00 LA0 uso0 1a00 1fA0 1cM0 1cM0 1cM0 1fA0 1a00 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cN0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|17e5", "Europe/Zaporozhye|CUT EET MSK CEST CET MSD EEST|-2k -20 -30 -20 -10 -40 -30|01234342525252525252525252526161616161616161616161616161616161616161616161616161616161616161616161616161616161616161616161|-1Pc2k eUok rdb0 2RE0 WM0 1fA0 8m0 1v9a0 1db0 1cN0 1db0 1cN0 1db0 1dd0 1cO0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cK0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cQ0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00|77e4", "HST|HST|a0|0|", "Indian/Chagos|LMT IOT IOT|-4N.E -50 -60|012|-2xosN.E 3AGLN.E|30e2", "Indian/Christmas|CXT|-70|0||21e2", "Indian/Cocos|CCT|-6u|0||596", "Indian/Kerguelen|-00 +05|0 -50|01|-MG00|130", "Indian/Mahe|LMT SCT|-3F.M -40|01|-2yO3F.M|79e3", "Indian/Maldives|MMT MVT|-4S -50|01|-olgS|35e4", "Indian/Mauritius|LMT MUT MUST|-3O -40 -50|012121|-2xorO 34unO 14L0 12kr0 11z0|15e4", "Indian/Reunion|LMT RET|-3F.Q -40|01|-2mDDF.Q|84e4", "Pacific/Kwajalein|MHT KWAT MHT|-b0 c0 -c0|012|-AX0 W9X0|14e3", "MET|MET MEST|-10 -20|01010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-2aFe0 11d0 1iO0 11A0 1o00 11A0 Qrc0 6i00 WM0 1fA0 1cM0 1cM0 1cM0 16M0 1gMM0 1a00 1fA0 1cM0 1cM0 1cM0 1fA0 1a00 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00", "MST|MST|70|0|", "MST7MDT|MST MDT MWT MPT|70 60 60 60|010102301010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-261r0 1nX0 11B0 1nX0 SgN0 8x20 ix0 QwN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0", "Pacific/Chatham|CHAST CHAST CHADT|-cf -cJ -dJ|012121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212121212|-WqAf 1adef IM0 1C00 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Oo0 1zc0 Rc0 1zc0 Oo0 1qM0 14o0 1lc0 14o0 1lc0 14o0 1lc0 17c0 1io0 17c0 1io0 17c0 1io0 17c0 1lc0 14o0 1lc0 14o0 1lc0 17c0 1io0 17c0 1io0 17c0 1lc0 14o0 1lc0 14o0 1lc0 17c0 1io0 17c0 1io0 17c0 1io0 17c0 1io0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1io0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1fA0 1a00 1fA0 1a00|600", "PST8PDT|PST PDT PWT PPT|80 70 70 70|010102301010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|-261q0 1nX0 11B0 1nX0 SgN0 8x10 iy0 QwN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1cN0 1cL0 1cN0 1cL0 s10 1Vz0 LB0 1BX0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1fz0 1a10 1fz0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 14p0 1lb0 14p0 1lb0 14p0 1nX0 11B0 1nX0 11B0 1nX0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Rd0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0 Op0 1zb0", "Pacific/Apia|LMT WSST SST SDT WSDT WSST|bq.U bu b0 a0 -e0 -d0|01232345454545454545454545454545454545454545454545454545454|-2nDMx.4 1yW03.4 2rRbu 1ff0 1a00 CI0 AQ0 1cM0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1io0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1a00 1fA0 1cM0 1fA0 1a00 1fA0 1a00|37e3", "Pacific/Bougainville|PGT JST BST|-a0 -90 -b0|0102|-16Wy0 7CN0 2MQp0|18e4", "Pacific/Chuuk|CHUT|-a0|0||49e3", "Pacific/Efate|LMT VUT VUST|-bd.g -b0 -c0|0121212121212121212121|-2l9nd.g 2Szcd.g 1cL0 1oN0 10L0 1fB0 19X0 1fB0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1cN0 1cL0 1fB0 Lz0 1Nd0 An0|66e3", "Pacific/Enderbury|PHOT PHOT PHOT|c0 b0 -d0|012|nIc0 B8n0|1", "Pacific/Fakaofo|TKT TKT|b0 -d0|01|1Gfn0|483", "Pacific/Fiji|LMT FJT FJST|-bT.I -c0 -d0|0121212121212121212121212121212121212121212121212121212121212121|-2bUzT.I 3m8NT.I LA0 1EM0 IM0 nJc0 LA0 1o00 Rc0 1wo0 Ao0 1Nc0 Ao0 1Q00 xz0 1SN0 uM0 1SM0 uM0 1VA0 s00 1VA0 uM0 1SM0 uM0 1SM0 uM0 1SM0 uM0 1VA0 s00 1VA0 s00 1VA0 uM0 1SM0 uM0 1SM0 uM0 1SM0 uM0 1VA0 s00 1VA0 uM0 1SM0 uM0 1SM0 uM0 1SM0 uM0 1VA0 s00 1VA0 s00 1VA0 uM0 1SM0 uM0 1SM0 uM0 1SM0 uM0|88e4", "Pacific/Funafuti|TVT|-c0|0||45e2", "Pacific/Galapagos|LMT ECT GALT|5W.o 50 60|012|-1yVS1.A 2dTz1.A|25e3", "Pacific/Gambier|LMT GAMT|8X.M 90|01|-2jof0.c|125", "Pacific/Guadalcanal|LMT SBT|-aD.M -b0|01|-2joyD.M|11e4", "Pacific/Guam|GST ChST|-a0 -a0|01|1fpq0|17e4", "Pacific/Honolulu|HST HDT HST|au 9u a0|010102|-1thLu 8x0 lef0 8Pz0 46p0|37e4", "Pacific/Kiritimati|LINT LINT LINT|aE a0 -e0|012|nIaE B8nk|51e2", "Pacific/Kosrae|KOST KOST|-b0 -c0|010|-AX0 1bdz0|66e2", "Pacific/Majuro|MHT MHT|-b0 -c0|01|-AX0|28e3", "Pacific/Marquesas|LMT MART|9i 9u|01|-2joeG|86e2", "Pacific/Pago_Pago|LMT NST BST SST|bm.M b0 b0 b0|0123|-2nDMB.c 2gVzB.c EyM0|37e2", "Pacific/Nauru|LMT NRT JST NRT|-b7.E -bu -90 -c0|01213|-1Xdn7.E PvzB.E 5RCu 1ouJu|10e3", "Pacific/Niue|NUT NUT NUT|bk bu b0|012|-KfME 17y0a|12e2", "Pacific/Norfolk|NMT NFT NFST NFT|-bc -bu -cu -b0|01213|-Kgbc W01G On0 1COp0|25e4", "Pacific/Noumea|LMT NCT NCST|-b5.M -b0 -c0|01212121|-2l9n5.M 2EqM5.M xX0 1PB0 yn0 HeP0 Ao0|98e3", "Pacific/Palau|PWT|-90|0||21e3", "Pacific/Pitcairn|PNT PST|8u 80|01|18Vku|56", "Pacific/Pohnpei|PONT|-b0|0||34e3", "Pacific/Port_Moresby|PGT|-a0|0||25e4", "Pacific/Rarotonga|CKT CKHST CKT|au 9u a0|012121212121212121212121212|lyWu IL0 1zcu Onu 1zcu Onu 1zcu Rbu 1zcu Onu 1zcu Onu 1zcu Onu 1zcu Onu 1zcu Onu 1zcu Rbu 1zcu Onu 1zcu Onu 1zcu Onu|13e3", "Pacific/Tahiti|LMT TAHT|9W.g a0|01|-2joe1.I|18e4", "Pacific/Tarawa|GILT|-c0|0||29e3", "Pacific/Tongatapu|+1220 +13 +14|-ck -d0 -e0|0121212121212121212121212121212121212121212121212121|-1aB0k 2n5dk 15A0 1wo0 xz0 1Q10 xz0 zWN0 s00 1VA0 uM0 1SM0 uM0 1SM0 uM0 1SM0 uM0 1VA0 s00 1VA0 s00 1VA0 uM0 1SM0 uM0 1SM0 uM0 1SM0 uM0 1VA0 s00 1VA0 uM0 1SM0 uM0 1SM0 uM0 1SM0 uM0 1VA0 s00 1VA0 s00 1VA0 uM0 1SM0 uM0 1SM0 uM0 1SM0 uM0|75e3", "Pacific/Wake|WAKT|-c0|0||16e3", "Pacific/Wallis|WFT|-c0|0||94", "WET|WET WEST|0 -10|010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010101010|hDB0 1a00 1fA0 1cM0 1cM0 1cM0 1fA0 1a00 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1cM0 1fA0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00 11A0 1qM0 WM0 1qM0 WM0 1qM0 WM0 1qM0 11A0 1o00 11A0 1o00"],
    links: ["Africa/Abidjan|Africa/Bamako", "Africa/Abidjan|Africa/Banjul", "Africa/Abidjan|Africa/Conakry", "Africa/Abidjan|Africa/Dakar", "Africa/Abidjan|Africa/Freetown", "Africa/Abidjan|Africa/Lome", "Africa/Abidjan|Africa/Nouakchott", "Africa/Abidjan|Africa/Ouagadougou", "Africa/Abidjan|Africa/Sao_Tome", "Africa/Abidjan|Africa/Timbuktu", "Africa/Abidjan|Atlantic/St_Helena", "Africa/Cairo|Egypt", "Africa/Johannesburg|Africa/Maseru", "Africa/Johannesburg|Africa/Mbabane", "Africa/Khartoum|Africa/Juba", "Africa/Lagos|Africa/Bangui", "Africa/Lagos|Africa/Brazzaville", "Africa/Lagos|Africa/Douala", "Africa/Lagos|Africa/Kinshasa", "Africa/Lagos|Africa/Libreville", "Africa/Lagos|Africa/Luanda", "Africa/Lagos|Africa/Malabo", "Africa/Lagos|Africa/Niamey", "Africa/Lagos|Africa/Porto-Novo", "Africa/Maputo|Africa/Blantyre", "Africa/Maputo|Africa/Bujumbura", "Africa/Maputo|Africa/Gaborone", "Africa/Maputo|Africa/Harare", "Africa/Maputo|Africa/Kigali", "Africa/Maputo|Africa/Lubumbashi", "Africa/Maputo|Africa/Lusaka", "Africa/Nairobi|Africa/Addis_Ababa", "Africa/Nairobi|Africa/Asmara", "Africa/Nairobi|Africa/Asmera", "Africa/Nairobi|Africa/Dar_es_Salaam", "Africa/Nairobi|Africa/Djibouti", "Africa/Nairobi|Africa/Kampala", "Africa/Nairobi|Africa/Mogadishu", "Africa/Nairobi|Indian/Antananarivo", "Africa/Nairobi|Indian/Comoro", "Africa/Nairobi|Indian/Mayotte", "Africa/Tripoli|Libya", "America/Adak|America/Atka", "America/Adak|US/Aleutian", "America/Anchorage|US/Alaska", "America/Argentina/Buenos_Aires|America/Buenos_Aires", "America/Argentina/Catamarca|America/Argentina/ComodRivadavia", "America/Argentina/Catamarca|America/Catamarca", "America/Argentina/Cordoba|America/Cordoba", "America/Argentina/Cordoba|America/Rosario", "America/Argentina/Jujuy|America/Jujuy", "America/Argentina/Mendoza|America/Mendoza", "America/Atikokan|America/Coral_Harbour", "America/Chicago|US/Central", "America/Curacao|America/Aruba", "America/Curacao|America/Kralendijk", "America/Curacao|America/Lower_Princes", "America/Denver|America/Shiprock", "America/Denver|Navajo", "America/Denver|US/Mountain", "America/Detroit|US/Michigan", "America/Edmonton|Canada/Mountain", "America/Fort_Wayne|America/Indiana/Indianapolis", "America/Fort_Wayne|America/Indianapolis", "America/Fort_Wayne|US/East-Indiana", "America/Halifax|Canada/Atlantic", "America/Havana|Cuba", "America/Indiana/Knox|America/Knox_IN", "America/Indiana/Knox|US/Indiana-Starke", "America/Jamaica|Jamaica", "America/Kentucky/Louisville|America/Louisville", "America/Los_Angeles|US/Pacific", "America/Los_Angeles|US/Pacific-New", "America/Manaus|Brazil/West", "America/Mazatlan|Mexico/BajaSur", "America/Mexico_City|Mexico/General", "America/New_York|US/Eastern", "America/Noronha|Brazil/DeNoronha", "America/Panama|America/Cayman", "America/Phoenix|US/Arizona", "America/Port_of_Spain|America/Anguilla", "America/Port_of_Spain|America/Antigua", "America/Port_of_Spain|America/Dominica", "America/Port_of_Spain|America/Grenada", "America/Port_of_Spain|America/Guadeloupe", "America/Port_of_Spain|America/Marigot", "America/Port_of_Spain|America/Montserrat", "America/Port_of_Spain|America/St_Barthelemy", "America/Port_of_Spain|America/St_Kitts", "America/Port_of_Spain|America/St_Lucia", "America/Port_of_Spain|America/St_Thomas", "America/Port_of_Spain|America/St_Vincent", "America/Port_of_Spain|America/Tortola", "America/Port_of_Spain|America/Virgin", "America/Regina|Canada/East-Saskatchewan", "America/Regina|Canada/Saskatchewan", "America/Rio_Branco|America/Porto_Acre", "America/Rio_Branco|Brazil/Acre", "America/Santiago|Chile/Continental", "America/Sao_Paulo|Brazil/East", "America/St_Johns|Canada/Newfoundland", "America/Tijuana|America/Ensenada", "America/Tijuana|America/Santa_Isabel", "America/Tijuana|Mexico/BajaNorte", "America/Toronto|America/Montreal", "America/Toronto|Canada/Eastern", "America/Vancouver|Canada/Pacific", "America/Whitehorse|Canada/Yukon", "America/Winnipeg|Canada/Central", "Asia/Ashgabat|Asia/Ashkhabad", "Asia/Bangkok|Asia/Phnom_Penh", "Asia/Bangkok|Asia/Vientiane", "Asia/Dhaka|Asia/Dacca", "Asia/Dubai|Asia/Muscat", "Asia/Ho_Chi_Minh|Asia/Saigon", "Asia/Hong_Kong|Hongkong", "Asia/Jerusalem|Asia/Tel_Aviv", "Asia/Jerusalem|Israel", "Asia/Kathmandu|Asia/Katmandu", "Asia/Kolkata|Asia/Calcutta", "Asia/Macau|Asia/Macao", "Asia/Makassar|Asia/Ujung_Pandang", "Asia/Nicosia|Europe/Nicosia", "Asia/Qatar|Asia/Bahrain", "Asia/Rangoon|Asia/Yangon", "Asia/Riyadh|Asia/Aden", "Asia/Riyadh|Asia/Kuwait", "Asia/Seoul|ROK", "Asia/Shanghai|Asia/Chongqing", "Asia/Shanghai|Asia/Chungking", "Asia/Shanghai|Asia/Harbin", "Asia/Shanghai|PRC", "Asia/Singapore|Singapore", "Asia/Taipei|ROC", "Asia/Tehran|Iran", "Asia/Thimphu|Asia/Thimbu", "Asia/Tokyo|Japan", "Asia/Ulaanbaatar|Asia/Ulan_Bator", "Asia/Urumqi|Asia/Kashgar", "Atlantic/Faroe|Atlantic/Faeroe", "Atlantic/Reykjavik|Iceland", "Australia/Adelaide|Australia/South", "Australia/Brisbane|Australia/Queensland", "Australia/Broken_Hill|Australia/Yancowinna", "Australia/Darwin|Australia/North", "Australia/Hobart|Australia/Tasmania", "Australia/Lord_Howe|Australia/LHI", "Australia/Melbourne|Australia/Victoria", "Australia/Perth|Australia/West", "Australia/Sydney|Australia/ACT", "Australia/Sydney|Australia/Canberra", "Australia/Sydney|Australia/NSW", "Etc/GMT+0|Etc/GMT", "Etc/GMT+0|Etc/GMT-0", "Etc/GMT+0|Etc/GMT0", "Etc/GMT+0|Etc/Greenwich", "Etc/GMT+0|GMT", "Etc/GMT+0|GMT+0", "Etc/GMT+0|GMT-0", "Etc/GMT+0|GMT0", "Etc/GMT+0|Greenwich", "Etc/UCT|UCT", "Etc/UTC|Etc/Universal", "Etc/UTC|Etc/Zulu", "Etc/UTC|UTC", "Etc/UTC|Universal", "Etc/UTC|Zulu", "Europe/Belgrade|Europe/Ljubljana", "Europe/Belgrade|Europe/Podgorica", "Europe/Belgrade|Europe/Sarajevo", "Europe/Belgrade|Europe/Skopje", "Europe/Belgrade|Europe/Zagreb", "Europe/Chisinau|Europe/Tiraspol", "Europe/Dublin|Eire", "Europe/Helsinki|Europe/Mariehamn", "Europe/Istanbul|Asia/Istanbul", "Europe/Istanbul|Turkey", "Europe/Lisbon|Portugal", "Europe/London|Europe/Belfast", "Europe/London|Europe/Guernsey", "Europe/London|Europe/Isle_of_Man", "Europe/London|Europe/Jersey", "Europe/London|GB", "Europe/London|GB-Eire", "Europe/Moscow|W-SU", "Europe/Oslo|Arctic/Longyearbyen", "Europe/Oslo|Atlantic/Jan_Mayen", "Europe/Prague|Europe/Bratislava", "Europe/Rome|Europe/San_Marino", "Europe/Rome|Europe/Vatican", "Europe/Warsaw|Poland", "Europe/Zurich|Europe/Busingen", "Europe/Zurich|Europe/Vaduz", "Pacific/Auckland|Antarctica/McMurdo", "Pacific/Auckland|Antarctica/South_Pole", "Pacific/Auckland|NZ", "Pacific/Chatham|NZ-CHAT", "Pacific/Chuuk|Pacific/Truk", "Pacific/Chuuk|Pacific/Yap", "Pacific/Easter|Chile/EasterIsland", "Pacific/Guam|Pacific/Saipan", "Pacific/Honolulu|Pacific/Johnston", "Pacific/Honolulu|US/Hawaii", "Pacific/Kwajalein|Kwajalein", "Pacific/Pago_Pago|Pacific/Midway", "Pacific/Pago_Pago|Pacific/Samoa", "Pacific/Pago_Pago|US/Samoa", "Pacific/Pohnpei|Pacific/Ponape"]
  }), a;
});

})();
System.registerDynamic('node_modules/rxjs/observable/BoundCallbackObservable.js', ['../Observable', '../util/tryCatch', '../util/errorObject', '../AsyncSubject'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Observable_1 = $__require('../Observable');
    var tryCatch_1 = $__require('../util/tryCatch');
    var errorObject_1 = $__require('../util/errorObject');
    var AsyncSubject_1 = $__require('../AsyncSubject');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var BoundCallbackObservable = function (_super) {
        __extends(BoundCallbackObservable, _super);
        function BoundCallbackObservable(callbackFunc, selector, args, scheduler) {
            _super.call(this);
            this.callbackFunc = callbackFunc;
            this.selector = selector;
            this.args = args;
            this.scheduler = scheduler;
        }
        /* tslint:enable:max-line-length */
        /**
         * Converts a callback function to an observable sequence.
         * @param {function} callbackFunc Function with a callback as the last
         * parameter.
         * @param {function} selector A selector which takes the arguments from the
         * callback to produce a single item to yield on next.
         * @param {Scheduler} [scheduler] The scheduler on which to schedule
         * the callbacks.
         * @return {function(...params: *): Observable<T>} a function which returns the
         * Observable that corresponds to the callback.
         * @static true
         * @name bindCallback
         * @owner Observable
         */
        BoundCallbackObservable.create = function (callbackFunc, selector, scheduler) {
            if (selector === void 0) {
                selector = undefined;
            }
            return function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i - 0] = arguments[_i];
                }
                return new BoundCallbackObservable(callbackFunc, selector, args, scheduler);
            };
        };
        BoundCallbackObservable.prototype._subscribe = function (subscriber) {
            var callbackFunc = this.callbackFunc;
            var args = this.args;
            var scheduler = this.scheduler;
            var subject = this.subject;
            if (!scheduler) {
                if (!subject) {
                    subject = this.subject = new AsyncSubject_1.AsyncSubject();
                    var handler = function handlerFn() {
                        var innerArgs = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            innerArgs[_i - 0] = arguments[_i];
                        }
                        var source = handlerFn.source;
                        var selector = source.selector,
                            subject = source.subject;
                        if (selector) {
                            var result_1 = tryCatch_1.tryCatch(selector).apply(this, innerArgs);
                            if (result_1 === errorObject_1.errorObject) {
                                subject.error(errorObject_1.errorObject.e);
                            } else {
                                subject.next(result_1);
                                subject.complete();
                            }
                        } else {
                            subject.next(innerArgs.length === 1 ? innerArgs[0] : innerArgs);
                            subject.complete();
                        }
                    };
                    // use named function instance to avoid closure.
                    handler.source = this;
                    var result = tryCatch_1.tryCatch(callbackFunc).apply(this, args.concat(handler));
                    if (result === errorObject_1.errorObject) {
                        subject.error(errorObject_1.errorObject.e);
                    }
                }
                return subject.subscribe(subscriber);
            } else {
                return scheduler.schedule(dispatch, 0, { source: this, subscriber: subscriber });
            }
        };
        return BoundCallbackObservable;
    }(Observable_1.Observable);
    exports.BoundCallbackObservable = BoundCallbackObservable;
    function dispatch(state) {
        var self = this;
        var source = state.source,
            subscriber = state.subscriber;
        var callbackFunc = source.callbackFunc,
            args = source.args,
            scheduler = source.scheduler;
        var subject = source.subject;
        if (!subject) {
            subject = source.subject = new AsyncSubject_1.AsyncSubject();
            var handler = function handlerFn() {
                var innerArgs = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    innerArgs[_i - 0] = arguments[_i];
                }
                var source = handlerFn.source;
                var selector = source.selector,
                    subject = source.subject;
                if (selector) {
                    var result_2 = tryCatch_1.tryCatch(selector).apply(this, innerArgs);
                    if (result_2 === errorObject_1.errorObject) {
                        self.add(scheduler.schedule(dispatchError, 0, { err: errorObject_1.errorObject.e, subject: subject }));
                    } else {
                        self.add(scheduler.schedule(dispatchNext, 0, { value: result_2, subject: subject }));
                    }
                } else {
                    var value = innerArgs.length === 1 ? innerArgs[0] : innerArgs;
                    self.add(scheduler.schedule(dispatchNext, 0, { value: value, subject: subject }));
                }
            };
            // use named function to pass values in without closure
            handler.source = source;
            var result = tryCatch_1.tryCatch(callbackFunc).apply(this, args.concat(handler));
            if (result === errorObject_1.errorObject) {
                subject.error(errorObject_1.errorObject.e);
            }
        }
        self.add(subject.subscribe(subscriber));
    }
    function dispatchNext(arg) {
        var value = arg.value,
            subject = arg.subject;
        subject.next(value);
        subject.complete();
    }
    function dispatchError(arg) {
        var err = arg.err,
            subject = arg.subject;
        subject.error(err);
    }
    
});
System.registerDynamic("node_modules/rxjs/observable/bindCallback.js", ["./BoundCallbackObservable"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var BoundCallbackObservable_1 = $__require('./BoundCallbackObservable');
  exports.bindCallback = BoundCallbackObservable_1.BoundCallbackObservable.create;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/bindCallback.js', ['../../Observable', '../../observable/bindCallback'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var bindCallback_1 = $__require('../../observable/bindCallback');
  Observable_1.Observable.bindCallback = bindCallback_1.bindCallback;
  
});
System.registerDynamic('node_modules/rxjs/observable/BoundNodeCallbackObservable.js', ['../Observable', '../util/tryCatch', '../util/errorObject', '../AsyncSubject'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Observable_1 = $__require('../Observable');
    var tryCatch_1 = $__require('../util/tryCatch');
    var errorObject_1 = $__require('../util/errorObject');
    var AsyncSubject_1 = $__require('../AsyncSubject');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var BoundNodeCallbackObservable = function (_super) {
        __extends(BoundNodeCallbackObservable, _super);
        function BoundNodeCallbackObservable(callbackFunc, selector, args, scheduler) {
            _super.call(this);
            this.callbackFunc = callbackFunc;
            this.selector = selector;
            this.args = args;
            this.scheduler = scheduler;
        }
        /* tslint:enable:max-line-length */
        /**
         * Converts a node callback to an Observable.
         * @param callbackFunc
         * @param selector
         * @param scheduler
         * @return {function(...params: *): Observable<T>}
         * @static true
         * @name bindNodeCallback
         * @owner Observable
         */
        BoundNodeCallbackObservable.create = function (callbackFunc, selector, scheduler) {
            if (selector === void 0) {
                selector = undefined;
            }
            return function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i - 0] = arguments[_i];
                }
                return new BoundNodeCallbackObservable(callbackFunc, selector, args, scheduler);
            };
        };
        BoundNodeCallbackObservable.prototype._subscribe = function (subscriber) {
            var callbackFunc = this.callbackFunc;
            var args = this.args;
            var scheduler = this.scheduler;
            var subject = this.subject;
            if (!scheduler) {
                if (!subject) {
                    subject = this.subject = new AsyncSubject_1.AsyncSubject();
                    var handler = function handlerFn() {
                        var innerArgs = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            innerArgs[_i - 0] = arguments[_i];
                        }
                        var source = handlerFn.source;
                        var selector = source.selector,
                            subject = source.subject;
                        var err = innerArgs.shift();
                        if (err) {
                            subject.error(err);
                        } else if (selector) {
                            var result_1 = tryCatch_1.tryCatch(selector).apply(this, innerArgs);
                            if (result_1 === errorObject_1.errorObject) {
                                subject.error(errorObject_1.errorObject.e);
                            } else {
                                subject.next(result_1);
                                subject.complete();
                            }
                        } else {
                            subject.next(innerArgs.length === 1 ? innerArgs[0] : innerArgs);
                            subject.complete();
                        }
                    };
                    // use named function instance to avoid closure.
                    handler.source = this;
                    var result = tryCatch_1.tryCatch(callbackFunc).apply(this, args.concat(handler));
                    if (result === errorObject_1.errorObject) {
                        subject.error(errorObject_1.errorObject.e);
                    }
                }
                return subject.subscribe(subscriber);
            } else {
                return scheduler.schedule(dispatch, 0, { source: this, subscriber: subscriber });
            }
        };
        return BoundNodeCallbackObservable;
    }(Observable_1.Observable);
    exports.BoundNodeCallbackObservable = BoundNodeCallbackObservable;
    function dispatch(state) {
        var self = this;
        var source = state.source,
            subscriber = state.subscriber;
        var callbackFunc = source.callbackFunc,
            args = source.args,
            scheduler = source.scheduler;
        var subject = source.subject;
        if (!subject) {
            subject = source.subject = new AsyncSubject_1.AsyncSubject();
            var handler = function handlerFn() {
                var innerArgs = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    innerArgs[_i - 0] = arguments[_i];
                }
                var source = handlerFn.source;
                var selector = source.selector,
                    subject = source.subject;
                var err = innerArgs.shift();
                if (err) {
                    subject.error(err);
                } else if (selector) {
                    var result_2 = tryCatch_1.tryCatch(selector).apply(this, innerArgs);
                    if (result_2 === errorObject_1.errorObject) {
                        self.add(scheduler.schedule(dispatchError, 0, { err: errorObject_1.errorObject.e, subject: subject }));
                    } else {
                        self.add(scheduler.schedule(dispatchNext, 0, { value: result_2, subject: subject }));
                    }
                } else {
                    var value = innerArgs.length === 1 ? innerArgs[0] : innerArgs;
                    self.add(scheduler.schedule(dispatchNext, 0, { value: value, subject: subject }));
                }
            };
            // use named function to pass values in without closure
            handler.source = source;
            var result = tryCatch_1.tryCatch(callbackFunc).apply(this, args.concat(handler));
            if (result === errorObject_1.errorObject) {
                subject.error(errorObject_1.errorObject.e);
            }
        }
        self.add(subject.subscribe(subscriber));
    }
    function dispatchNext(arg) {
        var value = arg.value,
            subject = arg.subject;
        subject.next(value);
        subject.complete();
    }
    function dispatchError(arg) {
        var err = arg.err,
            subject = arg.subject;
        subject.error(err);
    }
    
});
System.registerDynamic("node_modules/rxjs/observable/bindNodeCallback.js", ["./BoundNodeCallbackObservable"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var BoundNodeCallbackObservable_1 = $__require('./BoundNodeCallbackObservable');
  exports.bindNodeCallback = BoundNodeCallbackObservable_1.BoundNodeCallbackObservable.create;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/bindNodeCallback.js', ['../../Observable', '../../observable/bindNodeCallback'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var bindNodeCallback_1 = $__require('../../observable/bindNodeCallback');
  Observable_1.Observable.bindNodeCallback = bindNodeCallback_1.bindNodeCallback;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/combineLatest.js', ['../../Observable', '../../operator/combineLatest'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var combineLatest_1 = $__require('../../operator/combineLatest');
  Observable_1.Observable.combineLatest = combineLatest_1.combineLatestStatic;
  
});
System.registerDynamic("node_modules/rxjs/observable/concat.js", ["../operator/concat"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var concat_1 = $__require('../operator/concat');
  exports.concat = concat_1.concatStatic;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/concat.js', ['../../Observable', '../../observable/concat'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var concat_1 = $__require('../../observable/concat');
  Observable_1.Observable.concat = concat_1.concat;
  
});
System.registerDynamic('node_modules/rxjs/observable/DeferObservable.js', ['../Observable', '../util/subscribeToResult', '../OuterSubscriber'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Observable_1 = $__require('../Observable');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var DeferObservable = function (_super) {
        __extends(DeferObservable, _super);
        function DeferObservable(observableFactory) {
            _super.call(this);
            this.observableFactory = observableFactory;
        }
        /**
         * Creates an Observable that, on subscribe, calls an Observable factory to
         * make an Observable for each new Observer.
         *
         * <span class="informal">Creates the Observable lazily, that is, only when it
         * is subscribed.
         * </span>
         *
         * <img src="./img/defer.png" width="100%">
         *
         * `defer` allows you to create the Observable only when the Observer
         * subscribes, and create a fresh Observable for each Observer. It waits until
         * an Observer subscribes to it, and then it generates an Observable,
         * typically with an Observable factory function. It does this afresh for each
         * subscriber, so although each subscriber may think it is subscribing to the
         * same Observable, in fact each subscriber gets its own individual
         * Observable.
         *
         * @example <caption>Subscribe to either an Observable of clicks or an Observable of interval, at random</caption>
         * var clicksOrInterval = Rx.Observable.defer(function () {
         *   if (Math.random() > 0.5) {
         *     return Rx.Observable.fromEvent(document, 'click');
         *   } else {
         *     return Rx.Observable.interval(1000);
         *   }
         * });
         * clicksOrInterval.subscribe(x => console.log(x));
         *
         * @see {@link create}
         *
         * @param {function(): Observable|Promise} observableFactory The Observable
         * factory function to invoke for each Observer that subscribes to the output
         * Observable. May also return a Promise, which will be converted on the fly
         * to an Observable.
         * @return {Observable} An Observable whose Observers' subscriptions trigger
         * an invocation of the given Observable factory function.
         * @static true
         * @name defer
         * @owner Observable
         */
        DeferObservable.create = function (observableFactory) {
            return new DeferObservable(observableFactory);
        };
        DeferObservable.prototype._subscribe = function (subscriber) {
            return new DeferSubscriber(subscriber, this.observableFactory);
        };
        return DeferObservable;
    }(Observable_1.Observable);
    exports.DeferObservable = DeferObservable;
    var DeferSubscriber = function (_super) {
        __extends(DeferSubscriber, _super);
        function DeferSubscriber(destination, factory) {
            _super.call(this, destination);
            this.factory = factory;
            this.tryDefer();
        }
        DeferSubscriber.prototype.tryDefer = function () {
            try {
                this._callFactory();
            } catch (err) {
                this._error(err);
            }
        };
        DeferSubscriber.prototype._callFactory = function () {
            var result = this.factory();
            if (result) {
                this.add(subscribeToResult_1.subscribeToResult(this, result));
            }
        };
        return DeferSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic("node_modules/rxjs/observable/defer.js", ["./DeferObservable"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var DeferObservable_1 = $__require('./DeferObservable');
  exports.defer = DeferObservable_1.DeferObservable.create;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/defer.js', ['../../Observable', '../../observable/defer'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var defer_1 = $__require('../../observable/defer');
  Observable_1.Observable.defer = defer_1.defer;
  
});
System.registerDynamic("node_modules/rxjs/observable/empty.js", ["./EmptyObservable"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var EmptyObservable_1 = $__require('./EmptyObservable');
  exports.empty = EmptyObservable_1.EmptyObservable.create;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/empty.js', ['../../Observable', '../../observable/empty'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var empty_1 = $__require('../../observable/empty');
  Observable_1.Observable.empty = empty_1.empty;
  
});
System.registerDynamic('node_modules/rxjs/observable/ForkJoinObservable.js', ['../Observable', './EmptyObservable', '../util/isArray', '../util/subscribeToResult', '../OuterSubscriber'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Observable_1 = $__require('../Observable');
    var EmptyObservable_1 = $__require('./EmptyObservable');
    var isArray_1 = $__require('../util/isArray');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var ForkJoinObservable = function (_super) {
        __extends(ForkJoinObservable, _super);
        function ForkJoinObservable(sources, resultSelector) {
            _super.call(this);
            this.sources = sources;
            this.resultSelector = resultSelector;
        }
        /**
         * @param sources
         * @return {any}
         * @static true
         * @name forkJoin
         * @owner Observable
         */
        ForkJoinObservable.create = function () {
            var sources = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                sources[_i - 0] = arguments[_i];
            }
            if (sources === null || arguments.length === 0) {
                return new EmptyObservable_1.EmptyObservable();
            }
            var resultSelector = null;
            if (typeof sources[sources.length - 1] === 'function') {
                resultSelector = sources.pop();
            }
            // if the first and only other argument besides the resultSelector is an array
            // assume it's been called with `forkJoin([obs1, obs2, obs3], resultSelector)`
            if (sources.length === 1 && isArray_1.isArray(sources[0])) {
                sources = sources[0];
            }
            if (sources.length === 0) {
                return new EmptyObservable_1.EmptyObservable();
            }
            return new ForkJoinObservable(sources, resultSelector);
        };
        ForkJoinObservable.prototype._subscribe = function (subscriber) {
            return new ForkJoinSubscriber(subscriber, this.sources, this.resultSelector);
        };
        return ForkJoinObservable;
    }(Observable_1.Observable);
    exports.ForkJoinObservable = ForkJoinObservable;
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var ForkJoinSubscriber = function (_super) {
        __extends(ForkJoinSubscriber, _super);
        function ForkJoinSubscriber(destination, sources, resultSelector) {
            _super.call(this, destination);
            this.sources = sources;
            this.resultSelector = resultSelector;
            this.completed = 0;
            this.haveValues = 0;
            var len = sources.length;
            this.total = len;
            this.values = new Array(len);
            for (var i = 0; i < len; i++) {
                var source = sources[i];
                var innerSubscription = subscribeToResult_1.subscribeToResult(this, source, null, i);
                if (innerSubscription) {
                    innerSubscription.outerIndex = i;
                    this.add(innerSubscription);
                }
            }
        }
        ForkJoinSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            this.values[outerIndex] = innerValue;
            if (!innerSub._hasValue) {
                innerSub._hasValue = true;
                this.haveValues++;
            }
        };
        ForkJoinSubscriber.prototype.notifyComplete = function (innerSub) {
            var destination = this.destination;
            var _a = this,
                haveValues = _a.haveValues,
                resultSelector = _a.resultSelector,
                values = _a.values;
            var len = values.length;
            if (!innerSub._hasValue) {
                destination.complete();
                return;
            }
            this.completed++;
            if (this.completed !== len) {
                return;
            }
            if (haveValues === len) {
                var value = resultSelector ? resultSelector.apply(this, values) : values;
                destination.next(value);
            }
            destination.complete();
        };
        return ForkJoinSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic("node_modules/rxjs/observable/forkJoin.js", ["./ForkJoinObservable"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var ForkJoinObservable_1 = $__require('./ForkJoinObservable');
  exports.forkJoin = ForkJoinObservable_1.ForkJoinObservable.create;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/forkJoin.js', ['../../Observable', '../../observable/forkJoin'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var forkJoin_1 = $__require('../../observable/forkJoin');
  Observable_1.Observable.forkJoin = forkJoin_1.forkJoin;
  
});
System.registerDynamic('node_modules/rxjs/observable/IteratorObservable.js', ['../util/root', '../util/isObject', '../util/tryCatch', '../Observable', '../util/isFunction', '../symbol/iterator', '../util/errorObject'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var root_1 = $__require('../util/root');
    var isObject_1 = $__require('../util/isObject');
    var tryCatch_1 = $__require('../util/tryCatch');
    var Observable_1 = $__require('../Observable');
    var isFunction_1 = $__require('../util/isFunction');
    var iterator_1 = $__require('../symbol/iterator');
    var errorObject_1 = $__require('../util/errorObject');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var IteratorObservable = function (_super) {
        __extends(IteratorObservable, _super);
        function IteratorObservable(iterator, project, thisArg, scheduler) {
            _super.call(this);
            if (iterator == null) {
                throw new Error('iterator cannot be null.');
            }
            if (isObject_1.isObject(project)) {
                this.thisArg = project;
                this.scheduler = thisArg;
            } else if (isFunction_1.isFunction(project)) {
                this.project = project;
                this.thisArg = thisArg;
                this.scheduler = scheduler;
            } else if (project != null) {
                throw new Error('When provided, `project` must be a function.');
            }
            this.iterator = getIterator(iterator);
        }
        IteratorObservable.create = function (iterator, project, thisArg, scheduler) {
            return new IteratorObservable(iterator, project, thisArg, scheduler);
        };
        IteratorObservable.dispatch = function (state) {
            var index = state.index,
                hasError = state.hasError,
                thisArg = state.thisArg,
                project = state.project,
                iterator = state.iterator,
                subscriber = state.subscriber;
            if (hasError) {
                subscriber.error(state.error);
                return;
            }
            var result = iterator.next();
            if (result.done) {
                subscriber.complete();
                return;
            }
            if (project) {
                result = tryCatch_1.tryCatch(project).call(thisArg, result.value, index);
                if (result === errorObject_1.errorObject) {
                    state.error = errorObject_1.errorObject.e;
                    state.hasError = true;
                } else {
                    subscriber.next(result);
                    state.index = index + 1;
                }
            } else {
                subscriber.next(result.value);
                state.index = index + 1;
            }
            if (subscriber.isUnsubscribed) {
                return;
            }
            this.schedule(state);
        };
        IteratorObservable.prototype._subscribe = function (subscriber) {
            var index = 0;
            var _a = this,
                iterator = _a.iterator,
                project = _a.project,
                thisArg = _a.thisArg,
                scheduler = _a.scheduler;
            if (scheduler) {
                return scheduler.schedule(IteratorObservable.dispatch, 0, {
                    index: index, thisArg: thisArg, project: project, iterator: iterator, subscriber: subscriber
                });
            } else {
                do {
                    var result = iterator.next();
                    if (result.done) {
                        subscriber.complete();
                        break;
                    } else if (project) {
                        result = tryCatch_1.tryCatch(project).call(thisArg, result.value, index++);
                        if (result === errorObject_1.errorObject) {
                            subscriber.error(errorObject_1.errorObject.e);
                            break;
                        }
                        subscriber.next(result);
                    } else {
                        subscriber.next(result.value);
                    }
                    if (subscriber.isUnsubscribed) {
                        break;
                    }
                } while (true);
            }
        };
        return IteratorObservable;
    }(Observable_1.Observable);
    exports.IteratorObservable = IteratorObservable;
    var StringIterator = function () {
        function StringIterator(str, idx, len) {
            if (idx === void 0) {
                idx = 0;
            }
            if (len === void 0) {
                len = str.length;
            }
            this.str = str;
            this.idx = idx;
            this.len = len;
        }
        StringIterator.prototype[iterator_1.$$iterator] = function () {
            return this;
        };
        StringIterator.prototype.next = function () {
            return this.idx < this.len ? {
                done: false,
                value: this.str.charAt(this.idx++)
            } : {
                done: true,
                value: undefined
            };
        };
        return StringIterator;
    }();
    var ArrayIterator = function () {
        function ArrayIterator(arr, idx, len) {
            if (idx === void 0) {
                idx = 0;
            }
            if (len === void 0) {
                len = toLength(arr);
            }
            this.arr = arr;
            this.idx = idx;
            this.len = len;
        }
        ArrayIterator.prototype[iterator_1.$$iterator] = function () {
            return this;
        };
        ArrayIterator.prototype.next = function () {
            return this.idx < this.len ? {
                done: false,
                value: this.arr[this.idx++]
            } : {
                done: true,
                value: undefined
            };
        };
        return ArrayIterator;
    }();
    function getIterator(obj) {
        var i = obj[iterator_1.$$iterator];
        if (!i && typeof obj === 'string') {
            return new StringIterator(obj);
        }
        if (!i && obj.length !== undefined) {
            return new ArrayIterator(obj);
        }
        if (!i) {
            throw new TypeError('Object is not iterable');
        }
        return obj[iterator_1.$$iterator]();
    }
    var maxSafeInteger = Math.pow(2, 53) - 1;
    function toLength(o) {
        var len = +o.length;
        if (isNaN(len)) {
            return 0;
        }
        if (len === 0 || !numberIsFinite(len)) {
            return len;
        }
        len = sign(len) * Math.floor(Math.abs(len));
        if (len <= 0) {
            return 0;
        }
        if (len > maxSafeInteger) {
            return maxSafeInteger;
        }
        return len;
    }
    function numberIsFinite(value) {
        return typeof value === 'number' && root_1.root.isFinite(value);
    }
    function sign(value) {
        var valueAsNumber = +value;
        if (valueAsNumber === 0) {
            return valueAsNumber;
        }
        if (isNaN(valueAsNumber)) {
            return valueAsNumber;
        }
        return valueAsNumber < 0 ? -1 : 1;
    }
    
});
System.registerDynamic('node_modules/rxjs/observable/ArrayLikeObservable.js', ['../Observable', './ScalarObservable', './EmptyObservable'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Observable_1 = $__require('../Observable');
    var ScalarObservable_1 = $__require('./ScalarObservable');
    var EmptyObservable_1 = $__require('./EmptyObservable');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var ArrayLikeObservable = function (_super) {
        __extends(ArrayLikeObservable, _super);
        function ArrayLikeObservable(arrayLike, mapFn, thisArg, scheduler) {
            _super.call(this);
            this.arrayLike = arrayLike;
            this.scheduler = scheduler;
            if (!mapFn && !scheduler && arrayLike.length === 1) {
                this._isScalar = true;
                this.value = arrayLike[0];
            }
            if (mapFn) {
                this.mapFn = mapFn.bind(thisArg);
            }
        }
        ArrayLikeObservable.create = function (arrayLike, mapFn, thisArg, scheduler) {
            var length = arrayLike.length;
            if (length === 0) {
                return new EmptyObservable_1.EmptyObservable();
            } else if (length === 1 && !mapFn) {
                return new ScalarObservable_1.ScalarObservable(arrayLike[0], scheduler);
            } else {
                return new ArrayLikeObservable(arrayLike, mapFn, thisArg, scheduler);
            }
        };
        ArrayLikeObservable.dispatch = function (state) {
            var arrayLike = state.arrayLike,
                index = state.index,
                length = state.length,
                mapFn = state.mapFn,
                subscriber = state.subscriber;
            if (subscriber.isUnsubscribed) {
                return;
            }
            if (index >= length) {
                subscriber.complete();
                return;
            }
            var result = mapFn ? mapFn(arrayLike[index], index) : arrayLike[index];
            subscriber.next(result);
            state.index = index + 1;
            this.schedule(state);
        };
        ArrayLikeObservable.prototype._subscribe = function (subscriber) {
            var index = 0;
            var _a = this,
                arrayLike = _a.arrayLike,
                mapFn = _a.mapFn,
                scheduler = _a.scheduler;
            var length = arrayLike.length;
            if (scheduler) {
                return scheduler.schedule(ArrayLikeObservable.dispatch, 0, {
                    arrayLike: arrayLike, index: index, length: length, mapFn: mapFn, subscriber: subscriber
                });
            } else {
                for (var i = 0; i < length && !subscriber.isUnsubscribed; i++) {
                    var result = mapFn ? mapFn(arrayLike[i], i) : arrayLike[i];
                    subscriber.next(result);
                }
                subscriber.complete();
            }
        };
        return ArrayLikeObservable;
    }(Observable_1.Observable);
    exports.ArrayLikeObservable = ArrayLikeObservable;
    
});
System.registerDynamic('node_modules/rxjs/observable/FromObservable.js', ['../util/isArray', '../util/isFunction', '../util/isPromise', '../util/isScheduler', './PromiseObservable', './IteratorObservable', './ArrayObservable', './ArrayLikeObservable', '../symbol/observable', '../symbol/iterator', '../Observable', '../operator/observeOn'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var isArray_1 = $__require('../util/isArray');
    var isFunction_1 = $__require('../util/isFunction');
    var isPromise_1 = $__require('../util/isPromise');
    var isScheduler_1 = $__require('../util/isScheduler');
    var PromiseObservable_1 = $__require('./PromiseObservable');
    var IteratorObservable_1 = $__require('./IteratorObservable');
    var ArrayObservable_1 = $__require('./ArrayObservable');
    var ArrayLikeObservable_1 = $__require('./ArrayLikeObservable');
    var observable_1 = $__require('../symbol/observable');
    var iterator_1 = $__require('../symbol/iterator');
    var Observable_1 = $__require('../Observable');
    var observeOn_1 = $__require('../operator/observeOn');
    var isArrayLike = function (x) {
        return x && typeof x.length === 'number';
    };
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var FromObservable = function (_super) {
        __extends(FromObservable, _super);
        function FromObservable(ish, scheduler) {
            _super.call(this, null);
            this.ish = ish;
            this.scheduler = scheduler;
        }
        FromObservable.create = function (ish, mapFnOrScheduler, thisArg, lastScheduler) {
            var scheduler = null;
            var mapFn = null;
            if (isFunction_1.isFunction(mapFnOrScheduler)) {
                scheduler = lastScheduler || null;
                mapFn = mapFnOrScheduler;
            } else if (isScheduler_1.isScheduler(scheduler)) {
                scheduler = mapFnOrScheduler;
            }
            if (ish != null) {
                if (typeof ish[observable_1.$$observable] === 'function') {
                    if (ish instanceof Observable_1.Observable && !scheduler) {
                        return ish;
                    }
                    return new FromObservable(ish, scheduler);
                } else if (isArray_1.isArray(ish)) {
                    return new ArrayObservable_1.ArrayObservable(ish, scheduler);
                } else if (isPromise_1.isPromise(ish)) {
                    return new PromiseObservable_1.PromiseObservable(ish, scheduler);
                } else if (typeof ish[iterator_1.$$iterator] === 'function' || typeof ish === 'string') {
                    return new IteratorObservable_1.IteratorObservable(ish, null, null, scheduler);
                } else if (isArrayLike(ish)) {
                    return new ArrayLikeObservable_1.ArrayLikeObservable(ish, mapFn, thisArg, scheduler);
                }
            }
            throw new TypeError((ish !== null && typeof ish || ish) + ' is not observable');
        };
        FromObservable.prototype._subscribe = function (subscriber) {
            var ish = this.ish;
            var scheduler = this.scheduler;
            if (scheduler == null) {
                return ish[observable_1.$$observable]().subscribe(subscriber);
            } else {
                return ish[observable_1.$$observable]().subscribe(new observeOn_1.ObserveOnSubscriber(subscriber, scheduler, 0));
            }
        };
        return FromObservable;
    }(Observable_1.Observable);
    exports.FromObservable = FromObservable;
    
});
System.registerDynamic("node_modules/rxjs/observable/from.js", ["./FromObservable"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var FromObservable_1 = $__require('./FromObservable');
  exports.from = FromObservable_1.FromObservable.create;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/from.js', ['../../Observable', '../../observable/from'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var from_1 = $__require('../../observable/from');
  Observable_1.Observable.from = from_1.from;
  
});
System.registerDynamic('node_modules/rxjs/observable/FromEventObservable.js', ['../Observable', '../util/tryCatch', '../util/errorObject', '../Subscription'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Observable_1 = $__require('../Observable');
    var tryCatch_1 = $__require('../util/tryCatch');
    var errorObject_1 = $__require('../util/errorObject');
    var Subscription_1 = $__require('../Subscription');
    function isNodeStyleEventEmmitter(sourceObj) {
        return !!sourceObj && typeof sourceObj.addListener === 'function' && typeof sourceObj.removeListener === 'function';
    }
    function isJQueryStyleEventEmitter(sourceObj) {
        return !!sourceObj && typeof sourceObj.on === 'function' && typeof sourceObj.off === 'function';
    }
    function isNodeList(sourceObj) {
        return !!sourceObj && sourceObj.toString() === '[object NodeList]';
    }
    function isHTMLCollection(sourceObj) {
        return !!sourceObj && sourceObj.toString() === '[object HTMLCollection]';
    }
    function isEventTarget(sourceObj) {
        return !!sourceObj && typeof sourceObj.addEventListener === 'function' && typeof sourceObj.removeEventListener === 'function';
    }
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var FromEventObservable = function (_super) {
        __extends(FromEventObservable, _super);
        function FromEventObservable(sourceObj, eventName, selector) {
            _super.call(this);
            this.sourceObj = sourceObj;
            this.eventName = eventName;
            this.selector = selector;
        }
        /**
         * @param sourceObj
         * @param eventName
         * @param selector
         * @return {FromEventObservable}
         * @static true
         * @name fromEvent
         * @owner Observable
         */
        FromEventObservable.create = function (sourceObj, eventName, selector) {
            return new FromEventObservable(sourceObj, eventName, selector);
        };
        FromEventObservable.setupSubscription = function (sourceObj, eventName, handler, subscriber) {
            var unsubscribe;
            if (isNodeList(sourceObj) || isHTMLCollection(sourceObj)) {
                for (var i = 0, len = sourceObj.length; i < len; i++) {
                    FromEventObservable.setupSubscription(sourceObj[i], eventName, handler, subscriber);
                }
            } else if (isEventTarget(sourceObj)) {
                sourceObj.addEventListener(eventName, handler);
                unsubscribe = function () {
                    return sourceObj.removeEventListener(eventName, handler);
                };
            } else if (isJQueryStyleEventEmitter(sourceObj)) {
                sourceObj.on(eventName, handler);
                unsubscribe = function () {
                    return sourceObj.off(eventName, handler);
                };
            } else if (isNodeStyleEventEmmitter(sourceObj)) {
                sourceObj.addListener(eventName, handler);
                unsubscribe = function () {
                    return sourceObj.removeListener(eventName, handler);
                };
            }
            subscriber.add(new Subscription_1.Subscription(unsubscribe));
        };
        FromEventObservable.prototype._subscribe = function (subscriber) {
            var sourceObj = this.sourceObj;
            var eventName = this.eventName;
            var selector = this.selector;
            var handler = selector ? function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i - 0] = arguments[_i];
                }
                var result = tryCatch_1.tryCatch(selector).apply(void 0, args);
                if (result === errorObject_1.errorObject) {
                    subscriber.error(errorObject_1.errorObject.e);
                } else {
                    subscriber.next(result);
                }
            } : function (e) {
                return subscriber.next(e);
            };
            FromEventObservable.setupSubscription(sourceObj, eventName, handler, subscriber);
        };
        return FromEventObservable;
    }(Observable_1.Observable);
    exports.FromEventObservable = FromEventObservable;
    
});
System.registerDynamic("node_modules/rxjs/observable/fromEvent.js", ["./FromEventObservable"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var FromEventObservable_1 = $__require('./FromEventObservable');
  exports.fromEvent = FromEventObservable_1.FromEventObservable.create;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/fromEvent.js', ['../../Observable', '../../observable/fromEvent'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var fromEvent_1 = $__require('../../observable/fromEvent');
  Observable_1.Observable.fromEvent = fromEvent_1.fromEvent;
  
});
System.registerDynamic('node_modules/rxjs/observable/FromEventPatternObservable.js', ['../Observable', '../Subscription', '../util/tryCatch', '../util/errorObject'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Observable_1 = $__require('../Observable');
    var Subscription_1 = $__require('../Subscription');
    var tryCatch_1 = $__require('../util/tryCatch');
    var errorObject_1 = $__require('../util/errorObject');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var FromEventPatternObservable = function (_super) {
        __extends(FromEventPatternObservable, _super);
        function FromEventPatternObservable(addHandler, removeHandler, selector) {
            _super.call(this);
            this.addHandler = addHandler;
            this.removeHandler = removeHandler;
            this.selector = selector;
        }
        /**
         * @param addHandler
         * @param removeHandler
         * @param selector
         * @return {FromEventPatternObservable}
         * @static true
         * @name fromEventPattern
         * @owner Observable
         */
        FromEventPatternObservable.create = function (addHandler, removeHandler, selector) {
            return new FromEventPatternObservable(addHandler, removeHandler, selector);
        };
        FromEventPatternObservable.prototype._subscribe = function (subscriber) {
            var addHandler = this.addHandler;
            var removeHandler = this.removeHandler;
            var selector = this.selector;
            var handler = selector ? function (e) {
                var result = tryCatch_1.tryCatch(selector).apply(null, arguments);
                if (result === errorObject_1.errorObject) {
                    subscriber.error(result.e);
                } else {
                    subscriber.next(result);
                }
            } : function (e) {
                subscriber.next(e);
            };
            var result = tryCatch_1.tryCatch(addHandler)(handler);
            if (result === errorObject_1.errorObject) {
                subscriber.error(result.e);
            }
            subscriber.add(new Subscription_1.Subscription(function () {
                //TODO: determine whether or not to forward to error handler
                removeHandler(handler);
            }));
        };
        return FromEventPatternObservable;
    }(Observable_1.Observable);
    exports.FromEventPatternObservable = FromEventPatternObservable;
    
});
System.registerDynamic("node_modules/rxjs/observable/fromEventPattern.js", ["./FromEventPatternObservable"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var FromEventPatternObservable_1 = $__require('./FromEventPatternObservable');
  exports.fromEventPattern = FromEventPatternObservable_1.FromEventPatternObservable.create;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/fromEventPattern.js', ['../../Observable', '../../observable/fromEventPattern'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var fromEventPattern_1 = $__require('../../observable/fromEventPattern');
  Observable_1.Observable.fromEventPattern = fromEventPattern_1.fromEventPattern;
  
});
System.registerDynamic('node_modules/rxjs/observable/PromiseObservable.js', ['../util/root', '../Observable'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var root_1 = $__require('../util/root');
    var Observable_1 = $__require('../Observable');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var PromiseObservable = function (_super) {
        __extends(PromiseObservable, _super);
        function PromiseObservable(promise, scheduler) {
            if (scheduler === void 0) {
                scheduler = null;
            }
            _super.call(this);
            this.promise = promise;
            this.scheduler = scheduler;
        }
        /**
         * @param promise
         * @param scheduler
         * @return {PromiseObservable}
         * @static true
         * @name fromPromise
         * @owner Observable
         */
        PromiseObservable.create = function (promise, scheduler) {
            if (scheduler === void 0) {
                scheduler = null;
            }
            return new PromiseObservable(promise, scheduler);
        };
        PromiseObservable.prototype._subscribe = function (subscriber) {
            var _this = this;
            var promise = this.promise;
            var scheduler = this.scheduler;
            if (scheduler == null) {
                if (this._isScalar) {
                    if (!subscriber.isUnsubscribed) {
                        subscriber.next(this.value);
                        subscriber.complete();
                    }
                } else {
                    promise.then(function (value) {
                        _this.value = value;
                        _this._isScalar = true;
                        if (!subscriber.isUnsubscribed) {
                            subscriber.next(value);
                            subscriber.complete();
                        }
                    }, function (err) {
                        if (!subscriber.isUnsubscribed) {
                            subscriber.error(err);
                        }
                    }).then(null, function (err) {
                        // escape the promise trap, throw unhandled errors
                        root_1.root.setTimeout(function () {
                            throw err;
                        });
                    });
                }
            } else {
                if (this._isScalar) {
                    if (!subscriber.isUnsubscribed) {
                        return scheduler.schedule(dispatchNext, 0, { value: this.value, subscriber: subscriber });
                    }
                } else {
                    promise.then(function (value) {
                        _this.value = value;
                        _this._isScalar = true;
                        if (!subscriber.isUnsubscribed) {
                            subscriber.add(scheduler.schedule(dispatchNext, 0, { value: value, subscriber: subscriber }));
                        }
                    }, function (err) {
                        if (!subscriber.isUnsubscribed) {
                            subscriber.add(scheduler.schedule(dispatchError, 0, { err: err, subscriber: subscriber }));
                        }
                    }).then(null, function (err) {
                        // escape the promise trap, throw unhandled errors
                        root_1.root.setTimeout(function () {
                            throw err;
                        });
                    });
                }
            }
        };
        return PromiseObservable;
    }(Observable_1.Observable);
    exports.PromiseObservable = PromiseObservable;
    function dispatchNext(arg) {
        var value = arg.value,
            subscriber = arg.subscriber;
        if (!subscriber.isUnsubscribed) {
            subscriber.next(value);
            subscriber.complete();
        }
    }
    function dispatchError(arg) {
        var err = arg.err,
            subscriber = arg.subscriber;
        if (!subscriber.isUnsubscribed) {
            subscriber.error(err);
        }
    }
    
});
System.registerDynamic("node_modules/rxjs/observable/fromPromise.js", ["./PromiseObservable"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var PromiseObservable_1 = $__require('./PromiseObservable');
  exports.fromPromise = PromiseObservable_1.PromiseObservable.create;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/fromPromise.js', ['../../Observable', '../../observable/fromPromise'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var fromPromise_1 = $__require('../../observable/fromPromise');
  Observable_1.Observable.fromPromise = fromPromise_1.fromPromise;
  
});
System.registerDynamic('node_modules/rxjs/observable/IntervalObservable.js', ['../util/isNumeric', '../Observable', '../scheduler/async'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var isNumeric_1 = $__require('../util/isNumeric');
    var Observable_1 = $__require('../Observable');
    var async_1 = $__require('../scheduler/async');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var IntervalObservable = function (_super) {
        __extends(IntervalObservable, _super);
        function IntervalObservable(period, scheduler) {
            if (period === void 0) {
                period = 0;
            }
            if (scheduler === void 0) {
                scheduler = async_1.async;
            }
            _super.call(this);
            this.period = period;
            this.scheduler = scheduler;
            if (!isNumeric_1.isNumeric(period) || period < 0) {
                this.period = 0;
            }
            if (!scheduler || typeof scheduler.schedule !== 'function') {
                this.scheduler = async_1.async;
            }
        }
        /**
         * Creates an Observable that emits sequential numbers every specified
         * interval of time, on a specified Scheduler.
         *
         * <span class="informal">Emits incremental numbers periodically in time.
         * </span>
         *
         * <img src="./img/interval.png" width="100%">
         *
         * `interval` returns an Observable that emits an infinite sequence of
         * ascending integers, with a constant interval of time of your choosing
         * between those emissions. The first emission is not sent immediately, but
         * only after the first period has passed. By default, this operator uses the
         * `async` Scheduler to provide a notion of time, but you may pass any
         * Scheduler to it.
         *
         * @example <caption>Emits ascending numbers, one every second (1000ms)</caption>
         * var numbers = Rx.Observable.interval(1000);
         * numbers.subscribe(x => console.log(x));
         *
         * @see {@link timer}
         * @see {@link delay}
         *
         * @param {number} [period=0] The interval size in milliseconds (by default)
         * or the time unit determined by the scheduler's clock.
         * @param {Scheduler} [scheduler=async] The Scheduler to use for scheduling
         * the emission of values, and providing a notion of "time".
         * @return {Observable} An Observable that emits a sequential number each time
         * interval.
         * @static true
         * @name interval
         * @owner Observable
         */
        IntervalObservable.create = function (period, scheduler) {
            if (period === void 0) {
                period = 0;
            }
            if (scheduler === void 0) {
                scheduler = async_1.async;
            }
            return new IntervalObservable(period, scheduler);
        };
        IntervalObservable.dispatch = function (state) {
            var index = state.index,
                subscriber = state.subscriber,
                period = state.period;
            subscriber.next(index);
            if (subscriber.isUnsubscribed) {
                return;
            }
            state.index += 1;
            this.schedule(state, period);
        };
        IntervalObservable.prototype._subscribe = function (subscriber) {
            var index = 0;
            var period = this.period;
            var scheduler = this.scheduler;
            subscriber.add(scheduler.schedule(IntervalObservable.dispatch, period, {
                index: index, subscriber: subscriber, period: period
            }));
        };
        return IntervalObservable;
    }(Observable_1.Observable);
    exports.IntervalObservable = IntervalObservable;
    
});
System.registerDynamic("node_modules/rxjs/observable/interval.js", ["./IntervalObservable"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var IntervalObservable_1 = $__require('./IntervalObservable');
  exports.interval = IntervalObservable_1.IntervalObservable.create;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/interval.js', ['../../Observable', '../../observable/interval'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var interval_1 = $__require('../../observable/interval');
  Observable_1.Observable.interval = interval_1.interval;
  
});
System.registerDynamic("node_modules/rxjs/observable/merge.js", ["../operator/merge"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var merge_1 = $__require('../operator/merge');
  exports.merge = merge_1.mergeStatic;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/merge.js', ['../../Observable', '../../observable/merge'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var merge_1 = $__require('../../observable/merge');
  Observable_1.Observable.merge = merge_1.merge;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/race.js', ['../../Observable', '../../operator/race'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var race_1 = $__require('../../operator/race');
  Observable_1.Observable.race = race_1.raceStatic;
  
});
System.registerDynamic('node_modules/rxjs/observable/NeverObservable.js', ['../Observable', '../util/noop'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Observable_1 = $__require('../Observable');
    var noop_1 = $__require('../util/noop');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var NeverObservable = function (_super) {
        __extends(NeverObservable, _super);
        function NeverObservable() {
            _super.call(this);
        }
        /**
         * Creates an Observable that emits no items to the Observer.
         *
         * <span class="informal">An Observable that never emits anything.</span>
         *
         * <img src="./img/never.png" width="100%">
         *
         * This static operator is useful for creating a simple Observable that emits
         * neither values nor errors nor the completion notification. It can be used
         * for testing purposes or for composing with other Observables. Please not
         * that by never emitting a complete notification, this Observable keeps the
         * subscription from being disposed automatically. Subscriptions need to be
         * manually disposed.
         *
         * @example <caption>Emit the number 7, then never emit anything else (not even complete).</caption>
         * function info() {
         *   console.log('Will not be called');
         * }
         * var result = Rx.Observable.never().startWith(7);
         * result.subscribe(x => console.log(x), info, info);
         *
         * @see {@link create}
         * @see {@link empty}
         * @see {@link of}
         * @see {@link throw}
         *
         * @return {Observable} A "never" Observable: never emits anything.
         * @static true
         * @name never
         * @owner Observable
         */
        NeverObservable.create = function () {
            return new NeverObservable();
        };
        NeverObservable.prototype._subscribe = function (subscriber) {
            noop_1.noop();
        };
        return NeverObservable;
    }(Observable_1.Observable);
    exports.NeverObservable = NeverObservable;
    
});
System.registerDynamic("node_modules/rxjs/observable/never.js", ["./NeverObservable"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var NeverObservable_1 = $__require('./NeverObservable');
  exports.never = NeverObservable_1.NeverObservable.create;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/never.js', ['../../Observable', '../../observable/never'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var never_1 = $__require('../../observable/never');
  Observable_1.Observable.never = never_1.never;
  
});
System.registerDynamic("node_modules/rxjs/observable/of.js", ["./ArrayObservable"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var ArrayObservable_1 = $__require('./ArrayObservable');
  exports.of = ArrayObservable_1.ArrayObservable.of;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/of.js', ['../../Observable', '../../observable/of'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var of_1 = $__require('../../observable/of');
  Observable_1.Observable.of = of_1.of;
  
});
System.registerDynamic("node_modules/rxjs/observable/RangeObservable.js", ["../Observable"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Observable_1 = $__require('../Observable');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var RangeObservable = function (_super) {
        __extends(RangeObservable, _super);
        function RangeObservable(start, count, scheduler) {
            _super.call(this);
            this.start = start;
            this._count = count;
            this.scheduler = scheduler;
        }
        /**
         * Creates an Observable that emits a sequence of numbers within a specified
         * range.
         *
         * <span class="informal">Emits a sequence of numbers in a range.</span>
         *
         * <img src="./img/range.png" width="100%">
         *
         * `range` operator emits a range of sequential integers, in order, where you
         * select the `start` of the range and its `length`. By default, uses no
         * Scheduler and just delivers the notifications synchronously, but may use
         * an optional Scheduler to regulate those deliveries.
         *
         * @example <caption>Emits the numbers 1 to 10</caption>
         * var numbers = Rx.Observable.range(1, 10);
         * numbers.subscribe(x => console.log(x));
         *
         * @see {@link timer}
         * @see {@link interval}
         *
         * @param {number} [start=0] The value of the first integer in the sequence.
         * @param {number} [count=0] The number of sequential integers to generate.
         * @param {Scheduler} [scheduler] A {@link Scheduler} to use for scheduling
         * the emissions of the notifications.
         * @return {Observable} An Observable of numbers that emits a finite range of
         * sequential integers.
         * @static true
         * @name range
         * @owner Observable
         */
        RangeObservable.create = function (start, count, scheduler) {
            if (start === void 0) {
                start = 0;
            }
            if (count === void 0) {
                count = 0;
            }
            return new RangeObservable(start, count, scheduler);
        };
        RangeObservable.dispatch = function (state) {
            var start = state.start,
                index = state.index,
                count = state.count,
                subscriber = state.subscriber;
            if (index >= count) {
                subscriber.complete();
                return;
            }
            subscriber.next(start);
            if (subscriber.isUnsubscribed) {
                return;
            }
            state.index = index + 1;
            state.start = start + 1;
            this.schedule(state);
        };
        RangeObservable.prototype._subscribe = function (subscriber) {
            var index = 0;
            var start = this.start;
            var count = this._count;
            var scheduler = this.scheduler;
            if (scheduler) {
                return scheduler.schedule(RangeObservable.dispatch, 0, {
                    index: index, count: count, start: start, subscriber: subscriber
                });
            } else {
                do {
                    if (index++ >= count) {
                        subscriber.complete();
                        break;
                    }
                    subscriber.next(start++);
                    if (subscriber.isUnsubscribed) {
                        break;
                    }
                } while (true);
            }
        };
        return RangeObservable;
    }(Observable_1.Observable);
    exports.RangeObservable = RangeObservable;
    
});
System.registerDynamic("node_modules/rxjs/observable/range.js", ["./RangeObservable"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var RangeObservable_1 = $__require('./RangeObservable');
  exports.range = RangeObservable_1.RangeObservable.create;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/range.js', ['../../Observable', '../../observable/range'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var range_1 = $__require('../../observable/range');
  Observable_1.Observable.range = range_1.range;
  
});
System.registerDynamic("node_modules/rxjs/observable/ErrorObservable.js", ["../Observable"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Observable_1 = $__require('../Observable');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var ErrorObservable = function (_super) {
        __extends(ErrorObservable, _super);
        function ErrorObservable(error, scheduler) {
            _super.call(this);
            this.error = error;
            this.scheduler = scheduler;
        }
        /**
         * Creates an Observable that emits no items to the Observer and immediately
         * emits an error notification.
         *
         * <span class="informal">Just emits 'error', and nothing else.
         * </span>
         *
         * <img src="./img/throw.png" width="100%">
         *
         * This static operator is useful for creating a simple Observable that only
         * emits the error notification. It can be used for composing with other
         * Observables, such as in a {@link mergeMap}.
         *
         * @example <caption>Emit the number 7, then emit an error.</caption>
         * var result = Rx.Observable.throw(new Error('oops!')).startWith(7);
         * result.subscribe(x => console.log(x), e => console.error(e));
         *
         * @example <caption>Map and flattens numbers to the sequence 'a', 'b', 'c', but throw an error for 13</caption>
         * var interval = Rx.Observable.interval(1000);
         * var result = interval.mergeMap(x =>
         *   x === 13 ?
         *     Rx.Observable.throw('Thirteens are bad') :
         *     Rx.Observable.of('a', 'b', 'c')
         * );
         * result.subscribe(x => console.log(x), e => console.error(e));
         *
         * @see {@link create}
         * @see {@link empty}
         * @see {@link never}
         * @see {@link of}
         *
         * @param {any} error The particular Error to pass to the error notification.
         * @param {Scheduler} [scheduler] A {@link Scheduler} to use for scheduling
         * the emission of the error notification.
         * @return {Observable} An error Observable: emits only the error notification
         * using the given error argument.
         * @static true
         * @name throw
         * @owner Observable
         */
        ErrorObservable.create = function (error, scheduler) {
            return new ErrorObservable(error, scheduler);
        };
        ErrorObservable.dispatch = function (arg) {
            var error = arg.error,
                subscriber = arg.subscriber;
            subscriber.error(error);
        };
        ErrorObservable.prototype._subscribe = function (subscriber) {
            var error = this.error;
            var scheduler = this.scheduler;
            if (scheduler) {
                return scheduler.schedule(ErrorObservable.dispatch, 0, {
                    error: error, subscriber: subscriber
                });
            } else {
                subscriber.error(error);
            }
        };
        return ErrorObservable;
    }(Observable_1.Observable);
    exports.ErrorObservable = ErrorObservable;
    
});
System.registerDynamic("node_modules/rxjs/observable/throw.js", ["./ErrorObservable"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var ErrorObservable_1 = $__require('./ErrorObservable');
  exports._throw = ErrorObservable_1.ErrorObservable.create;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/throw.js', ['../../Observable', '../../observable/throw'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var throw_1 = $__require('../../observable/throw');
  Observable_1.Observable.throw = throw_1._throw;
  
});
System.registerDynamic('node_modules/rxjs/observable/TimerObservable.js', ['../util/isNumeric', '../Observable', '../scheduler/async', '../util/isScheduler', '../util/isDate'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var isNumeric_1 = $__require('../util/isNumeric');
    var Observable_1 = $__require('../Observable');
    var async_1 = $__require('../scheduler/async');
    var isScheduler_1 = $__require('../util/isScheduler');
    var isDate_1 = $__require('../util/isDate');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var TimerObservable = function (_super) {
        __extends(TimerObservable, _super);
        function TimerObservable(dueTime, period, scheduler) {
            if (dueTime === void 0) {
                dueTime = 0;
            }
            _super.call(this);
            this.period = -1;
            this.dueTime = 0;
            if (isNumeric_1.isNumeric(period)) {
                this.period = Number(period) < 1 && 1 || Number(period);
            } else if (isScheduler_1.isScheduler(period)) {
                scheduler = period;
            }
            if (!isScheduler_1.isScheduler(scheduler)) {
                scheduler = async_1.async;
            }
            this.scheduler = scheduler;
            this.dueTime = isDate_1.isDate(dueTime) ? +dueTime - this.scheduler.now() : dueTime;
        }
        /**
         * Creates an Observable that starts emitting after an `initialDelay` and
         * emits ever increasing numbers after each `period` of time thereafter.
         *
         * <span class="informal">Its like {@link interval}, but you can specify when
         * should the emissions start.</span>
         *
         * <img src="./img/timer.png" width="100%">
         *
         * `timer` returns an Observable that emits an infinite sequence of ascending
         * integers, with a constant interval of time, `period` of your choosing
         * between those emissions. The first emission happens after the specified
         * `initialDelay`. The initial delay may be a {@link Date}. By default, this
         * operator uses the `async` Scheduler to provide a notion of time, but you
         * may pass any Scheduler to it. If `period` is not specified, the output
         * Observable emits only one value, `0`. Otherwise, it emits an infinite
         * sequence.
         *
         * @example <caption>Emits ascending numbers, one every second (1000ms), starting after 3 seconds</caption>
         * var numbers = Rx.Observable.timer(3000, 1000);
         * numbers.subscribe(x => console.log(x));
         *
         * @example <caption>Emits one number after five seconds</caption>
         * var numbers = Rx.Observable.timer(5000);
         * numbers.subscribe(x => console.log(x));
         *
         * @see {@link interval}
         * @see {@link delay}
         *
         * @param {number|Date} initialDelay The initial delay time to wait before
         * emitting the first value of `0`.
         * @param {number} [period] The period of time between emissions of the
         * subsequent numbers.
         * @param {Scheduler} [scheduler=async] The Scheduler to use for scheduling
         * the emission of values, and providing a notion of "time".
         * @return {Observable} An Observable that emits a `0` after the
         * `initialDelay` and ever increasing numbers after each `period` of time
         * thereafter.
         * @static true
         * @name timer
         * @owner Observable
         */
        TimerObservable.create = function (initialDelay, period, scheduler) {
            if (initialDelay === void 0) {
                initialDelay = 0;
            }
            return new TimerObservable(initialDelay, period, scheduler);
        };
        TimerObservable.dispatch = function (state) {
            var index = state.index,
                period = state.period,
                subscriber = state.subscriber;
            var action = this;
            subscriber.next(index);
            if (subscriber.isUnsubscribed) {
                return;
            } else if (period === -1) {
                return subscriber.complete();
            }
            state.index = index + 1;
            action.schedule(state, period);
        };
        TimerObservable.prototype._subscribe = function (subscriber) {
            var index = 0;
            var _a = this,
                period = _a.period,
                dueTime = _a.dueTime,
                scheduler = _a.scheduler;
            return scheduler.schedule(TimerObservable.dispatch, dueTime, {
                index: index, period: period, subscriber: subscriber
            });
        };
        return TimerObservable;
    }(Observable_1.Observable);
    exports.TimerObservable = TimerObservable;
    
});
System.registerDynamic("node_modules/rxjs/observable/timer.js", ["./TimerObservable"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var TimerObservable_1 = $__require('./TimerObservable');
  exports.timer = TimerObservable_1.TimerObservable.create;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/timer.js', ['../../Observable', '../../observable/timer'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var timer_1 = $__require('../../observable/timer');
  Observable_1.Observable.timer = timer_1.timer;
  
});
System.registerDynamic("node_modules/rxjs/observable/zip.js", ["../operator/zip"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var zip_1 = $__require('../operator/zip');
  exports.zip = zip_1.zipStatic;
  
});
System.registerDynamic('node_modules/rxjs/add/observable/zip.js', ['../../Observable', '../../observable/zip'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var zip_1 = $__require('../../observable/zip');
  Observable_1.Observable.zip = zip_1.zip;
  
});
System.registerDynamic('node_modules/rxjs/operator/buffer.js', ['../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Buffers the source Observable values until `closingNotifier` emits.
     *
     * <span class="informal">Collects values from the past as an array, and emits
     * that array only when another Observable emits.</span>
     *
     * <img src="./img/buffer.png" width="100%">
     *
     * Buffers the incoming Observable values until the given `closingNotifier`
     * Observable emits a value, at which point it emits the buffer on the output
     * Observable and starts a new buffer internally, awaiting the next time
     * `closingNotifier` emits.
     *
     * @example <caption>On every click, emit array of most recent interval events</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var interval = Rx.Observable.interval(1000);
     * var buffered = interval.buffer(clicks);
     * buffered.subscribe(x => console.log(x));
     *
     * @see {@link bufferCount}
     * @see {@link bufferTime}
     * @see {@link bufferToggle}
     * @see {@link bufferWhen}
     * @see {@link window}
     *
     * @param {Observable<any>} closingNotifier An Observable that signals the
     * buffer to be emitted on the output Observable.
     * @return {Observable<T[]>} An Observable of buffers, which are arrays of
     * values.
     * @method buffer
     * @owner Observable
     */
    function buffer(closingNotifier) {
        return this.lift(new BufferOperator(closingNotifier));
    }
    exports.buffer = buffer;
    var BufferOperator = function () {
        function BufferOperator(closingNotifier) {
            this.closingNotifier = closingNotifier;
        }
        BufferOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new BufferSubscriber(subscriber, this.closingNotifier));
        };
        return BufferOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var BufferSubscriber = function (_super) {
        __extends(BufferSubscriber, _super);
        function BufferSubscriber(destination, closingNotifier) {
            _super.call(this, destination);
            this.buffer = [];
            this.add(subscribeToResult_1.subscribeToResult(this, closingNotifier));
        }
        BufferSubscriber.prototype._next = function (value) {
            this.buffer.push(value);
        };
        BufferSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            var buffer = this.buffer;
            this.buffer = [];
            this.destination.next(buffer);
        };
        return BufferSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/buffer.js', ['../../Observable', '../../operator/buffer'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var buffer_1 = $__require('../../operator/buffer');
  Observable_1.Observable.prototype.buffer = buffer_1.buffer;
  
});
System.registerDynamic("node_modules/rxjs/operator/bufferCount.js", ["../Subscriber"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * Buffers the source Observable values until the size hits the maximum
     * `bufferSize` given.
     *
     * <span class="informal">Collects values from the past as an array, and emits
     * that array only when its size reaches `bufferSize`.</span>
     *
     * <img src="./img/bufferCount.png" width="100%">
     *
     * Buffers a number of values from the source Observable by `bufferSize` then
     * emits the buffer and clears it, and starts a new buffer each
     * `startBufferEvery` values. If `startBufferEvery` is not provided or is
     * `null`, then new buffers are started immediately at the start of the source
     * and when each buffer closes and is emitted.
     *
     * @example <caption>Emit the last two click events as an array</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var buffered = clicks.bufferCount(2);
     * buffered.subscribe(x => console.log(x));
     *
     * @example <caption>On every click, emit the last two click events as an array</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var buffered = clicks.bufferCount(2, 1);
     * buffered.subscribe(x => console.log(x));
     *
     * @see {@link buffer}
     * @see {@link bufferTime}
     * @see {@link bufferToggle}
     * @see {@link bufferWhen}
     * @see {@link windowCount}
     *
     * @param {number} bufferSize The maximum size of the buffer emitted.
     * @param {number} [startBufferEvery] Interval at which to start a new buffer.
     * For example if `startBufferEvery` is `2`, then a new buffer will be started
     * on every other value from the source. A new buffer is started at the
     * beginning of the source by default.
     * @return {Observable<T[]>} An Observable of arrays of buffered values.
     * @method bufferCount
     * @owner Observable
     */
    function bufferCount(bufferSize, startBufferEvery) {
        if (startBufferEvery === void 0) {
            startBufferEvery = null;
        }
        return this.lift(new BufferCountOperator(bufferSize, startBufferEvery));
    }
    exports.bufferCount = bufferCount;
    var BufferCountOperator = function () {
        function BufferCountOperator(bufferSize, startBufferEvery) {
            this.bufferSize = bufferSize;
            this.startBufferEvery = startBufferEvery;
        }
        BufferCountOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new BufferCountSubscriber(subscriber, this.bufferSize, this.startBufferEvery));
        };
        return BufferCountOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var BufferCountSubscriber = function (_super) {
        __extends(BufferCountSubscriber, _super);
        function BufferCountSubscriber(destination, bufferSize, startBufferEvery) {
            _super.call(this, destination);
            this.bufferSize = bufferSize;
            this.startBufferEvery = startBufferEvery;
            this.buffers = [[]];
            this.count = 0;
        }
        BufferCountSubscriber.prototype._next = function (value) {
            var count = this.count += 1;
            var destination = this.destination;
            var bufferSize = this.bufferSize;
            var startBufferEvery = this.startBufferEvery == null ? bufferSize : this.startBufferEvery;
            var buffers = this.buffers;
            var len = buffers.length;
            var remove = -1;
            if (count % startBufferEvery === 0) {
                buffers.push([]);
            }
            for (var i = 0; i < len; i++) {
                var buffer = buffers[i];
                buffer.push(value);
                if (buffer.length === bufferSize) {
                    remove = i;
                    destination.next(buffer);
                }
            }
            if (remove !== -1) {
                buffers.splice(remove, 1);
            }
        };
        BufferCountSubscriber.prototype._complete = function () {
            var destination = this.destination;
            var buffers = this.buffers;
            while (buffers.length > 0) {
                var buffer = buffers.shift();
                if (buffer.length > 0) {
                    destination.next(buffer);
                }
            }
            _super.prototype._complete.call(this);
        };
        return BufferCountSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/bufferCount.js', ['../../Observable', '../../operator/bufferCount'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var bufferCount_1 = $__require('../../operator/bufferCount');
  Observable_1.Observable.prototype.bufferCount = bufferCount_1.bufferCount;
  
});
System.registerDynamic('node_modules/rxjs/operator/bufferTime.js', ['../Subscriber', '../scheduler/async'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var async_1 = $__require('../scheduler/async');
    /**
     * Buffers the source Observable values for a specific time period.
     *
     * <span class="informal">Collects values from the past as an array, and emits
     * those arrays periodically in time.</span>
     *
     * <img src="./img/bufferTime.png" width="100%">
     *
     * Buffers values from the source for a specific time duration `bufferTimeSpan`.
     * Unless the optional argument `bufferCreationInterval` is given, it emits and
     * resets the buffer every `bufferTimeSpan` milliseconds. If
     * `bufferCreationInterval` is given, this operator opens the buffer every
     * `bufferCreationInterval` milliseconds and closes (emits and resets) the
     * buffer every `bufferTimeSpan` milliseconds.
     *
     * @example <caption>Every second, emit an array of the recent click events</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var buffered = clicks.bufferTime(1000);
     * buffered.subscribe(x => console.log(x));
     *
     * @example <caption>Every 5 seconds, emit the click events from the next 2 seconds</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var buffered = clicks.bufferTime(2000, 5000);
     * buffered.subscribe(x => console.log(x));
     *
     * @see {@link buffer}
     * @see {@link bufferCount}
     * @see {@link bufferToggle}
     * @see {@link bufferWhen}
     * @see {@link windowTime}
     *
     * @param {number} bufferTimeSpan The amount of time to fill each buffer array.
     * @param {number} [bufferCreationInterval] The interval at which to start new
     * buffers.
     * @param {Scheduler} [scheduler=async] The scheduler on which to schedule the
     * intervals that determine buffer boundaries.
     * @return {Observable<T[]>} An observable of arrays of buffered values.
     * @method bufferTime
     * @owner Observable
     */
    function bufferTime(bufferTimeSpan, bufferCreationInterval, scheduler) {
        if (bufferCreationInterval === void 0) {
            bufferCreationInterval = null;
        }
        if (scheduler === void 0) {
            scheduler = async_1.async;
        }
        return this.lift(new BufferTimeOperator(bufferTimeSpan, bufferCreationInterval, scheduler));
    }
    exports.bufferTime = bufferTime;
    var BufferTimeOperator = function () {
        function BufferTimeOperator(bufferTimeSpan, bufferCreationInterval, scheduler) {
            this.bufferTimeSpan = bufferTimeSpan;
            this.bufferCreationInterval = bufferCreationInterval;
            this.scheduler = scheduler;
        }
        BufferTimeOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new BufferTimeSubscriber(subscriber, this.bufferTimeSpan, this.bufferCreationInterval, this.scheduler));
        };
        return BufferTimeOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var BufferTimeSubscriber = function (_super) {
        __extends(BufferTimeSubscriber, _super);
        function BufferTimeSubscriber(destination, bufferTimeSpan, bufferCreationInterval, scheduler) {
            _super.call(this, destination);
            this.bufferTimeSpan = bufferTimeSpan;
            this.bufferCreationInterval = bufferCreationInterval;
            this.scheduler = scheduler;
            this.buffers = [];
            var buffer = this.openBuffer();
            if (bufferCreationInterval !== null && bufferCreationInterval >= 0) {
                var closeState = { subscriber: this, buffer: buffer };
                var creationState = { bufferTimeSpan: bufferTimeSpan, bufferCreationInterval: bufferCreationInterval, subscriber: this, scheduler: scheduler };
                this.add(scheduler.schedule(dispatchBufferClose, bufferTimeSpan, closeState));
                this.add(scheduler.schedule(dispatchBufferCreation, bufferCreationInterval, creationState));
            } else {
                var timeSpanOnlyState = { subscriber: this, buffer: buffer, bufferTimeSpan: bufferTimeSpan };
                this.add(scheduler.schedule(dispatchBufferTimeSpanOnly, bufferTimeSpan, timeSpanOnlyState));
            }
        }
        BufferTimeSubscriber.prototype._next = function (value) {
            var buffers = this.buffers;
            var len = buffers.length;
            for (var i = 0; i < len; i++) {
                buffers[i].push(value);
            }
        };
        BufferTimeSubscriber.prototype._error = function (err) {
            this.buffers.length = 0;
            _super.prototype._error.call(this, err);
        };
        BufferTimeSubscriber.prototype._complete = function () {
            var _a = this,
                buffers = _a.buffers,
                destination = _a.destination;
            while (buffers.length > 0) {
                destination.next(buffers.shift());
            }
            _super.prototype._complete.call(this);
        };
        BufferTimeSubscriber.prototype._unsubscribe = function () {
            this.buffers = null;
        };
        BufferTimeSubscriber.prototype.openBuffer = function () {
            var buffer = [];
            this.buffers.push(buffer);
            return buffer;
        };
        BufferTimeSubscriber.prototype.closeBuffer = function (buffer) {
            this.destination.next(buffer);
            var buffers = this.buffers;
            buffers.splice(buffers.indexOf(buffer), 1);
        };
        return BufferTimeSubscriber;
    }(Subscriber_1.Subscriber);
    function dispatchBufferTimeSpanOnly(state) {
        var subscriber = state.subscriber;
        var prevBuffer = state.buffer;
        if (prevBuffer) {
            subscriber.closeBuffer(prevBuffer);
        }
        state.buffer = subscriber.openBuffer();
        if (!subscriber.isUnsubscribed) {
            this.schedule(state, state.bufferTimeSpan);
        }
    }
    function dispatchBufferCreation(state) {
        var bufferCreationInterval = state.bufferCreationInterval,
            bufferTimeSpan = state.bufferTimeSpan,
            subscriber = state.subscriber,
            scheduler = state.scheduler;
        var buffer = subscriber.openBuffer();
        var action = this;
        if (!subscriber.isUnsubscribed) {
            action.add(scheduler.schedule(dispatchBufferClose, bufferTimeSpan, { subscriber: subscriber, buffer: buffer }));
            action.schedule(state, bufferCreationInterval);
        }
    }
    function dispatchBufferClose(arg) {
        var subscriber = arg.subscriber,
            buffer = arg.buffer;
        subscriber.closeBuffer(buffer);
    }
    
});
System.registerDynamic('node_modules/rxjs/add/operator/bufferTime.js', ['../../Observable', '../../operator/bufferTime'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var bufferTime_1 = $__require('../../operator/bufferTime');
  Observable_1.Observable.prototype.bufferTime = bufferTime_1.bufferTime;
  
});
System.registerDynamic('node_modules/rxjs/operator/bufferToggle.js', ['../Subscription', '../util/subscribeToResult', '../OuterSubscriber'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscription_1 = $__require('../Subscription');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    /**
     * Buffers the source Observable values starting from an emission from
     * `openings` and ending when the output of `closingSelector` emits.
     *
     * <span class="informal">Collects values from the past as an array. Starts
     * collecting only when `opening` emits, and calls the `closingSelector`
     * function to get an Observable that tells when to close the buffer.</span>
     *
     * <img src="./img/bufferToggle.png" width="100%">
     *
     * Buffers values from the source by opening the buffer via signals from an
     * Observable provided to `openings`, and closing and sending the buffers when
     * a Subscribable or Promise returned by the `closingSelector` function emits.
     *
     * @example <caption>Every other second, emit the click events from the next 500ms</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var openings = Rx.Observable.interval(1000);
     * var buffered = clicks.bufferToggle(openings, i =>
     *   i % 2 ? Rx.Observable.interval(500) : Rx.Observable.empty()
     * );
     * buffered.subscribe(x => console.log(x));
     *
     * @see {@link buffer}
     * @see {@link bufferCount}
     * @see {@link bufferTime}
     * @see {@link bufferWhen}
     * @see {@link windowToggle}
     *
     * @param {SubscribableOrPromise<O>} openings A Subscribable or Promise of notifications to start new
     * buffers.
     * @param {function(value: O): SubscribableOrPromise} closingSelector A function that takes
     * the value emitted by the `openings` observable and returns a Subscribable or Promise,
     * which, when it emits, signals that the associated buffer should be emitted
     * and cleared.
     * @return {Observable<T[]>} An observable of arrays of buffered values.
     * @method bufferToggle
     * @owner Observable
     */
    function bufferToggle(openings, closingSelector) {
        return this.lift(new BufferToggleOperator(openings, closingSelector));
    }
    exports.bufferToggle = bufferToggle;
    var BufferToggleOperator = function () {
        function BufferToggleOperator(openings, closingSelector) {
            this.openings = openings;
            this.closingSelector = closingSelector;
        }
        BufferToggleOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new BufferToggleSubscriber(subscriber, this.openings, this.closingSelector));
        };
        return BufferToggleOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var BufferToggleSubscriber = function (_super) {
        __extends(BufferToggleSubscriber, _super);
        function BufferToggleSubscriber(destination, openings, closingSelector) {
            _super.call(this, destination);
            this.openings = openings;
            this.closingSelector = closingSelector;
            this.contexts = [];
            this.add(subscribeToResult_1.subscribeToResult(this, openings));
        }
        BufferToggleSubscriber.prototype._next = function (value) {
            var contexts = this.contexts;
            var len = contexts.length;
            for (var i = 0; i < len; i++) {
                contexts[i].buffer.push(value);
            }
        };
        BufferToggleSubscriber.prototype._error = function (err) {
            var contexts = this.contexts;
            while (contexts.length > 0) {
                var context = contexts.shift();
                context.subscription.unsubscribe();
                context.buffer = null;
                context.subscription = null;
            }
            this.contexts = null;
            _super.prototype._error.call(this, err);
        };
        BufferToggleSubscriber.prototype._complete = function () {
            var contexts = this.contexts;
            while (contexts.length > 0) {
                var context = contexts.shift();
                this.destination.next(context.buffer);
                context.subscription.unsubscribe();
                context.buffer = null;
                context.subscription = null;
            }
            this.contexts = null;
            _super.prototype._complete.call(this);
        };
        BufferToggleSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            outerValue ? this.closeBuffer(outerValue) : this.openBuffer(innerValue);
        };
        BufferToggleSubscriber.prototype.notifyComplete = function (innerSub) {
            this.closeBuffer(innerSub.context);
        };
        BufferToggleSubscriber.prototype.openBuffer = function (value) {
            try {
                var closingSelector = this.closingSelector;
                var closingNotifier = closingSelector.call(this, value);
                if (closingNotifier) {
                    this.trySubscribe(closingNotifier);
                }
            } catch (err) {
                this._error(err);
            }
        };
        BufferToggleSubscriber.prototype.closeBuffer = function (context) {
            var contexts = this.contexts;
            if (contexts && context) {
                var buffer = context.buffer,
                    subscription = context.subscription;
                this.destination.next(buffer);
                contexts.splice(contexts.indexOf(context), 1);
                this.remove(subscription);
                subscription.unsubscribe();
            }
        };
        BufferToggleSubscriber.prototype.trySubscribe = function (closingNotifier) {
            var contexts = this.contexts;
            var buffer = [];
            var subscription = new Subscription_1.Subscription();
            var context = { buffer: buffer, subscription: subscription };
            contexts.push(context);
            var innerSubscription = subscribeToResult_1.subscribeToResult(this, closingNotifier, context);
            if (!innerSubscription || innerSubscription.isUnsubscribed) {
                this.closeBuffer(context);
            } else {
                innerSubscription.context = context;
                this.add(innerSubscription);
                subscription.add(innerSubscription);
            }
        };
        return BufferToggleSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/bufferToggle.js', ['../../Observable', '../../operator/bufferToggle'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var bufferToggle_1 = $__require('../../operator/bufferToggle');
  Observable_1.Observable.prototype.bufferToggle = bufferToggle_1.bufferToggle;
  
});
System.registerDynamic('node_modules/rxjs/operator/bufferWhen.js', ['../Subscription', '../util/tryCatch', '../util/errorObject', '../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscription_1 = $__require('../Subscription');
    var tryCatch_1 = $__require('../util/tryCatch');
    var errorObject_1 = $__require('../util/errorObject');
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Buffers the source Observable values, using a factory function of closing
     * Observables to determine when to close, emit, and reset the buffer.
     *
     * <span class="informal">Collects values from the past as an array. When it
     * starts collecting values, it calls a function that returns an Observable that
     * tells when to close the buffer and restart collecting.</span>
     *
     * <img src="./img/bufferWhen.png" width="100%">
     *
     * Opens a buffer immediately, then closes the buffer when the observable
     * returned by calling `closingSelector` function emits a value. When it closes
     * the buffer, it immediately opens a new buffer and repeats the process.
     *
     * @example <caption>Emit an array of the last clicks every [1-5] random seconds</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var buffered = clicks.bufferWhen(() =>
     *   Rx.Observable.interval(1000 + Math.random() * 4000)
     * );
     * buffered.subscribe(x => console.log(x));
     *
     * @see {@link buffer}
     * @see {@link bufferCount}
     * @see {@link bufferTime}
     * @see {@link bufferToggle}
     * @see {@link windowWhen}
     *
     * @param {function(): Observable} closingSelector A function that takes no
     * arguments and returns an Observable that signals buffer closure.
     * @return {Observable<T[]>} An observable of arrays of buffered values.
     * @method bufferWhen
     * @owner Observable
     */
    function bufferWhen(closingSelector) {
        return this.lift(new BufferWhenOperator(closingSelector));
    }
    exports.bufferWhen = bufferWhen;
    var BufferWhenOperator = function () {
        function BufferWhenOperator(closingSelector) {
            this.closingSelector = closingSelector;
        }
        BufferWhenOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new BufferWhenSubscriber(subscriber, this.closingSelector));
        };
        return BufferWhenOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var BufferWhenSubscriber = function (_super) {
        __extends(BufferWhenSubscriber, _super);
        function BufferWhenSubscriber(destination, closingSelector) {
            _super.call(this, destination);
            this.closingSelector = closingSelector;
            this.subscribing = false;
            this.openBuffer();
        }
        BufferWhenSubscriber.prototype._next = function (value) {
            this.buffer.push(value);
        };
        BufferWhenSubscriber.prototype._complete = function () {
            var buffer = this.buffer;
            if (buffer) {
                this.destination.next(buffer);
            }
            _super.prototype._complete.call(this);
        };
        BufferWhenSubscriber.prototype._unsubscribe = function () {
            this.buffer = null;
            this.subscribing = false;
        };
        BufferWhenSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            this.openBuffer();
        };
        BufferWhenSubscriber.prototype.notifyComplete = function () {
            if (this.subscribing) {
                this.complete();
            } else {
                this.openBuffer();
            }
        };
        BufferWhenSubscriber.prototype.openBuffer = function () {
            var closingSubscription = this.closingSubscription;
            if (closingSubscription) {
                this.remove(closingSubscription);
                closingSubscription.unsubscribe();
            }
            var buffer = this.buffer;
            if (this.buffer) {
                this.destination.next(buffer);
            }
            this.buffer = [];
            var closingNotifier = tryCatch_1.tryCatch(this.closingSelector)();
            if (closingNotifier === errorObject_1.errorObject) {
                this.error(errorObject_1.errorObject.e);
            } else {
                closingSubscription = new Subscription_1.Subscription();
                this.closingSubscription = closingSubscription;
                this.add(closingSubscription);
                this.subscribing = true;
                closingSubscription.add(subscribeToResult_1.subscribeToResult(this, closingNotifier));
                this.subscribing = false;
            }
        };
        return BufferWhenSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/bufferWhen.js', ['../../Observable', '../../operator/bufferWhen'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var bufferWhen_1 = $__require('../../operator/bufferWhen');
  Observable_1.Observable.prototype.bufferWhen = bufferWhen_1.bufferWhen;
  
});
System.registerDynamic("node_modules/rxjs/operator/cache.js", ["./publishReplay"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var publishReplay_1 = $__require('./publishReplay');
  /**
   * @param bufferSize
   * @param windowTime
   * @param scheduler
   * @return {Observable<any>}
   * @method cache
   * @owner Observable
   */
  function cache(bufferSize, windowTime, scheduler) {
    if (bufferSize === void 0) {
      bufferSize = Number.POSITIVE_INFINITY;
    }
    if (windowTime === void 0) {
      windowTime = Number.POSITIVE_INFINITY;
    }
    return publishReplay_1.publishReplay.call(this, bufferSize, windowTime, scheduler).refCount();
  }
  exports.cache = cache;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/cache.js', ['../../Observable', '../../operator/cache'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var cache_1 = $__require('../../operator/cache');
  Observable_1.Observable.prototype.cache = cache_1.cache;
  
});
System.registerDynamic("node_modules/rxjs/operator/catch.js", ["../Subscriber"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * Catches errors on the observable to be handled by returning a new observable or throwing an error.
     * @param {function} selector a function that takes as arguments `err`, which is the error, and `caught`, which
     *  is the source observable, in case you'd like to "retry" that observable by returning it again. Whatever observable
     *  is returned by the `selector` will be used to continue the observable chain.
     * @return {Observable} an observable that originates from either the source or the observable returned by the
     *  catch `selector` function.
     * @method catch
     * @owner Observable
     */
    function _catch(selector) {
        var operator = new CatchOperator(selector);
        var caught = this.lift(operator);
        return operator.caught = caught;
    }
    exports._catch = _catch;
    var CatchOperator = function () {
        function CatchOperator(selector) {
            this.selector = selector;
        }
        CatchOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new CatchSubscriber(subscriber, this.selector, this.caught));
        };
        return CatchOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var CatchSubscriber = function (_super) {
        __extends(CatchSubscriber, _super);
        function CatchSubscriber(destination, selector, caught) {
            _super.call(this, destination);
            this.selector = selector;
            this.caught = caught;
        }
        // NOTE: overriding `error` instead of `_error` because we don't want
        // to have this flag this subscriber as `isStopped`.
        CatchSubscriber.prototype.error = function (err) {
            if (!this.isStopped) {
                var result = void 0;
                try {
                    result = this.selector(err, this.caught);
                } catch (err) {
                    this.destination.error(err);
                    return;
                }
                this._innerSub(result);
            }
        };
        CatchSubscriber.prototype._innerSub = function (result) {
            this.unsubscribe();
            this.destination.remove(this);
            result.subscribe(this.destination);
        };
        return CatchSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/catch.js', ['../../Observable', '../../operator/catch'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var catch_1 = $__require('../../operator/catch');
  Observable_1.Observable.prototype.catch = catch_1._catch;
  
});
System.registerDynamic("node_modules/rxjs/operator/combineAll.js", ["./combineLatest"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var combineLatest_1 = $__require('./combineLatest');
  /**
   * Converts a higher-order Observable into a first-order Observable by waiting
   * for the outer Observable to complete, then applying {@link combineLatest}.
   *
   * <span class="informal">Flattens an Observable-of-Observables by applying
   * {@link combineLatest} when the Observable-of-Observables completes.</span>
   *
   * <img src="./img/combineAll.png" width="100%">
   *
   * Takes an Observable of Observables, and collects all Observables from it.
   * Once the outer Observable completes, it subscribes to all collected
   * Observables and combines their values using the {@link combineLatest}
   * strategy, such that:
   * - Every time an inner Observable emits, the output Observable emits.
   * - When the returned observable emits, it emits all of the latest values by:
   *   - If a `project` function is provided, it is called with each recent value
   *     from each inner Observable in whatever order they arrived, and the result
   *     of the `project` function is what is emitted by the output Observable.
   *   - If there is no `project` function, an array of all of the most recent
   *     values is emitted by the output Observable.
   *
   * @example <caption>Map two click events to a finite interval Observable, then apply combineAll</caption>
   * var clicks = Rx.Observable.fromEvent(document, 'click');
   * var higherOrder = clicks.map(ev =>
   *   Rx.Observable.interval(Math.random()*2000).take(3)
   * ).take(2);
   * var result = higherOrder.combineAll();
   * result.subscribe(x => console.log(x));
   *
   * @see {@link combineLatest}
   * @see {@link mergeAll}
   *
   * @param {function} [project] An optional function to map the most recent
   * values from each inner Observable into a new result. Takes each of the most
   * recent values from each collected inner Observable as arguments, in order.
   * @return {Observable} An Observable of projected results or arrays of recent
   * values.
   * @method combineAll
   * @owner Observable
   */
  function combineAll(project) {
    return this.lift(new combineLatest_1.CombineLatestOperator(project));
  }
  exports.combineAll = combineAll;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/combineAll.js', ['../../Observable', '../../operator/combineAll'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var combineAll_1 = $__require('../../operator/combineAll');
  Observable_1.Observable.prototype.combineAll = combineAll_1.combineAll;
  
});
System.registerDynamic('node_modules/rxjs/operator/combineLatest.js', ['../observable/ArrayObservable', '../util/isArray', '../util/isScheduler', '../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var ArrayObservable_1 = $__require('../observable/ArrayObservable');
    var isArray_1 = $__require('../util/isArray');
    var isScheduler_1 = $__require('../util/isScheduler');
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Combines multiple Observables to create an Observable whose values are
     * calculated from the latest values of each of its input Observables.
     *
     * <span class="informal">Whenever any input Observable emits a value, it
     * computes a formula using the latest values from all the inputs, then emits
     * the output of that formula.</span>
     *
     * <img src="./img/combineLatest.png" width="100%">
     *
     * `combineLatest` combines the values from this Observable with values from
     * Observables passed as arguments. This is done by subscribing to each
     * Observable, in order, and collecting an array of each of the most recent
     * values any time any of the input Observables emits, then either taking that
     * array and passing it as arguments to an optional `project` function and
     * emitting the return value of that, or just emitting the array of recent
     * values directly if there is no `project` function.
     *
     * @example <caption>Dynamically calculate the Body-Mass Index from an Observable of weight and one for height</caption>
     * var weight = Rx.Observable.of(70, 72, 76, 79, 75);
     * var height = Rx.Observable.of(1.76, 1.77, 1.78);
     * var bmi = weight.combineLatest(height, (w, h) => w / (h * h));
     * bmi.subscribe(x => console.log('BMI is ' + x));
     *
     * @see {@link combineAll}
     * @see {@link merge}
     * @see {@link withLatestFrom}
     *
     * @param {Observable} other An input Observable to combine with the source
     * Observable. More than one input Observables may be given as argument.
     * @param {function} [project] An optional function to project the values from
     * the combined latest values into a new value on the output Observable.
     * @return {Observable} An Observable of projected values from the most recent
     * values from each input Observable, or an array of the most recent values from
     * each input Observable.
     * @method combineLatest
     * @owner Observable
     */
    function combineLatest() {
        var observables = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            observables[_i - 0] = arguments[_i];
        }
        var project = null;
        if (typeof observables[observables.length - 1] === 'function') {
            project = observables.pop();
        }
        // if the first and only other argument besides the resultSelector is an array
        // assume it's been called with `combineLatest([obs1, obs2, obs3], project)`
        if (observables.length === 1 && isArray_1.isArray(observables[0])) {
            observables = observables[0];
        }
        observables.unshift(this);
        return new ArrayObservable_1.ArrayObservable(observables).lift(new CombineLatestOperator(project));
    }
    exports.combineLatest = combineLatest;
    /* tslint:enable:max-line-length */
    /**
     * Combines the values from observables passed as arguments. This is done by subscribing
     * to each observable, in order, and collecting an array of each of the most recent values any time any of the observables
     * emits, then either taking that array and passing it as arguments to an option `project` function and emitting the return
     * value of that, or just emitting the array of recent values directly if there is no `project` function.
     * @param {...Observable} observables the observables to combine
     * @param {function} [project] an optional function to project the values from the combined recent values into a new value for emission.
     * @return {Observable} an observable of other projected values from the most recent values from each observable, or an array of each of
     * the most recent values from each observable.
     * @static true
     * @name combineLatest
     * @owner Observable
     */
    function combineLatestStatic() {
        var observables = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            observables[_i - 0] = arguments[_i];
        }
        var project = null;
        var scheduler = null;
        if (isScheduler_1.isScheduler(observables[observables.length - 1])) {
            scheduler = observables.pop();
        }
        if (typeof observables[observables.length - 1] === 'function') {
            project = observables.pop();
        }
        // if the first and only other argument besides the resultSelector is an array
        // assume it's been called with `combineLatest([obs1, obs2, obs3], project)`
        if (observables.length === 1 && isArray_1.isArray(observables[0])) {
            observables = observables[0];
        }
        return new ArrayObservable_1.ArrayObservable(observables, scheduler).lift(new CombineLatestOperator(project));
    }
    exports.combineLatestStatic = combineLatestStatic;
    var CombineLatestOperator = function () {
        function CombineLatestOperator(project) {
            this.project = project;
        }
        CombineLatestOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new CombineLatestSubscriber(subscriber, this.project));
        };
        return CombineLatestOperator;
    }();
    exports.CombineLatestOperator = CombineLatestOperator;
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var CombineLatestSubscriber = function (_super) {
        __extends(CombineLatestSubscriber, _super);
        function CombineLatestSubscriber(destination, project) {
            _super.call(this, destination);
            this.project = project;
            this.active = 0;
            this.values = [];
            this.observables = [];
            this.toRespond = [];
        }
        CombineLatestSubscriber.prototype._next = function (observable) {
            var toRespond = this.toRespond;
            toRespond.push(toRespond.length);
            this.observables.push(observable);
        };
        CombineLatestSubscriber.prototype._complete = function () {
            var observables = this.observables;
            var len = observables.length;
            if (len === 0) {
                this.destination.complete();
            } else {
                this.active = len;
                for (var i = 0; i < len; i++) {
                    var observable = observables[i];
                    this.add(subscribeToResult_1.subscribeToResult(this, observable, observable, i));
                }
            }
        };
        CombineLatestSubscriber.prototype.notifyComplete = function (unused) {
            if ((this.active -= 1) === 0) {
                this.destination.complete();
            }
        };
        CombineLatestSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            var values = this.values;
            values[outerIndex] = innerValue;
            var toRespond = this.toRespond;
            if (toRespond.length > 0) {
                var found = toRespond.indexOf(outerIndex);
                if (found !== -1) {
                    toRespond.splice(found, 1);
                }
            }
            if (toRespond.length === 0) {
                if (this.project) {
                    this._tryProject(values);
                } else {
                    this.destination.next(values);
                }
            }
        };
        CombineLatestSubscriber.prototype._tryProject = function (values) {
            var result;
            try {
                result = this.project.apply(this, values);
            } catch (err) {
                this.destination.error(err);
                return;
            }
            this.destination.next(result);
        };
        return CombineLatestSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    exports.CombineLatestSubscriber = CombineLatestSubscriber;
    
});
System.registerDynamic('node_modules/rxjs/add/operator/combineLatest.js', ['../../Observable', '../../operator/combineLatest'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var combineLatest_1 = $__require('../../operator/combineLatest');
  Observable_1.Observable.prototype.combineLatest = combineLatest_1.combineLatest;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/concat.js', ['../../Observable', '../../operator/concat'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var concat_1 = $__require('../../operator/concat');
  Observable_1.Observable.prototype.concat = concat_1.concat;
  
});
System.registerDynamic("node_modules/rxjs/operator/concatAll.js", ["./mergeAll"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var mergeAll_1 = $__require('./mergeAll');
  /**
   * Converts a higher-order Observable into a first-order Observable by
   * concatenating the inner Observables in order.
   *
   * <span class="informal">Flattens an Observable-of-Observables by putting one
   * inner Observable after the other.</span>
   *
   * <img src="./img/concatAll.png" width="100%">
   *
   * Joins every Observable emitted by the source (a higher-order Observable), in
   * a serial fashion. It subscribes to each inner Observable only after the
   * previous inner Observable has completed, and merges all of their values into
   * the returned observable.
   *
   * __Warning:__ If the source Observable emits Observables quickly and
   * endlessly, and the inner Observables it emits generally complete slower than
   * the source emits, you can run into memory issues as the incoming Observables
   * collect in an unbounded buffer.
   *
   * Note: `concatAll` is equivalent to `mergeAll` with concurrency parameter set
   * to `1`.
   *
   * @example <caption>For each click event, tick every second from 0 to 3, with no concurrency</caption>
   * var clicks = Rx.Observable.fromEvent(document, 'click');
   * var higherOrder = clicks.map(ev => Rx.Observable.interval(1000).take(4));
   * var firstOrder = higherOrder.concatAll();
   * firstOrder.subscribe(x => console.log(x));
   *
   * @see {@link combineAll}
   * @see {@link concat}
   * @see {@link concatMap}
   * @see {@link concatMapTo}
   * @see {@link exhaust}
   * @see {@link mergeAll}
   * @see {@link switch}
   * @see {@link zipAll}
   *
   * @return {Observable} An Observable emitting values from all the inner
   * Observables concatenated.
   * @method concatAll
   * @owner Observable
   */
  function concatAll() {
    return this.lift(new mergeAll_1.MergeAllOperator(1));
  }
  exports.concatAll = concatAll;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/concatAll.js', ['../../Observable', '../../operator/concatAll'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var concatAll_1 = $__require('../../operator/concatAll');
  Observable_1.Observable.prototype.concatAll = concatAll_1.concatAll;
  
});
System.registerDynamic("node_modules/rxjs/operator/concatMap.js", ["./mergeMap"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var mergeMap_1 = $__require('./mergeMap');
  /**
   * Projects each source value to an Observable which is merged in the output
   * Observable, in a serialized fashion waiting for each one to complete before
   * merging the next.
   *
   * <span class="informal">Maps each value to an Observable, then flattens all of
   * these inner Observables using {@link concatAll}.</span>
   *
   * <img src="./img/concatMap.png" width="100%">
   *
   * Returns an Observable that emits items based on applying a function that you
   * supply to each item emitted by the source Observable, where that function
   * returns an (so-called "inner") Observable. Each new inner Observable is
   * concatenated with the previous inner Observable.
   *
   * __Warning:__ if source values arrive endlessly and faster than their
   * corresponding inner Observables can complete, it will result in memory issues
   * as inner Observables amass in an unbounded buffer waiting for their turn to
   * be subscribed to.
   *
   * Note: `concatMap` is equivalent to `mergeMap` with concurrency parameter set
   * to `1`.
   *
   * @example <caption>For each click event, tick every second from 0 to 3, with no concurrency</caption>
   * var clicks = Rx.Observable.fromEvent(document, 'click');
   * var result = clicks.concatMap(ev => Rx.Observable.interval(1000).take(4));
   * result.subscribe(x => console.log(x));
   *
   * @see {@link concat}
   * @see {@link concatAll}
   * @see {@link concatMapTo}
   * @see {@link exhaustMap}
   * @see {@link mergeMap}
   * @see {@link switchMap}
   *
   * @param {function(value: T, ?index: number): Observable} project A function
   * that, when applied to an item emitted by the source Observable, returns an
   * Observable.
   * @param {function(outerValue: T, innerValue: I, outerIndex: number, innerIndex: number): any} [resultSelector]
   * A function to produce the value on the output Observable based on the values
   * and the indices of the source (outer) emission and the inner Observable
   * emission. The arguments passed to this function are:
   * - `outerValue`: the value that came from the source
   * - `innerValue`: the value that came from the projected Observable
   * - `outerIndex`: the "index" of the value that came from the source
   * - `innerIndex`: the "index" of the value from the projected Observable
   * @return {Observable} an observable of values merged from the projected
   * Observables as they were subscribed to, one at a time. Optionally, these
   * values may have been projected from a passed `projectResult` argument.
   * @return {Observable} An Observable that emits the result of applying the
   * projection function (and the optional `resultSelector`) to each item emitted
   * by the source Observable and taking values from each projected inner
   * Observable sequentially.
   * @method concatMap
   * @owner Observable
   */
  function concatMap(project, resultSelector) {
    return this.lift(new mergeMap_1.MergeMapOperator(project, resultSelector, 1));
  }
  exports.concatMap = concatMap;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/concatMap.js', ['../../Observable', '../../operator/concatMap'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var concatMap_1 = $__require('../../operator/concatMap');
  Observable_1.Observable.prototype.concatMap = concatMap_1.concatMap;
  
});
System.registerDynamic("node_modules/rxjs/operator/concatMapTo.js", ["./mergeMapTo"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var mergeMapTo_1 = $__require('./mergeMapTo');
  /**
   * Projects each source value to the same Observable which is merged multiple
   * times in a serialized fashion on the output Observable.
   *
   * <span class="informal">It's like {@link concatMap}, but maps each value
   * always to the same inner Observable.</span>
   *
   * <img src="./img/concatMapTo.png" width="100%">
   *
   * Maps each source value to the given Observable `innerObservable` regardless
   * of the source value, and then flattens those resulting Observables into one
   * single Observable, which is the output Observable. Each new `innerObservable`
   * instance emitted on the output Observable is concatenated with the previous
   * `innerObservable` instance.
   *
   * __Warning:__ if source values arrive endlessly and faster than their
   * corresponding inner Observables can complete, it will result in memory issues
   * as inner Observables amass in an unbounded buffer waiting for their turn to
   * be subscribed to.
   *
   * Note: `concatMapTo` is equivalent to `mergeMapTo` with concurrency parameter
   * set to `1`.
   *
   * @example <caption>For each click event, tick every second from 0 to 3, with no concurrency</caption>
   * var clicks = Rx.Observable.fromEvent(document, 'click');
   * var result = clicks.concatMapTo(Rx.Observable.interval(1000).take(4));
   * result.subscribe(x => console.log(x));
   *
   * @see {@link concat}
   * @see {@link concatAll}
   * @see {@link concatMap}
   * @see {@link mergeMapTo}
   * @see {@link switchMapTo}
   *
   * @param {Observable} innerObservable An Observable to replace each value from
   * the source Observable.
   * @param {function(outerValue: T, innerValue: I, outerIndex: number, innerIndex: number): any} [resultSelector]
   * A function to produce the value on the output Observable based on the values
   * and the indices of the source (outer) emission and the inner Observable
   * emission. The arguments passed to this function are:
   * - `outerValue`: the value that came from the source
   * - `innerValue`: the value that came from the projected Observable
   * - `outerIndex`: the "index" of the value that came from the source
   * - `innerIndex`: the "index" of the value from the projected Observable
   * @return {Observable} An observable of values merged together by joining the
   * passed observable with itself, one after the other, for each value emitted
   * from the source.
   * @method concatMapTo
   * @owner Observable
   */
  function concatMapTo(innerObservable, resultSelector) {
    return this.lift(new mergeMapTo_1.MergeMapToOperator(innerObservable, resultSelector, 1));
  }
  exports.concatMapTo = concatMapTo;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/concatMapTo.js', ['../../Observable', '../../operator/concatMapTo'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var concatMapTo_1 = $__require('../../operator/concatMapTo');
  Observable_1.Observable.prototype.concatMapTo = concatMapTo_1.concatMapTo;
  
});
System.registerDynamic("node_modules/rxjs/operator/count.js", ["../Subscriber"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * Counts the number of emissions on the source and emits that number when the
     * source completes.
     *
     * <span class="informal">Tells how many values were emitted, when the source
     * completes.</span>
     *
     * <img src="./img/count.png" width="100%">
     *
     * `count` transforms an Observable that emits values into an Observable that
     * emits a single value that represents the number of values emitted by the
     * source Observable. If the source Observable terminates with an error, `count`
     * will pass this error notification along without emitting an value first. If
     * the source Observable does not terminate at all, `count` will neither emit
     * a value nor terminate. This operator takes an optional `predicate` function
     * as argument, in which case the output emission will represent the number of
     * source values that matched `true` with the `predicate`.
     *
     * @example <caption>Counts how many seconds have passed before the first click happened</caption>
     * var seconds = Rx.Observable.interval(1000);
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var secondsBeforeClick = seconds.takeUntil(clicks);
     * var result = secondsBeforeClick.count();
     * result.subscribe(x => console.log(x));
     *
     * @example <caption>Counts how many odd numbers are there between 1 and 7</caption>
     * var numbers = Rx.Observable.range(1, 7);
     * var result = numbers.count(i => i % 2 === 1);
     * result.subscribe(x => console.log(x));
     *
     * @see {@link max}
     * @see {@link min}
     * @see {@link reduce}
     *
     * @param {function(value: T, i: number, source: Observable<T>): boolean} [predicate] A
     * boolean function to select what values are to be counted. It is provided with
     * arguments of:
     * - `value`: the value from the source Observable.
     * - `index`: the (zero-based) "index" of the value from the source Observable.
     * - `source`: the source Observable instance itself.
     * @return {Observable} An Observable of one number that represents the count as
     * described above.
     * @method count
     * @owner Observable
     */
    function count(predicate) {
        return this.lift(new CountOperator(predicate, this));
    }
    exports.count = count;
    var CountOperator = function () {
        function CountOperator(predicate, source) {
            this.predicate = predicate;
            this.source = source;
        }
        CountOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new CountSubscriber(subscriber, this.predicate, this.source));
        };
        return CountOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var CountSubscriber = function (_super) {
        __extends(CountSubscriber, _super);
        function CountSubscriber(destination, predicate, source) {
            _super.call(this, destination);
            this.predicate = predicate;
            this.source = source;
            this.count = 0;
            this.index = 0;
        }
        CountSubscriber.prototype._next = function (value) {
            if (this.predicate) {
                this._tryPredicate(value);
            } else {
                this.count++;
            }
        };
        CountSubscriber.prototype._tryPredicate = function (value) {
            var result;
            try {
                result = this.predicate(value, this.index++, this.source);
            } catch (err) {
                this.destination.error(err);
                return;
            }
            if (result) {
                this.count++;
            }
        };
        CountSubscriber.prototype._complete = function () {
            this.destination.next(this.count);
            this.destination.complete();
        };
        return CountSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/count.js', ['../../Observable', '../../operator/count'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var count_1 = $__require('../../operator/count');
  Observable_1.Observable.prototype.count = count_1.count;
  
});
System.registerDynamic("node_modules/rxjs/operator/dematerialize.js", ["../Subscriber"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * Returns an Observable that transforms Notification objects into the items or notifications they represent.
     *
     * @see {@link Notification}
     *
     * @return {Observable} an Observable that emits items and notifications embedded in Notification objects emitted by the source Observable.
     * @method dematerialize
     * @owner Observable
     */
    function dematerialize() {
        return this.lift(new DeMaterializeOperator());
    }
    exports.dematerialize = dematerialize;
    var DeMaterializeOperator = function () {
        function DeMaterializeOperator() {}
        DeMaterializeOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new DeMaterializeSubscriber(subscriber));
        };
        return DeMaterializeOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var DeMaterializeSubscriber = function (_super) {
        __extends(DeMaterializeSubscriber, _super);
        function DeMaterializeSubscriber(destination) {
            _super.call(this, destination);
        }
        DeMaterializeSubscriber.prototype._next = function (value) {
            value.observe(this.destination);
        };
        return DeMaterializeSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/dematerialize.js', ['../../Observable', '../../operator/dematerialize'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var dematerialize_1 = $__require('../../operator/dematerialize');
  Observable_1.Observable.prototype.dematerialize = dematerialize_1.dematerialize;
  
});
System.registerDynamic('node_modules/rxjs/operator/debounce.js', ['../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Returns the source Observable delayed by the computed debounce duration,
     * with the duration lengthened if a new source item arrives before the delay
     * duration ends.
     * In practice, for each item emitted on the source, this operator holds the
     * latest item, waits for a silence as long as the `durationSelector` specifies,
     * and only then emits the latest source item on the result Observable.
     * @param {function} durationSelector function for computing the timeout duration for each item.
     * @return {Observable} an Observable the same as source Observable, but drops items.
     * @method debounce
     * @owner Observable
     */
    function debounce(durationSelector) {
        return this.lift(new DebounceOperator(durationSelector));
    }
    exports.debounce = debounce;
    var DebounceOperator = function () {
        function DebounceOperator(durationSelector) {
            this.durationSelector = durationSelector;
        }
        DebounceOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new DebounceSubscriber(subscriber, this.durationSelector));
        };
        return DebounceOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var DebounceSubscriber = function (_super) {
        __extends(DebounceSubscriber, _super);
        function DebounceSubscriber(destination, durationSelector) {
            _super.call(this, destination);
            this.durationSelector = durationSelector;
            this.hasValue = false;
            this.durationSubscription = null;
        }
        DebounceSubscriber.prototype._next = function (value) {
            try {
                var result = this.durationSelector.call(this, value);
                if (result) {
                    this._tryNext(value, result);
                }
            } catch (err) {
                this.destination.error(err);
            }
        };
        DebounceSubscriber.prototype._complete = function () {
            this.emitValue();
            this.destination.complete();
        };
        DebounceSubscriber.prototype._tryNext = function (value, duration) {
            var subscription = this.durationSubscription;
            this.value = value;
            this.hasValue = true;
            if (subscription) {
                subscription.unsubscribe();
                this.remove(subscription);
            }
            subscription = subscribeToResult_1.subscribeToResult(this, duration);
            if (!subscription.isUnsubscribed) {
                this.add(this.durationSubscription = subscription);
            }
        };
        DebounceSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            this.emitValue();
        };
        DebounceSubscriber.prototype.notifyComplete = function () {
            this.emitValue();
        };
        DebounceSubscriber.prototype.emitValue = function () {
            if (this.hasValue) {
                var value = this.value;
                var subscription = this.durationSubscription;
                if (subscription) {
                    this.durationSubscription = null;
                    subscription.unsubscribe();
                    this.remove(subscription);
                }
                this.value = null;
                this.hasValue = false;
                _super.prototype._next.call(this, value);
            }
        };
        return DebounceSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/debounce.js', ['../../Observable', '../../operator/debounce'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var debounce_1 = $__require('../../operator/debounce');
  Observable_1.Observable.prototype.debounce = debounce_1.debounce;
  
});
System.registerDynamic('node_modules/rxjs/operator/debounceTime.js', ['../Subscriber', '../scheduler/async'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var async_1 = $__require('../scheduler/async');
    /**
     * Returns the source Observable delayed by the computed debounce duration,
     * with the duration lengthened if a new source item arrives before the delay
     * duration ends.
     * In practice, for each item emitted on the source, this operator holds the
     * latest item, waits for a silence for the `dueTime` length, and only then
     * emits the latest source item on the result Observable.
     * Optionally takes a scheduler for manging timers.
     * @param {number} dueTime the timeout value for the window of time required to not drop the item.
     * @param {Scheduler} [scheduler] the Scheduler to use for managing the timers that handle the timeout for each item.
     * @return {Observable} an Observable the same as source Observable, but drops items.
     * @method debounceTime
     * @owner Observable
     */
    function debounceTime(dueTime, scheduler) {
        if (scheduler === void 0) {
            scheduler = async_1.async;
        }
        return this.lift(new DebounceTimeOperator(dueTime, scheduler));
    }
    exports.debounceTime = debounceTime;
    var DebounceTimeOperator = function () {
        function DebounceTimeOperator(dueTime, scheduler) {
            this.dueTime = dueTime;
            this.scheduler = scheduler;
        }
        DebounceTimeOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new DebounceTimeSubscriber(subscriber, this.dueTime, this.scheduler));
        };
        return DebounceTimeOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var DebounceTimeSubscriber = function (_super) {
        __extends(DebounceTimeSubscriber, _super);
        function DebounceTimeSubscriber(destination, dueTime, scheduler) {
            _super.call(this, destination);
            this.dueTime = dueTime;
            this.scheduler = scheduler;
            this.debouncedSubscription = null;
            this.lastValue = null;
            this.hasValue = false;
        }
        DebounceTimeSubscriber.prototype._next = function (value) {
            this.clearDebounce();
            this.lastValue = value;
            this.hasValue = true;
            this.add(this.debouncedSubscription = this.scheduler.schedule(dispatchNext, this.dueTime, this));
        };
        DebounceTimeSubscriber.prototype._complete = function () {
            this.debouncedNext();
            this.destination.complete();
        };
        DebounceTimeSubscriber.prototype.debouncedNext = function () {
            this.clearDebounce();
            if (this.hasValue) {
                this.destination.next(this.lastValue);
                this.lastValue = null;
                this.hasValue = false;
            }
        };
        DebounceTimeSubscriber.prototype.clearDebounce = function () {
            var debouncedSubscription = this.debouncedSubscription;
            if (debouncedSubscription !== null) {
                this.remove(debouncedSubscription);
                debouncedSubscription.unsubscribe();
                this.debouncedSubscription = null;
            }
        };
        return DebounceTimeSubscriber;
    }(Subscriber_1.Subscriber);
    function dispatchNext(subscriber) {
        subscriber.debouncedNext();
    }
    
});
System.registerDynamic('node_modules/rxjs/add/operator/debounceTime.js', ['../../Observable', '../../operator/debounceTime'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var debounceTime_1 = $__require('../../operator/debounceTime');
  Observable_1.Observable.prototype.debounceTime = debounceTime_1.debounceTime;
  
});
System.registerDynamic("node_modules/rxjs/operator/defaultIfEmpty.js", ["../Subscriber"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * Returns an Observable that emits the elements of the source or a specified default value if empty.
     * @param {any} defaultValue the default value used if source is empty; defaults to null.
     * @return {Observable} an Observable of the items emitted by the where empty values are replaced by the specified default value or null.
     * @method defaultIfEmpty
     * @owner Observable
     */
    function defaultIfEmpty(defaultValue) {
        if (defaultValue === void 0) {
            defaultValue = null;
        }
        return this.lift(new DefaultIfEmptyOperator(defaultValue));
    }
    exports.defaultIfEmpty = defaultIfEmpty;
    var DefaultIfEmptyOperator = function () {
        function DefaultIfEmptyOperator(defaultValue) {
            this.defaultValue = defaultValue;
        }
        DefaultIfEmptyOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new DefaultIfEmptySubscriber(subscriber, this.defaultValue));
        };
        return DefaultIfEmptyOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var DefaultIfEmptySubscriber = function (_super) {
        __extends(DefaultIfEmptySubscriber, _super);
        function DefaultIfEmptySubscriber(destination, defaultValue) {
            _super.call(this, destination);
            this.defaultValue = defaultValue;
            this.isEmpty = true;
        }
        DefaultIfEmptySubscriber.prototype._next = function (value) {
            this.isEmpty = false;
            this.destination.next(value);
        };
        DefaultIfEmptySubscriber.prototype._complete = function () {
            if (this.isEmpty) {
                this.destination.next(this.defaultValue);
            }
            this.destination.complete();
        };
        return DefaultIfEmptySubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/defaultIfEmpty.js', ['../../Observable', '../../operator/defaultIfEmpty'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var defaultIfEmpty_1 = $__require('../../operator/defaultIfEmpty');
  Observable_1.Observable.prototype.defaultIfEmpty = defaultIfEmpty_1.defaultIfEmpty;
  
});
System.registerDynamic('node_modules/rxjs/operator/delay.js', ['../scheduler/async', '../util/isDate', '../Subscriber', '../Notification'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var async_1 = $__require('../scheduler/async');
    var isDate_1 = $__require('../util/isDate');
    var Subscriber_1 = $__require('../Subscriber');
    var Notification_1 = $__require('../Notification');
    /**
     * Delays the emission of items from the source Observable by a given timeout or
     * until a given Date.
     *
     * <span class="informal">Time shifts each item by some specified amount of
     * milliseconds.</span>
     *
     * <img src="./img/delay.png" width="100%">
     *
     * If the delay argument is a Number, this operator time shifts the source
     * Observable by that amount of time expressed in milliseconds. The relative
     * time intervals between the values are preserved.
     *
     * If the delay argument is a Date, this operator time shifts the start of the
     * Observable execution until the given date occurs.
     *
     * @example <caption>Delay each click by one second</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var delayedClicks = clicks.delay(1000); // each click emitted after 1 second
     * delayedClicks.subscribe(x => console.log(x));
     *
     * @example <caption>Delay all clicks until a future date happens</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var date = new Date('March 15, 2050 12:00:00'); // in the future
     * var delayedClicks = clicks.delay(date); // click emitted only after that date
     * delayedClicks.subscribe(x => console.log(x));
     *
     * @see {@link debounceTime}
     * @see {@link delayWhen}
     *
     * @param {number|Date} delay The delay duration in milliseconds (a `number`) or
     * a `Date` until which the emission of the source items is delayed.
     * @param {Scheduler} [scheduler=async] The Scheduler to use for
     * managing the timers that handle the time-shift for each item.
     * @return {Observable} An Observable that delays the emissions of the source
     * Observable by the specified timeout or Date.
     * @method delay
     * @owner Observable
     */
    function delay(delay, scheduler) {
        if (scheduler === void 0) {
            scheduler = async_1.async;
        }
        var absoluteDelay = isDate_1.isDate(delay);
        var delayFor = absoluteDelay ? +delay - scheduler.now() : Math.abs(delay);
        return this.lift(new DelayOperator(delayFor, scheduler));
    }
    exports.delay = delay;
    var DelayOperator = function () {
        function DelayOperator(delay, scheduler) {
            this.delay = delay;
            this.scheduler = scheduler;
        }
        DelayOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new DelaySubscriber(subscriber, this.delay, this.scheduler));
        };
        return DelayOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var DelaySubscriber = function (_super) {
        __extends(DelaySubscriber, _super);
        function DelaySubscriber(destination, delay, scheduler) {
            _super.call(this, destination);
            this.delay = delay;
            this.scheduler = scheduler;
            this.queue = [];
            this.active = false;
            this.errored = false;
        }
        DelaySubscriber.dispatch = function (state) {
            var source = state.source;
            var queue = source.queue;
            var scheduler = state.scheduler;
            var destination = state.destination;
            while (queue.length > 0 && queue[0].time - scheduler.now() <= 0) {
                queue.shift().notification.observe(destination);
            }
            if (queue.length > 0) {
                var delay_1 = Math.max(0, queue[0].time - scheduler.now());
                this.schedule(state, delay_1);
            } else {
                source.active = false;
            }
        };
        DelaySubscriber.prototype._schedule = function (scheduler) {
            this.active = true;
            this.add(scheduler.schedule(DelaySubscriber.dispatch, this.delay, {
                source: this, destination: this.destination, scheduler: scheduler
            }));
        };
        DelaySubscriber.prototype.scheduleNotification = function (notification) {
            if (this.errored === true) {
                return;
            }
            var scheduler = this.scheduler;
            var message = new DelayMessage(scheduler.now() + this.delay, notification);
            this.queue.push(message);
            if (this.active === false) {
                this._schedule(scheduler);
            }
        };
        DelaySubscriber.prototype._next = function (value) {
            this.scheduleNotification(Notification_1.Notification.createNext(value));
        };
        DelaySubscriber.prototype._error = function (err) {
            this.errored = true;
            this.queue = [];
            this.destination.error(err);
        };
        DelaySubscriber.prototype._complete = function () {
            this.scheduleNotification(Notification_1.Notification.createComplete());
        };
        return DelaySubscriber;
    }(Subscriber_1.Subscriber);
    var DelayMessage = function () {
        function DelayMessage(time, notification) {
            this.time = time;
            this.notification = notification;
        }
        return DelayMessage;
    }();
    
});
System.registerDynamic('node_modules/rxjs/add/operator/delay.js', ['../../Observable', '../../operator/delay'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var delay_1 = $__require('../../operator/delay');
  Observable_1.Observable.prototype.delay = delay_1.delay;
  
});
System.registerDynamic('node_modules/rxjs/operator/delayWhen.js', ['../Subscriber', '../Observable', '../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var Observable_1 = $__require('../Observable');
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Returns an Observable that delays the emission of items from the source Observable
     * by a subscription delay and a delay selector function for each element.
     * @param {Function} selector function to retrieve a sequence indicating the delay for each given element.
     * @param {Observable} sequence indicating the delay for the subscription to the source.
     * @return {Observable} an Observable that delays the emissions of the source Observable by the specified timeout or Date.
     * @method delayWhen
     * @owner Observable
     */
    function delayWhen(delayDurationSelector, subscriptionDelay) {
        if (subscriptionDelay) {
            return new SubscriptionDelayObservable(this, subscriptionDelay).lift(new DelayWhenOperator(delayDurationSelector));
        }
        return this.lift(new DelayWhenOperator(delayDurationSelector));
    }
    exports.delayWhen = delayWhen;
    var DelayWhenOperator = function () {
        function DelayWhenOperator(delayDurationSelector) {
            this.delayDurationSelector = delayDurationSelector;
        }
        DelayWhenOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new DelayWhenSubscriber(subscriber, this.delayDurationSelector));
        };
        return DelayWhenOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var DelayWhenSubscriber = function (_super) {
        __extends(DelayWhenSubscriber, _super);
        function DelayWhenSubscriber(destination, delayDurationSelector) {
            _super.call(this, destination);
            this.delayDurationSelector = delayDurationSelector;
            this.completed = false;
            this.delayNotifierSubscriptions = [];
            this.values = [];
        }
        DelayWhenSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            this.destination.next(outerValue);
            this.removeSubscription(innerSub);
            this.tryComplete();
        };
        DelayWhenSubscriber.prototype.notifyError = function (error, innerSub) {
            this._error(error);
        };
        DelayWhenSubscriber.prototype.notifyComplete = function (innerSub) {
            var value = this.removeSubscription(innerSub);
            if (value) {
                this.destination.next(value);
            }
            this.tryComplete();
        };
        DelayWhenSubscriber.prototype._next = function (value) {
            try {
                var delayNotifier = this.delayDurationSelector(value);
                if (delayNotifier) {
                    this.tryDelay(delayNotifier, value);
                }
            } catch (err) {
                this.destination.error(err);
            }
        };
        DelayWhenSubscriber.prototype._complete = function () {
            this.completed = true;
            this.tryComplete();
        };
        DelayWhenSubscriber.prototype.removeSubscription = function (subscription) {
            subscription.unsubscribe();
            var subscriptionIdx = this.delayNotifierSubscriptions.indexOf(subscription);
            var value = null;
            if (subscriptionIdx !== -1) {
                value = this.values[subscriptionIdx];
                this.delayNotifierSubscriptions.splice(subscriptionIdx, 1);
                this.values.splice(subscriptionIdx, 1);
            }
            return value;
        };
        DelayWhenSubscriber.prototype.tryDelay = function (delayNotifier, value) {
            var notifierSubscription = subscribeToResult_1.subscribeToResult(this, delayNotifier, value);
            this.add(notifierSubscription);
            this.delayNotifierSubscriptions.push(notifierSubscription);
            this.values.push(value);
        };
        DelayWhenSubscriber.prototype.tryComplete = function () {
            if (this.completed && this.delayNotifierSubscriptions.length === 0) {
                this.destination.complete();
            }
        };
        return DelayWhenSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var SubscriptionDelayObservable = function (_super) {
        __extends(SubscriptionDelayObservable, _super);
        function SubscriptionDelayObservable(source, subscriptionDelay) {
            _super.call(this);
            this.source = source;
            this.subscriptionDelay = subscriptionDelay;
        }
        SubscriptionDelayObservable.prototype._subscribe = function (subscriber) {
            this.subscriptionDelay.subscribe(new SubscriptionDelaySubscriber(subscriber, this.source));
        };
        return SubscriptionDelayObservable;
    }(Observable_1.Observable);
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var SubscriptionDelaySubscriber = function (_super) {
        __extends(SubscriptionDelaySubscriber, _super);
        function SubscriptionDelaySubscriber(parent, source) {
            _super.call(this);
            this.parent = parent;
            this.source = source;
            this.sourceSubscribed = false;
        }
        SubscriptionDelaySubscriber.prototype._next = function (unused) {
            this.subscribeToSource();
        };
        SubscriptionDelaySubscriber.prototype._error = function (err) {
            this.unsubscribe();
            this.parent.error(err);
        };
        SubscriptionDelaySubscriber.prototype._complete = function () {
            this.subscribeToSource();
        };
        SubscriptionDelaySubscriber.prototype.subscribeToSource = function () {
            if (!this.sourceSubscribed) {
                this.sourceSubscribed = true;
                this.unsubscribe();
                this.source.subscribe(this.parent);
            }
        };
        return SubscriptionDelaySubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/delayWhen.js', ['../../Observable', '../../operator/delayWhen'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var delayWhen_1 = $__require('../../operator/delayWhen');
  Observable_1.Observable.prototype.delayWhen = delayWhen_1.delayWhen;
  
});
System.registerDynamic('node_modules/rxjs/operator/distinctUntilChanged.js', ['../Subscriber', '../util/tryCatch', '../util/errorObject'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var tryCatch_1 = $__require('../util/tryCatch');
    var errorObject_1 = $__require('../util/errorObject');
    /**
     * Returns an Observable that emits all items emitted by the source Observable that are distinct by comparison from the previous item.
     * If a comparator function is provided, then it will be called for each item to test for whether or not that value should be emitted.
     * If a comparator function is not provided, an equality check is used by default.
     * @param {function} [compare] optional comparison function called to test if an item is distinct from the previous item in the source.
     * @return {Observable} an Observable that emits items from the source Observable with distinct values.
     * @method distinctUntilChanged
     * @owner Observable
     */
    function distinctUntilChanged(compare, keySelector) {
        return this.lift(new DistinctUntilChangedOperator(compare, keySelector));
    }
    exports.distinctUntilChanged = distinctUntilChanged;
    var DistinctUntilChangedOperator = function () {
        function DistinctUntilChangedOperator(compare, keySelector) {
            this.compare = compare;
            this.keySelector = keySelector;
        }
        DistinctUntilChangedOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new DistinctUntilChangedSubscriber(subscriber, this.compare, this.keySelector));
        };
        return DistinctUntilChangedOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var DistinctUntilChangedSubscriber = function (_super) {
        __extends(DistinctUntilChangedSubscriber, _super);
        function DistinctUntilChangedSubscriber(destination, compare, keySelector) {
            _super.call(this, destination);
            this.keySelector = keySelector;
            this.hasKey = false;
            if (typeof compare === 'function') {
                this.compare = compare;
            }
        }
        DistinctUntilChangedSubscriber.prototype.compare = function (x, y) {
            return x === y;
        };
        DistinctUntilChangedSubscriber.prototype._next = function (value) {
            var keySelector = this.keySelector;
            var key = value;
            if (keySelector) {
                key = tryCatch_1.tryCatch(this.keySelector)(value);
                if (key === errorObject_1.errorObject) {
                    return this.destination.error(errorObject_1.errorObject.e);
                }
            }
            var result = false;
            if (this.hasKey) {
                result = tryCatch_1.tryCatch(this.compare)(this.key, key);
                if (result === errorObject_1.errorObject) {
                    return this.destination.error(errorObject_1.errorObject.e);
                }
            } else {
                this.hasKey = true;
            }
            if (Boolean(result) === false) {
                this.key = key;
                this.destination.next(value);
            }
        };
        return DistinctUntilChangedSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/distinctUntilChanged.js', ['../../Observable', '../../operator/distinctUntilChanged'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var distinctUntilChanged_1 = $__require('../../operator/distinctUntilChanged');
  Observable_1.Observable.prototype.distinctUntilChanged = distinctUntilChanged_1.distinctUntilChanged;
  
});
System.registerDynamic("node_modules/rxjs/operator/do.js", ["../Subscriber"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * Perform a side effect for every emission on the source Observable, but return
     * an Observable that is identical to the source.
     *
     * <span class="informal">Intercepts each emission on the source and runs a
     * function, but returns an output which is identical to the source.</span>
     *
     * <img src="./img/do.png" width="100%">
     *
     * Returns a mirrored Observable of the source Observable, but modified so that
     * the provided Observer is called to perform a side effect for every value,
     * error, and completion emitted by the source. Any errors that are thrown in
     * the aforementioned Observer or handlers are safely sent down the error path
     * of the output Observable.
     *
     * This operator is useful for debugging your Observables for the correct values
     * or performing other side effects.
     *
     * Note: this is different to a `subscribe` on the Observable. If the Observable
     * returned by `do` is not subscribed, the side effects specified by the
     * Observer will never happen. `do` therefore simply spies on existing
     * execution, it does not trigger an execution to happen like `subscribe` does.
     *
     * @example <caption>Map every every click to the clientX position of that click, while also logging the click event</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var positions = clicks
     *   .do(ev => console.log(ev))
     *   .map(ev => ev.clientX);
     * positions.subscribe(x => console.log(x));
     *
     * @see {@link map}
     * @see {@link subscribe}
     *
     * @param {Observer|function} [nextOrObserver] A normal Observer object or a
     * callback for `next`.
     * @param {function} [error] Callback for errors in the source.
     * @param {function} [complete] Callback for the completion of the source.
     * @return {Observable} An Observable identical to the source, but runs the
     * specified Observer or callback(s) for each item.
     * @method do
     * @name do
     * @owner Observable
     */
    function _do(nextOrObserver, error, complete) {
        return this.lift(new DoOperator(nextOrObserver, error, complete));
    }
    exports._do = _do;
    var DoOperator = function () {
        function DoOperator(nextOrObserver, error, complete) {
            this.nextOrObserver = nextOrObserver;
            this.error = error;
            this.complete = complete;
        }
        DoOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new DoSubscriber(subscriber, this.nextOrObserver, this.error, this.complete));
        };
        return DoOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var DoSubscriber = function (_super) {
        __extends(DoSubscriber, _super);
        function DoSubscriber(destination, nextOrObserver, error, complete) {
            _super.call(this, destination);
            var safeSubscriber = new Subscriber_1.Subscriber(nextOrObserver, error, complete);
            safeSubscriber.syncErrorThrowable = true;
            this.add(safeSubscriber);
            this.safeSubscriber = safeSubscriber;
        }
        DoSubscriber.prototype._next = function (value) {
            var safeSubscriber = this.safeSubscriber;
            safeSubscriber.next(value);
            if (safeSubscriber.syncErrorThrown) {
                this.destination.error(safeSubscriber.syncErrorValue);
            } else {
                this.destination.next(value);
            }
        };
        DoSubscriber.prototype._error = function (err) {
            var safeSubscriber = this.safeSubscriber;
            safeSubscriber.error(err);
            if (safeSubscriber.syncErrorThrown) {
                this.destination.error(safeSubscriber.syncErrorValue);
            } else {
                this.destination.error(err);
            }
        };
        DoSubscriber.prototype._complete = function () {
            var safeSubscriber = this.safeSubscriber;
            safeSubscriber.complete();
            if (safeSubscriber.syncErrorThrown) {
                this.destination.error(safeSubscriber.syncErrorValue);
            } else {
                this.destination.complete();
            }
        };
        return DoSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/do.js', ['../../Observable', '../../operator/do'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var do_1 = $__require('../../operator/do');
  Observable_1.Observable.prototype.do = do_1._do;
  
});
System.registerDynamic('node_modules/rxjs/operator/expand.js', ['../util/tryCatch', '../util/errorObject', '../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var tryCatch_1 = $__require('../util/tryCatch');
    var errorObject_1 = $__require('../util/errorObject');
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Returns an Observable where for each item in the source Observable, the supplied function is applied to each item,
     * resulting in a new value to then be applied again with the function.
     * @param {function} project the function for projecting the next emitted item of the Observable.
     * @param {number} [concurrent] the max number of observables that can be created concurrently. defaults to infinity.
     * @param {Scheduler} [scheduler] The Scheduler to use for managing the expansions.
     * @return {Observable} an Observable containing the expansions of the source Observable.
     * @method expand
     * @owner Observable
     */
    function expand(project, concurrent, scheduler) {
        if (concurrent === void 0) {
            concurrent = Number.POSITIVE_INFINITY;
        }
        if (scheduler === void 0) {
            scheduler = undefined;
        }
        concurrent = (concurrent || 0) < 1 ? Number.POSITIVE_INFINITY : concurrent;
        return this.lift(new ExpandOperator(project, concurrent, scheduler));
    }
    exports.expand = expand;
    var ExpandOperator = function () {
        function ExpandOperator(project, concurrent, scheduler) {
            this.project = project;
            this.concurrent = concurrent;
            this.scheduler = scheduler;
        }
        ExpandOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new ExpandSubscriber(subscriber, this.project, this.concurrent, this.scheduler));
        };
        return ExpandOperator;
    }();
    exports.ExpandOperator = ExpandOperator;
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var ExpandSubscriber = function (_super) {
        __extends(ExpandSubscriber, _super);
        function ExpandSubscriber(destination, project, concurrent, scheduler) {
            _super.call(this, destination);
            this.project = project;
            this.concurrent = concurrent;
            this.scheduler = scheduler;
            this.index = 0;
            this.active = 0;
            this.hasCompleted = false;
            if (concurrent < Number.POSITIVE_INFINITY) {
                this.buffer = [];
            }
        }
        ExpandSubscriber.dispatch = function (arg) {
            var subscriber = arg.subscriber,
                result = arg.result,
                value = arg.value,
                index = arg.index;
            subscriber.subscribeToProjection(result, value, index);
        };
        ExpandSubscriber.prototype._next = function (value) {
            var destination = this.destination;
            if (destination.isUnsubscribed) {
                this._complete();
                return;
            }
            var index = this.index++;
            if (this.active < this.concurrent) {
                destination.next(value);
                var result = tryCatch_1.tryCatch(this.project)(value, index);
                if (result === errorObject_1.errorObject) {
                    destination.error(errorObject_1.errorObject.e);
                } else if (!this.scheduler) {
                    this.subscribeToProjection(result, value, index);
                } else {
                    var state = { subscriber: this, result: result, value: value, index: index };
                    this.add(this.scheduler.schedule(ExpandSubscriber.dispatch, 0, state));
                }
            } else {
                this.buffer.push(value);
            }
        };
        ExpandSubscriber.prototype.subscribeToProjection = function (result, value, index) {
            this.active++;
            this.add(subscribeToResult_1.subscribeToResult(this, result, value, index));
        };
        ExpandSubscriber.prototype._complete = function () {
            this.hasCompleted = true;
            if (this.hasCompleted && this.active === 0) {
                this.destination.complete();
            }
        };
        ExpandSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            this._next(innerValue);
        };
        ExpandSubscriber.prototype.notifyComplete = function (innerSub) {
            var buffer = this.buffer;
            this.remove(innerSub);
            this.active--;
            if (buffer && buffer.length > 0) {
                this._next(buffer.shift());
            }
            if (this.hasCompleted && this.active === 0) {
                this.destination.complete();
            }
        };
        return ExpandSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    exports.ExpandSubscriber = ExpandSubscriber;
    
});
System.registerDynamic('node_modules/rxjs/add/operator/expand.js', ['../../Observable', '../../operator/expand'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var expand_1 = $__require('../../operator/expand');
  Observable_1.Observable.prototype.expand = expand_1.expand;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/filter.js', ['../../Observable', '../../operator/filter'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var filter_1 = $__require('../../operator/filter');
  Observable_1.Observable.prototype.filter = filter_1.filter;
  
});
System.registerDynamic('node_modules/rxjs/operator/finally.js', ['../Subscriber', '../Subscription'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var Subscription_1 = $__require('../Subscription');
    /**
     * Returns an Observable that mirrors the source Observable, but will call a specified function when
     * the source terminates on complete or error.
     * @param {function} finallySelector function to be called when source terminates.
     * @return {Observable} an Observable that mirrors the source, but will call the specified function on termination.
     * @method finally
     * @owner Observable
     */
    function _finally(finallySelector) {
        return this.lift(new FinallyOperator(finallySelector));
    }
    exports._finally = _finally;
    var FinallyOperator = function () {
        function FinallyOperator(finallySelector) {
            this.finallySelector = finallySelector;
        }
        FinallyOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new FinallySubscriber(subscriber, this.finallySelector));
        };
        return FinallyOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var FinallySubscriber = function (_super) {
        __extends(FinallySubscriber, _super);
        function FinallySubscriber(destination, finallySelector) {
            _super.call(this, destination);
            this.add(new Subscription_1.Subscription(finallySelector));
        }
        return FinallySubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/finally.js', ['../../Observable', '../../operator/finally'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var finally_1 = $__require('../../operator/finally');
  Observable_1.Observable.prototype.finally = finally_1._finally;
  
});
System.registerDynamic('node_modules/rxjs/operator/first.js', ['../Subscriber', '../util/EmptyError'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var EmptyError_1 = $__require('../util/EmptyError');
    /**
     * Emits only the first value (or the first value that meets some condition)
     * emitted by the source Observable.
     *
     * <span class="informal">Emits only the first value. Or emits only the first
     * value that passes some test.</span>
     *
     * <img src="./img/first.png" width="100%">
     *
     * If called with no arguments, `first` emits the first value of the source
     * Observable, then completes. If called with a `predicate` function, `first`
     * emits the first value of the source that matches the specified condition. It
     * may also take a `resultSelector` function to produce the output value from
     * the input value, and a `defaultValue` to emit in case the source completes
     * before it is able to emit a valid value. Throws an error if `defaultValue`
     * was not provided and a matching element is not found.
     *
     * @example <caption>Emit only the first click that happens on the DOM</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var result = clicks.first();
     * result.subscribe(x => console.log(x));
     *
     * @example <caption>Emits the first click that happens on a DIV</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var result = clicks.first(ev => ev.target.tagName === 'DIV');
     * result.subscribe(x => console.log(x));
     *
     * @see {@link filter}
     * @see {@link find}
     * @see {@link take}
     *
     * @throws {EmptyError} Delivers an EmptyError to the Observer's `error`
     * callback if the Observable completes before any `next` notification was sent.
     *
     * @param {function(value: T, index: number, source: Observable<T>): boolean} [predicate]
     * An optional function called with each item to test for condition matching.
     * @param {function(value: T, index: number): R} [resultSelector] A function to
     * produce the value on the output Observable based on the values
     * and the indices of the source Observable. The arguments passed to this
     * function are:
     * - `value`: the value that was emitted on the source.
     * - `index`: the "index" of the value from the source.
     * @param {R} [defaultValue] The default value emitted in case no valid value
     * was found on the source.
     * @return {Observable<T|R>} an Observable of the first item that matches the
     * condition.
     * @method first
     * @owner Observable
     */
    function first(predicate, resultSelector, defaultValue) {
        return this.lift(new FirstOperator(predicate, resultSelector, defaultValue, this));
    }
    exports.first = first;
    var FirstOperator = function () {
        function FirstOperator(predicate, resultSelector, defaultValue, source) {
            this.predicate = predicate;
            this.resultSelector = resultSelector;
            this.defaultValue = defaultValue;
            this.source = source;
        }
        FirstOperator.prototype.call = function (observer, source) {
            return source._subscribe(new FirstSubscriber(observer, this.predicate, this.resultSelector, this.defaultValue, this.source));
        };
        return FirstOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var FirstSubscriber = function (_super) {
        __extends(FirstSubscriber, _super);
        function FirstSubscriber(destination, predicate, resultSelector, defaultValue, source) {
            _super.call(this, destination);
            this.predicate = predicate;
            this.resultSelector = resultSelector;
            this.defaultValue = defaultValue;
            this.source = source;
            this.index = 0;
            this.hasCompleted = false;
        }
        FirstSubscriber.prototype._next = function (value) {
            var index = this.index++;
            if (this.predicate) {
                this._tryPredicate(value, index);
            } else {
                this._emit(value, index);
            }
        };
        FirstSubscriber.prototype._tryPredicate = function (value, index) {
            var result;
            try {
                result = this.predicate(value, index, this.source);
            } catch (err) {
                this.destination.error(err);
                return;
            }
            if (result) {
                this._emit(value, index);
            }
        };
        FirstSubscriber.prototype._emit = function (value, index) {
            if (this.resultSelector) {
                this._tryResultSelector(value, index);
                return;
            }
            this._emitFinal(value);
        };
        FirstSubscriber.prototype._tryResultSelector = function (value, index) {
            var result;
            try {
                result = this.resultSelector(value, index);
            } catch (err) {
                this.destination.error(err);
                return;
            }
            this._emitFinal(result);
        };
        FirstSubscriber.prototype._emitFinal = function (value) {
            var destination = this.destination;
            destination.next(value);
            destination.complete();
            this.hasCompleted = true;
        };
        FirstSubscriber.prototype._complete = function () {
            var destination = this.destination;
            if (!this.hasCompleted && typeof this.defaultValue !== 'undefined') {
                destination.next(this.defaultValue);
                destination.complete();
            } else if (!this.hasCompleted) {
                destination.error(new EmptyError_1.EmptyError());
            }
        };
        return FirstSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/first.js', ['../../Observable', '../../operator/first'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var first_1 = $__require('../../operator/first');
  Observable_1.Observable.prototype.first = first_1.first;
  
});
System.registerDynamic("node_modules/rxjs/util/MapPolyfill.js", [], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var MapPolyfill = function () {
        function MapPolyfill() {
            this.size = 0;
            this._values = [];
            this._keys = [];
        }
        MapPolyfill.prototype.get = function (key) {
            var i = this._keys.indexOf(key);
            return i === -1 ? undefined : this._values[i];
        };
        MapPolyfill.prototype.set = function (key, value) {
            var i = this._keys.indexOf(key);
            if (i === -1) {
                this._keys.push(key);
                this._values.push(value);
                this.size++;
            } else {
                this._values[i] = value;
            }
            return this;
        };
        MapPolyfill.prototype.delete = function (key) {
            var i = this._keys.indexOf(key);
            if (i === -1) {
                return false;
            }
            this._values.splice(i, 1);
            this._keys.splice(i, 1);
            this.size--;
            return true;
        };
        MapPolyfill.prototype.clear = function () {
            this._keys.length = 0;
            this._values.length = 0;
            this.size = 0;
        };
        MapPolyfill.prototype.forEach = function (cb, thisArg) {
            for (var i = 0; i < this.size; i++) {
                cb.call(thisArg, this._values[i], this._keys[i]);
            }
        };
        return MapPolyfill;
    }();
    exports.MapPolyfill = MapPolyfill;
    
});
System.registerDynamic('node_modules/rxjs/util/Map.js', ['./root', './MapPolyfill'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var root_1 = $__require('./root');
  var MapPolyfill_1 = $__require('./MapPolyfill');
  exports.Map = root_1.root.Map || function () {
    return MapPolyfill_1.MapPolyfill;
  }();
  
});
System.registerDynamic("node_modules/rxjs/util/FastMap.js", [], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var FastMap = function () {
        function FastMap() {
            this.values = {};
        }
        FastMap.prototype.delete = function (key) {
            this.values[key] = null;
            return true;
        };
        FastMap.prototype.set = function (key, value) {
            this.values[key] = value;
            return this;
        };
        FastMap.prototype.get = function (key) {
            return this.values[key];
        };
        FastMap.prototype.forEach = function (cb, thisArg) {
            var values = this.values;
            for (var key in values) {
                if (values.hasOwnProperty(key) && values[key] !== null) {
                    cb.call(thisArg, values[key], key);
                }
            }
        };
        FastMap.prototype.clear = function () {
            this.values = {};
        };
        return FastMap;
    }();
    exports.FastMap = FastMap;
    
});
System.registerDynamic('node_modules/rxjs/operator/groupBy.js', ['../Subscriber', '../Subscription', '../Observable', '../Subject', '../util/Map', '../util/FastMap'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var Subscription_1 = $__require('../Subscription');
    var Observable_1 = $__require('../Observable');
    var Subject_1 = $__require('../Subject');
    var Map_1 = $__require('../util/Map');
    var FastMap_1 = $__require('../util/FastMap');
    /**
     * Groups the items emitted by an Observable according to a specified criterion,
     * and emits these grouped items as `GroupedObservables`, one
     * {@link GroupedObservable} per group.
     *
     * <img src="./img/groupBy.png" width="100%">
     *
     * @param {function(value: T): K} keySelector a function that extracts the key
     * for each item.
     * @param {function(value: T): R} [elementSelector] a function that extracts the
     * return element for each item.
     * @param {function(grouped: GroupedObservable<K,R>): Observable<any>} [durationSelector]
     * a function that returns an Observable to determine how long each group should
     * exist.
     * @return {Observable<GroupedObservable<K,R>>} an Observable that emits
     * GroupedObservables, each of which corresponds to a unique key value and each
     * of which emits those items from the source Observable that share that key
     * value.
     * @method groupBy
     * @owner Observable
     */
    function groupBy(keySelector, elementSelector, durationSelector) {
        return this.lift(new GroupByOperator(this, keySelector, elementSelector, durationSelector));
    }
    exports.groupBy = groupBy;
    var GroupByOperator = function () {
        function GroupByOperator(source, keySelector, elementSelector, durationSelector) {
            this.source = source;
            this.keySelector = keySelector;
            this.elementSelector = elementSelector;
            this.durationSelector = durationSelector;
        }
        GroupByOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new GroupBySubscriber(subscriber, this.keySelector, this.elementSelector, this.durationSelector));
        };
        return GroupByOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var GroupBySubscriber = function (_super) {
        __extends(GroupBySubscriber, _super);
        function GroupBySubscriber(destination, keySelector, elementSelector, durationSelector) {
            _super.call(this);
            this.keySelector = keySelector;
            this.elementSelector = elementSelector;
            this.durationSelector = durationSelector;
            this.groups = null;
            this.attemptedToUnsubscribe = false;
            this.count = 0;
            this.destination = destination;
            this.add(destination);
        }
        GroupBySubscriber.prototype._next = function (value) {
            var key;
            try {
                key = this.keySelector(value);
            } catch (err) {
                this.error(err);
                return;
            }
            this._group(value, key);
        };
        GroupBySubscriber.prototype._group = function (value, key) {
            var groups = this.groups;
            if (!groups) {
                groups = this.groups = typeof key === 'string' ? new FastMap_1.FastMap() : new Map_1.Map();
            }
            var group = groups.get(key);
            if (!group) {
                groups.set(key, group = new Subject_1.Subject());
                var groupedObservable = new GroupedObservable(key, group, this);
                if (this.durationSelector) {
                    this._selectDuration(key, group);
                }
                this.destination.next(groupedObservable);
            }
            if (this.elementSelector) {
                this._selectElement(value, group);
            } else {
                this.tryGroupNext(value, group);
            }
        };
        GroupBySubscriber.prototype._selectElement = function (value, group) {
            var result;
            try {
                result = this.elementSelector(value);
            } catch (err) {
                this.error(err);
                return;
            }
            this.tryGroupNext(result, group);
        };
        GroupBySubscriber.prototype._selectDuration = function (key, group) {
            var duration;
            try {
                duration = this.durationSelector(new GroupedObservable(key, group));
            } catch (err) {
                this.error(err);
                return;
            }
            this.add(duration.subscribe(new GroupDurationSubscriber(key, group, this)));
        };
        GroupBySubscriber.prototype.tryGroupNext = function (value, group) {
            if (!group.isUnsubscribed) {
                group.next(value);
            }
        };
        GroupBySubscriber.prototype._error = function (err) {
            var groups = this.groups;
            if (groups) {
                groups.forEach(function (group, key) {
                    group.error(err);
                });
                groups.clear();
            }
            this.destination.error(err);
        };
        GroupBySubscriber.prototype._complete = function () {
            var groups = this.groups;
            if (groups) {
                groups.forEach(function (group, key) {
                    group.complete();
                });
                groups.clear();
            }
            this.destination.complete();
        };
        GroupBySubscriber.prototype.removeGroup = function (key) {
            this.groups.delete(key);
        };
        GroupBySubscriber.prototype.unsubscribe = function () {
            if (!this.isUnsubscribed && !this.attemptedToUnsubscribe) {
                this.attemptedToUnsubscribe = true;
                if (this.count === 0) {
                    _super.prototype.unsubscribe.call(this);
                }
            }
        };
        return GroupBySubscriber;
    }(Subscriber_1.Subscriber);
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var GroupDurationSubscriber = function (_super) {
        __extends(GroupDurationSubscriber, _super);
        function GroupDurationSubscriber(key, group, parent) {
            _super.call(this);
            this.key = key;
            this.group = group;
            this.parent = parent;
        }
        GroupDurationSubscriber.prototype._next = function (value) {
            this.tryComplete();
        };
        GroupDurationSubscriber.prototype._error = function (err) {
            this.tryError(err);
        };
        GroupDurationSubscriber.prototype._complete = function () {
            this.tryComplete();
        };
        GroupDurationSubscriber.prototype.tryError = function (err) {
            var group = this.group;
            if (!group.isUnsubscribed) {
                group.error(err);
            }
            this.parent.removeGroup(this.key);
        };
        GroupDurationSubscriber.prototype.tryComplete = function () {
            var group = this.group;
            if (!group.isUnsubscribed) {
                group.complete();
            }
            this.parent.removeGroup(this.key);
        };
        return GroupDurationSubscriber;
    }(Subscriber_1.Subscriber);
    /**
     * An Observable representing values belonging to the same group represented by
     * a common key. The values emitted by a GroupedObservable come from the source
     * Observable. The common key is available as the field `key` on a
     * GroupedObservable instance.
     *
     * @class GroupedObservable<K, T>
     */
    var GroupedObservable = function (_super) {
        __extends(GroupedObservable, _super);
        function GroupedObservable(key, groupSubject, refCountSubscription) {
            _super.call(this);
            this.key = key;
            this.groupSubject = groupSubject;
            this.refCountSubscription = refCountSubscription;
        }
        GroupedObservable.prototype._subscribe = function (subscriber) {
            var subscription = new Subscription_1.Subscription();
            var _a = this,
                refCountSubscription = _a.refCountSubscription,
                groupSubject = _a.groupSubject;
            if (refCountSubscription && !refCountSubscription.isUnsubscribed) {
                subscription.add(new InnerRefCountSubscription(refCountSubscription));
            }
            subscription.add(groupSubject.subscribe(subscriber));
            return subscription;
        };
        return GroupedObservable;
    }(Observable_1.Observable);
    exports.GroupedObservable = GroupedObservable;
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var InnerRefCountSubscription = function (_super) {
        __extends(InnerRefCountSubscription, _super);
        function InnerRefCountSubscription(parent) {
            _super.call(this);
            this.parent = parent;
            parent.count++;
        }
        InnerRefCountSubscription.prototype.unsubscribe = function () {
            var parent = this.parent;
            if (!parent.isUnsubscribed && !this.isUnsubscribed) {
                _super.prototype.unsubscribe.call(this);
                parent.count -= 1;
                if (parent.count === 0 && parent.attemptedToUnsubscribe) {
                    parent.unsubscribe();
                }
            }
        };
        return InnerRefCountSubscription;
    }(Subscription_1.Subscription);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/groupBy.js', ['../../Observable', '../../operator/groupBy'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var groupBy_1 = $__require('../../operator/groupBy');
  Observable_1.Observable.prototype.groupBy = groupBy_1.groupBy;
  
});
System.registerDynamic("node_modules/rxjs/util/noop.js", [], true, function ($__require, exports, module) {
  "use strict";
  /* tslint:disable:no-empty */

  var global = this || self,
      GLOBAL = global;
  function noop() {}
  exports.noop = noop;
  
});
System.registerDynamic('node_modules/rxjs/operator/ignoreElements.js', ['../Subscriber', '../util/noop'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var noop_1 = $__require('../util/noop');
    /**
     * Ignores all items emitted by the source Observable and only passes calls of `complete` or `error`.
     *
     * <img src="./img/ignoreElements.png" width="100%">
     *
     * @return {Observable} an empty Observable that only calls `complete`
     * or `error`, based on which one is called by the source Observable.
     * @method ignoreElements
     * @owner Observable
     */
    function ignoreElements() {
        return this.lift(new IgnoreElementsOperator());
    }
    exports.ignoreElements = ignoreElements;
    ;
    var IgnoreElementsOperator = function () {
        function IgnoreElementsOperator() {}
        IgnoreElementsOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new IgnoreElementsSubscriber(subscriber));
        };
        return IgnoreElementsOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var IgnoreElementsSubscriber = function (_super) {
        __extends(IgnoreElementsSubscriber, _super);
        function IgnoreElementsSubscriber() {
            _super.apply(this, arguments);
        }
        IgnoreElementsSubscriber.prototype._next = function (unused) {
            noop_1.noop();
        };
        return IgnoreElementsSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/ignoreElements.js', ['../../Observable', '../../operator/ignoreElements'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var ignoreElements_1 = $__require('../../operator/ignoreElements');
  Observable_1.Observable.prototype.ignoreElements = ignoreElements_1.ignoreElements;
  
});
System.registerDynamic('node_modules/rxjs/operator/audit.js', ['../util/tryCatch', '../util/errorObject', '../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var tryCatch_1 = $__require('../util/tryCatch');
    var errorObject_1 = $__require('../util/errorObject');
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * @param durationSelector
     * @return {Observable<R>|WebSocketSubject<T>|Observable<T>}
     * @method audit
     * @owner Observable
     */
    function audit(durationSelector) {
        return this.lift(new AuditOperator(durationSelector));
    }
    exports.audit = audit;
    var AuditOperator = function () {
        function AuditOperator(durationSelector) {
            this.durationSelector = durationSelector;
        }
        AuditOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new AuditSubscriber(subscriber, this.durationSelector));
        };
        return AuditOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var AuditSubscriber = function (_super) {
        __extends(AuditSubscriber, _super);
        function AuditSubscriber(destination, durationSelector) {
            _super.call(this, destination);
            this.durationSelector = durationSelector;
            this.hasValue = false;
        }
        AuditSubscriber.prototype._next = function (value) {
            this.value = value;
            this.hasValue = true;
            if (!this.throttled) {
                var duration = tryCatch_1.tryCatch(this.durationSelector)(value);
                if (duration === errorObject_1.errorObject) {
                    this.destination.error(errorObject_1.errorObject.e);
                } else {
                    this.add(this.throttled = subscribeToResult_1.subscribeToResult(this, duration));
                }
            }
        };
        AuditSubscriber.prototype.clearThrottle = function () {
            var _a = this,
                value = _a.value,
                hasValue = _a.hasValue,
                throttled = _a.throttled;
            if (throttled) {
                this.remove(throttled);
                this.throttled = null;
                throttled.unsubscribe();
            }
            if (hasValue) {
                this.value = null;
                this.hasValue = false;
                this.destination.next(value);
            }
        };
        AuditSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex) {
            this.clearThrottle();
        };
        AuditSubscriber.prototype.notifyComplete = function () {
            this.clearThrottle();
        };
        return AuditSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/audit.js', ['../../Observable', '../../operator/audit'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var audit_1 = $__require('../../operator/audit');
  Observable_1.Observable.prototype.audit = audit_1.audit;
  
});
System.registerDynamic('node_modules/rxjs/operator/auditTime.js', ['../scheduler/async', '../Subscriber'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var async_1 = $__require('../scheduler/async');
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * @param delay
     * @param scheduler
     * @return {Observable<R>|WebSocketSubject<T>|Observable<T>}
     * @method auditTime
     * @owner Observable
     */
    function auditTime(delay, scheduler) {
        if (scheduler === void 0) {
            scheduler = async_1.async;
        }
        return this.lift(new AuditTimeOperator(delay, scheduler));
    }
    exports.auditTime = auditTime;
    var AuditTimeOperator = function () {
        function AuditTimeOperator(delay, scheduler) {
            this.delay = delay;
            this.scheduler = scheduler;
        }
        AuditTimeOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new AuditTimeSubscriber(subscriber, this.delay, this.scheduler));
        };
        return AuditTimeOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var AuditTimeSubscriber = function (_super) {
        __extends(AuditTimeSubscriber, _super);
        function AuditTimeSubscriber(destination, delay, scheduler) {
            _super.call(this, destination);
            this.delay = delay;
            this.scheduler = scheduler;
            this.hasValue = false;
        }
        AuditTimeSubscriber.prototype._next = function (value) {
            this.value = value;
            this.hasValue = true;
            if (!this.throttled) {
                this.add(this.throttled = this.scheduler.schedule(dispatchNext, this.delay, this));
            }
        };
        AuditTimeSubscriber.prototype.clearThrottle = function () {
            var _a = this,
                value = _a.value,
                hasValue = _a.hasValue,
                throttled = _a.throttled;
            if (throttled) {
                this.remove(throttled);
                this.throttled = null;
                throttled.unsubscribe();
            }
            if (hasValue) {
                this.value = null;
                this.hasValue = false;
                this.destination.next(value);
            }
        };
        return AuditTimeSubscriber;
    }(Subscriber_1.Subscriber);
    function dispatchNext(subscriber) {
        subscriber.clearThrottle();
    }
    
});
System.registerDynamic('node_modules/rxjs/add/operator/auditTime.js', ['../../Observable', '../../operator/auditTime'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var auditTime_1 = $__require('../../operator/auditTime');
  Observable_1.Observable.prototype.auditTime = auditTime_1.auditTime;
  
});
System.registerDynamic('node_modules/rxjs/operator/last.js', ['../Subscriber', '../util/EmptyError'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var EmptyError_1 = $__require('../util/EmptyError');
    /**
     * Returns an Observable that emits only the last item emitted by the source Observable.
     * It optionally takes a predicate function as a parameter, in which case, rather than emitting
     * the last item from the source Observable, the resulting Observable will emit the last item
     * from the source Observable that satisfies the predicate.
     *
     * <img src="./img/last.png" width="100%">
     *
     * @throws {EmptyError} Delivers an EmptyError to the Observer's `error`
     * callback if the Observable completes before any `next` notification was sent.
     * @param {function} predicate - the condition any source emitted item has to satisfy.
     * @return {Observable} an Observable that emits only the last item satisfying the given condition
     * from the source, or an NoSuchElementException if no such items are emitted.
     * @throws - Throws if no items that match the predicate are emitted by the source Observable.
     * @method last
     * @owner Observable
     */
    function last(predicate, resultSelector, defaultValue) {
        return this.lift(new LastOperator(predicate, resultSelector, defaultValue, this));
    }
    exports.last = last;
    var LastOperator = function () {
        function LastOperator(predicate, resultSelector, defaultValue, source) {
            this.predicate = predicate;
            this.resultSelector = resultSelector;
            this.defaultValue = defaultValue;
            this.source = source;
        }
        LastOperator.prototype.call = function (observer, source) {
            return source._subscribe(new LastSubscriber(observer, this.predicate, this.resultSelector, this.defaultValue, this.source));
        };
        return LastOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var LastSubscriber = function (_super) {
        __extends(LastSubscriber, _super);
        function LastSubscriber(destination, predicate, resultSelector, defaultValue, source) {
            _super.call(this, destination);
            this.predicate = predicate;
            this.resultSelector = resultSelector;
            this.defaultValue = defaultValue;
            this.source = source;
            this.hasValue = false;
            this.index = 0;
            if (typeof defaultValue !== 'undefined') {
                this.lastValue = defaultValue;
                this.hasValue = true;
            }
        }
        LastSubscriber.prototype._next = function (value) {
            var index = this.index++;
            if (this.predicate) {
                this._tryPredicate(value, index);
            } else {
                if (this.resultSelector) {
                    this._tryResultSelector(value, index);
                    return;
                }
                this.lastValue = value;
                this.hasValue = true;
            }
        };
        LastSubscriber.prototype._tryPredicate = function (value, index) {
            var result;
            try {
                result = this.predicate(value, index, this.source);
            } catch (err) {
                this.destination.error(err);
                return;
            }
            if (result) {
                if (this.resultSelector) {
                    this._tryResultSelector(value, index);
                    return;
                }
                this.lastValue = value;
                this.hasValue = true;
            }
        };
        LastSubscriber.prototype._tryResultSelector = function (value, index) {
            var result;
            try {
                result = this.resultSelector(value, index);
            } catch (err) {
                this.destination.error(err);
                return;
            }
            this.lastValue = result;
            this.hasValue = true;
        };
        LastSubscriber.prototype._complete = function () {
            var destination = this.destination;
            if (this.hasValue) {
                destination.next(this.lastValue);
                destination.complete();
            } else {
                destination.error(new EmptyError_1.EmptyError());
            }
        };
        return LastSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/last.js', ['../../Observable', '../../operator/last'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var last_1 = $__require('../../operator/last');
  Observable_1.Observable.prototype.last = last_1.last;
  
});
System.registerDynamic("node_modules/rxjs/operator/let.js", [], true, function ($__require, exports, module) {
  "use strict";
  /**
   * @param func
   * @return {Observable<R>}
   * @method let
   * @owner Observable
   */

  var global = this || self,
      GLOBAL = global;
  function letProto(func) {
    return func(this);
  }
  exports.letProto = letProto;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/let.js', ['../../Observable', '../../operator/let'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var let_1 = $__require('../../operator/let');
  Observable_1.Observable.prototype.let = let_1.letProto;
  Observable_1.Observable.prototype.letBind = let_1.letProto;
  
});
System.registerDynamic("node_modules/rxjs/operator/every.js", ["../Subscriber"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * Returns an Observable that emits whether or not every item of the source satisfies the condition specified.
     * @param {function} predicate a function for determining if an item meets a specified condition.
     * @param {any} [thisArg] optional object to use for `this` in the callback
     * @return {Observable} an Observable of booleans that determines if all items of the source Observable meet the condition specified.
     * @method every
     * @owner Observable
     */
    function every(predicate, thisArg) {
        var source = this;
        return source.lift(new EveryOperator(predicate, thisArg, source));
    }
    exports.every = every;
    var EveryOperator = function () {
        function EveryOperator(predicate, thisArg, source) {
            this.predicate = predicate;
            this.thisArg = thisArg;
            this.source = source;
        }
        EveryOperator.prototype.call = function (observer, source) {
            return source._subscribe(new EverySubscriber(observer, this.predicate, this.thisArg, this.source));
        };
        return EveryOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var EverySubscriber = function (_super) {
        __extends(EverySubscriber, _super);
        function EverySubscriber(destination, predicate, thisArg, source) {
            _super.call(this, destination);
            this.predicate = predicate;
            this.thisArg = thisArg;
            this.source = source;
            this.index = 0;
            this.thisArg = thisArg || this;
        }
        EverySubscriber.prototype.notifyComplete = function (everyValueMatch) {
            this.destination.next(everyValueMatch);
            this.destination.complete();
        };
        EverySubscriber.prototype._next = function (value) {
            var result = false;
            try {
                result = this.predicate.call(this.thisArg, value, this.index++, this.source);
            } catch (err) {
                this.destination.error(err);
                return;
            }
            if (!result) {
                this.notifyComplete(false);
            }
        };
        EverySubscriber.prototype._complete = function () {
            this.notifyComplete(true);
        };
        return EverySubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/every.js', ['../../Observable', '../../operator/every'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var every_1 = $__require('../../operator/every');
  Observable_1.Observable.prototype.every = every_1.every;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/map.js', ['../../Observable', '../../operator/map'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var map_1 = $__require('../../operator/map');
  Observable_1.Observable.prototype.map = map_1.map;
  
});
System.registerDynamic("node_modules/rxjs/operator/mapTo.js", ["../Subscriber"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * Emits the given constant value on the output Observable every time the source
     * Observable emits a value.
     *
     * <span class="informal">Like {@link map}, but it maps every source value to
     * the same output value every time.</span>
     *
     * <img src="./img/mapTo.png" width="100%">
     *
     * Takes a constant `value` as argument, and emits that whenever the source
     * Observable emits a value. In other words, ignores the actual source value,
     * and simply uses the emission moment to know when to emit the given `value`.
     *
     * @example <caption>Map every every click to the string 'Hi'</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var greetings = clicks.mapTo('Hi');
     * greetings.subscribe(x => console.log(x));
     *
     * @see {@link map}
     *
     * @param {any} value The value to map each source value to.
     * @return {Observable} An Observable that emits the given `value` every time
     * the source Observable emits something.
     * @method mapTo
     * @owner Observable
     */
    function mapTo(value) {
        return this.lift(new MapToOperator(value));
    }
    exports.mapTo = mapTo;
    var MapToOperator = function () {
        function MapToOperator(value) {
            this.value = value;
        }
        MapToOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new MapToSubscriber(subscriber, this.value));
        };
        return MapToOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var MapToSubscriber = function (_super) {
        __extends(MapToSubscriber, _super);
        function MapToSubscriber(destination, value) {
            _super.call(this, destination);
            this.value = value;
        }
        MapToSubscriber.prototype._next = function (x) {
            this.destination.next(this.value);
        };
        return MapToSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/mapTo.js', ['../../Observable', '../../operator/mapTo'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var mapTo_1 = $__require('../../operator/mapTo');
  Observable_1.Observable.prototype.mapTo = mapTo_1.mapTo;
  
});
System.registerDynamic('node_modules/rxjs/operator/materialize.js', ['../Subscriber', '../Notification'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var Notification_1 = $__require('../Notification');
    /**
     * Returns an Observable that represents all of the emissions and notifications
     * from the source Observable into emissions marked with their original types
     * within a `Notification` objects.
     *
     * <img src="./img/materialize.png" width="100%">
     *
     * @see {@link Notification}
     *
     * @scheduler materialize does not operate by default on a particular Scheduler.
     * @return {Observable<Notification<T>>} an Observable that emits items that are the result of
     * materializing the items and notifications of the source Observable.
     * @method materialize
     * @owner Observable
     */
    function materialize() {
        return this.lift(new MaterializeOperator());
    }
    exports.materialize = materialize;
    var MaterializeOperator = function () {
        function MaterializeOperator() {}
        MaterializeOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new MaterializeSubscriber(subscriber));
        };
        return MaterializeOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var MaterializeSubscriber = function (_super) {
        __extends(MaterializeSubscriber, _super);
        function MaterializeSubscriber(destination) {
            _super.call(this, destination);
        }
        MaterializeSubscriber.prototype._next = function (value) {
            this.destination.next(Notification_1.Notification.createNext(value));
        };
        MaterializeSubscriber.prototype._error = function (err) {
            var destination = this.destination;
            destination.next(Notification_1.Notification.createError(err));
            destination.complete();
        };
        MaterializeSubscriber.prototype._complete = function () {
            var destination = this.destination;
            destination.next(Notification_1.Notification.createComplete());
            destination.complete();
        };
        return MaterializeSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/materialize.js', ['../../Observable', '../../operator/materialize'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var materialize_1 = $__require('../../operator/materialize');
  Observable_1.Observable.prototype.materialize = materialize_1.materialize;
  
});
System.registerDynamic('node_modules/rxjs/operator/merge.js', ['../observable/ArrayObservable', './mergeAll', '../util/isScheduler'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var ArrayObservable_1 = $__require('../observable/ArrayObservable');
    var mergeAll_1 = $__require('./mergeAll');
    var isScheduler_1 = $__require('../util/isScheduler');
    /**
     * Creates an output Observable which concurrently emits all values from every
     * given input Observable.
     *
     * <span class="informal">Flattens multiple Observables together by blending
     * their values into one Observable.</span>
     *
     * <img src="./img/merge.png" width="100%">
     *
     * `merge` subscribes to each given input Observable (either the source or an
     * Observable given as argument), and simply forwards (without doing any
     * transformation) all the values from all the input Observables to the output
     * Observable. The output Observable only completes once all input Observables
     * have completed. Any error delivered by an input Observable will be immediately
     * emitted on the output Observable.
     *
     * @example <caption>Merge together two Observables: 1s interval and clicks</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var timer = Rx.Observable.interval(1000);
     * var clicksOrTimer = clicks.merge(timer);
     * clicksOrTimer.subscribe(x => console.log(x));
     *
     * @example <caption>Merge together 3 Observables, but only 2 run concurrently</caption>
     * var timer1 = Rx.Observable.interval(1000).take(10);
     * var timer2 = Rx.Observable.interval(2000).take(6);
     * var timer3 = Rx.Observable.interval(500).take(10);
     * var concurrent = 2; // the argument
     * var merged = timer1.merge(timer2, timer3, concurrent);
     * merged.subscribe(x => console.log(x));
     *
     * @see {@link mergeAll}
     * @see {@link mergeMap}
     * @see {@link mergeMapTo}
     * @see {@link mergeScan}
     *
     * @param {Observable} other An input Observable to merge with the source
     * Observable. More than one input Observables may be given as argument.
     * @param {number} [concurrent=Number.POSITIVE_INFINITY] Maximum number of input
     * Observables being subscribed to concurrently.
     * @param {Scheduler} [scheduler=null] The Scheduler to use for managing
     * concurrency of input Observables.
     * @return {Observable} an Observable that emits items that are the result of
     * every input Observable.
     * @method merge
     * @owner Observable
     */
    function merge() {
        var observables = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            observables[_i - 0] = arguments[_i];
        }
        observables.unshift(this);
        return mergeStatic.apply(this, observables);
    }
    exports.merge = merge;
    /* tslint:enable:max-line-length */
    /**
     * Creates an output Observable which concurrently emits all values from every
     * given input Observable.
     *
     * <span class="informal">Flattens multiple Observables together by blending
     * their values into one Observable.</span>
     *
     * <img src="./img/merge.png" width="100%">
     *
     * `merge` subscribes to each given input Observable (as arguments), and simply
     * forwards (without doing any transformation) all the values from all the input
     * Observables to the output Observable. The output Observable only completes
     * once all input Observables have completed. Any error delivered by an input
     * Observable will be immediately emitted on the output Observable.
     *
     * @example <caption>Merge together two Observables: 1s interval and clicks</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var timer = Rx.Observable.interval(1000);
     * var clicksOrTimer = Rx.Observable.merge(clicks, timer);
     * clicksOrTimer.subscribe(x => console.log(x));
     *
     * @example <caption>Merge together 3 Observables, but only 2 run concurrently</caption>
     * var timer1 = Rx.Observable.interval(1000).take(10);
     * var timer2 = Rx.Observable.interval(2000).take(6);
     * var timer3 = Rx.Observable.interval(500).take(10);
     * var concurrent = 2; // the argument
     * var merged = Rx.Observable.merge(timer1, timer2, timer3, concurrent);
     * merged.subscribe(x => console.log(x));
     *
     * @see {@link mergeAll}
     * @see {@link mergeMap}
     * @see {@link mergeMapTo}
     * @see {@link mergeScan}
     *
     * @param {Observable} input1 An input Observable to merge with others.
     * @param {Observable} input2 An input Observable to merge with others.
     * @param {number} [concurrent=Number.POSITIVE_INFINITY] Maximum number of input
     * Observables being subscribed to concurrently.
     * @param {Scheduler} [scheduler=null] The Scheduler to use for managing
     * concurrency of input Observables.
     * @return {Observable} an Observable that emits items that are the result of
     * every input Observable.
     * @static true
     * @name merge
     * @owner Observable
     */
    function mergeStatic() {
        var observables = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            observables[_i - 0] = arguments[_i];
        }
        var concurrent = Number.POSITIVE_INFINITY;
        var scheduler = null;
        var last = observables[observables.length - 1];
        if (isScheduler_1.isScheduler(last)) {
            scheduler = observables.pop();
            if (observables.length > 1 && typeof observables[observables.length - 1] === 'number') {
                concurrent = observables.pop();
            }
        } else if (typeof last === 'number') {
            concurrent = observables.pop();
        }
        if (observables.length === 1) {
            return observables[0];
        }
        return new ArrayObservable_1.ArrayObservable(observables, scheduler).lift(new mergeAll_1.MergeAllOperator(concurrent));
    }
    exports.mergeStatic = mergeStatic;
    
});
System.registerDynamic('node_modules/rxjs/add/operator/merge.js', ['../../Observable', '../../operator/merge'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var merge_1 = $__require('../../operator/merge');
  Observable_1.Observable.prototype.merge = merge_1.merge;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/mergeAll.js', ['../../Observable', '../../operator/mergeAll'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var mergeAll_1 = $__require('../../operator/mergeAll');
  Observable_1.Observable.prototype.mergeAll = mergeAll_1.mergeAll;
  
});
System.registerDynamic('node_modules/rxjs/operator/mergeMap.js', ['../util/subscribeToResult', '../OuterSubscriber'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    /**
     * Projects each source value to an Observable which is merged in the output
     * Observable.
     *
     * <span class="informal">Maps each value to an Observable, then flattens all of
     * these inner Observables using {@link mergeAll}.</span>
     *
     * <img src="./img/mergeMap.png" width="100%">
     *
     * Returns an Observable that emits items based on applying a function that you
     * supply to each item emitted by the source Observable, where that function
     * returns an Observable, and then merging those resulting Observables and
     * emitting the results of this merger.
     *
     * @example <caption>Map and flatten each letter to an Observable ticking every 1 second</caption>
     * var letters = Rx.Observable.of('a', 'b', 'c');
     * var result = letters.mergeMap(x =>
     *   Rx.Observable.interval(1000).map(i => x+i)
     * );
     * result.subscribe(x => console.log(x));
     *
     * @see {@link concatMap}
     * @see {@link exhaustMap}
     * @see {@link merge}
     * @see {@link mergeAll}
     * @see {@link mergeMapTo}
     * @see {@link mergeScan}
     * @see {@link switchMap}
     *
     * @param {function(value: T, ?index: number): Observable} project A function
     * that, when applied to an item emitted by the source Observable, returns an
     * Observable.
     * @param {function(outerValue: T, innerValue: I, outerIndex: number, innerIndex: number): any} [resultSelector]
     * A function to produce the value on the output Observable based on the values
     * and the indices of the source (outer) emission and the inner Observable
     * emission. The arguments passed to this function are:
     * - `outerValue`: the value that came from the source
     * - `innerValue`: the value that came from the projected Observable
     * - `outerIndex`: the "index" of the value that came from the source
     * - `innerIndex`: the "index" of the value from the projected Observable
     * @param {number} [concurrent=Number.POSITIVE_INFINITY] Maximum number of input
     * Observables being subscribed to concurrently.
     * @return {Observable} An Observable that emits the result of applying the
     * projection function (and the optional `resultSelector`) to each item emitted
     * by the source Observable and merging the results of the Observables obtained
     * from this transformation.
     * @method mergeMap
     * @owner Observable
     */
    function mergeMap(project, resultSelector, concurrent) {
        if (concurrent === void 0) {
            concurrent = Number.POSITIVE_INFINITY;
        }
        if (typeof resultSelector === 'number') {
            concurrent = resultSelector;
            resultSelector = null;
        }
        return this.lift(new MergeMapOperator(project, resultSelector, concurrent));
    }
    exports.mergeMap = mergeMap;
    var MergeMapOperator = function () {
        function MergeMapOperator(project, resultSelector, concurrent) {
            if (concurrent === void 0) {
                concurrent = Number.POSITIVE_INFINITY;
            }
            this.project = project;
            this.resultSelector = resultSelector;
            this.concurrent = concurrent;
        }
        MergeMapOperator.prototype.call = function (observer, source) {
            return source._subscribe(new MergeMapSubscriber(observer, this.project, this.resultSelector, this.concurrent));
        };
        return MergeMapOperator;
    }();
    exports.MergeMapOperator = MergeMapOperator;
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var MergeMapSubscriber = function (_super) {
        __extends(MergeMapSubscriber, _super);
        function MergeMapSubscriber(destination, project, resultSelector, concurrent) {
            if (concurrent === void 0) {
                concurrent = Number.POSITIVE_INFINITY;
            }
            _super.call(this, destination);
            this.project = project;
            this.resultSelector = resultSelector;
            this.concurrent = concurrent;
            this.hasCompleted = false;
            this.buffer = [];
            this.active = 0;
            this.index = 0;
        }
        MergeMapSubscriber.prototype._next = function (value) {
            if (this.active < this.concurrent) {
                this._tryNext(value);
            } else {
                this.buffer.push(value);
            }
        };
        MergeMapSubscriber.prototype._tryNext = function (value) {
            var result;
            var index = this.index++;
            try {
                result = this.project(value, index);
            } catch (err) {
                this.destination.error(err);
                return;
            }
            this.active++;
            this._innerSub(result, value, index);
        };
        MergeMapSubscriber.prototype._innerSub = function (ish, value, index) {
            this.add(subscribeToResult_1.subscribeToResult(this, ish, value, index));
        };
        MergeMapSubscriber.prototype._complete = function () {
            this.hasCompleted = true;
            if (this.active === 0 && this.buffer.length === 0) {
                this.destination.complete();
            }
        };
        MergeMapSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            if (this.resultSelector) {
                this._notifyResultSelector(outerValue, innerValue, outerIndex, innerIndex);
            } else {
                this.destination.next(innerValue);
            }
        };
        MergeMapSubscriber.prototype._notifyResultSelector = function (outerValue, innerValue, outerIndex, innerIndex) {
            var result;
            try {
                result = this.resultSelector(outerValue, innerValue, outerIndex, innerIndex);
            } catch (err) {
                this.destination.error(err);
                return;
            }
            this.destination.next(result);
        };
        MergeMapSubscriber.prototype.notifyComplete = function (innerSub) {
            var buffer = this.buffer;
            this.remove(innerSub);
            this.active--;
            if (buffer.length > 0) {
                this._next(buffer.shift());
            } else if (this.active === 0 && this.hasCompleted) {
                this.destination.complete();
            }
        };
        return MergeMapSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    exports.MergeMapSubscriber = MergeMapSubscriber;
    
});
System.registerDynamic('node_modules/rxjs/add/operator/mergeMap.js', ['../../Observable', '../../operator/mergeMap'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var mergeMap_1 = $__require('../../operator/mergeMap');
  Observable_1.Observable.prototype.mergeMap = mergeMap_1.mergeMap;
  Observable_1.Observable.prototype.flatMap = mergeMap_1.mergeMap;
  
});
System.registerDynamic('node_modules/rxjs/operator/mergeMapTo.js', ['../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Projects each source value to the same Observable which is merged multiple
     * times in the output Observable.
     *
     * <span class="informal">It's like {@link mergeMap}, but maps each value always
     * to the same inner Observable.</span>
     *
     * <img src="./img/mergeMapTo.png" width="100%">
     *
     * Maps each source value to the given Observable `innerObservable` regardless
     * of the source value, and then merges those resulting Observables into one
     * single Observable, which is the output Observable.
     *
     * @example <caption>For each click event, start an interval Observable ticking every 1 second</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var result = clicks.mergeMapTo(Rx.Observable.interval(1000));
     * result.subscribe(x => console.log(x));
     *
     * @see {@link concatMapTo}
     * @see {@link merge}
     * @see {@link mergeAll}
     * @see {@link mergeMap}
     * @see {@link mergeScan}
     * @see {@link switchMapTo}
     *
     * @param {Observable} innerObservable An Observable to replace each value from
     * the source Observable.
     * @param {function(outerValue: T, innerValue: I, outerIndex: number, innerIndex: number): any} [resultSelector]
     * A function to produce the value on the output Observable based on the values
     * and the indices of the source (outer) emission and the inner Observable
     * emission. The arguments passed to this function are:
     * - `outerValue`: the value that came from the source
     * - `innerValue`: the value that came from the projected Observable
     * - `outerIndex`: the "index" of the value that came from the source
     * - `innerIndex`: the "index" of the value from the projected Observable
     * @param {number} [concurrent=Number.POSITIVE_INFINITY] Maximum number of input
     * Observables being subscribed to concurrently.
     * @return {Observable} An Observable that emits items from the given
     * `innerObservable` (and optionally transformed through `resultSelector`) every
     * time a value is emitted on the source Observable.
     * @method mergeMapTo
     * @owner Observable
     */
    function mergeMapTo(innerObservable, resultSelector, concurrent) {
        if (concurrent === void 0) {
            concurrent = Number.POSITIVE_INFINITY;
        }
        if (typeof resultSelector === 'number') {
            concurrent = resultSelector;
            resultSelector = null;
        }
        return this.lift(new MergeMapToOperator(innerObservable, resultSelector, concurrent));
    }
    exports.mergeMapTo = mergeMapTo;
    // TODO: Figure out correct signature here: an Operator<Observable<T>, R>
    //       needs to implement call(observer: Subscriber<R>): Subscriber<Observable<T>>
    var MergeMapToOperator = function () {
        function MergeMapToOperator(ish, resultSelector, concurrent) {
            if (concurrent === void 0) {
                concurrent = Number.POSITIVE_INFINITY;
            }
            this.ish = ish;
            this.resultSelector = resultSelector;
            this.concurrent = concurrent;
        }
        MergeMapToOperator.prototype.call = function (observer, source) {
            return source._subscribe(new MergeMapToSubscriber(observer, this.ish, this.resultSelector, this.concurrent));
        };
        return MergeMapToOperator;
    }();
    exports.MergeMapToOperator = MergeMapToOperator;
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var MergeMapToSubscriber = function (_super) {
        __extends(MergeMapToSubscriber, _super);
        function MergeMapToSubscriber(destination, ish, resultSelector, concurrent) {
            if (concurrent === void 0) {
                concurrent = Number.POSITIVE_INFINITY;
            }
            _super.call(this, destination);
            this.ish = ish;
            this.resultSelector = resultSelector;
            this.concurrent = concurrent;
            this.hasCompleted = false;
            this.buffer = [];
            this.active = 0;
            this.index = 0;
        }
        MergeMapToSubscriber.prototype._next = function (value) {
            if (this.active < this.concurrent) {
                var resultSelector = this.resultSelector;
                var index = this.index++;
                var ish = this.ish;
                var destination = this.destination;
                this.active++;
                this._innerSub(ish, destination, resultSelector, value, index);
            } else {
                this.buffer.push(value);
            }
        };
        MergeMapToSubscriber.prototype._innerSub = function (ish, destination, resultSelector, value, index) {
            this.add(subscribeToResult_1.subscribeToResult(this, ish, value, index));
        };
        MergeMapToSubscriber.prototype._complete = function () {
            this.hasCompleted = true;
            if (this.active === 0 && this.buffer.length === 0) {
                this.destination.complete();
            }
        };
        MergeMapToSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            var _a = this,
                resultSelector = _a.resultSelector,
                destination = _a.destination;
            if (resultSelector) {
                this.trySelectResult(outerValue, innerValue, outerIndex, innerIndex);
            } else {
                destination.next(innerValue);
            }
        };
        MergeMapToSubscriber.prototype.trySelectResult = function (outerValue, innerValue, outerIndex, innerIndex) {
            var _a = this,
                resultSelector = _a.resultSelector,
                destination = _a.destination;
            var result;
            try {
                result = resultSelector(outerValue, innerValue, outerIndex, innerIndex);
            } catch (err) {
                destination.error(err);
                return;
            }
            destination.next(result);
        };
        MergeMapToSubscriber.prototype.notifyError = function (err) {
            this.destination.error(err);
        };
        MergeMapToSubscriber.prototype.notifyComplete = function (innerSub) {
            var buffer = this.buffer;
            this.remove(innerSub);
            this.active--;
            if (buffer.length > 0) {
                this._next(buffer.shift());
            } else if (this.active === 0 && this.hasCompleted) {
                this.destination.complete();
            }
        };
        return MergeMapToSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    exports.MergeMapToSubscriber = MergeMapToSubscriber;
    
});
System.registerDynamic('node_modules/rxjs/add/operator/mergeMapTo.js', ['../../Observable', '../../operator/mergeMapTo'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var mergeMapTo_1 = $__require('../../operator/mergeMapTo');
  Observable_1.Observable.prototype.flatMapTo = mergeMapTo_1.mergeMapTo;
  Observable_1.Observable.prototype.mergeMapTo = mergeMapTo_1.mergeMapTo;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/multicast.js', ['../../Observable', '../../operator/multicast'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var multicast_1 = $__require('../../operator/multicast');
  Observable_1.Observable.prototype.multicast = multicast_1.multicast;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/observeOn.js', ['../../Observable', '../../operator/observeOn'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var observeOn_1 = $__require('../../operator/observeOn');
  Observable_1.Observable.prototype.observeOn = observeOn_1.observeOn;
  
});
System.registerDynamic("node_modules/rxjs/util/not.js", [], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    function not(pred, thisArg) {
        function notPred() {
            return !notPred.pred.apply(notPred.thisArg, arguments);
        }
        notPred.pred = pred;
        notPred.thisArg = thisArg;
        return notPred;
    }
    exports.not = not;
    
});
System.registerDynamic("node_modules/rxjs/operator/filter.js", ["../Subscriber"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * Filter items emitted by the source Observable by only emitting those that
     * satisfy a specified predicate.
     *
     * <span class="informal">Like
     * [Array.prototype.filter()](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/filter),
     * it only emits a value from the source if it passes a criterion function.</span>
     *
     * <img src="./img/filter.png" width="100%">
     *
     * Similar to the well-known `Array.prototype.filter` method, this operator
     * takes values from the source Observable, passes them through a `predicate`
     * function and only emits those values that yielded `true`.
     *
     * @example <caption>Emit only click events whose target was a DIV element</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var clicksOnDivs = clicks.filter(ev => ev.target.tagName === 'DIV');
     * clicksOnDivs.subscribe(x => console.log(x));
     *
     * @see {@link distinct}
     * @see {@link distinctKey}
     * @see {@link distinctUntilChanged}
     * @see {@link distinctUntilKeyChanged}
     * @see {@link ignoreElements}
     * @see {@link partition}
     * @see {@link skip}
     *
     * @param {function(value: T, index: number): boolean} predicate A function that
     * evaluates each value emitted by the source Observable. If it returns `true`,
     * the value is emitted, if `false` the value is not passed to the output
     * Observable. The `index` parameter is the number `i` for the i-th source
     * emission that has happened since the subscription, starting from the number
     * `0`.
     * @param {any} [thisArg] An optional argument to determine the value of `this`
     * in the `predicate` function.
     * @return {Observable} An Observable of values from the source that were
     * allowed by the `predicate` function.
     * @method filter
     * @owner Observable
     */
    function filter(predicate, thisArg) {
        return this.lift(new FilterOperator(predicate, thisArg));
    }
    exports.filter = filter;
    var FilterOperator = function () {
        function FilterOperator(predicate, thisArg) {
            this.predicate = predicate;
            this.thisArg = thisArg;
        }
        FilterOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new FilterSubscriber(subscriber, this.predicate, this.thisArg));
        };
        return FilterOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var FilterSubscriber = function (_super) {
        __extends(FilterSubscriber, _super);
        function FilterSubscriber(destination, predicate, thisArg) {
            _super.call(this, destination);
            this.predicate = predicate;
            this.thisArg = thisArg;
            this.count = 0;
            this.predicate = predicate;
        }
        // the try catch block below is left specifically for
        // optimization and perf reasons. a tryCatcher is not necessary here.
        FilterSubscriber.prototype._next = function (value) {
            var result;
            try {
                result = this.predicate.call(this.thisArg, value, this.count++);
            } catch (err) {
                this.destination.error(err);
                return;
            }
            if (result) {
                this.destination.next(value);
            }
        };
        return FilterSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/operator/partition.js', ['../util/not', './filter'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var not_1 = $__require('../util/not');
    var filter_1 = $__require('./filter');
    /**
     * @param predicate
     * @param thisArg
     * @return {Observable<T>[]}
     * @method partition
     * @owner Observable
     */
    function partition(predicate, thisArg) {
        return [filter_1.filter.call(this, predicate), filter_1.filter.call(this, not_1.not(predicate, thisArg))];
    }
    exports.partition = partition;
    
});
System.registerDynamic('node_modules/rxjs/add/operator/partition.js', ['../../Observable', '../../operator/partition'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var partition_1 = $__require('../../operator/partition');
  Observable_1.Observable.prototype.partition = partition_1.partition;
  
});
System.registerDynamic('node_modules/rxjs/operator/map.js', ['../Subscriber'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * Applies a given `project` function to each value emitted by the source
     * Observable, and emits the resulting values as an Observable.
     *
     * <span class="informal">Like [Array.prototype.map()](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/map),
     * it passes each source value through a transformation function to get
     * corresponding output values.</span>
     *
     * <img src="./img/map.png" width="100%">
     *
     * Similar to the well known `Array.prototype.map` function, this operator
     * applies a projection to each value and emits that projection in the output
     * Observable.
     *
     * @example <caption>Map every every click to the clientX position of that click</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var positions = clicks.map(ev => ev.clientX);
     * positions.subscribe(x => console.log(x));
     *
     * @see {@link mapTo}
     * @see {@link pluck}
     *
     * @param {function(value: T, index: number): R} project The function to apply
     * to each `value` emitted by the source Observable. The `index` parameter is
     * the number `i` for the i-th emission that has happened since the
     * subscription, starting from the number `0`.
     * @param {any} [thisArg] An optional argument to define what `this` is in the
     * `project` function.
     * @return {Observable<R>} An Observable that emits the values from the source
     * Observable transformed by the given `project` function.
     * @method map
     * @owner Observable
     */
    function map(project, thisArg) {
        if (typeof project !== 'function') {
            throw new TypeError('argument is not a function. Are you looking for `mapTo()`?');
        }
        return this.lift(new MapOperator(project, thisArg));
    }
    exports.map = map;
    var MapOperator = function () {
        function MapOperator(project, thisArg) {
            this.project = project;
            this.thisArg = thisArg;
        }
        MapOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new MapSubscriber(subscriber, this.project, this.thisArg));
        };
        return MapOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var MapSubscriber = function (_super) {
        __extends(MapSubscriber, _super);
        function MapSubscriber(destination, project, thisArg) {
            _super.call(this, destination);
            this.project = project;
            this.count = 0;
            this.thisArg = thisArg || this;
        }
        // NOTE: This looks unoptimized, but it's actually purposefully NOT
        // using try/catch optimizations.
        MapSubscriber.prototype._next = function (value) {
            var result;
            try {
                result = this.project.call(this.thisArg, value, this.count++);
            } catch (err) {
                this.destination.error(err);
                return;
            }
            this.destination.next(result);
        };
        return MapSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/operator/pluck.js', ['./map'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var map_1 = $__require('./map');
    /**
     * Maps each source value (an object) to its specified nested property.
     *
     * <span class="informal">Like {@link map}, but meant only for picking one of
     * the nested properties of every emitted object.</span>
     *
     * <img src="./img/pluck.png" width="100%">
     *
     * Given a list of strings describing a path to an object property, retrieves
     * the value of a specified nested property from all values in the source
     * Observable. If a property can't be resolved, it will return `undefined` for
     * that value.
     *
     * @example <caption>Map every every click to the tagName of the clicked target element</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var tagNames = clicks.pluck('target', 'tagName');
     * tagNames.subscribe(x => console.log(x));
     *
     * @see {@link map}
     *
     * @param {...string} properties The nested properties to pluck from each source
     * value (an object).
     * @return {Observable} Returns a new Observable of property values from the
     * source values.
     * @method pluck
     * @owner Observable
     */
    function pluck() {
        var properties = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            properties[_i - 0] = arguments[_i];
        }
        var length = properties.length;
        if (length === 0) {
            throw new Error('List of properties cannot be empty.');
        }
        return map_1.map.call(this, plucker(properties, length));
    }
    exports.pluck = pluck;
    function plucker(props, length) {
        var mapper = function (x) {
            var currentProp = x;
            for (var i = 0; i < length; i++) {
                var p = currentProp[props[i]];
                if (typeof p !== 'undefined') {
                    currentProp = p;
                } else {
                    return undefined;
                }
            }
            return currentProp;
        };
        return mapper;
    }
    
});
System.registerDynamic('node_modules/rxjs/add/operator/pluck.js', ['../../Observable', '../../operator/pluck'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var pluck_1 = $__require('../../operator/pluck');
  Observable_1.Observable.prototype.pluck = pluck_1.pluck;
  
});
System.registerDynamic('node_modules/rxjs/operator/publish.js', ['../Subject', './multicast'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Subject_1 = $__require('../Subject');
  var multicast_1 = $__require('./multicast');
  /**
   * Returns a ConnectableObservable, which is a variety of Observable that waits until its connect method is called
   * before it begins emitting items to those Observers that have subscribed to it.
   *
   * <img src="./img/publish.png" width="100%">
   *
   * @return a ConnectableObservable that upon connection causes the source Observable to emit items to its Observers.
   * @method publish
   * @owner Observable
   */
  function publish() {
    return multicast_1.multicast.call(this, new Subject_1.Subject());
  }
  exports.publish = publish;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/publish.js', ['../../Observable', '../../operator/publish'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var publish_1 = $__require('../../operator/publish');
  Observable_1.Observable.prototype.publish = publish_1.publish;
  
});
System.registerDynamic('node_modules/rxjs/operator/publishBehavior.js', ['../BehaviorSubject', './multicast'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var BehaviorSubject_1 = $__require('../BehaviorSubject');
  var multicast_1 = $__require('./multicast');
  /**
   * @param value
   * @return {ConnectableObservable<T>}
   * @method publishBehavior
   * @owner Observable
   */
  function publishBehavior(value) {
    return multicast_1.multicast.call(this, new BehaviorSubject_1.BehaviorSubject(value));
  }
  exports.publishBehavior = publishBehavior;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/publishBehavior.js', ['../../Observable', '../../operator/publishBehavior'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var publishBehavior_1 = $__require('../../operator/publishBehavior');
  Observable_1.Observable.prototype.publishBehavior = publishBehavior_1.publishBehavior;
  
});
System.registerDynamic('node_modules/rxjs/operator/publishReplay.js', ['../ReplaySubject', './multicast'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var ReplaySubject_1 = $__require('../ReplaySubject');
  var multicast_1 = $__require('./multicast');
  /**
   * @param bufferSize
   * @param windowTime
   * @param scheduler
   * @return {ConnectableObservable<T>}
   * @method publishReplay
   * @owner Observable
   */
  function publishReplay(bufferSize, windowTime, scheduler) {
    if (bufferSize === void 0) {
      bufferSize = Number.POSITIVE_INFINITY;
    }
    if (windowTime === void 0) {
      windowTime = Number.POSITIVE_INFINITY;
    }
    return multicast_1.multicast.call(this, new ReplaySubject_1.ReplaySubject(bufferSize, windowTime, scheduler));
  }
  exports.publishReplay = publishReplay;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/publishReplay.js', ['../../Observable', '../../operator/publishReplay'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var publishReplay_1 = $__require('../../operator/publishReplay');
  Observable_1.Observable.prototype.publishReplay = publishReplay_1.publishReplay;
  
});
System.registerDynamic('node_modules/rxjs/operator/publishLast.js', ['../AsyncSubject', './multicast'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var AsyncSubject_1 = $__require('../AsyncSubject');
  var multicast_1 = $__require('./multicast');
  /**
   * @return {ConnectableObservable<T>}
   * @method publishLast
   * @owner Observable
   */
  function publishLast() {
    return multicast_1.multicast.call(this, new AsyncSubject_1.AsyncSubject());
  }
  exports.publishLast = publishLast;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/publishLast.js', ['../../Observable', '../../operator/publishLast'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var publishLast_1 = $__require('../../operator/publishLast');
  Observable_1.Observable.prototype.publishLast = publishLast_1.publishLast;
  
});
System.registerDynamic('node_modules/rxjs/operator/race.js', ['../util/isArray', '../observable/ArrayObservable', '../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var isArray_1 = $__require('../util/isArray');
    var ArrayObservable_1 = $__require('../observable/ArrayObservable');
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Returns an Observable that mirrors the first source Observable to emit an item
     * from the combination of this Observable and supplied Observables
     * @param {...Observables} ...observables sources used to race for which Observable emits first.
     * @return {Observable} an Observable that mirrors the output of the first Observable to emit an item.
     * @method race
     * @owner Observable
     */
    function race() {
        var observables = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            observables[_i - 0] = arguments[_i];
        }
        // if the only argument is an array, it was most likely called with
        // `pair([obs1, obs2, ...])`
        if (observables.length === 1 && isArray_1.isArray(observables[0])) {
            observables = observables[0];
        }
        observables.unshift(this);
        return raceStatic.apply(this, observables);
    }
    exports.race = race;
    function raceStatic() {
        var observables = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            observables[_i - 0] = arguments[_i];
        }
        // if the only argument is an array, it was most likely called with
        // `pair([obs1, obs2, ...])`
        if (observables.length === 1) {
            if (isArray_1.isArray(observables[0])) {
                observables = observables[0];
            } else {
                return observables[0];
            }
        }
        return new ArrayObservable_1.ArrayObservable(observables).lift(new RaceOperator());
    }
    exports.raceStatic = raceStatic;
    var RaceOperator = function () {
        function RaceOperator() {}
        RaceOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new RaceSubscriber(subscriber));
        };
        return RaceOperator;
    }();
    exports.RaceOperator = RaceOperator;
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var RaceSubscriber = function (_super) {
        __extends(RaceSubscriber, _super);
        function RaceSubscriber(destination) {
            _super.call(this, destination);
            this.hasFirst = false;
            this.observables = [];
            this.subscriptions = [];
        }
        RaceSubscriber.prototype._next = function (observable) {
            this.observables.push(observable);
        };
        RaceSubscriber.prototype._complete = function () {
            var observables = this.observables;
            var len = observables.length;
            if (len === 0) {
                this.destination.complete();
            } else {
                for (var i = 0; i < len; i++) {
                    var observable = observables[i];
                    var subscription = subscribeToResult_1.subscribeToResult(this, observable, observable, i);
                    this.subscriptions.push(subscription);
                    this.add(subscription);
                }
                this.observables = null;
            }
        };
        RaceSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            if (!this.hasFirst) {
                this.hasFirst = true;
                for (var i = 0; i < this.subscriptions.length; i++) {
                    if (i !== outerIndex) {
                        var subscription = this.subscriptions[i];
                        subscription.unsubscribe();
                        this.remove(subscription);
                    }
                }
                this.subscriptions = null;
            }
            this.destination.next(innerValue);
        };
        return RaceSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    exports.RaceSubscriber = RaceSubscriber;
    
});
System.registerDynamic('node_modules/rxjs/add/operator/race.js', ['../../Observable', '../../operator/race'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var race_1 = $__require('../../operator/race');
  Observable_1.Observable.prototype.race = race_1.race;
  
});
System.registerDynamic('node_modules/rxjs/operator/reduce.js', ['../Subscriber'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * Returns an Observable that applies a specified accumulator function to the first item emitted by a source Observable,
     * then feeds the result of that function along with the second item emitted by the source Observable into the same
     * function, and so on until all items have been emitted by the source Observable, and emits the final result from
     * the final call to your function as its sole item.
     * This technique, which is called "reduce" here, is sometimes called "aggregate," "fold," "accumulate," "compress," or
     * "inject" in other programming contexts.
     *
     * <img src="./img/reduce.png" width="100%">
     *
     * @param {initialValue} the initial (seed) accumulator value
     * @param {accumulator} an accumulator function to be invoked on each item emitted by the source Observable, the
     * result of which will be used in the next accumulator call.
     * @return {Observable} an Observable that emits a single item that is the result of accumulating the output from the
     * items emitted by the source Observable.
     * @method reduce
     * @owner Observable
     */
    function reduce(project, seed) {
        return this.lift(new ReduceOperator(project, seed));
    }
    exports.reduce = reduce;
    var ReduceOperator = function () {
        function ReduceOperator(project, seed) {
            this.project = project;
            this.seed = seed;
        }
        ReduceOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new ReduceSubscriber(subscriber, this.project, this.seed));
        };
        return ReduceOperator;
    }();
    exports.ReduceOperator = ReduceOperator;
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var ReduceSubscriber = function (_super) {
        __extends(ReduceSubscriber, _super);
        function ReduceSubscriber(destination, project, seed) {
            _super.call(this, destination);
            this.hasValue = false;
            this.acc = seed;
            this.project = project;
            this.hasSeed = typeof seed !== 'undefined';
        }
        ReduceSubscriber.prototype._next = function (value) {
            if (this.hasValue || (this.hasValue = this.hasSeed)) {
                this._tryReduce(value);
            } else {
                this.acc = value;
                this.hasValue = true;
            }
        };
        ReduceSubscriber.prototype._tryReduce = function (value) {
            var result;
            try {
                result = this.project(this.acc, value);
            } catch (err) {
                this.destination.error(err);
                return;
            }
            this.acc = result;
        };
        ReduceSubscriber.prototype._complete = function () {
            if (this.hasValue || this.hasSeed) {
                this.destination.next(this.acc);
            }
            this.destination.complete();
        };
        return ReduceSubscriber;
    }(Subscriber_1.Subscriber);
    exports.ReduceSubscriber = ReduceSubscriber;
    
});
System.registerDynamic('node_modules/rxjs/add/operator/reduce.js', ['../../Observable', '../../operator/reduce'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var reduce_1 = $__require('../../operator/reduce');
  Observable_1.Observable.prototype.reduce = reduce_1.reduce;
  
});
System.registerDynamic('node_modules/rxjs/operator/repeat.js', ['../Subscriber', '../observable/EmptyObservable'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var EmptyObservable_1 = $__require('../observable/EmptyObservable');
    /**
     * Returns an Observable that repeats the stream of items emitted by the source Observable at most count times,
     * on a particular Scheduler.
     *
     * <img src="./img/repeat.png" width="100%">
     *
     * @param {Scheduler} [scheduler] the Scheduler to emit the items on.
     * @param {number} [count] the number of times the source Observable items are repeated, a count of 0 will yield
     * an empty Observable.
     * @return {Observable} an Observable that repeats the stream of items emitted by the source Observable at most
     * count times.
     * @method repeat
     * @owner Observable
     */
    function repeat(count) {
        if (count === void 0) {
            count = -1;
        }
        if (count === 0) {
            return new EmptyObservable_1.EmptyObservable();
        } else if (count < 0) {
            return this.lift(new RepeatOperator(-1, this));
        } else {
            return this.lift(new RepeatOperator(count - 1, this));
        }
    }
    exports.repeat = repeat;
    var RepeatOperator = function () {
        function RepeatOperator(count, source) {
            this.count = count;
            this.source = source;
        }
        RepeatOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new RepeatSubscriber(subscriber, this.count, this.source));
        };
        return RepeatOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var RepeatSubscriber = function (_super) {
        __extends(RepeatSubscriber, _super);
        function RepeatSubscriber(destination, count, source) {
            _super.call(this, destination);
            this.count = count;
            this.source = source;
        }
        RepeatSubscriber.prototype.complete = function () {
            if (!this.isStopped) {
                var _a = this,
                    source = _a.source,
                    count = _a.count;
                if (count === 0) {
                    return _super.prototype.complete.call(this);
                } else if (count > -1) {
                    this.count = count - 1;
                }
                this.unsubscribe();
                this.isStopped = false;
                this.isUnsubscribed = false;
                source.subscribe(this);
            }
        };
        return RepeatSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/repeat.js', ['../../Observable', '../../operator/repeat'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var repeat_1 = $__require('../../operator/repeat');
  Observable_1.Observable.prototype.repeat = repeat_1.repeat;
  
});
System.registerDynamic("node_modules/rxjs/operator/retry.js", ["../Subscriber"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * Returns an Observable that mirrors the source Observable, resubscribing to it if it calls `error` and the
     * predicate returns true for that specific exception and retry count.
     * If the source Observable calls `error`, this method will resubscribe to the source Observable for a maximum of
     * count resubscriptions (given as a number parameter) rather than propagating the `error` call.
     *
     * <img src="./img/retry.png" width="100%">
     *
     * Any and all items emitted by the source Observable will be emitted by the resulting Observable, even those emitted
     * during failed subscriptions. For example, if an Observable fails at first but emits [1, 2] then succeeds the second
     * time and emits: [1, 2, 3, 4, 5] then the complete stream of emissions and notifications
     * would be: [1, 2, 1, 2, 3, 4, 5, `complete`].
     * @param {number} number of retry attempts before failing.
     * @return {Observable} the source Observable modified with the retry logic.
     * @method retry
     * @owner Observable
     */
    function retry(count) {
        if (count === void 0) {
            count = -1;
        }
        return this.lift(new RetryOperator(count, this));
    }
    exports.retry = retry;
    var RetryOperator = function () {
        function RetryOperator(count, source) {
            this.count = count;
            this.source = source;
        }
        RetryOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new RetrySubscriber(subscriber, this.count, this.source));
        };
        return RetryOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var RetrySubscriber = function (_super) {
        __extends(RetrySubscriber, _super);
        function RetrySubscriber(destination, count, source) {
            _super.call(this, destination);
            this.count = count;
            this.source = source;
        }
        RetrySubscriber.prototype.error = function (err) {
            if (!this.isStopped) {
                var _a = this,
                    source = _a.source,
                    count = _a.count;
                if (count === 0) {
                    return _super.prototype.error.call(this, err);
                } else if (count > -1) {
                    this.count = count - 1;
                }
                this.unsubscribe();
                this.isStopped = false;
                this.isUnsubscribed = false;
                source.subscribe(this);
            }
        };
        return RetrySubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/retry.js', ['../../Observable', '../../operator/retry'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var retry_1 = $__require('../../operator/retry');
  Observable_1.Observable.prototype.retry = retry_1.retry;
  
});
System.registerDynamic('node_modules/rxjs/operator/retryWhen.js', ['../Subject', '../util/tryCatch', '../util/errorObject', '../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subject_1 = $__require('../Subject');
    var tryCatch_1 = $__require('../util/tryCatch');
    var errorObject_1 = $__require('../util/errorObject');
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Returns an Observable that emits the same values as the source observable with the exception of an `error`.
     * An `error` will cause the emission of the Throwable that cause the error to the Observable returned from
     * notificationHandler. If that Observable calls onComplete or `error` then retry will call `complete` or `error`
     * on the child subscription. Otherwise, this Observable will resubscribe to the source observable, on a particular
     * Scheduler.
     *
     * <img src="./img/retryWhen.png" width="100%">
     *
     * @param {notificationHandler} receives an Observable of notifications with which a user can `complete` or `error`,
     * aborting the retry.
     * @param {scheduler} the Scheduler on which to subscribe to the source Observable.
     * @return {Observable} the source Observable modified with retry logic.
     * @method retryWhen
     * @owner Observable
     */
    function retryWhen(notifier) {
        return this.lift(new RetryWhenOperator(notifier, this));
    }
    exports.retryWhen = retryWhen;
    var RetryWhenOperator = function () {
        function RetryWhenOperator(notifier, source) {
            this.notifier = notifier;
            this.source = source;
        }
        RetryWhenOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new RetryWhenSubscriber(subscriber, this.notifier, this.source));
        };
        return RetryWhenOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var RetryWhenSubscriber = function (_super) {
        __extends(RetryWhenSubscriber, _super);
        function RetryWhenSubscriber(destination, notifier, source) {
            _super.call(this, destination);
            this.notifier = notifier;
            this.source = source;
        }
        RetryWhenSubscriber.prototype.error = function (err) {
            if (!this.isStopped) {
                var errors = this.errors;
                var retries = this.retries;
                var retriesSubscription = this.retriesSubscription;
                if (!retries) {
                    errors = new Subject_1.Subject();
                    retries = tryCatch_1.tryCatch(this.notifier)(errors);
                    if (retries === errorObject_1.errorObject) {
                        return _super.prototype.error.call(this, errorObject_1.errorObject.e);
                    }
                    retriesSubscription = subscribeToResult_1.subscribeToResult(this, retries);
                } else {
                    this.errors = null;
                    this.retriesSubscription = null;
                }
                this.unsubscribe();
                this.isUnsubscribed = false;
                this.errors = errors;
                this.retries = retries;
                this.retriesSubscription = retriesSubscription;
                errors.next(err);
            }
        };
        RetryWhenSubscriber.prototype._unsubscribe = function () {
            var _a = this,
                errors = _a.errors,
                retriesSubscription = _a.retriesSubscription;
            if (errors) {
                errors.unsubscribe();
                this.errors = null;
            }
            if (retriesSubscription) {
                retriesSubscription.unsubscribe();
                this.retriesSubscription = null;
            }
            this.retries = null;
        };
        RetryWhenSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            var _a = this,
                errors = _a.errors,
                retries = _a.retries,
                retriesSubscription = _a.retriesSubscription;
            this.errors = null;
            this.retries = null;
            this.retriesSubscription = null;
            this.unsubscribe();
            this.isStopped = false;
            this.isUnsubscribed = false;
            this.errors = errors;
            this.retries = retries;
            this.retriesSubscription = retriesSubscription;
            this.source.subscribe(this);
        };
        return RetryWhenSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/retryWhen.js', ['../../Observable', '../../operator/retryWhen'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var retryWhen_1 = $__require('../../operator/retryWhen');
  Observable_1.Observable.prototype.retryWhen = retryWhen_1.retryWhen;
  
});
System.registerDynamic('node_modules/rxjs/operator/sample.js', ['../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Returns an Observable that, when the specified sampler Observable emits an item or completes, it then emits the most
     * recently emitted item (if any) emitted by the source Observable since the previous emission from the sampler
     * Observable.
     *
     * <img src="./img/sample.png" width="100%">
     *
     * @param {Observable} sampler - the Observable to use for sampling the source Observable.
     * @return {Observable<T>} an Observable that emits the results of sampling the items emitted by this Observable
     * whenever the sampler Observable emits an item or completes.
     * @method sample
     * @owner Observable
     */
    function sample(notifier) {
        return this.lift(new SampleOperator(notifier));
    }
    exports.sample = sample;
    var SampleOperator = function () {
        function SampleOperator(notifier) {
            this.notifier = notifier;
        }
        SampleOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new SampleSubscriber(subscriber, this.notifier));
        };
        return SampleOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var SampleSubscriber = function (_super) {
        __extends(SampleSubscriber, _super);
        function SampleSubscriber(destination, notifier) {
            _super.call(this, destination);
            this.hasValue = false;
            this.add(subscribeToResult_1.subscribeToResult(this, notifier));
        }
        SampleSubscriber.prototype._next = function (value) {
            this.value = value;
            this.hasValue = true;
        };
        SampleSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            this.emitValue();
        };
        SampleSubscriber.prototype.notifyComplete = function () {
            this.emitValue();
        };
        SampleSubscriber.prototype.emitValue = function () {
            if (this.hasValue) {
                this.hasValue = false;
                this.destination.next(this.value);
            }
        };
        return SampleSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/sample.js', ['../../Observable', '../../operator/sample'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var sample_1 = $__require('../../operator/sample');
  Observable_1.Observable.prototype.sample = sample_1.sample;
  
});
System.registerDynamic('node_modules/rxjs/operator/sampleTime.js', ['../Subscriber', '../scheduler/async'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var async_1 = $__require('../scheduler/async');
    /**
     * @param delay
     * @param scheduler
     * @return {Observable<R>|WebSocketSubject<T>|Observable<T>}
     * @method sampleTime
     * @owner Observable
     */
    function sampleTime(delay, scheduler) {
        if (scheduler === void 0) {
            scheduler = async_1.async;
        }
        return this.lift(new SampleTimeOperator(delay, scheduler));
    }
    exports.sampleTime = sampleTime;
    var SampleTimeOperator = function () {
        function SampleTimeOperator(delay, scheduler) {
            this.delay = delay;
            this.scheduler = scheduler;
        }
        SampleTimeOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new SampleTimeSubscriber(subscriber, this.delay, this.scheduler));
        };
        return SampleTimeOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var SampleTimeSubscriber = function (_super) {
        __extends(SampleTimeSubscriber, _super);
        function SampleTimeSubscriber(destination, delay, scheduler) {
            _super.call(this, destination);
            this.delay = delay;
            this.scheduler = scheduler;
            this.hasValue = false;
            this.add(scheduler.schedule(dispatchNotification, delay, { subscriber: this, delay: delay }));
        }
        SampleTimeSubscriber.prototype._next = function (value) {
            this.lastValue = value;
            this.hasValue = true;
        };
        SampleTimeSubscriber.prototype.notifyNext = function () {
            if (this.hasValue) {
                this.hasValue = false;
                this.destination.next(this.lastValue);
            }
        };
        return SampleTimeSubscriber;
    }(Subscriber_1.Subscriber);
    function dispatchNotification(state) {
        var subscriber = state.subscriber,
            delay = state.delay;
        subscriber.notifyNext();
        this.schedule(state, delay);
    }
    
});
System.registerDynamic('node_modules/rxjs/add/operator/sampleTime.js', ['../../Observable', '../../operator/sampleTime'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var sampleTime_1 = $__require('../../operator/sampleTime');
  Observable_1.Observable.prototype.sampleTime = sampleTime_1.sampleTime;
  
});
System.registerDynamic('node_modules/rxjs/operator/scan.js', ['../Subscriber'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * Applies an accumulation function over the source Observable, and returns each
     * intermediate result, with an optional seed value.
     *
     * <span class="informal">It's like {@link reduce}, but emits the current
     * accumulation whenever the source emits a value.</span>
     *
     * <img src="./img/scan.png" width="100%">
     *
     * Combines together all values emitted on the source, using an accumulator
     * function that knows how to join a new source value into the accumulation from
     * the past. Is similar to {@link reduce}, but emits the intermediate
     * accumulations.
     *
     * Returns an Observable that applies a specified `accumulator` function to each
     * item emitted by the source Observable. If a `seed` value is specified, then
     * that value will be used as the initial value for the accumulator. If no seed
     * value is specified, the first item of the source is used as the seed.
     *
     * @example <caption>Count the number of click events</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var ones = clicks.mapTo(1);
     * var seed = 0;
     * var count = ones.scan((acc, one) => acc + one, seed);
     * count.subscribe(x => console.log(x));
     *
     * @see {@link expand}
     * @see {@link mergeScan}
     * @see {@link reduce}
     *
     * @param {function(acc: R, value: T): R} accumulator The accumulator function
     * called on each source value.
     * @param {T|R} [seed] The initial accumulation value.
     * @return {Observable<R>} An observable of the accumulated values.
     * @method scan
     * @owner Observable
     */
    function scan(accumulator, seed) {
        return this.lift(new ScanOperator(accumulator, seed));
    }
    exports.scan = scan;
    var ScanOperator = function () {
        function ScanOperator(accumulator, seed) {
            this.accumulator = accumulator;
            this.seed = seed;
        }
        ScanOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new ScanSubscriber(subscriber, this.accumulator, this.seed));
        };
        return ScanOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var ScanSubscriber = function (_super) {
        __extends(ScanSubscriber, _super);
        function ScanSubscriber(destination, accumulator, seed) {
            _super.call(this, destination);
            this.accumulator = accumulator;
            this.accumulatorSet = false;
            this.seed = seed;
            this.accumulator = accumulator;
            this.accumulatorSet = typeof seed !== 'undefined';
        }
        Object.defineProperty(ScanSubscriber.prototype, "seed", {
            get: function () {
                return this._seed;
            },
            set: function (value) {
                this.accumulatorSet = true;
                this._seed = value;
            },
            enumerable: true,
            configurable: true
        });
        ScanSubscriber.prototype._next = function (value) {
            if (!this.accumulatorSet) {
                this.seed = value;
                this.destination.next(value);
            } else {
                return this._tryNext(value);
            }
        };
        ScanSubscriber.prototype._tryNext = function (value) {
            var result;
            try {
                result = this.accumulator(this.seed, value);
            } catch (err) {
                this.destination.error(err);
            }
            this.seed = result;
            this.destination.next(result);
        };
        return ScanSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/scan.js', ['../../Observable', '../../operator/scan'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var scan_1 = $__require('../../operator/scan');
  Observable_1.Observable.prototype.scan = scan_1.scan;
  
});
System.registerDynamic('node_modules/rxjs/operator/multicast.js', ['../observable/ConnectableObservable'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var ConnectableObservable_1 = $__require('../observable/ConnectableObservable');
    /**
     * Returns an Observable that emits the results of invoking a specified selector on items
     * emitted by a ConnectableObservable that shares a single subscription to the underlying stream.
     *
     * <img src="./img/multicast.png" width="100%">
     *
     * @param {Function} selector - a function that can use the multicasted source stream
     * as many times as needed, without causing multiple subscriptions to the source stream.
     * Subscribers to the given source will receive all notifications of the source from the
     * time of the subscription forward.
     * @return {Observable} an Observable that emits the results of invoking the selector
     * on the items emitted by a `ConnectableObservable` that shares a single subscription to
     * the underlying stream.
     * @method multicast
     * @owner Observable
     */
    function multicast(subjectOrSubjectFactory) {
        var subjectFactory;
        if (typeof subjectOrSubjectFactory === 'function') {
            subjectFactory = subjectOrSubjectFactory;
        } else {
            subjectFactory = function subjectFactory() {
                return subjectOrSubjectFactory;
            };
        }
        return new ConnectableObservable_1.ConnectableObservable(this, subjectFactory);
    }
    exports.multicast = multicast;
    
});
System.registerDynamic('node_modules/rxjs/operator/share.js', ['./multicast', '../Subject'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var multicast_1 = $__require('./multicast');
    var Subject_1 = $__require('../Subject');
    function shareSubjectFactory() {
        return new Subject_1.Subject();
    }
    /**
     * Returns a new Observable that multicasts (shares) the original Observable. As long as there is at least one
     * Subscriber this Observable will be subscribed and emitting data. When all subscribers have unsubscribed it will
     * unsubscribe from the source Observable. Because the Observable is multicasting it makes the stream `hot`.
     * This is an alias for .publish().refCount().
     *
     * <img src="./img/share.png" width="100%">
     *
     * @return {Observable<T>} an Observable that upon connection causes the source Observable to emit items to its Observers
     * @method share
     * @owner Observable
     */
    function share() {
        return multicast_1.multicast.call(this, shareSubjectFactory).refCount();
    }
    exports.share = share;
    ;
    
});
System.registerDynamic('node_modules/rxjs/add/operator/share.js', ['../../Observable', '../../operator/share'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var share_1 = $__require('../../operator/share');
  Observable_1.Observable.prototype.share = share_1.share;
  
});
System.registerDynamic('node_modules/rxjs/operator/single.js', ['../Subscriber', '../util/EmptyError'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var EmptyError_1 = $__require('../util/EmptyError');
    /**
     * Returns an Observable that emits the single item emitted by the source Observable that matches a specified
     * predicate, if that Observable emits one such item. If the source Observable emits more than one such item or no
     * such items, notify of an IllegalArgumentException or NoSuchElementException respectively.
     *
     * <img src="./img/single.png" width="100%">
     *
     * @throws {EmptyError} Delivers an EmptyError to the Observer's `error`
     * callback if the Observable completes before any `next` notification was sent.
     * @param {Function} a predicate function to evaluate items emitted by the source Observable.
     * @return {Observable<T>} an Observable that emits the single item emitted by the source Observable that matches
     * the predicate.
     .
     * @method single
     * @owner Observable
     */
    function single(predicate) {
        return this.lift(new SingleOperator(predicate, this));
    }
    exports.single = single;
    var SingleOperator = function () {
        function SingleOperator(predicate, source) {
            this.predicate = predicate;
            this.source = source;
        }
        SingleOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new SingleSubscriber(subscriber, this.predicate, this.source));
        };
        return SingleOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var SingleSubscriber = function (_super) {
        __extends(SingleSubscriber, _super);
        function SingleSubscriber(destination, predicate, source) {
            _super.call(this, destination);
            this.predicate = predicate;
            this.source = source;
            this.seenValue = false;
            this.index = 0;
        }
        SingleSubscriber.prototype.applySingleValue = function (value) {
            if (this.seenValue) {
                this.destination.error('Sequence contains more than one element');
            } else {
                this.seenValue = true;
                this.singleValue = value;
            }
        };
        SingleSubscriber.prototype._next = function (value) {
            var predicate = this.predicate;
            this.index++;
            if (predicate) {
                this.tryNext(value);
            } else {
                this.applySingleValue(value);
            }
        };
        SingleSubscriber.prototype.tryNext = function (value) {
            try {
                var result = this.predicate(value, this.index, this.source);
                if (result) {
                    this.applySingleValue(value);
                }
            } catch (err) {
                this.destination.error(err);
            }
        };
        SingleSubscriber.prototype._complete = function () {
            var destination = this.destination;
            if (this.index > 0) {
                destination.next(this.seenValue ? this.singleValue : undefined);
                destination.complete();
            } else {
                destination.error(new EmptyError_1.EmptyError());
            }
        };
        return SingleSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/single.js', ['../../Observable', '../../operator/single'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var single_1 = $__require('../../operator/single');
  Observable_1.Observable.prototype.single = single_1.single;
  
});
System.registerDynamic("node_modules/rxjs/operator/skip.js", ["../Subscriber"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * Returns an Observable that skips `n` items emitted by an Observable.
     *
     * <img src="./img/skip.png" width="100%">
     *
     * @param {Number} the `n` of times, items emitted by source Observable should be skipped.
     * @return {Observable} an Observable that skips values emitted by the source Observable.
     *
     * @method skip
     * @owner Observable
     */
    function skip(total) {
        return this.lift(new SkipOperator(total));
    }
    exports.skip = skip;
    var SkipOperator = function () {
        function SkipOperator(total) {
            this.total = total;
        }
        SkipOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new SkipSubscriber(subscriber, this.total));
        };
        return SkipOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var SkipSubscriber = function (_super) {
        __extends(SkipSubscriber, _super);
        function SkipSubscriber(destination, total) {
            _super.call(this, destination);
            this.total = total;
            this.count = 0;
        }
        SkipSubscriber.prototype._next = function (x) {
            if (++this.count > this.total) {
                this.destination.next(x);
            }
        };
        return SkipSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/skip.js', ['../../Observable', '../../operator/skip'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var skip_1 = $__require('../../operator/skip');
  Observable_1.Observable.prototype.skip = skip_1.skip;
  
});
System.registerDynamic('node_modules/rxjs/operator/skipUntil.js', ['../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Returns an Observable that skips items emitted by the source Observable until a second Observable emits an item.
     *
     * <img src="./img/skipUntil.png" width="100%">
     *
     * @param {Observable} the second Observable that has to emit an item before the source Observable's elements begin to
     * be mirrored by the resulting Observable.
     * @return {Observable<T>} an Observable that skips items from the source Observable until the second Observable emits
     * an item, then emits the remaining items.
     * @method skipUntil
     * @owner Observable
     */
    function skipUntil(notifier) {
        return this.lift(new SkipUntilOperator(notifier));
    }
    exports.skipUntil = skipUntil;
    var SkipUntilOperator = function () {
        function SkipUntilOperator(notifier) {
            this.notifier = notifier;
        }
        SkipUntilOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new SkipUntilSubscriber(subscriber, this.notifier));
        };
        return SkipUntilOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var SkipUntilSubscriber = function (_super) {
        __extends(SkipUntilSubscriber, _super);
        function SkipUntilSubscriber(destination, notifier) {
            _super.call(this, destination);
            this.hasValue = false;
            this.isInnerStopped = false;
            this.add(subscribeToResult_1.subscribeToResult(this, notifier));
        }
        SkipUntilSubscriber.prototype._next = function (value) {
            if (this.hasValue) {
                _super.prototype._next.call(this, value);
            }
        };
        SkipUntilSubscriber.prototype._complete = function () {
            if (this.isInnerStopped) {
                _super.prototype._complete.call(this);
            } else {
                this.unsubscribe();
            }
        };
        SkipUntilSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            this.hasValue = true;
        };
        SkipUntilSubscriber.prototype.notifyComplete = function () {
            this.isInnerStopped = true;
            if (this.isStopped) {
                _super.prototype._complete.call(this);
            }
        };
        return SkipUntilSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/skipUntil.js', ['../../Observable', '../../operator/skipUntil'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var skipUntil_1 = $__require('../../operator/skipUntil');
  Observable_1.Observable.prototype.skipUntil = skipUntil_1.skipUntil;
  
});
System.registerDynamic("node_modules/rxjs/operator/skipWhile.js", ["../Subscriber"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * Returns an Observable that skips all items emitted by the source Observable as long as a specified condition holds
     * true, but emits all further source items as soon as the condition becomes false.
     *
     * <img src="./img/skipWhile.png" width="100%">
     *
     * @param {Function} predicate - a function to test each item emitted from the source Observable.
     * @return {Observable<T>} an Observable that begins emitting items emitted by the source Observable when the
     * specified predicate becomes false.
     * @method skipWhile
     * @owner Observable
     */
    function skipWhile(predicate) {
        return this.lift(new SkipWhileOperator(predicate));
    }
    exports.skipWhile = skipWhile;
    var SkipWhileOperator = function () {
        function SkipWhileOperator(predicate) {
            this.predicate = predicate;
        }
        SkipWhileOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new SkipWhileSubscriber(subscriber, this.predicate));
        };
        return SkipWhileOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var SkipWhileSubscriber = function (_super) {
        __extends(SkipWhileSubscriber, _super);
        function SkipWhileSubscriber(destination, predicate) {
            _super.call(this, destination);
            this.predicate = predicate;
            this.skipping = true;
            this.index = 0;
        }
        SkipWhileSubscriber.prototype._next = function (value) {
            var destination = this.destination;
            if (this.skipping) {
                this.tryCallPredicate(value);
            }
            if (!this.skipping) {
                destination.next(value);
            }
        };
        SkipWhileSubscriber.prototype.tryCallPredicate = function (value) {
            try {
                var result = this.predicate(value, this.index++);
                this.skipping = Boolean(result);
            } catch (err) {
                this.destination.error(err);
            }
        };
        return SkipWhileSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/skipWhile.js', ['../../Observable', '../../operator/skipWhile'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var skipWhile_1 = $__require('../../operator/skipWhile');
  Observable_1.Observable.prototype.skipWhile = skipWhile_1.skipWhile;
  
});
System.registerDynamic('node_modules/rxjs/operator/mergeAll.js', ['../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Converts a higher-order Observable into a first-order Observable which
     * concurrently delivers all values that are emitted on the inner Observables.
     *
     * <span class="informal">Flattens an Observable-of-Observables.</span>
     *
     * <img src="./img/mergeAll.png" width="100%">
     *
     * `mergeAll` subscribes to an Observable that emits Observables, also known as
     * a higher-order Observable. Each time it observes one of these emitted inner
     * Observables, it subscribes to that and delivers all the values from the
     * inner Observable on the output Observable. The output Observable only
     * completes once all inner Observables have completed. Any error delivered by
     * a inner Observable will be immediately emitted on the output Observable.
     *
     * @example <caption>Spawn a new interval Observable for each click event, and blend their outputs as one Observable</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var higherOrder = clicks.map((ev) => Rx.Observable.interval(1000));
     * var firstOrder = higherOrder.mergeAll();
     * firstOrder.subscribe(x => console.log(x));
     *
     * @example <caption>Count from 0 to 9 every second for each click, but only allow 2 concurrent timers</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var higherOrder = clicks.map((ev) => Rx.Observable.interval(1000).take(10));
     * var firstOrder = higherOrder.mergeAll(2);
     * firstOrder.subscribe(x => console.log(x));
     *
     * @see {@link combineAll}
     * @see {@link concatAll}
     * @see {@link exhaust}
     * @see {@link merge}
     * @see {@link mergeMap}
     * @see {@link mergeMapTo}
     * @see {@link mergeScan}
     * @see {@link switch}
     * @see {@link zipAll}
     *
     * @param {number} [concurrent=Number.POSITIVE_INFINITY] Maximum number of inner
     * Observables being subscribed to concurrently.
     * @return {Observable} An Observable that emits values coming from all the
     * inner Observables emitted by the source Observable.
     * @method mergeAll
     * @owner Observable
     */
    function mergeAll(concurrent) {
        if (concurrent === void 0) {
            concurrent = Number.POSITIVE_INFINITY;
        }
        return this.lift(new MergeAllOperator(concurrent));
    }
    exports.mergeAll = mergeAll;
    var MergeAllOperator = function () {
        function MergeAllOperator(concurrent) {
            this.concurrent = concurrent;
        }
        MergeAllOperator.prototype.call = function (observer, source) {
            return source._subscribe(new MergeAllSubscriber(observer, this.concurrent));
        };
        return MergeAllOperator;
    }();
    exports.MergeAllOperator = MergeAllOperator;
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var MergeAllSubscriber = function (_super) {
        __extends(MergeAllSubscriber, _super);
        function MergeAllSubscriber(destination, concurrent) {
            _super.call(this, destination);
            this.concurrent = concurrent;
            this.hasCompleted = false;
            this.buffer = [];
            this.active = 0;
        }
        MergeAllSubscriber.prototype._next = function (observable) {
            if (this.active < this.concurrent) {
                this.active++;
                this.add(subscribeToResult_1.subscribeToResult(this, observable));
            } else {
                this.buffer.push(observable);
            }
        };
        MergeAllSubscriber.prototype._complete = function () {
            this.hasCompleted = true;
            if (this.active === 0 && this.buffer.length === 0) {
                this.destination.complete();
            }
        };
        MergeAllSubscriber.prototype.notifyComplete = function (innerSub) {
            var buffer = this.buffer;
            this.remove(innerSub);
            this.active--;
            if (buffer.length > 0) {
                this._next(buffer.shift());
            } else if (this.active === 0 && this.hasCompleted) {
                this.destination.complete();
            }
        };
        return MergeAllSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    exports.MergeAllSubscriber = MergeAllSubscriber;
    
});
System.registerDynamic('node_modules/rxjs/operator/concat.js', ['../util/isScheduler', '../observable/ArrayObservable', './mergeAll'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var isScheduler_1 = $__require('../util/isScheduler');
    var ArrayObservable_1 = $__require('../observable/ArrayObservable');
    var mergeAll_1 = $__require('./mergeAll');
    /**
     * Creates an output Observable which sequentially emits all values from every
     * given input Observable after the current Observable.
     *
     * <span class="informal">Concatenates multiple Observables together by
     * sequentially emitting their values, one Observable after the other.</span>
     *
     * <img src="./img/concat.png" width="100%">
     *
     * Joins this Observable with multiple other Observables by subscribing to them
     * one at a time, starting with the source, and merging their results into the
     * output Observable. Will wait for each Observable to complete before moving
     * on to the next.
     *
     * @example <caption>Concatenate a timer counting from 0 to 3 with a synchronous sequence from 1 to 10</caption>
     * var timer = Rx.Observable.interval(1000).take(4);
     * var sequence = Rx.Observable.range(1, 10);
     * var result = timer.concat(sequence);
     * result.subscribe(x => console.log(x));
     *
     * @example <caption>Concatenate 3 Observables</caption>
     * var timer1 = Rx.Observable.interval(1000).take(10);
     * var timer2 = Rx.Observable.interval(2000).take(6);
     * var timer3 = Rx.Observable.interval(500).take(10);
     * var result = timer1.concat(timer2, timer3);
     * result.subscribe(x => console.log(x));
     *
     * @see {@link concatAll}
     * @see {@link concatMap}
     * @see {@link concatMapTo}
     *
     * @param {Observable} other An input Observable to concatenate after the source
     * Observable. More than one input Observables may be given as argument.
     * @param {Scheduler} [scheduler=null] An optional Scheduler to schedule each
     * Observable subscription on.
     * @return {Observable} All values of each passed Observable merged into a
     * single Observable, in order, in serial fashion.
     * @method concat
     * @owner Observable
     */
    function concat() {
        var observables = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            observables[_i - 0] = arguments[_i];
        }
        return concatStatic.apply(void 0, [this].concat(observables));
    }
    exports.concat = concat;
    /* tslint:enable:max-line-length */
    /**
     * Creates an output Observable which sequentially emits all values from every
     * given input Observable after the current Observable.
     *
     * <span class="informal">Concatenates multiple Observables together by
     * sequentially emitting their values, one Observable after the other.</span>
     *
     * <img src="./img/concat.png" width="100%">
     *
     * Joins multiple Observables together by subscribing to them one at a time and
     * merging their results into the output Observable. Will wait for each
     * Observable to complete before moving on to the next.
     *
     * @example <caption>Concatenate a timer counting from 0 to 3 with a synchronous sequence from 1 to 10</caption>
     * var timer = Rx.Observable.interval(1000).take(4);
     * var sequence = Rx.Observable.range(1, 10);
     * var result = Rx.Observable.concat(timer, sequence);
     * result.subscribe(x => console.log(x));
     *
     * @example <caption>Concatenate 3 Observables</caption>
     * var timer1 = Rx.Observable.interval(1000).take(10);
     * var timer2 = Rx.Observable.interval(2000).take(6);
     * var timer3 = Rx.Observable.interval(500).take(10);
     * var result = Rx.Observable.concat(timer1, timer2, timer3);
     * result.subscribe(x => console.log(x));
     *
     * @see {@link concatAll}
     * @see {@link concatMap}
     * @see {@link concatMapTo}
     *
     * @param {Observable} input1 An input Observable to concatenate with others.
     * @param {Observable} input2 An input Observable to concatenate with others.
     * More than one input Observables may be given as argument.
     * @param {Scheduler} [scheduler=null] An optional Scheduler to schedule each
     * Observable subscription on.
     * @return {Observable} All values of each passed Observable merged into a
     * single Observable, in order, in serial fashion.
     * @static true
     * @name concat
     * @owner Observable
     */
    function concatStatic() {
        var observables = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            observables[_i - 0] = arguments[_i];
        }
        var scheduler = null;
        var args = observables;
        if (isScheduler_1.isScheduler(args[observables.length - 1])) {
            scheduler = args.pop();
        }
        return new ArrayObservable_1.ArrayObservable(observables, scheduler).lift(new mergeAll_1.MergeAllOperator(1));
    }
    exports.concatStatic = concatStatic;
    
});
System.registerDynamic('node_modules/rxjs/operator/startWith.js', ['../observable/ArrayObservable', '../observable/ScalarObservable', '../observable/EmptyObservable', './concat', '../util/isScheduler'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var ArrayObservable_1 = $__require('../observable/ArrayObservable');
    var ScalarObservable_1 = $__require('../observable/ScalarObservable');
    var EmptyObservable_1 = $__require('../observable/EmptyObservable');
    var concat_1 = $__require('./concat');
    var isScheduler_1 = $__require('../util/isScheduler');
    /**
     * Returns an Observable that emits the items in a specified Iterable before it begins to emit items emitted by the
     * source Observable.
     *
     * <img src="./img/startWith.png" width="100%">
     *
     * @param {Values} an Iterable that contains the items you want the modified Observable to emit first.
     * @return {Observable} an Observable that emits the items in the specified Iterable and then emits the items
     * emitted by the source Observable.
     * @method startWith
     * @owner Observable
     */
    function startWith() {
        var array = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            array[_i - 0] = arguments[_i];
        }
        var scheduler = array[array.length - 1];
        if (isScheduler_1.isScheduler(scheduler)) {
            array.pop();
        } else {
            scheduler = null;
        }
        var len = array.length;
        if (len === 1) {
            return concat_1.concatStatic(new ScalarObservable_1.ScalarObservable(array[0], scheduler), this);
        } else if (len > 1) {
            return concat_1.concatStatic(new ArrayObservable_1.ArrayObservable(array, scheduler), this);
        } else {
            return concat_1.concatStatic(new EmptyObservable_1.EmptyObservable(scheduler), this);
        }
    }
    exports.startWith = startWith;
    
});
System.registerDynamic('node_modules/rxjs/add/operator/startWith.js', ['../../Observable', '../../operator/startWith'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var startWith_1 = $__require('../../operator/startWith');
  Observable_1.Observable.prototype.startWith = startWith_1.startWith;
  
});
System.registerDynamic("node_modules/rxjs/util/isNumeric.js", ["../util/isArray"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var isArray_1 = $__require('../util/isArray');
    function isNumeric(val) {
        // parseFloat NaNs numeric-cast false positives (null|true|false|"")
        // ...but misinterprets leading-number strings, particularly hex literals ("0x...")
        // subtraction forces infinities to NaN
        // adding 1 corrects loss of precision from parseFloat (#15100)
        return !isArray_1.isArray(val) && val - parseFloat(val) + 1 >= 0;
    }
    exports.isNumeric = isNumeric;
    ;
    
});
System.registerDynamic('node_modules/rxjs/observable/SubscribeOnObservable.js', ['../Observable', '../scheduler/asap', '../util/isNumeric'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Observable_1 = $__require('../Observable');
    var asap_1 = $__require('../scheduler/asap');
    var isNumeric_1 = $__require('../util/isNumeric');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var SubscribeOnObservable = function (_super) {
        __extends(SubscribeOnObservable, _super);
        function SubscribeOnObservable(source, delayTime, scheduler) {
            if (delayTime === void 0) {
                delayTime = 0;
            }
            if (scheduler === void 0) {
                scheduler = asap_1.asap;
            }
            _super.call(this);
            this.source = source;
            this.delayTime = delayTime;
            this.scheduler = scheduler;
            if (!isNumeric_1.isNumeric(delayTime) || delayTime < 0) {
                this.delayTime = 0;
            }
            if (!scheduler || typeof scheduler.schedule !== 'function') {
                this.scheduler = asap_1.asap;
            }
        }
        SubscribeOnObservable.create = function (source, delay, scheduler) {
            if (delay === void 0) {
                delay = 0;
            }
            if (scheduler === void 0) {
                scheduler = asap_1.asap;
            }
            return new SubscribeOnObservable(source, delay, scheduler);
        };
        SubscribeOnObservable.dispatch = function (arg) {
            var source = arg.source,
                subscriber = arg.subscriber;
            return source.subscribe(subscriber);
        };
        SubscribeOnObservable.prototype._subscribe = function (subscriber) {
            var delay = this.delayTime;
            var source = this.source;
            var scheduler = this.scheduler;
            return scheduler.schedule(SubscribeOnObservable.dispatch, delay, {
                source: source, subscriber: subscriber
            });
        };
        return SubscribeOnObservable;
    }(Observable_1.Observable);
    exports.SubscribeOnObservable = SubscribeOnObservable;
    
});
System.registerDynamic("node_modules/rxjs/operator/subscribeOn.js", ["../observable/SubscribeOnObservable"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var SubscribeOnObservable_1 = $__require('../observable/SubscribeOnObservable');
  /**
   * Asynchronously subscribes Observers to this Observable on the specified Scheduler.
   *
   * <img src="./img/subscribeOn.png" width="100%">
   *
   * @param {Scheduler} the Scheduler to perform subscription actions on.
   * @return {Observable<T>} the source Observable modified so that its subscriptions happen on the specified Scheduler
   .
   * @method subscribeOn
   * @owner Observable
   */
  function subscribeOn(scheduler, delay) {
    if (delay === void 0) {
      delay = 0;
    }
    return new SubscribeOnObservable_1.SubscribeOnObservable(this, delay, scheduler);
  }
  exports.subscribeOn = subscribeOn;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/subscribeOn.js', ['../../Observable', '../../operator/subscribeOn'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var subscribeOn_1 = $__require('../../operator/subscribeOn');
  Observable_1.Observable.prototype.subscribeOn = subscribeOn_1.subscribeOn;
  
});
System.registerDynamic('node_modules/rxjs/operator/switch.js', ['../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Converts a higher-order Observable into a first-order Observable by
     * subscribing to only the most recently emitted of those inner Observables.
     *
     * <span class="informal">Flattens an Observable-of-Observables by dropping the
     * previous inner Observable once a new one appears.</span>
     *
     * <img src="./img/switch.png" width="100%">
     *
     * `switch` subscribes to an Observable that emits Observables, also known as a
     * higher-order Observable. Each time it observes one of these emitted inner
     * Observables, the output Observable subscribes to the inner Observable and
     * begins emitting the items emitted by that. So far, it behaves
     * like {@link mergeAll}. However, when a new inner Observable is emitted,
     * `switch` unsubscribes from the earlier-emitted inner Observable and
     * subscribes to the new inner Observable and begins emitting items from it. It
     * continues to behave like this for subsequent inner Observables.
     *
     * @example <caption>Rerun an interval Observable on every click event</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * // Each click event is mapped to an Observable that ticks every second
     * var higherOrder = clicks.map((ev) => Rx.Observable.interval(1000));
     * var switched = higherOrder.switch();
     * // The outcome is that `switched` is essentially a timer that restarts
     * // on every click. The interval Observables from older clicks do not merge
     * // with the current interval Observable.
     * switched.subscribe(x => console.log(x));
     *
     * @see {@link combineAll}
     * @see {@link concatAll}
     * @see {@link exhaust}
     * @see {@link mergeAll}
     * @see {@link switchMap}
     * @see {@link switchMapTo}
     * @see {@link zipAll}
     *
     * @return {Observable<T>} An Observable that emits the items emitted by the
     * Observable most recently emitted by the source Observable.
     * @method switch
     * @name switch
     * @owner Observable
     */
    function _switch() {
        return this.lift(new SwitchOperator());
    }
    exports._switch = _switch;
    var SwitchOperator = function () {
        function SwitchOperator() {}
        SwitchOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new SwitchSubscriber(subscriber));
        };
        return SwitchOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var SwitchSubscriber = function (_super) {
        __extends(SwitchSubscriber, _super);
        function SwitchSubscriber(destination) {
            _super.call(this, destination);
            this.active = 0;
            this.hasCompleted = false;
        }
        SwitchSubscriber.prototype._next = function (value) {
            this.unsubscribeInner();
            this.active++;
            this.add(this.innerSubscription = subscribeToResult_1.subscribeToResult(this, value));
        };
        SwitchSubscriber.prototype._complete = function () {
            this.hasCompleted = true;
            if (this.active === 0) {
                this.destination.complete();
            }
        };
        SwitchSubscriber.prototype.unsubscribeInner = function () {
            this.active = this.active > 0 ? this.active - 1 : 0;
            var innerSubscription = this.innerSubscription;
            if (innerSubscription) {
                innerSubscription.unsubscribe();
                this.remove(innerSubscription);
            }
        };
        SwitchSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            this.destination.next(innerValue);
        };
        SwitchSubscriber.prototype.notifyError = function (err) {
            this.destination.error(err);
        };
        SwitchSubscriber.prototype.notifyComplete = function () {
            this.unsubscribeInner();
            if (this.hasCompleted && this.active === 0) {
                this.destination.complete();
            }
        };
        return SwitchSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/switch.js', ['../../Observable', '../../operator/switch'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var switch_1 = $__require('../../operator/switch');
  Observable_1.Observable.prototype.switch = switch_1._switch;
  
});
System.registerDynamic('node_modules/rxjs/operator/switchMap.js', ['../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Projects each source value to an Observable which is merged in the output
     * Observable, emitting values only from the most recently projected Observable.
     *
     * <span class="informal">Maps each value to an Observable, then flattens all of
     * these inner Observables using {@link switch}.</span>
     *
     * <img src="./img/switchMap.png" width="100%">
     *
     * Returns an Observable that emits items based on applying a function that you
     * supply to each item emitted by the source Observable, where that function
     * returns an (so-called "inner") Observable. Each time it observes one of these
     * inner Observables, the output Observable begins emitting the items emitted by
     * that inner Observable. When a new inner Observable is emitted, `switchMap`
     * stops emitting items from the earlier-emitted inner Observable and begins
     * emitting items from the new one. It continues to behave like this for
     * subsequent inner Observables.
     *
     * @example <caption>Rerun an interval Observable on every click event</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var result = clicks.switchMap((ev) => Rx.Observable.interval(1000));
     * result.subscribe(x => console.log(x));
     *
     * @see {@link concatMap}
     * @see {@link exhaustMap}
     * @see {@link mergeMap}
     * @see {@link switch}
     * @see {@link switchMapTo}
     *
     * @param {function(value: T, ?index: number): Observable} project A function
     * that, when applied to an item emitted by the source Observable, returns an
     * Observable.
     * @param {function(outerValue: T, innerValue: I, outerIndex: number, innerIndex: number): any} [resultSelector]
     * A function to produce the value on the output Observable based on the values
     * and the indices of the source (outer) emission and the inner Observable
     * emission. The arguments passed to this function are:
     * - `outerValue`: the value that came from the source
     * - `innerValue`: the value that came from the projected Observable
     * - `outerIndex`: the "index" of the value that came from the source
     * - `innerIndex`: the "index" of the value from the projected Observable
     * @return {Observable} An Observable that emits the result of applying the
     * projection function (and the optional `resultSelector`) to each item emitted
     * by the source Observable and taking only the values from the most recently
     * projected inner Observable.
     * @method switchMap
     * @owner Observable
     */
    function switchMap(project, resultSelector) {
        return this.lift(new SwitchMapOperator(project, resultSelector));
    }
    exports.switchMap = switchMap;
    var SwitchMapOperator = function () {
        function SwitchMapOperator(project, resultSelector) {
            this.project = project;
            this.resultSelector = resultSelector;
        }
        SwitchMapOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new SwitchMapSubscriber(subscriber, this.project, this.resultSelector));
        };
        return SwitchMapOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var SwitchMapSubscriber = function (_super) {
        __extends(SwitchMapSubscriber, _super);
        function SwitchMapSubscriber(destination, project, resultSelector) {
            _super.call(this, destination);
            this.project = project;
            this.resultSelector = resultSelector;
            this.index = 0;
        }
        SwitchMapSubscriber.prototype._next = function (value) {
            var result;
            var index = this.index++;
            try {
                result = this.project(value, index);
            } catch (error) {
                this.destination.error(error);
                return;
            }
            this._innerSub(result, value, index);
        };
        SwitchMapSubscriber.prototype._innerSub = function (result, value, index) {
            var innerSubscription = this.innerSubscription;
            if (innerSubscription) {
                innerSubscription.unsubscribe();
            }
            this.add(this.innerSubscription = subscribeToResult_1.subscribeToResult(this, result, value, index));
        };
        SwitchMapSubscriber.prototype._complete = function () {
            var innerSubscription = this.innerSubscription;
            if (!innerSubscription || innerSubscription.isUnsubscribed) {
                _super.prototype._complete.call(this);
            }
        };
        SwitchMapSubscriber.prototype._unsubscribe = function () {
            this.innerSubscription = null;
        };
        SwitchMapSubscriber.prototype.notifyComplete = function (innerSub) {
            this.remove(innerSub);
            this.innerSubscription = null;
            if (this.isStopped) {
                _super.prototype._complete.call(this);
            }
        };
        SwitchMapSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            if (this.resultSelector) {
                this._tryNotifyNext(outerValue, innerValue, outerIndex, innerIndex);
            } else {
                this.destination.next(innerValue);
            }
        };
        SwitchMapSubscriber.prototype._tryNotifyNext = function (outerValue, innerValue, outerIndex, innerIndex) {
            var result;
            try {
                result = this.resultSelector(outerValue, innerValue, outerIndex, innerIndex);
            } catch (err) {
                this.destination.error(err);
                return;
            }
            this.destination.next(result);
        };
        return SwitchMapSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/switchMap.js', ['../../Observable', '../../operator/switchMap'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var switchMap_1 = $__require('../../operator/switchMap');
  Observable_1.Observable.prototype.switchMap = switchMap_1.switchMap;
  
});
System.registerDynamic('node_modules/rxjs/operator/switchMapTo.js', ['../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Projects each source value to the same Observable which is flattened multiple
     * times with {@link switch} in the output Observable.
     *
     * <span class="informal">It's like {@link switchMap}, but maps each value
     * always to the same inner Observable.</span>
     *
     * <img src="./img/switchMapTo.png" width="100%">
     *
     * Maps each source value to the given Observable `innerObservable` regardless
     * of the source value, and then flattens those resulting Observables into one
     * single Observable, which is the output Observable. The output Observables
     * emits values only from the most recently emitted instance of
     * `innerObservable`.
     *
     * @example <caption>Rerun an interval Observable on every click event</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var result = clicks.switchMapTo(Rx.Observable.interval(1000));
     * result.subscribe(x => console.log(x));
     *
     * @see {@link concatMapTo}
     * @see {@link switch}
     * @see {@link switchMap}
     * @see {@link mergeMapTo}
     *
     * @param {Observable} innerObservable An Observable to replace each value from
     * the source Observable.
     * @param {function(outerValue: T, innerValue: I, outerIndex: number, innerIndex: number): any} [resultSelector]
     * A function to produce the value on the output Observable based on the values
     * and the indices of the source (outer) emission and the inner Observable
     * emission. The arguments passed to this function are:
     * - `outerValue`: the value that came from the source
     * - `innerValue`: the value that came from the projected Observable
     * - `outerIndex`: the "index" of the value that came from the source
     * - `innerIndex`: the "index" of the value from the projected Observable
     * @return {Observable} An Observable that emits items from the given
     * `innerObservable` every time a value is emitted on the source Observable.
     * @return {Observable} An Observable that emits items from the given
     * `innerObservable` (and optionally transformed through `resultSelector`) every
     * time a value is emitted on the source Observable, and taking only the values
     * from the most recently projected inner Observable.
     * @method switchMapTo
     * @owner Observable
     */
    function switchMapTo(innerObservable, resultSelector) {
        return this.lift(new SwitchMapToOperator(innerObservable, resultSelector));
    }
    exports.switchMapTo = switchMapTo;
    var SwitchMapToOperator = function () {
        function SwitchMapToOperator(observable, resultSelector) {
            this.observable = observable;
            this.resultSelector = resultSelector;
        }
        SwitchMapToOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new SwitchMapToSubscriber(subscriber, this.observable, this.resultSelector));
        };
        return SwitchMapToOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var SwitchMapToSubscriber = function (_super) {
        __extends(SwitchMapToSubscriber, _super);
        function SwitchMapToSubscriber(destination, inner, resultSelector) {
            _super.call(this, destination);
            this.inner = inner;
            this.resultSelector = resultSelector;
            this.index = 0;
        }
        SwitchMapToSubscriber.prototype._next = function (value) {
            var innerSubscription = this.innerSubscription;
            if (innerSubscription) {
                innerSubscription.unsubscribe();
            }
            this.add(this.innerSubscription = subscribeToResult_1.subscribeToResult(this, this.inner, value, this.index++));
        };
        SwitchMapToSubscriber.prototype._complete = function () {
            var innerSubscription = this.innerSubscription;
            if (!innerSubscription || innerSubscription.isUnsubscribed) {
                _super.prototype._complete.call(this);
            }
        };
        SwitchMapToSubscriber.prototype._unsubscribe = function () {
            this.innerSubscription = null;
        };
        SwitchMapToSubscriber.prototype.notifyComplete = function (innerSub) {
            this.remove(innerSub);
            this.innerSubscription = null;
            if (this.isStopped) {
                _super.prototype._complete.call(this);
            }
        };
        SwitchMapToSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            var _a = this,
                resultSelector = _a.resultSelector,
                destination = _a.destination;
            if (resultSelector) {
                this.tryResultSelector(outerValue, innerValue, outerIndex, innerIndex);
            } else {
                destination.next(innerValue);
            }
        };
        SwitchMapToSubscriber.prototype.tryResultSelector = function (outerValue, innerValue, outerIndex, innerIndex) {
            var _a = this,
                resultSelector = _a.resultSelector,
                destination = _a.destination;
            var result;
            try {
                result = resultSelector(outerValue, innerValue, outerIndex, innerIndex);
            } catch (err) {
                destination.error(err);
                return;
            }
            destination.next(result);
        };
        return SwitchMapToSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/switchMapTo.js', ['../../Observable', '../../operator/switchMapTo'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var switchMapTo_1 = $__require('../../operator/switchMapTo');
  Observable_1.Observable.prototype.switchMapTo = switchMapTo_1.switchMapTo;
  
});
System.registerDynamic('node_modules/rxjs/operator/take.js', ['../Subscriber', '../util/ArgumentOutOfRangeError', '../observable/EmptyObservable'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var ArgumentOutOfRangeError_1 = $__require('../util/ArgumentOutOfRangeError');
    var EmptyObservable_1 = $__require('../observable/EmptyObservable');
    /**
     * @throws {ArgumentOutOfRangeError} When using `take(i)`, it delivers an
     * ArgumentOutOrRangeError to the Observer's `error` callback if `i < 0`.
     * @param total
     * @return {any}
     * @method take
     * @owner Observable
     */
    function take(total) {
        if (total === 0) {
            return new EmptyObservable_1.EmptyObservable();
        } else {
            return this.lift(new TakeOperator(total));
        }
    }
    exports.take = take;
    var TakeOperator = function () {
        function TakeOperator(total) {
            this.total = total;
            if (this.total < 0) {
                throw new ArgumentOutOfRangeError_1.ArgumentOutOfRangeError();
            }
        }
        TakeOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new TakeSubscriber(subscriber, this.total));
        };
        return TakeOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var TakeSubscriber = function (_super) {
        __extends(TakeSubscriber, _super);
        function TakeSubscriber(destination, total) {
            _super.call(this, destination);
            this.total = total;
            this.count = 0;
        }
        TakeSubscriber.prototype._next = function (value) {
            var total = this.total;
            if (++this.count <= total) {
                this.destination.next(value);
                if (this.count === total) {
                    this.destination.complete();
                    this.unsubscribe();
                }
            }
        };
        return TakeSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/take.js', ['../../Observable', '../../operator/take'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var take_1 = $__require('../../operator/take');
  Observable_1.Observable.prototype.take = take_1.take;
  
});
System.registerDynamic('node_modules/rxjs/operator/takeLast.js', ['../Subscriber', '../util/ArgumentOutOfRangeError', '../observable/EmptyObservable'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var ArgumentOutOfRangeError_1 = $__require('../util/ArgumentOutOfRangeError');
    var EmptyObservable_1 = $__require('../observable/EmptyObservable');
    /**
     * @throws {ArgumentOutOfRangeError} When using `takeLast(i)`, it delivers an
     * ArgumentOutOrRangeError to the Observer's `error` callback if `i < 0`.
     * @param total
     * @return {any}
     * @method takeLast
     * @owner Observable
     */
    function takeLast(total) {
        if (total === 0) {
            return new EmptyObservable_1.EmptyObservable();
        } else {
            return this.lift(new TakeLastOperator(total));
        }
    }
    exports.takeLast = takeLast;
    var TakeLastOperator = function () {
        function TakeLastOperator(total) {
            this.total = total;
            if (this.total < 0) {
                throw new ArgumentOutOfRangeError_1.ArgumentOutOfRangeError();
            }
        }
        TakeLastOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new TakeLastSubscriber(subscriber, this.total));
        };
        return TakeLastOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var TakeLastSubscriber = function (_super) {
        __extends(TakeLastSubscriber, _super);
        function TakeLastSubscriber(destination, total) {
            _super.call(this, destination);
            this.total = total;
            this.ring = new Array();
            this.count = 0;
        }
        TakeLastSubscriber.prototype._next = function (value) {
            var ring = this.ring;
            var total = this.total;
            var count = this.count++;
            if (ring.length < total) {
                ring.push(value);
            } else {
                var index = count % total;
                ring[index] = value;
            }
        };
        TakeLastSubscriber.prototype._complete = function () {
            var destination = this.destination;
            var count = this.count;
            if (count > 0) {
                var total = this.count >= this.total ? this.total : this.count;
                var ring = this.ring;
                for (var i = 0; i < total; i++) {
                    var idx = count++ % total;
                    destination.next(ring[idx]);
                }
            }
            destination.complete();
        };
        return TakeLastSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/takeLast.js', ['../../Observable', '../../operator/takeLast'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var takeLast_1 = $__require('../../operator/takeLast');
  Observable_1.Observable.prototype.takeLast = takeLast_1.takeLast;
  
});
System.registerDynamic('node_modules/rxjs/operator/takeUntil.js', ['../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * @param notifier
     * @return {Observable<R>|WebSocketSubject<T>|Observable<T>}
     * @method takeUntil
     * @owner Observable
     */
    function takeUntil(notifier) {
        return this.lift(new TakeUntilOperator(notifier));
    }
    exports.takeUntil = takeUntil;
    var TakeUntilOperator = function () {
        function TakeUntilOperator(notifier) {
            this.notifier = notifier;
        }
        TakeUntilOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new TakeUntilSubscriber(subscriber, this.notifier));
        };
        return TakeUntilOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var TakeUntilSubscriber = function (_super) {
        __extends(TakeUntilSubscriber, _super);
        function TakeUntilSubscriber(destination, notifier) {
            _super.call(this, destination);
            this.notifier = notifier;
            this.add(subscribeToResult_1.subscribeToResult(this, notifier));
        }
        TakeUntilSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            this.complete();
        };
        TakeUntilSubscriber.prototype.notifyComplete = function () {
            // noop
        };
        return TakeUntilSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/takeUntil.js', ['../../Observable', '../../operator/takeUntil'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var takeUntil_1 = $__require('../../operator/takeUntil');
  Observable_1.Observable.prototype.takeUntil = takeUntil_1.takeUntil;
  
});
System.registerDynamic("node_modules/rxjs/operator/takeWhile.js", ["../Subscriber"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * @param predicate
     * @return {Observable<R>|WebSocketSubject<T>|Observable<T>}
     * @method takeWhile
     * @owner Observable
     */
    function takeWhile(predicate) {
        return this.lift(new TakeWhileOperator(predicate));
    }
    exports.takeWhile = takeWhile;
    var TakeWhileOperator = function () {
        function TakeWhileOperator(predicate) {
            this.predicate = predicate;
        }
        TakeWhileOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new TakeWhileSubscriber(subscriber, this.predicate));
        };
        return TakeWhileOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var TakeWhileSubscriber = function (_super) {
        __extends(TakeWhileSubscriber, _super);
        function TakeWhileSubscriber(destination, predicate) {
            _super.call(this, destination);
            this.predicate = predicate;
            this.index = 0;
        }
        TakeWhileSubscriber.prototype._next = function (value) {
            var destination = this.destination;
            var result;
            try {
                result = this.predicate(value, this.index++);
            } catch (err) {
                destination.error(err);
                return;
            }
            this.nextOrComplete(value, result);
        };
        TakeWhileSubscriber.prototype.nextOrComplete = function (value, predicateResult) {
            var destination = this.destination;
            if (Boolean(predicateResult)) {
                destination.next(value);
            } else {
                destination.complete();
            }
        };
        return TakeWhileSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/takeWhile.js', ['../../Observable', '../../operator/takeWhile'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var takeWhile_1 = $__require('../../operator/takeWhile');
  Observable_1.Observable.prototype.takeWhile = takeWhile_1.takeWhile;
  
});
System.registerDynamic('node_modules/rxjs/operator/throttle.js', ['../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * @param durationSelector
     * @return {Observable<R>|WebSocketSubject<T>|Observable<T>}
     * @method throttle
     * @owner Observable
     */
    function throttle(durationSelector) {
        return this.lift(new ThrottleOperator(durationSelector));
    }
    exports.throttle = throttle;
    var ThrottleOperator = function () {
        function ThrottleOperator(durationSelector) {
            this.durationSelector = durationSelector;
        }
        ThrottleOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new ThrottleSubscriber(subscriber, this.durationSelector));
        };
        return ThrottleOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var ThrottleSubscriber = function (_super) {
        __extends(ThrottleSubscriber, _super);
        function ThrottleSubscriber(destination, durationSelector) {
            _super.call(this, destination);
            this.destination = destination;
            this.durationSelector = durationSelector;
        }
        ThrottleSubscriber.prototype._next = function (value) {
            if (!this.throttled) {
                this.tryDurationSelector(value);
            }
        };
        ThrottleSubscriber.prototype.tryDurationSelector = function (value) {
            var duration = null;
            try {
                duration = this.durationSelector(value);
            } catch (err) {
                this.destination.error(err);
                return;
            }
            this.emitAndThrottle(value, duration);
        };
        ThrottleSubscriber.prototype.emitAndThrottle = function (value, duration) {
            this.add(this.throttled = subscribeToResult_1.subscribeToResult(this, duration));
            this.destination.next(value);
        };
        ThrottleSubscriber.prototype._unsubscribe = function () {
            var throttled = this.throttled;
            if (throttled) {
                this.remove(throttled);
                this.throttled = null;
                throttled.unsubscribe();
            }
        };
        ThrottleSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            this._unsubscribe();
        };
        ThrottleSubscriber.prototype.notifyComplete = function () {
            this._unsubscribe();
        };
        return ThrottleSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/throttle.js', ['../../Observable', '../../operator/throttle'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var throttle_1 = $__require('../../operator/throttle');
  Observable_1.Observable.prototype.throttle = throttle_1.throttle;
  
});
System.registerDynamic('node_modules/rxjs/operator/throttleTime.js', ['../Subscriber', '../scheduler/async'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var async_1 = $__require('../scheduler/async');
    /**
     * @param delay
     * @param scheduler
     * @return {Observable<R>|WebSocketSubject<T>|Observable<T>}
     * @method throttleTime
     * @owner Observable
     */
    function throttleTime(delay, scheduler) {
        if (scheduler === void 0) {
            scheduler = async_1.async;
        }
        return this.lift(new ThrottleTimeOperator(delay, scheduler));
    }
    exports.throttleTime = throttleTime;
    var ThrottleTimeOperator = function () {
        function ThrottleTimeOperator(delay, scheduler) {
            this.delay = delay;
            this.scheduler = scheduler;
        }
        ThrottleTimeOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new ThrottleTimeSubscriber(subscriber, this.delay, this.scheduler));
        };
        return ThrottleTimeOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var ThrottleTimeSubscriber = function (_super) {
        __extends(ThrottleTimeSubscriber, _super);
        function ThrottleTimeSubscriber(destination, delay, scheduler) {
            _super.call(this, destination);
            this.delay = delay;
            this.scheduler = scheduler;
        }
        ThrottleTimeSubscriber.prototype._next = function (value) {
            if (!this.throttled) {
                this.add(this.throttled = this.scheduler.schedule(dispatchNext, this.delay, { subscriber: this }));
                this.destination.next(value);
            }
        };
        ThrottleTimeSubscriber.prototype.clearThrottle = function () {
            var throttled = this.throttled;
            if (throttled) {
                throttled.unsubscribe();
                this.remove(throttled);
                this.throttled = null;
            }
        };
        return ThrottleTimeSubscriber;
    }(Subscriber_1.Subscriber);
    function dispatchNext(arg) {
        var subscriber = arg.subscriber;
        subscriber.clearThrottle();
    }
    
});
System.registerDynamic('node_modules/rxjs/add/operator/throttleTime.js', ['../../Observable', '../../operator/throttleTime'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var throttleTime_1 = $__require('../../operator/throttleTime');
  Observable_1.Observable.prototype.throttleTime = throttleTime_1.throttleTime;
  
});
System.registerDynamic('node_modules/rxjs/operator/timeout.js', ['../scheduler/async', '../util/isDate', '../Subscriber'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var async_1 = $__require('../scheduler/async');
    var isDate_1 = $__require('../util/isDate');
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * @param due
     * @param errorToSend
     * @param scheduler
     * @return {Observable<R>|WebSocketSubject<T>|Observable<T>}
     * @method timeout
     * @owner Observable
     */
    function timeout(due, errorToSend, scheduler) {
        if (errorToSend === void 0) {
            errorToSend = null;
        }
        if (scheduler === void 0) {
            scheduler = async_1.async;
        }
        var absoluteTimeout = isDate_1.isDate(due);
        var waitFor = absoluteTimeout ? +due - scheduler.now() : Math.abs(due);
        return this.lift(new TimeoutOperator(waitFor, absoluteTimeout, errorToSend, scheduler));
    }
    exports.timeout = timeout;
    var TimeoutOperator = function () {
        function TimeoutOperator(waitFor, absoluteTimeout, errorToSend, scheduler) {
            this.waitFor = waitFor;
            this.absoluteTimeout = absoluteTimeout;
            this.errorToSend = errorToSend;
            this.scheduler = scheduler;
        }
        TimeoutOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new TimeoutSubscriber(subscriber, this.absoluteTimeout, this.waitFor, this.errorToSend, this.scheduler));
        };
        return TimeoutOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var TimeoutSubscriber = function (_super) {
        __extends(TimeoutSubscriber, _super);
        function TimeoutSubscriber(destination, absoluteTimeout, waitFor, errorToSend, scheduler) {
            _super.call(this, destination);
            this.absoluteTimeout = absoluteTimeout;
            this.waitFor = waitFor;
            this.errorToSend = errorToSend;
            this.scheduler = scheduler;
            this.index = 0;
            this._previousIndex = 0;
            this._hasCompleted = false;
            this.scheduleTimeout();
        }
        Object.defineProperty(TimeoutSubscriber.prototype, "previousIndex", {
            get: function () {
                return this._previousIndex;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TimeoutSubscriber.prototype, "hasCompleted", {
            get: function () {
                return this._hasCompleted;
            },
            enumerable: true,
            configurable: true
        });
        TimeoutSubscriber.dispatchTimeout = function (state) {
            var source = state.subscriber;
            var currentIndex = state.index;
            if (!source.hasCompleted && source.previousIndex === currentIndex) {
                source.notifyTimeout();
            }
        };
        TimeoutSubscriber.prototype.scheduleTimeout = function () {
            var currentIndex = this.index;
            this.scheduler.schedule(TimeoutSubscriber.dispatchTimeout, this.waitFor, { subscriber: this, index: currentIndex });
            this.index++;
            this._previousIndex = currentIndex;
        };
        TimeoutSubscriber.prototype._next = function (value) {
            this.destination.next(value);
            if (!this.absoluteTimeout) {
                this.scheduleTimeout();
            }
        };
        TimeoutSubscriber.prototype._error = function (err) {
            this.destination.error(err);
            this._hasCompleted = true;
        };
        TimeoutSubscriber.prototype._complete = function () {
            this.destination.complete();
            this._hasCompleted = true;
        };
        TimeoutSubscriber.prototype.notifyTimeout = function () {
            this.error(this.errorToSend || new Error('timeout'));
        };
        return TimeoutSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/timeout.js', ['../../Observable', '../../operator/timeout'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var timeout_1 = $__require('../../operator/timeout');
  Observable_1.Observable.prototype.timeout = timeout_1.timeout;
  
});
System.registerDynamic("node_modules/rxjs/util/isDate.js", [], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    function isDate(value) {
        return value instanceof Date && !isNaN(+value);
    }
    exports.isDate = isDate;
    
});
System.registerDynamic('node_modules/rxjs/operator/timeoutWith.js', ['../scheduler/async', '../util/isDate', '../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var async_1 = $__require('../scheduler/async');
    var isDate_1 = $__require('../util/isDate');
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * @param due
     * @param withObservable
     * @param scheduler
     * @return {Observable<R>|WebSocketSubject<T>|Observable<T>}
     * @method timeoutWith
     * @owner Observable
     */
    function timeoutWith(due, withObservable, scheduler) {
        if (scheduler === void 0) {
            scheduler = async_1.async;
        }
        var absoluteTimeout = isDate_1.isDate(due);
        var waitFor = absoluteTimeout ? +due - scheduler.now() : Math.abs(due);
        return this.lift(new TimeoutWithOperator(waitFor, absoluteTimeout, withObservable, scheduler));
    }
    exports.timeoutWith = timeoutWith;
    var TimeoutWithOperator = function () {
        function TimeoutWithOperator(waitFor, absoluteTimeout, withObservable, scheduler) {
            this.waitFor = waitFor;
            this.absoluteTimeout = absoluteTimeout;
            this.withObservable = withObservable;
            this.scheduler = scheduler;
        }
        TimeoutWithOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new TimeoutWithSubscriber(subscriber, this.absoluteTimeout, this.waitFor, this.withObservable, this.scheduler));
        };
        return TimeoutWithOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var TimeoutWithSubscriber = function (_super) {
        __extends(TimeoutWithSubscriber, _super);
        function TimeoutWithSubscriber(destination, absoluteTimeout, waitFor, withObservable, scheduler) {
            _super.call(this);
            this.destination = destination;
            this.absoluteTimeout = absoluteTimeout;
            this.waitFor = waitFor;
            this.withObservable = withObservable;
            this.scheduler = scheduler;
            this.timeoutSubscription = undefined;
            this.index = 0;
            this._previousIndex = 0;
            this._hasCompleted = false;
            destination.add(this);
            this.scheduleTimeout();
        }
        Object.defineProperty(TimeoutWithSubscriber.prototype, "previousIndex", {
            get: function () {
                return this._previousIndex;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(TimeoutWithSubscriber.prototype, "hasCompleted", {
            get: function () {
                return this._hasCompleted;
            },
            enumerable: true,
            configurable: true
        });
        TimeoutWithSubscriber.dispatchTimeout = function (state) {
            var source = state.subscriber;
            var currentIndex = state.index;
            if (!source.hasCompleted && source.previousIndex === currentIndex) {
                source.handleTimeout();
            }
        };
        TimeoutWithSubscriber.prototype.scheduleTimeout = function () {
            var currentIndex = this.index;
            var timeoutState = { subscriber: this, index: currentIndex };
            this.scheduler.schedule(TimeoutWithSubscriber.dispatchTimeout, this.waitFor, timeoutState);
            this.index++;
            this._previousIndex = currentIndex;
        };
        TimeoutWithSubscriber.prototype._next = function (value) {
            this.destination.next(value);
            if (!this.absoluteTimeout) {
                this.scheduleTimeout();
            }
        };
        TimeoutWithSubscriber.prototype._error = function (err) {
            this.destination.error(err);
            this._hasCompleted = true;
        };
        TimeoutWithSubscriber.prototype._complete = function () {
            this.destination.complete();
            this._hasCompleted = true;
        };
        TimeoutWithSubscriber.prototype.handleTimeout = function () {
            if (!this.isUnsubscribed) {
                var withObservable = this.withObservable;
                this.unsubscribe();
                this.destination.add(this.timeoutSubscription = subscribeToResult_1.subscribeToResult(this, withObservable));
            }
        };
        return TimeoutWithSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/timeoutWith.js', ['../../Observable', '../../operator/timeoutWith'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var timeoutWith_1 = $__require('../../operator/timeoutWith');
  Observable_1.Observable.prototype.timeoutWith = timeoutWith_1.timeoutWith;
  
});
System.registerDynamic("node_modules/rxjs/operator/toArray.js", ["../Subscriber"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    /**
     * @return {Observable<any[]>|WebSocketSubject<T>|Observable<T>}
     * @method toArray
     * @owner Observable
     */
    function toArray() {
        return this.lift(new ToArrayOperator());
    }
    exports.toArray = toArray;
    var ToArrayOperator = function () {
        function ToArrayOperator() {}
        ToArrayOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new ToArraySubscriber(subscriber));
        };
        return ToArrayOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var ToArraySubscriber = function (_super) {
        __extends(ToArraySubscriber, _super);
        function ToArraySubscriber(destination) {
            _super.call(this, destination);
            this.array = [];
        }
        ToArraySubscriber.prototype._next = function (x) {
            this.array.push(x);
        };
        ToArraySubscriber.prototype._complete = function () {
            this.destination.next(this.array);
            this.destination.complete();
        };
        return ToArraySubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/toArray.js', ['../../Observable', '../../operator/toArray'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var toArray_1 = $__require('../../operator/toArray');
  Observable_1.Observable.prototype.toArray = toArray_1.toArray;
  
});
System.registerDynamic('node_modules/rxjs/operator/toPromise.js', ['../util/root'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var root_1 = $__require('../util/root');
    /**
     * @param PromiseCtor
     * @return {Promise<T>}
     * @method toPromise
     * @owner Observable
     */
    function toPromise(PromiseCtor) {
        var _this = this;
        if (!PromiseCtor) {
            if (root_1.root.Rx && root_1.root.Rx.config && root_1.root.Rx.config.Promise) {
                PromiseCtor = root_1.root.Rx.config.Promise;
            } else if (root_1.root.Promise) {
                PromiseCtor = root_1.root.Promise;
            }
        }
        if (!PromiseCtor) {
            throw new Error('no Promise impl found');
        }
        return new PromiseCtor(function (resolve, reject) {
            var value;
            _this.subscribe(function (x) {
                return value = x;
            }, function (err) {
                return reject(err);
            }, function () {
                return resolve(value);
            });
        });
    }
    exports.toPromise = toPromise;
    
});
System.registerDynamic('node_modules/rxjs/add/operator/toPromise.js', ['../../Observable', '../../operator/toPromise'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var toPromise_1 = $__require('../../operator/toPromise');
  Observable_1.Observable.prototype.toPromise = toPromise_1.toPromise;
  
});
System.registerDynamic('node_modules/rxjs/operator/window.js', ['../Subject', '../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subject_1 = $__require('../Subject');
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Branch out the source Observable values as a nested Observable whenever
     * `windowBoundaries` emits.
     *
     * <span class="informal">It's like {@link buffer}, but emits a nested Observable
     * instead of an array.</span>
     *
     * <img src="./img/window.png" width="100%">
     *
     * Returns an Observable that emits windows of items it collects from the source
     * Observable. The output Observable emits connected, non-overlapping
     * windows. It emits the current window and opens a new one whenever the
     * Observable `windowBoundaries` emits an item. Because each window is an
     * Observable, the output is a higher-order Observable.
     *
     * @example <caption>In every window of 1 second each, emit at most 2 click events</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var interval = Rx.Observable.interval(1000);
     * var result = clicks.window(interval)
     *   .map(win => win.take(2)) // each window has at most 2 emissions
     *   .mergeAll(); // flatten the Observable-of-Observables
     * result.subscribe(x => console.log(x));
     *
     * @see {@link windowCount}
     * @see {@link windowTime}
     * @see {@link windowToggle}
     * @see {@link windowWhen}
     * @see {@link buffer}
     *
     * @param {Observable<any>} windowBoundaries An Observable that completes the
     * previous window and starts a new window.
     * @return {Observable<Observable<T>>} An Observable of windows, which are
     * Observables emitting values of the source Observable.
     * @method window
     * @owner Observable
     */
    function window(windowBoundaries) {
        return this.lift(new WindowOperator(windowBoundaries));
    }
    exports.window = window;
    var WindowOperator = function () {
        function WindowOperator(windowBoundaries) {
            this.windowBoundaries = windowBoundaries;
        }
        WindowOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new WindowSubscriber(subscriber, this.windowBoundaries));
        };
        return WindowOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var WindowSubscriber = function (_super) {
        __extends(WindowSubscriber, _super);
        function WindowSubscriber(destination, windowBoundaries) {
            _super.call(this, destination);
            this.destination = destination;
            this.windowBoundaries = windowBoundaries;
            this.add(subscribeToResult_1.subscribeToResult(this, windowBoundaries));
            this.openWindow();
        }
        WindowSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            this.openWindow();
        };
        WindowSubscriber.prototype.notifyError = function (error, innerSub) {
            this._error(error);
        };
        WindowSubscriber.prototype.notifyComplete = function (innerSub) {
            this._complete();
        };
        WindowSubscriber.prototype._next = function (value) {
            this.window.next(value);
        };
        WindowSubscriber.prototype._error = function (err) {
            this.window.error(err);
            this.destination.error(err);
        };
        WindowSubscriber.prototype._complete = function () {
            this.window.complete();
            this.destination.complete();
        };
        WindowSubscriber.prototype.openWindow = function () {
            var prevWindow = this.window;
            if (prevWindow) {
                prevWindow.complete();
            }
            var destination = this.destination;
            var newWindow = this.window = new Subject_1.Subject();
            destination.add(newWindow);
            destination.next(newWindow);
        };
        return WindowSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/window.js', ['../../Observable', '../../operator/window'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var window_1 = $__require('../../operator/window');
  Observable_1.Observable.prototype.window = window_1.window;
  
});
System.registerDynamic('node_modules/rxjs/operator/windowCount.js', ['../Subscriber', '../Subject'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var Subject_1 = $__require('../Subject');
    /**
     * Branch out the source Observable values as a nested Observable with each
     * nested Observable emitting at most `windowSize` values.
     *
     * <span class="informal">It's like {@link bufferCount}, but emits a nested
     * Observable instead of an array.</span>
     *
     * <img src="./img/windowCount.png" width="100%">
     *
     * Returns an Observable that emits windows of items it collects from the source
     * Observable. The output Observable emits windows every `startWindowEvery`
     * items, each containing no more than `windowSize` items. When the source
     * Observable completes or encounters an error, the output Observable emits
     * the current window and propagates the notification from the source
     * Observable. If `startWindowEvery` is not provided, then new windows are
     * started immediately at the start of the source and when each window completes
     * with size `windowSize`.
     *
     * @example <caption>Ignore every 3rd click event, starting from the first one</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var result = clicks.windowCount(3)
     *   .map(win => win.skip(1)) // skip first of every 3 clicks
     *   .mergeAll(); // flatten the Observable-of-Observables
     * result.subscribe(x => console.log(x));
     *
     * @example <caption>Ignore every 3rd click event, starting from the third one</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var result = clicks.windowCount(2, 3)
     *   .mergeAll(); // flatten the Observable-of-Observables
     * result.subscribe(x => console.log(x));
     *
     * @see {@link window}
     * @see {@link windowTime}
     * @see {@link windowToggle}
     * @see {@link windowWhen}
     * @see {@link bufferCount}
     *
     * @param {number} windowSize The maximum number of values emitted by each
     * window.
     * @param {number} [startWindowEvery] Interval at which to start a new window.
     * For example if `startWindowEvery` is `2`, then a new window will be started
     * on every other value from the source. A new window is started at the
     * beginning of the source by default.
     * @return {Observable<Observable<T>>} An Observable of windows, which in turn
     * are Observable of values.
     * @method windowCount
     * @owner Observable
     */
    function windowCount(windowSize, startWindowEvery) {
        if (startWindowEvery === void 0) {
            startWindowEvery = 0;
        }
        return this.lift(new WindowCountOperator(windowSize, startWindowEvery));
    }
    exports.windowCount = windowCount;
    var WindowCountOperator = function () {
        function WindowCountOperator(windowSize, startWindowEvery) {
            this.windowSize = windowSize;
            this.startWindowEvery = startWindowEvery;
        }
        WindowCountOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new WindowCountSubscriber(subscriber, this.windowSize, this.startWindowEvery));
        };
        return WindowCountOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var WindowCountSubscriber = function (_super) {
        __extends(WindowCountSubscriber, _super);
        function WindowCountSubscriber(destination, windowSize, startWindowEvery) {
            _super.call(this, destination);
            this.destination = destination;
            this.windowSize = windowSize;
            this.startWindowEvery = startWindowEvery;
            this.windows = [new Subject_1.Subject()];
            this.count = 0;
            var firstWindow = this.windows[0];
            destination.add(firstWindow);
            destination.next(firstWindow);
        }
        WindowCountSubscriber.prototype._next = function (value) {
            var startWindowEvery = this.startWindowEvery > 0 ? this.startWindowEvery : this.windowSize;
            var destination = this.destination;
            var windowSize = this.windowSize;
            var windows = this.windows;
            var len = windows.length;
            for (var i = 0; i < len; i++) {
                windows[i].next(value);
            }
            var c = this.count - windowSize + 1;
            if (c >= 0 && c % startWindowEvery === 0) {
                windows.shift().complete();
            }
            if (++this.count % startWindowEvery === 0) {
                var window_1 = new Subject_1.Subject();
                windows.push(window_1);
                destination.add(window_1);
                destination.next(window_1);
            }
        };
        WindowCountSubscriber.prototype._error = function (err) {
            var windows = this.windows;
            while (windows.length > 0) {
                windows.shift().error(err);
            }
            this.destination.error(err);
        };
        WindowCountSubscriber.prototype._complete = function () {
            var windows = this.windows;
            while (windows.length > 0) {
                windows.shift().complete();
            }
            this.destination.complete();
        };
        return WindowCountSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/windowCount.js', ['../../Observable', '../../operator/windowCount'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var windowCount_1 = $__require('../../operator/windowCount');
  Observable_1.Observable.prototype.windowCount = windowCount_1.windowCount;
  
});
System.registerDynamic('node_modules/rxjs/operator/windowTime.js', ['../Subscriber', '../Subject', '../scheduler/async'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var Subject_1 = $__require('../Subject');
    var async_1 = $__require('../scheduler/async');
    /**
     * Branch out the source Observable values as a nested Observable periodically
     * in time.
     *
     * <span class="informal">It's like {@link bufferTime}, but emits a nested
     * Observable instead of an array.</span>
     *
     * <img src="./img/windowTime.png" width="100%">
     *
     * Returns an Observable that emits windows of items it collects from the source
     * Observable. The output Observable starts a new window periodically, as
     * determined by the `windowCreationInterval` argument. It emits each window
     * after a fixed timespan, specified by the `windowTimeSpan` argument. When the
     * source Observable completes or encounters an error, the output Observable
     * emits the current window and propagates the notification from the source
     * Observable. If `windowCreationInterval` is not provided, the output
     * Observable starts a new window when the previous window of duration
     * `windowTimeSpan` completes.
     *
     * @example <caption>In every window of 1 second each, emit at most 2 click events</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var result = clicks.windowTime(1000)
     *   .map(win => win.take(2)) // each window has at most 2 emissions
     *   .mergeAll(); // flatten the Observable-of-Observables
     * result.subscribe(x => console.log(x));
     *
     * @example <caption>Every 5 seconds start a window 1 second long, and emit at most 2 click events per window</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var result = clicks.windowTime(1000, 5000)
     *   .map(win => win.take(2)) // each window has at most 2 emissions
     *   .mergeAll(); // flatten the Observable-of-Observables
     * result.subscribe(x => console.log(x));
     *
     * @see {@link window}
     * @see {@link windowCount}
     * @see {@link windowToggle}
     * @see {@link windowWhen}
     * @see {@link bufferTime}
     *
     * @param {number} windowTimeSpan The amount of time to fill each window.
     * @param {number} [windowCreationInterval] The interval at which to start new
     * windows.
     * @param {Scheduler} [scheduler=async] The scheduler on which to schedule the
     * intervals that determine window boundaries.
     * @return {Observable<Observable<T>>} An observable of windows, which in turn
     * are Observables.
     * @method windowTime
     * @owner Observable
     */
    function windowTime(windowTimeSpan, windowCreationInterval, scheduler) {
        if (windowCreationInterval === void 0) {
            windowCreationInterval = null;
        }
        if (scheduler === void 0) {
            scheduler = async_1.async;
        }
        return this.lift(new WindowTimeOperator(windowTimeSpan, windowCreationInterval, scheduler));
    }
    exports.windowTime = windowTime;
    var WindowTimeOperator = function () {
        function WindowTimeOperator(windowTimeSpan, windowCreationInterval, scheduler) {
            this.windowTimeSpan = windowTimeSpan;
            this.windowCreationInterval = windowCreationInterval;
            this.scheduler = scheduler;
        }
        WindowTimeOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new WindowTimeSubscriber(subscriber, this.windowTimeSpan, this.windowCreationInterval, this.scheduler));
        };
        return WindowTimeOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var WindowTimeSubscriber = function (_super) {
        __extends(WindowTimeSubscriber, _super);
        function WindowTimeSubscriber(destination, windowTimeSpan, windowCreationInterval, scheduler) {
            _super.call(this, destination);
            this.destination = destination;
            this.windowTimeSpan = windowTimeSpan;
            this.windowCreationInterval = windowCreationInterval;
            this.scheduler = scheduler;
            this.windows = [];
            if (windowCreationInterval !== null && windowCreationInterval >= 0) {
                var window_1 = this.openWindow();
                var closeState = { subscriber: this, window: window_1, context: null };
                var creationState = { windowTimeSpan: windowTimeSpan, windowCreationInterval: windowCreationInterval, subscriber: this, scheduler: scheduler };
                this.add(scheduler.schedule(dispatchWindowClose, windowTimeSpan, closeState));
                this.add(scheduler.schedule(dispatchWindowCreation, windowCreationInterval, creationState));
            } else {
                var window_2 = this.openWindow();
                var timeSpanOnlyState = { subscriber: this, window: window_2, windowTimeSpan: windowTimeSpan };
                this.add(scheduler.schedule(dispatchWindowTimeSpanOnly, windowTimeSpan, timeSpanOnlyState));
            }
        }
        WindowTimeSubscriber.prototype._next = function (value) {
            var windows = this.windows;
            var len = windows.length;
            for (var i = 0; i < len; i++) {
                var window_3 = windows[i];
                if (!window_3.isUnsubscribed) {
                    window_3.next(value);
                }
            }
        };
        WindowTimeSubscriber.prototype._error = function (err) {
            var windows = this.windows;
            while (windows.length > 0) {
                windows.shift().error(err);
            }
            this.destination.error(err);
        };
        WindowTimeSubscriber.prototype._complete = function () {
            var windows = this.windows;
            while (windows.length > 0) {
                var window_4 = windows.shift();
                if (!window_4.isUnsubscribed) {
                    window_4.complete();
                }
            }
            this.destination.complete();
        };
        WindowTimeSubscriber.prototype.openWindow = function () {
            var window = new Subject_1.Subject();
            this.windows.push(window);
            var destination = this.destination;
            destination.add(window);
            destination.next(window);
            return window;
        };
        WindowTimeSubscriber.prototype.closeWindow = function (window) {
            window.complete();
            var windows = this.windows;
            windows.splice(windows.indexOf(window), 1);
        };
        return WindowTimeSubscriber;
    }(Subscriber_1.Subscriber);
    function dispatchWindowTimeSpanOnly(state) {
        var subscriber = state.subscriber,
            windowTimeSpan = state.windowTimeSpan,
            window = state.window;
        if (window) {
            window.complete();
        }
        state.window = subscriber.openWindow();
        this.schedule(state, windowTimeSpan);
    }
    function dispatchWindowCreation(state) {
        var windowTimeSpan = state.windowTimeSpan,
            subscriber = state.subscriber,
            scheduler = state.scheduler,
            windowCreationInterval = state.windowCreationInterval;
        var window = subscriber.openWindow();
        var action = this;
        var context = { action: action, subscription: null };
        var timeSpanState = { subscriber: subscriber, window: window, context: context };
        context.subscription = scheduler.schedule(dispatchWindowClose, windowTimeSpan, timeSpanState);
        action.add(context.subscription);
        action.schedule(state, windowCreationInterval);
    }
    function dispatchWindowClose(arg) {
        var subscriber = arg.subscriber,
            window = arg.window,
            context = arg.context;
        if (context && context.action && context.subscription) {
            context.action.remove(context.subscription);
        }
        subscriber.closeWindow(window);
    }
    
});
System.registerDynamic('node_modules/rxjs/add/operator/windowTime.js', ['../../Observable', '../../operator/windowTime'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var windowTime_1 = $__require('../../operator/windowTime');
  Observable_1.Observable.prototype.windowTime = windowTime_1.windowTime;
  
});
System.registerDynamic('node_modules/rxjs/operator/windowToggle.js', ['../Subject', '../Subscription', '../util/tryCatch', '../util/errorObject', '../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subject_1 = $__require('../Subject');
    var Subscription_1 = $__require('../Subscription');
    var tryCatch_1 = $__require('../util/tryCatch');
    var errorObject_1 = $__require('../util/errorObject');
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Branch out the source Observable values as a nested Observable starting from
     * an emission from `openings` and ending when the output of `closingSelector`
     * emits.
     *
     * <span class="informal">It's like {@link bufferToggle}, but emits a nested
     * Observable instead of an array.</span>
     *
     * <img src="./img/windowToggle.png" width="100%">
     *
     * Returns an Observable that emits windows of items it collects from the source
     * Observable. The output Observable emits windows that contain those items
     * emitted by the source Observable between the time when the `openings`
     * Observable emits an item and when the Observable returned by
     * `closingSelector` emits an item.
     *
     * @example <caption>Every other second, emit the click events from the next 500ms</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var openings = Rx.Observable.interval(1000);
     * var result = clicks.windowToggle(openings, i =>
     *   i % 2 ? Rx.Observable.interval(500) : Rx.Observable.empty()
     * ).mergeAll();
     * result.subscribe(x => console.log(x));
     *
     * @see {@link window}
     * @see {@link windowCount}
     * @see {@link windowTime}
     * @see {@link windowWhen}
     * @see {@link bufferToggle}
     *
     * @param {Observable<O>} openings An observable of notifications to start new
     * windows.
     * @param {function(value: O): Observable} closingSelector A function that takes
     * the value emitted by the `openings` observable and returns an Observable,
     * which, when it emits (either `next` or `complete`), signals that the
     * associated window should complete.
     * @return {Observable<Observable<T>>} An observable of windows, which in turn
     * are Observables.
     * @method windowToggle
     * @owner Observable
     */
    function windowToggle(openings, closingSelector) {
        return this.lift(new WindowToggleOperator(openings, closingSelector));
    }
    exports.windowToggle = windowToggle;
    var WindowToggleOperator = function () {
        function WindowToggleOperator(openings, closingSelector) {
            this.openings = openings;
            this.closingSelector = closingSelector;
        }
        WindowToggleOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new WindowToggleSubscriber(subscriber, this.openings, this.closingSelector));
        };
        return WindowToggleOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var WindowToggleSubscriber = function (_super) {
        __extends(WindowToggleSubscriber, _super);
        function WindowToggleSubscriber(destination, openings, closingSelector) {
            _super.call(this, destination);
            this.openings = openings;
            this.closingSelector = closingSelector;
            this.contexts = [];
            this.add(this.openSubscription = subscribeToResult_1.subscribeToResult(this, openings, openings));
        }
        WindowToggleSubscriber.prototype._next = function (value) {
            var contexts = this.contexts;
            if (contexts) {
                var len = contexts.length;
                for (var i = 0; i < len; i++) {
                    contexts[i].window.next(value);
                }
            }
        };
        WindowToggleSubscriber.prototype._error = function (err) {
            var contexts = this.contexts;
            this.contexts = null;
            if (contexts) {
                var len = contexts.length;
                var index = -1;
                while (++index < len) {
                    var context = contexts[index];
                    context.window.error(err);
                    context.subscription.unsubscribe();
                }
            }
            _super.prototype._error.call(this, err);
        };
        WindowToggleSubscriber.prototype._complete = function () {
            var contexts = this.contexts;
            this.contexts = null;
            if (contexts) {
                var len = contexts.length;
                var index = -1;
                while (++index < len) {
                    var context = contexts[index];
                    context.window.complete();
                    context.subscription.unsubscribe();
                }
            }
            _super.prototype._complete.call(this);
        };
        WindowToggleSubscriber.prototype._unsubscribe = function () {
            var contexts = this.contexts;
            this.contexts = null;
            if (contexts) {
                var len = contexts.length;
                var index = -1;
                while (++index < len) {
                    var context = contexts[index];
                    context.window.unsubscribe();
                    context.subscription.unsubscribe();
                }
            }
        };
        WindowToggleSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            if (outerValue === this.openings) {
                var closingSelector = this.closingSelector;
                var closingNotifier = tryCatch_1.tryCatch(closingSelector)(innerValue);
                if (closingNotifier === errorObject_1.errorObject) {
                    return this.error(errorObject_1.errorObject.e);
                } else {
                    var window_1 = new Subject_1.Subject();
                    var subscription = new Subscription_1.Subscription();
                    var context = { window: window_1, subscription: subscription };
                    this.contexts.push(context);
                    var innerSubscription = subscribeToResult_1.subscribeToResult(this, closingNotifier, context);
                    if (innerSubscription.isUnsubscribed) {
                        this.closeWindow(this.contexts.length - 1);
                    } else {
                        innerSubscription.context = context;
                        subscription.add(innerSubscription);
                    }
                    this.destination.next(window_1);
                }
            } else {
                this.closeWindow(this.contexts.indexOf(outerValue));
            }
        };
        WindowToggleSubscriber.prototype.notifyError = function (err) {
            this.error(err);
        };
        WindowToggleSubscriber.prototype.notifyComplete = function (inner) {
            if (inner !== this.openSubscription) {
                this.closeWindow(this.contexts.indexOf(inner.context));
            }
        };
        WindowToggleSubscriber.prototype.closeWindow = function (index) {
            if (index === -1) {
                return;
            }
            var contexts = this.contexts;
            var context = contexts[index];
            var window = context.window,
                subscription = context.subscription;
            contexts.splice(index, 1);
            window.complete();
            subscription.unsubscribe();
        };
        return WindowToggleSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/windowToggle.js', ['../../Observable', '../../operator/windowToggle'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var windowToggle_1 = $__require('../../operator/windowToggle');
  Observable_1.Observable.prototype.windowToggle = windowToggle_1.windowToggle;
  
});
System.registerDynamic('node_modules/rxjs/operator/windowWhen.js', ['../Subject', '../util/tryCatch', '../util/errorObject', '../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subject_1 = $__require('../Subject');
    var tryCatch_1 = $__require('../util/tryCatch');
    var errorObject_1 = $__require('../util/errorObject');
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Branch out the source Observable values as a nested Observable using a
     * factory function of closing Observables to determine when to start a new
     * window.
     *
     * <span class="informal">It's like {@link bufferWhen}, but emits a nested
     * Observable instead of an array.</span>
     *
     * <img src="./img/windowWhen.png" width="100%">
     *
     * Returns an Observable that emits windows of items it collects from the source
     * Observable. The output Observable emits connected, non-overlapping windows.
     * It emits the current window and opens a new one whenever the Observable
     * produced by the specified `closingSelector` function emits an item. The first
     * window is opened immediately when subscribing to the output Observable.
     *
     * @example <caption>Emit only the first two clicks events in every window of [1-5] random seconds</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var result = clicks
     *   .windowWhen(() => Rx.Observable.interval(1000 + Math.random() * 4000))
     *   .map(win => win.take(2)) // each window has at most 2 emissions
     *   .mergeAll(); // flatten the Observable-of-Observables
     * result.subscribe(x => console.log(x));
     *
     * @see {@link window}
     * @see {@link windowCount}
     * @see {@link windowTime}
     * @see {@link windowToggle}
     * @see {@link bufferWhen}
     *
     * @param {function(): Observable} closingSelector A function that takes no
     * arguments and returns an Observable that signals (on either `next` or
     * `complete`) when to close the previous window and start a new one.
     * @return {Observable<Observable<T>>} An observable of windows, which in turn
     * are Observables.
     * @method windowWhen
     * @owner Observable
     */
    function windowWhen(closingSelector) {
        return this.lift(new WindowOperator(closingSelector));
    }
    exports.windowWhen = windowWhen;
    var WindowOperator = function () {
        function WindowOperator(closingSelector) {
            this.closingSelector = closingSelector;
        }
        WindowOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new WindowSubscriber(subscriber, this.closingSelector));
        };
        return WindowOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var WindowSubscriber = function (_super) {
        __extends(WindowSubscriber, _super);
        function WindowSubscriber(destination, closingSelector) {
            _super.call(this, destination);
            this.destination = destination;
            this.closingSelector = closingSelector;
            this.openWindow();
        }
        WindowSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            this.openWindow(innerSub);
        };
        WindowSubscriber.prototype.notifyError = function (error, innerSub) {
            this._error(error);
        };
        WindowSubscriber.prototype.notifyComplete = function (innerSub) {
            this.openWindow(innerSub);
        };
        WindowSubscriber.prototype._next = function (value) {
            this.window.next(value);
        };
        WindowSubscriber.prototype._error = function (err) {
            this.window.error(err);
            this.destination.error(err);
            this.unsubscribeClosingNotification();
        };
        WindowSubscriber.prototype._complete = function () {
            this.window.complete();
            this.destination.complete();
            this.unsubscribeClosingNotification();
        };
        WindowSubscriber.prototype.unsubscribeClosingNotification = function () {
            if (this.closingNotification) {
                this.closingNotification.unsubscribe();
            }
        };
        WindowSubscriber.prototype.openWindow = function (innerSub) {
            if (innerSub === void 0) {
                innerSub = null;
            }
            if (innerSub) {
                this.remove(innerSub);
                innerSub.unsubscribe();
            }
            var prevWindow = this.window;
            if (prevWindow) {
                prevWindow.complete();
            }
            var window = this.window = new Subject_1.Subject();
            this.destination.next(window);
            var closingNotifier = tryCatch_1.tryCatch(this.closingSelector)();
            if (closingNotifier === errorObject_1.errorObject) {
                var err = errorObject_1.errorObject.e;
                this.destination.error(err);
                this.window.error(err);
            } else {
                this.add(this.closingNotification = subscribeToResult_1.subscribeToResult(this, closingNotifier));
                this.add(window);
            }
        };
        return WindowSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/windowWhen.js', ['../../Observable', '../../operator/windowWhen'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var windowWhen_1 = $__require('../../operator/windowWhen');
  Observable_1.Observable.prototype.windowWhen = windowWhen_1.windowWhen;
  
});
System.registerDynamic('node_modules/rxjs/operator/withLatestFrom.js', ['../OuterSubscriber', '../util/subscribeToResult'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    /**
     * Combines the source Observable with other Observables to create an Observable
     * whose values are calculated from the latest values of each, only when the
     * source emits.
     *
     * <span class="informal">Whenever the source Observable emits a value, it
     * computes a formula using that value plus the latest values from other input
     * Observables, then emits the output of that formula.</span>
     *
     * <img src="./img/withLatestFrom.png" width="100%">
     *
     * `withLatestFrom` combines each value from the source Observable (the
     * instance) with the latest values from the other input Observables only when
     * the source emits a value, optionally using a `project` function to determine
     * the value to be emitted on the output Observable. All input Observables must
     * emit at least one value before the output Observable will emit a value.
     *
     * @example <caption>On every click event, emit an array with the latest timer event plus the click event</caption>
     * var clicks = Rx.Observable.fromEvent(document, 'click');
     * var timer = Rx.Observable.interval(1000);
     * var result = clicks.withLatestFrom(timer);
     * result.subscribe(x => console.log(x));
     *
     * @see {@link combineLatest}
     *
     * @param {Observable} other An input Observable to combine with the source
     * Observable. More than one input Observables may be given as argument.
     * @param {Function} [project] Projection function for combining values
     * together. Receives all values in order of the Observables passed, where the
     * first parameter is a value from the source Observable. (e.g.
     * `a.withLatestFrom(b, c, (a1, b1, c1) => a1 + b1 + c1)`). If this is not
     * passed, arrays will be emitted on the output Observable.
     * @return {Observable} An Observable of projected values from the most recent
     * values from each input Observable, or an array of the most recent values from
     * each input Observable.
     * @method withLatestFrom
     * @owner Observable
     */
    function withLatestFrom() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i - 0] = arguments[_i];
        }
        var project;
        if (typeof args[args.length - 1] === 'function') {
            project = args.pop();
        }
        var observables = args;
        return this.lift(new WithLatestFromOperator(observables, project));
    }
    exports.withLatestFrom = withLatestFrom;
    /* tslint:enable:max-line-length */
    var WithLatestFromOperator = function () {
        function WithLatestFromOperator(observables, project) {
            this.observables = observables;
            this.project = project;
        }
        WithLatestFromOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new WithLatestFromSubscriber(subscriber, this.observables, this.project));
        };
        return WithLatestFromOperator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var WithLatestFromSubscriber = function (_super) {
        __extends(WithLatestFromSubscriber, _super);
        function WithLatestFromSubscriber(destination, observables, project) {
            _super.call(this, destination);
            this.observables = observables;
            this.project = project;
            this.toRespond = [];
            var len = observables.length;
            this.values = new Array(len);
            for (var i = 0; i < len; i++) {
                this.toRespond.push(i);
            }
            for (var i = 0; i < len; i++) {
                var observable = observables[i];
                this.add(subscribeToResult_1.subscribeToResult(this, observable, observable, i));
            }
        }
        WithLatestFromSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            this.values[outerIndex] = innerValue;
            var toRespond = this.toRespond;
            if (toRespond.length > 0) {
                var found = toRespond.indexOf(outerIndex);
                if (found !== -1) {
                    toRespond.splice(found, 1);
                }
            }
        };
        WithLatestFromSubscriber.prototype.notifyComplete = function () {
            // noop
        };
        WithLatestFromSubscriber.prototype._next = function (value) {
            if (this.toRespond.length === 0) {
                var args = [value].concat(this.values);
                if (this.project) {
                    this._tryProject(args);
                } else {
                    this.destination.next(args);
                }
            }
        };
        WithLatestFromSubscriber.prototype._tryProject = function (args) {
            var result;
            try {
                result = this.project.apply(this, args);
            } catch (err) {
                this.destination.error(err);
                return;
            }
            this.destination.next(result);
        };
        return WithLatestFromSubscriber;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic('node_modules/rxjs/add/operator/withLatestFrom.js', ['../../Observable', '../../operator/withLatestFrom'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var withLatestFrom_1 = $__require('../../operator/withLatestFrom');
  Observable_1.Observable.prototype.withLatestFrom = withLatestFrom_1.withLatestFrom;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/zip.js', ['../../Observable', '../../operator/zip'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var zip_1 = $__require('../../operator/zip');
  Observable_1.Observable.prototype.zip = zip_1.zipProto;
  
});
System.registerDynamic("node_modules/rxjs/observable/ScalarObservable.js", ["../Observable"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Observable_1 = $__require('../Observable');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var ScalarObservable = function (_super) {
        __extends(ScalarObservable, _super);
        function ScalarObservable(value, scheduler) {
            _super.call(this);
            this.value = value;
            this.scheduler = scheduler;
            this._isScalar = true;
        }
        ScalarObservable.create = function (value, scheduler) {
            return new ScalarObservable(value, scheduler);
        };
        ScalarObservable.dispatch = function (state) {
            var done = state.done,
                value = state.value,
                subscriber = state.subscriber;
            if (done) {
                subscriber.complete();
                return;
            }
            subscriber.next(value);
            if (subscriber.isUnsubscribed) {
                return;
            }
            state.done = true;
            this.schedule(state);
        };
        ScalarObservable.prototype._subscribe = function (subscriber) {
            var value = this.value;
            var scheduler = this.scheduler;
            if (scheduler) {
                return scheduler.schedule(ScalarObservable.dispatch, 0, {
                    done: false, value: value, subscriber: subscriber
                });
            } else {
                subscriber.next(value);
                if (!subscriber.isUnsubscribed) {
                    subscriber.complete();
                }
            }
        };
        return ScalarObservable;
    }(Observable_1.Observable);
    exports.ScalarObservable = ScalarObservable;
    
});
System.registerDynamic("node_modules/rxjs/observable/EmptyObservable.js", ["../Observable"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Observable_1 = $__require('../Observable');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var EmptyObservable = function (_super) {
        __extends(EmptyObservable, _super);
        function EmptyObservable(scheduler) {
            _super.call(this);
            this.scheduler = scheduler;
        }
        /**
         * Creates an Observable that emits no items to the Observer and immediately
         * emits a complete notification.
         *
         * <span class="informal">Just emits 'complete', and nothing else.
         * </span>
         *
         * <img src="./img/empty.png" width="100%">
         *
         * This static operator is useful for creating a simple Observable that only
         * emits the complete notification. It can be used for composing with other
         * Observables, such as in a {@link mergeMap}.
         *
         * @example <caption>Emit the number 7, then complete.</caption>
         * var result = Rx.Observable.empty().startWith(7);
         * result.subscribe(x => console.log(x));
         *
         * @example <caption>Map and flatten only odd numbers to the sequence 'a', 'b', 'c'</caption>
         * var interval = Rx.Observable.interval(1000);
         * var result = interval.mergeMap(x =>
         *   x % 2 === 1 ? Rx.Observable.of('a', 'b', 'c') : Rx.Observable.empty()
         * );
         * result.subscribe(x => console.log(x));
         *
         * @see {@link create}
         * @see {@link never}
         * @see {@link of}
         * @see {@link throw}
         *
         * @param {Scheduler} [scheduler] A {@link Scheduler} to use for scheduling
         * the emission of the complete notification.
         * @return {Observable} An "empty" Observable: emits only the complete
         * notification.
         * @static true
         * @name empty
         * @owner Observable
         */
        EmptyObservable.create = function (scheduler) {
            return new EmptyObservable(scheduler);
        };
        EmptyObservable.dispatch = function (arg) {
            var subscriber = arg.subscriber;
            subscriber.complete();
        };
        EmptyObservable.prototype._subscribe = function (subscriber) {
            var scheduler = this.scheduler;
            if (scheduler) {
                return scheduler.schedule(EmptyObservable.dispatch, 0, { subscriber: subscriber });
            } else {
                subscriber.complete();
            }
        };
        return EmptyObservable;
    }(Observable_1.Observable);
    exports.EmptyObservable = EmptyObservable;
    
});
System.registerDynamic("node_modules/rxjs/util/isScheduler.js", [], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    function isScheduler(value) {
        return value && typeof value.schedule === 'function';
    }
    exports.isScheduler = isScheduler;
    
});
System.registerDynamic('node_modules/rxjs/observable/ArrayObservable.js', ['../Observable', './ScalarObservable', './EmptyObservable', '../util/isScheduler'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Observable_1 = $__require('../Observable');
    var ScalarObservable_1 = $__require('./ScalarObservable');
    var EmptyObservable_1 = $__require('./EmptyObservable');
    var isScheduler_1 = $__require('../util/isScheduler');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @extends {Ignored}
     * @hide true
     */
    var ArrayObservable = function (_super) {
        __extends(ArrayObservable, _super);
        function ArrayObservable(array, scheduler) {
            _super.call(this);
            this.array = array;
            this.scheduler = scheduler;
            if (!scheduler && array.length === 1) {
                this._isScalar = true;
                this.value = array[0];
            }
        }
        ArrayObservable.create = function (array, scheduler) {
            return new ArrayObservable(array, scheduler);
        };
        /**
         * Creates an Observable that emits some values you specify as arguments,
         * immediately one after the other, and then emits a complete notification.
         *
         * <span class="informal">Emits the arguments you provide, then completes.
         * </span>
         *
         * <img src="./img/of.png" width="100%">
         *
         * This static operator is useful for creating a simple Observable that only
         * emits the arguments given, and the complete notification thereafter. It can
         * be used for composing with other Observables, such as with {@link concat}.
         * By default, it uses a `null` Scheduler, which means the `next`
         * notifications are sent synchronously, although with a different Scheduler
         * it is possible to determine when those notifications will be delivered.
         *
         * @example <caption>Emit 10, 20, 30, then 'a', 'b', 'c', then start ticking every second.</caption>
         * var numbers = Rx.Observable.of(10, 20, 30);
         * var letters = Rx.Observable.of('a', 'b', 'c');
         * var interval = Rx.Observable.interval(1000);
         * var result = numbers.concat(letters).concat(interval);
         * result.subscribe(x => console.log(x));
         *
         * @see {@link create}
         * @see {@link empty}
         * @see {@link never}
         * @see {@link throw}
         *
         * @param {...T} values Arguments that represent `next` values to be emitted.
         * @param {Scheduler} [scheduler] A {@link Scheduler} to use for scheduling
         * the emissions of the `next` notifications.
         * @return {Observable<T>} An Observable that emits each given input value.
         * @static true
         * @name of
         * @owner Observable
         */
        ArrayObservable.of = function () {
            var array = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                array[_i - 0] = arguments[_i];
            }
            var scheduler = array[array.length - 1];
            if (isScheduler_1.isScheduler(scheduler)) {
                array.pop();
            } else {
                scheduler = null;
            }
            var len = array.length;
            if (len > 1) {
                return new ArrayObservable(array, scheduler);
            } else if (len === 1) {
                return new ScalarObservable_1.ScalarObservable(array[0], scheduler);
            } else {
                return new EmptyObservable_1.EmptyObservable(scheduler);
            }
        };
        ArrayObservable.dispatch = function (state) {
            var array = state.array,
                index = state.index,
                count = state.count,
                subscriber = state.subscriber;
            if (index >= count) {
                subscriber.complete();
                return;
            }
            subscriber.next(array[index]);
            if (subscriber.isUnsubscribed) {
                return;
            }
            state.index = index + 1;
            this.schedule(state);
        };
        ArrayObservable.prototype._subscribe = function (subscriber) {
            var index = 0;
            var array = this.array;
            var count = array.length;
            var scheduler = this.scheduler;
            if (scheduler) {
                return scheduler.schedule(ArrayObservable.dispatch, 0, {
                    array: array, index: index, count: count, subscriber: subscriber
                });
            } else {
                for (var i = 0; i < count && !subscriber.isUnsubscribed; i++) {
                    subscriber.next(array[i]);
                }
                subscriber.complete();
            }
        };
        return ArrayObservable;
    }(Observable_1.Observable);
    exports.ArrayObservable = ArrayObservable;
    
});
System.registerDynamic("node_modules/rxjs/OuterSubscriber.js", ["./Subscriber"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('./Subscriber');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var OuterSubscriber = function (_super) {
        __extends(OuterSubscriber, _super);
        function OuterSubscriber() {
            _super.apply(this, arguments);
        }
        OuterSubscriber.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            this.destination.next(innerValue);
        };
        OuterSubscriber.prototype.notifyError = function (error, innerSub) {
            this.destination.error(error);
        };
        OuterSubscriber.prototype.notifyComplete = function (innerSub) {
            this.destination.complete();
        };
        return OuterSubscriber;
    }(Subscriber_1.Subscriber);
    exports.OuterSubscriber = OuterSubscriber;
    
});
System.registerDynamic('node_modules/rxjs/util/isPromise.js', [], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    function isPromise(value) {
        return value && typeof value.subscribe !== 'function' && typeof value.then === 'function';
    }
    exports.isPromise = isPromise;
    
});
System.registerDynamic("node_modules/rxjs/InnerSubscriber.js", ["./Subscriber"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('./Subscriber');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var InnerSubscriber = function (_super) {
        __extends(InnerSubscriber, _super);
        function InnerSubscriber(parent, outerValue, outerIndex) {
            _super.call(this);
            this.parent = parent;
            this.outerValue = outerValue;
            this.outerIndex = outerIndex;
            this.index = 0;
        }
        InnerSubscriber.prototype._next = function (value) {
            this.parent.notifyNext(this.outerValue, value, this.outerIndex, this.index++, this);
        };
        InnerSubscriber.prototype._error = function (error) {
            this.parent.notifyError(error, this);
            this.unsubscribe();
        };
        InnerSubscriber.prototype._complete = function () {
            this.parent.notifyComplete(this);
            this.unsubscribe();
        };
        return InnerSubscriber;
    }(Subscriber_1.Subscriber);
    exports.InnerSubscriber = InnerSubscriber;
    
});
System.registerDynamic('node_modules/rxjs/util/subscribeToResult.js', ['./root', './isArray', './isPromise', '../Observable', '../symbol/iterator', '../symbol/observable', '../InnerSubscriber'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var root_1 = $__require('./root');
    var isArray_1 = $__require('./isArray');
    var isPromise_1 = $__require('./isPromise');
    var Observable_1 = $__require('../Observable');
    var iterator_1 = $__require('../symbol/iterator');
    var observable_1 = $__require('../symbol/observable');
    var InnerSubscriber_1 = $__require('../InnerSubscriber');
    function subscribeToResult(outerSubscriber, result, outerValue, outerIndex) {
        var destination = new InnerSubscriber_1.InnerSubscriber(outerSubscriber, outerValue, outerIndex);
        if (destination.isUnsubscribed) {
            return;
        }
        if (result instanceof Observable_1.Observable) {
            if (result._isScalar) {
                destination.next(result.value);
                destination.complete();
                return;
            } else {
                return result.subscribe(destination);
            }
        }
        if (isArray_1.isArray(result)) {
            for (var i = 0, len = result.length; i < len && !destination.isUnsubscribed; i++) {
                destination.next(result[i]);
            }
            if (!destination.isUnsubscribed) {
                destination.complete();
            }
        } else if (isPromise_1.isPromise(result)) {
            result.then(function (value) {
                if (!destination.isUnsubscribed) {
                    destination.next(value);
                    destination.complete();
                }
            }, function (err) {
                return destination.error(err);
            }).then(null, function (err) {
                // Escaping the Promise trap: globally throw unhandled errors
                root_1.root.setTimeout(function () {
                    throw err;
                });
            });
            return destination;
        } else if (typeof result[iterator_1.$$iterator] === 'function') {
            for (var _i = 0, _a = result; _i < _a.length; _i++) {
                var item = _a[_i];
                destination.next(item);
                if (destination.isUnsubscribed) {
                    break;
                }
            }
            if (!destination.isUnsubscribed) {
                destination.complete();
            }
        } else if (typeof result[observable_1.$$observable] === 'function') {
            var obs = result[observable_1.$$observable]();
            if (typeof obs.subscribe !== 'function') {
                destination.error('invalid observable');
            } else {
                return obs.subscribe(new InnerSubscriber_1.InnerSubscriber(outerSubscriber, outerValue, outerIndex));
            }
        } else {
            destination.error(new TypeError('unknown type returned'));
        }
    }
    exports.subscribeToResult = subscribeToResult;
    
});
System.registerDynamic('node_modules/rxjs/operator/zip.js', ['../observable/ArrayObservable', '../util/isArray', '../Subscriber', '../OuterSubscriber', '../util/subscribeToResult', '../symbol/iterator'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var ArrayObservable_1 = $__require('../observable/ArrayObservable');
    var isArray_1 = $__require('../util/isArray');
    var Subscriber_1 = $__require('../Subscriber');
    var OuterSubscriber_1 = $__require('../OuterSubscriber');
    var subscribeToResult_1 = $__require('../util/subscribeToResult');
    var iterator_1 = $__require('../symbol/iterator');
    /**
     * @param observables
     * @return {Observable<R>}
     * @method zip
     * @owner Observable
     */
    function zipProto() {
        var observables = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            observables[_i - 0] = arguments[_i];
        }
        observables.unshift(this);
        return zipStatic.apply(this, observables);
    }
    exports.zipProto = zipProto;
    /* tslint:enable:max-line-length */
    /**
     * @param observables
     * @return {Observable<R>}
     * @static true
     * @name zip
     * @owner Observable
     */
    function zipStatic() {
        var observables = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            observables[_i - 0] = arguments[_i];
        }
        var project = observables[observables.length - 1];
        if (typeof project === 'function') {
            observables.pop();
        }
        return new ArrayObservable_1.ArrayObservable(observables).lift(new ZipOperator(project));
    }
    exports.zipStatic = zipStatic;
    var ZipOperator = function () {
        function ZipOperator(project) {
            this.project = project;
        }
        ZipOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new ZipSubscriber(subscriber, this.project));
        };
        return ZipOperator;
    }();
    exports.ZipOperator = ZipOperator;
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var ZipSubscriber = function (_super) {
        __extends(ZipSubscriber, _super);
        function ZipSubscriber(destination, project, values) {
            if (values === void 0) {
                values = Object.create(null);
            }
            _super.call(this, destination);
            this.index = 0;
            this.iterators = [];
            this.active = 0;
            this.project = typeof project === 'function' ? project : null;
            this.values = values;
        }
        ZipSubscriber.prototype._next = function (value) {
            var iterators = this.iterators;
            var index = this.index++;
            if (isArray_1.isArray(value)) {
                iterators.push(new StaticArrayIterator(value));
            } else if (typeof value[iterator_1.$$iterator] === 'function') {
                iterators.push(new StaticIterator(value[iterator_1.$$iterator]()));
            } else {
                iterators.push(new ZipBufferIterator(this.destination, this, value, index));
            }
        };
        ZipSubscriber.prototype._complete = function () {
            var iterators = this.iterators;
            var len = iterators.length;
            this.active = len;
            for (var i = 0; i < len; i++) {
                var iterator = iterators[i];
                if (iterator.stillUnsubscribed) {
                    this.add(iterator.subscribe(iterator, i));
                } else {
                    this.active--; // not an observable
                }
            }
        };
        ZipSubscriber.prototype.notifyInactive = function () {
            this.active--;
            if (this.active === 0) {
                this.destination.complete();
            }
        };
        ZipSubscriber.prototype.checkIterators = function () {
            var iterators = this.iterators;
            var len = iterators.length;
            var destination = this.destination;
            // abort if not all of them have values
            for (var i = 0; i < len; i++) {
                var iterator = iterators[i];
                if (typeof iterator.hasValue === 'function' && !iterator.hasValue()) {
                    return;
                }
            }
            var shouldComplete = false;
            var args = [];
            for (var i = 0; i < len; i++) {
                var iterator = iterators[i];
                var result = iterator.next();
                // check to see if it's completed now that you've gotten
                // the next value.
                if (iterator.hasCompleted()) {
                    shouldComplete = true;
                }
                if (result.done) {
                    destination.complete();
                    return;
                }
                args.push(result.value);
            }
            if (this.project) {
                this._tryProject(args);
            } else {
                destination.next(args);
            }
            if (shouldComplete) {
                destination.complete();
            }
        };
        ZipSubscriber.prototype._tryProject = function (args) {
            var result;
            try {
                result = this.project.apply(this, args);
            } catch (err) {
                this.destination.error(err);
                return;
            }
            this.destination.next(result);
        };
        return ZipSubscriber;
    }(Subscriber_1.Subscriber);
    exports.ZipSubscriber = ZipSubscriber;
    var StaticIterator = function () {
        function StaticIterator(iterator) {
            this.iterator = iterator;
            this.nextResult = iterator.next();
        }
        StaticIterator.prototype.hasValue = function () {
            return true;
        };
        StaticIterator.prototype.next = function () {
            var result = this.nextResult;
            this.nextResult = this.iterator.next();
            return result;
        };
        StaticIterator.prototype.hasCompleted = function () {
            var nextResult = this.nextResult;
            return nextResult && nextResult.done;
        };
        return StaticIterator;
    }();
    var StaticArrayIterator = function () {
        function StaticArrayIterator(array) {
            this.array = array;
            this.index = 0;
            this.length = 0;
            this.length = array.length;
        }
        StaticArrayIterator.prototype[iterator_1.$$iterator] = function () {
            return this;
        };
        StaticArrayIterator.prototype.next = function (value) {
            var i = this.index++;
            var array = this.array;
            return i < this.length ? { value: array[i], done: false } : { done: true };
        };
        StaticArrayIterator.prototype.hasValue = function () {
            return this.array.length > this.index;
        };
        StaticArrayIterator.prototype.hasCompleted = function () {
            return this.array.length === this.index;
        };
        return StaticArrayIterator;
    }();
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var ZipBufferIterator = function (_super) {
        __extends(ZipBufferIterator, _super);
        function ZipBufferIterator(destination, parent, observable, index) {
            _super.call(this, destination);
            this.parent = parent;
            this.observable = observable;
            this.index = index;
            this.stillUnsubscribed = true;
            this.buffer = [];
            this.isComplete = false;
        }
        ZipBufferIterator.prototype[iterator_1.$$iterator] = function () {
            return this;
        };
        // NOTE: there is actually a name collision here with Subscriber.next and Iterator.next
        //    this is legit because `next()` will never be called by a subscription in this case.
        ZipBufferIterator.prototype.next = function () {
            var buffer = this.buffer;
            if (buffer.length === 0 && this.isComplete) {
                return { done: true };
            } else {
                return { value: buffer.shift(), done: false };
            }
        };
        ZipBufferIterator.prototype.hasValue = function () {
            return this.buffer.length > 0;
        };
        ZipBufferIterator.prototype.hasCompleted = function () {
            return this.buffer.length === 0 && this.isComplete;
        };
        ZipBufferIterator.prototype.notifyComplete = function () {
            if (this.buffer.length > 0) {
                this.isComplete = true;
                this.parent.notifyInactive();
            } else {
                this.destination.complete();
            }
        };
        ZipBufferIterator.prototype.notifyNext = function (outerValue, innerValue, outerIndex, innerIndex, innerSub) {
            this.buffer.push(innerValue);
            this.parent.checkIterators();
        };
        ZipBufferIterator.prototype.subscribe = function (value, index) {
            return subscribeToResult_1.subscribeToResult(this, this.observable, this, index);
        };
        return ZipBufferIterator;
    }(OuterSubscriber_1.OuterSubscriber);
    
});
System.registerDynamic("node_modules/rxjs/operator/zipAll.js", ["./zip"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var zip_1 = $__require('./zip');
  /**
   * @param project
   * @return {Observable<R>|WebSocketSubject<T>|Observable<T>}
   * @method zipAll
   * @owner Observable
   */
  function zipAll(project) {
    return this.lift(new zip_1.ZipOperator(project));
  }
  exports.zipAll = zipAll;
  
});
System.registerDynamic('node_modules/rxjs/add/operator/zipAll.js', ['../../Observable', '../../operator/zipAll'], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var Observable_1 = $__require('../../Observable');
  var zipAll_1 = $__require('../../operator/zipAll');
  Observable_1.Observable.prototype.zipAll = zipAll_1.zipAll;
  
});
System.registerDynamic("node_modules/rxjs/Operator.js", ["./Subscriber"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var Subscriber_1 = $__require('./Subscriber');
    var Operator = function () {
        function Operator() {}
        Operator.prototype.call = function (subscriber, source) {
            return source._subscribe(new Subscriber_1.Subscriber(subscriber));
        };
        return Operator;
    }();
    exports.Operator = Operator;
    
});
System.registerDynamic("node_modules/rxjs/AsyncSubject.js", ["./Subject"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subject_1 = $__require('./Subject');
    /**
     * @class AsyncSubject<T>
     */
    var AsyncSubject = function (_super) {
        __extends(AsyncSubject, _super);
        function AsyncSubject() {
            _super.apply(this, arguments);
            this.value = null;
            this.hasNext = false;
        }
        AsyncSubject.prototype._subscribe = function (subscriber) {
            if (this.hasCompleted && this.hasNext) {
                subscriber.next(this.value);
            }
            return _super.prototype._subscribe.call(this, subscriber);
        };
        AsyncSubject.prototype._next = function (value) {
            this.value = value;
            this.hasNext = true;
        };
        AsyncSubject.prototype._complete = function () {
            var index = -1;
            var observers = this.observers;
            var len = observers.length;
            // optimization to block our SubjectSubscriptions from
            // splicing themselves out of the observers list one by one.
            this.isUnsubscribed = true;
            if (this.hasNext) {
                while (++index < len) {
                    var o = observers[index];
                    o.next(this.value);
                    o.complete();
                }
            } else {
                while (++index < len) {
                    observers[index].complete();
                }
            }
            this.isUnsubscribed = false;
            this.unsubscribe();
        };
        return AsyncSubject;
    }(Subject_1.Subject);
    exports.AsyncSubject = AsyncSubject;
    
});
System.registerDynamic('node_modules/rxjs/operator/observeOn.js', ['../Subscriber', '../Notification'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscriber_1 = $__require('../Subscriber');
    var Notification_1 = $__require('../Notification');
    /**
     * @see {@link Notification}
     *
     * @param scheduler
     * @param delay
     * @return {Observable<R>|WebSocketSubject<T>|Observable<T>}
     * @method observeOn
     * @owner Observable
     */
    function observeOn(scheduler, delay) {
        if (delay === void 0) {
            delay = 0;
        }
        return this.lift(new ObserveOnOperator(scheduler, delay));
    }
    exports.observeOn = observeOn;
    var ObserveOnOperator = function () {
        function ObserveOnOperator(scheduler, delay) {
            if (delay === void 0) {
                delay = 0;
            }
            this.scheduler = scheduler;
            this.delay = delay;
        }
        ObserveOnOperator.prototype.call = function (subscriber, source) {
            return source._subscribe(new ObserveOnSubscriber(subscriber, this.scheduler, this.delay));
        };
        return ObserveOnOperator;
    }();
    exports.ObserveOnOperator = ObserveOnOperator;
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var ObserveOnSubscriber = function (_super) {
        __extends(ObserveOnSubscriber, _super);
        function ObserveOnSubscriber(destination, scheduler, delay) {
            if (delay === void 0) {
                delay = 0;
            }
            _super.call(this, destination);
            this.scheduler = scheduler;
            this.delay = delay;
        }
        ObserveOnSubscriber.dispatch = function (arg) {
            var notification = arg.notification,
                destination = arg.destination;
            notification.observe(destination);
        };
        ObserveOnSubscriber.prototype.scheduleMessage = function (notification) {
            this.add(this.scheduler.schedule(ObserveOnSubscriber.dispatch, this.delay, new ObserveOnMessage(notification, this.destination)));
        };
        ObserveOnSubscriber.prototype._next = function (value) {
            this.scheduleMessage(Notification_1.Notification.createNext(value));
        };
        ObserveOnSubscriber.prototype._error = function (err) {
            this.scheduleMessage(Notification_1.Notification.createError(err));
        };
        ObserveOnSubscriber.prototype._complete = function () {
            this.scheduleMessage(Notification_1.Notification.createComplete());
        };
        return ObserveOnSubscriber;
    }(Subscriber_1.Subscriber);
    exports.ObserveOnSubscriber = ObserveOnSubscriber;
    var ObserveOnMessage = function () {
        function ObserveOnMessage(notification, destination) {
            this.notification = notification;
            this.destination = destination;
        }
        return ObserveOnMessage;
    }();
    exports.ObserveOnMessage = ObserveOnMessage;
    
});
System.registerDynamic('node_modules/rxjs/ReplaySubject.js', ['./Subject', './scheduler/queue', './operator/observeOn'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subject_1 = $__require('./Subject');
    var queue_1 = $__require('./scheduler/queue');
    var observeOn_1 = $__require('./operator/observeOn');
    /**
     * @class ReplaySubject<T>
     */
    var ReplaySubject = function (_super) {
        __extends(ReplaySubject, _super);
        function ReplaySubject(bufferSize, windowTime, scheduler) {
            if (bufferSize === void 0) {
                bufferSize = Number.POSITIVE_INFINITY;
            }
            if (windowTime === void 0) {
                windowTime = Number.POSITIVE_INFINITY;
            }
            _super.call(this);
            this.events = [];
            this.scheduler = scheduler;
            this.bufferSize = bufferSize < 1 ? 1 : bufferSize;
            this._windowTime = windowTime < 1 ? 1 : windowTime;
        }
        ReplaySubject.prototype._next = function (value) {
            var now = this._getNow();
            this.events.push(new ReplayEvent(now, value));
            this._trimBufferThenGetEvents(now);
            _super.prototype._next.call(this, value);
        };
        ReplaySubject.prototype._subscribe = function (subscriber) {
            var events = this._trimBufferThenGetEvents(this._getNow());
            var scheduler = this.scheduler;
            if (scheduler) {
                subscriber.add(subscriber = new observeOn_1.ObserveOnSubscriber(subscriber, scheduler));
            }
            var index = -1;
            var len = events.length;
            while (++index < len && !subscriber.isUnsubscribed) {
                subscriber.next(events[index].value);
            }
            return _super.prototype._subscribe.call(this, subscriber);
        };
        ReplaySubject.prototype._getNow = function () {
            return (this.scheduler || queue_1.queue).now();
        };
        ReplaySubject.prototype._trimBufferThenGetEvents = function (now) {
            var bufferSize = this.bufferSize;
            var _windowTime = this._windowTime;
            var events = this.events;
            var eventsCount = events.length;
            var spliceCount = 0;
            // Trim events that fall out of the time window.
            // Start at the front of the list. Break early once
            // we encounter an event that falls within the window.
            while (spliceCount < eventsCount) {
                if (now - events[spliceCount].time < _windowTime) {
                    break;
                }
                spliceCount += 1;
            }
            if (eventsCount > bufferSize) {
                spliceCount = Math.max(spliceCount, eventsCount - bufferSize);
            }
            if (spliceCount > 0) {
                events.splice(0, spliceCount);
            }
            return events;
        };
        return ReplaySubject;
    }(Subject_1.Subject);
    exports.ReplaySubject = ReplaySubject;
    var ReplayEvent = function () {
        function ReplayEvent(time, value) {
            this.time = time;
            this.value = value;
        }
        return ReplayEvent;
    }();
    
});
System.registerDynamic("node_modules/rxjs/SubjectSubscription.js", ["./Subscription"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subscription_1 = $__require('./Subscription');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var SubjectSubscription = function (_super) {
        __extends(SubjectSubscription, _super);
        function SubjectSubscription(subject, observer) {
            _super.call(this);
            this.subject = subject;
            this.observer = observer;
            this.isUnsubscribed = false;
        }
        SubjectSubscription.prototype.unsubscribe = function () {
            if (this.isUnsubscribed) {
                return;
            }
            this.isUnsubscribed = true;
            var subject = this.subject;
            var observers = subject.observers;
            this.subject = null;
            if (!observers || observers.length === 0 || subject.isUnsubscribed) {
                return;
            }
            var subscriberIndex = observers.indexOf(this.observer);
            if (subscriberIndex !== -1) {
                observers.splice(subscriberIndex, 1);
            }
        };
        return SubjectSubscription;
    }(Subscription_1.Subscription);
    exports.SubjectSubscription = SubjectSubscription;
    
});
System.registerDynamic('node_modules/rxjs/Subject.js', ['./Observable', './Subscriber', './Subscription', './SubjectSubscription', './symbol/rxSubscriber', './util/throwError', './util/ObjectUnsubscribedError'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Observable_1 = $__require('./Observable');
    var Subscriber_1 = $__require('./Subscriber');
    var Subscription_1 = $__require('./Subscription');
    var SubjectSubscription_1 = $__require('./SubjectSubscription');
    var rxSubscriber_1 = $__require('./symbol/rxSubscriber');
    var throwError_1 = $__require('./util/throwError');
    var ObjectUnsubscribedError_1 = $__require('./util/ObjectUnsubscribedError');
    /**
     * @class Subject<T>
     */
    var Subject = function (_super) {
        __extends(Subject, _super);
        function Subject(destination, source) {
            _super.call(this);
            this.destination = destination;
            this.source = source;
            this.observers = [];
            this.isUnsubscribed = false;
            this.isStopped = false;
            this.hasErrored = false;
            this.dispatching = false;
            this.hasCompleted = false;
            this.source = source;
        }
        Subject.prototype.lift = function (operator) {
            var subject = new Subject(this.destination || this, this);
            subject.operator = operator;
            return subject;
        };
        Subject.prototype.add = function (subscription) {
            return Subscription_1.Subscription.prototype.add.call(this, subscription);
        };
        Subject.prototype.remove = function (subscription) {
            Subscription_1.Subscription.prototype.remove.call(this, subscription);
        };
        Subject.prototype.unsubscribe = function () {
            Subscription_1.Subscription.prototype.unsubscribe.call(this);
        };
        Subject.prototype._subscribe = function (subscriber) {
            if (this.source) {
                return this.source.subscribe(subscriber);
            } else {
                if (subscriber.isUnsubscribed) {
                    return;
                } else if (this.hasErrored) {
                    return subscriber.error(this.errorValue);
                } else if (this.hasCompleted) {
                    return subscriber.complete();
                }
                this.throwIfUnsubscribed();
                var subscription = new SubjectSubscription_1.SubjectSubscription(this, subscriber);
                this.observers.push(subscriber);
                return subscription;
            }
        };
        Subject.prototype._unsubscribe = function () {
            this.source = null;
            this.isStopped = true;
            this.observers = null;
            this.destination = null;
        };
        Subject.prototype.next = function (value) {
            this.throwIfUnsubscribed();
            if (this.isStopped) {
                return;
            }
            this.dispatching = true;
            this._next(value);
            this.dispatching = false;
            if (this.hasErrored) {
                this._error(this.errorValue);
            } else if (this.hasCompleted) {
                this._complete();
            }
        };
        Subject.prototype.error = function (err) {
            this.throwIfUnsubscribed();
            if (this.isStopped) {
                return;
            }
            this.isStopped = true;
            this.hasErrored = true;
            this.errorValue = err;
            if (this.dispatching) {
                return;
            }
            this._error(err);
        };
        Subject.prototype.complete = function () {
            this.throwIfUnsubscribed();
            if (this.isStopped) {
                return;
            }
            this.isStopped = true;
            this.hasCompleted = true;
            if (this.dispatching) {
                return;
            }
            this._complete();
        };
        Subject.prototype.asObservable = function () {
            var observable = new SubjectObservable(this);
            return observable;
        };
        Subject.prototype._next = function (value) {
            if (this.destination) {
                this.destination.next(value);
            } else {
                this._finalNext(value);
            }
        };
        Subject.prototype._finalNext = function (value) {
            var index = -1;
            var observers = this.observers.slice(0);
            var len = observers.length;
            while (++index < len) {
                observers[index].next(value);
            }
        };
        Subject.prototype._error = function (err) {
            if (this.destination) {
                this.destination.error(err);
            } else {
                this._finalError(err);
            }
        };
        Subject.prototype._finalError = function (err) {
            var index = -1;
            var observers = this.observers;
            // optimization to block our SubjectSubscriptions from
            // splicing themselves out of the observers list one by one.
            this.observers = null;
            this.isUnsubscribed = true;
            if (observers) {
                var len = observers.length;
                while (++index < len) {
                    observers[index].error(err);
                }
            }
            this.isUnsubscribed = false;
            this.unsubscribe();
        };
        Subject.prototype._complete = function () {
            if (this.destination) {
                this.destination.complete();
            } else {
                this._finalComplete();
            }
        };
        Subject.prototype._finalComplete = function () {
            var index = -1;
            var observers = this.observers;
            // optimization to block our SubjectSubscriptions from
            // splicing themselves out of the observers list one by one.
            this.observers = null;
            this.isUnsubscribed = true;
            if (observers) {
                var len = observers.length;
                while (++index < len) {
                    observers[index].complete();
                }
            }
            this.isUnsubscribed = false;
            this.unsubscribe();
        };
        Subject.prototype.throwIfUnsubscribed = function () {
            if (this.isUnsubscribed) {
                throwError_1.throwError(new ObjectUnsubscribedError_1.ObjectUnsubscribedError());
            }
        };
        Subject.prototype[rxSubscriber_1.$$rxSubscriber] = function () {
            return new Subscriber_1.Subscriber(this);
        };
        Subject.create = function (destination, source) {
            return new Subject(destination, source);
        };
        return Subject;
    }(Observable_1.Observable);
    exports.Subject = Subject;
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var SubjectObservable = function (_super) {
        __extends(SubjectObservable, _super);
        function SubjectObservable(source) {
            _super.call(this);
            this.source = source;
        }
        return SubjectObservable;
    }(Observable_1.Observable);
    
});
System.registerDynamic("node_modules/rxjs/util/throwError.js", [], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  function throwError(e) {
    throw e;
  }
  exports.throwError = throwError;
  
});
System.registerDynamic('node_modules/rxjs/BehaviorSubject.js', ['./Subject', './util/throwError', './util/ObjectUnsubscribedError'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Subject_1 = $__require('./Subject');
    var throwError_1 = $__require('./util/throwError');
    var ObjectUnsubscribedError_1 = $__require('./util/ObjectUnsubscribedError');
    /**
     * @class BehaviorSubject<T>
     */
    var BehaviorSubject = function (_super) {
        __extends(BehaviorSubject, _super);
        function BehaviorSubject(_value) {
            _super.call(this);
            this._value = _value;
        }
        BehaviorSubject.prototype.getValue = function () {
            if (this.hasErrored) {
                throwError_1.throwError(this.errorValue);
            } else if (this.isUnsubscribed) {
                throwError_1.throwError(new ObjectUnsubscribedError_1.ObjectUnsubscribedError());
            } else {
                return this._value;
            }
        };
        Object.defineProperty(BehaviorSubject.prototype, "value", {
            get: function () {
                return this.getValue();
            },
            enumerable: true,
            configurable: true
        });
        BehaviorSubject.prototype._subscribe = function (subscriber) {
            var subscription = _super.prototype._subscribe.call(this, subscriber);
            if (subscription && !subscription.isUnsubscribed) {
                subscriber.next(this._value);
            }
            return subscription;
        };
        BehaviorSubject.prototype._next = function (value) {
            _super.prototype._next.call(this, this._value = value);
        };
        BehaviorSubject.prototype._error = function (err) {
            this.hasErrored = true;
            _super.prototype._error.call(this, this.errorValue = err);
        };
        return BehaviorSubject;
    }(Subject_1.Subject);
    exports.BehaviorSubject = BehaviorSubject;
    
});
System.registerDynamic('node_modules/rxjs/observable/ConnectableObservable.js', ['../Observable', '../Subscriber', '../Subscription'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Observable_1 = $__require('../Observable');
    var Subscriber_1 = $__require('../Subscriber');
    var Subscription_1 = $__require('../Subscription');
    /**
     * @class ConnectableObservable<T>
     */
    var ConnectableObservable = function (_super) {
        __extends(ConnectableObservable, _super);
        function ConnectableObservable(source, subjectFactory) {
            _super.call(this);
            this.source = source;
            this.subjectFactory = subjectFactory;
        }
        ConnectableObservable.prototype._subscribe = function (subscriber) {
            return this.getSubject().subscribe(subscriber);
        };
        ConnectableObservable.prototype.getSubject = function () {
            var subject = this.subject;
            if (subject && !subject.isUnsubscribed) {
                return subject;
            }
            return this.subject = this.subjectFactory();
        };
        ConnectableObservable.prototype.connect = function () {
            var source = this.source;
            var subscription = this.subscription;
            if (subscription && !subscription.isUnsubscribed) {
                return subscription;
            }
            subscription = source.subscribe(this.getSubject());
            subscription.add(new ConnectableSubscription(this));
            return this.subscription = subscription;
        };
        ConnectableObservable.prototype.refCount = function () {
            return new RefCountObservable(this);
        };
        /**
         * This method is opened for `ConnectableSubscription`.
         * Not to call from others.
         */
        ConnectableObservable.prototype._closeSubscription = function () {
            this.subject = null;
            this.subscription = null;
        };
        return ConnectableObservable;
    }(Observable_1.Observable);
    exports.ConnectableObservable = ConnectableObservable;
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var ConnectableSubscription = function (_super) {
        __extends(ConnectableSubscription, _super);
        function ConnectableSubscription(connectable) {
            _super.call(this);
            this.connectable = connectable;
        }
        ConnectableSubscription.prototype._unsubscribe = function () {
            var connectable = this.connectable;
            connectable._closeSubscription();
            this.connectable = null;
        };
        return ConnectableSubscription;
    }(Subscription_1.Subscription);
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var RefCountObservable = function (_super) {
        __extends(RefCountObservable, _super);
        function RefCountObservable(connectable, refCount) {
            if (refCount === void 0) {
                refCount = 0;
            }
            _super.call(this);
            this.connectable = connectable;
            this.refCount = refCount;
        }
        RefCountObservable.prototype._subscribe = function (subscriber) {
            var connectable = this.connectable;
            var refCountSubscriber = new RefCountSubscriber(subscriber, this);
            var subscription = connectable.subscribe(refCountSubscriber);
            if (!subscription.isUnsubscribed && ++this.refCount === 1) {
                refCountSubscriber.connection = this.connection = connectable.connect();
            }
            return subscription;
        };
        return RefCountObservable;
    }(Observable_1.Observable);
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var RefCountSubscriber = function (_super) {
        __extends(RefCountSubscriber, _super);
        function RefCountSubscriber(destination, refCountObservable) {
            _super.call(this, null);
            this.destination = destination;
            this.refCountObservable = refCountObservable;
            this.connection = refCountObservable.connection;
            destination.add(this);
        }
        RefCountSubscriber.prototype._next = function (value) {
            this.destination.next(value);
        };
        RefCountSubscriber.prototype._error = function (err) {
            this._resetConnectable();
            this.destination.error(err);
        };
        RefCountSubscriber.prototype._complete = function () {
            this._resetConnectable();
            this.destination.complete();
        };
        RefCountSubscriber.prototype._resetConnectable = function () {
            var observable = this.refCountObservable;
            var obsConnection = observable.connection;
            var subConnection = this.connection;
            if (subConnection && subConnection === obsConnection) {
                observable.refCount = 0;
                obsConnection.unsubscribe();
                observable.connection = null;
                this.unsubscribe();
            }
        };
        RefCountSubscriber.prototype._unsubscribe = function () {
            var observable = this.refCountObservable;
            if (observable.refCount === 0) {
                return;
            }
            if (--observable.refCount === 0) {
                var obsConnection = observable.connection;
                var subConnection = this.connection;
                if (subConnection && subConnection === obsConnection) {
                    obsConnection.unsubscribe();
                    observable.connection = null;
                }
            }
        };
        return RefCountSubscriber;
    }(Subscriber_1.Subscriber);
    
});
System.registerDynamic("node_modules/rxjs/Observer.js", [], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    exports.empty = {
        isUnsubscribed: true,
        next: function (value) {},
        error: function (err) {
            throw err;
        },
        complete: function () {}
    };
    
});
System.registerDynamic('node_modules/rxjs/Subscriber.js', ['./util/isFunction', './Subscription', './symbol/rxSubscriber', './Observer'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var isFunction_1 = $__require('./util/isFunction');
    var Subscription_1 = $__require('./Subscription');
    var rxSubscriber_1 = $__require('./symbol/rxSubscriber');
    var Observer_1 = $__require('./Observer');
    /**
     * Implements the {@link Observer} interface and extends the
     * {@link Subscription} class. While the {@link Observer} is the public API for
     * consuming the values of an {@link Observable}, all Observers get converted to
     * a Subscriber, in order to provide Subscription-like capabilities such as
     * `unsubscribe`. Subscriber is a common type in RxJS, and crucial for
     * implementing operators, but it is rarely used as a public API.
     *
     * @class Subscriber<T>
     */
    var Subscriber = function (_super) {
        __extends(Subscriber, _super);
        /**
         * @param {Observer|function(value: T): void} [destinationOrNext] A partially
         * defined Observer or a `next` callback function.
         * @param {function(e: ?any): void} [error] The `error` callback of an
         * Observer.
         * @param {function(): void} [complete] The `complete` callback of an
         * Observer.
         */
        function Subscriber(destinationOrNext, error, complete) {
            _super.call(this);
            this.syncErrorValue = null;
            this.syncErrorThrown = false;
            this.syncErrorThrowable = false;
            this.isStopped = false;
            switch (arguments.length) {
                case 0:
                    this.destination = Observer_1.empty;
                    break;
                case 1:
                    if (!destinationOrNext) {
                        this.destination = Observer_1.empty;
                        break;
                    }
                    if (typeof destinationOrNext === 'object') {
                        if (destinationOrNext instanceof Subscriber) {
                            this.destination = destinationOrNext;
                            this.destination.add(this);
                        } else {
                            this.syncErrorThrowable = true;
                            this.destination = new SafeSubscriber(this, destinationOrNext);
                        }
                        break;
                    }
                default:
                    this.syncErrorThrowable = true;
                    this.destination = new SafeSubscriber(this, destinationOrNext, error, complete);
                    break;
            }
        }
        /**
         * A static factory for a Subscriber, given a (potentially partial) definition
         * of an Observer.
         * @param {function(x: ?T): void} [next] The `next` callback of an Observer.
         * @param {function(e: ?any): void} [error] The `error` callback of an
         * Observer.
         * @param {function(): void} [complete] The `complete` callback of an
         * Observer.
         * @return {Subscriber<T>} A Subscriber wrapping the (partially defined)
         * Observer represented by the given arguments.
         */
        Subscriber.create = function (next, error, complete) {
            var subscriber = new Subscriber(next, error, complete);
            subscriber.syncErrorThrowable = false;
            return subscriber;
        };
        /**
         * The {@link Observer} callback to receive notifications of type `next` from
         * the Observable, with a value. The Observable may call this method 0 or more
         * times.
         * @param {T} [value] The `next` value.
         * @return {void}
         */
        Subscriber.prototype.next = function (value) {
            if (!this.isStopped) {
                this._next(value);
            }
        };
        /**
         * The {@link Observer} callback to receive notifications of type `error` from
         * the Observable, with an attached {@link Error}. Notifies the Observer that
         * the Observable has experienced an error condition.
         * @param {any} [err] The `error` exception.
         * @return {void}
         */
        Subscriber.prototype.error = function (err) {
            if (!this.isStopped) {
                this.isStopped = true;
                this._error(err);
            }
        };
        /**
         * The {@link Observer} callback to receive a valueless notification of type
         * `complete` from the Observable. Notifies the Observer that the Observable
         * has finished sending push-based notifications.
         * @return {void}
         */
        Subscriber.prototype.complete = function () {
            if (!this.isStopped) {
                this.isStopped = true;
                this._complete();
            }
        };
        Subscriber.prototype.unsubscribe = function () {
            if (this.isUnsubscribed) {
                return;
            }
            this.isStopped = true;
            _super.prototype.unsubscribe.call(this);
        };
        Subscriber.prototype._next = function (value) {
            this.destination.next(value);
        };
        Subscriber.prototype._error = function (err) {
            this.destination.error(err);
            this.unsubscribe();
        };
        Subscriber.prototype._complete = function () {
            this.destination.complete();
            this.unsubscribe();
        };
        Subscriber.prototype[rxSubscriber_1.$$rxSubscriber] = function () {
            return this;
        };
        return Subscriber;
    }(Subscription_1.Subscription);
    exports.Subscriber = Subscriber;
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var SafeSubscriber = function (_super) {
        __extends(SafeSubscriber, _super);
        function SafeSubscriber(_parent, observerOrNext, error, complete) {
            _super.call(this);
            this._parent = _parent;
            var next;
            var context = this;
            if (isFunction_1.isFunction(observerOrNext)) {
                next = observerOrNext;
            } else if (observerOrNext) {
                context = observerOrNext;
                next = observerOrNext.next;
                error = observerOrNext.error;
                complete = observerOrNext.complete;
                if (isFunction_1.isFunction(context.unsubscribe)) {
                    this.add(context.unsubscribe.bind(context));
                }
                context.unsubscribe = this.unsubscribe.bind(this);
            }
            this._context = context;
            this._next = next;
            this._error = error;
            this._complete = complete;
        }
        SafeSubscriber.prototype.next = function (value) {
            if (!this.isStopped && this._next) {
                var _parent = this._parent;
                if (!_parent.syncErrorThrowable) {
                    this.__tryOrUnsub(this._next, value);
                } else if (this.__tryOrSetError(_parent, this._next, value)) {
                    this.unsubscribe();
                }
            }
        };
        SafeSubscriber.prototype.error = function (err) {
            if (!this.isStopped) {
                var _parent = this._parent;
                if (this._error) {
                    if (!_parent.syncErrorThrowable) {
                        this.__tryOrUnsub(this._error, err);
                        this.unsubscribe();
                    } else {
                        this.__tryOrSetError(_parent, this._error, err);
                        this.unsubscribe();
                    }
                } else if (!_parent.syncErrorThrowable) {
                    this.unsubscribe();
                    throw err;
                } else {
                    _parent.syncErrorValue = err;
                    _parent.syncErrorThrown = true;
                    this.unsubscribe();
                }
            }
        };
        SafeSubscriber.prototype.complete = function () {
            if (!this.isStopped) {
                var _parent = this._parent;
                if (this._complete) {
                    if (!_parent.syncErrorThrowable) {
                        this.__tryOrUnsub(this._complete);
                        this.unsubscribe();
                    } else {
                        this.__tryOrSetError(_parent, this._complete);
                        this.unsubscribe();
                    }
                } else {
                    this.unsubscribe();
                }
            }
        };
        SafeSubscriber.prototype.__tryOrUnsub = function (fn, value) {
            try {
                fn.call(this._context, value);
            } catch (err) {
                this.unsubscribe();
                throw err;
            }
        };
        SafeSubscriber.prototype.__tryOrSetError = function (parent, fn, value) {
            try {
                fn.call(this._context, value);
            } catch (err) {
                parent.syncErrorValue = err;
                parent.syncErrorThrown = true;
                return true;
            }
            return false;
        };
        SafeSubscriber.prototype._unsubscribe = function () {
            var _parent = this._parent;
            this._context = null;
            this._parent = null;
            _parent.unsubscribe();
        };
        return SafeSubscriber;
    }(Subscriber);
    
});
System.registerDynamic('node_modules/rxjs/util/toSubscriber.js', ['../Subscriber', '../symbol/rxSubscriber'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var Subscriber_1 = $__require('../Subscriber');
    var rxSubscriber_1 = $__require('../symbol/rxSubscriber');
    function toSubscriber(nextOrObserver, error, complete) {
        if (nextOrObserver && typeof nextOrObserver === 'object') {
            if (nextOrObserver instanceof Subscriber_1.Subscriber) {
                return nextOrObserver;
            } else if (typeof nextOrObserver[rxSubscriber_1.$$rxSubscriber] === 'function') {
                return nextOrObserver[rxSubscriber_1.$$rxSubscriber]();
            }
        }
        return new Subscriber_1.Subscriber(nextOrObserver, error, complete);
    }
    exports.toSubscriber = toSubscriber;
    
});
System.registerDynamic('node_modules/rxjs/Observable.js', ['./util/root', './symbol/observable', './util/toSubscriber'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var root_1 = $__require('./util/root');
    var observable_1 = $__require('./symbol/observable');
    var toSubscriber_1 = $__require('./util/toSubscriber');
    /**
     * A representation of any set of values over any amount of time. This the most basic building block
     * of RxJS.
     *
     * @class Observable<T>
     */
    var Observable = function () {
        /**
         * @constructor
         * @param {Function} subscribe the function that is  called when the Observable is
         * initially subscribed to. This function is given a Subscriber, to which new values
         * can be `next`ed, or an `error` method can be called to raise an error, or
         * `complete` can be called to notify of a successful completion.
         */
        function Observable(subscribe) {
            this._isScalar = false;
            if (subscribe) {
                this._subscribe = subscribe;
            }
        }
        /**
         * Creates a new Observable, with this Observable as the source, and the passed
         * operator defined as the new observable's operator.
         * @method lift
         * @param {Operator} operator the operator defining the operation to take on the observable
         * @return {Observable} a new observable with the Operator applied
         */
        Observable.prototype.lift = function (operator) {
            var observable = new Observable();
            observable.source = this;
            observable.operator = operator;
            return observable;
        };
        /**
         * Registers handlers for handling emitted values, error and completions from the observable, and
         *  executes the observable's subscriber function, which will take action to set up the underlying data stream
         * @method subscribe
         * @param {PartialObserver|Function} observerOrNext (optional) either an observer defining all functions to be called,
         *  or the first of three possible handlers, which is the handler for each value emitted from the observable.
         * @param {Function} error (optional) a handler for a terminal event resulting from an error. If no error handler is provided,
         *  the error will be thrown as unhandled
         * @param {Function} complete (optional) a handler for a terminal event resulting from successful completion.
         * @return {ISubscription} a subscription reference to the registered handlers
         */
        Observable.prototype.subscribe = function (observerOrNext, error, complete) {
            var operator = this.operator;
            var sink = toSubscriber_1.toSubscriber(observerOrNext, error, complete);
            sink.add(operator ? operator.call(sink, this) : this._subscribe(sink));
            if (sink.syncErrorThrowable) {
                sink.syncErrorThrowable = false;
                if (sink.syncErrorThrown) {
                    throw sink.syncErrorValue;
                }
            }
            return sink;
        };
        /**
         * @method forEach
         * @param {Function} next a handler for each value emitted by the observable
         * @param {PromiseConstructor} [PromiseCtor] a constructor function used to instantiate the Promise
         * @return {Promise} a promise that either resolves on observable completion or
         *  rejects with the handled error
         */
        Observable.prototype.forEach = function (next, PromiseCtor) {
            var _this = this;
            if (!PromiseCtor) {
                if (root_1.root.Rx && root_1.root.Rx.config && root_1.root.Rx.config.Promise) {
                    PromiseCtor = root_1.root.Rx.config.Promise;
                } else if (root_1.root.Promise) {
                    PromiseCtor = root_1.root.Promise;
                }
            }
            if (!PromiseCtor) {
                throw new Error('no Promise impl found');
            }
            return new PromiseCtor(function (resolve, reject) {
                var subscription = _this.subscribe(function (value) {
                    if (subscription) {
                        // if there is a subscription, then we can surmise
                        // the next handling is asynchronous. Any errors thrown
                        // need to be rejected explicitly and unsubscribe must be
                        // called manually
                        try {
                            next(value);
                        } catch (err) {
                            reject(err);
                            subscription.unsubscribe();
                        }
                    } else {
                        // if there is NO subscription, then we're getting a nexted
                        // value synchronously during subscription. We can just call it.
                        // If it errors, Observable's `subscribe` imple will ensure the
                        // unsubscription logic is called, then synchronously rethrow the error.
                        // After that, Promise will trap the error and send it
                        // down the rejection path.
                        next(value);
                    }
                }, reject, resolve);
            });
        };
        Observable.prototype._subscribe = function (subscriber) {
            return this.source.subscribe(subscriber);
        };
        /**
         * An interop point defined by the es7-observable spec https://github.com/zenparsing/es-observable
         * @method Symbol.observable
         * @return {Observable} this instance of the observable
         */
        Observable.prototype[observable_1.$$observable] = function () {
            return this;
        };
        // HACK: Since TypeScript inherits static properties too, we have to
        // fight against TypeScript here so Subject can have a different static create signature
        /**
         * Creates a new cold Observable by calling the Observable constructor
         * @static true
         * @owner Observable
         * @method create
         * @param {Function} subscribe? the subscriber function to be passed to the Observable constructor
         * @return {Observable} a new cold observable
         */
        Observable.create = function (subscribe) {
            return new Observable(subscribe);
        };
        return Observable;
    }();
    exports.Observable = Observable;
    
});
System.registerDynamic('node_modules/rxjs/Notification.js', ['./Observable'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var Observable_1 = $__require('./Observable');
    /**
     * Represents a push-based event or value that an {@link Observable} can emit.
     * This class is particularly useful for operators that manage notifications,
     * like {@link materialize}, {@link dematerialize}, {@link observeOn}, and
     * others. Besides wrapping the actual delivered value, it also annotates it
     * with metadata of, for instance, what type of push message it is (`next`,
     * `error`, or `complete`).
     *
     * @see {@link materialize}
     * @see {@link dematerialize}
     * @see {@link observeOn}
     *
     * @class Notification<T>
     */
    var Notification = function () {
        function Notification(kind, value, exception) {
            this.kind = kind;
            this.value = value;
            this.exception = exception;
            this.hasValue = kind === 'N';
        }
        /**
         * Delivers to the given `observer` the value wrapped by this Notification.
         * @param {Observer} observer
         * @return
         */
        Notification.prototype.observe = function (observer) {
            switch (this.kind) {
                case 'N':
                    return observer.next && observer.next(this.value);
                case 'E':
                    return observer.error && observer.error(this.exception);
                case 'C':
                    return observer.complete && observer.complete();
            }
        };
        /**
         * Given some {@link Observer} callbacks, deliver the value represented by the
         * current Notification to the correctly corresponding callback.
         * @param {function(value: T): void} next An Observer `next` callback.
         * @param {function(err: any): void} [error] An Observer `error` callback.
         * @param {function(): void} [complete] An Observer `complete` callback.
         * @return {any}
         */
        Notification.prototype.do = function (next, error, complete) {
            var kind = this.kind;
            switch (kind) {
                case 'N':
                    return next && next(this.value);
                case 'E':
                    return error && error(this.exception);
                case 'C':
                    return complete && complete();
            }
        };
        /**
         * Takes an Observer or its individual callback functions, and calls `observe`
         * or `do` methods accordingly.
         * @param {Observer|function(value: T): void} nextOrObserver An Observer or
         * the `next` callback.
         * @param {function(err: any): void} [error] An Observer `error` callback.
         * @param {function(): void} [complete] An Observer `complete` callback.
         * @return {any}
         */
        Notification.prototype.accept = function (nextOrObserver, error, complete) {
            if (nextOrObserver && typeof nextOrObserver.next === 'function') {
                return this.observe(nextOrObserver);
            } else {
                return this.do(nextOrObserver, error, complete);
            }
        };
        /**
         * Returns a simple Observable that just delivers the notification represented
         * by this Notification instance.
         * @return {any}
         */
        Notification.prototype.toObservable = function () {
            var kind = this.kind;
            switch (kind) {
                case 'N':
                    return Observable_1.Observable.of(this.value);
                case 'E':
                    return Observable_1.Observable.throw(this.exception);
                case 'C':
                    return Observable_1.Observable.empty();
            }
        };
        /**
         * A shortcut to create a Notification instance of the type `next` from a
         * given value.
         * @param {T} value The `next` value.
         * @return {Notification<T>} The "next" Notification representing the
         * argument.
         */
        Notification.createNext = function (value) {
            if (typeof value !== 'undefined') {
                return new Notification('N', value);
            }
            return this.undefinedValueNotification;
        };
        /**
         * A shortcut to create a Notification instance of the type `error` from a
         * given error.
         * @param {any} [err] The `error` exception.
         * @return {Notification<T>} The "error" Notification representing the
         * argument.
         */
        Notification.createError = function (err) {
            return new Notification('E', undefined, err);
        };
        /**
         * A shortcut to create a Notification instance of the type `complete`.
         * @return {Notification<any>} The valueless "complete" Notification.
         */
        Notification.createComplete = function () {
            return this.completeNotification;
        };
        Notification.completeNotification = new Notification('C');
        Notification.undefinedValueNotification = new Notification('N', undefined);
        return Notification;
    }();
    exports.Notification = Notification;
    
});
System.registerDynamic('node_modules/rxjs/util/EmptyError.js', [], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    /**
     * An error thrown when an Observable or a sequence was queried but has no
     * elements.
     *
     * @see {@link first}
     * @see {@link last}
     * @see {@link single}
     *
     * @class EmptyError
     */
    var EmptyError = function (_super) {
        __extends(EmptyError, _super);
        function EmptyError() {
            _super.call(this, 'no elements in sequence');
            this.name = 'EmptyError';
        }
        return EmptyError;
    }(Error);
    exports.EmptyError = EmptyError;
    
});
System.registerDynamic('node_modules/rxjs/util/ArgumentOutOfRangeError.js', [], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    /**
     * An error thrown when an element was queried at a certain index of an
     * Observable, but no such index or position exists in that sequence.
     *
     * @see {@link elementAt}
     * @see {@link take}
     * @see {@link takeLast}
     *
     * @class ArgumentOutOfRangeError
     */
    var ArgumentOutOfRangeError = function (_super) {
        __extends(ArgumentOutOfRangeError, _super);
        function ArgumentOutOfRangeError() {
            _super.call(this, 'argument out of range');
            this.name = 'ArgumentOutOfRangeError';
        }
        return ArgumentOutOfRangeError;
    }(Error);
    exports.ArgumentOutOfRangeError = ArgumentOutOfRangeError;
    
});
System.registerDynamic('node_modules/rxjs/util/ObjectUnsubscribedError.js', [], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    /**
     * An error thrown when an action is invalid because the object has been
     * unsubscribed.
     *
     * @see {@link Subject}
     * @see {@link BehaviorSubject}
     *
     * @class ObjectUnsubscribedError
     */
    var ObjectUnsubscribedError = function (_super) {
        __extends(ObjectUnsubscribedError, _super);
        function ObjectUnsubscribedError() {
            _super.call(this, 'object unsubscribed');
            this.name = 'ObjectUnsubscribedError';
        }
        return ObjectUnsubscribedError;
    }(Error);
    exports.ObjectUnsubscribedError = ObjectUnsubscribedError;
    
});
System.registerDynamic('node_modules/rxjs/util/Immediate.js', ['./root'], true, function ($__require, exports, module) {
    /**
    Some credit for this helper goes to http://github.com/YuzuJS/setImmediate
    */
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var root_1 = $__require('./root');
    var ImmediateDefinition = function () {
        function ImmediateDefinition(root) {
            this.root = root;
            if (root.setImmediate && typeof root.setImmediate === 'function') {
                this.setImmediate = root.setImmediate.bind(root);
                this.clearImmediate = root.clearImmediate.bind(root);
            } else {
                this.nextHandle = 1;
                this.tasksByHandle = {};
                this.currentlyRunningATask = false;
                // Don't get fooled by e.g. browserify environments.
                if (this.canUseProcessNextTick()) {
                    // For Node.js before 0.9
                    this.setImmediate = this.createProcessNextTickSetImmediate();
                } else if (this.canUsePostMessage()) {
                    // For non-IE10 modern browsers
                    this.setImmediate = this.createPostMessageSetImmediate();
                } else if (this.canUseMessageChannel()) {
                    // For web workers, where supported
                    this.setImmediate = this.createMessageChannelSetImmediate();
                } else if (this.canUseReadyStateChange()) {
                    // For IE 6–8
                    this.setImmediate = this.createReadyStateChangeSetImmediate();
                } else {
                    // For older browsers
                    this.setImmediate = this.createSetTimeoutSetImmediate();
                }
                var ci = function clearImmediate(handle) {
                    delete clearImmediate.instance.tasksByHandle[handle];
                };
                ci.instance = this;
                this.clearImmediate = ci;
            }
        }
        ImmediateDefinition.prototype.identify = function (o) {
            return this.root.Object.prototype.toString.call(o);
        };
        ImmediateDefinition.prototype.canUseProcessNextTick = function () {
            return this.identify(this.root.process) === '[object process]';
        };
        ImmediateDefinition.prototype.canUseMessageChannel = function () {
            return Boolean(this.root.MessageChannel);
        };
        ImmediateDefinition.prototype.canUseReadyStateChange = function () {
            var document = this.root.document;
            return Boolean(document && 'onreadystatechange' in document.createElement('script'));
        };
        ImmediateDefinition.prototype.canUsePostMessage = function () {
            var root = this.root;
            // The test against `importScripts` prevents this implementation from being installed inside a web worker,
            // where `root.postMessage` means something completely different and can't be used for this purpose.
            if (root.postMessage && !root.importScripts) {
                var postMessageIsAsynchronous_1 = true;
                var oldOnMessage = root.onmessage;
                root.onmessage = function () {
                    postMessageIsAsynchronous_1 = false;
                };
                root.postMessage('', '*');
                root.onmessage = oldOnMessage;
                return postMessageIsAsynchronous_1;
            }
            return false;
        };
        // This function accepts the same arguments as setImmediate, but
        // returns a function that requires no arguments.
        ImmediateDefinition.prototype.partiallyApplied = function (handler) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            var fn = function result() {
                var _a = result,
                    handler = _a.handler,
                    args = _a.args;
                if (typeof handler === 'function') {
                    handler.apply(undefined, args);
                } else {
                    new Function('' + handler)();
                }
            };
            fn.handler = handler;
            fn.args = args;
            return fn;
        };
        ImmediateDefinition.prototype.addFromSetImmediateArguments = function (args) {
            this.tasksByHandle[this.nextHandle] = this.partiallyApplied.apply(undefined, args);
            return this.nextHandle++;
        };
        ImmediateDefinition.prototype.createProcessNextTickSetImmediate = function () {
            var fn = function setImmediate() {
                var instance = setImmediate.instance;
                var handle = instance.addFromSetImmediateArguments(arguments);
                instance.root.process.nextTick(instance.partiallyApplied(instance.runIfPresent, handle));
                return handle;
            };
            fn.instance = this;
            return fn;
        };
        ImmediateDefinition.prototype.createPostMessageSetImmediate = function () {
            // Installs an event handler on `global` for the `message` event: see
            // * https://developer.mozilla.org/en/DOM/window.postMessage
            // * http://www.whatwg.org/specs/web-apps/current-work/multipage/comms.html#crossDocumentMessages
            var root = this.root;
            var messagePrefix = 'setImmediate$' + root.Math.random() + '$';
            var onGlobalMessage = function globalMessageHandler(event) {
                var instance = globalMessageHandler.instance;
                if (event.source === root && typeof event.data === 'string' && event.data.indexOf(messagePrefix) === 0) {
                    instance.runIfPresent(+event.data.slice(messagePrefix.length));
                }
            };
            onGlobalMessage.instance = this;
            root.addEventListener('message', onGlobalMessage, false);
            var fn = function setImmediate() {
                var _a = setImmediate,
                    messagePrefix = _a.messagePrefix,
                    instance = _a.instance;
                var handle = instance.addFromSetImmediateArguments(arguments);
                instance.root.postMessage(messagePrefix + handle, '*');
                return handle;
            };
            fn.instance = this;
            fn.messagePrefix = messagePrefix;
            return fn;
        };
        ImmediateDefinition.prototype.runIfPresent = function (handle) {
            // From the spec: 'Wait until any invocations of this algorithm started before this one have completed.'
            // So if we're currently running a task, we'll need to delay this invocation.
            if (this.currentlyRunningATask) {
                // Delay by doing a setTimeout. setImmediate was tried instead, but in Firefox 7 it generated a
                // 'too much recursion' error.
                this.root.setTimeout(this.partiallyApplied(this.runIfPresent, handle), 0);
            } else {
                var task = this.tasksByHandle[handle];
                if (task) {
                    this.currentlyRunningATask = true;
                    try {
                        task();
                    } finally {
                        this.clearImmediate(handle);
                        this.currentlyRunningATask = false;
                    }
                }
            }
        };
        ImmediateDefinition.prototype.createMessageChannelSetImmediate = function () {
            var _this = this;
            var channel = new this.root.MessageChannel();
            channel.port1.onmessage = function (event) {
                var handle = event.data;
                _this.runIfPresent(handle);
            };
            var fn = function setImmediate() {
                var _a = setImmediate,
                    channel = _a.channel,
                    instance = _a.instance;
                var handle = instance.addFromSetImmediateArguments(arguments);
                channel.port2.postMessage(handle);
                return handle;
            };
            fn.channel = channel;
            fn.instance = this;
            return fn;
        };
        ImmediateDefinition.prototype.createReadyStateChangeSetImmediate = function () {
            var fn = function setImmediate() {
                var instance = setImmediate.instance;
                var root = instance.root;
                var doc = root.document;
                var html = doc.documentElement;
                var handle = instance.addFromSetImmediateArguments(arguments);
                // Create a <script> element; its readystatechange event will be fired asynchronously once it is inserted
                // into the document. Do so, thus queuing up the task. Remember to clean up once it's been called.
                var script = doc.createElement('script');
                script.onreadystatechange = function () {
                    instance.runIfPresent(handle);
                    script.onreadystatechange = null;
                    html.removeChild(script);
                    script = null;
                };
                html.appendChild(script);
                return handle;
            };
            fn.instance = this;
            return fn;
        };
        ImmediateDefinition.prototype.createSetTimeoutSetImmediate = function () {
            var fn = function setImmediate() {
                var instance = setImmediate.instance;
                var handle = instance.addFromSetImmediateArguments(arguments);
                instance.root.setTimeout(instance.partiallyApplied(instance.runIfPresent, handle), 0);
                return handle;
            };
            fn.instance = this;
            return fn;
        };
        return ImmediateDefinition;
    }();
    exports.ImmediateDefinition = ImmediateDefinition;
    exports.Immediate = new ImmediateDefinition(root_1.root);
    
});
System.registerDynamic('node_modules/rxjs/scheduler/AsapAction.js', ['../util/Immediate', './FutureAction'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var Immediate_1 = $__require('../util/Immediate');
    var FutureAction_1 = $__require('./FutureAction');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var AsapAction = function (_super) {
        __extends(AsapAction, _super);
        function AsapAction() {
            _super.apply(this, arguments);
        }
        AsapAction.prototype._schedule = function (state, delay) {
            if (delay === void 0) {
                delay = 0;
            }
            if (delay > 0) {
                return _super.prototype._schedule.call(this, state, delay);
            }
            this.delay = delay;
            this.state = state;
            var scheduler = this.scheduler;
            scheduler.actions.push(this);
            if (!scheduler.scheduledId) {
                scheduler.scheduledId = Immediate_1.Immediate.setImmediate(function () {
                    scheduler.scheduledId = null;
                    scheduler.flush();
                });
            }
            return this;
        };
        AsapAction.prototype._unsubscribe = function () {
            var scheduler = this.scheduler;
            var scheduledId = scheduler.scheduledId,
                actions = scheduler.actions;
            _super.prototype._unsubscribe.call(this);
            if (actions.length === 0) {
                scheduler.active = false;
                if (scheduledId != null) {
                    scheduler.scheduledId = null;
                    Immediate_1.Immediate.clearImmediate(scheduledId);
                }
            }
        };
        return AsapAction;
    }(FutureAction_1.FutureAction);
    exports.AsapAction = AsapAction;
    
});
System.registerDynamic('node_modules/rxjs/scheduler/AsapScheduler.js', ['./AsapAction', './QueueScheduler'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var AsapAction_1 = $__require('./AsapAction');
    var QueueScheduler_1 = $__require('./QueueScheduler');
    var AsapScheduler = function (_super) {
        __extends(AsapScheduler, _super);
        function AsapScheduler() {
            _super.apply(this, arguments);
        }
        AsapScheduler.prototype.scheduleNow = function (work, state) {
            return new AsapAction_1.AsapAction(this, work).schedule(state);
        };
        return AsapScheduler;
    }(QueueScheduler_1.QueueScheduler);
    exports.AsapScheduler = AsapScheduler;
    
});
System.registerDynamic("node_modules/rxjs/scheduler/asap.js", ["./AsapScheduler"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var AsapScheduler_1 = $__require('./AsapScheduler');
  exports.asap = new AsapScheduler_1.AsapScheduler();
  
});
System.registerDynamic('node_modules/rxjs/scheduler/AsyncScheduler.js', ['./FutureAction', './QueueScheduler'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var FutureAction_1 = $__require('./FutureAction');
    var QueueScheduler_1 = $__require('./QueueScheduler');
    var AsyncScheduler = function (_super) {
        __extends(AsyncScheduler, _super);
        function AsyncScheduler() {
            _super.apply(this, arguments);
        }
        AsyncScheduler.prototype.scheduleNow = function (work, state) {
            return new FutureAction_1.FutureAction(this, work).schedule(state, 0);
        };
        return AsyncScheduler;
    }(QueueScheduler_1.QueueScheduler);
    exports.AsyncScheduler = AsyncScheduler;
    
});
System.registerDynamic("node_modules/rxjs/scheduler/async.js", ["./AsyncScheduler"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var AsyncScheduler_1 = $__require('./AsyncScheduler');
  exports.async = new AsyncScheduler_1.AsyncScheduler();
  
});
System.registerDynamic("node_modules/rxjs/scheduler/QueueAction.js", ["./FutureAction"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var FutureAction_1 = $__require('./FutureAction');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var QueueAction = function (_super) {
        __extends(QueueAction, _super);
        function QueueAction() {
            _super.apply(this, arguments);
        }
        QueueAction.prototype._schedule = function (state, delay) {
            if (delay === void 0) {
                delay = 0;
            }
            if (delay > 0) {
                return _super.prototype._schedule.call(this, state, delay);
            }
            this.delay = delay;
            this.state = state;
            var scheduler = this.scheduler;
            scheduler.actions.push(this);
            scheduler.flush();
            return this;
        };
        return QueueAction;
    }(FutureAction_1.FutureAction);
    exports.QueueAction = QueueAction;
    
});
System.registerDynamic("node_modules/rxjs/util/isArray.js", [], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  exports.isArray = Array.isArray || function (x) {
    return x && typeof x.length === 'number';
  };
  
});
System.registerDynamic("node_modules/rxjs/util/isObject.js", [], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    function isObject(x) {
        return x != null && typeof x === 'object';
    }
    exports.isObject = isObject;
    
});
System.registerDynamic("node_modules/rxjs/util/isFunction.js", [], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    function isFunction(x) {
        return typeof x === 'function';
    }
    exports.isFunction = isFunction;
    
});
System.registerDynamic("node_modules/rxjs/util/tryCatch.js", ["./errorObject"], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var errorObject_1 = $__require('./errorObject');
    var tryCatchTarget;
    function tryCatcher() {
        try {
            return tryCatchTarget.apply(this, arguments);
        } catch (e) {
            errorObject_1.errorObject.e = e;
            return errorObject_1.errorObject;
        }
    }
    function tryCatch(fn) {
        tryCatchTarget = fn;
        return tryCatcher;
    }
    exports.tryCatch = tryCatch;
    ;
    
});
System.registerDynamic("node_modules/rxjs/util/errorObject.js", [], true, function ($__require, exports, module) {
  "use strict";
  // typeof any so that it we don't have to cast when comparing a result to the error object

  var global = this || self,
      GLOBAL = global;
  exports.errorObject = { e: {} };
  
});
System.registerDynamic("node_modules/rxjs/util/UnsubscriptionError.js", [], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    /**
     * An error thrown when one or more errors have occurred during the
     * `unsubscribe` of a {@link Subscription}.
     */
    var UnsubscriptionError = function (_super) {
        __extends(UnsubscriptionError, _super);
        function UnsubscriptionError(errors) {
            _super.call(this);
            this.errors = errors;
            this.name = 'UnsubscriptionError';
            this.message = errors ? errors.length + " errors occurred during unsubscription:\n" + errors.map(function (err, i) {
                return i + 1 + ") " + err.toString();
            }).join('\n') : '';
        }
        return UnsubscriptionError;
    }(Error);
    exports.UnsubscriptionError = UnsubscriptionError;
    
});
System.registerDynamic('node_modules/rxjs/Subscription.js', ['./util/isArray', './util/isObject', './util/isFunction', './util/tryCatch', './util/errorObject', './util/UnsubscriptionError'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var isArray_1 = $__require('./util/isArray');
    var isObject_1 = $__require('./util/isObject');
    var isFunction_1 = $__require('./util/isFunction');
    var tryCatch_1 = $__require('./util/tryCatch');
    var errorObject_1 = $__require('./util/errorObject');
    var UnsubscriptionError_1 = $__require('./util/UnsubscriptionError');
    /**
     * Represents a disposable resource, such as the execution of an Observable. A
     * Subscription has one important method, `unsubscribe`, that takes no argument
     * and just disposes the resource held by the subscription.
     *
     * Additionally, subscriptions may be grouped together through the `add()`
     * method, which will attach a child Subscription to the current Subscription.
     * When a Subscription is unsubscribed, all its children (and its grandchildren)
     * will be unsubscribed as well.
     *
     * @class Subscription
     */
    var Subscription = function () {
        /**
         * @param {function(): void} [unsubscribe] A function describing how to
         * perform the disposal of resources when the `unsubscribe` method is called.
         */
        function Subscription(unsubscribe) {
            /**
             * A flag to indicate whether this Subscription has already been unsubscribed.
             * @type {boolean}
             */
            this.isUnsubscribed = false;
            if (unsubscribe) {
                this._unsubscribe = unsubscribe;
            }
        }
        /**
         * Disposes the resources held by the subscription. May, for instance, cancel
         * an ongoing Observable execution or cancel any other type of work that
         * started when the Subscription was created.
         * @return {void}
         */
        Subscription.prototype.unsubscribe = function () {
            var hasErrors = false;
            var errors;
            if (this.isUnsubscribed) {
                return;
            }
            this.isUnsubscribed = true;
            var _a = this,
                _unsubscribe = _a._unsubscribe,
                _subscriptions = _a._subscriptions;
            this._subscriptions = null;
            if (isFunction_1.isFunction(_unsubscribe)) {
                var trial = tryCatch_1.tryCatch(_unsubscribe).call(this);
                if (trial === errorObject_1.errorObject) {
                    hasErrors = true;
                    (errors = errors || []).push(errorObject_1.errorObject.e);
                }
            }
            if (isArray_1.isArray(_subscriptions)) {
                var index = -1;
                var len = _subscriptions.length;
                while (++index < len) {
                    var sub = _subscriptions[index];
                    if (isObject_1.isObject(sub)) {
                        var trial = tryCatch_1.tryCatch(sub.unsubscribe).call(sub);
                        if (trial === errorObject_1.errorObject) {
                            hasErrors = true;
                            errors = errors || [];
                            var err = errorObject_1.errorObject.e;
                            if (err instanceof UnsubscriptionError_1.UnsubscriptionError) {
                                errors = errors.concat(err.errors);
                            } else {
                                errors.push(err);
                            }
                        }
                    }
                }
            }
            if (hasErrors) {
                throw new UnsubscriptionError_1.UnsubscriptionError(errors);
            }
        };
        /**
         * Adds a tear down to be called during the unsubscribe() of this
         * Subscription.
         *
         * If the tear down being added is a subscription that is already
         * unsubscribed, is the same reference `add` is being called on, or is
         * `Subscription.EMPTY`, it will not be added.
         *
         * If this subscription is already in an `isUnsubscribed` state, the passed
         * tear down logic will be executed immediately.
         *
         * @param {TeardownLogic} teardown The additional logic to execute on
         * teardown.
         * @return {Subscription} Returns the Subscription used or created to be
         * added to the inner subscriptions list. This Subscription can be used with
         * `remove()` to remove the passed teardown logic from the inner subscriptions
         * list.
         */
        Subscription.prototype.add = function (teardown) {
            if (!teardown || teardown === this || teardown === Subscription.EMPTY) {
                return;
            }
            var sub = teardown;
            switch (typeof teardown) {
                case 'function':
                    sub = new Subscription(teardown);
                case 'object':
                    if (sub.isUnsubscribed || typeof sub.unsubscribe !== 'function') {
                        break;
                    } else if (this.isUnsubscribed) {
                        sub.unsubscribe();
                    } else {
                        (this._subscriptions || (this._subscriptions = [])).push(sub);
                    }
                    break;
                default:
                    throw new Error('Unrecognized teardown ' + teardown + ' added to Subscription.');
            }
            return sub;
        };
        /**
         * Removes a Subscription from the internal list of subscriptions that will
         * unsubscribe during the unsubscribe process of this Subscription.
         * @param {Subscription} subscription The subscription to remove.
         * @return {void}
         */
        Subscription.prototype.remove = function (subscription) {
            // HACK: This might be redundant because of the logic in `add()`
            if (subscription == null || subscription === this || subscription === Subscription.EMPTY) {
                return;
            }
            var subscriptions = this._subscriptions;
            if (subscriptions) {
                var subscriptionIndex = subscriptions.indexOf(subscription);
                if (subscriptionIndex !== -1) {
                    subscriptions.splice(subscriptionIndex, 1);
                }
            }
        };
        Subscription.EMPTY = function (empty) {
            empty.isUnsubscribed = true;
            return empty;
        }(new Subscription());
        return Subscription;
    }();
    exports.Subscription = Subscription;
    
});
System.registerDynamic('node_modules/rxjs/scheduler/FutureAction.js', ['../util/root', '../Subscription'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var __extends = this && this.__extends || function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() {
            this.constructor = d;
        }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
    var root_1 = $__require('../util/root');
    var Subscription_1 = $__require('../Subscription');
    /**
     * We need this JSDoc comment for affecting ESDoc.
     * @ignore
     * @extends {Ignored}
     */
    var FutureAction = function (_super) {
        __extends(FutureAction, _super);
        function FutureAction(scheduler, work) {
            _super.call(this);
            this.scheduler = scheduler;
            this.work = work;
            this.pending = false;
        }
        FutureAction.prototype.execute = function () {
            if (this.isUnsubscribed) {
                this.error = new Error('executing a cancelled action');
            } else {
                try {
                    this.work(this.state);
                } catch (e) {
                    this.unsubscribe();
                    this.error = e;
                }
            }
        };
        FutureAction.prototype.schedule = function (state, delay) {
            if (delay === void 0) {
                delay = 0;
            }
            if (this.isUnsubscribed) {
                return this;
            }
            return this._schedule(state, delay);
        };
        FutureAction.prototype._schedule = function (state, delay) {
            var _this = this;
            if (delay === void 0) {
                delay = 0;
            }
            // Always replace the current state with the new state.
            this.state = state;
            // Set the pending flag indicating that this action has been scheduled, or
            // has recursively rescheduled itself.
            this.pending = true;
            var id = this.id;
            // If this action has an intervalID and the specified delay matches the
            // delay we used to create the intervalID, don't call `setInterval` again.
            if (id != null && this.delay === delay) {
                return this;
            }
            this.delay = delay;
            // If this action has an intervalID, but was rescheduled with a different
            // `delay` time, cancel the current intervalID and call `setInterval` with
            // the new `delay` time.
            if (id != null) {
                this.id = null;
                root_1.root.clearInterval(id);
            }
            //
            // Important implementation note:
            //
            // By default, FutureAction only executes once. However, Actions have the
            // ability to be rescheduled from within the scheduled callback (mimicking
            // recursion for asynchronous methods). This allows us to implement single
            // and repeated actions with the same code path without adding API surface
            // area, and implement tail-call optimization over asynchronous boundaries.
            //
            // However, JS runtimes make a distinction between intervals scheduled by
            // repeatedly calling `setTimeout` vs. a single `setInterval` call, with
            // the latter providing a better guarantee of precision.
            //
            // In order to accommodate both single and repeatedly rescheduled actions,
            // use `setInterval` here for both cases. By default, the interval will be
            // canceled after its first execution, or if the action schedules itself to
            // run again with a different `delay` time.
            //
            // If the action recursively schedules itself to run again with the same
            // `delay` time, the interval is not canceled, but allowed to loop again.
            // The check of whether the interval should be canceled or not is run every
            // time the interval is executed. The first time an action fails to
            // reschedule itself, the interval is canceled.
            //
            this.id = root_1.root.setInterval(function () {
                _this.pending = false;
                var _a = _this,
                    id = _a.id,
                    scheduler = _a.scheduler;
                scheduler.actions.push(_this);
                scheduler.flush();
                //
                // Terminate this interval if the action didn't reschedule itself.
                // Don't call `this.unsubscribe()` here, because the action could be
                // rescheduled later. For example:
                //
                // ```
                // scheduler.schedule(function doWork(counter) {
                //   /* ... I'm a busy worker bee ... */
                //   var originalAction = this;
                //   /* wait 100ms before rescheduling this action again */
                //   setTimeout(function () {
                //     originalAction.schedule(counter + 1);
                //   }, 100);
                // }, 1000);
                // ```
                if (_this.pending === false && id != null) {
                    _this.id = null;
                    root_1.root.clearInterval(id);
                }
            }, delay);
            return this;
        };
        FutureAction.prototype._unsubscribe = function () {
            this.pending = false;
            var _a = this,
                id = _a.id,
                scheduler = _a.scheduler;
            var actions = scheduler.actions;
            var index = actions.indexOf(this);
            if (id != null) {
                this.id = null;
                root_1.root.clearInterval(id);
            }
            if (index !== -1) {
                actions.splice(index, 1);
            }
            this.work = null;
            this.state = null;
            this.scheduler = null;
        };
        return FutureAction;
    }(Subscription_1.Subscription);
    exports.FutureAction = FutureAction;
    
});
System.registerDynamic('node_modules/rxjs/scheduler/QueueScheduler.js', ['./QueueAction', './FutureAction'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var QueueAction_1 = $__require('./QueueAction');
    var FutureAction_1 = $__require('./FutureAction');
    var QueueScheduler = function () {
        function QueueScheduler() {
            this.active = false;
            this.actions = []; // XXX: use `any` to remove type param `T` from `VirtualTimeScheduler`.
            this.scheduledId = null;
        }
        QueueScheduler.prototype.now = function () {
            return Date.now();
        };
        QueueScheduler.prototype.flush = function () {
            if (this.active || this.scheduledId) {
                return;
            }
            this.active = true;
            var actions = this.actions;
            // XXX: use `any` to remove type param `T` from `VirtualTimeScheduler`.
            for (var action = null; action = actions.shift();) {
                action.execute();
                if (action.error) {
                    this.active = false;
                    throw action.error;
                }
            }
            this.active = false;
        };
        QueueScheduler.prototype.schedule = function (work, delay, state) {
            if (delay === void 0) {
                delay = 0;
            }
            return delay <= 0 ? this.scheduleNow(work, state) : this.scheduleLater(work, delay, state);
        };
        QueueScheduler.prototype.scheduleNow = function (work, state) {
            return new QueueAction_1.QueueAction(this, work).schedule(state);
        };
        QueueScheduler.prototype.scheduleLater = function (work, delay, state) {
            return new FutureAction_1.FutureAction(this, work).schedule(state, delay);
        };
        return QueueScheduler;
    }();
    exports.QueueScheduler = QueueScheduler;
    
});
System.registerDynamic("node_modules/rxjs/scheduler/queue.js", ["./QueueScheduler"], true, function ($__require, exports, module) {
  "use strict";

  var global = this || self,
      GLOBAL = global;
  var QueueScheduler_1 = $__require('./QueueScheduler');
  exports.queue = new QueueScheduler_1.QueueScheduler();
  
});
System.registerDynamic('node_modules/rxjs/symbol/rxSubscriber.js', ['../util/root'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var root_1 = $__require('../util/root');
    var Symbol = root_1.root.Symbol;
    exports.$$rxSubscriber = typeof Symbol === 'function' && typeof Symbol.for === 'function' ? Symbol.for('rxSubscriber') : '@@rxSubscriber';
    
});
System.registerDynamic('node_modules/rxjs/symbol/observable.js', ['../util/root'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var root_1 = $__require('../util/root');
    var Symbol = root_1.root.Symbol;
    if (typeof Symbol === 'function') {
        if (Symbol.observable) {
            exports.$$observable = Symbol.observable;
        } else {
            if (typeof Symbol.for === 'function') {
                exports.$$observable = Symbol.for('observable');
            } else {
                exports.$$observable = Symbol('observable');
            }
            Symbol.observable = exports.$$observable;
        }
    } else {
        exports.$$observable = '@@observable';
    }
    
});
System.registerDynamic('node_modules/rxjs/util/root.js', [], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var objectTypes = {
        'boolean': false,
        'function': true,
        'object': true,
        'number': false,
        'string': false,
        'undefined': false
    };
    exports.root = objectTypes[typeof self] && self || objectTypes[typeof window] && window;
    /* tslint:disable:no-unused-variable */
    var freeExports = objectTypes[typeof exports] && exports && !exports.nodeType && exports;
    var freeModule = objectTypes[typeof module] && module && !module.nodeType && module;
    var freeGlobal = objectTypes[typeof global] && global;
    if (freeGlobal && (freeGlobal.global === freeGlobal || freeGlobal.window === freeGlobal)) {
        exports.root = freeGlobal;
    }
    
});
System.registerDynamic('node_modules/rxjs/symbol/iterator.js', ['../util/root'], true, function ($__require, exports, module) {
    "use strict";

    var global = this || self,
        GLOBAL = global;
    var root_1 = $__require('../util/root');
    var Symbol = root_1.root.Symbol;
    if (typeof Symbol === 'function') {
        if (Symbol.iterator) {
            exports.$$iterator = Symbol.iterator;
        } else if (typeof Symbol.for === 'function') {
            exports.$$iterator = Symbol.for('iterator');
        }
    } else {
        if (root_1.root.Set && typeof new root_1.root.Set()['@@iterator'] === 'function') {
            // Bug for mozilla version
            exports.$$iterator = '@@iterator';
        } else if (root_1.root.Map) {
            // es6-shim specific logic
            var keys = Object.getOwnPropertyNames(root_1.root.Map.prototype);
            for (var i = 0; i < keys.length; ++i) {
                var key = keys[i];
                if (key !== 'entries' && key !== 'size' && root_1.root.Map.prototype[key] === root_1.root.Map.prototype['entries']) {
                    exports.$$iterator = key;
                    break;
                }
            }
        } else {
            exports.$$iterator = '@@iterator';
        }
    }
    
});
System.registerDynamic('node_modules/rxjs/Rx.js', ['./Subject', './Observable', './add/observable/bindCallback', './add/observable/bindNodeCallback', './add/observable/combineLatest', './add/observable/concat', './add/observable/defer', './add/observable/empty', './add/observable/forkJoin', './add/observable/from', './add/observable/fromEvent', './add/observable/fromEventPattern', './add/observable/fromPromise', './add/observable/interval', './add/observable/merge', './add/observable/race', './add/observable/never', './add/observable/of', './add/observable/range', './add/observable/throw', './add/observable/timer', './add/observable/zip', './add/operator/buffer', './add/operator/bufferCount', './add/operator/bufferTime', './add/operator/bufferToggle', './add/operator/bufferWhen', './add/operator/cache', './add/operator/catch', './add/operator/combineAll', './add/operator/combineLatest', './add/operator/concat', './add/operator/concatAll', './add/operator/concatMap', './add/operator/concatMapTo', './add/operator/count', './add/operator/dematerialize', './add/operator/debounce', './add/operator/debounceTime', './add/operator/defaultIfEmpty', './add/operator/delay', './add/operator/delayWhen', './add/operator/distinctUntilChanged', './add/operator/do', './add/operator/expand', './add/operator/filter', './add/operator/finally', './add/operator/first', './add/operator/groupBy', './add/operator/ignoreElements', './add/operator/audit', './add/operator/auditTime', './add/operator/last', './add/operator/let', './add/operator/every', './add/operator/map', './add/operator/mapTo', './add/operator/materialize', './add/operator/merge', './add/operator/mergeAll', './add/operator/mergeMap', './add/operator/mergeMapTo', './add/operator/multicast', './add/operator/observeOn', './add/operator/partition', './add/operator/pluck', './add/operator/publish', './add/operator/publishBehavior', './add/operator/publishReplay', './add/operator/publishLast', './add/operator/race', './add/operator/reduce', './add/operator/repeat', './add/operator/retry', './add/operator/retryWhen', './add/operator/sample', './add/operator/sampleTime', './add/operator/scan', './add/operator/share', './add/operator/single', './add/operator/skip', './add/operator/skipUntil', './add/operator/skipWhile', './add/operator/startWith', './add/operator/subscribeOn', './add/operator/switch', './add/operator/switchMap', './add/operator/switchMapTo', './add/operator/take', './add/operator/takeLast', './add/operator/takeUntil', './add/operator/takeWhile', './add/operator/throttle', './add/operator/throttleTime', './add/operator/timeout', './add/operator/timeoutWith', './add/operator/toArray', './add/operator/toPromise', './add/operator/window', './add/operator/windowCount', './add/operator/windowTime', './add/operator/windowToggle', './add/operator/windowWhen', './add/operator/withLatestFrom', './add/operator/zip', './add/operator/zipAll', './Operator', './Subscription', './Subscriber', './AsyncSubject', './ReplaySubject', './BehaviorSubject', './observable/ConnectableObservable', './Notification', './util/EmptyError', './util/ArgumentOutOfRangeError', './util/ObjectUnsubscribedError', './util/UnsubscriptionError', './scheduler/asap', './scheduler/async', './scheduler/queue', './symbol/rxSubscriber', './symbol/observable', './symbol/iterator'], true, function ($__require, exports, module) {
  "use strict";
  /* tslint:disable:no-unused-variable */
  // Subject imported before Observable to bypass circular dependency issue since
  // Subject extends Observable and Observable references Subject in it's
  // definition

  var global = this || self,
      GLOBAL = global;
  var Subject_1 = $__require('./Subject');
  exports.Subject = Subject_1.Subject;
  /* tslint:enable:no-unused-variable */
  var Observable_1 = $__require('./Observable');
  exports.Observable = Observable_1.Observable;
  // statics
  /* tslint:disable:no-use-before-declare */
  $__require('./add/observable/bindCallback');
  $__require('./add/observable/bindNodeCallback');
  $__require('./add/observable/combineLatest');
  $__require('./add/observable/concat');
  $__require('./add/observable/defer');
  $__require('./add/observable/empty');
  $__require('./add/observable/forkJoin');
  $__require('./add/observable/from');
  $__require('./add/observable/fromEvent');
  $__require('./add/observable/fromEventPattern');
  $__require('./add/observable/fromPromise');
  $__require('./add/observable/interval');
  $__require('./add/observable/merge');
  $__require('./add/observable/race');
  $__require('./add/observable/never');
  $__require('./add/observable/of');
  $__require('./add/observable/range');
  $__require('./add/observable/throw');
  $__require('./add/observable/timer');
  $__require('./add/observable/zip');
  //operators
  $__require('./add/operator/buffer');
  $__require('./add/operator/bufferCount');
  $__require('./add/operator/bufferTime');
  $__require('./add/operator/bufferToggle');
  $__require('./add/operator/bufferWhen');
  $__require('./add/operator/cache');
  $__require('./add/operator/catch');
  $__require('./add/operator/combineAll');
  $__require('./add/operator/combineLatest');
  $__require('./add/operator/concat');
  $__require('./add/operator/concatAll');
  $__require('./add/operator/concatMap');
  $__require('./add/operator/concatMapTo');
  $__require('./add/operator/count');
  $__require('./add/operator/dematerialize');
  $__require('./add/operator/debounce');
  $__require('./add/operator/debounceTime');
  $__require('./add/operator/defaultIfEmpty');
  $__require('./add/operator/delay');
  $__require('./add/operator/delayWhen');
  $__require('./add/operator/distinctUntilChanged');
  $__require('./add/operator/do');
  $__require('./add/operator/expand');
  $__require('./add/operator/filter');
  $__require('./add/operator/finally');
  $__require('./add/operator/first');
  $__require('./add/operator/groupBy');
  $__require('./add/operator/ignoreElements');
  $__require('./add/operator/audit');
  $__require('./add/operator/auditTime');
  $__require('./add/operator/last');
  $__require('./add/operator/let');
  $__require('./add/operator/every');
  $__require('./add/operator/map');
  $__require('./add/operator/mapTo');
  $__require('./add/operator/materialize');
  $__require('./add/operator/merge');
  $__require('./add/operator/mergeAll');
  $__require('./add/operator/mergeMap');
  $__require('./add/operator/mergeMapTo');
  $__require('./add/operator/multicast');
  $__require('./add/operator/observeOn');
  $__require('./add/operator/partition');
  $__require('./add/operator/pluck');
  $__require('./add/operator/publish');
  $__require('./add/operator/publishBehavior');
  $__require('./add/operator/publishReplay');
  $__require('./add/operator/publishLast');
  $__require('./add/operator/race');
  $__require('./add/operator/reduce');
  $__require('./add/operator/repeat');
  $__require('./add/operator/retry');
  $__require('./add/operator/retryWhen');
  $__require('./add/operator/sample');
  $__require('./add/operator/sampleTime');
  $__require('./add/operator/scan');
  $__require('./add/operator/share');
  $__require('./add/operator/single');
  $__require('./add/operator/skip');
  $__require('./add/operator/skipUntil');
  $__require('./add/operator/skipWhile');
  $__require('./add/operator/startWith');
  $__require('./add/operator/subscribeOn');
  $__require('./add/operator/switch');
  $__require('./add/operator/switchMap');
  $__require('./add/operator/switchMapTo');
  $__require('./add/operator/take');
  $__require('./add/operator/takeLast');
  $__require('./add/operator/takeUntil');
  $__require('./add/operator/takeWhile');
  $__require('./add/operator/throttle');
  $__require('./add/operator/throttleTime');
  $__require('./add/operator/timeout');
  $__require('./add/operator/timeoutWith');
  $__require('./add/operator/toArray');
  $__require('./add/operator/toPromise');
  $__require('./add/operator/window');
  $__require('./add/operator/windowCount');
  $__require('./add/operator/windowTime');
  $__require('./add/operator/windowToggle');
  $__require('./add/operator/windowWhen');
  $__require('./add/operator/withLatestFrom');
  $__require('./add/operator/zip');
  $__require('./add/operator/zipAll');
  /* tslint:disable:no-unused-variable */
  var Operator_1 = $__require('./Operator');
  exports.Operator = Operator_1.Operator;
  var Subscription_1 = $__require('./Subscription');
  exports.Subscription = Subscription_1.Subscription;
  var Subscriber_1 = $__require('./Subscriber');
  exports.Subscriber = Subscriber_1.Subscriber;
  var AsyncSubject_1 = $__require('./AsyncSubject');
  exports.AsyncSubject = AsyncSubject_1.AsyncSubject;
  var ReplaySubject_1 = $__require('./ReplaySubject');
  exports.ReplaySubject = ReplaySubject_1.ReplaySubject;
  var BehaviorSubject_1 = $__require('./BehaviorSubject');
  exports.BehaviorSubject = BehaviorSubject_1.BehaviorSubject;
  var ConnectableObservable_1 = $__require('./observable/ConnectableObservable');
  exports.ConnectableObservable = ConnectableObservable_1.ConnectableObservable;
  var Notification_1 = $__require('./Notification');
  exports.Notification = Notification_1.Notification;
  var EmptyError_1 = $__require('./util/EmptyError');
  exports.EmptyError = EmptyError_1.EmptyError;
  var ArgumentOutOfRangeError_1 = $__require('./util/ArgumentOutOfRangeError');
  exports.ArgumentOutOfRangeError = ArgumentOutOfRangeError_1.ArgumentOutOfRangeError;
  var ObjectUnsubscribedError_1 = $__require('./util/ObjectUnsubscribedError');
  exports.ObjectUnsubscribedError = ObjectUnsubscribedError_1.ObjectUnsubscribedError;
  var UnsubscriptionError_1 = $__require('./util/UnsubscriptionError');
  exports.UnsubscriptionError = UnsubscriptionError_1.UnsubscriptionError;
  var asap_1 = $__require('./scheduler/asap');
  var async_1 = $__require('./scheduler/async');
  var queue_1 = $__require('./scheduler/queue');
  var rxSubscriber_1 = $__require('./symbol/rxSubscriber');
  var observable_1 = $__require('./symbol/observable');
  var iterator_1 = $__require('./symbol/iterator');
  /* tslint:enable:no-unused-variable */
  /**
   * @typedef {Object} Rx.Scheduler
   * @property {Scheduler} queue Schedules on a queue in the current event frame
   * (trampoline scheduler). Use this for iteration operations.
   * @property {Scheduler} asap Schedules on the micro task queue, which uses the
   * fastest transport mechanism available, either Node.js' `process.nextTick()`
   * or Web Worker MessageChannel or setTimeout or others. Use this for
   * asynchronous conversions.
   * @property {Scheduler} async Schedules work with `setInterval`. Use this for
   * time-based operations.
   */
  var Scheduler = {
    asap: asap_1.asap,
    async: async_1.async,
    queue: queue_1.queue
  };
  exports.Scheduler = Scheduler;
  /**
   * @typedef {Object} Rx.Symbol
   * @property {Symbol|string} rxSubscriber A symbol to use as a property name to
   * retrieve an "Rx safe" Observer from an object. "Rx safety" can be defined as
   * an object that has all of the traits of an Rx Subscriber, including the
   * ability to add and remove subscriptions to the subscription chain and
   * guarantees involving event triggering (can't "next" after unsubscription,
   * etc).
   * @property {Symbol|string} observable A symbol to use as a property name to
   * retrieve an Observable as defined by the [ECMAScript "Observable" spec](https://github.com/zenparsing/es-observable).
   * @property {Symbol|string} iterator The ES6 symbol to use as a property name
   * to retrieve an iterator from an object.
   */
  var Symbol = {
    rxSubscriber: rxSubscriber_1.$$rxSubscriber,
    observable: observable_1.$$observable,
    iterator: iterator_1.$$iterator
  };
  exports.Symbol = Symbol;
  
});